@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -41,20 +41,24 @@ if not hasattr(sys, 'version_info') or sys.version_info <= (2, 4, 0, 'final'):
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-# to use PyQt API 2
-#if sys.version_info[0] == 2:
-#    import sip
-#    sip.setapi('QString', 2)
-#    sip.setapi('QVariant', 2)
-#    sip.setapi('QTime', 2)
-#    sip.setapi('QUrl', 2)
+# Force PyQt API 2 for PyQt4
+
+if sys.version_info[0] == 2:
+    try:
+        import sip
+        for api in ('QDate', 'QDateTime', 'QString', 'QTextStream',
+                    'QTime', 'QUrl', 'QVariant'):
+            sip.setapi(api, 2)
+    except Exception:
+        pass
 
 try:
-    from PyQt4.QtCore import *
-    from PyQt4.QtGui  import *
+    from code_saturne.Base.QtCore    import *
+    from code_saturne.Base.QtGui     import *
+    from code_saturne.Base.QtWidgets import *
 except ImportError:
-    print("\n  Error: Unable to import PyQt4.QtCore or PyQt4.QtGui modules.")
-    print("  Please check your PyQt4 installation.\n")
+    print("\n  Error: Unable to import QtCore or QtGui modules.")
+    print("  Please check your PyQt4 or PyQt5 installation.\n")
     sys.exit(0)
 
 
@@ -101,8 +105,7 @@ def process_cmd_line(argv):
                       action="store_false",
                       help="deactivate splash screen")
 
-
-    parser.set_defaults(splash_screen=True)
+    parser.set_defaults(splash_screen=True, enable_neptune_cfd=True)
 
     (options, args) = parser.parse_args(argv)
 
@@ -136,11 +139,12 @@ def main(argv, pkg):
 
     # Test the package name to know which modules have to be imported
     if pkg.name == 'code_saturne':
-        icons_path = os.path.join(pkg.get_dir('pkgpythondir'), 'Base', 'icons')
+        images_path = os.path.join(pkg.get_dir('pkgdatadir'), 'images')
         sys.path.insert(1, os.path.join(pkg.get_dir('pkgpythondir'), 'Base'))
     else:
-        icons_path = os.path.join(pkg.get_dir('pkgpythondir'), 'core', 'icons')
+        images_path = os.path.join(pkg.get_dir('pkgpythondir'), 'core', 'icons')
         sys.path.insert(1, os.path.join(pkg.get_dir('pkgpythondir'), 'core'))
+        sys.path.insert(1, pkg.get_dir('pythondir'))
 
     # Test if EOS modules could be imported
     cfg = cs_config.config()
@@ -163,12 +167,12 @@ def main(argv, pkg):
 
     case, spl = process_cmd_line(argv)
 
-    app = QApplication(argv)
+    app = QApplication(sys.argv)
     app.setOrganizationName(pkg.code_name) # Defines the name of subdirectory under .config
     app.setOrganizationDomain(pkg.url)
     app.setApplicationName("gui") # Defines the name of the configuration file
     #app.setWindowIcon(QIcon(":/icon.png"))
-    app.connect(app, SIGNAL("lastWindowClosed()"), app, SLOT("quit()"))
+    app.lastWindowClosed.connect(app.quit)
 
     # Locale detection
     locale = QLocale.system().name()
@@ -179,7 +183,7 @@ def main(argv, pkg):
 
     if spl:
         app.setOverrideCursor(QCursor(Qt.WaitCursor))
-        pixmap = QPixmap('%s/splashscreen.png' % icons_path)
+        pixmap = QPixmap('%s/splashscreen.png' % images_path)
         splash = QSplashScreen(pixmap, Qt.WindowStaysOnTopHint)
         splash.setMask(pixmap.mask()) # this is usefull if the splashscreen is not a regular ractangle...
         splash.show()
@@ -190,7 +194,7 @@ def main(argv, pkg):
         app.processEvents()
         QTimer.singleShot(1500, splash.hide)
 
-    from Base.MainView import MainView
+    from code_saturne.Base.MainView import MainView
     mv = MainView(cmd_package = pkg, cmd_case = case)
 
     try:
