@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -34,7 +34,6 @@ This module defines the following functions:
 # Import required Python modules
 #===============================================================================
 
-import datetime
 import os, sys
 import types, string, re, fnmatch
 from optparse import OptionParser
@@ -64,10 +63,17 @@ def process_cmd_line(argv, pkg):
 
     parser = OptionParser(usage=usage)
 
+    parser.add_option("--compute-build", dest="compute_build", type="string",
+                      metavar="<build>",
+                      help="base name or full path to the compute build")
+
     parser.add_option("-n", "--nprocs", dest="nprocs", type="int",
                       metavar="<nprocs>",
                       help="number of MPI processes for the computation")
 
+    parser.add_option("--nt", "--threads-per-task", dest="nthreads", type="int",
+                      help="number of OpenMP threads per task")
+
     parser.add_option("-p", "--param", dest="param", type="string",
                       metavar="<param>",
                       help="path or name of the parameters file")
@@ -101,6 +107,10 @@ def process_cmd_line(argv, pkg):
                       help="run the data preparation stage even if " \
                            + "the matching execution directory exists")
 
+    parser.add_option("--stage", dest="stage",
+                      action="store_true",
+                      help="stage data prior to preparation and execution")
+
     parser.add_option("--initialize", dest="initialize",
                       action="store_true",
                       help="run the data preparation stage")
@@ -113,7 +123,9 @@ def process_cmd_line(argv, pkg):
                       action="store_true",
                       help="run the results copy/cleanup stage")
 
+    parser.set_defaults(compute_build=False)
     parser.set_defaults(suggest_id=False)
+    parser.set_defaults(stage=False)
     parser.set_defaults(initialize=False)
     parser.set_defaults(execute=False)
     parser.set_defaults(finalize=False)
@@ -122,6 +134,7 @@ def process_cmd_line(argv, pkg):
     parser.set_defaults(domain=None)
     parser.set_defaults(id=None)
     parser.set_defaults(nprocs=None)
+    parser.set_defaults(nthreads=None)
 
     # Note: we could use args to pass a calculation status file as an argument,
     # which would allow pursuing the later calculation stages.
@@ -131,125 +144,109 @@ def process_cmd_line(argv, pkg):
     # Try to determine case directory
 
     casedir = None
+    staging_dir = None
     param = None
-    coupling= None
-    data = None
-    src = None
-
-    if options.coupling:
+    compute_build = None
 
+    if options.coupling and options.param:
         # Multiple domain case
+        cmd_line = sys.argv[0]
+        for arg in sys.argv[1:]:
+            cmd_line += ' ' + arg
+        err_str = 'Error:\n' + cmd_line + '\n' \
+                  '--coupling and -p/--param options are incompatible.\n'
+        sys.stderr.write(err_str)
+        sys.exit(1)
 
-        if options.param:
-            cmd_line = sys.argv[0]
-            for arg in sys.argv[1:]:
-                cmd_line += ' ' + arg
-            err_str = 'Error:\n' + cmd_line + '\n' \
-                      '--coupling and -p/--param options are incompatible.\n'
-            sys.stderr.write(err_str)
-            sys.exit(1)
+    casedir, staging_dir = cs_case.get_case_dir(case=options.case,
+                                                param=options.param,
+                                                coupling=options.coupling,
+                                                id=options.id)
 
-        coupling = os.path.realpath(options.coupling)
-        if not os.path.isfile(coupling):
-            cmd_line = sys.argv[0]
-            for arg in sys.argv[1:]:
-                cmd_line += ' ' + arg
-            err_str = 'Error:\n' + cmd_line + '\n' \
-                      'coupling parameters: ' + options.coupling + '\n' \
-                      'not found or not a file.\n'
-            sys.stderr.write(err_str)
-            sys.exit(1)
+    if casedir == None:
+        cmd_line = sys.argv[0]
+        for arg in sys.argv[1:]:
+            cmd_line += ' ' + arg
+        err_str = 'Error:\n' + cmd_line + '\n' \
+                  'run from directory \"' + str(os.getcwd()) + '\",\n' \
+                  'which does not seem to be inside a case directory.\n'
+        sys.stderr.write(err_str)
 
-        if options.case:
-            casedir = os.path.realpath(options.case)
-        else:
-            casedir = os.path.split(coupling)[0]
-
-    else:
-
-        # Single domain case
-
-        if options.param:
-            param = os.path.basename(options.param)
-            if param != options.param:
-                datadir = os.path.split(os.path.realpath(options.param))[0]
-                (casedir, data) = os.path.split(datadir)
-                if data != 'DATA': # inconsistent paramaters location.
-                    casedir = None
-
-        if options.case:
-            casedir = os.path.realpath(options.case)
-            data = os.path.join(casedir, 'DATA')
-            src = os.path.join(casedir, 'SRC')
-        else:
-            casedir = os.getcwd()
-            while os.path.basename(casedir):
-                data = os.path.join(casedir, 'DATA')
-                src = os.path.join(casedir, 'SRC')
-                if os.path.isdir(data) and os.path.isdir(src):
-                    break
-                casedir = os.path.split(casedir)[0]
-
-        if not (os.path.isdir(data) and os.path.isdir(src)):
-            casedir = None
-            cmd_line = sys.argv[0]
-            for arg in sys.argv[1:]:
-                cmd_line += ' ' + arg
-            err_str = 'Error:\n' + cmd_line + '\n' \
-                      'run from directory \"' + str(os.getcwd()) + '\",\n' \
-                      'which does not seem to be inside a case directory.\n'
-            sys.stderr.write(err_str)
+    param = options.param
 
     # Stages to run (if no filter given, all are done).
 
-    prepare_data = options.initialize
-    run_solver = options.execute
-    save_results = options.finalize
+    compute_build = options.compute_build
+
+    stages = {'prepare_data':options.stage,
+              'initialize':options.initialize,
+              'run_solver':options.execute,
+              'save_results':options.finalize}
 
     if not options.force:
         force_id = False
     else:
         force_id = True
 
-    if not (prepare_data or run_solver or save_results):
-        prepare_data = True
-        run_solver = True
-        save_results = True
+    # Stages to run (if no filter given, all are run; specific stages
+    # are given, all stages in thar range are run).
+
+    ordered_stages = ['prepare_data',
+                      'initialize',
+                      'run_solver',
+                      'save_results']
+
+    stages = {'prepare_data':options.stage,
+              'initialize':options.initialize,
+              'run_solver':options.execute,
+              'save_results':options.finalize}
+
+    stages_start = len(ordered_stages)
+    stages_end = -1
+
+    i = 0
+    for k in ordered_stages:
+        if stages[k] == True and stages_start > i:
+            stages_start = i
+        if stages[k] and stages_end < i+1:
+            stages_end = i + 1
+        i += 1
+
+    # Default if nothing provided, ensure range is filled otherwise
+
+    if stages_end < 0:
+        for k in ordered_stages:
+            stages[k] = True
+
+    else:
+        for i in range(stages_start + 1, stages_end -1):
+            stages[ordered_stages[i]] = True
+
+    # Forced number of ranks and threads
 
     n_procs = options.nprocs
+    n_threads = options.nthreads
 
-    return  (casedir, options.id, param, coupling,
+    return  (casedir, staging_dir, options.id, param, options.coupling,
              options.id_prefix, options.id_suffix, options.suggest_id, force_id,
-             n_procs, prepare_data, run_solver, save_results)
+             n_procs, n_threads, stages, compute_build)
 
 #===============================================================================
 # Run the calculation
 #===============================================================================
 
-def main(argv, pkg):
+def run(argv, pkg):
     """
-    Main function.
+    Run calculation;
+    returns return code, run id, and results directory path when created.
     """
 
-    (casedir, run_id, param, coupling,
-     id_prefix, id_suffix, suggest_id, force, n_procs,
-     prepare_data, run_solver, save_results) = process_cmd_line(argv, pkg)
+    (casedir, staging_dir, run_id, param, coupling,
+     id_prefix, id_suffix, suggest_id, force, n_procs, n_threads,
+     stages, compute_build) = process_cmd_line(argv, pkg)
 
     if not casedir:
-        return 1
-
-    if not run_id or suggest_id:
-        now = datetime.datetime.now()
-        run_id = now.strftime('%Y%m%d-%H%M')
-
-    if id_prefix:
-        run_id = id_prefix + run_id
-    if id_suffix:
-        run_id += id_suffix
-
-    if suggest_id:
-        print(run_id)
-        return 0
+        return 1, None, None
 
     # Use alternate compute (back-end) package if defined
 
@@ -257,10 +254,13 @@ def main(argv, pkg):
     config.read(pkg.get_global_configfile())
 
     pkg_compute = None
-    if config.has_option('install', 'compute_versions'):
-        compute_versions = config.get('install', 'compute_versions').split(':')
-        if compute_versions[0]:
-            pkg_compute = pkg.get_alternate_version(compute_versions[0])
+    if not compute_build:
+        if config.has_option('install', 'compute_versions'):
+            compute_versions = config.get('install', 'compute_versions').split(':')
+            if compute_versions[0]:
+                pkg_compute = pkg.get_alternate_version(compute_versions[0])
+    else:
+        pkg_compute = pkg.get_alternate_version(compute_build)
 
     if coupling:
 
@@ -269,13 +269,23 @@ def main(argv, pkg):
 
         if os.path.isfile(coupling):
             try:
-                exec(compile(open(coupling).read(), user_scripts, 'exec'))
+                c_locals = {}
+                exec(compile(open(coupling).read(), coupling, 'exec'),
+                     globals(),
+                     c_locals)
+                domains = c_locals['domains']
             except Exception:
                 execfile(coupling)
 
+        verbose = True
+        if suggest_id:
+            verbose = False
+
         c = cs_case_coupling.coupling(pkg,
                                       domains,
                                       casedir,
+                                      staging_dir=staging_dir,
+                                      verbose=verbose,
                                       package_compute=pkg_compute)
 
     else:
@@ -286,18 +296,37 @@ def main(argv, pkg):
         c = cs_case.case(pkg,
                          package_compute=pkg_compute,
                          case_dir=casedir,
+                         staging_dir=staging_dir,
                          domains=d)
 
+    # Determine run id if not forced
+
+    if not run_id or suggest_id:
+        run_id = c.suggest_id(id_prefix, id_suffix)
+
+        if suggest_id:
+            print(run_id)
+            return 0, run_id, None
+
     # Now run case
 
     retval = c.run(n_procs=n_procs,
+                   n_threads=n_threads,
                    run_id=run_id,
                    force_id=force,
-                   prepare_data=prepare_data,
-                   run_solver=run_solver,
-                   save_results=save_results)
+                   stages=stages)
 
-    return retval
+    return retval, c.run_id, c.result_dir
+
+#===============================================================================
+# Main function
+#===============================================================================
+
+def main(argv, pkg):
+    """
+    Main function.
+    """
+    return run(argv, pkg)[0]
 
 #-------------------------------------------------------------------------------
 
