@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -71,6 +71,15 @@ def main(argv, pkg):
     Main function.
     """
 
+    # Save PYTHONPATH as CS_SALOME_TOP_PYTHONPATH to avoid issues
+    # with launch of code due to PYTHONPATH with extra (cumulative) entries
+    # encountered with Python 3.6 (i.e. post Python 3.3, with current
+    # import logic).
+    pythonpath = os.getenv('PYTHONPATH')
+    if not pythonpath:
+        pythonpath = ""
+    os.environ['CS_SALOME_TOP_PYTHONPATH'] = pythonpath
+
     template = """\
 %(salomeenv)s
 CFDSTUDY_ROOT_DIR=%(prefix)s
@@ -83,38 +92,32 @@ export CFDSTUDY_ROOT_DIR PYTHONPATH
         sys.stderr.write("SALOME is not available in this installation.\n")
         sys.exit(1)
 
-    # Skipped modules (version 6.3.0): YACS,JOBMANAGER,HOMARD,OPENTURNS
-    default_modules = "GEOM,SMESH,MED,CFDSTUDY,PARAVIS,VISU"
+    default_modules = "GEOM,SHAPER,SMESH,FIELDS,CFDSTUDY,PARAVIS,YACS,JOBMANAGER,HOMARD,OPENTURNS"
 
     run_cmd = pkg.config.salome_run
     if not run_cmd:
         run_cmd = "${KERNEL_ROOT_DIR}/bin/salome/envSalome.py python ${KERNEL_ROOT_DIR}/bin/salome/runSalome.py"
 
     path = pkg.get_dir('pkgpythondir')
+    path = path+":"+pkg.get_dir('pythondir')
     if pkg.name == 'neptune_cfd':
-        cspath = os.path.join(pkg.get_dir('csdir'), "lib",
-                                                   "python" + sys.version[:3],
-                                                   "site-packages", "code_saturne")
+        cspath = os.path.join(pkg.get_dir('pythondir'), 'code_saturne')
         path = path+":"+cspath
 
-        # Test if EOS modules could be imported
-        if pkg.config.libs['eos'].have == "yes":
-            eosprefix = pkg.config.libs['eos'].prefix
-            try:
-                from distutils import sysconfig
-                eospath = os.path.join(sysconfig.get_python_lib(0, 0, prefix=eosprefix), 'eos')
-            except Exception:
-                eospath = ''
-            if eospath:
-                if os.path.isdir(eospath) and not eospath in sys.path:
-                    path = path+":"+eospath
+    # Test if EOS modules could be imported
+    if pkg.config.libs['eos'].have == "yes":
+        eosprefix = pkg.config.libs['eos'].prefix
+        try:
+            from distutils import sysconfig
+            eospath = os.path.join(sysconfig.get_python_lib(0, 0, prefix=eosprefix), 'eos')
+        except Exception:
+            eospath = ''
+        if eospath:
+            if os.path.isdir(eospath) and not eospath in sys.path:
+                path = path+":"+eospath
 
-    if pkg.name == 'neptune_cfd':
-        prefix = pkg.get_dir('csdir')
-        pythondir = pkg.get_dir('cspythondir')
-    else:
-        prefix = pkg.get_dir('prefix')
-        pythondir = pkg.get_dir('pythondir')
+    prefix = pkg.get_dir('prefix')
+    pythondir = pkg.get_dir('pythondir')
     cmd = template % {'salomeenv': pkg.config.salome_env,
                       'prefix': prefix,
                       'pythondir': pythondir,
