@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -28,45 +28,6 @@ import os
 # Local functions
 #===============================================================================
 
-def domain_auto_restart(domain, n_add):
-    """
-    Select latest valid checkpoint for restart, and
-    create ficstp file to add n_add time steps.
-    """
-
-    from cs_exec_environment import get_command_output
-
-    results_dir = os.path.abspath(os.path.join(self.result_dir, '..'))
-    results = os.listdir(results_dir)
-    results.sort(reverse=True)
-    for r in results:
-        m = os.path.join(results_dir, r, 'checkpoint', 'main')
-        if os.path.isfile(m):
-            try:
-                cmd = self.package.get_io_dump()
-                cmd += ' --section nbre_pas_de_temps --extract ' + m
-                res = get_command_output(cmd)
-                n_steps = int(get_command_output(cmd))
-            except Exception:
-                print('checkpoint of result: ' + r + ' does not seem usable')
-                continue
-            info_line = 'Restart from iterations ' + str(n_steps)
-            n_steps += n_add
-            info_line += ' to ' + str(n_steps)
-            print(info_line)
-            f = open(os.path.join(self.exec_dir, 'control_file'), 'w')
-            l = ['#Target number of time steps, determined by script\n',
-                 str(n_steps)]
-            f.writelines(l)
-            f.close()
-            domain.restart_input = os.path.join('RESU',
-                                                r,
-                                                'checkpoint')
-            print('using ' + domain.restart_input)
-            break
-
-    return
-
 #===============================================================================
 # Defining parameters for a calculation domain
 #===============================================================================
@@ -78,11 +39,6 @@ def domain_prepare_data_add(domain):
     in DATA and copy of link of restart files as defined by domain).
     """
 
-    # Example: select latest valid checkpoint file for restart
-
-    if False:
-        domain_auto_restart(domain, 200)
-
     return
 
 #-------------------------------------------------------------------------------
@@ -93,16 +49,6 @@ def domain_copy_results_add(domain):
     (called at beginning of data copy stage).
     """
 
-    # Example: clean some temporary files
-
-    if False:
-        import fnmatch
-        dir_files = os.listdir(self.exec_dir)
-        tmp_files = (fnmatch.filter(dir_files, '*.tmp')
-                     + fnmatch.filter(dir_files, '*.fort'))
-        for f in tmp_files:
-            os.remove(os.path.join(self.exec_dir, f))
-
     return
 
 #-------------------------------------------------------------------------------
@@ -112,16 +58,14 @@ def define_domain_parameters(domain):
     Define domain execution parameters.
     """
 
-    # Read parameters file
-    # (already done just prior to this stage when
-    # running script with --param option)
-
-    if False:
-        domain.read_parameter_file('param2.xml')
-
     # Reusing output from previous runs
     #----------------------------------
 
+    # By default, upon restart, if the previous mesh is available in the
+    # checkpoint, preprocessing is skipped. To allow additional preprocessing,
+    # set:
+    #   domain.preprocess_on_restart = True
+
     # To use the output of a previous Preprocessor (mesh import) run, set:
     #   domain.mesh_input = 'RESU/<run_id>/mesh_input'
 
@@ -135,6 +79,7 @@ def define_domain_parameters(domain):
     #  domain.restart_input = 'RESU/<run_id>/checkpoint'
 
     if domain.param == None:
+        domain.preprocess_on_restart = False
         domain.mesh_input = None
         domain.partition_input = None
         domain.restart_input = None
@@ -171,7 +116,7 @@ def define_domain_parameters(domain):
 
     # Command-line arguments useful for logging, or determining the calculation
     # type may be defined here, for example:
-    #   domain.logging_args = '--logp 1'
+    #   domain.logging_args = '--logp'
 
     if domain.param == None:
         domain.logging_args = None
@@ -185,8 +130,7 @@ def define_domain_parameters(domain):
     # The type of run may be determined using domain.solver_args.
     # For example:
     #   domain.solver_args = '--quality'
-    # allows activation of elementary mesh quality and gradient tests
-    # (for a fixed function: sin(x+2y+3z), while:
+    # allows activation of elementary mesh quality criteria output, while:
     #   domain.solver_args = '--benchmark'
     # allows running  basic linear algebra operation benchmarks.
     # To run the solver's preprocessing stage only (mesh joining, smoothing,
@@ -202,7 +146,7 @@ def define_domain_parameters(domain):
 
     # Additionnal compiler flags may be passed to the C, C++, or Fortran
     # compilers, and libraries may be added, in case linking of user
-    # subroutines against external libraries is needed.
+    # functions against external libraries is needed.
 
     # Note that compiler flags will be added before the default flags;
     # this helps ensure added search paths have priority, but also implies
@@ -217,13 +161,16 @@ def define_domain_parameters(domain):
     # Debugging options
     #------------------
 
-    # The solver may be run through Valgrind if this memory-checking tool
-    # is available. In this case, domain.valgrind should contain the matching
-    # command-line arguments, such as:
-    #   domain.valgrind = 'valgrind --tool=memcheck'
+    # To run the solver through a debugger, domain.debug should contain
+    # the matching command-line arguments, such as:
+    #   domain.debug = '--debugger=gdb'
+    # or (for Valgrind):
+    #   domain.debug = 'valgrind --tool=memcheck'
+    # or (for Valgrind and ddd):
+    #   domain.debug = '--debugger=ddd valgrind --tool=memcheck --vgdb-error=1'
 
     if domain.param == None:
-        domain.valgrind = None
+        domain.debug = None
 
     # import pprint
     # pprint.pprint(domain.__dict__)
