@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2019 EDF S.A.
+# Copyright (C) 1998-2020 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -108,6 +108,7 @@ class Dico:
         self.data['batch']            = None
         self.data['no_boundary_conditions'] = False
         self.data['salome']           = False
+        self.data['module']           = None
         self.data['package']          = None
         self.data['current_page']     = ""
         self.data['current_index']    = None
@@ -305,7 +306,7 @@ class XMLElement:
     def xmlCreateAttribute(self, **kwargs):
         """
         Set attributes to a XMLElement node, only if these attributes
-        does not allready exist.
+        do not already exist.
         """
         for attr, value in list(kwargs.items()):
             if not self.el.hasAttribute(attr):
@@ -1179,16 +1180,14 @@ class XMLDocument(XMLElement):
         if isinstance(node, XMLElement):
             node = node.el
 
-        if 'formula' not in node.tagName or 'dirichlet_formula' in node.tagName:
-            for n in node.childNodes:
-                if n.nodeType == Node.TEXT_NODE:
+        for n in node.childNodes:
+            if n.nodeType == Node.TEXT_NODE:
+                if node.tagName[-7:] != 'formula':
                     n.data = self.xmlNormalizeWhitespace(n.data)
-        else:
-            for n in node.childNodes:
-                if n.nodeType == Node.TEXT_NODE:
-                    while n.data[0] in (" ", "\n", "\t"):
+                else:
+                    while n.data[:1] in (" ", "\n", "\t"):
                         n.data = n.data[1:]
-                    while n.data[-1] in (" ", "\n", "\t"):
+                    while n.data[-1:] in (" ", "\n", "\t"):
                         n.data = n.data[:-1]
 
         node.normalize()
@@ -1237,7 +1236,11 @@ class XMLDocument(XMLElement):
 
 class Case(Dico, XMLDocument):
 
-    def __init__(self, package=None, file_name="", studymanager=False):
+    def __init__(self,
+                 package=None,
+                 file_name="",
+                 module=None,
+                 studymanager=False):
         """
         Instantiate a new dico and a new xml doc
         """
@@ -1246,10 +1249,20 @@ class Case(Dico, XMLDocument):
 
         if package:
             self['package'] = package
-            rootNode = '<' + self['package'].code_name +'_GUI study="" case="" version="2.0"/>'
         else:
-            rootNode = '<' + '_GUI study="" case="" version="2.0"/>'
+            from code_saturne import cs_package
+            self['package'] = cs_package.package()
 
+        code_name = ''
+        if module:
+            if module == 'code_saturne':
+                code_name = 'Code_Saturne'
+            elif module == 'neptune_cfd':
+                code_name = 'NEPTUNE_CFD'
+        if package and not code_name:
+            code_name = package.code_name
+
+        rootNode = '<' + code_name +'_GUI study="" case="" version="2.0"/>'
         if studymanager:
             rootNode = '<studymanager/>'
 
@@ -1268,6 +1281,14 @@ class Case(Dico, XMLDocument):
         self.xml_saved = self.toString()
 
 
+    def module_name(self):
+        # Specific module
+        if (self.doc.documentElement.tagName == "NEPTUNE_CFD_GUI"):
+            return 'neptune_cfd'
+
+        # General case
+        return 'code_saturne'
+
     def xmlRootNode(self):
         """
         This function return the only one root element of the document
@@ -1460,12 +1481,9 @@ class Case(Dico, XMLDocument):
             file.write('sys.path.insert(0, "' + self['package'].dirs['pythondir'][1] + '")\n')
             file.write('sys.path.insert(0, "' + os.path.join(self['package'].dirs['pythondir'][1], self['package'].name) + '")\n\n')
 
-            if self['package'].code_name == 'Code_Saturne':
-                file.write('import cs_package\n')
+            if self.module_name() == 'code_saturne':
                 file.write('from model.XMLinitialize import XMLinit\n')
-            else:
-                file.write('sys.path.insert(0, "' + os.path.join(self['package'].dirs['pythondir'][1], "code_saturne") + '")\n\n')
-                file.write('import nc_package\n')
+            elif self.module_name() == 'neptune_cfd':
                 file.write('from model.XMLinitializeNeptune import XMLinit\n')
             file.write('from model.XMLengine import Case\n\n')
 
@@ -1475,10 +1493,7 @@ class Case(Dico, XMLDocument):
                 name = os.path.splitext(self['pythonfile'])[0] + ".xml"
                 file.write("fp = '" + name + "'\n")
 
-            if self['package'].code_name == 'Code_Saturne':
-                file.write("case = Case(package = cs_package.package())\n")
-            else:
-                file.write("case = Case(package = nc_package.package())\n")
+            file.write("case = Case(module=" + self.module_name + ")\n")
             file.write("case['xmlfile'] = fp\n")
             file.write("case.xmlCleanAllBlank(case.xmlRootNode())\n")
             file.write("XMLinit(case).initialize()\n")
