@@ -5,7 +5,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2013 EDF S.A.
+# Copyright (C) 1998-2018 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -36,10 +36,8 @@ import os.path
 # This is not quite standard (though methods and automation level used by other
 # tools vary widely), but we want to minimize files that need to be edited
 # when releasing a version, but use a robust mechanism which will work
-# for versions pulled by different version control tools (at least
-# Subversion, used for the main repository, git svn clones, which may or
-# may not have up-to-date fetched remotes information, and clones of those
-# clones...). As we need to maintain the NEWS file anyways to provide a clear
+# for versions either pulled by Git or simple release tarballs.
+# As we need to maintain the NEWS file anyways to provide a clear
 # user-readable summary of changes between versions, using a consistent
 # presentation, using this file to determine version information is a
 # practical solution, which requires enforcing that the NEWS file is
@@ -61,25 +59,25 @@ def version_from_news(srcdir):
     release = ''
     extra = ''
 
-    trunk = -1
+    master = -1
     version = [-1]
     release = [-1, '', '', '']
 
-    f = open(os.path.join(srcdir, "NEWS"))
+    f = open(os.path.join(srcdir, "NEWS.md"))
     lines = f.readlines()
     f.close()
 
     for i in range(1, len(lines)):
 
-        # Release names are underlined by '==='
+        # Release names are underlined by '---'
 
-        if lines[i][0:7] == '=======':
+        if lines[i][0:7] == '-------':
 
             j = i-1
             l = lines[j]
 
-            if l[0:6] == 'Trunk ' and trunk < 0:
-                trunk = j
+            if l[0:7] == 'Master ' and master < 0:
+                master = j
 
             # If release appears, at which line, and is there a date,
             # or is it not yet released ?
@@ -92,28 +90,20 @@ def version_from_news(srcdir):
                     version = [j] + n.split('.')
 
                 if release[0] < 0:
-                    try:
-                        dateinfo = info.split('(')[1].split(')')[0].split()
-                        num_count = 0
-                        for d in dateinfo:
-                            try:
-                                if int(d) > 0:
-                                    num_count += 1
-                            except Exception:
-                                pass
-                        if num_count > 1:
+                    s = ''.join(c for c in info if c.isdigit())
+                    if len(s) > 0:
                             release = [j] + n.split('.')
                             break
-                    except Exception:
-                        pass
 
-    if trunk > -1:
+    if master > -1:
         if version[0] > -1:
-            major, minor = trunk_major_minor(version[1], version[2])
+            major, minor = master_major_minor(version[1], version[2])
             release = ''
             extra = '-alpha'
 
     elif version[0] > -1:
+        while len(version) < 4:
+            version.append('')
         major, minor, release, extra \
             = branch_release_extra(version[1], version[2], version[3],
                                    release[1], release[2], release[3])
@@ -122,9 +112,9 @@ def version_from_news(srcdir):
 
 #-------------------------------------------------------------------------------
 
-def trunk_major_minor(prev_major, prev_minor):
+def master_major_minor(prev_major, prev_minor):
     """
-    For trunk, determine next version based on previous branches
+    For master, determine next version based on previous branches
     """
 
     # Current release cycle plans for 4 minor releases per major release
@@ -161,7 +151,7 @@ def branch_release_extra(cur_major, cur_minor, cur_release,
 
     else:
         try:
-            if int(cur_release) == 0:
+            if not cur_release or int(cur_release) == 0:
                 release = ''
                 extra = '-beta'
         except Exception:
@@ -171,219 +161,6 @@ def branch_release_extra(cur_major, cur_minor, cur_release,
 
 #-------------------------------------------------------------------------------
 
-def svn_version_from_branch(branch):
-    """
-    Determine branch version number from Subversion branch
-    """
-
-    major = ''
-    minor = ''
-
-    if branch[0:7] == 'Version':
-        v = branch[7:]
-        sep = v.find('_')
-        if (sep > -1):
-            major = v[0:sep]
-            minor = v[sep+1:]
-
-    return major, minor
-
-#-------------------------------------------------------------------------------
-
-def svn_version_from_tag(tag):
-    """
-    Determine branch version number from Subversion tag
-    """
-
-    major = ''
-    minor = ''
-    release = ''
-    extra = ''
-
-    if tag[0] == 'V':
-        v = tag[1:].split('_', 3)
-        if len(v) >= 1:
-            major = v[0]
-        if len(v) >= 2:
-            minor = v[1]
-        if len(v) >= 3:
-            if v[2] > '-1':
-                release = v[2]
-        if len(v) >= 4:
-            extra = v[3]
-
-    return major, minor, release, extra
-
-#-------------------------------------------------------------------------------
-
-def svn_version(srcdir, defaults):
-    """
-    Determine version information from Subversion.
-    """
-
-    # Use default info to pre-determine next version
-
-    major, minor, release, extra = defaults
-
-    import subprocess
-
-    # Use the XML output from Subversion, to avoid formatting/language
-    # environment parsing issues
-
-    try:
-        from xml.dom import minidom
-    except Exception:
-        return major, minor, release, extra, revision
-
-    # Get base info from 'svn info'
-
-    url = None
-    revision = ''
-
-    cmd = ['svn', 'info', '--xml', srcdir]
-    p = subprocess.Popen(cmd,
-                         stdout=subprocess.PIPE,
-                         stderr=subprocess.PIPE)
-    output = p.communicate()
-    if p.returncode != 0:
-        return major, minor, release, extra, revision
-    else:
-        try:
-            svn_info = minidom.parseString(output[0]).documentElement
-            url = svn_info.getElementsByTagName("url").item(0).firstChild.data
-            entry_node = svn_info.getElementsByTagName("entry")[0]
-            revision = entry_node.getAttribute("revision")
-        except Exception:
-            pass
-
-    if not url:
-        return major, minor, release, extra, revision
-
-    # If we are in trunk, use previous local info to determine next version
-
-    # If we are in a branch, use tags to determine next version
-
-    if url.find('/branches/') > -1:
-        url_id_0 = url.find('/branches/')
-        url_id_1 = url_id_0 + len('/branches/')
-        url_root = url[:url_id_0]
-
-        if url[url_id_1:url_id_1+7] == 'Version':
-            major, minor = svn_version_from_branch(url[url_id_1:])
-
-        cmd = ['svn', 'ls', '--xml', url_root + '/tags']
-
-        p = subprocess.Popen(cmd,
-                             stdout=subprocess.PIPE,
-                             stderr=subprocess.PIPE)
-        output = p.communicate()
-        if p.returncode != 0:
-            return major, minor, release, extra, revision
-        else:
-            try:
-                tag_ref = 'V' + str(major) + '_' + str(minor) + '_'
-                tag_ref_l = len(tag_ref)
-                name_max = tag_ref + '-1'
-                release = ''
-                svn_info = minidom.parseString(output[0]).documentElement
-                for tag in svn_info.getElementsByTagName("name"):
-                    name = tag.firstChild.data
-                    if name[0:tag_ref_l] == tag_ref and name > name_max:
-                        rev_node = tag.parentNode.getElementsByTagName("commit")[0]
-                        tag_rev = rev_node.getAttribute("revision")
-                        if (tag_rev == revision):
-                            release = tag[tag_ref_l:]
-                            break
-                        elif tag_rev < revision:
-                            name_max = name
-                major, minor, release, extra = svn_version_from_tag(name_max)
-                if not release:
-                    extra += '-beta'
-                if tag_rev == revision:
-                    revision = ''
-            except Exception:
-                pass
-
-    # If we are at a tag, simply use if
-
-    elif url.find('/tags/') > -1:
-        url_id_0 = url.find('/tags/')
-        url_id_1 = url_id_0 + len('/tags/')
-        tag = url[url_id_1:]
-        major, minor, release, extra = svn_version_from_tag(url[url_id_1:])
-        revision = ''
-
-    if revision:
-        revision = '-r' + revision
-    else:
-        revision = ''
-
-    return major, minor, release, extra, revision
-
-#-------------------------------------------------------------------------------
-
-def svn_version_is_modified(srcdir):
-    """
-    Determine if version is modifed relative to Subversion revision
-    (requires a local operation only).
-    """
-
-    import subprocess
-
-    p = subprocess.Popen(['svn', 'status', '--xml', srcdir],
-                         stdout=subprocess.PIPE,
-                         stderr=subprocess.PIPE)
-    output = p.communicate()
-
-    if p.returncode == 0:
-        if output[0].find('entry') > -1:
-            return True
-
-    return False
-
-#-------------------------------------------------------------------------------
-
-def git_svn_version(srcdir, url, revision, defaults):
-    """
-    Determine partial version information from Subversion.
-    """
-
-    # Use known info to pre-determine next version
-
-    major, minor, release, extra = defaults
-
-    import subprocess
-
-    # If we are in a branch, use tags if available to determine next version
-
-    if url.find('/branches/') > -1:
-        url_id_0 = url.find('/branches/')
-        url_id_1 = url_id_0 + len('/branches/')
-        url_root = url[:url_id_0]
-
-        if url[url_id_1:url_id_1+7] == 'Version':
-            major, minor = svn_version_from_branch(url[url_id_1:])
-            revision = '-svn' + revision
-
-    # If we are at a tag, simply use if
-
-    elif url.find('/tags/') > -1:
-        url_id_0 = url.find('/tags/')
-        url_id_1 = url_id_0 + len('/tags/')
-        tag = url[url_id_1:]
-        major, minor, release, extra = svn_version_from_tag(url[url_id_1:])
-        revision = ''
-
-    # If we are in trunk, branch data might not be available to determine
-    # next version, so use defaults
-
-    else:
-        revision = '-svn' + revision
-
-    return major, minor, release, extra, revision
-
-#-------------------------------------------------------------------------------
-
 def git_version(srcdir, defaults):
     """
     Determine version information from Git.
@@ -395,38 +172,30 @@ def git_version(srcdir, defaults):
 
     import subprocess
 
-    # Get base info from 'svn info'
+    # Get base info
 
     url = None
     revision = ''
-    head_is_svn = False
+    newline_failed = False
 
     cmd = ['git', '--no-pager', '--git-dir='+os.path.join(srcdir, '.git'),
-           '--work-tree='+srcdir, 'log', '-n', '200']
+           '--work-tree='+srcdir, 'log', '--pretty=format:%h','-n', '1']
+
+    # Universal newlines options may fail in some cases with
+    # Python 3, so check for newlines using '\n' and '\\n' later.
+
     p = subprocess.Popen(cmd,
                          stdout=subprocess.PIPE,
-                         stderr=subprocess.PIPE)
+                         stderr=subprocess.PIPE,
+                         universal_newlines=False)
     output = p.communicate()
     if p.returncode != 0:
         return major, minor, release, extra
     else:
-        svn_id = output[0].find('git-svn-id:')
-        if svn_id > 1:
-            svn_rev = output[0][svn_id:].split(' ', 2)[1]
-            svn_url, svn_revision = svn_rev.split('@')
-            major, minor, release, extra, revision \
-                = git_svn_version(srcdir, svn_url, svn_revision, defaults)
-            # Try to count commits before the one with svn
-            svn_log = output[0][:svn_id].split()
-            c1 = svn_log.count('commit')
-            c2 = svn_log.count('Author:')
-            if c1 == 1 and c2 == 1:
-                head_is_svn = True
-
-    if not head_is_svn:
-        commit_id = output[0].find('commit')
-        commit_rev = output[0][commit_id:].split(' ', 2)[1].split('\n')[0]
-        revision += '-git' + commit_rev
+        commit_rev = output[0]
+        if sys.version[0] == '3':
+            commit_rev = commit_rev.decode('utf-8')
+        revision += '-' + commit_rev
 
     return major, minor, release, extra, revision
 
@@ -444,10 +213,17 @@ def git_version_is_modified(srcdir):
            '--work-tree='+srcdir, 'status', '-s', '-uno']
     p = subprocess.Popen(cmd,
                          stdout=subprocess.PIPE,
-                         stderr=subprocess.PIPE)
+                         stderr=subprocess.PIPE,
+                         universal_newlines=True)
     output = p.communicate()
+    o0 = str(output[0])
     if p.returncode == 0:
-        if len(output[0].split('\n')) > 1:
+        changes = o0.split('\n')
+        n_changes = len(changes) - 1
+        for c in changes:
+            if c[-3:] == '.po':
+                n_changes -= 1
+        if n_changes > 0:
             return True
 
     return False
@@ -466,6 +242,7 @@ def build_version_string(major, minor, release, extra, revision = None):
         version += extra
     if revision:
         version += revision
+    version = version.replace('\r', '')
     return version
 
 #-------------------------------------------------------------------------------
@@ -482,39 +259,63 @@ if __name__ == '__main__':
 
     if len(sys.argv) == 1:
         major, minor, release, extra = version_from_news(srcdir)
-        print(build_version_string(major, minor, release, extra))
+        sys.stdout.write(build_version_string(major, minor, release, extra))
 
     elif sys.argv[1] == '--short':
         major, minor, release, extra = version_from_news(srcdir)
-        print(build_version_string(major, minor, '', extra))
+        sys.stdout.write(build_version_string(major, minor, '', extra))
 
     # Call tools
 
     elif sys.argv[1] in ['--full', '--revision-only', '--verbose']:
 
         defaults = version_from_news(srcdir)
+        release = ''
         modified = ''
         revision = ''
+        extra = ''
 
-        if os.path.isdir(os.path.join(srcdir, ".svn")):
-            major, minor, release, extra, revision \
-                = svn_version(srcdir, defaults)
-            modified = svn_version_is_modified(srcdir)
-        elif os.path.isdir(os.path.join(srcdir, ".git")):
+        if os.path.isdir(os.path.join(srcdir, ".git")):
             major, minor, release, extra, revision \
                 = git_version(srcdir, defaults)
             modified = git_version_is_modified(srcdir)
+        elif os.path.isfile(os.path.join(srcdir, "build-aux", "version")):
+            f = open(os.path.join(srcdir, "build-aux", "version"))
+            lines = f.readlines()
+            f.close()
+            v = lines[0].strip()
+            for e in ['-alpha', '-beta', '-rc']:
+                i = v.find(e)
+                if i > -1:
+                    j = v[i+1:].find('-')
+                    if j < 0:
+                        j = v[i+1:].find('_')
+                    if j > -1:
+                        extra = v[i:i+j+1]
+                    else:
+                        extra = v[i:]
+                    v = v.replace(extra, '')
+            vl = v.split(".")
+            l = len(vl)
+            vt = vl.pop(l-1)
+            vl += vt.split("-", 1)
+            major = vl[0]
+            minor = vl[1]
+            if l > 2:
+                release = vl[2]
+            if len(vl) > l:
+                revision = '-' + vl[l]
         else:
             major, minor, release, extra = version_from_news(srcdir)
 
         if sys.argv[1] == '--full':
             if modified:
                 revision += '-m'
-            print(build_version_string(major, minor, release, extra, revision))
+            sys.stdout.write(build_version_string(major, minor, release, extra, revision))
         elif sys.argv[1] == '--revision-only':
             if modified:
                 revision += '-m'
-            print(revision[1:]) # remove first '-' character
+            sys.stdout.write(revision[1:]) # remove first '-' character
         elif sys.argv[1] == '--verbose':
             print('major:    ' + major)
             print('minor:    ' + minor)
