@@ -1,10 +1,10 @@
-#!/bin/sh
+# Shell script
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -32,35 +32,34 @@
 # cflags_default_dbg     # Added to $CFLAGS for debugging    (default: "-g")
 # cflags_default_opt     # Added to $CFLAGS for optimization (default: "-O")
 # cflags_default_hot     # Optimization for specific files   (default: "-O")
-# cflags_default_prf     # Added to $CFLAGS for profiling    (default: "")
+# cflags_default_prf     # Added to $CFLAGS for profiling    (default: "-g")
 # cflags_default_omp     # Added to $CFLAGS for OpenMP       (default: "")
-# cflags_default_ext     # Added to $CFLAGS for extended     (default: "")
-#                        # precision if available
 
 # cxxflags_default       # Base CXXFLAGS                       (default: "")
 # cxxflags_default_dbg   # Added to $CXXFLAGS for debugging    (default: "-g")
 # cxxflags_default_opt   # Added to $CXXFLAGS for optimization (default: "-O")
 # cxxflags_default_hot   # Optimization for specific files     (default: "-O")
-# cxxflags_default_prf   # Added to $CXXFLAGS for profiling    (default: "")
+# cxxflags_default_prf   # Added to $CXXFLAGS for profiling    (default: "-g")
 # cxxflags_default_omp   # Added to $CXXFLAGS for OpenMP       (default: "")
+# cxxflags_default_std   # C++ standard variant                (default: "")
 
 # fcflags_default        # Base FCFLAGS                       (default: "")
 # fcflags_default_dbg    # Added to $FCFLAGS for debugging    (default: "-g")
 # fcflags_default_opt    # Added to $FCFLAGS for optimization (default: "-O")
 # fcflags_default_hot    # Optimization for specific files    (default: "-O")
-# fcflags_default_prf    # Added to $FCFLAGS for profiling    (default: "")
+# fcflags_default_prf    # Added to $FCFLAGS for profiling    (default: "-g")
 # fcflags_default_omp    # Added to $FCFLAGS for OpenMP       (default: "")
 #
 # ldflags_default        # Base LDFLAGS                       (default: "")
 # ldflags_default_dbg    # Added to $LDFLAGS for debugging    (default: "-g")
 # ldflags_default_opt    # Added to $LDFLAGS for optimization (default: "-O")
-# ldflags_default_prf    # Added to $LDFLAGS for profiling    (default: "")
+# ldflags_default_prf    # Added to $LDFLAGS for profiling    (default: "-g")
 # ldflags_rpath          # Added to $LDFLAGS for shared libs  (default: "")
 
 # libs_default           # Base LIBS                          (default: "")
-# libs_default_dbg       # Added to $LDFLAGS for debugging    (default: "")
-# libs_default_opt       # Added to $LDFLAGS for optimization (default: "")
-# libs_default_prf       # Added to $LDFLAGS for profiling    (default: "")
+# libs_default_dbg       # Added to $LIBS for debugging       (default: "")
+# libs_default_opt       # Added to $LIBS for optimization    (default: "")
+# libs_default_prf       # Added to $LIBS for profiling       (default: "")
 
 # Two other environment variable strings are defined, containing possibly
 # more detailed compiler information:
@@ -130,6 +129,8 @@ esac
 cs_ac_cc_version=unknown
 cs_cc_compiler_known=no
 
+cflags_default_prf="-g"
+
 # Are we using gcc ?
 #-------------------
 
@@ -142,10 +143,10 @@ if test "x$GCC" = "xyes"; then
 
   if test -n "`$CC --version | grep icc`" ; then
     cs_gcc=icc
+  elif test -n "`$CC --version | grep clang`" ; then
+    cs_gcc=clang
   elif test -n "`$CC --version 2>&1 | grep PathScale`" ; then
     cs_gcc=pathcc
-  elif test -n "`$CC --version 2>&1 | grep Open64`" ; then
-    cs_gcc=open64
   else
     cs_gcc=gcc
   fi
@@ -185,7 +186,6 @@ if test "x$cs_gcc" = "xgcc"; then
   cflags_default_dbg="-g"
   cflags_default_opt="-O2"
   cflags_default_hot="-O3"
-  cflags_default_prf="-pg"
   cflags_default_omp="-fopenmp"
 
   # Modify default flags on certain systems
@@ -196,13 +196,8 @@ if test "x$cs_gcc" = "xgcc"; then
       cflags_default_opt="-funroll-loops -O2 -Wuninitialized"
       case "$host_cpu" in
         i686)
-          case "$cs_cc_vendor-$cs_cc_version" in
-            gcc-3*|gcc-4*)
-              cflags_default_opt="$cflags_default_opt -march=i686"
-              ;;
-          esac
+          cflags_default_opt="$cflags_default_opt -march=i686"
           ;;
-
       esac
       ;;
 
@@ -212,13 +207,29 @@ if test "x$cs_gcc" = "xgcc"; then
   # may not handle all flags)
 
   case "$cs_cc_vendor-$cs_cc_version" in
-    gcc-3*|gcc-4.[012]*)
+    gcc-[45]*)
+      ;;
+    *)
+      cflags_default="$cflags_default -Wmisleading-indentation -Wduplicated-cond"
+      ;;
+  esac
+
+  case "$cs_cc_vendor-$cs_cc_version" in
+    gcc-4.[012345678]*)
+      ;;
+    *)
+      cflags_default="$cflags_default -fdiagnostics-color=auto"
+      ;;
+  esac
+
+  case "$cs_cc_vendor-$cs_cc_version" in
+    gcc-4.[012]*)
       cflags_default_omp=""
       ;;
   esac
 
   case "$cs_cc_vendor-$cs_cc_version" in
-    gcc-3*|gcc-4.[01234]*)
+    gcc-4.[01234]*)
       ;;
     *)
       cflags_default_opt="$cflags_default_opt -fexcess-precision=fast"
@@ -247,25 +258,49 @@ elif test "x$cs_gcc" = "xicc"; then
   cs_ac_cc_version=`$CC --version 2>&1 | head -1`
   cs_cc_compiler_known=yes
 
+  # Some version numbers
+  cs_cc_vers_major=`echo $cs_ac_cc_version | cut -f 3 -d" " | cut -f1 -d.`
+  cs_cc_vers_minor=`echo $cs_ac_cc_version | cut -f 3 -d" " | cut -f2 -d.`
+  cs_cc_vers_patch=`echo $cs_ac_cc_version | cut -f 3 -d" " | cut -f3 -d.`
+  test -n "$cs_cc_vers_major" || cs_cc_vers_major=0
+  test -n "$cs_cc_vers_minor" || cs_cc_vers_minor=0
+  test -n "$cs_cc_vers_patch" || cs_cc_vers_patch=0
+
   # Default compiler flags
   # (temporarily disable "operands evaluated in unspecified order" remark -- 981)
-  cflags_default="-strict-ansi -std=c99 -funsigned-char -Wall -Wcheck -Wshadow -Wpointer-arith -Wmissing-prototypes -Wuninitialized -Wunused -wd981"
+  cflags_default="-std=c99 -restrict -funsigned-char -Wall -Wcheck -Wshadow -Wpointer-arith -Wmissing-prototypes -Wuninitialized -Wunused -wd981"
   cflags_default_dbg="-g -O0 -traceback -w2 -Wp64 -ftrapuv"
   cflags_default_opt="-O2"
   cflags_default_hot="-O3"
-  cflags_default_prf="-p"
-  cflags_default_omp="-openmp"
-
-  # Modify default flags on certain systems
-
-  case "$host-os-$host_cpu" in
-    *ia64)
-      cflags_default_opt="-O2 -mcpu=itanium2-p9000"
-      cflags_default_hot="-O3 -mcpu=itanium2-p9000"
-      cflags_default_ext="-fp-model extended"
+  cflags_default_omp="-qopenmp"
+  case "$cs_cc_vers_major" in
+    1[0123456])
+      cflags_default_omp="-openmp"
       ;;
   esac
 
+# Otherwise, are we using clang ?
+#--------------------------------
+
+elif test "x$cs_gcc" = "xclang"; then
+
+  cs_cc_version=`echo $CC --version | grep clang | cut -f 3 -d ' '`
+
+  echo "compiler '$CC' is clang"
+
+  # Version strings for logging purposes and known compiler flag
+  $CC -v > $outfile 2>&1
+  cs_ac_cc_version=`$CC --version 2>&1 | head -1`
+  cs_cc_compiler_known=yes
+
+  # Default compiler flags
+  # (temporarily disable "operands evaluated in unspecified order" remark -- 981)
+  cflags_default="-std=c99 -funsigned-char -Wall -Wshadow -Wpointer-arith -Wmissing-prototypes -Wuninitialized -Wunused"
+  cflags_default_dbg="-g -O0"
+  cflags_default_opt="-O2"
+  cflags_default_hot="-O3"
+  cflags_default_omp="-fopenmp=libomp"
+
 # Otherwise, are we using pathcc ?
 #---------------------------------
 
@@ -287,33 +322,6 @@ elif test "x$cs_gcc" = "xpathcc"; then
     cflags_default_dbg="-g"
     cflags_default_opt="-O2"
     cflags_default_hot="-Ofast"
-    cflags_default_prf=""
-    cflags_default_omp="-openmp"
-
-  fi
-
-# Otherwise, are we using opencc ?
-#---------------------------------
-
-elif test "x$cs_gcc" = "xopen64"; then
-
-  $CC --version 2>&1 | grep 'Open64' > /dev/null
-  if test "$?" = "0" ; then
-
-    echo "compiler '$CC' is Open64 C compiler"
-
-    # Version strings for logging purposes and known compiler flag
-    $CC --version > $outfile 2>&1
-    cs_ac_cc_version=`grep -i Compiler $outfile`
-    cs_cc_compiler_known=yes
-
-    # Default compiler flags
-    cflags_default="-std=c99"
-    cflags_default="-std=c99 -funsigned-char -W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wunused -Wunused-value"
-    cflags_default_dbg="-g"
-    cflags_default_opt="-O2"
-    cflags_default_hot="-Ofast"
-    cflags_default_prf=""
     cflags_default_omp="-openmp"
 
   fi
@@ -340,7 +348,6 @@ if test "x$cs_cc_compiler_known" != "xyes" ; then
     cflags_default_dbg="-g -Mbounds"
     cflags_default_opt="-O2"
     cflags_default_hot="-fast"
-    cflags_default_prf="-Mprof=func,lines"
     cflags_default_omp="-mp"
 
   fi
@@ -368,14 +375,12 @@ if test "x$cs_cc_compiler_known" != "xyes" ; then
     cflags_default_opt="-O3"
     cflags_default_hot="-O3"
     cflags_default_dbg="-g -qfullpath"
-    cflags_default_prf="-pg -qfullpath"
     cflags_default_omp="-qsmp=omp -qthreaded"
 
     # Default  linker flags
     ldflags_default=""
     ldflags_default_opt="-O3"
     ldflags_default_dbg="-g -qfullpath"
-    ldflags_default_prf="-pg -qfullpath"
     ldflags_rpath="-R"
 
     # Adjust options for IBM Blue Gene cross-compiler
@@ -383,14 +388,7 @@ if test "x$cs_cc_compiler_known" != "xyes" ; then
     grep 'Blue Gene' $outfile > /dev/null
     if test "$?" = "0" ; then
       # Default compiler flags (we assume that MPI wrappers are used)
-      cs_ibm_bg_type=`grep 'Blue Gene' $outfile | sed -e 's/.*Blue Gene\/\([A-Z]\).*/\1/'`
-      if test "x$cs_ibm_bg_type" = "xP" ; then
-        cppflags_default="-I/bgsys/drivers/ppcfloor/arch/include"
-        cflags_default="-qlanglvl=extc99"
-        cflags_default_opt="-O3"
-        cflags_default_hot="-O3 -qhot"
-        cflags_default_dbg="-g"
-      elif test -f /bgsys/drivers/ppcfloor/cnk/bin/bgq_kernel.elf ; then
+      if test -f /bgsys/drivers/ppcfloor/cnk/bin/bgq_kernel.elf ; then
         cs_ibm_bg_type="Q"
         cppflags_default=""
         cflags_default=""                    # "-qlanglvl=extc99" by default
@@ -423,14 +421,12 @@ if test "x$cs_cc_compiler_known" != "xyes" ; then
     cflags_default_opt="-O2"
     cflags_default_hot="-O3"
     cflags_default_dbg="-g"
-    cflags_default_prf="-h profile_generate" # resulting code must be run under CrayPat
     cflags_default_omp="-h omp"              # default: use "-h noomp" to disable
 
     # Default  linker flags
     ldflags_default=""
     ldflags_default_opt="-O2"
     ldflags_default_dbg="-g"
-    ldflags_default_prf="-h profile_generate"
 
   fi
 fi
@@ -442,115 +438,14 @@ if test "x$cs_cc_compiler_known" != "xyes" ; then
 
   case "$host_os" in
 
-    SUPER-UX* | superux*)
-
-      # Native NEC SX vectorizing C compiler (sxmpicc)
-      #-------------------------------------
-
-      $CC -V conftest.c 2>&1 | grep 'NEC' | grep 'SX' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$CC' is NEC SX compiler"
-
-        # Version strings for logging purposes and known compiler flag
-        $CC -V conftest.c > $outfile 2>&1
-        cs_ac_cc_version=`grep ccom $outfile`
-        cs_cc_compiler_known=yes
-        cs_linker_set=yes
-
-        # Default compiler flags
-        cflags_default="-Kc99 -pvctl,loopcnt=2147483647"
-        cflags_default_opt=""
-        cflags_default_dbg=""
-        cflags_default_prf=""
-        cflags_default_omp=""
-
-        # Default linker flags
-        ldflags_default=""
-        ldflags_default_opt="-O"
-        ldflags_default_dbg="-g"
-        ldflags_default_prf="-pg"
-
-      fi
-      ;;
-
-    hpux*)
-
-      # Native HP-UX C compiler
-      #------------------------
-
-      $CC -V conftest.c 2>&1 | grep 'HP' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$CC' is HP compiler"
-
-        # Version strings for logging purposes and known compiler flag
-        $CC -V conftest.c > $outfile 2>&1
-        cs_ac_cc_version=`grep ccom $outfile`
-        cs_cc_compiler_known=yes
-        cs_linker_set=yes
-
-        # Default compiler flags
-        cflags_default="-AC99 +e"
-        cflags_default_opt="+O2"
-        cflags_default_hot="+O3"
-        cflags_default_dbg="-g"
-        cflags_default_prf="-G"
-        cflags_default_omp="+Oopenmp" # most pragmas require +O3
-
-        # Default linker flags
-        ldflags_default="+FPVZOUD +U77"
-        ldflags_default_opt="+O1"
-        ldflags_default_dbg="-g"
-        ldflags_default_prf="-fbexe"
-        cflags_default_omp="+Oopenmp"
-
-        if test "$host_cpu" = "ia64" ; then
-          cflags_default="$cflags_default +DD64"
-          ldflags_default="$ldflags_default +DD64"
-        else
-          cflags_default="$cflags_default +DA2.0w"
-          ldflags_default="$ldflags_default +DA2.0w"
-        fi
-
-      fi
-      ;;
-
-    solaris2.*)
-
-      # Sun Workshop compiler
-      #----------------------
-
-      $CC -V 2>&1 | grep 'WorkShop Compilers' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$CC' is Sun WorkShop Compilers"
-
-        # Version strings for logging purposes and known compiler flag
-        $CC -V conftest.c > $outfile 2>&1
-        cs_ac_cc_version=`grep cc $outfile`
-        cs_cc_compiler_known=yes
-
-        # Default compiler flags
-        cflags_default="-Xa -Xc99"
-        cflags_default_opt="-xO2"
-        cflags_default_hot="-xO3"
-        cflags_default_dbg="-g"
-        cflags_default_prf="-pg"
-        cflags_default_omp="-xopenmp"
-
-     fi
-     ;;
-
     *)
 
-      # Unknown
+      # Generic
       #--------
 
       cflags_default=""
       cflags_default_opt="-O"
       cflags_default_dbg="-g"
-      cflags_default_prf=""
       cflags_default_omp=""
       ;;
 
@@ -577,6 +472,8 @@ fi
 cs_ac_cxx_version=unknown
 cs_cxx_compiler_known=no
 
+cxxflags_default_prf="-g"
+
 # Are we using g++ ?
 #-------------------
 
@@ -587,8 +484,12 @@ if test "x$GXX" = "xyes"; then
   # Intel and Pathscale compilers may pass as GXX but
   # may be recognized by version string
 
-  if test -n "`$CXX --version | grep icc`" ; then
+  if test -n "`$CXX --version | grep icpc`" ; then
+    cs_gxx=icpc
+  elif test -n "`$CXX --version | grep icc`" ; then
     cs_gxx=icc
+  elif test -n "`$CXX --version | grep clang`" ; then
+    cs_gxx=clang
   elif test -n "`$CXX --version 2>&1 | grep PathScale`" ; then
     cs_gxx=pathCC
   else
@@ -605,8 +506,8 @@ if test "x$cs_gxx" = "xg++"; then
   cs_cxx_compiler_known=yes
 
   # Practical version info for option setting
-  cs_cxx_version="`$CXX -v 2>&1 |grep 'g++ version' |\
-                  sed 's/.*g++ version \([-a-z0-9\.]*\).*/\1/'`"
+  cs_cxx_version="`$CXX -v 2>&1 |grep 'gcc version' |\
+                  sed 's/.*gcc version \([-a-z0-9\.]*\).*/\1/'`"
   cs_cxx_vendor=`echo $cxx_version |sed 's/\([a-z]*\).*/\1/'`
   cs_cxx_version=`echo $cs_cxx_version |sed 's/[-a-z]//g'`
 
@@ -618,20 +519,20 @@ if test "x$cs_gxx" = "xg++"; then
   fi
 
   # Some version numbers
-  cs_cxx_vers_major=`echo $cxx_version | cut -f1 -d.`
-  cs_cxx_vers_minor=`echo $cxx_version | cut -f2 -d.`
-  cs_cxx_vers_patch=`echo $cxx_version | cut -f3 -d.`
+  cs_cxx_vers_major=`echo $cs_cxx_version | cut -f1 -d.`
+  cs_cxx_vers_minor=`echo $cs_cxx_version | cut -f2 -d.`
+  cs_cxx_vers_patch=`echo $cs_cxx_version | cut -f3 -d.`
   test -n "$cs_cxx_vers_major" || cs_cxx_vers_major=0
   test -n "$cs_cxx_vers_minor" || cs_cxx_vers_minor=0
   test -n "$cs_cxx_vers_patch" || cs_cxx_vers_patch=0
 
   # Default compiler flags
-  cxxflags_default="-ansi -funsigned-char -W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wunused -Wno-long-long -Wfloat-equal"
+  cxxflags_default="-W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wunused -Wfloat-equal"
   cxxflags_default_dbg="-g"
   cxxflags_default_opt="-O2"
   cxxflags_default_hot="-O3"
-  cxxflags_default_prf="-pg"
   cxxflags_default_omp="-fopenmp"
+  cxxflags_default_std="-funsigned-char"
 
   # Modify default flags on certain systems
 
@@ -641,11 +542,8 @@ if test "x$cs_gxx" = "xg++"; then
       cxxflags_default_opt="-funroll-loops -O2 -Wuninitialized"
       case "$host_cpu" in
         i686)
-          case "$cs_cxx_vendor-$cs_cxx_version" in
-            g++-3*|g++-4*)
-              cxxflags_default_opt="$cxxflags_default_opt -march=i686"
-            ;;
-          esac
+          cxxflags_default_opt="$cxxflags_default_opt -march=i686"
+          ;;
       esac
       ;;
 
@@ -654,6 +552,22 @@ if test "x$cs_gxx" = "xg++"; then
   # Modify default flags depending on g++ version (as older versions
   # may not handle all flags)
 
+  case "$cs_cxx_vendor-$cs_cxx_version" in
+    g++-[45]*)
+      ;;
+    *)
+      cxxflags_default="$cxxflags_default -std=c++11 -Wmisleading-indentation -Wduplicated-cond"
+      ;;
+  esac
+
+  case "$cs_cxx_vendor-$cs_cxx_version" in
+    g++-4.[012345678]*)
+      ;;
+    *)
+      cxxflags_default="$cxxflags_default -fdiagnostics-color=auto"
+      ;;
+  esac
+
   case "$cs_cxx_vendor-$cs_cxx_version" in
     g++-2.*|g++-3*|g++-4.[012]*)
       cxxflags_default_omp=""
@@ -663,9 +577,13 @@ if test "x$cs_gxx" = "xg++"; then
 # Otherwise, are we using icc ?
 #------------------------------
 
-elif test "x$cs_gxx" = "xicc"; then
+elif test "x$cs_gxx" = "xicpc" -o "x$cs_gxx" = "xicc"; then
 
-  cs_cxx_version=`echo $CXX --version | grep icc |sed 's/[a-zA-Z()]//g'`
+  if test "x$cs_gxx" = "xicpc"; then
+    cs_cxx_version=`echo $CXX --version | grep icpc |sed 's/[a-zA-Z()]//g'`
+  else
+    cs_cxx_version=`echo $CXX --version | grep icc |sed 's/[a-zA-Z()]//g'`
+  fi
 
   echo "compiler '$CXX' is Intel ICC"
 
@@ -674,22 +592,50 @@ elif test "x$cs_gxx" = "xicc"; then
   cs_ac_cxx_version=`$CXX --version 2>&1 | head -1`
   cs_cxx_compiler_known=yes
 
+  cs_cxx_vers_major=`echo $cs_ac_cxx_version | cut -f 3 -d" " | cut -f1 -d.`
+  cs_cxx_vers_minor=`echo $cs_ac_cxx_version | cut -f 3 -d" " | cut -f2 -d.`
+  cs_cxx_vers_patch=`echo $cs_ac_cxx_version | cut -f 3 -d" " | cut -f3 -d.`
+  test -n "$cs_cxx_vers_major" || cs_cxx_vers_major=0
+  test -n "$cs_cxx_vers_minor" || cs_cxx_vers_minor=0
+  test -n "$cs_cxx_vers_patch" || cs_cxx_vers_patch=0
+
   # Default compiler flags
-  cxxflags_default="-strict-ansi -funsigned-char -Wall -Wcheck -Wshadow -Wpointer-arith -Wmissing-prototypes -Wuninitialized -Wunused"
+  cxxflags_default="-std=c++11 -Wall -Wcheck -Wshadow -Wpointer-arith -Wmissing-prototypes -Wuninitialized -Wunused"
   cxxflags_default_dbg="-g -O0 -traceback -w2 -Wp64 -ftrapuv"
   cxxflags_default_opt="-O2"
   cxxflags_default_hot="-O3"
-  cxxflags_default_prf="-p"
-  cxxflags_default_omp="-openmp"
+  cxxflags_default_omp="-qopenmp"
+  cxxflags_default_std="-funsigned-char"
 
-  # Modify default flags on certain systems
-
-  case "$host-os-$host_cpu" in
-    *ia64)
-      cxxflags_default_opt="-O2 -mcpu=itanium2-p9000"
+  case "$cs_cxx_vers_major" in
+    1[0123456])
+      cxxflags_default_omp="-openmp"
       ;;
   esac
 
+# Otherwise, are we using clang ?
+#--------------------------------
+
+elif test "x$cs_gxx" = "xclang"; then
+
+  cs_cc_version=`echo $CXX --version | grep clang | cut -f 3 -d ' '`
+
+  echo "compiler '$CXX' is clang"
+
+  # Version strings for logging purposes and known compiler flag
+  $CXX -v > $outfile 2>&1
+  cs_ac_cxx_version=`$CXX --version 2>&1 | head -1`
+  cs_cxx_compiler_known=yes
+
+  # Default compiler flags
+  # (temporarily disable "operands evaluated in unspecified order" remark -- 981)
+  cxxlags_default="-Wall -Wshadow -Wpointer-arith -Wmissing-prototypes -Wuninitialized -Wunused"
+  cxxflags_default_dbg="-g -O0"
+  cxxflags_default_opt="-O2"
+  cxxflags_default_hot="-O3"
+  cxxflags_default_omp="-fopenmp=libomp"
+  cxxflags_default_std="-funsigned-char"
+
 # Otherwise, are we using pgcc ?
 #-------------------------------
 
@@ -706,11 +652,11 @@ else
     cs_cxx_compiler_known=yes
 
     # Default compiler flags
-    cxxflags_default="-Xa -noswitcherror"
+    cxxflags_default="-noswitcherror"
     cxxflags_default_dbg="-g -Mbounds"
     cxxflags_default_opt="-fast -fastsse"
-    cxxflags_default_prf="-Mprof=func,lines"
     cxxflags_default_omp="-mp"
+    cxxflags_default_std="-Xa"
 
   fi
 
@@ -736,20 +682,15 @@ if test "x$cs_cxx_compiler_known" != "xyes" ; then
     cxxflags_default_opt="-O3"
     cxxflags_default_hot="-O3"
     cxxflags_default_dbg="-g"
-    cxxflags_default_prf="-pg"
     cxxflags_default_omp="-qsmp=omp -qthreaded"
+    cxxflags_default_std=""
 
     # Adjust options for IBM Blue Gene cross-compiler
 
     grep 'Blue Gene' $outfile > /dev/null
     if test "$?" = "0" ; then
       # Default compiler flags (we assume that MPI wrappers are used)
-      if test "x$cs_ibm_bg_type" = "xP" ; then
-        cxxflags_default=""
-        cxxflags_default_opt="-O3"
-        cxxflags_default_hot="-O3 -qhot"
-        cxxflags_default_dbg="-g"
-      elif test "x$cs_ibm_bg_type" = "xQ" ; then
+      if test -f /bgsys/drivers/ppcfloor/cnk/bin/bgq_kernel.elf ; then
         cxxflags_default="-qlanglvl=redefmac"
         cxxflags_default_opt="-g -O3"
         cxxflags_default_hot="-g -O3 -qhot"
@@ -779,8 +720,8 @@ if test "x$cs_cxx_compiler_known" != "xyes" ; then
     cxxflags_default_opt="-O2"
     cxxflags_default_hot="-O3"
     cxxflags_default_dbg="-g"
-    cfxxlags_default_prf="-h profile_generate" # resulting code must be run under CrayPat
     cfxxlags_default_omp="-h omp"              # default: use "-h noomp" to disable
+    cxxflags_default_std=""
 
   fi
 
@@ -802,39 +743,12 @@ if test "x$cs_cxx_compiler_known" != "xyes" ; then
     cs_cxx_compiler_known=yes
 
     # Default compiler flags
-    cxxflags_default="-ansi -W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wunused -Wunused-value"
+    cxxflags_default="-W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wunused -Wunused-value"
     cxxflags_default_dbg="-g"
     cxxflags_default_opt="-O2"
     cxxflags_default_hot="-Ofast"
-    cxxflags_default_prf=""
-    cxxflags_default_omp="-openmp"
-
-  fi
-
-fi
-
-# Otherwise, are we using openCC ?
-#---------------------------------
-
-if test "x$cs_cxx_compiler_known" != "xyes" ; then
-
-  $CXX --version 2>&1 | grep 'Open64' > /dev/null
-  if test "$?" = "0" ; then
-
-    echo "compiler '$CXX' is Open64"
-
-    # Version strings for logging purposes and known compiler flag
-    $CXX --version > $outfile 2>&1
-    cs_ac_cxx_version=`grep -i Compiler $outfile`
-    cs_cxx_compiler_known=yes
-
-    # Default compiler flags
-    cxxflags_default="-ansi -W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wunused -Wunused-value"
-    cxxflags_default_dbg="-g"
-    cxxflags_default_opt="-O2"
-    cxxflags_default_hot="-Ofast"
-    cxxflags_default_prf=""
     cxxflags_default_omp="-openmp"
+    cxxflags_default_std=""
 
   fi
 
@@ -847,101 +761,16 @@ if test "x$cs_cxx_compiler_known" != "xyes" ; then
 
   case "$host_os" in
 
-    SUPER-UX*)
-
-      # Native NEC SX vectorizing C compiler (sxmpicc)
-      #-------------------------------------
-
-      $CXX -V conftest.c 2>&1 | grep 'NEC' | grep 'SX' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$CXX' is NEC SX compiler"
-
-        # Version strings for logging purposes and known compiler flag
-        $CXX -V conftest.c > $outfile 2>&1
-        cs_ac_cxx_version=`grep ccom $outfile`
-        cs_cxx_compiler_known=yes
-        cs_linker_set=yes
-
-        # Default compiler flags
-        cxxflags_default="-ftrace --pvctl,loopcnt=2147483647"
-        cxxflags_default_opt=""
-        cxxflags_default_dbg=""
-        cxxflags_default_prf=""
-        cxxflags_default_omp=""
-
-      fi
-      ;;
-
-    hpux*)
-
-      # Native HP-UX C compiler
-      #------------------------
-
-      $CXX -V conftest.c 2>&1 | grep 'HP' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$CXX' is HP compiler"
-
-        # Version strings for logging purposes and known compiler flag
-        $CXX -V conftest.c > $outfile 2>&1
-        cs_ac_cxx_version=`grep ccom $outfile`
-        cs_cxx_compiler_known=yes
-        cs_linker_set=yes
-
-        # Default compiler flags
-        cxxflags_default="-Aa +e"
-        cxxflags_default_opt="+O2"
-        cxxflags_default_hot="+O3"
-        cxxflags_default_dbg="-g"
-        cxxflags_default_prf="-G"
-        cxxflags_default_omp="+Oopenmp" # most pragmas require +O3
-
-        if test "$host_cpu" = "ia64" ; then
-          cxxflags_default="$cxxflags_default +DD64"
-        else
-          cxxflags_default="$cxxflags_default +DA2.0w"
-        fi
-
-      fi
-      ;;
-
-    solaris2.*)
-
-      # Sun Workshop compiler
-      #----------------------
-
-      $CXX -V 2>&1 | grep 'WorkShop Compilers' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$CXX' is Sun WorkShop Compilers"
-
-        # Version strings for logging purposes and known compiler flag
-        $CXX -V conftest.c > $outfile 2>&1
-        cs_ac_cxx_version=`grep cc $outfile`
-        cs_cxx_compiler_known=yes
-
-        # Default compiler flags
-        cxxflags_default="-Xa"
-        cxxflags_default_opt="-xO2"
-        cxxflags_default_hot="-xO3"
-        cxxflags_default_dbg="-g"
-        cxxflags_default_prf="-pg"
-        cxxflags_default_omp="-xopenmp"
-
-     fi
-     ;;
-
     *)
 
-      # Unknown
+      # Generic
       #--------
 
       cxxflags_default=""
       cxxflags_default_opt="-O"
       cxxflags_default_dbg="-g"
-      cxxflags_default_prf=""
       cxxflags_default_omp=""
+      cxxflags_default_std=""
       ;;
 
   esac
@@ -967,6 +796,8 @@ fi
 cs_ac_fc_version=unknown
 cs_fc_compiler_known=no
 
+fcflags_default_prf="-g"
+
 # Are we using gfortran ?
 #------------------------
 
@@ -983,8 +814,6 @@ if test "$?" = "0" ; then
   cs_fc_version="`$FC -v 2>&1 |grep 'gcc version' |\
                   sed 's/.*gcc version \([-a-z0-9\.]*\).*/\1/'`"
 
-  echo "compiler '$FC' is gfortran"
-
   cs_fc_compiler_known=yes
   cs_gfortran=gfortran
 
@@ -1004,6 +833,8 @@ if test "$?" = "0" ; then
   fi
   if test "-" != "$cs_fc_vendor-$cs_fc_version"; then
     echo "compiler '$FC' is GNU $cs_fc_vendor-$cs_fc_version"
+  else
+    echo "compiler '$FC' is gfortran"
   fi
 
   # Some version numbers
@@ -1019,13 +850,20 @@ if test "$?" = "0" ; then
   fcflags_default_dbg="-g -fcheck=bounds"
   fcflags_default_opt="-O"
   fcflags_default_hot="-O2"
-  fcflags_default_prf="-pg"
   fcflags_default_omp="-fopenmp"
 
-  case "$cs_fc_vendor-$cs_fc_version" in
-    gfortran-4.[234]*)
-    fcflags_default_dbg="`echo $fcflags_default_dbg | sed -e 's/-fcheck=bounds/-fbounds-check/g'`"
-  esac
+  if test "xgfortran" = "x$cs_fc_vendor"; then
+    case "$cs_fc_version" in
+      4.[234]*)
+        fcflags_default_dbg="`echo $fcflags_default_dbg | sed -e 's/-fcheck=bounds/-fbounds-check/g'`"
+        ;;
+      4.[56789]*)
+        ;;
+      *)
+        fcflags_default="$fcflags_default -fdiagnostics-color=auto"
+       ;;
+    esac
+  fi
 
 fi
 
@@ -1046,21 +884,24 @@ if test "x$cs_fc_compiler_known" != "xyes" ; then
     cs_ac_fc_version=`$FC --version 2>&1 | head -1`
     cs_fc_compiler_known=yes
 
+    cs_fc_vers_major=`echo $cs_ac_fc_version | cut -f 3 -d" " | cut -f1 -d.`
+    cs_fc_vers_minor=`echo $cs_ac_fc_version | cut -f 3 -d" " | cut -f2 -d.`
+    cs_fc_vers_patch=`echo $cs_ac_fc_version | cut -f 3 -d" " | cut -f3 -d.`
+    test -n "$cs_fc_vers_major" || cs_fc_vers_major=0
+    test -n "$cs_fc_vers_minor" || cs_fc_vers_minor=0
+    test -n "$cs_fc_vers_patch" || cs_fc_vers_patch=0
+
     # Default compiler flags
     # (temporarily disable "unused variable" remark -- 7712)
     fcflags_default="-cpp -fpic -warn -diag-disable 7712"
     fcflags_default_dbg="-g -O0 -traceback -check all -check nopointer -fpe0 -ftrapuv"
     fcflags_default_opt="-O2"
     fcflags_default_hot="-O3"
-    fcflags_default_prf="-p"
-    fcflags_default_omp="-openmp"
+    fcflags_default_omp="-qopenmp"
 
-    # Modify default flags on certain systems
-
-    case "$host-os-$host_cpu" in
-      *ia64)
-        fcflags_default_opt="-O2 -mcpu=itanium2-p9000"
-        fcflags_default_hot="-O3 -mcpu=itanium2-p9000"
+    case "$cs_cxx_vers_major" in
+      1[0123456])
+        fcflags_default_omp="-openmp"
         ;;
     esac
 
@@ -1087,7 +928,6 @@ if test "x$cs_fc_compiler_known" != "xyes" ; then
     fcflags_default_dbg="-g -Mbounds"
     fcflags_default_opt="-O2"
     fcflags_default_hot="-fast"
-    fcflags_default_prf="-Mprof=func,lines"
     fcflags_default_omp="-mp"
 
   fi
@@ -1113,7 +953,6 @@ if test "x$cs_fc_compiler_known" != "xyes" ; then
     fcflags_default="-q64 -qextname -qsuffix=cpp=f90"
     fcflags_default_dbg="-g"
     fcflags_default_opt="-O3"
-    fcflags_default_prf="-pg"
     fcflags_default_omp="-qsmp=omp -qthreaded"
 
     # Adjust options for IBM Blue Gene cross-compiler
@@ -1123,12 +962,7 @@ if test "x$cs_fc_compiler_known" != "xyes" ; then
     if test "$?" = "0" ; then
 
       # Default compiler flags (we assume that MPI wrappers are used)
-      if test "$cs_ibm_bg_type" = "P" ; then
-        fcflags_default="-qextname -qsuffix=cpp=f90"
-        fcflags_default_dbg="-g"
-        fcflags_default_opt="-O3"
-        fcflags_default_hot="-O3 -qhot"
-      elif test "x$cs_ibm_bg_type" = "xQ" ; then
+      if test -f /bgsys/drivers/ppcfloor/cnk/bin/bgq_kernel.elf ; then
         fcflags_default="-qextname -qsuffix=cpp=f90"
         fcflags_default_dbg="-g -qcheck"
         fcflags_default_opt="-g -O3"
@@ -1154,10 +988,9 @@ if test "x$cs_fc_compiler_known" != "xyes" ; then
     cs_ac_fc_version=`$FC -V 2>&1 | grep "Cray Fortran" | head -1`
     cs_fc_compiler_known=yes
 
-    fcflags_default="-eF -em"
+    fcflags_default="-eF -em -J."
     fcflags_default_dbg="-g"
     fcflags_default_opt="-O2"
-    fcflags_default_prf="-h profile_generate" # resulting code must be run under CrayPat
     fcflags_default_omp="-h omp"              # default: use "-h noomp" to disable
 
   fi
@@ -1183,33 +1016,6 @@ if test "x$cs_fc_compiler_known" != "xyes" ; then
     fcflags_default_dbg="-g -ffortran-bounds-check"
     fcflags_default_opt="-O"
     fcflags_default_hot="-fast"
-    fcflags_default_prf=""
-    fcflags_default_omp="-openmp"
-
-  fi
-fi
-
-if test "x$cs_fc_compiler_known" != "xyes" ; then
-
-  # Are we using openf95?
-  #---------------------
-
-  $FC --version 2>&1 | grep 'Open64' > /dev/null
-
-  if test "$?" = "0" ; then
-
-    echo "compiler '$FC' is Open64 Fortran compiler"
-
-    # Version strings for logging purposes and known compiler flag
-    $FC --version > $outfile 2>&1
-    cs_ac_fc_version=`grep 'Open64' $outfile`
-    cs_fc_compiler_known=yes
-
-    fcflags_default="-Wall -Wno-unused -cpp -fno-second-underscore"
-    fcflags_default_dbg="-g -ffortran-bounds-check"
-    fcflags_default_opt="-O"
-    fcflags_default_hot="-fast"
-    fcflags_default_prf=""
     fcflags_default_omp="-openmp"
 
   fi
@@ -1219,68 +1025,15 @@ if test "x$cs_fc_compiler_known" != "xyes" ; then
 
   case "$host_os" in
 
-    SUPER-UX* | superux*)
-
-      # Native NEC SX vectorizing Fortran compiler (sxmpif90)
-      #-------------------------------------------
-
-      $FC -V conftest.f 2>&1 | grep 'NEC' | grep 'SX' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$FC' is NEC SX compiler"
-
-        # Version strings for logging purposes and known compiler flag
-        $FC -V conftest.f > $outfile 2>&1
-        cs_ac_fc_version=`grep ccom $outfile`
-        cs_fc_compiler_known=yes
-
-        # Default compiler flags
-        fcflags_default="-Ep -C hopt -I. -Wf,-pvctl,loopcnt=2147483647"
-        fcflags_default_opt=""
-        fcflags_default_hot=""
-        fcflags_default_dbg=""
-        fcflags_default_prf=""
-        fcflags_default_omp=""
-
-      fi
-      ;;
-
-    hpux*)
-
-      # Native HP-UX Fortran compiler
-      #------------------------------
-
-      $FC +version 2>&1 | grep 'HP' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$FC' is HP compiler"
-
-        # Version strings for logging purposes and known compiler flag
-        $FC -V conftest.f > $outfile 2>&1
-        cs_ac_fc_version=`$FC +version`
-        cs_fc_compiler_known=yes
-
-        # Default compiler flags
-        fcflags_default="+cpp=yes +fp_exception +FPVZOUD +U77 +DA2.0W +noppu"
-        fcflags_default_opt="+O1"
-        fcflags_default_hot="+O2"
-        fcflags_default_dbg="-g"
-        fcflags_default_prf="-G"
-        fcflags_default_omp="+Oopenmp" # most pragmas require +O3
-
-      fi
-      ;;
-
     *)
 
-      # Unknown
+      # Generic
       #--------
 
       fcflags_default=""
       fcflags_default_opt="-O"
       fcflags_default_hot="-O"
       fcflags_default_dbg="-g"
-      fcflags_default_prf=""
       ;;
 
   esac
@@ -1306,6 +1059,8 @@ fi
 # Default linker flags
 #---------------------
 
+ldflags_default_prf="-g"
+
 if test "x$cs_linker_set" != "xyes" ; then
 
   case "$host_os" in
@@ -1314,7 +1069,6 @@ if test "x$cs_linker_set" != "xyes" ; then
       ldflags_default=""
       ldflags_default_opt="-O"
       ldflags_default_dbg="-g"
-      ldflags_default_prf="-pg"
       ldflags_rpath="-Wl,-rpath -Wl,"
       if test "x$cs_gfortran" = "xgfortran"; then
         libgfortran_path=`$FC --print-file-name=libgfortran.so`
@@ -1333,16 +1087,9 @@ if test "x$cs_linker_set" != "xyes" ; then
       fi
       ;;
 
-    solaris2.*)
-      ldflags_default_opt=""
-      ldflags_default_dbg="-g"
-      ldflags_default_prf=""
-      ;;
-
     *)
       ldflags_default_opt="-O"
       ldflags_default_dbg="-g"
-      ldflags_default_prf="-pg"
       ;;
 
   esac
