@@ -1,7 +1,7 @@
 #-------------------------------------------------------------------------------
 #   This file is part of the Code_Saturne CFD tool.
 #
-#   Copyright (C) 2011 EDF S.A.
+#   Copyright (C) 2011-2019 EDF S.A.
 #
 #   The Code_Saturne CFD tool is free software; you can redistribute it
 #   and/or modify it under the terms of the GNU General Public License
@@ -29,9 +29,6 @@ from module_generator import ASTERComponent, CPPComponent
 
 asterdir = os.path.expanduser("/home/aster/NEW11")
 
-milieuinc = "-I$(top_srcdir)/salome/libmilieu"
-milieulib = "$(top_builddir)/salome/libmilieu/libmilieu.la"
-
 # SALOME environment be must loaded beforehand
 
 KERNEL_ROOT_DIR = os.getenv("KERNEL_ROOT_DIR")
@@ -47,42 +44,31 @@ context = {'update':1,
 # Definition of all the in/out streams
 # ------------------------------------
 
-astDisplacement = ("DEPAST","CALCIUM_double","I")
-astVelocity = ("VITAST","CALCIUM_double","I")
-astForces = ("FORAST","CALCIUM_double","I")
+nbIter = ("NBPDTM", "CALCIUM_integer","I")
+nbSubIter = ("NBSSIT", "CALCIUM_integer","I")
 
-satDisplacement = ("DEPSAT","CALCIUM_double","I")
-satForces = ("FORSAT","CALCIUM_double","I")
+indSync =("ISYNCP", "CALCIUM_integer","I")
+frequency = ("NTCHRO", "CALCIUM_integer","I")
 
-geometricData = ("DONGEO","CALCIUM_integer","I")
-satLengthScale = ("ALMAXI","CALCIUM_double","I")
+timeStep = ("PDTREF", "CALCIUM_double","I")
+prevTime = ("TTINIT", "CALCIUM_double","I")
+epsilon = ("EPSILO", "CALCIUM_double","I")
 
-nbFor = ("NB_FOR","CALCIUM_integer","I")
-nbDyn = ("NB_DYN","CALCIUM_integer","I")
+newTimeStep = ("DTCALC", "CALCIUM_double","I")
+nbDyn = ("NB_DYN", "CALCIUM_integer","I")
+nbFor = ("NB_FOR", "CALCIUM_integer","I")
 
-nodesXYZ = ("COONOD","CALCIUM_double","I")
-facesXYZ = ("COOFAC","CALCIUM_double","I")
+nodesColor = ("COLNOD", "CALCIUM_integer","I")
+facesColor = ("COLFAC", "CALCIUM_integer","I")
+nodesXYZ = ("COONOD", "CALCIUM_double","I")
+facesXYZ = ("COOFAC", "CALCIUM_double","I")
 
-nodesColor = ("COLNOD","CALCIUM_integer","I")
-facesColor = ("COLFAC","CALCIUM_integer","I")
-
-nbIter = ("NBPDTM","CALCIUM_integer","I")
-nbSubIter = ("NBSSIT","CALCIUM_integer","I")
-
-indSync =("ISYNCP","CALCIUM_integer","I")
-frequency = ("NTCHRO","CALCIUM_integer","I")
-
-epsilon = ("EPSILO","CALCIUM_double","I")
-astConvergence = ("ICVAST","CALCIUM_integer","I")
-satConvergence = ("ICV",   "CALCIUM_integer","I")
-extConvergence = ("ICVEXT","CALCIUM_integer","I")
-
-prevTime = ("TTINIT","CALCIUM_double","I")
-timeStep = ("PDTREF","CALCIUM_double","I")
+astForces = ("FORAST", "CALCIUM_double","I")
+astConvergence = ("ICVAST", "CALCIUM_integer","I")
 
 astTimeStep = ("DTAST", "CALCIUM_double","I")
-satTimeStep = ("DTSAT", "CALCIUM_double","I")
-newTimeStep = ("DTCALC","CALCIUM_double","I")
+astDisplacement = ("DEPAST", "CALCIUM_double","I")
+astVelocity = ("VITAST", "CALCIUM_double","I")
 
 # Code_Aster streams
 
@@ -99,44 +85,21 @@ aster_instream = [nbFor, nbDyn,
 
 # Code_Saturne streams
 
-saturne_instream = [satDisplacement,
-                    epsilon,
-                    newTimeStep,
-                    prevTime,
-                    timeStep,
-                    nbIter, nbSubIter,
-                    extConvergence]
+saturne_instream = [astDisplacement, astVelocity, astTimeStep]
 
-saturne_outstream = [satTimeStep,
-                     satForces,
-                     satLengthScale,
+saturne_outstream = [nbFor, nbDyn,
                      nodesXYZ, facesXYZ,
                      nodesColor, facesColor,
-                     satConvergence,
-                     geometricData]
-
-# Milieu streams
-
-milieu_outstream = [newTimeStep,
-                    epsilon,
-                    prevTime, timeStep,
-                    satDisplacement, astForces,
-                    nbIter, nbSubIter,
-                    indSync, frequency,
-                    extConvergence, astConvergence,
-                    nbDyn, nbFor]
-
-milieu_instream = [satLengthScale,
-                   satTimeStep, astTimeStep,
-                   astDisplacement, astVelocity,
-                   satForces,
-                   geometricData,
-                   satConvergence]
+                     astForces,
+                     nbIter, nbSubIter,
+                     epsilon, astConvergence,
+                     indSync, frequency,
+                     prevTime, timeStep, newTimeStep]
 
 # Creation of the different components
 # ------------------------------------
 
-# Code_Aster component
+# code_aster component
 
 aster_service = Service("op0117",
                         instream = aster_instream,
@@ -172,39 +135,7 @@ saturne_service = Service("run",
 c2 = CPPComponent("FSI_SATURNE",
                   services = [saturne_service],
                   kind = "exe",
-                  exe_path = "./run_solver.sh")
-
-# Milieu component
-
-milieu_defs = """#include <runmilieu.h>
-#include <donnees.h>"""
-
-milieu_body = """inter_cs_ast_set_nbpdtm(NBPDTM);
-inter_cs_ast_set_nbssit(NBSSIT);
-inter_cs_ast_set_isyncp(ISYNCP);
-inter_cs_ast_set_ntchr(NTCHR);
-inter_cs_ast_set_dtref(DTREF);
-inter_cs_ast_set_ttinit(TTINIT);
-inter_cs_ast_set_epsilo(EPSILO);
-runmilieu(component);"""
-
-milieu_service = Service("inter_run",
-                         inport=[("NBPDTM", "long"),
-                                 ("NBSSIT", "long"),
-                                 ("ISYNCP", "long"),
-                                 ("NTCHR",  "long"),
-                                 ("DTREF",  "double"),
-                                 ("TTINIT", "double"),
-                                 ("EPSILO", "double")],
-                         outstream = milieu_outstream,
-                         instream = milieu_instream,
-                         defs = milieu_defs,
-                         body = milieu_body)
-
-c3 = CPPComponent("FSI_MILIEU",
-                  services = [milieu_service],
-                  includes = milieuinc,
-                  libs = milieulib)
+                  exe_path = "./run_solver")
 
 # Creation of the FSI module
 # --------------------------
