@@ -519,7 +519,6 @@ Postprocessing
     Profiles
     Balance by zone
 Performance settings
-OpenTurns study
 """
         return tree
 
@@ -811,8 +810,6 @@ OpenTurns study
         self.setRowClose(self.tr('Profiles'))
         self.setRowClose(self.tr('Balance by zone'))
 
-        self.setRowClose(self.tr('OpenTurns study'))
-
         self.__hideRow()
 
 
@@ -1074,7 +1071,6 @@ OpenTurns study
 
         self.setRowShow(self.tr('Performance tuning'), True)
         self.setRowShow(self.tr('Prepare batch calculation'), True)
-        self.setRowShow(self.tr('OpenTurns study'), case['salome'])
 
         # End of test of physical module
 
