@@ -1,30 +1,25 @@
-# -*- coding: iso-8859-1 -*-
-#
+# -*- coding: utf-8 -*-
+
 #-------------------------------------------------------------------------------
+
+# This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-#     This file is part of the Code_Saturne User Interface, element of the
-#     Code_Saturne CFD tool.
+# Copyright (C) 1998-2019 EDF S.A.
 #
-#     Copyright (C) 1998-2010 EDF S.A., France
+# This program is free software; you can redistribute it and/or modify it under
+# the terms of the GNU General Public License as published by the Free Software
+# Foundation; either version 2 of the License, or (at your option) any later
+# version.
 #
-#     contact: saturne-support@edf.fr
-#
-#     The Code_Saturne User Interface is free software; you can redistribute it
-#     and/or modify it under the terms of the GNU General Public License
-#     as published by the Free Software Foundation; either version 2 of
-#     the License, or (at your option) any later version.
-#
-#     The Code_Saturne User Interface is distributed in the hope that it will be
-#     useful, but WITHOUT ANY WARRANTY; without even the implied warranty
-#     of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-#     GNU General Public License for more details.
-#
-#     You should have received a copy of the GNU General Public License
-#     along with the Code_Saturne Kernel; if not, write to the
-#     Free Software Foundation, Inc.,
-#     51 Franklin St, Fifth Floor,
-#     Boston, MA  02110-1301  USA
+# This program is distributed in the hope that it will be useful, but WITHOUT
+# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+# details.
 #
+# You should have received a copy of the GNU General Public License along with
+# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
+# Street, Fifth Floor, Boston, MA 02110-1301, USA.
+
 #-------------------------------------------------------------------------------
 
 """
@@ -37,15 +32,16 @@ Generic Dialog window to handle execution of a list of external scripts.
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import os, string, logging
+import os, logging
 import signal, subprocess
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules
@@ -93,15 +89,11 @@ class CommandMgrLinesDisplayedDialogView(QDialog, Ui_CommandMgrLinesDisplayedDia
         self.lines = self.default['lines']
         self.lineEditLines.setText(str(self.lines))
 
-        self.connect(self.lineEditLines,
-                     SIGNAL("textChanged(const QString &)"),
-                     self.__slotLines)
-        self.connect(self.pushButtonLines,
-                     SIGNAL("clicked()"),
-                     self.__slotUnlimited)
+        self.lineEditLines.textChanged[str].connect(self.__slotLines)
+        self.pushButtonLines.clicked.connect(self.__slotUnlimited)
 
 
-    @pyqtSignature("const QString &")
+    @pyqtSlot(str)
     def __slotLines(self, text):
         """
         Private slot. Manage the number of lines allowed in the display zone.
@@ -111,7 +103,7 @@ class CommandMgrLinesDisplayedDialogView(QDialog, Ui_CommandMgrLinesDisplayedDia
             self.lines = lines
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotUnlimited(self):
         """
         Private slot. Set a unlimited number of lines in the display zone.
@@ -149,7 +141,7 @@ class CommandMgrDialogView(QDialog, Ui_CommandMgrDialogForm):
     """
     Open a dialog to start external programs and display its output.
     """
-    def __init__(self, parent, title, cmd_list, start_directory="", obj_salome=""):
+    def __init__(self, parent, title, cmd, start_directory="", obj_salome=""):
         """
         Constructor. Must be overriden.
         """
@@ -171,60 +163,47 @@ class CommandMgrDialogView(QDialog, Ui_CommandMgrDialogForm):
 
         self.objBr = obj_salome
 
-        self.connect(self.proc,
-                     SIGNAL('readyReadStandardOutput()'),
-                     self.slotReadFromStdout)
-        self.connect(self.proc,
-                     SIGNAL('readyReadStandardError()'),
-                     self.slotReadFromStderr)
-        self.connect(self.proc,
-                     SIGNAL('stateChanged(QProcess::ProcessState)'),
-                     self.slotStateChanged)
-        self.connect(self.pushButtonLines,
-                     SIGNAL('clicked()'),
-                     self.__slotLines)
-        self.connect(self.pushButtonSaveAs,
-                     SIGNAL('clicked()'),
-                     self.__slotSaveAs)
-        self.connect(self.pushButtonKill,
-                     SIGNAL('clicked()'),
-                     self.__slotKill)
+        self.proc.readyReadStandardOutput.connect(self.slotReadFromStdout)
+        self.proc.readyReadStandardError.connect(self.slotReadFromStderr)
+        self.pushButtonLines.clicked.connect(self.__slotLines)
+        self.pushButtonSaveAs.clicked.connect(self.__slotSaveAs)
+        self.pushButtonKill.clicked.connect(self.__slotKill)
+        self.proc.started.connect(self.slotStarted)
+        self.proc.finished.connect(self.slotFinished)
 
-        self.procErrorFlag = False
-
-        self.cmd_list = cmd_list
-        if self.cmd_list:
-            self.cmd = self.cmd_list.pop(0)
-        else:
-            self.cmd = None
+        self.cmd = cmd
 
         cursor = QCursor(Qt.BusyCursor)
         QApplication.setOverrideCursor(cursor)
 
 
-    @pyqtSignature("int, QProcess::ExitStatus")
-    def slotProcess(self):
+    def slotStarted(self):
         """
-        Public slot. Run a list of commands.
+        Public slot. Process is started.
         """
-        if self.proc.exitStatus() == QProcess.NormalExit and not self.procErrorFlag:
-            self.proc.start(self.cmd)
-            if self.cmd_list:
-                self.cmd = self.cmd_list.pop(0)
-                self.connect(self.proc,
-                             SIGNAL('finished(int, QProcess::ExitStatus)'),
-                             self.slotProcess)
-            else:
-                self.connect(self.proc,
-                             SIGNAL('finished(int, QProcess::ExitStatus)'),
-                             self.slotFinished)
+        print("started: " + self.cmd)
 
 
-    @pyqtSignature("int, QProcess::ExitStatus")
     def slotFinished(self, exitCode, exitStatus):
         """
         Public slot. Enable the close button of the dialog window.
         """
+
+        if exitStatus != QProcess.NormalExit:
+            error = self.proc.error()
+            if error == QProcess.FailedToStart:
+                print("failed to start")
+            elif error == QProcess.Timedout:
+                print("timed out")
+            elif error == QProcess.WriteError:
+                print("error trying to write to process")
+            elif error == QProcess.ReadError:
+                print("error trying to read from process")
+            else:
+                print("crashed or killed")
+        else:
+            print("finished with exit code " + str(exitCode))
+
         # if the GUI is launched through SALOME, update the object browser
         # in order to display results
         if self.objBr:
@@ -239,7 +218,7 @@ class CommandMgrDialogView(QDialog, Ui_CommandMgrDialogForm):
         self.pushButtonOK.setEnabled(True)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotLines(self):
         """
         Private slot. Manage the number of lines allowed in the display zone.
@@ -254,7 +233,7 @@ class CommandMgrDialogView(QDialog, Ui_CommandMgrDialogForm):
                 self.logText.document().setMaximumBlockCount(n)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotKill(self):
         """
         Private slot. Kill the subprocess.
@@ -280,7 +259,10 @@ class CommandMgrDialogView(QDialog, Ui_CommandMgrDialogForm):
         Private slot. Find and kill all children of the spawned subprocess.
         """
         cmd = "ps eo pid,ppid --sort=pid --no-headers"
-        psraw = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout.readlines()
+        psraw = subprocess.Popen(cmd,
+                                 shell=True,
+                                 stdout=subprocess.PIPE,
+                                 universal_newlines=True).stdout.readlines()
         psList = []
         killList = []
 
@@ -298,47 +280,14 @@ class CommandMgrDialogView(QDialog, Ui_CommandMgrDialogForm):
         if len(killList) <= 0:
             return
 
-        cmd = "kill -9 %s" % string.join(list(map(str, killList)), ' ')
-        subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
+        cmd = "kill -9 %s" % " ".join(list(map(str, killList)))
+        subprocess.Popen(cmd,
+                         shell=True,
+                         stdout=subprocess.PIPE,
+                         universal_newlines=True)
 
 
-    #def killPGroup(self):
-        #"""
-        #Attempts to find all childred & grandchildren of the spawned subprocess
-        #"""
-        ## get the pid, pgid, ppid of our current processes:
-        #cmd = "ps eo pid,pgid --no-headers"
-        #psraw = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE).stdout.readlines()
-        #psList = []
-        #killList = []
-        #for ps in psraw: # 1: gets rid of header
-            #psList.append(map(int, ps.split()))
-
-        #pgid = 0
-
-        ## find the pgid of the spawned subprocess:
-        #for ps in psList:
-            #if int(self.proc.pid()) in ps:
-                #pgid = ps[1]
-                #break
-
-        #if pgid == 0:
-            #return
-
-        ## get a list of all pids in the pgid except the group owner:
-        #for ps in psList:
-            #if pgid in ps and pgid != ps[0]:  # check [0] so we don't kill ourselves
-                #killList.append(ps[0])
-
-        ## don't do anything if we didn't find anything:
-        #if len(killList) <= 0:
-            #return
-
-        #cmd = "kill %s" % string.join(map(str, killList[1:-1]), ' ')
-        #subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
-
-
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotSaveAs(self):
         """
         Private slot. Save the contain of the display zone.
@@ -367,7 +316,7 @@ class CommandMgrDialogView(QDialog, Ui_CommandMgrDialogForm):
         logFile.close()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotReadFromStdout(self):
         """
         Public slot. Handle the readyReadStandardOutput signal of the subprocess.
@@ -383,7 +332,7 @@ class CommandMgrDialogView(QDialog, Ui_CommandMgrDialogForm):
             self.logText.append(s)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotReadFromStderr(self):
         """
         Public slot. Handle the readyReadStandardError signal of the subprocess.
@@ -397,15 +346,6 @@ class CommandMgrDialogView(QDialog, Ui_CommandMgrDialogForm):
             if ba.isNull(): return
             s = (ba.data()).decode("utf-8")[:-1]
             self.logText.append('<font color="red">' + s + '</font>')
-            self.procErrorFlag = True
-
-
-    @pyqtSignature("QProcess::ProcessState")
-    def slotStateChanged(self, state):
-        """
-        Public slot. Handle the current status of the subprocess.
-        """
-        pass
 
 
     def closeEvent(self, event):
