@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -34,29 +34,31 @@ This module contains the following classes:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import string, logging
+import logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import DoubleValidator, ComboModel, setGreenColor
-from Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import DoubleValidator, ComboModel
+from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
 
-from Pages.BoundaryConditionsCoalInletForm import Ui_BoundaryConditionsCoalInletForm
-import Pages.CoalCombustionModel as CoalCombustion
+from code_saturne.Pages.BoundaryConditionsCoalInletForm import Ui_BoundaryConditionsCoalInletForm
+import code_saturne.model.CoalCombustionModel as CoalCombustion
 
-from Pages.LocalizationModel import LocalizationModel, Zone
-from Pages.Boundary import Boundary
-from Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.model.LocalizationModel import LocalizationModel, Zone
+from code_saturne.model.Boundary import Boundary
+from code_saturne.Pages.QMegEditorView import QMegEditorView
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -82,6 +84,7 @@ class ValueDelegate(QItemDelegate):
         return editor
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(value)
 
@@ -102,11 +105,11 @@ class StandardItemModelCoal(QStandardItemModel):
                         self.tr("Temperature \n(K)")]
         self.setColumnCount(len(self.headers))
         self.dataCoal = []
-        self.__case = case
+        self.case = case
 
 
     def setBoundaryFromLabel(self, label):
-        self.modelBoundary = Boundary('coal_inlet', label, self.__case)
+        self.modelBoundary = Boundary('coal_inlet', label, self.case)
 
 
     def data(self, index, role):
@@ -144,7 +147,7 @@ class StandardItemModelCoal(QStandardItemModel):
             self.modelBoundary.setCoalFlow(v, row)
         elif col == 2:
             self.modelBoundary.setCoalTemperature(v, row)
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -167,7 +170,7 @@ class StandardItemModelCoalMass(QStandardItemModel):
 
     def __init__(self, case, coalNumber, coalClassesNumber):
         QStandardItemModel.__init__(self)
-        self.__case = case
+        self.case = case
         self.coalNumber = coalNumber
         self.coalClassesNumber = coalClassesNumber
 
@@ -185,7 +188,7 @@ class StandardItemModelCoalMass(QStandardItemModel):
 
     def setBoundaryFromLabel(self, label):
         log.debug("setBoundaryFromLabel")
-        self.modelBoundary = Boundary('coal_inlet', label, self.__case)
+        self.modelBoundary = Boundary('coal_inlet', label, self.case)
 
 
     def data(self, index, role):
@@ -225,6 +228,8 @@ class StandardItemModelCoalMass(QStandardItemModel):
             return
         classe = index.row()
         coal   = index.column()
+        if not value:
+            return False
         v = from_qvariant(value, float)
         self.ratio[coal][classe] = v
         log.debug("setData v = %f "%v)
@@ -243,7 +248,7 @@ class StandardItemModelCoalMass(QStandardItemModel):
         else :
             self.ratio[coal][classe] = lst[classe]
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -270,40 +275,23 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
+        self.case.undoStopGlobal()
+        self.notebook = NotebookModel(self.case)
 
         # Connections
-        self.connect(self.comboBoxTypeInlet,
-                     SIGNAL("activated(const QString&)"),
-                     self.__slotInletType)
-        self.connect(self.comboBoxVelocity,
-                     SIGNAL("activated(const QString&)"),
-                     self.__slotChoiceVelocity)
-        self.connect(self.lineEditVelocity,
-                     SIGNAL("textChanged(const QString &)"),
-                     self.__slotVelocityValue)
-        self.connect(self.lineEditTemperature,
-                     SIGNAL("textChanged(const QString &)"),
-                     self.__slotTemperature)
-        self.connect(self.spinBoxOxydantNumber,
-                     SIGNAL("valueChanged(int)"),
-                     self.__slotOxydantNumber)
+        self.comboBoxTypeInlet.activated[str].connect(self.__slotInletType)
+        self.comboBoxVelocity.activated[str].connect(self.__slotChoiceVelocity)
+        self.lineEditVelocity.textChanged[str].connect(self.__slotVelocityValue)
+        self.lineEditTemperature.textChanged[str].connect(self.__slotTemperature)
+        self.spinBoxOxydantNumber.valueChanged[int].connect(self.__slotOxydantNumber)
 
-        self.connect(self.comboBoxDirection,
-                     SIGNAL("activated(const QString&)"),
-                     self.__slotChoiceDirection)
-        self.connect(self.lineEditDirectionX,
-                     SIGNAL("textChanged(const QString &)"),
-                     self.__slotDirX)
-        self.connect(self.lineEditDirectionY,
-                     SIGNAL("textChanged(const QString &)"),
-                     self.__slotDirY)
-        self.connect(self.lineEditDirectionZ,
-                     SIGNAL("textChanged(const QString &)"),
-                     self.__slotDirZ)
+        self.comboBoxDirection.activated[str].connect(self.__slotChoiceDirection)
+        self.lineEditDirectionX.textChanged[str].connect(self.__slotDirX)
+        self.lineEditDirectionY.textChanged[str].connect(self.__slotDirY)
+        self.lineEditDirectionZ.textChanged[str].connect(self.__slotDirZ)
 
         # Combo models
         self.modelTypeInlet = ComboModel(self.comboBoxTypeInlet, 2, 1)
@@ -335,16 +323,12 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
         self.lineEditDirectionZ.setValidator(validatorZ)
         self.lineEditTemperature.setValidator(validatorTemp)
 
-        self.connect(self.pushButtonVelocityFormula,
-                     SIGNAL("clicked()"),
-                     self.__slotVelocityFormula)
-        self.connect(self.pushButtonDirectionFormula,
-                     SIGNAL("clicked()"),
-                     self.__slotDirectionFormula)
+        self.pushButtonVelocityFormula.clicked.connect(self.__slotVelocityFormula)
+        self.pushButtonDirectionFormula.clicked.connect(self.__slotDirectionFormula)
 
         # Useful information about coals, classes, and ratios
 
-        mdl =  CoalCombustion.CoalCombustionModel(self.__case)
+        mdl =  CoalCombustion.CoalCombustionModel(self.case)
         if mdl.getCoalCombustionModel() != "off":
             self.__coalNumber = mdl.getCoalNumber()
             self.__coalClassesNumber = []
@@ -362,7 +346,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
 
         # Coal table
 
-        self.__modelCoal = StandardItemModelCoal(self.__case)
+        self.__modelCoal = StandardItemModelCoal(self.case)
         self.tableViewCoal.setModel(self.__modelCoal)
         delegateValue = ValueDelegate(self.tableViewCoal)
         self.tableViewCoal.setItemDelegateForColumn(1, delegateValue)
@@ -370,7 +354,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
 
         # Coal mass ratio table
 
-        self.__modelCoalMass = StandardItemModelCoalMass(self.__case,
+        self.__modelCoalMass = StandardItemModelCoalMass(self.case,
                                                          self.__coalNumber,
                                                          self.__coalClassesNumber)
         self.tableViewCoalMass.setModel(self.__modelCoalMass)
@@ -379,7 +363,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
         for c in range(self.__modelCoalMass.columnCount()):
             self.tableViewCoalMass.setItemDelegateForColumn(c, delegateValueMass)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
     def showWidget(self, b):
@@ -387,7 +371,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
         Show the widget
         """
         label = b.getLabel()
-        self.__boundary = Boundary('coal_inlet', label, self.__case)
+        self.__boundary = Boundary('coal_inlet', label, self.case)
 
         # Initialize velocity
         choice = self.__boundary.getVelocityChoice()
@@ -487,7 +471,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
         self.__modelCoalMass.setRatio(self.__ratio)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotChoiceVelocity(self, text):
         """
         Private slot.
@@ -503,12 +487,17 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
 
         if c[-7:] == "formula":
             self.pushButtonVelocityFormula.setEnabled(True)
-            setGreenColor(self.pushButtonVelocityFormula, True)
+            exp = self.__boundary.getVelocity()
+            if exp:
+                self.pushButtonVelocityFormula.setStyleSheet("background-color: green")
+                self.pushButtonVelocityFormula.setToolTip(exp)
+            else:
+                self.pushButtonVelocityFormula.setStyleSheet("background-color: red")
             self.lineEditVelocity.setEnabled(False)
             self.lineEditVelocity.setText("")
         else:
             self.pushButtonVelocityFormula.setEnabled(False)
-            setGreenColor(self.pushButtonVelocityFormula, False)
+            self.pushButtonVelocityFormula.setStyleSheet("background-color: None")
             self.lineEditVelocity.setEnabled(True)
             v = self.__boundary.getVelocity()
             self.lineEditVelocity.setText(str(v))
@@ -529,7 +518,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
             self.labelUnitVelocity.setText(str('m<sup>3</sup>/s'))
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotVelocityValue(self, text):
         """
         Private slot.
@@ -544,7 +533,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
             self.__boundary.setVelocity(v)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotVelocityFormula(self):
         """
         """
@@ -565,22 +554,31 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
+
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
+        dialog = QMegEditorView(parent = self,
+                                function_type = 'bnd',
+                                zone_name     = self.__boundary._label,
+                                variable_name = 'velocity',
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = c,
+                                examples      = exa)
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotFormulaVelocity -> %s" % str(result))
-            self.__boundary.setVelocity(result)
-            setGreenColor(self.pushButtonVelocityFormula, False)
+            self.__boundary.setVelocity(str(result))
+            self.pushButtonVelocityFormula.setToolTip(result)
+            self.pushButtonVelocityFormula.setStyleSheet("background-color: green")
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotChoiceDirection(self, text):
         """
         Input the direction type choice.
@@ -591,11 +589,16 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
 
         if c == "formula":
             self.pushButtonDirectionFormula.setEnabled(True)
-            setGreenColor(self.pushButtonDirectionFormula, True)
+            exp = self.__boundary.getDirection('direction_formula')
+            if exp:
+                self.pushButtonDirectionFormula.setStyleSheet("background-color: green")
+                self.pushButtonDirectionFormula.setToolTip(exp)
+            else:
+                self.pushButtonDirectionFormula.setStyleSheet("background-color: red")
             self.frameDirectionCoordinates.hide()
         elif c == "coordinates":
             self.pushButtonDirectionFormula.setEnabled(False)
-            setGreenColor(self.pushButtonDirectionFormula, False)
+            self.pushButtonDirectionFormula.setStyleSheet("background-color: None")
             self.frameDirectionCoordinates.show()
             v = self.__boundary.getDirection('direction_x')
             self.lineEditDirectionX.setText(str(v))
@@ -605,11 +608,11 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
             self.lineEditDirectionZ.setText(str(v))
         elif c == "normal":
             self.pushButtonDirectionFormula.setEnabled(False)
-            setGreenColor(self.pushButtonDirectionFormula, False)
+            self.pushButtonDirectionFormula.setStyleSheet("background-color: None")
             self.frameDirectionCoordinates.hide()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotDirX(self, text):
         """
         INPUT value into direction of inlet flow
@@ -619,7 +622,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
             self.__boundary.setDirection('direction_x', value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotDirY(self, text):
         """
         INPUT value into direction of inlet flow
@@ -629,7 +632,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
             self.__boundary.setDirection('direction_y', value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotDirZ(self, text):
         """
         INPUT value into direction of inlet flow
@@ -639,7 +642,7 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
             self.__boundary.setDirection('direction_z', value)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotDirectionFormula(self):
         """
         """
@@ -656,23 +659,32 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
+
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
+        dialog = QMegEditorView(parent = self,
+                                function_type = 'bnd',
+                                zone_name     = self.__boundary._label,
+                                variable_name = 'direction',
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = 'formula',
+                                examples      = exa)
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotFormulaDirection -> %s" % str(result))
-            self.__boundary.setDirection('direction_formula', result)
-            setGreenColor(self.pushButtonDirectionFormula, False)
+            self.__boundary.setDirection('direction_formula', str(result))
+            self.pushButtonDirectionFormula.setToolTip(result)
+            self.pushButtonDirectionFormula.setStyleSheet("background-color: green")
 
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotInletType(self, text):
         """
         INPUT inlet type : 'oxydant' or 'oxydant + coal'
@@ -691,14 +703,14 @@ class BoundaryConditionsCoalInletView(QWidget, Ui_BoundaryConditionsCoalInletFor
             self.__updateTables()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotTemperature(self, text):
         if self.sender().validator().state == QValidator.Acceptable:
             t = from_qvariant(text, float)
             self.__boundary.setOxydantTemperature(t)
 
 
-    @pyqtSignature("int")
+    @pyqtSlot(int)
     def __slotOxydantNumber(self, i):
         self.__boundary.setOxydantNumber(i)
 
