@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -37,21 +37,22 @@ import string, logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import DoubleValidator, ComboModel, setGreenColor, from_qvariant
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import DoubleValidator, ComboModel, from_qvariant
 
-from Pages.BoundaryConditionsCompressibleOutletForm import \
+from code_saturne.Pages.BoundaryConditionsCompressibleOutletForm import \
      Ui_BoundaryConditionsCompressibleOutletForm
-from Pages.LocalizationModel import LocalizationModel, Zone
-from Pages.Boundary import Boundary
-from Pages.CompressibleModel import CompressibleModel
+from code_saturne.model.LocalizationModel import LocalizationModel, Zone
+from code_saturne.model.Boundary import Boundary
+from code_saturne.model.CompressibleModel import CompressibleModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -80,16 +81,16 @@ class BoundaryConditionsCompressibleOutletView(QWidget, Ui_BoundaryConditionsCom
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
+        self.case.undoStopGlobal()
 
-        self.mdl = CompressibleModel(self.__case)
+        self.mdl = CompressibleModel(self.case)
 
         # Connections
-        self.connect(self.comboBoxTypeOutlet, SIGNAL("activated(const QString&)"),    self.slotOutletType)
-        self.connect(self.lineEditPressure,   SIGNAL("textChanged(const QString &)"), self.slotPressureValue)
+        self.comboBoxTypeOutlet.activated[str].connect(self.slotOutletType)
+        self.lineEditPressure.textChanged[str].connect(self.slotPressureValue)
 
         # Combo models
         self.modelTypeOutlet = ComboModel(self.comboBoxTypeOutlet, 2, 1)
@@ -102,7 +103,7 @@ class BoundaryConditionsCompressibleOutletView(QWidget, Ui_BoundaryConditionsCom
         # Apply validators
         self.lineEditPressure.setValidator(validatorP)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
     def showWidget(self, boundary):
@@ -110,7 +111,7 @@ class BoundaryConditionsCompressibleOutletView(QWidget, Ui_BoundaryConditionsCom
         Show the widget
         """
         label = boundary.getLabel()
-        self.__boundary = Boundary('compressible_outlet', label, self.__case)
+        self.__boundary = Boundary('compressible_outlet', label, self.case)
         self.initialize()
 
 
@@ -138,7 +139,7 @@ class BoundaryConditionsCompressibleOutletView(QWidget, Ui_BoundaryConditionsCom
         self.hide()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotOutletType(self, text):
         """
         INPUT outlet type
@@ -150,7 +151,7 @@ class BoundaryConditionsCompressibleOutletView(QWidget, Ui_BoundaryConditionsCom
         self.initialize()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotPressureValue(self, text):
         """
         INPUT outlet pressure
