@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -31,28 +31,30 @@ This module contains the following classes:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import string, logging
+import logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import ComboModel, setGreenColor
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import ComboModel
 
-from Pages.BoundaryConditionsMobileMeshForm import Ui_BoundaryConditionsMobileMeshForm
-from Pages.MobileMeshModel import MobileMeshModel
-from Pages.LocalizationModel import LocalizationModel, Zone
-from Pages.Boundary import Boundary
+from code_saturne.Pages.BoundaryConditionsMobileMeshForm import Ui_BoundaryConditionsMobileMeshForm
+from code_saturne.model.MobileMeshModel import MobileMeshModel
+from code_saturne.model.LocalizationModel import LocalizationModel, Zone
+from code_saturne.model.Boundary import Boundary
 
-from Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -84,12 +86,13 @@ class BoundaryConditionsMobileMeshView(QWidget, Ui_BoundaryConditionsMobileMeshF
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
+        self.case.undoStopGlobal()
 
-        self.__model = MobileMeshModel(self.__case)
+        self.__model = MobileMeshModel(self.case)
+        self.notebook = NotebookModel(self.case)
 
         self.__comboModel = ComboModel(self.comboMobilBoundary, 6, 1)
         self.__comboModel.addItem(self.tr("Fixed boundary"), "fixed_boundary")
@@ -99,18 +102,14 @@ class BoundaryConditionsMobileMeshView(QWidget, Ui_BoundaryConditionsMobileMeshF
         self.__comboModel.addItem(self.tr("Fixed velocity"), "fixed_velocity")
         self.__comboModel.addItem(self.tr("Fixed displacement"), "fixed_displacement")
 
-        self.connect(self.comboMobilBoundary,
-                     SIGNAL("activated(const QString&)"),
-                     self.__slotCombo)
-        self.connect(self.pushButtonMobilBoundary,
-                     SIGNAL("clicked(bool)"),
-                     self.__slotFormula)
+        self.comboMobilBoundary.activated[str].connect(self.__slotCombo)
+        self.pushButtonMobilBoundary.clicked.connect(self.__slotFormula)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
-    @pyqtSignature("const QString&")
-    def __slotFormula(self, text):
+    @pyqtSlot()
+    def __slotFormula(self):
         """
         Run formula editor.
         """
@@ -119,11 +118,11 @@ class BoundaryConditionsMobileMeshView(QWidget, Ui_BoundaryConditionsMobileMeshF
 
         if aleChoice == "fixed_velocity":
             if not exp:
-                exp = 'mesh_u ='
-            req = [('mesh_u', 'Fixed velocity of the mesh'),
-                   ('mesh_v', 'Fixed velocity of the mesh'),
-                   ('mesh_w', 'Fixed velocity of the mesh')]
-            exa = 'mesh_u = 1000;\nmesh_v = 1000;\nmesh_w = 1000;'
+                exp = 'mesh_velocity_U ='
+            req = [('mesh_velocity_U', 'Fixed velocity of the mesh'),
+                   ('mesh_velocity_V', 'Fixed velocity of the mesh'),
+                   ('mesh_velocity_W', 'Fixed velocity of the mesh')]
+            exa = 'mesh_velocity_U = 1000;\nmesh_velocity_V = 1000;\nmesh_velocity_W = 1000;'
         elif aleChoice == "fixed_displacement":
             if not exp:
                 exp = 'mesh_x ='
@@ -136,8 +135,11 @@ class BoundaryConditionsMobileMeshView(QWidget, Ui_BoundaryConditionsMobileMeshF
                  ('t', 'current time'),
                  ('iter', 'number of iteration')]
 
+        for (nme, val) in self.notebook.getNotebookList():
+            symbs.append((nme, 'value (notebook) = ' + str(val)))
+
         dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
+                                check_syntax = self.case['package'].get_check_syntax(),
                                 expression = exp,
                                 required   = req,
                                 symbols    = symbs,
@@ -145,21 +147,36 @@ class BoundaryConditionsMobileMeshView(QWidget, Ui_BoundaryConditionsMobileMeshF
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotFormulaMobileMeshBoundary -> %s" % str(result))
-            self.__boundary.setFormula(result)
-            setGreenColor(self.pushButtonMobilBoundary, False)
+            self.__boundary.setFormula(str(result))
+            self.pushButtonMobilBoundary.setStyleSheet("background-color: green")
+            self.pushButtonMobilBoundary.setToolTip(result)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotCombo(self, text):
         """
         Called when the combobox changed.
         """
         modelData = self.__comboModel.dicoV2M[str(text)]
-        # Enable/disable formula button.
-        isFormulaEnabled = modelData in ["fixed_velocity", "fixed_displacement"]
-        self.pushButtonMobilBoundary.setEnabled(isFormulaEnabled)
-        setGreenColor(self.pushButtonMobilBoundary, isFormulaEnabled)
+
+        if modelData == self.__boundary.getALEChoice():
+            return
+
         self.__boundary.setALEChoice(modelData)
+        exp = self.__boundary.getFormula()
+
+        # Hide/Show formula button.
+        # Formula is always reset when changing values, so set
+        # color to red.
+        if modelData in ["fixed_velocity", "fixed_displacement"]:
+            self.pushButtonMobilBoundary.show()
+        else:
+            self.pushButtonMobilBoundary.hide()
+        if exp:
+            self.pushButtonMobilBoundary.setStyleSheet("background-color: red")
+            self.pushButtonMobilBoundary.setToolTip(exp)
+        else:
+            self.pushButtonMobilBoundary.setStyleSheet("background-color: red")
 
 
     def showWidget(self, b):
@@ -167,11 +184,13 @@ class BoundaryConditionsMobileMeshView(QWidget, Ui_BoundaryConditionsMobileMeshF
         Show the widget
         """
         if self.__model.getMethod() != "off":
-            self.__boundary = Boundary("mobile_boundary", b.getLabel(), self.__case)
-            modelData = self.__boundary.getALEChoice()
+            self.__boundary = b
+            modelData = b.getALEChoice()
             self.__comboModel.setItem(str_model=modelData)
-            isFormulaEnabled = modelData in ["fixed_velocity", "fixed_displacement"]
-            self.pushButtonMobilBoundary.setEnabled(isFormulaEnabled)
+            if modelData in ["fixed_velocity", "fixed_displacement"]:
+                self.pushButtonMobilBoundary.show()
+            else:
+                self.pushButtonMobilBoundary.hide()
             self.show()
         else:
             self.hideWidget()
