@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -37,24 +37,26 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import DoubleValidator, ComboModel, setGreenColor, from_qvariant
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage  import DoubleValidator, ComboModel, from_qvariant
 
-from Pages.BoundaryConditionsScalarsForm import Ui_BoundaryConditionsScalarsForm
-from Pages.LocalizationModel import LocalizationModel, Zone
-from Pages.DefineUserScalarsModel import DefineUserScalarsModel
-from Pages.ThermalScalarModel import ThermalScalarModel
-from Pages.QMeiEditorView import QMeiEditorView
-from Pages.Boundary import Boundary
-from Pages.CompressibleModel import CompressibleModel
-from Pages.AtmosphericFlowsModel import AtmosphericFlowsModel
+from code_saturne.Pages.BoundaryConditionsScalarsForm import Ui_BoundaryConditionsScalarsForm
+from code_saturne.model.LocalizationModel             import LocalizationModel, Zone
+from code_saturne.model.DefineUserScalarsModel        import DefineUserScalarsModel
+from code_saturne.model.ThermalScalarModel            import ThermalScalarModel
+from code_saturne.Pages.QMegEditorView                import QMegEditorView
+from code_saturne.model.Boundary                      import Boundary
+from code_saturne.model.CompressibleModel             import CompressibleModel
+from code_saturne.model.AtmosphericFlowsModel         import AtmosphericFlowsModel
+from code_saturne.model.NotebookModel                 import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -85,27 +87,28 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
+        self.case.undoStopGlobal()
+        self.notebook = NotebookModel(self.case)
 
-        self.connect(self.lineEditValueThermal, SIGNAL("textChanged(const QString &)"), self.slotValueThermal)
-        self.connect(self.lineEditValueSpecies, SIGNAL("textChanged(const QString &)"), self.slotValueSpecies)
-        self.connect(self.lineEditValueMeteo,   SIGNAL("textChanged(const QString &)"), self.slotValueMeteo)
-        self.connect(self.lineEditExThermal,    SIGNAL("textChanged(const QString &)"), self.slotExThermal)
-        self.connect(self.lineEditExSpecies,    SIGNAL("textChanged(const QString &)"), self.slotExSpecies)
-        self.connect(self.lineEditExMeteo,      SIGNAL("textChanged(const QString &)"), self.slotExMeteo)
+        self.lineEditValueThermal.textChanged[str].connect(self.slotValueThermal)
+        self.lineEditValueSpecies.textChanged[str].connect(self.slotValueSpecies)
+        self.lineEditValueMeteo.textChanged[str].connect(self.slotValueMeteo)
+        self.lineEditExThermal.textChanged[str].connect(self.slotExThermal)
+        self.lineEditExSpecies.textChanged[str].connect(self.slotExSpecies)
+        self.lineEditExMeteo.textChanged[str].connect(self.slotExMeteo)
 
-        self.connect(self.pushButtonThermal,    SIGNAL("clicked()"), self.slotThermalFormula)
-        self.connect(self.pushButtonSpecies,    SIGNAL("clicked()"), self.slotSpeciesFormula)
-        self.connect(self.pushButtonMeteo,      SIGNAL("clicked()"), self.slotMeteoFormula)
-        self.connect(self.comboBoxThermal,      SIGNAL("activated(const QString&)"), self.slotThermalChoice)
-        self.connect(self.comboBoxTypeThermal,  SIGNAL("activated(const QString&)"), self.slotThermalTypeChoice)
-        self.connect(self.comboBoxSpecies,      SIGNAL("activated(const QString&)"), self.slotSpeciesChoice)
-        self.connect(self.comboBoxTypeSpecies,  SIGNAL("activated(const QString&)"), self.slotSpeciesTypeChoice)
-        self.connect(self.comboBoxMeteo,        SIGNAL("activated(const QString&)"), self.slotMeteoChoice)
-        self.connect(self.comboBoxTypeMeteo,    SIGNAL("activated(const QString&)"), self.slotMeteoTypeChoice)
+        self.pushButtonThermal.clicked.connect(self.slotThermalFormula)
+        self.pushButtonSpecies.clicked.connect(self.slotSpeciesFormula)
+        self.pushButtonMeteo.clicked.connect(self.slotMeteoFormula)
+        self.comboBoxThermal.activated[str].connect(self.slotThermalChoice)
+        self.comboBoxTypeThermal.activated[str].connect(self.slotThermalTypeChoice)
+        self.comboBoxSpecies.activated[str].connect(self.slotSpeciesChoice)
+        self.comboBoxTypeSpecies.activated[str].connect(self.slotSpeciesTypeChoice)
+        self.comboBoxMeteo.activated[str].connect(self.slotMeteoChoice)
+        self.comboBoxTypeMeteo.activated[str].connect(self.slotMeteoTypeChoice)
 
         ## Validators
         validatorValueThermal = DoubleValidator(self.lineEditValueThermal)
@@ -122,7 +125,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.lineEditExSpecies.setValidator(validatorExSpecies)
         self.lineEditExMeteo.setValidator(validatorExMeteo)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
     def __setBoundary(self, boundary):
@@ -132,10 +135,10 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.__boundary = boundary
 
         self.nature  = boundary.getNature()
-        self.therm   = ThermalScalarModel(self.__case)
-        self.sca_mo  = DefineUserScalarsModel(self.__case)
-        self.comp    = CompressibleModel(self.__case)
-        self.atm     = AtmosphericFlowsModel(self.__case)
+        self.therm   = ThermalScalarModel(self.case)
+        self.sca_mo  = DefineUserScalarsModel(self.case)
+        self.comp    = CompressibleModel(self.case)
+        self.atm     = AtmosphericFlowsModel(self.case)
 
         self.modelTypeThermal = ComboModel(self.comboBoxTypeThermal, 1, 1)
         self.modelTypeSpecies = ComboModel(self.comboBoxTypeSpecies, 1, 1)
@@ -145,30 +148,32 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.modelTypeSpecies.addItem(self.tr("Prescribed value"), 'dirichlet')
         self.modelTypeMeteo.addItem(  self.tr("Prescribed value"), 'dirichlet')
 
-        self.modelTypeThermal.addItem(self.tr("Prescribed value  (user law)"), 'dirichlet_formula')
+        self.modelTypeThermal.addItem(self.tr("Prescribed value (user law)"), 'dirichlet_formula')
         self.modelTypeSpecies.addItem(self.tr("Prescribed value (user law)"), 'dirichlet_formula')
         self.modelTypeMeteo.addItem(  self.tr("Prescribed value (user law)"), 'dirichlet_formula')
 
         if self.nature == 'outlet':
-            self.modelTypeThermal.addItem(self.tr("Prescribed flux"), 'neumann')
-            self.modelTypeSpecies.addItem(self.tr("Prescribed flux"), 'neumann')
-            self.modelTypeMeteo.addItem(  self.tr("Prescribed flux"), 'neumann')
+            self.modelTypeThermal.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
+            self.modelTypeSpecies.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
+            self.modelTypeMeteo.addItem(  self.tr("Prescribed (outgoing) flux"), 'neumann')
         elif self.nature == 'wall':
-            self.modelTypeThermal.addItem(self.tr("Prescribed flux"), 'neumann')
-            self.modelTypeSpecies.addItem(self.tr("Prescribed flux"), 'neumann')
-            self.modelTypeMeteo.addItem(  self.tr("Prescribed flux"), 'neumann')
-            self.modelTypeThermal.addItem(self.tr("Prescribed flux (user law)"), 'neumann_formula')
-            self.modelTypeSpecies.addItem(self.tr("Prescribed flux (user law)"), 'neumann_formula')
-            self.modelTypeMeteo.addItem(  self.tr("Prescribed flux (user law)"), 'neumann_formula')
+            self.modelTypeThermal.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
+            self.modelTypeSpecies.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
+            self.modelTypeMeteo.addItem(  self.tr("Prescribed (outgoing) flux"), 'neumann')
+            self.modelTypeThermal.addItem(self.tr("Prescribed (outgoing) flux (user law)"), 'neumann_formula')
+            self.modelTypeSpecies.addItem(self.tr("Prescribed (outgoing) flux (user law)"), 'neumann_formula')
+            self.modelTypeMeteo.addItem(  self.tr("Prescribed (outgoing) flux (user law)"), 'neumann_formula')
             self.modelTypeThermal.addItem(self.tr("Exchange coefficient"), 'exchange_coefficient')
             self.modelTypeSpecies.addItem(self.tr("Exchange coefficient"), 'exchange_coefficient')
             self.modelTypeMeteo.addItem(  self.tr("Exchange coefficient"), 'exchange_coefficient')
             self.modelTypeThermal.addItem(self.tr("Exchange coefficient (user law)"), 'exchange_coefficient_formula')
             self.modelTypeSpecies.addItem(self.tr("Exchange coefficient (user law)"), 'exchange_coefficient_formula')
             self.modelTypeMeteo.addItem(  self.tr("Exchange coefficient (user law)"), 'exchange_coefficient_formula')
+        elif self.nature == 'groundwater':
+            self.modelTypeSpecies.addItem(self.tr("Prescribed (outgoing) flux"), 'neumann')
 
         self.species = ""
-        self.species_list = self.sca_mo.getUserScalarLabelsList()
+        self.species_list = self.sca_mo.getUserScalarNameList()
         for s in self.sca_mo.getScalarsVarianceList():
             if s in self.species_list:
                 self.species_list.remove(s)
@@ -188,25 +193,31 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         if self.model_th != 'off' and self.comp.getCompressibleModel() == 'off':
             self.groupBoxThermal.show()
             self.modelThermal = ComboModel(self.comboBoxThermal,1,1)
-            self.thermal = self.therm.getThermalScalarLabel()
+            self.thermal = self.therm.getThermalScalarName()
             self.modelThermal.addItem(self.tr(self.thermal),self.thermal)
             self.modelThermal.setItem(str_model = self.thermal)
         else:
             self.groupBoxThermal.hide()
 
         self.meteo_list = ""
-        self.meteo_list = self.sca_mo.getMeteoScalarsList()
+        self.meteo_list = self.sca_mo.getMeteoScalarsNameList()
 
         self.groupBoxMeteo.hide()
 
         if (self.atm.getAtmosphericFlowsModel() != "off" and self.nature == 'wall'):
-            self.groupBoxThermal.hide()
+            self.modelMeteo = ComboModel(self.comboBoxMeteo, 1, 1)
+            if len(self.meteo_list) > 0:
+                self.groupBoxMeteo.show()
+                for m in self.meteo_list:
+                    self.modelMeteo.addItem(self.tr(m), m)
+                self.meteo = self.meteo_list[0]
+                self.modelMeteo.setItem(str_model = self.meteo)
 
         if (self.atm.getAtmosphericFlowsModel() != "off" and \
            (self.nature == 'inlet' or self.nature == 'outlet')):
             label = self.__boundary.getLabel()
             nature = "meteo_" + self.nature
-            bb = Boundary(nature, label, self.__case)
+            bb = Boundary(nature, label, self.case)
 
             if bb.getMeteoDataStatus() == 'off':
                 self.groupBoxMeteo.hide()
@@ -241,7 +252,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.lineEditValueThermal.hide()
         self.labelValueThermal.hide()
         self.pushButtonThermal.setEnabled(False)
-        setGreenColor(self.pushButtonThermal, False)
+        self.pushButtonThermal.setStyleSheet("background-color: None")
 
         if self.model_th != 'off' and self.comp.getCompressibleModel() == 'off':
             self.thermal_type = self.__boundary.getScalarChoice(self.thermal)
@@ -271,13 +282,18 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
 
             elif self.thermal_type in ('exchange_coefficient_formula', 'dirichlet_formula', 'neumann_formula'):
                 self.pushButtonThermal.setEnabled(True)
-                setGreenColor(self.pushButtonThermal, True)
+                exp = self.__boundary.getScalarFormula(self.thermal, self.thermal_type)
+                if exp:
+                    self.pushButtonThermal.setStyleSheet("background-color: green")
+                    self.pushButtonThermal.setToolTip(exp)
+                else:
+                    self.pushButtonThermal.setStyleSheet("background-color: red")
 
         # Initalize species
         self.labelValueSpecies.hide()
         self.lineEditValueSpecies.hide()
         self.pushButtonSpecies.setEnabled(False)
-        setGreenColor(self.pushButtonSpecies, False)
+        self.pushButtonSpecies.setStyleSheet("background-color: None")
 
         if self.species_list != None and self.species_list != []:
             self.species_type = self.__boundary.getScalarChoice(self.species)
@@ -294,6 +310,9 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                     self.labelExSpecies.show()
                     v = self.__boundary.getScalarValue(self.species, 'dirichlet')
                     w = self.__boundary.getScalarValue(self.species, 'exchange_coefficient')
+                    if self.nature == 'groundwater':
+                        self.labelValueSpecies.setText('Velocity')
+                        self.labelExSpecies.setText('Concentration')
                     self.lineEditValueSpecies.setText(str(v))
                     self.lineEditExSpecies.setText(str(w))
                 else:
@@ -307,20 +326,31 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
 
             elif self.species_type in ('exchange_coefficient_formula', 'dirichlet_formula', 'neumann_formula'):
                 self.pushButtonSpecies.setEnabled(True)
-                setGreenColor(self.pushButtonSpecies, True)
+                exp = self.__boundary.getScalarFormula(self.species, self.species_type)
+                if exp:
+                    self.pushButtonSpecies.setStyleSheet("background-color: green")
+                    self.pushButtonSpecies.setToolTip(exp)
+                else:
+                    self.pushButtonSpecies.setStyleSheet("background-color: red")
+
+            if self.nature == 'groundwater':
+                self.groupBoxSpecies.setTitle('Transport equation')
 
         # Initalize meteo
         self.labelValueMeteo.hide()
         self.lineEditValueMeteo.hide()
         self.pushButtonMeteo.setEnabled(False)
-        setGreenColor(self.pushButtonMeteo, False)
+        self.pushButtonMeteo.setStyleSheet("background-color: None")
 
-        if (self.meteo_list and (self.nature == 'inlet' or self.nature == 'outlet')):
+        if (self.meteo_list):
             label = self.__boundary.getLabel()
-            nature = "meteo_" + self.nature
-            bb = Boundary(nature, label, self.__case)
+            if self.nature != 'wall':
+                nature = "meteo_" + self.nature
+            else:
+                nature = self.nature
+            bb = Boundary(nature, label, self.case)
 
-            if bb.getMeteoDataStatus() == 'off':
+            if self.nature == 'wall' or bb.getMeteoDataStatus() == 'off':
                 self.meteo_type = self.__boundary.getScalarChoice(self.meteo)
                 self.modelTypeMeteo.setItem(str_model = self.meteo_type)
                 self.labelValueMeteo.setText('Value')
@@ -348,16 +378,21 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
 
                 if self.meteo_type in ('exchange_coefficient_formula', 'dirichlet_formula', 'neumann_formula'):
                     self.pushButtonMeteo.setEnabled(True)
-                    setGreenColor(self.pushButtonMeteo, True)
+                    exp = self.__boundary.getScalarFormula(self.meteo, self.meteo_type)
+                    if exp:
+                        self.pushButtonMeteo.setStyleSheet("background-color: green")
+                        self.pushButtonMeteo.setToolTip(exp)
+                    else:
+                        self.pushButtonMeteo.setStyleSheet("background-color: red")
 
 
     def showWidget(self, boundary):
         """
         Show the widget
         """
-        if DefineUserScalarsModel(self.__case).getScalarLabelsList() or\
-           DefineUserScalarsModel(self.__case).getMeteoScalarsList() or\
-           DefineUserScalarsModel(self.__case).getThermalScalarLabelsList():
+        if DefineUserScalarsModel(self.case).getScalarNameList() or\
+           DefineUserScalarsModel(self.case).getMeteoScalarsNameList() or\
+           DefineUserScalarsModel(self.case).getThermalScalarName():
             self.__setBoundary(boundary)
             self.show()
         else:
@@ -371,7 +406,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.hide()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotThermalChoice(self, text):
         """
         INPUT label for choice of zone
@@ -380,7 +415,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.initializeVariables()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotThermalTypeChoice(self, text):
         """
         INPUT label for choice of zone
@@ -390,7 +425,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.initializeVariables()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotSpeciesChoice(self, text):
         """
         INPUT label for choice of zone
@@ -399,7 +434,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.initializeVariables()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotSpeciesTypeChoice(self, text):
         """
         INPUT label for choice of zone
@@ -409,7 +444,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.initializeVariables()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotMeteoChoice(self, text):
         """
         INPUT label for choice of zone
@@ -418,7 +453,7 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.initializeVariables()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotMeteoTypeChoice(self, text):
         """
         INPUT label for choice of zone
@@ -428,40 +463,51 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         self.initializeVariables()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotThermalFormula(self):
         """
         """
+        name = self.thermal
         exp = self.__boundary.getScalarFormula(self.thermal, self.thermal_type)
         exa = """#example: """
         if self.thermal_type == 'dirichlet_formula':
-            req = [(self.thermal, str(self.thermal))]
+            req = [(name, str(name))]
         elif self.thermal_type == 'neumann_formula':
             req = [("flux", "flux")]
         elif self.thermal_type == 'exchange_coefficient_formula':
-            req = [(self.thermal, str(self.thermal)),("hc", "heat coefficient")]
+            req = [(name, str(name)),("hc", "heat coefficient")]
 
         sym = [('x', "X face's gravity center"),
                ('y', "Y face's gravity center"),
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
+
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
+        c = self.__boundary.getScalarChoice(name)
+        dialog = QMegEditorView(parent      = self,
+                                function_type = 'bnd',
+                                zone_name     = self.__boundary._label,
+                                variable_name = name,
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = c,
+                                examples      = exa)
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotThermalFormula -> %s" % str(result))
-            self.__boundary.setScalarFormula(self.thermal, self.thermal_type, result)
-            setGreenColor(self.pushButtonThermal, False)
+            self.__boundary.setScalarFormula(self.thermal, self.thermal_type, str(result))
+            self.pushButtonThermal.setStyleSheet("background-color: green")
+            self.pushButtonThermal.setToolTip(exp)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotSpeciesFormula(self):
         """
         """
@@ -479,22 +525,32 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
+
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
+        c = self.__boundary.getScalarChoice(self.species)
+        dialog = QMegEditorView(parent        = self,
+                                function_type = 'bnd',
+                                zone_name     = self.__boundary._label,
+                                variable_name = self.species,
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = c,
+                                examples      = exa)
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotSpeciesFormula -> %s" % str(result))
-            self.__boundary.setScalarFormula(self.species, self.species_type, result)
-            setGreenColor(self.pushButtonSpecies, False)
+            self.__boundary.setScalarFormula(self.species, self.species_type, str(result))
+            self.pushButtonSpecies.setStyleSheet("background-color: green")
+            self.pushButtonSpecies.setToolTip(exp)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotMeteoFormula(self):
         """
         """
@@ -505,33 +561,43 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
         elif self.meteo_type == 'neumann_formula':
             req = [("flux", "flux")]
         elif self.meteo_type == 'exchange_coefficient_formula':
-            req = [(self.meteo, str(self.meteo)),("hc", "heat coefficient")]
+            req = [(self.meteo, str(self.meteo)),
+                   ("hc", "heat coefficient")]
 
         sym = [('x', "X face's gravity center"),
                ('y', "Y face's gravity center"),
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
+
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
+        dialog = QMegEditorView(parent        = self,
+                                function_type = 'bnd',
+                                zone_name     = self.__boundary._label,
+                                variable_name = self.meteo,
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = self.meteo_type,
+                                examples      = exa)
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotMeteoFormula -> %s" % str(result))
-            self.__boundary.setScalarFormula(self.meteo, self.meteo_type, result)
-            setGreenColor(self.pushButtonMeteo, False)
+            self.__boundary.setScalarFormula(self.meteo, self.meteo_type, str(result))
+            self.pushButtonMeteo.setStyleSheet("background-color: green")
+            self.pushButtonMeteo.setToolTip(exp)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotValueThermal(self, var):
         """
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditValueThermal.validator().state == QValidator.Acceptable:
             value = from_qvariant(var, float)
             if self.thermal_type in ('dirichlet', 'neumann'):
                 self.__boundary.setScalarValue(self.thermal, self.thermal_type, value)
@@ -539,11 +605,11 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                 self.__boundary.setScalarValue(self.thermal, 'dirichlet', value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotValueSpecies(self, var):
         """
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditValueSpecies.validator().state == QValidator.Acceptable:
             value = from_qvariant(var, float)
             if self.species_type in ('dirichlet', 'neumann'):
                 self.__boundary.setScalarValue(self.species, self.species_type, value)
@@ -551,11 +617,11 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                 self.__boundary.setScalarValue(self.species, 'dirichlet', value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotValueMeteo(self, var):
         """
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditValueMeteo.validator().state == QValidator.Acceptable:
             value = from_qvariant(var, float)
             if self.meteo_type in ('dirichlet', 'neumann'):
                 self.__boundary.setScalarValue(self.meteo, self.meteo_type, value)
@@ -563,29 +629,29 @@ class BoundaryConditionsScalarsView(QWidget, Ui_BoundaryConditionsScalarsForm):
                 self.__boundary.setScalarValue(self.meteo, 'dirichlet', value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotExThermal(self, var):
         """
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditExThermal.validator().state == QValidator.Acceptable:
             value = from_qvariant(var, float)
             self.__boundary.setScalarValue(self.thermal, 'exchange_coefficient', value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotExSpecies(self, var):
         """
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditExSpecies.validator().state == QValidator.Acceptable:
             value = from_qvariant(var, float)
             self.__boundary.setScalarValue(self.species, 'exchange_coefficient', value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotExMeteo(self, var):
         """
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditExMeteo.validator().state == QValidator.Acceptable:
             value = from_qvariant(var, float)
             self.__boundary.setScalarValue(self.meteo, 'exchange_coefficient', value)
 
