@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -31,26 +31,27 @@ This module contains the following classes:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import string, logging
+import logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Pages.BoundaryConditionsSlidingWallForm import Ui_BoundaryConditionsSlidingWallForm
-from Pages.MobileMeshModel import MobileMeshModel
+from code_saturne.Pages.BoundaryConditionsSlidingWallForm import Ui_BoundaryConditionsSlidingWallForm
+from code_saturne.model.MobileMeshModel import MobileMeshModel
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import DoubleValidator, ComboModel, from_qvariant
-from Pages.LocalizationModel import LocalizationModel, Zone
-from Pages.Boundary import Boundary
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import DoubleValidator, ComboModel, from_qvariant
+from code_saturne.model.LocalizationModel import LocalizationModel, Zone
+from code_saturne.model.Boundary import Boundary
 
 #-------------------------------------------------------------------------------
 # log config
@@ -82,16 +83,16 @@ class BoundaryConditionsSlidingWallView(QWidget, Ui_BoundaryConditionsSlidingWal
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
+        self.case.undoStopGlobal()
 
-        self.connect(self.groupBoxSliding, SIGNAL("clicked(bool)"), self.__slotSlidingWall)
+        self.groupBoxSliding.clicked[bool].connect(self.__slotSlidingWall)
 
-        self.connect(self.lineEditSlideU, SIGNAL("textChanged(const QString &)"), self.__slotVelocityU)
-        self.connect(self.lineEditSlideV, SIGNAL("textChanged(const QString &)"), self.__slotVelocityV)
-        self.connect(self.lineEditSlideW, SIGNAL("textChanged(const QString &)"), self.__slotVelocityW)
+        self.lineEditSlideU.textChanged[str].connect(self.__slotVelocityU)
+        self.lineEditSlideV.textChanged[str].connect(self.__slotVelocityV)
+        self.lineEditSlideW.textChanged[str].connect(self.__slotVelocityW)
 
         validatorSlideU = DoubleValidator(self.lineEditSlideU)
         validatorSlideV = DoubleValidator(self.lineEditSlideV)
@@ -101,14 +102,14 @@ class BoundaryConditionsSlidingWallView(QWidget, Ui_BoundaryConditionsSlidingWal
         self.lineEditSlideV.setValidator(validatorSlideV)
         self.lineEditSlideW.setValidator(validatorSlideW)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
     def showWidget(self, boundary):
         """
         Show the widget
         """
-        if MobileMeshModel(self.__case).getMethod() == "off":
+        if MobileMeshModel(self.case).getMethod() == "off":
             self.__boundary = boundary
             if self.__boundary.getVelocityChoice() == "on":
                 self.groupBoxSliding.setChecked(True)
@@ -129,7 +130,7 @@ class BoundaryConditionsSlidingWallView(QWidget, Ui_BoundaryConditionsSlidingWal
         self.hide()
 
 
-    @pyqtSignature("bool")
+    @pyqtSlot(bool)
     def __slotSlidingWall(self, checked):
         """
         Private slot.
@@ -154,7 +155,7 @@ class BoundaryConditionsSlidingWallView(QWidget, Ui_BoundaryConditionsSlidingWal
         self.lineEditSlideW.setText(str(w))
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotVelocityU(self, text):
         """
         Private slot.
@@ -164,12 +165,12 @@ class BoundaryConditionsSlidingWallView(QWidget, Ui_BoundaryConditionsSlidingWal
         @type text: C{QString}
         @param text: sliding wall U velocity component.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditSlideU.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
             self.__boundary.setVelocityComponent(value, '0')
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotVelocityV(self, text):
         """
         Private slot.
@@ -179,12 +180,12 @@ class BoundaryConditionsSlidingWallView(QWidget, Ui_BoundaryConditionsSlidingWal
         @type text: C{QString}
         @param text: sliding wall V velocity component.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditSlideV.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
             self.__boundary.setVelocityComponent(value, '1')
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotVelocityW(self, text):
         """
         Private slot.
@@ -194,7 +195,7 @@ class BoundaryConditionsSlidingWallView(QWidget, Ui_BoundaryConditionsSlidingWal
         @type text: C{QString}
         @param text: sliding wall W velocity component.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditSlideW.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
             self.__boundary.setVelocityComponent(value, '2')
 
