@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -31,25 +31,27 @@ This module contains the following classes:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import string, logging
+import logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Pages.BoundaryConditionsTurbulenceInletForm import Ui_BoundaryConditionsTurbulenceInletForm
-from Pages.TurbulenceModel import TurbulenceModel
+from code_saturne.Pages.BoundaryConditionsTurbulenceInletForm import Ui_BoundaryConditionsTurbulenceInletForm
+from code_saturne.model.TurbulenceModel import TurbulenceModel
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import DoubleValidator, ComboModel, setGreenColor, from_qvariant
-from Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import DoubleValidator, ComboModel, from_qvariant
+from code_saturne.Pages.QMegEditorView import QMegEditorView
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -81,22 +83,23 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
+        self.case.undoStopGlobal()
+        self.notebook = NotebookModel(self.case)
 
-        self.connect(self.comboBoxTurbulence, SIGNAL("activated(const QString&)"), self.__slotChoiceTurbulence)
+        self.comboBoxTurbulence.activated[str].connect(self.__slotChoiceTurbulence)
 
         self.__modelTurbulence = ComboModel(self.comboBoxTurbulence, 2, 1)
         self.__modelTurbulence.addItem(self.tr("Calculation by hydraulic diameter"), 'hydraulic_diameter')
         self.__modelTurbulence.addItem(self.tr("Calculation by turbulent intensity"), 'turbulent_intensity')
         self.__modelTurbulence.addItem(self.tr("Calculation by formula"), 'formula')
 
-        self.connect(self.lineEditDiameter, SIGNAL("textChanged(const QString &)"), self.__slotDiam)
-        self.connect(self.lineEditIntensity, SIGNAL("textChanged(const QString &)"), self.__slotIntensity)
-        self.connect(self.lineEditDiameterIntens, SIGNAL("textChanged(const QString &)"), self.__slotDiam)
-        self.connect(self.pushButtonTurb, SIGNAL("clicked()"), self.__slotTurbulenceFormula)
+        self.lineEditDiameter.textChanged[str].connect(self.__slotDiam)
+        self.lineEditIntensity.textChanged[str].connect(self.__slotIntensity)
+        self.lineEditDiameterIntens.textChanged[str].connect(self.__slotDiam)
+        self.pushButtonTurb.clicked.connect(self.__slotTurbulenceFormula)
 
         validatorDiam = DoubleValidator(self.lineEditDiameter, min=0.)
         validatorDiam.setExclusiveMin(True)
@@ -106,7 +109,7 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
         self.lineEditDiameterIntens.setValidator(validatorDiam)
         self.lineEditIntensity.setValidator(validatorIntensity)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
     def showWidget(self, boundary):
@@ -115,21 +118,19 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
         """
         self.__boundary = boundary
 
-        if TurbulenceModel(self.__case).getTurbulenceVariable():
+        if TurbulenceModel(self.case).getTurbulenceVariable():
             turb_choice = boundary.getTurbulenceChoice()
             self.__modelTurbulence.setItem(str_model=turb_choice)
+            self.pushButtonTurb.setEnabled(False)
+            self.pushButtonTurb.setStyleSheet("background-color: None")
             if turb_choice == "hydraulic_diameter":
                 self.frameTurbDiameter.show()
                 self.frameTurbIntensity.hide()
-                self.pushButtonTurb.setEnabled(False)
-                setGreenColor(self.pushButtonTurb, False)
                 d = boundary.getHydraulicDiameter()
                 self.lineEditDiameter.setText(str(d))
             elif turb_choice == "turbulent_intensity":
                 self.frameTurbIntensity.show()
                 self.frameTurbDiameter.hide()
-                self.pushButtonTurb.setEnabled(False)
-                setGreenColor(self.pushButtonTurb, False)
                 i = boundary.getTurbulentIntensity()
                 d = boundary.getHydraulicDiameter()
                 self.lineEditIntensity.setText(str(i))
@@ -138,7 +139,12 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
                 self.frameTurbIntensity.hide()
                 self.frameTurbDiameter.hide()
                 self.pushButtonTurb.setEnabled(True)
-                setGreenColor(self.pushButtonTurb, True)
+                exp = self.__boundary.getTurbFormula()
+                if exp:
+                    self.pushButtonTurb.setStyleSheet("background-color: green")
+                    self.pushButtonTurb.setToolTip(exp)
+                else:
+                    self.pushButtonTurb.setStyleSheet("background-color: red")
             self.show()
         else:
             self.hideWidget()
@@ -151,7 +157,7 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
         self.hide()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotChoiceTurbulence(self, text):
         """
         INPUT choice of method of calculation of the turbulence
@@ -162,7 +168,7 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
         self.frameTurbDiameter.hide()
         self.frameTurbIntensity.hide()
         self.pushButtonTurb.setEnabled(False)
-        setGreenColor(self.pushButtonTurb, False)
+        self.pushButtonTurb.setStyleSheet("background-color: None")
 
         if turb_choice  == 'hydraulic_diameter':
             self.frameTurbDiameter.show()
@@ -176,35 +182,40 @@ class BoundaryConditionsTurbulenceInletView(QWidget, Ui_BoundaryConditionsTurbul
             self.lineEditDiameterIntens.setText(str(d))
         elif turb_choice == 'formula':
             self.pushButtonTurb.setEnabled(True)
-            setGreenColor(self.pushButtonTurb, True)
+            exp = self.__boundary.getTurbFormula()
+            if exp:
+                self.pushButtonTurb.setStyleSheet("background-color: green")
+                self.pushButtonTurb.setToolTip(exp)
+            else:
+                self.pushButtonTurb.setStyleSheet("background-color: red")
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotDiam(self, text):
         """
         INPUT hydraulic diameter
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditDiameter.validator().state == QValidator.Acceptable:
             diam = from_qvariant(text, float)
             self.__boundary.setHydraulicDiameter(diam)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotIntensity(self, text):
         """
         INPUT turbulent intensity
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIntensity.validator().state == QValidator.Acceptable:
             intens = from_qvariant(text, float)
             self.__boundary.setTurbulentIntensity(intens)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotTurbulenceFormula(self):
         """
         INPUT user formula
         """
-        turb_model = TurbulenceModel(self.__case).getTurbulenceModel()
+        turb_model = TurbulenceModel(self.case).getTurbulenceModel()
         if turb_model in ('k-epsilon', 'k-epsilon-PL'):
 
             exp = self.__boundary.getTurbFormula()
@@ -235,31 +246,45 @@ cmu = 0.09;
 kappa = 0.42;
 k   = ustar2/sqrt(cmu);
 epsilon = ustar2^1.5/(kappa*dh*0.1);"""
+
             req = [('k', "turbulent energy"),
                    ('epsilon', "turbulent dissipation")]
+
             sym = [('x','cell center coordinate'),
                    ('y','cell center coordinate'),
                    ('z','cell center coordinate'),
                    ('t','time'),
                    ('dt','time step'),
-                   ('iter','number of time step')]
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
+                   ('iter','number of time step'),
+                   ('surface', 'Boundary zone surface')]
+
+            for (nme, val) in self.notebook.getNotebookList():
+                sym.append((nme, 'value (notebook) = ' + str(val)))
+
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_ke',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    condition     = 'formula',
+                                    examples      = exa)
+
+
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
-                self.__boundary.setTurbFormula(result)
-                setGreenColor(self.pushButtonTurb, False)
+                self.__boundary.setTurbFormula(str(result))
+                self.pushButtonTurb.setStyleSheet("background-color: green")
+                self.pushButtonTurb.setToolTip(result)
 
         elif turb_model in ('Rij-epsilon', 'Rij-SSG'):
 
             exp = self.__boundary.getTurbFormula()
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
+
             exa = """#exemple :
 uref2 = 10.;
 dh = 0.2;
@@ -293,36 +318,48 @@ r12 = 0;
 r13 = 0;
 r23 = 0;
 """
+
             req = [('r11', "Reynolds stress R11"),
                    ('r22', "Reynolds stress R22"),
                    ('r33', "Reynolds stress R33"),
                    ('r12', "Reynolds stress R12"),
-                   ('r13', "Reynolds stress R13"),
-                   ('r23', "Reynolds stress R23"),
+                   ('r23', "Reynolds stress R13"),
+                   ('r13', "Reynolds stress R23"),
                    ('epsilon', "turbulent dissipation")]
+
             sym = [('x','cell center coordinate'),
                    ('y','cell center coordinate'),
                    ('z','cell center coordinate'),
                    ('t','time'),
                    ('dt','time step'),
-                   ('iter','number of time step')]
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
+                   ('iter','number of time step'),
+                   ('surface', 'Boundary zone surface')]
+
+            for (nme, val) in self.notebook.getNotebookList():
+                sym.append((nme, 'value (notebook) = ' + str(val)))
+
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_rije',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    examples      = exa)
+
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
-                self.__boundary.setTurbFormula(result)
-                setGreenColor(self.pushButtonTurb, False)
+                self.__boundary.setTurbFormula(str(result))
+                self.pushButtonTurb.setStyleSheet("background-color: green")
+                self.pushButtonTurb.setToolTip(result)
 
         elif turb_model == 'Rij-EBRSM':
 
             exp = self.__boundary.getTurbFormula()
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
+
             exa = """#exemple :
 uref2 = 10.;
 dh = 0.2;
@@ -357,37 +394,49 @@ r13 = 0;
 r23 = 0;
 alpha =  1.;
 """
+
             req = [('r11', "Reynolds stress R11"),
                    ('r22', "Reynolds stress R22"),
                    ('r33', "Reynolds stress R33"),
                    ('r12', "Reynolds stress R12"),
-                   ('r13', "Reynolds stress R13"),
-                   ('r23', "Reynolds stress R23"),
+                   ('r23', "Reynolds stress R13"),
+                   ('r13', "Reynolds stress R23"),
                    ('epsilon', "turbulent dissipation"),
                    ('alpha', "alpha")]
+
             sym = [('x','cell center coordinate'),
                    ('y','cell center coordinate'),
                    ('z','cell center coordinate'),
                    ('t','time'),
                    ('dt','time step'),
-                   ('iter','number of time step')]
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
+                   ('iter','number of time step'),
+                   ('surface', 'Boundary zone surface')]
+
+            for (nme, val) in self.notebook.getNotebookList():
+                sym.append((nme, 'value (notebook) = ' + str(val)))
+
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_rij_ebrsm',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    examples      = exa)
+
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
-                self.__boundary.setTurbFormula(result)
-                setGreenColor(self.pushButtonTurb, False)
+                self.__boundary.setTurbFormula(str(result))
+                self.pushButtonTurb.setStyleSheet("background-color: green")
+                self.pushButtonTurb.setToolTip(result)
 
         elif turb_model == 'v2f-BL-v2/k':
 
             exp = self.__boundary.getTurbFormula()
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
+
             exa = """#exemple :
 uref2 = 10.;
 dh = 0.2;
@@ -416,33 +465,46 @@ k   = ustar2/sqrt(cmu);
 epsilon = ustar2^1.5/(kappa*dh*0.1);
 phi = d2s3;
 alpha = 0;"""
+
             req = [('k', "turbulent energy"),
                    ('epsilon', "turbulent dissipation"),
                    ('phi', "variable phi in v2f model"),
                    ('alpha', "variable alpha in v2f model")]
+
             sym = [('x','cell center coordinate'),
                    ('y','cell center coordinate'),
                    ('z','cell center coordinate'),
                    ('t','time'),
                    ('dt','time step'),
-                   ('iter','number of time step')]
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
+                   ('iter','number of time step'),
+                   ('surface', 'Boundary zone surface')]
+
+            for (nme, val) in self.notebook.getNotebookList():
+                sym.append((nme, 'value (notebook) = ' + str(val)))
+
+            dialog = QMegEditorView(parent = self,
+                                    function_type = 'bnd',
+                                    zone_name = self.__boundary._label,
+                                    variable_name = 'turbulence_v2f',
                                     expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
+                                    required = req,
+                                    symbols = sym,
+                                    condition = 'formula',
+                                    examples = exa)
+
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
-                self.__boundary.setTurbFormula(result)
-                setGreenColor(self.pushButtonTurb, False)
+                self.__boundary.setTurbFormula(str(result))
+                self.pushButtonTurb.setStyleSheet("background-color: green")
+                self.pushButtonTurb.setToolTip(result)
 
         elif turb_model == 'k-omega-SST':
 
             exp = self.__boundary.getTurbFormula()
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
+
             exa = """#exemple :
 uref2 = 10.;
 dh = 0.2;
@@ -469,31 +531,45 @@ kappa = 0.42;
 k   = ustar2/sqrt(cmu);
 eps = ustar2^1.5/(kappa*dh*0.1);
 omega = eps/(cmu * k);"""
+
             req = [('k', "turbulent energy"),
                    ('omega', "specific dissipation rate")]
+
             sym = [('x','cell center coordinate'),
                    ('y','cell center coordinate'),
                    ('z','cell center coordinate'),
                    ('t','time'),
                    ('dt','time step'),
-                   ('iter','number of time step')]
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
+                   ('iter','number of time step'),
+                   ('surface', 'Boundary zone surface')]
+
+            for (nme, val) in self.notebook.getNotebookList():
+                sym.append((nme, 'value (notebook) = ' + str(val)))
+
+
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_kw',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    condition     = 'formula',
+                                    examples      = exa)
+
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
-                self.__boundary.setTurbFormula(result)
-                setGreenColor(self.pushButtonTurb, False)
+                self.__boundary.setTurbFormula(str(result))
+                self.pushButtonTurb.setStyleSheet("background-color: green")
+                self.pushButtonTurb.setToolTip(result)
 
         elif turb_model == 'Spalart-Allmaras':
 
             exp = self.__boundary.getTurbFormula()
             if not exp:
                 exp = self.__boundary.getDefaultTurbFormula(turb_model)
+
             exa = """#exemple :
 uref2 = 10.;
 dh = 0.2;
@@ -520,24 +596,36 @@ kappa = 0.42;
 k   = ustar2/sqrt(cmu);
 eps = ustar2^1.5/(kappa*dh*0.1);
 nu_tilda = eps/(cmu * k);"""
+
             req = [('nu_tilda', "nu_tilda")]
+
             sym = [('x','cell center coordinate'),
                    ('y','cell center coordinate'),
                    ('z','cell center coordinate'),
                    ('t','time'),
                    ('dt','time step'),
-                   ('iter','number of time step')]
-            dialog = QMeiEditorView(self,
-                                    check_syntax = self.__case['package'].get_check_syntax(),
-                                    expression = exp,
-                                    required   = req,
-                                    symbols    = sym,
-                                    examples   = exa)
+                   ('iter','number of time step'),
+                   ('surface', 'Boundary zone surface')]
+
+            for (nme, val) in self.notebook.getNotebookList():
+                sym.append((nme, 'value (notebook) = ' + str(val)))
+
+            dialog = QMegEditorView(parent        = self,
+                                    function_type = 'bnd',
+                                    zone_name     = self.__boundary._label,
+                                    variable_name = 'turbulence_spalart',
+                                    expression    = exp,
+                                    required      = req,
+                                    symbols       = sym,
+                                    condition     = 'formula',
+                                    examples      = exa)
+
             if dialog.exec_():
                 result = dialog.get_result()
                 log.debug("slotFormulaTurb -> %s" % str(result))
-                self.__boundary.setTurbFormula(result)
-                setGreenColor(self.pushButtonTurb, False)
+                self.__boundary.setTurbFormula(str(result))
+                self.pushButtonTurb.setStyleSheet("background-color: green")
+                self.pushButtonTurb.setToolTip(result)
 
 
     def tr(self, text):
