@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -31,29 +31,31 @@ This module contains the following classes:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import string, logging
+import logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Pages.BoundaryConditionsVelocityInletForm import Ui_BoundaryConditionsVelocityInletForm
+from code_saturne.Pages.BoundaryConditionsVelocityInletForm import Ui_BoundaryConditionsVelocityInletForm
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import DoubleValidator, ComboModel, setGreenColor, from_qvariant
-from Pages.LocalizationModel import LocalizationModel, Zone
-from Pages.Boundary import Boundary
-from Pages.CompressibleModel import CompressibleModel
-from Pages.GasCombustionModel import GasCombustionModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import DoubleValidator, ComboModel, from_qvariant
+from code_saturne.model.LocalizationModel import LocalizationModel, Zone
+from code_saturne.model.Boundary import Boundary
+from code_saturne.model.CompressibleModel import CompressibleModel
+from code_saturne.model.GasCombustionModel import GasCombustionModel
 
-from Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.Pages.QMegEditorView import QMegEditorView
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -86,38 +88,39 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         """
         Setup the widget
         """
-        self.__case = case
+        self.case = case
         self.__boundary = None
 
-        self.__case.undoStopGlobal()
+        self.case.undoStopGlobal()
 
-        self.mdl = CompressibleModel(self.__case)
-        self.gas = GasCombustionModel(self.__case)
+        self.mdl = CompressibleModel(self.case)
+        self.gas = GasCombustionModel(self.case)
+        self.notebook = NotebookModel(self.case)
 
         # Connections
-        self.connect(self.comboBoxVelocity, SIGNAL("activated(const QString&)"), self.__slotChoiceVelocity)
-        self.connect(self.lineEditVelocity, SIGNAL("textChanged(const QString &)"), self.__slotVelocityValue)
+        self.comboBoxVelocity.activated[str].connect(self.__slotChoiceVelocity)
+        self.lineEditVelocity.textChanged[str].connect(self.__slotVelocityValue)
 
-        self.connect(self.comboBoxDirection,  SIGNAL("activated(const QString&)"), self.__slotChoiceDirection)
-        self.connect(self.lineEditDirectionX, SIGNAL("textChanged(const QString &)"), self.__slotDirX)
-        self.connect(self.lineEditDirectionY, SIGNAL("textChanged(const QString &)"), self.__slotDirY)
-        self.connect(self.lineEditDirectionZ, SIGNAL("textChanged(const QString &)"), self.__slotDirZ)
+        self.comboBoxDirection.activated[str].connect(self.__slotChoiceDirection)
+        self.lineEditDirectionX.textChanged[str].connect(self.__slotDirX)
+        self.lineEditDirectionY.textChanged[str].connect(self.__slotDirY)
+        self.lineEditDirectionZ.textChanged[str].connect(self.__slotDirZ)
 
-        self.connect(self.comboBoxTypeInlet,     SIGNAL("activated(const QString&)"),    self.__slotInletType)
-        self.connect(self.checkBoxPressure,      SIGNAL("clicked()"),                    self.__slotPressure)
-        self.connect(self.checkBoxDensity,       SIGNAL("clicked()"),                    self.__slotDensity)
-        self.connect(self.checkBoxTemperature,   SIGNAL("clicked()"),                    self.__slotTemperature)
-        self.connect(self.checkBoxEnergy,        SIGNAL("clicked()"),                    self.__slotEnergy)
-        self.connect(self.lineEditPressure,      SIGNAL("textChanged(const QString &)"), self.__slotPressureValue)
-        self.connect(self.lineEditDensity,       SIGNAL("textChanged(const QString &)"), self.__slotDensityValue)
-        self.connect(self.lineEditTotalPressure, SIGNAL("textChanged(const QString &)"), self.__slotTotalPressure)
-        self.connect(self.lineEditTotalEnthalpy, SIGNAL("textChanged(const QString &)"), self.__slotTotalEnthalpy)
-        self.connect(self.lineEditTemperature,   SIGNAL("textChanged(const QString &)"), self.__slotTemperatureValue)
-        self.connect(self.lineEditEnergy,        SIGNAL("textChanged(const QString &)"), self.__slotEnergyValue)
+        self.comboBoxTypeInlet.activated[str].connect(self.__slotInletType)
+        self.checkBoxPressure.clicked.connect(self.__slotPressure)
+        self.checkBoxDensity.clicked.connect(self.__slotDensity)
+        self.checkBoxTemperature.clicked.connect(self.__slotTemperature)
+        self.checkBoxEnergy.clicked.connect(self.__slotEnergy)
+        self.lineEditPressure.textChanged[str].connect(self.__slotPressureValue)
+        self.lineEditDensity.textChanged[str].connect(self.__slotDensityValue)
+        self.lineEditTotalPressure.textChanged[str].connect(self.__slotTotalPressure)
+        self.lineEditTotalEnthalpy.textChanged[str].connect(self.__slotTotalEnthalpy)
+        self.lineEditTemperature.textChanged[str].connect(self.__slotTemperatureValue)
+        self.lineEditEnergy.textChanged[str].connect(self.__slotEnergyValue)
 
-        self.connect(self.comboBoxTypeInletGasComb,   SIGNAL("activated(const QString&)"), self.__slotInletTypeGasComb)
-        self.connect(self.lineEditTemperatureGasComb, SIGNAL("textChanged(const QString &)"),  self.__slotTemperatureGasComb)
-        self.connect(self.lineEditFraction,           SIGNAL("textChanged(const QString &)"),  self.__slotMeanMixtureFraction)
+        self.comboBoxTypeInletGasComb.activated[str].connect(self.__slotInletTypeGasComb)
+        self.lineEditTemperatureGasComb.textChanged[str].connect(self.__slotTemperatureGasComb)
+        self.lineEditFraction.textChanged[str].connect(self.__slotMeanMixtureFraction)
 
         # Combo models
         self.modelVelocity = ComboModel(self.comboBoxVelocity, 6, 1)
@@ -135,7 +138,8 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
 
         self.modelTypeInlet = ComboModel(self.comboBoxTypeInlet, 2, 1)
         self.modelTypeInlet.addItem(self.tr("imposed inlet"), 'imposed_inlet')
-        self.modelTypeInlet.addItem(self.tr("subsonic inlet (imposed total pressure and total enthalpy)"), 'subsonic_inlet_PH')
+        self.modelTypeInlet.addItem(self.tr("subsonic inlet (imposed total pressure and total enthalpy)"), \
+                                            'subsonic_inlet_PH')
 
         self.modelTypeInletGasComb = ComboModel(self.comboBoxTypeInletGasComb, 2, 1)
         model = self.gas.getGasCombustionModel()
@@ -174,10 +178,10 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.lineEditTemperatureGasComb.setValidator(validatorTemp)
         self.lineEditFraction.setValidator(validatorFrac)
 
-        self.connect(self.pushButtonVelocityFormula, SIGNAL("clicked()"), self.__slotVelocityFormula)
-        self.connect(self.pushButtonDirectionFormula, SIGNAL("clicked()"), self.__slotDirectionFormula)
+        self.pushButtonVelocityFormula.clicked.connect(self.__slotVelocityFormula)
+        self.pushButtonDirectionFormula.clicked.connect(self.__slotDirectionFormula)
 
-        self.__case.undoStartGlobal()
+        self.case.undoStartGlobal()
 
 
     def showWidget(self, boundary):
@@ -341,7 +345,22 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.hide()
 
 
-    @pyqtSignature("const QString&")
+    def getVelocityExpression(self, boundary):
+        """
+        Get the mathematical expression for velocity
+        """
+        exp = boundary.getVelocity()
+        c = boundary.getVelocityChoice()
+        if c == 'norm_formula':
+            n = 'u_norm'
+        elif c == 'flow1_formula':
+            n = 'q_m'
+        elif c == 'flow2_formula':
+            n = 'q_v'
+
+        return exp, n
+
+    @pyqtSlot(str)
     def __slotChoiceVelocity(self, text):
         """
         Private slot.
@@ -357,12 +376,17 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
 
         if c[-7:] == "formula":
             self.pushButtonVelocityFormula.setEnabled(True)
-            setGreenColor(self.pushButtonVelocityFormula, True)
+            exp = self.__boundary.getVelocity()
+            if exp:
+                self.pushButtonVelocityFormula.setStyleSheet("background-color: green")
+                self.pushButtonVelocityFormula.setToolTip(exp)
+            else:
+                self.pushButtonVelocityFormula.setStyleSheet("background-color: red")
             self.lineEditVelocity.setEnabled(False)
             self.lineEditVelocity.setText("")
         else:
             self.pushButtonVelocityFormula.setEnabled(False)
-            setGreenColor(self.pushButtonVelocityFormula, False)
+            self.pushButtonVelocityFormula.setStyleSheet("background-color: None")
             self.lineEditVelocity.setEnabled(True)
             v = self.__boundary.getVelocity()
             self.lineEditVelocity.setText(str(v))
@@ -383,7 +407,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.labelUnitVelocity.setText(str('m<sup>3</sup>/s'))
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotVelocityValue(self, text):
         """
         Private slot.
@@ -393,12 +417,12 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         @type text: C{QString}
         @param text: value
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditVelocity.validator().state == QValidator.Acceptable:
             v = from_qvariant(text, float)
             self.__boundary.setVelocity(v)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotVelocityFormula(self):
         """
         """
@@ -419,22 +443,31 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
+
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
+        dialog = QMegEditorView(parent        = self,
+                                function_type = "bnd",
+                                zone_name     = self.__boundary._label,
+                                variable_name = "velocity",
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = c,
+                                examples      = exa)
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotFormulaVelocity -> %s" % str(result))
-            self.__boundary.setVelocity(result)
-            setGreenColor(self.pushButtonVelocityFormula, False)
+            self.__boundary.setVelocity(str(result))
+            self.pushButtonVelocityFormula.setStyleSheet("background-color: green")
+            self.pushButtonVelocityFormula.setToolTip(result)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotChoiceDirection(self, text):
         """
         Input the direction type choice.
@@ -445,11 +478,16 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
 
         if c == "formula":
             self.pushButtonDirectionFormula.setEnabled(True)
-            setGreenColor(self.pushButtonDirectionFormula, True)
+            exp = self.__boundary.getDirection('direction_formula')
+            if exp:
+                self.pushButtonDirectionFormula.setStyleSheet("background-color: green")
+                self.pushButtonDirectionFormula.setToolTip(exp)
+            else:
+                self.pushButtonDirectionFormula.setStyleSheet("background-color: red")
             self.frameDirectionCoordinates.hide()
         elif c == "coordinates":
             self.pushButtonDirectionFormula.setEnabled(False)
-            setGreenColor(self.pushButtonDirectionFormula, False)
+            self.pushButtonDirectionFormula.setStyleSheet("background-color: None")
             self.frameDirectionCoordinates.show()
             v = self.__boundary.getDirection('direction_x')
             self.lineEditDirectionX.setText(str(v))
@@ -459,11 +497,11 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.lineEditDirectionZ.setText(str(v))
         elif c == "normal":
             self.pushButtonDirectionFormula.setEnabled(False)
-            setGreenColor(self.pushButtonDirectionFormula, False)
+            self.pushButtonDirectionFormula.setStyleSheet("background-color: None")
             self.frameDirectionCoordinates.hide()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotDirX(self, text):
         """
         INPUT value into direction of inlet flow
@@ -473,7 +511,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setDirection('direction_x', value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotDirY(self, text):
         """
         INPUT value into direction of inlet flow
@@ -483,7 +521,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setDirection('direction_y', value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotDirZ(self, text):
         """
         INPUT value into direction of inlet flow
@@ -493,7 +531,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setDirection('direction_z', value)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotDirectionFormula(self):
         """
         """
@@ -510,22 +548,31 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
                ('z', "Z face's gravity center"),
                ('dt', 'time step'),
                ('t', 'current time'),
-               ('iter', 'number of iteration')]
+               ('iter', 'number of iteration'),
+               ('surface', 'Boundary zone surface')]
+
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
+        dialog = QMegEditorView(parent        = self,
+                                function_type = "bnd",
+                                zone_name     = self.__boundary._label,
+                                variable_name = "direction",
+                                expression    = exp,
+                                required      = req,
+                                symbols       = sym,
+                                condition     = "formula",
+                                examples      = exa)
 
-        dialog = QMeiEditorView(self,
-                                check_syntax = self.__case['package'].get_check_syntax(),
-                                expression = exp,
-                                required   = req,
-                                symbols    = sym,
-                                examples   = exa)
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotFormulaDirection -> %s" % str(result))
-            self.__boundary.setDirection('direction_formula', result)
-            setGreenColor(self.pushButtonDirectionFormula, False)
+            self.__boundary.setDirection('direction_formula', str(result))
+            self.pushButtonDirectionFormula.setToolTip(result)
+            self.pushButtonDirectionFormula.setStyleSheet("background-color: green")
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotInletType(self, text):
         """
         INPUT inlet type : 'oxydant'/'fuel' or 'burned'/'unburned'
@@ -537,7 +584,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.initialize()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotPressure(self):
         """
         Pressure selected or not for the initialisation.
@@ -549,7 +596,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.initialize()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotDensity(self):
         """
         Density selected or not for the initialisation.
@@ -561,7 +608,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.initialize()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotTemperature(self):
         """
         Temperature selected or not for the initialisation.
@@ -573,7 +620,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.initialize()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotEnergy(self):
         """
         Energy selected or not for the initialisation.
@@ -585,7 +632,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
         self.initialize()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotPressureValue(self, text):
         """
         INPUT inlet Pressure
@@ -595,7 +642,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setThermoValue('pressure', t)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotDensityValue(self, text):
         """
         INPUT inlet Density
@@ -605,7 +652,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setThermoValue('density', t)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotTemperatureValue(self, text):
         """
         INPUT inlet Temperature
@@ -615,7 +662,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setThermoValue('temperature', t)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotEnergyValue(self, text):
         """
         INPUT inlet Energy
@@ -625,7 +672,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setThermoValue('energy', t)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotTotalPressure(self, text):
         """
         INPUT inlet total pressure
@@ -635,7 +682,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setThermoValue('total_pressure', t)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotTotalEnthalpy(self, text):
         """
         INPUT inlet total enthalpy
@@ -645,7 +692,7 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setThermoValue('enthalpy', t)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotTemperatureGasComb(self, text):
         """
         INPUT inlet temperature
@@ -655,17 +702,17 @@ class BoundaryConditionsVelocityInletView(QWidget, Ui_BoundaryConditionsVelocity
             self.__boundary.setGasCombustionTemperature(t)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotMeanMixtureFraction(self, text):
         """
         INPUT inlet mean mixutre fraction
         """
         if self.sender().validator().state == QValidator.Acceptable:
-            t = from_qvariant(text, float)
+            f = from_qvariant(text, float)
             self.__boundary.setMeanMixtureFraction(f)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotInletTypeGasComb(self, text):
         """
         INPUT inlet type : 'oxydant'/'fuel' or 'burned'/'unburned'
