@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,8 @@
 This module defines the 'Additional user's scalars' page.
 
 This module contains the following classes:
-- LabelDelegate
+- NameDelegate
+- GGDHDelegate
 - VarianceNameDelegate
 - VarianceDelegate
 - StandardItemModelScalars
@@ -43,21 +44,23 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Common import LABEL_LENGTH_MAX
-from Base.Toolbox import GuiParam
-from Base.QtPage import ComboModel, DoubleValidator, RegExpValidator
-from Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
+from code_saturne.Base.QtPage import ComboModel, DoubleValidator, RegExpValidator
+from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
 
-from Pages.DefineUserScalarsForm import Ui_DefineUserScalarsForm
-from Pages.LocalizationModel import LocalizationModel
-from Pages.DefineUserScalarsModel import DefineUserScalarsModel
+from code_saturne.Pages.DefineUserScalarsForm import Ui_DefineUserScalarsForm
+from code_saturne.model.LocalizationModel import LocalizationModel
+from code_saturne.model.DefineUserScalarsModel import DefineUserScalarsModel
+from code_saturne.model.TurbulenceModel import TurbulenceModel
+from code_saturne.model.GroundwaterModel import GroundwaterModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -68,22 +71,22 @@ log = logging.getLogger("DefineUserScalarsView")
 log.setLevel(GuiParam.DEBUG)
 
 #-------------------------------------------------------------------------------
-# Line edit delegate for the label
+# Line edit delegate for the name
 #-------------------------------------------------------------------------------
 
-class LabelDelegate(QItemDelegate):
+class NameDelegate(QItemDelegate):
     """
     Use of a QLineEdit in the table.
     """
     def __init__(self, parent=None):
         QItemDelegate.__init__(self, parent)
         self.parent = parent
-        self.old_plabel = ""
+        self.old_pname = ""
 
 
     def createEditor(self, parent, option, index):
         editor = QLineEdit(parent)
-        self.old_label = ""
+        self.old_pname = ""
         #editor.installEventFilter(self)
         rx = "[_a-zA-Z][_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX-1) + "}"
         self.regExp = QRegExp(rx)
@@ -93,8 +96,9 @@ class LabelDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
-        self.old_plabel = str(value)
+        self.old_pname = str(value)
         editor.setText(value)
 
 
@@ -103,25 +107,78 @@ class LabelDelegate(QItemDelegate):
             return
 
         if editor.validator().state == QValidator.Acceptable:
-            new_plabel = str(editor.text())
+            new_pname = str(editor.text())
 
-            if new_plabel in model.mdl.getScalarLabelsList():
+            if new_pname in model.mdl.getScalarNameList():
                 default = {}
-                default['label']  = self.old_plabel
-                default['list']   = model.mdl.getScalarLabelsList()
+                default['name']  = self.old_pname
+                default['list']   = model.mdl.getScalarNameList()
                 default['regexp'] = self.regExp
                 log.debug("setModelData -> default = %s" % default)
 
-                from Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
+                from code_saturne.Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
                 dialog = VerifyExistenceLabelDialogView(self.parent, default)
                 if dialog.exec_():
                     result = dialog.get_result()
-                    new_plabel = result['label']
+                    new_pname = result['name']
                     log.debug("setModelData -> result = %s" % result)
                 else:
-                    new_plabel = self.old_plabel
+                    new_pname = self.old_pname
 
-            model.setData(index, to_qvariant(str(new_plabel)), Qt.DisplayRole)
+            model.setData(index, to_qvariant(str(new_pname)), Qt.DisplayRole)
+
+
+#-------------------------------------------------------------------------------
+# Combo box delegate for the modelized turbulent fluxes
+#-------------------------------------------------------------------------------
+
+class GGDHDelegate(QItemDelegate):
+    """
+    Use of a combo box in the table.
+    """
+    def __init__(self, parent, case):
+        super(GGDHDelegate, self).__init__(parent)
+        self.parent   = parent
+        self.case     = case
+
+
+    def createEditor(self, parent, option, index):
+        editor = QComboBox(parent)
+        self.modelCombo = ComboModel(editor, 1, 1)
+        editor.installEventFilter(self)
+        return editor
+
+
+    def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
+
+        if GroundwaterModel(self.case).getGroundwaterModel() == "groundwater":
+            self.modelCombo.addItem(self.tr("OFF"), "OFF")
+        else:
+            self.modelCombo.addItem(self.tr("SGDH"), "SGDH")
+            if TurbulenceModel(self.case).getTurbulenceModel() == "Rij-epsilon" or \
+               TurbulenceModel(self.case).getTurbulenceModel() == "Rij-SSG" or \
+               TurbulenceModel(self.case).getTurbulenceModel() == "Rij-EBRSM":
+                self.modelCombo.addItem(self.tr("GGDH"), "GGDH")
+                self.modelCombo.addItem(self.tr("AFM"), "AFM")
+                self.modelCombo.addItem(self.tr("DFM"), "DFM")
+            if TurbulenceModel(self.case).getTurbulenceModel() == "Rij-EBRSM":
+                self.modelCombo.addItem(self.tr("EB-GGDH"), "EB-GGDH")
+                self.modelCombo.addItem(self.tr("EB-AFM"),  "EB-AFM")
+                self.modelCombo.addItem(self.tr("EB-DFM"),  "EB-DFM")
+
+
+    def setModelData(self, comboBox, model, index):
+        txt = str(comboBox.currentText())
+        value = self.modelCombo.dicoV2M[txt]
+        model.setData(index, to_qvariant(value), Qt.DisplayRole)
+
+
+    def tr(self, text):
+        """
+        Translation
+        """
+        return text
 
 #-------------------------------------------------------------------------------
 # Line edit delegate for the variance name
@@ -134,12 +191,12 @@ class VarianceNameDelegate(QItemDelegate):
     def __init__(self, parent=None):
         QItemDelegate.__init__(self, parent)
         self.parent = parent
-        self.old_plabel = ""
+        self.old_pname = ""
 
 
     def createEditor(self, parent, option, index):
         editor = QLineEdit(parent)
-        self.old_label = ""
+        self.old_pname = ""
         rx = "[_a-zA-Z][_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX-1) + "}"
         self.regExp = QRegExp(rx)
         v = RegExpValidator(editor, self.regExp)
@@ -148,8 +205,9 @@ class VarianceNameDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
-        self.old_plabel = str(value)
+        self.old_pname = str(value)
         editor.setText(value)
 
 
@@ -158,25 +216,25 @@ class VarianceNameDelegate(QItemDelegate):
             return
 
         if editor.validator().state == QValidator.Acceptable:
-            new_plabel = str(editor.text())
+            new_pname = str(editor.text())
 
-            if new_plabel in model.mdl.getScalarLabelsList():
+            if new_pname in model.mdl.getScalarNameList():
                 default = {}
-                default['label']  = self.old_plabel
-                default['list']   = model.mdl.getScalarLabelsList()
+                default['name']  = self.old_pname
+                default['list']   = model.mdl.getScalarNameList()
                 default['regexp'] = self.regExp
                 log.debug("setModelData -> default = %s" % default)
 
-                from Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
+                from code_saturne.Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
                 dialog = VerifyExistenceLabelDialogView(self.parent, default)
                 if dialog.exec_():
                     result = dialog.get_result()
-                    new_plabel = result['label']
+                    new_pname = result['name']
                     log.debug("setModelData -> result = %s" % result)
                 else:
-                    new_plabel = self.old_plabel
+                    new_pname = self.old_pname
 
-            model.setData(index, to_qvariant(str(new_plabel)), Qt.DisplayRole)
+            model.setData(index, to_qvariant(str(new_pname)), Qt.DisplayRole)
 
 
 #-------------------------------------------------------------------------------
@@ -200,8 +258,9 @@ class VarianceDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
-        l1 = index.model().mdl.getScalarLabelsList()
-        for label in index.model().mdl.getThermalScalarLabelsList():
+        editor.setAutoFillBackground(True)
+        l1 = index.model().mdl.getScalarNameList()
+        for label in index.model().mdl.getThermalScalarName():
             l1.append(label)
         for s in index.model().mdl.getScalarsVarianceList():
             if s in l1: l1.remove(s)
@@ -234,12 +293,10 @@ class StandardItemModelScalars(QStandardItemModel):
         """
         QStandardItemModel.__init__(self)
 
-        self.headers = [self.tr("Name")]
+        self.headers = [self.tr("Name"), self.tr("Turbulent flux model")]
 
         self.setColumnCount(len(self.headers))
 
-        self.toolTipRole = [self.tr("Code_Saturne keyword: NSCAUS")]
-
         self._data = []
         self.parent = parent
         self.mdl  = mdl
@@ -252,8 +309,6 @@ class StandardItemModelScalars(QStandardItemModel):
         row = index.row()
         col = index.column()
 
-        if role == Qt.ToolTipRole:
-            return to_qvariant(self.toolTipRole[col])
         if role == Qt.DisplayRole:
             return to_qvariant(self._data[row][col])
 
@@ -261,6 +316,10 @@ class StandardItemModelScalars(QStandardItemModel):
 
 
     def flags(self, index):
+        # first variable if thermal scalar
+        if self.mdl.getThermalScalarName():
+            if index.row() == 0 and index.column() == 0 :
+                return Qt.ItemIsEnabled | Qt.ItemIsSelectable
         if not index.isValid():
             return Qt.ItemIsEnabled
         return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable
@@ -280,14 +339,21 @@ class StandardItemModelScalars(QStandardItemModel):
         row = index.row()
         col = index.column()
 
-        # Label
+        # Name
         if col == 0:
-            old_plabel = self._data[row][col]
-            new_plabel = str(from_qvariant(value, to_text_string))
-            self._data[row][col] = new_plabel
-            self.mdl.renameScalarLabel(old_plabel, new_plabel)
+            old_pname = self._data[row][col]
+            new_pname = str(from_qvariant(value, to_text_string))
+            self._data[row][col] = new_pname
+            self.mdl.renameScalarLabel(old_pname, new_pname)
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        # GGDH
+        elif col == 1:
+            turbFlux = str(from_qvariant(value, to_text_string))
+            self._data[row][col] = turbFlux
+            name = self._data[row][0]
+            self.mdl.setTurbulentFluxModel(name, turbFlux)
+
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -296,14 +362,15 @@ class StandardItemModelScalars(QStandardItemModel):
         return self._data[row]
 
 
-    def newItem(self, existing_label=None):
+    def newItem(self, existing_name=None):
         """
         Add an item in the table view
         """
         row = self.rowCount()
 
-        label = self.mdl.addUserScalar(existing_label)
-        scalar = [label]
+        name = self.mdl.addUserScalar(existing_name)
+        turbFlux = self.mdl.getTurbulentFluxModel(name)
+        scalar = [name, turbFlux]
 
         self.setRowCount(row+1)
         self._data.append(scalar)
@@ -313,8 +380,8 @@ class StandardItemModelScalars(QStandardItemModel):
         """
         Return the values for an item.
         """
-        [label] = self._data[row]
-        return label
+        [name, turbFlux] = self._data[row]
+        return name, turbFlux
 
 
     def deleteItem(self, row):
@@ -345,9 +412,6 @@ class StandardItemModelVariance(QStandardItemModel):
 
         self.setColumnCount(len(self.headers))
 
-        self.toolTipRole = [self.tr("Code_Saturne keyword: ???"),
-                            self.tr("Code_Saturne keyword: ???")]
-
         self._data = []
         self.parent = parent
         self.mdl  = mdl
@@ -360,8 +424,6 @@ class StandardItemModelVariance(QStandardItemModel):
         row = index.row()
         col = index.column()
 
-        if role == Qt.ToolTipRole:
-            return to_qvariant(self.toolTipRole[col])
         if role == Qt.DisplayRole:
             return to_qvariant(self._data[row][col])
 
@@ -388,22 +450,22 @@ class StandardItemModelVariance(QStandardItemModel):
         row = index.row()
         col = index.column()
 
-        # Label
+        # name
         if col == 0:
-            old_plabel = self._data[row][col]
-            new_plabel = str(from_qvariant(value, to_text_string))
-            self._data[row][col] = new_plabel
-            self.mdl.renameScalarLabel(old_plabel, new_plabel)
+            old_pname = self._data[row][col]
+            new_pname = str(from_qvariant(value, to_text_string))
+            self._data[row][col] = new_pname
+            self.mdl.renameScalarLabel(old_pname, new_pname)
 
 
-        # Variance
+        # Variance (associated scalar name)
         elif col == 1:
             variance = str(from_qvariant(value, to_text_string))
             self._data[row][col] = variance
-            [label, var] = self._data[row]
-            self.mdl.setScalarVariance(label,var)
+            name = self._data[row][0]
+            self.mdl.setScalarVariance(name, variance)
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -412,25 +474,25 @@ class StandardItemModelVariance(QStandardItemModel):
         return self._data[row]
 
 
-    def newItem(self, existing_label=None):
+    def newItem(self, existing_name=None):
         """
         Add an item in the table view
         """
-        if not self.mdl.getScalarLabelsList() and not self.mdl.getThermalScalarLabelsList():
+        if not self.mdl.getScalarNameList() and not self.mdl.getThermalScalarName():
             title = self.tr("Warning")
             msg   = self.tr("There is no user scalar.\n"\
                             "Please define a user scalar.")
             QMessageBox.warning(self.parent, title, msg)
             return
         row = self.rowCount()
-        if existing_label == None:
-            label = self.mdl.addVariance()
+        if existing_name == None:
+            name = self.mdl.addVariance()
         else:
-            label = self.mdl.addVariance(existing_label)
-        var = self.mdl.getScalarVariance(label)
+            name = self.mdl.addVariance(existing_name)
+        var = self.mdl.getScalarVariance(name)
         if var in ("", "no variance", "no_variance"):
             var = "no"
-        scalar = [label, var]
+        scalar = [name, var]
 
         self.setRowCount(row+1)
         self._data.append(scalar)
@@ -440,8 +502,8 @@ class StandardItemModelVariance(QStandardItemModel):
         """
         Return the values for an item.
         """
-        [label, var] = self._data[row]
-        return label, var
+        [name, var] = self._data[row]
+        return name, var
 
 
     def deleteItem(self, row):
@@ -462,7 +524,7 @@ class StandardItemModelVariance(QStandardItemModel):
 class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
     """
     """
-    def __init__(self, parent, case, stbar):
+    def __init__(self, parent, case, stbar, tree):
         """
         Constructor
         """
@@ -476,29 +538,36 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         self.case.undoStopGlobal()
 
         self.mdl = DefineUserScalarsModel(self.case)
+        self.browser = tree
 
         # tableView
         self.modelScalars = StandardItemModelScalars(self, self.mdl)
         self.modelVariance = StandardItemModelVariance(self, self.mdl)
-        self.tableScalars.horizontalHeader().setResizeMode(QHeaderView.Stretch)
-        self.tableVariance.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableScalars.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+            self.tableVariance.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableScalars.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
+            self.tableVariance.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
         # Delegates
-        delegateLabel        = LabelDelegate(self.tableScalars)
+        delegateLabel        = NameDelegate(self.tableScalars)
+        delegateGGDH         = GGDHDelegate(self.tableScalars, self.case)
         delegateVarianceName = VarianceNameDelegate(self.tableVariance)
-        delegateVariance     = VarianceDelegate(self.tableVariance)
+        delegateVariance     = VarianceDelegate(self.tableScalars)
 
         self.tableScalars.setItemDelegateForColumn(0, delegateLabel)
+        self.tableScalars.setItemDelegateForColumn(1, delegateGGDH)
         self.tableVariance.setItemDelegateForColumn(0, delegateVarianceName)
         self.tableVariance.setItemDelegateForColumn(1, delegateVariance)
 
         # Connections
-        self.connect(self.pushButtonNew,       SIGNAL("clicked()"), self.slotAddScalar)
-        self.connect(self.pushButtonDelete,    SIGNAL("clicked()"), self.slotDeleteScalar)
-        self.connect(self.modelScalars,        SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), self.dataChanged)
-        self.connect(self.pushButtonVarNew,    SIGNAL("clicked()"), self.slotAddVariance)
-        self.connect(self.pushButtonVarDelete, SIGNAL("clicked()"), self.slotDeleteVariance)
-        self.connect(self.modelVariance,       SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), self.dataChanged)
+        self.pushButtonNew.clicked.connect(self.slotAddScalar)
+        self.pushButtonDelete.clicked.connect(self.slotDeleteScalar)
+        self.modelScalars.dataChanged.connect(self.dataChanged)
+        self.pushButtonVarNew.clicked.connect(self.slotAddVariance)
+        self.pushButtonVarDelete.clicked.connect(self.slotDeleteVariance)
+        self.modelVariance.dataChanged.connect(self.dataChanged)
 
         # widget initialization
         self.tableScalars.reset()
@@ -509,20 +578,22 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         self.modelVariance = StandardItemModelVariance(self, self.mdl)
         self.tableVariance.setModel(self.modelVariance)
 
-        l1 = self.mdl.getScalarLabelsList()
+        l1 = self.mdl.getScalarNameList()
         for s in self.mdl.getScalarsVarianceList():
             if s in l1: l1.remove(s)
-        for label in l1:
-            self.modelScalars.newItem(label)
-        for label in self.mdl.getThermalScalarLabelsList():
-            self.modelScalars.newItem(label)
-        for label in self.mdl.getScalarsVarianceList():
-            self.modelVariance.newItem(label)
+        for name in self.mdl.getThermalScalarName():
+            self.modelScalars.newItem(name)
+        for name in l1:
+            self.modelScalars.newItem(name)
+        for name in self.mdl.getScalarsVarianceList():
+            self.modelVariance.newItem(name)
 
+        if GroundwaterModel(self.case).getGroundwaterModel() != "off":
+            self.groupBox_3.hide()
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotAddScalar(self):
         """
         Add a new item in the table when the 'Create' button is pushed.
@@ -531,7 +602,7 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         self.modelScalars.newItem()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotDeleteScalar(self):
         """
         Just delete the current selected entries from the table and
@@ -546,14 +617,14 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         lst.reverse()
 
         for row in lst:
-            label = self.modelScalars.getItem(row)
-            if self.mdl.getScalarType(label) == 'user':
-                self.mdl.deleteScalar(label)
+            name = self.modelScalars.getItem(row)[0]
+            if self.mdl.getScalarType(name) == 'user':
+                self.mdl.deleteScalar(name)
                 self.modelScalars.deleteItem(row)
             row_var = self.modelVariance.rowCount()
             del_var = []
             for r in range(row_var):
-                if label == self.modelVariance.getItem(r)[1]:
+                if name == self.modelVariance.getItem(r)[1]:
                     del_var.append(self.modelVariance.getItem(r)[0])
             for var in del_var:
                 tot_row = self.modelVariance.rowCount()
@@ -565,9 +636,10 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
                             self.modelVariance.deleteItem(rr)
 
         self.tableScalars.clearSelection()
+        self.browser.configureTree(self.case)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotAddVariance(self):
         """
         Add a new item in the table when the 'Create' button is pushed.
@@ -576,7 +648,7 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         self.modelVariance.newItem()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotDeleteVariance(self):
         """
         Just delete the current selected entries from the table and
@@ -591,14 +663,14 @@ class DefineUserScalarsView(QWidget, Ui_DefineUserScalarsForm):
         lst.reverse()
 
         for row in lst:
-            label = self.modelVariance.getItem(row)[0]
-            self.mdl.deleteScalar(label)
+            name = self.modelVariance.getItem(row)[0]
+            self.mdl.deleteScalar(name)
             self.modelVariance.deleteItem(row)
 
         self.tableVariance.clearSelection()
 
 
-    @pyqtSignature("const QModelIndex &, const QModelIndex &")
+    @pyqtSlot("QModelIndex, QModelIndex")
     def dataChanged(self, topLeft, bottomRight):
         for row in range(topLeft.row(), bottomRight.row()+1):
             self.tableView.resizeRowToContents(row)
