@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -40,18 +40,18 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.QtPage import RegExpValidator, DoubleValidator
-from Base.QtPage import to_qvariant, from_qvariant, to_text_string
-from Base.Toolbox import GuiParam
-from Pages.FacesSelectionForm import Ui_FacesSelectionForm
-from Pages.SolutionDomainModel import SolutionDomainModel
+from code_saturne.Base.QtPage import RegExpValidator, DoubleValidator
+from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.model.Common import GuiParam
+from code_saturne.Pages.FacesSelectionForm import Ui_FacesSelectionForm
 
 #-------------------------------------------------------------------------------
 # log config
@@ -105,10 +105,33 @@ class LineEditDelegateSelector(QItemDelegate):
         editor = QLineEdit(parent)
         validator =  RegExpValidator(editor, QRegExp("[ -~]*"))
         editor.setValidator(validator)
+
+        # Autocompletion for selection criteria!
+        comp_list = ['all[]']
+        comp_list += ['plane[a, b, c, d, epsilon]',
+                      'plane[a, b, c, d, inside]',
+                      'plane[a, b, c, d, outside]',
+                      'plane[n_x, n_y, n_z, x0, y0, z0, epsilon]',
+                      'plane[n_x, n_y, n_z, x0, y0, z0, inside]',
+                      'plane[n_x, n_y, n_z, x0, y0, z0, outside]',
+                      'box[xmin, ymin, zmin, xmax, ymax, zmax]',
+                      'box[x0, y0, z0, dx1, dy1, dz1, dx2, dy2, dz2, dx3, dy3, dz3]',
+                      'cylinder[x0, y0, z0, x1, y1, z1, radius]',
+                      'sphere[x_c, y_c, z_c, radius]']
+
+        completer = QCompleter()
+        editor.setCompleter(completer)
+        model = QStringListModel()
+        completer.setModel(model)
+        model.setStringList(comp_list)
+
         return editor
 
 
     def setEditorData(self, lineEdit, index):
+        # This line is used to avoid an overlay of old and new text
+        lineEdit.setAutoFillBackground(True)
+
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         lineEdit.setText(value)
 
@@ -135,6 +158,7 @@ class FractionPlaneDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(value)
 
@@ -165,8 +189,8 @@ class StandardItemModelFaces(QStandardItemModel):
                         self.tr("Visualization"),
                         self.tr("Selection criteria")]
 
-        self.tooltip = [self.tr("Relative merge tolerance"),
-                        self.tr("Maximum angle for normals of coplanar faces"),
+        self.tooltip = [self.tr("Relative merge tolerance (between 0 and 0.49, 0.1 by default) : maximum intersection distance"),
+                        self.tr("Maximum angle between normals of coplanar faces (25 degrees by default)"),
                         self.tr("Verbosity level"),
                         self.tr("Visualization output level (0 for none)"),
                         self.tr("Selection criteria string")]
@@ -259,7 +283,7 @@ class StandardItemModelFaces(QStandardItemModel):
             self.mdl.replacePeriodicFaces(row, self.dataFaces[row])
 
         log.debug("setData -> dataFaces = %s" % self.dataFaces)
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -318,8 +342,12 @@ class FacesSelectionView(QWidget, Ui_FacesSelectionForm):
 
         self.tableView.setModel(self.modelFaces)
 
-        self.tableView.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
-        self.tableView.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
+        if QT_API == "PYQT4":
+            self.tableView.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
+            self.tableView.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
+        elif QT_API == "PYQT5":
+            self.tableView.verticalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+            self.tableView.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
         self.tableView.horizontalHeader().setStretchLastSection(True)
 
         delegateFraction = FractionPlaneDelegate(self.tableView)
@@ -340,11 +368,10 @@ class FacesSelectionView(QWidget, Ui_FacesSelectionForm):
 
         # Connections
 
-        self.connect(self.pushButtonAdd,    SIGNAL("clicked()"), self.slotAddItem)
-        self.connect(self.pushButtonDelete, SIGNAL("clicked()"), self.slotDelItem)
+        self.pushButtonAdd.clicked.connect(self.slotAddItem)
+        self.pushButtonDelete.clicked.connect(self.slotDelItem)
 
 
-    @pyqtSignature("")
     def slotAddItem(self):
         """
         Create a new faces selection.
@@ -352,7 +379,6 @@ class FacesSelectionView(QWidget, Ui_FacesSelectionForm):
         self.modelFaces.addItem()
 
 
-    @pyqtSignature("")
     def slotDelItem(self):
         """
         Delete a single selected row.
