@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -46,24 +46,25 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import Qt, SIGNAL, pyqtSignature
-from PyQt4.QtGui  import QDialog, QHeaderView, QStandardItemModel, QWidget
-from PyQt4.QtGui  import QAbstractItemView, QItemSelectionModel, QValidator
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.QtPage                          import DoubleValidator, IntValidator
-from Base.QtPage                          import to_qvariant, from_qvariant
-from Pages.FluidStructureInteractionForm  import Ui_FluidStructureInteractionForm
-from Pages.FluidStructureInteractionModel import FluidStructureInteractionModel
-from Pages.LocalizationModel              import LocalizationModel
-from Pages.Boundary                       import Boundary
-from Pages.FluidStructureInteractionAdvancedOptionsDialogForm import \
+from code_saturne.Base.QtPage                          import DoubleValidator, IntValidator
+from code_saturne.Base.QtPage                          import to_qvariant, from_qvariant
+from code_saturne.Pages.FluidStructureInteractionForm  import Ui_FluidStructureInteractionForm
+from code_saturne.model.FluidStructureInteractionModel import FluidStructureInteractionModel
+from code_saturne.model.LocalizationModel              import LocalizationModel
+from code_saturne.model.Boundary                       import Boundary
+from code_saturne.Pages.FluidStructureInteractionAdvancedOptionsDialogForm import \
 Ui_FluidStructureInteractionAdvancedOptionsDialogForm
 
-from Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -71,16 +72,14 @@ from Pages.QMeiEditorView import QMeiEditorView
 
 logging.basicConfig()
 log = logging.getLogger("FluidStructureInteractionView")
-#log.setLevel(GuiParam.DEBUG)
 
 #-------------------------------------------------------------------------------
-# Constantes
+# Constants
 #-------------------------------------------------------------------------------
 
 displacement_prediction_alpha          = 'displacement_prediction_alpha'
 displacement_prediction_beta           = 'displacement_prediction_beta'
 stress_prediction_alpha                = 'stress_prediction_alpha'
-external_coupling_post_synchronization = 'external_coupling_post_synchronization'
 monitor_point_synchronisation          = 'monitor_point_synchronisation'
 
 #-------------------------------------------------------------------------------
@@ -193,10 +192,10 @@ class FluidStructureInteractionAdvancedOptionsView(QDialog,
         return text
 
 
-
 #-------------------------------------------------------------------------------
 # StandarItemModel class
 #-------------------------------------------------------------------------------
+
 class StandardItemModel(QStandardItemModel):
     """
     StandardItemModel for table view
@@ -282,17 +281,17 @@ class StandardItemModel(QStandardItemModel):
 
 class Coupling:
     """
-    Coupling is the base class to manage all widgets which value depend on the
+    Coupling is the base class to manage all widgets whose value depend on the
     boundary.
 
     It provides getBoundaryDefinedValue/setBoundaryDefinedValue methods which
-    get/set the value of a boundary attribute. Getter and setter are specified
-    int the constructor
+    gets/sets the value of a boundary attribute. Getter and setter are specified
+    in the constructor
 
-    It also automatically enable/disable the widget when boundary is present
+    It also automatically enables/disables the widget when boundary is present
     or not
 
-    Derived class can override onBoundarySet method to specify the initial
+    Derived classes can override onBoundarySet method to specify the initial
     value of the widget base on the boundary
     """
 
@@ -350,14 +349,13 @@ class Coupling:
         getattr(self.__boundary, self.__setterStr)(value)
 
 
-
 #-------------------------------------------------------------------------------
 # LineEdit Coupling class
 #-------------------------------------------------------------------------------
 
 class LineEditCoupling(Coupling):
     """
-    LineEdit that depend on a boundary
+    LineEdit that depends on a boundary
     """
 
     def __init__(self, lineEdit, getter, setter):
@@ -369,8 +367,7 @@ class LineEditCoupling(Coupling):
         # Add validator.
         validator = DoubleValidator(lineEdit)
         lineEdit.setValidator(validator)
-        lineEdit.connect(lineEdit, SIGNAL("textChanged(const QString &)"),
-                         self.__slotTextChanged)
+        lineEdit.textChanged[str].connect(self.__slotTextChanged)
 
 
     def onBoundarySet(self):
@@ -381,7 +378,11 @@ class LineEditCoupling(Coupling):
         self.getWidget().setText(str(value))
 
 
-    @pyqtSignature("const QString&")
+    # NOTE: using a decorated slot to connect to a signal is usually recommended,
+    # as it is slightly faster and uses less memory, but is causes a crash
+    # with PyQt5 (not PyQt4) when connecting to a signal from another class.
+
+    # @pyqtSlot(str)
     def __slotTextChanged(self, text):
         """
         Update the model
@@ -407,7 +408,8 @@ class FormulaCoupling(Coupling):
         self.__required = required
         self.__examples = examples
         self.__symbols  = symbols
-        button.connect(button, SIGNAL("clicked(bool)"), self.__slotFormula)
+
+        button.clicked.connect(self.__slotFormula)
 
 
     def onBoundarySet(self):
@@ -418,8 +420,10 @@ class FormulaCoupling(Coupling):
         self.getBoundaryDefinedValue()
 
 
-    @pyqtSignature("const QString&")
-    def __slotFormula(self, text):
+    # NOTE: as above, do not use decorator to avoid crash in PyQt5.
+
+    # @pyqtSlot(bool)
+    def __slotFormula(self, checked):
         """
         Run formula editor.
         """
@@ -455,8 +459,7 @@ class CheckBoxCoupling(Coupling):
         Constructor
         """
         Coupling.__init__(self, checkBox, getter, setter)
-        checkBox.connect(checkBox, SIGNAL("stateChanged(int)"),
-                         self.__slotStateChanged)
+        checkBox.stateChanged[int].connect(self.__slotStateChanged)
 
 
     def onBoundarySet(self):
@@ -471,8 +474,9 @@ class CheckBoxCoupling(Coupling):
             state = Qt.Unchecked
         self.getWidget().setCheckState(state)
 
+    # NOTE: as above, do not use decorator to avoid crash in PyQt5.
 
-    @pyqtSignature("int")
+    # @pyqtSlot(int)
     def __slotStateChanged(self, state):
         """
         Called when checkbox state changed
@@ -491,12 +495,16 @@ class CouplingManager:
     Manage and initialize coupling derived objects
     """
 
-    def __init__(self, mainView, case, internalTableModel, externalTableModel):
+    def __init__(self, mainView, case,
+                 internalTableView, internalTableModel,
+                 externalTableView, externalTableModel):
         """
         Constructor
         """
         self.case               = case
         self.case.undoStopGlobal()
+        self.__internalTableView = internalTableView
+        self.__externalTableView = externalTableView
         self.__internalTableModel = internalTableModel
         self.__externalTableModel = externalTableModel
         self.__internalCouplings = []
@@ -513,35 +521,38 @@ class CouplingManager:
         """
         Initialize the creation of LineEditCoupling
         """
-        coupings = []
-        coupings.append( LineEditCoupling( mainView.lineEditInitialDisplacementX
-                       , "getInitialDisplacementX", "setInitialDisplacementX"))
-        coupings.append( LineEditCoupling( mainView.lineEditInitialDisplacementY
-                       , "getInitialDisplacementY", "setInitialDisplacementY"))
-        coupings.append( LineEditCoupling( mainView.lineEditInitialDisplacementZ
-                       , "getInitialDisplacementZ", "setInitialDisplacementZ"))
+        couplings = []
+        couplings.append(LineEditCoupling(mainView.lineEditInitialDisplacementX,
+                                          "getInitialDisplacementX",
+                                          "setInitialDisplacementX"))
+        couplings.append(LineEditCoupling(mainView.lineEditInitialDisplacementY,
+                                          "getInitialDisplacementY",
+                                          "setInitialDisplacementY"))
+        couplings.append(LineEditCoupling(mainView.lineEditInitialDisplacementZ,
+                                          "getInitialDisplacementZ",
+                                          "setInitialDisplacementZ"))
 
-        coupings.append(
-            LineEditCoupling( mainView.lineEditEquilibriumDisplacementX,
-                              "getEquilibriumDisplacementX",
-                              "setEquilibriumDisplacementX"))
+        couplings.append(LineEditCoupling(mainView.lineEditEquilibriumDisplacementX,
+                                          "getEquilibriumDisplacementX",
+                                          "setEquilibriumDisplacementX"))
 
-        coupings.append(
-            LineEditCoupling( mainView.lineEditEquilibriumDisplacementY,
-                              "getEquilibriumDisplacementY",
-                              "setEquilibriumDisplacementY"))
-        coupings.append(
-            LineEditCoupling( mainView.lineEditEquilibriumDisplacementZ,
-                              "getEquilibriumDisplacementZ",
-                              "setEquilibriumDisplacementZ"))
+        couplings.append(LineEditCoupling(mainView.lineEditEquilibriumDisplacementY,
+                                          "getEquilibriumDisplacementY",
+                                          "setEquilibriumDisplacementY"))
+        couplings.append(LineEditCoupling(mainView.lineEditEquilibriumDisplacementZ,
+                                          "getEquilibriumDisplacementZ",
+                                          "setEquilibriumDisplacementZ"))
 
-        coupings.append( LineEditCoupling( mainView.lineEditInitialVelocityX
-                       , "getInitialVelocityX", "setInitialVelocityX"))
-        coupings.append( LineEditCoupling( mainView.lineEditInitialVelocityY
-                       , "getInitialVelocityY", "setInitialVelocityY"))
-        coupings.append( LineEditCoupling( mainView.lineEditInitialVelocityZ
-                       , "getInitialVelocityZ", "setInitialVelocityZ"))
-        self.__internalCouplings.extend(coupings)
+        couplings.append(LineEditCoupling(mainView.lineEditInitialVelocityX,
+                                          "getInitialVelocityX",
+                                          "setInitialVelocityX"))
+        couplings.append(LineEditCoupling(mainView.lineEditInitialVelocityY,
+                                          "getInitialVelocityY",
+                                          "setInitialVelocityY"))
+        couplings.append(LineEditCoupling(mainView.lineEditInitialVelocityZ,
+                                          "getInitialVelocityZ",
+                                          "setInitialVelocityZ"))
+        self.__internalCouplings.extend(couplings)
 
 
     def __initCheckBoxCouplings(self, mainView):
@@ -549,12 +560,12 @@ class CouplingManager:
         Initialize the creation of the checkbox coupling
         """
         couplings = []
-        couplings.append(CheckBoxCoupling( mainView.checkBoxDDLX, "getDDLX",
-                                           "setDDLX"))
-        couplings.append(CheckBoxCoupling( mainView.checkBoxDDLY, "getDDLY",
-                                           "setDDLY"))
-        couplings.append(CheckBoxCoupling( mainView.checkBoxDDLZ, "getDDLZ",
-                                           "setDDLZ"))
+        couplings.append(CheckBoxCoupling(mainView.checkBoxDDLX,
+                                          "getDDLX", "setDDLX"))
+        couplings.append(CheckBoxCoupling(mainView.checkBoxDDLY,
+                                          "getDDLY", "setDDLY"))
+        couplings.append(CheckBoxCoupling(mainView.checkBoxDDLZ,
+                                          "getDDLZ", "setDDLZ"))
         self.__externalCouplings.extend(couplings)
 
 
@@ -576,6 +587,11 @@ class CouplingManager:
                    ('t', 'current time'),
                    ('nbIter', 'number of iteration')]
 
+        # Add notebook symbols
+        self.notebook = NotebookModel(self.case)
+        for (nme, val) in self.notebook.getNotebookList():
+            symbols.append((nme, 'value (notebook) = ' + str(val)))
+
         m_default = default % {'t':'m'}
         c_default = default % {'t':'c'}
         k_default = default % {'t':'k'}
@@ -602,68 +618,73 @@ k11 = 2;\nk22 = 2;\nk33 = 2;\nk12 = 0;\nk13 = 0;\nk23 = 0;\nk21 = 0;\nk31 = 0;\n
 """
 
         couplings = []
-        couplings.append( FormulaCoupling( mainView.pushButtonMassMatrix,
-                                           "getMassMatrix", "setMassMatrix",
-                                           m_default, m_default_required,
-                                           symbols, m_examples))
+        couplings.append(FormulaCoupling(mainView.pushButtonMassMatrix,
+                                         "getMassMatrix", "setMassMatrix",
+                                         m_default, m_default_required,
+                                         symbols, m_examples))
 
-        couplings.append( FormulaCoupling( mainView.pushButtonDampingMatrix,
-                                           "getDampingMatrix", "setDampingMatrix",
-                                           c_default, c_default_required,
-                                           symbols, c_examples))
+        couplings.append(FormulaCoupling(mainView.pushButtonDampingMatrix,
+                                         "getDampingMatrix", "setDampingMatrix",
+                                         c_default, c_default_required,
+                                         symbols, c_examples))
 
-        couplings.append( FormulaCoupling( mainView.pushButtonStiffnessMatrix,
-                                           "getStiffnessMatrix", "setStiffnessMatrix",
-                                           k_default, k_default_required,
-                                           symbols, k_examples))
+        couplings.append(FormulaCoupling(mainView.pushButtonStiffnessMatrix,
+                                         "getStiffnessMatrix", "setStiffnessMatrix",
+                                         k_default, k_default_required,
+                                         symbols, k_examples))
 
         defaultFluidForce  = "fx = "
         requiredFluidForce = [('fx', 'force applied to the structure along X'),
                               ('fy', 'force applied to the structure along Y'),
                               ('fz', 'force applied to the structure along Z')]
         symbolsFluidForce = symbols[:];
-        symbolsFluidForce.append( ('fluid_fx', 'force of flow along X'))
-        symbolsFluidForce.append( ('fluid_fy', 'force of flow along Y'))
-        symbolsFluidForce.append( ('fluid_fz', 'force of flow along Z'))
+        symbolsFluidForce.append(('fluid_fx', 'force of flow along X'))
+        symbolsFluidForce.append(('fluid_fy', 'force of flow along Y'))
+        symbolsFluidForce.append(('fluid_fz', 'force of flow along Z'))
 
         examplesFluidForce = """# The fluid force is zero in the Y direction.
 #
 fx = fluid_fx;\nfy = 0;\nfz = fluid_fz;"""
-        couplings.append( FormulaCoupling( mainView.pushButtonFluidForce,
-                                           "getFluidForceMatrix",
-                                           "setFluidForceMatrix",
-                                           defaultFluidForce,
-                                           requiredFluidForce,
-                                           symbolsFluidForce,
-                                           examplesFluidForce))
+        couplings.append( FormulaCoupling(mainView.pushButtonFluidForce,
+                                          "getFluidForceMatrix",
+                                          "setFluidForceMatrix",
+                                          defaultFluidForce,
+                                          requiredFluidForce,
+                                          symbolsFluidForce,
+                                          examplesFluidForce))
         self.__internalCouplings.extend(couplings)
 
 
+    # NOTE: as above, do not use decorator to avoid crash in PyQt5.
 
-    @pyqtSignature("QModelIndex  const&, QModelIndex  const&")
+    # @pyqtSlot(QItemSelection, QItemSelection)
     def slotInternalSelectionChanged(self, selected, deselected):
         """
         Called when internal tableView selection changed
         """
-        self.__selectionChanged(self.__internalTableModel,
-                               self.__internalCouplings, selected)
+        self.__selectionChanged(self.__internalTableView,
+                                self.__internalTableModel,
+                                self.__internalCouplings, selected)
 
+    # NOTE: as above, do not use decorator to avoid crash in PyQt5.
 
-    @pyqtSignature("QModelIndex  const&, QModelIndex  const&")
+    # @pyqtSlot(QItemSelection, QItemSelection)
     def slotExternalSelectionChanged(self, selected, deselected):
         """
         Called when external tableView selection changed
         """
-        self.__selectionChanged(self.__externalTableModel,
-                               self.__externalCouplings, selected)
+        self.__selectionChanged(self.__externalTableView,
+                                self.__externalTableModel,
+                                self.__externalCouplings, selected)
 
 
-    def __selectionChanged(self, tableModel, couplings, selected):
+    def __selectionChanged(self, tableView, tableModel, couplings, selected):
         """
         Called when a tableView selection changed
         """
         # Get Boundary
-        label = tableModel.getLabel(selected)
+        index = tableView.currentIndex()
+        label = tableModel.getLabel(index)
 
         boundary = Boundary("coupling_mobile_boundary", label, self.case)
 
@@ -710,36 +731,33 @@ class FluidStructureInteractionView(QWidget, Ui_FluidStructureInteractionForm):
                                                                     'external_coupling')
 
         # Coupling Manager
-        couplingManager = CouplingManager(self, case, self.__internalTableModel
-                                         , self.__externalTableModel)
+        couplingManager = CouplingManager(self, case,
+                                          self.tableInternalCoupling,
+                                          self.__internalTableModel,
+                                          self.tableExternalCoupling,
+                                          self.__externalTableModel)
         # Avoid garbage collector to delete couplingManager
         self.__couplingManager = couplingManager
 
         # Initialize internal / external table view
         self.__initTableView(self.tableInternalCoupling,
-                            self.__internalTableModel,
-                            couplingManager.slotInternalSelectionChanged)
+                             self.__internalTableModel,
+                             couplingManager.slotInternalSelectionChanged)
 
         self.__initTableView(self.tableExternalCoupling,
-                            self.__externalTableModel,
-                            couplingManager.slotExternalSelectionChanged)
+                             self.__externalTableModel,
+                             couplingManager.slotExternalSelectionChanged)
         self.case.undoStartGlobal()
 
 
     def __defineConnection(self):
         """
-        Define coonection for widget that do not depend on the boundary
+        Define connection for widgets that do not depend on the boundary
         """
-        self.connect(self.lineEditNALIMX,
-                     SIGNAL("textChanged(const QString &)"), self.__slotNalimx)
+        self.lineEditNALIMX.textChanged[str].connect(self.__slotNalimx)
 
-        self.connect(self.lineEditEPALIM,
-                     SIGNAL("textChanged(const QString &)"), self.__slotEpalim)
-        self.connect(self.pushButtonAdvanced,
-                     SIGNAL("clicked(bool)"), self.__slotAdvanced)
-        self.connect(self.checkBoxPostSynchronization,
-                     SIGNAL("stateChanged (int)"),
-                     self.__slotPostSynchronization)
+        self.lineEditEPALIM.textChanged[str].connect(self.__slotEpalim)
+        self.pushButtonAdvanced.clicked.connect(self.__slotAdvanced)
 
 
     def __addValidators(self):
@@ -762,8 +780,6 @@ class FluidStructureInteractionView(QWidget, Ui_FluidStructureInteractionForm):
         self.lineEditNALIMX.setText(str(nalimx))
         epalim = self.__model.getPrecision()
         self.lineEditEPALIM.setText(str(epalim))
-        postSynchronization      = self.__model.getExternalCouplingPostSynchronization()
-        self.checkBoxPostSynchronization.setChecked(postSynchronization == 'on')
 
 
     def __createTableViewItemModel(self, modelLocalization, filterALE):
@@ -774,7 +790,7 @@ class FluidStructureInteractionView(QWidget, Ui_FluidStructureInteractionForm):
 
         # Populate QTableView model
         for zone in modelLocalization.getZones():
-            boundary = Boundary("mobile_boundary", zone.getLabel(), self.case)
+            boundary = Boundary(zone.getNature(), zone.getLabel(), self.case)
             if boundary.getALEChoice() == filterALE:
                 tableViewItemModel.addItem(zone)
         return tableViewItemModel
@@ -788,21 +804,23 @@ class FluidStructureInteractionView(QWidget, Ui_FluidStructureInteractionForm):
         tableView.setModel(tableViewItemModel)
 
         # set the column size
-        tableView.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
-        tableView.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
-        tableView.horizontalHeader().setResizeMode(2, QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            tableView.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
+            tableView.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
+            tableView.horizontalHeader().setResizeMode(2, QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            tableView.verticalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+            tableView.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+            tableView.horizontalHeader().setSectionResizeMode(2, QHeaderView.Stretch)
 
         # Connect slot when selection changed
         tableView.setSelectionBehavior(QAbstractItemView.SelectRows)
         selectionModel = QItemSelectionModel(tableViewItemModel, tableView)
         tableView.setSelectionModel(selectionModel)
-
-        self.connect(selectionModel,
-                     SIGNAL( "currentChanged(const QModelIndex &, const QModelIndex &)"),
-                     slotSelectionChanged)
+        selectionModel.selectionChanged.connect(slotSelectionChanged)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotNalimx(self, text):
         """
         Input viscosity type of mesh : isotrop or orthotrop.
@@ -812,7 +830,7 @@ class FluidStructureInteractionView(QWidget, Ui_FluidStructureInteractionForm):
             self.__model.setMaxIterations(nalimx)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def __slotEpalim(self, text):
         """
         Input viscosity type of mesh : isotrop or orthotrop.
@@ -822,20 +840,8 @@ class FluidStructureInteractionView(QWidget, Ui_FluidStructureInteractionForm):
             self.__model.setPrecision(epalim)
 
 
-    @pyqtSignature("int")
-    def __slotPostSynchronization(self, value):
-        """
-        Called when check box state changed
-        """
-        postSynchronization = 'on'
-        if not value:
-            postSynchronization = 'off'
-        self.__model.setExternalCouplingPostSynchronization(postSynchronization)
-
-
-
-    @pyqtSignature("const QString&")
-    def __slotAdvanced(self, text):
+    @pyqtSlot()
+    def __slotAdvanced(self):
         """
         Private slot.
         Ask one popup for advanced specifications
