@@ -1,10 +1,10 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -37,17 +37,18 @@ import logging, os
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import ComboModel, setGreenColor
-from Pages.GasCombustionForm import Ui_GasCombustionForm
-from Pages.GasCombustionModel import GasCombustionModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import ComboModel
+from code_saturne.Pages.GasCombustionForm import Ui_GasCombustionForm
+from code_saturne.model.GasCombustionModel import GasCombustionModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -84,8 +85,8 @@ class GasCombustionView(QWidget, Ui_GasCombustionForm):
         self.modelGasCombustionOption = ComboModel(self.comboBoxGasCombustionOption,1,1)
 
         # Connections
-        self.connect(self.comboBoxGasCombustionOption, SIGNAL("activated(const QString&)"), self.slotGasCombustionOption)
-        self.connect(self.pushButtonThermochemistryData, SIGNAL("pressed()"), self.__slotSearchThermochemistryData)
+        self.comboBoxGasCombustionOption.activated[str].connect(self.slotGasCombustionOption)
+        self.pushButtonThermochemistryData.pressed.connect(self.__slotSearchThermochemistryData)
 
         # Initialize Widgets
         model = self.mdl.getGasCombustionModel()
@@ -112,14 +113,14 @@ class GasCombustionView(QWidget, Ui_GasCombustionForm):
         name = self.mdl.getThermoChemistryDataFileName()
         if name != None:
             self.labelThermochemistryFile.setText(str(name))
-            setGreenColor(self.pushButtonThermochemistryData, False)
+            self.pushButtonThermochemistryData.setStyleSheet("background-color: green")
         else:
-            setGreenColor(self.pushButtonThermochemistryData, True)
+            self.pushButtonThermochemistryData.setStyleSheet("background-color: red")
 
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotGasCombustionOption(self, text):
         """
         Private slot.
@@ -129,15 +130,17 @@ class GasCombustionView(QWidget, Ui_GasCombustionForm):
         self.mdl.setGasCombustionOption(option)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def __slotSearchThermochemistryData(self):
         """
         Select a properties file of data for electric arc
         """
         data = self.case['data_path']
+        if not data:
+            data = "."
         title = self.tr("Thermochemistry file of data.")
         filetypes = self.tr("Thermochemistry (*dp_*);;All Files (*)")
-        file = QFileDialog.getOpenFileName(self, title, data, filetypes)
+        file = QFileDialog.getOpenFileName(self, title, data, filetypes)[0]
         file = str(file)
         if not file:
             return
@@ -149,7 +152,7 @@ class GasCombustionView(QWidget, Ui_GasCombustionForm):
         else:
             self.labelThermochemistryFile.setText(str(file))
             self.mdl.setThermoChemistryDataFileName(file)
-            setGreenColor(self.pushButtonThermochemistryData, False)
+            self.pushButtonThermochemistryData.setStyleSheet("background-color: green")
 
 
     def tr(self, text):
