@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -34,18 +34,21 @@ This module contains the following classes:
 # Library modules import
 #-------------------------------------------------------------------------------
 
-import os, string
+import os
 import logging
 
-from PyQt4 import QtGui, QtCore
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Pages.IdentityAndPathesForm import Ui_IdentityAndPathesForm
-from Pages.IdentityAndPathesModel import IdentityAndPathesModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.MainView import MainView
+from code_saturne.Pages.IdentityAndPathesForm import Ui_IdentityAndPathesForm
+from code_saturne.model.IdentityAndPathesModel import IdentityAndPathesModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -67,21 +70,23 @@ unknown_dir = "????????"
 #
 #-------------------------------------------------------------------------------
 
-class IdentityAndPathesView(QtGui.QWidget, Ui_IdentityAndPathesForm):
+class IdentityAndPathesView(QWidget, Ui_IdentityAndPathesForm):
     """
     Class to open Identity and Pathes Page.
     """
-    def __init__(self, parent, case, study):
+    def __init__(self, parent, case):
         """
         Constructor
         """
-        QtGui.QWidget.__init__(self, parent)
+
+        self.parent = parent
+
+        QWidget.__init__(self, parent)
 
         Ui_IdentityAndPathesForm.__init__(self)
         self.setupUi(self)
 
         self.case = case
-        self.study = study
 
         self.case.undoStopGlobal()
 
@@ -94,14 +99,13 @@ class IdentityAndPathesView(QtGui.QWidget, Ui_IdentityAndPathesForm):
         self.mdl = IdentityAndPathesModel(self.case)
 
         # Create the Page layout.
-        self.connect(self.toolButton, QtCore.SIGNAL("pressed()"), self.searchDir)
+        self.toolButton.pressed.connect(self.searchDir)
         self.lineEditCasePath.setReadOnly(True)
 
         # Initialize the contens of the widgets.
         if not self.case['case_path']:
 
             # Set case path to its default value
-            #
             fic = self.mdl.getXmlFileName()
             if not fic:
                 f = os.getcwd()
@@ -116,19 +120,29 @@ class IdentityAndPathesView(QtGui.QWidget, Ui_IdentityAndPathesForm):
                         title = self.tr("WARNING")
                         msg   = self.tr("Warning: the xml file must be in directory "\
                                         "DATA of the case.")
-                        QtGui.QMessageBox.warning(self, title, msg)
+                        QMessageBox.warning(self, title, msg)
                         self.mdl.setCasePath(file_dir)
                 else:
                     self.mdl.setCasePath(os.path.split(os.getcwd())[0])
 
         self.case_path = self.mdl.getCasePath()
-        self.lineEditCasePath.setText(self.case_path)
-        self.getAbsolutePath()
+
+        study_path = ''
+        case_path = self.case_path
+        if case_path:
+            study_path = os.path.split(os.path.abspath(self.case_path))[0]
+            if study_path:
+                case_path = os.path.basename(case_path)
+
+        self.__getAbsolutePath()
+
+        self.lineEditStudyPath.setText(study_path)
+        self.lineEditCasePath.setText(case_path)
 
         self.case.undoStartGlobal()
 
 
-    def updateId(self, case_path):
+    def __updateId(self, case_path):
         """
         Update Study and Case names in the StudyId bar.
         """
@@ -138,7 +152,9 @@ class IdentityAndPathesView(QtGui.QWidget, Ui_IdentityAndPathesForm):
 
         self.mdl.setId(case_name, study_name)
         fic = self.mdl.getXmlFileName()
-        self.study.set(study=study_name, case=case_name, filename=fic)
+
+        if hasattr(self.parent, 'updateTitleBar'):
+            self.parent.updateTitleBar()
 
 
     def searchDir(self):
@@ -149,22 +165,22 @@ class IdentityAndPathesView(QtGui.QWidget, Ui_IdentityAndPathesForm):
 
         title    = self.tr("Select directory")
         default  = path_case
-        options  = QtGui.QFileDialog.ShowDirsOnly # | QtGui.QFileDialog.DontResolveSymlinks
-        dir_name = QtGui.QFileDialog.getExistingDirectory(self, title, default, options)
+        options  = QFileDialog.ShowDirsOnly # | QFileDialog.DontResolveSymlinks
+        dir_name = QFileDialog.getExistingDirectory(self, title, default, options)
         dir_name = str(dir_name)
 
         if dir_name:
             self.case_path = dir_name
-            self.lineEditCasePath.setText(self.case_path)
-            self.getAbsolutePath()
-            self.updateId(dir_name)
+            self.mdl.setCasePath(self.case_path)
+            self.__getAbsolutePath()
+            self.__updateId(dir_name)
 
 
-    def getAbsolutePath(self):
+    def __getAbsolutePath(self):
         """
         Get absolute path for the case sub-directories and the meshes.
         """
-        self.case_path = str(self.lineEditCasePath.text())
+        self.case_path = self.mdl.getCasePath()
         self.lineEditCasePath.setText(self.case_path)
         case_dir = os.path.abspath(self.case_path)
         self.case_path = case_dir
@@ -173,7 +189,7 @@ class IdentityAndPathesView(QtGui.QWidget, Ui_IdentityAndPathesForm):
 
         if os.path.isdir(case_dir) :
             self.mdl.setCasePath(case_dir)
-            self.updateId(case_dir)
+            self.__updateId(case_dir)
 
             msg = [self.tr("Warning: the DATA sub-directory DATA is required."),
                    self.tr("Warning: the RESU sub-directory RESU is required."),
@@ -197,7 +213,13 @@ class IdentityAndPathesView(QtGui.QWidget, Ui_IdentityAndPathesForm):
                     line.setStatusTip(msg[i])
                     self.mdl.setRelevantSubdir("no", sub_dir[i])
 
-            if os.path.isdir(self.spath) and meshes_dir in os.listdir(self.spath):
+            set_subdir = False
+            try:
+                if os.path.isdir(self.spath) and meshes_dir in os.listdir(self.spath):
+                    set_subdr = True
+            except Exception:
+                pass
+            if set_subdir:
                 self.mdl.setPath(self.path[4], os.path.abspath(self.spath + '/' + meshes_dir))
                 self.mdl.setRelevantSubdir("yes", self.path[4])
             else:
