@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -38,24 +38,25 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import IntValidator, DoubleValidator, ComboModel
-from Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import IntValidator, DoubleValidator, ComboModel
+from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
 
-from Pages.LagrangianBoundariesForm import Ui_LagrangianBoundariesForm
-from Pages.LocalizationModel import LocalizationModel, Zone
-from Pages.LagrangianBoundariesModel import LagrangianBoundariesModel
-from Pages.LagrangianModel import LagrangianModel
-from Pages.LagrangianStatisticsModel import LagrangianStatisticsModel
-from Pages.CoalCombustionModel import CoalCombustionModel
+from code_saturne.Pages.LagrangianBoundariesForm import Ui_LagrangianBoundariesForm
+from code_saturne.model.LocalizationModel import LocalizationModel, Zone
+from code_saturne.model.LagrangianBoundariesModel import LagrangianBoundariesModel
+from code_saturne.model.LagrangianModel import LagrangianModel
+from code_saturne.model.LagrangianStatisticsModel import LagrangianStatisticsModel
+from code_saturne.model.CoalCombustionModel import CoalCombustionModel
 
 
 #-------------------------------------------------------------------------------
@@ -77,11 +78,12 @@ class ValueDelegate(QItemDelegate):
 
     def createEditor(self, parent, option, index):
         editor = QLineEdit(parent)
-        validator = IntValidator(editor, min=0) # nb max classes
+        validator = IntValidator(editor, min=0) # nb max sets
         editor.setValidator(validator)
         return editor
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(value)
 
@@ -147,7 +149,7 @@ class StandardItemModelBoundaries(QStandardItemModel):
         QStandardItemModel.__init__(self)
         self.headers = [self.tr("Label"), self.tr("Nature"),
                         self.tr("Particle-boundary\ninteraction"),
-                        self.tr("Number of classes")]
+                        self.tr("Number of sets")]
         self.setColumnCount(len(self.headers))
         self.case = case
         self.model = model
@@ -161,9 +163,15 @@ class StandardItemModelBoundaries(QStandardItemModel):
                            "deposit1" : self.tr("Deposition and elimination"),
                            "deposit2" : self.tr("Deposition"),
                            "fouling" : self.tr("Fouling") },
-                "inlet" : { "inlet" : self.tr("Particles inlet") },
+                "inlet" : { "inlet" : self.tr("Particles inlet"),
+                            "bounce" : self.tr("Particles rebound"),
+                            "outlet" : self.tr("Particles outlet") },
                 "outlet" : { "outlet" : self.tr("Particles outlet") },
-                "symmetry" : { "part_symmetry" : self.tr("Particles zero-flux") }
+                "free_inlet_outlet" : { "inlet" : self.tr("Particles inlet"),
+                                        "outlet" : self.tr("Particles outlet") },
+                "imposed_p_outlet" : { "outlet" : self.tr("Particles outlet") },
+                "symmetry" : { "part_symmetry" : self.tr("Particles zero-flux"),
+                               "bounce" : self.tr("Particles rebound") }
                 }
         else:
             self.dicoM2V = {
@@ -171,9 +179,15 @@ class StandardItemModelBoundaries(QStandardItemModel):
                            "bounce" : self.tr("Particles rebound"),
                            "deposit1" : self.tr("Deposition and elimination"),
                            "deposit2" : self.tr("Deposition") },
-                "inlet" : { "inlet" : self.tr("Particles inlet") },
+                "inlet" : { "inlet" : self.tr("Particles inlet"),
+                            "bounce" : self.tr("Particles rebound"),
+                            "outlet" : self.tr("Particles outlet") },
                 "outlet" : { "outlet" : self.tr("Particles outlet") },
-                "symmetry" : { "part_symmetry" : self.tr("Particles zero-flux") }
+                "free_inlet_outlet" : { "inlet" : self.tr("Particles inlet"),
+                                        "outlet" : self.tr("Particles outlet") },
+                "imposed_p_outlet" : { "outlet" : self.tr("Particles outlet") },
+                "symmetry" : { "part_symmetry" : self.tr("Particles zero-flux"),
+                               "bounce" : self.tr("Particles rebound")}
                 }
 
         self.dicoV2M = {}
@@ -188,8 +202,8 @@ class StandardItemModelBoundaries(QStandardItemModel):
             label = zone.getLabel()
             nature = zone.getNature()
             interaction = self.model.getBoundaryChoice(nature, label)
-            nclasses = self.model.getNumberOfClassesValue(label)
-            line = [label, nature, interaction, nclasses]
+            n_sets = self.model.getNumberOfSetsValue(label)
+            line = [label, nature, interaction, n_sets]
             self._data.append(line)
             row = self.rowCount()
             self.setRowCount(row+1)
@@ -251,14 +265,14 @@ class StandardItemModelBoundaries(QStandardItemModel):
                 self._data[row][3] = 0
 
         elif col == 3:
-            nclasses = from_qvariant(value, int)
-            self._data[row][col] = nclasses
+            n_sets = from_qvariant(value, int)
+            self._data[row][col] = n_sets
             label = self._data[row][0]
-            nn = self.model.getNumberOfClassesValue(label)
+            nn = self.model.getNumberOfSetsValue(label)
             label = self._data[row][0]
-            self.model.setNumberOfClassesValue(label, nclasses)
+            self.model.setNumberOfSetsValue(label, n_sets)
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -294,66 +308,58 @@ class LagrangianBoundariesView(QWidget, Ui_LagrangianBoundariesForm):
         self.modelBoundaries = StandardItemModelBoundaries(self.case, self.model)
         self.tableViewBoundaries.setModel(self.modelBoundaries)
         self.tableViewBoundaries.setAlternatingRowColors(True)
-        self.tableViewBoundaries.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableViewBoundaries.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableViewBoundaries.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
         delegateInteraction = ParticleBoundaryInteractionDelegate(self.tableViewBoundaries)
-        delegateClassNumber = ValueDelegate(self.tableViewBoundaries)
+        delegateSetNumber = ValueDelegate(self.tableViewBoundaries)
         self.tableViewBoundaries.setItemDelegateForColumn(2,delegateInteraction)
-        self.tableViewBoundaries.setItemDelegateForColumn(3,delegateClassNumber)
+        self.tableViewBoundaries.setItemDelegateForColumn(3,delegateSetNumber)
 
         self.modelIPOIT = ComboModel(self.comboBoxIPOIT,3,1)
-        self.modelIPOIT.addItem(self.tr("Volumic flow rate"), "rate")
+        self.modelIPOIT.addItem(self.tr("Mass flow rate"), "rate")
         self.modelIPOIT.addItem(self.tr("Statistical weight set by values"), "prescribed")
-        self.modelIPOIT.addItem(self.tr("User defined statistical weight"), "subroutine")
 
         self.modelIJUVW = ComboModel(self.comboBoxIJUVW,4,1)
         self.modelIJUVW.addItem(self.tr("Fluid velocity"), "fluid")
         self.modelIJUVW.addItem(self.tr("Normal direction velocity"), "norm")
         self.modelIJUVW.addItem(self.tr("Velocity given by values"), "components")
-        self.modelIJUVW.addItem(self.tr("User defined velocity"), "subroutine")
 
         self.modelIJRTP = ComboModel(self.comboBoxIJRTP,2,1)
+        self.modelIJRTP.addItem(self.tr("Fluid temperature"), "fluid")
         self.modelIJRTP.addItem(self.tr("Temperature set by values"), "prescribed")
-        self.modelIJRTP.addItem(self.tr("User defined temperature"), "subroutine")
 
-        self.modelIJRDP = ComboModel(self.comboBoxIJRDP,2,1)
-        self.modelIJRDP.addItem(self.tr("Diameter set by values"), "prescribed")
-        self.modelIJRDP.addItem(self.tr("User defined diameter"), "subroutine")
+        self.tableViewBoundaries.clicked[QModelIndex].connect(self.slotSelectBoundary)
+        self.modelBoundaries.dataChanged.connect(self.dataChanged)
+        self.spinBoxICLAS.valueChanged[int].connect(self.slotICLAS)
 
-        self.modelIRAWCL = ComboModel(self.comboBoxIRAWCL,2,1)
-        self.modelIRAWCL.addItem(self.tr("Raw coal"), "raw_coal_as_received")
-        self.modelIRAWCL.addItem(self.tr("User defined"), "subroutine")
+        self.lineEditIJNBP.textChanged[str].connect(self.slotIJNBP)
+        self.lineEditIJFRE.textChanged[str].connect(self.slotIJFRE)
+        self.lineEditICLST.textChanged[str].connect(self.slotICLST)
+        self.lineEditIDEBT.textChanged[str].connect(self.slotIDEBT)
+        self.comboBoxIPOIT.activated[str].connect(self.slotIPOITChoice)
+        self.lineEditIPOIT.textChanged[str].connect(self.slotIPOIT)
+        self.lineEditIROPT.textChanged[str].connect(self.slotIROPT)
+        self.lineEditIRCOLM.textChanged[str].connect(self.slotIRCOLM)
 
-        self.connect(self.tableViewBoundaries, SIGNAL("clicked(const QModelIndex &)"), self.slotSelectBoundary)
-        self.connect(self.modelBoundaries,     SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), self.dataChanged)
-        self.connect(self.spinBoxICLAS, SIGNAL("valueChanged(int)"), self.slotICLAS)
+        self.comboBoxIJUVW.activated[str].connect(self.slotIJUVW)
+        self.lineEditIUNO.textChanged[str].connect(self.slotIUNO)
+        self.lineEditIUPT.textChanged[str].connect(self.slotIUPT)
+        self.lineEditIVPT.textChanged[str].connect(self.slotIVPT)
+        self.lineEditIWPT.textChanged[str].connect(self.slotIWPT)
 
-        self.connect(self.lineEditIJNBP,  SIGNAL("textChanged(const QString &)"), self.slotIJNBP)
-        self.connect(self.lineEditIJFRE,  SIGNAL("textChanged(const QString &)"), self.slotIJFRE)
-        self.connect(self.lineEditICLST,  SIGNAL("textChanged(const QString &)"), self.slotICLST)
-        self.connect(self.lineEditIDEBT,  SIGNAL("textChanged(const QString &)"), self.slotIDEBT)
-        self.connect(self.comboBoxIPOIT,  SIGNAL("activated(const QString&)"), self.slotIPOITChoice)
-        self.connect(self.lineEditIPOIT,  SIGNAL("textChanged(const QString &)"), self.slotIPOIT)
-        self.connect(self.lineEditIROPT,  SIGNAL("textChanged(const QString &)"), self.slotIROPT)
+        self.comboBoxIJRTP.activated[str].connect(self.slotIJRTP)
+        self.lineEditITPT.textChanged[str].connect(self.slotITPT)
+        self.lineEditICPT.textChanged[str].connect(self.slotICPT)
+        self.lineEditIEPSI.textChanged[str].connect(self.slotIEPSI)
 
-        self.connect(self.comboBoxIJUVW, SIGNAL("activated(const QString&)"),    self.slotIJUVW)
-        self.connect(self.lineEditIUNO,  SIGNAL("textChanged(const QString &)"), self.slotIUNO)
-        self.connect(self.lineEditIUPT,  SIGNAL("textChanged(const QString &)"), self.slotIUPT)
-        self.connect(self.lineEditIVPT,  SIGNAL("textChanged(const QString &)"), self.slotIVPT)
-        self.connect(self.lineEditIWPT,  SIGNAL("textChanged(const QString &)"), self.slotIWPT)
+        self.lineEditIDPT.textChanged[str].connect(self.slotIDPT)
+        self.lineEditIVDPT.textChanged[str].connect(self.slotIVDPT)
 
-        self.connect(self.comboBoxIJRTP, SIGNAL("activated(const QString&)"),    self.slotIJRTP)
-        self.connect(self.lineEditITPT,  SIGNAL("textChanged(const QString &)"), self.slotITPT)
-        self.connect(self.lineEditICPT,  SIGNAL("textChanged(const QString &)"), self.slotICPT)
-        self.connect(self.lineEditIEPSI, SIGNAL("textChanged(const QString &)"), self.slotIEPSI)
-
-        self.connect(self.comboBoxIJRDP, SIGNAL("activated(const QString&)"),    self.slotIJRDP)
-        self.connect(self.lineEditIDPT,  SIGNAL("textChanged(const QString &)"), self.slotIDPT)
-        self.connect(self.lineEditIVDPT, SIGNAL("textChanged(const QString &)"), self.slotIVDPT)
-
-        self.connect(self.lineEditINUCHL, SIGNAL("textChanged(const QString &)"), self.slotINUCHL)
-        self.connect(self.lineEditIHPT,   SIGNAL("textChanged(const QString &)"), self.slotIHPT)
-        self.connect(self.comboBoxIRAWCL, SIGNAL("activated(const QString&)"),    self.slotIRAWCL)
+        self.lineEditINUCHL.textChanged[str].connect(self.slotINUCHL)
+        self.lineEditIHPT.textChanged[str].connect(self.slotIHPT)
 
         # Validators
         validatorIJNBP  = IntValidator(self.lineEditIJNBP, min=0)
@@ -364,6 +370,7 @@ class LagrangianBoundariesView(QWidget, Ui_LagrangianBoundariesForm):
         validatorIPOIT.setExclusiveMin(True)
         validatorIROPT  = DoubleValidator(self.lineEditIROPT, min=0.)
         validatorIROPT.setExclusiveMin(True)
+        validatorIRCOLM = DoubleValidator(self.lineEditIRCOLM, min=0.)
 
         validatorIUNO = DoubleValidator(self.lineEditIUNO)
         validatorIUPT = DoubleValidator(self.lineEditIUPT)
@@ -386,6 +393,7 @@ class LagrangianBoundariesView(QWidget, Ui_LagrangianBoundariesForm):
         self.lineEditIDEBT.setValidator(validatorIDEBT)
         self.lineEditIPOIT.setValidator(validatorIPOIT)
         self.lineEditIROPT.setValidator(validatorIROPT)
+        self.lineEditIRCOLM.setValidator(validatorIRCOLM)
 
         self.lineEditIUNO.setValidator(validatorIUNO)
         self.lineEditIUPT.setValidator(validatorIUPT)
@@ -408,7 +416,7 @@ class LagrangianBoundariesView(QWidget, Ui_LagrangianBoundariesForm):
 
 
     def _hideAllWidgets(self):
-        self.groupBoxClassNumber.hide()
+        self.groupBoxSetNumber.hide()
         self.groupBoxMain.hide()
         self.groupBoxRate.hide()
         self.groupBoxVelocity.hide()
@@ -417,120 +425,111 @@ class LagrangianBoundariesView(QWidget, Ui_LagrangianBoundariesForm):
         self.groupBoxCoal.hide()
 
 
-    @pyqtSignature("const QModelIndex&")
+    @pyqtSlot("QModelIndex")
     def slotSelectBoundary(self, index):
         """
         """
         self._hideAllWidgets()
-        label, nature, interaction, nclasses = self.modelBoundaries.getItem(index.row())
+        label, nature, interaction, n_sets = self.modelBoundaries.getItem(index.row())
         self.label = label
         if interaction != "inlet":
             return
         self.model.setCurrentBoundaryNode(nature, label)
-        if nclasses > 0:
-            self.groupBoxClassNumber.show()
+        if n_sets > 0:
+            self.groupBoxSetNumber.show()
             self.spinBoxICLAS.setMinimum(1)
-            self.spinBoxICLAS.setMaximum(nclasses)
+            self.spinBoxICLAS.setMaximum(n_sets)
             self.spinBoxICLAS.setValue(1)
             self.slotICLAS(1)
 
 
-    @pyqtSignature("const QModelIndex &, const QModelIndex &")
     def dataChanged(self, topLeft, bottomRight):
         """
         """
         self.slotSelectBoundary(topLeft)
 
 
-    @pyqtSignature("int")
-    def slotICLAS(self, iclass):
+    @pyqtSlot(int)
+    def slotICLAS(self, iset):
         """
         Input ICLAS.
         """
-        self.iclass = iclass
+        self.iset = iset
         index = self.tableViewBoundaries.currentIndex()
-        label, nature, interaction, nclasses = self.modelBoundaries.getItem(index.row())
+        label, nature, interaction, n_sets = self.modelBoundaries.getItem(index.row())
         if interaction == "inlet":
-            self.model.setCurrentClassNode(self.label, iclass)
+            self.model.setCurrentSetNode(self.label, iset)
 
         self.LM = LagrangianModel(self.case)
         part_model = self.LM.getParticlesModel()
 
         # Main variables
         self.groupBoxMain.show()
-        npart = self.model.getNumberOfParticulesInClassValue(self.label, self.iclass)
+        npart = self.model.getNumberOfParticulesInSetValue(self.label, self.iset)
         self.lineEditIJNBP.setText(str(npart))
-        freq = self.model.getInjectionFrequencyValue(self.label, self.iclass)
+        freq = self.model.getInjectionFrequencyValue(self.label, self.iset)
         self.lineEditIJFRE.setText(str(freq))
 
         self.LSM = LagrangianStatisticsModel(self.case)
         if self.LSM.getGroupOfParticlesValue() > 0:
-            igroup = self.model.getParticleGroupNumberValue(self.label, self.iclass)
+            igroup = self.model.getParticleGroupNumberValue(self.label, self.iset)
             self.lineEditICLST.setText(str(igroup))
+            self.labelICLST.show()
+            self.lineEditICLST.show()
         else:
-            self.labelICLST.setDisabled(True)
-            self.lineEditICLST.setDisabled(True)
+            self.labelICLST.hide()
+            self.lineEditICLST.hide()
 
         # Rate / stat. weight
         self.groupBoxRate.show()
-        choice = self.model.getStatisticalWeightChoice(self.label, self.iclass)
+        choice = self.model.getStatisticalWeightChoice(self.label, self.iset)
         self.modelIPOIT.setItem(str_model=choice)
         text = self.modelIPOIT.dicoM2V[choice]
         self.slotIPOITChoice(text)
 
         # Velocity
         self.groupBoxVelocity.show()
-        choice = self.model.getVelocityChoice(self.label, self.iclass)
+        choice = self.model.getVelocityChoice(self.label, self.iset)
         self.modelIJUVW.setItem(str_model=choice)
         text = self.modelIJUVW.dicoM2V[choice]
         self.slotIJUVW(text)
 
+        # Fouling
+        colm = self.model.getFoulingIndexValue(self.label, self.iset)
+        self.lineEditIRCOLM.setText(str(colm))
+
         # Temperature
         status = self.LM.getHeating()
         if part_model == "thermal" and status == "on":
             self.groupBoxTemperature.show()
-            choice = self.model.getTemperatureChoice(self.label, self.iclass)
+            choice = self.model.getTemperatureChoice(self.label, self.iset)
             self.modelIJRTP.setItem(str_model=choice)
             text = self.modelIJRTP.dicoM2V[choice]
             self.slotIJRTP(text)
 
-            cp = self.model.getSpecificHeatValue(self.label, self.iclass)
+            cp = self.model.getSpecificHeatValue(self.label, self.iset)
             self.lineEditICPT.setText(str(cp))
-            eps = self.model.getEmissivityValue(self.label, self.iclass)
+            eps = self.model.getEmissivityValue(self.label, self.iset)
             self.lineEditIEPSI.setText(str(eps))
 
         # Coals
-        if CoalCombustionModel(self.case).getCoalCombustionModel() != 'off':
+        if CoalCombustionModel(self.case).getCoalCombustionModel("only") != 'off':
             self.groupBoxCoal.show()
-            icoal = self.model.getCoalNumberValue(self.label, self.iclass)
+            icoal = self.model.getCoalNumberValue(self.label, self.iset)
             self.lineEditINUCHL.setText(str(icoal))
-            temp  = self.model.getCoalTemperatureValue(self.label, self.iclass)
+            temp  = self.model.getCoalTemperatureValue(self.label, self.iset)
             self.lineEditIHPT.setText(str(temp))
-            choice = self.model.getCoalCompositionChoice(self.label, self.iclass)
-            self.modelIRAWCL.setItem(str_model=choice)
 
         # Diameter
         self.groupBoxDiameter.show()
-        choice = self.model.getDiameterChoice(self.label, self.iclass)
 
-        if part_model == "coal":
-            self.modelIJRDP.setItem(str_model="prescribed")
-        else:
-            self.modelIJRDP.setItem(str_model=choice)
-            text = self.modelIJRDP.dicoM2V[choice]
-            self.slotIJRDP(text)
-
-        if choice == "prescribed":
-            self.frameDiameter.show()
-            diam = self.model.getDiameterValue(self.label, self.iclass)
-            vdiam = self.model.getDiameterVarianceValue(self.label, self.iclass)
-            self.lineEditIDPT.setText(str(diam))
-            self.lineEditIVDPT.setText(str(vdiam))
-        elif choice == "subroutine":
-            self.frameDiameter.hide()
+        diam = self.model.getDiameterValue(self.label, self.iset)
+        vdiam = self.model.getDiameterVarianceValue(self.label, self.iset)
+        self.lineEditIDPT.setText(str(diam))
+        self.lineEditIVDPT.setText(str(vdiam))
 
         #Coal
-        if CoalCombustionModel(self.case).getCoalCombustionModel() != 'off':
+        if CoalCombustionModel(self.case).getCoalCombustionModel("only") != 'off':
             self.labelIROPT.hide()
             self.labelUnitIROPT.hide()
             self.lineEditIROPT.hide()
@@ -538,264 +537,245 @@ class LagrangianBoundariesView(QWidget, Ui_LagrangianBoundariesForm):
             self.labelIROPT.show()
             self.labelUnitIROPT.show()
             self.lineEditIROPT.show()
-            rho = self.model.getDensityValue(self.label, self.iclass)
+            rho = self.model.getDensityValue(self.label, self.iset)
             self.lineEditIROPT.setText(str(rho))
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIJNBP(self, text):
         """
         Input IJNBP.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIJNBP.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, int)
-            self.model.setNumberOfParticulesInClassValue(self.label, self.iclass, value)
+            self.model.setNumberOfParticulesInSetValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIJFRE(self, text):
         """
         Input IJFRE.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIJFRE.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, int)
-            self.model.setInjectionFrequencyValue(self.label, self.iclass, value)
+            self.model.setInjectionFrequencyValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotICLST(self, text):
         """
         Input ICLST.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditICLST.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, int)
-            self.model.setParticleGroupNumberValue(self.label, self.iclass, value)
+            self.model.setParticleGroupNumberValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIDEBT(self, text):
         """
         Input IDEBT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIDEBT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setMassFlowRateValue(self.label, self.iclass, value)
+            self.model.setMassFlowRateValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIPOITChoice(self, text):
         """
         Input IPOIT.
         """
         choice = self.modelIPOIT.dicoV2M[str(text)]
-        self.model.setStatisticalWeightChoice(self.label, self.iclass, choice)
-        self.frameVolumicRate.hide()
+        self.model.setStatisticalWeightChoice(self.label, self.iset, choice)
+        self.frameMassRate.hide()
         self.frameStatisticalWeight.hide()
         if choice == "rate":
-            self.frameVolumicRate.show()
-            rate = self.model.getMassFlowRateValue(self.label, self.iclass)
+            self.frameMassRate.show()
+            rate = self.model.getMassFlowRateValue(self.label, self.iset)
             self.lineEditIDEBT.setText(str(rate))
-            self.model.setStatisticalWeightValue(self.label, self.iclass, 1)
+            self.model.setStatisticalWeightValue(self.label, self.iset, 1)
         elif choice == "prescribed":
             self.frameStatisticalWeight.show()
-            weight = self.model.getStatisticalWeightValue(self.label, self.iclass)
+            weight = self.model.getStatisticalWeightValue(self.label, self.iset)
             self.lineEditIPOIT.setText(str(weight))
-        elif choice == "subroutine":
-            pass
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIPOIT(self, text):
         """
         Input IPOIT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIPOIT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setStatisticalWeightValue(self.label, self.iclass, value)
+            self.model.setStatisticalWeightValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIROPT(self, text):
         """
         Input IROPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIROPT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setDensityValue(self.label, self.iclass, value)
+            self.model.setDensityValue(self.label, self.iset, value)
+
+    @pyqtSlot(str)
+    def slotIRCOLM(self, text):
+        """
+        Input IRCOLM.
+        """
+        if self.lineEditIRCOLM.validator().state == QValidator.Acceptable:
+            value = from_qvariant(text, float)
+            self.model.setFoulingIndexValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIJUVW(self, text):
         """
         Input IJUVW.
         """
         choice = self.modelIJUVW.dicoV2M[str(text)]
-        self.model.setVelocityChoice(self.label, self.iclass, choice)
+        self.model.setVelocityChoice(self.label, self.iset, choice)
         self.frameVelocityNorm.hide()
         self.frameVelocityValues.hide()
         if choice == "norm":
             self.frameVelocityNorm.show()
-            norm = self.model.getVelocityNormValue(self.label, self.iclass)
+            norm = self.model.getVelocityNormValue(self.label, self.iset)
             self.lineEditIUNO.setText(str(norm))
         elif choice == "components":
             self.frameVelocityValues.show()
-            vu = self.model.getVelocityDirectionValue(self.label, self.iclass, "u")
-            vv = self.model.getVelocityDirectionValue(self.label, self.iclass, "v")
-            vw = self.model.getVelocityDirectionValue(self.label, self.iclass, "w")
+            vu = self.model.getVelocityDirectionValue(self.label, self.iset, "x")
+            vv = self.model.getVelocityDirectionValue(self.label, self.iset, "y")
+            vw = self.model.getVelocityDirectionValue(self.label, self.iset, "z")
             self.lineEditIUPT.setText(str(vu))
             self.lineEditIVPT.setText(str(vv))
             self.lineEditIWPT.setText(str(vw))
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIUNO(self, text):
         """
         Input IUNO.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIUNO.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setVelocityNormValue(self.label, self.iclass, value)
+            self.model.setVelocityNormValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIUPT(self, text):
         """
         Input IUPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIUPT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setVelocityDirectionValue(self.label, self.iclass, "u", value)
+            self.model.setVelocityDirectionValue(self.label, self.iset, "x", value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIVPT(self, text):
         """
         Input IVPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIVPT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setVelocityDirectionValue(self.label, self.iclass, "v", value)
+            self.model.setVelocityDirectionValue(self.label, self.iset, "y", value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIWPT(self, text):
         """
         Input IWPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIWPT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setVelocityDirectionValue(self.label, self.iclass, "w", value)
+            self.model.setVelocityDirectionValue(self.label, self.iset, "z", value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIJRTP(self, text):
         """
         Input IJRTP.
         """
         choice = self.modelIJRTP.dicoV2M[str(text)]
-        self.model.setTemperatureChoice(self.label, self.iclass, choice)
+        self.model.setTemperatureChoice(self.label, self.iset, choice)
         if choice == "prescribed":
             self.frameTemperature.show()
-            temp = self.model.getTemperatureValue(self.label, self.iclass)
+            temp = self.model.getTemperatureValue(self.label, self.iset)
             self.lineEditITPT.setText(str(temp))
-        elif choice == "subroutine":
+        else:
             self.frameTemperature.hide()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotITPT(self, text):
         """
         Input ITPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditITPT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setTemperatureValue(self.label, self.iclass, value)
+            self.model.setTemperatureValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotICPT(self, text):
         """
         Input ICPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditICPT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setSpecificHeatValue(self.label, self.iclass, value)
+            self.model.setSpecificHeatValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIEPSI(self, text):
         """
         Input IEPSI.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIEPSI.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setEmissivityValue(self.label, self.iclass, value)
+            self.model.setEmissivityValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
-    def slotIJRDP(self, text):
-        """
-        Input IJRDP.
-        """
-        choice = self.modelIJRDP.dicoV2M[str(text)]
-        self.model.setDiameterChoice(self.label, self.iclass, choice)
-        if choice == "prescribed":
-            self.frameDiameter.show()
-            diam = self.model.getDiameterValue(self.label, self.iclass)
-            vdiam = self.model.getDiameterVarianceValue(self.label, self.iclass)
-            self.lineEditIDPT.setText(str(diam))
-            self.lineEditIVDPT.setText(str(vdiam))
-        elif choice == "subroutine":
-            self.frameDiameter.hide()
-
-
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIDPT(self, text):
         """
         Input IDPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIDPT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setDiameterValue(self.label, self.iclass, value)
+            self.model.setDiameterValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIVDPT(self, text):
         """
         Input IVDPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIVDPT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setDiameterVarianceValue(self.label, self.iclass, value)
+            self.model.setDiameterVarianceValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotINUCHL(self, text):
         """
         Input IHPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditINUCHL.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, int)
-            self.model.setCoalNumberValue(self.label, self.iclass, value)
+            self.model.setCoalNumberValue(self.label, self.iset, value)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIHPT(self, text):
         """
         Input IHPT.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditIHPT.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, float)
-            self.model.setCoalTemperatureValue(self.label, self.iclass, value)
-
-
-    @pyqtSignature("const QString&")
-    def slotIRAWCL(self, text):
-        """
-        Input IJRDP.
-        """
-        choice = self.modelIRAWCL.dicoV2M[str(text)]
-        self.model.setCoalCompositionChoice(self.label, self.iclass, choice)
+            self.model.setCoalTemperatureValue(self.label, self.iset, value)
 
 
     def tr(self, text):
