@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -37,17 +37,19 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import ComboModel, IntValidator, DoubleValidator
-from Pages.LagrangianOutputForm import Ui_LagrangianOutputForm
-from Pages.LagrangianOutputModel import LagrangianOutputModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import ComboModel, IntValidator, DoubleValidator
+from code_saturne.Pages.LagrangianOutputForm import Ui_LagrangianOutputForm
+from code_saturne.model.LagrangianOutputModel import LagrangianOutputModel
+from code_saturne.model.CoalCombustionModel import CoalCombustionModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -78,16 +80,16 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
         self.model = LagrangianOutputModel(self.case)
 
         # Connections
-        self.connect(self.checkBoxIVISV1,   SIGNAL("clicked()"),    self.slotIVISV1)
-        self.connect(self.checkBoxIVISV2,   SIGNAL("clicked()"),    self.slotIVISV2)
-        self.connect(self.checkBoxIVISTP,   SIGNAL("clicked()"),    self.slotIVISTP)
-        self.connect(self.checkBoxIVISDM,   SIGNAL("clicked()"),    self.slotIVISDM)
-        self.connect(self.checkBoxIVISTE,   SIGNAL("clicked()"),    self.slotIVISTE)
-        self.connect(self.checkBoxIVISMP,   SIGNAL("clicked()"),    self.slotIVISMP)
-        self.connect(self.checkBoxIVISDK,   SIGNAL("clicked()"),    self.slotIVISDK)
-        self.connect(self.checkBoxIVISCH,   SIGNAL("clicked()"),    self.slotIVISCH)
-        self.connect(self.checkBoxIVISCK,   SIGNAL("clicked()"),    self.slotIVISCK)
-        self.connect(self.checkBoxMoisture, SIGNAL("clicked()"),    self.slotMoisture)
+        self.checkBoxIVISV1.clicked.connect(self.slotIVISV1)
+        self.checkBoxIVISV2.clicked.connect(self.slotIVISV2)
+        self.checkBoxIVISTP.clicked.connect(self.slotIVISTP)
+        self.checkBoxIVISDM.clicked.connect(self.slotIVISDM)
+        self.checkBoxIVISTE.clicked.connect(self.slotIVISTE)
+        self.checkBoxIVISMP.clicked.connect(self.slotIVISMP)
+        self.checkBoxIVISDK.clicked.connect(self.slotIVISDK)
+        self.checkBoxIVISCH.clicked.connect(self.slotIVISCH)
+        self.checkBoxIVISCK.clicked.connect(self.slotIVISCK)
+        self.checkBoxMoisture.clicked.connect(self.slotMoisture)
 
         # initialize Widgets
         status = self.model.getFluidVelocityStatus()
@@ -126,36 +128,45 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
         else:
             self.checkBoxIVISMP.setChecked(False)
 
-        status = self.model.getCoalParticleDiameterStatus()
-        if status == "on":
-            self.checkBoxIVISDK.setChecked(True)
-        else:
-            self.checkBoxIVISDK.setChecked(False)
+        if CoalCombustionModel(self.case).getCoalCombustionModel("only") == \
+           "homogeneous_fuel_moisture":
+            status = self.model.getCoalParticleDiameterStatus()
+            if status == "on":
+                self.checkBoxIVISDK.setChecked(True)
+            else:
+                self.checkBoxIVISDK.setChecked(False)
 
-        status = self.model.getCoalParticleMassStatus()
-        if status == "on":
-            self.checkBoxIVISCH.setChecked(True)
-        else:
-            self.checkBoxIVISCH.setChecked(False)
+            status = self.model.getCoalParticleMassStatus()
+            if status == "on":
+                self.checkBoxIVISCH.setChecked(True)
+            else:
+                self.checkBoxIVISCH.setChecked(False)
 
-        status = self.model.getCokeParticleMassStatus()
-        if status == "on":
-            self.checkBoxIVISCK.setChecked(True)
-        else:
-            self.checkBoxIVISCK.setChecked(False)
+            status = self.model.getCokeParticleMassStatus()
+            if status == "on":
+                self.checkBoxIVISCK.setChecked(True)
+            else:
+                self.checkBoxIVISCK.setChecked(False)
 
-        self.case.undoStartGlobal()
-
-        status = self.model.getMoistureMassStatus()
-        if status == "on":
-            self.checkBoxMoisture.setChecked(True)
+            status = self.model.getMoistureMassStatus()
+            if status == "on":
+                self.checkBoxMoisture.setChecked(True)
+            else:
+                self.checkBoxMoisture.setChecked(False)
         else:
-            self.checkBoxMoisture.setChecked(False)
+            self.checkBoxIVISDK.hide()
+            self.labelIVISDK.hide()
+            self.checkBoxIVISCH.hide()
+            self.labelIVISCH.hide()
+            self.checkBoxIVISCK.hide()
+            self.labelIVISCK.hide()
+            self.checkBoxMoisture.hide()
+            self.labelIMoisture.hide()
 
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISV1(self):
         """
         Input IVISV1.
@@ -166,7 +177,7 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
             self.model.setFluidVelocityStatus("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISV2(self):
         """
         Input IVISV2.
@@ -177,7 +188,7 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
             self.model.setParticlesVelocityStatus("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISTP(self):
         """
         Input IVISTP.
@@ -188,7 +199,7 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
             self.model.setResidentTimeStatus("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISDM(self):
         """
         Input IVISDM.
@@ -199,7 +210,7 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
             self.model.setParticleDiameterStatus("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISTE(self):
         """
         Input IVISTE.
@@ -210,7 +221,7 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
             self.model.setParticleTemperatureStatus("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISMP(self):
         """
         Input IVISMP.
@@ -221,7 +232,7 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
             self.model.setParticleMassStatus("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISDK(self):
         """
         Input IVISDK.
@@ -232,7 +243,7 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
             self.model.setCoalParticleDiameterStatus("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISCH(self):
         """
         Input IVISCH.
@@ -243,7 +254,7 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
             self.model.setCoalParticleMassStatus("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIVISCK(self):
         """
         Input IVISCK.
@@ -254,7 +265,7 @@ class LagrangianOutputView(QWidget, Ui_LagrangianOutputForm):
             self.model.setCokeParticleMassStatus("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotMoisture(self):
         """
         Input IVISCK.
