@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -40,20 +40,21 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import ComboModel, IntValidator, DoubleValidator
-from Base.QtPage import to_qvariant, from_qvariant, to_text_string
-from Pages.LagrangianForm import Ui_LagrangianForm
-from Pages.LagrangianAdvancedOptionsDialogForm import Ui_LagrangianAdvancedOptionsDialogForm
-from Pages.LagrangianModel import LagrangianModel
-from Pages.StartRestartModel import StartRestartModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import ComboModel, IntValidator, DoubleValidator
+from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.Pages.LagrangianForm import Ui_LagrangianForm
+from code_saturne.Pages.LagrangianAdvancedOptionsDialogForm import Ui_LagrangianAdvancedOptionsDialogForm
+from code_saturne.model.LagrangianModel import LagrangianModel
+from code_saturne.model.CoalCombustionModel import CoalCombustionModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -98,12 +99,12 @@ class LagrangianAdvancedOptionsDialogView(QDialog, Ui_LagrangianAdvancedOptionsD
         self.modelIDIRLA.addItem(self.tr("Z"), "3")
 
         # Connections
-        self.connect(self.comboBoxNORDRE, SIGNAL("activated(const QString&)"),    self.slotNORDRE)
-        self.connect(self.checkBoxIDISTU, SIGNAL("clicked()"),                    self.slotIDISTU)
-        self.connect(self.checkBoxIDIFFL, SIGNAL("clicked()"),                    self.slotIDIFFL)
-        self.connect(self.groupBoxModel,  SIGNAL("clicked(bool)"),                self.slotModel)
-        self.connect(self.lineEditMODCPL, SIGNAL("textChanged(const QString &)"), self.slotMODCPL)
-        self.connect(self.comboBoxIDIRLA, SIGNAL("activated(const QString&)"),    self.slotIDIRLA)
+        self.comboBoxNORDRE.activated[str].connect(self.slotNORDRE)
+        self.checkBoxIDISTU.clicked.connect(self.slotIDISTU)
+        self.checkBoxIDIFFL.clicked.connect(self.slotIDIFFL)
+        self.groupBoxModel.clicked[bool].connect(self.slotModel)
+        self.lineEditMODCPL.textChanged[str].connect(self.slotMODCPL)
+        self.comboBoxIDIRLA.activated[str].connect(self.slotIDIRLA)
 
         validatorMODCPL = IntValidator(self.lineEditMODCPL, min=1)
         self.lineEditMODCPL.setValidator(validatorMODCPL)
@@ -134,7 +135,7 @@ class LagrangianAdvancedOptionsDialogView(QDialog, Ui_LagrangianAdvancedOptionsD
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotNORDRE(self, text):
         """
         Input NORDRE.
@@ -143,7 +144,7 @@ class LagrangianAdvancedOptionsDialogView(QDialog, Ui_LagrangianAdvancedOptionsD
         self.result['scheme_order'] = value
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIDISTU(self):
         """
         Input IDISTU.
@@ -155,7 +156,7 @@ class LagrangianAdvancedOptionsDialogView(QDialog, Ui_LagrangianAdvancedOptionsD
         self.result['turbulent_dispertion'] = status
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIDIFFL(self):
         """
         Input IDIFFL.
@@ -167,7 +168,7 @@ class LagrangianAdvancedOptionsDialogView(QDialog, Ui_LagrangianAdvancedOptionsD
         self.result['fluid_particles_turbulent_diffusion'] = status
 
 
-    @pyqtSignature("bool")
+    @pyqtSlot(bool)
     def slotModel(self, checked):
         if checked:
              value = self.default['complete_model_iteration']
@@ -179,16 +180,16 @@ class LagrangianAdvancedOptionsDialogView(QDialog, Ui_LagrangianAdvancedOptionsD
              self.result['complete_model_iteration'] = 0
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotMODCPL(self, text):
         """
         Input MODCPL.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditMODCPL.validator().state == QValidator.Acceptable:
             self.result['complete_model_iteration'] = from_qvariant(text, int)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIDIRLA(self, text):
         """
         Input IDIRLA.
@@ -249,6 +250,7 @@ class LabelDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         self.old_plabel = str(value)
         editor.setText(value)
@@ -275,6 +277,7 @@ class ValueDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(value)
 
@@ -311,8 +314,7 @@ class StandardItemModelCoals(QStandardItemModel):
         self.case = case
         self.model = model
 
-        import Pages.CoalCombustionModel as CoalCombustionModel
-        self.coalModel = CoalCombustionModel.CoalCombustionModel(self.case)
+        self.coalModel = CoalCombustionModel(self.case)
         CoalsNumber = self.coalModel.getCoalNumber()
 
         for icoal in range(CoalsNumber):
@@ -326,8 +328,6 @@ class StandardItemModelCoals(QStandardItemModel):
             row = self.rowCount()
             self.setRowCount(row+1)
 
-        del CoalCombustionModel
-
 
     def data(self, index, role):
         if not index.isValid():
@@ -375,7 +375,7 @@ class StandardItemModelCoals(QStandardItemModel):
         elif col == 4:
             self.model.setCoef2OfFouling(row+1, val)
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -401,58 +401,34 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
         self.model = LagrangianModel(self.case)
 
         # Combo model
-        self.modelIILAGR = ComboModel(self.comboBoxIILAGR,3,1)
-        self.modelIILAGR.addItem(self.tr("One-way coupling"), "one_way")
-        self.modelIILAGR.addItem(self.tr("Two-way coupling"), "two_way")
-        self.modelIILAGR.addItem(self.tr("Frozen carrier flow"), "frozen")
-
         self.modelIPHYLA = ComboModel(self.comboBoxIPHYLA,2,1)
         self.modelIPHYLA.addItem(self.tr("No model"), 'off')
         self.modelIPHYLA.addItem(self.tr("Heat transfer and evaporation"), 'thermal')
-        self.modelIPHYLA.addItem(self.tr("Pulverised coal model"), 'coal')
+        if CoalCombustionModel(self.case).getCoalCombustionModel("only") != 'off':
+            self.modelIPHYLA.addItem(self.tr("Pulverised coal model"), 'coal')
 
         # Connections
-        self.connect(self.comboBoxIILAGR, SIGNAL("activated(const QString&)"), self.slotIILAGR)
-        self.connect(self.checkBoxISUILA, SIGNAL("clicked()"), self.slotISUILA)
-        self.connect(self.checkBoxISTTIO, SIGNAL("clicked()"), self.slotISTTIO)
-        self.connect(self.lineEditNBPMAX, SIGNAL("textChanged(const QString &)"), self.slotNBPMAX)
-        self.connect(self.checkBoxINJCON, SIGNAL("clicked()"), self.slotINJCON)
-        self.connect(self.checkBoxIDEPST, SIGNAL("clicked()"), self.slotIDEPST)
-        self.connect(self.comboBoxIPHYLA, SIGNAL("activated(const QString&)"), self.slotIPHYLA)
-        self.connect(self.checkBoxITPVAR, SIGNAL("clicked()"), self.slotITPVAR)
-        self.connect(self.checkBoxIMPVAR, SIGNAL("clicked()"), self.slotIMPVAR)
-        self.connect(self.checkBoxIENCRA, SIGNAL("clicked()"), self.slotIENCRA)
+        self.checkBoxISUILA.clicked.connect(self.slotISUILA)
+        self.checkBoxISTTIO.clicked.connect(self.slotISTTIO)
+        self.checkBoxIDEPST.clicked.connect(self.slotIDEPST)
+        self.comboBoxIPHYLA.activated[str].connect(self.slotIPHYLA)
+        self.checkBoxITPVAR.clicked.connect(self.slotITPVAR)
+        self.checkBoxIMPVAR.clicked.connect(self.slotIMPVAR)
+        self.checkBoxIENCRA.clicked.connect(self.slotIENCRA)
         #
-        self.connect(self.lineEditNSTITS, SIGNAL("textChanged(const QString &)"), self.slotNSTITS)
-        self.connect(self.checkBoxLTSDYN, SIGNAL("clicked()"), self.slotLTSDYN)
-        self.connect(self.checkBoxLTSMAS, SIGNAL("clicked()"), self.slotLTSMAS)
-        self.connect(self.checkBoxLTSTHE, SIGNAL("clicked()"), self.slotLTSTHE)
-        self.connect(self.toolButtonAdvanced, SIGNAL("clicked()"), self.slotAdvancedOptions)
+        self.lineEditNSTITS.textChanged[str].connect(self.slotNSTITS)
+        self.checkBoxLTSDYN.clicked.connect(self.slotLTSDYN)
+        self.checkBoxLTSMAS.clicked.connect(self.slotLTSMAS)
+        self.checkBoxLTSTHE.clicked.connect(self.slotLTSTHE)
+        self.toolButtonAdvanced.clicked.connect(self.slotAdvancedOptions)
 
         # Validators
-        validatorNBPMAX = IntValidator(self.lineEditNBPMAX, min=0)
         validatorNSTITS = IntValidator(self.lineEditNSTITS)
 
-        self.lineEditNBPMAX.setValidator(validatorNBPMAX)
         self.lineEditNSTITS.setValidator(validatorNSTITS)
 
         # initialize Widgets
-
-#        # Update the lagrangian list with the calculation features
-#
-#        for mdl in self.model.lagrangianCouplingMode():
-#            if mdl not in self.model.lagrangianStatus():
-#                self.modelIILAGR.disableItem(str_model=mdl)
-
-        from Pages.CoalCombustionModel import CoalCombustionModel
-        if CoalCombustionModel(self.case).getCoalCombustionModel() != 'off':
-            self.modelIILAGR.disableItem(str_model="one_way")
-            self.modelIILAGR.disableItem(str_model="two_way")
-        del CoalCombustionModel
-
-        model = self.model.getCouplingMode()
-        self.modelIILAGR.setItem(str_model=model)
-        self.slotIILAGR(self.modelIILAGR.dicoM2V[model])
+        model = self.model.getLagrangianModel()
 
         status = self.model.getRestart()
         if status == "on":
@@ -466,12 +442,6 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
         else:
             self.checkBoxISTTIO.setChecked(False)
 
-        status = self.model.getContinuousInjection()
-        if status == "on":
-            self.checkBoxINJCON.setChecked(True)
-        else:
-            self.checkBoxINJCON.setChecked(False)
-
         status = self.model.getDepositionSubmodel()
         if status == "on":
             self.checkBoxIDEPST.setChecked(True)
@@ -483,30 +453,6 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.checkBoxISTTIO.setChecked(True)
             self.checkBoxISTTIO.setDisabled(True)
 
-        nmax = self.model.getMaxNumber()
-        self.lineEditNBPMAX.setText(str(nmax))
-
-        status = self.model.getContinuousInjection()
-        if status == "on":
-            self.checkBoxINJCON.setChecked(True)
-        else:
-            self.checkBoxINJCON.setChecked(False)
-
-        part_model = self.model.getParticlesModel()
-        self.modelIPHYLA.setItem(str_model=part_model)
-        self.slotIPHYLA(self.modelIPHYLA.dicoM2V[part_model])
-
-        self.case.undoStartGlobal()
-
-
-    @pyqtSignature("const QString&")
-    def slotIILAGR(self, text):
-        """
-        Input IILAGR.
-        """
-        model = self.modelIILAGR.dicoV2M[str(text)]
-        self.model.setCouplingMode(model)
-
         self.groupBox2way.hide()
 
         self.labelISTTIO.setDisabled(False)
@@ -543,8 +489,14 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.labelISTTIO.setDisabled(True)
             self.checkBoxISTTIO.setDisabled(True)
 
+        part_model = self.model.getParticlesModel()
+        self.modelIPHYLA.setItem(str_model=part_model)
+        self.slotIPHYLA(self.modelIPHYLA.dicoM2V[part_model])
 
-    @pyqtSignature("")
+        self.case.undoStartGlobal()
+
+
+    @pyqtSlot()
     def slotISUILA(self):
         """
         Input ISUILA.
@@ -555,7 +507,7 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.model.setRestart("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotISTTIO(self):
         """
         Input ISTTIO.
@@ -566,18 +518,7 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.model.setCarrierFlowStationary("off")
 
 
-    @pyqtSignature("")
-    def slotINJCON(self):
-        """
-        Input INJCON.
-        """
-        if self.checkBoxINJCON.isChecked():
-            self.model.setContinuousInjection("on")
-        else:
-            self.model.setContinuousInjection("off")
-
-
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIDEPST(self):
         """
         Input IDEPST.
@@ -588,17 +529,7 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.model.setDepositionSubmodel("off")
 
 
-    @pyqtSignature("const QString&")
-    def slotNBPMAX(self, text):
-        """
-        Input NBPMAX.
-        """
-        if self.sender().validator().state == QValidator.Acceptable:
-            value = from_qvariant(text, int)
-            self.model.setMaxNumber(value)
-
-
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotIPHYLA(self, text):
         """
         Input IPHYLA.
@@ -644,7 +575,7 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.slotIENCRA()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotITPVAR(self):
         """
         Input ITPVAR.
@@ -655,7 +586,7 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.model.setHeating("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIMPVAR(self):
         """
         Input IMPVAR.
@@ -666,7 +597,7 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.model.setEvaporation("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotIENCRA(self):
         """
         Input IENCRA.
@@ -685,9 +616,14 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.tableViewCoals.setItemDelegateForColumn(3, delegateValue2)
             self.tableViewCoals.setItemDelegateForColumn(4, delegateValue2)
             self.tableViewCoals.show()
-            self.tableViewCoals.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
-            self.tableViewCoals.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
-            self.tableViewCoals.horizontalHeader().setResizeMode(0, QHeaderView.Stretch)
+            if QT_API == "PYQT4":
+                self.tableViewCoals.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
+                self.tableViewCoals.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
+                self.tableViewCoals.horizontalHeader().setResizeMode(0, QHeaderView.Stretch)
+            elif QT_API == "PYQT5":
+                self.tableViewCoals.verticalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+                self.tableViewCoals.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+                self.tableViewCoals.horizontalHeader().setSectionResizeMode(0, QHeaderView.Stretch)
         else:
             self.model.setCoalFouling("off")
             if hasattr(self, "modelCoals"):
@@ -695,17 +631,17 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.tableViewCoals.hide()
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotNSTITS(self, text):
         """
         Input NSTITS.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditNSTITS.validator().state == QValidator.Acceptable:
             value = from_qvariant(text, int)
             self.model.set2WayCouplingStartIteration(value)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotLTSDYN(self):
         """
         Input LTSDYN.
@@ -716,7 +652,7 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.model.set2WayCouplingDynamic("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotLTSMAS(self):
         """
         Input LTSMAS.
@@ -727,7 +663,7 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.model.set2WayCouplingMass("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotLTSTHE(self):
         """
         Input LTSTHE.
@@ -738,7 +674,7 @@ class LagrangianView(QWidget, Ui_LagrangianForm):
             self.model.set2WayCouplingTemperature("off")
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotAdvancedOptions(self):
         """
         Ask one popup for advanced specifications
