@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -24,6 +24,7 @@
 
 """
 This module contains the following classes:
+- VolumicZoneAdvancedView
 - LabelDelegate
 - CodeNumberDelegate
 - BoundaryNatureDelegate
@@ -41,26 +42,28 @@ This module contains the following classes:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import string, logging
+import logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.Common import LABEL_LENGTH_MAX
-from Base.QtPage import IntValidator, RegExpValidator
-from Base.QtPage import to_qvariant, from_qvariant, to_text_string
-from Pages.LocalizationForm import Ui_LocalizationForm
-from Pages.PreProcessingInformationsView import Informations, preprocessorFile
-from Pages.LocalizationModel import LocalizationModel, Zone
+from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
+from code_saturne.Base.QtPage import IntValidator, RegExpValidator
+from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.Pages.LocalizationForm import Ui_LocalizationForm
+from code_saturne.Pages.VolumicZoneAdvancedDialogForm import Ui_VolumicZoneAdvancedDialogForm
+from code_saturne.Pages.PreProcessingInformationsView import Informations, preprocessorFile
+from code_saturne.model.LocalizationModel import LocalizationModel, Zone
+from code_saturne.model.OutputControlModel import OutputControlModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -70,6 +73,181 @@ logging.basicConfig()
 log = logging.getLogger("LocalizationView")
 log.setLevel(GuiParam.DEBUG)
 
+#-------------------------------------------------------------------------------
+# Advanced dialog
+#-------------------------------------------------------------------------------
+
+class VolumicZoneAdvancedView(QDialog, Ui_VolumicZoneAdvancedDialogForm):
+    """
+    Advanced dialog
+    """
+    def __init__(self, parent, case, keys, default):
+        """
+        Constructor
+        """
+        QDialog.__init__(self, parent)
+
+        Ui_VolumicZoneAdvancedDialogForm.__init__(self)
+        self.setupUi(self)
+
+        self.case = case
+        self.case.undoStopGlobal()
+
+        self.default = default
+        self.keys    = keys
+        self.result  = self.default.copy()
+
+        if 'initialization' not in self.keys:
+            self.labelInitialization.hide()
+            self.checkBoxInitialization.hide()
+        else:
+            if self.default['initialization'] == 'on':
+                self.checkBoxInitialization.setChecked(True)
+            else:
+                self.checkBoxInitialization.setChecked(False)
+
+        if 'head_losses' not in self.keys:
+            self.labelHeadLosses.hide()
+            self.checkBoxHeadLosses.hide()
+        else:
+            if self.default['head_losses'] == 'on':
+                self.checkBoxHeadLosses.setChecked(True)
+            else:
+                self.checkBoxHeadLosses.setChecked(False)
+
+        if 'porosity' not in self.keys:
+            self.labelPorosity.hide()
+            self.checkBoxPorosity.hide()
+        else:
+            if self.default['porosity'] == 'on':
+                self.checkBoxPorosity.setChecked(True)
+            else:
+                self.checkBoxPorosity.setChecked(False)
+
+        if 'momentum_source_term' not in self.keys:
+            self.labelMomentum.hide()
+            self.checkBoxMomentum.hide()
+        else:
+            if self.default['momentum_source_term'] == 'on':
+                self.checkBoxMomentum.setChecked(True)
+            else:
+                self.checkBoxMomentum.setChecked(False)
+
+        if 'mass_source_term' not in self.keys:
+            self.labelMass.hide()
+            self.checkBoxMass.hide()
+        else:
+            if self.default['mass_source_term'] == 'on':
+                self.checkBoxMass.setChecked(True)
+            else:
+                self.checkBoxMass.setChecked(False)
+
+        if 'thermal_source_term' not in self.keys:
+            self.labelThermal.hide()
+            self.checkBoxThermal.hide()
+        else:
+            if self.default['thermal_source_term'] == 'on':
+                self.checkBoxThermal.setChecked(True)
+            else:
+                self.checkBoxThermal.setChecked(False)
+
+        if 'scalar_source_term' not in self.keys:
+            self.labelScalar.hide()
+            self.checkBoxScalar.hide()
+        else:
+            if self.default['scalar_source_term'] == 'on':
+                self.checkBoxScalar.setChecked(True)
+            else:
+                self.checkBoxScalar.setChecked(False)
+
+        if 'groundwater_law' not in self.keys:
+            self.labelGroundWater.hide()
+            self.checkBoxGroundWater.hide()
+        else:
+            if self.default['groundwater_law'] == 'on':
+                self.checkBoxGroundWater.setChecked(True)
+            else:
+                self.checkBoxGroundWater.setChecked(False)
+
+        self.case.undoStartGlobal()
+
+
+    def get_result(self):
+        """
+        Method to get the result
+        """
+        return self.result
+
+
+    def accept(self):
+        """
+        Method called when user clicks 'OK'
+        """
+        if 'initialization' in self.keys:
+            if self.checkBoxInitialization.isChecked():
+                self.result['initialization'] = 'on'
+            else:
+                self.result['initialization'] = 'off'
+
+        if 'head_losses' in self.keys:
+            if self.checkBoxHeadLosses.isChecked():
+                self.result['head_losses'] = 'on'
+            else:
+                self.result['head_losses'] = 'off'
+
+        if 'porosity' in self.keys:
+            if self.checkBoxPorosity.isChecked():
+                self.result['porosity'] = 'on'
+            else:
+                self.result['porosity'] = 'off'
+
+        if 'momentum_source_term' in self.keys:
+            if self.checkBoxMomentum.isChecked():
+                self.result['momentum_source_term'] = 'on'
+            else:
+                self.result['momentum_source_term'] = 'off'
+
+        if 'mass_source_term' in self.keys:
+            if self.checkBoxMass.isChecked():
+                self.result['mass_source_term'] = 'on'
+            else:
+                self.result['mass_source_term'] = 'off'
+
+        if 'thermal_source_term' in self.keys:
+            if self.checkBoxThermal.isChecked():
+                self.result['thermal_source_term'] = 'on'
+            else:
+                self.result['thermal_source_term'] = 'off'
+
+        if 'scalar_source_term' in self.keys:
+            if self.checkBoxScalar.isChecked():
+                self.result['scalar_source_term'] = 'on'
+            else:
+                self.result['scalar_source_term'] = 'off'
+
+        if 'groundwater_law' in self.keys:
+            if self.checkBoxGroundWater.isChecked():
+                self.result['groundwater_law'] = 'on'
+            else:
+                self.result['groundwater_law'] = 'off'
+
+        QDialog.accept(self)
+
+
+    def reject(self):
+        """
+        Method called when user clicks 'Cancel'
+        """
+        QDialog.reject(self)
+
+
+    def tr(self, text):
+        """
+        Translation
+        """
+        return text
+
+
 #-------------------------------------------------------------------------------
 # Line edit delegate for the label
 #-------------------------------------------------------------------------------
@@ -94,6 +272,7 @@ class LabelDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         v = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         self.p_value = str(v)
         editor.setText(v)
@@ -113,7 +292,7 @@ class LabelDelegate(QItemDelegate):
                 default['regexp'] = self.regExp
                 log.debug("setModelData-> default = %s" % default)
 
-                from Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
+                from code_saturne.Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
                 dialog = VerifyExistenceLabelDialogView(self.parent, default)
                 if dialog.exec_():
                     result = dialog.get_result()
@@ -143,6 +322,7 @@ class CodeNumberDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         v = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(self.value)
 
@@ -184,7 +364,7 @@ class BoundaryNatureDelegate(QItemDelegate):
         for k in list(self.dicoV2M.keys()):
             editor.addItem(k)
         editor.installEventFilter(self)
-        editor.setMinimumWidth(100)
+        editor.setMinimumWidth(120)
         return editor
 
 
@@ -294,7 +474,7 @@ class StandardItemVolumeNature(QStandardItemModel):
 
         id1 = self.index(0, 0)
         id2 = self.index(self.rowCount(), 0)
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), id1, id2)
+        self.dataChanged.emit(id1, id2)
         return True
 
 
@@ -303,7 +483,7 @@ class StandardItemVolumeNature(QStandardItemModel):
         for k, v in list(self.dicoNature.items()):
             if v == "on":
                 s.append(k)
-        return string.join(s, ";")
+        return ";".join(s)
 
 #-------------------------------------------------------------------------------
 # FlagBox: new QComboBox to construct Delegate for the volume nature
@@ -319,10 +499,10 @@ class FlagBox(QComboBox):
         if (self.style().styleHint(QStyle.SH_ComboBox_Popup, opt)):
             self.setItemDelegate(QItemDelegate(self))
 
-        self.connect(self, SIGNAL("activated(int)"), self.slotActivated)
+        self.activated[int].connect(self.slotActivated)
 
 
-    @pyqtSignature("int")
+    @pyqtSlot(int)
     def slotActivated(self, index):
         value = self.itemData(index, Qt.CheckStateRole)
         state = from_qvariant(value, int)
@@ -355,6 +535,7 @@ class VolumeNatureDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         data = index.model().getData(index.row(), index.column())
         dicoM2V = index.model().dicoM2V
 
@@ -378,11 +559,37 @@ class LocalizationSelectorDelegate(QItemDelegate):
 
 
     def createEditor(self, parent, option, index):
+
         editor = QLineEdit(parent)
+
+        # Autocompletion for selection criteria!
+        comp_list = ['all[]']
+        if self.mdl._typeZone == 'BoundaryZone':
+            comp_list += ['normal[x, y, z, epsilon]']
+        comp_list += ['plane[a, b, c, d, epsilon]',
+                      'plane[a, b, c, d, inside]',
+                      'plane[a, b, c, d, outside]',
+                      'plane[n_x, n_y, n_z, x0, y0, z0, epsilon]',
+                      'plane[n_x, n_y, n_z, x0, y0, z0, inside]',
+                      'plane[n_x, n_y, n_z, x0, y0, z0, outside]',
+                      'box[xmin, ymin, zmin, xmax, ymax, zmax]',
+                      'box[x0, y0, z0, dx1, dy1, dz1, dx2, dy2, dz2, dx3, dy3, dz3]',
+                      'cylinder[x0, y0, z0, x1, y1, z1, radius]',
+                      'sphere[x_c, y_c, z_c, radius]']
+
+        completer = QCompleter()
+        editor.setCompleter(completer)
+        model = QStringListModel()
+        completer.setModel(model)
+        model.setStringList(comp_list)
+
         return editor
 
 
     def setEditorData(self, editor, index):
+        # This line is used to avoid an overlay of old and new text
+        editor.setAutoFillBackground(True)
+
         self.value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(self.value)
 
@@ -439,7 +646,7 @@ class StandardItemModelLocalization(QStandardItemModel):
             elif col == 2:
                 if self.zoneType == "VolumicZone":
                     data = self._data[row][col]
-                    item = string.join([self.dicoM2V[key] for key in list(self.dicoM2V.keys()) if data[key] == "on"], "\n")
+                    item = "\n".join([self.dicoM2V[key] for key in list(self.dicoM2V.keys()) if data[key] == "on"])
 
                     return to_qvariant(item)
 
@@ -521,7 +728,7 @@ class StandardItemModelLocalization(QStandardItemModel):
 
         self.mdl.replaceZone(old_zone, new_zone)
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         self.browser.configureTree(self.case)
         return True
 
@@ -541,13 +748,15 @@ class StandardItemModelLocalization(QStandardItemModel):
         row = self.rowCount()
         self.setRowCount(row+1)
 
-        # Warning: the Volume region 'all_cells' is mandatory, and can not be modified.
+        # Warning: the Volume region 'all_cells' is mandatory, and can not be removed.
         if self.zoneType == "VolumicZone":
             if zone.getLabel() == "all_cells":
-                for c in range(self.columnCount()):
+                for c in [0, 3]:
                     self._disable.append((row, c))
+            # self._disable.append((row, 2))
         self._disable.append((row, 1))
         self.browser.configureTree(self.case)
+        return zone
 
 
     def getItem(self, row):
@@ -560,11 +769,18 @@ class StandardItemModelLocalization(QStandardItemModel):
             self._data[id][1] = id + 1
 
 
+    def updateZone(self, index, nature):
+        self._data[index.row()][2] = nature
+        self.dataChanged.emit(index, index)
+
+
     def deleteItem(self, irow):
         del self._data[irow]
         nb_rows = self.rowCount()
         self.setRowCount(nb_rows-1)
         self.updateItem()
+        if irow < nb_rows:
+            self.browser.configureTree(self.case)
 
 
     def deleteItems(self):
@@ -597,7 +813,10 @@ class LocalizationView(QWidget, Ui_LocalizationForm):
         self.case.undoStopGlobal()
 
         self.zoneType = zoneType
+
         self.mdl = LocalizationModel(zoneType, case)
+        self.case['dump_python'].append([self.mdl.__module__, zoneType, ()])
+
         self.browser = tree
 
         # Model for table View
@@ -617,37 +836,123 @@ class LocalizationView(QWidget, Ui_LocalizationForm):
         for zone in self.mdl.getZones():
             self.modelLocalization.addItem(zone)
 
-        self.tableView.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
-        self.tableView.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
-        self.tableView.horizontalHeader().setResizeMode(3, QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableView.verticalHeader().setResizeMode(QHeaderView.ResizeToContents)
+            self.tableView.horizontalHeader().setResizeMode(QHeaderView.ResizeToContents)
+            self.tableView.horizontalHeader().setResizeMode(3, QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableView.verticalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+            self.tableView.horizontalHeader().setSectionResizeMode(QHeaderView.ResizeToContents)
+            self.tableView.horizontalHeader().setSectionResizeMode(3, QHeaderView.Stretch)
 
         # Connections
-        self.connect(self.pushButtonNew,      SIGNAL("clicked()"), self.slotAddZone)
-        self.connect(self.pushButtonDelete,   SIGNAL("clicked()"), self.slotDeleteZone)
-        self.connect(self.toolButtonCreation, SIGNAL("clicked()"), self.slotAddFromPrePro)
-        self.connect(self.modelLocalization,  SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), self.dataChanged)
+        self.pushButtonNew.clicked.connect(self.slotAddZone)
+        self.pushButtonDelete.clicked.connect(self.slotDeleteZone)
+        self.pushButtonModify.clicked.connect(self.slotModifyZone)
+        self.toolButtonCreation.clicked.connect(self.slotAddFromPrePro)
+        self.modelLocalization.dataChanged.connect(self.dataChanged)
+        self.tableView.clicked.connect(self.slotChangeSelection)
 
         self.pushButtonSalome.hide()
+        self.pushButtonModify.setEnabled(False)
         if case['salome']:
             self.pushButtonSalome.show()
-            self.connect(self.pushButtonSalome, SIGNAL("clicked()"), self.slotAddFromSalome)
+            self.pushButtonSalome.clicked.connect(self.slotAddFromSalome)
 
         # Context menu
         self.tableView.setContextMenuPolicy(Qt.CustomContextMenu)
-        self.connect(self.tableView, SIGNAL("customContextMenuRequested(QPoint)"), self.slotContextMenu)
+        self.tableView.customContextMenuRequested[QPoint].connect(self.slotContextMenu)
 
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("")
+    def slotChangeSelection(self):
+        """
+        """
+        self.pushButtonModify.setEnabled(False)
+        current = self.tableView.currentIndex()
+        if current != self.tableView.rootIndex():
+            self.pushButtonModify.setEnabled(True)
+
+
+    @pyqtSlot()
     def slotAddZone(self):
         """
         Insert a new item in the table view.
         """
-        self.modelLocalization.addItem()
+        zone = self.modelLocalization.addItem()
+
+        if self.zoneType == 'VolumicZone':
+            label  = zone.getLabel()
+            code   = zone.getCodeNumber()
+            nature = zone.getNature()
+            local  = zone.getLocalization()
+
+            self.keys = Zone('VolumicZone', case = self.case).getNatureList()
+
+            log.debug("slotAddZone -> %s" % str(nature))
+
+            dialog = VolumicZoneAdvancedView(self, self.case, self.keys, nature)
+
+            if dialog.exec_():
+                result = dialog.get_result()
+                log.debug("slotAddZone -> %s" % str(result))
+
+                new_zone = Zone(self.zoneType,
+                                case         = self.case,
+                                label        = label,
+                                codeNumber   = code,
+                                localization = local,
+                                nature       = result)
+
+                self.mdl.replaceZone(zone, new_zone)
+                index = self.tableView.model().index(code - 1, 2)
+                self.modelLocalization.updateZone(index, result)
+            self.browser.configureTree(self.case)
+        self.slotChangeSelection()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
+    def slotModifyZone(self):
+        """
+        Modify volumic zone nature
+        """
+        index = self.tableView.selectionModel().selectedRows()[0]
+
+        [label, code, nature, local] = self.modelLocalization.getItem(index.row())
+
+        self.keys = Zone('VolumicZone', case = self.case).getNatureList()
+
+        zone = Zone(self.zoneType,
+                    case         = self.case,
+                    label        = label,
+                    codeNumber   = code,
+                    localization = local,
+                    nature       = nature)
+
+        log.debug("slotAddZone -> %s" % str(nature))
+
+        dialog = VolumicZoneAdvancedView(self, self.case, self.keys, nature)
+
+        if dialog.exec_():
+            result = dialog.get_result()
+            log.debug("slotAddZone -> %s" % str(result))
+
+            new_zone = Zone(self.zoneType,
+                            case         = self.case,
+                            label        = label,
+                            codeNumber   = code,
+                            localization = local,
+                            nature       = result)
+
+            self.mdl.replaceZone(zone, new_zone)
+            index = self.tableView.model().index(code - 1, 2)
+            self.modelLocalization.updateZone(index, result)
+        self.browser.configureTree(self.case)
+        self.slotChangeSelection()
+
+
+    @pyqtSlot()
     def slotDeleteZone(self):
         """
         Private Slot.
@@ -664,14 +969,20 @@ class LocalizationView(QWidget, Ui_LocalizationForm):
         for row in lst:
             [label, codeNumber, nature, localization] = self.modelLocalization.getItem(row)
             if not (label == "all_cells" and self.zoneType == 'VolumicZone'):
+                # We also need to delete postprocessing meshes based on the zone
+                OutputControlModel(self.case).deleteZone(label, self.zoneType)
+                # Delete the zone itself
                 self.mdl.deleteZone(label)
                 self.modelLocalization.deleteItem(row)
+        self.slotChangeSelection()
+        for index in self.tableView.selectionModel().selectedRows():
+            self.modelLocalization.dataChanged.emit(index, index)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotAddFromPrePro(self):
         """
-        Research a Preprocessor listing to pick colors or groups of cells or faces.
+        Research a preprocessor log to pick colors or groups of cells or faces.
         """
         if self.zoneType == 'VolumicZone':
             entity = 'cells'
@@ -688,7 +999,7 @@ class LocalizationView(QWidget, Ui_LocalizationForm):
                     self.modelLocalization.addItem(zone)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotContextMenu(self):
         """
         Public slot
@@ -698,33 +1009,33 @@ class LocalizationView(QWidget, Ui_LocalizationForm):
         fileMenu = QMenu(self.tableView)
 
         actionMerge = QAction(self.tr("Merge selected zones"), self.tableView)
-        self.connect(actionMerge, SIGNAL("triggered()"), self.slotMerge)
+        actionMerge.triggered.connect(self.slotMerge)
         fileMenu.addAction(actionMerge)
 
         if self.zoneType == 'BoundaryZone':
             fileMenu.addSeparator()
 
             self.actionInlet = QAction(self.tr("Select all inlets"), self.tableView)
-            self.connect(self.actionInlet, SIGNAL("triggered()"), self.slotSelectBoudaries)
+            self.actionInlet.triggered.connect(self.slotSelectBoudaries)
             fileMenu.addAction(self.actionInlet)
 
             self.actionOutlet = QAction(self.tr("Select all outlets"), self.tableView)
-            self.connect(self.actionOutlet, SIGNAL("triggered()"), self.slotSelectBoudaries)
+            self.actionOutlet.triggered.connect(self.slotSelectBoudaries)
             fileMenu.addAction(self.actionOutlet)
 
             self.actionWall = QAction(self.tr("Select all walls"), self.tableView)
-            self.connect(self.actionWall, SIGNAL("triggered()"), self.slotSelectBoudaries)
+            self.actionWall.triggered.connect(self.slotSelectBoudaries)
             fileMenu.addAction(self.actionWall)
 
-            self.actionSymmetry = QAction(self.tr("Select all symmeties"), self.tableView)
-            self.connect(self.actionSymmetry, SIGNAL("triggered()"), self.slotSelectBoudaries)
+            self.actionSymmetry = QAction(self.tr("Select all symmetries"), self.tableView)
+            self.actionSymmetry.triggered.connect(self.slotSelectBoudaries)
             fileMenu.addAction(self.actionSymmetry)
 
         fileMenu.popup(QCursor().pos())
         fileMenu.show()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotMerge(self):
         """
         public slot
@@ -752,14 +1063,14 @@ class LocalizationView(QWidget, Ui_LocalizationForm):
             self.modelLocalization.addItem(zone)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotAddFromSalome(self):
         """
         When GUI is embeded in the Salome desktop. Add selection criteria from
         graphical selection in the VTK viwver, or in the ObjectBrowser.
         """
         if self.case['salome']:
-            from Pages.SalomeHandler import BoundaryGroup, VolumeGroup
+            from code_saturne.Pages.SalomeHandler import BoundaryGroup, VolumeGroup
 
             log.debug("slotAddFromSalome: zoneType -> %s" % self.zoneType)
             if self.zoneType == 'VolumicZone':
@@ -768,13 +1079,12 @@ class LocalizationView(QWidget, Ui_LocalizationForm):
                 loc = BoundaryGroup()
 
             log.debug("slotAddFromSalome: selection criteria -> %s" % loc)
-            if loc not in self.mdl.getLocalizationsZonesList():
+            if loc not in self.mdl.getLocalizationsZonesList() and loc != "" :
                 zone = Zone(self.zoneType, localization = loc, case = self.case)
                 self.mdl.addZone(zone)
                 self.modelLocalization.addItem(zone)
 
 
-    @pyqtSignature("const QModelIndex &, const QModelIndex &")
     def dataChanged(self, topLeft, bottomRight):
         for row in range(topLeft.row(), bottomRight.row()+1):
             self.tableView.resizeRowToContents(row)
@@ -827,20 +1137,22 @@ class BoundaryLocalizationView(LocalizationView):
         Constructor.
         """
         self.case = case
-        dicoM2V = Zone("BoundaryZone").getModel2ViewDictionary()
+        dicoM2V = Zone("BoundaryZone", case = self.case).getModel2ViewDictionary()
         self.browser = tree
 
         LocalizationView.__init__(self, "BoundaryZone", parent, self.case, dicoM2V, tree)
 
-        title = self.tr("Definition of boundary regions")
+        title = self.tr("Boundary regions definition")
         self.groupBoxLocalization.setTitle(title)
 
         # Delegates
         delegateNature = BoundaryNatureDelegate(self.tableView, dicoM2V)
         self.tableView.setItemDelegateForColumn(2, delegateNature)
 
+        self.pushButtonModify.hide()
 
-    @pyqtSignature("")
+
+    @pyqtSlot()
     def slotSelectBoudaries(self):
         """
         Public slot.
