@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -39,20 +39,20 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import DoubleValidator, IntValidator, to_qvariant
-from Base.QtPage import from_qvariant, to_text_string
-from Pages.NumericalParamEquationForm import Ui_NumericalParamEquationForm
-from Pages.NumericalParamEquationModel import NumericalParamEquatModel
-from Pages.TurbulenceModel import TurbulenceModel
-from Pages.SteadyManagementModel import SteadyManagementModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import DoubleValidator, IntValidator, to_qvariant
+from code_saturne.Base.QtPage import from_qvariant, to_text_string
+from code_saturne.Pages.NumericalParamEquationForm import Ui_NumericalParamEquationForm
+from code_saturne.model.NumericalParamEquationModel import NumericalParamEquationModel
+from code_saturne.model.TurbulenceModel import TurbulenceModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -78,6 +78,7 @@ class SchemeOrderDelegate(QItemDelegate):
 
     def createEditor(self, parent, option, index):
         editor = QComboBox(parent)
+        editor.addItem("Automatic")
         editor.addItem("Upwind")
         editor.addItem("Centered")
         editor.addItem("SOLU")
@@ -86,7 +87,7 @@ class SchemeOrderDelegate(QItemDelegate):
 
 
     def setEditorData(self, comboBox, index):
-        dico = {"upwind": 0, "centered": 1, "solu": 2}
+        dico = {"automatic": 0, "upwind": 1, "centered": 2, "solu": 3}
         row = index.row()
         string = index.model().dataScheme[row]['ischcv']
         idx = dico[string]
@@ -117,20 +118,39 @@ class SolverChoiceDelegate(QItemDelegate):
     def createEditor(self, parent, option, index):
         editor = QComboBox(parent)
 
+        mg = index.model().dataSolver[index.row()]['mg']
         editor.addItem("Automatic")
         editor.addItem("Conjugate gradient")
+        editor.addItem("Flexible conjugate gradient")
+        editor.addItem("Inexact conjugate gradient")
         editor.addItem("Jacobi")
-        editor.addItem("BI-CGSTAB")
+        editor.addItem("BiCGstab")
+        editor.addItem("BiCGstab2")
         editor.addItem("GMRES")
-        editor.addItem("Multigrid")
+        editor.addItem("Gauss Seidel")
+        editor.addItem("Symmetric Gauss Seidel")
+        editor.addItem("conjugate residual")
+        if mg:
+            editor.addItem("Multigrid, V-cycle")
+            editor.addItem("Multigrid, K-cycle")
         editor.installEventFilter(self)
-        if index.row() > 0:
-            editor.removeItem(4)
         return editor
 
 
     def setEditorData(self, comboBox, index):
-        dico = {"automatic": 0, "conjugate_gradient": 1, "jacobi": 2, "bi_cgstab": 3, "gmres": 4, "multigrid": 5}
+        dico = {"automatic": 0,
+                "conjugate_gradient": 1,
+                "flexible_conjugate_gradient": 2,
+                "inexact_conjugate_gradient": 3,
+                "jacobi": 4,
+                "bi_cgstab": 5,
+                "bi_cgstab2": 6,
+                "gmres": 7,
+                "gauss_seidel": 8,
+                "symmetric_gauss_seidel": 9,
+                "PCR3": 10,
+                "multigrid": 11,
+                "multigrid_k_cycle": 12}
         row = index.row()
         string = index.model().dataSolver[row]['iresol']
         idx = dico[string]
@@ -144,6 +164,47 @@ class SolverChoiceDelegate(QItemDelegate):
             if idx.column() == index.column():
                 model.setData(idx, to_qvariant(value))
 
+#-------------------------------------------------------------------------------
+# Combo box delegate for preconditioning
+#-------------------------------------------------------------------------------
+
+class PreconditioningChoiceDelegate(QItemDelegate):
+    """
+    Use of a combo box in the table.
+    """
+    def __init__(self, parent=None, xml_model=None):
+        super(PreconditioningChoiceDelegate, self).__init__(parent)
+        self.parent = parent
+        self.mdl = xml_model
+
+
+    def createEditor(self, parent, option, index):
+        editor = QComboBox(parent)
+
+        editor.addItem("Automatic")
+        editor.addItem("None")
+        editor.addItem("Multigrid, V-cycle")
+        editor.addItem("Multigrid, K-cycle")
+        editor.addItem("Jacobi")
+        editor.addItem("Polynomial")
+
+        solver = index.model().dataSolver[index.row()]['iresol']
+        if solver in ('multigrid', 'multigrid_k_cycle'):
+            editor.model().item(1).setEnabled(False)
+        editor.installEventFilter(self)
+        return editor
+
+
+    def setEditorData(self, comboBox, index):
+        row = index.row()
+        string = index.model().dataSolver[row]['precond']
+        comboBox.setEditText(string)
+
+
+    def setModelData(self, comboBox, model, index):
+        value = comboBox.currentText()
+        model.setData(index, to_qvariant(value), Qt.DisplayRole)
+
 #-------------------------------------------------------------------------------
 # Line edit delegate for BLENCV
 #-------------------------------------------------------------------------------
@@ -161,12 +222,13 @@ class BlendingFactorDelegate(QItemDelegate):
             validator = DoubleValidator(editor, min=0.95, max=1.)
         else:
             validator = DoubleValidator(editor, min=0., max=1.)
-            validator.setExclusiveMin(True)
+            # validator.setExclusiveMin(True)
         editor.setValidator(validator)
         return editor
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(value)
 
@@ -198,6 +260,7 @@ class RhsReconstructionDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(value)
 
@@ -227,19 +290,23 @@ class SolverDelegate(QItemDelegate):
             validator.setExclusiveMin(True)
         elif (index.column() == 2 or index.column() == 4):
             validator = IntValidator(editor, min=1)
+        elif index.column() == 5:
+            validator = DoubleValidator(editor, min=0.)
+            validator.setExclusiveMin(True)
         editor.setValidator(validator)
         editor.installEventFilter(self)
         return editor
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(value)
 
 
     def setModelData(self, editor, model, index):
         if editor.validator().state == QValidator.Acceptable:
-            if index.column() == 3:
+            if index.column() == 3 or index.column() == 5:
                 value = from_qvariant(editor.text(), float)
             elif (index.column() == 2 or index.column() == 4):
                 value = from_qvariant(editor.text(), int)
@@ -269,7 +336,7 @@ class StandardItemModelScheme(QStandardItemModel):
                         self.tr("Slope\nTest"),
                         self.tr("Flux\nReconstruction"),
                         self.tr("RHS Sweep\nReconstruction")]
-        self.keys = ['label', 'ischcv', 'blencv', 'isstpc', 'ircflu', 'nswrsm']
+        self.keys = ['name', 'ischcv', 'blencv', 'isstpc', 'ircflu', 'nswrsm']
         self.setColumnCount(len(self.headers))
 
         # Initialize the flags
@@ -285,17 +352,17 @@ class StandardItemModelScheme(QStandardItemModel):
 
 
     def populateModel(self):
-        self.dicoV2M= {"Upwind" : 'upwind', "Centered": 'centered', "SOLU": 'solu'}
-        self.dicoM2V= {"upwind" : 'Upwind', "centered": 'Centered', "solu": 'SOLU'}
+        self.dicoV2M= {"Automatic" : 'automatic', "Upwind" : 'upwind', "Centered": 'centered', "SOLU": 'solu'}
+        self.dicoM2V= {'automatic' : "Automatic", "upwind" : 'Upwind', "centered": 'Centered', "solu": 'SOLU'}
 
-        for label in self.NPE.getSchemeList():
+        for name in self.NPE.getSchemeList():
             dico           = {}
-            dico['label']  = label
-            dico['blencv'] = self.NPE.getBlendingFactor(label)
-            dico['ischcv'] = self.NPE.getScheme(label)
-            dico['isstpc'] = self.NPE.getSlopeTest(label)
-            dico['ircflu'] = self.NPE.getFluxReconstruction(label)
-            dico['nswrsm'] = self.NPE.getRhsReconstruction(label)
+            dico['name']  = name
+            dico['blencv'] = self.NPE.getBlendingFactor(name)
+            dico['ischcv'] = self.NPE.getScheme(name)
+            dico['isstpc'] = self.NPE.getSlopeTest(name)
+            dico['ircflu'] = self.NPE.getFluxReconstruction(name)
+            dico['nswrsm'] = self.NPE.getRhsReconstruction(name)
             self.dataScheme.append(dico)
             log.debug("populateModel-> dataScheme = %s" % dico)
             row = self.rowCount()
@@ -366,7 +433,7 @@ class StandardItemModelScheme(QStandardItemModel):
     def setData(self, index, value, role=None):
         row = index.row()
         column = index.column()
-        label = self.dataScheme[row]['label']
+        name = self.dataScheme[row]['name']
 
         # for Pressure, most fields are empty
         if column > 0 and str(from_qvariant(value, to_text_string)) in ['', 'None']:
@@ -392,14 +459,14 @@ class StandardItemModelScheme(QStandardItemModel):
                     self.disabledItem.remove((row, 3))
                     self.dataScheme[row]['isstpc'] = "on"
 
-            self.NPE.setScheme(label, self.dataScheme[row]['ischcv'])
-            self.NPE.setBlendingFactor(label, self.dataScheme[row]['blencv'])
+            self.NPE.setScheme(name, self.dataScheme[row]['ischcv'])
+            self.NPE.setBlendingFactor(name, self.dataScheme[row]['blencv'])
 
         # set BLENCV
         elif column == 2:
             if self.dataScheme[row]['ischcv'] != "upwind":
                 self.dataScheme[row]['blencv'] = from_qvariant(value, float)
-                self.NPE.setBlendingFactor(label, self.dataScheme[row]['blencv'])
+                self.NPE.setBlendingFactor(name, self.dataScheme[row]['blencv'])
 
         # set ISSTPC
         elif column == 3:
@@ -409,7 +476,7 @@ class StandardItemModelScheme(QStandardItemModel):
                     self.dataScheme[row]['isstpc'] = "off"
                 else:
                     self.dataScheme[row]['isstpc'] = "on"
-                self.NPE.setSlopeTest(label, self.dataScheme[row]['isstpc'])
+                self.NPE.setSlopeTest(name, self.dataScheme[row]['isstpc'])
 
         # set IRCFLU
         elif column == 4:
@@ -418,14 +485,14 @@ class StandardItemModelScheme(QStandardItemModel):
                 self.dataScheme[row]['ircflu'] = "off"
             else:
                 self.dataScheme[row]['ircflu'] = "on"
-            self.NPE.setFluxReconstruction(label, self.dataScheme[row]['ircflu'])
+            self.NPE.setFluxReconstruction(name, self.dataScheme[row]['ircflu'])
 
         # set NSWRSM
         elif column == 5:
             self.dataScheme[row]['nswrsm'] = from_qvariant(value, int)
-            self.NPE.setRhsReconstruction(label, self.dataScheme[row]['nswrsm'])
+            self.NPE.setRhsReconstruction(name, self.dataScheme[row]['nswrsm'])
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 #-------------------------------------------------------------------------------
@@ -436,13 +503,12 @@ class StandardItemModelSolver(QStandardItemModel):
     """
     Model associated with a QTableView.
     """
-    def __init__(self, NPE, SM):
+    def __init__(self, NPE):
         """
         """
         QStandardItemModel.__init__(self)
         self.NPE = NPE
-        self.SM = SM
-        self.setColumnCount(5)
+        self.setColumnCount(6)
         self.dataSolver = []
         # list of items to be disabled in the view
         self.disabledItem = []
@@ -450,26 +516,53 @@ class StandardItemModelSolver(QStandardItemModel):
 
 
     def populateModel(self):
-        self.dicoV2M= {"Multigrid": 'multigrid',"Conjugate gradient" : 'conjugate_gradient',
-                       "Jacobi": 'jacobi', "BI-CGSTAB": 'bi_cgstab', "GMRES": 'gmres',
-                       "Automatic": "automatic"}
-        self.dicoM2V= {"multigrid" : 'Multigrid',"conjugate_gradient" : 'Conjugate gradient',
-                       "jacobi": 'Jacobi', "bi_cgstab": 'BI-CGSTAB', 'gmres': "GMRES",
-                       "automatic": "Automatic"}
-        for label in self.NPE.getSolverList():
+        self.dicoV2M= {"Multigrid, V-cycle"     : 'multigrid',
+                       "Multigrid, K-cycle"     : 'multigrid_k_cycle',
+                       "Conjugate gradient"     : 'conjugate_gradient',
+                       "Flexible conjugate gradient" : 'flexible_conjugate_gradient',
+                       "Inexact conjugate gradient"  : 'inexact_conjugate_gradient',
+                       "Jacobi"                 : 'jacobi',
+                       "BiCGstab"               : 'bi_cgstab',
+                       "BiCGstab2"              : 'bi_cgstab2',
+                       "GMRES"                  : 'gmres',
+                       "Automatic"              : "automatic",
+                       "Gauss Seidel"           : "gauss_seidel",
+                       "Symmetric Gauss Seidel" : "symmetric_gauss_seidel",
+                       "conjugate residual"     : "PCR3",
+                       "None"                   : "none",
+                       "Polynomial"             : "polynomial"}
+        self.dicoM2V= {"multigrid"              : 'Multigrid, V-cycle',
+                       "multigrid_k_cycle"      : 'Multigrid, K-cycle',
+                       "conjugate_gradient"     : 'Conjugate gradient',
+                       "inexact_conjugate_gradient"  : 'Inexact conjugate gradient',
+                       "flexible_conjugate_gradient" : 'Flexible conjugate gradient',
+                       "jacobi"                 : 'Jacobi',
+                       "bi_cgstab"              : 'BiCGstab',
+                       "bi_cgstab2"             : 'BiCGstab2',
+                       'gmres'                  : "GMRES",
+                       "automatic"              : "Automatic",
+                       "gauss_seidel"           : "Gauss Seidel",
+                       "symmetric_gauss_seidel" : "Symmetric Gauss Seidel",
+                       "PCR3"                   : "conjugate residual",
+                       "none"                   : "None",
+                       "polynomial"             : "Polynomial"}
+
+        for name in self.NPE.getSolverList():
             row = self.rowCount()
             self.setRowCount(row + 1)
 
-            dico           = {}
-            dico['label']  = label
-            dico['iresol'] = self.NPE.getSolverChoice(label)
-            dico['nitmax'] = self.NPE.getMaxIterNumber(label)
-            dico['epsilo'] = self.NPE.getSolverPrecision(label)
-            if self.NPE.isScalar(label):
-                dico['cdtvar'] = self.NPE.getScalarTimeStepFactor(label)
+            dico            = {}
+            dico['name']    = name
+            dico['mg']      = self.NPE.getSolverAllowMultigrid(name)
+            dico['iresol']  = self.NPE.getSolverChoice(name)
+            dico['precond'] = self.NPE.getPreconditioningChoice(name)
+            dico['epsilo']  = self.NPE.getSolverPrecision(name)
+            dico['verbo']   = self.NPE.getVerbosity(name)
+            if self.NPE.isScalar(name):
+                dico['cdtvar'] = self.NPE.getScalarTimeStepFactor(name)
             else:
                 dico['cdtvar'] = ""
-                self.disabledItem.append((row,4))
+                self.disabledItem.append((row,5))
 
             self.dataSolver.append(dico)
             log.debug("populateModel-> dataSolver = %s" % dico)
@@ -481,13 +574,11 @@ class StandardItemModelSolver(QStandardItemModel):
             return to_qvariant()
 
         if role == Qt.ToolTipRole:
-            if index.column() == 1:
-                return to_qvariant(self.tr("Code_Saturne keyword: IRESOL"))
-            elif index.column() == 2:
-                return to_qvariant(self.tr("Code_Saturne keyword: NITMAX"))
-            elif index.column() == 3:
-                return to_qvariant(self.tr("Code_Saturne keyword: EPSILO"))
+            if index.column() == 3:
+                return to_qvariant(self.tr("Field variable calculation option keyword: epsilo"))
             elif index.column() == 4:
+                return to_qvariant(self.tr("Field variable calculation option keyword: iwarni"))
+            elif index.column() == 5:
                 return to_qvariant(self.tr("Code_Saturne keyword: CDTVAR"))
 
         elif role == Qt.DisplayRole:
@@ -495,14 +586,16 @@ class StandardItemModelSolver(QStandardItemModel):
             dico = self.dataSolver[row]
 
             if index.column() == 0:
-                return to_qvariant(dico['label'])
+                return to_qvariant(dico['name'])
             elif index.column() == 1:
                 return to_qvariant(self.dicoM2V[dico['iresol']])
             elif index.column() == 2:
-                return to_qvariant(dico['nitmax'])
+                return to_qvariant(self.dicoM2V[dico['precond']])
             elif index.column() == 3:
                 return to_qvariant(dico['epsilo'])
             elif index.column() == 4:
+                return to_qvariant(dico['verbo'])
+            elif index.column() == 5:
                 return to_qvariant(dico['cdtvar'])
             else:
                 return to_qvariant()
@@ -534,10 +627,12 @@ class StandardItemModelSolver(QStandardItemModel):
             elif section == 1:
                 return to_qvariant(self.tr("Solver\nChoice"))
             elif section == 2:
-                return to_qvariant(self.tr("Maximum\nIteration Number"))
+                return to_qvariant(self.tr("Preconditioning\nChoice"))
             elif section == 3:
                 return to_qvariant(self.tr("Solver\nPrecision"))
             elif section == 4:
+                return to_qvariant(self.tr("Verbosity"))
+            elif section == 5:
                 return to_qvariant(self.tr("Time Step\nFactor"))
             else:
                 return to_qvariant()
@@ -546,27 +641,32 @@ class StandardItemModelSolver(QStandardItemModel):
 
     def setData(self, index, value, role=None):
         row = index.row()
-        label = self.dataSolver[row]['label']
+        name = self.dataSolver[row]['name']
 
         if index.column() == 1:
             self.dataSolver[row]['iresol'] = self.dicoV2M[from_qvariant(value, to_text_string)]
-            self.NPE.setSolverChoice(label, self.dataSolver[row]['iresol'])
+            self.NPE.setSolverChoice(name, self.dataSolver[row]['iresol'])
 
         elif index.column() == 2:
-            self.dataSolver[row]['nitmax'] = from_qvariant(value, int)
-            self.NPE.setMaxIterNumber(label, self.dataSolver[row]['nitmax'])
+            self.dataSolver[row]['precond'] = self.dicoV2M[from_qvariant(value, to_text_string)]
+            self.NPE.setPreconditioningChoice(name, self.dataSolver[row]['precond'])
 
         elif index.column() == 3:
             self.dataSolver[row]['epsilo'] = from_qvariant(value, float)
-            self.NPE.setSolverPrecision(label, self.dataSolver[row]['epsilo'])
+            self.NPE.setSolverPrecision(name, self.dataSolver[row]['epsilo'])
 
         elif index.column() == 4:
-            self.dataSolver[row]['cdtvar'] = from_qvariant(value, float)
-            self.NPE.setScalarTimeStepFactor(label, self.dataSolver[row]['cdtvar'])
+            self.dataSolver[row]['verbo'] = from_qvariant(value, int)
+            self.NPE.setVerbosity(name, self.dataSolver[row]['verbo'])
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        elif index.column() == 5:
+            self.dataSolver[row]['cdtvar'] = from_qvariant(value, float)
+            self.NPE.setScalarTimeStepFactor(name, self.dataSolver[row]['cdtvar'])
+
+        self.dataChanged.emit(index, index)
         return True
 
+
 #-------------------------------------------------------------------------------
 # Line edit delegate for minimum value
 #-------------------------------------------------------------------------------
@@ -585,6 +685,7 @@ class MinimumDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(value)
 
@@ -596,11 +697,45 @@ class MinimumDelegate(QItemDelegate):
             for idx in self.parent.selectionModel().selectedIndexes():
                 if idx.column() == index.column():
                     maxi = model.getData(idx)['scamax']
-                    label = model.getData(idx)['label']
-                    if model.checkMinMax(label, value, maxi):
+                    name = model.getData(idx)['name']
+                    if model.checkMinMax(name, value, maxi):
                         model.setData(idx, to_qvariant(value), Qt.DisplayRole)
 
 
+#-------------------------------------------------------------------------------
+# Line edit delegate for verbosity
+#-------------------------------------------------------------------------------
+
+class VerbosityDelegate(QItemDelegate):
+    def __init__(self, parent=None, xml_model=None):
+        super(VerbosityDelegate, self).__init__(parent)
+        self.parent = parent
+        self.turb = xml_model
+
+
+    def createEditor(self, parent, option, index):
+        editor = QLineEdit(parent)
+        validator = IntValidator(editor, min=0)
+        validator.setExclusiveMin(False)
+        editor.setValidator(validator)
+        return editor
+
+
+    def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
+        value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
+        editor.setText(value)
+
+
+    def setModelData(self, editor, model, index):
+        if editor.validator().state == QValidator.Acceptable:
+            value = from_qvariant(editor.text(), int)
+            selectionModel = self.parent.selectionModel()
+            for idx in selectionModel.selectedIndexes():
+                if idx.column() == index.column():
+                    model.setData(idx, to_qvariant(value))
+
+
 #-------------------------------------------------------------------------------
 # Line edit delegate for maximum value
 #-------------------------------------------------------------------------------
@@ -619,6 +754,7 @@ class MaximumDelegate(QItemDelegate):
 
 
     def setEditorData(self, editor, index):
+        editor.setAutoFillBackground(True)
         value = from_qvariant(index.model().data(index, Qt.DisplayRole), to_text_string)
         editor.setText(value)
 
@@ -631,8 +767,8 @@ class MaximumDelegate(QItemDelegate):
             for idx in self.parent.selectionModel().selectedIndexes():
                 if idx.column() == index.column():
                     mini = model.getData(idx)['scamin']
-                    label = model.getData(idx)['label']
-                    if model.checkMinMax(label, mini, value):
+                    name = model.getData(idx)['name']
+                    if model.checkMinMax(name, mini, value):
                         model.setData(idx, to_qvariant(value), Qt.DisplayRole)
 
 
@@ -665,13 +801,13 @@ class StandardItemModelClipping(QStandardItemModel):
         self.populateModel()
 
     def populateModel(self):
-        for label in self.NPE.getClippingList():
+        for name in self.NPE.getClippingList():
             row = self.rowCount()
             self.setRowCount(row + 1)
             dico             = {}
-            dico['label']    = label
-            dico['scamin']   = self.NPE.getMinValue(label)
-            dico['scamax']   = self.NPE.getMaxValue(label)
+            dico['name']    = name
+            dico['scamin']   = self.NPE.getMinValue(name)
+            dico['scamax']   = self.NPE.getMaxValue(name)
 
             self._data.append(dico)
             log.debug("populateModel-> _data = %s" % dico)
@@ -690,7 +826,7 @@ class StandardItemModelClipping(QStandardItemModel):
             row = index.row()
             dico = self._data[row]
             if col == 0:
-                return to_qvariant(dico['label'])
+                return to_qvariant(dico['name'])
             elif col == 1:
                 return to_qvariant(dico['scamin'])
             elif col == 2:
@@ -723,17 +859,17 @@ class StandardItemModelClipping(QStandardItemModel):
         if not index.isValid():
             return Qt.ItemIsEnabled
         row = index.row()
-        label = self._data[row]['label']
+        name = self._data[row]['name']
 
         if index.column() == 1:
             self._data[row]['scamin'] = from_qvariant(value, float)
-            self.NPE.setMinValue(label, self._data[row]['scamin'])
+            self.NPE.setMinValue(name, self._data[row]['scamin'])
 
         elif index.column() == 2:
             self._data[row]['scamax'] = from_qvariant(value, float)
-            self.NPE.setMaxValue(label, self._data[row]['scamax'])
+            self.NPE.setMaxValue(name, self._data[row]['scamax'])
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -742,7 +878,7 @@ class StandardItemModelClipping(QStandardItemModel):
         return self._data[row]
 
 
-    def checkMinMax(self, label, mini, maxi):
+    def checkMinMax(self, name, mini, maxi):
         """
         Verify the coherence between mini and maxi
         """
@@ -752,7 +888,7 @@ class StandardItemModelClipping(QStandardItemModel):
             title = self.tr("Information")
             msg = self.tr("The minimal value is greater than the maximal "\
                           "value. Therefore there will be no clipping for the "\
-                          "scalar named:\n\n%1").arg(label)
+                          "scalar named:\n\n%1").arg(name)
             QMessageBox.information(self.parent, title, msg)
             return OK
 
@@ -776,8 +912,7 @@ class NumericalParamEquationView(QWidget, Ui_NumericalParamEquationForm):
 
         self.case = case
         self.case.undoStopGlobal()
-        self.NPE = NumericalParamEquatModel(self.case)
-        self.SM  = SteadyManagementModel(self.case)
+        self.NPE = NumericalParamEquationModel(self.case)
         self.turb = TurbulenceModel(self.case)
 
         # Scheme
@@ -789,7 +924,10 @@ class NumericalParamEquationView(QWidget, Ui_NumericalParamEquationForm):
         self.tableViewScheme.setSelectionBehavior(QAbstractItemView.SelectItems)
         self.tableViewScheme.setSelectionMode(QAbstractItemView.ExtendedSelection)
         self.tableViewScheme.setEditTriggers(QAbstractItemView.DoubleClicked)
-        self.tableViewScheme.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableViewScheme.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableViewScheme.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
         delegateISCHCV = SchemeOrderDelegate(self.tableViewScheme)
         self.tableViewScheme.setItemDelegateForColumn(1, delegateISCHCV)
@@ -801,7 +939,7 @@ class NumericalParamEquationView(QWidget, Ui_NumericalParamEquationForm):
         self.tableViewScheme.setItemDelegateForColumn(5, delegateNSWRSM)
 
         # Solver
-        self.modelSolver = StandardItemModelSolver(self.NPE, self.SM)
+        self.modelSolver = StandardItemModelSolver(self.NPE)
         self.tableViewSolver.setModel(self.modelSolver)
         self.tableViewSolver.setAlternatingRowColors(True)
         self.tableViewSolver.resizeColumnToContents(0)
@@ -809,10 +947,15 @@ class NumericalParamEquationView(QWidget, Ui_NumericalParamEquationForm):
         self.tableViewSolver.setSelectionBehavior(QAbstractItemView.SelectItems)
         self.tableViewSolver.setSelectionMode(QAbstractItemView.ExtendedSelection)
         self.tableViewSolver.setEditTriggers(QAbstractItemView.DoubleClicked)
-        self.tableViewSolver.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableViewSolver.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableViewSolver.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
-        if self.SM.getSteadyFlowManagement() == 'on':
-            self.tableViewSolver.setColumnHidden(4, True)
+        from code_saturne.model.TimeStepModel import TimeStepModel
+        idtvar = TimeStepModel(self.case).getTimePassing()
+        if idtvar in [-1, 2]:
+            self.tableViewSolver.setColumnHidden(5, True)
 
         delegate = SolverDelegate(self.tableViewSolver)
         self.tableViewSolver.setItemDelegate(delegate)
@@ -820,6 +963,12 @@ class NumericalParamEquationView(QWidget, Ui_NumericalParamEquationForm):
         delegateSolverChoice = SolverChoiceDelegate(self.tableViewSolver)
         self.tableViewSolver.setItemDelegateForColumn(1, delegateSolverChoice)
 
+        delegatePrecondChoice = PreconditioningChoiceDelegate(self.tableViewSolver)
+        self.tableViewSolver.setItemDelegateForColumn(2, delegatePrecondChoice)
+
+        delegateVerbosity = VerbosityDelegate(self.tableViewSolver)
+        self.tableViewSolver.setItemDelegateForColumn(4, delegateVerbosity)
+
         # Clipping
         self.modelClipping = StandardItemModelClipping(self, self.NPE)
         self.tableViewClipping.setModel(self.modelClipping)
@@ -829,7 +978,10 @@ class NumericalParamEquationView(QWidget, Ui_NumericalParamEquationForm):
         self.tableViewClipping.setSelectionBehavior(QAbstractItemView.SelectItems)
         self.tableViewClipping.setSelectionMode(QAbstractItemView.ExtendedSelection)
         self.tableViewClipping.setEditTriggers(QAbstractItemView.DoubleClicked)
-        self.tableViewClipping.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableViewClipping.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableViewClipping.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
         delegateMin = MinimumDelegate(self.tableViewClipping)
         self.tableViewClipping.setItemDelegateForColumn(1, delegateMin)
@@ -842,12 +994,12 @@ class NumericalParamEquationView(QWidget, Ui_NumericalParamEquationForm):
 
         self.tabWidgetScheme.setCurrentIndex(self.case['current_tab'])
 
-        self.connect(self.tabWidgetScheme, SIGNAL("currentChanged(int)"), self.slotchanged)
+        self.tabWidgetScheme.currentChanged[int].connect(self.slotchanged)
 
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("int")
+    @pyqtSlot(int)
     def slotchanged(self, index):
         """
         Changed tab
