@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -37,19 +37,21 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import to_qvariant, from_qvariant, to_text_string
-from Pages.OutputSurfacicVariablesForm import Ui_OutputSurfacicVariablesForm
-from Pages.OutputControlModel import OutputControlModel
-from Pages.OutputSurfacicVariablesModel import OutputSurfacicVariablesModel
-from Pages.OutputVolumicVariablesView import LabelDelegate
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import to_qvariant, from_qvariant, to_text_string
+from code_saturne.Pages.OutputSurfacicVariablesForm import Ui_OutputSurfacicVariablesForm
+from code_saturne.model.OutputControlModel import OutputControlModel
+from code_saturne.model.OutputSurfacicVariablesModel import OutputSurfacicVariablesModel
+from code_saturne.model.OutputSurfacicFieldsModel import OutputSurfacicFieldsModel #AZ
+from code_saturne.Pages.OutputVolumicVariablesView import LabelDelegate
 
 #-------------------------------------------------------------------------------
 # log config
@@ -73,8 +75,9 @@ class StandardItemModelOutput(QStandardItemModel):
         self.case = case
         self.mdl = mdl
 
-        self.setColumnCount(2)
+        self.setColumnCount(3)
         self.dataLabel      = []
+        self.dataName       = []
         self.dataPost       = []
         self.disableItem    = []
         self.populateModel()
@@ -89,11 +92,13 @@ class StandardItemModelOutput(QStandardItemModel):
 
             label = self.mdl.dicoLabelName[name]
             post  = self.mdl.getPostProcessing(label)
+
             if OutputControlModel(self.case).getAssociatedWriterIdList("-2") == []:
                 self.disableItem.append((row, 1))
                 post = "off"
 
             self.dataLabel.append(label)
+            self.dataName.append(name)
             self.dataPost.append(post)
 
 
@@ -103,16 +108,12 @@ class StandardItemModelOutput(QStandardItemModel):
 
         # ToolTips BUG
         if role == Qt.ToolTipRole:
-            if index.column() == 0 and index.column() > 3:
-                return to_qvariant(self.tr("Code_Saturne keyword: nbrvaf"))
-            elif index.column() == 1 and index.column() > 3:
-                return to_qvariant(self.tr("Code_Saturne keyword: irayvf"))
-            elif index.column() == 1 and index.column() <= 3:
+            if index.column() == 2:
                 return to_qvariant(self.tr("Code_Saturne keyword: ipstdv"))
 
         # StatusTips
         if role == Qt.StatusTipRole:
-            if index.column() == 1:
+            if index.column() == 2:
                 return to_qvariant("Post-processing")
 
         # Display
@@ -120,13 +121,15 @@ class StandardItemModelOutput(QStandardItemModel):
             row = index.row()
             if index.column() == 0:
                 return to_qvariant(self.dataLabel[row])
+            elif index.column() == 1:
+                return to_qvariant(self.dataName[row])
             else:
                 return to_qvariant()
 
         # CheckState
         if role == Qt.CheckStateRole:
             row = index.row()
-            if index.column() == 1:
+            if index.column() == 2:
                 value = self.dataPost[row]
                 if value == 'on':
                     return to_qvariant(Qt.Checked)
@@ -145,6 +148,8 @@ class StandardItemModelOutput(QStandardItemModel):
         elif index.column() == 0 :
             return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable
         elif index.column() == 1 :
+            return  Qt.ItemIsEnabled | Qt.ItemIsSelectable
+        elif index.column() == 2 :
             return  Qt.ItemIsEnabled | Qt.ItemIsUserCheckable
         else:
             return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable
@@ -153,8 +158,10 @@ class StandardItemModelOutput(QStandardItemModel):
     def headerData(self, section, orientation, role):
         if orientation == Qt.Horizontal and role == Qt.DisplayRole:
             if section == 0:
-                return to_qvariant(self.tr("Name"))
-            elif section == 1:
+                return to_qvariant(self.tr("Output label"))
+            if section == 1:
+                return to_qvariant(self.tr("Internal name"))
+            elif section == 2:
                 return to_qvariant(self.tr("Post-\nprocessing"))
         return to_qvariant()
 
@@ -163,11 +170,12 @@ class StandardItemModelOutput(QStandardItemModel):
         row = index.row()
         if index.column() == 0:
             label = str(from_qvariant(value, to_text_string))
-            if label == "": label = self.dataLabel[row]
+            if label == "":
+                label = self.dataLabel[row]
             self.mdl.setPropertyLabel(self.dataLabel[row], label)
             self.dataLabel[row] = label
 
-        elif index.column() == 1:
+        elif index.column() == 2:
             v = from_qvariant(value, int)
             if v == Qt.Checked:
                 self.dataPost[row] = "on"
@@ -178,7 +186,7 @@ class StandardItemModelOutput(QStandardItemModel):
 
             self.mdl.setPostProcessing(self.dataLabel[row], self.dataPost[row])
 
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 #-------------------------------------------------------------------------------
@@ -199,7 +207,11 @@ class OutputSurfacicVariablesView(QWidget, Ui_OutputSurfacicVariablesForm):
 
         self.case = case
         self.case.undoStopGlobal()
-        self.mdl = OutputSurfacicVariablesModel(self.case)
+
+        if self.case.xmlRootNode().tagName == "Code_Saturne_GUI":
+            self.mdl = OutputSurfacicVariablesModel(self.case)
+        else:
+            self.mdl = OutputSurfacicFieldsModel(self.case)
 
         self.modelOutput = StandardItemModelOutput(self.case, self.mdl)
         self.tableViewOutput.setModel(self.modelOutput)
@@ -209,7 +221,10 @@ class OutputSurfacicVariablesView(QWidget, Ui_OutputSurfacicVariablesForm):
         self.tableViewOutput.setSelectionBehavior(QAbstractItemView.SelectItems)
         self.tableViewOutput.setSelectionMode(QAbstractItemView.ExtendedSelection)
         self.tableViewOutput.setEditTriggers(QAbstractItemView.DoubleClicked)
-        self.tableViewOutput.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        if QT_API == "PYQT4":
+            self.tableViewOutput.horizontalHeader().setResizeMode(QHeaderView.Stretch)
+        elif QT_API == "PYQT5":
+            self.tableViewOutput.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
 
         labelDelegate = LabelDelegate(self.tableViewOutput, self.mdl)
         self.tableViewOutput.setItemDelegateForColumn(0, labelDelegate)
