@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -23,8 +23,6 @@
 #-------------------------------------------------------------------------------
 
 """
-This module defines the 'Meshes and Enveloppe' page.
-
 This module contains the following classes:
 - ProfilesView
 """
@@ -33,27 +31,27 @@ This module contains the following classes:
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import string
 import logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Common import LABEL_LENGTH_MAX
-from Base.Toolbox import GuiParam
-from Base.QtPage import IntValidator, DoubleValidator, RegExpValidator, ComboModel
-from Base.QtPage import setGreenColor, to_qvariant, from_qvariant
-from Pages.ProfilesForm import Ui_ProfilesForm
-from Pages.ProfilesModel import ProfilesModel
-from Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.model.Common import LABEL_LENGTH_MAX, GuiParam
+from code_saturne.Base.QtPage import IntValidator, DoubleValidator, RegExpValidator, ComboModel
+from code_saturne.Base.QtPage import to_qvariant, from_qvariant
+from code_saturne.Pages.ProfilesForm import Ui_ProfilesForm
+from code_saturne.model.ProfilesModel import ProfilesModel
+from code_saturne.Pages.QMeiEditorView import QMeiEditorView
+from code_saturne.model.NotebookModel import NotebookModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -106,7 +104,7 @@ class StandardItemModelProfile(QStandardItemModel):
 
 
     def setData(self, index, value, role):
-        self.emit(SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)
+        self.dataChanged.emit(index, index)
         return True
 
 
@@ -167,6 +165,7 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
         self.case = case
         self.case.undoStopGlobal()
         self.mdl = ProfilesModel(self.case)
+        self.notebook = NotebookModel(self.case)
 
         #  Initialize variables concerning the display of the Hlist
 
@@ -179,15 +178,14 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
 
         # QListView layout
         self.gridlayout1 = QGridLayout(self.widgetDrag)
-        self.gridlayout1.setMargin(0)
+        self.gridlayout1.setContentsMargins(0, 0, 0, 0)
         self.DragList = QListView(self.widgetDrag)
         self.gridlayout1.addWidget(self.DragList,0,0,1,1)
 
         self.gridlayout2 = QGridLayout(self.widgetDrop)
-        self.gridlayout2.setMargin(0)
+        self.gridlayout2.setContentsMargins(0, 0, 0, 0)
         self.DropList = QListView(self.widgetDrop)
         self.gridlayout2.addWidget(self.DropList,0,0,1,1)
-        self.line_formula = "x = 0;\ny = 0;\nz = 0;\n"
 
         self.modelDrag = QStringListModel()
         self.modelDrop = QStringListModel()
@@ -209,15 +207,18 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
         self.modelFormat.addItem(self.tr(".csv"), "CSV")
 
         # Connections
-        self.connect(self.treeViewProfile,       SIGNAL("pressed(const QModelIndex &)"), self.slotSelectProfile)
-        self.connect(self.pushButtonAdd,         SIGNAL("clicked()"), self.slotAddProfile)
-        self.connect(self.pushButtonEdit,        SIGNAL("clicked()"), self.slotEditProfile)
-        self.connect(self.pushButtonDelete,      SIGNAL("clicked()"), self.slotDeleteProfile)
-        self.connect(self.pushButtonAddVar,      SIGNAL("clicked()"), self.slotAddVarProfile)
-        self.connect(self.pushButtonSuppressVar, SIGNAL("clicked()"), self.slotDeleteVarProfile)
-        self.connect(self.comboBoxFreq,          SIGNAL("activated(const QString&)"), self.slotFrequencyType)
-        self.connect(self.comboBoxFormat,        SIGNAL("activated(const QString&)"), self.slotFormatType)
-        self.connect(self.pushButtonFormula,     SIGNAL("clicked()"), self.slotFormula)
+        self.treeViewProfile.pressed[QModelIndex].connect(self.slotSelectProfile)
+        self.pushButtonAdd.clicked.connect(self.slotAddProfile)
+        self.pushButtonDelete.clicked.connect(self.slotDeleteProfile)
+        self.pushButtonAddVar.clicked.connect(self.slotAddVarProfile)
+        self.pushButtonSuppressVar.clicked.connect(self.slotDeleteVarProfile)
+        self.comboBoxFreq.activated[str].connect(self.slotFrequencyType)
+        self.comboBoxFormat.activated[str].connect(self.slotFormatType)
+        self.pushButtonFormula.clicked.connect(self.slotFormula)
+        self.lineEditBaseName.textChanged[str].connect(self.slotBaseName)
+        self.lineEditFreq.textChanged[str].connect(self.slotFrequence)
+        self.lineEditFreqTime.textChanged[str].connect(self.slotFrequenceTime)
+        self.lineEditNbPoint.textChanged[str].connect(self.slotNbPoint)
 
         # Validators
         validatorFreq = IntValidator(self.lineEditFreq, min=0)
@@ -231,10 +232,6 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
         validatorNbPoint = IntValidator(self.lineEditNbPoint, min=0)
         self.lineEditNbPoint.setValidator(validatorNbPoint)
 
-        rx = "[\- _A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX) + "}"
-        validatorTitle =  RegExpValidator(self.lineEditTitle, QRegExp(rx))
-        self.lineEditTitle.setValidator(validatorTitle)
-
         rx = "[\-_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX) + "}"
         validatorBaseName =  RegExpValidator(self.lineEditBaseName, QRegExp(rx))
         self.lineEditBaseName.setValidator(validatorBaseName)
@@ -243,15 +240,14 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
         liste_label = []
         for label in self.mdl.getVariablesAndVolumeProperties():
             liste_label.append(label)
-        self.modelDrag.setStringList(liste_label)
+        self.modelDrag.setStringList(sorted(liste_label, key=str.lower))
 
         #update list of profiles for view from xml file
         for lab in self.mdl.getProfilesLabelsList():
             self.entriesNumber = self.entriesNumber + 1
-            label, title, format, list, choice, freq, formula, nb_point = self.mdl.getProfileData(lab)
-            self.__insertProfile(label, list)
+            label, fmt, lst, choice, freq, formula, nb_point = self.mdl.getProfileData(lab)
+            self.__insertProfile(label, lst)
 
-        setGreenColor(self.pushButtonFormula, True)
         self.__eraseEntries()
 
         self.case.undoStartGlobal()
@@ -268,7 +264,7 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
             default['list'] = self.mdl.getProfilesLabelsList()
             rx = "[\-_A-Za-z0-9]{1," + str(LABEL_LENGTH_MAX) + "}"
             default['regexp'] = QRegExp(rx)
-            from Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
+            from code_saturne.Pages.VerifyExistenceLabelDialogView import VerifyExistenceLabelDialogView
             dialog = VerifyExistenceLabelDialogView(self, default)
             if dialog.exec_():
                 result = dialog.get_result()
@@ -278,7 +274,7 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
         return label
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotFrequencyType(self, text):
         """
         Input choice for frequency for profile.
@@ -291,37 +287,35 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
             self.lineEditFreq.show()
             self.lineEditFreqTime.hide()
             self.lineEditFreq.setDisabled(True)
-            self.labelBaseName.setText("Filename")
 
         elif choice == "frequency":
             self.lineEditFreq.show()
             self.lineEditFreqTime.hide()
-            nfreq = from_qvariant(self.lineEditFreq.text(), int)
-            if nfreq == -1: nfreq = 1
+            nfreq = self.mdl.getOutputFrequency(self.label_select)
+            if nfreq == -1:
+                nfreq = 1
             self.lineEditFreq.setEnabled(True)
             self.lineEditFreq.setText(str(nfreq))
-            self.labelBaseName.setText("Filename")
-
 
         elif choice == "time_value":
             self.lineEditFreq.hide()
             self.lineEditFreqTime.show()
-            if self.lineEditFreqTime.text() == "":
-                nfreq = 1.
-            else:
-                nfreq = from_qvariant(self.lineEditFreqTime.text(), float)
+            nfreq = self.mdl.getOutputFrequency(self.label_select)
+            if nfreq == -1:
+                nfreq = 1.0
             self.lineEditFreqTime.setText(str(nfreq))
-            self.labelBaseName.setText("Filename")
 
-        return choice, nfreq
+        self.mdl.setOutputType(self.label_select, choice)
+        self.mdl.setOutputFrequency(self.label_select, nfreq)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotFormatType(self, text):
         """
         Input choice for frequency for profile.
         """
-        return self.modelFormat.dicoV2M[str(text)]
+        fmt = self.modelFormat.dicoV2M[str(text)]
+        self.mdl.setFormat(self.label_select, fmt)
 
 
     def __infoProfile(self, row):
@@ -329,64 +323,29 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
         Return info from the argument entry.
         """
         label = self.modelProfile.getLabel(row)
-        lab, title, format, list, choice, freq, formula, nb_point = self.mdl.getProfileData(label)
-        return label, title, format, list, choice, freq, formula, nb_point
+        lab, fmt, lst, choice, freq, formula, nb_point = self.mdl.getProfileData(label)
+        return label, fmt, lst, choice, freq, formula, nb_point
 
 
-    def __insertProfile(self, label, list):
+    def __insertProfile(self, label, lst):
         """
         Insert values in table view.
         """
-        self.modelProfile.addItem(label, " ; ".join(list))
+        self.modelProfile.addItem(label, " ; ".join(lst))
 
 
-    def __replaceProfile(self, num, label, list):
-        """
-        Insert values in table view.
-        """
-        self.modelProfile.replaceItem(num-1, label, " ; ".join(list))
-
-
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotAddProfile(self):
         """
         Set in view label and variables to see on profile
         """
-        var_prof = [str(s) for s in self.modelDrop.stringList()]
-        log.debug("slotAddProfile -> %s" % (var_prof,))
-        if not var_prof:
-            title = self.tr("Warning")
-            msg   = self.tr("You must select at least one variable or property from list")
-            QMessageBox.information(self, title, msg)
-
-        else:
-            label = self.__verifLabel()
-
-            self.entriesNumber = self.entriesNumber + 1
-            if label == '':
-                label = 'profile' + repr(self.entriesNumber)
-
-            if label in self.mdl.getProfilesLabelsList():
-                title = self.tr("Warning")
-                msg = self.tr("This label already exists")
-                QMessageBox.information(self, title, msg)
-                return
-
-            log.debug("slotAddProfile -> %s" % (label,))
-            self.__insertProfile(label, var_prof)
-
-            choice, freq = self.slotFrequencyType(self.comboBoxFreq.currentText())
-            format = self.slotFormatType(self.comboBoxFormat.currentText())
-            title = str(self.lineEditTitle.text())
-            if not title: title = label
-            nb_point = from_qvariant(self.lineEditNbPoint.text(), int)
-            formula = self.line_formula
-
-            self.mdl.setProfile(label, title, format, var_prof, choice, freq, formula, nb_point)
-            self.__eraseEntries()
+        var_prof = []
+        label = self.mdl.addProfile()
+        self.__insertProfile(label, var_prof)
+        self.__eraseEntries()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotDeleteProfile(self):
         """
         Delete the profile from the list (one by one).
@@ -398,75 +357,27 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
             msg   = self.tr("You must select an existing profile")
             QMessageBox.information(self, title, msg)
         else:
-            label, title, format, list, choice, freq, formula, nb_point = self.__infoProfile(row)
+            label, fmt, lst, choice, freq, formula, nb_point = self.__infoProfile(row)
             self.modelProfile.deleteRow(row)
             self.mdl.deleteProfile(label)
             self.__eraseEntries()
 
 
-    @pyqtSignature("")
-    def slotEditProfile(self):
-        """
-        Edit profile to modify its characteristics.
-        """
-        row = self.treeViewProfile.currentIndex().row()
-        log.debug("slotEditProfile -> %s" % (row,))
-        if row == -1:
-            title = self.tr("Warning")
-            msg   = self.tr("You must select an existing profile")
-            QMessageBox.information(self, title, msg)
-        else:
-            old_label, title, format, vlist, choice, freq, formula, nb_point = self.__infoProfile(row)
-
-            var_prof = [str(s) for s in self.modelDrop.stringList()]
-            log.debug("slotEditProfile -> %s" % (var_prof,))
-            if not var_prof:
-                title = self.tr("Warning")
-                msg   = self.tr("You must select at least one variable or property from list")
-                QMessageBox.information(self, title, msg)
-
-            else:
-                new_label = str(self.lineEditBaseName.text())
-
-                if new_label == '':
-                    new_label = old_label
-
-                if new_label != old_label and new_label in self.mdl.getProfilesLabelsList():
-                    title = self.tr("Warning")
-                    msg = self.tr("This label already exists: the old label is kept")
-                    QMessageBox.information(self, title, msg)
-                    new_label = old_label
-
-                log.debug("slotEditedProfile -> %s" % (new_label,))
-                self.__replaceProfile(row+1, new_label, var_prof)
-                idx = self.treeViewProfile.currentIndex()
-                self.treeViewProfile.dataChanged(idx, idx)
-
-                choice, freq = self.slotFrequencyType(self.comboBoxFreq.currentText())
-                format = self.slotFormatType(self.comboBoxFormat.currentText())
-                title = str(self.lineEditTitle.text())
-                if not title: title = new_label
-                nb_point = from_qvariant(self.lineEditNbPoint.text(), int)
-                formula = self.line_formula
-
-                self.mdl.replaceProfile(old_label, new_label, title, format, var_prof, choice, freq, formula, nb_point)
-                self.__eraseEntries()
-
-
-    @pyqtSignature("const QModelIndex &")
+    @pyqtSlot("QModelIndex")
     def slotSelectProfile(self, index):
         """
         Return the selected item from the list.
         """
+        self.groupBoxProfile.show()
+
         row = index.row()
         log.debug("slotSelectProfile -> %s" % (row,))
 
-        label, title, format, liste, choice, freq, formula, nb_point = self.__infoProfile(row)
-        self.line_formula = formula
+        label, fmt, liste, choice, freq, formula, nb_point = self.__infoProfile(row)
+        self.label_select = label
 
-        self.lineEditTitle.setText(str(title))
         self.lineEditBaseName.setText(str(label))
-        self.modelFormat.setItem(str_model=format)
+        self.modelFormat.setItem(str_model=fmt)
 
         self.modelFreq.setItem(str_model=choice)
         if choice == "end":
@@ -474,20 +385,17 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
             self.lineEditFreqTime.hide()
             self.lineEditFreq.setText(str("-1"))
             self.lineEditFreq.setDisabled(True)
-            self.labelBaseName.setText("Filename")
 
         elif choice == "frequency":
             self.lineEditFreq.show()
             self.lineEditFreqTime.hide()
             self.lineEditFreq.setEnabled(True)
             self.lineEditFreq.setText(str(freq))
-            self.labelBaseName.setText("Filename")
 
         elif choice == "time_value":
             self.lineEditFreq.hide()
             self.lineEditFreqTime.show()
             self.lineEditFreqTime.setText(str(freq))
-            self.labelBaseName.setText("Filename")
 
         self.lineEditNbPoint.setText(str(nb_point))
 
@@ -496,8 +404,15 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
 
         self.modelDrop.setStringList(liste)
 
+        exp = self.mdl.getFormula(self.label_select)
+        if exp:
+            self.pushButtonFormula.setStyleSheet("background-color: green")
+            self.pushButtonFormula.setToolTip(exp)
+        else:
+            self.pushButtonFormula.setStyleSheet("background-color: red")
 
-    @pyqtSignature("")
+
+    @pyqtSlot()
     def slotAddVarProfile(self):
         """
         Add a new var from list to profile
@@ -507,53 +422,63 @@ class ProfilesView(QWidget, Ui_ProfilesForm):
             var = self.modelDrag.stringList()[self.DragList.currentIndex().row()]
             if var not in liste :
                 liste.append(var)
+            liste = [str(s) for s in liste]
             self.modelDrop.setStringList(liste)
+            self.mdl.setVariable(self.label_select, liste)
+
+            row = self.treeViewProfile.currentIndex().row()
+            liste = self.mdl.getVariable(self.label_select)
+            self.modelProfile.replaceItem(row, self.label_select, " ; ".join(liste))
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotDeleteVarProfile(self):
         """
         Supress a var from profile
         """
         self.modelDrop.removeRows(self.DropList.currentIndex().row(), 1)
+        liste = self.modelDrop.stringList()
+        liste = [str(s) for s in liste]
+        self.mdl.setVariable(self.label_select, liste)
+
+        row = self.treeViewProfile.currentIndex().row()
+        liste = self.mdl.getVariable(self.label_select)
+        self.modelProfile.replaceItem(row, self.label_select, " ; ".join(liste))
 
 
     def __eraseEntries(self):
         """
         Delete all caracters in the entries.
         """
-        self.lineEditTitle.setText(str(""))
-        self.lineEditBaseName.setText(str(""))
-        self.lineEditNbPoint.setText(str(""))
-
-        self.lineEditFreq.show()
-        self.lineEditFreqTime.hide()
-        self.modelFreq.setItem(str_model='end')
-        self.lineEditFreq.setText(str("-1"))
-
-        self.lineEditFreq.setDisabled(True)
-        self.labelBaseName.setText("Filename")
-
-        self.modelDrop.setStringList([])
-
+        self.groupBoxProfile.hide()
+        self.label_select = None
         self.treeViewProfile.clearSelection()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotFormula(self):
         """
         """
-        exp = self.line_formula
-        exa = """#example: a line segment
+        exp = self.mdl.getFormula(self.label_select)
+        exa = """#example: a line segment between (x0, y0, z0) and (x1, y1, z1)
 #(s, the parameter is always between 0 and 1)
-x = 2*s + 3.2;
-y = 2;
-z = -0.5*s+5;"""
+x0 = 3.2;
+x1 = 2;
+y0 = 2;
+y1 = 2;
+z0 = 5;
+z1 = -0.5;
+x = x1*s + x0*(1.-s);
+y = y1*s + y0*(1.-s);
+z = z1*s + z0*(1.-s);"""
         req = [('x', "x formula"),
                ('y', "y formula"),
                ('z', "z formula")]
         sym = [('s', 'parameter')]
 
+        for (nme, val) in self.notebook.getNotebookList():
+            sym.append((nme, 'value (notebook) = ' + str(val)))
+
         dialog = QMeiEditorView(self,
                                 check_syntax = self.case['package'].get_check_syntax(),
                                 expression = exp,
@@ -563,8 +488,48 @@ z = -0.5*s+5;"""
         if dialog.exec_():
             result = dialog.get_result()
             log.debug("slotLineFormula -> %s" % str(result))
-            setGreenColor(self.pushButtonFormula, False)
-            self.line_formula = result
+            self.mdl.setFormula(self.label_select, str(result))
+            self.pushButtonFormula.setToolTip(result)
+            self.pushButtonFormula.setStyleSheet("background-color: green")
+
+
+    @pyqtSlot(str)
+    def slotBaseName(self, text):
+        """
+        """
+        lst = self.mdl.getProfilesLabelsList()
+        if text not in lst:
+            if self.lineEditBaseName.validator().state == QValidator.Acceptable:
+                self.mdl.setLabel(self.label_select, str(text))
+                self.label_select = str(text)
+
+                row = self.treeViewProfile.currentIndex().row()
+                liste = self.mdl.getVariable(self.label_select)
+                self.modelProfile.replaceItem(row, self.label_select, " ; ".join(liste))
+
+
+    @pyqtSlot(str)
+    def slotFrequence(self, text):
+        """
+        """
+        if self.lineEditFreq.validator().state == QValidator.Acceptable:
+            self.mdl.setOutputFrequency(self.label_select, int(text))
+
+
+    @pyqtSlot(str)
+    def slotFrequenceTime(self, text):
+        """
+        """
+        if self.lineEditFreqTime.validator().state == QValidator.Acceptable:
+            self.mdl.setOutputFrequency(self.label_select, float(text))
+
+
+    @pyqtSlot(str)
+    def slotNbPoint(self, text):
+        """
+        """
+        if self.lineEditNbPoint.validator().state == QValidator.Acceptable:
+            self.mdl.setNbPoint(self.label_select, int(text))
 
 
     def tr(self, text):
