@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -36,7 +36,7 @@ This module contains the following classes and function:
 #-------------------------------------------------------------------------------
 
 import os
-import sys, string
+import sys
 import logging
 import subprocess
 
@@ -44,15 +44,16 @@ import subprocess
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Pages.QMeiEditorForm import Ui_QMeiDialog
+from code_saturne.model.Common import GuiParam
+from code_saturne.Pages.QMeiEditorForm import Ui_QMeiDialog
 
 #-------------------------------------------------------------------------------
 # log config
@@ -101,7 +102,7 @@ class QMeiHighlighter(QSyntaxHighlighter):
          'acos', 'asin', 'atan', 'atan2', 'cosh', 'sinh',
          'tanh', 'abs', 'mod', 'int', 'min', 'max',
          'pi', 'e', 'while', 'if', 'else', 'print',
-         'while', 'interp1d'
+         'while'
     ]
 
     operators = [
@@ -241,7 +242,6 @@ class QMeiEditorView(QDialog, Ui_QMeiDialog):
                   "<b>int</b>: floor<br>"\
                   "<b>min</b>: minimum<br>"\
                   "<b>max</b>: maximum<br>"\
-                  "<b>interp1d</b>: 1D linear interpolation<br>"\
                   "<br>"\
                   "<big><u>Useful constants:</u></big><br>"\
                   "<b>pi</b> = 3.14159265358979323846<br>"\
@@ -262,12 +262,12 @@ class QMeiEditorView(QDialog, Ui_QMeiDialog):
         self.expressionDoc = self.textEditExpression.document()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotClearBackground(self):
         """
         Private slot.
         """
-        QObject.disconnect(self.textEditExpression, SIGNAL("textChanged()"), self.slotClearBackground)
+        QObject.disconnect(self.textEditExpression)
         doc = self.textEditExpression.document()
 
         for i in range(0, doc.blockCount()):
@@ -297,7 +297,8 @@ class QMeiEditorView(QDialog, Ui_QMeiDialog):
         check = subprocess.Popen([self.check_syntax],
                                  stdin=subprocess.PIPE,
                                  stdout=subprocess.PIPE,
-                                 stderr=subprocess.PIPE)
+                                 stderr=subprocess.PIPE,
+                                 universal_newlines=True)
         check.stdin.write(str(self.textEditExpression.toPlainText()) + '\30')
 
         check.stdin.write(str(len(self.symbols)) + '\30')
@@ -338,7 +339,7 @@ class QMeiEditorView(QDialog, Ui_QMeiDialog):
                        "    column: " + str(c) + " \n\n"
 
             QMessageBox.critical(self, self.tr('Expression Editor'), str(msg))
-            QObject.connect(self.textEditExpression, SIGNAL("textChanged()"), self.slotClearBackground)
+            self.textEditExpression.textChanged.connect(self.slotClearBackground)
             return
 
         # If required symbols are missing
@@ -361,7 +362,7 @@ class QMeiEditorView(QDialog, Ui_QMeiDialog):
                 cursor.setBlockFormat(block_format)
             for i in range(0, n_errors): msg += errors[i+1] + " \n"
             QMessageBox.critical(self, self.tr('Expression Editor'), str(msg))
-            QObject.connect(self.textEditExpression, SIGNAL("textChanged()"), self.slotClearBackground)
+            self.textEditExpression.textChanged.connect(self.slotClearBackground)
             return
 
         # If another error was found
@@ -373,7 +374,7 @@ class QMeiEditorView(QDialog, Ui_QMeiDialog):
 
             msg = "Warning, expression check failed unexpectedly:\n\n"
             QMessageBox.critical(self, self.tr('Expression Editor'), str(msg))
-            QObject.connect(self.textEditExpression, SIGNAL("textChanged()"), self.slotClearBackground)
+            self.textEditExpression.textChanged.connect(self.slotClearBackground)
             return
 
         QDialog.accept(self)
@@ -407,7 +408,7 @@ class QMeiEditorView(QDialog, Ui_QMeiDialog):
 if __name__ == "__main__":
     import sys, signal
     app = QApplication(sys.argv)
-    app.connect(app, SIGNAL("lastWindowClosed()"), app, SLOT("quit()"))
+    app.lastWindowClosed.connect(app.quit)
     parent = QWidget()
     dlg = QMeiEditorView(parent)
     dlg.show()
