@@ -1,10 +1,10 @@
-# -*- coding: iso-8859-1 -*-
+# -*- coding: utf-8 -*-
 
 #-------------------------------------------------------------------------------
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -32,8 +32,9 @@ import logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore     import *
+from code_saturne.Base.QtGui      import *
+from code_saturne.Base.QtWidgets  import *
 
 import vtk
 
@@ -43,24 +44,29 @@ import SMESH
 import CFDSTUDYGUI_DataModel
 
 try:
-    from libvtkRenderingCorePython import *
-    from libvtkFiltersSourcesPython import *
+    from vtkmodules.vtkRenderingCorePython import *
+    from vtkmodules.vtkFiltersSourcesPython import *
 except:
     try:
         from vtkRenderingCorePython import *
         from vtkFiltersSourcesPython import *
     except:
-        # for compatibility with salome 6.6
+        # for compatibility with salome 7.2
         try:
-            from libvtkRenderingPython import *
+            from libvtkRenderingCorePython import *
+            from libvtkFiltersSourcesPython import *
         except:
-            from vtkRenderingPython import *
+            # for compatibility with salome 6.6
+            try:
+                from libvtkRenderingPython import *
+            except:
+                from vtkRenderingPython import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
+from code_saturne.model.Common import GuiParam
 
 #-------------------------------------------------------------------------------
 # log config
@@ -563,12 +569,12 @@ class ProfilActors(Actors):
 
 #def getMesh(label):
     #for aStudy in CFDSTUDYGUI_DataModel.GetStudyList():
-        #print "study = ", aStudy, aStudy.GetName()
+        #print("study = " +  aStudy + " " + aStudy.GetName())
         #aChildList = CFDSTUDYGUI_DataModel.ScanChildren(aStudy, label)
 
-    #print "tototototo = ", aChildList
-    #aDataObj =  aChildList[0]
-    #print aDataObj.GetName()
+    #print("tototototo = " + " ".join(aChildList))
+    #aDataObj = aChildList[0]
+    #print(aDataObj.GetName())
 
 
 #def BoundingBox(my_mesh):
@@ -585,6 +591,3 @@ class ProfilActors(Actors):
 #-------------------------------------------------------------------------------
 #
 #-------------------------------------------------------------------------------
-
-
-
