@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -34,26 +34,27 @@ This module contains the following classes:
 # Library modules import
 #-------------------------------------------------------------------------------
 
-import os, sys, string, types, shutil
+import os, sys, types, shutil
 import logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import ComboModel, IntValidator, setGreenColor, from_qvariant
-from Pages.SolutionDomainModel import RelOrAbsPath
-from Pages.StartRestartForm import Ui_StartRestartForm
-from Pages.StartRestartAdvancedDialogForm import Ui_StartRestartAdvancedDialogForm
-from Pages.StartRestartModel import StartRestartModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import ComboModel, IntValidator, from_qvariant
+from code_saturne.model.SolutionDomainModel import RelOrAbsPath
+from code_saturne.Pages.StartRestartForm import Ui_StartRestartForm
+from code_saturne.Pages.StartRestartAdvancedDialogForm import Ui_StartRestartAdvancedDialogForm
+from code_saturne.model.StartRestartModel import StartRestartModel, getRestartInfo
 
 #-------------------------------------------------------------------------------
 # log config
@@ -91,14 +92,15 @@ class StartRestartAdvancedDialogView(QDialog, Ui_StartRestartAdvancedDialogForm)
         self.modelFreq   = ComboModel(self.comboBoxFreq, 4, 1)
 
         self.modelFreq.addItem(self.tr("Never"), 'Never')
-        self.modelFreq.addItem(self.tr("Only at the end of the calculation"), 'At the end')
+        self.modelFreq.addItem(self.tr("Only at the end of the calculation"),
+                               'At the end')
         self.modelFreq.addItem(self.tr("4 restart checkpoints"), '4 output')
         self.modelFreq.addItem(self.tr("Checkpoints frequency :"), 'Frequency')
 
         # Connections
 
-        self.connect(self.comboBoxFreq, SIGNAL("activated(const QString&)"), self.slotFreq)
-        self.connect(self.lineEditNSUIT, SIGNAL("textChanged(const QString&)"), self.slotNsuit)
+        self.comboBoxFreq.activated[str].connect(self.slotFreq)
+        self.lineEditNSUIT.textChanged[str].connect(self.slotNsuit)
 
         # Validator
 
@@ -141,7 +143,7 @@ class StartRestartAdvancedDialogView(QDialog, Ui_StartRestartAdvancedDialogForm)
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("const QString &")
+    @pyqtSlot(str)
     def slotFreq(self, text):
         """
         Creation of popup window's widgets
@@ -170,9 +172,9 @@ class StartRestartAdvancedDialogView(QDialog, Ui_StartRestartAdvancedDialogForm)
             self.lineEditNSUIT.setEnabled(True)
 
 
-    @pyqtSignature("const QString &")
+    @pyqtSlot(str)
     def slotNsuit(self, text):
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditNSUIT.validator().state == QValidator.Acceptable:
             n = from_qvariant(text, int)
             self.nsuit = n
             log.debug("getNsuit-> nsuit = %s" % n)
@@ -233,31 +235,42 @@ class StartRestartView(QWidget, Ui_StartRestartForm):
         self.case = case
         self.case.undoStopGlobal()
 
-        self.connect(self.radioButtonYes, SIGNAL("clicked()"), self.slotStartRestart)
-        self.connect(self.radioButtonNo, SIGNAL("clicked()"), self.slotStartRestart)
-        self.connect(self.toolButton, SIGNAL("pressed()"), self.slotSearchRestartDirectory)
-        self.connect(self.checkBox, SIGNAL("clicked()"), self.slotFrozenField)
-        self.connect(self.toolButtonAdvanced, SIGNAL("pressed()"), self.slotAdvancedOptions)
+        self.radioButtonYes.clicked.connect(self.slotStartRestart)
+        self.radioButtonNo.clicked.connect(self.slotStartRestart)
+        self.radioButtonAuto.clicked.connect(self.slotStartRestart)
+        self.toolButton.pressed.connect(self.slotSearchRestartDirectory)
+        self.toolButtonRestartMesh.pressed.connect(self.slotSearchRestartMesh)
+        self.checkBox.clicked.connect(self.slotFrozenField)
+        self.toolButtonAdvanced.pressed.connect(self.slotAdvancedOptions)
+        self.checkBoxRestartMesh.stateChanged.connect(self.slotRestartMesh)
 
         self.model = StartRestartModel(self.case)
 
         # Widget initialization
 
         self.restart_path = self.model.getRestartPath()
+        self.restart_mesh_path = self.model.getRestartMeshPath()
 
         if self.restart_path:
-            if not os.path.isdir(os.path.join(self.case['case_path'],
-                                              self.restart_path)):
-                title = self.tr("WARNING")
-                msg   = self.tr("Invalid path in %s!" % self.restart_path)
-                QMessageBox.warning(self, title, msg)
+            if self.restart_path == '*':
+                self.radioButtonNo.setChecked(False)
+                self.radioButtonYes.setChecked(False)
+                self.radioButtonAuto.setChecked(True)
+            else:
+                if not os.path.isdir(os.path.join(self.case['case_path'],
+                                                  self.restart_path)):
+                    title = self.tr("WARNING")
+                    msg   = self.tr("Invalid path in %s!" % self.restart_path)
+                    QMessageBox.warning(self, title, msg)
 
-            self.radioButtonYes.setChecked(True)
-            self.radioButtonNo.setChecked(False)
+                self.radioButtonNo.setChecked(False)
+                self.radioButtonYes.setChecked(True)
+                self.radioButtonAuto.setChecked(False)
 
         else:
-            self.radioButtonYes.setChecked(False)
             self.radioButtonNo.setChecked(True)
+            self.radioButtonYes.setChecked(False)
+            self.radioButtonAuto.setChecked(False)
 
         self.slotStartRestart()
 
@@ -266,15 +279,85 @@ class StartRestartView(QWidget, Ui_StartRestartForm):
         else:
             self.checkBox.setChecked(False)
 
+        self.updateRestartTimes()
+        self.updateRestartMeshView()
+
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("")
+    def updateRestartTimes(self):
+        """
+        Update information on restart times
+        """
+
+        # FIXME: ensure correct path is used in coupling situations;
+        # currently, if in doubt, leave it empty (we prefer to have
+        # no information than false information)
+        restart_dir = None
+        restart_path = None
+
+        if self.restart_path == '*':
+            d = os.path.join(os.path.split(self.case['case_path'])[0],
+                             'RESU_COUPLING')
+            if not os.path.isdir(d):
+                restart_dir = os.path.join(self.case['case_path'], 'RESU')
+
+            # isdir returns an error if restart_dir == None, hence the test
+            if restart_dir:
+                if os.path.isdir(restart_dir):
+                    restart_path = '*'
+                else:
+                    restart_dir = None
+        elif self.restart_path:
+            if os.path.isabs(self.restart_path):
+                restart_path = self.restart_path
+            else:
+                restart_path = os.path.join(self.case['case_path'],
+                                            self.restart_path)
+
+        rinfo = getRestartInfo(self.case['package'],
+                               restart_dir,
+                               restart_path)
+
+        self.lineEdit.setEnabled(self.restart_path != '*')
+        self.lineEdit.setFrame(self.restart_path != '*')
+
+        if rinfo:
+            self.lineEdit.setText(rinfo[0])
+            self.labelIteration.show()
+            self.labelTime.show()
+            self.lineEditIteration.show()
+            self.lineEditTime.show()
+            self.lineEditIteration.setText(str(rinfo[1]))
+            self.lineEditTime.setText(str(rinfo[2]))
+        else:
+            self.labelIteration.hide()
+            self.labelTime.hide()
+            self.lineEditIteration.hide()
+            self.lineEditTime.hide()
+
+
+    def updateRestartMeshView(self):
+        """
+        Upate restart mesh path view
+        """
+        if self.restart_mesh_path:
+            self.checkBoxRestartMesh.setChecked(True)
+            self.lineEditRestartMesh.setText(self.restart_mesh_path)
+            self.lineEditRestartMesh.show()
+            self.toolButtonRestartMesh.show()
+        else:
+            self.checkBoxRestartMesh.setChecked(False)
+            self.lineEditRestartMesh.setText("")
+            self.lineEditRestartMesh.hide()
+            self.toolButtonRestartMesh.hide()
+
+
+    @pyqtSlot()
     def slotSearchRestartDirectory(self):
         """
         Search restart file (directory) in list of directories
         """
-        title    = self.tr("Select checkpoint/restart directory")
 
         default = None
         l_restart_dirs = []
@@ -289,17 +372,22 @@ class StartRestartView(QWidget, Ui_StartRestartForm):
         if not default:
             default = self.case['case_path']
 
-        if hasattr(QFileDialog, 'ReadOnly'):
-            options  = QFileDialog.DontUseNativeDialog | QFileDialog.ReadOnly
-        else:
-            options  = QFileDialog.DontUseNativeDialog
+        title = self.tr("Select checkpoint/restart directory")
+        options = QFileDialog.DontUseNativeDialog | QFileDialog.ReadOnly | QFileDialog.ShowDirsOnly
 
-        dialog = QFileDialog(self, title, default)
-        if hasattr(dialog, 'setOptions'):
-            dialog.setOptions(options)
+        dialog = QFileDialog()
+        dialog.setWindowTitle(title)
+        dialog.setDirectory(default)
+
+        dialog.setOptions(options)
         dialog.setSidebarUrls(l_restart_dirs)
         dialog.setFileMode(QFileDialog.Directory)
 
+        name_filter = str(self.tr("Checkpoint directory (checkpoint*)"))
+        dialog.setNameFilter(name_filter)
+
+        dialog.setLabelText(QFileDialog.Accept, str(self.tr("Select")))
+
         if dialog.exec_() == 1:
 
             s = dialog.selectedFiles()
@@ -310,45 +398,120 @@ class StartRestartView(QWidget, Ui_StartRestartForm):
             self.restart_path = RelOrAbsPath(dir_path, self.case['case_path'])
             self.model.setRestartPath(self.restart_path)
             self.lineEdit.setText(self.restart_path)
+            self.updateRestartTimes()
 
             log.debug("slotSearchRestartDirectory-> %s" % self.restart_path)
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
+    def slotSearchRestartMesh(self):
+        """
+        Search restart mesh (file) in list of directories
+        """
+        title    = self.tr("Select checkpoint/restart mesh_input/output")
+
+        default = None
+        l_restart_dirs = []
+        for d in [os.path.join(os.path.split(self.case['case_path'])[0],
+                               'RESU_COUPLING'),
+                  os.path.join(self.case['case_path'], 'RESU')]:
+            if os.path.isdir(d):
+                l_restart_dirs.append(QUrl.fromLocalFile(d))
+                if not default:
+                    default = d
+
+        if not default:
+            default = self.case['case_path']
+
+        options  = QFileDialog.DontUseNativeDialog | QFileDialog.ReadOnly
+
+        dialog = QFileDialog()
+        dialog.setWindowTitle(title)
+        dialog.setDirectory(default)
+
+        dialog.setOptions(options)
+        dialog.setSidebarUrls(l_restart_dirs)
+        dialog.setFileMode(QFileDialog.ExistingFile)
+
+        name_filter = str(self.tr("Imported or preprocessed meshes (mesh_input mesh_output)"))
+        dialog.setNameFilter(name_filter)
+
+        dialog.setLabelText(QFileDialog.Accept, str(self.tr("Select")))
+
+        if dialog.exec_() == 1:
+
+            s = dialog.selectedFiles()
+
+            path = str(s[0])
+            path = os.path.abspath(path)
+
+            self.restart_mesh_path = RelOrAbsPath(path, self.case['case_path'])
+            self.model.setRestartMeshPath(self.restart_mesh_path)
+            self.lineEditRestartMesh.setText(self.restart_mesh_path)
+
+            log.debug("slotSearchRestartDirectory-> %s" % self.restart_mesh_path)
+
+
+    @pyqtSlot()
     def slotStartRestart(self):
         """
         Input IRESTART Code_Saturne keyword.
         """
         if self.radioButtonYes.isChecked():
-            if not self.restart_path:
+            if not self.restart_path or self.restart_path == '*':
                 self.slotSearchRestartDirectory()
-
+        elif self.radioButtonAuto.isChecked():
+            self.restart_path = '*'
         else:
             self.restart_path = None
 
         if self.restart_path:
             self.model.setRestartPath(self.restart_path)
-            self.radioButtonYes.setChecked(True)
+            if self.restart_path == '*':
+                self.radioButtonYes.setChecked(False)
+                self.radioButtonAuto.setChecked(True)
+                self.labelRestartDir.setEnabled(False)
+                self.toolButton.hide()
+                self.updateRestartTimes()
+            else:
+                self.radioButtonYes.setChecked(True)
+                self.radioButtonAuto.setChecked(False)
+                self.labelRestartDir.setEnabled(True)
+                self.toolButton.show()
             self.radioButtonNo.setChecked(False)
             self.frameRestart.show()
-            self.lineEdit.setText(self.restart_path)
-            if not os.path.isdir(os.path.join(self.case['resu_path'],
-                                              self.restart_path)):
-                setGreenColor(self.toolButton)
-            else:
-                setGreenColor(self.toolButton, False)
-
         else:
             self.model.setRestartPath(None)
+            self.model.setRestartMeshPath(None)
             self.model.setFrozenField("off")
             self.radioButtonYes.setChecked(False)
             self.radioButtonNo.setChecked(True)
-
+            self.checkBoxRestartMesh.setChecked(False)
             self.frameRestart.hide()
             self.lineEdit.setText("")
+            self.updateRestartTimes()
+
+        self.updateRestartMeshView()
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
+    def slotRestartMesh(self):
+        """
+        Input different restart mesh.
+        """
+        if self.checkBoxRestartMesh.isChecked():
+            if not self.restart_mesh_path:
+                self.slotSearchRestartMesh()
+
+        else:
+            self.restart_mesh_path = None
+
+        self.model.setRestartMeshPath(self.restart_mesh_path)
+
+        self.updateRestartMeshView()
+
+
+    @pyqtSlot()
     def slotFrozenField(self):
         """
         Input if calculation on frozen velocity and pressure fields or not
@@ -359,7 +522,7 @@ class StartRestartView(QWidget, Ui_StartRestartForm):
             self.model.setFrozenField('off')
 
 
-    @pyqtSignature("")
+    @pyqtSlot()
     def slotAdvancedOptions(self):
         """
         Ask one popup for advanced specifications
