@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -40,18 +40,19 @@ import sys, logging
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtCore import *
-from PyQt4.QtGui  import *
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Application modules import
 #-------------------------------------------------------------------------------
 
-from Base.Toolbox import GuiParam
-from Base.QtPage import ComboModel, DoubleValidator, from_qvariant
-from Pages.TurbulenceForm import Ui_TurbulenceForm
-from Pages.TurbulenceAdvancedOptionsDialogForm import Ui_TurbulenceAdvancedOptionsDialogForm
-from Pages.TurbulenceModel import TurbulenceModel
+from code_saturne.model.Common import GuiParam
+from code_saturne.Base.QtPage import ComboModel, DoubleValidator, from_qvariant
+from code_saturne.Pages.TurbulenceForm import Ui_TurbulenceForm
+from code_saturne.Pages.TurbulenceAdvancedOptionsDialogForm import Ui_TurbulenceAdvancedOptionsDialogForm
+from code_saturne.model.TurbulenceModel import TurbulenceModel
 
 #-------------------------------------------------------------------------------
 # log config
@@ -96,31 +97,52 @@ class TurbulenceAdvancedOptionsDialogView(QDialog, Ui_TurbulenceAdvancedOptionsD
         self.result  = self.default.copy()
 
         self.checkBoxGravity.setEnabled(True)
-        self.comboBoxScales.setEnabled(True)
+        self.comboBoxWallFunctions.setEnabled(True)
 
-        if default['model'] == 'Spalart-Allmaras' or \
-           default['model'] == 'v2f-BL-v2/k' or \
-           default['model'] == 'Rij-EBRSM':
-            self.modelScales = ComboModel(self.comboBoxScales, 1, 1)
-            self.modelScales.addItem(self.tr("One scale model"), '0')
-            self.checkBoxGravity.setChecked(False)
-            self.checkBoxGravity.setEnabled(False)
-            self.comboBoxScales.setEnabled(False)
+        if default['model'] == 'Rij-EBRSM':
+            # Combo - piecewise laws (iwallf=2,3) unavailable through the GUI
+            self.wallFunctions = ComboModel(self.comboBoxWallFunctions, 2, 1)
+            self.wallFunctions.addItem(self.tr("No wall function"), '0')
+            self.wallFunctions.addItem(self.tr("2-scale model (all y+)"), '7')
+
+            # Initialization of wall function model
+            self.wallFunctions.setItem(str_model=str(self.result['wall_function']))
+
+        elif default['model'] == 'v2f-BL-v2/k':
+            self.wallFunctions = ComboModel(self.comboBoxWallFunctions, 1, 1)
+            self.wallFunctions.addItem(self.tr("No wall function"), '0')
+            self.comboBoxWallFunctions.setEnabled(False)
+        elif default['model'] == 'Spalart-Allmaras':
+            self.wallFunctions = ComboModel(self.comboBoxWallFunctions, 1, 1)
+            self.wallFunctions.addItem(self.tr("One scale model (log law)"), '2')
+            self.comboBoxWallFunctions.setEnabled(False)
+        elif default['model'] == 'k-omega-SST':
+            # Combo - power law (iwallf=1) unavailable through the GUI
+            self.wallFunctions = ComboModel(self.comboBoxWallFunctions, 5, 1)
+            self.wallFunctions.addItem(self.tr("No wall function"), '0')
+            self.wallFunctions.addItem(self.tr("1-scale model (log law)"), '2')
+            self.wallFunctions.addItem(self.tr("2-scale model (log law)"), '3')
+            self.wallFunctions.addItem(self.tr("2-scale model (all y+)"), '7')
+            self.wallFunctions.addItem(self.tr("Scalable 2-scale model (log law)"), '4')
+
+            # Initialization of wall function model
+            self.wallFunctions.setItem(str_model=str(self.result['wall_function']))
         else:
-            # Combo
-            self.modelScales = ComboModel(self.comboBoxScales, 3, 1)
+            # Combo - power law (iwallf=1) unavailable through the GUI
+            self.wallFunctions = ComboModel(self.comboBoxWallFunctions, 4, 1)
+            self.wallFunctions.addItem(self.tr("No wall function"), '0')
+            self.wallFunctions.addItem(self.tr("1-scale model (log law)"), '2')
+            self.wallFunctions.addItem(self.tr("2-scale model (log law)"), '3')
+            self.wallFunctions.addItem(self.tr("Scalable 2-scale model (log law)"), '4')
 
-            self.modelScales.addItem(self.tr("One scale model"), '0')
-            self.modelScales.addItem(self.tr("Two scale model"), '1')
-            self.modelScales.addItem(self.tr("Scalable wall function"), '2')
+            # Initialization of wall function model
+            self.wallFunctions.setItem(str_model=str(self.result['wall_function']))
 
-            # Initialization
-
-            self.modelScales.setItem(str_model=str(self.result['scale_model']))
-            if self.result['gravity_terms'] == 'on':
-                self.checkBoxGravity.setChecked(True)
-            else:
-                self.checkBoxGravity.setChecked(False)
+        # Initialization gravity terms
+        if self.result['gravity_terms'] == 'on':
+            self.checkBoxGravity.setChecked(True)
+        else:
+            self.checkBoxGravity.setChecked(False)
 
         self.case.undoStartGlobal()
 
@@ -140,8 +162,8 @@ class TurbulenceAdvancedOptionsDialogView(QDialog, Ui_TurbulenceAdvancedOptionsD
             self.result['gravity_terms'] = "on"
         else:
             self.result['gravity_terms'] = "off"
-        self.result['scale_model'] = \
-        int(self.modelScales.dicoV2M[str(self.comboBoxScales.currentText())])
+        self.result['wall_function'] = \
+        int(self.wallFunctions.dicoV2M[str(self.comboBoxWallFunctions.currentText())])
 
         QDialog.accept(self)
 
@@ -188,7 +210,7 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
         self.modelTurbModel.addItem(self.tr("Mixing length"), "mixing_length")
         self.modelTurbModel.addItem(self.tr("k-epsilon"), "k-epsilon")
         self.modelTurbModel.addItem(self.tr("k-epsilon Linear Production"), "k-epsilon-PL")
-        self.modelTurbModel.addItem(self.tr("Rij-epsilon LLR"), "Rij-epsilon")
+        self.modelTurbModel.addItem(self.tr("Rij-epsilon LRR"), "Rij-epsilon")
         self.modelTurbModel.addItem(self.tr("Rij-epsilon SSG"), "Rij-SSG")
         self.modelTurbModel.addItem(self.tr("Rij-epsilon EBRSM"), "Rij-EBRSM")
         self.modelTurbModel.addItem(self.tr("v2f BL-v2/k"), "v2f-BL-v2/k")
@@ -198,23 +220,37 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
         self.modelTurbModel.addItem(self.tr("LES (classical dynamic model)"), "LES_dynamique")
         self.modelTurbModel.addItem(self.tr("LES (WALE)"), "LES_WALE")
 
+        self.modelLength = ComboModel(self.comboBoxLength,2,1)
+        self.modelLength.addItem(self.tr("Automatic"), 'automatic')
+        self.modelLength.addItem(self.tr("Prescribed"), 'prescribed')
+        self.comboBoxLength.setSizeAdjustPolicy(QComboBox.AdjustToContents)
+
         # Connections
 
-        self.connect(self.comboBoxTurbModel, SIGNAL("activated(const QString&)"), self.slotTurbulenceModel)
-        self.connect(self.pushButtonAdvanced, SIGNAL("clicked()"), self.slotAdvancedOptions)
-        self.connect(self.lineEditLength, SIGNAL("textChanged(const QString &)"), self.slotLengthScale)
+        self.comboBoxTurbModel.activated[str].connect(self.slotTurbulenceModel)
+        self.pushButtonAdvanced.clicked.connect(self.slotAdvancedOptions)
+        self.lineEditLength.textChanged[str].connect(self.slotLengthScale)
+
+        self.lineEditV0.textChanged[str].connect(self.slotVelocity)
+        self.comboBoxLength.activated[str].connect(self.slotLengthChoice)
+        self.lineEditL0.textChanged[str].connect(self.slotLength)
 
         # Frames display
 
         self.frameAdvanced.hide()
         self.frameLength.hide()
 
-        # Validator
+        # Validators
 
         validator = DoubleValidator(self.lineEditLength, min=0.0)
         validator.setExclusiveMin(True)
         self.lineEditLength.setValidator(validator)
 
+        validatorV0 = DoubleValidator(self.lineEditV0, min=0.0)
+        self.lineEditV0.setValidator(validatorV0)
+
+        validatorL0 = DoubleValidator(self.lineEditL0, min=0.0)
+        self.lineEditL0.setValidator(validatorL0)
 
         # Update the turbulence models list with the calculation features
 
@@ -226,28 +262,73 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
 
         model = self.model.getTurbulenceModel()
         self.modelTurbModel.setItem(str_model=model)
-        self.slotTurbulenceModel(self.comboBoxTurbModel.currentText())
+        self.__initializeView()
 
         # Length scale
 
         l_scale = self.model.getLengthScale()
         self.lineEditLength.setText(str(l_scale))
 
+        # Initialization
+
+        v = self.model.getVelocity()
+        self.lineEditV0.setText(str(v))
+
+        init_length_choice = self.model.getLengthChoice()
+        self.modelLength.setItem(str_model=init_length_choice)
+        if init_length_choice == 'automatic':
+            self.lineEditL0.setText(str())
+            self.lineEditL0.hide()
+            self.labelUnitL0.hide()
+        else:
+            self.lineEditL0.show()
+            self.labelUnitL0.show()
+            l = self.model.getLength()
+            self.lineEditL0.setText(str(l))
+
         self.case.undoStartGlobal()
 
 
-    @pyqtSignature("const QString&")
+    def __initializeView(self):
+        """
+        Private Method.
+        initalize view for a turbulence model
+        """
+        model = self.model.getTurbulenceModel()
+
+        self.frameAdvanced.hide()
+        self.frameLength.hide()
+        self.groupBoxReferenceValues.hide()
+
+        if model not in ('off', 'mixing_length', 'LES_Smagorinsky', 'LES_dynamique', 'LES_WALE'):
+            self.groupBoxReferenceValues.show()
+
+        if model == 'mixing_length':
+            self.frameLength.show()
+            self.frameAdvanced.hide()
+            self.model.getLengthScale()
+        elif model not in ('off', 'LES_Smagorinsky', 'LES_dynamique', 'LES_WALE', 'Spalart-Allmaras'):
+            self.frameLength.hide()
+            self.frameAdvanced.show()
+
+        if model in ('off', 'LES_Smagorinsky', 'LES_dynamique', 'LES_WALE', 'Spalart-Allmaras'):
+            self.line.hide()
+        else:
+            self.line.show()
+
+
+    @pyqtSlot(str)
     def slotLengthScale(self, text):
         """
         Private slot.
         Input XLOMLG.
         """
-        if self.sender().validator().state == QValidator.Acceptable:
+        if self.lineEditLength.validator().state == QValidator.Acceptable:
             l_scale = from_qvariant(text, float)
             self.model.setLengthScale(l_scale)
 
 
-    @pyqtSignature("const QString&")
+    @pyqtSlot(str)
     def slotTurbulenceModel(self, text):
         """
         Private slot.
@@ -255,25 +336,51 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
         """
         model = self.modelTurbModel.dicoV2M[str(text)]
         self.model.setTurbulenceModel(model)
+        self.__initializeView()
 
-        self.frameAdvanced.hide()
-        self.frameLength.hide()
+    @pyqtSlot(str)
+    def slotVelocity(self,  text):
+        """
+        Private slot.
+        Input reference velocity.
+        """
+        if self.lineEditV0.validator().state == QValidator.Acceptable:
+            v = from_qvariant(text, float)
+            self.model.setVelocity(v)
 
-        if model == 'mixing_length':
-            self.frameLength.show()
-            self.frameAdvanced.hide()
-            self.model.getLengthScale()
-        elif model not in ('off', 'LES_Smagorinsky', 'LES_dynamique', 'LES_WALE'):
-            self.frameLength.hide()
-            self.frameAdvanced.show()
 
-        if model in ('off', 'LES_Smagorinsky', 'LES_dynamique', 'LES_WALE'):
-            self.line.hide()
+    @pyqtSlot(str)
+    def slotLengthChoice(self,text):
+        """
+        Private slot.
+        Input mode for reference length.
+        """
+        choice = self.modelLength.dicoV2M[str(text)]
+        self.model.setLengthChoice(choice)
+        if choice == 'automatic':
+            self.lineEditL0.setText(str())
+            self.lineEditL0.hide()
+            self.labelUnitL0.hide()
         else:
-            self.line.show()
+            self.lineEditL0.show()
+            self.labelUnitL0.show()
+            value = self.model.getLength()
+            self.lineEditL0.setText(str(value))
+        log.debug("slotlengthchoice-> %s" % choice)
 
 
-    @pyqtSignature("")
+    @pyqtSlot(str)
+    def slotLength(self,  text):
+        """
+        Private slot.
+        Input reference length.
+        """
+        if self.lineEditL0.validator().state == QValidator.Acceptable:
+            l = from_qvariant(text, float)
+            self.model.setLength(l)
+
+
+    @pyqtSlot()
     def slotAdvancedOptions(self):
         """
         Private slot.
@@ -281,7 +388,7 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
         """
         default = {}
         default['model']         = self.model.getTurbulenceModel()
-        default['scale_model']   = self.model.getScaleModel()
+        default['wall_function'] = self.model.getWallFunction()
         default['gravity_terms'] = self.model.getGravity()
         log.debug("slotAdvancedOptions -> %s" % str(default))
 
@@ -290,7 +397,7 @@ class TurbulenceView(QWidget, Ui_TurbulenceForm):
             result = dialog.get_result()
             log.debug("slotAdvancedOptions -> %s" % str(result))
             self.model.setTurbulenceModel(result['model'])
-            self.model.setScaleModel(result['scale_model'])
+            self.model.setWallFunction(result['wall_function'])
             self.model.setGravity(result['gravity_terms'])
 
 
