@@ -1,6 +1,6 @@
 # Shell script
 
-# Copyright (C) 2005-2013 EDF
+# Copyright (C) 2005-2017 EDF
 
 # This file is part of the PLE software package.  For license
 # information, see the COPYING file in the top level directory of the
@@ -15,14 +15,14 @@
 # cflags_default         # Base CFLAGS                       (default: "")
 # cflags_default_dbg     # Added to $CFLAGS for debugging    (default: "-g")
 # cflags_default_opt     # Added to $CFLAGS for optimization (default: "-O")
-# cflags_default_prf     # Added to $CFLAGS for profiling    (default: "")
+# cflags_default_prf     # Added to $CFLAGS for profiling    (default: "-g")
+# cflags_default_omp     # Added to $CFLAGS for OpenMP       (default: "")
 #
 # ldflags_default        # Base LDFLAGS                       (default: "")
 # ldflags_default_dbg    # Added to $LDFLAGS for debugging    (default: "-g")
 # ldflags_default_opt    # Added to $LDFLAGS for optimization (default: "-O")
-# ldflags_default_prf    # Added to $LDFLAGS for profiling    (default: "")
-#
-# ple_disable_shared     # Disable shared librairies          (default: "")
+# ldflags_default_prf    # Added to $LDFLAGS for profiling    (default: "-g")
+# ldflags_rpath          # Added to $LDFLAGS for shared libs  (default: "")
 
 # Two other environment variable strings are defined, containing possibly
 # more detailed compiler information:
@@ -44,8 +44,6 @@
 
 outfile=ple_ac_cc_env-tmp
 
-ple_ac_cc_version=unknown
-
 # Some compilers require a file to compile even for version info.
 
 cat > conftest.c <<\_______EOF
@@ -69,6 +67,12 @@ case "$host_os" in
     ;;
 esac
 
+cflags_default_prf="-g"
+ldflags_default_prf="-g"
+
+ple_ac_cc_version=unknown
+ple_cc_compiler_known=no
+
 # Are we using gcc ?
 #-------------------
 
@@ -76,13 +80,15 @@ ple_gcc=no
 
 if test "x$GCC" = "xyes"; then
 
-  # Intel compiler passes as GCC but may be recognized by version string
+  # Intel and Pathscale compilers may pass as GCC but
+  # may be recognized by version string
+
   if test -n "`$CC --version | grep icc`" ; then
     ple_gcc=icc
+  elif test -n "`$CC --version | grep clang`" ; then
+    ple_gcc=clang
   elif test -n "`$CC --version 2>&1 | grep PathScale`" ; then
     ple_gcc=pathcc
-  elif test -n "`$CC --version 2>&1 | grep Open64`" ; then
-    ple_gcc=open64
   else
     ple_gcc=gcc
   fi
@@ -94,7 +100,7 @@ if test "x$ple_gcc" = "xgcc"; then
   # Version strings for logging purposes and known compiler flag
   $CC -v > $outfile 2>&1
   ple_ac_cc_version=`$CC --version 2>&1 | head -1`
-  ple_compiler_known=yes
+  ple_cc_compiler_known=yes
 
   # Practical version info for option setting
   ple_cc_version="`$CC -v 2>&1 |grep 'gcc version' |\
@@ -121,7 +127,7 @@ if test "x$ple_gcc" = "xgcc"; then
   cflags_default="-std=c99 -funsigned-char -pedantic -W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wunused"
   cflags_default_dbg="-g"
   cflags_default_opt="-O2"
-  cflags_default_prf="-pg"
+  cflags_default_omp="-fopenmp"
 
   # Modify default flags on certain systems
 
@@ -131,11 +137,7 @@ if test "x$ple_gcc" = "xgcc"; then
       cflags_default_opt="-funroll-loops -O2 -Wuninitialized"
       case "$host_cpu" in
         i686)
-          case "$ple_cc_vendor-$ple_cc_version" in
-            gcc-3*|gcc-4*)
-              cflags_default_opt="$cflags_default_opt -march=i686"
-              ;;
-          esac
+          cflags_default_opt="$cflags_default_opt -march=i686"
           ;;
       esac
       ;;
@@ -146,15 +148,29 @@ if test "x$ple_gcc" = "xgcc"; then
   # may not handle all flags)
 
   case "$ple_cc_vendor-$ple_cc_version" in
-    gcc-3.*|gcc-4.*)
-      cflags_default="`echo $cflags_default | sed -e 's/-ansi/-std=c99/g'`"
-      cflags_default="$cflags_default -Wfloat-equal"
+    gcc-[45]*)
+      ;;
+    *)
+      cflags_default="$cflags_default -Wmisleading-indentation -Wduplicated-cond"
       ;;
-
   esac
 
   case "$ple_cc_vendor-$ple_cc_version" in
-    gcc-3*|gcc-4.[01234]*)
+    gcc-4.[012345678]*)
+      ;;
+    *)
+      cflags_default="$cflags_default -fdiagnostics-color=auto"
+      ;;
+  esac
+
+  case "$ple_cc_vendor-$ple_cc_version" in
+    gcc-4.[012]*)
+      cflags_default_omp=""
+      ;;
+  esac
+
+  case "$ple_cc_vendor-$ple_cc_version" in
+    gcc-4.[01234]*)
       ;;
     *)
       cflags_default_opt="$cflags_default_opt -fexcess-precision=fast"
@@ -180,18 +196,51 @@ elif test "x$ple_gcc" = "xicc"; then
   # Version strings for logging purposes and known compiler flag
   $CC -V conftest.c > $outfile 2>&1
   ple_ac_cc_version=`$CC --version 2>&1 | head -1`
-  ple_compiler_known=yes
+  ple_cc_compiler_known=yes
+
+  # Some version numbers
+  ple_cc_vers_major=`echo $ple_ac_cc_version | cut -f 3 -d" " | cut -f1 -d.`
+  ple_cc_vers_minor=`echo $ple_ac_cc_version | cut -f 3 -d" " | cut -f2 -d.`
+  ple_cc_vers_patch=`echo $ple_ac_cc_version | cut -f 3 -d" " | cut -f3 -d.`
+  test -n "$ple_cc_vers_major" || ple_cc_vers_major=0
+  test -n "$ple_cc_vers_minor" || ple_cc_vers_minor=0
+  test -n "$ple_cc_vers_patch" || ple_cc_vers_patch=0
 
   # Default compiler flags
   cflags_default="-strict-ansi -std=c99 -funsigned-char -Wall -Wcheck -Wshadow -Wpointer-arith -Wmissing-prototypes -Wuninitialized -Wunused"
   cflags_default_dbg="-g -O0 -traceback -w2 -Wp64 -ftrapuv"
   cflags_default_opt="-O2"
-  cflags_default_prf="-p"
+  cflags_default_omp="-qopenmp"
+  case "$ple_cc_vers_major" in
+    1[0123456])
+      cflags_default_omp="-openmp"
+      ;;
+  esac
+
+# Otherwise, are we using clang ?
+#--------------------------------
+
+elif test "x$ple_gcc" = "xclang"; then
+
+  ple_cc_version=`echo $CC --version | grep clang | cut -f 3 -d ' '`
+
+  echo "compiler '$CC' is clang"
+
+  # Version strings for logging purposes and known compiler flag
+  $CC -v > $outfile 2>&1
+  ple_ac_cc_version=`$CC --version 2>&1 | head -1`
+  ple_cc_compiler_known=yes
+
+  # Default compiler flags
+  cflags_default="-std=c99 -funsigned-char -Wall -Wshadow -Wpointer-arith -Wmissing-prototypes -Wuninitialized -Wunused"
+  cflags_default_dbg="-g -O0"
+  cflags_default_opt="-O2"
+  cflags_default_omp="-openmp"
 
 # Otherwise, are we using pathcc ?
 #---------------------------------
 
-elif test "x$ple_cc_compiler_known" != "xyes" ; then
+elif test "x$ple_gcc" = "xpathcc"; then
 
   $CC --version 2>&1 | grep 'PathScale' > /dev/null
   if test "$?" = "0" ; then
@@ -208,31 +257,7 @@ elif test "x$ple_cc_compiler_known" != "xyes" ; then
     cflags_default="-std=c99 -funsigned-char -W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wunused -Wunused-value"
     cflags_default_dbg="-g"
     cflags_default_opt="-O2"
-    cflags_default_prf=""
-
-  fi
-
-# Otherwise, are we using opencc ?
-#---------------------------------
-
-elif test "x$ple_gcc" = "xopen64"; then
-
-  $CC --version 2>&1 | grep 'Open64' > /dev/null
-  if test "$?" = "0" ; then
-
-    echo "compiler '$CC' is Open64 C compiler"
-
-    # Version strings for logging purposes and known compiler flag
-    $CC --version > $outfile 2>&1
-    ple_ac_cc_version=`grep -i Compiler $outfile`
-    ple_cc_compiler_known=yes
-
-    # Default compiler flags
-    cflags_default="-std=c99"
-    cflags_default="-std=c99 -funsigned-char -W -Wall -Wshadow -Wpointer-arith -Wcast-qual -Wcast-align -Wwrite-strings -Wstrict-prototypes -Wmissing-prototypes -Wmissing-declarations -Wnested-externs -Wunused -Wunused-value"
-    cflags_default_dbg="-g"
-    cflags_default_opt="-O2"
-    cflags_default_prf=""
+    cflags_default_omp="-openmp"
 
   fi
 
@@ -241,7 +266,7 @@ fi
 # Otherwise, are we using pgcc ?
 #-------------------------------
 
-if test "x$ple_compiler_known" != "xyes" ; then
+if test "x$ple_cc_compiler_known" != "xyes" ; then
 
   $CC -V 2>&1 | grep 'The Portland Group' > /dev/null
   if test "$?" = "0" ; then
@@ -251,13 +276,13 @@ if test "x$ple_compiler_known" != "xyes" ; then
     # Version strings for logging purposes and known compiler flag
     $CC -V > $outfile 2>&1
     ple_ac_cc_version=`grep -i pgcc $outfile`
-    ple_compiler_known=yes
+    ple_cc_compiler_known=yes
 
     # Default compiler flags
-    cflags_default="-c99"
+    cflags_default="-c99 -noswitcherror"
     cflags_default_dbg="-g -Mbounds"
     cflags_default_opt="-O2"
-    cflags_default_prf="-Mprof=func,lines"
+    cflags_default_omp="-mp"
 
   fi
 
@@ -266,7 +291,7 @@ fi
 # Otherwise, are we using xlc ?
 #------------------------------
 
-if test "x$ple_compiler_known" != "xyes" ; then
+if test "x$ple_cc_compiler_known" != "xyes" ; then
 
   $CC -qversion 2>&1 | grep 'XL C' > /dev/null
   if test "$?" = "0" ; then
@@ -276,46 +301,36 @@ if test "x$ple_compiler_known" != "xyes" ; then
     # Version strings for logging purposes and known compiler flag
     $CC -qversion > $outfile 2>&1
     ple_ac_cc_version=`grep 'XL C' $outfile`
-    ple_compiler_known=yes
+    ple_cc_compiler_known=yes
     ple_linker_set=yes
 
     # Default compiler flags
     cflags_default="-qlanglvl=stdc99 -q64"
-    cflags_default_opt="-O2"
-    cflags_default_dbg="-g"
-    cflags_default_prf="-pg"
+    cflags_default_opt="-O3"
+    cflags_default_dbg="-g -qfullpath"
+    cflags_default_omp="-qsmp=omp -qthreaded"
 
     # Default  linker flags
     ldflags_default=""
-    ldflags_default_opt="-O2"
-    ldflags_default_dbg="-g"
-    ldflags_default_prf="-pg"
+    ldflags_default_opt="-O3"
+    ldflags_default_dbg="-g -qfullpath"
 
     # Adjust options for IBM Blue Gene cross-compiler
 
     grep 'Blue Gene' $outfile > /dev/null
     if test "$?" = "0" ; then
-      # Default compiler flags
+      # Default compiler flags (we assume that MPI wrappers are used)
       ple_ibm_bg_type=`grep 'Blue Gene' $outfile | sed -e 's/.*Blue Gene\/\([A-Z]\).*/\1/'`
-      if test "$ple_ibm_bg_type" = "L" ; then
-        cppflags_default="-I/bgl/BlueLight/ppcfloor/bglsys/include"
-        cflags_default="-g -qmaxmem=-1 -qarch=440d -qtune=440"
-        cflags_default_opt="-O2"
-        cflags_default_dbg=""
-        ldflags_default="-L/bgl/BlueLight/ppcfloor/bglsys/lib -lmpich.rts -lmsglayer.rts -lrts.rts -ldevices.rts -lnss_files -lnss_dns -lresolv"
-        ple_disable_shared=yes # Disable shared libraries
-      elif test "$ple_ibm_bg_type" = "P" ; then
-        cppflags_default="-I/bgsys/drivers/ppcfloor/comm/include"
-        cflags_default="-g -qmaxmem=-1 -qarch=450d -qtune=450"
-        cflags_default_opt="-O1"
-        cflags_default_dbg=""
-        ldflags_default=""
-        ple_disable_shared=yes # Disable shared libraries
-      else
+      if test "x$ple_ibm_bg_type" = "xP" ; then
+        cppflags_default="-I/bgsys/drivers/ppcfloor/arch/include"
+        cflags_default="-qlanglvl=extc99"
+        cflags_default_opt="-O3"
+        cflags_default_dbg="-g"
+      elif test -f /bgsys/drivers/ppcfloor/cnk/bin/bgq_kernel.elf ; then
         ple_ibm_bg_type="Q"
         cppflags_default=""
-        cflags_default=""
-        cflags_default_opt="-O3"
+        cflags_default=""                    # "-qlanglvl=extc99" by default
+        cflags_default_opt="-g -O3"
         cflags_default_dbg="-g"
       fi
     fi
@@ -342,13 +357,12 @@ if test "x$ple_cc_compiler_known" != "xyes" ; then
     cflags_default=""                        # "-h c99" by default
     cflags_default_opt="-O2"
     cflags_default_dbg="-g"
-    cflags_default_prf="-h profile_generate" # resulting code must be run under CrayPat
+    cflags_default_omp="-h omp"              # default: use "-h noomp" to disable
 
     # Default  linker flags
     ldflags_default=""
     ldflags_default_opt="-O2"
     ldflags_default_dbg="-g"
-    ldflags_default_prf="-h profile_generate"
 
   fi
 fi
@@ -356,100 +370,36 @@ fi
 # Compiler still not identified
 #------------------------------
 
-if test "x$ple_compiler_known" != "xyes" ; then
+if test "x$ple_cc_compiler_known" != "xyes" ; then
 
   case "$host_os" in
 
     SUPER-UX* | superux*)
 
-      # Native NEC SX vectorizing C compiler (sxmpicc)
-      #-------------------------------------
+     # Native NEC SX vectorizing C compiler (sxmpicc)
+     #-------------------------------------
 
-      $CC -V conftest.c 2>&1 | grep 'NEC' | grep 'SX' > /dev/null
-      if test "$?" = "0" ; then
+     $CC -V conftest.c 2>&1 | grep 'NEC' | grep 'SX' > /dev/null
+     if test "$?" = "0" ; then
 
-        echo "compiler '$CC' is NEC SX compiler"
+       echo "compiler '$CC' is NEC SX compiler"
 
-        # Version strings for logging purposes and known compiler flag
-        $CC -V conftest.c > $outfile 2>&1
-        ple_ac_cc_version=`grep ccom $outfile`
-        ple_compiler_known=yes
-        ple_linker_set=yes
+       # Version strings for logging purposes and known compiler flag
+       $CC -V conftest.c > $outfile 2>&1
+       ple_ac_cc_version=`grep ccom $outfile`
+       ple_cc_compiler_known=yes
+       ple_linker_set=yes
 
-        # Default compiler flags
-        cflags_default="-Kc99 -pvctl,loopcnt=2147483647"
-        cflags_default_opt=""
-        cflags_default_dbg=""
-        cflags_default_prf=""
+       # Default compiler flags
+       cflags_default="-Kc99 -pvctl,loopcnt=2147483647"
+       cflags_default_opt=""
+       cflags_default_dbg=""
+       cflags_default_omp=""
 
-        # Default linker flags
-        ldflags_default=""
-        ldflags_default_opt="-O"
-        ldflags_default_dbg="-g"
-        ldflags_default_prf="-pg"
-
-      fi
-      ;;
-
-    hpux*)
-
-      # Native HP-UX C compiler
-      #------------------------
-
-      $CC -V conftest.c 2>&1 | grep 'HP' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$CC' is HP compiler"
-
-        # Version strings for logging purposes and known compiler flag
-        $CC -V conftest.c > $outfile 2>&1
-        ple_ac_cc_version=`grep ccom $outfile`
-        ple_compiler_known=yes
-        ple_linker_set=yes
-
-        # Default compiler flags
-        cflags_default="-AC99 +e"
-        cflags_default_opt="+O2"
-        cflags_default_dbg="-g"
-        cflags_default_prf="-G" # -G for gprof, -p for prof
-
-        # Default linker flags
-        ldflags_default="+FPVZOUD +U77"
-        ldflags_default_opt="+O1"
-        ldflags_default_dbg="-g"
-        ldflags_default_prf="-fbexe"
-
-        if test "$host_cpu" = "ia64" ; then
-          cflags_default="$cflags_default +DD64"
-          ldflags_default="$ldflags_default +DD64"
-        else
-          cflags_default="$cflags_default +DA2.0w"
-          ldflags_default="$ldflags_default +DA2.0w"
-        fi
-
-      fi
-      ;;
-
-    solaris2.*)
-
-      # Sun Workshop compiler
-      #----------------------
-
-      $CC -V 2>&1 | grep 'WorkShop Compilers' > /dev/null
-      if test "$?" = "0" ; then
-
-        echo "compiler '$CC' is Sun WorkShop Compilers"
-
-        # Version strings for logging purposes and known compiler flag
-        $CC -V conftest.c > $outfile 2>&1
-        ple_ac_cc_version=`grep cc $outfile`
-        ple_compiler_known=yes
-
-        # Default compiler flags
-        cflags_default="-Xa -Xc99"
-        cflags_default_opt="-xO2"
-        cflags_default_dbg="-g"
-        cflags_default_prf="-pg"
+       # Default linker flags
+       ldflags_default=""
+       ldflags_default_opt="-O"
+       ldflags_default_dbg="-g"
 
      fi
      ;;
@@ -462,14 +412,19 @@ if test "x$ple_compiler_known" != "xyes" ; then
       cflags_default=""
       cflags_default_opt="-O"
       cflags_default_dbg="-g"
-      cflags_default_prf=""
-
+      cflags_default_omp=""
       ;;
 
   esac
 
 fi
 
+
+if test -f $outfile ; then 
+  ple_ac_cc_version_full=`sed -e '11,$d' $outfile`
+  rm -f $outfile
+fi
+
 # Default linker flags
 #---------------------
 
@@ -481,20 +436,12 @@ if test "x$ple_linker_set" != "xyes" ; then
       ldflags_default=""
       ldflags_default_opt="-O"
       ldflags_default_dbg="-g"
-      ldflags_default_prf="-pg"
-      ;;
-
-    solaris2.*)
-      ldflags_default_opt=""
-      ldflags_default_dbg="-g"
-      ldflags_default_prf=""
+      ldflags_rpath="-Wl,-rpath -Wl,"
       ;;
 
     *)
-      ldflags_default=""
       ldflags_default_opt="-O"
       ldflags_default_dbg="-g"
-      ldflags_default_prf="-pg"
       ;;
 
   esac
@@ -505,10 +452,6 @@ fi
 
 export LANG=$save_LANG
 
-if test -f $outfile ; then 
-  ple_ac_cc_version_full=`sed -e '11,$d' $outfile`
-fi
-
 # Clean temporary files
 
 rm -f conftest* a.out $outfile
