@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -305,7 +305,6 @@ _maillage__extrait_coords(ecs_maillage_t         *maillage_new,
 {
   size_t  cpt_elt_new;
   size_t  cpt_val_new;
-  size_t  cpt_val_old_new;
   size_t  nbr_elt_ref;
   size_t  nbr_val_ref;
   size_t  pos_ref_inf;
@@ -329,7 +328,6 @@ _maillage__extrait_coords(ecs_maillage_t         *maillage_new,
 
   cpt_elt_new     = 0;
   cpt_val_new     = 0;
-  cpt_val_old_new = 0;
 
   for (ielt_ref = 0; ielt_ref < nbr_elt_ref; ielt_ref++) {
 
@@ -701,7 +699,7 @@ _maillage__liste_fac_err(const ecs_tab_int_t  *typ_fac,
   /*------------------------------------------*/
 
   for (ifac = 0; ifac < typ_fac->nbr; ifac++) {
-    if (typ_fac->val[ifac] == 4)
+    if (typ_fac->val[ifac] >= 4)
       cpt_fac_erreur++;
   }
 
@@ -716,7 +714,7 @@ _maillage__liste_fac_err(const ecs_tab_int_t  *typ_fac,
   cpt_fac_erreur  = 0;
 
   for (ifac = 0; ifac < typ_fac->nbr; ifac++) {
-    if (typ_fac->val[ifac] == 4)
+    if (typ_fac->val[ifac] >= 4)
       liste_fac_erreur->val[cpt_fac_erreur++] = ifac;
   }
 }
@@ -728,7 +726,7 @@ _maillage__liste_fac_err(const ecs_tab_int_t  *typ_fac,
  *  et sortante pour une cellule) et de faces isolées.
  *
  *  Un tableau indiquant le type associé à chaque face (0 pour face isolée,
- *  1 ou 2 pour face de bord, 3 pour face interne, et 4 pour tous les autres
+ *  1 ou 2 pour face de bord, 3 pour face interne, et >= 4 pour tous les autres
  *  cas (faces voyant au moins deux cellules sur un même côté, d'ou erreur
  *  de connectivité) doit être fourni en entrée.
  *----------------------------------------------------------------------------*/
