@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -1191,7 +1191,6 @@ ecs_table__extrait(ecs_table_t  *table_ref,
 {
   size_t        cpt_elt_new;
   size_t        cpt_val_new;
-  size_t        cpt_val_old_new;
   size_t        nbr_elt_ref;
   size_t        nbr_val_ref;
   size_t        pos_ref_inf;
@@ -1220,7 +1219,6 @@ ecs_table__extrait(ecs_table_t  *table_ref,
   table_new->pos[0] = 1;
   cpt_elt_new           = 0;
   cpt_val_new           = 0;
-  cpt_val_old_new       = 0;
 
   for (ielt_ref = 0; ielt_ref < nbr_elt_ref; ielt_ref++) {
 
