@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -441,6 +441,8 @@ _maillage_ncs__cree_fam(const ecs_maillage_t    *maillage,
 
   } /* Fin : si affichage de la famille par défaut */
 
+  ECS_FREE(cpt_elt_fam);
+
   *nbr_fam = cpt_fam;
   if (bool_cree_fam_par_defaut == true)
     *nbr_fam += 1;
@@ -547,8 +549,8 @@ ecs_maillage_ncs__ecr(const char      *output,
 {
   size_t          n_cells, n_faces, n_vertices, face_vertices_size;
 
-  int             nbr_fam;
-  int             nbr_max_propr;
+  int             nbr_fam, nbr_max_propr;
+  ecs_int_t       int_tmp_comm;
 
   size_t          ind_grp;
   ecs_int_t      *pos_nom_grp;
@@ -686,19 +688,23 @@ ecs_maillage_ncs__ecr(const char      *output,
 
   /* Écriture du nombre de familles (on ne compte pas la famille `0') */
 
+  int_tmp_comm = nbr_fam;  /* int type may differ */
+
   ecs_comm_write_section("n_group_classes",
                          1,
                          0, 0, 1, true,
-                         &nbr_fam,
+                         &int_tmp_comm,
                          ECS_TYPE_ecs_int_t,
                          comm);
 
   /* Écriture du nombre de propriétés max. des familles */
 
+  int_tmp_comm = nbr_max_propr;  /* int type may differ */
+
   ecs_comm_write_section("n_group_class_props_max",
                          1,
                          0, 0, 1, true,
-                         &nbr_max_propr,
+                         &int_tmp_comm,
                          ECS_TYPE_ecs_int_t,
                          comm);
 
