@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -300,8 +300,6 @@ ecs_maillage_post__ecr_fac_liste(const char           *nom_liste,
 {
   ecs_maillage_t  * maillage_extrait;
 
-  ecs_tab_char_t    liste_groupe;
-
   /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/
 
   assert(maillage != NULL);
@@ -309,9 +307,6 @@ ecs_maillage_post__ecr_fac_liste(const char           *nom_liste,
   /* Initialisations */
   /*-----------------*/
 
-  liste_groupe.nbr = 0;
-  liste_groupe.val = NULL;
-
   maillage_extrait = NULL;
 
   /* Si aucun post-traitement, rien à faire */
