@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -194,7 +194,7 @@ ecs_loc_pre_ens__ouverture_fic_geo(const char  *nom_fic_geo)
   /* Si le fichier est binaire, on vérifiera plus tard l'aspect
      "big-endian/little-endian" */
 
-  else if (strncmp(chaine, "C Binary", strlen("C Binary") != 0)) {
+  else if (strncmp(chaine, "C Binary", strlen("C Binary")) != 0) {
 
     ecs_file_set_type(fic, ECS_FILE_TYPE_TEXT);
 
@@ -869,9 +869,10 @@ ecs_loc_pre_ens__ele_lin(int32_t     *connect_loc,
     for (iloc = 0; iloc < taille_ele_lin; iloc++)
       connect_loc[ipos_lin++] = connect_loc[ipos_ens++];
 
-    ECS_REALLOC(connect_loc, ipos_lin, int32_t);
   }
 
+  ECS_REALLOC(connect_loc, ipos_lin, int32_t);
+
   return connect_loc;
 }
 
@@ -894,7 +895,6 @@ ecs_loc_pre_ens__lit_elem_gold(ecs_file_t                   *fic,
   int            nbr_som_elt;
 
   ecs_int_t      ind;
-  ecs_int_t      nbr_som_elt_lin;
   ecs_int_t      taille_connect;
   ecs_int_t      taille_lect;
 
@@ -904,6 +904,7 @@ ecs_loc_pre_ens__lit_elem_gold(ecs_file_t                   *fic,
 
   ecs_loc_elems_ens_t  *elems_loc;
 
+  ecs_int_t  nbr_som_elt_lin = 0;
   int32_t    nbr_elt_loc = 0;
   int32_t  * nbr_n_loc = NULL;
   int32_t  * nbr_f_loc = NULL;
@@ -1709,11 +1710,12 @@ ecs_loc_pre_ens__concat_elems(ecs_maillage_t         *maillage,
   if (liste_noeuds->id_noeud != NULL) {
 
     for (ient = ECS_ENTMAIL_FAC ; ient < ECS_N_ENTMAIL ; ient++)
-      ecs_maillage_pre__label_en_indice
-        (liste_noeuds->nbr_noeuds,
-         elt_pos_som_ent[ient][cpt_elt_ent[ient]] - 1,
-         liste_noeuds->id_noeud,
-         elt_val_som_ent[ient]) ;
+      if (elt_val_som_ent[ient] != NULL)
+        ecs_maillage_pre__label_en_indice
+          (liste_noeuds->nbr_noeuds,
+           elt_pos_som_ent[ient][cpt_elt_ent[ient]],
+           liste_noeuds->id_noeud,
+           elt_val_som_ent[ient]) ;
 
     ECS_FREE(liste_noeuds->id_noeud) ;
   }
@@ -1871,7 +1873,6 @@ ecs_loc_pre_ens__lit_geo_gold(const char       *nom_fic_geo,
   bool        lire_elem_id;
   bool        importer_part;
 
-  bool         swap_endian;
   ecs_file_t  *fic;
 
   float    coo_min_max[6];
@@ -2012,7 +2013,6 @@ ecs_loc_pre_ens__lit_geo_gold(const char       *nom_fic_geo,
                "\"big-endian/little-endian\" detection."),
              ecs_file_get_name(fic));
         else {
-          swap_endian = true;
           if (ecs_file_get_swap_endian(fic) == 1)
             ecs_file_set_swap_endian(fic, 0);
           else
