@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -262,12 +262,6 @@ static const ecs_loc_gambit_elt_t  ecs_loc_gambit_elt_liste_c[7] = {
 /* fichiers au format 'DOS'                     */
 #define ECS_LOC_LNG_MAX_CHAINE_GAMBIT  84        /* Dimension des chaines */
 
-static const int ecs_loc_nbr_max_elt_c = 1000; /* Nombre initial d'elements  */
-static const int ecs_loc_nbr_moy_som_c =    8; /* Nombre initial de sommets  */
-                                                /*  par element en moyenne    */
-
-
-
 /*============================================================================
  *  Fonctions privées
  *============================================================================*/
@@ -481,7 +475,6 @@ ecs_loc_pre_gambit__lit_coords(ecs_file_t    *fic_maillage,
                                ecs_int_t      som_val_label[],
                                int           *num_ligne)
 {
-  ecs_int_t    retour;
   ecs_int_t    isom;
   ecs_int_t    icoo;
 
@@ -501,6 +494,8 @@ ecs_loc_pre_gambit__lit_coords(ecs_file_t    *fic_maillage,
 
   for (isom = 0; isom < nbr_som; isom++) {
 
+    ecs_int_t  retour = 0;
+
     ecs_file_gets(chaine, ECS_LOC_LNG_MAX_CHAINE_GAMBIT,
                   fic_maillage, num_ligne);
 
@@ -537,7 +532,6 @@ ecs_loc_pre_gambit__lit_coords(ecs_file_t    *fic_maillage,
 
   ecs_loc_pre_gambit__fin_section(fic_maillage, num_ligne);
 
-
   /* Maillage en dimension 3 */
 
   if (dim_e == 2) {
@@ -579,7 +573,6 @@ ecs_loc_pre_gambit__lit_elements(ecs_file_t  *fic_maillage,
   ecs_int_t    taille_connect_max;
 
   ecs_int_t    ind_ent;              /* Indice de l'entité concernée         */
-  ecs_int_t    num_pos;              /* Nouvel indice de stockage            */
 
   size_t       ielt;                 /* Indice de boucle sur les éléments    */
   ecs_int_t    inod;                 /* Indice de boucle sur les noeuds      */
@@ -735,9 +728,6 @@ ecs_loc_pre_gambit__lit_elements(ecs_file_t  *fic_maillage,
 
       }
 
-      num_pos =   elt_pos_som_ent[ind_ent][cpt_elt_ent[ind_ent]] - 1
-                + nbr_som_elt;
-
       /* Connectivité de l'élément par ses numéros de sommets */
       /*------------------------------------------------------*/
 
