@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -316,7 +316,6 @@ ecs_loc_pre_med__lit_noeud(ecs_maillage_t   *maillage,
   med_bool     transformation = MED_FALSE;
 
   char         nom_maillage_med[MED_NAME_SIZE + 1];
-  char        *nom_noe_med;
 
   /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/
 
@@ -353,7 +352,6 @@ ecs_loc_pre_med__lit_noeud(ecs_maillage_t   *maillage,
   ECS_MALLOC(coord_med,   nbr_noe_med * mdim_med,            med_float);
   ECS_MALLOC(fam_noe_med, nbr_noe_med,                       med_int);
   ECS_MALLOC(num_noe_med, nbr_noe_med,                       med_int);
-  ECS_MALLOC(nom_noe_med, nbr_noe_med * MED_SNAME_SIZE + 1, char);
 
   ret_med = MEDmeshNodeCoordinateRd(fic_maillage->fid,
                                     nom_maillage_med,
@@ -370,7 +368,6 @@ ecs_loc_pre_med__lit_noeud(ecs_maillage_t   *maillage,
 
   ECS_FREE(fam_noe_med);
   ECS_FREE(num_noe_med);
-  ECS_FREE(nom_noe_med);
 
   /* Transformation du tableau des coord au type `med_float'   */
   /*             en un tableau des coord au type `ecs_coord_t' */
@@ -422,8 +419,7 @@ ecs_loc_pre_med__lit_noeud(ecs_maillage_t   *maillage,
 static void
 ecs_loc_pre_med__lit_maille(ecs_maillage_t   *maillage,
                             const ecs_med_t  *fic_maillage,
-                            const char       *nom_maillage,
-                            int               dim_e)
+                            const char       *nom_maillage)
 {
   ecs_entmail_t   entmail_e;
 
@@ -451,9 +447,6 @@ ecs_loc_pre_med__lit_maille(ecs_maillage_t   *maillage,
   ecs_int_t  *elt_val_som_ent[ECS_N_ENTMAIL];    /* Numeros des sommets  */
   int        *elt_val_fam_ent[ECS_N_ENTMAIL];    /* Familles elements    */
 
-  ecs_size_t *pos_fac_connect;
-  ecs_int_t  *val_fac_connect;
-
   /* Déclarations des variables pour MED */
   /*-------------------------------------*/
 
@@ -468,7 +461,6 @@ ecs_loc_pre_med__lit_maille(ecs_maillage_t   *maillage,
   med_int            nbr_equiv = 0;
   med_int            ind_equiv = -1;
   med_int            edim_med;
-  med_int            mdim_med;
   med_int            nbr_ele_med;
   med_int            taille_med;
 
@@ -507,10 +499,6 @@ ecs_loc_pre_med__lit_maille(ecs_maillage_t   *maillage,
 
   }
 
-  pos_fac_connect = NULL;
-  val_fac_connect = NULL;
-
-  mdim_med = (med_int)dim_e;
   strcpy(nom_maillage_med, nom_maillage);
 
   /*------------------------------------------------*/
@@ -565,7 +553,7 @@ ecs_loc_pre_med__lit_maille(ecs_maillage_t   *maillage,
       ecs_warn();
       printf(_("The MED mesh contains %d elements of type %s\n"
                "which are ignored by the Preprocessor.\n\n"),
-             nbr_ele_med, nom_typ_ignore[ityp]);
+             (int)nbr_ele_med, nom_typ_ignore[ityp]);
 
     }
   }
@@ -628,6 +616,7 @@ ecs_loc_pre_med__lit_maille(ecs_maillage_t   *maillage,
 
   for (ityp = 0; ityp < ECS_MED_NBR_TYP_ELT; ityp++) {
 
+    taille_med = 0;
     typ_geo_med = ecs_fic_med_init_elt_liste_c[ityp].med_typ;
 
     /*
@@ -1527,7 +1516,7 @@ ecs_pre_med__lit_maillage(const char  *nom_fic_maillage,
 {
   char            *nom_maillage;
   int              ind;
-  int              dim_e;
+  med_int          dim_e;
   ecs_med_t       *fic_maillage;
   ecs_famille_t  **vect_famille;
 
@@ -1661,8 +1650,7 @@ ecs_pre_med__lit_maillage(const char  *nom_fic_maillage,
 
   ecs_loc_pre_med__lit_maille(maillage,
                               fic_maillage,
-                              nom_maillage,
-                              dim_e);
+                              nom_maillage);
 
   /* Lecture des familles */
   /*----------------------*/
