@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -214,7 +214,6 @@ ecs_loc_table_post_med__ecr_fam(const char           *prefixe_nom_fam,
   size_t   ipropr;
 
   ecs_tab_char_t  tab_nom_descr;
-  ecs_tab_int_t   tab_ide_descr;
 
   /* Déclarations des variables pour MED */
   /*-------------------------------------*/
@@ -278,7 +277,6 @@ ecs_loc_table_post_med__ecr_fam(const char           *prefixe_nom_fam,
 
   }
 
-  ECS_FREE(tab_ide_descr.val);
   ECS_FREE(tab_nom_descr.val);
 
   /* Appel à la fonction d'écriture MED */
@@ -354,7 +352,7 @@ ecs_table_post_med__ecr_famille(const char           *nom_maillage,
 
   num_fam_med = 0;
 
-  sprintf(str_num_fam_med, "%d", num_fam_med);
+  sprintf(str_num_fam_med, "%d", (int)num_fam_med);
 
   assert(strlen("FAMILLE_") + strlen(str_num_fam_med) <= MED_NAME_SIZE);
 
