@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -33,15 +33,16 @@ Main file of the CFD_STUDY module. Defines the standard SALOME callback.
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import os
+import os, string
 import logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtGui import QDialog, QMessageBox, QDockWidget
-from PyQt4.QtCore import Qt, QObject, SIGNAL
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Salome modules
@@ -51,6 +52,7 @@ import SalomePyQt
 from SalomePyQt import WT_ObjectBrowser, WT_PyConsole, WT_LogWindow
 import SALOMEDS
 import SALOMEDS_Attributes_idl
+import salome, salome_version
 
 #-------------------------------------------------------------------------------
 # Application modules
@@ -60,11 +62,12 @@ import CFDSTUDYGUI_ActionsHandler
 import CFDSTUDYGUI_DataModel
 import CFDSTUDYGUI_Commons
 import CFDSTUDYGUI_DesktopMgr
+import CFDSTUDYGUI_SolverGUI
 
 from CFDSTUDYGUI_Commons import sg, sgPyQt
 from CFDSTUDYGUI_Commons import CFD_Saturne, CFD_Neptune
 from CFDSTUDYGUI_Commons import CheckCFD_CodeEnv
-
+from CFDSTUDYGUI_Message import cfdstudyMess
 #-------------------------------------------------------------------------------
 # log config
 #-------------------------------------------------------------------------------
@@ -76,9 +79,8 @@ log.setLevel(logging.NOTSET)
 #-------------------------------------------------------------------------------
 # Global definitions
 #-------------------------------------------------------------------------------
+__MODULE_NAME__ = "CFDSTUDY"
 
-studyId = 1
-d_activation = {}
 
 # Desktop manager: instance of CFDSTUDYGUI_DesktopMgr class to store the SALOME Workspace
 _DesktopMgr = CFDSTUDYGUI_DesktopMgr.CFDSTUDYGUI_DesktopMgr()
@@ -88,6 +90,10 @@ ObjectTR = QObject()
 
 DEFAULT_EDITOR_NAME = ObjectTR.tr("CFDSTUDY_PREF_EDITOR")
 
+DEFAULT_READER_NAME = ObjectTR.tr("CFDSTUDY_PREF_READER")
+
+DEFAULT_DISPLAY_VIEWER_NAME = ObjectTR.tr("CFDSTUDY_PREF_DISPLAY_VIEWER")
+
 #-------------------------------------------------------------------------------
 # Callback GUI functions
 #-------------------------------------------------------------------------------
@@ -100,6 +106,10 @@ def initialize():
     log.debug("initialize")
     if not sgPyQt.hasSetting( "CFDSTUDY", "ExternalEditor"):
         sgPyQt.addSetting( "CFDSTUDY", "ExternalEditor", DEFAULT_EDITOR_NAME )
+    if not sgPyQt.hasSetting( "CFDSTUDY", "ExternalReader"):
+        sgPyQt.addSetting( "CFDSTUDY", "ExternalReader", DEFAULT_READER_NAME )
+    if not sgPyQt.hasSetting( "CFDSTUDY", "ExternalDisplay"):
+        sgPyQt.addSetting( "CFDSTUDY", "ExternalDisplay", DEFAULT_DISPLAY_VIEWER_NAME )
     pass
 
 
@@ -122,6 +132,17 @@ def windows():
     return winMap
 
 
+def closeStudy() :
+    """
+    This method is called when salome study is closed (Salome desktop button File -> close -> close w/o saving button) and Salome Main window desktop is already available
+    """
+    log.debug("closeStudy")
+    if salome_version.getVersion() >= '7.5.0' :
+        dsk = sgPyQt.getDesktop()
+        CFDSTUDYGUI_SolverGUI._c_CFDGUI.cleanAllDock(dsk)
+        ActionHandler = _DesktopMgr.getActionHandler(dsk)
+        ActionHandler._SolverGUI.resizeObjBrowserDock()
+
 def views():
     """
     This method is called when GUI module is being created and initialized.
@@ -152,19 +173,24 @@ def setWorkSpace(ws):
 
     dsk = sgPyQt.getDesktop()
     _DesktopMgr.setWorkspace(dsk, ws)
-    ActionHandler = _DesktopMgr.getActionHandler(dsk)
-
+    #MPActionHandler = _DesktopMgr.getActionHandler(dsk)
+    #MPActionHandler.connectSolverGUI()
 
 def createPreferences():
     """
     Manages the preferences QDialog of the module.
     """
     log.debug("createPreferences")
-    sgPyQt = CFDSTUDYGUI_DataModel.sgPyQt
     genTab = sgPyQt.addPreference(ObjectTR.tr("CFDSTUDY_PREF_GEN_GROUP"))
-    editorGroup = sgPyQt.addPreference("Editor",genTab)
-    editor      = sgPyQt.addPreference("External Editor",editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalEditor")
-    Reader      = sgPyQt.addPreference("External Reader",editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalReader")
+    EditorField = str(ObjectTR.tr("EDITOR"))
+    editorGroup = sgPyQt.addPreference(EditorField,genTab)
+    externalEditorField = str(ObjectTR.tr("EXTERNAL_EDITOR"))
+    externalReaderField= str(ObjectTR.tr("EXTERNAL_READER"))
+    editor      = sgPyQt.addPreference(externalEditorField,editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalEditor")
+    Reader      = sgPyQt.addPreference(externalReaderField,editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalReader")
+    externalDisplayField = str(ObjectTR.tr("EXTERNAL_DISPLAY"))
+    displayViewer        = sgPyQt.addPreference(externalDisplayField,editorGroup,SalomePyQt.PT_String, "CFDSTUDY","ExternalDisplay")
+
 
 
 def preferenceChanged( section, setting ):
@@ -180,79 +206,69 @@ def activate():
     @return: C{True} only if the activation is successful.
     """
     log.debug("activate")
-    global d_activation, studyId
-
     dsk = sgPyQt.getDesktop()
-    studyId = sgPyQt.getStudyId()
-    # instance of the CFDSTUDYGUI_ActionsHandler class for the current desktop
+    dsk.setTabPosition(Qt.RightDockWidgetArea,QTabWidget.South)
+    dsk.setTabPosition(Qt.LeftDockWidgetArea,QTabWidget.South)
+
     ActionHandler = _DesktopMgr.getActionHandler(dsk)
 
-    if studyId not in d_activation.keys():
-        d_activation[studyId] = 1
+    env_saturne, mess1 = CheckCFD_CodeEnv(CFD_Saturne)
+    env_neptune, mess2 = CheckCFD_CodeEnv(CFD_Neptune)
 
-    if d_activation[studyId] == 1:
-        d_activation[studyId] = 0
-        env_saturne, mess1 = CheckCFD_CodeEnv(CFD_Saturne)
-        env_neptune, mess2 = CheckCFD_CodeEnv(CFD_Neptune)
+    log.debug("activate -> env_saturne = %s" % env_saturne)
+    log.debug("activate -> env_neptune = %s" % env_neptune)
 
-        log.debug("activate -> env_saturne = %s" % env_saturne)
-        log.debug("activate -> env_neptune = %s" % env_neptune)
+    if not env_saturne and not env_neptune:
+        QMessageBox.critical(ActionHandler.dskAgent().workspace(),
+                             "Error", mess1, QMessageBox.Ok, 0)
+        QMessageBox.critical(ActionHandler.dskAgent().workspace(),
+                             "Error", mess2, QMessageBox.Ok, 0)
+        return False
 
-        if not env_saturne and not env_neptune:
-            QMessageBox.critical(ActionHandler.dskAgent().workspace(),
-                                 "Error", mess1, QMessageBox.Ok, 0)
-            QMessageBox.critical(ActionHandler.dskAgent().workspace(),
-                                 "Error", mess2, QMessageBox.Ok, 0)
-            d_activation[studyId] = 1
+    if env_neptune:
+        if mess2 != "":
+            mess = cfdstudyMess.trMessage(ObjectTR.tr("CFDSTUDY_INVALID_ENV"),[]) + " ; "+ mess2
+            cfdstudyMess.aboutMessage(mess)
             return False
+        else:
+            ActionHandler.DialogCollector.InfoDialog.setCode(env_saturne, env_neptune)
 
-        if env_neptune:
-            if mess2 != "":
-                Error = "Error: "+ ObjectTR.tr("CFDSTUDY_INVALID_ENV")
-                QMessageBox.critical(ActionHandler.dskAgent().workspace(),
-                                     Error, mess2, QMessageBox.Ok, 0)
-                d_activation[studyId] = 1
-                return False
-            else:
-                ActionHandler.DialogCollector.InfoDialog.setCode(env_saturne, env_neptune)
-
-        elif env_saturne:
-            if mess1 != "":
-                Error = "Error: "+ ObjectTR.tr("CFDSTUDY_INVALID_ENV")
-                QMessageBox.critical(ActionHandler.dskAgent().workspace(),
-                                     Error, mess1, QMessageBox.Ok, 0)
-                d_activation[studyId] = 1
-                return False
-            else:
-                ActionHandler.DialogCollector.InfoDialog.setCode(env_saturne, False)
-
-        ActionHandler.DialogCollector.InfoDialog.exec_()
-
-        if not ActionHandler.DialogCollector.InfoDialog.result() == QDialog.Accepted:
-            d_activation[studyId] = 1
+    elif env_saturne:
+        if mess1 != "":
+            mess = cfdstudyMess.trMessage(ObjectTR.tr("CFDSTUDY_INVALID_ENV"),[]) + " ; "+ mess2
+            cfdstudyMess.aboutMessage(mess)
             return False
+        else:
+            ActionHandler.DialogCollector.InfoDialog.setCode(env_saturne, False)
 
-    ActionHandler.connect(ActionHandler._SalomeSelection,
-                          SIGNAL('currentSelectionChanged()'),
-                          ActionHandler.updateActions)
+    ActionHandler._SalomeSelection.currentSelectionChanged.connect(ActionHandler.updateActions)
 
     ActionHandler.connectSolverGUI()
-    ActionHandler.updateObjBrowser()
 
     # Hide the Python Console window layout
     for dock in sgPyQt.getDesktop().findChildren(QDockWidget):
         dockTitle = dock.windowTitle()
         log.debug("activate -> QDockWidget: %s" % dockTitle)
-        if dockTitle in (u"Python Console", u"Console Python",  u"Message Window"):
+        if "Object Browser" in str(dockTitle):
+            dock.raise_()
+            dock.show()
+            if dsk.dockWidgetArea(dock) == 0:
+                dsk.addDockWidget(Qt.LeftDockWidgetArea,dock)
+        if dockTitle in ("Python Console", "Console Python",  "Message Window"):
             dock.setVisible(False)
-
+            dock.hide()
+#MP Analyser si on peut faire expand sur les case dans OB
+#    ob = sgPyQt.getObjectBrowser()
+#    if ob != None:
+#        ob.expandAll()
+#        ob.expandToDepth(2)
     return True
 
 
 def setSettings():
     """
     Stores the selected CFD code and updates action according with current
-    selection and study states in the dekstop manager.
+    selection and study states in the desktop manager.
     """
     log.debug("setSettings")
 
@@ -268,8 +284,14 @@ def deactivate():
     log.debug("deactivate")
     dsk = sgPyQt.getDesktop()
     ActionHandler = _DesktopMgr.getActionHandler(dsk)
-    ActionHandler.disconnect(ActionHandler._SalomeSelection, SIGNAL('currentSelectionChanged()'), ActionHandler.updateActions)
+
+    ActionHandler._SalomeSelection.currentSelectionChanged.disconnect(ActionHandler.updateActions)
+
     ActionHandler.disconnectSolverGUI()
+    ob = sgPyQt.getObjectBrowser()
+    if ob != None:
+        ob.collapseAll()
+
 
 
 def createPopupMenu(popup, context):
@@ -291,28 +313,45 @@ def createPopupMenu(popup, context):
     #log.debug("createPopupMenu -> SelectedCount = %s" % sg.SelectedCount())
 
     if sg.SelectedCount() > 0:
+        dictSobj = {}
         # Custom Popup menu added or removed regards to the type of the object
         for i in range(sg.SelectedCount()):
             entry = sg.getSelected(i)
             if entry != '':
                 sobj = study.FindObjectID(entry)
-                if sobj is not None:
+                if sobj != None:
                     test, anAttr = sobj.FindAttribute("AttributeLocalID")
                     if test:
                         id = anAttr._narrow(SALOMEDS.AttributeLocalID).Value()
                         if id >= 0:
+                            dictSobj[sobj] = id
 
-                            if sobj.GetFatherComponent().GetName() == "Mesh":
-                                if CFDSTUDYGUI_DataModel.getMeshFromMesh(sobj) == None:
-                                    meshGroupObject,group = CFDSTUDYGUI_DataModel.getMeshFromGroup(sobj)
-                                    if meshGroupObject != None:
-                                        ActionHandler.customPopup(id, popup)
-                                        if sg.SelectedCount() > 1:
-                                            popup.removeAction(ActionHandler.commonAction(CFDSTUDYGUI_ActionsHandler.DisplayOnlyGroupMESHAction))
-                                else:
-                                    ActionHandler.customPopup(id, popup)
+        if dictSobj != {}:
+            if CFDSTUDYGUI_DataModel.isASmeshListObject(list(dictSobj.keys())) :
+                for sobj in list(dictSobj.keys()):
+                    if sobj.GetFatherComponent().GetName() == "Mesh":
+                        if CFDSTUDYGUI_DataModel.getMeshFromMesh(sobj) == None:
+                            meshGroupObject,group = CFDSTUDYGUI_DataModel.getMeshFromGroup(sobj)
+                            if meshGroupObject != None:
+                                id = dictSobj[sobj]
+                                ActionHandler.customPopup(id, popup)
+                                if sg.SelectedCount() > 1:
                                     popup.removeAction(ActionHandler.commonAction(CFDSTUDYGUI_ActionsHandler.DisplayOnlyGroupMESHAction))
+                        else:
+                            ActionHandler.customPopup(id, popup)
+                            popup.removeAction(ActionHandler.commonAction(CFDSTUDYGUI_ActionsHandler.DisplayOnlyGroupMESHAction))
 
-                            else:
-                                if not CFDSTUDYGUI_DataModel.isLinkPathObject(sobj):
-                                    ActionHandler.customPopup(id, popup)
+            elif CFDSTUDYGUI_DataModel.isACFDSTUDYListObject(list(dictSobj.keys())):
+                if CFDSTUDYGUI_DataModel.hasTheSameType(list(dictSobj.keys())):
+                    for sobj in list(dictSobj.keys()):
+                        id = dictSobj[sobj]
+                        ActionHandler.customPopup(id, popup)
+                        fathername = sobj.GetFather().GetName()
+                        if CFDSTUDYGUI_DataModel.checkType(sobj, CFDSTUDYGUI_DataModel.dict_object["RESUPNGFile"]):
+                            ActionHandler.customPopup(id, popup)
+                            ActionHandler.commonAction(ActionHandler.DisplayImageAction).setEnabled(True)
+                        if  fathername in ["RESU","RESU_COUPLING"]:
+                            if CFDSTUDYGUI_DataModel.checkType(sobj, CFDSTUDYGUI_DataModel.dict_object["RESUSubFolder"]) or \
+                                CFDSTUDYGUI_DataModel.checkType(sobj, CFDSTUDYGUI_DataModel.dict_object["RESU_COUPLINGSubFolder"]):
+                                ActionHandler.customPopup(id, popup)
+                                ActionHandler.commonAction(ActionHandler.RemoveAction).setEnabled(True)
