@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -27,19 +27,19 @@ Common
 ======
 
 """
-from PyQt4.QtCore import QObject
+from code_saturne.Base.QtCore    import *
 
 #-------------------------------------------------------------------------------
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import os, re
+import os, re, subprocess
 import logging
 
 #-------------------------------------------------------------------------------
 # Salome modules
 #-------------------------------------------------------------------------------
-
+from CFDSTUDYGUI_Message import cfdstudyMess
 # Get SALOME PyQt interface
 import SalomePyQt
 sgPyQt = SalomePyQt.SalomePyQt()
@@ -66,7 +66,7 @@ ObjectTR = QObject()
 _CFD_Code = None #By default
 
 # True or false for log tracing
-_Trace = False  #True
+_Trace = False  #
 
 # If True all stdout redirected to MassageWindow
 _LogModeOn = True
@@ -95,6 +95,7 @@ def CFD_Code():
 
 
 def _SetCFDCode(var):
+    log.debug("_SetCFDCode : var = %s" % var)
     global _CFD_Code
     _CFD_Code = var
 
@@ -130,33 +131,34 @@ def CheckCFD_CodeEnv(code):
     bindir = ""
 
     if code not in [CFD_Saturne, CFD_Neptune]:
-        mess = ObjectTR.tr("CFDSTUDY_INVALID_SOLVER_NAME").arg(code).arg(CFD_Saturne).arg(CFD_Neptune)
+        mess = cfdstudyMess.trMessage(ObjectTR.tr("CFDSTUDY_INVALID_SOLVER_NAME"),[code,CFD_Saturne,CFD_Neptune])
         iok= False
+        return iok,mess
 
     if code == CFD_Saturne:
         try:
             from cs_package import package
             iok = True
-        except ImportError,e:
-            mess = ObjectTR.tr("INFO_DLG_INVALID_ENV").arg(code) + e.__str__()
+        except ImportError as e:
+            mess = cfdstudyMess.trMessage(ObjectTR.tr("INFO_DLG_INVALID_ENV"),[code]) + e.__str__()
             if "cs_package" in e.__str__():
-                mess = mess + " ; Check for cs_package file in Code_Saturne python package"
+                mess = mess + cfdstudyMess.trMessage(ObjectTR.tr("CHECK_CODE_PACKAGE"),["cs_package",code])
             elif "code_saturne" in e.__str__():
-                mess = mess + " ; Check PYTHONPATH then your installation "
+                mess = mess + cfdstudyMess.trMessage(ObjectTR.tr("CHECK_PYTHON_PATH"),[])
             iok = False
     elif code == CFD_Neptune:
         try:
             from nc_package import package
             iok = True
-        except ImportError,e:
-            mess = ObjectTR.tr("INFO_DLG_INVALID_ENV").arg(code) + e.__str__()
+        except ImportError as e:
+            mess = cfdstudyMess.trMessage(ObjectTR.tr("INFO_DLG_INVALID_ENV"),[code]) + e.__str__()
             if "nc_package" in e.__str__():
-                mess = mess + " ; Check for nc_package file in NEPTUNE_CFD python package"
+                mess = mess + cfdstudyMess.trMessage(ObjectTR.tr("CHECK_CODE_PACKAGE"),["nc_package",code])
             elif "neptune_cfd" in e.__str__():
-                mess = mess + " ; Check PYTHONPATH then your installation "
+                mess = mess + cfdstudyMess.trMessage(ObjectTR.tr("CHECK_PYTHON_PATH"),[])
             iok = False
     else:
-        raise ApplicationError, "Invalid name of solver!"
+        raise ValueError("Invalid name of solver!")
 
     if iok:
         pkg = package()
@@ -167,13 +169,11 @@ def CheckCFD_CodeEnv(code):
         log.debug("CheckCFD_CodeEnv -> prefix = %s" % (bindir))
 
         if not os.path.exists(prefix):
-            mess1 = ObjectTR.tr("ENV_DLG_INVALID_DIRECTORY")
-            mess = mess + mess1.arg(prefix)
+            mess = cfdstudyMess.trMessage(ObjectTR.tr("ENV_DLG_INVALID_DIRECTORY"),[prefix])
             iok = False
         else:
             if not os.path.exists(bindir):
-                mess2 =  ObjectTR.tr("ENV_DLG_INVALID_DIRECTORY")
-                mess = mess + mess2.arg(bindir)
+                mess = cfdstudyMess.trMessage(ObjectTR.tr("ENV_DLG_INVALID_DIRECTORY"),[bindir])
                 iok = False
 
     log.debug("CheckCFD_CodeEnv -> %s = %s" % (code, iok))
@@ -185,9 +185,11 @@ def BinCode():
     b = ""
     c = ""
     mess = ""
+    # default package is code_saturne (for convert...)
+    from cs_package import package
+    pkg = package()
+
     if CFD_Code() == CFD_Saturne:
-        from cs_package import package
-        pkg = package()
         bindir = pkg.get_dir('bindir')
         b = os.path.join(bindir, "code_saturne")
     elif CFD_Code() == CFD_Neptune:
@@ -195,11 +197,76 @@ def BinCode():
         pkg = package()
         bindir = pkg.get_dir('bindir')
         b = os.path.join(bindir, "neptune_cfd")
-
     c = pkg.get_preprocessor()
     log.debug("BinCode -> \n    %s\n    %s" % (b, c))
     return b, c, mess
 
+def isaCFDCase(theCasePath):
+    log.debug("isaCFDCase")
+    iok = True
+    dirList = []
+    if os.path.isdir(theCasePath):
+        try:#python3
+            dirList = os.walk(theCasePath).__next__()[1]
+        except: #python27
+            dirList = os.walk(theCasePath).next()[1]
+        if (dirList.count("DATA") and \
+           dirList.count("SRC")  and \
+           dirList.count("SCRIPTS")):
+            if not (dirList.count("RESU")):
+                subprocess.call(["mkdir","-p",os.path.join(theCasePath,"RESU")])
+        else:
+            iok = False
+    return iok
+
+def isaCFDStudy(theStudyPath):
+    log.debug("isaCFDStudy")
+    dirList = []
+    if os.path.isdir(theStudyPath):
+        try:#python3
+            dirList = os.walk(theStudyPath).__next__()[1]
+        except: #python27
+            dirList = os.walk(theStudyPath).next()[1]
+        for i in dirList:
+            if i not in ["MESH"] :
+                if isaCFDCase(os.path.join(theStudyPath,i)) :
+                    return True
+    return False
+
+def isSyrthesCase(theCasePath):
+    log.debug("isSyrthesCase")
+#a minima
+    iok = True
+    if os.path.isdir(theCasePath):
+        dirList = os.listdir(theCasePath)
+        if not dirList.count("Makefile") and not dirList.count("syrthes.py"):
+            iok = False
+    return iok
+
+def isaSaturneSyrthesCouplingStudy(theStudyPath):
+    log.debug("isaSaturneSyrthesCouplingStudy")
+    iok = False
+    hasCFDCase     = False
+    hasSyrthesCase = False
+    if not os.path.isdir(theStudyPath):
+        mess = cfdstudyMess.trMessage(ObjectTR.tr("MUST_BE_A_DIRECTORY"),[theStudyPath])
+        cfdstudyMess.criticalMessage(mess)
+        return False
+    dirList = os.listdir(theStudyPath)
+    if not (dirList.count("RESU_COUPLING") and dirList.count("coupling_parameters.py") and dirList.count("runcase")):
+        return False
+    for i in dirList:
+        ipath = os.path.join(theStudyPath,i)
+        if os.path.isdir(ipath):
+            if i not in ["MESH", "RESU_COUPLING"]:
+                if isaCFDCase(ipath):
+                    hasCFDCase = True
+                if isSyrthesCase(ipath):
+                    hasSyrthesCase = True
+    if hasCFDCase and hasSyrthesCase:
+        iok = True
+    return iok
+
 #-------------------------------------------------------------------------------
 # Classes definitions
 #-------------------------------------------------------------------------------
