@@ -4,7 +4,7 @@
 
 # This file is part of Code_Saturne, a general-purpose CFD tool.
 #
-# Copyright (C) 1998-2014 EDF S.A.
+# Copyright (C) 1998-2019 EDF S.A.
 #
 # This program is free software; you can redistribute it and/or modify it under
 # the terms of the GNU General Public License as published by the Free Software
@@ -35,14 +35,15 @@ the selected code in the SALOME workspace.
 # Standard modules
 #-------------------------------------------------------------------------------
 
-import os, sys, string, logging
+import os, sys, logging
 
 #-------------------------------------------------------------------------------
 # Third-party modules
 #-------------------------------------------------------------------------------
 
-from PyQt4.QtGui import QDockWidget, QTreeView, QMessageBox
-from PyQt4.QtCore import Qt, QObject, SIGNAL
+from code_saturne.Base.QtCore    import *
+from code_saturne.Base.QtGui     import *
+from code_saturne.Base.QtWidgets import *
 
 #-------------------------------------------------------------------------------
 # Salome modules
@@ -52,10 +53,11 @@ from PyQt4.QtCore import Qt, QObject, SIGNAL
 # Application modules
 #-------------------------------------------------------------------------------
 
-from CFDSTUDYGUI_Commons import CFD_Code, CFD_Saturne, CFD_Neptune, sgPyQt
+from CFDSTUDYGUI_Commons import CFD_Code, CFD_Saturne, CFD_Neptune, sgPyQt, sg
 from CFDSTUDYGUI_Commons import LoggingMgr
 import CFDSTUDYGUI_DataModel
 from CFDSTUDYGUI_Management import CFDGUI_Management
+import cs_info
 
 #-------------------------------------------------------------------------------
 # log config
@@ -77,45 +79,53 @@ _c_CFDGUI = CFDGUI_Management()
 # Function definitions
 #-------------------------------------------------------------------------------
 
-def getObjectBrowserDock():
-    dock = None
+
+def findObjectBrowserDockWindow():
     dsk = sgPyQt.getDesktop()
-    studyId = sgPyQt.getStudyId()
-    for dock in dsk.findChildren(QDockWidget):
-        dockTitle = str(dock.windowTitle())
-        if (dockTitle == 'Object Browser'):
-            return dock
+    ldock = []
+    if dsk != None:
+        ldock = dsk.findChildren(QDockWidget)
+    objectBrowserDockWindow = None
+    if ldock != []:
+        for i in ldock:
+            if 'Object Browser' in str(i.windowTitle()):
+                objectBrowserDockWindow = i
+    return objectBrowserDockWindow
 
 
-def tabObjectBrowser():
+def tabifyCfdGui():
     """
-    tabify DockWidgets which contains QTreeView:
-    Object Browser and CFDSTUDY tree
+    tabify DockWidgets which contains CFD study CASE QMainview :
+    CFDSTUDY Main Window
     """
+    log.debug("tabifyCfdGui")
     dsk = sgPyQt.getDesktop()
-    ldock = dsk.findChildren(QDockWidget)
-    ldocktree = []
-    for i in ldock:
-        lo = i.findChildren(QTreeView)
-        if len(lo):
-            ldocktree.append(i)
-    for i in range(1, len(ldocktree)):
-        dsk.tabifyDockWidget(ldocktree[0], ldocktree[i])
+    ldockMainWin = []
+
+    ldockMainWin = _c_CFDGUI.getDockListe()
+
+    objectBrowserDockWindow = findObjectBrowserDockWindow()
+    if len(ldockMainWin) >= 1:
+        dsk.splitDockWidget(objectBrowserDockWindow,ldockMainWin[0],Qt.Horizontal)
+        dsk.tabifyDockWidget(objectBrowserDockWindow,ldockMainWin[0])
+    for i in range(1, len(ldockMainWin)):
+        dsk.tabifyDockWidget(ldockMainWin[0], ldockMainWin[i])
+        dsk.tabifyDockWidget(objectBrowserDockWindow, ldockMainWin[i])
 
 
 def updateObjectBrowser():
     """
     force le regroupement en onglets des QTreeView apres updateObjBrowser
     """
-    studyId = sgPyQt.getStudyId()
-    sgPyQt.updateObjBrowser(studyId, 1)
-    tabObjectBrowser()
+    sg.updateObjBrowser()
+    tabifyCfdGui()
 
 
 def findDockWindow(xmlName, caseName, studyCFDName):
     """
     Find if the dockwindow corresponding to this xmlcase is already opened
     """
+    log.debug("findDockWindow")
     bool_findDockWindow = False
 
     if _c_CFDGUI != None:
@@ -136,7 +146,7 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         log.debug("CFDSTUDY_SolverGUI.__init__: ")
         QObject.__init__(self, None)
         self._CurrentWindow = None
-
+        self.dockMainWin = None
 
     def ExecGUI(self, WorkSpace, sobjXML, aCase, Args=''):
         """
@@ -157,23 +167,21 @@ class CFDSTUDYGUI_SolverGUI(QObject):
             aTitle = "unnamed"
             if aCase != None:
                 if findDockWindow(aTitle, aCase.GetName(), aCase.GetFather().GetName()):
-                    mess = "A case not finished to be set is already opened"
+                    mess = "A new case is already opened"
                     QMessageBox.warning(None, "Warning: ",mess)
                     return
-
         if aCase != None:
             aChildList = CFDSTUDYGUI_DataModel.ScanChildren(aCase, "^DATA$")
             if not len(aChildList)== 1:
                 # no DATA folder
-                mess = "There are not DATA directory  in the case"
+                mess = "DATA directory is not present in the case"
                 QMessageBox.warning(None, "Warning: ", mess)
                 return None
 
             aStartPath = CFDSTUDYGUI_DataModel._GetPath(aChildList[0])
             if aStartPath != None and aStartPath != '':
                 os.chdir(aStartPath)
-
-        mw = self.lauchGUI(WorkSpace, aCase, sobjXML, Args)
+        mw = self.launchGUI(WorkSpace, aCase, sobjXML, Args)
         if mw != None:
             self._CurrentWindow = mw
 
@@ -181,10 +189,8 @@ class CFDSTUDYGUI_SolverGUI(QObject):
 
 
     def isActive(self):
-        studyId = sgPyQt.getStudyId()
-        if _c_CFDGUI.getDocks(studyId) == {}:
+        if _c_CFDGUI.getDocks() == {}:
             self._CurrentWindow = None
-
         if self._CurrentWindow != None:
             return True
         else:
@@ -201,14 +207,13 @@ class CFDSTUDYGUI_SolverGUI(QObject):
             return False
 
 
-    def onSaveXmlFile(self):
-        log.debug("onSaveXmlFile")
+    def SaveXmlFile(self):
+        log.debug("SaveXmlFile")
+        xml_file = None
         if self._CurrentWindow != None:
-            if self._CurrentWindow.case['xmlfile'] != "":
-                self._CurrentWindow.fileSave()
-            else:
-                self.SaveAsXmlFile()
-
+            self._CurrentWindow.fileSave()
+            xml_file = self._CurrentWindow.case['xmlfile']
+        return xml_file
 
     def SaveAsXmlFile(self):
         """
@@ -218,14 +223,14 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         """
         old_xml_file = None
         xml_file = None
-
         if self._CurrentWindow != None:
             old_xml_file = self._CurrentWindow.case['xmlfile']
             self._CurrentWindow.fileSaveAs()
             xml_file = self._CurrentWindow.case['xmlfile']
-
             if old_xml_file == "":
                 old_xml_file = None
+            if xml_file == "":
+                xml_file = None
 
         return old_xml_file, xml_file
 
@@ -234,17 +239,17 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         """
         Build the Dock Title Name STUDY.CASE.file.xml with the entire file Name path
         """
-        lnames = string.split(xml_file, "/")
+        lnames = xml_file.split("/")
         if len(lnames) < 4:
             return None
         xmlname   = lnames[-1]
         casename  = lnames[-3]
         studyname = lnames[-4]
-        return string.join([studyname, casename, xmlname], ".")
+        return '.'.join([studyname, casename, xmlname])
 
 
     def getDockTitleNameFromOB(self, studyname, casename, xmlname):
-        return string.join([studyname, casename, xmlname], ".")
+        return '.'.join([studyname, casename, xmlname])
 
 
     def onUndo(self):
@@ -256,6 +261,9 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         if self._CurrentWindow != None:
             self._CurrentWindow.slotRedo()
 
+    def onOTStudyMode(self):
+        if self._CurrentWindow != None:
+            self._CurrentWindow.slotOpenTurnsMode()
 
     def onOpenShell(self):
         if self._CurrentWindow != None:
@@ -267,6 +275,30 @@ class CFDSTUDYGUI_SolverGUI(QObject):
             self._CurrentWindow.displayCase()
 
 
+    def onEditSRCFiles(self):
+        if self._CurrentWindow != None:
+            self._CurrentWindow.fileEditorOpen()
+
+
+    def onCheckSRCFiles(self):
+        if self._CurrentWindow != None:
+            self._CurrentWindow.testUserFilesCompilation()
+
+
+    def onViewLogFiles(self):
+        if self._CurrentWindow != None:
+            self._CurrentWindow.fileViewerOpen()
+
+
+    def onLaunchSolver(self):
+        if self._CurrentWindow != None:
+            self._CurrentWindow.runOrSubmit()
+
+
+    def onLaunchOT(self):
+        if self._CurrentWindow != None:
+            self._CurrentWindow.runOTMode()
+
     def onHelpAbout(self):
         if self._CurrentWindow != None:
             self._CurrentWindow.displayAbout()
@@ -278,27 +310,57 @@ class CFDSTUDYGUI_SolverGUI(QObject):
 
 
     def onSaturneHelpManual(self):
-        if self._CurrentWindow != None:
-            if CFD_Code() == CFD_Saturne:
-                self._CurrentWindow.displayCSManual()
+        from cs_package import package
+        argv_info = ['--guide', 'user']
+        cs_info.main(argv_info, package())
 
 
     def onSaturneHelpTutorial(self):
-        if self._CurrentWindow != None:
-            if CFD_Code() == CFD_Saturne:
-                self._CurrentWindow.displayCSTutorial()
+        from cs_package import package
+        msg = "See http://code-saturne.org web site for tutorials."
+        QMessageBox.about(self._CurrentWindow, 'code_saturne Interface', msg)
 
 
     def onSaturneHelpKernel(self):
-        if self._CurrentWindow != None:
-            if CFD_Code() == CFD_Saturne:
-                self._CurrentWindow.displayCSKernel()
+        from cs_package import package
+        argv_info = ['--guide', 'theory']
+        cs_info.main(argv_info, package())
 
 
     def onSaturneHelpRefcard(self):
-        if self._CurrentWindow != None:
-            if CFD_Code() == CFD_Saturne:
-                self._CurrentWindow.displayCSRefcard()
+        from cs_package import package
+        argv_info = ['--guide', 'refcard']
+        cs_info.main(argv_info, package())
+
+
+    def onSaturneHelpDoxygen(self):
+        from cs_package import package
+        argv_info = ['--guide', 'Doxygen']
+        cs_info.main(argv_info, package())
+
+
+    def onNeptuneHelpManual(self):
+        from nc_package import package
+        argv_info = ['--guide', 'user']
+        cs_info.main(argv_info, package())
+
+
+    def onNeptuneHelpTutorial(self):
+        from nc_package import package
+        argv_info = ['--guide', 'tutorial']
+        cs_info.main(argv_info, package())
+
+
+    def onNeptuneHelpKernel(self):
+        from nc_package import package
+        argv_info = ['--guide', 'theory']
+        cs_info.main(argv_info, package())
+
+
+    def onNeptuneHelpDoxygen(self):
+        from nc_package import package
+        argv_info = ['--guide', 'Doxygen']
+        cs_info.main(argv_info, package())
 
 
     def setWindowTitle_CFD(self,mw,aCase,baseTitleName):
@@ -310,19 +372,17 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         return aTitle
 
 
-    def lauchGUI(self, WorkSpace, aCase, sobjXML, Args):
+    def launchGUI(self, WorkSpace, aCase, sobjXML, Args):
         """
         mw.dockWidgetBrowser is the Browser of the CFD MainView
         """
-        log.debug("lauchGUI")
+        log.debug("launchGUI")
         from cs_gui import process_cmd_line
-
+        from code_saturne.Base.MainView import MainView
         if CFD_Code() == CFD_Saturne:
             from cs_package import package
-            from Base.MainView import MainView
         elif CFD_Code() == CFD_Neptune:
             from nc_package import package
-            from core.MainView import MainView
 
         if sobjXML == None:
             Title = "unnamed"
@@ -332,154 +392,101 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         self.Workspace = WorkSpace
         pkg = package()
         case, splash = process_cmd_line(Args)
-        mw = MainView(pkg, case, aCase)
+        try:
+            mw = MainView(pkg, case, aCase)
+        except:
+            mess = "Error in Opening CFD GUI"
+            QMessageBox.warning(None, "Warning", mess, QMessageBox.Ok, QMessageBox.NoButton)
+            return None
 
         # Put the standard panel of the MainView inside a QDockWidget
         # in the SALOME Desktop
         aTitle = self.setWindowTitle_CFD(mw, aCase, Title)
         dsk = sgPyQt.getDesktop()
-        dock = QDockWidget(aTitle)
+#####
+        objectBrowserDockWindow = findObjectBrowserDockWindow()
 
-        dock.setWidget(mw.frame)
-        dock.setMinimumWidth(520)
-        dsk.addDockWidget(Qt.RightDockWidgetArea, dock)
+        self.mainWin = QMainWindow()
+        self.mainWin.setWindowTitle(aTitle)
+        self.mainWin.setCentralWidget(mw.centralwidget)
+        self.mainWin.addDockWidget(Qt.LeftDockWidgetArea,mw.dockWidgetBrowser)
+#####
+        self.dockMainWin = QDockWidget(aTitle)
+        self.dockMainWin.setWidget(self.mainWin)
+##
 
-        dock.setVisible(True)
-        dock.show()
+        dsk.addDockWidget(Qt.LeftDockWidgetArea,self.dockMainWin)
+        self.dockMainWin.setVisible(True)
+        self.dockMainWin.show()
+        self.dockMainWin.raise_()
 
-        # Put the QTreeView of the MainView which is already inside a QDockWidget
-        # in the SALOME Desktop
-        BrowserTitle = aTitle  + " Browser"
-        mw.dockWidgetBrowser.setWindowTitle(BrowserTitle)
-        dsk.addDockWidget(Qt.LeftDockWidgetArea, mw.dockWidgetBrowser)
-
-        mw.dockWidgetBrowser.setVisible(True)
-        mw.dockWidgetBrowser.show()
-        mw.dockWidgetBrowser.raise_()
-        dock.raise_()
+        objectBrowserDockWindow.visibilityChanged["bool"].connect(self.resizeObjectBrowserDock)
 
         #Add Dock windows are managed by CFDGUI_Management class
-        studyId = sgPyQt.getStudyId()
         aStudyCFD = aCase.GetFather()
         aCaseCFD  = aCase
         xmlFileName = str(Title)
-        _c_CFDGUI.set_d_CfdCases(studyId, dock, mw.dockWidgetBrowser, mw, aStudyCFD, aCaseCFD, xmlFileName, sobjXML)
+        _c_CFDGUI.set_d_CfdCases(self.dockMainWin, mw, aStudyCFD, aCaseCFD, xmlFileName, sobjXML)
+        dockMain = _c_CFDGUI.getDockWithCFDNames(aStudyCFD.GetName(), aCaseCFD.GetName(), xmlFileName)
+        if dockMain != None:
+            dockMain.visibilityChanged["bool"].connect(self.resizeMainWindowDock)
 
-        self.connect(dock, SIGNAL("visibilityChanged(bool)"), self.setdockWindowBrowserActivated)
-        self.connect(mw.dockWidgetBrowser, SIGNAL("visibilityChanged(bool)"),self.setdockWindowActivated)
-
-        self.connect(dock.toggleViewAction(), SIGNAL("toggled(bool)"), self.setdockWB)
-        self.connect(mw.dockWidgetBrowser.toggleViewAction(), SIGNAL("toggled(bool)"), self.setdock)
-
-        _c_CFDGUI.tabifyDockWindows(dsk, studyId)
-        self.showDockWindows(studyId, xmlFileName, aCaseCFD.GetName(), aStudyCFD.GetName())
         updateObjectBrowser()
-
         return mw
 
 
-    def setdockWB(self, istoggled):
-        studyId = sgPyQt.getStudyId()
-        dock = self.sender().parent()
-        log.debug("setdockWB -> %s" % (dock,))
-
-        if _c_CFDGUI != None:
-            dockWB = _c_CFDGUI.getdockWB(studyId, dock)
-            if dockWB != None:
-                dockWB.setVisible(dock.isVisible())
-                if istoggled:
-                    dock.show()
-                    dock.raise_()
-                    dockWB.show()
-                    dockWB.raise_()
-                mw = _c_CFDGUI.getMW(studyId, dock)
-                self._CurrentWindow = mw
-                mw.activateWindow()
-                log.debug("setdockWB -> mw = %s" % (mw,))
-        else:
-            self._CurrentWindow = None
-
-
-    def setdock(self, istoggled):
-        studyId = sgPyQt.getStudyId()
-        dockWB = self.sender().parent()
-        log.debug("setdock -> %s" % (dockWB,))
-
-        if _c_CFDGUI != None:
-            dock = _c_CFDGUI.getdock(studyId, dockWB)
-            if dock != None:
-                dock.setVisible(dockWB.isVisible())
-                if istoggled:
-                    dock.show()
-                    dock.raise_()
-                    dockWB.show()
-                    dockWB.raise_()
-                mw = _c_CFDGUI.getMW(studyId, dock)
-                self._CurrentWindow = mw
-                mw.activateWindow()
-                log.debug("setdock -> mw = %s" % (mw,))
-        else:
-            self._CurrentWindow = None
-
-
-    def setdockWindowBrowserActivated(self, visible):
+    def resizeObjBrowserDock(self):
         """
-        mw is the Main CFD window allocated by MainView code
-        When we click on a cfd study window tab, the cfd study window appears and the associated CFD window browser raises too
+        called by closeStudy in CFDSTUDYGUI.py because the Object Browser size is stored into ~/.config/salome/SalomeApprc.xxx file xxx is the salome version
         """
-        studyId = sgPyQt.getStudyId()
-        dock = self.sender()
-        log.debug("setdockWindowBrowserActivated -> %s" % (dock,))
-
-        if not visible:
-            return
-        #if dock.isActiveWindow() == False:
-            #return
-        if _c_CFDGUI != None:
-            dockWB = _c_CFDGUI.getdockWB(studyId, dock)
-            if dockWB != None:
-                dockWB.activateWindow()
-                dockWB.show()
-                dockWB.raise_()
-                mw = _c_CFDGUI.getMW(studyId, dock)
-                self._CurrentWindow = mw
-                mw.activateWindow()
-                log.debug("setdockWindowBrowserActivated -> mw = %s" % (mw,))
-                ob = sgPyQt.getObjectBrowser()
-                # Clear the current selection in the SALOME object browser, which does not match with the shown dock window
-                ob.clearSelection()
-        else:
-            self._CurrentWindow = None
+        log.debug("resizeObjectBrowserDock")
+        dsk = sgPyQt.getDesktop()
+        if dsk != None:
+            objectBrowserDockWindow = findObjectBrowserDockWindow()
+            if objectBrowserDockWindow != None:
+                dsk.resizeDocks({objectBrowserDockWindow}, {300},Qt.Horizontal)
 
 
-    def setdockWindowActivated(self, visible):
+    def resizeMainWindowDock(self,visible):
         """
-        mv is the Main CFD window allocated by MainView code
-        When we click on a  CFD window browser tab, the CFD window browser appears and the associated cfd study window raises too
+        visible referred to Object Browser dock widget
         """
-        studyId = sgPyQt.getStudyId()
-        dockWB = self.sender()
-        log.debug("setdockWindowActivated -> %s" % (dockWB,))
+        log.debug("resizeMainWindowDock")
+        dsk = sgPyQt.getDesktop()
+        if dsk != None:
+            dock = self.sender()
+            if visible:
+                dsk.resizeDocks({dock}, {900},Qt.Horizontal)
 
-        if not visible:
-            return
-        #if dockWB.isActiveWindow() == False:
-            #return
-        if _c_CFDGUI != None:
-            dock = _c_CFDGUI.getdock(studyId, dockWB)
-            if dock != None:
-                dock.activateWindow()
-                dock.show()
-                dock.raise_()
-                mw = _c_CFDGUI.getMW(studyId, dock)
-                self._CurrentWindow = mw
-                mw.activateWindow()
-                log.debug("setdockWindowActivated -> mw = %s" % (mw,))
-                ob = sgPyQt.getObjectBrowser()
-                # effacer la selection en cours
-                ob.clearSelection()
-        else:
-            self._CurrentWindow = None
+    def resizeObjectBrowserDock(self,visible):
+        """
+        visible referred to Object Browser dock widget
+        """
+        log.debug("resizeObjectBrowserDock")
+        dsk = sgPyQt.getDesktop()
+        if dsk != None:
+            if visible:
+                objectBrowserDockWindow = findObjectBrowserDockWindow()
+                dsk.resizeDocks({objectBrowserDockWindow}, {300},Qt.Horizontal)
+
+            else:
+                if self.dockMainWin != None :
+                    dsk.resizeDocks({self.dockMainWin}, {900},Qt.Horizontal)
+
+    def hideDocks(self):
+        _c_CFDGUI.hideDocks()
+        ob = sgPyQt.getObjectBrowser()
+        # Clear the current selection in the SALOME object browser, which does not match with the shown dock window
+        if ob != None:
+            ob.clearSelection()
+
+
+    def showDocks(self):
+        _c_CFDGUI.showDocks()
+        ob = sgPyQt.getObjectBrowser()
+        # Clear the current selection in the SALOME object browser, which does not match with the shown dock window
+        if ob != None:
+            ob.clearSelection()
 
 
     def disconnectDockWindows(self):
@@ -488,52 +495,49 @@ class CFDSTUDYGUI_SolverGUI(QObject):
         We can have one or several of them with the right click on the main menu bar of
         Salome
         """
-        studyId = sgPyQt.getStudyId()
         if _c_CFDGUI != None:
-          _c_CFDGUI.hideDocks(studyId)
+            if _c_CFDGUI.d_CfdCases != []:
+                self.hideDocks()
 
 
     def connectDockWindows(self):
         """
         Show all the dock windows of CFDSTUDY GUI, when activating Salome CFDSTUDY module
         """
-        studyId = sgPyQt.getStudyId()
         if _c_CFDGUI != None:
-            _c_CFDGUI.showDocks(studyId)
-
-        updateObjectBrowser()
-
-
-    def showDockWindows(self, studyId, xmlName, caseName, studyCFDName):
-        """
-        Find if the dockwindow corresponding to this xmlcase is already opened
-        """
-        if _c_CFDGUI != None:
-            _c_CFDGUI.showDockWindows(studyId, xmlName, caseName, studyCFDName)
+            if _c_CFDGUI.d_CfdCases != []:
+                self.showDocks()
+                tabifyCfdGui()
 
 
     def getStudyCaseXmlNames(self, mw):
-        studyId = sgPyQt.getStudyId()
         if _c_CFDGUI != None:
-            studyCFDName, caseName, xmlName  = _c_CFDGUI.getStudyCaseXmlNames(studyId, mw)
+            studyCFDName, caseName, xmlName  = _c_CFDGUI.getStudyCaseXmlNames(mw)
         return studyCFDName, caseName, xmlName
 
 
     def getCase(self, mw):
-        studyId = sgPyQt.getStudyId()
         if _c_CFDGUI != None:
-            case  = _c_CFDGUI.getCase(studyId,mw)
+            case  = _c_CFDGUI.getCase(mw)
         return case
 
 
+    def removeDockWindowfromStudyAndCaseNames(self, studyCFDName, caseName):
+        """
+        Close the CFD_study_dock_windows if opened from close Study popup menu into the object browser
+        """
+        log.debug("removeDockWindowfromStudyAndCaseNames -> %s %s" % (studyCFDName, caseName))
+        dsk = sgPyQt.getDesktop()
+        if _c_CFDGUI != None:
+            _c_CFDGUI.delDockfromStudyAndCaseNames(dsk, studyCFDName, caseName)
+
+
     def removeDockWindow(self, studyCFDName, caseName, xmlName):
         """
         Close the CFD_study_dock_windows from remove  popup menu in object browser
         """
         log.debug("removeDockWindow -> %s %s %s" % (studyCFDName, caseName, xmlName))
         dsk = sgPyQt.getDesktop()
-        studyId = sgPyQt.getStudyId()
         if _c_CFDGUI != None:
-           _c_CFDGUI.delDock(dsk, studyId, studyCFDName, caseName, xmlName)
-
+            _c_CFDGUI.delDock(dsk, studyCFDName, caseName, xmlName)
 #-------------------------------------------------------------------------------
