@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -72,6 +72,7 @@
 
 #include "bft_mem.h"
 #include "bft_error.h"
+#include "bft_printf.h"
 
 #include "cs_base.h"
 #include "cs_blas.h"
@@ -79,8 +80,11 @@
 #include "cs_halo_perio.h"
 #include "cs_log.h"
 #include "cs_mesh.h"
+#include "cs_mesh_adjacencies.h"
 #include "cs_mesh_quantities.h"
 #include "cs_matrix.h"
+#include "cs_matrix_assembler.h"
+#include "cs_matrix_default.h"
 #include "cs_matrix_tuning.h"
 #include "cs_timer.h"
 
@@ -94,6 +98,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -111,6 +117,20 @@ BEGIN_C_DECLS
  *  Global variables
  *============================================================================*/
 
+static const char *_matrix_operation_name[CS_MATRIX_N_FILL_TYPES][2]
+  = {{N_("y <- A.x"),
+      N_("y <- (A-D).x")},
+     {N_("Symmetric y <- A.x"),
+      N_("Symmetric y <- (A-D).x")},
+     {N_("Block diagonal y <- A.x"),
+      N_("Block diagonal y <- (A-D).x")},
+     {N_("Block 6 diagonal y <- A.x"),
+      N_("Block 6 diagonal y <- (A-D).x")},
+     {N_("Block diagonal symmetric y <- A.x"),
+      N_("Block diagonal symmetric y <- (A-D).x")},
+     {N_("Block y <- A.x"),
+      N_("Block y <- (A-D).x")}};
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -214,7 +234,7 @@ _print_stats(long    n_runs,
  *   n_cells_ext <-- number of cells including ghost cells (array size)
  *   n_faces     <-- local number of internal faces
  *   cell_num    <-- global cell numbers (1 to n)
- *   face_cell   <-- face -> cells connectivity (1 to n)
+ *   face_cell   <-- face -> cells connectivity
  *   halo        <-- cell halo structure
  *   numbering   <-- vectorization or thread-related numbering info, or NULL
  *   da          <-- diagonal values
@@ -230,8 +250,7 @@ _matrix_vector_test(double                 t_measure,
                     cs_int_t               n_cells,
                     cs_int_t               n_cells_ext,
                     cs_int_t               n_faces,
-                    const cs_gnum_t       *cell_num,
-                    const cs_int_t        *face_cell,
+                    const cs_lnum_2_t     *face_cell,
                     const cs_halo_t       *halo,
                     const cs_numbering_t  *numbering,
                     const cs_real_t       *restrict da,
@@ -239,7 +258,7 @@ _matrix_vector_test(double                 t_measure,
                     cs_real_t             *restrict x,
                     cs_real_t             *restrict y)
 {
-  cs_int_t ii;
+  cs_lnum_t ii;
   double wt0, wt1;
   int    run_id, n_runs;
   long   n_ops, n_ops_glob;
@@ -264,7 +283,6 @@ _matrix_vector_test(double                 t_measure,
                                   n_cells,
                                   n_cells_ext,
                                   n_faces,
-                                  cell_num,
                                   face_cell,
                                   halo,
                                   numbering);
@@ -275,6 +293,8 @@ _matrix_vector_test(double                 t_measure,
                              sym_coeffs,
                              NULL,
                              NULL,
+                             n_faces,
+                             face_cell,
                              da,
                              xa);
 
@@ -415,7 +435,6 @@ _matrix_vector_test(double                 t_measure,
 
   cs_matrix_destroy(&m);
   cs_matrix_structure_destroy(&ms);
-
 }
 
 /*----------------------------------------------------------------------------
@@ -424,31 +443,32 @@ _matrix_vector_test(double                 t_measure,
  *
  * parameters:
  *   n_faces         <-- local number of internal faces
- *   face_cell       <-- face -> cells connectivity (1 to n)
+ *   face_cell       <-- face -> cells connectivity
  *   xa              <-- extradiagonal values
  *   x               <-- vector
  *   y               <-> vector
  *----------------------------------------------------------------------------*/
 
 static void
-_mat_vec_exdiag_native(cs_int_t             n_faces,
-                       const cs_int_t      *face_cell,
+_mat_vec_exdiag_native(cs_lnum_t            n_faces,
+                       const cs_lnum_2_t   *face_cell,
                        const cs_real_t     *restrict xa,
                        cs_real_t           *restrict x,
                        cs_real_t           *restrict y)
 {
-  cs_int_t  ii, jj, face_id;
+  cs_lnum_t  ii, jj, face_id;
 
   /* Tell IBM compiler not to alias */
 #if defined(__xlc__)
 #pragma disjoint(*x, *y, *xa)
 #endif
 
-  const cs_int_t *restrict face_cel_p = face_cell;
+  const cs_lnum_t *restrict face_cel_p
+    = (const cs_lnum_t *restrict)face_cell;
 
   for (face_id = 0; face_id < n_faces; face_id++) {
-    ii = *face_cel_p++ - 1;
-    jj = *face_cel_p++ - 1;
+    ii = *face_cel_p++;
+    jj = *face_cel_p++;
     y[ii] += xa[face_id] * x[jj];
     y[jj] += xa[face_id] * x[ii];
   }
@@ -467,13 +487,13 @@ _mat_vec_exdiag_native(cs_int_t             n_faces,
  *----------------------------------------------------------------------------*/
 
 static void
-_mat_vec_exdiag_native_v1(cs_int_t             n_faces,
-                          const cs_int_t      *face_cell,
+_mat_vec_exdiag_native_v1(cs_lnum_t            n_faces,
+                          const cs_lnum_2_t   *face_cell,
                           const cs_real_t     *restrict xa,
                           cs_real_t           *restrict x,
                           cs_real_t           *restrict y)
 {
-  cs_int_t  ii, ii_prev, kk, face_id, kk_max;
+  cs_lnum_t  ii, ii_prev, kk, face_id, kk_max;
   cs_real_t y_it, y_it_prev;
 
   const int l1_cache_size = 508;
@@ -487,7 +507,8 @@ _mat_vec_exdiag_native_v1(cs_int_t             n_faces,
    *    to another in y[ii] loop (nonzero matrix value on the same line ii).
    */
 
-  const cs_int_t *restrict face_cel_p = face_cell;
+  const cs_lnum_t *restrict face_cel_p
+    = (const cs_lnum_t *restrict)face_cell;
 
   for (face_id = 0;
        face_id < n_faces;
@@ -497,12 +518,12 @@ _mat_vec_exdiag_native_v1(cs_int_t             n_faces,
 
     /* sub-loop to compute y[ii] += xa[face_id] * x[jj] */
 
-    ii = face_cel_p[0] - 1;
+    ii = face_cel_p[0];
     ii_prev = ii;
-    y_it_prev = y[ii_prev] + xa[face_id] * x[face_cel_p[1] - 1];
+    y_it_prev = y[ii_prev] + xa[face_id] * x[face_cel_p[1]];
 
     for (kk = 1; kk < kk_max; ++kk) {
-      ii = face_cel_p[2*kk] - 1;
+      ii = face_cel_p[2*kk];
       /* y[ii] += xa[face_id+kk] * x[jj]; */
       if(ii == ii_prev) {
         y_it = y_it_prev;
@@ -512,15 +533,15 @@ _mat_vec_exdiag_native_v1(cs_int_t             n_faces,
         y[ii_prev] = y_it_prev;
       }
       ii_prev = ii;
-      y_it_prev = y_it + xa[face_id+kk] * x[face_cel_p[2*kk+1] - 1];
+      y_it_prev = y_it + xa[face_id+kk] * x[face_cel_p[2*kk+1]];
     }
     y[ii] = y_it_prev;
 
     /* sub-loop to compute y[ii] += xa[face_id] * x[jj] */
 
     for (kk = 0; kk < kk_max; ++kk) {
-      y[face_cel_p[2*kk+1] - 1]
-        += xa[face_id+kk] * x[face_cel_p[2*kk] - 1];
+      y[face_cel_p[2*kk+1]]
+        += xa[face_id+kk] * x[face_cel_p[2*kk]];
     }
     face_cel_p += 2 * l1_cache_size;
   }
@@ -533,31 +554,32 @@ _mat_vec_exdiag_native_v1(cs_int_t             n_faces,
  *
  * parameters:
  *   n_faces         <-- local number of internal faces
- *   face_cell       <-- face -> cells connectivity (1 to n)
+ *   face_cell       <-- face -> cells connectivity
  *   xa              <-- extradiagonal values
  *   x               <-- vector
  *   ya              <-> vector
  *----------------------------------------------------------------------------*/
 
 static void
-_mat_vec_exdiag_part_p1(cs_int_t             n_faces,
-                        const cs_int_t      *face_cell,
+_mat_vec_exdiag_part_p1(cs_lnum_t            n_faces,
+                        const cs_lnum_2_t   *face_cell,
                         const cs_real_t     *restrict xa,
                         cs_real_t           *restrict x,
                         cs_real_t           *restrict ya)
 {
-  cs_int_t  ii, jj, face_id;
+  cs_lnum_t  ii, jj, face_id;
 
   /* Tell IBM compiler not to alias */
 #if defined(__xlc__)
 #pragma disjoint(*x, *xa, *ya)
 #endif
 
-  const cs_int_t *restrict face_cel_p = face_cell;
+  const cs_lnum_t *restrict face_cel_p
+    = (const cs_lnum_t *restrict)face_cell;
 
   for (face_id = 0; face_id < n_faces; face_id++) {
-    ii = *face_cel_p++ - 1;
-    jj = *face_cel_p++ - 1;
+    ii = *face_cel_p++;
+    jj = *face_cel_p++;
     ya[face_id] += xa[face_id] * x[ii];
     ya[face_id] += xa[face_id] * x[jj];
   }
@@ -571,7 +593,7 @@ _mat_vec_exdiag_part_p1(cs_int_t             n_faces,
  *   n_cells     <-- number of cells
  *   n_cells_ext <-- number of cells including ghost cells (array size)
  *   n_faces     <-- local number of internal faces
- *   face_cell   <-- face -> cells connectivity (1 to n)
+ *   face_cell   <-- face -> cells connectivity
  *   xa          <-- extradiagonal values
  *   x           <-> vector
  *   y           --> vector
@@ -579,15 +601,15 @@ _mat_vec_exdiag_part_p1(cs_int_t             n_faces,
 
 static void
 _sub_matrix_vector_test(double               t_measure,
-                        cs_int_t             n_cells,
-                        cs_int_t             n_cells_ext,
-                        cs_int_t             n_faces,
-                        const cs_int_t      *face_cell,
+                        cs_lnum_t            n_cells,
+                        cs_lnum_t            n_cells_ext,
+                        cs_lnum_t            n_faces,
+                        const cs_lnum_2_t   *face_cell,
                         const cs_real_t     *restrict xa,
                         cs_real_t           *restrict x,
                         cs_real_t           *restrict y)
 {
-  cs_int_t  jj;
+  cs_lnum_t  jj;
   double wt0, wt1;
   int    run_id, n_runs;
   long   n_ops, n_ops_glob;
@@ -726,6 +748,408 @@ _sub_matrix_vector_test(double               t_measure,
 
 }
 
+/*----------------------------------------------------------------------------
+ * Copy array to reference for matrix computation check.
+ *
+ * parameters:
+ *   n_elts      <-- number values to compare
+ *   y           <-- array to copare or copy
+ *   yr          <-- reference array
+ *
+ * returns:
+ *   maximum difference between values
+ *----------------------------------------------------------------------------*/
+
+static double
+_matrix_check_compare(cs_lnum_t        n_elts,
+                      const cs_real_t  y[],
+                      cs_real_t        yr[])
+{
+  cs_lnum_t  ii;
+
+  double dmax = 0.0;
+
+  for (ii = 0; ii < n_elts; ii++) {
+    double d = CS_ABS(y[ii] - yr[ii]);
+    if (d > dmax)
+      dmax = d;
+  }
+
+#if defined(HAVE_MPI)
+
+  if (cs_glob_n_ranks > 1) {
+    double dmaxg;
+    MPI_Allreduce(&dmax, &dmaxg, 1, MPI_DOUBLE, MPI_MAX, cs_glob_mpi_comm);
+    dmax = dmaxg;
+  }
+
+#endif
+
+  return dmax;
+}
+
+/*----------------------------------------------------------------------------
+ * Check local matrix.vector product operations using matrix assembler
+ *
+ * parameters:
+ *   n_rows      <-- local number of rows
+ *   n_cols_ext  <-- number of local + ghost columns
+ *   n_edges     <-- local number of (undirected) graph edges
+ *   cell_num    <-- optional global cell numbers (1 to n), or NULL
+ *   edges       <-- edges (symmetric row <-> column) connectivity
+ *   halo        <-- cell halo structure
+ *----------------------------------------------------------------------------*/
+
+static void
+_matrix_check_asmb(cs_lnum_t              n_rows,
+                   cs_lnum_t              n_cols_ext,
+                   cs_lnum_t              n_edges,
+                   const cs_lnum_2_t     *edges,
+                   const cs_halo_t       *halo)
+{
+  cs_lnum_t  ii;
+  int  f_id;
+
+  cs_real_t  *da = NULL, *xa = NULL, *x = NULL, *y = NULL;
+  cs_real_t  *yr0 = NULL;
+  int d_block_size[4] = {3, 3, 3, 9};
+  int ed_block_size[4] = {3, 3, 3, 9};
+
+  cs_matrix_fill_type_t f_type[]
+    = {CS_MATRIX_SCALAR,           /* Simple scalar matrix */
+       CS_MATRIX_BLOCK_D};
+
+  const char *t_name[] = {"general assembly",
+                          "local rows assembly",
+                          "assembly from shared"};
+
+  /* Allocate and initialize  working arrays */
+
+  if (CS_MEM_ALIGN > 0) {
+    BFT_MEMALIGN(x, CS_MEM_ALIGN, n_cols_ext*d_block_size[1], cs_real_t);
+    BFT_MEMALIGN(y, CS_MEM_ALIGN, n_cols_ext*d_block_size[1], cs_real_t);
+    BFT_MEMALIGN(yr0, CS_MEM_ALIGN, n_cols_ext*d_block_size[1], cs_real_t);
+  }
+  else {
+    BFT_MALLOC(x, n_cols_ext*d_block_size[1], cs_real_t);
+    BFT_MALLOC(y, n_cols_ext*d_block_size[1], cs_real_t);
+    BFT_MALLOC(yr0, n_cols_ext*d_block_size[1], cs_real_t);
+  }
+
+  BFT_MALLOC(da, n_cols_ext*d_block_size[3], cs_real_t);
+  BFT_MALLOC(xa, n_edges*2*ed_block_size[3], cs_real_t);
+
+  cs_gnum_t *cell_gnum = NULL;
+  if (cs_glob_mesh->global_cell_num != NULL) {
+    BFT_MALLOC(cell_gnum, n_cols_ext, cs_gnum_t);
+    for (ii = 0; ii < n_rows; ii++)
+      cell_gnum[ii] = cs_glob_mesh->global_cell_num[ii];
+    if (halo != NULL)
+      cs_halo_sync_untyped(halo,
+                           CS_HALO_STANDARD,
+                           sizeof(cs_gnum_t),
+                           cell_gnum);
+  }
+
+  /* Global cell ids, based on range/scan */
+
+  cs_gnum_t l_range[2] = {0, n_rows};
+  cs_gnum_t n_g_rows = n_rows;
+
+#if defined(HAVE_MPI)
+  if (cs_glob_n_ranks > 1) {
+    cs_gnum_t g_shift;
+    cs_gnum_t l_shift = n_rows;
+    MPI_Scan(&l_shift, &g_shift, 1, CS_MPI_GNUM, MPI_SUM, cs_glob_mpi_comm);
+    MPI_Allreduce(&l_shift, &n_g_rows, 1, CS_MPI_GNUM, MPI_SUM,
+                  cs_glob_mpi_comm);
+    l_range[0] = g_shift - l_shift;
+    l_range[1] = g_shift;
+  }
+#endif
+
+  cs_gnum_t *r_g_id;
+  BFT_MALLOC(r_g_id, n_cols_ext, cs_gnum_t);
+  for (ii = 0; ii < n_rows; ii++)
+    r_g_id[ii] = ii + l_range[0];
+  if (halo != NULL)
+    cs_halo_sync_untyped(halo,
+                         CS_HALO_STANDARD,
+                         sizeof(cs_gnum_t),
+                         r_g_id);
+
+  /* Loop on fill options */
+
+  for (f_id = 0; f_id < 2; f_id++) {
+
+    const int *_d_block_size
+      = (f_type[f_id] >= CS_MATRIX_BLOCK_D) ? d_block_size : NULL;
+    const cs_lnum_t stride = (_d_block_size != NULL) ? d_block_size[1] : 1;
+    const cs_lnum_t sd = stride*stride; /* for current fill types */
+    const cs_lnum_t se = 1;             /* for current fill types */
+
+    /* Initialize arrays; we need to be careful here, so that
+       the array values are consisten across MPI ranks.
+       This requires using a specific initialiation for each fill type. */
+
+    if (cell_gnum != NULL) {
+#     pragma omp parallel for
+      for (ii = 0; ii < n_cols_ext; ii++) {
+        cs_gnum_t jj = (cell_gnum[ii] - 1)*sd;
+        for (cs_lnum_t kk = 0; kk < sd; kk++) {
+          da[ii*sd+kk] = 1.0 + cos(jj*sd+kk);
+        }
+      }
+    }
+    else {
+#     pragma omp parallel for
+      for (ii = 0; ii < n_cols_ext*sd; ii++)
+        da[ii] = 1.0 + cos(ii);
+    }
+
+    const cs_gnum_t *face_gnum = cs_glob_mesh->global_i_face_num;
+    if (face_gnum != NULL) {
+#     pragma omp parallel for
+      for (ii = 0; ii < n_edges; ii++) {
+        cs_gnum_t jj = (face_gnum[ii] - 1)*se;
+        for (cs_lnum_t kk = 0; kk < se; kk++) {
+          xa[(ii*se+kk)*2]
+            = 0.5*(0.9 + cos(jj*se+kk));
+          xa[(ii*se+kk)*2 + 1]
+            = -0.5*(0.9 + cos(jj*se+kk));
+        }
+      }
+    }
+    else {
+#     pragma omp parallel for
+      for (ii = 0; ii < n_edges*se; ii++) {
+        xa[ii*2] = 0.5*(0.9 + cos(ii));
+        xa[ii*2 + 1] = -0.5*(0.9 + cos(ii));
+      }
+    }
+
+    if (cell_gnum != NULL) {
+#     pragma omp parallel for
+      for (ii = 0; ii < n_cols_ext; ii++) {
+        cs_gnum_t jj = (cell_gnum[ii] - 1)*stride;
+        for (cs_lnum_t kk = 0; kk < stride; kk++)
+          x[ii*stride+kk] = sin(jj*stride+kk);
+      }
+    }
+    else {
+#     pragma omp parallel for
+      for (ii = 0; ii < n_cols_ext*stride; ii++)
+        x[ii] = sin(ii);
+    }
+
+    /* Reference */
+
+    cs_matrix_vector_native_multiply(false,  /* symmetric */
+                                     _d_block_size,
+                                     NULL,   /* extra diag block size */
+                                     CS_HALO_ROTATION_COPY,
+                                     -1,     /* field id or -1 */
+                                     da,
+                                     xa,
+                                     x,
+                                     yr0);
+
+    /* Test for matrix assembler (for MSR case) */
+
+    const cs_lnum_t block_size = 800;
+    cs_gnum_t g_row_id[800];
+    cs_gnum_t g_col_id[800];
+    cs_real_t val[1600];
+
+    cs_matrix_assembler_t *ma = NULL;
+
+    /* 3 variant construction methods, 2 coefficient methods */
+
+    const char *ma_name[] = {"distributed contribution assember",
+                             "local rows assembler",
+                             "shared index assembler"};
+
+    for (int c_id = 0; c_id < 3; c_id++) {
+
+      /* Matrix created from shared index may not always handle
+         periodic elements in the same manner */
+
+      if (halo != NULL) {
+        if (halo->n_transforms > 0 && c_id == 2)
+          continue;
+      }
+
+      if (c_id < 2) {
+
+        ma = cs_matrix_assembler_create(l_range, true);
+
+        /* Add connectivities */
+
+        cs_matrix_assembler_add_g_ids(ma, n_rows, r_g_id, r_g_id);
+
+        if (c_id == 0) { /* Rank contributions through global edges */
+          cs_lnum_t jj = 0;
+          for (ii = 0; ii < n_edges; ii++) {
+            cs_lnum_t i0 = edges[ii][0];
+            cs_lnum_t i1 = edges[ii][1];
+            g_row_id[jj] = r_g_id[i0];
+            g_col_id[jj] = r_g_id[i1];
+            jj++;
+            g_row_id[jj] = r_g_id[i1];
+            g_col_id[jj] = r_g_id[i0];
+            jj++;
+            if (jj >= block_size - 1) {
+              cs_matrix_assembler_add_g_ids(ma, jj, g_row_id, g_col_id);
+              jj = 0;
+            }
+          }
+          cs_matrix_assembler_add_g_ids(ma, jj, g_row_id, g_col_id);
+        }
+        else { /* Rank contributions are local */
+          cs_lnum_t jj = 0;
+          for (ii = 0; ii < n_edges; ii++) {
+            cs_lnum_t i0 = edges[ii][0];
+            cs_lnum_t i1 = edges[ii][1];
+            if (i0 < n_rows) {
+              g_row_id[jj] = r_g_id[i0];
+              g_col_id[jj] = r_g_id[i1];
+              jj++;
+            }
+            if (i1 < n_rows) {
+              g_row_id[jj] = r_g_id[i1];
+              g_col_id[jj] = r_g_id[i0];
+              jj++;
+            }
+            if (jj >= block_size - 1) {
+              cs_matrix_assembler_add_g_ids(ma, jj, g_row_id, g_col_id);
+              jj = 0;
+            }
+          }
+          cs_matrix_assembler_add_g_ids(ma, jj, g_row_id, g_col_id);
+        }
+
+        cs_matrix_assembler_compute(ma);
+      }
+      else {
+        const cs_mesh_adjacencies_t  *madj = cs_glob_mesh_adjacencies;
+        ma = cs_matrix_assembler_create_from_shared(n_rows,
+                                                    true,
+                                                    madj->cell_cells_idx,
+                                                    madj->cell_cells,
+                                                    halo);
+      }
+
+      cs_matrix_structure_t  *ms
+        = cs_matrix_structure_create_from_assembler(CS_MATRIX_MSR, ma);
+
+      cs_matrix_t  *m = cs_matrix_create(ms);
+
+      cs_matrix_assembler_values_t *mav = NULL;
+
+      if (f_type[f_id] == CS_MATRIX_SCALAR)
+        mav = cs_matrix_assembler_values_init(m, NULL, NULL);
+      else if (f_type[f_id] == CS_MATRIX_BLOCK_D)
+        mav = cs_matrix_assembler_values_init(m, d_block_size, NULL);
+
+      cs_matrix_assembler_values_add_g(mav, n_rows,
+                                       r_g_id, r_g_id, da);
+
+      if (c_id == 0) { /* Rank contributions through global edges */
+        cs_lnum_t jj = 0;
+        for (ii = 0; ii < n_edges; ii++) {
+          cs_lnum_t i0 = edges[ii][0];
+          cs_lnum_t i1 = edges[ii][1];
+          g_row_id[jj] = r_g_id[i0];
+          g_col_id[jj] = r_g_id[i1];
+          if (i0 < n_rows && i1 < n_rows)
+            val[jj] = xa[ii*2];
+          else
+            val[jj] = xa[ii*2]*0.5; /* count half contribution twice */
+          jj++;
+          g_row_id[jj] = r_g_id[i1];
+          g_col_id[jj] = r_g_id[i0];
+          if (i0 < n_rows && i1 < n_rows)
+            val[jj] = xa[ii*2+1];
+          else
+            val[jj] = xa[ii*2+1]*0.5;
+          jj++;
+          if (jj >= block_size - 1) {
+            cs_matrix_assembler_values_add_g(mav, jj,
+                                             g_row_id, g_col_id, val);
+            jj = 0;
+          }
+        }
+        for (ii = 0; ii < jj; ii++) {
+        }
+
+        cs_matrix_assembler_values_add_g(mav, jj, g_row_id, g_col_id, val);
+      }
+      else { /* Rank contributions are local */
+        cs_lnum_t jj = 0;
+        for (ii = 0; ii < n_edges; ii++) {
+          cs_lnum_t i0 = edges[ii][0];
+          cs_lnum_t i1 = edges[ii][1];
+          if (i0 < n_rows) {
+            g_row_id[jj] = r_g_id[i0];
+            g_col_id[jj] = r_g_id[i1];
+            val[jj] = xa[ii*2];
+            jj++;
+          }
+          if (i1 < n_rows) {
+            g_row_id[jj] = r_g_id[i1];
+            g_col_id[jj] = r_g_id[i0];
+            val[jj] = xa[ii*2+1];
+            jj++;
+          }
+          if (jj >= block_size - 1) {
+            cs_matrix_assembler_values_add_g(mav, jj,
+                                             g_row_id, g_col_id, val);
+            jj = 0;
+          }
+        }
+        cs_matrix_assembler_values_add_g(mav, jj, g_row_id, g_col_id, val);
+      }
+
+      cs_matrix_assembler_values_finalize(&mav);
+
+      cs_matrix_vector_multiply(CS_HALO_ROTATION_COPY, m, x, y);
+
+      cs_matrix_release_coefficients(m);
+
+      cs_matrix_destroy(&m);
+      cs_matrix_structure_destroy(&ms);
+
+      if (f_id == 0) /* no need to log multiple identical assemblers */
+        cs_matrix_assembler_log_rank_counts(ma, CS_LOG_DEFAULT, ma_name[c_id]);
+
+      cs_matrix_assembler_destroy(&ma);
+
+      double dmax = _matrix_check_compare(n_rows*stride, y, yr0);
+      bft_printf("\n%s\n",
+                 _matrix_operation_name[f_type[f_id]][0]);
+      bft_printf("  %-32s : %12.5e\n",
+                 t_name[c_id],
+                 dmax);
+      bft_printf_flush();
+
+    } /* end of loop on construction method */
+
+  } /* end of loop on fill types */
+
+  BFT_FREE(r_g_id);
+  BFT_FREE(cell_gnum);
+
+  BFT_FREE(yr0);
+
+  BFT_FREE(y);
+  BFT_FREE(x);
+
+  BFT_FREE(xa);
+  BFT_FREE(da);
+}
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -755,21 +1179,42 @@ cs_benchmark(int  mpi_trace_mode)
 
   const cs_mesh_t *mesh = cs_glob_mesh;
   const cs_mesh_quantities_t *mesh_v = cs_glob_mesh_quantities;
+  const cs_lnum_2_t *i_face_cells = (const cs_lnum_2_t *)(mesh->i_face_cells);
 
   size_t n_cells = mesh->n_cells;
   size_t n_cells_ext = mesh->n_cells_with_ghosts;
   size_t n_faces = mesh->n_i_faces;
 
-  int                    n_fill_types_nsym = 3;
+  int                    n_fill_types_nsym = 4;
   int                    n_fill_types_sym = 2;
   cs_matrix_fill_type_t  fill_types_nsym[] = {CS_MATRIX_SCALAR,
-                                              CS_MATRIX_33_BLOCK_D,
-                                              CS_MATRIX_33_BLOCK};
+                                              CS_MATRIX_BLOCK_D,
+                                              CS_MATRIX_BLOCK_D_66,
+                                              CS_MATRIX_BLOCK};
   cs_matrix_fill_type_t  fill_types_sym[] = {CS_MATRIX_SCALAR_SYM,
-                                             CS_MATRIX_33_BLOCK_D_SYM};
-  double                 fill_weights_nsym[] = {0.5, 0.4, 0.1};
+                                             CS_MATRIX_BLOCK_D_SYM};
+  double                 fill_weights_nsym[] = {0.5, 0.3, 0.1, 0.1};
   double                 fill_weights_sym[] = {0.8, 0.2};
 
+  cs_mesh_adjacencies_initialize();
+  cs_mesh_adjacencies_update_mesh();
+
+  cs_matrix_initialize();
+
+  cs_log_printf(CS_LOG_PERFORMANCE,
+                _("\n"
+                  "Benchmark mode activated\n"
+                  "========================\n"));
+
+  /* Run some feature tests */
+  /*------------------------*/
+
+  _matrix_check_asmb(n_cells,
+                     n_cells_ext,
+                     n_faces,
+                     i_face_cells,
+                     mesh->halo);
+
   /* Allocate and initialize  working arrays */
   /*-----------------------------------------*/
 
@@ -794,14 +1239,6 @@ cs_benchmark(int  mpi_trace_mode)
     xa[ii*2 + 1] = -0.5;
   }
 
-  /* Run tests */
-  /*-----------*/
-
-  cs_log_printf(CS_LOG_PERFORMANCE,
-                _("\n"
-                  "Benchmark mode activated\n"
-                  "========================\n"));
-
   /* Call matrix tuning */
   /*--------------------*/
 
@@ -810,8 +1247,7 @@ cs_benchmark(int  mpi_trace_mode)
   cs_matrix_variant_test(n_cells,
                          n_cells_ext,
                          n_faces,
-                         mesh->global_cell_num,
-                         mesh->i_face_cells,
+                         i_face_cells,
                          mesh->halo,
                          mesh->i_face_numbering);
 
@@ -828,19 +1264,18 @@ cs_benchmark(int  mpi_trace_mode)
                                NULL,
                                fill_types_nsym,
                                fill_weights_nsym,
-                               10,       /* min expected SpMV products */
+                               50,       /* min expected SpMV products */
                                n_cells,
                                n_cells_ext,
                                n_faces,
-                               mesh->global_cell_num,
-                               mesh->i_face_cells,
+                               i_face_cells,
                                mesh->halo,
                                mesh->i_face_numbering);
 
   _matrix_vector_test(t_measure,
                       mv, false,
                       n_cells, n_cells_ext, n_faces,
-                      mesh->global_cell_num, mesh->i_face_cells, mesh->halo,
+                      i_face_cells, mesh->halo,
                       mesh->i_face_numbering, da, xa, x, y);
 
   cs_matrix_variant_destroy(&mv);
@@ -856,19 +1291,18 @@ cs_benchmark(int  mpi_trace_mode)
                                NULL,
                                fill_types_sym,
                                fill_weights_sym,
-                               10,  /* min expected SpMV products */
+                               50,  /* min expected SpMV products */
                                n_cells,
                                n_cells_ext,
                                n_faces,
-                               mesh->global_cell_num,
-                               mesh->i_face_cells,
+                               i_face_cells,
                                mesh->halo,
                                mesh->i_face_numbering);
 
   _matrix_vector_test(t_measure,
                       mv, true,
                       n_cells, n_cells_ext, n_faces,
-                      mesh->global_cell_num, mesh->i_face_cells, mesh->halo,
+                      i_face_cells, mesh->halo,
                       mesh->i_face_numbering, da, xa, x, y);
 
   cs_matrix_variant_destroy(&mv);
@@ -877,11 +1311,15 @@ cs_benchmark(int  mpi_trace_mode)
                           n_cells,
                           n_cells_ext,
                           n_faces,
-                          mesh->i_face_cells,
+                          i_face_cells,
                           xa,
                           x,
                           y);
 
+  cs_matrix_finalize();
+
+  cs_mesh_adjacencies_finalize();
+
   cs_log_separator(CS_LOG_PERFORMANCE);
 
   /* Free working arrays */
