@@ -2,13 +2,13 @@
 #define __CS_BLAS_H__
 
 /*============================================================================
- * Portability and fallback layer for BLAS functions
+ * BLAS (Basic Linear Algebra Subroutine) functions
  *============================================================================*/
 
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,26 +48,36 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 /*============================================================================
- *  Public function prototypes for Fortran API
+ * Type definitions
  *============================================================================*/
 
-/* Return the dot product of 2 vectors: x.y */
+/* BLAS reduction algorithm families */
 
-double CS_PROCF(csdot, CSDOT)(const cs_int_t   *n,
-                              const cs_real_t  *x,
-                              const cs_real_t  *y);
+typedef enum {
 
-/* Return the global residual of 2 extensiv vectors: x.y */
+  CS_BLAS_REDUCE_SUPERBLOCK,
+  CS_BLAS_REDUCE_KAHAN
 
-double CS_PROCF(csres, CSRES)(const cs_int_t   *n,
-                              const cs_real_t  *vol,
-                              const cs_real_t  *x,
-                              const cs_real_t  *y);
+} cs_blas_reduce_t;
 
 /*============================================================================
- *  Public function prototypes or wrapper macros
+ *  Public function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set the preferred BLAS reduction algorithm family.
+ *
+ * This may not be enforced for all algorithms, though it should at least
+ * be enforced for the most general functions such as \ref cs_dot.
+ *
+ * \param[in]  mode   BLAS mode to use
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_blas_set_reduce_algorithm(cs_blas_reduce_t  mode);
+
 /*----------------------------------------------------------------------------
  * Constant times a vector plus a vector: y <-- ax + y
  *
@@ -85,14 +95,44 @@ cs_axpy(cs_lnum_t         n,
         cs_real_t        *restrict y);
 
 /*----------------------------------------------------------------------------
- * Return the dot product of 2 vectors: x.y
+ * Return the sum of a vector. For better precision, a superblock algorithm
+ * is used.
  *
- * For better precision, a superblock algorithm is used.
+ * parameters:
+ *   n <-- size of array x
+ *   x <-- array of floating-point values
+ *
+ * returns:
+ *   the resulting sum
+ *----------------------------------------------------------------------------*/
+
+double
+cs_sum(cs_lnum_t         n,
+       const cs_real_t  *x);
+
+/*----------------------------------------------------------------------------
+ * Return the weighted sum of a vector. For better precision, a superblock
+ * algorithm is used.
+ *
+ * \param[in]  n  size of array x
+ * \param[in]  w  array of floating-point weights
+ * \param[in]  x  array of floating-point values
+ *
+ * \return the resulting weighted sum
+ *----------------------------------------------------------------------------*/
+
+double
+cs_weighted_sum(cs_lnum_t         n,
+                const cs_real_t  *w,
+                const cs_real_t  *x);
+
+/*----------------------------------------------------------------------------
+ * Return the dot product of 2 vectors: x.y
  *
  * parameters:
  *   n <-- size of arrays x and y
  *   x <-- array of floating-point values
- *   y<-- array of floating-point values
+ *   y <-- array of floating-point values
  *
  * returns:
  *   dot product
@@ -103,35 +143,11 @@ cs_dot(cs_lnum_t         n,
        const cs_real_t  *x,
        const cs_real_t  *y);
 
-/*----------------------------------------------------------------------------
- * Return the global resildual of 2 extensive vectors:
- *  1/sum(vol) . sum(X.Y/vol)
- *
- * For better precision, a superblock algorithm is used.
- *
- * parameters:
- *   n   <-- size of arrays x and y
- *   vol <-- array of floating-point values
- *   x   <-- array of floating-point values
- *   y   <-- array of floating-point values
- *
- * returns:
- *   dot product
- *----------------------------------------------------------------------------*/
-
-double
-cs_gres(cs_lnum_t         n,
-       const cs_real_t  *vol,
-       const cs_real_t  *x,
-       const cs_real_t  *y);
-
 /*----------------------------------------------------------------------------
  * Return dot products of a vector with itself: x.x
  *
- * For better precision, a superblock algorithm is used.
- *
  * parameters:
- *   n  <-- size of arrays x and y
+ *   n  <-- size of arrays x
  *   x  <-- array of floating-point values
  *
  * returns:
@@ -142,6 +158,25 @@ double
 cs_dot_xx(cs_lnum_t         n,
           const cs_real_t  *x);
 
+/*----------------------------------------------------------------------------
+ * Return weighted dot products of a vector with itself: x.x
+ *
+ * For better precision, a superblock algorithm is used.
+ *
+ * parameters:
+ *   n  <-- size of arrays x
+ *   w  <-- array of weights
+ *   x  <-- array of floating-point values
+ *
+ * returns:
+ *   dot product
+ *----------------------------------------------------------------------------*/
+
+double
+cs_dot_wxx(cs_lnum_t         n,
+           const cs_real_t  *w,
+           const cs_real_t  *x);
+
 /*----------------------------------------------------------------------------
  * Return the double dot product of 2 vectors: x.x, and x.y
  *
@@ -149,8 +184,6 @@ cs_dot_xx(cs_lnum_t         n,
  * simultaneously adds more optimization opportunities and possibly better
  * cache behavior.
  *
- * For better precision, a superblock algorithm is used.
- *
  * parameters:
  *   n  <-- size of arrays x and y
  *   x  <-- array of floating-point values
@@ -173,8 +206,6 @@ cs_dot_xx_xy(cs_lnum_t                    n,
  * simultaneously adds more optimization opportunities and possibly better
  * cache behavior.
  *
- * For better precision, a superblock algorithm is used.
- *
  * parameters:
  *   n  <-- size of arrays x and y
  *   x  <-- array of floating-point values
@@ -193,14 +224,12 @@ cs_dot_xy_yz(cs_lnum_t                    n,
              double                      *xy);
 
 /*----------------------------------------------------------------------------
- * Return 3 dot products of 3 vectors: x.y, x.y, and y.z
+ * Return 3 dot products of 3 vectors: x.x, x.y, and y.z
  *
  * The products could be computed separately, but computing them
  * simultaneously adds more optimization opportunities and possibly better
  * cache behavior.
  *
- * For better precision, a superblock algorithm is used.
- *
  * parameters:
  *   n  <-- size of arrays x and y
  *   x  <-- array of floating-point values
@@ -220,14 +249,42 @@ cs_dot_xx_xy_yz(cs_lnum_t                    n,
                 double                      *xy,
                 double                      *yz);
 
+/*----------------------------------------------------------------------------
+ * Return 5 dot products of 3 vectors: x.x, y.y, x.y, x.z, and y.z
+ *
+ * The products could be computed separately, but computing them
+ * simultaneously adds more optimization opportunities and possibly better
+ * cache behavior.
+ *
+ * parameters:
+ *   n  <-- size of arrays x and y
+ *   x  <-- array of floating-point values
+ *   y  <-- array of floating-point values
+ *   z  <-- array of floating-point values
+ *   xx --> x.y dot product
+ *   yy --> y.y dot product
+ *   xy --> x.y dot product
+ *   xz --> x.z dot product
+ *   yz --> y.z dot product
+ *----------------------------------------------------------------------------*/
+
+void
+cs_dot_xx_yy_xy_xz_yz(cs_lnum_t                    n,
+                      const cs_real_t  *restrict   x,
+                      const cs_real_t  *restrict   y,
+                      const cs_real_t  *restrict   z,
+                      double                      *xx,
+                      double                      *yy,
+                      double                      *xy,
+                      double                      *xz,
+                      double                      *yz);
+
 /*----------------------------------------------------------------------------
  * Return the global dot product of 2 vectors: x.y
  *
  * In parallel mode, the local results are summed on the default
  * global communicator.
  *
- * For better precision, a superblock algorithm is used.
- *
  * parameters:
  *   n <-- size of arrays x and y
  *   x <-- array of floating-point values
@@ -242,6 +299,26 @@ cs_gdot(cs_lnum_t         n,
         const cs_real_t  *x,
         const cs_real_t  *y);
 
+/*----------------------------------------------------------------------------
+ * Return the global residual of 2 extensive vectors:
+ *  1/sum(vol) . sum(X.Y/vol)
+ *
+ * parameters:
+ *   n   <-- size of arrays x and y
+ *   vol <-- array of floating-point values
+ *   x   <-- array of floating-point values
+ *   y   <-- array of floating-point values
+ *
+ * returns:
+ *   global residual
+ *----------------------------------------------------------------------------*/
+
+double
+cs_gres(cs_lnum_t         n,
+        const cs_real_t  *vol,
+        const cs_real_t  *x,
+        const cs_real_t  *y);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
