@@ -4,7 +4,7 @@
 
 /* This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -52,6 +52,7 @@
 #include "cs_blas.h"
 #include "cs_halo.h"
 #include "cs_halo_perio.h"
+#include "cs_internal_coupling.h"
 #include "cs_log.h"
 #include "cs_mesh.h"
 #include "cs_field.h"
@@ -80,26 +81,29 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 /*! \file  cs_face_viscosity.c
-
+ *
+ *  \brief Face viscosity.
+ *
 */
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local type definitions
  *============================================================================*/
 
-/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
-/*! \endcond (end ignore by Doxygen) */
-
 /*============================================================================
  * Private function definitions
  *============================================================================*/
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions for Fortran API
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- * Wrapper to cs_face_viscosity_scalar
+ * Wrapper to cs_face_viscosity
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (viscfa, VISCFA)
@@ -113,16 +117,16 @@ void CS_PROCF (viscfa, VISCFA)
   const cs_mesh_t  *m = cs_glob_mesh;
   cs_mesh_quantities_t  *fvq = cs_glob_mesh_quantities;
 
-  cs_face_viscosity_scalar(m,
-                           fvq,
-                           *visc_mean_type,
-                           c_visc,
-                           i_visc,
-                           b_visc);
+  cs_face_viscosity(m,
+                    fvq,
+                    *visc_mean_type,
+                    c_visc,
+                    i_visc,
+                    b_visc);
 }
 
 /*----------------------------------------------------------------------------
- * Wrapper to cs_face_viscosity_tensor_velocity
+ * Wrapper to cs_face_anisotropic_viscosity_vector
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (vistnv, VISTNV)
@@ -136,16 +140,16 @@ void CS_PROCF (vistnv, VISTNV)
   const cs_mesh_t  *m = cs_glob_mesh;
   cs_mesh_quantities_t  *fvq = cs_glob_mesh_quantities;
 
-  cs_face_viscosity_tensor_velocity(m,
-                                    fvq,
-                                    *visc_mean_type,
-                                    c_visc,
-                                    i_visc,
-                                    b_visc);
+  cs_face_anisotropic_viscosity_vector(m,
+                                       fvq,
+                                       *visc_mean_type,
+                                       c_visc,
+                                       i_visc,
+                                       b_visc);
 }
 
 /*----------------------------------------------------------------------------
- * Wrapper to cs_face_viscosity_tensor
+ * Wrapper to cs_face_anisotropic_viscosity_scalar
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (vitens, VITENS)
@@ -161,14 +165,14 @@ void CS_PROCF (vitens, VITENS)
   const cs_mesh_t  *m = cs_glob_mesh;
   cs_mesh_quantities_t  *fvq = cs_glob_mesh_quantities;
 
-  cs_face_viscosity_tensor(m,
-                           fvq,
-                           c_visc,
-                           *iwarnp,
-                           weighf,
-                           weighb,
-                           i_visc,
-                           b_visc);
+  cs_face_anisotropic_viscosity_scalar(m,
+                                       fvq,
+                                       c_visc,
+                                       *iwarnp,
+                                       weighf,
+                                       weighb,
+                                       i_visc,
+                                       b_visc);
 }
 
 /*============================================================================
@@ -178,10 +182,17 @@ void CS_PROCF (vitens, VITENS)
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Compute the diffusion velocity at faces.
+ *
  * i_visc,b_visc = viscosity*surface/distance, homogeneous to a rate of flow
  * in kg/s.
  *
- * Remark: a priori, no need of reconstruction techniques
+ * <a name="viscfa"></a>
+ *
+ * Please refer to the
+ * <a href="../../theory.pdf#viscfa"><b>viscfa</b></a> section of the theory
+ * guide for more informations.
+ *
+ * \remark: a priori, no need of reconstruction techniques
  * (to improve if necessary).
  *
  * \param[in]     m              pointer to mesh
@@ -198,12 +209,12 @@ void CS_PROCF (vitens, VITENS)
 /*----------------------------------------------------------------------------*/
 
 void
-cs_face_viscosity_scalar(const cs_mesh_t               *m,
-                         cs_mesh_quantities_t          *fvq,
-                         const int                      visc_mean_type,
-                         cs_real_t            *restrict c_visc,
-                         cs_real_t            *restrict i_visc,
-                         cs_real_t            *restrict b_visc)
+cs_face_viscosity(const cs_mesh_t               *m,
+                  const cs_mesh_quantities_t    *fvq,
+                  const int                      visc_mean_type,
+                  cs_real_t            *restrict c_visc,
+                  cs_real_t            *restrict i_visc,
+                  cs_real_t            *restrict b_visc)
 {
   const cs_halo_t  *halo = m->halo;
 
@@ -213,15 +224,15 @@ cs_face_viscosity_scalar(const cs_mesh_t               *m,
     = (const cs_lnum_t *restrict)m->b_face_cells;
   const cs_real_t *restrict weight = fvq->weight;
   const cs_real_t *restrict i_dist = fvq->i_dist;
-  const cs_real_t *restrict i_face_surf = fvq->i_face_surf;
-  const cs_real_t *restrict b_face_surf = fvq->b_face_surf;
+  const cs_real_t *restrict i_f_face_surf = fvq->i_f_face_surf;
+  const cs_real_t *restrict b_f_face_surf = fvq->b_f_face_surf;
 
   /* Porosity field */
-  cs_field_t *fporo = CS_F_(poro);
+  cs_field_t *fporo = cs_field_by_name_try("porosity");
 
   cs_real_t *porosi = NULL;
 
-  if (fporo != NULL) {
+  if (cs_glob_porous_model == 1 || cs_glob_porous_model == 2) {
     porosi = fporo->val;
   }
 
@@ -229,6 +240,7 @@ cs_face_viscosity_scalar(const cs_mesh_t               *m,
   if (halo != NULL) {
     cs_halo_type_t halo_type = CS_HALO_STANDARD;
     cs_halo_sync_var(halo, halo_type, c_visc);
+    if (porosi != NULL) cs_halo_sync_var(halo, halo_type, porosi);
   }
 
   /* Without porosity */
@@ -238,14 +250,14 @@ cs_face_viscosity_scalar(const cs_mesh_t               *m,
 
       for (cs_lnum_t face_id = 0; face_id < m->n_i_faces; face_id++) {
 
-        cs_lnum_t ii = i_face_cells[face_id][0] - 1;
-        cs_lnum_t jj = i_face_cells[face_id][1] - 1;
+        cs_lnum_t ii = i_face_cells[face_id][0];
+        cs_lnum_t jj = i_face_cells[face_id][1];
 
         double visci = c_visc[ii];
         double viscj = c_visc[jj];
 
         i_visc[face_id] = 0.5*(visci+viscj)
-                         *i_face_surf[face_id]/i_dist[face_id];
+                         *i_f_face_surf[face_id]/i_dist[face_id];
 
       }
 
@@ -253,15 +265,16 @@ cs_face_viscosity_scalar(const cs_mesh_t               *m,
 
       for (cs_lnum_t face_id = 0; face_id < m->n_i_faces; face_id++) {
 
-        cs_lnum_t ii = i_face_cells[face_id][0] - 1;
-        cs_lnum_t jj = i_face_cells[face_id][1] - 1;
+        cs_lnum_t ii = i_face_cells[face_id][0];
+        cs_lnum_t jj = i_face_cells[face_id][1];
 
         double visci = c_visc[ii];
         double viscj = c_visc[jj];
         double pnd   = weight[face_id];
 
-        i_visc[face_id] = visci*viscj/(pnd*visci+(1.-pnd)*viscj)
-                         *i_face_surf[face_id]/i_dist[face_id];
+        i_visc[face_id] = visci*viscj/CS_MAX(pnd*visci+(1.-pnd)*viscj,
+                                             DBL_MIN)
+                         *i_f_face_surf[face_id]/i_dist[face_id];
 
       }
 
@@ -269,7 +282,7 @@ cs_face_viscosity_scalar(const cs_mesh_t               *m,
 
     for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
 
-      b_visc[face_id] = b_face_surf[face_id];
+      b_visc[face_id] = b_f_face_surf[face_id];
 
     }
 
@@ -280,14 +293,14 @@ cs_face_viscosity_scalar(const cs_mesh_t               *m,
 
       for (cs_lnum_t face_id = 0; face_id < m->n_i_faces; face_id++) {
 
-        cs_lnum_t ii = i_face_cells[face_id][0] - 1;
-        cs_lnum_t jj = i_face_cells[face_id][1] - 1;
+        cs_lnum_t ii = i_face_cells[face_id][0];
+        cs_lnum_t jj = i_face_cells[face_id][1];
 
         double visci = c_visc[ii] * porosi[ii];
         double viscj = c_visc[jj] * porosi[jj];
 
         i_visc[face_id] = 0.5*(visci+viscj)
-                         *i_face_surf[face_id]/i_dist[face_id];
+                         *i_f_face_surf[face_id]/i_dist[face_id];
 
       }
 
@@ -295,15 +308,16 @@ cs_face_viscosity_scalar(const cs_mesh_t               *m,
 
       for (cs_lnum_t face_id = 0; face_id <m->n_i_faces; face_id++) {
 
-        cs_lnum_t ii = i_face_cells[face_id][0] - 1;
-        cs_lnum_t jj = i_face_cells[face_id][1] - 1;
+        cs_lnum_t ii = i_face_cells[face_id][0];
+        cs_lnum_t jj = i_face_cells[face_id][1];
 
         double visci = c_visc[ii] * porosi[ii];
         double viscj = c_visc[jj] * porosi[jj];
         double pnd   = weight[face_id];
 
-        i_visc[face_id] = visci*viscj/(pnd*visci+(1.-pnd)*viscj)
-                         *i_face_surf[face_id]/i_dist[face_id];
+        i_visc[face_id] = visci*viscj/CS_MAX(pnd*visci+(1.-pnd)*viscj,
+                                             DBL_MIN)
+                         *i_f_face_surf[face_id]/i_dist[face_id];
 
       }
 
@@ -311,9 +325,9 @@ cs_face_viscosity_scalar(const cs_mesh_t               *m,
 
     for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
 
-      cs_lnum_t ii = b_face_cells[face_id] - 1;
+      cs_lnum_t ii = b_face_cells[face_id];
 
-      b_visc[face_id] = b_face_surf[face_id]*porosi[ii];
+      b_visc[face_id] = b_f_face_surf[face_id]*porosi[ii];
 
     }
 
@@ -339,12 +353,12 @@ cs_face_viscosity_scalar(const cs_mesh_t               *m,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
-                                  cs_mesh_quantities_t          *fvq,
-                                  const int                      visc_mean_type,
-                                  cs_real_6_t          *restrict c_visc,
-                                  cs_real_33_t         *restrict i_visc,
-                                  cs_real_t            *restrict b_visc)
+cs_face_anisotropic_viscosity_vector(const cs_mesh_t             *m,
+                                     const cs_mesh_quantities_t  *fvq,
+                                     const int                    visc_mean_type,
+                                     cs_real_6_t        *restrict c_visc,
+                                     cs_real_33_t       *restrict i_visc,
+                                     cs_real_t          *restrict b_visc)
 {
   const cs_halo_t  *halo = m->halo;
 
@@ -357,8 +371,8 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
     = (const cs_lnum_t *restrict)m->b_face_cells;
   const cs_real_t *restrict weight = fvq->weight;
   const cs_real_t *restrict i_dist = fvq->i_dist;
-  const cs_real_t *restrict i_face_surf = fvq->i_face_surf;
-  const cs_real_t *restrict b_face_surf = fvq->b_face_surf;
+  const cs_real_t *restrict i_f_face_surf = fvq->i_f_face_surf;
+  const cs_real_t *restrict b_f_face_surf = fvq->b_f_face_surf;
 
   double visci[3][3], viscj[3][3], s1[6], s2[6];
 
@@ -366,13 +380,13 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
   cs_real_6_t *w2 = NULL;
 
   /* Porosity fields */
-  cs_field_t *fporo = CS_F_(poro);
-  cs_field_t *ftporo = CS_F_(t_poro);
+  cs_field_t *fporo = cs_field_by_name_try("porosity");
+  cs_field_t *ftporo = cs_field_by_name_try("tensorial_porosity");
 
   cs_real_t *porosi = NULL;
   cs_real_6_t *porosf = NULL;
 
-  if (fporo != NULL) {
+  if (cs_glob_porous_model == 1 || cs_glob_porous_model == 2) {
     porosi = fporo->val;
     if (ftporo != NULL) {
       porosf = (cs_real_6_t *)ftporo->val;
@@ -400,9 +414,9 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
 
     BFT_MALLOC(w2, n_cells_ext, cs_real_6_t);
     for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-      cs_math_sym_33_product(w2[cell_id],
-                             porosf[cell_id],
-                             c_visc[cell_id]);
+      cs_math_sym_33_product(porosf[cell_id],
+                             c_visc[cell_id],
+                             w2[cell_id]);
     }
     c_poro_visc = w2;
 
@@ -423,8 +437,8 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
 
     for (cs_lnum_t face_id = 0; face_id < m->n_i_faces; face_id++) {
 
-      cs_lnum_t ii = i_face_cells[face_id][0] - 1;
-      cs_lnum_t jj = i_face_cells[face_id][1] - 1;
+      cs_lnum_t ii = i_face_cells[face_id][0];
+      cs_lnum_t jj = i_face_cells[face_id][1];
 
       visci[0][0] = c_poro_visc[ii][0];
       visci[1][1] = c_poro_visc[ii][1];
@@ -450,7 +464,7 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
         for (int jsou = 0; jsou < 3; jsou++) {
           i_visc[face_id][jsou][isou] =  0.5*(visci[jsou][isou]
                                              +viscj[jsou][isou])
-                                       * i_face_surf[face_id]/i_dist[face_id];
+                                       * i_f_face_surf[face_id]/i_dist[face_id];
         }
       }
 
@@ -461,8 +475,8 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
 
     for (cs_lnum_t face_id = 0; face_id < m->n_i_faces; face_id++) {
 
-      cs_lnum_t ii = i_face_cells[face_id][0] - 1;
-      cs_lnum_t jj = i_face_cells[face_id][1] - 1;
+      cs_lnum_t ii = i_face_cells[face_id][0];
+      cs_lnum_t jj = i_face_cells[face_id][1];
 
       double pnd = weight[face_id];
 
@@ -470,13 +484,13 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
         s1[isou] = pnd*c_poro_visc[ii][isou] + (1.-pnd)*c_poro_visc[jj][isou];
       }
 
-      cs_math_sym_33_inv_cramer(s2, s1);
+      cs_math_sym_33_inv_cramer(s1, s2);
 
-      cs_math_sym_33_product(s1, s2, c_poro_visc[jj]);
+      cs_math_sym_33_product(s2, c_poro_visc[jj], s1);
 
-      cs_math_sym_33_product(s2, c_poro_visc[ii], s1);
+      cs_math_sym_33_product(c_poro_visc[ii], s1, s2);
 
-      double srfddi = i_face_surf[face_id]/i_dist[face_id];
+      double srfddi = i_f_face_surf[face_id]/i_dist[face_id];
 
       i_visc[face_id][0][0] = s2[0]*srfddi;
       i_visc[face_id][1][1] = s2[1]*srfddi;
@@ -497,7 +511,7 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
 
     for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
 
-      b_visc[face_id] = b_face_surf[face_id];
+      b_visc[face_id] = b_f_face_surf[face_id];
 
     }
 
@@ -506,9 +520,9 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
 
     for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
 
-      cs_lnum_t ii = b_face_cells[face_id] - 1;
+      cs_lnum_t ii = b_face_cells[face_id];
 
-      b_visc[face_id] = b_face_surf[face_id]*porosi[ii];
+      b_visc[face_id] = b_f_face_surf[face_id]*porosi[ii];
 
     }
 
@@ -517,9 +531,9 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
 
     for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
 
-      cs_lnum_t ii = b_face_cells[face_id] - 1;
+      cs_lnum_t ii = b_face_cells[face_id];
 
-      b_visc[face_id] = b_face_surf[face_id]*porosi[ii];
+      b_visc[face_id] = b_f_face_surf[face_id]*porosi[ii];
 
     }
 
@@ -554,14 +568,14 @@ cs_face_viscosity_tensor_velocity(const cs_mesh_t               *m,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_face_viscosity_tensor(const cs_mesh_t               *m,
-                         cs_mesh_quantities_t          *fvq,
-                         cs_real_6_t          *restrict c_visc,
-                         const int                      iwarnp,
-                         cs_real_2_t          *restrict weighf,
-                         cs_real_t            *restrict weighb,
-                         cs_real_t            *restrict i_visc,
-                         cs_real_t            *restrict b_visc)
+cs_face_anisotropic_viscosity_scalar(const cs_mesh_t               *m,
+                                     const cs_mesh_quantities_t    *fvq,
+                                     cs_real_6_t          *restrict c_visc,
+                                     const int                      iwarnp,
+                                     cs_real_2_t          *restrict weighf,
+                                     cs_real_t            *restrict weighb,
+                                     cs_real_t            *restrict i_visc,
+                                     cs_real_t            *restrict b_visc)
 {
   const cs_halo_t  *halo = m->halo;
 
@@ -575,11 +589,15 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
   const cs_real_t *restrict weight = fvq->weight;
   const cs_real_t *restrict i_dist = fvq->i_dist;
   const cs_real_t *restrict b_dist = fvq->b_dist;
-  const cs_real_t *restrict b_face_surf = fvq->b_face_surf;
+  const cs_real_t *restrict b_f_face_surf = fvq->b_f_face_surf;
   const cs_real_3_t *restrict cell_cen
     = (const cs_real_3_t *restrict)fvq->cell_cen;
   const cs_real_3_t *restrict i_face_normal
     = (const cs_real_3_t *restrict)fvq->i_face_normal;
+  const cs_real_t *restrict i_face_surf
+    = (const cs_real_t *restrict)fvq->i_face_surf;
+  const cs_real_t *restrict i_f_face_surf
+    = (const cs_real_t *restrict)fvq->i_f_face_surf;
   const cs_real_3_t *restrict b_face_normal
     = (const cs_real_3_t *restrict)fvq->b_face_normal;
   const cs_real_3_t *restrict i_face_cog
@@ -588,20 +606,19 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
     = (const cs_real_3_t *restrict)fvq->b_face_cog;
 
   cs_gnum_t nclipf = 0, nclipb = 0;
-  cs_real_33_t visci, viscj;
   const double eps = 0.1;
 
   cs_real_6_t *c_poro_visc = NULL;
   cs_real_6_t *w2 = NULL;
 
   /* Porosity fields */
-  cs_field_t *fporo = CS_F_(poro);
-  cs_field_t *ftporo = CS_F_(t_poro);
+  cs_field_t *fporo = cs_field_by_name_try("porosity");
+  cs_field_t *ftporo = cs_field_by_name_try("tensorial_porosity");
 
   cs_real_t *porosi = NULL;
   cs_real_6_t *porosf = NULL;
 
-  if (fporo != NULL) {
+  if (cs_glob_porous_model == 1 || cs_glob_porous_model == 2) {
     porosi = fporo->val;
     if (ftporo != NULL) {
       porosf = (cs_real_6_t *)ftporo->val;
@@ -629,9 +646,9 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
 
     BFT_MALLOC(w2, n_cells_ext, cs_real_6_t);
     for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
-      cs_math_sym_33_product(w2[cell_id],
-                             porosf[cell_id],
-                             c_visc[cell_id]);
+      cs_math_sym_33_product(porosf[cell_id],
+                             c_visc[cell_id],
+                             w2[cell_id]);
     }
     c_poro_visc = w2;
 
@@ -650,22 +667,12 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
   /* Always Harmonic mean */
   for (cs_lnum_t face_id = 0; face_id < m->n_i_faces; face_id++) {
 
-    cs_lnum_t ii = i_face_cells[face_id][0] - 1;
-    cs_lnum_t jj = i_face_cells[face_id][1] - 1;
-
-    visci[0][0] = c_poro_visc[ii][0];
-    visci[1][1] = c_poro_visc[ii][1];
-    visci[2][2] = c_poro_visc[ii][2];
-    visci[1][0] = c_poro_visc[ii][3];
-    visci[0][1] = c_poro_visc[ii][3];
-    visci[2][1] = c_poro_visc[ii][4];
-    visci[1][2] = c_poro_visc[ii][4];
-    visci[2][0] = c_poro_visc[ii][5];
-    visci[0][2] = c_poro_visc[ii][5];
+    cs_lnum_t ii = i_face_cells[face_id][0];
+    cs_lnum_t jj = i_face_cells[face_id][1];
 
     /* ||Ki.S||^2 */
     cs_real_3_t viscisv;
-    cs_math_33_3_product(visci, i_face_normal[face_id], viscisv);
+    cs_math_sym_33_3_product(c_poro_visc[ii], i_face_normal[face_id], viscisv);
     cs_real_t viscis = cs_math_3_square_norm(viscisv);
 
     /* IF */
@@ -675,7 +682,7 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
 
     /* IF.Ki.S */
     cs_real_3_t fiki;
-    cs_math_33_3_product(visci, fi, fiki);
+    cs_math_sym_33_3_product(c_poro_visc[ii], fi, fiki);
     cs_real_t fikis = cs_math_3_dot_product(fiki, i_face_normal[face_id]);
 
     double distfi = (1. - weight[face_id])*i_dist[face_id];
@@ -687,19 +694,9 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
       nclipf++;
     }
 
-    viscj[0][0] = c_poro_visc[jj][0];
-    viscj[1][1] = c_poro_visc[jj][1];
-    viscj[2][2] = c_poro_visc[jj][2];
-    viscj[1][0] = c_poro_visc[jj][3];
-    viscj[0][1] = c_poro_visc[jj][3];
-    viscj[2][1] = c_poro_visc[jj][4];
-    viscj[1][2] = c_poro_visc[jj][4];
-    viscj[2][0] = c_poro_visc[jj][5];
-    viscj[0][2] = c_poro_visc[jj][5];
-
     /* ||Kj.S||^2 */
     cs_real_3_t viscjsv;
-    cs_math_33_3_product(viscj, i_face_normal[face_id], viscjsv);
+    cs_math_sym_33_3_product(c_poro_visc[jj], i_face_normal[face_id], viscjsv);
     cs_real_t viscjs = cs_math_3_square_norm(viscjsv);
 
     /* FJ */
@@ -709,7 +706,7 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
 
     /* FJ.Kj.S */
     cs_real_3_t fjkj;
-    cs_math_33_3_product(viscj, fj, fjkj);
+    cs_math_sym_33_3_product(c_poro_visc[jj], fj, fjkj);
     cs_real_t fjkjs = cs_math_3_dot_product(fjkj, i_face_normal[face_id]);
 
     double distfj = weight[face_id]*i_dist[face_id];
@@ -728,23 +725,20 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
 
   }
 
+  /* For the porous modelling based on integral formulation Section and fluid
+   * Section are different */
+  if (cs_glob_porous_model == 3) {
+    for (cs_lnum_t face_id = 0; face_id < m->n_i_faces; face_id++)
+      i_visc[face_id] *= i_f_face_surf[face_id] / i_face_surf[face_id];
+  }
+
   for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
 
-    cs_lnum_t ii = b_face_cells[face_id] - 1;
-
-    visci[0][0] = c_poro_visc[ii][0];
-    visci[1][1] = c_poro_visc[ii][1];
-    visci[2][2] = c_poro_visc[ii][2];
-    visci[1][0] = c_poro_visc[ii][3];
-    visci[0][1] = c_poro_visc[ii][3];
-    visci[2][1] = c_poro_visc[ii][4];
-    visci[1][2] = c_poro_visc[ii][4];
-    visci[2][0] = c_poro_visc[ii][5];
-    visci[0][2] = c_poro_visc[ii][5];
+    cs_lnum_t ii = b_face_cells[face_id];
 
     /* ||Ki.S||^2 */
     cs_real_3_t viscisv;
-    cs_math_33_3_product(visci, b_face_normal[face_id], viscisv);
+    cs_math_sym_33_3_product(c_poro_visc[ii], b_face_normal[face_id], viscisv);
     cs_real_t viscis = cs_math_3_square_norm(viscisv);
 
     /* IF */
@@ -754,7 +748,7 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
 
     /* IF.Ki.S */
     cs_real_3_t fiki;
-    cs_math_33_3_product(visci, fi, fiki);
+    cs_math_sym_33_3_product(c_poro_visc[ii], fi, fiki);
     cs_real_t fikis = cs_math_3_dot_product(fiki, b_face_normal[face_id]);
 
     double distfi = b_dist[face_id];
@@ -776,7 +770,7 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
     for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
 
       /* Warning: hint must be ||Ki.n||/I"F */
-      b_visc[face_id] = b_face_surf[face_id];
+      b_visc[face_id] = b_f_face_surf[face_id];
     }
 
   /* With porosity */
@@ -784,10 +778,10 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
 
     for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
 
-      cs_lnum_t ii = b_face_cells[face_id] - 1;
+      cs_lnum_t ii = b_face_cells[face_id];
 
       /* Warning: hint must be ||Ki.n||/I"F */
-      b_visc[face_id] = b_face_surf[face_id]*porosi[ii];
+      b_visc[face_id] = b_f_face_surf[face_id]*porosi[ii];
 
     }
 
@@ -796,10 +790,10 @@ cs_face_viscosity_tensor(const cs_mesh_t               *m,
 
     for (cs_lnum_t face_id = 0; face_id < m->n_b_faces; face_id++) {
 
-      cs_lnum_t ii = b_face_cells[face_id] - 1;
+      cs_lnum_t ii = b_face_cells[face_id];
 
       /* Warning: hint must be ||Ki.n||/I"F */
-      b_visc[face_id] = b_face_surf[face_id]*porosi[ii];
+      b_visc[face_id] = b_f_face_surf[face_id]*porosi[ii];
 
     }
 
