@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -55,55 +55,32 @@ BEGIN_C_DECLS
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- * Wrapper to cs_matrix_scalar (or its counterpart for
+ * Fortran wrapper to cs_matrix_scalar (or its counterpart for
  * symmetric matrices)
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (matrix, MATRIX)
 (
- const cs_int_t  *const   iconvp,
- const cs_int_t  *const   idiffp,
- const cs_int_t  *const   ndircp,
- const cs_int_t  *const   isym,
- const cs_real_t *const   thetap,
- const cs_int_t  *const   imucpp,
- const cs_real_t          coefbp[],
- const cs_real_t          cofbfp[],
- const cs_real_t          rovsdt[],
- const cs_real_t          i_massflux[],
- const cs_real_t          b_massflux[],
- const cs_real_t          i_visc[],
- const cs_real_t          b_visc[],
- const cs_real_t          xcpp[],
- cs_real_t                da[],
- cs_real_t                xa[]
+ const int        *iconvp,
+ const int        *idiffp,
+ const int        *ndircp,
+ const int        *isym,
+ const cs_real_t  *thetap,
+ const int        *imucpp,
+ const cs_real_t   coefbp[],
+ const cs_real_t   cofbfp[],
+ const cs_real_t   rovsdt[],
+ const cs_real_t   i_massflux[],
+ const cs_real_t   b_massflux[],
+ const cs_real_t   i_visc[],
+ const cs_real_t   b_visc[],
+ const cs_real_t   xcpp[],
+ cs_real_t         da[],
+ cs_real_t         xa[]
 );
 
 /*----------------------------------------------------------------------------
- * Wrapper to cs_matrix_vector (or its counterpart for
- * symmetric matrices)
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (matrxv, MATRXV)
-(
- const cs_int_t  *const   iconvp,
- const cs_int_t  *const   idiffp,
- const cs_int_t  *const   ndircp,
- const cs_int_t  *const   isym,
- const cs_real_t *const   thetap,
- const cs_real_33_t       coefbu[],
- const cs_real_33_t       cofbfu[],
- const cs_real_33_t       fimp[],
- const cs_real_t          i_massflux[],
- const cs_real_t          b_massflux[],
- const cs_real_t          i_visc[],
- const cs_real_t          b_visc[],
- cs_real_33_t             da[],
- cs_real_t                xa[]
-);
-
-/*----------------------------------------------------------------------------
- * Wrapper to cs_matrix_time_step
+ * Fortran wrapper to cs_matrix_time_step
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF (matrdt, MATRDT)
@@ -120,33 +97,103 @@ void CS_PROCF (matrdt, MATRDT)
  cs_real_t                da[]
 );
 
-/*----------------------------------------------------------------------------
- * Wrapper to cs_matrix_anisotropic_diffusion (or its counterpart for
- * symmetric matrices)
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (matrvv, MATRVV)
-(
- const cs_int_t  *const   iconvp,
- const cs_int_t  *const   idiffp,
- const cs_int_t  *const   ndircp,
- const cs_int_t  *const   isym,
- const cs_real_t *const   thetap,
- const cs_real_33_t       coefbu[],
- const cs_real_33_t       cofbfu[],
- const cs_real_33_t       fimp[],
- const cs_real_t          i_massflux[],
- const cs_real_t          b_massflux[],
- const cs_real_33_t       i_visc[],
- const cs_real_t          b_visc[],
- cs_real_33_t             da[],
- cs_real_332_t            xa[]
-);
-
 /*=============================================================================
  * Public function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Wrapper to cs_matrix_scalar (or its counterpart for
+ * symmetric matrices)
+ *----------------------------------------------------------------------------*/
+
+void
+cs_matrix_wrapper_scalar(int               iconvp,
+                         int               idiffp,
+                         int               ndircp,
+                         int               isym,
+                         double            thetap,
+                         int               imucpp,
+                         const cs_real_t   coefbp[],
+                         const cs_real_t   cofbfp[],
+                         const cs_real_t   rovsdt[],
+                         const cs_real_t   i_massflux[],
+                         const cs_real_t   b_massflux[],
+                         const cs_real_t   i_visc[],
+                         const cs_real_t   b_visc[],
+                         const cs_real_t   xcpp[],
+                         cs_real_t         da[],
+                         cs_real_t         xa[]);
+
+/*----------------------------------------------------------------------------
+ * Wrapper to cs_matrix_scalar for convection/diffusion multigrid
+ *----------------------------------------------------------------------------*/
+
+void
+cs_matrix_wrapper_scalar_conv_diff(int               iconvp,
+                                   int               idiffp,
+                                   int               ndircp,
+                                   double            thetap,
+                                   int               imucpp,
+                                   const cs_real_t   coefbp[],
+                                   const cs_real_t   cofbfp[],
+                                   const cs_real_t   rovsdt[],
+                                   const cs_real_t   i_massflux[],
+                                   const cs_real_t   b_massflux[],
+                                   const cs_real_t   i_visc[],
+                                   const cs_real_t   b_visc[],
+                                   const cs_real_t   xcpp[],
+                                   cs_real_t         da[],
+                                   cs_real_t         xa[],
+                                   cs_real_t         da_conv[],
+                                   cs_real_t         xa_conv[],
+                                   cs_real_t         da_diff[],
+                                   cs_real_t         xa_diff[]);
+
+/*----------------------------------------------------------------------------
+ * Wrapper to cs_matrix_vector (or its counterpart for
+ * symmetric matrices)
+ *----------------------------------------------------------------------------*/
+
+void
+cs_matrix_wrapper_vector(int                  iconvp,
+                         int                  idiffp,
+                         int                  tensorial_diffusion,
+                         int                  ndircp,
+                         int                  isym,
+                         int                  eb_size[4],
+                         double               thetap,
+                         const cs_real_33_t   coefbu[],
+                         const cs_real_33_t   cofbfu[],
+                         const cs_real_33_t   fimp[],
+                         const cs_real_t      i_massflux[],
+                         const cs_real_t      b_massflux[],
+                         const cs_real_t      i_visc[],
+                         const cs_real_t      b_visc[],
+                         cs_real_33_t         da[],
+                         cs_real_t            xa[]);
+
+/*----------------------------------------------------------------------------
+ * Wrapper to cs_matrix_tensor (or its counterpart for
+ * symmetric matrices)
+ *----------------------------------------------------------------------------*/
+
+void
+cs_matrix_wrapper_tensor(int                  iconvp,
+                         int                  idiffp,
+                         int                  tensorial_diffusion,
+                         int                  ndircp,
+                         int                  isym,
+                         double               thetap,
+                         const cs_real_66_t   coefbts[],
+                         const cs_real_66_t   cofbfts[],
+                         const cs_real_66_t   fimp[],
+                         const cs_real_t      i_massflux[],
+                         const cs_real_t      b_massflux[],
+                         const cs_real_t      i_visc[],
+                         const cs_real_t      b_visc[],
+                         cs_real_66_t         da[],
+                         cs_real_t            xa[]);
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Build the diffusion matrix for a scalar field.
@@ -161,16 +208,14 @@ void CS_PROCF (matrvv, MATRVV)
  * \param[in]     idiffp        indicator
  *                               - 1 diffusion
  *                               - 0 otherwise
- * \param[in]     ndircp        indicator
- *                               - 0 if the diagonal stepped aside
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
  * \param[in]     cofbfp        boundary condition array for the variable flux
- *                               (Implicit part)
+ *                               (implicit part)
  * \param[in]     rovsdt        working array
  * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
  *                               at interior faces for the matrix
@@ -184,7 +229,6 @@ void CS_PROCF (matrvv, MATRVV)
 void
 cs_sym_matrix_scalar(const cs_mesh_t          *m,
                      int                       idiffp,
-                     int                       ndircp,
                      double                    thetap,
                      const cs_real_t           cofbfp[],
                      const cs_real_t           rovsdt[],
@@ -192,6 +236,102 @@ cs_sym_matrix_scalar(const cs_mesh_t          *m,
                      const cs_real_t           b_visc[],
                      cs_real_t       *restrict da,
                      cs_real_t       *restrict xa);
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Build the diffusion matrix for a tensor field
+ * (symmetric matrix).
+ *
+ * The diffusion is not reconstructed.
+ * The matrix is split into a diagonal block (6x6 times number of cells)
+ * and an extra diagonal part (of dimension the number of internal
+ * faces).
+ *
+ * \param[in]     m             pointer to mesh structure
+ * \param[in]     idiffp        indicator
+ *                               - 1 diffusion
+ *                               - 0 otherwise
+ * \param[in]     thetap        weighting coefficient for the theta-scheme,
+ *                               - thetap = 0: explicit scheme
+ *                               - thetap = 0.5: time-centered
+ *                               scheme (mix between Crank-Nicolson and
+ *                               Adams-Bashforth)
+ *                               - thetap = 1: implicit scheme
+ * \param[in]     cofbfts        boundary condition array for the variable flux
+ *                               (Implicit part - 6x6 tensor array)
+ * \param[in]     fimp          part of the diagonal
+ * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
+ *                               at interior faces for the matrix
+ * \param[in]     b_visc        \f$ \mu_\fib \dfrac{S_\fib}{\ipf \centf} \f$
+ *                               at border faces for the matrix
+ * \param[out]    da            diagonal part of the matrix
+ * \param[out]    xa            extra interleaved diagonal part of the matrix
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sym_matrix_tensor(const cs_mesh_t          *m,
+                     int                       idiffp,
+                     double                    thetap,
+                     const cs_real_66_t        cofbfts[],
+                     const cs_real_66_t        fimp[],
+                     const cs_real_t           i_visc[],
+                     const cs_real_t           b_visc[],
+                     cs_real_66_t              *restrict da,
+                     cs_real_t                 *restrict xa);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Build the advection/diffusion matrix for a tensor field
+ * (non-symmetric matrix).
+ *
+ * The advection is upwind, the diffusion is not reconstructed.
+ * The matrix is split into a diagonal block (6x6 times number of cells)
+ * and an extra diagonal part (of dimension 2 time the number of internal
+ * faces).
+ *
+ * \param[in]     m             pointer to mesh structure
+ * \param[in]     iconvp        indicator
+ *                               - 1 advection
+ *                               - 0 otherwise
+ * \param[in]     idiffp        indicator
+ *                               - 1 diffusion
+ *                               - 0 otherwise
+ * \param[in]     thetap        weighting coefficient for the theta-scheme,
+ *                               - thetap = 0: explicit scheme
+ *                               - thetap = 0.5: time-centered
+ *                               scheme (mix between Crank-Nicolson and
+ *                               Adams-Bashforth)
+ *                               - thetap = 1: implicit scheme
+ * \param[in]     coefbts        boundary condition array for the variable
+ *                               (Implicit part - 6x6 tensor array)
+ * \param[in]     cofbfts        boundary condition array for the variable flux
+ *                               (Implicit part - 6x6 tensor array)
+ * \param[in]     fimp          part of the diagonal
+ * \param[in]     i_massflux    mass flux at interior faces
+ * \param[in]     b_massflux    mass flux at border faces
+ * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
+ *                               at interior faces for the matrix
+ * \param[in]     b_visc        \f$ \mu_\fib \dfrac{S_\fib}{\ipf \centf} \f$
+ *                               at border faces for the matrix
+ * \param[out]    da            diagonal part of the matrix
+ * \param[out]    xa            extra interleaved diagonal part of the matrix
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_matrix_tensor(const cs_mesh_t          *m,
+                 int                       iconvp,
+                 int                       idiffp,
+                 double                    thetap,
+                 const cs_real_66_t        coefbts[],
+                 const cs_real_66_t        cofbfts[],
+                 const cs_real_66_t        fimp[],
+                 const cs_real_t           i_massflux[],
+                 const cs_real_t           b_massflux[],
+                 const cs_real_t           i_visc[],
+                 const cs_real_t           b_visc[],
+                 cs_real_66_t              *restrict da,
+                 cs_real_2_t               *restrict xa);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -209,11 +349,9 @@ cs_sym_matrix_scalar(const cs_mesh_t          *m,
  * \param[in]     idiffp        indicator
  *                               - 1 diffusion
  *                               - 0 otherwise
- * \param[in]     ndircp        indicator
- *                               - 0 if the diagonal stepped aside
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
@@ -221,9 +359,9 @@ cs_sym_matrix_scalar(const cs_mesh_t          *m,
  *                               - 0 do not multiply the convective term by Cp
  *                               - 1 do multiply the convective term by Cp
  * \param[in]     coefbp        boundary condition array for the variable
- *                               (Implicit part)
+ *                               (implicit part)
  * \param[in]     cofbfp        boundary condition array for the variable flux
- *                               (Implicit part)
+ *                               (implicit part)
  * \param[in]     rovsdt        working array
  * \param[in]     i_massflux    mass flux at interior faces
  * \param[in]     b_massflux    mass flux at border faces
@@ -241,7 +379,6 @@ void
 cs_matrix_scalar(const cs_mesh_t          *m,
                  int                       iconvp,
                  int                       idiffp,
-                 int                       ndircp,
                  double                    thetap,
                  int                       imucpp,
                  const cs_real_t           coefbp[],
@@ -269,16 +406,15 @@ cs_matrix_scalar(const cs_mesh_t          *m,
  * \param[in]     idiffp        indicator
  *                               - 1 diffusion
  *                               - 0 otherwise
- * \param[in]     ndircp        indicator
- *                               - 0 if the diagonal stepped aside
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
  * \param[in]     cofbfu        boundary condition array for the variable flux
- *                               (Implicit part - 3x3 tensor array)
+ *                               (implicit part - 3x3 tensor array)
+ * \param[in]     fimp          part of the diagonal
  * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
  *                               at interior faces for the matrix
  * \param[in]     b_visc        \f$ \mu_\fib \dfrac{S_\fib}{\ipf \centf} \f$
@@ -291,7 +427,6 @@ cs_matrix_scalar(const cs_mesh_t          *m,
 void
 cs_sym_matrix_vector(const cs_mesh_t          *m,
                      int                       idiffp,
-                     int                       ndircp,
                      double                    thetap,
                      const cs_real_33_t        cofbfu[],
                      const cs_real_33_t        fimp[],
@@ -311,24 +446,24 @@ cs_sym_matrix_vector(const cs_mesh_t          *m,
  * faces).
  *
  * \param[in]     m             pointer to mesh structure
+ * \param[in]     mq            pointer to mesh quantities structure
  * \param[in]     iconvp        indicator
  *                               - 1 advection
  *                               - 0 otherwise
  * \param[in]     idiffp        indicator
  *                               - 1 diffusion
  *                               - 0 otherwise
- * \param[in]     ndircp        indicator
- *                               - 0 if the diagonal stepped aside
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
  * \param[in]     coefbu        boundary condition array for the variable
- *                               (Implicit part - 3x3 tensor array)
+ *                               (implicit part - 3x3 tensor array)
  * \param[in]     cofbfu        boundary condition array for the variable flux
- *                               (Implicit part - 3x3 tensor array)
+ *                               (implicit part - 3x3 tensor array)
+ * \param[in]     fimp          part of the diagonal
  * \param[in]     i_massflux    mass flux at interior faces
  * \param[in]     b_massflux    mass flux at border faces
  * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
@@ -341,20 +476,21 @@ cs_sym_matrix_vector(const cs_mesh_t          *m,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_matrix_vector(const cs_mesh_t          *m,
-                 int                       iconvp,
-                 int                       idiffp,
-                 int                       ndircp,
-                 double                    thetap,
-                 const cs_real_33_t        coefbu[],
-                 const cs_real_33_t        cofbfu[],
-                 const cs_real_33_t        fimp[],
-                 const cs_real_t           i_massflux[],
-                 const cs_real_t           b_massflux[],
-                 const cs_real_t           i_visc[],
-                 const cs_real_t           b_visc[],
-                 cs_real_33_t    *restrict da,
-                 cs_real_2_t     *restrict xa);
+cs_matrix_vector(const cs_mesh_t            *m,
+                 const cs_mesh_quantities_t *mq,
+                 int                         iconvp,
+                 int                         idiffp,
+                 int                         eb_size[4],
+                 double                      thetap,
+                 const cs_real_33_t          coefbu[],
+                 const cs_real_33_t          cofbfu[],
+                 const cs_real_33_t          fimp[],
+                 const cs_real_t             i_massflux[],
+                 const cs_real_t             b_massflux[],
+                 const cs_real_t             i_visc[],
+                 const cs_real_t             b_visc[],
+                 cs_real_33_t      *restrict da,
+                 cs_real_2_t       *restrict xa);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -372,9 +508,9 @@ cs_matrix_vector(const cs_mesh_t          *m,
  *                               - 1 symmetric matrix
  *                               - 2 non symmmetric matrix
  * \param[in]     coefbp        boundary condition array for the variable
- *                               (Implicit part)
+ *                               (implicit part)
  * \param[in]     cofbfp        boundary condition array for the variable flux
- *                               (Implicit part)
+ *                               (implicit part)
  * \param[in]     i_massflux    mass flux at interior faces
  * \param[in]     b_massflux    mass flux at border faces
  * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
@@ -409,24 +545,23 @@ cs_matrix_time_step(const cs_mesh_t          *m,
  * faces).
  *
  * \param[in]     m             pointer to mesh structure
+ * \param[in]     mq            pointer to mesh quantities structure
  * \param[in]     iconvp        indicator
  *                               - 1 advection
  *                               - 0 otherwise
  * \param[in]     idiffp        indicator
  *                               - 1 diffusion
  *                               - 0 otherwise
- * \param[in]     ndircp        indicator
- *                               - 0 if the diagonal stepped aside
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
  * \param[in]     coefbu        boundary condition array for the variable
- *                               (Implicit part - 3x3 tensor array)
+ *                               (implicit part - 3x3 tensor array)
  * \param[in]     cofbfu        boundary condition array for the variable flux
- *                               (Implicit part - 3x3 tensor array)
+ *                               (implicit part - 3x3 tensor array)
  * \param[in]     fimp
  * \param[in]     i_massflux    mass flux at interior faces
  * \param[in]     b_massflux    mass flux at border faces
@@ -440,20 +575,74 @@ cs_matrix_time_step(const cs_mesh_t          *m,
 /*----------------------------------------------------------------------------*/
 
 void
-cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
-                                int                       iconvp,
-                                int                       idiffp,
-                                int                       ndircp,
-                                double                    thetap,
-                                const cs_real_33_t        coefbu[],
-                                const cs_real_33_t        cofbfu[],
-                                const cs_real_33_t        fimp[],
-                                const cs_real_t           i_massflux[],
-                                const cs_real_t           b_massflux[],
-                                const cs_real_33_t        i_visc[],
-                                const cs_real_t           b_visc[],
-                                cs_real_33_t    *restrict da,
-                                cs_real_332_t   *restrict xa);
+cs_matrix_anisotropic_diffusion(const cs_mesh_t            *m,
+                                const cs_mesh_quantities_t *mq,
+                                int                         iconvp,
+                                int                         idiffp,
+                                double                      thetap,
+                                const cs_real_33_t          coefbu[],
+                                const cs_real_33_t          cofbfu[],
+                                const cs_real_33_t          fimp[],
+                                const cs_real_t             i_massflux[],
+                                const cs_real_t             b_massflux[],
+                                const cs_real_33_t          i_visc[],
+                                const cs_real_t             b_visc[],
+                                cs_real_33_t      *restrict da,
+                                cs_real_332_t     *restrict xa);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Build the advection/diffusion matrix for a tensor field with a
+ * tensorial diffusivity.
+ *
+ * The advection is upwind, the diffusion is not reconstructed.
+ * The matrix is split into a diagonal block (3x3 times number of cells)
+ * and an extra diagonal part (of dimension 2 times 3x3 the number of internal
+ * faces).
+ *
+ * \param[in]     m             pointer to mesh structure
+ * \param[in]     iconvp        indicator
+ *                               - 1 advection
+ *                               - 0 otherwise
+ * \param[in]     idiffp        indicator
+ *                               - 1 diffusion
+ *                               - 0 otherwise
+ * \param[in]     thetap        weighting coefficient for the theta-scheme,
+ *                               - thetap = 0: explicit scheme
+ *                               - thetap = 0.5: time-centered
+ *                               scheme (mix between Crank-Nicolson and
+ *                               Adams-Bashforth)
+ *                               - thetap = 1: implicit scheme
+ * \param[in]     coefbu        boundary condition array for the variable
+ *                               (implicit part - 3x3 tensor array)
+ * \param[in]     cofbfu        boundary condition array for the variable flux
+ *                               (implicit part - 3x3 tensor array)
+ * \param[in]     fimp          part of the diagonal
+ * \param[in]     i_massflux    mass flux at interior faces
+ * \param[in]     b_massflux    mass flux at border faces
+ * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
+ *                               at interior faces for the matrix
+ * \param[in]     b_visc        \f$ \mu_\fib \dfrac{S_\fib}{\ipf \centf} \f$
+ *                               at border faces for the matrix
+ * \param[out]    da            diagonal part of the matrix
+ * \param[out]    xa            extra interleaved diagonal part of the matrix
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
+                                       int                       iconvp,
+                                       int                       idiffp,
+                                       double                    thetap,
+                                       const cs_real_66_t        coefbu[],
+                                       const cs_real_66_t        cofbfu[],
+                                       const cs_real_66_t        fimp[],
+                                       const cs_real_t           i_massflux[],
+                                       const cs_real_t           b_massflux[],
+                                       const cs_real_66_t        i_visc[],
+                                       const cs_real_t           b_visc[],
+                                       cs_real_66_t    *restrict da,
+                                       cs_real_662_t   *restrict xa);
 
 /*----------------------------------------------------------------------------*/
 /*!
@@ -468,16 +657,14 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
  * \param[in]     idiffp        indicator
  *                               - 1 diffusion
  *                               - 0 otherwise
- * \param[in]     ndircp        indicator
- *                               - 0 if the diagonal stepped aside
  * \param[in]     thetap        weighting coefficient for the theta-scheme,
  *                               - thetap = 0: explicit scheme
- *                               - thetap = 0.5: time-centred
+ *                               - thetap = 0.5: time-centered
  *                               scheme (mix between Crank-Nicolson and
  *                               Adams-Bashforth)
  *                               - thetap = 1: implicit scheme
  * \param[in]     cofbfu        boundary condition array for the variable flux
- *                               (Implicit part - 3x3 tensor array)
+ *                               (implicit part - 3x3 tensor array)
  * \param[in]     fimp
  * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
  *                               at interior faces for the matrix
@@ -491,7 +678,6 @@ cs_matrix_anisotropic_diffusion(const cs_mesh_t          *m,
 void
 cs_sym_matrix_anisotropic_diffusion(const cs_mesh_t           *m,
                                     int                       idiffp,
-                                    int                       ndircp,
                                     double                    thetap,
                                     const cs_real_33_t        cofbfu[],
                                     const cs_real_33_t        fimp[],
@@ -500,6 +686,49 @@ cs_sym_matrix_anisotropic_diffusion(const cs_mesh_t           *m,
                                     cs_real_33_t    *restrict da,
                                     cs_real_33_t    *restrict xa);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Build the diffusion matrix for a vector field with a
+ * tensorial diffusivity (symmetric matrix).
+ *
+ * The diffusion is not reconstructed.
+ * The matrix is split into a diagonal block (3x3 times number of cells)
+ * and an extra diagonal part (of dimension 3x3 the number of internal
+ * faces).
+ *
+ * \param[in]     m             pointer to mesh structure
+ * \param[in]     idiffp        indicator
+ *                               - 1 diffusion
+ *                               - 0 otherwise
+ * \param[in]     thetap        weighting coefficient for the theta-scheme,
+ *                               - thetap = 0: explicit scheme
+ *                               - thetap = 0.5: time-centered
+ *                               scheme (mix between Crank-Nicolson and
+ *                               Adams-Bashforth)
+ *                               - thetap = 1: implicit scheme
+ * \param[in]     cofbfu        boundary condition array for the variable flux
+ *                               (implicit part - 3x3 tensor array)
+ * \param[in]     fimp          \f$ \tens{f_s}^{imp} \f$
+ * \param[in]     i_visc        \f$ \mu_\fij \dfrac{S_\fij}{\ipf \jpf} \f$
+ *                               at interior faces for the matrix
+ * \param[in]     b_visc        \f$ \mu_\fib \dfrac{S_\fib}{\ipf \centf} \f$
+ *                               at border faces for the matrix
+ * \param[out]    da            diagonal part of the matrix
+ * \param[out]    xa            extra interleaved diagonal part of the matrix
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_sym_matrix_anisotropic_diffusion_tensor(const cs_mesh_t          *m,
+                                           int                       idiffp,
+                                           double                    thetap,
+                                           const cs_real_66_t        cofbfu[],
+                                           const cs_real_66_t        fimp[],
+                                           const cs_real_66_t        i_visc[],
+                                           const cs_real_t           b_visc[],
+                                           cs_real_66_t    *restrict da,
+                                           cs_real_66_t    *restrict xa);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
