@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -99,20 +99,12 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
 
-/* Variant default for Intel compiler and on Itanium (optimized by BULL)
-   (Use compile flag -DNO_BULL_OPTIM to switch default to general code) */
-
-#if (defined(__INTEL_COMPILER) && defined(__ia64__) && !defined(NO_BULL_OPTIM))
-#define IA64_OPTIM
-#endif
-
-/* Minimum size for OpenMP loops (needs benchmarking to adjust) */
-#define THR_MIN 128
-
 /*=============================================================================
  * Local Type Definitions
  *============================================================================*/
@@ -129,21 +121,24 @@ BEGIN_C_DECLS
 static const char *_matrix_fill_name[CS_MATRIX_N_FILL_TYPES]
   = {N_("scalar"),
      N_("scalar symmetric"),
-     N_("block diagonal"),
-     N_("block diagonal symmetric"),
-     N_("block")};
+     N_("block 3 diagonal"),
+     N_("block 6 diagonal"),
+     N_("block 3 diagonal symmetric"),
+     N_("block 3")};
 
 static const char *_matrix_operation_name[CS_MATRIX_N_FILL_TYPES][2]
   = {{N_("y <- A.x"),
       N_("y <- (A-D).x")},
      {N_("Symmetric y <- A.x"),
       N_("Symmetric y <- (A-D).x")},
-     {N_("Block diagonal y <- A.x"),
-      N_("Block diagonal y <- (A-D).x")},
-     {N_("Block diagonal symmetric y <- A.x"),
-      N_("Block diagonal symmetric y <- (A-D).x")},
-     {N_("Block y <- A.x"),
-      N_("Block y <- (A-D).x")}};
+     {N_("Block 3 diagonal y <- A.x"),
+      N_("Block 3 diagonal y <- (A-D).x")},
+     {N_("Block 6 diagonal y <- A.x"),
+      N_("Block 6 diagonal y <- (A-D).x")},
+     {N_("Block 3 diagonal symmetric y <- A.x"),
+      N_("Block 3 diagonal symmetric y <- (A-D).x")},
+     {N_("Block 3 y <- A.x"),
+      N_("Block 3 y <- (A-D).x")}};
 
 /*============================================================================
  * Private function definitions
@@ -159,7 +154,7 @@ static const char *_matrix_operation_name[CS_MATRIX_N_FILL_TYPES][2]
  *   n_cells_ext <-- number of cells including ghost cells (array size)
  *   n_faces     <-- local number of internal faces
  *   cell_num    <-- Optional global cell numbers (1 to n), or NULL
- *   face_cell   <-- face -> cells connectivity (1 to n)
+ *   face_cell   <-- face -> cells connectivity
  *   halo        <-- cell halo structure
  *   numbering   <-- vectorization or thread-related numbering info, or NULL
  *   m_variant   <-> array of matrix variants
@@ -171,8 +166,7 @@ _matrix_tune_test(double                 t_measure,
                   cs_lnum_t              n_cells,
                   cs_lnum_t              n_cells_ext,
                   cs_lnum_t              n_faces,
-                  const cs_gnum_t       *cell_num,
-                  const cs_lnum_t       *face_cell,
+                  const cs_lnum_2_t     *face_cell,
                   const cs_halo_t       *halo,
                   const cs_numbering_t  *numbering,
                   cs_matrix_variant_t   *m_variant)
@@ -180,6 +174,7 @@ _matrix_tune_test(double                 t_measure,
   cs_lnum_t  ii;
   int  n_runs, run_id, v_id, f_id, ed_flag;
   double  wt0, wt1, wtu;
+  double wti, wtf;
   cs_matrix_type_t  type, type_prev;
 
   double test_sum = 0.0;
@@ -247,7 +242,6 @@ _matrix_tune_test(double                 t_measure,
                                           n_cells,
                                           n_cells_ext,
                                           n_faces,
-                                          cell_num,
                                           face_cell,
                                           halo,
                                           numbering);
@@ -266,12 +260,12 @@ _matrix_tune_test(double                 t_measure,
     for (f_id = 0; f_id < CS_MATRIX_N_FILL_TYPES; f_id++) {
 
       const int *_d_block_size
-        = (f_id >= CS_MATRIX_33_BLOCK_D) ? d_block_size : NULL;
+        = (f_id >= CS_MATRIX_BLOCK_D) ? d_block_size : NULL;
       const int *_ed_block_size
-        = (f_id >= CS_MATRIX_33_BLOCK) ? ed_block_size : NULL;
+        = (f_id >= CS_MATRIX_BLOCK) ? ed_block_size : NULL;
       const bool sym_coeffs
         = (   f_id == CS_MATRIX_SCALAR_SYM
-           || f_id == CS_MATRIX_33_BLOCK_D_SYM) ? true : false;
+           || f_id == CS_MATRIX_BLOCK_D_SYM) ? true : false;
 
       /* Loop on diagonal exclusion flags */
 
@@ -281,9 +275,6 @@ _matrix_tune_test(double                 t_measure,
           && v->vector_multiply[f_id][1] == NULL)
         continue;
 
-      m->loop_length[f_id][0] = v->loop_length[f_id][0];
-      m->loop_length[f_id][1] = v->loop_length[f_id][1];
-
       /* Measure overhead of setting coefficients if not already done */
 
       if (test_assign) {
@@ -301,6 +292,8 @@ _matrix_tune_test(double                 t_measure,
                                      sym_coeffs,
                                      _d_block_size,
                                      _ed_block_size,
+                                     n_faces,
+                                     (const cs_lnum_2_t *)face_cell,
                                      da,
                                      xa);
           run_id++;
@@ -325,22 +318,33 @@ _matrix_tune_test(double                 t_measure,
         if (vector_multiply != NULL) {
           wt0 = cs_timer_wtime(), wt1 = wt0;
           run_id = 0, n_runs = 8;
+
+          double mean = 0.0;
+          double m2 = 0.0;
+          double delta = 0.0;
+
           while (run_id < n_runs) {
             while (run_id < n_runs) {
               if (run_id % 8)
                 test_sum = 0;
+              wti = cs_timer_wtime(), wtf = wti;
               vector_multiply(ed_flag, m, x, y);
+              wtf = cs_timer_wtime();
               test_sum += y[n_cells-1];
               run_id++;
+              delta = (wtf - wti) - mean;
+              mean = mean + delta/run_id;
+              m2 = m2 + delta * (wtf - wti - mean);
             }
             wt1 = cs_timer_wtime();
             if (wt1 - wt0 < t_measure)
               n_runs *= 2;
           }
           wtu = (wt1 - wt0) / n_runs;
-          v->matrix_vector_cost[f_id][ed_flag] = wtu;
+          m2 = sqrt(m2 / (n_runs - 1));
+          v->matrix_vector_cost[f_id][ed_flag][0] = wtu;
+          v->matrix_vector_cost[f_id][ed_flag][1] = m2;
         }
-
       } /* end of loop on ed_flag */
 
       cs_matrix_release_coefficients(m);
@@ -413,14 +417,13 @@ _matrix_tune_create_assign_title(int                    struct_flag,
 
     cs_log_strpadl(tmp_s[0], _("time (s)"), 16, 24);
     cs_log_strpadl(tmp_s[1], _(" mean"), 12, 24);
-    cs_log_strpadl(tmp_s[2], _("min"), 12, 24);
-    cs_log_strpadl(tmp_s[3], _("max"), 12, 24);
+    cs_log_strpadl(tmp_s[2], _("max"), 12, 24);
 
     cs_log_printf(CS_LOG_PERFORMANCE,
                   "  %24s %21s %s\n"
-                  "  %24s %s %s %s\n",
+                  "  %24s %s %s\n",
                   " ", " ", tmp_s[0],
-                  " ", tmp_s[1], tmp_s[2], tmp_s[3]);
+                  " ", tmp_s[1], tmp_s[2]);
   }
 
 #endif
@@ -473,13 +476,12 @@ _matrix_tune_create_assign_stats(const cs_matrix_variant_t  *m_variant,
 #if defined(HAVE_MPI)
 
   if (cs_glob_n_ranks > 1) {
-    double t_max, t_min, t_sum = -1;
+    double t_max, t_sum = -1;
     MPI_Allreduce(&t_loc, &t_sum, 1, MPI_DOUBLE, MPI_SUM, cs_glob_mpi_comm);
-    MPI_Allreduce(&t_loc, &t_min, 1, MPI_DOUBLE, MPI_MIN, cs_glob_mpi_comm);
     MPI_Allreduce(&t_loc, &t_max, 1, MPI_DOUBLE, MPI_MAX, cs_glob_mpi_comm);
     cs_log_printf(CS_LOG_PERFORMANCE,
-                  "  %s %12.5e %12.5e %12.5e\n",
-                  title, t_sum/cs_glob_n_ranks, t_min, t_max);
+                  "  %s %12.5e %12.5e\n",
+                  title, t_sum/cs_glob_n_ranks, t_max);
   }
 
 #endif
@@ -526,37 +528,40 @@ _matrix_tune_spmv_title(cs_matrix_fill_type_t  fill_type,
 
   if (cs_glob_n_ranks > 1) {
 
-    char tmp_s[8][24] =  {"", "", "", "", "", "", "", ""};
+    char tmp_s[12][24] =  {"", "", "", "", "", "", "", "", "", "", "", ""};
 
     cs_log_strpadl(tmp_s[0], _("time (s)"), 16, 24);
     cs_log_strpadl(tmp_s[1], _("speedup"), 16, 24);
-    cs_log_strpadl(tmp_s[2], _(" mean"), 12, 24);
-    cs_log_strpadl(tmp_s[3], _("min"), 12, 24);
+    cs_log_strpadl(tmp_s[2], _("std. dev."), 16, 24);
+    cs_log_strpadl(tmp_s[3], _("mean"), 12, 24);
     cs_log_strpadl(tmp_s[4], _("max"), 12, 24);
-    cs_log_strpadl(tmp_s[5], _(" mean"), 8, 24);
+    cs_log_strpadl(tmp_s[5], _("mean"), 8, 24);
     cs_log_strpadl(tmp_s[6], _("min"), 8, 24);
     cs_log_strpadl(tmp_s[7], _("max"), 8, 24);
+    cs_log_strpadl(tmp_s[8], _("mean"), 8, 24);
+    cs_log_strpadl(tmp_s[9], _("max"), 8, 24);
 
     cs_log_printf(CS_LOG_PERFORMANCE,
-                  "  %24s %21s %s %9s %s\n"
-                  "  %24s %s %s %s %s %s %s\n",
-                  " ", " ", tmp_s[0], " ", tmp_s[1],
-                  " ", tmp_s[2], tmp_s[3], tmp_s[4],
-                  tmp_s[5], tmp_s[6], tmp_s[7]);
+                  "  %24s %8s %s %9s %s  %s\n"
+                  "  %24s %s %s %s %s %s %s %s\n",
+                  " ", " ", tmp_s[0], " ", tmp_s[1], tmp_s[2],
+                  " ", tmp_s[3], tmp_s[4], tmp_s[5],
+                  tmp_s[6], tmp_s[7], tmp_s[8], tmp_s[9]);
   }
 
 #endif
 
   if (cs_glob_n_ranks == 1) {
 
-    char tmp_s[2][24] =  {"", ""};
+    char tmp_s[3][24] =  {"", "", ""};
 
     cs_log_strpadl(tmp_s[0], _("time (s)"), 12, 24);
     cs_log_strpadl(tmp_s[1], _("speedup"), 8, 24);
+    cs_log_strpadl(tmp_s[2], _("std. dev."), 8, 24);
 
     cs_log_printf(CS_LOG_PERFORMANCE,
-                  "  %24s %s %s\n",
-                  " ", tmp_s[0], tmp_s[1]);
+                  "  %24s %s %s %s\n",
+                  " ", tmp_s[0], tmp_s[1], tmp_s[2]);
 
   }
 }
@@ -579,7 +584,7 @@ _matrix_tune_spmv_stats(const cs_matrix_variant_t  *m_variant,
 {
   char title[32];
 
-  double v_loc[2] = {-1, -1};
+  double v_loc[3] = {-1, -1, 0};
 
   const cs_matrix_variant_t  *r = m_variant;
   const cs_matrix_variant_t  *v = m_variant + variant_id;
@@ -588,8 +593,9 @@ _matrix_tune_spmv_stats(const cs_matrix_variant_t  *m_variant,
 
   /* Get timing info */
 
-  v_loc[0] = v->matrix_vector_cost[fill_type][ed_flag];
-  v_loc[1] = r->matrix_vector_cost[fill_type][ed_flag] / v_loc[0];
+  v_loc[0] = v->matrix_vector_cost[fill_type][ed_flag][0];
+  v_loc[1] = r->matrix_vector_cost[fill_type][ed_flag][0];
+  v_loc[2] = v->matrix_vector_cost[fill_type][ed_flag][1];
 
   if (v_loc[0] < 0)
     return;
@@ -599,24 +605,46 @@ _matrix_tune_spmv_stats(const cs_matrix_variant_t  *m_variant,
 #if defined(HAVE_MPI)
 
   if (cs_glob_n_ranks > 1) {
-    double v_max[2], v_min[2], v_sum[2];
-    MPI_Allreduce(v_loc, v_sum, 2, MPI_DOUBLE, MPI_SUM, cs_glob_mpi_comm);
-    MPI_Allreduce(v_loc, v_min, 2, MPI_DOUBLE, MPI_MIN, cs_glob_mpi_comm);
-    MPI_Allreduce(v_loc, v_max, 2, MPI_DOUBLE, MPI_MAX, cs_glob_mpi_comm);
+
+    double v_max[3], speedup_min, v_sum[3];
+
+    MPI_Allreduce(v_loc, v_sum, 3, MPI_DOUBLE, MPI_SUM, cs_glob_mpi_comm);
+
+    if (v_loc[0] > 1e-9)
+      v_loc[1] = r->matrix_vector_cost[fill_type][ed_flag][0] / v_loc[0];
+    else
+      v_loc[1] = 1;
+    MPI_Allreduce(v_loc + 1, &speedup_min, 1, MPI_DOUBLE, MPI_MIN,
+                  cs_glob_mpi_comm);
+
+    if (v_loc[0] > 1e-9)
+      v_loc[2] = v->matrix_vector_cost[fill_type][ed_flag][1] / v_loc[0];
+    else {
+      v_loc[1] = 0;
+      v_loc[2] = 0;
+    }
+    MPI_Allreduce(v_loc, v_max, 3, MPI_DOUBLE, MPI_MAX, cs_glob_mpi_comm);
+
+    cs_real_t t_mean = v_sum[0]/cs_glob_n_ranks;
+    cs_real_t speedup_mean = v_sum[1] / v_sum[0]; /* weighted by v_loc[0] */
+    cs_real_t stddev_mean = v_sum[2]/v_sum[0];
     cs_log_printf(CS_LOG_PERFORMANCE,
-                  "  %s %12.5e %12.5e %12.5e %8.4f %8.4f %8.4f\n",
+                  "  %s %12.5e %12.5e %8.4f %8.4f %8.4f %8.4f %8.4f\n",
                   title,
-                  v_sum[0]/cs_glob_n_ranks, v_min[0], v_max[0],
-                  (v_sum[1]/cs_glob_n_ranks), v_min[1], v_max[1]);
+                  t_mean, v_max[0], speedup_mean, speedup_min, v_max[1],
+                  stddev_mean, v_max[2]);
   }
 
 #endif
 
-  if (cs_glob_n_ranks == 1)
+  if (cs_glob_n_ranks == 1) {
+    cs_real_t speedup = v_loc[1] / v_loc[0];
+    cs_real_t stddev = v_loc[2] / v_loc[0];
     cs_log_printf(CS_LOG_PERFORMANCE,
-                  "  %s %12.5e %8.4f\n",
+                  "  %s %12.5e %8.4f %8.4f\n",
                   title,
-                  v_loc[0], v_loc[1]);
+                  v_loc[0], speedup, stddev);
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -633,13 +661,14 @@ _variant_init(cs_matrix_variant_t  *v)
   for (int i = 0; i < CS_MATRIX_N_FILL_TYPES; i++) {
     for (int j = 0; j < 2; j++) {
       v->vector_multiply[i][j] = NULL;
-      v->loop_length[i][j] = 0;
-      v->matrix_vector_cost[i][j] = -1.;
+      v->matrix_vector_cost[i][j][0] = -1.;
     }
     v->matrix_assign_cost[i] = -1.;
   }
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -670,7 +699,7 @@ _variant_init(cs_matrix_variant_t  *v)
  *   n_cells_ext    <-- number of cells including ghost cells (array size)
  *   n_faces        <-- local number of internal faces
  *   cell_num       <-- Optional global cell numbers (1 to n), or NULL
- *   face_cell      <-- face -> cells connectivity (1 to n)
+ *   face_cell      <-- face -> cells connectivity
  *   halo           <-- cell halo structure
  *   numbering      <-- vectorization or thread-related numbering info, or NULL
  *
@@ -689,8 +718,7 @@ cs_matrix_variant_tuned(double                 t_measure,
                         cs_lnum_t              n_cells,
                         cs_lnum_t              n_cells_ext,
                         cs_lnum_t              n_faces,
-                        const cs_gnum_t       *cell_num,
-                        const cs_lnum_t       *face_cell,
+                        const cs_lnum_2_t     *face_cell,
                         const cs_halo_t       *halo,
                         const cs_numbering_t  *numbering)
 {
@@ -709,12 +737,11 @@ cs_matrix_variant_tuned(double                 t_measure,
   int                    _n_fill_types_default = 3;
   cs_matrix_fill_type_t  _fill_types_default[] = {CS_MATRIX_SCALAR,
                                                   CS_MATRIX_SCALAR_SYM,
-                                                  CS_MATRIX_33_BLOCK_D};
+                                                  CS_MATRIX_BLOCK_D};
   double                 _fill_weights_default[] = {0.5, 0.25, 0.25};
 
   int                    _n_types = n_types;
   int                    _n_fill_types = n_fill_types;
-  cs_matrix_type_t       *_types = _types;
   cs_matrix_fill_type_t  *_fill_types = fill_types;
   double                 *_fill_weights = fill_weights;
   double                  tot_weight = 0.;
@@ -771,7 +798,6 @@ cs_matrix_variant_tuned(double                 t_measure,
                     n_cells,
                     n_cells_ext,
                     n_faces,
-                    cell_num,
                     face_cell,
                     halo,
                     numbering,
@@ -816,9 +842,9 @@ cs_matrix_variant_tuned(double                 t_measure,
           && (n_min_products > 0 && n_min_products < 10000))
         t_overhead[v->type][fill_type]
           = v->matrix_assign_cost[fill_type] / n_min_products;
-      speedup = (  (  m_variant->matrix_vector_cost[fill_type][0]
+      speedup = (  (  m_variant->matrix_vector_cost[fill_type][0][0]
                     + t_overhead[m_variant->type][fill_type])
-                 / (  v->matrix_vector_cost[fill_type][0]
+                 / (  v->matrix_vector_cost[fill_type][0][0]
                     + t_overhead[v->type][fill_type]));
       if (t_speedup[v->type][fill_type] < speedup)
         t_speedup[v->type][fill_type] = speedup;
@@ -880,16 +906,15 @@ cs_matrix_variant_tuned(double                 t_measure,
 
         cs_matrix_fill_type_t  fill_type = _fill_types[f_id];
 
-        if (v->matrix_vector_cost[fill_type][ed_flag] > 0) {
-          if (   v->matrix_vector_cost[fill_type][ed_flag]
-               < r->matrix_vector_cost[fill_type][ed_flag]
-              || r->matrix_vector_cost[fill_type][ed_flag] < 0) {
+        if (v->matrix_vector_cost[fill_type][ed_flag][0] > 0) {
+          if (   v->matrix_vector_cost[fill_type][ed_flag][0]
+               < r->matrix_vector_cost[fill_type][ed_flag][0]
+              || r->matrix_vector_cost[fill_type][ed_flag][0] < 0) {
             r->vector_multiply[fill_type][ed_flag]
               = v->vector_multiply[fill_type][ed_flag];
-            r->matrix_vector_cost[fill_type][ed_flag]
-              = v->matrix_vector_cost[fill_type][ed_flag];
-            r->loop_length[fill_type][ed_flag]
-              = v->loop_length[fill_type][ed_flag];
+            for (int o_id = 0; o_id < 1; o_id++)
+              r->matrix_vector_cost[fill_type][ed_flag][o_id]
+                = v->matrix_vector_cost[fill_type][ed_flag][o_id];
             cur_select[f_id][ed_flag] = v_id;
           }
         }
