@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -45,7 +45,7 @@ parameter (Mair = 28.9d-3)           ! Kg/mol
 
 !> Choice of chemistry resolution scheme
 !> - 0 --> no atmospheric chemistry
-!> - 1 --> quasi stationary equilibrium NOx scheme with 4 species and 5 reactions
+!> - 1 --> quasi steady equilibrium NOx scheme with 4 species and 5 reactions
 !> - 2 --> scheme with 20 species and 34 reactions
 !> - 3 --> scheme CB05 with 52 species and 155 reactions
 !> - 4 --> user defined schema
@@ -85,7 +85,7 @@ double precision, save ::  lat, lon
 !> logical unit of the concentration profiles file
 integer, save         ::  impmec
 !> name of the concentration profiles file
-character*10, save    ::  ficmec
+character(len=10), save    ::  ficmec
 !> number of time steps for the concentration profiles file
 integer, save         ::  nbchim
 !> number of altitudes for the concentration profiles file
@@ -101,8 +101,10 @@ double precision, allocatable, dimension(:) :: espnum
 double precision, allocatable, dimension(:) :: zproc
 !> time steps of the concentration profiles
 double precision, allocatable, dimension(:) :: tchem
-!> coordinates of concentration profiles
-double precision, allocatable, dimension(:) :: xchem,ychem
+!> X coordinates of concentration profiles
+double precision, allocatable, dimension(:) :: xchem
+!> Y coordinates of concentration profiles
+double precision, allocatable, dimension(:) :: ychem
 !> read zone boundary conditions from profile
 integer, save :: iprofc(nozppm)
 
@@ -111,7 +113,7 @@ integer, save :: iprofc(nozppm)
 contains
 
 !=============================================================================
-
+!> \brief Allocate space
 subroutine init_chemistry
 
 use mesh, only: ncel
@@ -152,7 +154,7 @@ allocate(ychem(nbchim))
 end subroutine init_chemistry
 
 !=============================================================================
-
+!> \brief deallocate the space
 subroutine finalize_chemistry
 
 implicit none
