@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,56 +19,23 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-
-subroutine atiniv &
-!================
-
- ( nvar   , nscal  ,                                              &
-   dt     , rtp    , propce )
-
-!===============================================================================
-! FONCTION :
-! --------
-
-! INITIALISATION DES VARIABLES DE CALCUL
-!    POUR LA PHYSIQUE PARTICULIERE : ECOULEMENTS ATMOSPHERIQUES
-!    PENDANT DE USINIV.F
-
-! Cette routine est appelee en debut de calcul (suite ou non)
-!     avant le debut de la boucle en temps
-
-! Elle permet d'INITIALISER ou de MODIFIER (pour les calculs suite)
-!     les variables de calcul,
-!     les valeurs du pas de temps
-
-
-! On dispose ici de ROM et VISCL initialises par RO0 et VISCL0
-!     ou relues d'un fichier suite
-! On ne dispose des variables VISCLS, CP (quand elles sont
-!     definies) que si elles ont pu etre relues dans un fichier
-!     suite de calcul
-
-! LA MODIFICATION DES PROPRIETES PHYSIQUES (ROM, VISCL, VISCLS, CP)
-!     SE FERA EN STANDARD DANS LE SOUS PROGRAMME PPPHYV
-!     ET PAS ICI
-
+!> \file atiniv.f90
+!> \brief Initialisation of calculation variables for the atmospheric module,
+!> it is the counterpart of usiniv.f90.
+!>
+!> Initialise for example the meteorological field for each cell of
+!> the domain by interpolation of the data from the meteo file
+!-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! nvar             ! i  ! <-- ! total number of variables                      !
-! nscal            ! i  ! <-- ! total number of scalars                        !
-! dt(ncelet)       ! tr ! <-- ! valeur du pas de temps                         !
-! rtp              ! tr ! <-- ! variables de calcul au centre des              !
-! (ncelet,*)       !    !     !    cellules                                    !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-!__________________!____!_____!________________________________________________!
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   nvar        total number of variables
+!> \param[in]   nscal       total number of scalars
+!> \param[in]   dt          time step value
+!-------------------------------------------------------------------------------
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
+subroutine atiniv  ( nvar, nscal, dt )
 
 !===============================================================================
 ! Module files
@@ -95,7 +62,7 @@ implicit none
 
 integer          nvar   , nscal
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), propce(ncelet,*)
+double precision dt(ncelet)
 
 ! Local variables
 
@@ -107,6 +74,13 @@ integer k,ii, isc
 double precision xcent
 
 double precision, dimension(:,:), pointer :: vel
+double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_phi
+double precision, dimension(:), pointer :: cvar_fb, cvar_omg, cvar_nusa
+double precision, dimension(:,:), pointer :: cvar_rij
+double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
+double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
+double precision, dimension(:), pointer :: cvar_despgi, cvar_sc
+double precision, dimension(:), pointer :: cvar_scalt, cvar_totwt, cvar_ntdrp
 
 !===============================================================================
 
@@ -119,6 +93,33 @@ call field_get_val_v(ivarfl(iu), vel)
 
 d2s3 = 2.d0/3.d0
 
+if (itytur.eq.2) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (itytur.eq.3) then
+  if (irijco.eq.1) then
+    call field_get_val_v(ivarfl(irij), cvar_rij)
+  else
+    call field_get_val_s(ivarfl(ir11), cvar_r11)
+    call field_get_val_s(ivarfl(ir22), cvar_r22)
+    call field_get_val_s(ivarfl(ir33), cvar_r33)
+    call field_get_val_s(ivarfl(ir12), cvar_r12)
+    call field_get_val_s(ivarfl(ir23), cvar_r23)
+    call field_get_val_s(ivarfl(ir13), cvar_r13)
+  endif
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (iturb.eq.50) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+  call field_get_val_s(ivarfl(iphi), cvar_phi)
+  call field_get_val_s(ivarfl(ifb), cvar_fb)
+elseif (iturb.eq.60) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iomg), cvar_omg)
+elseif (iturb.eq.70) then
+  call field_get_val_s(ivarfl(inusa), cvar_nusa)
+endif
+
 !===============================================================================
 ! 2. READING THE METEO PROFILE FILE (IF IMETEO = 1 DEFAULT OPTION):
 !===============================================================================
@@ -160,18 +161,18 @@ if (ifilechemistry.ge.1) then
 
   ! Volume initilization with profiles for species present
   ! in the chemical profiles file
-  if (init_at_chem.eq.1) then
-    do iel = 1, ncel
+  if (isuite.ne.0.and.init_at_chem.eq.1) then
+    do k = 1, nespgi
+      call field_get_val_s(ivarfl(isca(isca_chem(idespgi(k)))), cvar_despgi)
 
-      zent = xyzcen(3,iel)
-
-      do k = 1, nespgi
+      do iel = 1, ncel
+        zent = xyzcen(3,iel)
         call intprf                                                         &
         !==========
         (nbchmz, nbchim,                                                    &
          zproc, tchem, espnum(1+(k-1)*nbchim*nbchmz), zent  , ttcabs, xcent )
         ! The first nespg user scalars are supposed to be chemical species
-        rtp(iel,isca(idespgi(k))) = xcent
+        cvar_despgi(iel) = xcent
       enddo
 
     enddo
@@ -186,10 +187,11 @@ if (iaerosol.eq.1) then
 
   ! Initialization
   if (init_at_chem.eq.1) then
-    do iel = 1, ncel
-      do ii = 1, nesp_aer*nbin_aer + nbin_aer
-        isc = (isca_chem(1) - 1) + nespg_siream + ii
-        rtp(iel,isca(isc)) = dlconc0(ii)
+    do ii = 1, nesp_aer*nbin_aer + nbin_aer
+      isc = (isca_chem(1) - 1) + nespg_siream + ii
+      call field_get_val_s(ivarfl(isca(isc)), cvar_sc)
+      do iel = 1, ncel
+        cvar_sc(iel) = dlconc0(ii)
       enddo
     enddo
   endif
@@ -218,21 +220,30 @@ endif
 if (isuite.eq.0) then
 
   if (initmeteo.eq.1) then
+
+    if (ippmod(iatmos).eq.1) then
+      call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
+    else if (ippmod(iatmos).eq.2) then
+      call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
+      call field_get_val_s(ivarfl(isca(itotwt)), cvar_totwt)
+      call field_get_val_s(ivarfl(isca(intdrp)), cvar_ntdrp)
+    endif
+
     if (imeteo.eq.0) then
 
       if (ippmod(iatmos).eq.1) then
         ! The thermal scalar is potential temperature
         do iel = 1, ncel
-          rtp(iel,isca(iscalt)) = t0
+          cvar_scalt(iel) = t0
         enddo
       endif
 
       if (ippmod(iatmos).eq.2) then
         ! The thermal scalar is liquid potential temperature
         do iel = 1, ncel
-          rtp(iel,isca(iscalt)) = t0
-          rtp(iel,isca(itotwt)) = 0.d0
-          rtp(iel,isca(intdrp)) = 0.d0
+          cvar_scalt(iel) = t0
+          cvar_totwt(iel) = 0.d0
+          cvar_ntdrp(iel) = 0.d0
         enddo
       endif
 
@@ -270,34 +281,43 @@ if (isuite.eq.0) then
     !     ITYTUR est un indicateur qui vaut ITURB/10
         if    (itytur.eq.2) then
 
-          rtp(iel,ik)  = xkent
-          rtp(iel,iep) = xeent
+          cvar_k(iel)  = xkent
+          cvar_ep(iel) = xeent
 
         elseif (itytur.eq.3) then
 
-          rtp(iel,ir11) = d2s3*xkent
-          rtp(iel,ir22) = d2s3*xkent
-          rtp(iel,ir33) = d2s3*xkent
-          rtp(iel,ir12) = 0.d0
-          rtp(iel,ir13) = 0.d0
-          rtp(iel,ir23) = 0.d0
-          rtp(iel,iep)  = xeent
+          if (irijco.eq.1) then
+            cvar_rij(1,iel) = d2s3*xkent
+            cvar_rij(2,iel) = d2s3*xkent
+            cvar_rij(3,iel) = d2s3*xkent
+            cvar_rij(4,iel) = 0.d0
+            cvar_rij(5,iel) = 0.d0
+            cvar_rij(6,iel) = 0.d0
+          else
+            cvar_r11(iel) = d2s3*xkent
+            cvar_r22(iel) = d2s3*xkent
+            cvar_r33(iel) = d2s3*xkent
+            cvar_r12(iel) = 0.d0
+            cvar_r13(iel) = 0.d0
+            cvar_r23(iel) = 0.d0
+          endif
+          cvar_ep(iel)  = xeent
 
         elseif (iturb.eq.50) then
 
-          rtp(iel,ik)   = xkent
-          rtp(iel,iep)  = xeent
-          rtp(iel,iphi) = d2s3
-          rtp(iel,ifb)  = 0.d0
+          cvar_k(iel)   = xkent
+          cvar_ep(iel)  = xeent
+          cvar_phi(iel) = d2s3
+          cvar_fb(iel)  = 0.d0
 
         elseif (iturb.eq.60) then
 
-          rtp(iel,ik)   = xkent
-          rtp(iel,iomg) = xeent/cmu/xkent
+          cvar_k(iel)   = xkent
+          cvar_omg(iel) = xeent/cmu/xkent
 
         elseif (iturb.eq.70) then
 
-          rtp(iel,inusa) = cmu*xkent**2/xeent
+          cvar_nusa(iel) = cmu*xkent**2/xeent
 
         endif
 
@@ -309,7 +329,7 @@ if (isuite.eq.0) then
          (nbmett, nbmetm,                                               &
           ztmet, tmmet, tpmet, zent  , ttcabs, tpent )
 
-            rtp(iel,isca(iscalt)) = tpent
+            cvar_scalt(iel) = tpent
         endif
 
         if (ippmod(iatmos).eq.2) then
@@ -318,19 +338,19 @@ if (isuite.eq.0) then
             !==========
          (nbmett, nbmetm,                                               &
           ztmet, tmmet, tpmet, zent  , ttcabs, tpent )
-            rtp(iel,isca(iscalt)) = tpent
+            cvar_scalt(iel) = tpent
 
             call intprf                                                 &
             !==========
          (nbmett, nbmetm,                                               &
           ztmet, tmmet, qvmet, zent  , ttcabs, qvent )
-            rtp(iel,isca(itotwt)) = qvent
+            cvar_totwt(iel) = qvent
 
             call intprf                                                 &
             !==========
          (nbmett, nbmetm,                                               &
           ztmet, tmmet, ncmet, zent  , ttcabs, ncent )
-            rtp(iel,isca(intdrp)) = ncent
+            cvar_ntdrp(iel) = ncent
         endif
 
       enddo
@@ -344,7 +364,7 @@ endif
 ! 4. USER  OPTIONS
 !===============================================================================
 
-call cs_user_initialization &
+call cs_user_f_initialization &
 !==========================
 ( nvar   , nscal  ,                                            &
   dt     )
@@ -441,7 +461,7 @@ call cs_user_initialization &
 '@    The simulation time is wrong                            ',/,&
 '@    Check variables syear, squant, shour, smin, ssec        ',/,&
 '@                                                            ',/,&
-'@    By decreasing priority these variablse can be defined   ',/,&
+'@    By decreasing priority, these variables can be defined  ',/,&
 '@    in cs_user_parameters or the meteo file                 ',/,&
 '@    or the chemistry file                                   ',/,&
 '@                                                            ',/,&
@@ -459,7 +479,7 @@ call cs_user_initialization &
 '@    The simulation time is wrong                            ',/,&
 '@    Check variables syear, squant, shour, smin, ssec        ',/,&
 '@                                                            ',/,&
-'@    By decreasing priority these variablse can be defined   ',/,&
+'@    By decreasing priority, these variables can be defined  ',/,&
 '@    in cs_user_parameters or the meteo file                 ',/,&
 '@    or the chemistry file                                   ',/,&
 '@                                                            ',/,&
