@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,13 +26,11 @@
 
 !> \file atlecc.f90
 !> \brief Reads the chemistry profile data for the atmospheric chemistry
-!>
-!>             imode = 0 : reading of dimensions only
-!>             imode = 1 : reading of data
-!>
-!>             warning : arguments are defined only on second call
-!>             of the routine (imode = 1)
 !
+!> \brief Reads the chemistry profile data for the atmospheric chemistry
+!>-     imode = 0 : reading of dimensions only, imode = 1 : reading of data
+!>-     warning : arguments are defined only on second call
+!>             of the routine (imode = 1)
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -40,13 +38,10 @@
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]     mode          execution mode
+!> \param[in]     imode         execution mode
 !_______________________________________________________________________________
 
-subroutine atlecc &
-!================
-
- ( imode)
+subroutine atlecc (imode )
 
 !===============================================================================
 ! Module files
@@ -78,13 +73,13 @@ integer itp, ii, ios, k
 integer sjday,minute
 double precision second
 integer year, month, quant, hour, day, jday
-character*80     ccomnt, label
-character*1      csaute
+character(len=80) :: ccomnt, label, oneline
+character(len=1) :: csaute
 
 ! altitudes and concentrations of every nespgi species
 double precision  zconctemp(nespgi+1)
 ! names of species defined in the file in case of a user defined chemical scheme
-character*80, allocatable, dimension(:) ::      namespg
+character(len=80), allocatable, dimension(:) ::      namespg
 
 !===============================================================================
 
@@ -119,27 +114,42 @@ itp = itp+1
 
  101  read (impmec,'(A80)',err=999,end=906) ccomnt
 
-if(ccomnt(1:1).eq.csaute) go to 101
+if (ccomnt(1:1).eq.csaute) go to 101
 backspace(impmec)
 
 !===============================================================================
 ! 2. reading the time of the profile
 !===============================================================================
 
-! --> year, month, day, hour (TU), minute, second
+! two formats possible :
+! --> year, month, day, hour, minute, second  of the profile (UTC)
+! --> year, quant-day, hour, minute, second  of the profile (UTC)
+! NB: second is real, all others are integers
 
 if (imode.eq.0) then
-  read(impmec,*,err=999,end=906)
+  read(impmec, *, err=999, end=906)
 else
-  read(impmec,*,err=907,end=906) year, month, day, hour, minute, second
-  call comp_quantile(day,month,year,quant)
+  second = -9999.d0
+  read(impmec, '(a80)', err=999, end=906) oneline
+  read(oneline,*,err=907,end=907) year, month, day, hour, minute, second
+  ! --> catch some read errors
+  if (month.gt.12.or.day.gt.31) then
+    write(nfecra,8005)
+    call csexit (1)
+  endif
+  call comp_quantile(day, month, year, quant)
   goto 908
 907  continue
-  backspace(impmec)
-  read(impmec,*,err=999,end=906) year, quant, hour, minute, second
+  read(oneline,*,err=999,end=906) year, quant, hour, minute, second
 908 continue
+! --> catch some read errors
+  if (second.lt.0d0.or.quant.gt.366) then
+    write(nfecra,8005)
+    call csexit (1)
+  endif
 
-  ! --> if the date and time are not completed in usati1.f90 nor and if no meteo file is given,
+  ! --> if the date and time are not completed in usati1.f90 nor and if no meteo
+  ! --> file is given,
   ! --> the date and time of the first concentration profile are taken as the
   !     starting time of the simulation
 
@@ -300,9 +310,9 @@ if (nespgi.ge.1) then
     read(impmec,*,err=999,end=999) idespgi(1:nespgi)
   endif
 
-!=================================================================================
+!===============================================================================
 ! 9. reading the concentration profiles
-!=================================================================================
+!===============================================================================
 
 109 read(impmec,'(a80)',err=999,end=999) ccomnt
 
@@ -339,9 +349,9 @@ if (nespgi.ge.1) then
 
 endif ! fin test nespgi
 
-!================================================================================
+!===============================================================================
 ! 10. logging
-!================================================================================
+!===============================================================================
 
 if (imode.eq.1) then
   if (itp.eq.1) then
@@ -357,7 +367,7 @@ if (imode.eq.1) then
 7996 format(1x, f10.2)
   write(nfecra, '(a)', advance='no') 'zproc, '
   do ii = 1, nespgi
-    f_id = ivarfl(isca(isca_chem(k)))
+    f_id = ivarfl(isca(isca_chem(ii)))
     call field_get_label(f_id, label)
     if (ii .lt. nespgi) then
       write(nfecra, '(a)', advance='no') trim(label)//', '
@@ -434,7 +444,7 @@ call csexit (1)
 '@    MODULE DE CHIMIE (ICHEMISTRY) DEMANDE                   ',/,&
 '@                                                            ',/,&
 '@              Erreur  dans le fichier chimie :              ',/,&
-'@  le nombre de mesures de concentrations doit tre          ',/,&
+'@  le nombre de mesures de concentrations doit etre          ',/,&
 '@  superieur a 2                                             ',/,&
 '@                                                            ',/,&
 '@  Le calcul ne sera pas execute.                            ',/,&
@@ -460,6 +470,21 @@ call csexit (1)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
+ 8005 format (                                                    &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@ATTENTION:VERIFICATION A L''ENTREE DES DONNEES (atlecc)  ',/,&
+'@   =========                                                ',/,&
+'@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
+'@                                                            ',/,&
+'@  erreur a la lecture de la date du fichier meteo           ',/,&
+'@  verifier le format (entiers, reel)                        ',/,&
+'@                                                            ',/,&
+'@  Le calcul ne sera pas execute.                            ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                             ',/)
  9998 format(                                                           &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -562,6 +587,21 @@ call csexit (1)
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
+ 8005 format (                                                    &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@ WARNING : STOP WHILE READING INPUT DATA (atlecc)        ',/,&
+'@    =========                                               ',/,&
+'@      ATMOSPHERIC CHEMISTRY                                 ',/,&
+'@                                                            ',/,&
+'@  Error opening the chemistry profile file                  ',/,&
+'@  verify input format (integer, real)                       ',/,&
+'@                                                            ',/,&
+'@  The computation will not be run                           ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                             ',/)
  9998 format(                                                           &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -570,8 +610,8 @@ call csexit (1)
 '@     =======                                                ',/,&
 '@      ATMOSPHERIC CHEMISTRY                                 ',/,&
 '@                                                            ',/,&
-'@  Error opening the chemistry profile file                     ',/,&
-'@  check the name of the chemistry file                         ',/,&
+'@  Error opening the chemistry profile file                  ',/,&
+'@  check the name of the chemistry file                      ',/,&
 '@                                                            ',/,&
 '@  The computation will not be run                           ',/,&
 '@                                                            ',/,&
