@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,33 +19,17 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-
-subroutine atlecm &
-     !================
-
-     ( imode )
-
-!===============================================================================
-!  Purpose:
-!  -------
-!             Reads the meteo profile data
-!             for the atmospheric module
-!
-!             IMODE = 0 : READING FOR DIMENSIONS ONLY
-!             IMODE = 1 : READING ACTUAL METEO DATA
-!
-!             WARNING : METEO DATA STRUCTURE CAN BE FOUND IN MODULE ATINCL
+!> \file atlecm.f90
+!> \brief Reads the meteo profile data for the atmospheric module
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________!____!_____!________________________________________________!
-
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   imode       IMODE = 0 : READING FOR DIMENSIONS ONLY
+!>                          IMODE = 1 : READING ACTUAL METEO DATA
+!-------------------------------------------------------------------------------
+subroutine atlecm ( imode )
 
 !===============================================================================
 ! Module files
@@ -57,6 +41,7 @@ use entsor
 use cstnum
 use cstphy
 use ppppar
+use optcal
 use atincl
 
 !===============================================================================
@@ -83,8 +68,8 @@ double precision rap,rscp,tmoy, rhmoy
 double precision ztop, zzmax, tlkelv, pptop, dum
 double precision rhum,q0,q1
 
-character*80     ccomnt
-character*1      csaute
+character(len=80) :: ccomnt,oneline
+character(len=1)  :: csaute
 
 !===============================================================================
 
@@ -136,19 +121,33 @@ backspace(impmet)
 ! 2. Read the time of the profile
 !===============================================================================
 
-! --> year, quant-day, hour, minute, second  of the profile (TU)
+! two formats possible :
+! --> year, month, day, hour, minute, second  of the profile (UTC)
+! --> year, quant-day, hour, minute, second  of the profile (UTC)
+! NB: second is real, all others are integers
 
 if (imode.eq.0) then
   read(impmet, *, err=999, end=906)
 else
-  read(impmet, *, err=907, end=906) year, month, day, hour, minute, second
+  second = -9999.d0
+  read(impmet, '(a80)', err=999, end=906) oneline
+  read(oneline, *, err=907, end=907) year, month, day,  &
+                                                  hour, minute, second
+! --> catch some read errors
+  if (month.gt.12.or.day.gt.31) then
+    write(nfecra,8005)
+    call csexit (1)
+  endif
   call comp_quantile(day, month, year, quant)
   goto 908
 907  continue
-  backspace(impmet)
-  read(impmet,*,err=999,end=906) year, quant, hour, minute, second
+  read(oneline, *, err=999, end=906) year, quant, hour, minute, second
 908 continue
-
+! --> catch some read errors
+  if (second.lt.0d0.or.quant.gt.366) then
+    write(nfecra,8005)
+    call csexit (1)
+  endif
 
   ! --> if the date and time are not completed in usati1.f90,
   !     the date and time of the first meteo profile are taken as the
@@ -320,6 +319,19 @@ else
 
 endif
 
+! For Boussinesq approximation, initialize p0, rho0 and theta0 at the first level
+if (imode.eq.1.and.idilat.eq.0.and.itp.eq.1) then
+  p0 = pmer(itp)
+  t0 = (ttmet(1,itp)+tkelvi)*(ps/p0)**rscp ! theta0
+  ! Humid atmosphere
+  if (ippmod(iatmos).eq.2) then
+    rhum = rair*(1.d0+(rvsra-1.d0)*qvmet(1,itp)*ih2o)
+  else
+    rhum = rair
+  endif
+  ro0 = p0 / (ttmet(1,itp) + tkelvi)/rhum
+endif
+
 !===============================================================================
 ! 6. Compute hydro pressure profile  (Laplace integration)
 !===============================================================================
@@ -454,7 +466,7 @@ if (imode.eq.1) then
 7995 format(1x, i4, i5, 2i4, f10.2)
   write(nfecra, *) 'tmmet(itp):'
   write(nfecra, 7996) tmmet(itp)
-7996 format(1x, f10.2)
+7996 format(1x, e10.3)
   write(nfecra, *) 'zdmet, umet, vmet, ekmet, epmet:'
   do ii = 1, nbmetd
     write(nfecra, 7997)                                            &
@@ -593,7 +605,7 @@ call csexit (1)
      '@                                                            ',/,&
      '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
      '@                                                            ',/,&
-     '@ @@ATTENTION:VERIFICIATION A L''ENTREE DES DONNEES (atlecm)',/,&
+     '@ @@ATTENTION:VERIFICATION A L''ENTREE DES DONNEES (atlecm)' ,/,&
      '@   =========                                               ',/,&
      '@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
      '@                                                            ',/,&
@@ -605,6 +617,21 @@ call csexit (1)
      '@                                                            ',/,&
      '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
      '@                                                             ',/)
+8005 format (                                                    &
+     '@                                                            ',/,&
+     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+     '@                                                            ',/,&
+     '@ @@ATTENTION:VERIFICATION A L''ENTREE DES DONNEES (atlecm)  ',/,&
+     '@   =========                                                ',/,&
+     '@        PHYSIQUE PARTICULIERE ATMOSPHERIQUE                 ',/,&
+     '@                                                            ',/,&
+     '@  erreur a la lecture de la date du fichier meteo           ',/,&
+     '@  verifier le format (entiers, reel)                        ',/,&
+     '@                                                            ',/,&
+     '@  Le calcul ne sera pas execute.                            ',/,&
+     '@                                                            ',/,&
+     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+     '@                                                             ',/)
 9998 format(                                                     &
      '@                                                            ',/,&
      '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -724,7 +751,21 @@ call csexit (1)
      '@      ATMOSPHERIC SPECIFIC PHYSICS                          ',/,&
      '@                                                            ',/,&
      '@              Error in the meteo profile file:              ',/,&
-     '@  Number of droplets readed  <  0                           ',/,&
+     '@  Number of droplets read  <  0                             ',/,&
+     '@  The computation will not be run                           ',/,&
+     '@                                                            ',/,&
+     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+     '@                                                             ',/)
+8005 format (                                                    &
+     '@                                                            ',/,&
+     '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+     '@                                                            ',/,&
+     '@ @@  WARNING:   CHECKING INPUT DATA (atlecm)                ',/,&
+     '@     =======                                                ',/,&
+     '@      ATMOSPHERIC SPECIFIC PHYSICS                          ',/,&
+     '@                                                            ',/,&
+     '@              Error in the date of meteo profile file:      ',/,&
+     '@  Check the format (integers,real) for the date             ',/,&
      '@  The computation will not be run                           ',/,&
      '@                                                            ',/,&
      '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
@@ -780,30 +821,18 @@ end subroutine atlecm
 
 
 !===============================================================================
-
-subroutine comp_quantile(jour,mois,annee,quant)
-
-!===============================================================================
-!  Purpose:
-!  -------
-!             Computes quantile
+!> \brief Compute the calendar day number from the date
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________!____!_____!________________________________________________!
-
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
-
-!===============================================================================
-! Module files
-!===============================================================================
-
-!===============================================================================
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]       jour        day
+!> \param[in]       mois        month
+!> \param[in]       annee       year
+!> \param[out]      quant       calendar day number
+!-------------------------------------------------------------------------------
+subroutine comp_quantile(jour,mois,annee,quant)
 
 implicit none
 
@@ -814,7 +843,6 @@ integer quant
 ! Local variables
 integer distrib, booll, retrait
 
-
 distrib = int(mois * 275 / 9.) - 30
 booll = int((mois + 9 ) / 12.) !boolean=0 for jan and feb and =1 for march to dec
 retrait = 1+int(mod(annee,4)+2)/3 !retrait=2 days non leap years and =1 day for leap years
@@ -822,5 +850,3 @@ quant = distrib-(booll*retrait)+jour
 
 return
 end subroutine comp_quantile
-
-
