@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,48 +19,30 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-
-subroutine atprke &
-!================
-
- ( nscal  ,                                                       &
-   rtpa   , propce ,                                              &
-   tinstk ,                                                       &
-   smbrk  , smbre )
-
-!===============================================================================
-! FONCTION :
-! --------
-!  SPECIFIQUE AU CAS ATMOSPHERIQUE :
-!  CALCUL DU TERME DE PRODUCTION LIEE A LA FLOTTABILITE :
-!  G = G*GRAD(THETA)/PRDTUR/THETA
+!> \file atprke.f90
+!> \brief Modify the \f$k-\varepsilon\f$ turbulence model formulation (cf.: turbke)
+!>  for the atmospheric module
+!
+!>\brief Adjonction of a production term for buyancy in the \f$k-\varepsilon\f$ model
+!>  in the context of the atmospheric module \n
+!>  G = G*GRAD(THETA)/PRDTUR/THETA
 !-------------------------------------------------------------------------------
-! ARGUMENTS
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! nscal            ! i  ! <-- ! total number of scalars                        !
-! rtpa             ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at previous time step)                       !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-! tinstk(ncelet)   ! tr ! --> ! Implicit part of the buoyancy term (for k)     !
-! smbrk(ncelet)    ! tr ! --> ! Explicit part of the buoyancy term (for k)     !
-! smbre(ncelet)    ! tr ! --> ! Explicit part of the buoyancy term (for eps)   !
-!__________________!____!_____!________________________________________________!
-
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   nscal           total number of scalars
+!> \param[in]   tinstk          Implicit part of the buoyancy term (for k)
+!> \param[in]   smbrk           Explicit part of the buoyancy term (for k)
+!> \param[in]   smbre           Explicit part of the buoyancy term (for eps)
 !-------------------------------------------------------------------------------
-!===============================================================================
+subroutine atprke ( nscal, tinstk, smbrk, smbre )
 
 !===============================================================================
 ! Module files
 !===============================================================================
 
 use paramx
-use dimens, only: nvar
 use numvar
 use entsor
 use cstnum
@@ -83,28 +65,32 @@ implicit none
 ! Arguments
 integer          nscal
 
-double precision rtpa (ncelet,nflown:nvar)
-double precision propce(ncelet,*)
 double precision smbrk(ncelet), smbre(ncelet)
 double precision tinstk(ncelet)
 
 ! Local variables
-integer         iel
-integer         ipcvto
-integer         itpp , iqw
-integer         ipcliq
-integer         iccocg, inc
-integer         iivar
+integer          iel
+integer          itpp , iqw
+integer          iccocg, inc
+integer          iivar
+integer          key_t_ext_id
+integer          iviext
+integer          iroext
 
 double precision gravke, prdtur
 double precision theta_virt
 double precision qldia,qw
 double precision xk, xeps, visct, ttke, rho
+double precision turb_schmidt
 
 double precision dum
 double precision, allocatable, dimension(:,:) :: grad
 double precision, dimension(:), pointer :: cromo
 
+double precision, dimension(:), pointer :: cvara_k, cvara_ep
+double precision, dimension(:), pointer :: cvara_tpp, cvara_qw
+double precision, dimension(:), pointer :: cpro_pcvto, cpro_pcliq
+
 !===============================================================================
 
 !===============================================================================
@@ -114,18 +100,28 @@ double precision, dimension(:), pointer :: cromo
 ! Allocate work arrays
 allocate(grad(3,ncelet))
 
+! Time extrapolation?
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
 ! Pointer to density and turbulent viscosity
 call field_get_val_s(icrom, cromo)
-ipcvto = ipproc(ivisct)
+call field_get_val_s(ivisct, cpro_pcvto)
 if(isto2t.gt.0) then
+  call field_get_key_int(icrom, key_t_ext_id, iroext)
   if (iroext.gt.0) then
     call field_get_val_prev_s(icrom, cromo)
   endif
+  call field_get_key_int(ivisct, key_t_ext_id, iviext)
   if(iviext.gt.0) then
-    ipcvto = ipproc(ivista)
+    call field_get_val_prev_s(ivisct, cpro_pcvto)
   endif
 endif
 
+if ((ippmod(iatmos).eq.1.and.itytur.eq.2).or.ippmod(iatmos).eq.2) then
+  call field_get_val_prev_s(ivarfl(ik), cvara_k)
+  call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+endif
+
 !===============================================================================
 ! 2. Calcul des derivees de la temperature potentielle
 !===============================================================================
@@ -140,6 +136,7 @@ endif
 deallocate(grad)
 
 return
+!**************************************************************************
 contains
 
 !**************************************************************************
@@ -148,16 +145,18 @@ contains
 !--------------------------------------------------------------------------
 !
 !--------------------------------------------------------------------------
-
+!> \brief Internal function -
+!> Computes the production term in case of dry atmosphere
+!> ie. when ippmod(iatmos) eq 1
+!--------------------------------------------------------------------------
 subroutine dry_atmosphere()
 
-! computes the production term in case of dry atmosphere
-! ie. when ippmod(iatmos) eq 1
-
 ! Computation of the gradient of the potential temperature
 
 itpp = isca(iscalt)
 
+call field_get_val_prev_s(ivarfl(itpp), cvara_tpp)
+
 ! computational options:
 
 iccocg = 1
@@ -166,7 +165,7 @@ inc = 1
 iivar = itpp
 
 ! computes the turbulent production/destruction terms:
-! dry atmo: (1/sigmas*theta)*(dtheta/dz)*gz
+! dry atmo: (1/turb_schmidt*theta)*(dtheta/dz)*gz
 
 call field_gradient_scalar(ivarfl(iivar), 1, imrgra, inc,           &
                            iccocg,                                  &
@@ -176,7 +175,8 @@ call field_gradient_scalar(ivarfl(iivar), 1, imrgra, inc,           &
 ! TINSTK=P+G et TINSTE = P + (1-CE3)*G
 
 if (iscalt.gt.0.and.nscal.ge.iscalt) then
-  prdtur = sigmas(iscalt)
+  call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
+  prdtur = turb_schmidt
 else
   prdtur = 1.d0
 endif
@@ -184,13 +184,13 @@ endif
 if (itytur.eq.2) then
   do iel = 1, ncel
     rho   = cromo(iel)
-    visct = propce(iel,ipcvto)
-    xeps = rtpa(iel,iep )
-    xk   = rtpa(iel,ik )
+    visct = cpro_pcvto(iel)
+    xeps = cvara_ep(iel)
+    xk   = cvara_k(iel)
     ttke = xk / xeps
 
     gravke = (grad(1,iel)*gx + grad(2,iel)*gy + grad(3,iel)*gz) &
-           / (rtpa(iel,itpp)*prdtur)
+           / (cvara_tpp(iel)*prdtur)
 
     ! Implicit part (no implicit part for epsilon because the source
     ! term is positive)
@@ -207,7 +207,10 @@ end subroutine dry_atmosphere
 !--------------------------------------------------------------------------
 !
 !--------------------------------------------------------------------------
-
+!> \brief Internal function -
+!> Computes the production term in case of humid atmosphere
+!> ie. when ippmod(iatmos) eq 2
+!--------------------------------------------------------------------------
 subroutine humid_atmosphere()
 
 ! computes the production term in case of humid atmosphere
@@ -230,7 +233,10 @@ allocate(gravke_qw(ncelet))
 
 itpp = isca(iscalt)
 iqw = isca(itotwt)
-ipcliq = ipproc(iliqwt)
+
+call field_get_val_prev_s(ivarfl(itpp), cvara_tpp)
+call field_get_val_prev_s(ivarfl(iqw), cvara_qw)
+call field_get_val_s(iliqwt, cpro_pcliq)
 
 ! compute the coefficients etheta,eq
 
@@ -243,9 +249,9 @@ do iel = 1, ncel
          nbmett, nbmetm,                                          &
          ztmet, tmmet, phmet, xyzcen(3,iel), ttcabs, pphy )
   endif
-  qw = rtpa(iel,iqw) ! total water content
-  qldia = propce(iel,ipcliq) ! liquid water content
-  call etheq(pphy,rtpa(iel,itpp),qw,qldia,                      &
+  qw = cvara_qw(iel) ! total water content
+  qldia = cpro_pcliq(iel) ! liquid water content
+  call etheq(pphy,cvara_tpp(iel),qw,qldia,                      &
              nebdia(iel),nn(iel),etheta(iel),eq(iel))
 enddo
 ! options for gradient calculation
@@ -256,7 +262,7 @@ inc = 1
 iivar = itpp
 
 ! computes the turbulent production/destruction terms:
-! humid atmo: (1/sigmas*theta_v)*(dtheta_l/dz)*gz
+! humid atmo: (1/turb_schmidt*theta_v)*(dtheta_l/dz)*gz
 
 call field_gradient_scalar(ivarfl(iivar), 1, imrgra, inc,           &
                            iccocg,                                  &
@@ -266,7 +272,8 @@ call field_gradient_scalar(ivarfl(iivar), 1, imrgra, inc,           &
 ! TINSTK = P + G et TINSTE = P + (1-CE3)*G
 
 if(iscalt.gt.0.and.nscal.ge.iscalt) then
-  prdtur = sigmas(iscalt)
+  call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
+  prdtur = turb_schmidt
 else
   prdtur = 1.d0
 endif
@@ -274,9 +281,9 @@ endif
 ! store now the production term due to theta_liq in gravke_theta
 if (itytur.eq.2) then
   do iel = 1, ncel
-    qw = rtpa(iel,iqw) ! total water content
-    qldia = propce(iel,ipcliq) ! liquid water content
-    theta_virt = rtpa(iel,itpp)*(1.d0 + (rvsra - 1)*qw - rvsra*qldia)
+    qw = cvara_qw(iel) ! total water content
+    qldia = cpro_pcliq(iel) ! liquid water content
+    theta_virt = cvara_tpp(iel)*(1.d0 + (rvsra - 1)*qw - rvsra*qldia)
     gravke = (grad(1,iel)*gx + grad(2,iel)*gy + grad(3,iel)*gz)            &
            / (theta_virt*prdtur)
     gravke_theta(iel) = gravke*etheta(iel)
@@ -290,7 +297,7 @@ endif
 iivar = iqw
 
 ! computes the turbulent production/destruction terms:
-! humid atmo: (1/sigmas*theta_v)*(dtheta_l/dz)*gz
+! humid atmo: (1/turb_schmidt*theta_v)*(dtheta_l/dz)*gz
 
 call field_gradient_scalar(ivarfl(iivar), 1, imrgra, inc,           &
                            iccocg,                                  &
@@ -300,7 +307,8 @@ call field_gradient_scalar(ivarfl(iivar), 1, imrgra, inc,           &
 ! TINSTK = P + G et TINSTE = P + (1-CE3)*G
 
 if (iscalt.gt.0.and.nscal.ge.iscalt) then
-  prdtur = sigmas(iscalt)
+  call field_get_key_double(ivarfl(isca(iscalt)), ksigmas, turb_schmidt)
+  prdtur = turb_schmidt
 else
   prdtur = 1.d0
 endif
@@ -309,9 +317,9 @@ endif
 
 if (itytur.eq.2) then
   do iel = 1, ncel
-    qw = rtpa(iel,iqw) ! total water content
-    qldia = propce(iel,ipcliq) !liquid water content
-    theta_virt = rtpa(iel,itpp)*(1.d0 + (rvsra - 1.d0)*qw - rvsra*qldia)
+    qw = cvara_qw(iel) ! total water content
+    qldia = cpro_pcliq(iel) !liquid water content
+    theta_virt = cvara_tpp(iel)*(1.d0 + (rvsra - 1.d0)*qw - rvsra*qldia)
     gravke = (grad(1,iel)*gx + grad(2,iel)*gy + grad(3,iel)*gz)                 &
            / (theta_virt*prdtur)
     gravke_qw(iel) = gravke*eq(iel)
@@ -321,9 +329,9 @@ endif
 ! Finalization
 do iel = 1, ncel
   rho   = cromo(iel)
-  visct = propce(iel,ipcvto)
-  xeps = rtpa(iel,iep)
-  xk   = rtpa(iel,ik)
+  visct = cpro_pcvto(iel)
+  xeps = cvara_ep(iel)
+  xk   = cvara_k(iel)
   ttke = xk / xeps
 
   gravke = gravke_theta(iel) + gravke_qw(iel)
