@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,37 +19,14 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-
-subroutine atr1vf &
-!================
-     ( rtpa, propce )
-
-!===============================================================================
-!  Purpose:
-!  --------
-
-!    Atmospheric module subroutine.
-
-!    Computes the source term for scalar equations
-!    from radiative forcing (uv and ir radiative fluxes)
-!    computed with the 1d atmo radiative scheme.
-!
+!> \file atr1vf.f90
+!>
+!> \brief Compute radiative fluxes for the atmospheric model.
+!> Computes the source term for scalar equations from radiative forcing
+!> (UV and IR radiative fluxes) with a 1D scheme.
 !-------------------------------------------------------------------------------
-! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! rtpa             ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (preceding time step)                         !
-! propce           ! ra ! <-- ! properties cell centers                        !
-!  (ncelet, *)     !    !     !  (current time step)                           !
-!__________________!____!_____!________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
+subroutine atr1vf
 
 !===============================================================================
 ! Module files
@@ -59,8 +36,8 @@ use paramx
 use numvar
 use entsor
 use optcal
-use dimens, only: nvar
 use cstphy
+use cstnum, only:pi
 use parall
 use period
 use ppppar
@@ -75,15 +52,10 @@ use atsoil
 
 implicit none
 
-! Arguments
-
-double precision propce(ncelet,*)
-double precision rtpa(ncelet,nflown:nvar)
-
 ! Local variables
 integer k, ii, jj
 integer k1
-integer iico2,imer1
+integer ico2,imer1
 integer ideb, icompt
 integer kmray, ktamp
 
@@ -92,12 +64,14 @@ double precision xvert, yvert
 double precision zrac,fpond,rap,tmoy,rhum,dum
 
 integer , allocatable :: cressm(:), interp(:)
-double precision, allocatable :: temray(:), qvray(:), qlray(:)
-double precision, allocatable :: fneray(:), romray(:), preray(:)
-double precision, allocatable :: zproj(:), ttvert(:), qvvert(:), romvert(:)
-double precision, allocatable :: aeroso(:)
-double precision, allocatable :: coords(:,:,:), infrad(:)
-double precision, dimension(:), pointer :: crom
+double precision, allocatable, dimension(:) :: temray, qvray, qlray, ncray
+double precision, allocatable, dimension(:) :: fneray, romray, preray
+double precision, allocatable, dimension(:) :: zproj, ttvert, qvvert, romvert
+double precision, allocatable, dimension(:) :: qwvert, qlvert, ncvert,fnvert
+double precision, allocatable, dimension(:) :: aeroso, infrad
+double precision, allocatable, dimension(:,:,:) :: coords(:,:,:)
+double precision, dimension(:), pointer :: crom, cpro_pcliq
+double precision, dimension(:), pointer :: cvara_totwt, cpro_tempc, cvara_ntdrp
 
 save ideb
 data ideb/0/
@@ -106,14 +80,16 @@ data ideb/0/
 ! 1.  INITIALISATIONS
 !===============================================================================
 
-! --- Radiative fluxes are computed at the fisrt call
+! --- Radiative fluxes are computed at the first call
 !     and then for the frequency nfatr1
 
 if (mod(ntcabs,nfatr1).eq.0.or.ideb.eq.0) then
 
-  allocate(temray(kmx), qvray(kmx), qlray(kmx))
+  allocate(temray(kmx), qvray(kmx), qlray(kmx), ncray(kmx))
   allocate(fneray(kmx), romray(kmx), preray(kmx))
   allocate(zproj(kmx), ttvert(kmx*nvert), qvvert(kmx*nvert), romvert(kmx*nvert))
+  allocate(qwvert(kmx*nvert), qlvert(kmx*nvert), ncvert(kmx*nvert))
+  allocate(fnvert(kmx*nvert))
   allocate(aeroso(kmx))
   allocate(coords(3,kmx,nvert))
   allocate(cressm(kmx*nvert), interp(kmx*nvert))
@@ -121,13 +97,13 @@ if (mod(ntcabs,nfatr1).eq.0.or.ideb.eq.0) then
 
   ideb = 1
 
-  heuray = float(shour) + float(smin)/60.d0+ssec/3600.d0                        &
+  heuray = float(shour) + float(smin)/60.d0+ssec/3600.d0                       &
          + (ntcabs-1)*dtref/3600.d0
 
-  if (ntcabs.le.2) then
-    iico2 = 1
+  if (ntcabs.le.2.or.isuite.eq.1) then
+    ico2 = 1
   else
-    iico2 = 0
+    ico2 = 0
   endif
 
   ! --- Initialization:
@@ -143,15 +119,24 @@ if (mod(ntcabs,nfatr1).eq.0.or.ideb.eq.0) then
     qvray(k) = 0.d0
     romray(k) = 0.d0
     qlray (k) = 0.d0
+    ncray (k) = 0.d0
     fneray(k) = 0.d0
     aeroso(k) = 0.d0
   enddo
 
   call field_get_val_s(icrom, crom)
+  call field_get_val_s(itempc, cpro_tempc)
 
-  !===============================================================================
+  if (ippmod(iatmos).eq.2) then
+    call field_get_val_s(iliqwt, cpro_pcliq)
+
+    call field_get_val_prev_s(ivarfl(isca(itotwt)), cvara_totwt)
+    call field_get_val_prev_s(ivarfl(isca(intdrp)), cvara_ntdrp)
+  endif
+
+  !=============================================================================
   ! 2.  Computing long-wave and short-wave radiative fluxes
-  !===============================================================================
+  !=============================================================================
 
   do ii = 1, nvert
 
@@ -166,27 +151,37 @@ if (mod(ntcabs,nfatr1).eq.0.or.ideb.eq.0) then
   enddo
 
   if (ntcabs.eq.1) then
-    call grimap &
-    !===========
-         (igrid, nvert*kmx, coords)
+    call grimap(igrid, nvert*kmx, coords)
   endif
 
-  call gripol(igrid, propce(:,ipproc(itempc)), ttvert)
-  !===========
-  call gripol(igrid, rtpa(:,isca(itotwt)), qvvert)
-  !===========
+  call gripol(igrid, cpro_tempc, ttvert)
   call gripol(igrid, crom, romvert)
-  !===========
+
+  if (ippmod(iatmos).eq.2) then
+    ! liquid content interpolation
+    call gripol(igrid, cpro_pcliq, qlvert)
+    ! total water content interpolation
+    call gripol(igrid, cvara_totwt, qwvert)
+
+    ! deduce vapor content interpolation
+    do ii = 1, nvert
+      do k = 1, kvert
+        jj = (ii-1)*kmx + k
+        qvvert(jj) = qwvert(jj)-qlvert(jj)
+      enddo
+    enddo
+
+    call gripol(igrid, cvara_ntdrp, ncvert)
+    call gripol(igrid, nebdia, fnvert)
+  endif
 
   ! --- Loop on the vertical array:
 
   do ii = 1, nvert
-
     xvert = xyvert(ii,1)
     yvert = xyvert(ii,2)
 
-    ! --- Soil constants:
-
+    ! Soil constants
     albedo = soilvert(ii)%albedo
     emis = soilvert(ii)%emissi
     fos = soilvert(ii)%fos
@@ -195,45 +190,49 @@ if (mod(ntcabs,nfatr1).eq.0.or.ideb.eq.0) then
     ! 2.1 Profiles used for the computation of the radiative fluxes
     !--------------------------------------------------------------
 
-    ! --- Soil variables:
-
+    ! Soil variables
     zray(1)   = zvert(1)
     temray(1) = soilvert(ii)%ttsoil
     qvray(1)  = soilvert(ii)%totwat
     romray(1) = soilvert(ii)%density
     preray(1) = soilvert(ii)%pressure
-    qlray (1) = 0.d0
+    qlray(1)  = 0.d0
+    ncray(1)  = 0.d0
     fneray(1) = 0.d0
     aeroso(1) = 0.d0
 
-
-    ! --- Interpolation of temperature, humidity, density on the vertical
-    !     The ref pressure profile is the one computed from the meteo profile
-
+    ! Interpolation of temperature, humidity, density on the vertical
+    ! The ref pressure profile is the one computed from the meteo profile
+    if (ippmod(iatmos).eq.2.and.moddis.eq.2) then
+      qlray(1) = qlvert(1 + (ii-1)*kmx)
+      ncray(1)  = ncvert(1 + (ii-1)*kmx)
+      fneray(1) = fnvert(1 + (ii-1)*kmx)
+    endif
 
     do k = 2, kvert
       zray(k) = zvert(k)
 
-      if (imeteo.eq.0) then
-        call atmstd(zray(k),preray(k),dum,dum)
-      else
-        call intprf(nbmetd, nbmetm, ztmet, tmmet,                           &
-                    phmet, zray(k), ttcabs, preray(k))
-      endif
-
       temray(k) = ttvert(k + (ii-1)*kmx)
       qvray(k)  = qvvert(k + (ii-1)*kmx)
       romray(k) = romvert(k + (ii-1)*kmx)
 
-      ! ---  Default values:
-      !      - liquide water = 0.
-      !      - cloud fract.  = 0.
-      !      - aerosols      = 0.
-
-      qlray (k) = 0.d0
+      ! default values
+      ncray(k) = 0.d0
+      qlray(k) = 0.d0
       fneray(k) = 0.d0
-      aeroso(k) = 0.d0
 
+      if (ippmod(iatmos).eq.2.and.moddis.eq.2) then
+        ncray(k)  = ncvert(k + (ii-1)*kmx)
+        qlray(k)  = qlvert(k + (ii-1)*kmx)
+        fneray(k) = fnvert(k + (ii-1)*kmx)
+      endif
+
+      if (imeteo.eq.0) then
+        call atmstd(zray(k), preray(k), dum, dum)
+      else
+        call intprf(nbmetd, nbmetm, ztmet, tmmet, phmet, zray(k), ttcabs, &
+                    preray(k))
+      endif
     enddo
 
     ! --- Filling the additional levels
@@ -245,30 +244,38 @@ if (mod(ntcabs,nfatr1).eq.0.or.ideb.eq.0) then
       qvray(k) = 0.d0
       fneray(k) = 0.d0
       aeroso(k) = 0.d0
+      ncray(k) = 0.d0
+
+      ! initialize with standard atmosphere
+      call atmstd(zray(k), preray(k), temray(k), romray(k))
     enddo
 
+    call cs_user_atmo_1d_rad_prf(preray, temray, romray, qvray, qlray, &
+                                 ncray, aeroso)
+
     ! --- Smoothing the temperature and humidity profile in the damping zone
+
     if (imeteo.eq.1) then
       ktamp = 6
       do k = kvert - ktamp+1, kmray
-        call intprf(nbmaxt,nbmetm, ztmet, tmmet,                                 &
-             ttmet, zray(k), ttcabs, temray(k))
-        call intprf(nbmaxt,nbmetm, ztmet, tmmet, qvmet,                          &
-             zray(k), ttcabs, qvray(k))
+        call intprf(nbmaxt, nbmetm, ztmet, tmmet,                              &
+                    ttmet, zray(k), ttcabs, temray(k))
+        call intprf(nbmaxt, nbmetm, ztmet, tmmet, qvmet,                       &
+                    zray(k), ttcabs, qvray(k))
       enddo
 
       icompt = 0
       do k = kvert,2,-1
         icompt = icompt+1
         if (icompt.le.6) then
-          zrac = 2.d0*(zray(k) - zray(nbmett-ktamp + 3))                         &
+          zrac = 2.d0*(zray(k) - zray(nbmett-ktamp + 3))                       &
                /(zray(nbmett) - zray(nbmett - ktamp))
           fpond = (1.d0 + tanh(zrac))/2.d0
           temray(k) = ttvert(k + (ii-1)*kmx)*(1.d0 - fpond) + temray(k)*fpond
           qvray(k) = qvvert(k + (ii-1)*kmx)*(1.d0 - fpond) + qvray(k)*fpond
+          qlray(k) = qlvert(k + (ii-1)*kmx)*(1.d0 - fpond) + qlray(k)*fpond
         endif
       enddo
-
     endif
 
     ! --- Clipping the humidity
@@ -292,24 +299,24 @@ if (mod(ntcabs,nfatr1).eq.0.or.ideb.eq.0) then
 
     k1 = 1
 
-    ! --- Long-wave:
-    call rayir ( k1,kmray,iico2,emis,                           &
-         tauzq, tauz, tausup, zq,                       &
-         acinfe, dacinfe, aco2, daco2, acsup, dacsup,   &
-         zray,temray,qvray,                             &
-         qlray,fneray,romray,preray,aeroso,             &
-         foir,rayi(:,ii) )
+    ! --- Long-wave: InfraRed
+    call rayir(ii, k1, kmray, ico2, emis,                         &
+               tauzq, tauz, tausup, zq,                           &
+               acinfe, dacinfe, aco2, daco2, aco2s, daco2s,       &
+               acsup, dacsup, acsups, dacsups,                    &
+               zray, temray, qvray,                               &
+               qlray, fneray, romray, preray, aeroso,             &
+               foir, rayi(:,ii), ncray)
 
-    ! --- Short-wave:
-    call rayso ( k1,kmray,heuray,imer1,albedo,                  &
-         tauzq, tauz, tausup, zq,                       &
-         zray,                                          &
-         qvray,qlray,fneray,romray,aeroso,              &
-         fos,rayst(:,ii) )
+    ! --- Short-wave: Sun
+    call rayso(ii, k1, kmray, heuray, imer1, albedo,             &
+               tauzq, tauz, tausup, zq,                          &
+               zray,                                             &
+               qvray, qlray, fneray, romray, preray, aeroso,     &
+               fos, rayst(:,ii), ncray)
 
-    soilvert(ii)%fos= fos
+    soilvert(ii)%fos = fos
     soilvert(ii)%foir = foir
-
   enddo
 
   do ii = 1, kmx*nvert
@@ -320,11 +327,13 @@ if (mod(ntcabs,nfatr1).eq.0.or.ideb.eq.0) then
     infrad(3*(ii-1) + 3) = 4.d0/1.d0    ! vertical(z) cressman radius
   enddo
 
+  ! Map Infra Red 1D (rayi) on the structure idrayi
   call mesmap (idrayi, kmx*nvert, rayi, coords,            &
-       cressm, interp, infrad)
-  call mesmap (idrayst, kmx*nvert, rayst, coords,          &
-       cressm, interp, infrad)
+               cressm, interp, infrad)
 
+  ! Map Sun 1D (rayst) on the structure idrayst
+  call mesmap (idrayst, kmx*nvert, rayst, coords,          &
+               cressm, interp, infrad)
 
   deallocate(temray, qvray, qlray)
   deallocate(fneray, romray, preray)
