@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,47 +19,50 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
+!> \file attycl.f90
+!> \brief Automatic boundary conditions for atmospheric module
+!>   (based on meteo file)
 
-subroutine attycl &
-!================
-
- ( itypfb , izfppp ,                                              &
-   propce ,                                                       &
-   rcodcl )
-
-!===============================================================================
-! FONCTION :
-! --------
-!    CONDITIONS AUX LIMITES AUTOMATIQUES
-!           ECOULEMENTS ATMOSPHERIQUES
+!> \brief Automatically compute the boundary conditions from the meteo file
+!>      or from the imbrication profiles
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! itypfb           ! ia ! <-- ! boundary face types                            !
-! izfppp           ! te ! <-- ! numero de zone de la face de bord              !
-! (nfabor)         !    !     !  pour le module phys. part.                    !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-! rcodcl           ! tr ! --> ! valeur des conditions aux limites              !
-!  (nfabor,nvarcl) !    !     !  aux faces de bord                             !
-!                  !    !     ! rcodcl(1) = valeur du dirichlet                !
-!                  !    !     ! rcodcl(2) = valeur du coef. d'echange          !
-!                  !    !     !  ext. (infinie si pas d'echange)               !
-!                  !    !     ! rcodcl(3) = valeur de la densite de            !
-!                  !    !     !  flux (negatif si gain) w/m2 ou                !
-!                  !    !     !  hauteur de rugosite (m) si icodcl=6           !
-!                  !    !     ! pour les vitesses (vistl+visct)*gradu          !
-!                  !    !     ! pour la pression             dt*gradp          !
-!                  !    !     ! pour les scalaires                             !
-!                  !    !     !        cp*(viscls+visct/sigmas)*gradt          !
-!__________________!____!_____!________________________________________________!
-
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   itypfb          boundary face types
+!> \param[in]   izfppp          boundary face zone number for atmospheric module
+!> \param[out]  icodcl          face boundary condition code
+!>                               - 1 Dirichlet
+!>                               - 2 Radiative outlet
+!>                               - 3 Neumann
+!>                               - 4 sliding and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 5 smooth wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 6 rough wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 9 free inlet/outlet
+!>                                 (input mass flux blocked to 0)
+!>                               - 13 Dirichlet for the advection operator and
+!>                                    Neumann for the diffusion operator
+!> \param[out]  rcodcl          Boundary conditions value
+!>                               - rcodcl(1) value of the dirichlet
+!>                               - rcodcl(2) value of the exterior exchange
+!>                                 coefficient (infinite if no exchange)
+!>                               - rcodcl(3) value flux density
+!>                                 (negative if gain) in w/m2 or roughness
+!>                                 in m if icodcl=6
+!>                                 -# for the velocity \f$ (\mu+\mu_T)
+!>                                    \gradv \vect{u} \cdot \vect{n}  \f$
+!>                                 -# for the pressure \f$ \Delta t
+!>                                    \grad P \cdot \vect{n}  \f$
+!>                                 -# for a scalar \f$ cp \left( K +
+!>                                     \dfrac{K_T}{\sigma_T} \right)
+!>                                     \grad T \cdot \vect{n} \f$
+!
+!-------------------------------------------------------------------------------
+subroutine attycl ( itypfb, izfppp, icodcl, rcodcl )
 
 !===============================================================================
 ! Module files
@@ -70,6 +73,7 @@ use numvar
 use optcal
 use cstphy
 use cstnum
+use dimens, only: nvar
 use entsor
 use parall
 use ppppar
@@ -81,6 +85,7 @@ use atincl
 use atchem
 use atimbr
 use siream
+use cs_c_bindings
 
 !===============================================================================
 
@@ -90,21 +95,19 @@ implicit none
 
 integer          itypfb(nfabor)
 integer          izfppp(nfabor)
-
-double precision propce(ncelet,*)
-double precision rcodcl(nfabor,nvarcl,3)
+integer          icodcl(nfabor,nvar)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
-integer          ifac, izone
-integer          icke, iel, ipcvis
+integer          ifac, iel, izone
 integer          ii
 integer jsp, isc
 double precision d2s3, zent, vs, xuent, xvent
 double precision xkent, xeent, tpent, qvent,ncent
-double precision dhy, rhomoy, uref2, ustar2, viscla, xiturb
 double precision xcent
-double precision, dimension(:), pointer ::  brom
+double precision viscla, uref2, rhomoy, dhy, xiturb
+double precision, dimension(:), pointer :: brom, coefap, viscl
 
 ! arrays for cressman interpolation
 double precision , dimension(:),allocatable :: u_bord
@@ -114,14 +117,11 @@ double precision , dimension(:),allocatable :: eps_bord
 double precision , dimension(:),allocatable :: theta_bord
 double precision , dimension(:),allocatable :: qw_bord
 double precision , dimension(:),allocatable :: nc_bord
-integer stat
 
-!===============================================================================
 !===============================================================================
 ! 1.  INITIALISATIONS
 !===============================================================================
 
-
 d2s3 = 2.d0/3.d0
 
 xuent = 0.d0
@@ -131,7 +131,7 @@ xeent = 0.d0
 tpent = 0.d0
 
 call field_get_val_s(ibrom, brom)
-ipcvis = ipproc(iviscl)
+call field_get_val_s(iviscl, viscl)
 
 !===============================================================================
 ! 2.  SI IPROFM = 1 : CHOIX ENTREE/SORTIE SUIVANT LE PROFIL METEO SI
@@ -241,10 +241,12 @@ if (imbrication_flag) then
 endif ! imbrication_flag
 
 ! ==============================================================================
-! the interpollated field u_bord,v_bord,  .. will replace the values from the
+! the interpolated field u_bord,v_bord,  .. will replace the values from the
 ! standard meteo profile
 ! ==============================================================================
 
+call field_get_coefa_s(ivarfl(ipr), coefap)
+
 do ifac = 1, nfabor
 
   izone = izfppp(ifac)
@@ -304,74 +306,26 @@ do ifac = 1, nfabor
 
     vs = xuent*surfbo(1,ifac) + xvent*surfbo(2,ifac)
 
-    !     On met a jour le type de face de bord s'il n'a pas ete specifie
-    !       par l'utilisateur.
-    !     Pour une entree, on remplit la condition de Dirichlet si elle n'a pas
-    !     ete  specifiee par utilisateur.
+    ! On met a jour le type de face de bord s'il n'a pas ete specifie
+    !   par l'utilisateur.
+    ! Pour une entree, on remplit la condition de Dirichlet si elle n'a pas
+    ! ete  specifiee par utilisateur.
 
-    if (vs.gt.0) then
-      if (itypfb(ifac).eq.0) itypfb(ifac) = isolib
-    else
-      if (itypfb(ifac).eq.0) itypfb(ifac) = ientre
+    if (iautom(ifac).ge.1) then
+      if (vs.gt.0) then
+        itypfb(ifac) = isolib
+      else
+        if (itypfb(ifac).eq.0) itypfb(ifac) = ientre
+      endif
     endif
 
-    if (itypfb(ifac).eq.ientre) then
+    if (itypfb(ifac).eq.ientre.or.itypfb(ifac).eq.i_convective_inlet) then
 
-      if (rcodcl(ifac,iu,1).gt.rinfin*0.5d0)             &
-           rcodcl(ifac,iu,1) = xuent
-      if (rcodcl(ifac,iv,1).gt.rinfin*0.5d0)             &
-           rcodcl(ifac,iv,1) = xvent
-      if (rcodcl(ifac,iw,1).gt.rinfin*0.5d0)             &
-           rcodcl(ifac,iw,1) = 0.d0
+      if (rcodcl(ifac,iu,1).gt.rinfin*0.5d0) rcodcl(ifac,iu,1) = xuent
+      if (rcodcl(ifac,iv,1).gt.rinfin*0.5d0) rcodcl(ifac,iv,1) = xvent
+      if (rcodcl(ifac,iw,1).gt.rinfin*0.5d0) rcodcl(ifac,iw,1) = 0.d0
 
-      if    (itytur.eq.2) then
-
-        if (rcodcl(ifac,ik,1).gt.rinfin*0.5d0)           &
-             rcodcl(ifac,ik,1) = xkent
-        if (rcodcl(ifac,iep,1).gt.rinfin*0.5d0)          &
-             rcodcl(ifac,iep,1) = xeent
-
-      elseif(itytur.eq.3) then
-
-        if (rcodcl(ifac,ir11,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir11,1) = d2s3*xkent
-        if (rcodcl(ifac,ir22,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir22,1) = d2s3*xkent
-        if (rcodcl(ifac,ir33,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir33,1) = d2s3*xkent
-        if (rcodcl(ifac,ir12,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir12,1) = 0.d0
-        if (rcodcl(ifac,ir13,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir13,1) = 0.d0
-        if (rcodcl(ifac,ir23,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,ir23,1) = 0.d0
-        if (rcodcl(ifac,iep,1).gt.rinfin*0.5d0)          &
-             rcodcl(ifac,iep,1) = xeent
-
-      elseif(iturb.eq.50) then
-
-        if (rcodcl(ifac,ik,1).gt.rinfin*0.5d0)           &
-             rcodcl(ifac,ik,1) = xkent
-        if (rcodcl(ifac,iep,1).gt.rinfin*0.5d0)          &
-             rcodcl(ifac,iep,1) = xeent
-        if (rcodcl(ifac,iphi,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,iphi,1) = d2s3
-        if (rcodcl(ifac,ifb,1).gt.rinfin*0.5d0)          &
-             rcodcl(ifac,ifb,1) = 0.d0
-
-      elseif(iturb.eq.60) then
-
-        if (rcodcl(ifac,ik,1).gt.rinfin*0.5d0)           &
-             rcodcl(ifac,ik,1) = xkent
-        if (rcodcl(ifac,iomg,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,iomg,1) = xeent/cmu/xkent
-
-      elseif(iturb.eq.70) then
-
-        if (rcodcl(ifac,inusa,1).gt.rinfin*0.5d0)         &
-             rcodcl(ifac,inusa,1) = cmu*xkent**2/xeent
-
-      endif
+      call turbulence_bc_set_uninit_inlet_k_eps(ifac, xkent, xeent, rcodcl)
 
       if (iscalt.ne.-1) then
 
@@ -407,6 +361,74 @@ do ifac = 1, nfabor
 
     endif
 
+    if (iatmst.gt.0) then
+
+      if (iautom(ifac).ge.1) then
+
+        ! Dirichlet on the pressure: expressed in solved pressure directly
+        ! (not in total pressure), that is why -1 is used (transformed as 1 in typecl.f90).
+        icodcl(ifac,ipr) = -1
+        rcodcl(ifac,ipr,1) = coefap(ifac)
+
+        ! Dirichlet on turbulent variables
+        call turbulence_bc_set_uninit_inlet_k_eps(ifac, xkent, xeent, rcodcl)
+
+        if (iautom(ifac).eq.1) then
+
+          ! Homogeneous Neumann on the velocity
+          icodcl(ifac,iu) = 3
+          icodcl(ifac,iv) = 3
+          icodcl(ifac,iw) = 3
+          rcodcl(ifac,iu,3) = 0.d0
+          rcodcl(ifac,iv,3) = 0.d0
+          rcodcl(ifac,iw,3) = 0.d0
+
+        else if (iautom(ifac).eq.2) then
+
+          ! Dirichlet on the velocity
+          icodcl(ifac,iu) = 1
+          icodcl(ifac,iv) = 1
+          icodcl(ifac,iw) = 1
+          rcodcl(ifac,iu,1) = xuent
+          rcodcl(ifac,iv,1) = xvent
+          rcodcl(ifac,iw,1) = 0.d0
+
+        endif
+
+      endif
+
+    endif
+
+  else
+
+    if (itypfb(ifac).eq.ientre.or.itypfb(ifac).eq.i_convective_inlet) then
+
+      if (icalke(izone).eq.1) then
+        uref2 =   rcodcl(ifac,iu,1)**2                       &
+                + rcodcl(ifac,iv,1)**2                       &
+                + rcodcl(ifac,iw,1)**2
+        uref2 = max(uref2,epzero)
+        rhomoy = brom(ifac)
+        iel    = ifabor(ifac)
+        viscla = viscl(iel)
+        dhy    = dh(izone)
+        call turbulence_bc_inlet_hyd_diam(ifac,                            &
+                                          uref2, dhy, rhomoy, viscla,      &
+                                          rcodcl)
+      else if (icalke(izone).eq.2) then
+        uref2 =   rcodcl(ifac,iu,1)**2                       &
+                + rcodcl(ifac,iv,1)**2                       &
+                + rcodcl(ifac,iw,1)**2
+        uref2 = max(uref2,epzero)
+        dhy    = dh(izone)
+        xiturb = xintur(izone)
+        call turbulence_bc_inlet_turb_intensity(ifac,                      &
+                                                uref2, xiturb, dhy,        &
+                                                rcodcl)
+      endif
+
+    endif
+
   endif
 
 enddo
@@ -428,13 +450,13 @@ if (ifilechemistry.ge.1) then
     ! profiles are used here as boundary conditions if boundary conditions have
     ! not been treated earier (eg, in usatcl)
     do ii = 1, nespgi
-      if (rcodcl(ifac,isca(idespgi(ii)),1).gt.0.5d0*rinfin) then
+      if (rcodcl(ifac,isca(isca_chem(idespgi(ii))),1).gt.0.5d0*rinfin) then
         call intprf                                                    &
         !==========
         (nbchmz, nbchim,                                               &
         zproc, tchem, espnum(1+(ii-1)*nbchim*nbchmz), zent  , ttcabs, xcent )
         ! The first nespg user scalars are supposed to be chemical species
-        rcodcl(ifac,isca(idespgi(ii)),1) = xcent
+        rcodcl(ifac,isca(isca_chem(idespgi(ii))),1) = xcent
       endif
     enddo
 
@@ -443,7 +465,9 @@ if (ifilechemistry.ge.1) then
    ! For other species zero dirichlet conditions are imposed,
    ! unless they have already been treated earlier (eg, in usatcl)
    do ii =1 , nespg
-    if (rcodcl(ifac,isca(ii),1).gt.0.5d0*rinfin) rcodcl(ifac,isca(ii),1) = 0.0d0
+    if (rcodcl(ifac,isca(isca_chem(ii)),1).gt.0.5d0*rinfin) then
+      rcodcl(ifac,isca(isca_chem(ii)),1) = 0.0d0
+    endif
    enddo
 
   endif
@@ -501,16 +525,16 @@ endif
 ! clean up the 'imbrication'
 ! ---------------------------------
 if (imbrication_flag)then
-  if(cressman_u .and. ippmod(iatmos).ge.1 ) then
+  if(cressman_u) then
     deallocate(u_bord)
   endif
-  if(cressman_v .and. ippmod(iatmos).ge.1 ) then
+  if(cressman_v) then
     deallocate(v_bord)
   endif
-  if(cressman_tke .and. ippmod(iatmos).ge.1 ) then
+  if(cressman_tke) then
     deallocate(tke_bord)
   endif
-  if(cressman_eps .and. ippmod(iatmos).ge.1 ) then
+  if(cressman_eps) then
     deallocate(eps_bord)
   endif
   if(cressman_theta .and. ippmod(iatmos).ge.1 ) then
