@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,28 +20,23 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine atvarp
-!================
-
-
 !===============================================================================
-!  FONCTION  :
-!  ---------
+! Function:
+! ---------
 
-!    INIT DES POSITIONS DES VARIABLES POUR LE MODULE ATMOSPHERIQUE
-!    REMPLISSAGE DES PARAMETRES (DEJA DEFINIS) POUR LES SCALAIRES PP
+!> \file atvarp.f90
+!> \brief Declare additional transported variables for atmospheric module.
+!>
+!-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________!____!_____!________________________________________________!
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!_______________________________________________________________________________
 
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
+subroutine atvarp
 
 !===============================================================================
 ! Module files
@@ -73,9 +68,9 @@ implicit none
 
 integer        ii, jj, isc, f_id
 integer        kscmin, kscmax
-character*80   name, label
-character*2 :: nbin
-character*10, dimension(nesp_aer) :: f_esp_siream
+character(len=80) :: name, label
+character(len=2) :: nbin
+character(len=10), dimension(nesp_aer) :: f_esp_siream
 
 integer(c_int) :: n_chem_species
 integer(c_int), dimension(65) :: species_f_id
@@ -94,12 +89,6 @@ interface
     integer(c_int), dimension(*) :: species_f_id
   end subroutine cs_field_pointer_map_atmospheric
 
-  subroutine cs_gui_labels_atmospheric()  &
-    bind(C, name='cs_gui_labels_atmospheric')
-    use, intrinsic :: iso_c_binding
-    implicit none
-  end subroutine cs_gui_labels_atmospheric
-
 end interface
 
 !===============================================================================
@@ -116,16 +105,14 @@ call field_get_key_id("max_scalar_clipping", kscmax)
 
 if (iihmpr.eq.1) then
   call uiati1 (imeteo, ficmet, len(ficmet))
-  !==========
 endif
 
 ! Set base default model parameters and call usati1
 
 call atini0
-!==========
 
 !===============================================================================
-! 1. DEFINITION DES POINTEURS
+! 1. Add variables
 !===============================================================================
 
 ! 1.1  Dry atmosphere
@@ -192,10 +179,10 @@ if (ifilechemistry.ge.1) then
     allocate(isca_chem(nespg))
     allocate(dmmk(nespg))
     allocate(chempoint(nespg))
-    call add_model_scalar_field('chemistry_no',  'NO',   isca_chem(1))
-    call add_model_scalar_field('chemistry_no2', 'NO2',  isca_chem(2))
-    call add_model_scalar_field('chemistry_o3',  'NO3',  isca_chem(3))
-    call add_model_scalar_field('chemistry_o3p', 'NO3P', isca_chem(4))
+    call add_model_scalar_field('chemistry_no',  'NO',  isca_chem(1))
+    call add_model_scalar_field('chemistry_no2', 'NO2', isca_chem(2))
+    call add_model_scalar_field('chemistry_o3',  'O3',  isca_chem(3))
+    call add_model_scalar_field('chemistry_o3p', 'O3P', isca_chem(4))
     dmmk(1)=30.d-3 ! Molar mass NO
     dmmk(2)=46.d-3 ! Molar mass NO2
     dmmk(3)=48.d-3 ! Molar mass O3
@@ -210,8 +197,8 @@ if (ifilechemistry.ge.1) then
     allocate(chempoint(nespg))
     call add_model_scalar_field('species_no',    'NO',    isca_chem(1))
     call add_model_scalar_field('species_no2',   'NO2',   isca_chem(2))
-    call add_model_scalar_field('species_o3',    'NO3',   isca_chem(3))
-    call add_model_scalar_field('species_o3p',   'NO3P',  isca_chem(4))
+    call add_model_scalar_field('species_o3',    'O3',    isca_chem(3))
+    call add_model_scalar_field('species_o3p',   'O3P',   isca_chem(4))
     call add_model_scalar_field('species_o1d',   'O1D',   isca_chem(5))
     call add_model_scalar_field('species_oh',    'OH',    isca_chem(6))
     call add_model_scalar_field('species_ho2',   'HO2',   isca_chem(7))
@@ -221,7 +208,7 @@ if (ifilechemistry.ge.1) then
     call add_model_scalar_field('species_hono',  'HONO',  isca_chem(11))
     call add_model_scalar_field('species_hno3',  'HNO3',  isca_chem(12))
     call add_model_scalar_field('species_co',    'CO',    isca_chem(13))
-    call add_model_scalar_field('species_hco',   'HCO',   isca_chem(14))
+    call add_model_scalar_field('species_hcho',  'HCHO',  isca_chem(14))
     call add_model_scalar_field('species_ald2',  'ALD2',  isca_chem(15))
     call add_model_scalar_field('species_c2o3',  'C2O3',  isca_chem(16))
     call add_model_scalar_field('species_pan',   'PAN',   isca_chem(17))
@@ -265,8 +252,8 @@ if (ifilechemistry.ge.1) then
 
     call add_model_scalar_field('species_no',    'NO',    isca_chem(1))
     call add_model_scalar_field('species_no2',   'NO2',   isca_chem(2))
-    call add_model_scalar_field('species_o3',    'NO3',   isca_chem(3))
-    call add_model_scalar_field('species_o3p',   'NO3P',  isca_chem(4))
+    call add_model_scalar_field('species_o3',    'O3',    isca_chem(3))
+    call add_model_scalar_field('species_o3p',   'O3P',   isca_chem(4))
     call add_model_scalar_field('species_o1d',   'O1D',   isca_chem(5))
     call add_model_scalar_field('species_oh',    'OH',    isca_chem(6))
     call add_model_scalar_field('species_ho2',   'HO2',   isca_chem(7))
@@ -277,7 +264,7 @@ if (ifilechemistry.ge.1) then
     call add_model_scalar_field('species_hno3',  'HNO3',  isca_chem(12))
     call add_model_scalar_field('species_hno4',  'HNO4',  isca_chem(13))
     call add_model_scalar_field('species_co',    'CO',    isca_chem(14))
-    call add_model_scalar_field('species_hco',   'HCO',   isca_chem(15))
+    call add_model_scalar_field('species_hcho',  'HCHO',  isca_chem(15))
     call add_model_scalar_field('species_ald2',  'ALD2',  isca_chem(16))
     call add_model_scalar_field('species_c2o3',  'C2O3',  isca_chem(17))
     call add_model_scalar_field('species_pan',   'PAN',   isca_chem(18))
@@ -301,6 +288,7 @@ if (ifilechemistry.ge.1) then
     call add_model_scalar_field('species_ror',   'ROR',   isca_chem(36))
     call add_model_scalar_field('species_eth',   'ETH',   isca_chem(37))
     call add_model_scalar_field('species_ole',   'OLE',   isca_chem(38))
+    call add_model_scalar_field('species_iole',  'IOLE',  isca_chem(39))
     call add_model_scalar_field('species_isop',  'ISOP',  isca_chem(40))
     call add_model_scalar_field('species_ispd',  'ISPD',  isca_chem(41))
     call add_model_scalar_field('species_terp',  'TERP',  isca_chem(42))
@@ -473,25 +461,12 @@ enddo
 
 call cs_field_pointer_map_atmospheric(n_chem_species, species_f_id)
 
-if (iihmpr.eq.1) then
-  call cs_gui_labels_atmospheric
-endif
-
 !===============================================================================
 ! 5. General field and physical properties
 !===============================================================================
 
-do isc = 1, nscapp
-
-  if (iscavr(iscapp(isc)).le.0) then
-    ! Reference dynamic viscosity relative to this scalar
-    ivisls(iscapp(isc)) = 0
-  endif
-
-enddo
-
 ! Cp is constant
-icp = 0
+icp = -1
 
 !----
 ! Formats
