@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,16 +36,12 @@
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 !> \param[in]     iscal         scalar number
-!> \param[in]     rtpa          calculated variables at cell centers
-!>                               (preceding time steps)
 !> \param[out]    crvexp        explicit part of the source term
 !> \param[out]    crvimp        implicit part of the source term
 !_______________________________________________________________________________
 
 subroutine chem_source_terms &
-!================
-
- ( iscal  , rtpa   ,                                      &
+ ( iscal  ,                  &
    crvexp , crvimp)
 
 
@@ -57,7 +53,6 @@ use paramx
 use pointe
 use numvar
 use entsor
-use dimens, only: nvar
 use optcal
 use cstphy
 use parall
@@ -74,7 +69,6 @@ implicit none
 ! Arguments
 
 integer          iscal
-double precision rtpa(ncelet,nflown:nvar)
 double precision crvexp(ncelet), crvimp(ncelet)
 
 ! Local variables
@@ -87,11 +81,20 @@ double precision rom
 double precision conv_factor(nespg) ! conversion factors for reaction rates
 
 double precision, dimension(:), pointer :: crom
+type(pmapper_double_r1), dimension(:), allocatable :: cvara_espg
 
 !===============================================================================
 
+allocate(cvara_espg(nespg))
+
 call field_get_val_s(icrom, crom)
 
+! Arrays of pointers containing the fields values for each species
+! (loop on cells outside loop on species)
+do ii = 1, nespg
+  call field_get_val_prev_s(ivarfl(isca(isca_chem(ii))), cvara_espg(ii)%p)
+enddo
+
 do iel = 1, ncel
 
   ! density
@@ -104,7 +107,7 @@ do iel = 1, ncel
 
   ! Filling working arrays
   do ii = 1, nespg
-    dlconc(chempoint(ii)) = rtpa(iel,isca(ii))  ! rtpa
+    dlconc(chempoint(ii)) = cvara_espg(ii)%p(iel)
     conv_factor(chempoint(ii)) = rom*navo*(1.0d-12)/dmmk(ii)
     source(ii) = 0.0d0
   enddo
@@ -131,6 +134,8 @@ do iel = 1, ncel
 
 enddo
 
+deallocate(cvara_espg)
+
 !--------
 ! FORMATS
 !--------
