@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -35,15 +35,9 @@
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 !> \param[in]     dt            time step (per cell)
-!> \param[in]     rtpa          calculated variables at cell centers
-!>                               (preceding time steps)
-!> \param[in,out] rtp           calculated variables at cell centers
 !_______________________________________________________________________________
 
-subroutine compute_gaseous_chemistry &
-!================
-
-( dt     , rtpa   , rtp    )
+subroutine compute_gaseous_chemistry ( dt )
 
 !===============================================================================
 ! Module files
@@ -53,15 +47,14 @@ use paramx
 use numvar
 use entsor
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use parall
 use period
+use pointe
 use ppppar
 use ppthch
 use ppincl
-use elincl
 use mesh
 use field
 use dimens
@@ -74,7 +67,7 @@ implicit none
 
 ! Arguments
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
+double precision dt(ncelet)
 
 ! Local Variables
 
@@ -90,14 +83,23 @@ double precision rk(nrg)
 double precision dtc
 integer ncycle
 double precision dtrest
-double precision rvoid(1)
 
 double precision, dimension(:), pointer :: crom
+type(pmapper_double_r1), dimension(:), allocatable :: cvar_espg, cvara_espg
 
 !===============================================================================
 
+allocate(cvar_espg(nespg), cvara_espg(nespg))
+
 call field_get_val_s(icrom, crom)
 
+! Arrays of pointers containing the fields values for each species
+! (loop on cells outside loop on species)
+do ii = 1, nespg
+  call field_get_val_s(ivarfl(isca(isca_chem(ii))), cvar_espg(ii)%p)
+  call field_get_val_prev_s(ivarfl(isca(isca_chem(ii))), cvara_espg(ii)%p)
+enddo
+
 do iel = 1, ncel
 
   ! time step
@@ -121,9 +123,9 @@ do iel = 1, ncel
     ! -- splitted Rosenbrock solver
     ! -----------------------------
 
-    ! Filling working array dlconc with rtp
+    ! Filling working array dlconc with values at current time step
     do ii = 1, nespg
-      dlconc(chempoint(ii)) = rtp(iel,isca(ii))
+      dlconc(chempoint(ii)) = cvar_espg(ii)%p(iel)
     enddo
 
   else
@@ -131,9 +133,9 @@ do iel = 1, ncel
     ! -- semi-coupled Rosenbrock solver
     ! -----------------------------
 
-    ! Filling working array dlconc with rtpa
+    ! Filling working array dlconc with values at previous time step
     do ii = 1, nespg
-      dlconc(chempoint(ii)) = rtpa(iel,isca(ii))
+      dlconc(chempoint(ii)) = cvara_espg(ii)%p(iel)
     enddo
 
     ! Computation of C(Xn)
@@ -155,7 +157,7 @@ do iel = 1, ncel
     ! (X*-Xn)/dt(dynamics) - C(Xn). See usatch.f90
     ! The first nespg user scalars are supposed to be chemical species
     do ii = 1, nespg
-      source(chempoint(ii)) = (rtp(iel,isca(ii))-rtpa(iel,isca(ii)))/dtc  &
+      source(chempoint(ii)) = (cvar_espg(ii)%p(iel)-cvara_espg(ii)%p(iel))/dtc  &
                             - dchema(chempoint(ii))
     enddo
 
@@ -175,17 +177,18 @@ do iel = 1, ncel
     call roschem (dlconc,source,source,conv_factor,dtrest,rk,rk)
   endif
 
-  ! Update of rtp
+  ! Update of values at current time step
   do ii = 1, nespg
-    rtp(iel,isca(ii)) = dlconc(chempoint(ii))
+    cvar_espg(ii)%p(iel) = dlconc(chempoint(ii))
   enddo
 
 enddo
 
-! TODO: clipping or not ?
+deallocate(cvar_espg, cvara_espg)
+
 ! Clipping
 do ii = 1, nespg
-  call clpsca (ncelet, ncel, ii, rvoid, rtp)
+  call clpsca(isca_chem(ii))
 enddo
 
 !--------
