@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -31,14 +31,9 @@
 !   mode          name          role
 !------------------------------------------------------------------------------
 !> \param[in]     dt             time step (per cell)
-!> \param[in,out] rtp            calculated variables at cell centers at current time step
-!> \param[in]     propce         physical properties at cell centers
 !______________________________________________________________________________
 
-subroutine compute_siream &
-!================
-
- ( dt     , rtp,   propce)
+subroutine compute_siream ( dt )
 
 !===============================================================================
 ! Module files
@@ -48,15 +43,14 @@ use paramx
 use numvar
 use entsor
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use parall
 use period
+use pointe
 use ppppar
 use ppthch
 use ppincl
-use elincl
 use mesh
 use field
 use dimens
@@ -70,13 +64,11 @@ implicit none
 
 ! Arguments
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
+double precision dt(ncelet)
 
 ! Local Variables
 
-integer ii, jb, jsp, iel
-double precision rvoid(1)
+integer ii, jb, jsp, iel, index
 double precision dlconc(nespg_siream)
 double precision dlconc_aer(nbin_aer,nesp_aer)
 double precision dlnum_aer(nbin_aer)
@@ -84,6 +76,11 @@ double precision dltemp, dlpress, dlhumid, dldens
 double precision zent                               ! Z coordinate of a cell
 
 double precision, dimension(:), pointer :: crom
+type(pmapper_double_r1), dimension(:), allocatable :: cvar_espg
+type(pmapper_double_r1), dimension(:), allocatable :: cvar_aer
+type(pmapper_double_r1), dimension(:), allocatable :: cvar_naer
+double precision, dimension(:), pointer :: cvar_totwt
+double precision, dimension(:), pointer :: cpro_tempc, cpro_liqwt
 
 ! Initialisation
 
@@ -92,41 +89,67 @@ dldens = ro0
 dlpress = dldens*rair*dltemp ! ideal gas law
 dlhumid = 0.0d0
 
+allocate(cvar_espg(nespg_siream))
+allocate(cvar_aer(nesp_aer*nbin_aer))
+allocate(cvar_naer(nbin_aer))
+
 ! Get Physical quantities
 ! Densisty
 call field_get_val_s(icrom, crom)
 
+if (ippmod(iatmos).ge.1) call field_get_val_s(itempc, cpro_tempc)
+
+if (ippmod(iatmos).ge.2) then
+  call field_get_val_s(ivarfl(isca(itotwt)), cvar_totwt)
+  call field_get_val_s(iliqwt, cpro_liqwt)
+endif
+
+! Arrays of pointers containing the fields values for each species
+! (loop on cells outside loop on species)
+do jsp = 1, nespg_siream
+  call field_get_val_s(ivarfl(isca(jsp)), cvar_espg(jsp)%p)
+enddo
+do jb = 1, nbin_aer
+  do jsp = 1, nesp_aer
+    index = nespg_siream+jb+(jsp-1)*nbin_aer
+    call field_get_val_s(ivarfl(isca(index)), cvar_aer(jb+(jsp-1)*nbin_aer)%p)
+  enddo
+enddo
+do jb = 1, nbin_aer
+  index = nespg_siream+nesp_aer*nbin_aer+jb
+  call field_get_val_s(ivarfl(isca(index)), cvar_naer(jb)%p)
+enddo
+
 ! Clipping before calling siream
 
 do ii = 1, nespg_siream+nesp_aer*nbin_aer
-  call clpsca(ncelet, ncel, ii, rvoid, rtp)
+  call clpsca(ii)
 enddo
 
 ! Loop on cells
 do iel = 1, ncel
 
   zent = xyzcen(3,iel)
-  ! Verifier l'ordre dans lequel on rentre rtp, dlconc et dlconc_aer
 
   ! Filling working arrays
   do jsp = 1, nespg_siream
-    dlconc(jsp) = rtp(iel,isca(jsp))
+    dlconc(jsp) = cvar_espg(jsp)%p(iel)
   enddo
 
   do jb = 1, nbin_aer
     do jsp = 1, nesp_aer
-      dlconc_aer(jb,jsp) = rtp(iel,isca(nespg_siream+jb+(jsp-1)*nbin_aer))
+      dlconc_aer(jb,jsp) = cvar_aer(jb+(jsp-1)*nbin_aer)%p(iel)
     enddo
   enddo
 
   do jb = 1, nbin_aer
-    dlnum_aer(jb) = rtp(iel,isca(nespg_siream+nesp_aer*nbin_aer+jb))
+    dlnum_aer(jb) = cvar_naer(jb)%p(iel)
   enddo
 
   ! Temperature and density
   ! Dry or humid atmosphere
   if (ippmod(iatmos).ge.1) then
-    dltemp = propce(iel,ipproc(itempc)) + tkelvi
+    dltemp = cpro_tempc(iel) + tkelvi
     dldens = crom(iel)
     dlpress = dldens*rair*dltemp
 
@@ -150,8 +173,8 @@ do iel = 1, ncel
   ! Specific hymidity
   ! Humid atmosphere
   if (ippmod(iatmos).ge.2) then
-    dlhumid = (rtp(iel,isca(itotwt))-propce(iel,ipproc(iliqwt)))   &
-              /(1.d0-propce(iel,ipproc(iliqwt)))
+    dlhumid = (cvar_totwt(iel)-cpro_liqwt(iel))   &
+              /(1.d0-cpro_liqwt(iel))
 
   ! Constant density or dry atmosphere with a meteo file
   else if (imeteo.eq.1) then
@@ -187,27 +210,29 @@ do iel = 1, ncel
        couples_coag, first_index_coag, second_index_coag,               &
        coefficient_coag, dlconc_aer, dlnum_aer)
 
-  ! Updating rtp
+  ! Updating arrays
   do jsp = 1, nespg_siream
-    rtp(iel,isca(jsp)) = dlconc(jsp)
+    cvar_espg(jsp)%p(iel) = dlconc(jsp)
   enddo
 
   do jb = 1, nbin_aer
     do jsp = 1, nesp_aer
-      rtp(iel,isca(nespg_siream+jb+(jsp-1)*nbin_aer)) = dlconc_aer(jb,jsp)
+      cvar_aer(jb+(jsp-1)*nbin_aer)%p(iel) = dlconc_aer(jb,jsp)
     enddo
   enddo
 
   do jb = 1, nbin_aer
-    rtp(iel,isca(nespg_siream+nesp_aer*nbin_aer+jb)) = dlnum_aer(jb)
+    cvar_naer(jb)%p(iel) = dlnum_aer(jb)
   enddo
 
 enddo ! Loop on cells
 
+deallocate(cvar_espg, cvar_aer, cvar_naer)
+
 ! Clipping after calling Siream
 
 do ii = 1, nespg_siream+nesp_aer*nbin_aer
-  call clpsca(ncelet, ncel, ii, rvoid, rtp)
+  call clpsca(ii)
 enddo
 
 !--------
