@@ -4,7 +4,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -31,6 +31,13 @@
 
 BEGIN_C_DECLS
 
+/*!
+  \file cs_at_plugin.h
+
+  \brief Plugin to get aerosol and compute_coagulation_coefficient functions
+    from SIREAM library (ENPC - INRIA - EDF R&D)
+*/
+
 /*============================================================================
  * Macro definitions
  *============================================================================*/
