@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,43 +19,25 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-
-subroutine intprf &
-!================
-
- ( nprofz , nproft ,                                                            &
-   profz  , proft  , profv  , xz     , temps  , var    )
-
-!===============================================================================
-!  Purpose:
-!  ---------
-
-!      Temporal and z-axis optimized interpolation
-
+!> \file intprf.f90
+!>
+!> \brief Temporal and z-axis interpolation for meteorological profiles
+!> An optimized linear interpolation is used.
 !-------------------------------------------------------------------------------
-! arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! nprofz           ! i  ! <-- ! total number of z-measure points               !
-! nproft           ! i  ! <-- ! total number of time dataset                   !
-! profz            ! tr ! <-- ! z coordonate of measure points                 !
-! proft            ! tr ! <-- ! physical time of dataset acquisition           !
-! profv            ! tr ! <-- ! measured values                                !
-! xz               ! tr ! <-- ! interpolation elevation                        !
-! temps            ! tr ! <-- ! interpolation time                             !
-! var              ! r  ! --> ! interpolation result                           !
-!__________________!____!_____!________________________________________________!
-
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-
-!===============================================================================
-! Module files
-!===============================================================================
-
-!===============================================================================
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]   nprofz      total number of z-measure points
+!> \param[in]   nproft      total number of time dataset
+!> \param[in]   profz       z coordonate of measure points
+!> \param[in]   proft       physical time of dataset acquisition
+!> \param[in]   profv       measured values
+!> \param[in]   xz          interpolation elevation
+!> \param[in]   temps       interpolation time
+!> \param[out]  var         interpolation result
+!-------------------------------------------------------------------------------
+subroutine intprf ( nprofz, nproft, profz, proft, profv, xz, temps, var )
 
 implicit none
 
@@ -72,7 +54,6 @@ integer          it, it1, it2
 integer          iz, iz1, iz2
 double precision alphaz, alphat, var1, var2
 
-!===============================================================================
 
 !===============================================================================
 ! 1. Time interpolation
@@ -86,7 +67,7 @@ else if (temps.ge.proft(nproft)) then
   it1 = nproft
   it2 = nproft
   alphat = 1.d0
-else !     else nproft > 1
+else ! else nproft > 1
   it = 1
  102    continue
   if (temps.gt.proft(it+1)) then
@@ -111,7 +92,7 @@ else if (xz.ge.profz(nprofz)) then
   iz1 = nprofz
   iz2 = nprofz
   alphaz = 1.d0
-else ! sinon on a forcement nprofz > 1
+else ! else nprofz > 1
   iz = 1
  103    continue
   if (xz.gt.profz(iz+1)) then
@@ -134,7 +115,3 @@ var2 = alphaz*profv(iz1,it2) + (1.d0-alphaz)*profv(iz2,it2)
 var = alphat*var1 + (1.d0 - alphat)*var2
 
 end subroutine intprf
-
-
-
-
