@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,22 +24,9 @@
 
 !> \file kinrates.f90
 !> \brief Calls the computation of reaction rates for atmospheric chemistry
-!
-!-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
-! Arguments
-!______________________________________________________________________________.
-!  mode           name          role                                           !
-!______________________________________________________________________________!
-!> \param[in]     rtp           calculated variables at cell centers
-!> \param[in]     propce        physical properties at cell centers
-!_______________________________________________________________________________
-
-subroutine kinrates &
-!================
-
- (rtp    , propce)
+subroutine kinrates ()
 
 !===============================================================================
 ! Module files
@@ -49,7 +36,6 @@ use paramx
 use numvar
 use entsor
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use parall
@@ -67,11 +53,6 @@ implicit none
 
 !===============================================================================
 
-! Arguments
-
-double precision rtp(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
-
 ! Local variables
 
 integer iel,ii
@@ -86,7 +67,9 @@ double precision heurtu              ! yime (UTC)
 double precision albe                ! albedo, useless here
 double precision fo                  ! solar constant, useless here
 
-double precision, dimension(:), pointer ::  crom
+double precision, dimension(:), pointer :: crom
+double precision, dimension(:), pointer :: cvar_totwt
+double precision, dimension(:), pointer :: cpro_tempc, cpro_liqwt
 
 ! Initialisation
 
@@ -95,7 +78,15 @@ dens = ro0
 press = dens*rair*temp ! ideal gas law
 hspec = 0.0d0
 
-if (ippmod(iatmos).ge.1) call field_get_val_s(icrom, crom)
+if (ippmod(iatmos).ge.1) then
+  call field_get_val_s(icrom, crom)
+  call field_get_val_s(itempc, cpro_tempc)
+endif
+
+if (ippmod(iatmos).ge.2) then
+  call field_get_val_s(ivarfl(isca(itotwt)), cvar_totwt)
+  call field_get_val_s(iliqwt, cpro_liqwt)
+endif
 
 ! Computation of kinetic rates in every cell
 
@@ -117,7 +108,7 @@ do iel = 1, ncel
   ! Temperature and density
   ! Dry or humid atmosphere
   if (ippmod(iatmos).ge.1) then
-    temp = propce(iel,ipproc(itempc)) + tkelvi
+    temp = cpro_tempc(iel) + tkelvi
     dens = crom(iel)
     press = dens*rair*temp
 
@@ -141,8 +132,8 @@ do iel = 1, ncel
   ! Specific hymidity
   ! Humid atmosphere
   if (ippmod(iatmos).ge.2) then
-    hspec = (rtp(iel,isca(itotwt))-propce(iel,ipproc(iliqwt)))    &
-          / (1.d0-propce(iel,ipproc(iliqwt)))
+    hspec = (cvar_totwt(iel)-cpro_liqwt(iel))    &
+          / (1.d0-cpro_liqwt(iel))
 
   ! Constant density or dry atmosphere with a meteo file
   else if (imeteo.eq.1) then
