@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -68,12 +68,12 @@ parameter (noptions_aer = 14)
 integer, save :: options_aer(noptions_aer)
 
 !> Names of particular species in SIREAM
-character*10, dimension(nesp_aer) :: esp_siream
+character(len=10), dimension(nesp_aer) :: esp_siream
 
 !> logical unit of the aerosol concentration profiles file
 integer, save         ::  impmea
 !> name of the aerosol concentration profiles file
-character*10, save    ::  ficmea
+character(len=10), save    ::  ficmea
 
 !> Aerosol diameters at bin bounds
 double precision, save :: bin_bound_aer(nbin_aer + 1)
