@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -46,7 +46,7 @@
 !> \param[in,out] viscb         work array
 !> \param[in,out] dam           work array
 !> \param[in,out] xam           work array
-!> \param[in,out] drtp          work array
+!> \param[in,out] dpvar         work array
 !> \param[in,out] smbr          work array
 !______________________________________________________________________________
 
@@ -58,12 +58,13 @@ subroutine calhyd &
    cofafp , cofbfp ,                                              &
    viscf  , viscb  ,                                              &
    dam    , xam    ,                                              &
-   drtp   , smbr   )
+   dpvar  , smbr   )
 
 !===============================================================================
 ! Module files
 !===============================================================================
 
+use atincl, only: iatmst
 use paramx
 use numvar
 use entsor
@@ -71,8 +72,8 @@ use cstnum
 use optcal
 use parall
 use period
-use mltgrd
 use mesh
+use cs_c_bindings
 
 !===============================================================================
 
@@ -90,25 +91,22 @@ double precision coefap(nfabor), coefbp(nfabor)
 double precision cofafp(nfabor), cofbfp(nfabor)
 double precision viscf(nfac), viscb(nfabor)
 double precision dam(ncelet), xam(nfac)
-double precision drtp(ncelet)
+double precision dpvar(ncelet)
 double precision smbr(ncelet)
 
 ! Local variables
 
-character*80     chaine
+character(len=80) :: chaine
 integer          lchain
-integer          iccocg, inc   , init  , isym  , ipol  , isqrt
+integer          f_id, iccocg, inc   , init  , isym
 integer          iel   , ical
-integer          ireslp, nswmpr
-integer          isweep, niterf, icycle
+integer          nswmpr
+integer          isweep, niterf
 integer          iphydp
 integer          nswrgp, imligp, iwarnp
-integer          iinvpe
 integer          idiffp, iconvp, ndircp
-integer          nitmap, imgrp , ncymap, nitmgp
-integer          nagmax, npstmg
 integer          ibsize, iesize
-integer          imucpp
+integer          imucpp, f_id0
 
 double precision residu, rnorm , rnrmf , rnrmdf
 double precision epsrgp, climgp, extrap, epsilp
@@ -116,35 +114,32 @@ double precision precre, precab, thetap
 
 double precision rvoid(1)
 
-double precision, allocatable, dimension(:) :: w1, w7, w10
+double precision, allocatable, dimension(:) :: rovsdt, div_dfext, viscce
+
+type(var_cal_opt) :: vcopt_u, vcopt_pr
 
 !===============================================================================
 
 !===============================================================================
-! 1.  INITIALISATIONS
+! 1.  Initialisations
 !===============================================================================
 
 ! Allocate temporary arrays
-allocate(w1(ncelet), w7(ncelet), w10(ncelet))
+allocate(rovsdt(ncelet), div_dfext(ncelet), viscce(ncelet))
 
-! --- Variable name
+! Get variables calculation options
 
-chaine = 'PresHydr'
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_pr)
+
+! Variable name
+
+f_id = -1
+chaine = 'hydrostatic_p'
 lchain = 16
 
-! --- Options de resolution
-!     Symetrique
-!     Preconditionnement diagonal par defaut
+! Symetric
 isym  = 1
-if (iresol(ipr).eq.-1) then
-  ireslp = 0
-  ipol   = 0
-else
-  ireslp = mod(iresol(ipr)+10000,1000)
-  ipol   = (iresol(ipr)-ireslp)/1000
-endif
-
-isqrt = 1
 
 ! Matrix block size
 ibsize = 1
@@ -170,16 +165,18 @@ enddo
 if (irangp.ge.0) then
   call parcpt (ical)
 endif
-if (ical.eq.0) then
-  do iel = 1,ncel
-    phydr(iel) = 0.d0
-  enddo
-  indhyd = 0
-  return
+
+if (iatmst.eq.0) then
+  if (ical.eq.0) then
+    do iel = 1,ncel
+      phydr(iel) = 0.d0
+    enddo
+    indhyd = 0
+    return
+  endif
 endif
 
-if ( mod(ntcabs,ntlist).eq.0 .or. iwarni(iu) .ge.0 )           &
-     write(nfecra,1000)
+if (mod(ntcabs,ntlist).eq.0.or.vcopt_u%iwarni.ge.0) write(nfecra,1000)
 
 #if defined(_CS_LANG_FR)
 
@@ -197,6 +194,8 @@ if ( mod(ntcabs,ntlist).eq.0 .or. iwarni(iu) .ge.0 )           &
 
 indhyd = 1
 
+f_id0 = -1
+
 !===============================================================================
 ! 2.  PREPARATION DE LA MATRICE DU SYSTEME A RESOUDRE
 !===============================================================================
@@ -204,19 +203,19 @@ indhyd = 1
 ! ---> TERME INSTATIONNAIRE
 
 do iel = 1, ncel
-  w1(iel) = 0.d0
+  rovsdt(iel) = 0.d0
 enddo
 
 ! ---> "VITESSE" DE DIFFUSION FACETTE
 
 do iel = 1, ncel
-  w10(iel) = 1.d0
+  viscce(iel) = 1.d0
 enddo
 
 call viscfa                                                       &
 !==========
  ( imvisf ,                                                       &
-   w10    ,                                                       &
+   viscce ,                                                       &
    viscf  , viscb  )
 
 
@@ -232,24 +231,25 @@ call matrix &
 !==========
  ( iconvp , idiffp , ndircp , isym ,                              &
    thetap , imucpp ,                                              &
-   coefbp , cofbfp , w1     ,                                     &
+   coefbp , cofbfp , rovsdt ,                                     &
    flumas , flumab , viscf  , viscb  ,                            &
    rvoid  , dam    , xam    )
 
 !===============================================================================
-! 4.  INITIALISATION DU FLUX DE MASSE
+! 3.  INITIALISATION DU FLUX DE MASSE
 !===============================================================================
 
 
 !     PROJECTION AUX FACES DES TERMES SOURCES
+
 init   = 1
 inc    = 0
 iccocg = 1
-nswrgp = nswrgr(ipr)
-imligp = imligr(ipr)
-iwarnp = iwarni(ipr)
-epsrgp = epsrgr(ipr)
-climgp = climgr(ipr)
+nswrgp = vcopt_pr%nswrgr
+imligp = vcopt_pr%imligr
+iwarnp = vcopt_pr%iwarni
+epsrgp = vcopt_pr%epsrgr
+climgp = vcopt_pr%climgr
 
 call projts                                                       &
 !==========
@@ -258,49 +258,26 @@ call projts                                                       &
    cofbfp ,                                                       &
    flumas, flumab ,                                               &
    viscf  , viscb  ,                                              &
-   w10    , w10    , w10    )
+   viscce , viscce , viscce    )
 
 init = 1
-call divmas(init,flumas,flumab,w7)
-call prodsc(ncel,isqrt,w7,w7,rnorm)
-
+call divmas(init,flumas,flumab,div_dfext)
+rnorm = sqrt(cs_gdot(ncel,div_dfext,div_dfext))
 
 !===============================================================================
-! 5.  PREPARATION DU MULTIGRILLE ALGEBRIQUE
-!===============================================================================
-
-if (imgr(ipr).gt.0) then
-
-!   --- Creation de la hierarchie de maillages
-
-  iwarnp = iwarni(ipr)
-  nagmax = nagmx0(ipr)
-  npstmg = ncpmgr(ipr)
-
-  call clmlga &
-  !==========
- ( chaine(1:16) ,    lchain ,                                     &
-   isym   , ibsize , iesize , nagmax , npstmg , iwarnp ,          &
-   ngrmax , ncegrm ,                                              &
-   rlxp1  ,                                                       &
-   dam    , xam    )
-
-endif
-
-!===============================================================================
-! 6.  BOUCLES SUR LES NON ORTHOGONALITES (RESOLUTION)
+! 4.  BOUCLES SUR LES NON ORTHOGONALITES (RESOLUTION)
 !===============================================================================
 
 ! --- Nombre de sweeps
-nswmpr = nswrsm(ipr)
+nswmpr = vcopt_pr%nswrsm
 
 ! --- Mise a zero des variables
-!       RTP(.,IPR) sera l'increment de pression cumule
-!       DRTP       sera l'increment d'increment a chaque sweep
-!       W7         sera la divergence du flux de masse predit
+!       phydr      sera l'increment de pression cumule
+!       dpvar      sera l'increment d'increment a chaque sweep
+!       div_dfext         sera la divergence du flux de masse predit
 do iel = 1,ncel
   phydr(iel) = 0.d0
-  drtp(iel) = 0.d0
+  dpvar(iel) = 0.d0
   smbr(iel) = 0.d0
 enddo
 
@@ -311,54 +288,42 @@ do isweep = 1, nswmpr
 ! --- Mise a jour du second membre
 !     (signe "-" a cause de celui qui est implicitement dans la matrice)
   do iel = 1, ncel
-    smbr(iel) = - w7(iel) - smbr(iel)
+    smbr(iel) = - div_dfext(iel) - smbr(iel)
   enddo
 
 ! --- Test de convergence du calcul
 
-  call prodsc(ncel,isqrt,smbr,smbr,residu)
-  if (iwarni(ipr).ge.2) then
-     write(nfecra,1400)chaine(1:16),isweep,residu
+  residu = sqrt(cs_gdot(ncel,smbr,smbr))
+  if (vcopt_pr%iwarni.ge.2) then
+    write(nfecra,1400)chaine(1:16),isweep,residu
   endif
 
-!MO IL FAUDRA VERIFIER LA PERTINENCE DU TEST
-
-  if( residu .le. 10.d0*epsrsm(ipr)*rnorm ) then
+  ! FIXME coherent with resopv!
+  if (residu.le.10.d0*vcopt_pr%epsrsm*rnorm) then
 !     Si convergence,  sortie
 
     goto 101
 
   endif
 
-! --- Resolution implicite sur l'increment d'increment DRTP
+! --- Resolution implicite sur l'increment d'increment DPVAR
   do iel = 1, ncel
-    drtp(iel) = 0.d0
+    dpvar(iel) = 0.d0
   enddo
 
-  nitmap = nitmax(ipr)
-  imgrp  = imgr  (ipr)
-  ncymap = ncymax(ipr)
-  nitmgp = nitmgf(ipr)
-  iwarnp = iwarni(ipr)
-  epsilp = epsilo(ipr)
-  iinvpe = 1
+  iwarnp = vcopt_pr%iwarni
+  epsilp = vcopt_pr%epsilo
   ibsize = 1
   iesize = 1
 
-  call invers &
-  !==========
- ( chaine(1:16)    , isym   , ibsize , iesize ,                   &
-   ipol   , ireslp , nitmap , imgrp  ,                            &
-   ncymap , nitmgp ,                                              &
-   iwarnp , niterf , icycle , iinvpe ,                            &
-   epsilp , rnorm  , residu ,                                     &
-   dam    , xam    , smbr   , drtp   )
+  call sles_solve_native(f_id, chaine,                            &
+                         isym, ibsize, iesize, dam, xam,          &
+                         epsilp, rnorm, niterf, residu, smbr, dpvar)
 
-
-  if( isweep.eq.nswmpr ) then
+  if (isweep.eq.nswmpr ) then
 !     Mise a jour de l'increment de pression
     do iel = 1, ncel
-      phydr(iel) = phydr(iel) + drtp(iel)
+      phydr(iel) = phydr(iel) + dpvar(iel)
     enddo
 
 
@@ -370,55 +335,53 @@ do isweep = 1, nswmpr
 !       (avec reconstruction)
 
     do iel = 1, ncel
-      phydr(iel) = phydr(iel) + drtp(iel)
+      phydr(iel) = phydr(iel) + dpvar(iel)
     enddo
 
     iccocg = 1
     init = 1
     inc  = 1
-    nswrgp = nswrgr(ipr)
-    imligp = imligr(ipr)
-    iwarnp = iwarni(ipr)
-    epsrgp = epsrgr(ipr)
-    climgp = climgr(ipr)
+    nswrgp = vcopt_pr%nswrgr
+    imligp = vcopt_pr%imligr
+    iwarnp = vcopt_pr%iwarni
+    epsrgp = vcopt_pr%epsrgr
+    climgp = vcopt_pr%climgr
     extrap = 0.d0
     iphydp = 1
 
     call itrgrp &
     !==========
- ( init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp , &
-   iwarnp ,                                                       &
-   epsrgp , climgp , extrap ,                                     &
-   dfext  ,                                                       &
-   phydr  ,                                                       &
-   coefap , coefbp ,                                              &
-   cofafp , cofbfp ,                                              &
-   viscf  , viscb  ,                                              &
-   w10         , w10         , w10         ,                      &
+ ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp ,     &
+   iwarnp ,                                                                    &
+   epsrgp , climgp , extrap ,                                                  &
+   dfext  ,                                                                    &
+   phydr  ,                                                                    &
+   coefap , coefbp ,                                                           &
+   cofafp , cofbfp ,                                                           &
+   viscf  , viscb  ,                                                           &
+   viscce ,                                                                    &
    smbr   )
 
   endif
 
 enddo
+
 ! --- Boucle de reconstruction : fin
 
-if(iwarni(ipr).ge.2) then
+if(vcopt_pr%iwarni.ge.2) then
    write( nfecra,1600)chaine(1:16),nswmpr
 endif
 
  101  continue
 
 ! Free memory
-deallocate(w1, w7, w10)
+deallocate(rovsdt, div_dfext, viscce)
 
 !===============================================================================
-! 7.  SUPPRESSION DE LA HIERARCHIE DE MAILLAGES
+! 5. Free solver setup
 !===============================================================================
 
-if (imgr(ipr).gt.0) then
-  call dsmlga(chaine(1:16), lchain)
-  !==========
-endif
+call sles_free_native(-1, chaine)
 
 !--------
 ! FORMATS
