@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,18 +36,15 @@
 !> \param[in]     iterns        iteration number on Navier-Stoke
 !> \param[in]     isnexp        sources terms of treated phasis extrapolation
 !>                              indicator
-!> \param[in]     thetv         theta scheme for the variable
-!>                              \f$ \varia^\theta = \theta \varia^{n+1}
-!>                                                + (1-\theta)\varia^n \f$
 !> \param[in]     icetsm        source mass cells pointer
 !> \param[in]     itpsmp        mass source type for the working variable
-!>                              (see \ref ustsma)
+!>                              (see \ref cs_user_mass_source_terms)
 !> \param[in]     volume        cells volume
 !> \param[in]     pvara         variable value at time step beginning
 !> \param[in]     smcelp        value of the variable associated with mass source
 !> \param[in]     gamma         flow mass value
 !> \param[in,out] tsexp         explicit source term part linear in the variable
-!> \param[in,out] tsimp         associated value withr \ref tsexp
+!> \param[in,out] tsimp         associated value with \c tsexp
 !>                              to be stored in the matrix
 !> \param[out]    gapinj        explicit source term part independant
 !>                              of the variable
@@ -55,7 +52,6 @@
 
 subroutine catsma &
  ( ncelet , ncel   , ncesmp , iterns , isnexp ,                   &
-   thetv  ,                                                       &
    icetsm , itpsmp ,                                              &
    volume , pvara  , smcelp , gamma  ,                            &
    tsexp  , tsimp  , gapinj )
@@ -70,7 +66,6 @@ implicit none
 
 integer          ncelet, ncel  , ncesmp, iterns, isnexp
 integer          icetsm(ncesmp), itpsmp(ncesmp)
-double precision thetv
 double precision volume(ncelet)
 double precision pvara (ncelet)
 double precision smcelp(ncesmp), gamma (ncesmp)
@@ -88,7 +83,7 @@ integer ii, iel
 !     Sinon, on ajoute le terme source GAMMA*(f_i-f^(n+1))
 
 !     Dans TSIMP, on ajoute le terme qui ira sur la diagonale,
-!       soit Gamma (*theta en cas d'ordre 2)
+!       soit Gamma
 !     Dans TSEXP on ajoute le terme correspondant du second membre
 !       cad Gamma * Pvar (avec Pvar)
 !     Dans GAPINJ on place le terme Gamma Pinj qui ira au second membre
@@ -114,7 +109,7 @@ if(isnexp.gt.0) then
   do ii = 1, ncesmp
     iel = icetsm(ii)
     if (gamma(ii).gt.0.d0 .and. itpsmp(ii).eq.1) then
-      tsimp(iel) = tsimp(iel) + volume(iel)*gamma(ii)*thetv
+      tsimp(iel) = tsimp(iel) + volume(iel)*gamma(ii)
     endif
   enddo
 else
