@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,19 +36,16 @@
 !> \param[in]     iterns        Navier-Stokes iteration number
 !> \param[in]     isnexp        sources terms of treated phase extrapolation
 !>                              indicator
-!> \param[in]     thetv         theta scheme for the variable
-!>                              \f$ \varia^\theta = \theta \varia^{n+1}
-!>                                                + (1-\theta)\varia^n \f$
 !> \param[in]     icetsm        source mass cells pointer
 !> \param[in]     itpsmp        mass source type for the working variable
-!>                              (cf. \ref ustsma)
-!> \param[in]     volume        cells volume
+!>                              (cf. \ref cs_user_mass_source_terms)
+!> \param[in]     cell_f_vol    cells fluid volume
 !> \param[in]     vela          variable value at time step beginning
-!> \param[in]     smcelv        valeur de la variable associee a la
-!>                              source de masse NOT INTERLEAVED
+!> \param[in]     smcelv        value of the variable associated to the mass
+!>                              source; NOT INTERLEAVED
 !> \param[in]     gamma         mass flow value
 !> \param[in,out] tsexpv        explicit source term part linear in the variable
-!> \param[in,out] tsimpv        associated value withr \ref tsexp
+!> \param[in,out] tsimpv        associated value with \c tsexp
 !>                              to be stored in the matrix
 !> \param[out]    gavinj        explicit source term part independant
 !>                              of the variable
@@ -56,9 +53,8 @@
 
 subroutine catsmv &
  ( ncelet , ncel   , ncesmp , iterns , isnexp ,                   &
-   thetv  ,                                                       &
    icetsm , itpsmp ,                                              &
-   volume , vela   , smcelv , gamma  ,                            &
+   cell_f_vol      , vela   , smcelv , gamma  ,                   &
    tsexpv , tsimpv , gavinj )
 
 !===============================================================================
@@ -71,8 +67,7 @@ implicit none
 
 integer          ncelet, ncel  , ncesmp, iterns, isnexp
 integer          icetsm(ncesmp), itpsmp(ncesmp)
-double precision thetv
-double precision volume(ncelet)
+double precision cell_f_vol(ncelet)
 double precision vela  (3,ncelet)
 double precision smcelv(ncesmp,3), gamma (ncesmp)
 double precision tsexpv(3,ncelet), tsimpv(3,3,ncelet), gavinj(3,ncelet)
@@ -89,7 +84,7 @@ integer ii, iel, isou
 !     Sinon, on ajoute le terme source GAMMA*(f_i-f^(n+1))
 
 !     Dans TSIMP, on ajoute le terme qui ira sur la diagonale,
-!       soit Gamma (*theta en cas d'ordre 2)
+!       soit Gamma
 !     Dans TSEXP on ajoute le terme correspondant du second membre
 !       cad Gamma * Pvar (avec Pvar)
 !     Dans GAPINJ on place le terme Gamma Pinj qui ira au second membre
@@ -107,8 +102,8 @@ if(iterns.eq.1) then
     iel = icetsm(ii)
     if (gamma(ii).gt.0.d0 .and. itpsmp(ii).eq.1) then
       do isou = 1, 3
-        tsexpv(isou,iel) = tsexpv(isou,iel)-volume(iel)*gamma(ii)*vela(isou,iel)
-        gavinj(isou,iel) = volume(iel)*gamma(ii) * smcelv(ii,isou)
+        tsexpv(isou,iel) = tsexpv(isou,iel)-cell_f_vol(iel)*gamma(ii)*vela(isou,iel)
+        gavinj(isou,iel) = cell_f_vol(iel)*gamma(ii) * smcelv(ii,isou)
       enddo
     endif
   enddo
@@ -120,7 +115,7 @@ if(isnexp.gt.0) then
     iel = icetsm(ii)
     if (gamma(ii).gt.0.d0 .and. itpsmp(ii).eq.1) then
       do isou = 1, 3
-        tsimpv(isou,isou,iel) = tsimpv(isou,isou,iel)+volume(iel)*gamma(ii)*thetv
+        tsimpv(isou,isou,iel) = tsimpv(isou,isou,iel)+cell_f_vol(iel)*gamma(ii)
       enddo
     endif
   enddo
@@ -129,7 +124,7 @@ else
     iel = icetsm(ii)
     if (gamma(ii).gt.0.d0 .and. itpsmp(ii).eq.1) then
       do isou = 1, 3
-        tsimpv(isou,isou,iel) = tsimpv(isou,isou,iel)+volume(iel)*gamma(ii)
+        tsimpv(isou,isou,iel) = tsimpv(isou,isou,iel)+cell_f_vol(iel)*gamma(ii)
       enddo
     endif
   enddo
