@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -41,14 +41,14 @@ subroutine coupbi &
 ! nfabor           ! i  ! <-- ! number of boundary faces                       !
 ! nscal            ! i  ! <-- ! total number of scalars                        !
 ! icodcl           ! te ! --> ! boundary condition code                        !
-!  (nfabor, nvarcl)!    !     ! = 1   -> dirichlet                             !
+!  (nfabor, nvar)  !    !     ! = 1   -> dirichlet                             !
 !                  !    !     ! = 3   -> flux density                          !
 !                  !    !     ! = 4   -> slip and u.n=0 (velocity)             !
 !                  !    !     ! = 5   -> friction and u.n=0 (velocity)         !
 !                  !    !     ! = 6   -> rugosity and u.n=0 (velocity)         !
 !                  !    !     ! = 9   -> free inlet/outlet (velocity)          !
 ! rcodcl           ! tr ! --> ! boundary condition values                      !
-!  (nfabor, nvarcl)!    !     ! rcodcl(1) = dirichlet value                    !
+!  (nfabor, nvar)  !    !     ! rcodcl(1) = dirichlet value                    !
 !                  !    !     ! rcodcl(2) = exchange coefficient value         !
 !                  !    !     !  (infinite if no exchange)                     !
 !                  !    !     ! rcodcl(3) = flux density value (negative       !
@@ -57,7 +57,7 @@ subroutine coupbi &
 !                  !    !     ! for velocities (vistl+visct)*gradu             !
 !                  !    !     ! for pressure              dt*gradp             !
 !                  !    !     ! for scalars                                    !
-!                  !    !     !        cp*(viscls+visct/sigmas)*gradt          !
+!                  !    !     !        cp*(viscls+visct/turb_schmidt)*gradt    !
 !__________________!____!_____!________________________________________________!
 
 !     Type: i (integer), r (real), s (string), a (array), l (logical),
@@ -74,11 +74,13 @@ use numvar
 use optcal
 use cstnum
 use cstphy
+use dimens, only: nvar
 use entsor
 use ppppar
 use ppthch
 use ppincl
 use pointe
+use field
 
 !===============================================================================
 
@@ -87,30 +89,49 @@ implicit none
 ! Arguments
 
 integer          nfabor, nscal
-integer          icodcl(nfabor,nvarcl)
-double precision rcodcl(nfabor,nvarcl,3)
+integer          icodcl(nfabor,nvar)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
 integer          ll, nbccou, inbcou, inbcoo, nbfcou
 integer          ifac, iloc, iscal
-integer          mode, flag
+integer          mode
 integer          issurf
-double precision temper, enthal
+integer          icpsyr, kcpsyr
 
 integer, dimension(:), allocatable :: lfcou
-double precision, dimension(:), allocatable :: thpar
+double precision, dimension(:), allocatable :: thpar, h_b
 
 !===============================================================================
+! Interfaces
+!===============================================================================
+
+interface
+
+  subroutine b_t_to_h(nlst, lstfac, t_b, h_b)
+
+    use mesh, only: nfabor
+    implicit none
+
+    integer :: nlst
+    integer, dimension(nlst) :: lstfac
+    double precision, dimension(nfabor), intent(in) :: t_b
+    double precision, dimension(nfabor), intent(out), target :: h_b
+
+  end subroutine b_t_to_h
+
+ end interface
 
 !===============================================================================
 ! SYRTHES coupling: get wall temperature
 !===============================================================================
 
+call field_get_key_id("syrthes_coupling", kcpsyr)
+
 ! Get number of coupling cases
 
 call nbcsyr (nbccou)
-!==========
 
 !---> Loop on couplings: get "tparoi" array for each coupling and apply
 !                        matching boundary condition.
@@ -123,7 +144,6 @@ do inbcou = 1, nbccou
   ! This is a surface coupling if issurf = 1
 
   call tsursy(inbcoo, issurf)
-  !==========
 
   if (issurf.eq.1) then
 
@@ -132,25 +152,25 @@ do inbcou = 1, nbccou
     ! Number of boundary faces per coupling case
 
     call nbesyr(inbcoo, mode, nbfcou)
-    !==========
 
-    ! Memory management to receive array
+    ! Memory management to receive arrays
+
     allocate(lfcou(nbfcou))
     allocate(thpar(nbfcou))
 
     ! Read wall temperature and interpolate if necessary.
 
     call varsyi(inbcou, mode, thpar)
-    !==========
 
     ! Prescribe wall temperature
     inbcoo = inbcou
     call leltsy(inbcoo, mode, lfcou)
-    !==========
 
     do iscal = 1, nscal
 
-      if (icpsyr(iscal).eq.1) then
+      call field_get_key_int(ivarfl(isca(iscal)), kcpsyr, icpsyr)
+
+      if (icpsyr.eq.1) then
 
         ! For scalars coupled with SYRTHES, prescribe a Dirichlet
         ! condition at coupled faces.
@@ -195,18 +215,26 @@ do inbcou = 1, nbccou
 
         if (iscal.eq.iscalt .and. itherm.eq.2) then
 
-          do iloc = 1, nbfcou
-
-            ifac = lfcou(iloc)
-
-            temper = rcodcl(ifac,ll,1)
-            flag   = -1
-            call usthht(flag, enthal, temper)
-            !==========
-            rcodcl(ifac,ll,1) = enthal
+          allocate(h_b(nfabor))
 
+          do iloc = 1, nfabor
+            h_b(iloc) = 0
           enddo
 
+          do iloc = 1, nbfcou
+            ifac = lfcou(iloc)
+            h_b(ifac) = thpar(iloc)
+          enddo
+
+          call b_t_to_h(nbfcou, lfcou, h_b, h_b)
+
+          do iloc = 1, nbfcou
+            ifac = lfcou(iloc)
+            rcodcl(ifac,ll,1) = h_b(ifac)
+          enddo
+
+          deallocate(h_b)
+
         endif
 
       endif
@@ -220,6 +248,7 @@ do inbcou = 1, nbccou
 
 enddo ! Loop on all syrthes couplings
 
+
 !===============================================================================
 ! End of boundary couplings
 !===============================================================================
