@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -43,8 +43,6 @@ module cplsat
   integer, save :: nbrcpl
   !> indicator coupling face / face only
   integer, save :: ifaccp
-  !> "Mobil Mesh" indicator for turbo-machine applications
-  integer, save :: imobil
   !> maximum permissible number of coupling
   integer   nbcpmx
   parameter(nbcpmx=10)
