@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,7 @@ subroutine cptssy &
 !================
 
  ( iscal  ,                                                       &
-   rtpa   , crvexp , crvimp )
+   crvexp , crvimp )
 
 !===============================================================================
 ! Purpose:
@@ -39,8 +39,6 @@ subroutine cptssy &
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
 ! iscal            ! i  ! <-- ! index number of the current scalar             !
-! rtpa             ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at previous time step)                       !
 ! crvexp           ! ra ! --> ! explicit part of the source term               !
 ! crvimp           ! ra ! --> ! implicit part of the source term               !
 !__________________!____!_____!________________________________________________!
@@ -57,12 +55,12 @@ subroutine cptssy &
 
 use paramx
 use numvar
-use dimens, only: nvar
 use entsor
 use cstphy
 use cstnum
 use mesh
 use optcal
+use field
 
 !===============================================================================
 
@@ -72,7 +70,6 @@ implicit none
 
 integer          iscal
 
-double precision rtpa(ncelet,nflown:nvar)
 double precision crvexp(ncelet), crvimp(ncelet)
 
 ! Local variables
@@ -83,6 +80,7 @@ double precision tsexp, tsimp
 
 integer, dimension(:), allocatable :: lcecpl
 double precision, dimension(:), allocatable :: tfluid, ctbimp, ctbexp
+double precision, dimension(:), pointer :: cvara_vart
 
 !===============================================================================
 
@@ -152,10 +150,12 @@ do inbcou = 1, nbccou
 
     ! Loop on coupled cells to initialize arrays
 
+    call field_get_val_prev_s(ivarfl(ivart), cvara_vart)
+
     do iloc = 1, ncecpl
 
       iel = lcecpl(iloc)
-      tfluid(iloc) = rtpa(iel, ivart)
+      tfluid(iloc) = cvara_vart(iel)
       ctbimp(iloc) = 0.0d0
       ctbexp(iloc) = 0.0d0
 
