@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,7 @@ subroutine cpvosy &
 !================
 
  ( isvtf  ,                                                       &
-   dt     , rtp    , rtpa   , propce )
+   dt     )
 
 !===============================================================================
 ! Purpose:
@@ -40,9 +40,6 @@ subroutine cpvosy &
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
 ! isvtf            ! i  ! <-- ! indicateur de scalaire pour la temp. fluide    !
-! rtp, rtpa        ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at current and previous time steps)          !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________!____!_____!________________________________________________!
 
 !     Type: i (integer), r (real), s (string), a (array), l (logical),
@@ -57,10 +54,10 @@ subroutine cpvosy &
 use paramx
 use numvar
 use entsor
-use dimens, only: nvar
 use cstphy
 use mesh
 use optcal
+use field
 
 !===============================================================================
 
@@ -70,16 +67,16 @@ implicit none
 
 integer          isvtf
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
+double precision dt(ncelet)
 
 ! Local variables
 
 integer          nbccou, inbcou, inbcoo, ncecpl, iloc, iel
-integer          mode, isvol, ivart
+integer          mode, isvol
 
 integer, dimension(:), allocatable :: lcecpl
 double precision, dimension(:), allocatable :: tfluid, hvol
+double precision, dimension(:), pointer :: cvar_scalt
 
 !===============================================================================
 
@@ -122,7 +119,7 @@ do inbcou = 1, nbccou
     endif
 
     mode = 1 ! Volume coupling
-    ivart = isca(iscalt)
+    call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
 
     ! Number of cells per coupling case
 
@@ -153,7 +150,7 @@ do inbcou = 1, nbccou
     do iloc = 1, ncecpl
 
       iel = lcecpl(iloc)
-      tfluid(iloc) = rtp(iel, ivart)
+      tfluid(iloc) = cvar_scalt(iel)
       hvol(iloc) = 0.0d0
 
     enddo
