@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -44,6 +44,20 @@
 #include <unistd.h>
 #endif
 
+#if defined(HAVE_DLOPEN)
+#include <dlfcn.h>
+#endif
+
+#if defined(HAVE_CATALYST)
+#  define _CS_EXIT_DEPLIB_CRASH_WORKAROUND 1
+#else
+#  define _CS_EXIT_DEPLIB_CRASH_WORKAROUND 0
+#endif
+
+#if _CS_EXIT_DEPLIB_CRASH_WORKAROUND
+#include <setjmp.h>
+#endif
+
 /*----------------------------------------------------------------------------
  * PLE library headers
  *----------------------------------------------------------------------------*/
@@ -60,7 +74,9 @@
 #include "bft_mem.h"
 #include "bft_printf.h"
 
-#include "cs_prototypes.h"
+#include "cs_file.h"
+#include "cs_fp_exception.h"
+#include "cs_log.h"
 #include "cs_timer.h"
 
 /*----------------------------------------------------------------------------
@@ -73,10 +89,14 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Local Macro Definitions
  *============================================================================*/
 
+#define DIR_SEPARATOR '/'
+
 /* Fortran API */
 /*-------------*/
 
@@ -86,7 +106,6 @@ BEGIN_C_DECLS
  */
 
 #define CS_BASE_N_STRINGS                               5
-#define CS_BASE_STRING_LEN                             64
 
 /*============================================================================
  * Local Type Definitions
@@ -127,6 +146,8 @@ static char  cs_glob_base_str[CS_BASE_N_STRINGS][CS_BASE_STRING_LEN + 1];
 
 /* Global variables associated with signal handling */
 
+static bool _cs_base_sighandlers_set = false;
+
 #if defined(SIGHUP)
 static _cs_base_sighandler_t cs_glob_base_sighup_save = SIG_DFL;
 #endif
@@ -145,6 +166,23 @@ static _cs_base_sighandler_t cs_glob_base_sigtrap_save = SIG_DFL;
 static _cs_base_sighandler_t cs_glob_base_sigcpu_save = SIG_DFL;
 #endif
 
+/* Workaround fo SIGSEGV at exit
+   with some ParaViw Catalyst/OpenGL driver combinations */
+
+#if _CS_EXIT_DEPLIB_CRASH_WORKAROUND
+static jmp_buf _cs_exit_jmp_buf;
+#endif
+
+/* Dynamic library handling */
+
+#if defined(HAVE_DLOPEN)
+#if defined(CS_DLOPEN_USE_RTLD_GLOBAL)
+static int _cs_dlopen_flags = RTLD_LAZY | RTLD_GLOBAL;
+#else
+static int _cs_dlopen_flags = RTLD_LAZY;
+#endif
+#endif
+
 /* Installation paths */
 
 static const char _cs_base_build_localedir[] = LOCALEDIR;
@@ -156,8 +194,10 @@ static char *_cs_base_env_pkglibdir = NULL;
 
 /* Log file */
 
+static FILE  *_bft_printf_file = NULL;
 static char  *_bft_printf_file_name = NULL;
 static bool   _bft_printf_suppress = false;
+static bool   _cs_trace = false;
 
 /* Additional cleanup steps */
 
@@ -175,9 +215,23 @@ static int
 _cs_base_bft_printf_null(const char  *format,
                          va_list      arg_ptr)
 {
+  CS_UNUSED(format);
+  CS_UNUSED(arg_ptr);
+
   return 0;
 }
 
+/*----------------------------------------------------------------------------
+ * False print of a message to standard output for discarded logs
+ *----------------------------------------------------------------------------*/
+
+static int
+_cs_base_bft_printf_file(const char  *format,
+                         va_list      arg_ptr)
+{
+  return  vfprintf(_bft_printf_file, format, arg_ptr);
+}
+
 /*----------------------------------------------------------------------------
  * Flush of log output buffer
  *----------------------------------------------------------------------------*/
@@ -198,6 +252,16 @@ _cs_base_bft_printf_flush_null(void)
   return 0;
 }
 
+/*----------------------------------------------------------------------------
+ * False flush of log output buffer for discarded logs
+ *----------------------------------------------------------------------------*/
+
+static int
+_cs_base_bft_printf_flush_file(void)
+{
+  return fflush(_bft_printf_file);
+}
+
 /*----------------------------------------------------------------------------
  * Print a message to the error output
  *
@@ -259,7 +323,7 @@ _cs_base_err_vprintf(const char  *format,
                                                             is unusable. */
 #endif
       for (i = cs_glob_n_ranks; i >= 10; i /= 10, n_dec += 1);
-      sprintf(err_file_name, "error_n%0*d", n_dec, cs_glob_rank_id + 1);
+      sprintf(err_file_name, "error_r%0*d", n_dec, cs_glob_rank_id);
     }
 
     freopen(err_file_name, "w", stderr);
@@ -276,25 +340,104 @@ _cs_base_err_vprintf(const char  *format,
  * The message is repeated on the standard output and an error file.
  *----------------------------------------------------------------------------*/
 
+#if defined(__GNUC__)
+__attribute__((format(printf, 1, 2)))
+#endif
+
 static void
 _cs_base_err_printf(const char  *format,
                     ...)
 {
-  /* Initialisation de la liste des arguments */
+  /* Initialize arguments list */
 
   va_list  arg_ptr;
-
   va_start(arg_ptr, format);
 
-  /* message sur les sorties */
+  /* message on outputs */
 
   _cs_base_err_vprintf(format, arg_ptr);
 
-  /* Finalisation de la liste des arguments */
+  /* Finalize arguments list */
 
   va_end(arg_ptr);
 }
 
+#if _CS_EXIT_DEPLIB_CRASH_WORKAROUND
+
+/*----------------------------------------------------------------------------
+ * Handle a signal as a warning at exit
+ *----------------------------------------------------------------------------*/
+
+static void
+_cs_base_sig_exit_crash_workaround(int  signum)
+{
+  bft_printf_flush();
+
+  char sig_name[32] = "";
+
+  switch (signum) {
+
+  case SIGFPE:
+    snprintf(sig_name, 31, "SIGFPE");
+    break;
+
+  case SIGSEGV:
+    snprintf(sig_name, 31, "SIGSEGV");
+    break;
+
+  }
+
+  if (cs_glob_rank_id <= 0) {
+    fprintf(stderr, _("Warning: signal %s received during program exit.\n"),
+            sig_name);
+    cs_base_backtrace_dump(stderr, 3);
+  }
+
+  longjmp(_cs_exit_jmp_buf, 1);
+}
+
+/*----------------------------------------------------------------------------
+ * Set signal handlers to workaround an external library issue at exit.
+ *
+ * This is used at exit to transform errors into warnings on rank 0
+ * only during a standard exit routine.
+ *
+ * This is an ugly workaroud to an ugly crash observed when exiting after
+ * have used VTK (Paraview Catalyst) libraries (on Debian Stretch with
+ * NVIDIA 390 driver at least).
+ *
+ * It is based on changing to a different signal handler for SIGSEGV
+ * (but could also be used for other signals), and using setjmp/longjmp
+ * to return from that handler and call _exit to avoid further issues.
+ * This avoids returning a nonzero error code to the caller when the
+ * code had finished correctly before calling exit(EXIT_SUCCESS).
+ *----------------------------------------------------------------------------*/
+
+static void
+_set_atexit_crash_workaround(void)
+{
+#if defined(HAVE_SIGACTION)
+
+  if (_cs_base_sighandlers_set) {
+    struct sigaction sa;
+    sa.sa_handler = *_cs_base_sig_exit_crash_workaround;
+    sigfillset(&sa.sa_mask);
+    sa.sa_flags = 0;
+    sa.sa_restorer = NULL;
+    if (sigaction(SIGSEGV, &sa, NULL) == -1 && cs_glob_rank_id <= 0)
+      perror("sigaction");
+
+    int i = setjmp(_cs_exit_jmp_buf);
+
+    if (i == 1)
+      _exit(EXIT_SUCCESS); /* This handler is only used during normal exit */
+  }
+
+#endif
+}
+
+#endif /* _CS_EXIT_DEPLIB_CRASH_WORKAROUND */
+
 /*----------------------------------------------------------------------------
  * Exit function
  *----------------------------------------------------------------------------*/
@@ -302,6 +445,9 @@ _cs_base_err_printf(const char  *format,
 static void
 _cs_base_exit(int status)
 {
+  if (status == EXIT_SUCCESS)
+    cs_base_update_status(NULL);
+
 #if defined(HAVE_MPI)
   {
     int mpi_flag;
@@ -319,6 +465,13 @@ _cs_base_exit(int status)
 
     if (mpi_flag != 0) {
 
+      /* For safety, flush all streams before calling MPI_Abort
+       * (should be done by exit, but in case we call MPI_Abort
+       * due to a SIGTERM received from another rank's MPI_Abort,
+       * make sure we avoid ill-defined behavior) */
+
+      fflush(NULL);
+
       if (status != EXIT_SUCCESS)
         MPI_Abort(cs_glob_mpi_comm, EXIT_FAILURE);
 
@@ -332,6 +485,13 @@ _cs_base_exit(int status)
   }
 #endif /* HAVE_MPI */
 
+#if _CS_EXIT_DEPLIB_CRASH_WORKAROUND
+
+  if (status == EXIT_SUCCESS)
+    _set_atexit_crash_workaround();
+
+#endif
+
   exit(status);
 }
 
@@ -388,9 +548,12 @@ _error_mem_summary(void)
     (_("Theoretical current allocated memory:   %llu kB\n"),
      (unsigned long long)(bft_mem_size_current()));
 
-  _cs_base_err_printf
-    (_("Theoretical maximum allocated memory:   %llu kB\n"),
-     (unsigned long long)(bft_mem_size_max()));
+  mem_usage = bft_mem_size_max();
+
+  if (mem_usage > 0)
+    _cs_base_err_printf
+      (_("Theoretical maximum allocated memory:   %llu kB\n"),
+       (unsigned long long)(bft_mem_size_max()));
 
   if (bft_mem_usage_initialized() == 1) {
 
@@ -447,9 +610,8 @@ _cs_mem_error_handler(const char  *file_name,
  *----------------------------------------------------------------------------*/
 
 static void
-_cs_base_backtrace_print(int  niv_debut)
+_cs_base_backtrace_print(int  lv_start)
 {
-  size_t  ind;
   bft_backtrace_t  *tr = NULL;
 
   tr = bft_backtrace_create();
@@ -462,23 +624,23 @@ _cs_base_backtrace_print(int  niv_debut)
     const char *s_func;
     const char *s_addr;
 
-    const char s_inconnu[] = "?";
+    const char s_unknown[] = "?";
     const char s_vide[] = "";
     const char *s_prefix = s_vide;
 
-    size_t nbr = bft_backtrace_size(tr);
+    int nbr = bft_backtrace_size(tr);
 
     if (nbr > 0)
       _cs_base_err_printf(_("\nCall stack:\n"));
 
-    for (ind = niv_debut ; ind < nbr ; ind++) {
+    for (int ind = lv_start; ind < nbr; ind++) {
 
       s_file = bft_backtrace_file(tr, ind);
       s_func = bft_backtrace_function(tr, ind);
       s_addr = bft_backtrace_address(tr, ind);
 
       if (s_file == NULL)
-        s_file = s_inconnu;
+        s_file = s_unknown;
       if (s_func == NULL)
         strcpy(s_func_buf, "?");
       else {
@@ -487,10 +649,10 @@ _cs_base_backtrace_print(int  niv_debut)
         strcat(s_func_buf, ">");
       }
       if (s_addr == NULL)
-        s_addr = s_inconnu;
+        s_addr = s_unknown;
 
       _cs_base_err_printf("%s%4d: %-12s %-32s (%s)\n", s_prefix,
-                          ind-niv_debut+1, s_addr, s_func_buf, s_file);
+                          ind-lv_start+1, s_addr, s_func_buf, s_file);
 
     }
 
@@ -641,6 +803,11 @@ _get_path(const char   *dir_path,
 
     return *env_path;
   }
+#else
+
+  CS_UNUSED(dir_path);
+  CS_UNUSED(env_path);
+
 #endif /* defined(HAVE_RELOCATABLE) */
 
   /* Standard install */
@@ -847,6 +1014,8 @@ _cs_base_mpi_setup(const char *app_name)
 
 #endif /* HAVE_MPI */
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -1057,7 +1226,7 @@ cs_base_logfile_head(int    argc,
              "                      Version %s\n\n",
              CS_APP_VERSION);
 
-  bft_printf("\n  Copyright (C) 1998-2014 EDF S.A., France\n\n");
+  bft_printf("\n  Copyright (C) 1998-2019 EDF S.A., France\n\n");
 
 #if defined(CS_REVISION)
   if (strlen(CS_REVISION) > 0)
@@ -1129,14 +1298,13 @@ cs_base_mpi_init(int    *argc,
 
 #if   defined(__bg__) || defined(__CRAYXT_COMPUTE_LINUX_TARGET)
 
-  /* Notes: Blue Gene/L also defines the BGLMPI_SIZE environment variable.
-   *        Blue Gene/P defines BG_SIZE (plus BG_MAPPING, and BG_RELEASE). */
+  /* Blue Gene/Q or Cray: assume MPI is always used. */
 
   use_mpi = true;
 
 #elif defined(MPICH) || defined(MPICH2) || defined(MSMPI_VER)
 
-  /* Notes: Microsoft MPI is based on standard MPICH2 */
+  /* Notes: Microsoft MPI is based on MPICH */
 
   if (getenv("PMI_RANK") != NULL)
     use_mpi = true;
@@ -1144,45 +1312,19 @@ cs_base_mpi_init(int    *argc,
   else if (getenv("PCMPI") != NULL) /* Platform MPI */
     use_mpi = true;
 
-#if defined(MPICH_NAME)
-#if (MPICH_NAME == 1)
-
-  /*
-    Using standard MPICH1 1.2.x with the p4 (default) mechanism,
-    the information required by MPI_Init() are transferred through
-    the command line, which is then modified by MPI_Init();
-    in this case, only rank 0 knows the "user" command line arguments
-    at program startup, the other processes obtaining them only upon
-    calling  MPI_Init(). In this case, it is thus necessary to initialize
-    MPI before parsing the command line.
-  */
-
-  for (arg_id = 0 ; arg_id < *argc ; arg_id++) {
-    if (   !strcmp((*argv)[arg_id], "-p4pg")         /* For process 0 */
-        || !strcmp((*argv)[arg_id], "-p4rmrank")) {  /* For other processes */
-      use_mpi = true;
-      break;
-    }
-  }
-
-  if (getenv("GMPI_ID") != NULL) /* In case we are using MPICH-GM */
-    use_mpi = true;
-
-#endif /* End for MPICH-1 or variants */
-#endif
-
-#elif defined(LAM_MPI)
-  if (getenv("LAMRANK") != NULL)
-    use_mpi = true;
-
 #elif defined(OPEN_MPI)
   if (getenv("OMPI_MCA_ns_nds_vpid") != NULL)         /* OpenMPI 1.2 */
     use_mpi = true;
-  else if (getenv("OMPI_COMM_WORLD_RANK") != NULL)    /* OpenMPI 1.3 */
+  else if (getenv("OMPI_COMM_WORLD_RANK") != NULL)    /* OpenMPI 1.3 + */
     use_mpi = true;
 
 #endif /* Tests for known MPI variants */
 
+  /* Test for run through SLURM's srun */
+
+  if (getenv("SLURM_SRUN_COMM_HOST") != NULL)
+    use_mpi = true;
+
   /* If we have determined from known MPI environment variables
      of command line arguments that we are running under MPI,
      initialize MPI */
@@ -1292,10 +1434,13 @@ cs_exit(int  status)
 
 /*----------------------------------------------------------------------------
  * Initialize error and signal handlers.
+ *
+ * parameters:
+ *   signal_defaults <-- leave default signal handlers in place if true
  *----------------------------------------------------------------------------*/
 
 void
-cs_base_error_init(void)
+cs_base_error_init(bool  signal_defaults)
 {
   /* Error handler */
 
@@ -1305,30 +1450,35 @@ cs_base_error_init(void)
 
   /* Signal handlers */
 
-  bft_backtrace_print_set(_cs_base_backtrace_print);
+  if (signal_defaults == false) {
+
+    bft_backtrace_print_set(_cs_base_backtrace_print);
 
 #if defined(SIGHUP)
-  if (cs_glob_rank_id <= 0)
-    cs_glob_base_sighup_save  = signal(SIGHUP, _cs_base_sig_fatal);
+    if (cs_glob_rank_id <= 0)
+      cs_glob_base_sighup_save  = signal(SIGHUP, _cs_base_sig_fatal);
 #endif
 
-  if (cs_glob_rank_id <= 0) {
-    cs_glob_base_sigint_save  = signal(SIGINT, _cs_base_sig_fatal);
-    cs_glob_base_sigterm_save = signal(SIGTERM, _cs_base_sig_fatal);
-  }
+    if (cs_glob_rank_id <= 0) {
+      cs_glob_base_sigint_save  = signal(SIGINT, _cs_base_sig_fatal);
+      cs_glob_base_sigterm_save = signal(SIGTERM, _cs_base_sig_fatal);
+    }
 
-  cs_glob_base_sigfpe_save  = signal(SIGFPE, _cs_base_sig_fatal);
-  cs_glob_base_sigsegv_save = signal(SIGSEGV, _cs_base_sig_fatal);
+    cs_glob_base_sigfpe_save  = signal(SIGFPE, _cs_base_sig_fatal);
+    cs_glob_base_sigsegv_save = signal(SIGSEGV, _cs_base_sig_fatal);
 
 #if defined(__bgq__)
-  cs_glob_base_sigabrt_save  = signal(SIGABRT, _cs_base_sig_fatal);
-  cs_glob_base_sigtrap_save  = signal(SIGTRAP, _cs_base_sig_fatal);
+    cs_glob_base_sigabrt_save  = signal(SIGABRT, _cs_base_sig_fatal);
+    cs_glob_base_sigtrap_save  = signal(SIGTRAP, _cs_base_sig_fatal);
 #endif
 
 #if defined(SIGXCPU)
-  if (cs_glob_rank_id <= 0)
-    cs_glob_base_sigcpu_save = signal(SIGXCPU, _cs_base_sig_fatal);
+    if (cs_glob_rank_id <= 0)
+      cs_glob_base_sigcpu_save = signal(SIGXCPU, _cs_base_sig_fatal);
 #endif
+
+    _cs_base_sighandlers_set = true;
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -1357,39 +1507,45 @@ cs_base_mem_init(void)
 
   /* Memory management initialization */
 
-  if ((base_name = getenv("CS_MEM_LOG")) != NULL) {
-
-    /* We may not use BFT_MALLOC here as memory management has
-       not yet been initialized using bft_mem_init() */
-
-    if (base_name != NULL) {
-
-      /* In parallel, we will have one trace file per MPI process */
-      if (cs_glob_rank_id >= 0) {
-        int i;
-        int n_dec = 1;
-        for (i = cs_glob_n_ranks; i >= 10; i /= 10, n_dec += 1);
-        file_name = malloc((strlen(base_name) + n_dec + 2) * sizeof (char));
-        sprintf(file_name, "%s.%0*d", base_name, n_dec, cs_glob_rank_id + 1);
-      }
-      else {
-        file_name = malloc((strlen(base_name) + 1) * sizeof (char));
-        strcpy(file_name, base_name);
-      }
-    }
-
-  }
-
   if (bft_mem_initialized())
     cs_glob_base_bft_mem_init = false;
 
   else {
-    cs_glob_base_bft_mem_init = true;
-    bft_mem_init(file_name);
-  }
 
-  if (file_name != NULL)
-    free (file_name);
+    if ((base_name = getenv("CS_MEM_LOG")) != NULL) {
+
+      /* We may not use BFT_MALLOC here as memory management has
+         not yet been initialized using bft_mem_init() */
+
+      if (base_name != NULL) {
+
+        /* In parallel, we will have one trace file per MPI process */
+        if (cs_glob_rank_id >= 0) {
+          int i;
+          int n_dec = 1;
+          for (i = cs_glob_n_ranks; i >= 10; i /= 10, n_dec += 1);
+          file_name = malloc((strlen(base_name) + n_dec + 2) * sizeof (char));
+          sprintf(file_name, "%s.%0*d", base_name, n_dec, cs_glob_rank_id);
+        }
+        else {
+          file_name = malloc((strlen(base_name) + 1) * sizeof (char));
+          strcpy(file_name, base_name);
+        }
+
+        /* Actually initialize bft_mem instrumentation only when
+           CS_MEM_LOG is defined (for better performance) */
+
+        bft_mem_init(file_name);
+
+        free (file_name);
+
+      }
+
+    }
+
+    cs_glob_base_bft_mem_init = true;
+
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -1419,7 +1575,8 @@ cs_base_mem_finalize(void)
 
   /* Memory summary */
 
-  bft_printf(_("\nMemory use summary:\n\n"));
+  cs_log_printf(CS_LOG_PERFORMANCE,
+                _("\nMemory use summary:\n\n"));
 
   valreal[0] = (double)bft_mem_usage_max_pr_size();
   valreal[1] = (double)bft_mem_size_max();
@@ -1482,23 +1639,29 @@ cs_base_mem_finalize(void)
 
       /* Print to log file */
 
-      bft_printf(_("  %s %12.3f %ciB\n"),
-                 _(type_bil[ind_bil]), valreal[ind_bil], unit[itot]);
+      cs_log_printf(CS_LOG_PERFORMANCE,
+                    _("  %s %12.3f %ciB\n"),
+                    _(type_bil[ind_bil]), valreal[ind_bil], unit[itot]);
 
 #if defined(HAVE_MPI)
       if (cs_glob_n_ranks > 1 && cs_glob_rank_id == 0) {
-        bft_printf(_("                             "
-                     "local minimum: %12.3f %ciB  (rank %d)\n"),
-                   val_min[ind_bil].val, unit[imin], val_min[ind_bil].rank);
-        bft_printf(_("                             "
-                     "local maximum: %12.3f %ciB  (rank %d)\n"),
-                   val_max[ind_bil].val, unit[imax], val_max[ind_bil].rank);
+        cs_log_printf(CS_LOG_PERFORMANCE,
+                      _("                             "
+                        "local minimum: %12.3f %ciB  (rank %d)\n"),
+                      val_min[ind_bil].val, unit[imin], val_min[ind_bil].rank);
+        cs_log_printf(CS_LOG_PERFORMANCE,
+                      _("                             "
+                        "local maximum: %12.3f %ciB  (rank %d)\n"),
+                      val_max[ind_bil].val, unit[imax], val_max[ind_bil].rank);
       }
 #endif
     }
 
   }
 
+  cs_log_printf(CS_LOG_PERFORMANCE, "\n");
+  cs_log_separator(CS_LOG_PERFORMANCE);
+
   /* Finalize memory handling */
 
   if (cs_glob_base_bft_mem_init == true) {
@@ -1531,7 +1694,8 @@ cs_base_time_summary(void)
 
   /*xxxxxxxxxxxxxxxxxxxxxxxxxxx Instructions xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx*/
 
-  bft_printf(_("\nCalculation time summary:\n"));
+  cs_log_printf(CS_LOG_PERFORMANCE,
+                _("\nCalculation time summary:\n"));
 
   cs_timer_cpu_times(&utime, &stime);
 
@@ -1544,15 +1708,18 @@ cs_base_time_summary(void)
   /* CPU time */
 
   if (utime > 0. || stime > 0.) {
-    bft_printf (_("\n  User CPU time:       %12.3f s\n"),
-                (float)utime);
-    bft_printf (_("  System CPU time:     %12.3f s\n"),
-                (float)stime);
+    cs_log_printf(CS_LOG_PERFORMANCE,
+                  _("\n  User CPU time:       %12.3f s\n"),
+                  (float)utime);
+    cs_log_printf(CS_LOG_PERFORMANCE,
+                  _("  System CPU time:     %12.3f s\n"),
+                  (float)stime);
   }
 
   else if (time_cpu > 0.)
-    bft_printf (_("\n  CPU time:            %12.3f s\n"),
-                (float)time_cpu);
+    cs_log_printf(CS_LOG_PERFORMANCE,
+                  _("\n  CPU time:            %12.3f s\n"),
+                  (float)time_cpu);
 
 #if defined(HAVE_MPI)
   if (cs_glob_n_ranks > 1) {
@@ -1560,8 +1727,9 @@ cs_base_time_summary(void)
     MPI_Reduce (&time_cpu, &time_cumul, 1, MPI_DOUBLE, MPI_SUM,
                 0, cs_glob_mpi_comm);
     if (cs_glob_rank_id == 0)
-      bft_printf (_("  Total CPU time:      %12.3f s\n"),
-                  time_cumul);
+      cs_log_printf(CS_LOG_PERFORMANCE,
+                    _("  Total CPU time:      %12.3f s\n"),
+                    time_cumul);
   }
 #endif
 
@@ -1571,14 +1739,113 @@ cs_base_time_summary(void)
 
   if (time_tot > 0.) {
 
-    bft_printf (_("\n  Elapsed time:        %12.3f s\n"),
-                time_tot);
+    cs_log_printf(CS_LOG_PERFORMANCE,
+                  _("\n  Elapsed time:        %12.3f s\n"),
+                  time_tot);
 
-    bft_printf (_("  CPU / elapsed time   %12.3f\n"),
-                (float)(time_cpu/time_tot));
+    cs_log_printf(CS_LOG_PERFORMANCE,
+                  _("  CPU / elapsed time   %12.3f\n"),
+                  (float)(time_cpu/time_tot));
 
   }
 
+  cs_log_printf(CS_LOG_PERFORMANCE, "\n");
+  cs_log_separator(CS_LOG_PERFORMANCE);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Update status file.
+ *
+ * If the format string is NULL, the file is removed.
+
+ * \param[in]  format  format string, or NULL
+ * \param[in]  ...     format arguments
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_update_status(const char  *format,
+                      ...)
+{
+  static const char _status_file_name[] = "run_status.running";
+  static FILE *_status_file = NULL;
+
+  if (cs_glob_rank_id < 1) {
+
+    if (format == NULL) {
+      if (_status_file != NULL) {
+        if (fclose(_status_file) == 0) {
+          _status_file = NULL;
+          remove(_status_file_name);
+        }
+      }
+    }
+
+    else {
+
+      va_list  arg_ptr;
+      va_start(arg_ptr, format);
+
+      /* Output to trace */
+
+#if defined(va_copy) || defined(__va_copy)
+      if (_cs_trace && format != NULL) {
+        va_list arg_ptr_2;
+#if defined(va_copy)
+        va_copy(arg_ptr_2, arg_ptr);
+#else
+        __va_copy(arg_ptr_2, arg_ptr);
+#endif
+        vprintf(format, arg_ptr_2);
+        va_end(arg_ptr_2);
+      }
+#endif
+
+      /* Status file */
+
+      if (_status_file == NULL)
+        _status_file = fopen(_status_file_name, "w");
+
+      if (_status_file != NULL) {
+        long p_size = ftell(_status_file);
+        fseek(_status_file, 0, SEEK_SET);
+        vfprintf(_status_file, format, arg_ptr);
+        long c_size = ftell(_status_file);
+
+        while (p_size > c_size) {
+          size_t l = 0;
+          char buf[64];
+          while (l < 64 && p_size > c_size) {
+            buf[l++] = ' ';
+            p_size--;
+          }
+          fwrite(buf, 1, l, _status_file);
+        }
+      }
+
+      va_end(arg_ptr);
+
+    }
+
+  }
+}
+
+/*----------------------------------------------------------------------------
+ * Set tracing of progress on or off.
+ *
+ * This function should be called before cs_base_bft_printf_set() if tracing
+ * is activated.
+ *
+ * parameters:
+ *   trace  <-- trace progress to stdout
+ *----------------------------------------------------------------------------*/
+
+void
+cs_base_trace_set(bool  trace)
+{
+  if (_bft_printf_file_name == NULL)
+    _cs_trace = trace;
 }
 
 /*----------------------------------------------------------------------------
@@ -1587,28 +1854,42 @@ cs_base_time_summary(void)
  * This allows redirecting or suppressing logging for different ranks.
  *
  * parameters:
- *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
+ *   log_name    <-- base file name for log
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
+ *                   false: to "/dev/null" (suppressed)
+ *                   true: to <log_name>_r*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_bft_printf_init(const char  *log_name,
-                        int          r0_log_flag,
-                        int          rn_log_flag)
+                        bool         rn_log_flag)
 {
   BFT_FREE(_bft_printf_file_name);
   _bft_printf_suppress = false;
 
+  const char ext[] = ".log";
+
+  /* Allow bypassing this with environment variable to accomodate
+     some debug habits */
+
+  bool log_to_stdout = false;
+  const char *p = getenv("CS_LOG_TO_STDOUT");
+  if (p != NULL) {
+    if (atoi(p) > 0)
+      log_to_stdout = true;
+  }
+
   /* Rank 0 */
 
-  if (cs_glob_rank_id < 1 && r0_log_flag == 1 && log_name != NULL) {
+  if (   cs_glob_rank_id < 1
+      && log_name != NULL
+      && log_to_stdout == false) {
 
-    BFT_MALLOC(_bft_printf_file_name, strlen(log_name) + 1, char);
+    BFT_MALLOC(_bft_printf_file_name,
+               strlen(log_name) + strlen(ext) + 1,
+               char);
     strcpy(_bft_printf_file_name, log_name);
+    strcat(_bft_printf_file_name, ext);
 
   }
 
@@ -1616,21 +1897,24 @@ cs_base_bft_printf_init(const char  *log_name,
 
   else if (cs_glob_rank_id > 0) {
 
-    if (log_name != NULL && rn_log_flag == 1) { /* Non-suppressed logs */
+    if (log_name != NULL && rn_log_flag > 0) { /* Non-suppressed logs */
 
-      int i;
-      int n_dec = 1;
-      for (i = cs_glob_n_ranks; i >= 10; i /= 10, n_dec += 1);
-      BFT_MALLOC(_bft_printf_file_name, strlen(log_name) + n_dec + 3, char);
-      sprintf(_bft_printf_file_name,
-              "%s_n%0*d",
-              log_name,
-              n_dec,
-              cs_glob_rank_id+1);
+      if (log_to_stdout == false) {
+        int n_dec = 1;
+        for (int i = cs_glob_n_ranks; i >= 10; i /= 10, n_dec += 1);
+        BFT_MALLOC(_bft_printf_file_name,
+                   strlen(log_name) + n_dec + 3 + strlen(ext), char);
+        sprintf(_bft_printf_file_name,
+                "%s_r%0*d%s",
+                log_name,
+                n_dec,
+                cs_glob_rank_id,
+                ext);
+      }
 
     }
 
-    else if (rn_log_flag == 2) { /* Suppressed logs */
+    else { /* Suppressed logs */
 
       _bft_printf_suppress = true;
       bft_printf_proxy_set(_cs_base_bft_printf_null);
@@ -1648,45 +1932,59 @@ cs_base_bft_printf_init(const char  *log_name,
  * This allows redirecting or suppressing logging for different ranks.
  *
  * parameters:
- *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
+ *   log_name    <-- base file name for log
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: suppressed
+ *                   false: to "/dev/null" (suppressed)
+ *                   true: to <log_name>_r*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_bft_printf_set(const char  *log_name,
-                       int          r0_log_flag,
-                       int          rn_log_flag)
+                       bool         rn_log_flag)
 {
-  cs_base_bft_printf_init(log_name, r0_log_flag, rn_log_flag);
+  cs_base_bft_printf_init(log_name, rn_log_flag);
 
   if (_bft_printf_file_name != NULL && _bft_printf_suppress == false) {
 
-    bft_printf_proxy_set(vprintf);
-    bft_printf_flush_proxy_set(_cs_base_bft_printf_flush);
-    ple_printf_function_set(vprintf);
-
     /* Redirect log */
 
     if (_bft_printf_file_name != NULL) {
 
-      FILE *fp = freopen(_bft_printf_file_name, "w", stdout);
+      bft_printf_proxy_set(vprintf);
+      bft_printf_flush_proxy_set(_cs_base_bft_printf_flush);
+      ple_printf_function_set(vprintf);
 
-      if (fp == NULL)
-        bft_error(__FILE__, __LINE__, errno,
-                  _("It is impossible to redirect the standard output "
-                    "to file:\n%s"), _bft_printf_file_name);
+      if (cs_glob_rank_id > 0 || _cs_trace == false) {
+
+        FILE *fp = freopen(_bft_printf_file_name, "w", stdout);
+
+        if (fp == NULL)
+          bft_error(__FILE__, __LINE__, errno,
+                    _("It is impossible to redirect the standard output "
+                      "to file:\n%s"), _bft_printf_file_name);
 
 #if defined(HAVE_DUP2)
-      if (dup2(fileno(fp), fileno(stderr)) == -1)
-        bft_error(__FILE__, __LINE__, errno,
-                  _("It is impossible to redirect the standard error "
-                    "to file:\n%s"), _bft_printf_file_name);
+        if (dup2(fileno(fp), fileno(stderr)) == -1)
+          bft_error(__FILE__, __LINE__, errno,
+                    _("It is impossible to redirect the standard error "
+                      "to file:\n%s"), _bft_printf_file_name);
 #endif
 
+      }
+      else {
+
+        _bft_printf_file = fopen(_bft_printf_file_name, "w");
+        if (_bft_printf_file == NULL)
+          bft_error(__FILE__, __LINE__, errno,
+                    _("Error opening log file:\n%s"),
+                    _bft_printf_file_name);
+
+        bft_printf_proxy_set(_cs_base_bft_printf_file);
+        bft_printf_flush_proxy_set(_cs_base_bft_printf_flush_file);
+        ple_printf_function_set(_cs_base_bft_printf_file);
+
+      }
+
     }
 
   }
@@ -1927,6 +2225,313 @@ cs_base_get_pkglibdir(void)
                    &_cs_base_env_pkglibdir);
 }
 
+/*----------------------------------------------------------------------------
+ * Ensure bool argument has value 0 or 1.
+ *
+ * This allows working around issues with Intel compiler C bindings,
+ * which seem to pass incorrect values in some cases.
+ *
+ * parameters:
+ *   b <-> pointer to bool
+ *----------------------------------------------------------------------------*/
+
+void
+cs_base_check_bool(bool *b)
+{
+  if (sizeof(bool) == 1) {
+    char *pb = (char *)b;
+    int i = *pb;
+    if (i != 0 && i != 1)
+      *b = true;
+  }
+  else if (sizeof(bool) == sizeof(int)) {
+    int *pb = (int *)b;
+    if (*pb != 0 && *pb != 1)
+      *b = true;
+  }
+}
+
+/*----------------------------------------------------------------------------
+ * Open a data file in read mode.
+ *
+ * If a file of the given name in the working directory is found, it
+ * will be opened. Otherwise, it will be searched for in the "data/thch"
+ * subdirectory of pkgdatadir.
+ *
+ * parameters:
+ *   base_name      <-- base file name
+ *
+ * returns:
+ *   pointer to opened file
+ *----------------------------------------------------------------------------*/
+
+FILE *
+cs_base_open_properties_data_file(const char  *base_name)
+{
+  FILE *f = NULL;
+
+  char *_f_name = NULL;
+  const char *file_name = base_name;
+
+  /* choose local file if present, default otherwise */
+
+  if (! cs_file_isreg(file_name)) {
+    const char *datadir = cs_base_get_pkgdatadir();
+    const char subdir[] = "/data/thch/";
+    BFT_MALLOC(_f_name,
+               strlen(datadir) + strlen(subdir) + strlen(base_name) + 1,
+               char);
+    sprintf(_f_name, "%s%s%s", datadir, subdir, base_name);
+    file_name = _f_name;
+  }
+
+  f = fopen(file_name, "r");
+
+  if (f == NULL)
+    bft_error(__FILE__, __LINE__, errno,
+              _("Error opening data file \"%s\""), file_name);
+
+  BFT_FREE(_f_name);
+
+  return f;
+}
+
+#if defined(HAVE_DLOPEN)
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Load a dynamic library.
+ *
+ * \param[in]  filename  path to shared library file
+ *
+ * \return  handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void*
+cs_base_dlopen(const char *filename)
+{
+  void *retval = NULL;
+
+  /* Disable floating-point traps as the initialization of some libraries
+     may interfere with this (for example, embree, and optional Paraview
+     depedency) */
+
+  cs_fp_exception_disable_trap();
+
+  /* Load symbols from shared library */
+
+  retval = dlopen(filename, _cs_dlopen_flags);
+
+  if (retval == NULL)
+    bft_error(__FILE__, __LINE__, 0,
+              _("Error loading %s: %s."), filename, dlerror());
+
+  /* Restore floating-point trap behavior */
+
+  cs_fp_exception_restore_trap();
+
+  return retval;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Load a plugin's dynamic library
+ *
+ * This function is similar to \ref cs_base_dlopen, execpt that only
+ * the base plugin file name (with no extension) needs to be given.
+ * It is assumed the file is available in the code's "pkglibdir" directory,
+ *
+ * \param[in]  name  path to shared library file
+ *
+ * \return  handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_base_dlopen_plugin(const char *name)
+{
+  void *retval = NULL;
+
+  char  *lib_path = NULL;
+  const char *pkglibdir = cs_base_get_pkglibdir();
+
+  /* Open shared library */
+
+  BFT_MALLOC(lib_path,
+             strlen(pkglibdir) + 1 + 3 + strlen(name) + 3 + 1,
+             char);
+
+  sprintf(lib_path, "%s%c%s.so", pkglibdir, DIR_SEPARATOR, name);
+
+  retval = cs_base_dlopen(lib_path);
+
+  BFT_FREE(lib_path);
+
+  return retval;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get flags for dlopen.
+ *
+ * \return  flags used for dlopen.
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_base_dlopen_get_flags(void)
+{
+  return _cs_dlopen_flags;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set flags for dlopen.
+ *
+ * \param[in]  flags  flags to set
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_dlopen_set_flags(int flags)
+{
+  _cs_dlopen_flags = flags;
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Unload a dynamic library.
+ *
+ * Note that the dlopen underlying mechanism uses a reference count, so
+ * a library is really unloaded only one \ref cs_base_dlclose has been called
+ * the same number of times as \ref cs_base_dlopen.
+ *
+ * \param[in]  filename  optional path to shared library file name for error
+ *                       logging, or NULL
+ * \param[in]  handle    handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_dlclose(const char  *filename,
+                void        *handle)
+{
+  int retval = 0;
+
+  if (handle != NULL)
+    retval = dlclose(handle);
+
+  if (retval != 0) {
+    if (filename != NULL)
+      bft_error(__FILE__, __LINE__, 0,
+                _("Error decrementing count or unloading %s: %s."),
+                filename, dlerror());
+    else
+      bft_error(__FILE__, __LINE__, 0,
+                _("Error decrementing count or unloading %s."),
+                dlerror());
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get a shared library function pointer
+ *
+ * \param[in]  handle            handle to shared library
+ * \param[in]  name              name of function symbol in library
+ * \param[in]  errors_are_fatal  abort if true, silently ignore if false
+ *
+ * \return  pointer to function in shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_base_get_dl_function_pointer(void        *handle,
+                                const char  *name,
+                                bool         errors_are_fatal)
+{
+  void  *retval = NULL;
+  char  *error = NULL;
+
+  dlerror();    /* Clear any existing error */
+
+  retval = dlsym(handle, name);
+  error = dlerror();
+
+  if (error != NULL && errors_are_fatal)
+    bft_error(__FILE__, __LINE__, 0,
+              _("Error calling dlsym for %s: %s\n"), name, error);
+
+  return retval;
+}
+
+#endif /* defined(HAVE_DLOPEN) */
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Dump a stack trace to a file
+ *
+ * \param[in]  f         pointer to file in which to dump trace
+ * \param[in]  lv_start  start level in stack trace
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_backtrace_dump(FILE  *f,
+                       int    lv_start)
+{
+  bft_backtrace_t  *tr = NULL;
+
+  tr = bft_backtrace_create();
+
+  if (tr != NULL) {
+
+    char s_func_buf[67];
+
+    const char *s_file;
+    const char *s_func;
+    const char *s_addr;
+
+    const char s_unknown[] = "?";
+    const char s_vide[] = "";
+    const char *s_prefix = s_vide;
+
+    int nbr = bft_backtrace_size(tr);
+
+    if (nbr > 0)
+      fprintf(f, "\nCall stack:\n");
+
+    for (int ind = lv_start; ind < nbr; ind++) {
+
+      s_file = bft_backtrace_file(tr, ind);
+      s_func = bft_backtrace_function(tr, ind);
+      s_addr = bft_backtrace_address(tr, ind);
+
+      if (s_file == NULL)
+        s_file = s_unknown;
+      if (s_func == NULL)
+        strcpy(s_func_buf, "?");
+      else {
+        s_func_buf[0] = '<';
+        strncpy(s_func_buf + 1, s_func, 64);
+        strcat(s_func_buf, ">");
+      }
+      if (s_addr == NULL)
+        s_addr = s_unknown;
+
+      fprintf(f, "%s%4d: %-12s %-32s (%s)\n", s_prefix,
+              ind-lv_start+1, s_addr, s_func_buf, s_file);
+
+    }
+
+    bft_backtrace_destroy(tr);
+
+    if (nbr > 0)
+      fprintf(f, "End of stack\n\n");
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
