@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -27,10 +27,18 @@
 
 /*----------------------------------------------------------------------------*/
 
+#include "cs_defs.h"
+
 /*----------------------------------------------------------------------------
  * Standard C library headers
  *----------------------------------------------------------------------------*/
 
+#include <stdio.h>
+
+/*----------------------------------------------------------------------------
+ *  Local headers
+ *----------------------------------------------------------------------------*/
+
 /*=============================================================================
  * Macro definitions
  *============================================================================*/
@@ -42,7 +50,7 @@
 
 /* System type name */
 
-#if defined(__blrts__) || defined(__bg__)
+#if defined(__bg__)
 #define _CS_ARCH_Blue_Gene
 
 #elif defined(__linux__) || defined(__linux) || defined(linux)
@@ -57,14 +65,14 @@
  * be better optimized on memory-aligned data (if 0 here, no alignment
  * is leveraged). This alignment is not exploited yet in Code_Saturne. */
 
-#if defined(__blrts__) || defined(__bgp__)
-#define CS_MEM_ALIGN 16
-#elif defined(__bgq__)
+#if defined(__bgq__)
 #define CS_MEM_ALIGN 32
 #else
 #define CS_MEM_ALIGN 0
 #endif
 
+#define CS_BASE_STRING_LEN                             80
+
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
@@ -86,6 +94,25 @@ typedef void (cs_base_atexit_t) (void);
  * Public function prototypes
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Return a string "true" or "false" according to the boolean
+ *
+ * \param[in]  boolean  bool type
+ *
+ * \return a string "true" or "false"
+ */
+/*----------------------------------------------------------------------------*/
+
+static inline const char *
+cs_base_strtf(bool  boolean)
+{
+  if (boolean)
+    return "**True**";
+  else
+    return "**False**";
+}
+
 /*----------------------------------------------------------------------------
  * First analysis of the command line to determine an application name.
  *
@@ -154,10 +181,13 @@ cs_exit(int  status);
 
 /*----------------------------------------------------------------------------
  * Initialize error and signal handlers.
+ *
+ * parameters:
+ *   signal_defaults <-- leave default signal handlers in place if true.
  *----------------------------------------------------------------------------*/
 
 void
-cs_base_error_init(void);
+cs_base_error_init(bool  signal_defaults);
 
 /*----------------------------------------------------------------------------
  * Initialize management of memory allocated through BFT.
@@ -182,24 +212,48 @@ cs_base_mem_finalize(void);
 void
 cs_base_time_summary(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Update status file.
+ *
+ * If the format string is NULL, the file is removed.
+
+ * \param[in]  format  format string, or NULL
+ * \param[in]  ...     format arguments
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_update_status(const char  *format,
+                      ...);
+
+/*----------------------------------------------------------------------------
+ * Set tracing of progress on or off.
+ *
+ * parameters:
+ *   trace  <-- trace progress to stdout
+ *----------------------------------------------------------------------------*/
+
+void
+cs_base_trace_set(bool trace);
+
+
 /*----------------------------------------------------------------------------
  * Set output file name and suppression flag for bft_printf().
  *
  * This allows redirecting or suppressing logging for different ranks.
  *
  * parameters:
- *   log_name    <-- base file name for log, or NULL for stdout
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to <log_name> file
+ *   log_name    <-- base file name for log
  *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to <log_name>_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
+ *   rn_log_flag <-- redirection for ranks > 0 log:
+ *                   false: to "/dev/null" (suppressed)
+ *                   true: to <log_name>_r*.log" file;
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_bft_printf_init(const char  *log_name,
-                        int          r0_log_flag,
-                        int          rn_log_flag);
+                        bool         rn_log_flag);
 
 /*----------------------------------------------------------------------------
  * Replace default bft_printf() mechanism with internal mechanism.
@@ -208,17 +262,11 @@ cs_base_bft_printf_init(const char  *log_name,
  *
  * parameters:
  *   log_name    <-- base file name for log
- *   r0_log_flag <-- redirection for rank 0 log;
- *                   0: not redirected; 1: redirected to "listing" file
- *   rn_log_flag <-- redirection for ranks > 0 log:
- *                   0: not redirected; 1: redirected to "listing_n*" file;
- *                   2: redirected to "/dev/null" (suppressed)
  *----------------------------------------------------------------------------*/
 
 void
 cs_base_bft_printf_set(const char  *log_name,
-                       int          r0_log_flag,
-                       int          rn_log_flag);
+                       bool         rn_log_flag);
 
 /*----------------------------------------------------------------------------
  * Return name of default log file.
@@ -352,6 +400,141 @@ cs_base_get_pkgdatadir(void);
 const char *
 cs_base_get_pkglibdir(void);
 
+/*----------------------------------------------------------------------------
+ * Ensure bool argument has value 0 or 1.
+ *
+ * This allows working around issues with Intel compiler C bindings,
+ * which seem to pass incorrect values in some cases.
+ *
+ * parameters:
+ *   b <-> pointer to bool
+ *----------------------------------------------------------------------------*/
+
+void
+cs_base_check_bool(bool *b);
+
+/*----------------------------------------------------------------------------
+ * Open a data file in read mode.
+ *
+ * If a file of the given name in the working directory is found, it
+ * will be opened. Otherwise, it will be searched for in the "data/thch"
+ * subdirectory of pkgdatadir.
+ *
+ * parameters:
+ *   base_name      <-- base file name
+ *
+ * returns:
+ *   pointer to opened file
+ *----------------------------------------------------------------------------*/
+
+FILE *
+cs_base_open_properties_data_file(const char  *base_name);
+
+#if defined(HAVE_DLOPEN)
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Load a dynamic library.
+ *
+ * \param[in]  filename  path to shared library file.
+ *
+ * \return  handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void*
+cs_base_dlopen(const char *filename);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Load a plugin's dynamic library
+ *
+ * This function is similar to \ref cs_base_dlopen, execpt that only
+ * the base plugin file name (with no extension) needs to be given.
+ * It is assumed the file is available in the code's "pkglibdir" directory,
+ *
+ * \param[in]  name  path to shared library file
+ *
+ * \return  handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void*
+cs_base_dlopen_plugin(const char *name);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get flags for dlopen.
+ *
+ * \return  flags used for dlopen.
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_base_dlopen_get_flags(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set flags for dlopen.
+ *
+ * \param[in]  flags  flags to set
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_dlopen_set_flags(int flags);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Unload a dynamic library.
+ *
+ * Note that the dlopen underlying mechanism uses a reference count, so
+ * a library is really unloaded only one \ref cs_base_dlclose has been called
+ * the same number of times as \ref cs_base_dlopen.
+ *
+ * \param[in]  filename  optional path to shared library file name for error
+ *                       logging, or NULL
+ * \param[in]  handle    handle to shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_dlclose(const char  *filename,
+                void        *handle);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Get a shared library function pointer
+ *
+ * \param[in]  handle            handle to shared library
+ * \param[in]  name              name of function symbol in library
+ * \param[in]  errors_are_fatal  abort if true, silently ignore if false
+ *
+ * \return  pointer to function in shared library
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_base_get_dl_function_pointer(void        *handle,
+                                const char  *name,
+                                bool         errors_are_fatal);
+
+
+#endif /* defined(HAVE_DLOPEN) */
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Dump a stack trace to a file
+ *
+ * \param[in]  f         pointer to file in which to dump trace
+ * \param[in]  lv_start  start level in stack trace
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_base_backtrace_dump(FILE  *f,
+                       int    lv_start);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
