@@ -234,6 +234,14 @@ cs_base_fortran_bft_printf_to_f(void);
 void
 cs_user_extra_operations_initialize_wrapper(void);
 
+void
+cs_user_physical_properties_wrapper(void);
+
+void
+cs_user_source_terms_wrapper(int       f_id,
+                             cs_real_t *st_exp,
+                             cs_real_t *st_imp);
+
 void
 cs_user_extra_operations_wrapper(void);
 
