@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Macro definitions
  *============================================================================*/
@@ -65,6 +67,8 @@ BEGIN_C_DECLS
  * Local function defintions
  *============================================================================*/
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -129,6 +133,8 @@ cs_block_dist_compute_sizes(int        rank_id,
     bi.rank_step *= 2;
     _n_ranks = n_ranks / bi.rank_step;
   }
+  if (n_ranks % bi.rank_step)
+    _n_ranks += 1;
   if (bi.rank_step > n_ranks) {
     bi.rank_step = n_ranks;
     _n_ranks = 1;
