@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,6 +36,7 @@
  * PLE library headers
  *----------------------------------------------------------------------------*/
 
+#include <ple_defs.h>
 #include <ple_coupling.h>
 
 /*----------------------------------------------------------------------------
@@ -58,6 +59,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -91,6 +94,8 @@ static double _cs_coupling_ts_multiplier = 1.0;
  * Private function definitions
  *============================================================================*/
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions for Fortran API
  *============================================================================*/
@@ -170,7 +175,8 @@ void CS_PROCF(cplact, CPLACT)
 /*----------------------------------------------------------------------------*/
 
 void
-cs_coupling_discover_mpi_apps(const char  *app_name)
+cs_coupling_discover_mpi_apps(const char  *app_name,
+                              const char  *forced_app_type)
 {
   int mpi_flag;
   int world_size;
@@ -204,12 +210,20 @@ cs_coupling_discover_mpi_apps(const char  *app_name)
       bft_printf_flush();
     }
 
-    _cs_glob_coupling_mpi_app_world
-      = ple_coupling_mpi_set_create(_cs_coupling_sync_flag,
-                                    app_type,
-                                    app_name,
-                                    MPI_COMM_WORLD,
-                                    cs_glob_mpi_comm);
+    if (forced_app_type == NULL)
+      _cs_glob_coupling_mpi_app_world
+        = ple_coupling_mpi_set_create(_cs_coupling_sync_flag,
+                                      app_type,
+                                      app_name,
+                                      MPI_COMM_WORLD,
+                                      cs_glob_mpi_comm);
+    else
+      _cs_glob_coupling_mpi_app_world
+        = ple_coupling_mpi_set_create(_cs_coupling_sync_flag,
+                                      forced_app_type,
+                                      app_name,
+                                      MPI_COMM_WORLD,
+                                      cs_glob_mpi_comm);
 
     n_apps = ple_coupling_mpi_set_n_apps(_cs_glob_coupling_mpi_app_world);
     app_id = ple_coupling_mpi_set_get_app_id(_cs_glob_coupling_mpi_app_world);
@@ -437,7 +451,7 @@ cs_coupling_sync_apps(int      flags,
     }
     sync_flags = sync_flags | flags | stop_mask;
 
-    if (current_ts_id >= *max_ts_id)
+    if (current_ts_id >= *max_ts_id && *max_ts_id >= 0)
       sync_flags = sync_flags | PLE_COUPLING_STOP;
     else {
       sync_flags = sync_flags | PLE_COUPLING_NEW_ITERATION;
@@ -502,7 +516,7 @@ cs_coupling_sync_apps(int      flags,
       /* Handle time stepping behavior */
 
       if (app_status[i] & PLE_COUPLING_STOP) {
-        if (*max_ts_id > current_ts_id) {
+        if (*max_ts_id > current_ts_id || *max_ts_id < 0) {
           ai = ple_coupling_mpi_set_get_info(_cs_glob_coupling_mpi_app_world, i);
           bft_printf
             (_("\nApplication \"%s\" (%s) requested calculation stop.\n"),
@@ -528,7 +542,7 @@ cs_coupling_sync_apps(int      flags,
       }
 
       if (app_status[i] & PLE_COUPLING_LAST) {
-        if (*max_ts_id > current_ts_id + 1) {
+        if (*max_ts_id > current_ts_id + 1 || *max_ts_id < 0) {
           ai = ple_coupling_mpi_set_get_info(_cs_glob_coupling_mpi_app_world, i);
           bft_printf
             (_("\nApplication \"%s\" (%s) requested last iteration.\n"),
@@ -660,32 +674,40 @@ cs_coupling_mesh_extents(const void  *mesh,
  * Location is relative to the id of a given element + 1 in
  * concatenated sections of same element dimension.
  *
- * \param[in]       mesh          pointer to mesh representation structure
- * \param[in]       tolerance     associated tolerance
- * \param[in]       n_points      number of points to locate
- * \param[in]       point_coords  point coordinates
- * \param[in, out]  location      number of element containing or closest to
- *                                each point (size: n_points)
- * \param[in, out]  distance      distance from point to element indicated by
- *                                location[]: < 0 if unlocated, 0 - 1 if inside,
- *                                and > 1 if outside a volume element, or
- *                                absolute distance to a surface element
- *                                (size: n_points)
+ * \param[in]       mesh                pointer to mesh representation structure
+ * \param[in]       tolerance_base      associated base tolerance (for bounding
+ *                                      box check only, not for location test)
+ * \param[in]       tolerance_fraction  associated fraction of element bounding
+ *                                      boxes added to tolerance
+ * \param[in]       n_points            number of points to locate
+ * \param[in]       point_coords        point coordinates
+ * \param[in]       point_tag           optional point tag
+ * \param[in, out]  location            number of element containing or closest
+ *                                      to each point (size: n_points)
+ * \param[in, out]  distance            distance from point to element indicated
+ *                                      by location[]: < 0 if unlocated,
+ *                                      0 - 1 if inside, and > 1 if outside a
+ *                                      volume element, or absolute distance to
+ *                                      a surface element (size: n_points)
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_coupling_point_in_mesh(const void         *mesh,
-                          double              tolerance,
+                          float               tolerance_base,
+                          float               tolerance_fraction,
                           ple_lnum_t          n_points,
                           const ple_coord_t   point_coords[],
+                          const int           point_tag[],
                           ple_lnum_t          location[],
                           float               distance[])
 {
   fvm_point_location_nodal((const fvm_nodal_t *)mesh,
-                           tolerance,
+                           tolerance_base,
+                           tolerance_fraction,
                            0, /* Do not locate on parents */
                            n_points,
+                           point_tag,
                            point_coords,
                            location,
                            distance);
@@ -700,116 +722,45 @@ cs_coupling_point_in_mesh(const void         *mesh,
  *
  * Location is relative to parent element numbers.
  *
- * \param[in]       mesh          pointer to mesh representation structure
- * \param[in]       tolerance     associated tolerance
- * \param[in]       n_points      number of points to locate
- * \param[in]       point_coords  point coordinates
- * \param[in, out]  location      number of element containing or closest to
- *                                each point (size: n_points)
- * \param[in, out]  distance      distance from point to element indicated by
- *                                location[]: < 0 if unlocated, 0 - 1 if inside,
- *                                and > 1 if outside a volume element, or
- *                                absolute distance to a surface element
- *                                (size: n_points)
+ * \param[in]       mesh                pointer to mesh representation structure
+ * \param[in]       tolerance_base      associated base tolerance (for bounding
+ *                                      box check only, not for location test)
+ * \param[in]       tolerance_fraction  associated fraction of element bounding
+ *                                      boxes added to tolerance
+ * \param[in]       n_points            number of points to locate
+ * \param[in]       point_coords        point coordinates
+ * \param[in]       point_tag           optional point tag
+ * \param[in, out]  location            number of element containing or closest
+ *                                      to each point (size: n_points)
+ * \param[in, out]  distance            distance from point to element indicated
+ *                                      by location[]: < 0 if unlocated,
+ *                                      0 - 1 if inside, and > 1 if outside a
+ *                                      volume element, or absolute distance to
+ *                                      a surface element (size: n_points)
  */
 /*----------------------------------------------------------------------------*/
 
 void
 cs_coupling_point_in_mesh_p(const void         *mesh,
-                            double              tolerance,
+                            float               tolerance_base,
+                            float               tolerance_fraction,
                             ple_lnum_t          n_points,
                             const ple_coord_t   point_coords[],
+                            const int           point_tag[],
                             ple_lnum_t          location[],
                             float               distance[])
 {
   fvm_point_location_nodal((const fvm_nodal_t *)mesh,
-                           tolerance,
+                           tolerance_base,
+                           tolerance_fraction,
                            1, /* Locate on parents */
                            n_points,
+                           point_tag,
                            point_coords,
                            location,
                            distance);
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Find elements in a given mesh closest to points: updates the
- * location[] and distance[] arrays associated with a set of points
- * for points that are closer to an element of this mesh than to previously
- * encountered elements.
- *
- * This function currently only handles elements of lower dimension than
- * the spatial dimension.
- *
- * Location is relative to the id of a given element + 1 in
- * concatenated sections of same element dimension.
- *
- * \param[in]       mesh          pointer to mesh representation structure
- * \param[in]       n_points      number of points to locate
- * \param[in]       point_coords  point coordinates
- * \param[in, out]  location      number of element containing or closest to
- *                                each point (size: n_points)
- * \param[in, out]  distance      distance from point to element indicated by
- *                                location[]: < 0 if unlocated, or absolute
- *                                distance to a surface element
- *                                (size: n_points)
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_coupling_point_closest_mesh(const void         *mesh,
-                               ple_lnum_t          n_points,
-                               const ple_coord_t   point_coords[],
-                               ple_lnum_t          location[],
-                               float               distance[])
-{
-  fvm_point_location_closest_nodal((const fvm_nodal_t *)mesh,
-                                   0, /* Do not locate_on_parents */
-                                   n_points,
-                                   point_coords,
-                                   location,
-                                   distance);
-}
-
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Find elements in a given mesh closest to points: updates the
- * location[] and distance[] arrays associated with a set of points
- * for points that are closer to an element of this mesh than to previously
- * encountered elements.
- *
- * This function currently only handles elements of lower dimension than
- * the spatial dimension.
- *
- * Location is relative to parent element numbers.
- *
- * \param[in]       mesh          pointer to mesh representation structure
- * \param[in]       n_points      number of points to locate
- * \param[in]       point_coords  point coordinates
- * \param[in, out]  location      number of element containing or closest to
- *                                each point (size: n_points)
- * \param[in, out]  distance      distance from point to element indicated by
- *                                location[]: < 0 if unlocated, or absolute
- *                                distance to a surface element
- *                                (size: n_points)
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_coupling_point_closest_mesh_p(const void         *mesh,
-                                 ple_lnum_t          n_points,
-                                 const ple_coord_t   point_coords[],
-                                 ple_lnum_t          location[],
-                                 float               distance[])
-{
-  fvm_point_location_closest_nodal((const fvm_nodal_t *)mesh,
-                                   1, /* Locate_on_parents */
-                                   n_points,
-                                   point_coords,
-                                   location,
-                                   distance);
-}
-
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
