@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -58,6 +58,11 @@ extern "C" {
 
 #else
 
+#  define _LIBINTL_H /* Prevent inclusion of <libintl.h> by other files
+                        with incorrect or missing checks;
+                        TODO locate files causing issues to avoid
+                        requiring this workaround */
+
 #  define _(String) (String)
 #  define N_(String) String
 #  define textdomain(String) (String)
@@ -258,6 +263,7 @@ typedef enum {
   CS_CHAR,               /* character values */
   CS_FLOAT,              /* 4-byte floating point values */
   CS_DOUBLE,             /* 8-byte floating point values */
+  CS_UINT16,             /* 2-byte unsigned integer values */
   CS_INT32,              /* 4-byte signed integer values */
   CS_INT64,              /* 8-byte signed integer values */
   CS_UINT32,             /* 4-byte unsigned integer values */
@@ -289,16 +295,18 @@ typedef enum {
 
 /* Other types */
 
-typedef int      cs_lnum_t;     /* Local integer index or number */
-typedef double   cs_coord_t;    /* Real number (coordinate value) */
+typedef int                 cs_lnum_t;    /* Local integer index or number */
+typedef double              cs_coord_t;   /* Real number (coordinate value) */
 
-typedef int      cs_int_t;      /* Fortran integer */
-typedef double   cs_real_t;     /* Fortran double precision */
-typedef char     cs_byte_t;     /* Byte (untyped memory unit) */
+typedef int                 cs_int_t;    /* Fortran integer */
+typedef double              cs_real_t;   /* Fortran double precision */
+typedef char                cs_byte_t;   /* Byte (untyped memory unit) */
+typedef unsigned short int  cs_flag_t;   /* Flag for storing metadata */
 
 /* Vector or array block types */
 
-typedef cs_lnum_t  cs_lnum_2_t[2];      /* Vector of 2 local numbers */
+typedef int        cs_lnum_2_t[2];      /* Vector of 2 local numbers */
+typedef int        cs_lnum_3_t[3];      /* Vector of 3 local numbers */
 
 typedef double     cs_coord_3_t[3];     /* Vector of 3 real (coordinate)
                                            values */
@@ -308,9 +316,36 @@ typedef cs_real_t  cs_real_3_t[3];      /* Vector of 3 real values */
 typedef cs_real_t  cs_real_4_t[4];      /* Vector of 4 real values */
 typedef cs_real_t  cs_real_6_t[6];      /* Vector of 6 real values
                                            (for symmetric tensor) */
+typedef cs_real_t  cs_real_9_t[9];      /* Vector of 9 real values */
+
 typedef cs_real_t  cs_real_33_t[3][3];  /* Matrix of 3x3 real values */
 typedef cs_real_t  cs_real_66_t[6][6];  /* Matrix of 6x6 real values */
-typedef cs_real_33_t  cs_real_332_t[2]; /* vector of 2 3x3 matrices of real values */
+typedef cs_real_t  cs_real_99_t[9][9];  /* Matrix of 9x9 real values */
+
+typedef cs_real_t  cs_real_34_t[3][4];  /* Matrix of 3x4 real values */
+
+typedef cs_real_t  cs_real_63_t[6][3];  /* Matrix of 6x3 real values */
+
+typedef cs_real_33_t  cs_real_332_t[2];  /* vector of 2 3x3 matrices
+                                            of real values */
+typedef cs_real_66_t  cs_real_662_t[2];  /* vector of 2 6x6 matrices
+                                            of real values */
+
+typedef struct {
+
+  double   val;  /* Value */
+  int      id;   /* Id related to value */
+
+} cs_double_int_t;
+
+/* Vector-valued quantity stored using its measure (i.e. length) and
+   its direction given by a unitary vector */
+typedef struct {
+
+  double  meas;
+  double  unitv[3];
+
+} cs_nvec3_t;
 
 /* Mappings to MPI datatypes */
 /*---------------------------*/
@@ -339,8 +374,9 @@ typedef cs_real_33_t  cs_real_332_t[2]; /* vector of 2 3x3 matrices of real valu
 #    define CS_MPI_GNUM       MPI_UNSIGNED
 #  endif
 
-#  define CS_MPI_LNUM     MPI_INT         /* MPI type for cs_lnum_t type */
-#  define CS_MPI_COORD    MPI_DOUBLE      /* MPI type for cs_coord_t type */
+#  define CS_MPI_FLAG     MPI_UNSIGNED_SHORT /* MPI type for cs_flag_t type */
+#  define CS_MPI_LNUM     MPI_INT            /* MPI type for cs_lnum_t type */
+#  define CS_MPI_COORD    MPI_DOUBLE         /* MPI type for cs_coord_t type */
 
 #endif /* defined(HAVE_MPI) && !defined(CS_IGNORE_MPI) */
 
@@ -367,9 +403,25 @@ typedef cs_real_33_t  cs_real_332_t[2]; /* vector of 2 3x3 matrices of real valu
 # define CS_INT_TYPE      CS_INT32
 #endif
 
+#define CS_FLAG_TYPE      CS_UINT16
 #define CS_REAL_TYPE      CS_DOUBLE
 #define CS_COORD_TYPE     CS_DOUBLE
 
+/* Minimum size for OpenMP loops
+ *  (based on initial benchmarking, which will need updates)
+ *-----------------------------------------------------------*/
+
+#if defined(__bgq__) && defined(__xlc__)
+#  define CS_THR_MIN 1014
+#else
+#  define CS_THR_MIN 128
+#endif
+
+/* Cache line size, or multiple thereof */
+/*--------------------------------------*/
+
+#define CS_CL_SIZE 64
+
 /*----------------------------------------------------------------------------
  * Type independent min an max (caution: the argument is evaluated)
  *----------------------------------------------------------------------------*/
@@ -391,6 +443,16 @@ typedef enum {
 
 } cs_interlace_t;
 
+/*----------------------------------------------------------------------------
+ * Macro used to silence "unused argument" warnings.
+ *
+ * This is useful when a function must match a given function pointer
+ * type, but does not use all possible arguments.
+ *----------------------------------------------------------------------------*/
+
+#define CS_UNUSED(x) (void)(x)
+#define CS_NO_WARN_IF_UNUSED(x) (void)(x)
+
 /*----------------------------------------------------------------------------
  * Macros for compilation with a C++ compiler
  *----------------------------------------------------------------------------*/
@@ -464,6 +526,28 @@ extern MPI_Comm       cs_glob_mpi_comm;      /* Main MPI intra-communicator */
 
 #endif
 
+/*=============================================================================
+ * Public functions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Given a base index i, return the next index aligned with a size m.
+ *
+ * \param[in]  i   base index
+ * \param[in]  m   block size to align with
+ *
+ * \return aligned index
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static cs_lnum_t
+cs_align(cs_lnum_t  i,
+         cs_lnum_t  m)
+{
+  return ((i > 0) ? ((i-1)/m+1)*m : 0);
+}
+
 /*----------------------------------------------------------------------------*/
 
 #ifdef __cplusplus
