@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -64,12 +64,12 @@ module cs_f_interfaces
     !---------------------------------------------------------------------------
 
     subroutine itrmav &
-     (init, inc, imrgra, iccocg, nswrgp, imligp, ircflp,                       &
-     iphydp, iwarnp, epsrgp, climgp, extrap, frcxt,                            &
+     (f_id, init, inc, imrgra, iccocg, nswrgp, imligp, ircflp,                 &
+     iphydp, iwgrp, iwarnp, epsrgp, climgp, extrap, frcxt,                     &
      pvar, coefap, coefbp, cofafp, cofbfp, viscf, viscb, viscel,               &
      weighf, weighb, flumas, flumab)
       use mesh
-      integer :: init, inc, imrgra
+      integer :: f_id, init, inc, imrgra
       integer :: iccocg, nswrgp, imligp, ircflp
       integer :: iwarnp, iphydp
       double precision :: epsrgp , climgp , extrap
@@ -88,13 +88,13 @@ module cs_f_interfaces
     !---------------------------------------------------------------------------
 
     subroutine itrgrv &
-      (init, inc, imrgra, iccocg, nswrgp, imligp, ircflp,                      &
+      (f_id, init, inc, imrgra, iccocg, nswrgp, imligp, ircflp,                &
       iphydp, iwarnp,                                                          &
       epsrgp, climgp, extrap, frcxt,                                           &
       pvar, coefap, coefbp, cofafp, cofbfp, viscf, viscb, viscel,              &
       weighf, weighb, diverg)
       use mesh
-      integer :: init, inc, imrgra, iccocg, nswrgp, imligp, ircflp
+      integer :: f_id, init, inc, imrgra, iccocg, nswrgp, imligp, ircflp
       integer :: iwarnp , iphydp
       double precision :: epsrgp, climgp, extrap
       double precision, dimension(ncelet) :: pvar
@@ -110,66 +110,76 @@ module cs_f_interfaces
 
     !---------------------------------------------------------------------------
 
+    subroutine matrix &
+      (iconvp, idiffp, ndircp, isym, thetap, imucpp, coefbp, cofbfp,           &
+      rovsdt, i_massflux, b_massflux, i_visc, b_visc, xcpp, da, xa)
+      use mesh
+      integer :: iconvp, idiffp, ndircp, isym, imucpp
+      double precision :: thetap
+      double precision, dimension(ncelet) :: rovsdt, xcpp, da
+      double precision, dimension(nfabor) :: coefbp, cofbfp, b_massflux, b_visc
+      double precision, dimension(nfac) :: i_massflux, i_visc
+      double precision, dimension(2,nfac) :: xa
+    end subroutine matrix
+
+    !---------------------------------------------------------------------------
+
+    subroutine matrdt &
+      (iconvp, idiffp, isym, coefbp, cofbfp,                                   &
+      i_massflux, b_massflux, i_visc, b_visc, da)
+      use mesh
+      integer :: iconvp, idiffp, isym
+      double precision, dimension(ncelet) :: da
+      double precision, dimension(nfabor) :: coefbp, cofbfp, b_massflux, b_visc
+      double precision, dimension(nfac) :: i_massflux, i_visc
+    end subroutine matrdt
+
+    !---------------------------------------------------------------------------
+
     subroutine post_boundary_thermal_flux &
-      (nfbrps, lstfbr, propce, bflux)
+      (nfbrps, lstfbr, bflux)
       use dimens
       use mesh
       integer, intent(in)                                        :: nfbrps
       integer, dimension(nfbrps), intent(in)                     :: lstfbr
-      double precision, dimension(ncelet, *), intent(in), target :: propce
       double precision, dimension(nfbrps), intent(out)           :: bflux
     end subroutine post_boundary_thermal_flux
 
     !---------------------------------------------------------------------------
 
-    subroutine post_boundary_temperature &
-      (nfbrps, lstfbr, propce, btemp)
-      use dimens
-      use mesh
-      integer, intent(in)                                        :: nfbrps
-      integer, dimension(nfbrps), intent(in)                     :: lstfbr
-      double precision, dimension(ncelet, *), intent(in), target :: propce
-      double precision, dimension(nfbrps), intent(out)           :: btemp
-    end subroutine post_boundary_temperature
-
-    !---------------------------------------------------------------------------
-
     subroutine post_boundary_nusselt &
-      (nfbrps, lstfbr, propce, bnussl)
+      (nfbrps, lstfbr, bnussl)
       use dimens
       use mesh
       integer, intent(in)                                        :: nfbrps
       integer, dimension(nfbrps), intent(in)                     :: lstfbr
-      double precision, dimension(ncelet, *), intent(in), target :: propce
       double precision, dimension(nfbrps), intent(out)           :: bnussl
     end subroutine post_boundary_nusselt
 
     !---------------------------------------------------------------------------
 
-    subroutine post_efforts &
-      (nfbrps, lstfbr, effort)
+    subroutine post_stress &
+      (nfbrps, lstfbr, stress)
       use dimens
       use mesh
       integer, intent(in)                                 :: nfbrps
       integer, dimension(nfbrps), intent(in)              :: lstfbr
-      double precision, dimension(3, nfbrps), intent(out) :: effort
-    end subroutine post_efforts
+      double precision, dimension(3, nfbrps), intent(out) :: stress
+    end subroutine post_stress
 
     !---------------------------------------------------------------------------
 
     subroutine turrij &
       (nvar, nscal, ncepdp, ncesmp, icepdc, icetsm, itypsm,                    &
-      dt, rtp, rtpa, propce, tslagr,                                           &
+      dt, tslagr,                                                              &
       ckupdc, smacel)
-      use dimens, only: ndimfb
-      use lagdim, only: ntersl
+      use lagran, only: ntersl
       use mesh
       integer :: nvar, nscal, ncepdp, ncesmp
       integer, dimension(ncepdp) :: icepdc
       integer, dimension(ncesmp) :: icetsm
       integer, dimension(ncesmp,nvar), target :: itypsm
       double precision, dimension(ncelet) :: dt
-      double precision, dimension(ncelet, *) :: rtp, rtpa, propce
       double precision, dimension(ncelet,ntersl), target :: tslagr
       double precision, dimension(ncepdp,6) :: ckupdc
       double precision, dimension(ncesmp,nvar), target ::  smacel
