@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,22 +41,44 @@ BEGIN_C_DECLS
  * Macro definitions
  *============================================================================*/
 
+/*!
+ * @defgroup field_flags Flags specifying general field attributes
+ *
+ * @{
+ */
+
 /*
  * Field property type
  */
 
+/*! represents an intensive quantity */
 #define CS_FIELD_INTENSIVE           (1 << 0)
+
+/*! represents an extensive quantity */
 #define CS_FIELD_EXTENSIVE           (1 << 1)
 
 /* Field category */
 
+/*! resolved variable */
 #define CS_FIELD_VARIABLE            (1 << 2)
+
+/*! computed property */
 #define CS_FIELD_PROPERTY            (1 << 3)
+
+/*! intended for postprocessing */
 #define CS_FIELD_POSTPROCESS         (1 << 4)
+
+/*! accumulator for some statisicial values */
 #define CS_FIELD_ACCUMULATOR         (1 << 5)
 
+/*! user-defined */
 #define CS_FIELD_USER                (1 << 6)
 
+/*! field associated to a CDO equation */
+#define CS_FIELD_CDO                 (1 << 7)
+
+/*! @} */
+
 /*============================================================================
  * Type definitions
  *============================================================================*/
@@ -70,7 +92,8 @@ typedef enum {
   CS_FIELD_INVALID_KEY_NAME,
   CS_FIELD_INVALID_KEY_ID,
   CS_FIELD_INVALID_CATEGORY,
-  CS_FIELD_INVALID_TYPE
+  CS_FIELD_INVALID_TYPE,
+  CS_FIELD_LOCKED
 
 } cs_field_error_type_t;
 
@@ -90,6 +113,9 @@ typedef struct {
   cs_real_t         *ac;           /* Explicit coefficient for convection */
   cs_real_t         *bc;           /* Implicit coefficient for convection */
 
+  cs_real_t         *hint;         /* coefficient for internal coupling */
+  cs_real_t         *hext;         /* coefficient for internal coupling */
+
 } cs_field_bc_coeffs_t;
 
 /* Field descriptor */
@@ -103,7 +129,6 @@ typedef struct {
   int                     type;         /* Field type flag */
 
   int                     dim;          /* Field dimension */
-  bool                    interleaved;  /* is field interleaved ? */
 
   int                     location_id;  /* Id of matching location */
 
@@ -173,8 +198,6 @@ cs_field_n_fields(void);
  *   type_flag    <-- mask of field property and category values
  *   location_id  <-- id of associated location
  *   dim          <-- field dimension (number of components)
- *   interleaved  <-- indicate if values are interleaved
- *                    (ignored if number of components < 2)
  *   has_previous <-- maintain values at the previous time step ?
  *
  * returns:
@@ -186,9 +209,36 @@ cs_field_create(const char   *name,
                 int           type_flag,
                 int           location_id,
                 int           dim,
-                bool          interleaved,
                 bool          has_previous);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a field matching a given name and attributes,
+ *        creating it if necessary.
+ *
+ * If a field with the same name but different attributes is present,
+ * this is considered an error.
+ *
+ * The default number of time values associated with a field created through
+ * this function is 1. To modify it, use \ref cs_field_set_n_time_vals.
+ *
+ * \param[in]  name          field name
+ * \param[in]  type_flag     mask of field property and category values
+ * \param[in]  location_id   id of associated location
+ * \param[in]  dim           field dimension (number of components)
+ * \param[in]  has_previous  maintain values at the previous time step ?
+ *
+ * \return  pointer to field
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_field_t *
+cs_field_find_or_create(const char   *name,
+                        int           type_flag,
+                        int           location_id,
+                        int           dim,
+                        bool          has_previous);
+
 /*----------------------------------------------------------------------------
  * Change the number of time values managed by a field.
  *
@@ -237,26 +287,25 @@ cs_field_map_values(cs_field_t   *f,
  * locations (though support could be added by mapping a boundary->location
  * indirection array in the cs_mesh_location_t structure).
  *
- * For multidimensional fields, arrays are assumed to have the same
- * interleaving behavior as the field, unless components are coupled.
- *
- * For multidimensional fields with coupled components, interleaving
- * is the norm, and implicit b and bf coefficient arrays are arrays of
- * block matrices, not vectors, so the number of entries for each boundary
- * face is dim*dim instead of dim.
+ * For multidimensional fields with coupled components, implicit b and bf
+ * coefficient arrays are arrays of block matrices, not vectors, so the
+ * number of entries for each boundary face is dim*dim instead of dim.
  *
  * parameters:
  *   f            <-- pointer to field structure
  *   have_flux_bc <-- if true, flux BC coefficients (af and bf) are added
  *   have_mom_bc  <-- if true, div BC coefficients (ad and bd) are added
  *   have_conv_bc <-- if true, convection BC coefficients (ac and bc) are added
+ *   have_exch_bc <-- if true, exchange boundary coefficients (hint and hext)
+ *                    are added
  *----------------------------------------------------------------------------*/
 
 void
 cs_field_allocate_bc_coeffs(cs_field_t  *f,
                             bool         have_flux_bc,
                             bool         have_mom_bc,
-                            bool         have_conv_bc);
+                            bool         have_conv_bc,
+                            bool         have_exch_bc);
 
 /*----------------------------------------------------------------------------*/
 /* Initialize boundary condition coefficients arrays.
@@ -268,13 +317,9 @@ cs_field_allocate_bc_coeffs(cs_field_t  *f,
  * locations (though support could be added by mapping a boundary->location
  * indirection array in the cs_mesh_location_t structure).
  *
- * For multidimensional fields, arrays are assumed to have the same
- * interleaving behavior as the field, unless components are coupled.
- *
- * For multidimensional fields with coupled components, interleaving
- * is the norm, and implicit b and bf coefficient arrays are arrays of
- * block matrices, not vectors, so the number of entries for each boundary
- * face is dim*dim instead of dim.
+ * For multidimensional fields with coupled components, implicit b and bf
+ * coefficient arrays are arrays of block matrices, not vectors, so the
+ * number of entries for each boundary face is dim*dim instead of dim.
  *
  * parameters:
  *   f <-> pointer to field structure
@@ -283,6 +328,18 @@ cs_field_allocate_bc_coeffs(cs_field_t  *f,
 void
 cs_field_init_bc_coeffs(cs_field_t  *f);
 
+/*----------------------------------------------------------------------------
+ * Set current field values to the given constant.
+ *
+ * parameters:
+ *   f <-> pointer to field structure
+ *   c <-- assigned value
+ *----------------------------------------------------------------------------*/
+
+void
+cs_field_set_values(cs_field_t  *f,
+                    cs_real_t    c);
+
 /*----------------------------------------------------------------------------
  * Copy current field values to previous values if applicable.
  *
@@ -495,12 +552,13 @@ cs_field_define_key_str(const char  *name,
  * by the current value, and its id is returned.
  *
  * parameters:
- *   name          <-- key name
- *   default_value <-- pointer to default value associated with key
- *   log_funct     <-- pointer to logging function
- *   size          <-- sizeof structure
- *   type_flag     <-- mask associated with field types with which
- *                     the key may be associated, or 0
+ *   name             <-- key name
+ *   default_value    <-- pointer to default value associated with key
+ *   log_funct        <-- pointer to logging function
+ *   log_func_default <-- pointer to default logging function
+ *   size             <-- sizeof structure
+ *   type_flag        <-- mask associated with field types with which
+ *                        the key may be associated, or 0
  *
  * returns:
  *   id associated with key
@@ -510,6 +568,7 @@ int
 cs_field_define_key_struct(const char                 *name,
                            const void                 *default_value,
                            cs_field_log_key_struct_t  *log_func,
+                           cs_field_log_key_struct_t  *log_func_default,
                            size_t                      size,
                            int                         type_flag);
 
@@ -555,6 +614,19 @@ cs_field_destroy_all_keys(void);
 int
 cs_field_key_flag(int key_id);
 
+/*----------------------------------------------------------------------------
+ * Disable logging setup values associated with a given key.
+ *
+ * This is useful when a key is used not for setup purposes, but to track
+ * values associated with a field, such as convergence or performance data.
+ *
+ * parameters:
+ *   key_id <-- id of associated key
+ *----------------------------------------------------------------------------*/
+
+void
+cs_field_key_disable_setup_log(int  key_id);
+
 /*----------------------------------------------------------------------------
  * Query if a given key has been set for a field.
  *
@@ -562,7 +634,7 @@ cs_field_key_flag(int key_id);
  * compatible, a fatal error is provoked.
  *
  * parameters:
- *   f             <-- pointer to field structure
+ *   f      <-- pointer to field structure
  *   key_id <-- id of associated key
  *
  * returns:
@@ -573,6 +645,44 @@ bool
 cs_field_is_key_set(const cs_field_t  *f,
                     int                key_id);
 
+/*----------------------------------------------------------------------------
+ * Query if a given key has been locked for a field.
+ *
+ * If the key id is not valid, or the field category is not
+ * compatible, a fatal error is provoked.
+ *
+ * parameters:
+ *   f      <-- pointer to field structure
+ *   key_id <-- id of associated key
+ *
+ * returns:
+ *   true if the key has been locked for this field, false otherwise
+ *----------------------------------------------------------------------------*/
+
+bool
+cs_field_is_key_locked(const cs_field_t  *f,
+                       int                key_id);
+
+/*----------------------------------------------------------------------------
+ * Lock a field relative to a given key.
+ *
+ * If the key id is not valid, CS_FIELD_INVALID_KEY_ID is returned.
+ * If the field category is not compatible with the key (as defined
+ * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
+ *
+ * parameters:
+ *   f      <-- pointer to field structure
+ *   key_id <-- id of associated key
+ *   value  <-- value associated with key
+ *
+ * returns:
+ *   0 in case of success, > 1 in case of error
+ *----------------------------------------------------------------------------*/
+
+int
+cs_field_lock_key(cs_field_t  *f,
+                  int          key_id);
+
 /*----------------------------------------------------------------------------
  * Assign a integer value for a given key to a field.
  *
@@ -580,6 +690,7 @@ cs_field_is_key_set(const cs_field_t  *f,
  * If the field category is not compatible with the key (as defined
  * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
  * If the data type does not match, CS_FIELD_INVALID_TYPE is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
  *
  * parameters:
  *   f      <-- pointer to field structure
@@ -613,6 +724,52 @@ int
 cs_field_get_key_int(const cs_field_t  *f,
                      int                key_id);
 
+/*----------------------------------------------------------------------------
+ * Set integer bits matching a mask to 1 for a given key for a field.
+ *
+ * If the key id is not valid, CS_FIELD_INVALID_KEY_ID is returned.
+ * If the field category is not compatible with the key (as defined
+ * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
+ * If the data type does not match, CS_FIELD_INVALID_TYPE is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
+ *
+ * parameters:
+ *   f             <-- pointer to field structure
+ *   key_id        <-- id of associated key
+ *   mask          <-- mask associated with key
+ *
+ * returns:
+ *   0 in case of success, > 1 in case of error
+ *----------------------------------------------------------------------------*/
+
+int
+cs_field_set_key_int_bits(cs_field_t  *f,
+                          int          key_id,
+                          int          mask);
+
+/*----------------------------------------------------------------------------
+ * Set integer bits matching a mask to 0 for a given key for a field.
+ *
+ * If the key id is not valid, CS_FIELD_INVALID_KEY_ID is returned.
+ * If the field category is not compatible with the key (as defined
+ * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
+ * If the data type does not match, CS_FIELD_INVALID_TYPE is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
+ *
+ * parameters:
+ *   f             <-- pointer to field structure
+ *   key_id        <-- id of associated key
+ *   mask          <-- mask associated with key
+ *
+ * returns:
+ *   0 in case of success, > 1 in case of error
+ *----------------------------------------------------------------------------*/
+
+int
+cs_field_clear_key_int_bits(cs_field_t  *f,
+                            int          key_id,
+                            int          mask);
+
 /*----------------------------------------------------------------------------
  * Assign a floating point value for a given key to a field.
  *
@@ -620,6 +777,7 @@ cs_field_get_key_int(const cs_field_t  *f,
  * If the field category is not compatible with the key (as defined
  * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
  * If the data type does not match, CS_FIELD_INVALID_TYPE is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
  *
  * parameters:
  *   f      <-- pointer to field structure
@@ -660,6 +818,7 @@ cs_field_get_key_double(const cs_field_t  *f,
  * If the field category is not compatible with the key (as defined
  * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
  * If the data type does not match, CS_FIELD_INVALID_TYPE is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
  *
  * parameters:
  *   f      <-- pointer to field structure
@@ -700,6 +859,7 @@ cs_field_get_key_str(const cs_field_t  *f,
  * If the key id is not valid, CS_FIELD_INVALID_KEY_ID is returned.
  * If the field category is not compatible with the key (as defined
  * by its type flag), CS_FIELD_INVALID_CATEGORY is returned.
+ * If the key value has been locked, CS_FIELD_LOCKED is returned.
  *
  * parameters:
  *   f      <-- pointer to field structure
@@ -736,6 +896,47 @@ cs_field_get_key_struct(const cs_field_t  *f,
                         int                key_id,
                         void              *s);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a pointer to a simple structure for a given key to a field.
+ *
+ * If the key id is not valid, the value type or field category is not
+ * compatible, or the structure has been locked, a fatal error is provoked.
+ *
+ * Note that using this function marks the field's value for this structure
+ * as set, and if no values have been set yet, the structure is set to
+ * default values.
+ *
+ * \param[in]  f       pointer to field structure
+ * \param[in]  key_id  id of associated key
+ *
+ * \return  pointer to key structure in case of success, NULL in case of error
+ */
+/*----------------------------------------------------------------------------*/
+
+void *
+cs_field_get_key_struct_ptr(cs_field_t  *f,
+                            int          key_id);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Return a read-only pointer to a simple structure for a given key
+ *        to a field.
+ *
+ * If the key id is not valid, the value type or field category is not
+ * compatible, a fatal error is provoked.
+ *
+ * \param[in]  f       pointer to field structure
+ * \param[in]  key_id  id of associated key
+ *
+ * \return  pointer to key structure in case of success, NULL in case of error
+ */
+/*----------------------------------------------------------------------------*/
+
+const void *
+cs_field_get_key_struct_const_ptr(const cs_field_t  *f,
+                                  int                key_id);
+
 /*----------------------------------------------------------------------------
  * Print info relative to all field definitions to log file.
  *----------------------------------------------------------------------------*/
@@ -805,7 +1006,6 @@ cs_field_log_all_key_vals(bool  log_defaults);
  *   "label"        (string)
  *   "log"          (integer)
  *   "post_vis"     (integer)
- *   "post_probes"  (integer)
  *   "coupled"      (integer, restricted to CS_FIELD_VARIABLE)
  *   "moment_id"    (integer, restricted to
  *                   CS_FIELD_ACCUMULATOR | CS_FIELD_POSTPROCESS);
