@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -66,6 +66,37 @@ BEGIN_C_DECLS
         Field pointers and ids for standard and model fields.
 */
 
+/*! \fn CS_ENUMF_(e)
+ * \brief Macro used for scoping of field pointer enums.
+ *
+ * This macro replaces CS_ENUMF_ by CS_FIELD_POINTER_ and allows
+ * to rebuild a full enumerated field pointer id.
+ *
+ * \param [in] e suffix of enumerated field pointer id.
+ */
+
+/*! \fn CS_F_(e)
+ * \brief Macro used to return a field pointer by its enumerated value.
+ *
+ * This macro replaces CS_F_ by an access to the global array of field pointers
+ * \ref cs_glob_field_pointers using a rebuilt enumerated field pointer id.
+ *
+ * \param [in] e suffix of enumerated field pointer id.
+ */
+
+/*! \fn CS_FI_(e, i)
+ * \brief Macro used to return a field pointer by its enumerated value.
+ *
+ * This macro replaces CS_FI_ by an access to the global array of field pointers
+ * \ref cs_glob_field_pointers using a rebuilt enumerated field pointer id and
+ * its field sublist index.
+ *
+ * \param [in] e suffix of enumerated field pointer id.
+ * \param [in] i field enumerator value.
+ */
+
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local macro definitions
  *============================================================================*/
@@ -74,8 +105,6 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
-/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
-
 /*============================================================================
  * Static global variables
  *============================================================================*/
@@ -90,6 +119,8 @@ static union cs_field_pointer_val_t  *_field_pointer = NULL;
 
 static bool  *_is_sublist = NULL;
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Global variables
  *============================================================================*/
@@ -98,6 +129,8 @@ static bool  *_is_sublist = NULL;
 
 union cs_field_pointer_val_t  *cs_glob_field_pointers = NULL;
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -123,6 +156,8 @@ _init_pointers(void)
   cs_glob_field_pointers = _field_pointer;
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
@@ -178,7 +213,7 @@ cs_field_pointer_map(cs_field_pointer_id_t   e,
  * This sort of mapping may be used for sets of fields whose size
  * is not known in advance.
  *
- * The associated field pointer may then be retreived using \ref CS_F_(e, i).
+ * The associated field pointer may then be retreived using \ref CS_FI_(e, i).
  *
  * \param[in]  e      field enumerator value
  * \param[in]  index  field enumerator index
@@ -258,7 +293,7 @@ cs_field_pointer_map_base(void)
 
   cs_field_pointer_map(CS_ENUMF_(p),
                        cs_field_by_name_try("pressure"));
-  cs_field_pointer_map(CS_ENUMF_(u),
+  cs_field_pointer_map(CS_ENUMF_(vel),
                        cs_field_by_name_try("velocity"));
 
   cs_field_pointer_map(CS_ENUMF_(k),
@@ -272,17 +307,23 @@ cs_field_pointer_map_base(void)
   cs_field_pointer_map(CS_ENUMF_(r12), cs_field_by_name_try("r12"));
   cs_field_pointer_map(CS_ENUMF_(r23), cs_field_by_name_try("r23"));
   cs_field_pointer_map(CS_ENUMF_(r13), cs_field_by_name_try("r13"));
+  cs_field_pointer_map(CS_ENUMF_(rij), cs_field_by_name_try("rij"));
 
   cs_field_pointer_map(CS_ENUMF_(phi), cs_field_by_name_try("phi"));
   cs_field_pointer_map(CS_ENUMF_(f_bar), cs_field_by_name_try("f_bar"));
-  cs_field_pointer_map(CS_ENUMF_(alpha), cs_field_by_name_try("alpha"));
+  cs_field_pointer_map(CS_ENUMF_(alp_bl), cs_field_by_name_try("alpha"));
 
   cs_field_pointer_map(CS_ENUMF_(omg), cs_field_by_name_try("omega"));
   cs_field_pointer_map(CS_ENUMF_(nusa), cs_field_by_name_try("nu_tilda"));
 
+  cs_field_pointer_map(CS_ENUMF_(hybrid_blend), cs_field_by_name_try("hybrid_blend"));
+
   cs_field_pointer_map(CS_ENUMF_(mesh_u),
                        cs_field_by_name_try("mesh_velocity"));
 
+  cs_field_pointer_map(CS_ENUMF_(void_f),
+                       cs_field_by_name_try("void_fraction"));
+
   cs_field_pointer_map(CS_ENUMF_(h),
                        cs_field_by_name_try("enthalpy"));
   cs_field_pointer_map(CS_ENUMF_(t),
@@ -290,8 +331,6 @@ cs_field_pointer_map_base(void)
 
   cs_field_pointer_map(CS_ENUMF_(rho),
                        cs_field_by_name_try("density"));
-  cs_field_pointer_map(CS_ENUMF_(rho_b),
-                       cs_field_by_name_try("boundary_density"));
 
   cs_field_pointer_map(CS_ENUMF_(cp),
                        cs_field_by_name_try("specific_heat"));
@@ -301,18 +340,42 @@ cs_field_pointer_map_base(void)
   cs_field_pointer_map(CS_ENUMF_(mu_t),
                        cs_field_by_name_try("turbulent_viscosity"));
 
+  cs_field_pointer_map(CS_ENUMF_(lambda),
+                       cs_field_by_name_try("thermal_conductivity"));
+  cs_field_pointer_map(CS_ENUMF_(th_diff),
+                       cs_field_by_name_try("thermal_diffusivity"));
+
+  cs_field_pointer_map(CS_ENUMF_(vism),
+                       cs_field_by_name_try("mesh_viscosity"));
+
   cs_field_pointer_map(CS_ENUMF_(poro),
                        cs_field_by_name_try("porosity"));
   cs_field_pointer_map(CS_ENUMF_(t_poro),
                        cs_field_by_name_try("tensorial_porosity"));
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Map some boundary fields to enumerated pointers.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_field_pointer_map_boundary(void)
+{
+  cs_field_pointer_map(CS_ENUMF_(t_b),
+                       cs_field_by_name_try("boundary_temperature"));
+
+  cs_field_pointer_map(CS_ENUMF_(rho_b),
+                       cs_field_by_name_try("boundary_density"));
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Map base fields to enumerated pointers for atmospheric models
  *
  * \param[in]  n_chem_species  number of chemical species
- * \param[in]  species_f_if    field id for each chemical species
+ * \param[in]  species_f_id    field id for each chemical species
  */
 /*----------------------------------------------------------------------------*/
 
@@ -353,55 +416,55 @@ cs_field_pointer_map_coal_combustion(int  n_coals,
                        cs_field_by_name_try("enthalpy"));
 
   for (int i = 0; i < n_classes; i++) {
-    snprintf(s, 63, "np_coal_%02d", i+1); s[63] = '\0';
+    snprintf(s, 63, "n_p_%02d", i+1); s[63] = '\0';
     cs_field_pointer_map_indexed(CS_ENUMF_(np), i, cs_field_by_name_try(s));
   }
 
   for (int i = 0; i < n_classes; i++) {
-    snprintf(s, 63, "x_coal_%02d", i+1); s[63] = '\0';
+    snprintf(s, 63, "x_p_coal_%02d", i+1); s[63] = '\0';
     cs_field_pointer_map_indexed(CS_ENUMF_(xch), i, cs_field_by_name_try(s));
   }
 
   for (int i = 0; i < n_classes; i++) {
-    snprintf(s, 63, "xck_coal_%02d", i+1); s[63] = '\0';
+    snprintf(s, 63, "x_p_char_%02d", i+1); s[63] = '\0';
     cs_field_pointer_map_indexed(CS_ENUMF_(xck), i, cs_field_by_name_try(s));
   }
 
   for (int i = 0; i < n_classes; i++) {
-    snprintf(s, 63, "xwt_coal_%02d", i+1); s[63] = '\0';
+    snprintf(s, 63, "x_p_wt_%02d", i+1); s[63] = '\0';
     cs_field_pointer_map_indexed(CS_ENUMF_(xwt), i, cs_field_by_name_try(s));
   }
 
   for (int i = 0; i < n_classes; i++) {
-    snprintf(s, 63, "h2_coal_%02d", i+1); s[63] = '\0';
+    snprintf(s, 63, "x_p_h_%02d", i+1); s[63] = '\0';
     cs_field_pointer_map_indexed(CS_ENUMF_(h2), i, cs_field_by_name_try(s));
   }
 
   for (int i = 0; i < n_coals; i++) {
-    snprintf(s, 63, "mv1_fraction_%02d", i+1); s[63] = '\0';
+    snprintf(s, 63, "fr_mv1_%02d", i+1); s[63] = '\0';
     cs_field_pointer_map_indexed(CS_ENUMF_(f1m), i, cs_field_by_name_try(s));
   }
 
   for (int i = 0; i < n_coals; i++) {
-    snprintf(s, 63, "mv2_fraction_%02d", i+1); s[63] = '\0';
+    snprintf(s, 63, "fr_mv2_%02d", i+1); s[63] = '\0';
     cs_field_pointer_map_indexed(CS_ENUMF_(f2m), i, cs_field_by_name_try(s));
   }
 
-  cs_field_pointer_map(CS_ENUMF_(f4m), cs_field_by_name_try("oxyd2_fraction"));
-  cs_field_pointer_map(CS_ENUMF_(f5m), cs_field_by_name_try("oxyd3_fraction"));
-  cs_field_pointer_map(CS_ENUMF_(f6m), cs_field_by_name_try("h2o_fraction"));
-  cs_field_pointer_map(CS_ENUMF_(f7m), cs_field_by_name_try("het_o2_fraction"));
-  cs_field_pointer_map(CS_ENUMF_(f8m), cs_field_by_name_try("het_co2_fraction"));
-  cs_field_pointer_map(CS_ENUMF_(f9m), cs_field_by_name_try("het_h2o_fraction"));
+  cs_field_pointer_map(CS_ENUMF_(f4m), cs_field_by_name_try("fr_oxyd2"));
+  cs_field_pointer_map(CS_ENUMF_(f5m), cs_field_by_name_try("fr_oxyd3"));
+  cs_field_pointer_map(CS_ENUMF_(f6m), cs_field_by_name_try("fr_h2o"));
+  cs_field_pointer_map(CS_ENUMF_(f7m), cs_field_by_name_try("fr_het_o2"));
+  cs_field_pointer_map(CS_ENUMF_(f8m), cs_field_by_name_try("fr_het_co2"));
+  cs_field_pointer_map(CS_ENUMF_(f9m), cs_field_by_name_try("fr_het_h2o"));
 
   cs_field_pointer_map(CS_ENUMF_(fvp2m), cs_field_by_name_try("f1f2_variance"));
 
-  cs_field_pointer_map(CS_ENUMF_(yco2), cs_field_by_name_try("co2_fraction"));
-  cs_field_pointer_map(CS_ENUMF_(yhcn), cs_field_by_name_try("hcn_fraction"));
-  cs_field_pointer_map(CS_ENUMF_(yno), cs_field_by_name_try("no_fraction"));
-  cs_field_pointer_map(CS_ENUMF_(ynh3), cs_field_by_name_try("nh3_fraction"));
+  cs_field_pointer_map(CS_ENUMF_(yco2), cs_field_by_name_try("x_c_co2"));
+  cs_field_pointer_map(CS_ENUMF_(yhcn), cs_field_by_name_try("x_c_hcn"));
+  cs_field_pointer_map(CS_ENUMF_(yno), cs_field_by_name_try("x_c_no"));
+  cs_field_pointer_map(CS_ENUMF_(ynh3), cs_field_by_name_try("x_c_nh3"));
 
-  cs_field_pointer_map(CS_ENUMF_(hox), cs_field_by_name_try("ox_enthalpy"));
+  cs_field_pointer_map(CS_ENUMF_(hox), cs_field_by_name_try("x_c_h_ox"));
 }
 
 /*----------------------------------------------------------------------------*/
@@ -413,38 +476,45 @@ cs_field_pointer_map_coal_combustion(int  n_coals,
 void
 cs_field_pointer_map_compressible(void)
 {
-  cs_field_pointer_map(CS_ENUMF_(energy),
+  cs_field_pointer_map(CS_ENUMF_(e_tot),
                        cs_field_by_name_try("total_energy"));
 
   cs_field_pointer_map(CS_ENUMF_(t_kelvin),
                        cs_field_by_name_try("temperature"));
+
+  /* Also map to main temperature pointer */
+
+  cs_field_pointer_map(CS_ENUMF_(t),
+                       cs_field_by_name_try("temperature"));
+
+  /* map volume specific heat if it is non constant */
+
+  cs_field_pointer_map(CS_ENUMF_(cv),
+                       cs_field_by_name_try("specific_heat_const_vol"));
+
+  /* map fractions for homogeneous two phase model */
+
+  cs_field_pointer_map(CS_ENUMF_(volume_f),
+                       cs_field_by_name_try("volume_fraction"));
+
+  cs_field_pointer_map(CS_ENUMF_(mass_f),
+                       cs_field_by_name_try("mass_fraction"));
+
+  cs_field_pointer_map(CS_ENUMF_(energy_f),
+                       cs_field_by_name_try("energy_fraction"));
 }
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Map base fields to enumerated pointers for electric arcs
- *
- * \param[in]  n_gasses    number of gasses
+ * \brief Map base fields to enumerated pointers for gas mix model
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_field_pointer_map_electric_arcs(int  n_gasses)
+cs_field_pointer_map_gas_mix(void)
 {
-  char s[64];
-
-  cs_field_pointer_map(CS_ENUMF_(h),
-                       cs_field_by_name_try("enthalpy"));
-
-  cs_field_pointer_map(CS_ENUMF_(potr), cs_field_by_name_try("elec_pot_r"));
-  cs_field_pointer_map(CS_ENUMF_(poti), cs_field_by_name_try("elec_pot_i"));
-
-  cs_field_pointer_map(CS_ENUMF_(potva), cs_field_by_name_try("vec_potential"));
-
-  for (int i = 0; i < n_gasses - 1; i++) {
-    snprintf(s, 63, "esl_fraction_%02d", i+1); s[63] = '\0';
-    cs_field_pointer_map_indexed(CS_ENUMF_(ycoel), i, cs_field_by_name_try(s));
-  }
+  cs_field_pointer_map(CS_ENUMF_(mol_mass),
+                       cs_field_by_name_try("mix_mol_mas"));
 }
 
 /*----------------------------------------------------------------------------*/
@@ -483,74 +553,18 @@ cs_field_pointer_map_gas_combustion(void)
 
 /*----------------------------------------------------------------------------*/
 /*!
- * \brief Map base fields to enumerated pointers for radiation module
- *
- * \param[in]  n_r_phases  number of radiating phases: 1 + possibly
- *                         number of combustible classes (coal, fuel)
+ * \brief Map base fields to enumerated pointers for groundwater flows
  */
 /*----------------------------------------------------------------------------*/
 
 void
-cs_field_pointer_map_radiation(int  n_r_phases)
+cs_field_pointer_map_groundwater(void)
 {
-  char s[64], suffix[16];
-
-  cs_field_pointer_map(CS_ENUMF_(rad_lumin),
-                       cs_field_by_name_try("luminance"));
-  cs_field_pointer_map(CS_ENUMF_(rad_q),
-                       cs_field_by_name_try("radiative_flux"));
-
-  for (int i = 0; i < n_r_phases; i++) {
-
-    if (i > 0) {
-      snprintf(suffix, 15, "_%02d", i+1);
-      suffix[15] = '\0';
-    }
-    else
-      suffix[0] = '\0';
-
-    snprintf(s, 63, "rad_st%s", suffix); s[63] = '\0';
-    cs_field_pointer_map_indexed(CS_ENUMF_(rad_ets), i, cs_field_by_name_try(s));
-
-    snprintf(s, 63, "rad_st_implicit%s", suffix); s[63] = '\0';
-    cs_field_pointer_map_indexed(CS_ENUMF_(rad_its), i, cs_field_by_name_try(s));
-
-    snprintf(s, 63, "rad_absorption%s", suffix); s[63] = '\0';
-    cs_field_pointer_map_indexed(CS_ENUMF_(rad_abs), i, cs_field_by_name_try(s));
-
-    snprintf(s, 63, "rad_emission%s", suffix); s[63] = '\0';
-    cs_field_pointer_map_indexed(CS_ENUMF_(rad_emi), i, cs_field_by_name_try(s));
-
-    snprintf(s, 63, "rad_absorption_coeff%s", suffix); s[63] = '\0';
-    cs_field_pointer_map_indexed(CS_ENUMF_(rad_cak), i, cs_field_by_name_try(s));
-
-  }
-
-  cs_field_pointer_map(CS_ENUMF_(tparo),
-                       cs_field_by_name_try("wall_temperature"));
-
-  cs_field_pointer_map(CS_ENUMF_(qinci),
-                       cs_field_by_name_try("rad_incident_flux"));
-
-  cs_field_pointer_map(CS_ENUMF_(xlam),
-                       cs_field_by_name_try("wall_thermal_conductivity"));
-
-  cs_field_pointer_map(CS_ENUMF_(epa),
-                       cs_field_by_name_try("wall_thickness"));
-
-  cs_field_pointer_map(CS_ENUMF_(emissivity),
-                       cs_field_by_name_try("emissivity"));
-
-  cs_field_pointer_map(CS_ENUMF_(fnet),
-                       cs_field_by_name_try("rad_net_flux"));
-
-  cs_field_pointer_map(CS_ENUMF_(fconv),
-                       cs_field_by_name_try("rad_convective_flux"));
-
-  cs_field_pointer_map(CS_ENUMF_(hconv),
-                       cs_field_by_name_try("rad_exchange_coefficient"));
+  cs_field_pointer_map(CS_ENUMF_(head),
+                       cs_field_by_name_try("hydraulic_head"));
 }
 
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
