@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_defs.h"
+#include "cs_field.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -45,7 +46,7 @@ BEGIN_C_DECLS
 
 #define CS_ENUMF_(e) CS_FIELD_POINTER_ ## e
 
-/* Macro used to return field a field pointer by its enumerated value */
+/* Macro used to return a field pointer by its enumerated value */
 
 #define CS_F_(e) cs_glob_field_pointers[CS_FIELD_POINTER_ ## e].f
 
@@ -55,7 +56,7 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
-/*! Enumerated field pointer ids */
+/*! Suffixes of enumerated field pointer ids, arguments of the macro CS_F_(). */
 
 typedef enum {
 
@@ -64,7 +65,7 @@ typedef enum {
   CS_ENUMF_(dt),           /*!< local time step */
 
   CS_ENUMF_(p),            /*!< pressure */
-  CS_ENUMF_(u),            /*!< velocity */
+  CS_ENUMF_(vel),          /*!< velocity */
 
   CS_ENUMF_(k),            /*!< turbulent kinetic energy \f$ k \f$ */
   CS_ENUMF_(eps),          /*!< turbulent dissipation \f$ \varepsilon \f$ */
@@ -75,36 +76,56 @@ typedef enum {
   CS_ENUMF_(r12),          /*!< Reynolds stress component \f$ R_{xy} \f$ */
   CS_ENUMF_(r23),          /*!< Reynolds stress component \f$ R_{yz} \f$ */
   CS_ENUMF_(r13),          /*!< Reynolds stress component \f$ R_{xz} \f$ */
+  CS_ENUMF_(rij),          /*!< Reynolds stress tensor \f$ R_{ij} \f$ */
 
   CS_ENUMF_(phi),          /*!< \f$ \phi \f$ for \f$ \phi-f_b \f$ model */
   CS_ENUMF_(f_bar),        /*!< \f$ f_b \f$ for \f$ \phi-f_b \f$ model */
-  CS_ENUMF_(alpha),        /*!< \f$ alpha \f$ for \f$ Bl-v^2-k \f$
-                             or EBRSM model */
+  CS_ENUMF_(alp_bl),        /*!< \f$ \alpha \f$ for \f$ Bl-v^2-k \f$
+                                 or EBRSM model */
 
-  CS_ENUMF_(omg),          /*!< \f$ omega \f$ for \f$ k-\omega \f$ SST model */
-  CS_ENUMF_(nusa),         /*!< \widetilde{\nu}_T \f$ for Spalart Allmaras */
+  CS_ENUMF_(omg),          /*!< \f$ \omega \f$ for \f$ k-\omega \f$ SST model */
+  CS_ENUMF_(nusa),         /*!< \f$ \widetilde{\nu}_T \f$ for Spalart-
+                                Allmaras */
+
+  CS_ENUMF_(hybrid_blend), /*!< Blending factor for DDES*/
 
   CS_ENUMF_(mesh_u),       /*!< mesh velocity */
 
+  CS_ENUMF_(void_f),       /*!< void fraction */
+  CS_ENUMF_(vol_f),        /*!< volume fraction */
+
   CS_ENUMF_(h),            /*!< enthalpy */
-  CS_ENUMF_(t),            /*!< temperature */
-  CS_ENUMF_(energy),       /*!< total energy */
+  CS_ENUMF_(t),            /*!< temperature*/
+  CS_ENUMF_(t_b),          /*!< temperature (at boundary faces)*/
+  CS_ENUMF_(e_tot),        /*!< total energy */
+  CS_ENUMF_(h_tot),        /*!< total enthalpy */
 
   CS_ENUMF_(rho),          /*!< density (at cells) */
   CS_ENUMF_(rho_b),        /*!< density (at boundary faces) */
 
-  CS_ENUMF_(cp),           /*!< specific heat */
+  CS_ENUMF_(cp),           /*!< isobaric specific heat */
+  CS_ENUMF_(cv),           /*!< isochoric specific heat */
 
   CS_ENUMF_(mu),           /*!< molecular viscosity */
   CS_ENUMF_(mu_t),         /*!< turbulent dynamic viscosity */
 
+  CS_ENUMF_(lambda),       /*!< Thermal conductivity */
+  CS_ENUMF_(th_diff),      /*!< Thermal diffusivity */
+
   CS_ENUMF_(poro),         /*!< porosity */
+  CS_ENUMF_(if_poro),      /*!< internal faces porosity */
   CS_ENUMF_(t_poro),       /*!< tensorial porosity */
 
   /* Specific physics variables and properties */
 
   CS_ENUMF_(t_kelvin),     /*!< temperature, in Kelvin */
 
+  CS_ENUMF_(vism),         /*!< mesh viscosity */
+
+  CS_ENUMF_(volume_f),     /*!< homogeneous model volume fraction */
+  CS_ENUMF_(mass_f),       /*!< homogeneous model mass fraction */
+  CS_ENUMF_(energy_f),     /*!< homogeneous model energy fraction */
+
   CS_ENUMF_(pot_t),        /*!< potential temperature */
   CS_ENUMF_(totwt),        /*!< total water content */
   CS_ENUMF_(ntdrp),        /*!< total number of droplets */
@@ -128,13 +149,13 @@ typedef enum {
   CS_ENUMF_(h2),           /*!< mass enthalpy for coal class (permeatic case) */
   CS_ENUMF_(f1m),          /*!< mean value light volatiles for coal class */
   CS_ENUMF_(f2m),          /*!< mean value heavy volatiles for coal class */
-  CS_ENUMF_(f4m),
-  CS_ENUMF_(f5m),
-  CS_ENUMF_(f6m),
-  CS_ENUMF_(f7m),
-  CS_ENUMF_(f8m),
-  CS_ENUMF_(f9m),
-  CS_ENUMF_(fvp2m),
+  CS_ENUMF_(f4m),          /*!< oxydant 2 mass fraction */
+  CS_ENUMF_(f5m),          /*!< oxydant 3 mass fraction */
+  CS_ENUMF_(f6m),          /*!< water from coal drying mass fraction */
+  CS_ENUMF_(f7m),          /*!< carbon from coal oxidyzed by O2 mass fraction */
+  CS_ENUMF_(f8m),          /*!< carbon from coal gasified by CO2 mass fraction */
+  CS_ENUMF_(f9m),          /*!< carbon from coal gasified by H2O mass fraction */
+  CS_ENUMF_(fvp2m),        /*!< f1f2 variance */
   CS_ENUMF_(yco2),         /*!< CO2 fraction */
   CS_ENUMF_(yhcn),         /*!< HCN fraction */
   CS_ENUMF_(yno),          /*!< NO fraction */
@@ -145,18 +166,26 @@ typedef enum {
   CS_ENUMF_(poti),         /*!< Electric potential, imaginary part */
   CS_ENUMF_(potva),        /*!< Vector potential */
   CS_ENUMF_(ycoel),        /*!< Constituent mass fraction */
+  CS_ENUMF_(joulp),        /*!< Joule power */
+  CS_ENUMF_(radsc),        /*!< radiation source */
+  CS_ENUMF_(elech),        /*!< electric charge */
+  CS_ENUMF_(curre),        /*!< current real */
+  CS_ENUMF_(curim),        /*!< current imaginary */
+  CS_ENUMF_(laplf),        /*!< laplace forces */
+  CS_ENUMF_(magfl),        /*!< magnetic field */
+  CS_ENUMF_(elefl),        /*!< electric field */
 
   CS_ENUMF_(rad_lumin),    /*!< Radiative luminance */
   CS_ENUMF_(rad_q),        /*!< Radiative flux */
 
-  CS_ENUMF_(rad_ets),      /*!< Radiative flux explicit source term */
-  CS_ENUMF_(rad_its),      /*!< Radiative flux implicit source term */
+  CS_ENUMF_(rad_est),      /*!< Radiative flux explicit source term */
+  CS_ENUMF_(rad_ist),      /*!< Radiative flux implicit source term */
   CS_ENUMF_(rad_abs),      /*!< Radiative absorption */
   CS_ENUMF_(rad_emi),      /*!< Radiative emission */
   CS_ENUMF_(rad_cak),      /*!< Radiative absorption coefficient */
 
-  CS_ENUMF_(tparo),        /*!< Wall temperature */
   CS_ENUMF_(qinci),        /*!< Radiative incident radiative flux density */
+  CS_ENUMF_(qinsp),        /*!< Spectral radiative incident flux */
   CS_ENUMF_(xlam),         /*!< Wall thermal conductivity */
   CS_ENUMF_(epa),          /*!< Wall thickness */
   CS_ENUMF_(emissivity),   /*!< Wall emissivity */
@@ -164,6 +193,58 @@ typedef enum {
   CS_ENUMF_(fconv),        /*!< Boundary radiative convective flux */
   CS_ENUMF_(hconv),        /*!< radiative exchange coefficient */
 
+  CS_ENUMF_(mol_mass),     /*!< gas mix molar max */
+
+  CS_ENUMF_(head),         /*!< hydraulic head */
+
+  /* Cooling tower fields */
+  CS_ENUMF_(humid),          /*!< Humidity */
+  CS_ENUMF_(ym_w),           /*!< Mass fraction of dry air in humid air */
+  CS_ENUMF_(t_l),            /*!< Injected liquid water temperature */
+  CS_ENUMF_(h_l),            /*!< Injected liquid water enthalpy */
+  CS_ENUMF_(y_l_pack),       /*!< Mass of liquid per unit volume of cell */
+  CS_ENUMF_(thermal_diff_h), /*!< Humid air thermal diffusivity  */
+  CS_ENUMF_(thermal_diff_l), /*!< Injected liquid water thermal diffusivity */
+  CS_ENUMF_(pack_zone_id),   /*!< Id of the packing zone */
+
+  /* NCFD fields */
+  CS_ENUMF_(yf_ncond),       /*!< non-condensable mass fraction */
+  CS_ENUMF_(qp),             /*!< Turbulent Kinetic Energy q2 */
+  CS_ENUMF_(qfp),            /*!< Covariance q12 */
+  CS_ENUMF_(qfpxx),          /*!< XX component of qfp */
+  CS_ENUMF_(qfpxy),          /*!< XY component of qfp */
+  CS_ENUMF_(qfpxz),          /*!< XZ component of qfp */
+  CS_ENUMF_(qfpyx),          /*!< YX component of qfp */
+  CS_ENUMF_(qfpyy),          /*!< YY component of qfp */
+  CS_ENUMF_(qfpyz),          /*!< YZ component of qfp */
+  CS_ENUMF_(qfpzx),          /*!< ZX component of qfp */
+  CS_ENUMF_(qfpzy),          /*!< ZY component of qfp */
+  CS_ENUMF_(qfpzz),          /*!< ZZ component of qfp */
+  CS_ENUMF_(gamma),          /*!< Interfacial mass transfer */
+  CS_ENUMF_(ia),             /*!< Interfacial area */
+  CS_ENUMF_(x2),             /*!< x2 for droplets */
+  CS_ENUMF_(d32),            /*!< Sauter diameter */
+  CS_ENUMF_(drag),           /*!< Phases drag */
+  CS_ENUMF_(ad_mass),        /*!< Added mass */
+  CS_ENUMF_(wlubr),          /*!< Wall lubrication */
+  CS_ENUMF_(th_diff_t),      /*!< Turbulent thermal diffusivity */
+  CS_ENUMF_(drho_dp),        /*!< drho over dp */
+  CS_ENUMF_(drho_dh),        /*!< drho over dh */
+  CS_ENUMF_(tau12_t),        /*!< turbulent tau12 */
+  CS_ENUMF_(lift),           /*!< Particles lift */
+  CS_ENUMF_(disp_t),         /*!< Turbulent dispersion */
+  CS_ENUMF_(drift_vel),      /*!< Particles drift velocity */
+  CS_ENUMF_(yplus),          /*!< Wall distance: y+ */
+  CS_ENUMF_(vel_mean),       /*!< Mean velocity (for dispersed phases) */
+  CS_ENUMF_(vel_rel),        /*!< Relative velocity (for dispersed phases) */
+  CS_ENUMF_(dt_dp),          /*!< dtemp/dpress derivative */
+
+  /* Added variables (scalars) */
+  CS_ENUMF_(add_var),        /*!< User added variables */
+
+  /* User-defined arrays */
+  CS_ENUMF_(user),
+
   /* End of attributes */
 
   CS_FIELD_N_POINTERS
@@ -243,6 +324,13 @@ cs_field_pointer_map_indexed(cs_field_pointer_id_t   e,
 void
 cs_field_pointer_map_base(void);
 
+/*----------------------------------------------------------------------------
+ * Map some boundary fields to enumerated pointers.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_field_pointer_map_boundary(void);
+
 /*----------------------------------------------------------------------------
  * Map base fields to enumerated pointers for atmospheric models
  *
@@ -274,15 +362,12 @@ cs_field_pointer_map_coal_combustion(int  n_coals,
 void
 cs_field_pointer_map_compressible(void);
 
-/*----------------------------------------------------------------------------
- * Map base fields to enumerated pointers for electric arcs
- *
- * parameters:
- *  n_gasses <-- number of gasses
+/*----------------------------------------------------------------------------*
+ * Map base fields to enumerated pointers for gas mix model
  *----------------------------------------------------------------------------*/
 
 void
-cs_field_pointer_map_electric_arcs(int  n_gasses);
+cs_field_pointer_map_gas_mix(void);
 
 /*----------------------------------------------------------------------------
  * Map base fields to enumerated pointers for gas combustion.
@@ -291,16 +376,13 @@ cs_field_pointer_map_electric_arcs(int  n_gasses);
 void
 cs_field_pointer_map_gas_combustion(void);
 
-/*----------------------------------------------------------------------------
- * Map base fields to enumerated pointers for radiation module
- *
- * parameters:
- *   n_r_phases <-- number of radiating phases: 1 + possibly
- *                  number of combustible classes (coal, fuel)
+/*----------------------------------------------------------------------------*/
+/*
+ * Map base fields to enumerated pointers for groundwater flows
  *----------------------------------------------------------------------------*/
 
 void
-cs_field_pointer_map_radiation(int  n_r_phases);
+cs_field_pointer_map_groundwater(void);
 
 /*----------------------------------------------------------------------------*/
 
