@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -185,6 +185,26 @@ cs_file_t *
 cs_file_open_default(const char        *name,
                      cs_file_mode_t     mode);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a file descriptor and open the associated file, using the
+ *        serial IO on the root rank.
+ *
+ * By default, data is written or read as native data. This behavior may be
+ * modified by cs_file_set_swap_endian().
+ *
+ * \param[in]  name   file name
+ * \param[in]  mode   file acces mode: read, write, or append
+ *
+ * \return pointer to cs_file_t file descriptor (NULL in case of failure);
+ *   currently, errors are fatal.
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_file_t *
+cs_file_open_serial(const char      *name,
+                    cs_file_mode_t   mode);
+
 /*----------------------------------------------------------------------------
  * Destroy a file descriptor and close the associated file.
  *
@@ -448,6 +468,13 @@ cs_file_tell(cs_file_t  *f);
 void
 cs_file_dump(const cs_file_t  *f);
 
+/*----------------------------------------------------------------------------
+ * Free the default options for file access.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_file_free_defaults(void);
+
 /*----------------------------------------------------------------------------
  * Get the default options for file access.
  *
@@ -565,6 +592,23 @@ cs_file_set_default_comm(int       block_rank_step,
                          int       block_min_size,
                          MPI_Comm  comm);
 
+/*----------------------------------------------------------------------------
+ * Create an MPI communicator for distributed block parallel IO.
+ *
+ * parameters:
+ *   block_rank_step <-- MPI rank stepping between non-empty blocks
+ *   comm            <-- Handle to main MPI communicator
+ *
+ * returns:
+ *   communicator associated with IO, MPI_COMM_NULL for ranks not
+ *   participating in parallel IO (including ranks participating in IO
+ *   where communicator size would be 1)
+ *----------------------------------------------------------------------------*/
+
+MPI_Comm
+cs_file_block_comm(int       block_rank_step,
+                   MPI_Comm  comm);
+
 #endif /* defined(HAVE_MPI) */
 
 /*----------------------------------------------------------------------------
@@ -759,7 +803,7 @@ cs_file_listdir(const char *path);
 /*----------------------------------------------------------------------------
  * Return the size of a file.
  *
- * If the file does not exit, 0 is returned.
+ * If the file does not exist, 0 is returned.
  *
  * Note that for some special files, such as files in the Linux /proc
  * directory, this may return 0.
