@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -46,6 +46,8 @@
 #include "cs_order.h"
 
 #include "cs_interface.h"
+#include "cs_rank_neighbors.h"
+
 #include "fvm_periodicity.h"
 
 /*----------------------------------------------------------------------------
@@ -58,6 +60,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Static global variables
  *============================================================================*/
@@ -307,22 +311,24 @@ _zero_rotation_values(const cs_halo_t  *halo,
   }
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Create a halo structure.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a halo structure given an interface set.
  *
- * parameters:
- *   ifs  <--  pointer to a cs_interface_set structure
+ * \param[in]  ifs  pointer to a cs_interface_set structure
  *
- * returns:
- *  pointer to created cs_halo_t structure
- *---------------------------------------------------------------------------*/
+ * \return  pointer to created cs_halo_t structure
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_halo_t *
-cs_halo_create(cs_interface_set_t  *ifs)
+cs_halo_create(const cs_interface_set_t  *ifs)
 {
   cs_lnum_t  i, tmp_id, perio_lst_size;
 
@@ -443,15 +449,15 @@ cs_halo_create(cs_interface_set_t  *ifs)
   return halo;
 }
 
-/*----------------------------------------------------------------------------
- * Create a halo structure, using a reference halo
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a halo structure, given a reference halo.
  *
- * parameters:
- *   ref  <--  pointer to reference halo
+ * \param[in]  ref  pointer to reference halo
  *
- * returns:
- *  pointer to created cs_halo_t structure
- *---------------------------------------------------------------------------*/
+ * \return  pointer to created cs_halo_t structure
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_halo_t *
 cs_halo_create_from_ref(const cs_halo_t  *ref)
@@ -507,34 +513,270 @@ cs_halo_create_from_ref(const cs_halo_t  *ref)
   return halo;
 }
 
-/*----------------------------------------------------------------------------
- * Destroy a halo structure
+#if defined(HAVE_MPI)
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create a halo structure from distant element distant ranks and ids.
  *
- * parameters:
- *   halo  <--  pointer to cs_halo_t structure to destroy
+ * \remark  This function does not handle periodicity. For most matrix-vector,
+ *          products and similar operations, periodicity of translation an
+ *          even rotation could be handled with no specific halo information,
+ *          simply by assigning an equivalence between two periodic elements.
+ *          For rotation, this would require also applying a rotation through
+ *          the matrix coefficients (this would have the advantage of being
+ *          compatible with external libraries). An alternative would be
+ *          to add rotation information to a given halo as a second stage,
+ *          through a specialized operator which can be added in the future.
  *
- * Returns:
- *  pointer to deleted halo structure (NULL)
- *---------------------------------------------------------------------------*/
+ * \param[in]  rn              associated rank neighbors info
+ * \param[in]  n_local_elts    number of elements for local rank
+ * \param[in]  n_distant_elts  number of distant elements for local rank
+ * \param[in]  elt_rank_id     distant element rank index in rank neighbors,
+ *                             ordered by rank (size: n_distant_elts)
+ * \param[in]  elt_id          distant element id (at distant rank),
+ *                             ordered by rank (size: n_distant_elts)
+ *
+ * \return  pointer to created cs_halo_t structure
+ */
+/*----------------------------------------------------------------------------*/
 
 cs_halo_t *
-cs_halo_destroy(cs_halo_t  *halo)
+cs_halo_create_from_rank_neighbors(const cs_rank_neighbors_t  *rn,
+                                   cs_lnum_t                   n_local_elts,
+                                   cs_lnum_t                   n_distant_elts,
+                                   const int                   elt_rank_id[],
+                                   const cs_lnum_t             elt_id[])
 {
-  if (halo == NULL)
-    return NULL;
+  cs_halo_t  *halo = NULL;
+
+  BFT_MALLOC(halo, 1, cs_halo_t);
 
   halo->n_c_domains = 0;
-  BFT_FREE(halo->c_domain_rank);
+  halo->n_transforms = 0;
 
-  BFT_FREE(halo->send_perio_lst);
-  BFT_FREE(halo->send_index);
-  BFT_FREE(halo->perio_lst);
-  BFT_FREE(halo->index);
+  halo->n_rotations = 0;
 
-  if (halo->send_list != NULL)
-    BFT_FREE(halo->send_list);
+  halo->periodicity = NULL;
+  halo->send_perio_lst = NULL;
+  halo->perio_lst = NULL;
 
-  BFT_FREE(halo);
+  halo->n_local_elts = n_local_elts;
+
+  for (int i = 0; i < CS_HALO_N_TYPES; i++) {
+    halo->n_send_elts[i] = 0;
+    halo->n_elts [i] = n_distant_elts;
+  }
+
+  /* Count elements for each rank;
+     check they are are ordered lexicographically */
+
+  cs_lnum_t *rank_count;
+  BFT_MALLOC(rank_count, rn->size*2, cs_lnum_t);
+  for (int i = 0; i < rn->size; i++)
+    rank_count[i] = 0;
+
+  int rank_prev = -1;
+  int elt_prev = -1;
+  for (cs_lnum_t i = 0; i < n_distant_elts; i++) {
+    int rank_id = elt_rank_id[i];
+    if (   rank_id < rank_prev
+        || (rank_id == rank_prev && elt_id[i] <= elt_prev))
+      bft_error
+        (__FILE__, __LINE__, 0,
+         "%s:\n"
+         "  Rank and distant element ids passed to this function must\n"
+         "  be lexicographically ordered; this is not the case here.",
+         __func__);
+    rank_count[rank_id] += 1;
+    rank_prev = rank_id;
+    elt_prev = elt_id[i];
+  }
+
+  /* Now exchange counts with neighboring elaments */
+
+  MPI_Comm comm = cs_glob_mpi_comm;
+  MPI_Request *request = NULL;
+  MPI_Status *status = NULL;
+
+  BFT_MALLOC(request, rn->size*2, MPI_Request);
+  BFT_MALLOC(status, rn->size*2, MPI_Status);
+
+  /* Exchange local range with neighbor ranks */
+
+  int request_count = 0;
+  const int local_rank = cs_glob_rank_id;
+
+  for (int i = 0; i < rn->size; i++) {
+    MPI_Irecv(rank_count + rn->size + i,
+              1,
+              CS_MPI_LNUM,
+              rn->rank[i],
+              local_rank,
+              comm,
+              &(request[request_count++]));
+  }
+
+  for (int i = 0; i < rn->size; i++) {
+    MPI_Isend(rank_count + i,
+              1,
+              CS_MPI_LNUM,
+              rn->rank[i],
+              rn->rank[i],
+              comm,
+              &(request[request_count++]));
+  }
+
+  MPI_Waitall(request_count, request, status);
+
+  /* Now build send and receive indexes to exchange data;
+     the receive index can be directly assigned to the halo;
+     also check if cs_glob_rank_id belongs to interface set in order to
+     order ranks with local rank at first place */
+
+  int        loc_r_index = -1;
+  cs_lnum_t  r_displ = 0, loc_r_displ = 0;
+  cs_lnum_t  recv_count = 0, send_count = 0;
+
+  halo->n_c_domains = 0;
+  for (int i = 0; i < rn->size; i++) {
+    if (rank_count[i] + rank_count[rn->size + i] > 0) {
+      halo->n_c_domains += 1;
+      if (rn->rank[i] == local_rank) {
+        loc_r_index = i;
+        loc_r_displ = r_displ;
+        assert(rank_count[i] == rank_count[rn->size + i]);
+      }
+      r_displ += rank_count[i];
+      recv_count += rank_count[rn->size + i];
+    }
+  }
+
+  BFT_MALLOC(halo->c_domain_rank, halo->n_c_domains, int);
+
+  BFT_MALLOC(halo->send_list, recv_count, cs_lnum_t);
+  BFT_MALLOC(halo->send_index, halo->n_c_domains*2+1, cs_lnum_t);
+  BFT_MALLOC(halo->index, halo->n_c_domains*2+1, cs_lnum_t);
+
+  halo->n_c_domains = 0;
+  send_count = 0;
+  recv_count = 0;
+
+  halo->index[0] = 0;
+  halo->send_index[0] = 0;
+
+  if (loc_r_index > -1) {
+    halo->c_domain_rank[0] = local_rank;
+    cs_lnum_t  l_count = rank_count[loc_r_index];
+    for (cs_lnum_t i = 0; i < l_count; i++)
+      halo->send_list[i] = elt_id[loc_r_displ + i];
+    send_count += l_count;
+    recv_count += l_count;
+    halo->n_c_domains = 1;
+    for (int j = 1; j < 3; j++) {
+      halo->index[j] = recv_count;
+      halo->send_index[j] = send_count;
+    }
+  }
+
+  for (int i = 0; i < rn->size; i++) {
+    if (   rank_count[i] + rank_count[rn->size + i] > 0
+        && rn->rank[i] != local_rank) {
+      halo->c_domain_rank[halo->n_c_domains] = rn->rank[i];
+      recv_count += rank_count[i];
+      send_count += rank_count[rn->size + i];
+      for (int j = 1; j < 3; j++) {
+        halo->index[halo->n_c_domains*2 + j] = recv_count;
+        halo->send_index[halo->n_c_domains*2 + j] = send_count;
+      }
+      halo->n_c_domains += 1;
+    }
+  }
+
+  BFT_FREE(rank_count);
+
+  for (int i = 0; i < CS_HALO_N_TYPES; i++)
+    halo->n_send_elts[i] = send_count;
+
+  /* Now send lists to matching ranks (reverse send and receive) */
+
+  request_count = 0;
+
+  for (int i = 0; i < halo->n_c_domains; i++) {
+    int rank_id = halo->c_domain_rank[i];
+    if (rank_id == local_rank) continue;
+    cs_lnum_t r_shift = halo->send_index[2*i];
+    cs_lnum_t r_size  = halo->send_index[2*i+1] - r_shift;
+    if (r_size > 0)
+      MPI_Irecv(halo->send_list + r_shift,
+                r_size,
+                CS_MPI_LNUM,
+                rank_id,
+                local_rank,
+                comm,
+                &(request[request_count++]));
+  }
+
+  for (int i = 0; i < halo->n_c_domains; i++) {
+    int rank_id = halo->c_domain_rank[i];
+    if (rank_id == local_rank) continue;
+    cs_lnum_t s_shift = halo->index[2*i];
+    cs_lnum_t s_size  = halo->index[2*i+1] - s_shift;
+    if (s_shift < loc_r_displ) { /* case with local rank first */
+      assert(halo->c_domain_rank[0] == local_rank);
+      s_shift -= halo->index[2];
+    }
+    if (s_size > 0)
+      MPI_Isend(elt_id + s_shift,
+                s_size,
+                CS_MPI_LNUM,
+                rank_id,
+                rank_id,
+                comm,
+                &(request[request_count++]));
+  }
+
+  MPI_Waitall(request_count, request, status);
+
+  BFT_FREE(request);
+  BFT_FREE(status);
+
+  _cs_glob_n_halos += 1;
+
+  return halo;
+}
+
+#endif /* HAVE_MPI */
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * brief Destroy a halo structure.
+ *
+ * \param[in, out]  halo  pointer to pointer to cs_halo structure to destroy.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_halo_destroy(cs_halo_t  **halo)
+{
+  if (halo == NULL)
+    return;
+
+  if (*halo == NULL)
+    return;
+
+  cs_halo_t  *_halo = *halo;
+
+  BFT_FREE(_halo->c_domain_rank);
+
+  BFT_FREE(_halo->send_perio_lst);
+  BFT_FREE(_halo->send_index);
+  BFT_FREE(_halo->perio_lst);
+  BFT_FREE(_halo->index);
+
+  BFT_FREE(_halo->send_list);
+
+  BFT_FREE(*halo);
 
   _cs_glob_n_halos -= 1;
 
@@ -558,8 +800,6 @@ cs_halo_destroy(cs_halo_t  *halo)
 #endif
 
   }
-
-  return NULL;
 }
 
 /*----------------------------------------------------------------------------
@@ -666,7 +906,7 @@ cs_halo_free_buffer(void)
 }
 
 /*----------------------------------------------------------------------------
- * Apply cell renumbering to a halo
+ * Apply local cells renumbering to a halo
  *
  * parameters:
  *   halo        <-- pointer to halo structure
@@ -679,14 +919,148 @@ cs_halo_renumber_cells(cs_halo_t        *halo,
 {
   if (halo != NULL) {
 
-    cs_lnum_t i;
     const cs_lnum_t n_elts = halo->n_send_elts[CS_HALO_EXTENDED];
 
-    for (i = 0; i < n_elts; i++)
-      halo->send_list[i] = new_cell_id[halo->send_list[i]];
+    for (cs_lnum_t j = 0; j < n_elts; j++)
+      halo->send_list[j] = new_cell_id[halo->send_list[j]];
+
   }
 }
 
+/*----------------------------------------------------------------------------
+ * Apply ghost cells renumbering to a halo
+ *
+ * parameters:
+ *   halo        <-- pointer to halo structure
+ *   old_cell_id <-- array indicating new -> old cell id (0 to n-1)
+ *---------------------------------------------------------------------------*/
+
+void
+cs_halo_renumber_ghost_cells(cs_halo_t        *halo,
+                             const cs_lnum_t   old_cell_id[])
+{
+  if (halo == NULL)
+    return;
+
+  /* Reverse update from distant cells */
+
+  cs_lnum_t *send_buf, *recv_buf;
+
+  BFT_MALLOC(send_buf, halo->n_send_elts[1], cs_lnum_t);
+  BFT_MALLOC(recv_buf, halo->n_elts[1], cs_lnum_t);
+
+  for (int i = 0; i < halo->n_c_domains; i++) {
+    cs_lnum_t start = halo->index[2*i];
+    cs_lnum_t end = halo->index[2*i+2];
+    cs_lnum_t shift = halo->n_local_elts + halo->index[2*i];
+    for (cs_lnum_t j = start; j < end; j++) {
+      recv_buf[j] = old_cell_id[halo->n_local_elts + j] - shift;
+      assert(recv_buf[j] >= 0 && recv_buf[j] < (end - start));
+    }
+  }
+
+  int local_rank_id = (cs_glob_n_ranks == 1) ? 0 : -1;
+
+#if defined(HAVE_MPI)
+
+  if (cs_glob_n_ranks > 1) {
+
+    int rank_id;
+    int request_count = 0;
+    const int local_rank = cs_glob_rank_id;
+
+    /* Receive data from distant ranks */
+
+    for (rank_id = 0; rank_id < halo->n_c_domains; rank_id++) {
+
+      cs_lnum_t start = halo->send_index[2*rank_id];
+      cs_lnum_t length = (  halo->send_index[2*rank_id + 2]
+                          - halo->send_index[2*rank_id]);
+
+      if (halo->c_domain_rank[rank_id] != local_rank) {
+        if (length > 0)
+          MPI_Irecv(send_buf + start,
+                    length,
+                    CS_MPI_LNUM,
+                    halo->c_domain_rank[rank_id],
+                    local_rank,
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
+      }
+      else
+        local_rank_id = rank_id;
+
+    }
+
+    /* We wait for posting all receives (often recommended) */
+
+    if (_cs_glob_halo_use_barrier)
+      MPI_Barrier(cs_glob_mpi_comm);
+
+    /* Send data to distant ranks */
+
+    for (rank_id = 0; rank_id < halo->n_c_domains; rank_id++) {
+
+      /* If this is not the local rank */
+
+      if (halo->c_domain_rank[rank_id] != local_rank) {
+
+        cs_lnum_t start = halo->index[2*rank_id];
+        cs_lnum_t length = (  halo->index[2*rank_id + 2]
+                            - halo->index[2*rank_id]);
+
+        if (length > 0)
+          MPI_Isend(recv_buf + start,
+                    length,
+                    CS_MPI_LNUM,
+                    halo->c_domain_rank[rank_id],
+                    halo->c_domain_rank[rank_id],
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
+
+      }
+
+    }
+
+    /* Wait for all exchanges */
+
+    MPI_Waitall(request_count, _cs_glob_halo_request, _cs_glob_halo_status);
+
+  }
+
+#endif /* defined(HAVE_MPI) */
+
+  /* Copy local values if present */
+
+  if (local_rank_id > -1) {
+
+    cs_lnum_t *recv = recv_buf + halo->index[2*local_rank_id];
+
+    cs_lnum_t start = halo->send_index[2*local_rank_id];
+    cs_lnum_t length = (  halo->send_index[2*local_rank_id + 2]
+                        - halo->send_index[2*local_rank_id]);
+
+    for (cs_lnum_t j = 0; j < length; j++)
+      send_buf[j+start] = recv[j];
+
+  }
+
+  BFT_FREE(recv_buf);
+
+  /* Now apply renumbering to send list */
+
+  for (int i = 0; i < halo->n_c_domains; i++) {
+    cs_lnum_t start = halo->send_index[2*i];
+    cs_lnum_t end = halo->send_index[2*i+2];
+    for (cs_lnum_t j = start; j < end; j++)
+      send_buf[j] = halo->send_list[start + send_buf[j]];
+    for (cs_lnum_t j = start; j < end; j++)
+      halo->send_list[j] = send_buf[j];
+  }
+
+  BFT_FREE(send_buf);
+}
+
 /*----------------------------------------------------------------------------
  * Update array of any type of halo values in case of parallelism or
  * periodicity.
@@ -707,6 +1081,10 @@ cs_halo_renumber_cells(cs_halo_t        *halo,
  *   num       <-> pointer to local number value array
  *----------------------------------------------------------------------------*/
 
+#if defined(__INTEL_COMPILER) && defined(__KNC__)
+#pragma optimization_level 1 /* Crash with O2 on KNC with icc 14.0.0 20130728 */
+#endif
+
 void
 cs_halo_sync_untyped(const cs_halo_t  *halo,
                      cs_halo_type_t    sync_mode,
@@ -724,8 +1102,8 @@ cs_halo_sync_untyped(const cs_halo_t  *halo,
 #if defined(HAVE_MPI)
 
   if (cs_glob_n_ranks > 1) {
-    const size_t send_buffer_size =   CS_MAX(halo->n_send_elts[CS_HALO_EXTENDED],
-                                             halo->n_elts[CS_HALO_EXTENDED])
+    const size_t send_buffer_size = CS_MAX(halo->n_send_elts[CS_HALO_EXTENDED],
+                                           halo->n_elts[CS_HALO_EXTENDED])
                                     * size;
 
     if (send_buffer_size > _cs_glob_halo_send_buffer_size) {
@@ -763,16 +1141,19 @@ cs_halo_sync_untyped(const cs_halo_t  *halo,
 
       if (halo->c_domain_rank[rank_id] != local_rank) {
 
-        unsigned char *dest = _val + (halo->n_local_elts*size) + start*size;
+        if (length > 0) {
 
-        MPI_Irecv(dest,
-                  length * size,
-                  MPI_UNSIGNED_CHAR,
-                  halo->c_domain_rank[rank_id],
-                  halo->c_domain_rank[rank_id],
-                  cs_glob_mpi_comm,
-                  &(_cs_glob_halo_request[request_count++]));
+          unsigned char *dest = _val + (halo->n_local_elts*size) + start*size;
 
+          MPI_Irecv(dest,
+                    length * size,
+                    MPI_UNSIGNED_CHAR,
+                    halo->c_domain_rank[rank_id],
+                    halo->c_domain_rank[rank_id],
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
+
+        }
       }
       else
         local_rank_id = rank_id;
@@ -780,9 +1161,10 @@ cs_halo_sync_untyped(const cs_halo_t  *halo,
     }
 
     /* Assemble buffers for halo exchange;
-       with threading, use dynamic scheduling ,as this is probably a small loop */
+       avoid threading for now, as dynamic scheduling led to slightly
+       higher cost here, and even static scheduling might lead to
+       false sharing for small halos. */
 
-    #pragma omp parallel for private(start, length, src, i, j) schedule(dynamic)
     for (rank_id = 0; rank_id < halo->n_c_domains; rank_id++) {
 
       if (halo->c_domain_rank[rank_id] != local_rank) {
@@ -819,13 +1201,14 @@ cs_halo_sync_untyped(const cs_halo_t  *halo,
         length = (  halo->send_index[2*rank_id + end_shift]
                   - halo->send_index[2*rank_id]);
 
-        MPI_Isend(build_buffer + start*size,
-                  length*size,
-                  MPI_UNSIGNED_CHAR,
-                  halo->c_domain_rank[rank_id],
-                  local_rank,
-                  cs_glob_mpi_comm,
-                  &(_cs_glob_halo_request[request_count++]));
+        if (length > 0)
+          MPI_Isend(build_buffer + start*size,
+                    length*size,
+                    MPI_UNSIGNED_CHAR,
+                    halo->c_domain_rank[rank_id],
+                    local_rank,
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
 
       }
 
@@ -907,13 +1290,14 @@ cs_halo_sync_num(const cs_halo_t  *halo,
 
       if (halo->c_domain_rank[rank_id] != local_rank) {
 
-        MPI_Irecv(num + halo->n_local_elts + start,
-                  length,
-                  CS_MPI_INT,
-                  halo->c_domain_rank[rank_id],
-                  halo->c_domain_rank[rank_id],
-                  cs_glob_mpi_comm,
-                  &(_cs_glob_halo_request[request_count++]));
+        if (length > 0)
+          MPI_Irecv(num + halo->n_local_elts + start,
+                    length,
+                    CS_MPI_INT,
+                    halo->c_domain_rank[rank_id],
+                    halo->c_domain_rank[rank_id],
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
 
       }
       else
@@ -922,9 +1306,10 @@ cs_halo_sync_num(const cs_halo_t  *halo,
     }
 
     /* Assemble buffers for halo exchange;
-       with threading, use dynamic scheduling ,as this is probably a small loop */
+       avoid threading for now, as dynamic scheduling led to slightly
+       higher cost here, and even static scheduling might lead to
+       false sharing for small halos. */
 
-    #pragma omp parallel for private(start, length, i) schedule(dynamic)
     for (rank_id = 0; rank_id < halo->n_c_domains; rank_id++) {
 
       if (halo->c_domain_rank[rank_id] != local_rank) {
@@ -955,13 +1340,14 @@ cs_halo_sync_num(const cs_halo_t  *halo,
         length =   halo->send_index[2*rank_id + end_shift]
                  - halo->send_index[2*rank_id];
 
-        MPI_Isend(build_buffer + start,
-                  length,
-                  CS_MPI_INT,
-                  halo->c_domain_rank[rank_id],
-                  local_rank,
-                  cs_glob_mpi_comm,
-                  &(_cs_glob_halo_request[request_count++]));
+        if (length > 0)
+          MPI_Isend(build_buffer + start,
+                    length,
+                    CS_MPI_INT,
+                    halo->c_domain_rank[rank_id],
+                    local_rank,
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
 
       }
 
@@ -1036,25 +1422,24 @@ cs_halo_sync_var(const cs_halo_t  *halo,
       length = halo->index[2*rank_id + end_shift] - halo->index[2*rank_id];
 
       if (halo->c_domain_rank[rank_id] != local_rank) {
-
-        MPI_Irecv(var + halo->n_local_elts + start,
-                  length,
-                  CS_MPI_REAL,
-                  halo->c_domain_rank[rank_id],
-                  halo->c_domain_rank[rank_id],
-                  cs_glob_mpi_comm,
-                  &(_cs_glob_halo_request[request_count++]));
-
+        if (length > 0)
+          MPI_Irecv(var + halo->n_local_elts + start,
+                    length,
+                    CS_MPI_REAL,
+                    halo->c_domain_rank[rank_id],
+                    halo->c_domain_rank[rank_id],
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
       }
       else
         local_rank_id = rank_id;
 
     }
 
-    /* Assemble buffers for halo exchange;
-       with threading, use dynamic scheduling ,as this is probably a small loop */
+    /* Assemble buffers for halo exchange; avoid threading for now, as dynamic
+       scheduling led to slightly higher cost here, and even static scheduling
+       might lead to false sharing for small halos. */
 
-    #pragma omp parallel for private(start, length, i) schedule(dynamic)
     for (rank_id = 0; rank_id < halo->n_c_domains; rank_id++) {
 
       if (halo->c_domain_rank[rank_id] != local_rank) {
@@ -1085,13 +1470,14 @@ cs_halo_sync_var(const cs_halo_t  *halo,
         length =   halo->send_index[2*rank_id + end_shift]
                  - halo->send_index[2*rank_id];
 
-        MPI_Isend(build_buffer + start,
-                  length,
-                  CS_MPI_REAL,
-                  halo->c_domain_rank[rank_id],
-                  local_rank,
-                  cs_glob_mpi_comm,
-                  &(_cs_glob_halo_request[request_count++]));
+        if (length > 0)
+          MPI_Isend(build_buffer + start,
+                    length,
+                    CS_MPI_REAL,
+                    halo->c_domain_rank[rank_id],
+                    local_rank,
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
 
       }
 
@@ -1180,26 +1566,29 @@ cs_halo_sync_var_strided(const cs_halo_t  *halo,
 
       if (halo->c_domain_rank[rank_id] != local_rank) {
 
-        buffer = var + (halo->n_local_elts + halo->index[2*rank_id])*stride;
+        if (length > 0) {
 
-        MPI_Irecv(buffer,
-                  length,
-                  CS_MPI_REAL,
-                  halo->c_domain_rank[rank_id],
-                  halo->c_domain_rank[rank_id],
-                  cs_glob_mpi_comm,
-                  &(_cs_glob_halo_request[request_count++]));
+          buffer = var + (halo->n_local_elts + halo->index[2*rank_id])*stride;
 
+          MPI_Irecv(buffer,
+                    length,
+                    CS_MPI_REAL,
+                    halo->c_domain_rank[rank_id],
+                    halo->c_domain_rank[rank_id],
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
+
+        }
       }
       else
         local_rank_id = rank_id;
 
     }
 
-    /* Assemble buffers for halo exchange;
-       with threading, use dynamic scheduling ,as this is probably a small loop */
+    /* Assemble buffers for halo exchange; avoid threading for now, as dynamic
+       scheduling led to slightly higher cost here, and even static scheduling
+       might lead to false sharing for small halos. */
 
-    #pragma omp parallel for private(start, length, i, j) schedule(dynamic)
     for (rank_id = 0; rank_id < halo->n_c_domains; rank_id++) {
 
       if (halo->c_domain_rank[rank_id] != local_rank) {
@@ -1247,13 +1636,14 @@ cs_halo_sync_var_strided(const cs_halo_t  *halo,
         length = (  halo->send_index[2*rank_id + end_shift]
                   - halo->send_index[2*rank_id]);
 
-        MPI_Isend(build_buffer + start*stride,
-                  length*stride,
-                  CS_MPI_REAL,
-                  halo->c_domain_rank[rank_id],
-                  local_rank,
-                  cs_glob_mpi_comm,
-                  &(_cs_glob_halo_request[request_count++]));
+        if (length > 0)
+          MPI_Isend(build_buffer + start*stride,
+                    length*stride,
+                    CS_MPI_REAL,
+                    halo->c_domain_rank[rank_id],
+                    local_rank,
+                    cs_glob_mpi_comm,
+                    &(_cs_glob_halo_request[request_count++]));
 
       }
 
@@ -1524,10 +1914,10 @@ cs_halo_dump(const cs_halo_t  *halo,
         bft_printf("  idx start %d:          idx end   %d:\n",
                    index[2*i], index[2*i+1]);
 
-        if (print_level == 1 && list != NULL) {
-          bft_printf("\n            id      cell number\n");
+        if (print_level > 0 && list != NULL) {
+          bft_printf("\n            idx     elt id\n");
           for (j = index[2*i]; j < index[2*i+1]; j++)
-            bft_printf("    %10d %10d\n", j, list[j]+1);
+            bft_printf("    %10d %10d\n", j, list[j]);
         }
 
       } /* there are elements on standard neighborhood */
@@ -1538,11 +1928,11 @@ cs_halo_dump(const cs_halo_t  *halo,
         bft_printf("  idx start %d:          idx end   %d:\n",
                    index[2*i+1], index[2*i+2]);
 
-        if (print_level == 1 && list != NULL) {
-          bft_printf("\n            id      cell number\n");
+        if (print_level > 0 && list != NULL) {
+          bft_printf("\n            idx     elt id\n");
           for (j = index[2*i+1]; j < index[2*i+2]; j++)
             bft_printf("    %10d %10d %10d\n",
-                       j, list[j]+1, halo->n_local_elts+j+1);
+                       j, list[j], halo->n_local_elts+j);
         }
 
       } /* If there are elements on extended neighborhood */
