@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -48,38 +48,6 @@ BEGIN_C_DECLS
  *  Public function header for Fortran API
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Rotate vector values for periodic cells on extended halos.
- *
- * Fortran API:
- *
- * subroutine pervec
- * *****************
- *
- * double precision var(3,ncelet) : <-> : interleaved vector
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (pervec, PERVEC) (cs_real_t  var[]);
-
-/*----------------------------------------------------------------------------
- * Rotate vector values for periodic cells on extended halos.
- *
- * Fortran API:
- *
- * subroutine perrve
- * *****************
- *
- * double precision var1(ncelet) : <-> : component 1 of vector
- * double precision var2(ncelet) : <-> : component 2 of vector
- * double precision var3(ncelet) : <-> : component 3 of vector
- *----------------------------------------------------------------------------*/
-
-void
-CS_PROCF (perrve, PERRVE) (cs_real_t  var1[],
-                           cs_real_t  var2[],
-                           cs_real_t  var3[]);
-
 /*----------------------------------------------------------------------------
  * Rotate tensor values for periodic cells on extended halos.
  *
@@ -202,9 +170,9 @@ cs_halo_perio_sync_var_tens_ni(const cs_halo_t *halo,
  *----------------------------------------------------------------------------*/
 
 void
-cs_halo_perio_sync_var_tens(const cs_halo_t *halo,
-                            cs_halo_type_t   sync_mode,
-                            cs_real_t        var[]);
+cs_halo_perio_sync_var_tens(const cs_halo_t  *halo,
+                            cs_halo_type_t    sync_mode,
+                            cs_real_t         var[]);
 
 /*----------------------------------------------------------------------------
  * Synchronize values for a real tensor (symmetric interleaved) between
@@ -241,23 +209,6 @@ cs_halo_perio_sync_var_diag_ni(const cs_halo_t *halo,
                                cs_real_t        var22[],
                                cs_real_t        var33[]);
 
-/*----------------------------------------------------------------------------
- * Synchronize values for a real diagonal tensor (interleaved)
- * between periodic cells.
- *
- * We only know the diagonal of the tensor.
- *
- * parameters:
- *   halo      <-> halo associated with variable to synchronize
- *   sync_mode --> kind of halo treatment (standard or extended)
- *   var       <-> diagonal tensor to update
- *----------------------------------------------------------------------------*/
-
-void
-cs_halo_perio_sync_var_diag(const cs_halo_t *halo,
-                            cs_halo_type_t   sync_mode,
-                            cs_real_t        var[]);
-
 /*----------------------------------------------------------------------------
  * Apply rotation on the gradient of Reynolds stress tensor
  *
@@ -268,6 +219,21 @@ cs_halo_perio_sync_var_diag(const cs_halo_t *halo,
 void
 cs_halo_perio_rotate_rij(cs_real_t  *drdxyz);
 
+/*----------------------------------------------------------------------------
+ * Synchronize values for a real gradient of a tensor (symmetric interleaved)
+ * between periodic cells.
+ *
+ * parameters:
+ *   halo      <-> halo associated with variable to synchronize
+ *   sync_mode <-- kind of halo treatment (standard or extended)
+ *   var       <-> symmetric tensor to update (6 values)
+ *----------------------------------------------------------------------------*/
+
+void
+cs_halo_perio_sync_var_sym_tens_grad(const cs_halo_t  *halo,
+                                     cs_halo_type_t    sync_mode,
+                                     cs_real_t         var[]);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
