@@ -10,7 +10,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,7 +54,7 @@ BEGIN_C_DECLS
 
 /*
   Pointer to structures representing an interface and a list of interfaces.
-  The structures themselves are private, and is defined in cs_interface.c
+  The structures themselves are private, and are defined in cs_interface.c
 */
 
 typedef struct _cs_interface_t     cs_interface_t;
@@ -184,11 +184,9 @@ cs_interface_get_tr_index(const cs_interface_t  *itf);
  * if the periodicity structure provides for composed periodicities, so they
  * need not be defined prior to this function.
  *
- * The corresponding elements must be locally ordered.
- *
  * parameters:
  *   n_elts             <-- number of local elements considered
- *                          (size of parent_element_number[]
+ *                          (size of parent_element_id[]
  *   parent_element_id  <-- pointer to list of selected elements local
  *                          numbers (0 to n-1), or NULL if all first n_elts
  *                          elements are used
@@ -269,7 +267,7 @@ cs_interface_set_n_elts(const cs_interface_set_t  *ifs);
 
 const cs_interface_t *
 cs_interface_set_get(const cs_interface_set_t  *ifs,
-                      int                       interface_id);
+                     int                        interface_id);
 
 /*----------------------------------------------------------------------------
  * Return pointer to the periocicity structure associated of an interface set.
@@ -416,12 +414,38 @@ cs_interface_set_copy_indexed(const cs_interface_set_t  *ifs,
  *----------------------------------------------------------------------------*/
 
 void
-cs_interface_set_sum(cs_interface_set_t  *ifs,
-                     cs_lnum_t            n_elts,
-                     cs_lnum_t            stride,
-                     bool                 interlace,
-                     cs_datatype_t        datatype,
-                     void                *var);
+cs_interface_set_sum(const cs_interface_set_t  *ifs,
+                     cs_lnum_t                  n_elts,
+                     cs_lnum_t                  stride,
+                     bool                       interlace,
+                     cs_datatype_t              datatype,
+                     void                      *var);
+
+/*----------------------------------------------------------------------------
+ * Update to minimum value for elements associated with an interface set.
+ *
+ * On input, the variable array should contain local contributions. On output,
+ * contributions from matching elements on parallel or periodic boundaries
+ * have been added.
+ *
+ * Only the values of elements belonging to the interfaces are modified.
+ *
+ * parameters:
+ *   ifs       <-- pointer to a fvm_interface_set_t structure
+ *   n_elts    <-- number of elements in var buffer
+ *   stride    <-- number of values (non interlaced) by entity
+ *   interlace <-- true if variable is interlaced (for stride > 1)
+ *   datatype  <-- type of data considered
+ *   var       <-> variable buffer
+ *----------------------------------------------------------------------------*/
+
+void
+cs_interface_set_min(const cs_interface_set_t  *ifs,
+                     cs_lnum_t                  n_elts,
+                     cs_lnum_t                  stride,
+                     bool                       interlace,
+                     cs_datatype_t              datatype,
+                     void                      *var);
 
 /*----------------------------------------------------------------------------
  * Update to maximum value for elements associated with an interface set.
@@ -442,38 +466,12 @@ cs_interface_set_sum(cs_interface_set_t  *ifs,
  *----------------------------------------------------------------------------*/
 
 void
-cs_interface_set_max(cs_interface_set_t  *ifs,
-                     cs_lnum_t            n_elts,
-                     cs_lnum_t            stride,
-                     bool                 interlace,
-                     cs_datatype_t        datatype,
-                     void                *var);
-
-/*----------------------------------------------------------------------------
- * Update to maximum value for elements associated with an interface set.
- *
- * On input, the variable array should contain local contributions. On output,
- * contributions from matching elements on parallel or periodic boundaries
- * have been added.
- *
- * Only the values of elements belonging to the interfaces are modified.
- *
- * parameters:
- *   ifs       <-- pointer to a fvm_interface_set_t structure
- *   n_elts    <-- number of elements in var buffer
- *   stride    <-- number of values (non interlaced) by entity
- *   interlace <-- true if variable is interlaced (for stride > 1)
- *   datatype  <-- type of data considered
- *   var       <-> variable buffer
- *----------------------------------------------------------------------------*/
-
-void
-cs_interface_set_max(cs_interface_set_t  *ifs,
-                     cs_lnum_t            n_elts,
-                     cs_lnum_t            stride,
-                     bool                 interlace,
-                     cs_datatype_t        datatype,
-                     void                *var);
+cs_interface_set_max(const cs_interface_set_t  *ifs,
+                     cs_lnum_t                  n_elts,
+                     cs_lnum_t                  stride,
+                     bool                       interlace,
+                     cs_datatype_t              datatype,
+                     void                      *var);
 
 /*----------------------------------------------------------------------------
  * Dump printout of an interface list.
