@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -72,6 +72,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Local types and structures
  *============================================================================*/
@@ -298,6 +300,7 @@ _convert_to_offset(const unsigned char  buf[],
 
 #if defined(HAVE_STDINT_H)
 
+  #pragma _NEC novector
   for (i = 0; i < n; i++)
     val[i] = ((const uint64_t *)buf)[i];
 
@@ -1636,10 +1639,7 @@ _write_header(const char     *sec_name,
   header_vals[2] = location_id;
   header_vals[3] = index_id;
 
-  if (n_vals > 0)
-    header_vals[4] = n_location_vals;
-  else
-    header_vals[4] = 0;
+  header_vals[4] = n_location_vals;
 
   name_size = strlen(sec_name);
   name_pad_size = 8-(name_size%8); /* At least 1 NULL
@@ -1678,49 +1678,47 @@ _write_header(const char     *sec_name,
 
   outp->type_name = (char *)outp->buffer + 48;
 
-  if (n_vals > 0) {
-    switch(elt_type) {
-    case CS_FLOAT:
-      outp->type_name[0] = 'r';
-      if (sizeof(float) == 4)
-        outp->type_name[1] = '4';
-      else
-        outp->type_name[1] = '8';
-      assert(sizeof(float) == 4 || sizeof(float) == 8);
-      break;
-    case CS_DOUBLE:
-      outp->type_name[0] = 'r';
-      if (sizeof(double) == 8)
-        outp->type_name[1] = '8';
-      else {
-        outp->type_name[1] = '1';
-        outp->type_name[2] = '6';
-      }
-      assert(sizeof(double) == 8 || sizeof(float) == 16);
-      break;
-    case CS_INT32:
-      outp->type_name[0] = 'i';
+  switch(elt_type) {
+  case CS_FLOAT:
+    outp->type_name[0] = 'r';
+    if (sizeof(float) == 4)
       outp->type_name[1] = '4';
-      break;
-    case CS_INT64:
-      outp->type_name[0] = 'i';
+    else
       outp->type_name[1] = '8';
-      break;
-    case CS_UINT32:
-      outp->type_name[0] = 'u';
-      outp->type_name[1] = '4';
-      break;
-    case CS_UINT64:
-      outp->type_name[0] = 'u';
+    assert(sizeof(float) == 4 || sizeof(float) == 8);
+    break;
+  case CS_DOUBLE:
+    outp->type_name[0] = 'r';
+    if (sizeof(double) == 8)
       outp->type_name[1] = '8';
-      break;
-    case CS_CHAR:
-      outp->type_name[0] = 'c';
-      outp->type_name[1] = ' ';
-      break;
-    default:
-      break;
+    else {
+      outp->type_name[1] = '1';
+      outp->type_name[2] = '6';
     }
+    assert(sizeof(double) == 8 || sizeof(float) == 16);
+    break;
+  case CS_INT32:
+    outp->type_name[0] = 'i';
+    outp->type_name[1] = '4';
+    break;
+  case CS_INT64:
+    outp->type_name[0] = 'i';
+    outp->type_name[1] = '8';
+    break;
+  case CS_UINT32:
+    outp->type_name[0] = 'u';
+    outp->type_name[1] = '4';
+    break;
+  case CS_UINT64:
+    outp->type_name[0] = 'u';
+    outp->type_name[1] = '8';
+    break;
+  case CS_CHAR:
+    outp->type_name[0] = 'c';
+    outp->type_name[1] = ' ';
+    break;
+  default:
+    break;
   }
 
   if (embed == true)
@@ -1808,6 +1806,8 @@ _dump_index(const cs_io_sec_index_t  *idx)
   bft_printf("\n");
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -2801,9 +2801,9 @@ cs_io_write_global(const char     *sec_name,
     _write_padding(outp->body_align, outp);
 
     n_written = cs_file_write_global(outp->f,
-                                      elts,
-                                      cs_datatype_size[elt_type],
-                                      n_vals);
+                                     elts,
+                                     cs_datatype_size[elt_type],
+                                     n_vals);
 
     if (n_vals != (cs_gnum_t)n_written)
       bft_error(__FILE__, __LINE__, 0,
