@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -59,6 +59,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*-----------------------------------------------------------------------------
  * Local type definitions
  *-----------------------------------------------------------------------------*/
@@ -73,6 +75,15 @@ static FILE* _cs_log[] = {NULL, NULL, NULL};
 static const char* _cs_log_name[] = {"",
                                      "setup.log",
                                      "performance.log"};
+int cs_glob_log_frequency = 1;
+
+/*============================================================================
+ * Prototypes for functions intended for use only by Fortran wrappers.
+ * (descriptions follow, with function bodies).
+ *============================================================================*/
+
+void
+cs_f_log_frequency_get_pointer(int     **ntlist);
 
 /*============================================================================
  * Private function definitions
@@ -204,6 +215,28 @@ _log_strpad(char        *dest,
   dest[j] = '\0';
 }
 
+/*============================================================================
+ * Fortran wrapper function definitions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------
+ * Get pointer to log frequency (ntlist in Fortran).
+ *
+ * This function is intended for use by Fortran wrappers, and
+ * enables mapping to Fortran global pointers.
+ *
+ * parameters:
+ *   ntlist  --> pointer to ntlist
+ *----------------------------------------------------------------------------*/
+
+void
+cs_f_log_frequency_get_pointer(int     **ntlist)
+{
+  *ntlist = &cs_glob_log_frequency;
+}
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -328,6 +361,86 @@ cs_log_strpadl(char        *dest,
   _log_strpad(dest, src, width, destsize, 1);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Pretty-print int-32 based bit field to string
+ *
+ * \param[in]   code  value to print
+ * \param[out]  buf   output buffer (must be at least 33 bytes).
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_log_binary_pp_int32(int32_t     code,
+                       char     buf[33])
+{
+  int i;
+  int32_t n = code;
+
+  for (i = 0; i < 33; i++)
+    buf[i] = ' ';
+  buf[32] = '\0';
+  buf[31] = '0';
+
+  i = 31;
+  while (n && i > -1) {
+    if (n & 1)
+      buf[i] = '1';
+    else
+      buf[i] = '0';
+    n = n >> 1;
+    i--;
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print log info to a given log type.
+ *
+ * The format and variable arguments are similar to those of the vprintf()
+ * type functions.
+ *
+ * In parallel, output is only handled by rank 0.
+ *
+ * \param[in]  log      log file type
+ * \param[in]  format   format string, as printf() and family.
+ * \param[in]  arg_ptr  variable arguments list pointer
+ *
+ * \return number of characters printed, not counting the trailing '\0' used
+ *         to end output strings
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_log_vprintf(cs_log_t     log,
+               const char  *format,
+               va_list      arg_ptr)
+{
+  int  retval;
+
+  if (cs_glob_rank_id > 0)
+    return 0;
+
+  if (log != CS_LOG_DEFAULT) {
+
+    if (_cs_log[log] == NULL && log != CS_LOG_DEFAULT)
+      _open_log(log);
+
+    retval = vfprintf(_cs_log[log], format, arg_ptr);
+
+  }
+
+  else {
+
+    bft_printf_proxy_t *_printf_proxy = bft_printf_proxy_get();
+
+    retval = _printf_proxy(format, arg_ptr);
+
+  }
+
+  return retval;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Print log info to a given log type.
@@ -444,7 +557,7 @@ cs_log_separator(cs_log_t log)
 
   for (i = 0; i < 80; i++)
     separator[i] = '-';
-    separator[80] = '\0';
+  separator[80] = '\0';
 
   cs_log_printf(log, "%s\n", separator);
 }
@@ -558,4 +671,6 @@ cs_log_timer_array(cs_log_t                   log,
 
 /*-----------------------------------------------------------------------------*/
 
+
+
 END_C_DECLS
