@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,7 @@
 
 #include "cs_defs.h"
 #include "cs_timer.h"
+#include "stdarg.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -53,6 +54,7 @@ typedef enum {
 
 } cs_log_t;
 
+extern int cs_glob_log_frequency;
 
 /*============================================================================
  * Public macros
@@ -124,6 +126,42 @@ cs_log_strpadl(char        *dest,
                size_t       width,
                size_t       destsize);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Pretty-print int-32 based bit field to string
+ *
+ * \param[in]   code  value to print
+ * \param[out]  buf   output buffer (must be at least 33 bytes).
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_log_binary_pp_int32(int32_t     code,
+                       char     buf[33]);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print log info to a given log type.
+ *
+ * The format and variable arguments are similar to those of the vprintf()
+ * type functions.
+ *
+ * In parallel, output is only handled by rank 0.
+ *
+ * \param[in]  log      log file type
+ * \param[in]  format   format string, as printf() and family.
+ * \param[in]  arg_ptr  variable arguments list pointer
+ *
+ * \return number of characters printed, not counting the trailing '\0' used
+ *         to end output strings
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+cs_log_vprintf(cs_log_t     log,
+               const char  *format,
+               va_list      arg_ptr);
+
 /*----------------------------------------------------------------------------
  * Print log info to a given log type.
  *
@@ -141,11 +179,24 @@ cs_log_strpadl(char        *dest,
  *   to end output strings
  *----------------------------------------------------------------------------*/
 
+#if defined(__GNUC__)
+
+int
+cs_log_printf(cs_log_t     log,
+              const char  *format,
+              ...)
+  __attribute__((format(printf, 2, 3)));
+
+#else
+
 int
 cs_log_printf(cs_log_t     log,
               const char  *format,
               ...);
 
+#endif
+
+
 /*----------------------------------------------------------------------------
  * Flush for output of cs_log_printf() with modifiable behavior.
  *
