@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -46,6 +46,8 @@
 
 #include "cs_array_reduce.h"
 #include "cs_base.h"
+#include "cs_ctwr.h"
+#include "cs_fan.h"
 #include "cs_field.h"
 #include "cs_log.h"
 #include "cs_map.h"
@@ -53,8 +55,13 @@
 #include "cs_mesh_location.h"
 #include "cs_parall.h"
 #include "cs_prototypes.h"
+#include "cs_range_set.h"
 #include "cs_time_moment.h"
+#include "cs_time_plot.h"
 #include "cs_time_step.h"
+#include "cs_lagr_stat.h"
+#include "cs_lagr_log.h"
+
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -135,16 +142,13 @@ static double *_clips_vmin = NULL;
 static double *_clips_vmax = NULL;
 static cs_log_clip_t  *_clips = NULL;
 
+static cs_time_plot_t  *_l2_residual_plot = NULL;
+
 /*============================================================================
  * Prototypes for functions intended for use only by Fortran wrappers.
  * (descriptions follow, with function bodies).
  *============================================================================*/
 
-void
-cs_f_log_iteration(void);
-
-/*! \endcond (end ignore by Doxygen) */
-
 /*============================================================================
  * Fortran function prototypes for subroutines from field.f90.
  *============================================================================*/
@@ -328,7 +332,6 @@ _find_clip(int  f_id,
  *   prefix       <-- string inserted before name
  *   name         <-- array name or label
  *   name_width   <-- width of "name" column
- *   interleaved  <-- is array interleaved ?
  *   dim          <-- array dimension
  *   n_g_elts     <-- global number of associated elements,
  *   total_weight <-- if > 0, weight (volume or surface) of array location
@@ -336,27 +339,27 @@ _find_clip(int  f_id,
  *   vmax         <-- maximum values of each component or norm
  *   vsum         <-- sum of each component or norm
  *   wsum         <-- weighted sum of each component or norm, or NULL
+ *   fpe_flag     <-- was a "not a number" or floating-point error detected ?
  *----------------------------------------------------------------------------*/
 
 static void
 _log_array_info(const char        *prefix,
                 const char        *name,
                 size_t             name_width,
-                bool               interleaved,
                 int                dim,
                 int                n_g_elts,
                 double             total_weight,
                 double             vmin[],
                 const double       vmax[],
                 const double       vsum[],
-                const double      *wsum)
+                const double      *wsum,
+                int               *fpe_flag)
 {
-  int c_id;
-  const int _dim = (interleaved && dim == 3) ? 4 : dim;
+  const int _dim = (dim == 3) ? 4 : dim;
 
   char tmp_s[2][64] =  {"", ""};
 
-  for (c_id = 0; c_id < _dim; c_id++) {
+  for (int c_id = 0; c_id < _dim; c_id++) {
 
     if (dim == 3) {
       if (c_id < 3)
@@ -392,12 +395,16 @@ _log_array_info(const char        *prefix,
       cs_log_printf(CS_LOG_DEFAULT,
                     "%s%s  %14.5g  %14.5g  %14.5g\n",
                     prefix,
-                    tmp_s[c_id],
+                    tmp_s[0],
                     vmin[c_id],
                     vmax[c_id],
                     vsum[c_id] / n_g_elts);
 
+    /* Check NAN  and exit */
+    if (isnan(vsum[c_id]))
+      *fpe_flag = 1;
   }
+
 }
 
 /*----------------------------------------------------------------------------
@@ -419,66 +426,78 @@ _log_clip_info(const char        *prefix,
                const char        *name,
                size_t             name_width,
                int                dim,
-               cs_gnum_t          count_min,
-               cs_gnum_t          count_max,
+               cs_gnum_t          count_min[],
+               cs_gnum_t          count_max[],
                const double       vmin[],
                const double       vmax[])
 {
   int c_id;
-  const int _dim = (dim == 3) ? 4 : dim;
+  const int _dim = (dim == 1) ? dim : dim + 1;
 
   char tmp_s[2][64] =  {"", ""};
 
   for (c_id = 0; c_id < _dim; c_id++) {
 
+    int    _count_max, _count_min;
     double _vmin, _vmax;
     const char *_name = (c_id == 0) ? name : " ";
 
-    if (c_id < dim) {
+    if (c_id < _dim) {
+      _count_min = count_min[2*c_id];
+      _count_max = count_max[2*c_id];
       _vmin = vmin[c_id];
       _vmax = vmax[c_id];
     }
 
-    if (dim == 3) {
-      snprintf(tmp_s[1], 63, "%s%s", _name, cs_glob_field_comp_name_3[c_id]);
-      tmp_s[1][63] = '\0';
-      cs_log_strpad(tmp_s[0], tmp_s[1], name_width, 64);
-      if (c_id == dim) {
-        _vmin = sqrt(vmin[0]*vmin[0] + vmin[1]*vmin[1] + vmin[2]*vmin[2]);
-        _vmax = sqrt(vmax[0]*vmax[0] + vmax[1]*vmax[1] + vmax[2]*vmax[2]);
+    if (dim > 1) {
+      if (c_id == 0) {
+        snprintf(tmp_s[1], 63, "%s", _name);
+        tmp_s[1][63] = '\0';
+        cs_log_strpad(tmp_s[0], tmp_s[1], name_width, 64);
+        _vmin = 0.;
+        _vmax = 0.;
+        for (int i = 0; i< dim; i++) {
+          _vmin += vmin[i]*vmin[i];
+          _vmax += vmax[i]*vmax[i];
+        }
+        _vmin = sqrt(_vmin);
+        _vmax = sqrt(_vmax);
+      }
+      else {
+        if (dim == 3)
+          snprintf(tmp_s[1], 63, "%s%s", name, cs_glob_field_comp_name_3[c_id - 1]);
+        else if (dim == 6)
+          snprintf(tmp_s[1], 63, "%s%s", name, cs_glob_field_comp_name_6[c_id - 1]);
+        tmp_s[1][63] = '\0';
+        cs_log_strpad(tmp_s[0], tmp_s[1], name_width, 64);
       }
-    }
-    else if (dim == 6) {
-      snprintf(tmp_s[1], 63, "%s%s", _name, cs_glob_field_comp_name_6[c_id]);
-      tmp_s[1][63] = '\0';
-      cs_log_strpad(tmp_s[0], tmp_s[1], name_width, 64);
     }
     else
       cs_log_strpad(tmp_s[0], name, name_width, 64);
 
-    if (count_min > 0 && count_max > 0)
+    if (_count_min > 0 && _count_max > 0)
       cs_log_printf(CS_LOG_DEFAULT,
                     "%s%s  %14.5g  %14.5g  %12llu  %12llu\n",
                     prefix,
                     tmp_s[0],
                     _vmin,
                     _vmax,
-                    (unsigned long long)count_min,
-                    (unsigned long long)count_max);
-    else if (count_min > 0)
+                    (unsigned long long)_count_min,
+                    (unsigned long long)_count_max);
+    else if (_count_min > 0)
       cs_log_printf(CS_LOG_DEFAULT,
                     "%s%s  %14.5g                  %12llu\n",
                     prefix,
                     tmp_s[0],
                     _vmin,
-                    (unsigned long long)count_min);
-    else if (count_max > 0)
+                    (unsigned long long)_count_min);
+    else if (_count_max > 0)
       cs_log_printf(CS_LOG_DEFAULT,
                     "%s%s                  %14.5g                %12llu\n",
                     prefix,
                     tmp_s[0],
                     _vmax,
-                    (unsigned long long)count_max);
+                    (unsigned long long)_count_max);
     else
       cs_log_printf(CS_LOG_DEFAULT,
                     "%s%s\n",
@@ -497,6 +516,7 @@ _log_fields(void)
   int f_id, li, log_count;
 
   int log_count_max = 0;
+  int fpe_flag = 0;
   int     *log_id = NULL, *moment_id = NULL;
   double  *vmin = NULL, *vmax = NULL, *vsum = NULL, *wsum = NULL;
 
@@ -509,9 +529,12 @@ _log_fields(void)
   const int label_key_id = cs_field_key_id("label");
 
   const cs_mesh_location_type_t m_l[] = {CS_MESH_LOCATION_CELLS,
-                                         CS_MESH_LOCATION_BOUNDARY_FACES};
+                                         CS_MESH_LOCATION_INTERIOR_FACES,
+                                         CS_MESH_LOCATION_BOUNDARY_FACES,
+                                         CS_MESH_LOCATION_VERTICES
+                                         };
 
-  const cs_mesh_t *m = cs_glob_mesh;
+  cs_mesh_t *m = cs_glob_mesh;
   const cs_mesh_quantities_t *mq = cs_glob_mesh_quantities;
 
   /* Allocate working arrays */
@@ -537,13 +560,14 @@ _log_fields(void)
 
   /* Loop on locations */
 
-  for (li = 0; li < 2; li++) {
+  for (li = 0; li < 4; li++) {
 
     size_t max_name_width = cs_log_strlen(_("field"));
     int loc_id = m_l[li];
     int have_weight = 0;
     double total_weight = -1;
     cs_gnum_t n_g_elts = 0;
+    cs_real_t *gather_array = NULL; /* only if CS_MESH_LOCATION_VERTICES */
     const cs_lnum_t *n_elts = cs_mesh_location_get_n_elts(loc_id);
     const cs_lnum_t _n_elts = n_elts[0];
     const cs_real_t *weight = NULL;
@@ -556,6 +580,13 @@ _log_fields(void)
         have_weight = 1;
         total_weight = mq->tot_vol;
         break;
+      case CS_MESH_LOCATION_INTERIOR_FACES:
+        n_g_elts = m->n_g_i_faces;
+        weight = mq->i_face_surf;
+        cs_array_reduce_sum_l(_n_elts, 1, NULL, weight, &total_weight);
+        cs_parall_sum(1, CS_DOUBLE, &total_weight);
+        have_weight = 1;
+        break;
       case CS_MESH_LOCATION_BOUNDARY_FACES:
         n_g_elts = m->n_g_b_faces;
         weight = mq->b_face_surf;
@@ -563,6 +594,11 @@ _log_fields(void)
         cs_parall_sum(1, CS_DOUBLE, &total_weight);
         have_weight = 1;
         break;
+      case CS_MESH_LOCATION_VERTICES:
+        n_g_elts = m->n_g_vertices;
+        have_weight = 0;
+        BFT_MALLOC(gather_array, m->n_vertices, cs_real_t);
+        break;
       default:
         n_g_elts = _n_elts;
         cs_parall_counter(&n_g_elts, 1);
@@ -570,6 +606,9 @@ _log_fields(void)
       }
     }
 
+    if (n_g_elts == 0)
+      continue;
+
     /* First loop on fields */
 
     log_count = 0;
@@ -600,7 +639,7 @@ _log_fields(void)
 
       log_id[f_id] = log_count;
 
-      _dim = (f->interleaved && f->dim == 3) ? 4 : f->dim;
+      _dim = (f->dim == 3) ? 4 : f->dim;
 
       while (log_count + _dim > log_count_max) {
         log_count_max *= 2;
@@ -610,53 +649,52 @@ _log_fields(void)
         BFT_REALLOC(wsum, log_count_max, double);
       }
 
-      if (have_weight && (f->type | CS_FIELD_INTENSIVE)) {
-        if (f->interleaved)
-          cs_array_reduce_simple_stats_l_w(_n_elts,
-                                           f->dim,
-                                           NULL,
-                                           NULL,
-                                           f->val,
-                                           weight,
-                                           vmin + log_count,
-                                           vmax + log_count,
-                                           vsum + log_count,
-                                           wsum + log_count);
-
-        else {
-          for (c_id = 0; c_id < f->dim; c_id++)
-            cs_array_reduce_simple_stats_l_w(_n_elts,
-                                             1,
-                                             NULL,
-                                             NULL,
-                                             f->val + n_elts[2]*c_id,
-                                             weight,
-                                             vmin + log_count + c_id,
-                                             vmax + log_count + c_id,
-                                             vsum + log_count + c_id,
-                                             wsum + log_count + c_id);
-        }
-      }
-      else {
-        if (f->interleaved)
-          cs_array_reduce_simple_stats_l(_n_elts,
+      if (have_weight && (f->type & CS_FIELD_INTENSIVE)) {
+        cs_array_reduce_simple_stats_l_w(_n_elts,
                                          f->dim,
                                          NULL,
+                                         NULL,
                                          f->val,
+                                         weight,
                                          vmin + log_count,
                                          vmax + log_count,
-                                         vsum + log_count);
+                                         vsum + log_count,
+                                         wsum + log_count);
 
-        else {
-          for (c_id = 0; c_id < f->dim; c_id++)
-            cs_array_reduce_simple_stats_l(_n_elts,
-                                           1,
-                                           NULL,
-                                           f->val + n_elts[2]*c_id,
-                                           vmin + log_count + c_id,
-                                           vmax + log_count + c_id,
-                                           vsum + log_count + c_id);
+      }
+      else {
+
+        cs_real_t  *field_val = f->val;
+        cs_lnum_t  _n_cur_elts = _n_elts;
+        if (gather_array != NULL) { /* Eliminate shared values whose local
+                                       rank is not owner and compact */
+
+          if (m->vtx_range_set == NULL)
+            m->vtx_range_set = cs_range_set_create(m->vtx_interfaces,
+                                                   NULL,
+                                                   m->n_vertices,
+                                                   false, /* balance */
+                                                   0); /* g_id_base */
+
+          if (f->dim > 1)
+            BFT_REALLOC(gather_array, (f->dim * m->n_vertices), cs_real_t);
+
+          cs_range_set_gather(m->vtx_range_set,
+                              CS_REAL_TYPE,
+                              f->dim,
+                              f->val,
+                              gather_array);
+          field_val = gather_array;
+          _n_cur_elts = m->vtx_range_set->n_elts[0];
         }
+        cs_array_reduce_simple_stats_l(_n_cur_elts,
+                                       f->dim,
+                                       NULL,
+                                       field_val,
+                                       vmin + log_count,
+                                       vmax + log_count,
+                                       vsum + log_count);
+
         if (have_weight) {
           for (c_id = 0; c_id < _dim; c_id++)
             wsum[log_count + c_id] = 0.;
@@ -670,11 +708,18 @@ _log_fields(void)
         name = f->name;
 
       size_t l_name_width = cs_log_strlen(name);
+      if (f->dim == 3)
+        l_name_width += 3;
+      else if (f->dim > 3)
+        l_name_width += 4;
 
       max_name_width = CS_MAX(max_name_width, l_name_width);
 
     } /* End of first loop on fields */
 
+    if (gather_array != NULL)
+      BFT_FREE(gather_array);
+
     if (log_count < 1)
       continue;
 
@@ -701,7 +746,7 @@ _log_fields(void)
                   _("\n"
                     "  ** Computed fields on %s\n"
                     "     -------------------%.*s\n"),
-                  loc_name, loc_name_w, _underline);
+                  loc_name, (int)loc_name_w, _underline);
 
     cs_log_strpad(tmp_s[0], _("field"), max_name_width, 64);
     cs_log_strpadl(tmp_s[1], _("minimum"), 14, 64);
@@ -753,14 +798,14 @@ _log_fields(void)
 
       /* Position in log */
 
-      _dim = (f->interleaved && f->dim == 3) ? 4 : f->dim;
+      _dim = (f->dim == 3) ? 4 : f->dim;
 
       const char *name = cs_field_get_key_str(f, label_key_id);
       if (name == NULL)
         name = f->name;
 
       double t_weight = -1;
-      if (total_weight > 0 && (f->type | CS_FIELD_INTENSIVE))
+      if (total_weight > 0 && (f->type & CS_FIELD_INTENSIVE))
         t_weight = total_weight;
 
       char prefix[] = "v  ";
@@ -768,18 +813,20 @@ _log_fields(void)
         if (moment_id[f_id] > -1)
           prefix[0] = 'm';
       }
+      if (f->type & CS_FIELD_ACCUMULATOR)
+        prefix[0] = 'm';
 
       _log_array_info(prefix,
                       name,
                       max_name_width,
-                      f->interleaved,
                       f->dim,
                       n_g_elts,
                       t_weight,
                       vmin + log_count,
                       vmax + log_count,
                       vsum + log_count,
-                      wsum + log_count);
+                      wsum + log_count,
+                      &fpe_flag);
 
       log_count += _dim;
 
@@ -787,6 +834,11 @@ _log_fields(void)
 
   } /* End of loop on mesh locations */
 
+  /* Check NaN and exit */
+  if (fpe_flag == 1)
+    bft_error(__FILE__, __LINE__, 0,
+                _("Invalid (not-a-number) values detected for a field."));
+
   BFT_FREE(moment_id);
   BFT_FREE(wsum);
   BFT_FREE(vsum);
@@ -805,7 +857,9 @@ static void
 _log_sstats(void)
 {
   int     stat_id;
+  int     fpe_flag = 0;
   double _boundary_surf = -1;
+  double _interior_surf = -1;
   double  *vmin = NULL, *vmax = NULL, *vsum = NULL, *wsum = NULL;
 
   char tmp_s[5][64] =  {"", "", "", "", ""};
@@ -894,6 +948,17 @@ _log_sstats(void)
           have_weight = 1;
           total_weight = mq->tot_vol;
           break;
+        case CS_MESH_LOCATION_INTERIOR_FACES:
+          n_g_elts = m->n_g_i_faces;
+          weight = mq->i_face_surf;
+          if (_interior_surf < 0) {
+            cs_array_reduce_sum_l(_n_elts, 1, NULL, weight, &_interior_surf);
+            cs_parall_sum(1, CS_DOUBLE, &_interior_surf);
+            if (_interior_surf < 0) _interior_surf = 0; /* just to be safe */
+          }
+          total_weight = _interior_surf;
+          have_weight = 1;
+          break;
         case CS_MESH_LOCATION_BOUNDARY_FACES:
           n_g_elts = m->n_g_b_faces;
           weight = mq->b_face_surf;
@@ -905,6 +970,10 @@ _log_sstats(void)
           total_weight = _boundary_surf;
           have_weight = 1;
           break;
+        case CS_MESH_LOCATION_VERTICES:
+          n_g_elts = m->n_g_vertices;
+          have_weight = 0;
+          break;
         default:
           n_g_elts = _n_elts;
           cs_parall_counter(&n_g_elts, 1);
@@ -926,7 +995,7 @@ _log_sstats(void)
                     _("\n"
                       "  ** Computed values on %s\n"
                       "     -------------------%.*s\n"),
-                    loc_name, loc_name_w, _underline);
+                    loc_name, (int)loc_name_w, _underline);
 
       cs_log_strpad(tmp_s[0], _(cat_name), max_name_width, 64);
       cs_log_strpadl(tmp_s[1], _("minimum"), 14, 64);
@@ -981,14 +1050,14 @@ _log_sstats(void)
         _log_array_info("   ",
                         name,
                         max_name_width,
-                        true,
                         _sstats[stat_id].dim,
                         n_g_elts,
                         t_weight,
                         vmin + stat_id,
                         vmax + stat_id,
                         vsum + stat_id,
-                        wsum + stat_id);
+                        wsum + stat_id,
+                        &fpe_flag);
 
       } /* End of loop on stats */
 
@@ -998,6 +1067,11 @@ _log_sstats(void)
 
   } /* End of loop on mesh categories */
 
+  /* Check NaN and exit */
+  if (fpe_flag == 1)
+    bft_error(__FILE__, __LINE__, 0,
+                _("Invalid (not-a-number) values detected for a statistic."));
+
   BFT_FREE(wsum);
   BFT_FREE(vsum);
   BFT_FREE(vmax);
@@ -1026,7 +1100,9 @@ _add_clipping(int               name_id,
               cs_lnum_t         n_clip_min,
               cs_lnum_t         n_clip_max,
               const cs_real_t   min_pre_clip[],
-              const cs_real_t   max_pre_clip[])
+              const cs_real_t   max_pre_clip[],
+              cs_lnum_t         n_clip_min_comp[],
+              cs_lnum_t         n_clip_max_comp[])
 {
   bool need_sort = false;
 
@@ -1037,7 +1113,7 @@ _add_clipping(int               name_id,
   if (clip_id < 0) {
 
     _n_clips += 1;
-    _clips_val_size += dim;
+    _clips_val_size += (dim == 1) ? 1 : dim + 1;
 
     /* Reallocate key definitions if necessary */
 
@@ -1065,7 +1141,7 @@ _add_clipping(int               name_id,
     _clips[clip_id].f_id = f_id;
     _clips[clip_id].name_id = name_id;
     _clips[clip_id].dim = dim;
-    _clips[clip_id].v_idx = _clips_val_size - dim;
+    _clips[clip_id].v_idx = (dim == 1) ? _clips_val_size - dim : _clips_val_size - dim - 1;
 
   }
 
@@ -1097,12 +1173,23 @@ _add_clipping(int               name_id,
     qsort(_clips, _n_clips, sizeof(cs_log_clip_t), &_compare_clips);
 
   /* Update values */
-
-  for (int i = 0; i < dim; i++) {
-    _clips_vmin[v_idx + i] = min_pre_clip[i];
-    _clips_vmax[v_idx + i] = max_pre_clip[i];
-    _clips_count[(v_idx + i)*2] = n_clip_min;
-    _clips_count[(v_idx + i)*2 + 1] = n_clip_max;
+  if (dim > 1) {
+      _clips_count[(v_idx)*2] = n_clip_min;
+      _clips_count[(v_idx)*2 + 1] = n_clip_max;
+      _clips_vmin[v_idx] = min_pre_clip[0];
+      _clips_vmax[v_idx] = max_pre_clip[0];
+    for (int i = 0; i < dim; i++) {
+      _clips_vmin[v_idx + i + 1] = min_pre_clip[i];
+      _clips_vmax[v_idx + i + 1] = max_pre_clip[i];
+      _clips_count[(v_idx + i + 1)*2] = n_clip_min_comp[i];
+      _clips_count[(v_idx + i + 1)*2 + 1] = n_clip_max_comp[i];
+    }
+  }
+  else {
+    _clips_vmin[v_idx] = min_pre_clip[0];
+    _clips_vmax[v_idx] = max_pre_clip[0];
+    _clips_count[(v_idx)*2] = n_clip_min;
+    _clips_count[(v_idx)*2 + 1] = n_clip_max;
   }
 }
 
@@ -1147,12 +1234,14 @@ _log_clips(void)
 
     const char *name = NULL;
     int f_id = _clips[clip_id].f_id;
+    int f_dim = 0;
     if (f_id > 0) {
       const cs_field_t  *f = cs_field_by_id(f_id);
       name = cs_field_get_key_str(f, label_key_id);
       if (name == NULL)
         name = f->name;
       type_idx[1] = clip_id + 1;
+      f_dim = f->dim;
     }
     else {
       name = cs_map_name_to_id_reverse(_name_map, _clips[clip_id].name_id);
@@ -1162,6 +1251,10 @@ _log_clips(void)
     assert(name != NULL);
 
     size_t l_name_width = cs_log_strlen(name);
+    if (f_dim == 3)
+      l_name_width += 3;
+    else if (f_dim > 3)
+      l_name_width += 4;
     max_name_width = CS_MAX(max_name_width, l_name_width);
 
   }
@@ -1247,10 +1340,10 @@ _log_clips(void)
                       name,
                       max_name_width,
                      _clips[clip_id].dim,
-                     _clips_count[v_idx*2],
-                     _clips_count[v_idx*2 + 1],
-                     _clips_vmin + v_idx,
-                     _clips_vmax + v_idx);
+                     vcount + v_idx*2,
+                     vcount + v_idx*2 + 1,
+                     vmin + v_idx,
+                     vmax + v_idx);
     }
 
   }
@@ -1266,9 +1359,7 @@ _log_clips(void)
  * Fortran wrapper function definitions
  *============================================================================*/
 
-/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
-
-/*! \endcond (end ignore by Doxygen) */
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
  * Public function definitions
@@ -1309,6 +1400,9 @@ cs_log_iteration_destroy_all(void)
 
   if (_name_map != NULL)
     cs_map_name_to_id_destroy(&_name_map);
+
+  if (_l2_residual_plot != NULL)
+    cs_time_plot_finalize(&_l2_residual_plot);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1329,6 +1423,11 @@ cs_log_iteration(void)
     _log_sstats();
 
   cs_time_moment_log_iteration();
+  cs_lagr_stat_log_iteration();
+  cs_lagr_log_iteration();
+
+  cs_fan_log_iteration();
+  cs_ctwr_log_balance();
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1443,10 +1542,17 @@ cs_log_iteration_add_array(const char                     *name,
       weight = mq->cell_vol;
       have_weight = true;
       break;
+    case CS_MESH_LOCATION_INTERIOR_FACES:
+      weight = mq->i_face_surf;
+      have_weight = true;
+      break;
     case CS_MESH_LOCATION_BOUNDARY_FACES:
       weight = mq->b_face_surf;
       have_weight = true;
       break;
+    case CS_MESH_LOCATION_VERTICES:
+      have_weight = false;
+      break;
     default:
       break;
     }
@@ -1509,18 +1615,20 @@ cs_log_iteration_clipping(const char       *name,
 
   _add_clipping(name_id, -1, dim,
                 n_clip_min, n_clip_max,
-                min_pre_clip, max_pre_clip);
+                min_pre_clip, max_pre_clip,0,0);
 }
 
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Add or update clipping info for a field
  *
- * \param[in]  f_id          associated field id
- * \param[in]  n_clip_min    number of local clippings to minimum value
- * \param[in]  n_clip_max    number of local clippings to maximum value
- * \param[in]  min_pre_clip  minimum values prior to clipping
- * \param[in]  max_pre_clip  maximum values prior to clipping
+ * \param[in]  f_id            associated field id
+ * \param[in]  n_clip_min      number of local clippings to minimum value
+ * \param[in]  n_clip_max      number of local clippings to maximum value
+ * \param[in]  min_pre_clip    minimum values prior to clipping
+ * \param[in]  max_pre_clip    maximum values prior to clipping
+ * \param[in]  n_clip_min_comp number of clip min by component
+ * \param[in]  n_clip_max_comp number of clip max by component
  */
 /*----------------------------------------------------------------------------*/
 
@@ -1529,13 +1637,92 @@ cs_log_iteration_clipping_field(int               f_id,
                                 cs_lnum_t         n_clip_min,
                                 cs_lnum_t         n_clip_max,
                                 const cs_real_t   min_pre_clip[],
-                                const cs_real_t   max_pre_clip[])
+                                const cs_real_t   max_pre_clip[],
+                                cs_lnum_t         n_clip_min_comp[],
+                                cs_lnum_t         n_clip_max_comp[])
 {
   const cs_field_t  *f = cs_field_by_id(f_id);
 
   _add_clipping(-1, f_id, f->dim,
                 n_clip_min, n_clip_max,
-                min_pre_clip, max_pre_clip);
+                min_pre_clip, max_pre_clip,n_clip_min_comp,n_clip_max_comp);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Log L2 time residual for every variable fields.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_log_l2residual(void)
+{
+  if (cs_glob_rank_id > 0)
+    return;
+
+  const cs_time_step_t *ts = cs_glob_time_step;
+  const int n_fields = cs_field_n_fields();
+
+  /* write header */
+
+  if (_l2_residual_plot == NULL) {
+
+    int                    _plot_buffer_steps = -1;
+    double                 _plot_flush_wtime = 3600;
+    cs_time_plot_format_t  _plot_format = CS_TIME_PLOT_CSV;
+    bool                   use_iteration = (ts->is_local) ? true : false;
+
+    const char **labels;
+    BFT_MALLOC(labels, n_fields + 1, const char *);
+
+    int n_variables = 0;
+    for (int f_id = 0 ; f_id < n_fields ; f_id++) {
+      const cs_field_t *f = cs_field_by_id(f_id);
+      if (f->type & CS_FIELD_VARIABLE) {
+        labels[n_variables] = f->name;
+        n_variables++;
+      }
+    }
+
+    _l2_residual_plot = cs_time_plot_init_probe("residuals",
+                                                "",
+                                                _plot_format,
+                                                use_iteration,
+                                                _plot_flush_wtime,
+                                                _plot_buffer_steps,
+                                                n_variables,
+                                                NULL,
+                                                NULL,
+                                                labels);
+
+    BFT_FREE(labels);
+  }
+
+  {
+    cs_real_t *vals;
+    BFT_MALLOC(vals, n_fields, cs_real_t);
+
+    int si_k_id = cs_field_key_id("solving_info");
+
+    int n_variables = 0;
+    for (int f_id = 0 ; f_id < n_fields ; f_id++) {
+      const cs_field_t *f = cs_field_by_id(f_id);
+      if (f->type & CS_FIELD_VARIABLE) {
+        const cs_solving_info_t *sinfo
+          = cs_field_get_key_struct_const_ptr(f, si_k_id);
+        vals[n_variables] = sinfo->l2residual;
+        n_variables += 1;
+      }
+    }
+
+    cs_time_plot_vals_write(_l2_residual_plot,
+                            ts->nt_cur,
+                            ts->t_cur,
+                            n_variables,
+                            vals);
+
+    BFT_FREE(vals);
+  }
 }
 
 /*----------------------------------------------------------------------------*/
