@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -134,7 +134,16 @@ cs_log_iteration_clipping_field(int               f_id,
                                 cs_lnum_t         n_clip_min,
                                 cs_lnum_t         n_clip_max,
                                 const cs_real_t   min_pre_clip[],
-                                const cs_real_t   max_pre_clip[]);
+                                const cs_real_t   max_pre_clip[],
+                                cs_lnum_t         n_clip_min_comp[],
+                                cs_lnum_t         n_clip_max_comp[]);
+
+/*----------------------------------------------------------------------------
+ * Log L2 time residual for every variable fields.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_log_l2residual(void);
 
 /*----------------------------------------------------------------------------*/
 
