@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -68,10 +68,14 @@
 #include "cs_mesh.h"
 #include "cs_measures_util.h"
 
+#include "cs_field.h"
+
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -145,9 +149,11 @@ _mesh_interpol_create_connect(cs_interpol_grid_t   *ig)
   }
 
   fvm_point_location_nodal(nodal_mesh,
+                           0,
                            0.1,
                            0,
                            nb_points,
+                           NULL,
                            coords,
                            location,
                            distance);
@@ -174,9 +180,10 @@ _mesh_interpol_create_connect(cs_interpol_grid_t   *ig)
   }
 #endif
 
+  /* copy location and switch to 0-based */
 #   pragma omp parallel for
   for (ii = 0; ii < nb_points; ii++) {
-    ig->cell_connect[ii] = location[ii];
+    ig->cell_connect[ii] = location[ii] - 1;
   }
 
   nodal_mesh = fvm_nodal_destroy(nodal_mesh);
@@ -206,10 +213,10 @@ cs_interpol_field_on_grid(cs_interpol_grid_t         *ig,
 
 #   pragma omp parallel for private(jj)
   for (ii = 0; ii < nb_points; ii++) {
-    if (ig->cell_connect[ii] > 0 && ig->cell_connect[ii] < mesh->n_cells +1)
+    if (ig->cell_connect[ii] > -1 && ig->cell_connect[ii] < mesh->n_cells)
       for (jj = 0; jj < ms_dim; jj++)
         interpoled_values[ii*ms_dim +jj] =
-          values_to_interpol[(ig->cell_connect[ii] -1)*ms_dim + jj];
+          values_to_interpol[(ig->cell_connect[ii])*ms_dim + jj];
   }
 
 #if defined(HAVE_MPI)
@@ -241,16 +248,7 @@ cs_cressman_interpol(cs_measures_set_t         *ms,
                      cs_real_t                 *interpolated_values,
                      int                        id_type)
 {
-  cs_lnum_t ii;
-  cs_lnum_t jj;
-  cs_lnum_t n_elts;
-  cs_real_t interpolated_value;
-  cs_real_t dist_x;
-  cs_real_t dist_y;
-  cs_real_t dist_z;
-  cs_real_t weight;
-  cs_real_t total_weight;
-  cs_real_t r2;
+  cs_lnum_t n_elts = 0;
   cs_real_t *xyz_cen;
   const cs_mesh_t *mesh = cs_glob_mesh;
   const cs_mesh_quantities_t *mesh_quantities = cs_glob_mesh_quantities;
@@ -264,29 +262,27 @@ cs_cressman_interpol(cs_measures_set_t         *ms,
     xyz_cen = mesh_quantities->b_face_cog;
   }
 
-#   pragma omp parallel for private(jj)
-  for (ii = 0; ii < n_elts; ii++) {
-    total_weight = 0.;
-    interpolated_value = 0.;
-    for (jj = 0; jj < ms->nb_measures; jj++) {
+# pragma omp parallel for
+  for (cs_lnum_t ii = 0; ii < n_elts; ii++) {
+    cs_real_t total_weight = 0.;
+    cs_real_t interpolated_value = 0.;
+    for (cs_lnum_t jj = 0; jj < ms->nb_measures; jj++) {
       if (ms->is_cressman[jj] == 1) {
-        dist_x = (xyz_cen[ii*3   ] - ms->coords[jj*3   ])
-                *ms->inf_radius[jj*3   ];
-        dist_y = (xyz_cen[ii*3 +1] - ms->coords[jj*3 +1])
-                *ms->inf_radius[jj*3 +1];
-        dist_z = (xyz_cen[ii*3 +2] - ms->coords[jj*3 +2])
-                *ms->inf_radius[jj*3 +2];
+        cs_real_t dist_x = (xyz_cen[ii*3   ] - ms->coords[jj*3   ])
+                          *ms->inf_radius[jj*3   ];
+        cs_real_t dist_y = (xyz_cen[ii*3 +1] - ms->coords[jj*3 +1])
+                          *ms->inf_radius[jj*3 +1];
+        cs_real_t dist_z = (xyz_cen[ii*3 +2] - ms->coords[jj*3 +2])
+                          *ms->inf_radius[jj*3 +2];
 
-        r2 = dist_x*dist_x + dist_y*dist_y + dist_z*dist_z;
+        cs_real_t r2 = dist_x*dist_x + dist_y*dist_y + dist_z*dist_z;
 
-          if (r2/4. > 700.)
-            weight = 0.;
-          else
-            weight = exp(-r2/4.);
+        cs_real_t weight = 0.;
+        if (r2/4. <= 700.)
+          weight = exp(-r2/4.);
 
-
-          total_weight += weight;
-          interpolated_value += (ms->measures[jj])*weight;
+        total_weight += weight;
+        interpolated_value += (ms->measures[jj])*weight;
       }
     }
 
@@ -411,8 +407,8 @@ cs_interpol_grid_init(cs_interpol_grid_t    *ig,
   cs_lnum_t ii;
   BFT_MALLOC(ig->cell_connect, nb_points, cs_lnum_t);
 #if defined(HAVE_MPI)
-      if (cs_glob_n_ranks > 1)
-        BFT_MALLOC(ig->rank_connect, nb_points, int);
+  if (cs_glob_n_ranks > 1)
+    BFT_MALLOC(ig->rank_connect, nb_points, int);
 #endif
   BFT_MALLOC(ig->coords, 3*nb_points, cs_real_t);
 
@@ -426,7 +422,6 @@ cs_interpol_grid_init(cs_interpol_grid_t    *ig,
   _mesh_interpol_create_connect(ig);
 
   ig->is_connect = true;
-
 }
 
 /*----------------------------------------------------------------------------
@@ -525,7 +520,7 @@ cs_measures_set_create(const char   *name,
     ms->is_cressman = NULL;
     ms->is_interpol = NULL;
     ms->inf_radius = NULL;
-
+    ms->comp_ids = NULL;
   }
   else {
     BFT_FREE(ms->coords);
@@ -533,6 +528,7 @@ cs_measures_set_create(const char   *name,
     BFT_FREE(ms->is_cressman);
     BFT_FREE(ms->is_interpol);
     BFT_FREE(ms->inf_radius);
+    BFT_FREE(ms->comp_ids);
   }
 
   return ms;
@@ -551,7 +547,7 @@ cs_measures_set_create(const char   *name,
  *                          0: not taken into account
  *                          1: taken into account
  *   measures_coords  <-- measures spaces coordonates
- *   measures         <-- measures values (associated to coordonates)
+ *   measures         <-- measures values (associated to coordinates)
  *   influence_radius <-- influence radius for interpolation (xyz interleaved)
  *----------------------------------------------------------------------------*/
 
@@ -595,7 +591,7 @@ cs_measures_set_map_values(cs_measures_set_t       *ms,
 #   pragma omp parallel for private(jj)
       for (ii = 0; ii < nb_measures; ii++) {
         for (jj = 0; jj < dim; jj++)
-          ms->measures[ii*dim + jj] = measures[ii*nb_measures + jj];
+          ms->measures[ii*dim + jj] = measures[jj*nb_measures + ii];
       }
     }
   }
@@ -614,7 +610,6 @@ cs_measures_set_map_values(cs_measures_set_t       *ms,
         ms->inf_radius[ii*3 +jj] = influence_radius[ii*3 + jj];
       }
   }
-
 }
 
 /*----------------------------------------------------------------------------
@@ -815,6 +810,7 @@ cs_measures_sets_destroy(void)
     BFT_FREE(ms->coords);
     BFT_FREE(ms->is_interpol);
     BFT_FREE(ms->is_cressman);
+    BFT_FREE(ms->comp_ids);
   }
 
   BFT_FREE(_measures_sets);
@@ -850,6 +846,8 @@ cs_interpol_grids_destroy(void)
   _n_grids_max = 0;
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions for Fortran API
  *============================================================================*/
