@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -51,6 +51,8 @@ typedef struct _cs_measures_set_t {
   int                 type;                /* Measures set type flag
                                               not yet used */
   int                 dim;                 /* Measures set dimension */
+  int                *comp_ids;            /* index of components of measures if
+                                              dim > 1 */
   cs_lnum_t           nb_measures;         /* Number of measures */
   cs_lnum_t           nb_measures_max;     /* Dynamic reallocation parameter */
   bool                interleaved;         /* Is measures set interleaved ? */
@@ -61,7 +63,7 @@ typedef struct _cs_measures_set_t {
                                               interpolation ?
                                                 0: no
                                                 1: yes */
-  cs_real_t          *coords;              /* measures coordonates */
+  cs_real_t          *coords;              /* measures coordinates */
   cs_real_t          *measures;            /* measures values */
   cs_real_t          *inf_radius;          /* influence radius */
 
@@ -73,7 +75,7 @@ typedef struct _cs_interpol_grid_t {
   int                 id;                  /* Grid id */
   cs_lnum_t           nb_points;           /* Number of grid points */
   bool                is_connect;          /* Is connectivity computed? */
-  cs_real_t          *coords;              /* measures coordonates */
+  cs_real_t          *coords;              /* measures coordinates */
   cs_lnum_t          *cell_connect;        /* Mesh -> grid connectivity */
   int                *rank_connect;        /* Rank location */
 
@@ -132,7 +134,6 @@ cs_interpol_grid_init(cs_interpol_grid_t    *ig,
  * parameters:
  *   name        <-- measures set name
  *   type_flag   <-- mask of field property and category values (not used yet)
- *   location_id <-- id of associated location
  *   dim         <-- measure set dimension (number of components)
  *   interleaved <-- if dim > 1, indicate if field is interleaved
  *
@@ -198,7 +199,7 @@ cs_measures_set_add_values(cs_measures_set_t       *ms,
                            const int               *is_interpol,
                            const cs_real_t         *measures_coords,
                            const cs_real_t         *measures,
-                           const cs_real_t         *influence_raidus);
+                           const cs_real_t         *influence_radius);
 
 /*----------------------------------------------------------------------------
  * Compute a Cressman interpolation on the global mesh.
