@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@
 
 #include <assert.h>
 #include <stdio.h>
+#include <string.h>
 
 /*----------------------------------------------------------------------------
  *  Local headers
@@ -50,6 +51,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Local structure definitions
  *============================================================================*/
@@ -277,7 +280,7 @@ _order_gnum_s(const cs_gnum_t   number[],
  *   true if element i1 is greater or equal than element i2, false otherwise
  *----------------------------------------------------------------------------*/
 
-inline static _Bool
+inline static bool
 _indexed_is_greater_or_equal(size_t            i1,
                              size_t            i2,
                              const cs_lnum_t   index[],
@@ -328,7 +331,7 @@ _indexed_is_greater_or_equal(size_t            i1,
  *   true if element i1 is strictly greater than element i2, false otherwise
  *----------------------------------------------------------------------------*/
 
-inline static _Bool
+inline static bool
 _indexed_is_greater(size_t            i1,
                     size_t            i2,
                     const cs_lnum_t   index[],
@@ -673,6 +676,98 @@ _order_lnum_s(const cs_lnum_t   number[],
   }
 }
 
+/*----------------------------------------------------------------------------
+ * Descend binary tree for the ordering of a cs_real_t array.
+ *
+ * parameters:
+ *   value    <-- pointer to values of entities that should be ordered.
+ *   level    <-- level of the binary tree to descend
+ *   nb_ent   <-- number of entities in the binary tree to descend
+ *   order    <-> ordering array
+ *----------------------------------------------------------------------------*/
+
+inline static void
+_order_real_descend_tree(const cs_real_t   value[],
+                         size_t            level,
+                         const size_t      nb_ent,
+                         cs_lnum_t         order[])
+{
+  size_t i_save, i1, i2, lv_cur;
+
+  i_save = (size_t)(order[level]);
+
+  while (level <= (nb_ent/2)) {
+
+    lv_cur = (2*level) + 1;
+
+    if (lv_cur < nb_ent - 1) {
+
+      i1 = (size_t)(order[lv_cur+1]);
+      i2 = (size_t)(order[lv_cur]);
+
+      if (value[i1] > value[i2]) lv_cur++;
+    }
+
+    if (lv_cur >= nb_ent) break;
+
+    i1 = i_save;
+    i2 = (size_t)(order[lv_cur]);
+
+    if (value[i1] >= value[i2]) break;
+
+    order[level] = order[lv_cur];
+    level = lv_cur;
+
+  }
+
+  order[level] = i_save;
+}
+
+/*----------------------------------------------------------------------------
+ * Order an array of local values.
+ *
+ * parameters:
+ *   value   <-- array of entity values
+ *   order   <-- pre-allocated ordering table
+ *   nb_ent  <-- number of entities considered
+ *----------------------------------------------------------------------------*/
+
+static void
+_order_real(const cs_real_t   value[],
+            cs_lnum_t         order[],
+            const size_t      nb_ent)
+{
+  size_t i;
+  cs_lnum_t o_save;
+
+  /* Initialize ordering array */
+
+  for (i = 0 ; i < nb_ent ; i++)
+    order[i] = i;
+
+  if (nb_ent < 2)
+    return;
+
+  /* Create binary tree */
+
+  i = (nb_ent / 2) ;
+  do {
+    i--;
+    _order_real_descend_tree(value, i, nb_ent, order);
+  } while (i > 0);
+
+  /* Sort binary tree */
+
+  for (i = nb_ent - 1 ; i > 0 ; i--) {
+    o_save   = order[0];
+    order[0] = order[i];
+    order[i] = o_save;
+    _order_real_descend_tree(value, 0, i, order);
+  }
+}
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
@@ -738,89 +833,6 @@ cs_order_gnum_test(const cs_lnum_t  list[],
     return 0;
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Test if an array of global numbers is lexicographically ordered.
- *
- * \param[in]  list    optional list (1 to n numbering) of selected entities
- *                     (or NULL if all nb_ent are selected). This list may
- *                     contain element numbers in any order
- * \param[in]  number  array of all entity numbers (number of entity i given
- *                     by number[i] or number[list[i] - 1]) if list exists
- *                     (if NULL, a default 1 to n numbering is considered)
- * \param[in]  stride  stride of number array (number of values to compare)
- * \param[in]  nb_ent  number of entities considered
- *
- * \return  1 if ordered, 0 otherwise.
- */
-/*----------------------------------------------------------------------------*/
-
-int
-cs_order_gnum_test_s(const cs_lnum_t  list[],
-                     const cs_gnum_t  number[],
-                     size_t           stride,
-                     size_t           nb_ent)
-{
-  size_t j;
-  size_t i = 0;
-
-  /* If numbering is explicit */
-
-  if (number != NULL) {
-
-    if (list != NULL) {
-      for (i = 1 ; i < nb_ent ; i++) {
-        size_t j_prev, k;
-        bool unordered = false;
-        j_prev = list[i-1] - 1;
-        j = list[i] - 1;
-        for (k = 0; k < stride; k++) {
-          if (number[j_prev*stride + k] < number[j*stride + k])
-            break;
-          else if (number[j_prev*stride + k] > number[j*stride + k])
-            unordered = true;
-        }
-        if (unordered == true)
-          break;
-      }
-    }
-    else {
-      for (i = 1 ; i < nb_ent ; i++) {
-        size_t i_prev, k;
-        bool unordered = false;
-        i_prev = i-1;
-        for (k = 0; k < stride; k++) {
-          if (number[i_prev*stride + k] < number[i*stride + k])
-            break;
-          else if (number[i_prev*stride + k] > number[i*stride + k])
-            unordered = true;
-        }
-        if (unordered == true)
-          break;
-      }
-    }
-
-  /* If numbering is implicit */
-
-  }
-  else {
-
-    if (list != NULL) {
-      for (i = 1 ; i < nb_ent ; i++) {
-        if (list[i] < list[i-1])
-          break;
-      }
-    }
-    else
-      i = nb_ent;
-  }
-
-  if (i == nb_ent || nb_ent == 0)
-    return 1;
-  else
-    return 0;
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Return an ordering table associated with an array of global numbers.
@@ -963,9 +975,7 @@ cs_order_gnum_allocated(const cs_lnum_t  list[],
       BFT_MALLOC(number_list, nb_ent, cs_gnum_t);
       for (i = 0 ; i < nb_ent ; i++)
         number_list[i] = number[list[i] - 1];
-      _order_gnum(number_list,
-                   order,
-                   nb_ent);
+      _order_gnum(number_list, order, nb_ent);
       BFT_FREE(number_list);
     }
     else
@@ -983,9 +993,7 @@ cs_order_gnum_allocated(const cs_lnum_t  list[],
       BFT_MALLOC(number_list, nb_ent, cs_gnum_t);
       for (i = 0 ; i < nb_ent ; i++)
         number_list[i] = (cs_gnum_t)(list[i]);
-      _order_gnum(number_list,
-                   order,
-                   nb_ent);
+      _order_gnum(number_list, order, nb_ent);
       BFT_FREE(number_list);
     }
     else {
@@ -1035,17 +1043,11 @@ cs_order_gnum_allocated_s(const cs_lnum_t  list[],
         for (j = 0; j < stride; j++)
           number_list[i*stride + j] = number[(list[i] - 1)*stride + j];
       }
-      _order_gnum_s(number_list,
-                     stride,
-                     order,
-                     nb_ent);
+      _order_gnum_s(number_list, stride, order, nb_ent);
       BFT_FREE(number_list);
     }
     else
-      _order_gnum_s(number,
-                     stride,
-                     order,
-                     nb_ent);
+      _order_gnum_s(number, stride, order, nb_ent);
 
   }
 
@@ -1053,10 +1055,7 @@ cs_order_gnum_allocated_s(const cs_lnum_t  list[],
 
   else
 
-    cs_order_gnum_allocated(list,
-                              NULL,
-                              order,
-                              nb_ent);
+    cs_order_gnum_allocated(list, NULL, order, nb_ent);
 
 }
 
@@ -1253,16 +1252,16 @@ cs_order_lnum_allocated_s(const cs_lnum_t  list[],
           number_list[i*stride + j] = number[(list[i] - 1)*stride + j];
       }
       _order_lnum_s(number_list,
-                     stride,
-                     order,
-                     nb_ent);
+                    stride,
+                    order,
+                    nb_ent);
       BFT_FREE(number_list);
     }
     else
       _order_lnum_s(number,
-                     stride,
-                     order,
-                     nb_ent);
+                    stride,
+                    order,
+                    nb_ent);
 
   }
 
@@ -1271,9 +1270,50 @@ cs_order_lnum_allocated_s(const cs_lnum_t  list[],
   else
 
     cs_order_lnum_allocated(list,
-                              NULL,
-                              order,
-                              nb_ent);
+                            NULL,
+                            order,
+                            nb_ent);
+
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute an ordering table associated with an array of local values.
+ *
+ * \param[in]   list    optional list (1 to n numbering) of selected entities
+ *                      (or NULL if all nb_ent are selected). This list may
+ *                      contain element numbers in any order
+ * \param[in]   val     array of all entity values (value of entity i given
+ *                      by value[i] or value[list[i] - 1]) if list exists
+ * \param[out]  order   pointer to pre-allocated ordering table
+ * \param[in]   nb_ent  number of entities considered
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_order_real_allocated(const cs_lnum_t  list[],
+                        const cs_real_t  val[],
+                        cs_lnum_t        order[],
+                        size_t           nb_ent)
+{
+  size_t i;
+  cs_real_t *val_list;
+
+  /* Explicit numbering */
+
+  if (list != NULL) {
+    BFT_MALLOC(val_list, nb_ent, cs_real_t);
+    for (i = 0 ; i < nb_ent ; i++)
+      val_list[i] = val[list[i] - 1];
+    _order_real(val_list,
+                order,
+                nb_ent);
+    BFT_FREE(val_list);
+  }
+  else
+    _order_real(val,
+                order,
+                nb_ent);
 
 }
 
@@ -1326,6 +1366,42 @@ cs_order_renumbering(const cs_lnum_t  order[],
   return number;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Reorder data based on ordering array.
+ *
+ * \param[in]      n_elts      number of elements
+ * \param[in]      elt_size    element size
+ * \param[in]      order       reordering array
+ * \param[in,out]  data        data
+ *
+ * \return  new size of data
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_order_reorder_data(cs_lnum_t         n_elts,
+                      size_t            elt_size,
+                      const cs_lnum_t   order[],
+                      void             *data)
+{
+  unsigned char *tmp;
+  unsigned char *_data = data;
+
+  BFT_MALLOC(tmp, n_elts*elt_size, unsigned char);
+
+  for (cs_lnum_t i = 0; i < n_elts; i++) {
+    cs_lnum_t j = order[i];
+    const unsigned char *src = _data + j*elt_size;
+    unsigned char *dest = tmp + i*elt_size;
+    for (size_t k = 0; k < elt_size; k++)
+      dest[k] = src[k];
+  }
+  memcpy(data, tmp, n_elts*elt_size);
+
+  BFT_FREE(tmp);
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
