@@ -1,11 +1,11 @@
 /*============================================================================
- * ICoCo coupling
+ * ParaMEDMEM coupling
  *============================================================================*/
 
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -42,20 +42,20 @@
 #endif
 
 /*----------------------------------------------------------------------------
- * MED library headers
+ * MEDCOUPLING library headers
  *----------------------------------------------------------------------------*/
 
-#include <mpi.h>
-#include <ParaFIELD.hxx>
-#include <ParaMESH.hxx>
-#include <InterpKernelDEC.hxx>
-
-#include <MED_version.h>
+#include <MEDCoupling_version.h>
 
 #include <MEDCouplingUMesh.hxx>
 #include <MEDCouplingField.hxx>
 #include <MEDCouplingFieldDouble.hxx>
 
+#if defined(HAVE_PARAMEDMEM)
+#include <ParaFIELD.hxx>
+#include <ParaMESH.hxx>
+#include <InterpKernelDEC.hxx>
+
 /*----------------------------------------------------------------------------
  *  Local headers
  *----------------------------------------------------------------------------*/
@@ -69,6 +69,7 @@
 #include "cs_parall.h"
 #include "cs_prototypes.h"
 #include "cs_selector.h"
+#include "cs_timer.h"
 
 #include "fvm_defs.h"
 #include "fvm_nodal_from_desc.h"
@@ -77,31 +78,43 @@
  *  Header for the current file
  *----------------------------------------------------------------------------*/
 
+#include "cs_medcoupling_utils.hxx"
 #include "cs_paramedmem_coupling.hxx"
 
 /*----------------------------------------------------------------------------*/
 
-using namespace ParaMEDMEM;
+using namespace MEDCoupling;
 
 /*=============================================================================
- * Local Macro Definitions
+ * Private global variables
  *============================================================================*/
 
+static int                          _n_paramed_couplers = 0;
+static cs_paramedmem_coupling_t   **_paramed_couplers = NULL;
+
+const int cs_medcpl_cell_field = 0;
+const int cs_medcpl_vertex_field = 1;
+
+const int cs_medcpl_no_time = 0;
+const int cs_medcpl_one_time = 1;
+const int cs_medcpl_linear_time = 2;
+
 /*=============================================================================
  * Local Structure Definitions
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- * MED field structure
+ * ParaMEDMED field structure
  *----------------------------------------------------------------------------*/
 
 typedef struct {
 
   int                       mesh_id;       /* Associated mesh structure id */
-
   int                       dim;           /* Field dimension */
+
   TypeOfTimeDiscretization  td;            /* NO_TIME, ONE_TIME, LINEAR_TIME,
                                               or CONST_ON_TIME_INTERVAL */
+
   MEDCouplingFieldDouble   *f;             /* Pointer to MED coupling field */
 
   ParaFIELD                *pf;            /* Pointer to ParaMEDMEM field */
@@ -109,23 +122,30 @@ typedef struct {
 } _paramedmem_field_t;
 
 /*----------------------------------------------------------------------------
- * MED mesh structure
+ * ParaMEDMED mesh structure
  *----------------------------------------------------------------------------*/
 
-/* Structure associated with ParaMEDMEM entity */
-
 typedef struct {
 
+  cs_medcoupling_mesh_t *mesh;        /* The Code_Saturne stracture englobing
+                                         the medcoupling mesh structure */
+
   char               *sel_criteria;   /* Element selection criteria */
 
-  int                 direction;      /* 1: send, 2: receive, 3: both */
   int                 elt_dim;        /* Element dimension */
 
   cs_lnum_t           n_elts;         /* Number of coupled elements */
   cs_lnum_t          *elt_list;       /* List of associated elements
-                                         (0 to n-1) */
+                                        (0 to n-1) */
+  int                *new_to_old;     /* Connectivity used if only a section of
+                                        the mesh is read */
 
   MEDCouplingUMesh   *med_mesh;       /* MED mesh structure */
+
+  cs_real_t          *bbox;           /* Bounding box to optimize search */
+
+  int                 direction;      /* 1: send, 2: receive, 3: both */
+
   ParaMESH           *para_mesh[2];   /* parallel MED mesh structures
                                          for send (0) and receive (1) */
 
@@ -150,405 +170,18 @@ struct _cs_paramedmem_coupling_t {
   InterpKernelDEC           *send_dec;       /* Send data exchange channel */
   InterpKernelDEC           *recv_dec;       /* Receive data exchange channel */
 
+  int                       send_synced;
+  int                       recv_synced;
 };
 
-/*============================================================================
- *  Global variables
- *============================================================================*/
+/*----------------------------------------------------------------------------
+ * MEDCoupling parallel interpolation structure
+ *----------------------------------------------------------------------------*/
 
 /*============================================================================
  * Private function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Assign vertex coordinates to MEDCoupling structures
- *
- * parameters:
- *   mesh     <-- pointer to parent mesh structure that should be written.
- *   n_vtx    <-- number of vertices to assign.
- *   vtx_id   <-- id of initial vertices in new (sub) mesh, or -1
- *                (size: mesh->n_vtx).
- *   med_mesh <-> pointer to associated MEDCoupling mesh structure.
- *----------------------------------------------------------------------------*/
-
-static void
-_assign_vertex_coords(const cs_mesh_t    *mesh,
-                      cs_lnum_t           n_vtx,
-                      const cs_lnum_t    *vtx_id,
-                      MEDCouplingUMesh   *med_mesh)
-{
-  int  i, j;
-
-  const int  dim = mesh->dim;
-  const cs_coord_t  *vertex_coords = mesh->vtx_coord;
-
-  assert(med_mesh != NULL);
-
-  /* Assign all coordinates */
-  /*------------------------*/
-
-  DataArrayDouble *med_coords = DataArrayDouble::New();
-  med_coords->alloc(n_vtx, dim);
-
-  if (vtx_id != NULL) {
-    for (i = 0; i < mesh->n_vertices; i++) {
-      if (vtx_id[i] > -1) {
-        for (j = 0; j < dim; j++)
-          med_coords->setIJ(vtx_id[i], j, vertex_coords[i*dim + j]);
-      }
-    }
-  }
-  else {
-    for (i = 0; i < mesh->n_vertices; i++) {
-      for (j = 0; j < dim; j++)
-        med_coords->setIJ(i, j, vertex_coords[i*dim + j]);
-    }
-  }
-
-  med_mesh->setCoords(med_coords);
-  med_coords->decrRef();
-
-  return;
-}
-
-/*----------------------------------------------------------------------------
- * Assign boundary faces to a MEDCoupling structure
- *
- * parameters:
- *   mesh      <-- parent mesh structure
- *   n_elts    <-- number of selected elements
- *   elts_list <-- list of selected elements (1 to n numbering),
- *                 or NULL (if all are selected)
- *   med_mesh  <-> associated MEDCouplingUMesh structure
- *----------------------------------------------------------------------------*/
-
-static void
-_assign_face_mesh(const cs_mesh_t   *mesh,
-                  cs_lnum_t          n_elts,
-                  const cs_lnum_t   *elts_list,
-                  MEDCouplingUMesh  *med_mesh)
-{
-  cs_lnum_t i, j, k;
-  INTERP_KERNEL::NormalizedCellType type;
-
-  cs_lnum_t vtx_count = 0;
-  int elt_buf_size = 4;
-  int *elt_buf = NULL;
-  cs_lnum_t *vtx_id = NULL;
-
-  const int perm_tri[4] = {0, 2, 1};
-  const int perm_quad[4] = {0, 3, 2, 1};
-
-  /* Mark and renumber vertices */
-
-  BFT_MALLOC(vtx_id, mesh->n_vertices, cs_lnum_t);
-
-  for (i = 0; i < mesh->n_vertices; i++)
-    vtx_id[i] = -1;
-
-  /* Case with filter list */
-
-  if (elts_list != NULL) {
-
-    for (i = 0; i < n_elts; i++) {
-      cs_lnum_t eid = elts_list[i] - 1;
-      for (j = mesh->b_face_vtx_idx[eid];
-           j < mesh->b_face_vtx_idx[eid+1];
-           j++) {
-        cs_lnum_t vid = mesh->b_face_vtx_lst[j-1] - 1;
-        if (vtx_id[vid] < 0)
-          vtx_id[vid] = vtx_count++;
-      }
-    }
-
-  }
-  else {
-
-    for (i = 0; i < n_elts; i++) {
-      for (j = mesh->b_face_vtx_idx[i];
-           j < mesh->b_face_vtx_idx[i+1];
-           j++) {
-        cs_lnum_t vid = mesh->b_face_vtx_lst[j-1] - 1;
-        if (vtx_id[vid] < 0)
-          vtx_id[vid] = vtx_count++;
-      }
-    }
-
-  }
-
-  /* Assign coordinates */
-
-  _assign_vertex_coords(mesh, vtx_count, vtx_id, med_mesh);
-
-  /* Assign faces */
-
-  BFT_MALLOC(elt_buf, elt_buf_size, int);
-  med_mesh->allocateCells(n_elts);
-
-  for (i = 0; i < n_elts; i++) {
-
-    cs_lnum_t eid = (elts_list != NULL) ? elts_list[i] - 1 : i;
-
-    assert(eid >= 0 && eid < mesh->n_b_faces);
-
-    int n_vtx = mesh->b_face_vtx_idx[eid+1] - mesh->b_face_vtx_idx[eid];
-
-    cs_lnum_t connect_start = mesh->b_face_vtx_idx[eid] - 1;
-
-    if (n_vtx > elt_buf_size) { /* reallocate buffer if required */
-      elt_buf_size *= 2;
-      BFT_REALLOC(elt_buf, elt_buf_size, int);
-    }
-
-    switch(n_vtx) {
-    case 3:
-      type = INTERP_KERNEL::NORM_TRI3;
-      for (j = 0; j < 3; j++)
-        elt_buf[j]
-          = vtx_id[mesh->b_face_vtx_lst[connect_start + perm_tri[j]] - 1];
-      break;
-    case 4:
-      type = INTERP_KERNEL::NORM_QUAD4;
-      for (j = 0; j < 4; j++)
-        elt_buf[j]
-          = vtx_id[mesh->b_face_vtx_lst[connect_start + perm_quad[j]] - 1];
-      break;
-    default:
-      type = INTERP_KERNEL::NORM_POLYGON;
-      for (j = 0; j < n_vtx; j++)
-        elt_buf[j]
-          = vtx_id[mesh->b_face_vtx_lst[connect_start + n_vtx - 1 - j] - 1];
-      break;
-    }
-
-    med_mesh->insertNextCell(type, n_vtx, elt_buf);
-
-  }
-
-  med_mesh->finishInsertingCells();
-
-  BFT_FREE(elt_buf);
-  BFT_FREE(vtx_id);
-}
-
-/*----------------------------------------------------------------------------
- * Assign cells to a MEDCoupling structure
- *
- * parameters:
- *   mesh      <-- parent mesh structure
- *   n_elts    <-- number of selected elements
- *   elts_list <-- list of selected elements (1 to n numbering),
- *                 or NULL (if all are selected)
- *   med_mesh  <-> associated MEDCouplingUMesh structure
- *----------------------------------------------------------------------------*/
-
-static void
-_assign_cell_mesh(const cs_mesh_t   *mesh,
-                  cs_lnum_t          n_elts,
-                  const cs_lnum_t   *elts_list,
-                  MEDCouplingUMesh  *med_mesh)
-{
-  cs_lnum_t i, j, k;
-  cs_lnum_t  c_id, c_id1, c_id2, face_id, n_loc_cells;
-  fvm_element_t fvm_type;
-  INTERP_KERNEL::NormalizedCellType type;
-
-  cs_lnum_t vtx_count = 0, cell_count = 0;
-
-  int elt_buf_size = 8;
-  int *elt_buf = NULL;
-  cs_lnum_t *vtx_id = NULL;
-  cs_lnum_t *cell_id = NULL;
-  cs_lnum_t *cell_faces_idx = NULL, *cell_faces_num = NULL;
-
-  /* Build old->new cell id indirection */
-
-  BFT_MALLOC(cell_id, mesh->n_cells, cs_lnum_t);
-  for (i = 0; i < mesh->n_cells; i++)
-    cell_id[i] = -1;
-
-  if (elts_list != NULL) {
-    for (i = 0; i < n_elts; i++)
-      cell_id[elts_list[i-1]] = cell_count++;
-  }
-  else {
-    for (i = 0; i < n_elts; i++)
-      cell_id[elts_list[i-1]] = i;
-  }
-
-  /* Mark and renumber vertices */
-
-  BFT_MALLOC(vtx_id, mesh->n_vertices, cs_lnum_t);
-
-  for (face_id = 0; face_id < mesh->n_b_faces; face_id++) {
-    c_id = cell_id[mesh->b_face_cells[face_id] - 1];
-    if (c_id > -1) {
-      for (j = mesh->b_face_vtx_idx[face_id];
-           j < mesh->b_face_vtx_idx[face_id+1];
-           j++) {
-        cs_lnum_t vid = mesh->b_face_vtx_lst[j-1] - 1;
-        if (vtx_id[vid] < 0)
-          vtx_id[vid] = vtx_count++;
-      }
-    }
-  }
-
-  for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
-    c_id1 = mesh->i_face_cells[face_id*2    ] - 1;
-    c_id2 = mesh->i_face_cells[face_id*2 + 1] - 1;
-    c_id1 = (c_id1 < mesh->n_cells) ? cell_id[c_id1] : -1;
-    c_id2 = (c_id2 < mesh->n_cells) ? cell_id[c_id2] : -1;
-    if (c_id1 > -1 || c_id2 > -1) {
-      for (j = mesh->i_face_vtx_idx[face_id];
-           j < mesh->i_face_vtx_idx[face_id+1];
-           j++) {
-        cs_lnum_t vid = mesh->i_face_vtx_lst[j-1] - 1;
-        if (vtx_id[vid] < 0)
-          vtx_id[vid] = vtx_count++;
-      }
-    }
-  }
-
-  /* Assign coordinates */
-
-  _assign_vertex_coords(mesh, vtx_count, vtx_id, med_mesh);
-
-  /* Build temporary descending connectivity */
-
-  cs_mesh_connect_get_cell_faces(mesh,
-                                 mesh->n_cells,
-                                 cell_id,
-                                 &cell_faces_idx,
-                                 &cell_faces_num);
-
-  BFT_FREE(cell_id);
-
-  /* Now loop on cells */
-
-  const cs_lnum_t  face_num_shift[3] = {0,
-                                        mesh->n_b_faces,
-                                        mesh->n_b_faces + mesh->n_i_faces};
-  const cs_lnum_t  *face_vertices_idx[2] = {mesh->b_face_vtx_idx,
-                                            mesh->i_face_vtx_idx};
-  const cs_lnum_t  *face_vertices_num[2] = {mesh->b_face_vtx_lst,
-                                            mesh->i_face_vtx_lst};
-
-  BFT_MALLOC(elt_buf, elt_buf_size, int);
-
-  for (i = 0; i < n_elts; i++) {
-
-    int n_vtx;
-    cs_lnum_t cell_vtx[8];
-
-    fvm_element_t fvm_type = fvm_nodal_from_desc_cell(i,
-                                                      2,
-                                                      face_num_shift,
-                                                      face_vertices_idx,
-                                                      face_vertices_num,
-                                                      cell_faces_idx,
-                                                      cell_faces_num,
-                                                      cell_vtx);
-
-    switch(fvm_type) {
-
-    case FVM_CELL_TETRA:
-      type = INTERP_KERNEL::NORM_TETRA4;
-      n_vtx = 4;
-      elt_buf[0] = vtx_id[0];
-      elt_buf[1] = vtx_id[2];
-      elt_buf[2] = vtx_id[1];
-      elt_buf[3] = vtx_id[3];
-      break;
-
-    case FVM_CELL_PYRAM:
-      type = INTERP_KERNEL::NORM_PYRA5;
-      n_vtx = 5;
-      elt_buf[0] = vtx_id[0];
-      elt_buf[1] = vtx_id[3];
-      elt_buf[2] = vtx_id[2];
-      elt_buf[3] = vtx_id[1];
-      elt_buf[4] = vtx_id[4];
-      break;
-
-    case FVM_CELL_PRISM:
-      type = INTERP_KERNEL::NORM_PENTA6;
-      n_vtx = 6;
-      elt_buf[0] = vtx_id[0];
-      elt_buf[1] = vtx_id[2];
-      elt_buf[2] = vtx_id[1];
-      elt_buf[3] = vtx_id[3];
-      elt_buf[4] = vtx_id[5];
-      elt_buf[5] = vtx_id[4];
-      break;
-
-    case FVM_CELL_HEXA:
-      type = INTERP_KERNEL::NORM_HEXA8;
-      n_vtx = 8;
-      elt_buf[0] = vtx_id[0];
-      elt_buf[1] = vtx_id[3];
-      elt_buf[2] = vtx_id[2];
-      elt_buf[3] = vtx_id[1];
-      elt_buf[4] = vtx_id[4];
-      elt_buf[5] = vtx_id[7];
-      elt_buf[6] = vtx_id[6];
-      elt_buf[7] = vtx_id[5];
-      break;
-
-    default:
-      type = INTERP_KERNEL::NORM_POLYHED;
-
-      int connect_size = 0;
-
-      n_vtx = 0;
-
-      for (j = cell_faces_idx[i]; j < cell_faces_idx[i]; j++) {
-
-        face_id = CS_ABS(cell_faces_num[j-1]) - 1;
-
-        int fl = 1;
-        if (face_id < face_num_shift[fl])
-          fl = 0;
-        face_id -= face_num_shift[fl];
-
-        cs_lnum_t v_id_start = face_vertices_idx[fl][face_id] - 1;
-        cs_lnum_t v_id_end   = face_vertices_idx[fl][face_id + 1] - 1;
-        int n_face_vertices = v_id_end - v_id_start;
-
-        connect_size += n_face_vertices + 1;
-        if (connect_size > elt_buf_size) { /* reallocate buffer if required */
-          elt_buf_size *= 2;
-          BFT_REALLOC(elt_buf, elt_buf_size, int);
-        }
-
-        /* Add separator after first face */
-        if (j > cell_faces_idx[i])
-          elt_buf[n_vtx++] = -1;
-
-        /* Add face vertices */
-        if (cell_faces_num[j-1] > 0) {
-          for (k = v_id_start; k < v_id_end; k++)
-            elt_buf[n_vtx++] = vtx_id[face_vertices_num[fl][k] - 1];
-        }
-        else {
-          for (k = v_id_end - 1; k >= v_id_start; k--)
-            elt_buf[n_vtx++] = vtx_id[face_vertices_num[fl][k] - 1];
-        }
-
-      }
-
-    }
-
-    med_mesh->insertNextCell(type, n_vtx, elt_buf);
-  }
-
-  med_mesh->finishInsertingCells();
-
-  BFT_FREE(elt_buf);
-  BFT_FREE(cell_faces_num);;
-  BFT_FREE(cell_faces_idx);
-  BFT_FREE(vtx_id);
-}
-
 /*----------------------------------------------------------------------------
  * Initialize mesh for ParaMEDMEM coupling.
  *
@@ -558,63 +191,32 @@ _assign_cell_mesh(const cs_mesh_t   *mesh,
  *----------------------------------------------------------------------------*/
 
 static void
-_init_mesh(cs_paramedmem_coupling_t  *coupling,
-           _paramedmem_mesh_t        *mesh)
+_init_mesh_coupling(cs_paramedmem_coupling_t  *coupling,
+                    _paramedmem_mesh_t        *mesh)
 {
-  cs_lnum_t i;
   cs_mesh_t *parent_mesh = cs_glob_mesh;
 
   assert(mesh != NULL);
 
-  /* Creation of a new nodal mesh from selected cells */
+  /* Building the MED representation of the internal mesh */
+  cs_medcoupling_mesh_copy_from_base(parent_mesh, mesh->mesh, 0);
 
-  if (mesh->elt_dim == 3) {
-
-    BFT_MALLOC(mesh->elt_list, parent_mesh->n_cells, cs_lnum_t);
-
-    cs_selector_get_cell_list(mesh->sel_criteria,
-                              &(mesh->n_elts),
-                              mesh->elt_list);
-
-    BFT_REALLOC(mesh->elt_list, mesh->n_elts, cs_lnum_t);
-
-    _assign_cell_mesh(parent_mesh,
-                      mesh->n_elts,
-                      mesh->elt_list,
-                      mesh->med_mesh);
-
-  }
-
-  /* Creation of a new nodal mesh from selected border faces */
-
-  else if (mesh->elt_dim == 2) {
-
-    BFT_MALLOC(mesh->elt_list, parent_mesh->n_b_faces, cs_lnum_t);
-
-    cs_selector_get_b_face_list(mesh->sel_criteria,
-                                &(mesh->n_elts),
-                                mesh->elt_list);
-
-    BFT_REALLOC(mesh->elt_list, mesh->n_elts, cs_lnum_t);
-
-    _assign_face_mesh(parent_mesh,
-                      mesh->n_elts,
-                      mesh->elt_list,
-                      mesh->med_mesh);
-
-  }
+  /* Linking the pointers in order to simplify the calls*/
+  mesh->sel_criteria = mesh->mesh->sel_criteria;
+  mesh->elt_dim      = mesh->mesh->elt_dim;
+  mesh->n_elts       = mesh->mesh->n_elts;
+  mesh->elt_list     = mesh->mesh->elt_list;
+  mesh->new_to_old   = mesh->mesh->new_to_old;
+  mesh->med_mesh     = mesh->mesh->med_mesh;
+  mesh->bbox         = mesh->mesh->bbox;
 
   /* Define associated ParaMESH */
-
-  if (mesh->direction & 1)
-    mesh->para_mesh[0] = new ParaMESH(mesh->med_mesh,
-                                      *(coupling->send_dec->getSourceGrp()),
-                                      "source mesh");
-
-  if (mesh->direction & 2)
-    mesh->para_mesh[1] = new ParaMESH(mesh->med_mesh,
-                                      *(coupling->recv_dec->getTargetGrp()),
-                                      "target mesh");
+  mesh->para_mesh[0] = new ParaMESH(mesh->med_mesh,
+                                    *(coupling->send_dec->getSourceGrp()),
+                                    "source mesh");
+  mesh->para_mesh[1] = new ParaMESH(mesh->med_mesh,
+                                    *(coupling->recv_dec->getTargetGrp()),
+                                    "target mesh");
 }
 
 /*----------------------------------------------------------------------------
@@ -625,7 +227,7 @@ _init_mesh(cs_paramedmem_coupling_t  *coupling,
  *----------------------------------------------------------------------------*/
 
 static void
-_destroy_mesh(_paramedmem_mesh_t **mesh)
+_destroy_mesh(_paramedmem_mesh_t  **mesh)
 {
   _paramedmem_mesh_t *m = *mesh;
 
@@ -644,6 +246,48 @@ _destroy_mesh(_paramedmem_mesh_t **mesh)
   BFT_FREE(*mesh);
 }
 
+/*----------------------------------------------------------------------------
+ * Create an InterpKernelDEC object based on two lists, and their sizes,
+ * of mpi ranks (within MPI_COMM_WORLD).
+ *
+ * parameters:
+ *   grp1_global_ranks <-- array of ranks of group 1
+ *   grp1_size         <-- size of grp1_global_ranks array
+ *   grp2_global_ranks <-- array of ranks of group 2
+ *   grp2_size         <-- size of grp2_global_ranks array
+ *
+ * return:
+ *   new InterpKernelDEC object
+ *----------------------------------------------------------------------------*/
+
+static InterpKernelDEC *
+_cs_paramedmem_create_InterpKernelDEC(int  *grp1_global_ranks,
+                                      int   grp1_size,
+                                      int  *grp2_global_ranks,
+                                      int   grp2_size)
+{
+  /* Group 1 id's */
+  std::set<int> grp1_ids;
+  for (int i = 0; i < grp1_size; i++) {
+    grp1_ids.insert(grp1_global_ranks[i]);
+  }
+
+  /* Group 2 id's */
+  std::set<int> grp2_ids;
+  for (int i = 0; i < grp2_size; i++) {
+    grp2_ids.insert(grp2_global_ranks[i]);
+  }
+
+  /* Create the InterpKernel DEC */
+  InterpKernelDEC *NewDec = new InterpKernelDEC(grp1_ids, grp2_ids);
+
+  return NewDec;
+}
+
+#if defined(HAVE_MEDCOUPLING_LOADER)
+
+#endif /* HAVE_MEDCOUPLING_LOADER */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -657,7 +301,7 @@ _destroy_mesh(_paramedmem_mesh_t **mesh)
  *   recv_dec <-- receive Data Exchange Channel
  *----------------------------------------------------------------------------*/
 
-cs_paramedmem_coupling_t  *
+cs_paramedmem_coupling_t *
 cs_paramedmem_create(const char       *name,
                      InterpKernelDEC  *send_dec,
                      InterpKernelDEC  *recv_dec)
@@ -683,6 +327,125 @@ cs_paramedmem_create(const char       *name,
   return c;
 }
 
+/*----------------------------------------------------------------------------
+ * Create a paramedmem coupling based on an InterpKernelDEC.
+ *
+ * The latter is created using the the lists of ranks provided as
+ * input to this function.
+ *
+ * parameters:
+ *   name              <-- coupling name
+ *   grp1_global_ranks <-- array of ranks of group 1
+ *   grp1_size         <-- size of grp1_global_ranks array
+ *   grp2_global_ranks <-- array of ranks of group 2
+ *   grp2_size         <-- size of grp2_global_ranks array
+ *
+ * return:
+ *   pointer to new coupling object
+ *----------------------------------------------------------------------------*/
+
+static void
+_add_paramedmem_interpkernel(const char  *name,
+                             int         *grp1_global_ranks,
+                             int          grp1_size,
+                             int         *grp2_global_ranks,
+                             int          grp2_size)
+{
+
+  if (_paramed_couplers == NULL)
+    BFT_MALLOC(_paramed_couplers,
+               1,
+               cs_paramedmem_coupling_t *);
+  else
+    BFT_REALLOC(_paramed_couplers,
+                _n_paramed_couplers + 1,
+                cs_paramedmem_coupling_t *);
+
+  cs_paramedmem_coupling_t *c = NULL;
+
+  /* Add corresponding coupling to temporary ICoCo couplings array */
+
+  BFT_MALLOC(c, 1, cs_paramedmem_coupling_t);
+
+  BFT_MALLOC(c->name, strlen(name) + 1, char);
+  strcpy(c->name, name);
+
+  c->n_meshes = 0;
+  c->meshes = NULL;
+
+  c->n_fields = 0;
+  c->fields = NULL;
+
+  c->send_synced = 0;
+  c->recv_synced = 0;
+
+  bool is_in_grp1 = false;
+  int my_rank;
+  MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);
+
+  for (int ii = 0; ii < grp1_size; ii++) {
+    if (my_rank == grp1_global_ranks[ii]) {
+      is_in_grp1 = true;
+      break;
+    }
+  }
+
+  if (is_in_grp1) {
+    c->send_dec = _cs_paramedmem_create_InterpKernelDEC(grp1_global_ranks,
+                                                        grp1_size,
+                                                        grp2_global_ranks,
+                                                        grp2_size);
+
+    c->recv_dec = _cs_paramedmem_create_InterpKernelDEC(grp2_global_ranks,
+                                                        grp2_size,
+                                                        grp1_global_ranks,
+                                                        grp1_size);
+  } else {
+    c->recv_dec = _cs_paramedmem_create_InterpKernelDEC(grp1_global_ranks,
+                                                        grp1_size,
+                                                        grp2_global_ranks,
+                                                        grp2_size);
+
+    c->send_dec = _cs_paramedmem_create_InterpKernelDEC(grp2_global_ranks,
+                                                        grp2_size,
+                                                        grp1_global_ranks,
+                                                        grp1_size);
+  }
+
+  _paramed_couplers[_n_paramed_couplers] = c;
+
+  _n_paramed_couplers++;
+
+}
+
+cs_paramedmem_coupling_t *
+cs_paramedmem_coupling_by_id(int  pc_id)
+{
+
+  cs_paramedmem_coupling_t * c = _paramed_couplers[pc_id];
+
+  return c;
+
+}
+
+cs_paramedmem_coupling_t *
+cs_paramedmem_interpkernel_create(const char  *name,
+                                  int         *grp1_global_ranks,
+                                  int          grp1_size,
+                                  int         *grp2_global_ranks,
+                                  int          grp2_size)
+{
+  _add_paramedmem_interpkernel(name,
+                              grp1_global_ranks,
+                              grp1_size,
+                              grp2_global_ranks,
+                              grp2_size);
+
+  cs_paramedmem_coupling_t *c = _paramed_couplers[_n_paramed_couplers-1];
+
+  return c;
+}
+
 /*----------------------------------------------------------------------------
  * Define new ParaMEDMEM coupling.
  *
@@ -712,7 +475,6 @@ cs_paramedmem_destroy(cs_paramedmem_coupling_t  **coupling)
     for (int i = 0; i < c->n_meshes; i++)
       _destroy_mesh(&(c->meshes[i]));
 
-    BFT_FREE(c->fields);
     c->n_meshes = 0;
     c->meshes = NULL;
 
@@ -746,26 +508,26 @@ cs_paramedmem_define_mesh(cs_paramedmem_coupling_t  *coupling,
                           bool                       is_dest)
 {
   int id;
-  char *coupled_mesh_name = NULL;
-  cs_lnum_t *elt_list = NULL;
 
-  _paramedmem_mesh_t *mesh = NULL;
+  _paramedmem_mesh_t *pmmesh = NULL;
+  cs_medcoupling_mesh_t *mesh = NULL;
 
   assert(coupling != NULL);
 
   /* Initialization */
 
-  BFT_MALLOC(mesh, 1, _paramedmem_mesh_t);
+  BFT_MALLOC(pmmesh, 1, _paramedmem_mesh_t);
+  BFT_MALLOC(mesh, 1, cs_medcoupling_mesh_t);
 
   BFT_MALLOC(mesh->sel_criteria, strlen(select_criteria) + 1, char);
   strcpy(mesh->sel_criteria, select_criteria);
 
-  mesh->direction = 0;
+  pmmesh->direction = 0;
 
   if (is_source)
-    mesh->direction += 1;
+    pmmesh->direction += 1;
   if (is_dest)
-    mesh->direction += 2;
+    pmmesh->direction += 2;
 
   mesh->elt_dim = elt_dim;
 
@@ -779,8 +541,10 @@ cs_paramedmem_define_mesh(cs_paramedmem_coupling_t  *coupling,
   mesh->med_mesh->setTimeUnit("s");
   mesh->med_mesh->setMeshDimension(elt_dim);
 
-  mesh->para_mesh[0] = NULL;
-  mesh->para_mesh[1] = NULL;
+  pmmesh->para_mesh[0] = NULL;
+  pmmesh->para_mesh[1] = NULL;
+
+  mesh->new_to_old = NULL;
 
   /* Add as new MEDCoupling mesh structure */
 
@@ -789,7 +553,8 @@ cs_paramedmem_define_mesh(cs_paramedmem_coupling_t  *coupling,
 
   BFT_REALLOC(coupling->meshes, coupling->n_meshes, _paramedmem_mesh_t *);
 
-  coupling->meshes[id] = mesh;
+  pmmesh->mesh = mesh;
+  coupling->meshes[id] = pmmesh;
 
   return id;
 }
@@ -798,14 +563,14 @@ cs_paramedmem_define_mesh(cs_paramedmem_coupling_t  *coupling,
  * Initialize nodal coupled meshes.
  *
  * parameters:
- *   coupling        <-- partially initialized ParaMEDMEM coupling structure
+ *   coupling  <-- partially initialized ParaMEDMEM coupling structure
  *----------------------------------------------------------------------------*/
 
 void
 cs_paramedmem_init_meshes(cs_paramedmem_coupling_t  *coupling)
 {
   for (int i = 0; i < coupling->n_meshes; i++)
-    _init_mesh(coupling, coupling->meshes[i]);
+    _init_mesh_coupling(coupling, coupling->meshes[i]);
 }
 
 /*----------------------------------------------------------------------------
@@ -831,12 +596,8 @@ cs_paramedmem_mesh_id(cs_paramedmem_coupling_t  *coupling,
   assert(coupling != NULL);
 
   for (i = 0; i < coupling->n_meshes; i++) {
-#if (SALOMEMED_VERSION  >= 0x070300)
-    if (   strcmp(mesh_name, coupling->meshes[i]->med_mesh->getName().c_str())
+    if (strcmp(mesh_name, coupling->meshes[i]->med_mesh->getName().c_str())
         == 0)
-#else
-    if (strcmp(mesh_name, coupling->meshes[i]->med_mesh->getName()) == 0)
-#endif
       break;
   }
 
@@ -858,8 +619,8 @@ cs_paramedmem_mesh_id(cs_paramedmem_coupling_t  *coupling,
  *----------------------------------------------------------------------------*/
 
 cs_lnum_t
-cs_paramedmem_mesh_get_n_elts(const cs_paramedmem_coupling_t *coupling,
-                              int                             mesh_id)
+cs_paramedmem_mesh_get_n_elts(const cs_paramedmem_coupling_t  *coupling,
+                              int                              mesh_id)
 {
   cs_lnum_t retval = 0;
 
@@ -878,8 +639,8 @@ cs_paramedmem_mesh_get_n_elts(const cs_paramedmem_coupling_t *coupling,
  *----------------------------------------------------------------------------*/
 
 const cs_lnum_t *
-cs_paramedmem_mesh_get_elt_list(const cs_paramedmem_coupling_t *coupling,
-                                int                             mesh_id)
+cs_paramedmem_mesh_get_elt_list(const cs_paramedmem_coupling_t  *coupling,
+                                int                              mesh_id)
 {
   const cs_lnum_t *retval = NULL;
 
@@ -910,14 +671,28 @@ cs_paramedmem_field_add(cs_paramedmem_coupling_t  *coupling,
                         const char                *name,
                         int                        mesh_id,
                         int                        dim,
-                        TypeOfField                type,
-                        TypeOfTimeDiscretization   td,
+                        int                        medcpl_field_type,
+                        int                        medcpl_time_discr,
                         int                        dirflag)
 {
   int f_id = -1;
   _paramedmem_mesh_t *mesh = coupling->meshes[mesh_id];
 
   /* Prepare coupling structure */
+  TypeOfField type = ON_CELLS;
+  if (medcpl_field_type == cs_medcpl_cell_field)
+    type = ON_CELLS;
+  else if (medcpl_field_type == cs_medcpl_vertex_field)
+    type = ON_NODES;
+
+  TypeOfTimeDiscretization td = NO_TIME;
+  if (medcpl_time_discr == cs_medcpl_no_time)
+    td = NO_TIME;
+  else if (medcpl_time_discr == cs_medcpl_one_time)
+    td = ONE_TIME;
+  else if (medcpl_time_discr == cs_medcpl_linear_time)
+    td = LINEAR_TIME;
+
 
   f_id = coupling->n_fields;
 
@@ -956,18 +731,20 @@ cs_paramedmem_field_add(cs_paramedmem_coupling_t  *coupling,
                                                td,
                                                mesh->para_mesh[1],
                                                comp_topo);
+
     f = coupling->fields[f_id]->pf->getField();
     coupling->recv_dec->attachLocalField(coupling->fields[f_id]->pf);
   }
-  else
+  else {
     f = MEDCouplingFieldDouble::New(type, td);
+  }
 
   coupling->fields[f_id]->td = td;
   coupling->fields[f_id]->mesh_id = mesh_id;
 
   /* TODO: setNature should be set by caller to allow for more options */
 
-  f->setNature(ConservativeVolumic);
+  f->setNature(IntensiveConservation);
 
   f->setName(name);
 
@@ -991,7 +768,6 @@ cs_paramedmem_field_add(cs_paramedmem_coupling_t  *coupling,
   coupling->fields[f_id]->dim = dim;
 
   coupling->fields[f_id]->f = f;
-  //coupling->fields[f_id]->pf = NULL;
 
   coupling->n_fields++;
 
@@ -1019,42 +795,14 @@ cs_paramedmem_field_get_id(cs_paramedmem_coupling_t  *coupling,
   /* Loop on fields to know if field has already been created */
 
   for (int f_id = 0; f_id < coupling->n_fields; f_id++) {
-#if (SALOMEMED_VERSION  >= 0x070300)
     if (   coupling->fields[f_id]->mesh_id == mesh_id
         && strcmp(name, coupling->fields[f_id]->f->getName().c_str()) == 0)
-#else
-    if (   coupling->fields[f_id]->mesh_id == mesh_id
-        && strcmp(name, coupling->fields[f_id]->f->getName()) == 0)
-#endif
       return f_id;
   }
 
   return -1;
 }
 
-/*----------------------------------------------------------------------------
- * Return ParaMEDMEM::ParaFIELD object associated with a given field id.
- *
- * parameters:
- *   coupling  <-- pointer to associated coupling
- *   field_id  <-- id of associated field structure
- *
- * returns:
- *   pointer to ParaFIELD to which values were assigned
- *----------------------------------------------------------------------------*/
-
-ParaMEDMEM::ParaFIELD *
-cs_paramedmem_field_get(cs_paramedmem_coupling_t  *coupling,
-                        int                        field_id)
-{
-  ParaFIELD *pf = NULL;
-
-  if (field_id >= 0)
-    pf = coupling->fields[field_id]->pf;
-
-  return pf;
-}
-
 /*----------------------------------------------------------------------------
  * Write field associated with a mesh to MEDCoupling.
  *
@@ -1084,7 +832,6 @@ cs_paramedmem_field_export(cs_paramedmem_coupling_t  *coupling,
   double  *val_ptr = f->getArray()->getPointer();
   const int dim = coupling->fields[field_id]->dim;
 
-
   /* Assign element values */
   /*-----------------------*/
 
@@ -1138,8 +885,11 @@ cs_paramedmem_field_import(cs_paramedmem_coupling_t  *coupling,
   /*-----------------------*/
 
   if (! on_parent) {
-    for (cs_lnum_t i = 0; i < dim*mesh->n_elts; i++)
-      field_values[i] = val_ptr[i];
+    for (cs_lnum_t i = 0; i < mesh->n_elts; i++)
+      for (int j = 0; j < dim; j++) {
+        cs_lnum_t c_id = mesh->new_to_old[i];
+        field_values[dim*c_id+j] = val_ptr[i*dim + j];
+      }
   }
   else {
     for (cs_lnum_t i = 0; i < mesh->n_elts; i++) {
@@ -1149,4 +899,115 @@ cs_paramedmem_field_import(cs_paramedmem_coupling_t  *coupling,
   }
 }
 
+/*----------------------------------------------------------------------------
+ * Synchronize DEC assciated with a given coupling.
+ *
+ * This sync function needs to be called at least once before exchanging data.
+ * dec->synchronize() creates the interpolation matrix between the two codes!
+ *
+ * parameters:
+ *   coupling    <-- coupling structure.
+ *   dec_to_sync <-- 1 for send_dec, != 1 for recv_dec
+ *----------------------------------------------------------------------------*/
+
+void
+cs_paramedmem_sync_dec(cs_paramedmem_coupling_t  *coupling,
+                       int                        dec_to_sync)
+{
+  if (dec_to_sync == 1) {
+    if (coupling->send_synced == 0) {
+      coupling->send_dec->synchronize();
+      coupling->send_synced = 1;
+    }
+  } else if (coupling->recv_synced == 0) {
+    coupling->recv_dec->synchronize();
+    coupling->recv_synced = 1;
+  }
+}
+
+/*----------------------------------------------------------------------------
+ * Send the values related to a coupling
+ *
+ * parameters:
+ *   coupling <-> coupling structure.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_paramedmem_send_data(cs_paramedmem_coupling_t  *coupling)
+{
+  coupling->send_dec->sendData();
+}
+
+/*----------------------------------------------------------------------------
+ * Receive the values related to a coupling
+ *
+ * parameters:
+ *   coupling <-> coupling structure.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_paramedmem_recv_data(cs_paramedmem_coupling_t  *coupling)
+{
+  coupling->recv_dec->recvData();
+}
+
+/*----------------------------------------------------------------------------
+ * Link a given field to the DEC before send/recv
+ *
+ * parameters:
+ *   coupling <-> coupling structure.
+ *   field_id <-> associated field id
+ *----------------------------------------------------------------------------*/
+
+void
+cs_paramedmem_reattach_field(cs_paramedmem_coupling_t  *coupling,
+                             int                        field_id)
+{
+  int mesh_id = coupling->fields[field_id]->mesh_id;
+  _paramedmem_mesh_t *mesh = coupling->meshes[mesh_id];
+
+  if (mesh->direction == 1)
+    coupling->send_dec->attachLocalField(coupling->fields[field_id]->pf);
+  else if (mesh->direction == 2)
+    coupling->recv_dec->attachLocalField(coupling->fields[field_id]->pf);
+}
+
+/*============================================================================
+ * Public C functions
+ *============================================================================*/
+
+BEGIN_C_DECLS
+
+/*----------------------------------------------------------------------------
+ * Map MPI ranks within cs_glob_mpi_comm to their values in MPI_COMM_WORLD.
+ *
+ * The caller is responsible for freeing the returned array
+ *
+ * return:
+ *   list of ranks in MPI_COMM_WORLD
+ *----------------------------------------------------------------------------*/
+
+int *
+cs_paramedmem_get_mpi_comm_world_ranks(void)
+{
+  /* Global rank of current rank */
+  int my_rank;
+  MPI_Comm_rank(MPI_COMM_WORLD, &my_rank);
+
+  /* Size of the local communicator */
+  int mycomm_size;
+  MPI_Comm_size(cs_glob_mpi_comm, &mycomm_size);
+
+  int *world_ranks;
+  BFT_MALLOC(world_ranks, mycomm_size, int);
+
+  MPI_Allgather(&my_rank, 1, MPI_INT, world_ranks, 1, MPI_INT, cs_glob_mpi_comm);
+
+  return world_ranks;
+}
+
 /*----------------------------------------------------------------------------*/
+
+END_C_DECLS
+
+#endif /* HAVE_PARAMEDMEM */
