@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,8 +45,10 @@
 #include "bft_error.h"
 #include "bft_printf.h"
 
+#include "cs_boundary_zone.h"
 #include "cs_gui.h"
 #include "cs_gui_mesh.h"
+#include "cs_internal_coupling.h"
 #include "cs_join.h"
 #include "cs_log.h"
 #include "cs_map.h"
@@ -63,6 +65,9 @@
 #include "cs_post.h"
 #include "cs_prototypes.h"
 #include "cs_preprocessor_data.h"
+#include "cs_stokes_model.h"
+#include "cs_timer_stats.h"
+#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -83,6 +88,8 @@ BEGIN_C_DECLS
         Handle successive preprocessing operations.
 */
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Macro definitions
  *============================================================================*/
@@ -91,8 +98,6 @@ BEGIN_C_DECLS
  * Type definitions
  *============================================================================*/
 
-/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
-
 /*============================================================================
  * Static global variables
  *============================================================================*/
@@ -103,55 +108,50 @@ BEGIN_C_DECLS
 
 /* majgeo */
 
-extern void cs_f_majgeo(const cs_int_t   *ncel,
-                        const cs_int_t   *ncelet,
-                        const cs_int_t   *nfac,
-                        const cs_int_t   *nfabor,
-                        const cs_int_t   *nsom,
-                        const cs_int_t   *lndfac,
-                        const cs_int_t   *lndfbr,
-                        const cs_int_t   *ncelbr,
-                        const int64_t    *ncelgb,
-                        const int64_t    *nfacgb,
-                        const int64_t    *nfbrgb,
-                        const int64_t    *nsomgb,
-                        const cs_int_t   *nfml,
-                        const cs_int_t   *nthrdi,
-                        const cs_int_t   *nthrdb,
-                        const cs_int_t   *ngrpi,
-                        const cs_int_t   *ngrpb,
-                        const cs_int_t   *idxfi,
-                        const cs_int_t   *idxfb,
-                        const cs_int_t    ifacel[],
-                        const cs_int_t    ifabor[],
-                        const cs_int_t    ifmfbr[],
-                        const cs_int_t    ifmcel[],
-                        const cs_int_t    ipnfac[],
-                        const cs_int_t    nodfac[],
-                        const cs_int_t    ipnfbr[],
-                        const cs_int_t    nodfbr[],
-                        const cs_int_t    icelbr[],
-                        const cs_int_t    isympa[],
-                        const cs_real_t  *volmin,
-                        const cs_real_t  *volmax,
-                        const cs_real_t  *voltot,
-                        const cs_real_t   xyzcen[],
-                        const cs_real_t   surfac[],
-                        const cs_real_t   surfbo[],
-                        const cs_real_t   cdgfac[],
-                        const cs_real_t   cdgfbo[],
-                        const cs_real_t   xyznod[],
-                        const cs_real_t   volume[],
-                        const cs_real_t   surfan[],
-                        const cs_real_t   surfbn[],
-                        const cs_real_t   dist[],
-                        const cs_real_t   distb[],
-                        const cs_real_t   pond[],
-                        const cs_real_t   dijpf[],
-                        const cs_real_t   diipb[],
-                        const cs_real_t   dofij[]);
-
-/*! \endcond (end ignore by Doxygen) */
+extern void cs_f_majgeo(const cs_int_t     *ncel,
+                        const cs_int_t     *ncelet,
+                        const cs_int_t     *nfac,
+                        const cs_int_t     *nfabor,
+                        const cs_int_t     *nsom,
+                        const cs_int_t     *lndfac,
+                        const cs_int_t     *lndfbr,
+                        const int64_t      *ncelgb,
+                        const int64_t      *nfacgb,
+                        const int64_t      *nfbrgb,
+                        const int64_t      *nsomgb,
+                        const cs_lnum_2_t   ifacel[],
+                        const cs_int_t      ifabor[],
+                        const cs_int_t      ifmfbr[],
+                        const cs_int_t      ifmcel[],
+                        const cs_int_t      ipnfac[],
+                        const cs_int_t      nodfac[],
+                        const cs_int_t      ipnfbr[],
+                        const cs_int_t      nodfbr[],
+                        const cs_int_t      isympa[],
+                        const cs_int_t      isolid_0[],
+                        const cs_real_t    *volmin,
+                        const cs_real_t    *volmax,
+                        const cs_real_t    *voltot,
+                        const cs_real_t     xyzcen[],
+                        const cs_real_t     surfac[],
+                        const cs_real_t     surfbo[],
+                        const cs_real_t     suffac[],
+                        const cs_real_t     suffbo[],
+                        const cs_real_t     cdgfac[],
+                        const cs_real_t     cdgfbo[],
+                        const cs_real_t     xyznod[],
+                        const cs_real_t     volume[],
+                        const cs_real_t     cell_f_vol[],
+                        const cs_real_t     surfan[],
+                        const cs_real_t     surfbn[],
+                        const cs_real_t     suffan[],
+                        const cs_real_t     suffbn[],
+                        const cs_real_t     dist[],
+                        const cs_real_t     distb[],
+                        const cs_real_t     pond[],
+                        const cs_real_t     dijpf[],
+                        const cs_real_t     diipb[],
+                        const cs_real_t     dofij[]);
 
 /*============================================================================
  * Private function definitions
@@ -161,15 +161,49 @@ extern void cs_f_majgeo(const cs_int_t   *ncel,
  * Fortran wrapper function definitions
  *============================================================================*/
 
-/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
-
-
-/*! \endcond (end ignore by Doxygen) */
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Determine if preprocessing is needed.
+ *
+ * Preprocessing is ignored when a ./restart/mesh_input file is present but
+ * no ./mesh_input file or directory is present. In this case, restart mesh
+ * file is read, and all other preprocessing steps are skipped.
+ *
+ * \return  true if preprocessing is needed, false if only reading is needed.
+ */
+/*----------------------------------------------------------------------------*/
+
+bool
+cs_preprocess_mesh_is_needed(void)
+{
+  bool retval = true;
+  int needed = 1;
+
+  if (cs_glob_rank_id < 1) {
+    if (cs_file_isreg("restart/mesh_input")) {
+      const char path[] = "mesh_input";
+      if (! (cs_file_isreg(path) || cs_file_isdir(path)))
+        needed = 0;
+    }
+  }
+
+#if defined(HAVE_MPI)
+  if (cs_glob_rank_id >= 0)
+    MPI_Bcast(&needed,  1, MPI_INT,  0, cs_glob_mpi_comm);
+#endif
+
+  if (! needed)
+    retval = false;
+
+  return retval;
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Define all mesh preprocessing operations.
@@ -179,6 +213,9 @@ extern void cs_f_majgeo(const cs_int_t   *ncel,
 void
 cs_preprocess_mesh_define(void)
 {
+  if (cs_preprocess_mesh_is_needed() == false)
+    return;
+
   /* Define meshes to read */
 
   cs_user_mesh_input();
@@ -207,6 +244,16 @@ cs_preprocess_mesh(cs_halo_type_t   halo_type)
 {
   double  t1, t2;
 
+  int t_stat_id = cs_timer_stats_id_by_name("mesh_processing");
+
+  int t_top_id = cs_timer_stats_switch(t_stat_id);
+
+  bool allow_modify = cs_preprocess_mesh_is_needed();
+
+  /* Disable all writers until explicitely enabled for this stage */
+
+  cs_post_disable_writer(0);
+
   /* Set partitioning options */
 
   {
@@ -231,13 +278,20 @@ cs_preprocess_mesh(cs_halo_type_t   halo_type)
   cs_preprocessor_data_read_mesh(cs_glob_mesh,
                                  cs_glob_mesh_builder);
 
-  /* Join meshes / build periodicity links if necessary */
+  if (allow_modify) {
 
-  cs_join_all(true);
+    /* Join meshes / build periodicity links if necessary */
 
-  /* Insert thin walls if necessary */
+    cs_join_all(true);
 
-  cs_user_mesh_thinwall(cs_glob_mesh);
+    /* Insert boundaries if necessary */
+
+    cs_gui_mesh_boundary(cs_glob_mesh);
+    cs_user_mesh_boundary(cs_glob_mesh);
+
+    cs_internal_coupling_preprocess(cs_glob_mesh);
+
+  }
 
   /* Initialize extended connectivity, ghost cells and other remaining
      parallelism-related structures */
@@ -245,48 +299,61 @@ cs_preprocess_mesh(cs_halo_type_t   halo_type)
   cs_mesh_init_halo(cs_glob_mesh, cs_glob_mesh_builder, halo_type);
   cs_mesh_update_auxiliary(cs_glob_mesh);
 
-  /* Possible geometry modification */
+  if (allow_modify) {
 
-  cs_user_mesh_modify(cs_glob_mesh);
+    /* Possible geometry modification */
 
-  /* Discard isolated faces if present */
+    cs_gui_mesh_extrude(cs_glob_mesh);
+    cs_user_mesh_modify(cs_glob_mesh);
 
-  cs_post_add_free_faces();
-  cs_mesh_discard_free_faces(cs_glob_mesh);
+    /* Discard isolated faces if present */
 
-  /* Smoothe mesh if required */
+    cs_post_add_free_faces();
+    cs_mesh_discard_free_faces(cs_glob_mesh);
 
-  cs_gui_mesh_smoothe(cs_glob_mesh);
-  cs_user_mesh_smoothe(cs_glob_mesh);
+    /* Smoothe mesh if required */
 
-  /* Triangulate warped faces if necessary */
+    cs_gui_mesh_smoothe(cs_glob_mesh);
+    cs_user_mesh_smoothe(cs_glob_mesh);
 
-  {
-    double  cwf_threshold = -1.0;
-    int  cwf_post = 0;
+    /* Triangulate warped faces if necessary */
 
-    cs_mesh_warping_get_defaults(&cwf_threshold, &cwf_post);
+    {
+      double  cwf_threshold = -1.0;
+      int  cwf_post = 0;
 
-    if (cwf_threshold >= 0.0) {
+      cs_mesh_warping_get_defaults(&cwf_threshold, &cwf_post);
 
-      t1 = cs_timer_wtime();
-      cs_mesh_warping_cut_faces(cs_glob_mesh, cwf_threshold, cwf_post);
-      t2 = cs_timer_wtime();
+      if (cwf_threshold >= 0.0) {
 
-      bft_printf(_("\n Cutting warped faces (%.3g s)\n"), t2-t1);
+        t1 = cs_timer_wtime();
+        cs_mesh_warping_cut_faces(cs_glob_mesh, cwf_threshold, cwf_post);
+        t2 = cs_timer_wtime();
 
+        bft_printf(_("\n Cutting warped faces (%.3g s)\n"), t2-t1);
+
+      }
     }
+
+    /* Now that mesh modification is finished, save mesh if modified */
+
+    cs_gui_mesh_save_if_modified(cs_glob_mesh);
+    cs_user_mesh_save(cs_glob_mesh); /* Disable or force */
+
   }
 
-  /* Now that mesh modification is finished, save mesh if modified */
-
-  cs_user_mesh_save(cs_glob_mesh); /* Disable or force */
-
   bool partition_preprocess = cs_partition_get_preprocess();
+  bool need_save = false;
+  if (   (cs_glob_mesh->modified > 0 && cs_glob_mesh->save_if_modified > 0)
+      || cs_glob_mesh->save_if_modified > 1)
+    need_save = true;
+
   if (cs_glob_mesh->modified > 0 || partition_preprocess) {
     if (partition_preprocess) {
-      if (cs_glob_mesh->modified > 0)
-        cs_mesh_save(cs_glob_mesh, cs_glob_mesh_builder, "mesh_output");
+      if (need_save) {
+        cs_mesh_save(cs_glob_mesh, cs_glob_mesh_builder, NULL, "mesh_output");
+        need_save = false;
+      }
       else
         cs_mesh_to_builder(cs_glob_mesh, cs_glob_mesh_builder, true, NULL);
       cs_partition(cs_glob_mesh, cs_glob_mesh_builder, CS_PARTITION_MAIN);
@@ -294,10 +361,11 @@ cs_preprocess_mesh(cs_halo_type_t   halo_type)
       cs_mesh_init_halo(cs_glob_mesh, cs_glob_mesh_builder, halo_type);
       cs_mesh_update_auxiliary(cs_glob_mesh);
     }
-    else
-      cs_mesh_save(cs_glob_mesh, NULL, "mesh_output");
   }
 
+  if (need_save)
+    cs_mesh_save(cs_glob_mesh, NULL, NULL, "mesh_output");
+
   /* Destroy the temporary structure used to build the main mesh */
 
   cs_mesh_builder_destroy(&cs_glob_mesh_builder);
@@ -306,10 +374,7 @@ cs_preprocess_mesh(cs_halo_type_t   halo_type)
 
   cs_user_numbering();
 
-  bft_printf(_("\n Renumbering mesh:\n"));
-  bft_printf_flush();
-  cs_renumber_mesh(cs_glob_mesh,
-                   cs_glob_mesh_quantities);
+  cs_renumber_mesh(cs_glob_mesh);
 
   /* Initialize group classes */
 
@@ -319,12 +384,26 @@ cs_preprocess_mesh(cs_halo_type_t   halo_type)
 
   cs_mesh_print_info(cs_glob_mesh, _("Mesh"));
 
+  /* Second pass to define internal coupling locators */
+  cs_internal_coupling_map(cs_glob_mesh);
+
   /* Compute geometric quantities related to the mesh */
 
   bft_printf_flush();
 
   t1 = cs_timer_wtime();
+
+  /* If fluid_solid mode is activate: disable solid cells for the dynamics */
+  cs_stokes_model_t *stokes = cs_get_glob_stokes_model();
+  if (stokes->fluid_solid)
+    cs_glob_mesh_quantities->has_disable_flag = 1;
+
   cs_mesh_quantities_compute(cs_glob_mesh, cs_glob_mesh_quantities);
+
+  /* If fluid_solid mode is activate: disable solid cells for the dynamics */
+  if (stokes->fluid_solid)
+    cs_internal_coupling_tag_disable_cells(cs_glob_mesh, cs_glob_mesh_quantities);
+
   cs_mesh_bad_cells_detect(cs_glob_mesh, cs_glob_mesh_quantities);
   cs_user_mesh_bad_cells_tag(cs_glob_mesh, cs_glob_mesh_quantities);
   t2 = cs_timer_wtime();
@@ -335,6 +414,10 @@ cs_preprocess_mesh(cs_halo_type_t   halo_type)
 
   cs_mesh_init_selectors();
   cs_mesh_location_build(cs_glob_mesh, -1);
+  cs_volume_zone_build_all(true);
+  cs_volume_zone_print_info();
+  cs_boundary_zone_build_all(true);
+  cs_boundary_zone_print_info();
 
   /* For debugging purposes */
 
@@ -342,6 +425,12 @@ cs_preprocess_mesh(cs_halo_type_t   halo_type)
   cs_mesh_dump(cs_glob_mesh);
   cs_mesh_quantities_dump(cs_glob_mesh, cs_glob_mesh_quantities);
 #endif
+
+  /* Re-enable writers disabled when entering this stage */
+
+  cs_post_enable_writer(0);
+
+  cs_timer_stats_switch(t_top_id);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -360,26 +449,6 @@ cs_preprocess_mesh_update_fortran(void)
   int64_t n_g_i_faces = m->n_g_i_faces;
   int64_t n_g_b_faces = m->n_g_b_faces;
   int64_t n_g_vertices = m->n_g_vertices;
-  cs_int_t nthrdi = 1;
-  cs_int_t nthrdb = 1;
-  cs_int_t ngrpi = 1;
-  cs_int_t ngrpb = 1;
-  const cs_int_t *idxfi = NULL;
-  const cs_int_t *idxfb = NULL;
-
-  if (m->i_face_numbering != NULL) {
-    const cs_numbering_t *_n = m->i_face_numbering;
-    nthrdi = _n->n_threads;
-    ngrpi = _n->n_groups;
-    idxfi = _n->group_index;
-  }
-
-  if (m->b_face_numbering != NULL) {
-    const cs_numbering_t *_n = m->b_face_numbering;
-    nthrdb = _n->n_threads;
-    ngrpb = _n->n_groups;
-    idxfb = _n->group_index;
-  }
 
   cs_f_majgeo(&(m->n_cells),
               &(m->n_cells_with_ghosts),
@@ -388,19 +457,11 @@ cs_preprocess_mesh_update_fortran(void)
               &(m->n_vertices),
               &(m->i_face_vtx_connect_size),
               &(m->b_face_vtx_connect_size),
-              &(m->n_b_cells),
               &n_g_cells,
               &n_g_i_faces,
               &n_g_b_faces,
               &n_g_vertices,
-              &(m->n_families),
-              &nthrdi,
-              &nthrdb,
-              &ngrpi,
-              &ngrpb,
-              idxfi,
-              idxfb,
-              m->i_face_cells,
+              (const cs_lnum_2_t *)(m->i_face_cells),
               m->b_face_cells,
               m->b_face_family,
               m->cell_family,
@@ -408,20 +469,25 @@ cs_preprocess_mesh_update_fortran(void)
               m->i_face_vtx_lst,
               m->b_face_vtx_idx,
               m->b_face_vtx_lst,
-              m->b_cells,
               mq->b_sym_flag,
+              mq->c_disable_flag,
               &(mq->min_vol),
               &(mq->max_vol),
               &(mq->tot_vol),
               mq->cell_cen,
               mq->i_face_normal,
               mq->b_face_normal,
+              mq->i_f_face_normal,
+              mq->b_f_face_normal,
               mq->i_face_cog,
               mq->b_face_cog,
               m->vtx_coord,
               mq->cell_vol,
+              mq->cell_f_vol,
               mq->i_face_surf,
               mq->b_face_surf,
+              mq->i_f_face_surf,
+              mq->b_f_face_surf,
               mq->i_dist,
               mq->b_dist,
               mq->weight,
