@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -72,6 +72,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Structure Definitions
  *============================================================================*/
@@ -84,21 +86,22 @@ typedef struct {
   char    *app_name;        /* Application name */
   char    *face_cpl_sel_c;  /* Face selection criteria */
   char    *cell_cpl_sel_c;  /* Cell selection criteria */
-  char    *face_sup_sel_c;  /* Face selection criteria */
-  char    *cell_sup_sel_c;  /* Cell selection criteria */
+  char    *face_loc_sel_c;  /* Face selection criteria */
+  char    *cell_loc_sel_c;  /* Cell selection criteria */
   int      verbosity;       /* Verbosity level */
 
 } _cs_sat_coupling_builder_t;
 
-
 struct _cs_sat_coupling_t {
 
-  char            *sat_name;     /* Application name */
+  char                   *sat_name;        /* Application name */
+  cs_sat_coupling_tag_t  *tag_func;        /* Tagging function pointer */
+  void                   *tag_context;     /* Tagging context */
 
   char            *face_cpl_sel; /* Face selection criteria */
   char            *cell_cpl_sel; /* Face selection criteria */
-  char            *face_sup_sel; /* Face selection criteria */
-  char            *cell_sup_sel; /* Face selection criteria */
+  char            *face_loc_sel; /* Face selection criteria */
+  char            *cell_loc_sel; /* Face selection criteria */
 
   ple_locator_t   *localis_cel;  /* Locator associated with cells */
   ple_locator_t   *localis_fbr;  /* Locator associated with boundary faces */
@@ -117,6 +120,7 @@ struct _cs_sat_coupling_t {
   cs_real_t       *distant_pond_fbr; /* Distant weighting coefficient */
   cs_real_t       *local_pond_fbr;   /* Local weighting coefficient */
 
+  cs_real_t        tolerance; /* location tolerance */
   int              verbosity; /* Verbosity level */
 
   /* Communication-related members */
@@ -166,11 +170,11 @@ _remove_matched_builder_entries(void)
 
     _cs_sat_coupling_builder_t *scb = _sat_coupling_builder + i;
 
-    if (scb->match_id > -1) {
+    if (scb->match_id != -1) {
       if (scb->face_cpl_sel_c != NULL) BFT_FREE(scb->face_cpl_sel_c);
       if (scb->cell_cpl_sel_c != NULL) BFT_FREE(scb->cell_cpl_sel_c);
-      if (scb->face_sup_sel_c != NULL) BFT_FREE(scb->face_sup_sel_c);
-      if (scb->cell_sup_sel_c != NULL) BFT_FREE(scb->cell_sup_sel_c);
+      if (scb->face_loc_sel_c != NULL) BFT_FREE(scb->face_loc_sel_c);
+      if (scb->cell_loc_sel_c != NULL) BFT_FREE(scb->cell_loc_sel_c);
       if (scb->app_name != NULL) BFT_FREE(scb->app_name);
     }
   }
@@ -298,8 +302,8 @@ _sat_add_mpi(int builder_id,
 
   cs_sat_coupling_add(scb->face_cpl_sel_c,
                       scb->cell_cpl_sel_c,
-                      scb->face_sup_sel_c,
-                      scb->cell_sup_sel_c,
+                      scb->face_loc_sel_c,
+                      scb->cell_loc_sel_c,
                       scb->app_name,
                       scb->verbosity);
 
@@ -514,6 +518,13 @@ _init_all_mpi_sat(void)
 static cs_sat_coupling_t *
 _sat_coupling_destroy(cs_sat_coupling_t  *couplage)
 {
+  BFT_FREE(couplage->sat_name);
+
+  BFT_FREE(couplage->face_cpl_sel);
+  BFT_FREE(couplage->cell_cpl_sel);
+  BFT_FREE(couplage->face_loc_sel);
+  BFT_FREE(couplage->cell_loc_sel);
+
   ple_locator_destroy(couplage->localis_cel);
   ple_locator_destroy(couplage->localis_fbr);
 
@@ -540,7 +551,7 @@ _sat_coupling_destroy(cs_sat_coupling_t  *couplage)
 }
 
 /*----------------------------------------------------------------------------
- * Computed some quantities needed for a centred-like interpolation
+ * Computed some quantities needed for a centered-like interpolation
  *  - distance JJ' for distant boundary faces
  *  - local weighting coefficients
  *----------------------------------------------------------------------------*/
@@ -576,9 +587,8 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
   const cs_lnum_t   *element       = NULL;
   const cs_coord_t  *distant_coord = NULL;
 
-  cs_mesh_t  *mesh = cs_glob_mesh;
-  cs_mesh_quantities_t  *mesh_quantities = cs_glob_mesh_quantities;
-
+  const cs_mesh_t  *mesh = cs_glob_mesh;
+  const cs_mesh_quantities_t  *mesh_quantities = cs_glob_mesh_quantities;
 
   /* Removing the connectivity and localization informations in case of
      coupling update */
@@ -590,11 +600,10 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
   if (couplage->local_of != NULL)
     BFT_FREE(couplage->local_of);
   if (couplage->distant_pond_fbr != NULL)
-    BFT_FREE(couplage->local_pond_fbr);
+    BFT_FREE(couplage->distant_pond_fbr);
   if (couplage->local_pond_fbr != NULL)
     BFT_FREE(couplage->local_pond_fbr);
 
-
   /* Interpolation structure */
 
   n_fbr_loc  = ple_locator_get_n_interior(couplage->localis_fbr);
@@ -606,7 +615,7 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
 
 
   /* Calculation of the distance DJJPB defining the distance from */
-  /* the local cell centre to the distant boundary face norm      */
+  /* the local cell center to the distant boundary face norm      */
   /*--------------------------------------------------------------*/
 
   BFT_MALLOC(couplage->distant_dist_fbr, 3*n_fbr_dist, cs_real_t);
@@ -684,7 +693,7 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
   BFT_MALLOC(couplage->distant_pond_fbr, n_fbr_dist, cs_real_t);
   BFT_MALLOC(couplage->local_pond_fbr, n_fbr_loc, cs_real_t);
 
-  /* Get the cell centres coordinates (reverse = 0) */
+  /* Get the cell center coordinates (reverse = 0) */
 
   reverse = 0;
 
@@ -705,7 +714,7 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
   for (ind = 0 ; ind < n_fbr_loc ; ind++) {
 
     ifac = lstfbr[ind] - 1;
-    iel  = mesh->b_face_cells[ifac] - 1;
+    iel  = mesh->b_face_cells[ifac];
 
     surface = 0.;
 
@@ -737,7 +746,6 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
 
   }
 
-
   /* Get the distant weighting coefficients (reverse = 1) */
 
   reverse = 1;
@@ -750,8 +758,6 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
                                  1,
                                  reverse);
 
-
-
   /* Calculation of the OF distance */
   /*--------------------------------*/
 
@@ -761,7 +767,7 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
   for (ind = 0 ; ind < n_fbr_loc ; ind++) {
 
     ifac = lstfbr[ind] - 1;
-    iel  = mesh->b_face_cells[ifac] - 1;
+    iel  = mesh->b_face_cells[ifac];
 
     surface = 0.;
 
@@ -786,14 +792,18 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
     distance_fbr_cel /= surface;
     distance_cel_cel /= surface;
 
-    for (icoo = 0 ; icoo < 3 ; icoo++)
-
-      couplage->local_of[ind*3 + icoo] =
-        mesh_quantities->b_face_cog[ifac*3 + icoo]
-        -  (mesh_quantities->b_face_cog[ifac*3 + icoo] /*  O'  */
-            + mesh_quantities->b_face_normal[ifac*3 + icoo] *distance_fbr_cel/surface   /*J'=F+n*FJ'*/
-            - 0.5*mesh_quantities->b_face_normal[ifac*3 + icoo] *distance_cel_cel/surface );  /*-n*I'J'/2*/
+    const cs_real_3_t *restrict b_face_cog
+      = (const cs_real_3_t *restrict)mesh_quantities->b_face_cog;
+    const cs_real_3_t *restrict b_face_normal
+      = (const cs_real_3_t *restrict)mesh_quantities->b_face_normal;
 
+    for (icoo = 0 ; icoo < 3 ; icoo++) {
+      couplage->local_of[ind*3 + icoo]
+        =     b_face_cog[ifac][icoo]
+          -  (      b_face_cog[ifac][icoo] /*  O'  */
+              +     b_face_normal[ifac][icoo] *distance_fbr_cel/surface    /*J'=F+n*FJ'*/
+              - 0.5*b_face_normal[ifac][icoo] *distance_cel_cel/surface);  /*-n*I'J'/2*/
+    }
   }
 
   reverse = 1;
@@ -807,33 +817,16 @@ _sat_coupling_interpolate(cs_sat_coupling_t  *couplage)
                                  reverse);
 
   BFT_FREE(local_xyzcen);
-
-
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions for Fortran API
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- * User function wrapper for definition of Code_Saturne couplings
- *
- * Fortran Interface:
- *
- * SUBROUTINE USSATC
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (ussatc, USSATC)
-(
- void
-)
-{
-  cs_user_saturne_coupling();
-}
-
-/*----------------------------------------------------------------------------
- * Get number of code coupling
+ * Get number of code couplings
  *
  * Fortran interface:
  *
@@ -849,10 +842,9 @@ void CS_PROCF (nbccpl, NBCCPL)
 )
 {
   if (_cs_glob_n_sat_cp < 0) {
+    _cs_glob_n_sat_cp = cs_sat_coupling_n_couplings();
     if (_sat_coupling_builder_size > 0)
-      _cs_glob_n_sat_cp = _sat_coupling_builder_size;
-    else
-      _cs_glob_n_sat_cp = cs_sat_coupling_n_couplings();
+      _cs_glob_n_sat_cp += _sat_coupling_builder_size;
   }
 
   *n_couplings = _cs_glob_n_sat_cp;
@@ -901,7 +893,10 @@ void CS_PROCF (defloc, DEFLOC)
   fvm_nodal_t  *support_fbr = NULL;
   cs_mesh_quantities_t  *mesh_quantities = cs_glob_mesh_quantities;
 
-  const double tolerance = 0.1;
+  int locator_options[PLE_LOCATOR_N_OPTIONS];
+  locator_options[PLE_LOCATOR_NUMBERING] = 1;
+
+  int *point_tag = NULL;
 
   /* Initializations and verifications */
 
@@ -920,23 +915,23 @@ void CS_PROCF (defloc, DEFLOC)
 
   /* Create the local lists */
 
-  if (coupl->cell_sup_sel != NULL) {
+  if (coupl->cell_loc_sel != NULL) {
 
     BFT_MALLOC(c_elt_list, cs_glob_mesh->n_cells, cs_lnum_t);
 
-    cs_selector_get_cell_list(coupl->cell_sup_sel,
-                              &(coupl->nbr_cel_sup),
-                              c_elt_list);
+    cs_selector_get_cell_num_list(coupl->cell_loc_sel,
+                                  &(coupl->nbr_cel_sup),
+                                  c_elt_list);
 
   }
 
-  if (coupl->face_sup_sel != NULL) {
+  if (coupl->face_loc_sel != NULL) {
 
     BFT_MALLOC(f_elt_list, cs_glob_mesh->n_b_faces, cs_lnum_t);
 
-    cs_selector_get_b_face_list(coupl->face_sup_sel,
-                                &(coupl->nbr_fbr_sup),
-                                f_elt_list);
+    cs_selector_get_b_face_num_list(coupl->face_loc_sel,
+                                    &(coupl->nbr_fbr_sup),
+                                    f_elt_list);
 
   }
 
@@ -978,27 +973,30 @@ void CS_PROCF (defloc, DEFLOC)
 
   }
 
-  if (coupl->cell_sup_sel != NULL) BFT_FREE(c_elt_list);
-  if (coupl->face_sup_sel != NULL) BFT_FREE(f_elt_list);
+  if (coupl->cell_loc_sel != NULL) BFT_FREE(c_elt_list);
+  if (coupl->face_loc_sel != NULL) BFT_FREE(f_elt_list);
 
   /* Build and initialize associated locator */
 
 #if defined(PLE_HAVE_MPI)
 
-  coupl->localis_cel = ple_locator_create(tolerance,
-                                          coupl->comm,
-                                          coupl->n_sat_ranks,
-                                          coupl->sat_root_rank);
+  if (coupl->localis_cel == NULL)
+    coupl->localis_cel = ple_locator_create(coupl->comm,
+                                            coupl->n_sat_ranks,
+                                            coupl->sat_root_rank);
 
-  coupl->localis_fbr = ple_locator_create(tolerance,
-                                          coupl->comm,
-                                          coupl->n_sat_ranks,
-                                          coupl->sat_root_rank);
+  if (coupl->localis_fbr == NULL)
+    coupl->localis_fbr = ple_locator_create(coupl->comm,
+                                            coupl->n_sat_ranks,
+                                            coupl->sat_root_rank);
 
 #else
 
-  coupl->localis_cel = ple_locator_create(tolerance);
-  coupl->localis_fbr = ple_locator_create(tolerance);
+  if (coupl->localis_cel == NULL)
+    coupl->localis_cel = ple_locator_create();
+
+  if (coupl->localis_fbr == NULL)
+    coupl->localis_fbr = ple_locator_create();
 
 #endif
 
@@ -1008,33 +1006,47 @@ void CS_PROCF (defloc, DEFLOC)
 
     BFT_MALLOC(c_elt_list, cs_glob_mesh->n_cells, cs_lnum_t);
 
-    cs_selector_get_cell_list(coupl->cell_cpl_sel,
-                              &nbr_cel_cpl,
-                              c_elt_list);
+    cs_selector_get_cell_num_list(coupl->cell_cpl_sel,
+                                  &nbr_cel_cpl,
+                                  c_elt_list);
 
   }
 
+  if (coupl->tag_func != NULL) {
+    BFT_MALLOC(point_tag, nbr_cel_cpl, int);
+    coupl->tag_func(coupl->tag_context,
+                    coupl->cells_sup,
+                    nbr_cel_cpl,
+                    1,
+                    c_elt_list,
+                    point_tag);
+  }
+
   ple_locator_set_mesh(coupl->localis_cel,
                        coupl->cells_sup,
+                       locator_options,
+                       0.,
+                       coupl->tolerance,
                        3,
                        nbr_cel_cpl,
                        c_elt_list,
+                       point_tag,
                        mesh_quantities->cell_cen,
                        NULL,
                        cs_coupling_mesh_extents,
-                       cs_coupling_point_in_mesh_p,
-                       NULL);
+                       cs_coupling_point_in_mesh_p);
+
+  BFT_FREE(point_tag);
 
   if (coupl->cell_cpl_sel != NULL) BFT_FREE(c_elt_list);
 
-
   if (coupl->face_cpl_sel != NULL) {
 
     BFT_MALLOC(f_elt_list, cs_glob_mesh->n_b_faces, cs_lnum_t);
 
-    cs_selector_get_b_face_list(coupl->face_cpl_sel,
-                                &nbr_fbr_cpl,
-                                f_elt_list);
+    cs_selector_get_b_face_num_list(coupl->face_cpl_sel,
+                                    &nbr_fbr_cpl,
+                                    f_elt_list);
 
   }
 
@@ -1043,26 +1055,39 @@ void CS_PROCF (defloc, DEFLOC)
   else
     support_fbr = coupl->cells_sup;
 
+  if (coupl->tag_func != NULL) {
+    BFT_MALLOC(point_tag, nbr_fbr_cpl, int);
+    coupl->tag_func(coupl->tag_context,
+                    support_fbr,
+                    nbr_fbr_cpl,
+                    1,
+                    f_elt_list,
+                    point_tag);
+  }
+
   ple_locator_set_mesh(coupl->localis_fbr,
                        support_fbr,
+                       locator_options,
+                       0.,
+                       coupl->tolerance,
                        3,
                        nbr_fbr_cpl,
                        f_elt_list,
+                       point_tag,
                        mesh_quantities->b_face_cog,
                        NULL,
                        cs_coupling_mesh_extents,
-                       cs_coupling_point_in_mesh_p,
-                       NULL);
+                       cs_coupling_point_in_mesh_p);
+
+  BFT_FREE(point_tag);
 
   if (coupl->face_cpl_sel != NULL) BFT_FREE(f_elt_list);
 
-
-  /* Computed some quantities needed for a centred-like interpolation */
+  /* Computed some quantities needed for a centered-like interpolation */
 
   if (coupl->localis_fbr != NULL)
     _sat_coupling_interpolate(coupl);
 
-
 #if 0
   /* TODO: associate the FVM meshes to the post-processing,
      with a fonction giving a pointer to the associated FVM structures,
@@ -1467,7 +1492,7 @@ void CS_PROCF (coocpl, COOCPL)
 }
 
 /*----------------------------------------------------------------------------
- * Get the weighting coefficient needed for a centred-like interpolation
+ * Get the weighting coefficient needed for a centered-like interpolation
  * in the case of a coupling on boundary faces.
  *
  * Fortran interface:
@@ -1508,7 +1533,7 @@ void CS_PROCF (pondcp, PONDCP)
 
   if (*ityloc == 1)
     bft_error(__FILE__, __LINE__, 0,
-              _("The centred interpolation scheme is not available\n"
+              _("The centered interpolation scheme is not available\n"
                 "when coupling cells"));
   else if (*ityloc == 2)
     localis = coupl->localis_fbr;
@@ -1628,7 +1653,6 @@ void CS_PROCF (varcpl, VARCPL)
                                    sizeof(cs_real_t),
                                    *stride,
                                    0);
-
   }
 
 }
@@ -1844,24 +1868,35 @@ void CS_PROCF (mxicpl, MXICPL)
  * Public function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Define new Code_Saturne coupling.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define new Code_Saturne coupling.
  *
- * arguments:
- *   saturne_name      <-- name of Code_Saturne instance
- *   boundary_criteria <-- boundary face selection criteria, or NULL
- *   volume_criteria   <-- volume cell selection criteria, or NULL
- *   projection_axis   <-- 'x', 'y', or 'y' for 2D projection axis (case
- *                         independent), or ' ' for standard 3D coupling
- *   verbosity         <-- verbosity level
- *----------------------------------------------------------------------------*/
+ * The arguments to \ref cs_sat_coupling_define are:
+ * \param[in] saturne_name          matching Code_Saturne application name
+ * \param[in] boundary_cpl_criteria boundary face selection criteria for coupled
+ *                                  faces, or NULL
+ * \param[in] volume_cpl_criteria   cell selection criteria for coupled cells, or
+                                    NULL
+ * \param[in] boundary_loc_criteria boundary face selection criteria for location
+ *                                  (not functional)
+ * \param[in] volume_loc_criteria   cell selection criteria for location
+ * \param[in] verbosity             verbosity level
+ *
+ * In the case of only 2 Code_Saturne instances, the 'saturne_name' argument
+ * is ignored, as there is only one matching possibility.
+ *
+ * In case of multiple couplings, a coupling will be matched with available
+ * Code_Saturne instances based on the 'saturne_name' argument.
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_sat_coupling_define(const char  *saturne_name,
                        const char  *boundary_cpl_criteria,
                        const char  *volume_cpl_criteria,
-                       const char  *boundary_sup_criteria,
-                       const char  *volume_sup_criteria,
+                       const char  *boundary_loc_criteria,
+                       const char  *volume_loc_criteria,
                        int          verbosity)
 {
   _cs_sat_coupling_builder_t *scb = NULL;
@@ -1894,16 +1929,16 @@ cs_sat_coupling_define(const char  *saturne_name,
     strcpy(scb->cell_cpl_sel_c, volume_cpl_criteria);
   }
 
-  scb->face_sup_sel_c = NULL;
-  if (boundary_sup_criteria != NULL) {
-    BFT_MALLOC(scb->face_sup_sel_c, strlen(boundary_sup_criteria) + 1, char);
-    strcpy(scb->face_sup_sel_c, boundary_sup_criteria);
+  scb->face_loc_sel_c = NULL;
+  if (boundary_loc_criteria != NULL) {
+    BFT_MALLOC(scb->face_loc_sel_c, strlen(boundary_loc_criteria) + 1, char);
+    strcpy(scb->face_loc_sel_c, boundary_loc_criteria);
   }
 
-  scb->cell_sup_sel_c = NULL;
-  if (volume_sup_criteria != NULL) {
-    BFT_MALLOC(scb->cell_sup_sel_c, strlen(volume_sup_criteria) + 1, char);
-    strcpy(scb->cell_sup_sel_c, volume_sup_criteria);
+  scb->cell_loc_sel_c = NULL;
+  if (volume_loc_criteria != NULL) {
+    BFT_MALLOC(scb->cell_loc_sel_c, strlen(volume_loc_criteria) + 1, char);
+    strcpy(scb->cell_loc_sel_c, volume_loc_criteria);
   }
 
   scb->verbosity = verbosity;
@@ -1956,7 +1991,7 @@ cs_sat_coupling_by_id(int coupling_id)
 void
 cs_sat_coupling_all_init(void)
 {
-  /* First try using MPI */
+  /* First, try using other MPI communicators */
 
 #if defined(HAVE_MPI)
 
@@ -1994,8 +2029,8 @@ cs_sat_coupling_all_init(void)
 void
 cs_sat_coupling_add(const char  *face_cpl_sel_c,
                     const char  *cell_cpl_sel_c,
-                    const char  *face_sup_sel_c,
-                    const char  *cell_sup_sel_c,
+                    const char  *face_loc_sel_c,
+                    const char  *cell_loc_sel_c,
                     const char  *sat_name,
                     int          verbosity)
 {
@@ -2008,6 +2043,8 @@ cs_sat_coupling_add(const char  *face_cpl_sel_c,
   BFT_MALLOC(sat_coupling, 1, cs_sat_coupling_t);
 
   sat_coupling->sat_name = NULL;
+  sat_coupling->tag_func = NULL;
+  sat_coupling->tag_context = NULL;
 
   if (sat_name != NULL) {
     BFT_MALLOC(sat_coupling->sat_name, strlen(sat_name) + 1, char);
@@ -2018,8 +2055,8 @@ cs_sat_coupling_add(const char  *face_cpl_sel_c,
 
   sat_coupling->face_cpl_sel = NULL;
   sat_coupling->cell_cpl_sel = NULL;
-  sat_coupling->face_sup_sel = NULL;
-  sat_coupling->cell_sup_sel = NULL;
+  sat_coupling->face_loc_sel = NULL;
+  sat_coupling->cell_loc_sel = NULL;
 
   if (face_cpl_sel_c != NULL) {
     BFT_MALLOC(sat_coupling->face_cpl_sel, strlen(face_cpl_sel_c) + 1, char);
@@ -2030,13 +2067,13 @@ cs_sat_coupling_add(const char  *face_cpl_sel_c,
     strcpy(sat_coupling->cell_cpl_sel, cell_cpl_sel_c);
   }
 
-  if (face_sup_sel_c != NULL) {
-    BFT_MALLOC(sat_coupling->face_sup_sel, strlen(face_sup_sel_c) + 1, char);
-    strcpy(sat_coupling->face_sup_sel, face_sup_sel_c);
+  if (face_loc_sel_c != NULL) {
+    BFT_MALLOC(sat_coupling->face_loc_sel, strlen(face_loc_sel_c) + 1, char);
+    strcpy(sat_coupling->face_loc_sel, face_loc_sel_c);
   }
-  if (cell_sup_sel_c != NULL) {
-    BFT_MALLOC(sat_coupling->cell_sup_sel, strlen(cell_sup_sel_c) + 1, char);
-    strcpy(sat_coupling->cell_sup_sel, cell_sup_sel_c);
+  if (cell_loc_sel_c != NULL) {
+    BFT_MALLOC(sat_coupling->cell_loc_sel, strlen(cell_loc_sel_c) + 1, char);
+    strcpy(sat_coupling->cell_loc_sel, cell_loc_sel_c);
   }
 
   sat_coupling->faces_sup = NULL;
@@ -2048,6 +2085,7 @@ cs_sat_coupling_add(const char  *face_cpl_sel_c,
   sat_coupling->nbr_fbr_sup = 0;
   sat_coupling->nbr_cel_sup = 0;
 
+  sat_coupling->tolerance = 0.1;
   sat_coupling->verbosity = verbosity;
 
   /* Geometric quantities arrays for interpolation */
@@ -2074,6 +2112,52 @@ cs_sat_coupling_add(const char  *face_cpl_sel_c,
   cs_glob_sat_n_couplings++;
 }
 
+/*----------------------------------------------------------------------------
+ * Create a new internal Code_Saturne coupling.
+ *
+ * arguments:
+ *   tag_func          <-- pointer to tagging function
+ *   tag_context       <-- pointer to tagging function context
+ *   boundary_criteria <-- boundary face selection criteria, or NULL
+ *   volume_criteria   <-- volume cell selection criteria, or NULL
+ *   loc_tolerance     <-- location tolerance factor (0.1 recommended)
+ *   verbosity         <-- verbosity level
+ *----------------------------------------------------------------------------*/
+
+void
+cs_sat_coupling_add_internal(cs_sat_coupling_tag_t  *tag_func,
+                             void                   *tag_context,
+                             const char             *boundary_cpl_criteria,
+                             const char             *volume_cpl_criteria,
+                             const char             *boundary_loc_criteria,
+                             const char             *volume_loc_criteria,
+                             float                   loc_tolerance,
+                             int                     verbosity)
+{
+  CS_UNUSED(loc_tolerance);
+
+  cs_sat_coupling_add(boundary_cpl_criteria,
+                      volume_cpl_criteria,
+                      boundary_loc_criteria,
+                      volume_loc_criteria,
+                      NULL,
+                      verbosity);
+
+  cs_sat_coupling_t *sat_coupling
+    = cs_sat_coupling_by_id(cs_sat_coupling_n_couplings() - 1);
+
+  sat_coupling->tag_func = tag_func;
+  sat_coupling->tag_context = tag_context;
+
+#if defined(HAVE_MPI)
+
+  sat_coupling->comm = cs_glob_mpi_comm;
+  sat_coupling->sat_root_rank = 0;
+  sat_coupling->n_sat_ranks = cs_glob_n_ranks;
+
+#endif
+}
+
 /*----------------------------------------------------------------------------
  * Destroy all couplings
  *----------------------------------------------------------------------------*/
