@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,6 +36,8 @@
  *----------------------------------------------------------------------------*/
 
 #include "fvm_defs.h"
+#include "fvm_nodal.h"
+
 #include "cs_base.h"
 
 /*----------------------------------------------------------------------------*/
@@ -43,29 +45,42 @@
 BEGIN_C_DECLS
 
 /*=============================================================================
- * Structure Definitions
+ * Type Definitions
  *============================================================================*/
 
 typedef struct _cs_sat_coupling_t cs_sat_coupling_t;
 
+/*----------------------------------------------------------------------------
+ * Function pointer to mesh tagging function.
+ *
+ * Each function of this sort may be used to tag a mesh and associated
+ * points for mocatin exclusion.
+ *
+ * Note: if the context pointer is non-NULL, it must point to valid data
+ * when the selection function is called, so that value or structure
+ * should not be temporary (i.e. local);
+ *
+ * parameters:
+ *   context         <-> pointer to optional (untyped) value or structure.
+ *   mesh            <-> nodal mesh which should be tagged
+ *   n_points        <-- number of points to tag
+ *   point_list_base <-- base numbering for point_list
+ *   point_list      <-- optional indirection for points
+ *   point_tag       --> point tag values (size: n_tags)
+ *----------------------------------------------------------------------------*/
+
+typedef void
+(cs_sat_coupling_tag_t) (void            *context,
+                         fvm_nodal_t     *mesh,
+                         cs_lnum_t        n_points,
+                         cs_lnum_t        point_list_base,
+                         const cs_lnum_t  point_list[],
+                         int             *point_tag);
+
 /*============================================================================
  *  Public function prototypes for Fortran API
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * User function wrapper for definition of Code_Saturne couplings
- *
- * Fortran Interface:
- *
- * SUBROUTINE USSATC
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (ussatc, USSATC)
-(
- void
-);
-
 /*----------------------------------------------------------------------------
  * Get number of code coupling
  *
@@ -261,7 +276,7 @@ void CS_PROCF (coocpl, COOCPL)
 );
 
 /*----------------------------------------------------------------------------
- * Get the weighting coefficient needed for a centred-like interpolation
+ * Get the weighting coefficient needed for a centered-like interpolation
  * in the case of a coupling on boundary faces.
  *
  * Fortran interface:
@@ -396,22 +411,35 @@ void CS_PROCF (mxicpl, MXICPL)
  * Public function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Define new Code_Saturne coupling.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define new Code_Saturne coupling.
  *
- * arguments:
- *   saturne_name      <-- name of Code_Saturne instance, or NULL
- *   boundary_criteria <-- boundary face selection criteria, or NULL
- *   volume_criteria   <-- volume cell selection criteria, or NULL
- *   verbosity         <-- verbosity level
- *----------------------------------------------------------------------------*/
+ * The arguments to \ref cs_sat_coupling_define are:
+ * \param[in] saturne_name          matching Code_Saturne application name
+ * \param[in] boundary_cpl_criteria boundary face selection criteria for coupled
+ *                                  faces, or NULL
+ * \param[in] volume_cpl_criteria   cell selection criteria for coupled cells, or
+                                    NULL
+ * \param[in] boundary_loc_criteria boundary face selection criteria for location
+ *                                  (not functional)
+ * \param[in] volume_loc_criteria   cell selection criteria for location
+ * \param[in] verbosity             verbosity level
+ *
+ * In the case of only 2 Code_Saturne instances, the 'saturne_name' argument
+ * is ignored, as there is only one matching possibility.
+ *
+ * In case of multiple couplings, a coupling will be matched with available
+ * Code_Saturne instances based on the 'saturne_name' argument.
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_sat_coupling_define(const char  *saturne_name,
                        const char  *boundary_cpl_criteria,
                        const char  *volume_cpl_criteria,
-                       const char  *boundary_sup_criteria,
-                       const char  *volume_sup_criteria,
+                       const char  *boundary_loc_criteria,
+                       const char  *volume_loc_criteria,
                        int          verbosity);
 
 /*----------------------------------------------------------------------------
@@ -451,11 +479,33 @@ cs_sat_coupling_by_id(int coupling_id);
 void
 cs_sat_coupling_add(const char  *face_cpl_sel_c,
                     const char  *cell_cpl_sel_c,
-                    const char  *face_sup_sel_c,
-                    const char  *cell_sup_sel_c,
+                    const char  *face_loc_sel_c,
+                    const char  *cell_loc_sel_c,
                     const char  *sat_name,
                     int          verbosity);
 
+/*----------------------------------------------------------------------------
+ * Create a new internal Code_Saturne coupling.
+ *
+ * arguments:
+ *   tag_func          <-- pointer to tagging function
+ *   tag_context       <-- pointer to tagging function context
+ *   boundary_criteria <-- boundary face selection criteria, or NULL
+ *   volume_criteria   <-- volume cell selection criteria, or NULL
+ *   loc_tolerance     <-- location tolerance factor (0.1 recommended)
+ *   verbosity         <-- verbosity level
+ *----------------------------------------------------------------------------*/
+
+void
+cs_sat_coupling_add_internal(cs_sat_coupling_tag_t  *tag_func,
+                             void                   *tag_context,
+                             const char             *boundary_cpl_criteria,
+                             const char             *volume_cpl_criteria,
+                             const char             *boundary_loc_criteria,
+                             const char             *volume_loc_criteria,
+                             float                   loc_tolerance,
+                             int                     verbosity);
+
 /*----------------------------------------------------------------------------
  * Initialize Code_Saturne couplings.
  *
