@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,6 +45,7 @@
  * PLE library headers
  *----------------------------------------------------------------------------*/
 
+#include <ple_defs.h>
 #include <ple_coupling.h>
 #include <ple_locator.h>
 
@@ -70,6 +71,7 @@
 #include "cs_post.h"
 #include "cs_selector.h"
 #include "cs_time_step.h"
+#include "cs_timer_stats.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
@@ -81,6 +83,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -343,6 +347,7 @@ _cs_syr4_coupling_post_function(void                  *coupling,
           cell_temp = coupling_ent->solid_temp;
 
         cs_post_write_var(coupling_ent->post_mesh_id,
+                          CS_POST_WRITER_ALL_ASSOCIATED,
                           _("Solid T"),
                           1,
                           false,
@@ -355,6 +360,7 @@ _cs_syr4_coupling_post_function(void                  *coupling,
 
         if (type_id == 1)
           cs_post_write_var(coupling_ent->post_mesh_id,
+                            CS_POST_WRITER_ALL_ASSOCIATED,
                             _("Solid heat flux"),
                             1,
                             false,
@@ -400,6 +406,9 @@ _post_init(cs_syr4_coupling_t      *syr_coupling,
   if (cs_post_writer_exists(writer_id) != true)
     return;
 
+  int t_top_id
+    = cs_timer_stats_switch(cs_timer_stats_id_by_name("postprocessing_stage"));
+
   /* Initialize post processing flag */
 
   coupling_ent->post_mesh_id = cs_post_get_free_mesh_id();
@@ -441,6 +450,57 @@ _post_init(cs_syr4_coupling_t      *syr_coupling,
     cs_glob_syr4_post_mesh_ext[0] = coupling_ent->post_mesh_id;
 
   cs_glob_syr4_post_mesh_ext[1] = coupling_ent->post_mesh_id;
+
+  cs_timer_stats_switch(t_top_id);
+}
+
+/*----------------------------------------------------------------------------
+ * Check if coupling location is complete
+ *
+ * parameters:
+ *   syr_coupling <-- partially initialized SYRTHES coupling structure
+ *   coupling_ent <-- coupling entity
+ *   n_ext        --> number of unlocated points
+ *   ext_syr      --> 1 if SYRTHES has some unlocted elements, 0 otherwise
+ *
+ * returns:
+ *   true if location is complete, false otherwise
+ *----------------------------------------------------------------------------*/
+
+static bool
+_is_location_complete(cs_syr4_coupling_t      *syr_coupling,
+                      cs_syr4_coupling_ent_t  *coupling_ent,
+                      cs_gnum_t               *n_ext,
+                      bool                    *ext_syr)
+{
+  bool location_complete = true;
+
+  /* Check that all points are effectively located */
+
+  ple_lnum_t n_exterior = ple_locator_get_n_exterior(coupling_ent->locator);
+  *n_ext = n_exterior;
+
+  char  op_name_send[32 + 1];
+  char  op_name_recv[32 + 1];
+
+  cs_parall_counter(n_ext, 1);
+
+  if (*n_ext > 0) {
+    strcpy(op_name_send, "coupling:location:incomplete");
+    location_complete = false;
+  }
+  else
+    strcpy(op_name_send, "coupling:location:ok");
+
+  _exchange_sync(syr_coupling, op_name_send, op_name_recv);
+  if (!strcmp(op_name_recv, "coupling:location:incomplete")) {
+    location_complete = false;
+    *ext_syr = true;
+  }
+  else
+    *ext_syr = false;
+
+  return location_complete;
 }
 
 /*----------------------------------------------------------------------------
@@ -461,17 +521,21 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                     int                  elt_dim)
 {
   char *coupled_mesh_name = NULL;
-  cs_gnum_t n_exterior = 0, n_dist_elts = 0;
+  bool      ext_syr = false;
+  cs_lnum_t n_exterior = 0;
   cs_gnum_t n_ext = 0;
   cs_lnum_t *elt_list = NULL;
   cs_coord_t *elt_centers = NULL;
   fvm_nodal_t *location_elts = NULL;
-  ple_mesh_elements_closest_t *locate_on_closest = NULL;
   float *cs_to_syr_dist = NULL;
   float *syr_to_cs_dist = NULL;
 
+  bool location_complete = false;
   cs_syr4_coupling_ent_t *coupling_ent = NULL;
 
+  int locator_options[PLE_LOCATOR_N_OPTIONS];
+  locator_options[PLE_LOCATOR_NUMBERING] = 1;
+
   assert(syr_coupling != NULL);
 
   /* Initialization */
@@ -507,9 +571,9 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
 
     BFT_MALLOC(elt_list, cs_glob_mesh->n_cells, cs_lnum_t);
 
-    cs_selector_get_cell_list(select_criteria,
-                              &(coupling_ent->n_elts),
-                              elt_list);
+    cs_selector_get_cell_num_list(select_criteria,
+                                  &(coupling_ent->n_elts),
+                                  elt_list);
 
     coupling_ent->elts
       = cs_mesh_connect_cells_to_nodal(cs_glob_mesh,
@@ -531,9 +595,6 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
 
   else if (elt_dim == syr_coupling->dim - 1) {
 
-    if (syr_coupling->allow_nearest)
-      locate_on_closest = cs_coupling_point_closest_mesh;
-
     BFT_MALLOC(coupled_mesh_name,
                strlen("SYRTHES  faces") + strlen(syr_coupling->syr_name) + 1,
                char);
@@ -541,9 +602,9 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
 
     BFT_MALLOC(elt_list, cs_glob_mesh->n_b_faces, cs_lnum_t);
 
-    cs_selector_get_b_face_list(select_criteria,
-                                &(coupling_ent->n_elts),
-                                elt_list);
+    cs_selector_get_b_face_num_list(select_criteria,
+                                    &(coupling_ent->n_elts),
+                                    elt_list);
 
     coupling_ent->elts
       = cs_mesh_connect_faces_to_nodal(cs_glob_mesh,
@@ -624,6 +685,12 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
 
   /* Initialize post-processing */
 
+  /* Precaution: deactivate visualization for time-dependent meshes,
+     as this would require regenerating visualization at each time step */
+
+  if (cs_post_get_writer_time_dep(-1) != FVM_WRITER_FIXED_MESH)
+    syr_coupling->visualization = 0;
+
   if (syr_coupling->visualization != 0)
     _post_init(syr_coupling, coupling_ent);
 
@@ -634,15 +701,6 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
     bft_printf_flush();
   }
 
-#if defined(PLE_HAVE_MPI)
-  coupling_ent->locator = ple_locator_create(syr_coupling->tolerance,
-                                             syr_coupling->comm,
-                                             syr_coupling->n_syr_ranks,
-                                             syr_coupling->syr_root_rank);
-#else
-  coupling_ent->locator = ple_locator_create(syr_coupling->tolerance);
-#endif
-
   /* Retrieve coordinates using FVM functions rather than previous list and
      coordinates, in case the extracted nodal mesh contains elements in a
      different order (multiple element types) or coordinates are projected
@@ -650,7 +708,7 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
 
   if (coupling_ent->n_elts > 0) {
 
-    if (syr_coupling->visualization >= 0)
+    if (syr_coupling->visualization != 0)
       BFT_MALLOC(cs_to_syr_dist, coupling_ent->n_elts, float);
 
     BFT_MALLOC(elt_centers,
@@ -664,16 +722,76 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
 
   /* Locate entities */
 
+#if defined(PLE_HAVE_MPI)
+  coupling_ent->locator = ple_locator_create(syr_coupling->comm,
+                                             syr_coupling->n_syr_ranks,
+                                             syr_coupling->syr_root_rank);
+#else
+  coupling_ent->locator = ple_locator_create();
+#endif
+
   ple_locator_set_mesh(coupling_ent->locator,
                        location_elts,
+                       locator_options,
+                       0.,
+                       syr_coupling->tolerance,
                        syr_coupling->dim,
                        coupling_ent->n_elts,
                        NULL,
+                       NULL,
                        elt_centers,
                        cs_to_syr_dist,
                        cs_coupling_mesh_extents,
-                       cs_coupling_point_in_mesh,
-                       locate_on_closest);
+                       cs_coupling_point_in_mesh);
+
+  /* Check that all points are effectively located */
+
+  location_complete = _is_location_complete(syr_coupling,
+                                            coupling_ent,
+                                            &n_ext,
+                                            &ext_syr);
+
+  if (syr_coupling->allow_nearest) {
+
+    float tolerance = syr_coupling->tolerance;
+
+    while (location_complete == false) {
+
+      tolerance *= 4;
+
+      if (syr_coupling->verbosity > 0) {
+        bft_printf(_(" [failed]\n"));
+        if (n_ext > 0)
+          bft_printf(_(" %llu fluid mesh elements not located on solid mesh\n"),
+                     (unsigned long long) n_ext);
+        if (ext_syr)
+          bft_printf(_(" Some solid mesh elements not located on fluid mesh\n"));
+        bft_printf(_("\n   Extending search with tolerance factor %f..."),
+                   tolerance);
+        bft_printf_flush();
+      }
+
+      ple_locator_extend_search(coupling_ent->locator,
+                                location_elts,
+                                locator_options,
+                                0.,
+                                tolerance,
+                                coupling_ent->n_elts,
+                                NULL,
+                                NULL,
+                                elt_centers,
+                                cs_to_syr_dist,
+                                cs_coupling_mesh_extents,
+                                cs_coupling_point_in_mesh);
+
+      location_complete = _is_location_complete(syr_coupling,
+                                                coupling_ent,
+                                                &n_ext,
+                                                &ext_syr);
+
+    }
+
+  }
 
   if (syr_coupling->verbosity > 0) {
     bft_printf(_(" [ok]\n"));
@@ -686,12 +804,13 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
   if (elt_centers != NULL)
     BFT_FREE(elt_centers);
 
-  if (syr_coupling->visualization >= 0) {
+  if (syr_coupling->visualization != 0) {
 
-    cs_post_activate_writer(0, 1);
+    cs_post_activate_writer(-1, 1);
     cs_post_write_meshes(cs_glob_time_step);
 
     cs_post_write_var(coupling_ent->post_mesh_id,
+                      CS_POST_WRITER_ALL_ASSOCIATED,
                       _("distance_to_solid"),
                       1,
                       false,
@@ -710,7 +829,8 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
 
   if (elt_dim == syr_coupling->dim - 1) {
 
-    n_dist_elts = ple_locator_get_n_dist_points(coupling_ent->locator);
+    cs_lnum_t n_dist_elts = ple_locator_get_n_dist_points(coupling_ent->locator);
+
     BFT_MALLOC(syr_to_cs_dist, n_dist_elts, float);
 
     ple_locator_exchange_point_var(coupling_ent->locator,
@@ -721,7 +841,8 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                                    1,
                                    1);
 
-    if (syr_coupling->visualization != 0 && locate_on_closest != NULL) {
+    if (   syr_coupling->visualization != 0
+        && syr_coupling->allow_nearest == false) {
 
       cs_lnum_t i;
       int writer_ids[] = {-1};
@@ -759,10 +880,11 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                                    1,
                                    writer_ids);
 
-      cs_post_activate_writer(0, 1);
-      cs_post_write_meshes(NULL);
+      cs_post_activate_writer(-1, 1);
+      cs_post_write_meshes(cs_glob_time_step);
 
       cs_post_write_vertex_var(mesh_id,
+                               CS_POST_WRITER_ALL_ASSOCIATED,
                                _("distance_to_fluid"),
                                1,
                                false,
@@ -782,13 +904,6 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
 
   }
 
-  /* Check that all points are effectively located */
-
-  n_exterior = ple_locator_get_n_exterior(coupling_ent->locator);
-  n_ext = n_exterior;
-
-  cs_parall_counter(&n_ext, 1);
-
   if (n_ext) {
 
     int i;
@@ -799,7 +914,8 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
     cs_coord_t *el_list = NULL;
     fvm_io_num_t *vtx_io_num = NULL;
     fvm_nodal_t *ulck_points = fvm_nodal_create("unlocated elements (centers)",
-                                                syr_coupling->dim);
+                                                3);
+    n_exterior = ple_locator_get_n_exterior(coupling_ent->locator);
     const ple_lnum_t *exterior_list
       = ple_locator_get_exterior_list(coupling_ent->locator);
 
@@ -846,8 +962,8 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                                  1,
                                  writer_ids);
 
-    cs_post_activate_writer(0, 1);
-    cs_post_write_meshes(NULL);
+    cs_post_activate_writer(writer_ids[0], 1);
+    cs_post_write_meshes(cs_glob_time_step);
     cs_post_free_mesh(mesh_id);
 
     if (cs_glob_n_ranks > 1)
@@ -863,12 +979,13 @@ _create_coupled_ent(cs_syr4_coupling_t  *syr_coupling,
                (unsigned long long)n_ext,
                fvm_nodal_get_name(coupling_ent->elts));
 
-    /* Ensure clean stop */
-
-    cs_coupling_set_sync_flag(PLE_COUPLING_STOP);
-
   }
 
+  /* Ensure clean stop */
+
+  if (location_complete == false)
+    cs_coupling_set_sync_flag(PLE_COUPLING_STOP);
+
   return coupling_ent;
 }
 
@@ -1115,6 +1232,59 @@ _ensure_conservativity(cs_syr4_coupling_t   *syr_coupling,
                  " coupling with SYRTHES: %5.3e\n"), coef);
 }
 
+/*----------------------------------------------------------------------------
+ * Exchange location synchronization status
+ *
+ * parameters:
+ *   syr_coupling <-- SYRTHES coupling structure
+ *
+ * returns:
+ *   0 in case of success, 1 otherwise
+ *----------------------------------------------------------------------------*/
+
+static int
+_sync_after_location(cs_syr4_coupling_t  *syr_coupling)
+{
+  int retval = 1;
+
+  char op_name_send[32 + 1];
+  char op_name_recv[32 + 1];
+
+  /* Communication with SYRTHES */
+  /*----------------------------*/
+
+  /* Ready to start time iterations */
+
+  strcpy(op_name_send, "coupling:start");
+
+  _exchange_sync(syr_coupling, op_name_send, op_name_recv);
+
+  if (!strcmp(op_name_recv, "coupling:error:location")) {
+
+    cs_coupling_set_sync_flag(PLE_COUPLING_STOP);
+
+    cs_base_warn(__FILE__, __LINE__);
+
+    bft_printf(_(" Message received from SYRTHES: \"%s\"\n"
+                 " indicates meshes have not been matched correctly.\n\n"
+                 " The calculation will not run.\n\n"),
+               op_name_recv);
+
+  }
+  else if (strcmp(op_name_recv, "coupling:start"))
+    bft_error(__FILE__, __LINE__, 0,
+              _(" Message received from SYRTHES: \"%s\"\n"
+                " indicates an error or is unexpected."),
+              op_name_recv);
+
+  else
+    retval = 0;
+
+  return retval;
+}
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -1341,6 +1511,7 @@ cs_syr4_coupling_init_comm(cs_syr4_coupling_t *syr_coupling,
   char  volume_flag = ' ';
   char  boundary_flag = ' ';
   char  conservativity_flag = '1';
+  char  allow_nearest_flag = '1';
   char  op_name_send[32 + 1];
   char  op_name_recv[32 + 1];
 
@@ -1357,17 +1528,16 @@ cs_syr4_coupling_init_comm(cs_syr4_coupling_t *syr_coupling,
     volume_flag = 'v';
   if (cs_syr4_coupling_conservativity == 0)
     conservativity_flag = '0';
+  if (syr_coupling->allow_nearest == false)
+    allow_nearest_flag = '0';
 
-  sprintf(op_name_send, "coupling:type:%c%c%c",
-          boundary_flag, volume_flag, conservativity_flag);
+  snprintf(op_name_send, 32, "coupling:type:%c%c%c \2\2%c(%6.2g)",
+           boundary_flag, volume_flag, conservativity_flag,
+           allow_nearest_flag, (double)syr_coupling->tolerance);
 
   _exchange_sync(syr_coupling, op_name_send, op_name_recv);
 
-  /* Only Code_Saturne set its conservativity flag.
-     To avoid an error, set the conservativity flag in op_name_recv */
-  op_name_recv[16] = op_name_send[16];
-
-  if (strcmp(op_name_recv, op_name_send))
+  if (strncmp(op_name_recv, op_name_send, 16))
     bft_error
       (__FILE__, __LINE__, 0,
        _("========================================================\n"
@@ -1393,9 +1563,6 @@ cs_syr4_coupling_init_comm(cs_syr4_coupling_t *syr_coupling,
 void
 cs_syr4_coupling_init_mesh(cs_syr4_coupling_t  *syr_coupling)
 {
-  char op_name_send[32 + 1];
-  char op_name_recv[32 + 1];
-
   const cs_lnum_t verbosity = syr_coupling->verbosity;
 
   if (verbosity > 0)
@@ -1407,50 +1574,30 @@ cs_syr4_coupling_init_mesh(cs_syr4_coupling_t  *syr_coupling)
 
   assert(syr_coupling->dim == 3 || syr_coupling->dim == 2);
 
-  if (syr_coupling->face_sel != NULL)
+  int match_flag = 0;
+
+  if (syr_coupling->face_sel != NULL) {
     syr_coupling->faces = _create_coupled_ent(syr_coupling,
                                               syr_coupling->face_sel,
                                               syr_coupling->dim - 1);
+    match_flag += _sync_after_location(syr_coupling);
+  }
 
   if (syr_coupling->cell_sel != NULL) {
     syr_coupling->cells = _create_coupled_ent(syr_coupling,
                                               syr_coupling->cell_sel,
                                               syr_coupling->dim);
+    match_flag += _sync_after_location(syr_coupling);
   }
 
   /* Communication with SYRTHES */
   /*----------------------------*/
 
-  /* Ready to start time iterations */
-
-  strcpy(op_name_send, "coupling:start");
-
-  _exchange_sync(syr_coupling, op_name_send, op_name_recv);
-
-  if (!strcmp(op_name_recv, "coupling:error:location")) {
-
-    cs_coupling_set_sync_flag(PLE_COUPLING_STOP);
-
-    cs_base_warn(__FILE__, __LINE__);
-
-    bft_printf(_(" Message received from SYRTHES: \"%s\"\n"
-                 " indicates meshes have not been matched correctly.\n\n"
-                 " The calculation will not run.\n\n"),
-               op_name_recv);
-
-  }
-  else if (strcmp(op_name_recv, "coupling:start"))
-    bft_error(__FILE__, __LINE__, 0,
-              _(" Message received from SYRTHES: \"%s\"\n"
-                " indicates an error or is unexpected."),
-              op_name_recv);
-
-  else if (verbosity > 0) {
+  if (match_flag == 0 && verbosity > 0) {
     bft_printf(_("\n ** Mesh located for SYRTHES coupling \"%s\".\n\n"),
                syr_coupling->syr_name);
     bft_printf_flush();
   }
-
 }
 
 /*----------------------------------------------------------------------------
