@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -59,9 +59,10 @@
 #include "cs_coupling.h"
 #endif
 
+#include "cs_log.h"
 #include "cs_parall.h"
 #include "cs_prototypes.h"
-
+#include "cs_thermal_model.h"
 #include "cs_syr4_coupling.h"
 
 /*----------------------------------------------------------------------------
@@ -74,6 +75,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -426,6 +429,8 @@ _init_all_mpi_syr(void)
 
 #endif /* defined(HAVE_MPI) */
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  *  Public function definitions for Fortran API
  *============================================================================*/
@@ -508,7 +513,6 @@ void CS_PROCF(tsursy, TSURSY)
 
       if (scb->face_sel_c != NULL)
         *issurf = 1;
-
     }
 
   }
@@ -581,32 +585,6 @@ void CS_PROCF(tvolsy, TVOLSY)
 
 }
 
-/*----------------------------------------------------------------------------
- * Create nodal coupled mesh.
- *
- * Setup PLE locator for SYRTHES 4.
- *
- * Fortran Interface:
- *
- * SUBROUTINE GEOSYR
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF(geosyr, GEOSYR)
-(
- void
-)
-{
-  int coupl_id;
-
-  _cs_glob_n_syr4_cp = cs_syr4_coupling_n_couplings();
-
-  for (coupl_id = 0; coupl_id < _cs_glob_n_syr4_cp; coupl_id++) {
-    cs_syr4_coupling_t *syr_coupling = cs_syr4_coupling_by_id(coupl_id);
-    cs_syr4_coupling_init_mesh(syr_coupling);
-  }
-}
-
 /*----------------------------------------------------------------------------
  * Get number of coupled elements with SYRTHES.
  *
@@ -676,23 +654,6 @@ void CS_PROCF(leltsy, LELTSY)
   }
 }
 
-/*----------------------------------------------------------------------------
- * User function wrapper for definition of SYRTHES couplings
- *
- * Fortran Interface:
- *
- * SUBROUTINE USSYRC
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (ussyrc, USSYRC)
-(
- void
-)
-{
-  cs_user_syrthes_coupling();
-}
-
 /*----------------------------------------------------------------------------
  * Receive coupling variables from SYRTHES
  *
@@ -809,28 +770,32 @@ void CS_PROCF (ctbvsy, CTBVSY)
  * Public function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Define new SYRTHES coupling.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define new SYRTHES coupling.
+ *
+ * The arguments to \ref cs_syr_coupling_define are:
+ * \param[in] syrthes_name      matching SYRTHES application name
+ * \param[in] boundary_criteria surface selection criteria, or NULL
+ * \param[in] volume_criteria   volume selection criteria, or NULL
+ * \param[in] projection_axis   x', 'y', or 'y' for 2D projection axis (case
+ *                              independent), or ' ' for standard 3D coupling
+ * \param[in] allow_nonmatching allow nearest-neighbor mapping where matching
+ *                              within tolerance is not available (useful
+ *                              when meshes have a different level of detail)
+ * \param[in] tolerance         addition to local extents of each element
+ *                              extent = base_extent * (1 + tolerance)
+ * \param[in] verbosity         verbosity level
+ * \param[in] visualization     visualization output level (0 or 1)
  *
  * In the case of a single Code_Saturne and single SYRTHES instance, the
- * syrthes_name argument is ignored.
+ * 'syrthes_name' argument is ignored, as there is only one matching
+ * possibility.
  *
  * In case of multiple couplings, a coupling will be matched with available
- * SYRTHES instances based on the syrthes_name argument.
- *
- * parameters:
- *   syrthes_name      <-- name of SYRTHES instance
- *   boundary_criteria <-- boundary face selection criteria, or NULL
- *   volume_criteria   <-- volume cell selection criteria, or NULL
- *   projection_axis   <-- 'x', 'y', or 'y' for 2D projection axis (case
- *                         independent), or ' ' for standard 3D coupling
- *   allow_nonmatching <-- allow nearest-neighbor mapping where matching
- *                         within tolerance is not available
- *   tolerance         <-- addition to local extents of each element
- *                         extent = base_extent * (1 + tolerance)
- *   verbosity         <-- verbosity level
- *   visualization     <-- visualization output level (0 or 1)
- *----------------------------------------------------------------------------*/
+ * SYRTHES instances based on the 'syrthes_name' argument.
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_syr_coupling_define(const char  *syrthes_name,
@@ -934,6 +899,8 @@ cs_syr_coupling_all_init(void)
               _("At least 1 SYRTHES coupling was defined for which\n"
                 "no communication with a SYRTHES instance is possible."));
   }
+
+  _cs_glob_n_syr4_cp = cs_syr4_coupling_n_couplings();
 }
 
 /*----------------------------------------------------------------------------
@@ -991,6 +958,93 @@ cs_syr_coupling_set_explicit_treatment(void)
   cs_syr4_coupling_set_explicit_treatment();
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Log SYRTHES coupling setup information.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_syr_coupling_log_setup(void)
+{
+  /* Get the number of SYRTHES couplings */
+  int n_coupl = cs_syr_coupling_n_couplings();
+  const int keysca = cs_field_key_id("scalar_id");
+  const int kcpsyr = cs_field_key_id("syrthes_coupling");
+  int icpsyr;
+
+  if (n_coupl >= 1) {
+
+    cs_log_printf
+      (CS_LOG_SETUP,
+       _("SYRTHES coupling\n"
+         "----------------\n\n"
+         "    number of couplings: %d\n"),
+         n_coupl);
+
+    int n_surf_coupl = 0, n_vol_coupl = 0, issurf, isvol;
+
+    for (int ii = 1 ; ii <= n_coupl ; ii++) {
+      /* Add a new surface coupling if detected */
+      issurf = 0;
+      CS_PROCF(tsursy, TSURSY)(&ii, &issurf);
+      n_surf_coupl += issurf;
+
+      /* Add a new volume coupling if detected */
+      isvol = 0;
+      CS_PROCF(tvolsy, TVOLSY)(&ii, &isvol);
+      n_vol_coupl += isvol;
+    }
+
+    cs_log_printf
+      (CS_LOG_SETUP,
+       _("    with             %d surface coupling(s)\n"
+         "    with             %d volume coupling(s)\n"),
+         n_surf_coupl, n_vol_coupl);
+
+    cs_log_printf
+      (CS_LOG_SETUP,
+       _("\n"
+         "   Coupled scalars\n"
+         "------------------------\n"
+         " Scalar    Number icpsyr\n"
+         "------------------------\n"));
+
+
+    for (int f_id = 0 ; f_id < cs_field_n_fields() ; f_id++) {
+      cs_field_t  *f = cs_field_by_id(f_id);
+      if ((f->type & CS_FIELD_VARIABLE) || (f->type & CS_FIELD_USER)) {
+        int ii = cs_field_get_key_int(f, keysca);
+        if (ii > 0) {
+          icpsyr = cs_field_get_key_int(f, kcpsyr);
+          cs_log_printf
+            (CS_LOG_SETUP,
+             _(" %s %7d %7d\n"),cs_field_get_label(f),ii, icpsyr);
+        }
+      }
+    }
+    cs_log_printf
+      (CS_LOG_SETUP,
+       _("------------------------\n\n"
+         "    icpsyr = 0 or 1         (1: scalar coupled to SYRTHES)\n"));
+  }
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create coupled meshes and setup PLE locator for Syrthes couplings.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_syr_coupling_init_meshes(void)
+{
+  for (int coupl_id = 0; coupl_id < _cs_glob_n_syr4_cp; coupl_id++) {
+    cs_syr4_coupling_t *syr_coupling = cs_syr4_coupling_by_id(coupl_id);
+    cs_syr4_coupling_init_mesh(syr_coupling);
+  }
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
