@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -113,21 +113,6 @@ void CS_PROCF(tvolsy, TVOLSY)
  cs_int_t  *isvol
 );
 
-/*----------------------------------------------------------------------------
- * Create nodal coupled mesh.
- * Send vertices's coordinates and connectivity of coupled mesh.
- *
- * Fortran Interface:
- *
- * SUBROUTINE GEOSYR
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF(geosyr, GEOSYR)
-(
- void
-);
-
 /*----------------------------------------------------------------------------
  * Get number of coupled elements with SYRTHES.
  *
@@ -168,20 +153,6 @@ void CS_PROCF(leltsy, LELTSY)
        cs_lnum_t   *coupl_elt_list
 );
 
-/*----------------------------------------------------------------------------
- * User function wrapper for definition of SYRTHES couplings
- *
- * Fortran Interface:
- *
- * SUBROUTINE USSYRC
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (ussyrc, USSYRC)
-(
- void
-);
-
 /*----------------------------------------------------------------------------
  * Receive coupling variables from SYRTHES
  *
@@ -331,6 +302,24 @@ cs_syr_coupling_set_conservativity(int  flag);
 void
 cs_syr_coupling_set_explicit_treatment(void);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Log SYRTHES coupling setup information.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_syr_coupling_log_setup(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Create coupled meshes and setup PLE locator for Syrthes couplings.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_syr_coupling_init_meshes(void);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
