@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -30,6 +30,7 @@
  * Standard C library headers
  *----------------------------------------------------------------------------*/
 
+#include <assert.h>
 #include <string.h>
 #include <time.h>
 
@@ -53,12 +54,6 @@
 #include <pwd.h>
 #endif
 
-#if defined(__bgp__)
-#include <spi/kernel_interface.h>
-#include <common/bgp_personality.h>
-#include <common/bgp_personality_inlines.h>
-#endif
-
 #if defined(__bgq__) && defined(__xlc__)
 #include <spi/include/kernel/location.h>
 #endif
@@ -68,6 +63,7 @@
  *----------------------------------------------------------------------------*/
 
 #include "bft_printf.h"
+#include "cs_log.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
@@ -79,6 +75,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Local type definitions
  *============================================================================*/
@@ -91,6 +89,38 @@ BEGIN_C_DECLS
  * Private function definitions
  *============================================================================*/
 
+/*----------------------------------------------------------------------------
+ * Remove leading and trailing whitespace from a string.
+ *
+ * parameters:
+ *   s <-> string to be cleaned
+ *----------------------------------------------------------------------------*/
+
+static void
+_string_clean(char  *s)
+{
+  assert(s != NULL);
+
+  int l = strlen(s);
+  int i = l - 1;
+  while (i > -1 && (   s[i] == ' ' || s[i] == '\t'
+                    || s[i] == '\n' || s[i] == '\r')) {
+    s[i--] = '\0';
+  }
+
+  i = 0;
+  while (i < l && (   s[i] == ' ' || s[i] == '\t'
+                   || s[i] == '\n' || s[i] == '\r')) {
+    i++;
+  }
+
+  if (i > 0) {
+    int j = 0;
+    while (i <= l)
+      s[j++] = s[i++];
+  }
+}
+
 /*----------------------------------------------------------------------------
  * Return basic available CPU info depending on system.
  *
@@ -105,7 +135,7 @@ _sys_info_cpu(char      *cpu_str,
 {
   strcpy(cpu_str, "");
 
-#if defined(__linux__)
+#if defined(__linux__) && !defined(__ve__)
 
   {
     unsigned _cpu_str_max = (cpu_str_max > 0) ? cpu_str_max - 1 : 0;
@@ -150,34 +180,97 @@ _sys_info_cpu(char      *cpu_str,
 #endif /* HAVE_SYS_UTSNAME_H */
 }
 
-/*============================================================================
- * Public function definitions
- *============================================================================*/
-
 /*----------------------------------------------------------------------------
- * Print available system information.
+ * Return Linux info based on /etc/issue.
+ *
+ * Only the information prior to a first escape sequence is returned.
+ *
+ * parameters:
+ *   issue_str     --> string with system description
+ *   issue_str_max <-- maximum length of string with system description
  *----------------------------------------------------------------------------*/
 
+static void
+_sys_info_issue(char      *issue_str,
+                unsigned   issue_str_max)
+{
+  issue_str[0] = '\0';
+
+#if defined(__linux__)
+
+  {
+    unsigned _issue_str_max = (issue_str_max > 0) ? issue_str_max - 1 : 0;
+
+    FILE *fp;
+    char *s;
+
+    fp = fopen("/etc/issue", "r");
+
+    if (fp != NULL) {
+
+      issue_str[0] = ' ';
+      issue_str[1] = '(';
+
+      s = fgets(issue_str + 2, _issue_str_max - 4, fp);
+
+      if (s != NULL) {
+        int l = strlen(s);
+        for (int i = 0; i < l; i++)
+          if (s[i] == '\\') {
+            s[i] = '\0';
+            l = i;
+          }
+        _string_clean(issue_str + 2);
+        l = strlen(issue_str);
+        if (l > 2) {
+          issue_str[l] = ')';
+          issue_str[l+1] = '\0';
+        }
+        else /* If no info was kept, empty string */
+          issue_str[0] = '\0';
+      }
+    }
+
+    fclose (fp);
+
+  }
+
+#endif
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print available system information.
+ *
+ * \param[in]  comm  associated MPI communicator
+ * \param[in]  log   if true, standard logging; otherwise, single output
+ */
+/*----------------------------------------------------------------------------*/
+
 #if defined(HAVE_MPI)
 
-void
-cs_system_info(MPI_Comm comm)
+static void
+_system_info(MPI_Comm comm,
+             bool     log)
 
 #else
 
-void
-cs_system_info(void)
+static void
+_system_info(bool  log)
 
 #endif
 
 {
   time_t          date;
   size_t          ram;
+  int             log_id;
+  int             n_logs = (log) ? 2 : 1;
+
+  cs_log_t logs[] = {CS_LOG_DEFAULT, CS_LOG_PERFORMANCE};
 
 #if defined(HAVE_UNAME)
   struct utsname  sys_config;
 #endif
-
 #if defined(HAVE_GETPWUID) && defined(HAVE_GETEUID)
   struct passwd   *pwd_user = NULL;
 #endif
@@ -188,16 +281,14 @@ cs_system_info(void)
 
   char  str_date[81];
   char  str_cpu[81];
+  char  str_issue[81];
   char  str_directory[PATH_MAX] = "";
 
 # if defined(HAVE_MPI)
   int mpi_flag = 0;
 #endif
 
-#if defined(__bgp__)
-  _BGP_Personality_t personality;
-  Kernel_GetPersonality(&personality, sizeof(personality));
-#elif defined(__bgq__) && defined(__xlc__)
+#if defined(__bgq__) && defined(__xlc__)
   Personality_t personality;
   Kernel_GetPersonality(&personality, sizeof(personality));
 #endif
@@ -218,30 +309,44 @@ cs_system_info(void)
   /* Print local configuration */
   /*---------------------------*/
 
-  bft_printf("\n%s\n", _("Local case configuration:\n"));
+  if (log) {
+    for (log_id = 0; log_id < n_logs; log_id++)
+      cs_log_printf(logs[log_id],
+                    "\n%s\n", _("Local case configuration:\n"));
+  }
 
-  bft_printf("  %s%s\n", _("Date:                "), str_date);
+  for (log_id = 0; log_id < n_logs; log_id++)
+    cs_log_printf(logs[log_id],
+                  "  %s%s\n", _("Date:                "), str_date);
 
   /* System and machine */
 
+  _sys_info_issue(str_issue, 81);
+
 #if defined(HAVE_UNAME)
 
   if (uname(&sys_config) != -1) {
-    bft_printf("  %s%s %s\n", _("System:              "),
-               sys_config.sysname, sys_config.release);
-    bft_printf("  %s%s\n", _("Machine:             "),  sys_config.nodename);
+    for (log_id = 0; log_id < n_logs; log_id++) {
+      cs_log_printf(logs[log_id],
+                    "  %s%s %s%s\n", _("System:              "),
+                    sys_config.sysname, sys_config.release,
+                    str_issue);
+      cs_log_printf(logs[log_id],
+                    "  %s%s\n", _("Machine:             "),
+                    sys_config.nodename);
+    }
   }
 #endif
 
   _sys_info_cpu(str_cpu, 81);
 
-  bft_printf("  %s%s\n", _("Processor:           "), str_cpu);
+  for (log_id = 0; log_id < n_logs; log_id++)
+    cs_log_printf(logs[log_id],
+                  "  %s%s\n", _("Processor:           "), str_cpu);
 
   /* Available memory */
 
-#if defined(__bgp__)
-  ram = personality.DDR_Config.DDRSizeMB;
-#elif defined(__bgq__) && defined(__xlc__)
+#if defined(__bgq__) && defined(__xlc__)
   ram = personality.DDR_Config.DDRSizeMB;
 #elif defined(__linux__) \
    && defined(HAVE_SYS_SYSINFO_H) && defined(HAVE_SYSINFO)
@@ -254,9 +359,12 @@ cs_system_info(void)
   ram = 0; /* TODO: complete for Windows systems */
 #endif
 
-  if (ram > 0)
-    bft_printf("  %s%llu %s\n", _("Memory:              "),
-               (unsigned long long)ram, _("MB"));
+  if (ram > 0) {
+    for (log_id = 0; log_id < n_logs; log_id++)
+      cs_log_printf(logs[log_id],
+                    "  %s%llu %s\n", _("Memory:              "),
+                    (unsigned long long)ram, _("MB"));
+  }
 
   /* User info */
 
@@ -264,8 +372,8 @@ cs_system_info(void)
 
   /* Functions not available on IBM Blue Gene or Cray XT,
      but a stub may exist, so we make sure we ignore it */
-#if   defined(__blrts__) || defined(__bg__) \
-   || defined(__CRAYXT_COMPUTE_LINUX_TARGET)
+#if   defined(__bg__) || defined(_CRAYC) \
+   || defined(__CRAYXT) || defined(__CRAYXE) || defined(__CRAYXC)
   pwd_user = NULL;
 #else
   pwd_user = getpwuid(geteuid());
@@ -275,7 +383,8 @@ cs_system_info(void)
 
     size_t l_info = 0;
 
-    bft_printf("  %s%s", _("User:                "), pwd_user->pw_name);
+    cs_log_printf(CS_LOG_DEFAULT,
+                  "  %s%s", _("User:                "), pwd_user->pw_name);
 
     if (pwd_user->pw_gecos != NULL) {
       for (l_info = 0;
@@ -284,17 +393,20 @@ cs_system_info(void)
            l_info++);
       if (pwd_user->pw_gecos[l_info] == ',')
         pwd_user->pw_gecos[l_info] = '\0';
-      bft_printf(" (%s)", pwd_user->pw_gecos);
+      cs_log_printf(CS_LOG_DEFAULT,
+                    " (%s)", pwd_user->pw_gecos);
     }
 
-    bft_printf("\n");
+    cs_log_printf(CS_LOG_DEFAULT, "\n");
   }
 
 #endif /* defined(HAVE_GETPWUID) && defined(HAVE_GETEUID) */
 
   /* Directory info */
 
-  bft_printf("  %s%s\n", _("Directory:           "), str_directory);
+  for (log_id = 0; log_id < n_logs; log_id++)
+    cs_log_printf(logs[log_id],
+                  "  %s%s\n", _("Directory:           "), str_directory);
 
   /* MPI Info */
 
@@ -309,63 +421,42 @@ cs_system_info(void)
     MPI_Comm_size(comm, &n_ranks);
     MPI_Comm_size(MPI_COMM_WORLD, &n_world_ranks);
 
-#   if defined(__bgp__)
-
-    {
-      int node_config = personality.Kernel_Config.ProcessConfig;
-      const char *_mode_names[] = {N_("SMP mode"),
-                                   N_("Virtual-node mode"),
-                                   N_("Dual mode"),
-                                   N_("Unknown mode")};
-      const char *mode_name = _mode_names[3];
-
-      if (node_config == _BGP_PERS_PROCESSCONFIG_SMP)
-        mode_name = _mode_names[0];
-      else if (node_config == _BGP_PERS_PROCESSCONFIG_VNM)
-        mode_name = _mode_names[1];
-      else if (node_config == _BGP_PERS_PROCESSCONFIG_2x2)
-        mode_name = _mode_names[2];
-
-      bft_printf("  %s%d (%s)\n", _("MPI ranks:           "),
-                 n_ranks, _(mode_name));
-      if (n_world_ranks > n_ranks)
-        bft_printf("  %s%d\n", _("MPI_COMM_WORLD size: "),
-                   n_world_ranks);
-      bft_printf("  %s%d\n", _("pset size:         "),
-                 personality.Network_Config.PSetSize);
-      bft_printf("  %s<%d,%d,%d>\n", _("Torus dimensions:    "),
-                 personality.Network_Config.Xnodes,
-                 personality.Network_Config.Ynodes,
-                 personality.Network_Config.Znodes);
-    }
-
-#   elif defined(__bgq__) && defined(__xlc__)
+#   if defined(__bgq__) && defined(__xlc__)
 
     {
       int a_torus, b_torus, c_torus, d_torus, e_torus;
       int n_flags = personality.Network_Config.NetFlags;
       int n_hw_threads = Kernel_ProcessorCount();
 
-      bft_printf("  %s%d\n", _("MPI ranks:           "), n_ranks);
-      if (n_world_ranks > n_ranks)
-        bft_printf("  %s%d\n", _("MPI_COMM_WORLD size: "),
-                   n_world_ranks);
-      bft_printf("  %s%d\n", _("Hardware threads:    "), n_hw_threads);
+      for (log_id = 0; log_id < n_logs; log_id++) {
+        cs_log_printf(logs[log_id],
+                      "  %s%d\n", _("MPI ranks:           "), n_ranks);
+        if (n_world_ranks > n_ranks)
+          cs_log_printf(logs[log_id],
+                        "  %s%d\n", _("MPI_COMM_WORLD size: "),
+                        n_world_ranks);
+        cs_log_printf(logs[log_id],
+                      "  %s%d\n", _("Hardware threads:    "), n_hw_threads);
+      }
+
       if (n_flags & ND_ENABLE_TORUS_DIM_A) a_torus = 1; else a_torus = 0;
       if (n_flags & ND_ENABLE_TORUS_DIM_B) b_torus = 1; else b_torus = 0;
       if (n_flags & ND_ENABLE_TORUS_DIM_C) c_torus = 1; else c_torus = 0;
       if (n_flags & ND_ENABLE_TORUS_DIM_D) d_torus = 1; else d_torus = 0;
       if (n_flags & ND_ENABLE_TORUS_DIM_E) e_torus = 1; else e_torus = 0;
 
-      bft_printf("  %s<%d,%d,%d,%d,%d>\n", _("Block shape:         "),
-                 personality.Network_Config.Anodes,
-                 personality.Network_Config.Bnodes,
-                 personality.Network_Config.Cnodes,
-                 personality.Network_Config.Dnodes,
-                 personality.Network_Config.Enodes);
-
-      bft_printf("  %s<%d,%d,%d,%d,%d>\n", _("Torus links enabled: "),
-                 a_torus, b_torus, c_torus, d_torus, e_torus);
+      for (log_id = 0; log_id < n_logs; log_id++) {
+        cs_log_printf(logs[log_id],
+                      "  %s<%d,%d,%d,%d,%d>\n", _("Block shape:         "),
+                      personality.Network_Config.Anodes,
+                      personality.Network_Config.Bnodes,
+                      personality.Network_Config.Cnodes,
+                      personality.Network_Config.Dnodes,
+                      personality.Network_Config.Enodes);
+        cs_log_printf(logs[log_id],
+                      "  %s<%d,%d,%d,%d,%d>\n", _("Torus links enabled: "),
+                      a_torus, b_torus, c_torus, d_torus, e_torus);
+      }
     }
 
 #   else
@@ -381,16 +472,21 @@ cs_system_info(void)
         appnum = *(int *)attp;
 #     endif
 
-      if (appnum > -1)
-        bft_printf("  %s%d (%s %d)\n",
-                   _("MPI ranks:           "), n_ranks,
-                   _("appnum attribute:"), appnum);
-      else
-        bft_printf("  %s%d\n", _("MPI ranks:           "), n_ranks);
+      for (log_id = 0; log_id < n_logs; log_id++) {
+        if (appnum > -1 && log_id == 0)
+          cs_log_printf(logs[log_id],
+                        "  %s%d (%s %d)\n",
+                        _("MPI ranks:           "), n_ranks,
+                        _("appnum attribute:"), appnum);
+        else
+          cs_log_printf(logs[log_id],
+                        "  %s%d\n", _("MPI ranks:           "), n_ranks);
 
-      if (n_world_ranks > n_ranks)
-        bft_printf("  %s%d\n", _("MPI_COMM_WORLD size: "),
-                   n_world_ranks);
+        if (n_world_ranks > n_ranks)
+          cs_log_printf(logs[log_id],
+                        "  %s%d\n", _("MPI_COMM_WORLD size: "),
+                        n_world_ranks);
+      }
     }
 
 #   endif /* defined(HAVE_MPI) */
@@ -403,17 +499,84 @@ cs_system_info(void)
   {
     int t_id = omp_get_thread_num();
     if (t_id == 0) {
-      bft_printf("  %s%d\n", _("OpenMP threads:      "),
-                 omp_get_max_threads());
-      if (omp_get_dynamic())
-        bft_printf("  %s\n", _("Dynamic scheduling allowed"));
-      bft_printf("  %s%d\n", _("Processors/node:     "),
-                 omp_get_num_procs());
+      for (log_id = 0; log_id < n_logs; log_id++) {
+        cs_log_printf(logs[log_id],
+                      "  %s%d\n", _("OpenMP threads:      "),
+                      omp_get_max_threads());
+        if (omp_get_dynamic())
+          cs_log_printf(logs[log_id],
+                        "  %s\n", _("Dynamic scheduling allowed"));
+        cs_log_printf(logs[log_id],
+                      "  %s%d\n", _("Processors/node:     "),
+                      omp_get_num_procs());
+      }
     }
   }
 #endif
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
+/*============================================================================
+ * Public function definitions
+ *============================================================================*/
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print available system information.
+ *
+ * \param[in]  comm  associated MPI communicator
+ */
+/*----------------------------------------------------------------------------*/
+
+#if defined(HAVE_MPI)
+
+void
+cs_system_info(MPI_Comm comm)
+
+#else
+
+void
+cs_system_info(void)
+
+#endif
+
+{
+#if defined(HAVE_MPI)
+  _system_info(comm, true);
+#else
+  _system_info(true);
+#endif
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Print available system information, without additional logging
+ *
+ * \param[in]  comm  associated MPI communicator
+ */
+/*----------------------------------------------------------------------------*/
+
+#if defined(HAVE_MPI)
+
+void
+cs_system_info_no_log(MPI_Comm comm)
+
+#else
+
+void
+cs_system_info_no_log(void)
+
+#endif
+
+{
+#if defined(HAVE_MPI)
+  _system_info(comm, false);
+#else
+  _system_info(false);
+#endif
+}
+
 /*-----------------------------------------------------------------------------*/
 
 END_C_DECLS
