@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -57,6 +57,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -98,16 +100,24 @@ struct _cs_time_plot_t {
   size_t      buffer_end;       /* Current buffer end */
   char       *buffer;           /* Associated buffer if required */
 
+  struct _cs_time_plot_t  *prev;  /* Previous in flush list */
+  struct _cs_time_plot_t  *next;  /* Next in flush list */
+
 };
 
 /*============================================================================
  *  Global variables
  *============================================================================*/
 
+static cs_time_plot_t   *_plots_head = NULL, *_plots_tail = NULL;
+
 static size_t            _n_files[2] = {0, 0};
 static size_t            _n_files_max[2] = {0, 0};
 static cs_time_plot_t  **_plot_files[2] = {NULL, NULL};
 
+static float             _flush_wtime_default = -1;
+static int               _n_buffer_steps_default = -1;
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -144,13 +154,15 @@ _ensure_buffer_size(cs_time_plot_t  *p,
  *   n_probes         <-- number of probes associated with this variable ?
  *   probe_list       <-- numbers (1 to n) of probes if filtered, or NULL
  *   probe_coords     <-- probe coordinates
+ *   probe_names      <-- probe names, or NULL
  *----------------------------------------------------------------------------*/
 
 static void
 _write_probe_header_dat(cs_time_plot_t    *p,
                         int                n_probes,
                         const int         *probe_list,
-                        const cs_real_t    probe_coords[])
+                        const cs_real_t    probe_coords[],
+                        const char        *probe_names[])
 {
   int i, probe_id;
   int col_id = 0;
@@ -177,14 +189,27 @@ _write_probe_header_dat(cs_time_plot_t    *p,
       probe_id = i;
       if (probe_list != NULL)
         probe_id = probe_list[i] - 1;
-      fprintf(_f, "#   %6i [%14.7e, %14.7e, %14.7e]\n",
-              probe_id + 1,
-              probe_coords[probe_id*3],
-              probe_coords[probe_id*3 + 1],
-              probe_coords[probe_id*3 + 2]);
+      if (probe_names != NULL)
+        fprintf(_f, "# %16s [%14.7e, %14.7e, %14.7e]\n",
+                probe_names[i],
+                probe_coords[probe_id*3],
+                probe_coords[probe_id*3 + 1],
+                probe_coords[probe_id*3 + 2]);
+      else
+        fprintf(_f, "#   %6i [%14.7e, %14.7e, %14.7e]\n",
+                probe_id + 1,
+                probe_coords[probe_id*3],
+                probe_coords[probe_id*3 + 1],
+                probe_coords[probe_id*3 + 2]);
     }
     fprintf(_f, "#\n");
   }
+  else if (probe_names != NULL) {
+    fprintf(_f, _("# Monitoring points:\n"));
+    for (i = 0; i < n_probes; i++)
+      fprintf(_f, "# %s\n", probe_names[i]);
+    fprintf(_f, "#\n");
+  }
 
   fprintf(_f, _("# Columns:\n"));
   if (p->use_iteration)
@@ -205,14 +230,26 @@ _write_probe_header_dat(cs_time_plot_t    *p,
     probe_id = i;
     if (probe_list != NULL)
       probe_id = probe_list[i] - 1;
-    if (probe_coords != NULL)
-      fprintf(_f, " | %d [%9.5e, %9.5e, %9.5e]",
-              probe_id + 1,
-              probe_coords[probe_id*3],
-              probe_coords[probe_id*3 + 1],
-              probe_coords[probe_id*3 + 2]);
-    else
-      fprintf(_f, " | %d", probe_id + 1);
+    if (probe_names != NULL) {
+      if (probe_coords != NULL)
+        fprintf(_f, " | %s [%9.5e, %9.5e, %9.5e]",
+                probe_names[i],
+                probe_coords[probe_id*3],
+                probe_coords[probe_id*3 + 1],
+                probe_coords[probe_id*3 + 2]);
+      else
+        fprintf(_f, " | %s", probe_names[i]);
+    }
+    else {
+      if (probe_coords != NULL)
+        fprintf(_f, " | %d [%9.5e, %9.5e, %9.5e]",
+                probe_id + 1,
+                probe_coords[probe_id*3],
+                probe_coords[probe_id*3 + 1],
+                probe_coords[probe_id*3 + 2]);
+      else
+        fprintf(_f, " | %d", probe_id + 1);
+    }
   }
   fprintf(_f, "\n");
 
@@ -225,7 +262,7 @@ _write_probe_header_dat(cs_time_plot_t    *p,
     fprintf(_f, " -");
   fprintf(_f, "\n#\n");
 
-  /* Close file or assign it ot handler depending on options */
+  /* Close file or assign it to handler depending on options */
 
   if (p->buffer_steps[0] > 0) {
     if (fclose(_f) != 0)
@@ -262,32 +299,35 @@ _write_probe_coords_csv(const char        *file_prefix,
              strlen(file_prefix) + strlen(plot_name) + strlen("_coords") + 4 + 1,
              char);
 
-  sprintf(file_name, "%s%s%s.csv", file_prefix, plot_name, "_coords");
+  if (probe_coords != NULL) {
+    sprintf(file_name, "%s%s%s.csv", file_prefix, plot_name, "_coords");
+
+    _f = fopen(file_name, "w");
+    if (_f == NULL) {
+      bft_error(__FILE__, __LINE__, errno,
+          _("Error opening file: \"%s\""), file_name);
+      return;
+    }
+
+    fprintf(_f, "x, y, z\n");
+    for (i = 0; i < n_probes; i++) {
+      probe_id = i;
+      if (probe_list != NULL)
+        probe_id = probe_list[i] - 1;
+      fprintf(_f, "%14.7e, %14.7e, %14.7e\n",
+          probe_coords[probe_id*3],
+          probe_coords[probe_id*3 + 1],
+          probe_coords[probe_id*3 + 2]);
+    }
+
+    /* Close file or assign it ot handler depending on options */
+
+    if (fclose(_f) != 0)
+      bft_error(__FILE__, __LINE__, errno,
+          _("Error closing file: \"%s\""), file_name);
 
-  _f = fopen(file_name, "w");
-  if (_f == NULL) {
-    bft_error(__FILE__, __LINE__, errno,
-              _("Error opening file: \"%s\""), file_name);
-    return;
   }
 
-  fprintf(_f, "x, y, z\n");
-  for (i = 0; i < n_probes; i++) {
-    probe_id = i;
-    if (probe_list != NULL)
-      probe_id = probe_list[i] - 1;
-    fprintf(_f, "%14.7e, %14.7e, %14.7e\n",
-            probe_coords[probe_id*3],
-            probe_coords[probe_id*3 + 1],
-            probe_coords[probe_id*3 + 2]);
-  }
-
-  /* Close file or assign it ot handler depending on options */
-
-  if (fclose(_f) != 0)
-    bft_error(__FILE__, __LINE__, errno,
-              _("Error closing file: \"%s\""), file_name);
-
   BFT_FREE(file_name);
 }
 
@@ -299,13 +339,15 @@ _write_probe_coords_csv(const char        *file_prefix,
  *   n_probes         <-- number of probes associated with this variable ?
  *   probe_list       <-- numbers (1 to n) of probes if filtered, or NULL
  *   probe_coords     <-- probe coordinates
+ *   probe_names      <-- probe names, or NULL
  *----------------------------------------------------------------------------*/
 
 static void
 _write_probe_header_csv(cs_time_plot_t    *p,
                         int                n_probes,
                         const int         *probe_list,
-                        const cs_real_t    probe_coords[])
+                        const cs_real_t    probe_coords[],
+                        const char        *probe_names[])
 {
   int i, probe_id;
   FILE *_f = p->f;
@@ -330,14 +372,24 @@ _write_probe_header_csv(cs_time_plot_t    *p,
     probe_id = i;
     if (probe_list != NULL)
       probe_id = probe_list[i] - 1;
-    if (probe_coords != NULL)
-      fprintf(_f, ",%d [%9.5e; %9.5e; %9.5e]",
-              probe_id + 1,
-              probe_coords[probe_id*3],
-              probe_coords[probe_id*3 + 1],
-              probe_coords[probe_id*3 + 2]);
+    if (probe_coords != NULL) {
+      if (probe_names != NULL)
+        fprintf(_f, ", %s [%9.5e; %9.5e; %9.5e]",
+                probe_names[i],
+                probe_coords[probe_id*3],
+                probe_coords[probe_id*3 + 1],
+                probe_coords[probe_id*3 + 2]);
+      else
+        fprintf(_f, ", %d [%9.5e; %9.5e; %9.5e]",
+                probe_id + 1,
+                probe_coords[probe_id*3],
+                probe_coords[probe_id*3 + 1],
+                probe_coords[probe_id*3 + 2]);
+    }
+    else if (probe_names != NULL)
+      fprintf(_f, ", %s", probe_names[i]);
     else
-      fprintf(_f, ",%d", probe_id + 1);
+      fprintf(_f, ", %d", probe_id + 1);
   }
   fprintf(_f, "\n");
 
@@ -513,6 +565,45 @@ _write_struct_header_csv(cs_time_plot_t  *p,
     p->f = _f;
 }
 
+/*----------------------------------------------------------------------------
+ * Add a time plot to the global time plots array.
+ *----------------------------------------------------------------------------*/
+
+static void
+_time_plot_register(cs_time_plot_t  *p)
+{
+  p->prev = _plots_tail;
+  p->next = NULL;
+
+  if (_plots_head == NULL)
+    _plots_head = p;
+  else if (_plots_head->next == NULL)
+    _plots_head->next = p;
+
+  if (_plots_tail != NULL)
+    _plots_tail->next = p;
+
+  _plots_tail = p;
+}
+
+/*----------------------------------------------------------------------------
+ * Remove a time plot from the global time plots array.
+ *----------------------------------------------------------------------------*/
+
+static void
+_time_plot_unregister(cs_time_plot_t  *p)
+{
+  if (_plots_head == p)
+    _plots_head = p->next;
+  if (_plots_tail == p)
+    _plots_tail = p->prev;
+
+  if (p->prev != NULL)
+    p->prev->next = p->next;
+  if (p->next != NULL)
+    p->next->prev = p->prev;
+}
+
 /*----------------------------------------------------------------------------
  * Create time plot writer for a given variable
  *
@@ -582,6 +673,8 @@ _plot_file_create(const char             *plot_name,
 
   BFT_MALLOC(p->buffer, p->buffer_size, char);
 
+  _time_plot_register(p);
+
   return p;
 }
 
@@ -686,95 +779,12 @@ _fortran_time_plot_realloc(int                     plot_num,
   _n_files[format] += 1;
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions for Fortran API
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Create a writer for time plot probe-type data.
- *
- * This subroutine should only be called by one rank for a given data series.
- *
- * subroutine tppini (tplnum, tplnam, tplpre, tplfmt, idtvar,
- * *****************
- *                    ntflsh, wtflsh, nprb,   lstprb, xyzprb,
- *                    lnam,   lpre)
- *
- * integer          tplnum      : <-- : number of plot to create (> 0)
- * character        tplnam      : <-- : name of associated plot
- * character        tplpre      : <-- : prefix for associated file
- * integer          tplfmt      : <-- : associated format
- *                                      (1: dat, 2: csv, 3: both)
- * integer          idtvar      : <-- : calculation time dependency
- * integer          ntflsh      : <-- : file write every ntflsh output
- *                                      time steps if > 0 (file kept
- *                                      open otherwise)
- * integer          wtflsh      : <-- : file flush forced every wtflsh
- *                                      elapsed seconds if > 0
- * integer          nprb        : <-- : number of probes
- * integer          lstprb      : <-- : list of probes (1 to n)
- * double precision xyzprb      : <-- : probe coordinates
- * integer          lnam        : <-- : name length
- * integer          lpre        : <-- : prefix length
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (tppini, TPPINI)
-(
- const cs_int_t  *tplnum,
- const char      *tplnam,
- const char      *tplpre,
- const cs_int_t  *tplfmt,
- const cs_int_t  *idtvar,
- const cs_int_t  *ntflsh,
- const cs_real_t *wtflsh,
- const cs_int_t  *nprb,
- const cs_int_t  *lstprb,
- const cs_real_t *xyzprb,
- const cs_int_t  *lnam,
- const cs_int_t  *lpre
- CS_ARGF_SUPP_CHAINE              /*     (possible 'length' arguments added
-                                         by many Fortran compilers) */
-)
-{
-  /* Copy Fortran strings to C strings */
-
-  cs_time_plot_format_t fmt;
-  char *plot_name   = cs_base_string_f_to_c_create(tplnam, *lnam);
-  char *file_prefix = cs_base_string_f_to_c_create(tplpre, *lpre);
-  bool use_iteration = false;
-
-  if (*idtvar < 0 || *idtvar == 2)
-    use_iteration = true;
-
-  /* Main processing */
-
-  for (fmt = CS_TIME_PLOT_DAT; fmt <= CS_TIME_PLOT_CSV; fmt++) {
-
-    int fmt_mask = fmt + 1;
-
-    if (*tplfmt & fmt_mask) {
-
-      _fortran_time_plot_realloc(*tplnum, plot_name, fmt);
-
-      _plot_files[fmt][*tplnum - 1] = cs_time_plot_init_probe(plot_name,
-                                                              file_prefix,
-                                                              fmt,
-                                                              use_iteration,
-                                                              *wtflsh,
-                                                              *ntflsh,
-                                                              *nprb,
-                                                              lstprb,
-                                                              xyzprb);
-    }
-
-  }
-
-  /* Free temporary C strings */
-
-  cs_base_string_f_to_c_free(&plot_name);
-  cs_base_string_f_to_c_free(&file_prefix);
-}
-
 /*----------------------------------------------------------------------------
  * Create a writer for time plot structure-type data.
  *
@@ -782,8 +792,7 @@ void CS_PROCF (tppini, TPPINI)
  *
  * subroutine tpsini (tplnum, tplnam, tplpre, tplfmt, idtvar,
  * *****************
- *                    ntflsh, wtflsh, nprb,   lstprb, xyzprb,
- *                    lnam,   lpre)
+ *                    nprb,   lstprb, xyzprb, lnam,   lpre)
  *
  * integer          tplnum      : <-- : number of plot to create (> 0)
  * character        tplnam      : <-- : name of associated plot
@@ -791,11 +800,6 @@ void CS_PROCF (tppini, TPPINI)
  * integer          tplfmt      : <-- : associated format
  *                                      (1: dat, 2: csv, 3: both)
  * integer          idtvar      : <-- : calculation time dependency
- * integer          ntflsh      : <-- : file write every ntflsh output
- *                                      time steps if > 0 (file kept
- *                                      open otherwise)
- * integer          wtflsh      : <-- : file flush forced every wtflsh
- *                                      elapsed seconds if > 0
  * integer          nstru       : <-- : number of structures
  * double precision xmstru      : <-- : mass matrixes
  * double precision xcstru      : <-- : damping matrixes
@@ -811,8 +815,6 @@ void CS_PROCF (tpsini, TPSINI)
  const char      *tplpre,
  const cs_int_t  *tplfmt,
  const cs_int_t  *idtvar,
- const cs_int_t  *ntflsh,
- const cs_real_t *wtflsh,
  const cs_int_t  *nstru,
  const cs_real_t *xmstru,
  const cs_real_t *xcstru,
@@ -843,16 +845,17 @@ void CS_PROCF (tpsini, TPSINI)
 
       _fortran_time_plot_realloc(*tplnum, plot_name, fmt);
 
-      _plot_files[fmt][*tplnum - 1] = cs_time_plot_init_struct(plot_name,
-                                                               file_prefix,
-                                                               fmt,
-                                                               use_iteration,
-                                                               *wtflsh,
-                                                               *ntflsh,
-                                                               *nstru,
-                                                               xmstru,
-                                                               xcstru,
-                                                               xkstru);
+      _plot_files[fmt][*tplnum - 1]
+        = cs_time_plot_init_struct(plot_name,
+                                   file_prefix,
+                                   fmt,
+                                   use_iteration,
+                                   _flush_wtime_default,
+                                   _n_buffer_steps_default,
+                                   *nstru,
+                                   xmstru,
+                                   xcstru,
+                                   xkstru);
     }
 
   }
@@ -892,7 +895,7 @@ void CS_PROCF (tplend, TPLEND)
     if (*tplfmt & fmt_mask) {
 
       if (*tplnum <= 0 || *tplnum > (int)_n_files_max[fmt])
-        bft_error(__FILE__, __LINE__, errno,
+        bft_error(__FILE__, __LINE__, 0,
                   _("Plot number must be in the interval [1, %d] and not %d."),
                   (int)(_n_files_max[fmt]), *tplnum);
 
@@ -980,7 +983,7 @@ void CS_PROCF (tplnbr, TPLNBR)
   *ntpl = 0;
 
   for (fmt = CS_TIME_PLOT_DAT; fmt <= CS_TIME_PLOT_CSV; fmt++) {
-    if (_n_files_max[fmt] > *ntpl)
+    if (_n_files_max[fmt] > (size_t)(*ntpl))
       *ntpl = _n_files_max[fmt];
   }
 }
@@ -1007,6 +1010,7 @@ void CS_PROCF (tplnbr, TPLNBR)
  *   n_probes         <-- number of probes associated with this plot
  *   probe_list       <-- numbers (1 to n) of probes if filtered, or NULL
  *   probe_coords     <-- probe coordinates, or NULL
+ *   probe_names      <-- probe names, or NULL
  *
  * returns:
  *   pointer to new time plot writer
@@ -1021,7 +1025,8 @@ cs_time_plot_init_probe(const char             *plot_name,
                         int                     n_buffer_steps,
                         int                     n_probes,
                         const int              *probe_list,
-                        const cs_real_t         probe_coords[])
+                        const cs_real_t         probe_coords[],
+                        const char             *probe_names[])
 {
   cs_time_plot_t  *p = _plot_file_create(plot_name,
                                          file_prefix,
@@ -1032,7 +1037,7 @@ cs_time_plot_init_probe(const char             *plot_name,
 
   switch (format) {
   case CS_TIME_PLOT_DAT:
-    _write_probe_header_dat(p, n_probes, probe_list, probe_coords);
+    _write_probe_header_dat(p, n_probes, probe_list, probe_coords, probe_names);
     break;
   case CS_TIME_PLOT_CSV:
     _write_probe_coords_csv(file_prefix,
@@ -1040,12 +1045,16 @@ cs_time_plot_init_probe(const char             *plot_name,
                             n_probes,
                             probe_list,
                             probe_coords);
-    _write_probe_header_csv(p, n_probes, probe_list, probe_coords);
+    _write_probe_header_csv(p, n_probes, probe_list, probe_coords, probe_names);
     break;
   default:
     break;
   }
 
+  cs_time_plot_flush(p); /* Flush after creation so plot tools can start
+                            analyzing file immediately, while computation
+                            is running */
+
   return p;
 }
 
@@ -1123,6 +1132,8 @@ cs_time_plot_finalize(cs_time_plot_t  **p)
 
     cs_time_plot_t *_p = *p;
 
+    _time_plot_unregister(_p);
+
     if (_p->buffer_steps[0] > 0)
       _p->buffer_steps[1] = _p->buffer_steps[0] + 1;
 
@@ -1216,6 +1227,81 @@ cs_time_plot_vals_write(cs_time_plot_t  *p,
   _plot_file_check_or_write(p);
 }
 
+/*----------------------------------------------------------------------------
+ * Flush buffered values to file if applicable
+ *
+ * parameters:
+ *   p <-> time plot values file handler
+ *----------------------------------------------------------------------------*/
+
+void
+cs_time_plot_flush(cs_time_plot_t  *p)
+{
+  /* Force buffered variant output */
+
+  if (p->buffer_end > 0) {
+    if (p->buffer_steps[0] > 0)
+      p->buffer_steps[1] = p->buffer_steps[0];
+    _plot_file_check_or_write(p);
+  }
+
+  if (p->f != NULL) {
+    if (p->flush_times[0] > 0)
+      p->flush_times[1] = cs_timer_wtime();
+    fflush(p->f);
+  }
+}
+
+/*----------------------------------------------------------------------------
+ * flush all time plots
+ *----------------------------------------------------------------------------*/
+
+void
+cs_time_plot_flush_all(void)
+{
+  for (cs_time_plot_t *p = _plots_head; p != NULL; p = p->next)
+    cs_time_plot_flush(p);
+}
+
+/*----------------------------------------------------------------------------
+ * Set time plot file writer flush behavior defaults.
+ *
+ * parameters:
+ *   flush_wtime     <-- elapsed time interval between file flushes;
+ *                       if < 0, no forced flush
+ *   n_buffer_steps  <-- number of time steps in output buffer if
+ *                       file is not to be kept open
+ *----------------------------------------------------------------------------*/
+
+void
+cs_time_plot_set_flush_default(float  flush_wtime,
+                               int    n_buffer_steps)
+{
+  _flush_wtime_default    = flush_wtime;
+  _n_buffer_steps_default = n_buffer_steps;
+}
+
+/*----------------------------------------------------------------------------
+ * Return time plot file writer flush behavior defaults.
+ *
+ * parameters:
+ *   flush_wtime     --> elapsed time interval between file flushes;
+ *                       if < 0, no forced flush (NULL if not queried)
+ *   n_buffer_steps  <-- number of time steps in output buffer if
+ *                       file is not to be kept open (NULL if not queried)
+ *----------------------------------------------------------------------------*/
+
+void
+cs_time_plot_get_flush_default(float  *flush_wtime,
+                               int    *n_buffer_steps)
+{
+  if (flush_wtime != NULL)
+    *flush_wtime = _flush_wtime_default;
+
+  if (n_buffer_steps != NULL)
+    *n_buffer_steps = _n_buffer_steps_default;
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
