@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -80,6 +80,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*-----------------------------------------------------------------------------
  * Local type definitions
  *-----------------------------------------------------------------------------*/
@@ -177,7 +179,7 @@ _cs_timer_cpu_stdc_clock(cs_timer_t  *timer);
  * Local static variable definitions
  *-----------------------------------------------------------------------------*/
 
-static _Bool _cs_timer_initialized = false;
+static bool  _cs_timer_initialized = false;
 static cs_timer_method_t _cs_timer_wall_method = CS_TIMER_DISABLE;
 static cs_timer_method_t _cs_timer_cpu_method = CS_TIMER_DISABLE;
 static _cs_timer_wall_func_t *_cs_timer_wall = _cs_timer_wall_null;
@@ -213,6 +215,7 @@ static cs_timer_t  _cs_timer_start;
 void
 _cs_timer_wall_null(cs_timer_t  *timer)
 {
+  CS_UNUSED(timer);
 }
 
 #if defined (HAVE_CLOCK_GETTIME)
@@ -283,6 +286,7 @@ _cs_timer_wall_stdc_time(cs_timer_t  *timer)
 void
 _cs_timer_cpu_null(cs_timer_t  *timer)
 {
+  CS_UNUSED(timer);
 }
 
 #if defined (HAVE_CLOCK_GETTIME)
@@ -389,6 +393,7 @@ _cs_timer_initialize(void)
     }
   }
 
+#if defined (HAVE_CLOCK_GETCPUCLOCKID)
   if (_cs_timer_cpu_method == CS_TIMER_DISABLE) {
     clockid_t clock_id;
     if (clock_getcpuclockid(0, &clock_id) == 0) {
@@ -400,6 +405,7 @@ _cs_timer_initialize(void)
       }
     }
   }
+#endif
 
 #endif
 
@@ -464,6 +470,8 @@ _cs_timer_initialize(void)
   _cs_timer_initialized = true;
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -480,8 +488,7 @@ _cs_timer_initialize(void)
 double
 cs_timer_wtime(void)
 {
-  cs_timer_t time_current;
-  cs_timer_counter_t time_diff;
+  cs_timer_t t1;
 
   /* Ensure initialization */
 
@@ -490,11 +497,13 @@ cs_timer_wtime(void)
 
   /* Compute elapsed time */
 
-  _cs_timer_wall(&time_current);
+  _cs_timer_wall(&t1);
 
-  time_diff = cs_timer_diff(&_cs_timer_start, &time_current);
+  long long wall_nsec
+    =  (t1.wall_sec - _cs_timer_start.wall_sec) * (long long)1000000000
+      + t1.wall_nsec - _cs_timer_start.wall_nsec;
 
-  return time_diff.wall_nsec*1.e-9;
+  return wall_nsec*1.e-9;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -515,8 +524,7 @@ cs_timer_wtime(void)
 double
 cs_timer_cpu_time(void)
 {
-  cs_timer_t time_current;
-  cs_timer_counter_t time_diff;
+  cs_timer_t t1;
 
   /* Ensure initialization */
 
@@ -525,11 +533,13 @@ cs_timer_cpu_time(void)
 
   /* Compute CPU time */
 
-  _cs_timer_cpu(&time_current);
+  _cs_timer_cpu(&t1);
 
-  time_diff = cs_timer_diff(&_cs_timer_start, &time_current);
+  long long cpu_nsec
+    =  (t1.cpu_sec - _cs_timer_start.cpu_sec) * (long long)1000000000
+       + t1.cpu_nsec - _cs_timer_start.cpu_nsec;
 
-  return time_diff.cpu_nsec*1.e-9;
+  return cpu_nsec*1.e-9;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -636,27 +646,6 @@ cs_timer_diff(const cs_timer_t  *t0,
   return retval;
 }
 
-/*----------------------------------------------------------------------------*/
-/*!
- * \brief Add the the difference between 2 timers to a counter.
- *
- * \param[in, out]  tc  pointer to timer counter
- * \param[in]       t0  oldest timer value
- * \param[in]       t1 most recent timer value
- */
-/*----------------------------------------------------------------------------*/
-
-void
-cs_timer_counter_add_diff(cs_timer_counter_t  *tc,
-                          const cs_timer_t    *t0,
-                          const cs_timer_t    *t1)
-{
-  tc->wall_nsec +=    (t1->wall_sec - t0->wall_sec) * (long long)1000000000
-                     + t1->wall_nsec - t0->wall_nsec;
-  tc->cpu_nsec +=    (t1->cpu_sec - t0->cpu_sec) * (long long)1000000000
-                    + t1->cpu_nsec - t0->cpu_nsec;
-}
-
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Return method used to return wall clock time.
