@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,12 @@
 
 #include "cs_defs.h"
 
+/*----------------------------------------------------------------------------
+ * Local headers
+ *----------------------------------------------------------------------------*/
+
+#include "cs_restart.h"
+
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
@@ -81,15 +87,17 @@ cs_turbomachinery_get_model(void);
  * Define a rotor by its axis and cell selection criteria.
  *
  * parameters:
- *   cell_criteria     <-- cell selection criteria string
- *   rotation_velocity <-- rotation velocity, in radians/second
- *   rotation_axis     <-- rotation axis vector
+ *   cell_criteria         <-- cell selection criteria string
+ *   rotation_velocity     <-- rotation velocity, in radians/second
+ *   rotation_axis         <-- rotation axis vector
+ *   rotation_invariant    <-- rotation invariant point
  *----------------------------------------------------------------------------*/
 
 void
 cs_turbomachinery_add_rotor(const char    *cell_criteria,
                             double         rotation_velocity,
-                            const double   rotation_axis[3]);
+                            const double   rotation_axis[3],
+                            const double   rotation_invariant[3]);
 
 /*----------------------------------------------------------------------------
  * Add a cs_join_t structure to the list of rotor/stator joinings.
@@ -112,10 +120,34 @@ cs_turbomachinery_join_add(const char  *sel_criteria,
                            int          verbosity,
                            int          visualization);
 
+/*----------------------------------------------------------------------------
+ * Add a cs_join_t structure to the list of rotor/stator couplings.
+ *
+ * parameters:
+ *   sel_criteria <-- boundary face selection criteria
+ *   tolerance    <-- value of the search tolerance
+ *   verbosity    <-- level of verbosity required
+ *
+ * returns:
+ *   number (1 to n) associated with new coupling
+ *----------------------------------------------------------------------------*/
+
+int
+cs_turbomachinery_coupling_add(const char  *sel_criteria,
+                               float        tolerance,
+                               int          verbosity);
+
+/*----------------------------------------------------------------------------
+ * Definitions for turbomachinery computation.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_define(void);
+
 /*----------------------------------------------------------------------------
  * Initializations for turbomachinery computation
  *
- * Note: this function should be called before once the mesh is built,
+ * Note: this function should be called after the mesh is built,
  *       but before cs_post_init_meshes() so that postprocessing meshes are
  *       updated correctly in the transient case.
  *----------------------------------------------------------------------------*/
@@ -134,14 +166,23 @@ cs_turbomachinery_finalize(void);
  * Update mesh for unsteady rotor/stator computation
  *
  * parameters:
- *   t_cur_mob <-- current rotor time
- *   t_elapsed --> elapsed computation time
+ *   t_cur_mob    <-- current rotor time
+ *   t_elapsed    --> elapsed computation time
  *----------------------------------------------------------------------------*/
 
 void
 cs_turbomachinery_update_mesh(double   t_cur_mob,
                               double  *t_elapsed);
 
+/*----------------------------------------------------------------------------
+ * Update mesh for unsteady rotor/stator computation in case of restart.
+ *
+ * Reads mesh from checkpoint when available.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_restart_mesh(void);
+
 /*----------------------------------------------------------------------------
  * Reinitialize interior face-based fields.
  *----------------------------------------------------------------------------*/
@@ -172,6 +213,19 @@ cs_turbomachinery_rotation_matrix(int        rotor_num,
                                   double     theta,
                                   cs_real_t  matrix[3][4]);
 
+/*----------------------------------------------------------------------------
+ * Return number of rotors.
+ *
+ * Note that the number of associated rotations is n_rotors + 1, as the
+ * first rotation id is reserved for the fixed portion of the domain.
+ *
+ * return:
+ *   number of rotors
+ *----------------------------------------------------------------------------*/
+
+int
+cs_turbomachinery_n_rotors(void);
+
 /*----------------------------------------------------------------------------
  * Return cell rotor number.
  *
@@ -196,6 +250,52 @@ cs_turbomachinery_get_cell_rotor_num(void);
 double
 cs_turbomachinery_get_rotation_velocity(int  rotor_num);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set rotation velocity
+ *
+ * param[in]  rotor_num  rotor number (1 to n numbering)
+ * param[in]  omega      rotation velocity
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_set_rotation_velocity(int     rotor_num,
+                                        double  omega);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Build rotation matrices for a given time interval.
+ *
+ * The caller is responsible for freeing the array when not needed.
+ *
+ * \param[in]  dt  associated time delta (0 for current, unmodified time)
+ *
+ * \return  array of rotation matrices.
+ */
+/*----------------------------------------------------------------------------*/
+
+cs_real_34_t *
+cs_turbomachinery_get_rotation_matrices(double dt);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Set turbomachinery joining retry parameters.
+ *
+ * When a joing leads to a different number of boundary faces from the
+ * previous position, the rotor positions may be perturbed by a small
+ * quantity to try to obtain a better joining.
+ *
+ * param[in]  n_max_join_retries   maximum number of retries before considering
+ *                                 the joining has failed
+ * param[in]  dt_retry_multiplier  time step multiplier for new position retry
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_set_rotation_retry(int     n_max_join_retries,
+                                     double  dt_retry_multiplier);
+
 /*----------------------------------------------------------------------------
  * Rotation of vector and tensor fields.
  *
@@ -209,6 +309,10 @@ cs_turbomachinery_rotate_fields(const cs_real_t dt[]);
 /*----------------------------------------------------------------------------
  * Compute velocity relative to fixed coordinates at a given point
  *
+ * Deprecated:
+ * Use cs_rotation_velocity for more consistent naming of this reference
+ * frame velocity.
+ *
  * parameters:
  *   rotor_num <-- associated rotor number (1 to n numbering)
  *   coords    <-- point coordinates
@@ -220,6 +324,32 @@ cs_turbomachinery_relative_velocity(int              rotor_num,
                                     const cs_real_t  coords[3],
                                     cs_real_t        velocity[3]);
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Read turbomachinery metadata from restart file.
+ *
+ * The mesh is handled separately.
+ *
+ * \param[in, out]  r  associated restart file pointer
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_restart_read(cs_restart_t  *r);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Write turbomachinery metadata to checkpoint file.
+ *
+ * The mesh is handled separately.
+ *
+ * \param[in, out]  r  associated restart file pointer
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_turbomachinery_restart_write(cs_restart_t  *r);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
