@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -42,7 +42,7 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
@@ -52,14 +52,12 @@
 !> \param[in]     lfbncp
 !> \param[out]    itypfb        boundary face types
 !> \param[in]     dt            time step (per cell)
-!> \param[in]     rtp           calculated variables at cell centers
-!>                              (at current time step)
 !> \param[out]    rcodcl        boundary condition values:
 !>                               - rcodcl(1) value of the dirichlet
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradv \vect{u} \cdot \vect{n}  \f$
@@ -77,7 +75,7 @@ subroutine csc2cl &
  ( nvcp   , nvcpto , nfbcpl , nfbncp ,                            &
    icodcl , itypfb ,                                              &
    lfbcpl , lfbncp ,                                              &
-   dt     , rtp    ,                                              &
+   dt     ,                                                       &
    rcodcl ,                                                       &
    rvcpfb , pndcpl , dofcpl )
 
@@ -108,18 +106,18 @@ implicit none
 integer          nvcp   , nvcpto
 integer          nfbcpl , nfbncp
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          lfbcpl(nfbcpl)  , lfbncp(nfbncp)
 integer          itypfb(nfabor)
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision dt(ncelet)
+double precision rcodcl(nfabor,nvar,3)
 double precision rvcpfb(nfbcpl,nvcpto), pndcpl(nfbcpl)
 double precision dofcpl(3,nfbcpl)
 
 ! Local variables
 
-integer          ifac, iel, isou
+integer          ifac, iel, isou, icscp
 integer          inc, iccocg, iprev
 integer          ivar, iflmab
 integer          ipt
@@ -136,6 +134,7 @@ double precision, allocatable, dimension(:,:) :: grad
 
 double precision, dimension(:), pointer :: bmasfl
 double precision, dimension(:,:), pointer :: vel
+double precision, dimension(:), pointer :: cvar_var
 
 !===============================================================================
 
@@ -146,6 +145,12 @@ call field_get_val_v(ivarfl(iu), vel)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
 call field_get_val_s(iflmab, bmasfl)
 
+if (ifaccp.eq.0) then
+  icscp = icscpl
+else
+  icscp = icscpd
+endif
+
 !===============================================================================
 ! 1.  Translation of the coupling to boundary conditions
 !===============================================================================
@@ -171,6 +176,7 @@ do ivar = 1, nvcp
 
   if (ivar.ne.iu .and. ivar.ne.iv .and. ivar.ne.iw) then
 
+    call field_get_val_s(ivarfl(ivar), cvar_var)
     call field_gradient_scalar(ivarfl(ivar), iprev, imrgra, inc,   &
                                iccocg,                             &
                                grad)
@@ -217,7 +223,7 @@ do ivar = 1, nvcp
         !     the pressure gradient through the coupling and remain consistent
         !     with the resolution of the pressure gradient on an orthogonal mesh.
 
-        xip = rtp(iel,ivar) &
+        xip = cvar_var(iel) &
             + (grad(1,iel)*xiip + grad(2,iel)*yiip + grad(3,iel)*ziip)
 
       else if (ivar.eq.iu.or.ivar.eq.iv.or.ivar.eq.iw) then
@@ -232,11 +238,11 @@ do ivar = 1, nvcp
 
         ! -- UPWIND
 
-        !        xip =  rtp(iel,ivar)
+        !        xip =  cvar_var(iel)
 
         ! -- SOLU
 
-        !        xip =   rtp(iel,ivar)                &
+        !        xip =   cvar_var(iel)                &
         !              + (   gradv(1,isou,iel)*xif    &
         !                 +  gradv(2,isou,iel)*yif    &
         !                 +  gradv(3,isou,iel)*zif)
@@ -252,16 +258,16 @@ do ivar = 1, nvcp
 
         ! -- UPWIND
 
-        !        xip =  rtp(iel,ivar)
+        !        xip =  cvar_var(iel)
 
         ! -- SOLU
 
-        !        xip = rtp(iel,ivar) &
+        !        xip = cvar_var(iel) &
         !            + (grad(1,iel)*xif + grad(2,iel)*yif + grad(3,iel)*zif)
 
         ! -- CENTERED
 
-        xip =  rtp(iel,ivar) &
+        xip =  cvar_var(iel) &
             + (grad(1,iel)*xiip + grad(2,iel)*yiip + grad(3,iel)*ziip)
 
       endif
@@ -274,8 +280,7 @@ do ivar = 1, nvcp
       ! Information received from distant instance at J'/O'
       xjp = rvcpfb(ipt,ivar)
 
-
-      itypfb(ifac)  = icscpl
+      itypfb(ifac) = icscp
 
       if (ivar.eq.ipr) then
 
@@ -367,7 +372,7 @@ do ivar = 1, nvcp
 
       else
 
-        xip =   rtp(iel,ivar)            &
+        xip =   cvar_var(iel)            &
               + grad(1,iel)*(xiip+xopf)  &
               + grad(2,iel)*(yiip+yopf)  &
               + grad(3,iel)*(ziip+zopf)
@@ -379,8 +384,7 @@ do ivar = 1, nvcp
       ! Information received from distant instance at J'/O'
       xjp = rvcpfb(ipt,ivar)
 
-
-      itypfb(ifac)  = icscpl
+      itypfb(ifac) = icscp
 
       if (ivar.ne.ipr) then
         icodcl(ifac,ivar  ) = 1
@@ -402,7 +406,7 @@ do ivar = 1, nvcp
 
     ifac = lfbncp(ipt)
 
-    itypfb(ifac)  = icscpl
+    itypfb(ifac) = icscp
 
     icodcl(ifac,ivar  ) = 3
     rcodcl(ifac,ivar,3) = 0.d0
@@ -425,3 +429,142 @@ deallocate(grad)
 
 return
 end subroutine csc2cl
+
+!===============================================================================
+
+!> \brief Initialization of the "itypfb(*, *) = icscpl" condition.
+
+!------------------------------------------------------------------------------
+! Arguments
+!------------------------------------------------------------------------------
+!   mode          name          role
+!------------------------------------------------------------------------------
+!> \param[in]     nvcp
+!> \param[in]     nfbcpl
+!> \param[in]     nfbncp
+!> \param[out]    icodcl        face boundary condition code:
+!>                               - 1 Dirichlet
+!>                               - 2 Radiative outlet
+!>                               - 3 Neumann
+!>                               - 4 sliding and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 5 smooth wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 6 rough wall and
+!>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
+!>                               - 9 free inlet/outlet
+!>                                 (input mass flux blocked to 0)
+!>                               - 13 Dirichlet for the advection operator and
+!>                                    Neumann for the diffusion operator
+!> \param[in]     lfbcpl
+!> \param[in]     lfbncp
+!> \param[out]    itypfb        boundary face types
+!______________________________________________________________________________
+
+subroutine csc2cl_init &
+ (nvcp, nfbcpl, nfbncp, icodcl, itypfb, lfbcpl, lfbncp)
+
+!===============================================================================
+! Module files
+!===============================================================================
+
+use paramx
+use numvar
+use optcal
+use cstphy
+use cstnum
+use dimens, only: nvar
+use entsor
+use parall
+use period
+use cplsat
+use mesh
+use field
+use field_operator
+
+!===============================================================================
+
+implicit none
+
+! Arguments
+
+integer          nvcp
+integer          nfbcpl , nfbncp
+
+integer          icodcl(nfabor,nvar)
+integer          lfbcpl(nfbcpl)  , lfbncp(nfbncp)
+integer          itypfb(nfabor)
+
+! Local variables
+
+integer          ifac, icscp
+integer          ivar
+integer          ipt
+
+!===============================================================================
+
+!===============================================================================
+! 1.  Translation of the coupling to boundary conditions
+!===============================================================================
+
+if (ifaccp.eq.0) then
+  icscp = icscpl
+else
+  icscp = icscpd
+endif
+
+! Reminder: variables are received in the order of VARPOS;
+! looping on variables is thus sufficient.
+
+do ivar = 1, nvcp
+
+  if (ifaccp.eq.1) then
+
+    do ipt = 1, nfbcpl
+
+      ifac = lfbcpl(ipt)
+
+      itypfb(ifac) = icscp
+      icodcl(ifac,ivar  ) = 1
+
+    enddo
+
+  else
+
+    do ipt = 1, nfbcpl
+
+      ifac = lfbcpl(ipt)
+
+      itypfb(ifac) = icscp
+
+      if (ivar.ne.ipr) then
+        icodcl(ifac,ivar  ) = 1
+      else
+        icodcl(ifac,ivar  ) = 3
+      endif
+
+    enddo
+
+  endif
+
+  ! Non-located boundary faces
+
+  do ipt = 1, nfbncp
+
+    ifac = lfbncp(ipt)
+
+    itypfb(ifac) = icscp
+
+    icodcl(ifac,ivar  ) = 3
+
+  enddo
+
+enddo
+
+!----
+! End
+!----
+
+return
+end subroutine csc2cl_init
+
