@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,8 +36,6 @@
 !> \param[in]     numcpl
 !> \param[in]     nvcpto
 !> \param[in]     locpts
-!> \param[in]     rtp           calculated variables at cell centers
-!>                              (at current and previous time steps)
 !> \param[in]     coopts
 !> \param[in]     djppts
 !> \param[in]     pndpts
@@ -49,7 +47,6 @@ subroutine cscpfb &
  ( nscal  ,                                                       &
    nptdis , numcpl , nvcpto,                                      &
    locpts ,                                                       &
-   rtp    ,                                                       &
    coopts , djppts , pndpts ,                                     &
    rvdis  , dofpts )
 
@@ -61,13 +58,13 @@ use paramx
 use pointe
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use entsor
 use parall
 use period
 use cplsat
+use rotation
 use mesh
 use field
 use field_operator
@@ -83,7 +80,6 @@ integer          nptdis , numcpl , nvcpto
 
 integer          locpts(nptdis)
 
-double precision rtp(ncelet,nflown:nvar)
 double precision coopts(3,nptdis), djppts(3,nptdis)
 double precision pndpts(nptdis), dofpts(3,nptdis)
 double precision rvdis(nptdis,nvcpto)
@@ -99,23 +95,40 @@ integer          itytu0
 double precision xjjp   , yjjp   , zjjp
 double precision d2s3
 double precision xjpf,yjpf,zjpf,jpf
-double precision xx, yy, zz
-double precision omegal(3), omegad(3), omegar(3), omgnrl, omgnrd, omgnrr
-double precision vitent, daxis2
 
-double precision, allocatable, dimension(:,:,:) :: gradv
+double precision, allocatable, dimension(:,:,:) :: gradv, gradts
 double precision, allocatable, dimension(:,:) :: grad
 double precision, allocatable, dimension(:) :: trav1, trav2, trav3, trav4
 double precision, allocatable, dimension(:) :: trav5, trav6, trav7, trav8
 
 double precision, dimension(:), pointer :: crom
-double precision, dimension(:,:), pointer :: vel
+double precision, dimension(:,:), pointer :: vel, cvar_rij
+double precision, dimension(:), pointer :: cvar_pr, cvar_k, cvar_ep
+double precision, dimension(:), pointer :: cvar_phi, cvar_fb, cvar_omg
+double precision, dimension(:), pointer :: cvar_var, cvar_scal
 
 !===============================================================================
 
 ! Map field arrays
 call field_get_val_v(ivarfl(iu), vel)
 
+call field_get_val_s(ivarfl(ipr), cvar_pr)
+
+if (itytur.eq.2) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+else if (itytur.eq.3) then
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+else if (iturb.eq.50) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+  call field_get_val_s(ivarfl(iphi), cvar_phi)
+  call field_get_val_s(ivarfl(ifb), cvar_fb)
+else if (iturb.eq.60) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iomg), cvar_omg)
+endif
+
 !=========================================================================
 ! 1.  INITIALISATIONS
 !=========================================================================
@@ -126,10 +139,6 @@ iprev  = 0
 inc    = 1
 iccocg = 1
 
-! Initialize variables to avoid compiler warnings
-
-vitent = 0.d0
-
 ! Allocate temporary arrays
 
 allocate(grad(3,ncelet))
@@ -148,46 +157,13 @@ d2s3 = 2.d0/3.d0
 
 call field_get_val_s(icrom, crom)
 
-if (icormx(numcpl).eq.1) then
-
-  ! On rcupre dans tous les cas le vecteur rotation de l'autre instance
-  omegal(1) = omegax
-  omegal(2) = omegay
-  omegal(3) = omegaz
-  call tbrcpl(numcpl,3,3,omegal,omegad)
-
-  ! Vecteur vitesse relatif d'une instance a l'autre
-  omegar(1) = omegal(1) - omegad(1)
-  omegar(2) = omegal(2) - omegad(2)
-  omegar(3) = omegal(3) - omegad(3)
-
-  omgnrl = sqrt(omegal(1)**2 + omegal(2)**2 + omegal(3)**2)
-  omgnrd = sqrt(omegad(1)**2 + omegad(2)**2 + omegad(3)**2)
-  omgnrr = sqrt(omegar(1)**2 + omegar(2)**2 + omegar(3)**2)
-
-else
-
-  omegal(1) = 0.d0
-  omegal(2) = 0.d0
-  omegal(3) = 0.d0
-
-  omegar(1) = 0.d0
-  omegar(2) = 0.d0
-  omegar(3) = 0.d0
-
-  omgnrl = 0.d0
-  omgnrd = 0.d0
-  omgnrr = 0.d0
-
-endif
-
 ! On part du principe que l'on envoie les bonnes variables 
 ! l'instance distante et uniquement celles-l.
 
 ! De plus, les variables sont envoyes dans l'ordre:
 
-!     - pression (unique pour toute les phases)
 !     - vitesse
+!     - pression
 !     - grandeurs turbulentes (selon le modle)
 !   Et ensuite :
 !     - scalaires physique particulire (pas encore traits)
@@ -195,87 +171,10 @@ endif
 !     - vitesse de maillage (non couple, donc non envoye)
 
 
-ipos = 1
+ipos = 0
 
 !=========================================================================
-! 1.  Prepare for pressure
-!=========================================================================
-
-call field_gradient_scalar(ivarfl(ipr), iprev, imrgra, inc,  &
-                           iccocg,                           &
-                           grad)
-
-! For a specific face to face coupling, geometric assumptions are made
-
-if (ifaccp.eq.1) then
-
-  do ipt = 1, nptdis
-
-    iel = locpts(ipt)
-
-    ! --- Pour la pression on veut imposer un dirichlet tel que le gradient
-    !     de pression se conserve entre les deux domaines coupls Pour cela
-    !     on impose une interpolation centre
-
-    xjjp = djppts(1,ipt)
-    yjjp = djppts(2,ipt)
-    zjjp = djppts(3,ipt)
-
-    rvdis(ipt,ipos) = rtp(iel,ipr) &
-         + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    ! On prend en compte le potentiel centrifuge en repre relatif
-    if (icormx(numcpl).eq.1) then
-
-      ! Calcul de la distance a l'axe de rotation
-      ! On suppose que les axes sont confondus...
-
-      xx = xyzcen(1,iel) + xjjp
-      yy = xyzcen(2,iel) + yjjp
-      zz = xyzcen(3,iel) + zjjp
-
-      daxis2 =   (omegar(2)*zz - omegar(3)*yy)**2 &
-           + (omegar(3)*xx - omegar(1)*zz)**2 &
-           + (omegar(1)*yy - omegar(2)*xx)**2
-
-      daxis2 = daxis2 / omgnrr**2
-
-      rvdis(ipt,ipos) = rvdis(ipt,ipos)                         &
-           + 0.5d0*crom(iel)*(omgnrl**2 - omgnrd**2)*daxis2
-
-    endif
-
-  enddo
-
-  ! For a generic coupling, no assumption can be made
-
-else
-
-  do ipt = 1, nptdis
-
-    iel = locpts(ipt)
-
-    xjpf = coopts(1,ipt) - xyzcen(1,iel)- djppts(1,ipt)
-    yjpf = coopts(2,ipt) - xyzcen(2,iel)- djppts(2,ipt)
-    zjpf = coopts(3,ipt) - xyzcen(3,iel)- djppts(3,ipt)
-
-    if (pndpts(ipt).ge.0.d0.and.pndpts(ipt).le.1.d0) then
-      jpf = -1.d0*sqrt(xjpf**2+yjpf**2+zjpf**2)
-    else
-      jpf =       sqrt(xjpf**2+yjpf**2+zjpf**2)
-    endif
-
-    rvdis(ipt,ipos) = (xjpf*grad(1,iel)+yjpf*grad(2,iel)+zjpf*grad(3,iel))  &
-         /jpf
-
-  enddo
-
-endif
-!       FIn pour la pression
-
-
-!=========================================================================
-! 2.  Prepare for velocity
+! 1.  Prepare for velocity
 !=========================================================================
 
 call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc,  &
@@ -328,24 +227,6 @@ do isou = 1, 3
                         + yjjp*gradv(2,isou,iel)   &
                         + zjjp*gradv(3,isou,iel)
 
-      ! On prend en compte la vitesse d'entrainement en repre relatif
-      if (icormx(numcpl).eq.1) then
-
-        if (isou.eq.1) then
-          vitent =   omegar(2)*(xyzcen(3,iel)+zjjp) &
-               - omegar(3)*(xyzcen(2,iel)+yjjp)
-        elseif (isou.eq.2) then
-          vitent =   omegar(3)*(xyzcen(1,iel)+xjjp) &
-               - omegar(1)*(xyzcen(3,iel)+zjjp)
-        elseif (isou.eq.3) then
-          vitent =   omegar(1)*(xyzcen(2,iel)+yjjp) &
-               - omegar(2)*(xyzcen(1,iel)+xjjp)
-        endif
-
-        rvdis(ipt,ipos) = rvdis(ipt,ipos) + vitent
-
-      endif
-
     enddo
 
     ! For a generic coupling, no assumption can be made
@@ -372,6 +253,63 @@ do isou = 1, 3
 enddo
 !       Fin de la boucle sur les composantes de la vitesse
 
+!=========================================================================
+! 2.  Prepare for pressure
+!=========================================================================
+
+ipos = ipos + 1
+
+call field_gradient_scalar(ivarfl(ipr), iprev, imrgra, inc,  &
+                           iccocg,                           &
+                           grad)
+
+! For a specific face to face coupling, geometric assumptions are made
+
+if (ifaccp.eq.1) then
+
+  do ipt = 1, nptdis
+
+    iel = locpts(ipt)
+
+    ! --- Pour la pression on veut imposer un dirichlet tel que le gradient
+    !     de pression se conserve entre les deux domaines coupls Pour cela
+    !     on impose une interpolation centre
+
+    xjjp = djppts(1,ipt)
+    yjjp = djppts(2,ipt)
+    zjjp = djppts(3,ipt)
+
+    rvdis(ipt,ipos) = cvar_pr(iel) &
+         + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
+
+  enddo
+
+  ! For a generic coupling, no assumption can be made
+
+else
+
+  do ipt = 1, nptdis
+
+    iel = locpts(ipt)
+
+    xjpf = coopts(1,ipt) - xyzcen(1,iel)- djppts(1,ipt)
+    yjpf = coopts(2,ipt) - xyzcen(2,iel)- djppts(2,ipt)
+    zjpf = coopts(3,ipt) - xyzcen(3,iel)- djppts(3,ipt)
+
+    if (pndpts(ipt).ge.0.d0.and.pndpts(ipt).le.1.d0) then
+      jpf = -1.d0*sqrt(xjpf**2+yjpf**2+zjpf**2)
+    else
+      jpf =       sqrt(xjpf**2+yjpf**2+zjpf**2)
+    endif
+
+    rvdis(ipt,ipos) = (xjpf*grad(1,iel)+yjpf*grad(2,iel)+zjpf*grad(3,iel))  &
+         /jpf
+
+  enddo
+
+endif
+!       Fin pour la pression
+
 !=========================================================================
 ! 3.  PREPARATION DES GRANDEURS TURBULENTES
 !=========================================================================
@@ -392,101 +330,56 @@ if (itytur.eq.2) then
                              iccocg,                              &
                              grad)
 
-  ! For a specific face to face coupling, geometric assumptions are made
+  do ipt = 1, nptdis
 
-  if (ifaccp.eq.1) then
+    iel = locpts(ipt)
 
-    do ipt = 1, nptdis
+    xjjp = djppts(1,ipt)
+    yjjp = djppts(2,ipt)
+    zjjp = djppts(3,ipt)
 
-      iel = locpts(ipt)
+    trav1(ipt) = cvar_k(iel) + xjjp*grad(1,iel)   &
+                             + yjjp*grad(2,iel)   &
+                             + zjjp*grad(3,iel)
 
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
+  enddo
 
-      trav1(ipt) = rtp(iel,ik)                         &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-    ! For a generic coupling, no assumption can be made
-
-  else
-
-    do ipt = 1, nptdis
-
-      iel = locpts(ipt)
-
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav1(ipt) = rtp(iel,ik)                         &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-  endif
-
-  !         Prparation des donnes: interpolation de epsilon en J'
+  ! Prepare data: interpolate epsilon at J'
 
   call field_gradient_scalar(ivarfl(iep), iprev, imrgra, inc,     &
                              iccocg,                              &
                              grad)
 
-  ! For a specific face to face coupling, geometric assumptions are made
+  do ipt = 1, nptdis
 
-  if (ifaccp.eq.1) then
+    iel = locpts(ipt)
 
-    do ipt = 1, nptdis
+    xjjp = djppts(1,ipt)
+    yjjp = djppts(2,ipt)
+    zjjp = djppts(3,ipt)
 
-      iel = locpts(ipt)
-
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav2(ipt) = rtp(iel,iep)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-    ! For a generic coupling, no assumption can be made
-
-  else
-
-    do ipt = 1, nptdis
-
-      iel = locpts(ipt)
-
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav2(ipt) = rtp(iel,iep)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-  endif
+    trav2(ipt) = cvar_ep(iel) + xjjp*grad(1,iel)  &
+                              + yjjp*grad(2,iel)  &
+                              + zjjp*grad(3,iel)
 
+  enddo
 
 !=======================================================================
-!          3.1.2.   Transfert de variable  "iso-modle"
+!          3.1.2.   Transfer variable as "iso-model"
 !=======================================================================
 
   if (itytu0.eq.2) then
 
-    !           Energie turbulente
-    !           ------------------
+    ! k
+    ! -
     ipos = ipos + 1
 
     do ipt = 1, nptdis
       rvdis(ipt,ipos) = trav1(ipt)
     enddo
 
-    !           Dissipation turbulente
-    !           ----------------------
+    ! epsilon
+    ! -------
     ipos = ipos + 1
 
     do ipt = 1, nptdis
@@ -494,7 +387,7 @@ if (itytur.eq.2) then
     enddo
 
     !=======================================================================
-    !          3.1.3.   Transfert de k-eps vers Rij-eps
+    !          3.1.3.   Transfer from k-eps to Rij-eps
     !=======================================================================
 
   elseif (itytu0.eq.3) then
@@ -570,7 +463,7 @@ if (itytur.eq.2) then
     enddo
 
     !=======================================================================
-    !          3.1.4.   Transfert de k-eps vers v2f
+    !          3.1.4.   Transfer from k-eps to v2f
     !=======================================================================
 
   elseif (iturcp(numcpl).eq.50) then
@@ -583,16 +476,16 @@ if (itytur.eq.2) then
 
   elseif (iturcp(numcpl).eq.60) then
 
-    !           Energie turbulente
-    !           -----------------
+    ! k
+    ! -
     ipos = ipos + 1
 
     do ipt = 1, nptdis
       rvdis(ipt,ipos) = trav1(ipt)
     enddo
 
-    !           Omega
-    !           -----
+    ! omega
+    ! -----
     ipos = ipos + 1
 
     do ipt = 1, nptdis
@@ -612,24 +505,26 @@ elseif (itytur.eq.3) then
   !          3.2.1. INTERPOLATION EN J'
   !=======================================================================
 
-  ! Prparation des donnes: interpolation des Rij en J'
+  ! Prepare data: interpolate Rij at J'
 
-  do isou = 1, 6
+  if (irijco.eq.0) then
 
-    if (isou.eq.1) ivar = ir11
-    if (isou.eq.2) ivar = ir22
-    if (isou.eq.3) ivar = ir33
-    if (isou.eq.4) ivar = ir12
-    if (isou.eq.5) ivar = ir13
-    if (isou.eq.6) ivar = ir23
+    do isou = 1, 6
 
-    call field_gradient_scalar(ivarfl(ivar), iprev, imrgra, inc,    &
-                               iccocg,                              &
-                               grad)
+      if (isou.eq.1) ivar = ir11
+      if (isou.eq.2) ivar = ir22
+      if (isou.eq.3) ivar = ir33
+      if (isou.eq.4) ivar = ir12
+      if (isou.eq.5) ivar = ir23
+      if (isou.eq.6) ivar = ir13
 
-    ! For a specific face to face coupling, geometric assumptions are made
+      call field_get_val_s(ivarfl(ivar), cvar_var)
 
-    if (ifaccp.eq.1) then
+      call field_gradient_scalar(ivarfl(ivar), iprev, imrgra, inc,    &
+                                 iccocg,                              &
+                                 grad)
+
+      ! For a specific face to face coupling, geometric assumptions are made
 
       do ipt = 1, nptdis
 
@@ -640,106 +535,93 @@ elseif (itytur.eq.3) then
         zjjp = djppts(3,ipt)
 
         if (isou.eq.1) then
-          trav1(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
+          trav1(ipt) =   cvar_var(iel) &
+                       + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
         else if (isou.eq.2) then
-          trav2(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
+          trav2(ipt) =   cvar_var(iel) &
+                      + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
         else if (isou.eq.3) then
-          trav3(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
+          trav3(ipt) =   cvar_var(iel) &
+                       + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
         else if (isou.eq.4) then
-          trav4(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
+          trav4(ipt) =   cvar_var(iel) &
+                       + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
         else if (isou.eq.5) then
-          trav5(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
+          trav5(ipt) =   cvar_var(iel) &
+                       + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
         else if (isou.eq.6) then
-          trav6(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
+          trav6(ipt) =   cvar_var(iel) &
+                       + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
         endif
 
       enddo
 
-      ! For a generic coupling, no assumption can be made
+    enddo
 
-    else
+  else if (irijco.eq.1) then
 
-      do ipt = 1, nptdis
+    call field_get_val_v(ivarfl(irij), cvar_rij)
 
-        iel = locpts(ipt)
+    ! allocate a temporary array
+    allocate(gradts(6,3,ncelet))
 
-        xjjp = djppts(1,ipt)
-        yjjp = djppts(2,ipt)
-        zjjp = djppts(3,ipt)
+    call field_gradient_tensor(ivarfl(irij), iprev, imrgra, inc, gradts)
 
-        if (isou.eq.1) then
-          trav1(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-        else if (isou.eq.2) then
-          trav2(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-        else if (isou.eq.3) then
-          trav3(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-        else if (isou.eq.4) then
-          trav4(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-        else if (isou.eq.5) then
-          trav5(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-        else if (isou.eq.6) then
-          trav6(ipt) = rtp(iel,ivar) &
-             + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-        endif
+    do ipt = 1, nptdis
 
-      enddo
+      iel = locpts(ipt)
 
-    endif
+      xjjp = djppts(1,ipt)
+      yjjp = djppts(2,ipt)
+      zjjp = djppts(3,ipt)
 
-  enddo
+      trav1(ipt) = cvar_var(iel) + xjjp*gradts(1,1,iel)  &
+                                 + yjjp*gradts(1,2,iel)  &
+                                 + zjjp*gradts(1,3,iel)
 
-  ! Prparation des donnes: interpolation de epsilon en J'
+      trav2(ipt) = cvar_var(iel) + xjjp*gradts(2,1,iel)  &
+                                 + yjjp*gradts(2,2,iel)  &
+                                 + zjjp*gradts(2,3,iel)
+
+      trav3(ipt) = cvar_var(iel) + xjjp*gradts(3,1,iel)  &
+                                 + yjjp*gradts(3,2,iel)  &
+                                 + zjjp*gradts(3,3,iel)
+
+      trav4(ipt) = cvar_var(iel) + xjjp*gradts(4,1,iel)  &
+                                 + yjjp*gradts(4,2,iel)  &
+                                 + zjjp*gradts(4,3,iel)
+
+      trav5(ipt) = cvar_var(iel) + xjjp*gradts(5,1,iel)  &
+                                 + yjjp*gradts(5,2,iel)  &
+                                 + zjjp*gradts(5,3,iel)
+
+      trav6(ipt) = cvar_var(iel) + xjjp*gradts(6,1,iel)  &
+                                 + yjjp*gradts(6,2,iel)  &
+                                 + zjjp*gradts(6,3,iel)
+
+    enddo
+
+  endif
+
+  ! Prepare data: interpolation of epsilon at J'
 
   call field_gradient_scalar(ivarfl(iep), iprev, imrgra, inc,     &
                              iccocg,                              &
                              grad)
 
-  ! For a specific face to face coupling, geometric assumptions are made
+  do ipt = 1, nptdis
 
-  if (ifaccp.eq.1) then
+    iel = locpts(ipt)
 
-    do ipt = 1, nptdis
+    xjjp = djppts(1,ipt)
+    yjjp = djppts(2,ipt)
+    zjjp = djppts(3,ipt)
 
-      iel = locpts(ipt)
+    trav7(ipt) = cvar_ep(iel) + xjjp*grad(1,iel)  &
+                              + yjjp*grad(2,iel)  &
+                              + zjjp*grad(3,iel)
 
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav7(ipt) = rtp(iel,iep)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-    ! For a generic coupling, no assumption can be made
-
-  else
-
-    do ipt = 1, nptdis
-
-      iel = locpts(ipt)
-
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav7(ipt) = rtp(iel,iep)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-  endif
+  enddo
 
 !=======================================================================
 !          3.2.2. Transfert de variable  "iso-modle"
@@ -853,91 +735,47 @@ elseif (iturb.eq.50) then
   !          3.3.1. INTERPOLATION EN J'
   !=======================================================================
 
-  !         Prparation des donnes: interpolation de k en J'
+  !         Prepare data: interpolation of k at J'
 
   call field_gradient_scalar(ivarfl(ik), iprev, imrgra, inc,      &
                              iccocg,                              &
                              grad)
 
-  ! For a specific face to face coupling, geometric assumptions are made
+  do ipt = 1, nptdis
 
-  if (ifaccp.eq.1) then
+    iel = locpts(ipt)
 
-    do ipt = 1, nptdis
+    xjjp = djppts(1,ipt)
+    yjjp = djppts(2,ipt)
+    zjjp = djppts(3,ipt)
 
-      iel = locpts(ipt)
+    trav1(ipt) = cvar_k(iel) + xjjp*grad(1,iel)  &
+                             + yjjp*grad(2,iel)  &
+                             + zjjp*grad(3,iel)
 
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
+  enddo
 
-      trav1(ipt) = rtp(iel,ik)                         &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-    ! For a generic coupling, no assumption can be made
-
-  else
-
-    do ipt = 1, nptdis
-
-      iel = locpts(ipt)
-
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav1(ipt) = rtp(iel,ik)                         &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-  endif
-
-  ! Prparation des donnes: interpolation de epsilon en J'
+  ! Prepare data: interpolation of epsilon at J'
 
   call field_gradient_scalar(ivarfl(iep), iprev, imrgra, inc,  &
                              iccocg,                           &
                              grad)
 
-  ! For a specific face to face coupling, geometric assumptions are made
+  do ipt = 1, nptdis
 
-  if (ifaccp.eq.1) then
+    iel = locpts(ipt)
 
-    do ipt = 1, nptdis
+    xjjp = djppts(1,ipt)
+    yjjp = djppts(2,ipt)
+    zjjp = djppts(3,ipt)
 
-      iel = locpts(ipt)
+    trav2(ipt) = cvar_ep(iel) + xjjp*grad(1,iel)  &
+                              + yjjp*grad(2,iel)  &
+                              + zjjp*grad(3,iel)
 
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
+  enddo
 
-      trav2(ipt) = rtp(iel,iep)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-    ! For a generic coupling, no assumption can be made
-
-  else
-
-    do ipt = 1, nptdis
-
-      iel = locpts(ipt)
-
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav2(ipt) = rtp(iel,iep)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-  endif
-
-  !         Prparation des donnes: interpolation de Phi en J'
+  !         Prepare data: interpolation of Phi at J'
 
   call field_gradient_scalar(ivarfl(iphi), iprev, imrgra, inc,  &
                              iccocg,                            &
@@ -951,52 +789,31 @@ elseif (iturb.eq.50) then
     yjjp = djppts(2,ipt)
     zjjp = djppts(3,ipt)
 
-    trav3(ipt) = rtp(iel,iphi)                        &
-         + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
+    trav3(ipt) = cvar_phi(iel) + xjjp*grad(1,iel)  &
+                               + yjjp*grad(2,iel)  &
+                               + zjjp*grad(3,iel)
 
   enddo
 
-  !         Prparation des donnes: interpolation de F-barre en J'
+  !         Prepare data: interpolation of F-bar at J'
 
   call field_gradient_scalar(ivarfl(ifb), iprev, imrgra, inc,  &
                              iccocg,                           &
                              grad)
 
-  ! For a specific face to face coupling, geometric assumptions are made
+  do ipt = 1, nptdis
 
-  if (ifaccp.eq.1) then
+    iel = locpts(ipt)
 
-    do ipt = 1, nptdis
+    xjjp = djppts(1,ipt)
+    yjjp = djppts(2,ipt)
+    zjjp = djppts(3,ipt)
 
-      iel = locpts(ipt)
+    trav4(ipt) = cvar_fb(iel) + xjjp*grad(1,iel)  &
+                              + yjjp*grad(2,iel)  &
+                              + zjjp*grad(3,iel)
 
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav4(ipt) = rtp(iel,ifb)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-    ! For a generic coupling, no assumption can be made
-
-  else
-
-    do ipt = 1, nptdis
-
-      iel = locpts(ipt)
-
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav4(ipt) = rtp(iel,ifb)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-  endif
+  enddo
 
 !=======================================================================
 !          3.3.2. Transfert de variable  "iso-modle"
@@ -1055,89 +872,45 @@ elseif (iturb.eq.60) then
   !          3.4.1. INTERPOLATION EN J'
   !=======================================================================
 
-  !         Prparation des donnes: interpolation de k en J'
+  !         Prepare data: interpolation of k at J'
 
   call field_gradient_scalar(ivarfl(ik), iprev, imrgra, inc,  &
                              iccocg,                          &
                              grad)
 
-  ! For a specific face to face coupling, geometric assumptions are made
+  do ipt = 1, nptdis
 
-  if (ifaccp.eq.1) then
+    iel = locpts(ipt)
 
-    do ipt = 1, nptdis
+    xjjp = djppts(1,ipt)
+    yjjp = djppts(2,ipt)
+    zjjp = djppts(3,ipt)
 
-      iel = locpts(ipt)
+    trav1(ipt) = cvar_k(iel) + xjjp*grad(1,iel)  &
+                             + yjjp*grad(2,iel)  &
+                             + zjjp*grad(3,iel)
 
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
+  enddo
 
-      trav1(ipt) = rtp(iel,ik)                         &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-    ! For a generic coupling, no assumption can be made
-
-  else
-
-    do ipt = 1, nptdis
-
-      iel = locpts(ipt)
-
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav1(ipt) = rtp(iel,ik)                         &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-  endif
-
-  !         Prparation des donnes: interpolation de omega en J'
+  !         Prepare data: interpolation of omega at J'
 
   call field_gradient_scalar(ivarfl(iomg), iprev, imrgra, inc,  &
                              iccocg,                            &
                              grad)
 
-  ! For a specific face to face coupling, geometric assumptions are made
+  do ipt = 1, nptdis
 
-  if (ifaccp.eq.1) then
+    iel = locpts(ipt)
 
-    do ipt = 1, nptdis
+    xjjp = djppts(1,ipt)
+    yjjp = djppts(2,ipt)
+    zjjp = djppts(3,ipt)
 
-      iel = locpts(ipt)
+    trav2(ipt) = cvar_omg(iel) + xjjp*grad(1,iel)  &
+                               + yjjp*grad(2,iel)  &
+                               + zjjp*grad(3,iel)
 
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav2(ipt) = rtp(iel,iomg)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-    ! For a generic coupling, no assumption can be made
-
-  else
-
-    do ipt = 1, nptdis
-
-      iel = locpts(ipt)
-
-      xjjp = djppts(1,ipt)
-      yjjp = djppts(2,ipt)
-      zjjp = djppts(3,ipt)
-
-      trav2(ipt) = rtp(iel,iomg)                        &
-           + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-    enddo
-
-  endif
+  enddo
 
   !=======================================================================
   !          3.4.2. Transfert de variable  "iso-modle"
@@ -1283,6 +1056,7 @@ if (nscal.gt.0) then
     ipos = ipos + 1
 
     ivar = isca(iscal)
+    call field_get_val_s(ivarfl(isca(iscal)), cvar_scal)
 
 ! --- Calcul du gradient du scalaire pour interpolation
 
@@ -1290,21 +1064,16 @@ if (nscal.gt.0) then
                                iccocg,                            &
                                grad)
 
-    ! For a specific face to face coupling, geometric assumptions are made
+    do ipt = 1, nptdis
 
-    if (ifaccp.eq.1) then
+      iel = locpts(ipt)
 
-      do ipt = 1, nptdis
-
-        iel = locpts(ipt)
-
-! --- Pour les scalaires on veut imposer un dirichlet. On se laisse
-!     le choix entre UPWIND, SOLU ou CENTRE. Seul le centr respecte
-!     la diffusion si il n'y avait qu'un seul domaine
+! --- For scalars we want to prescribe a Dirichlet condition.
+!     Only a centered scheme would respect diffusion in a single domain.
 
 ! -- UPWIND
 
-!        rvdis(ipt,ipos) = rtp(iel,ivar)
+!        rvdis(ipt,ipos) = cvar_scal(iel)
 
 ! -- SOLU
 
@@ -1312,38 +1081,20 @@ if (nscal.gt.0) then
 !        yjf = coopts(2,ipt) - xyzcen(2,iel)
 !        zjf = coopts(3,ipt) - xyzcen(3,iel)
 
-!        rvdis(ipt,ipos) = rtp(iel,ivar) &
+!        rvdis(ipt,ipos) = cvar_scal(iel) &
 !          + xjf*grad(1,iel) + yjf*grad(2,iel) + zjf*grad(3,iel)
 
 ! -- CENTRE
 
-        xjjp = djppts(1,ipt)
-        yjjp = djppts(2,ipt)
-        zjjp = djppts(3,ipt)
+      xjjp = djppts(1,ipt)
+      yjjp = djppts(2,ipt)
+      zjjp = djppts(3,ipt)
 
-        rvdis(ipt,ipos) = rtp(iel,ivar) &
-          + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
+      rvdis(ipt,ipos) = cvar_scal(iel) + xjjp*grad(1,iel)  &
+                                       + yjjp*grad(2,iel)  &
+                                       + zjjp*grad(3,iel)
 
-      enddo
-
-    ! For a generic coupling, no assumption can be made
-
-    else
-
-      do ipt = 1, nptdis
-
-        iel = locpts(ipt)
-
-        xjjp = djppts(1,ipt)
-        yjjp = djppts(2,ipt)
-        zjjp = djppts(3,ipt)
-
-        rvdis(ipt,ipos) = rtp(iel,ivar)                             &
-          + xjjp*grad(1,iel) + yjjp*grad(2,iel) + zjjp*grad(3,iel)
-
-      enddo
-
-    endif
+    enddo
 
   enddo
 
