@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -21,8 +21,6 @@
 !-------------------------------------------------------------------------------
 
 subroutine csinit &
-!================
-
  ( irgpar , nrgpar )
 
 !===============================================================================
@@ -59,18 +57,9 @@ integer  irgpar, nrgpar
 
 !===============================================================================
 
-!===============================================================================
-! Basic initialization for parall module
-!===============================================================================
-
 irangp = irgpar
 nrangp = nrgpar
 
-nthrdi = 1
-nthrdb = 1
-ngrpi = 1
-ngrpb = 1
-
 !----
 ! End
 !----
