@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -23,8 +23,7 @@
 subroutine diffst &
 !================
 
- ( nscal  ,                                              &
-   rtp    , propce )
+ ( nscal, iterns )
 
 !===============================================================================
 ! Function :
@@ -38,9 +37,7 @@ subroutine diffst &
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
 ! nscal            ! i  ! <-- ! total number of scalars                        !
-! rtp,             ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  at current time step                          !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
+! iterns           ! i  ! <-- ! Navier-Stokes iteration number                 !
 !__________________!____!_____!________________________________________________!
 
 !     Type: i (integer), r (real), s (string), a (array), l (logical),
@@ -55,7 +52,6 @@ subroutine diffst &
 use paramx
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use entsor
@@ -69,6 +65,7 @@ use ppthch
 use ppincl
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -76,24 +73,21 @@ implicit none
 
 ! Arguments
 
-integer          nscal
-
-double precision rtp(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
+integer          nscal , iterns
 
 ! Local variables
 
-integer          ivar  , iel   , ifac  , iscal
-integer          ipcvst
+integer          ivar  , iel   , ifac  , iscal, f_id0
 integer          nswrgp, imligp, iwarnp
 integer          iccocg, inc
 integer          iconvp, idiffp, ircflp
 integer          ischcp, isstpp
-integer          ipcvsl, iflmas, iflmab
-integer          imucpp, idftnp
+integer          ifcvsl, iflmas, iflmab
+integer          imucpp, idftnp, imasac
+integer          key_buoyant_id, is_buoyant_fld
 double precision epsrgp, climgp, extrap
 double precision blencp, relaxp, thetex
-double precision qimp , hint
+double precision turb_schmidt
 
 integer          icvflb
 integer          ivoid(1)
@@ -101,10 +95,14 @@ integer          ivoid(1)
 double precision rvoid(1)
 
 double precision, allocatable, dimension(:) :: vistot, viscf, viscb
-double precision, allocatable, dimension(:) :: whsad
 double precision, allocatable, dimension(:) :: xcpp
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
+double precision, dimension(:), pointer :: visct, cpro_cp, cpro_viscls
+double precision, dimension(:), pointer :: cvar_scal
+double precision, dimension(:), pointer :: cpro_tsscal
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -113,11 +111,24 @@ allocate(vistot(ncelet))
 allocate(viscf(nfac), viscb(nfabor))
 allocate(xcpp(ncelet))
 
+if (icp.ge.0) call field_get_val_s(icp, cpro_cp)
+
 do iscal = 1, nscal
 
   ! Index for variable
   ivar = isca(iscal)
 
+  call field_get_val_s(ivarfl(isca(iscal)), cvar_scal)
+
+  ! Key id for buoyant field (inside the Navier Stokes loop)
+  call field_get_key_id("is_buoyant", key_buoyant_id)
+  call field_get_key_int(ivarfl(isca(iscal)), key_buoyant_id, is_buoyant_fld)
+
+  ! If the scalar is buoyant, it is inside the Navier Stokes loop, and so iterns >=1
+  ! otherwise it is outside of the loop and iterns = -1.
+  if (  (is_buoyant_fld.eq. 1 .and. iterns.eq.-1) &
+    .or.(is_buoyant_fld.eq. 0 .and. iterns.ne.-1)) cycle
+
   imucpp = 0
   if (iscavr(iscal).gt.0) then
     if (abs(iscacp(iscavr(iscal))).eq.1) then
@@ -134,9 +145,9 @@ do iscal = 1, nscal
       xcpp(iel) = 1.d0
     enddo
   elseif (imucpp.eq.1) then
-    if (icp.gt.0) then
+    if (icp.ge.0) then
       do iel = 1, ncel
-        xcpp(iel) = propce(iel,ipproc(icp))
+        xcpp(iel) = cpro_cp(iel)
       enddo
    else
       do iel = 1, ncel
@@ -149,24 +160,30 @@ do iscal = 1, nscal
   if (irangp.ge.0.or.iperio.eq.1) then
     call synsca(xcpp)
   endif
-  !
-  iconvp = 0
+
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+  f_id0  = -1
+  iconvp = 0 ! diffusion term only
+  ischcp = 1
+  isstpp = 1
+  blencp = 0.d0
+  imasac = 0
   idiffp = 1
-  nswrgp = nswrgr(ivar)
-  imligp = imligr(ivar)
-  ircflp = ircflu(ivar)
-  ischcp = ischcv(ivar)
-  isstpp = isstpc(ivar)
+  nswrgp = vcopt%nswrgr
+  imligp = vcopt%imligr
+  ircflp = vcopt%ircflu
   inc    = 1
   iccocg = 1
-  idftnp = 1 !idften(ivar)!FIXME when activating GGDH
-  iwarnp = iwarni(ivar)
-  blencp = blencv(ivar)
-  epsrgp = epsrgr(ivar)
-  climgp = climgr(ivar)
-  extrap = extrag(ivar)
-  relaxp = relaxv(ivar)
+  idftnp = ISOTROPIC_DIFFUSION !FIXME when activating GGDH
+  iwarnp = vcopt%iwarni
+  epsrgp = vcopt%epsrgr
+  climgp = vcopt%climgr
+  extrap = vcopt%extrag
+  relaxp = vcopt%relaxv
   thetex = 1.d0
+  ! all boundary convective flux with upwind
+  icvflb = 0
 
   ! Pointers to the mass fluxes
   call field_get_key_int(ivarfl(iu), kimasf, iflmas)
@@ -177,31 +194,32 @@ do iscal = 1, nscal
   ! Diffusion velocity
 
   ! Index for molecular diffusivity
-  if (ivisls(iscal).gt.0) then
-    ipcvsl = ipproc(ivisls(iscal))
-  else
-    ipcvsl = 0
+  call field_get_key_int (ivarfl(isca(iscal)), kivisl, ifcvsl)
+  if (ifcvsl.ge.0) then
+    call field_get_val_s(ifcvsl, cpro_viscls)
   endif
 
   ! Index for turbulent diffusivity
-  ipcvst = ipproc(ivisct)
+  call field_get_val_s(ivisct, visct)
 
-  if (idiff(ivar).ge.1) then
+  if (vcopt%idiff.ge.1) then
 
     ! Only the positive part of mu_t is considered (MAX(mu_t,0)),
     ! Dynamic LES can cause negative mu_t (clipping on (mu+mu_t))
     ! The positive part of (K+K_t) would have been considered
     ! but should allow negative K_t that is considered non physical here
 
-    if(ipcvsl.eq.0)then
+    call field_get_key_double(ivarfl(isca(iscal)), ksigmas, turb_schmidt)
+
+    if(ifcvsl.lt.0)then
       do iel = 1, ncel
         vistot(iel) = visls0(iscal)                                     &
-           + idifft(ivar)*xcpp(iel)*max(propce(iel,ipcvst),zero)/sigmas(iscal)
+           + vcopt%idifft*xcpp(iel)*max(visct(iel),zero)/turb_schmidt
       enddo
     else
       do iel = 1, ncel
-        vistot(iel) = propce(iel,ipcvsl)                                &
-           + idifft(ivar)*xcpp(iel)*max(propce(iel,ipcvst),zero)/sigmas(iscal)
+        vistot(iel) = cpro_viscls(iel)                                  &
+           + vcopt%idifft*xcpp(iel)*max(visct(iel),zero)/turb_schmidt
       enddo
     endif
 
@@ -222,91 +240,28 @@ do iscal = 1, nscal
 
   endif
 
-  ! If the combustion model is used, gap between enthalpy and
-  ! adiabatic enthalpy has to be used. The last one is already considered
-  ! through the mixture fraction contribution.
+  ! Source term
+  call field_get_val_s(iustdy(iscal), cpro_tsscal)
 
-  if (ippmod(icod3p).eq.1.and.iscal.eq.iscalt) then
+  ! Diffusion term calculation
+  call field_get_coefa_s(ivarfl(ivar), coefap)
+  call field_get_coefb_s(ivarfl(ivar), coefbp)
+  call field_get_coefaf_s(ivarfl(ivar), cofafp)
+  call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
-    ! Memory allocation
-    allocate(coefap(nfabor), coefbp(nfabor))
-    allocate(cofafp(nfabor), cofbfp(nfabor))
-    allocate(whsad(ncelet))
-
-    ! Hs is store in a local array and the source term is initialized
-    do iel = 1, ncel
-      whsad(iel) = propce(iel,ipproc(iustdy(iscalt)))
-      propce(iel,ipproc(iustdy(iscalt))) = 0.d0
-    enddo
-
-    ! Parallel and periodic exchanges
-    if (irangp.ge.0.or.iperio.eq.1) then
-      call synsca(whsad)
-    endif
-
-    ! Boundary condition on Hsad: Homogenous Neumann
-    do ifac = 1, nfabor
-      iel = ifabor(ifac)
-
-      hint = vistot(iel)/distb(ifac)
-      qimp = 0.d0
-
-      call set_neumann_scalar &
-           !=================
-         ( coefap(ifac), cofafp(ifac),             &
-           coefbp(ifac), cofbfp(ifac),             &
-           qimp        , hint )
-
-    enddo
-
-    ! all boundary convective flux with upwind
-    icvflb = 0
-
-    ! Diffusion term calculation
-    call bilsca &
-   !==========
-  ( idtvar , ivar   , iconvp , idiffp , nswrgp , imligp , ircflp , &
+  call bilsca &
+  !==========
+  ( idtvar , f_id0  , iconvp , idiffp , nswrgp , imligp , ircflp , &
     ischcp , isstpp , inc    , imrgra , iccocg ,                   &
-    iwarnp , imucpp , idftnp ,                                     &
+    iwarnp , imucpp , idftnp , imasac ,                            &
     blencp , epsrgp , climgp , extrap , relaxp , thetex ,          &
-    whsad  , whsad  ,                                              &
-    coefap , coefbp ,                                              &
-    cofafp , cofbfp ,                                              &
-    imasfl , bmasfl ,                                              &
-    viscf  , viscb  , rvoid  , rvoid  ,                            &
-    rvoid  , rvoid  ,                                              &
-    icvflb , ivoid  ,                                              &
-    propce(1,ipproc(iustdy(iscal))) )
-
-    ! Free memory
-    deallocate(coefap, coefbp)
-    deallocate(cofafp, cofbfp)
-    deallocate(whsad)
-
-  else
-
-    ! Diffusion term calculation
-
-    call field_get_coefa_s(ivarfl(ivar), coefap)
-    call field_get_coefb_s(ivarfl(ivar), coefbp)
-    call field_get_coefaf_s(ivarfl(ivar), cofafp)
-    call field_get_coefbf_s(ivarfl(ivar), cofbfp)
-
-    call bilsca &
-    !==========
-  ( idtvar , ivar   , iconvp , idiffp , nswrgp , imligp , ircflp , &
-    ischcp , isstpp , inc    , imrgra , iccocg ,                   &
-    iwarnp , imucpp , idftnp ,                                     &
-    blencp , epsrgp , climgp , extrap , relaxp , thetex ,          &
-    rtp(1,ivar)     , rtp(1,ivar)     ,                            &
+    cvar_scal       , cvar_scal       ,                            &
     coefap , coefbp , cofafp , cofbfp ,                            &
     imasfl , bmasfl ,                                              &
     viscf  , viscb  , rvoid  , xcpp   ,                            &
     rvoid  , rvoid  ,                                              &
     icvflb , ivoid  ,                                              &
-    propce(1,ipproc(iustdy(iscal))) )
-
-  endif
+    cpro_tsscal     )
 
 enddo
 
