@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -41,11 +41,9 @@
 !   mode          name          role
 !------------------------------------------------------------------------------
 !> \param[in]     itypfb        boundary face types
-!> \param[out]    distpa        distance to wall
 !______________________________________________________________________________
 
-subroutine distpr &
- ( itypfb , distpa )
+subroutine distpr(itypfb)
 
 !===============================================================================
 ! Module files
@@ -60,8 +58,10 @@ use cstnum
 use ppppar
 use parall
 use period
-use mltgrd
 use mesh
+use field
+use field_operator
+use cs_c_bindings
 
 !===============================================================================
 
@@ -71,36 +71,35 @@ implicit none
 
 integer          itypfb(nfabor)
 
-double precision distpa(ncelet)
 
 ! Local variables
 
-integer          ndircp, iconvp, idiffp, isym
-integer          ipol  , ireslp, ipp
-integer          niterf, icycle, ncymxp, nitmfp
-integer          iinvpe
-integer          isqrt , iel   , ifac
-integer          inc   , iccocg, ivar
-integer          isweep, nittot, idtva0
-integer          ibsize, iesize, mmprpl, nswrsl
+integer          ndircp, iconvp, idiffp
+integer          iel   , ifac
+integer          inc   , iccocg, f_id
+integer          mmprpl, nswrsp
 integer          imucpp, idftnp
-
-integer          icvflb
+integer          nswrgp
+integer          icvflb, iescap, imligp, ircflp, iswdyp, isstpp, ischcp, iwarnp
 integer          ivoid(1)
+integer          init, counter
 
-double precision relaxp, thetap, rnorm, residu, rnoini
-double precision dismax, dismin, hint, pimp, qimp
+double precision relaxp, blencp, climgp, epsilp, epsrgp, epsrsp, extrap
+double precision dismax, dismin, hint, pimp, qimp, norm_grad, thetap
+double precision normp
 
 double precision rvoid(1)
 
 double precision, allocatable, dimension(:) :: viscf, viscb
-double precision, allocatable, dimension(:) :: coefad, coefbd
-double precision, allocatable, dimension(:) :: cofafd, cofbfd
-double precision, allocatable, dimension(:) :: dam, xam
-double precision, allocatable, dimension(:) :: rtpdp, smbdp, rovsdp
+double precision, allocatable, dimension(:) :: dpvar, smbrp, rovsdt
 double precision, allocatable, dimension(:,:) :: grad
-double precision, allocatable, dimension(:) :: w1, w2, w3
-double precision, allocatable, dimension(:) :: w7, w8, w9
+double precision, allocatable, dimension(:) :: w1
+double precision, allocatable, dimension(:) :: imasfl, bmasfl
+double precision, pointer, dimension(:)   :: cvar_var
+double precision, pointer, dimension(:)   :: cvara_var
+double precision, pointer, dimension(:) :: coefap, coefbp
+double precision, pointer, dimension(:) :: cofafp, cofbfp
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -111,20 +110,34 @@ double precision, allocatable, dimension(:) :: w7, w8, w9
 
 ! Allocate temporary arrays for the species resolution
 allocate(viscf(nfac), viscb(nfabor))
-allocate(coefad(nfabor), coefbd(nfabor))
-allocate(cofafd(nfabor), cofbfd(nfabor))
-allocate(dam(ncelet), xam(nfac))
-allocate(rtpdp(ncelet), smbdp(ncelet), rovsdp(ncelet))
+allocate(imasfl(nfac), bmasfl(nfabor))
+
+do ifac = 1, nfac
+  imasfl(ifac) = 0.d0
+enddo
+
+do ifac = 1, nfabor
+  bmasfl(ifac) = 0.d0
+enddo
+
+allocate(dpvar(ncelet), smbrp(ncelet), rovsdt(ncelet))
 
 ! Allocate work arrays
-allocate(w1(ncelet), w2(ncelet), w3(ncelet))
-allocate(w7(ncelet), w8(ncelet), w9(ncelet))
+allocate(w1(ncelet))
 
 ! Initialize variables to avoid compiler warnings
+call field_get_id("wall_distance", f_id)
 
-rnoini = 0.d0
+call field_get_key_struct_var_cal_opt(f_id, vcopt)
 
-nittot = 0
+call field_get_val_s(f_id, cvar_var)
+call field_get_val_prev_s(f_id, cvara_var)
+
+! Always start from a 0 previous wall distance
+do iel = 1, ncel
+  cvara_var(iel) = 0.d0
+enddo
+call synsce(cvara_var)
 
 !===============================================================================
 ! 2. Boundary conditions
@@ -133,13 +146,17 @@ nittot = 0
 !     Conditions aux limites pour le scalaire resolu T
 !       Dirichlet a 0 en paroi
 !       Neumann nul ailleurs
-!     On test aussi la pressence d'un Dirichlet
+!     On test aussi la presence d'un Dirichlet
 
 ndircp = 0
 
+call field_get_coefa_s( f_id, coefap)
+call field_get_coefb_s( f_id, coefbp)
+call field_get_coefaf_s(f_id, cofafp)
+call field_get_coefbf_s(f_id, cofbfp)
+
 do ifac = 1, nfabor
-  if(itypfb(ifac).eq.iparoi .or.                            &
-     itypfb(ifac).eq.iparug) then
+  if (itypfb(ifac).eq.iparoi .or. itypfb(ifac).eq.iparug) then
 
     ! Dirichlet Boundary Condition
     !-----------------------------
@@ -149,12 +166,12 @@ do ifac = 1, nfabor
 
     call set_dirichlet_scalar &
          !====================
-       ( coefad(ifac), cofafd(ifac),             &
-         coefbd(ifac), cofbfd(ifac),             &
+       ( coefap(ifac), cofafp(ifac),             &
+         coefbp(ifac), cofbfp(ifac),             &
          pimp        , hint        , rinfin )
 
 
-    ndircp = 1
+    ndircp = ndircp + 1
   else
 
     ! Neumann Boundary Conditions
@@ -165,13 +182,23 @@ do ifac = 1, nfabor
 
     call set_neumann_scalar &
          !==================
-       ( coefad(ifac), cofafd(ifac),             &
-         coefbd(ifac), cofbfd(ifac),             &
+       ( coefap(ifac), cofafp(ifac),             &
+         coefbp(ifac), cofbfp(ifac),             &
          qimp        , hint )
 
   endif
 enddo
 
+if (irangp.ge.0) call parcpt(ndircp)
+
+! If no wall initialization to a big value
+if (ndircp.eq.0) then
+  do iel = 1, ncel
+    cvar_var(iel) = grand
+  enddo
+  return
+endif
+
 !===============================================================================
 ! 3. Prepare system to solve
 !===============================================================================
@@ -179,7 +206,7 @@ enddo
 ! -- Diagonal
 
 do iel = 1, ncel
-  rovsdp(iel) = 0.d0
+  rovsdt(iel) = 0.d0
 enddo
 
 ! -- Diffusion at faces
@@ -194,142 +221,110 @@ call viscfa                                                       &
    w1     ,                                                       &
    viscf  , viscb  )
 
-iconvp = 0
-idiffp = 1
-isym   = 1
-thetap = 1.d0
-imucpp = 0
-
-call matrix &
-!==========
- ( iconvp , idiffp , ndircp , isym   ,                            &
-   thetap , imucpp ,                                              &
-   coefbd , cofbfd , rovsdp ,                                     &
-   viscf  , viscb  , viscf  , viscb  ,                            &
-   rvoid  , dam    , xam    )
-
 !===============================================================================
 ! 4. Solve system
 !===============================================================================
 
-ipp = 1
-nomva0 = 'DisParoi'
-ipol   = 0
-ireslp = 0
-! No multigrid (NCYMXP,NITMFP arbitrary)
-ncymxp = 100
-nitmfp = 10
-! Periodicity
-iinvpe = 0
-if(iperio.eq.1) iinvpe = 1
-isqrt = 1
-ibsize = 1
-iesize = 1
-nswrsl = nswrsy
-110 continue
-
 ! Distance to wall is initialized to 0 for reconstruction
 
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
+idftnp = vcopt%idften
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
+iescap = 0
+imucpp = 0
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
+thetap = vcopt%thetav
+! all boundary convective flux with upwind
+icvflb = 0
+init   = 1
+normp = -1.d0
+
+110 continue
+
 do iel = 1, ncelet
-  distpa(iel) = 0.d0
-  rtpdp(iel)  = 0.d0
+  dpvar(iel)  = 0.d0
 enddo
 
 ! -- RHS
 
 do iel = 1, ncel
-  smbdp(iel)  = volume(iel)
+  rovsdt(iel) = 0.d0
+  smbrp(iel)  = cell_f_vol(iel)
 enddo
 
-! -- Reconstruction loop;
-!   if NSWRSY = 1, we must solve twice
-
-do isweep = 0, nswrsl
-
-  call prodsc(ncel,isqrt,smbdp,smbdp,rnorm)
-  if (iwarny.ge.2) then
-     write(nfecra,5000) nomva0,isweep,rnorm
-  endif
-  if (isweep.le.1) rnoini = rnorm
-  ! Convergence test
-  if (rnorm.le.10.d0*epsily*rnoini) goto 100
-
-  do iel = 1, ncelet
-    rtpdp(iel) = 0.d0
-  enddo
-
-  call invers &
-  !==========
- ( nomva0 , isym   , ibsize , iesize ,                            &
-   ipol   , ireslp , nitmay , imgrpy ,                            &
-   ncymxp , nitmfp ,                                              &
-   iwarny , niterf , icycle , iinvpe ,                            &
-   epsily , rnorm  , residu ,                                     &
-   dam    , xam    , smbdp  , rtpdp  )
-
-  nittot = nittot + niterf
-  do iel = 1, ncel
-    distpa(iel) = distpa(iel) + rtpdp(iel)
-  enddo
-
-  ! - Synchronization for parallelism
-
-  if (irangp.ge.0.or.iperio.eq.1) then
-    call synsca(rtpdp)
-    !==========
-  endif
-
-  if (isweep.lt.nswrsl) then
-    inc    = 0
-    iccocg = 1
-    imucpp = 0
-    idftnp = 1 ! no tensorial diffusivity
-    ivar = 0
-    idtva0 = 0
-    relaxp = 1.d0
-
-    ! all boundary convective flux with upwind
-    icvflb = 0
-
-    call bilsca &
-    !==========
- ( idtva0 , ivar   , iconvp , idiffp , nswrgy , imligy , ircfly , &
-   ischcy , isstpy , inc    , imrgra , iccocg ,                   &
-   iwarny , imucpp , idftnp ,                                     &
-   blency , epsrgy , climgy , extray , relaxp , thetap ,          &
-   rtpdp  , rtpdp  , coefad , coefbd , coefad , cofbfd ,          &
-   viscf  , viscb  , viscf  , viscb  , rvoid  , rvoid  ,          &
+call codits &
+ ( idtvar , init, f_id   , iconvp , idiffp , ndircp ,             &
+   imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
+   ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
+   iwarnp , normp  ,                                              &
+   blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
+   relaxp , thetap ,                                              &
+   cvara_var       , cvara_var       ,                            &
+   coefap , coefbp , cofafp , cofbfp ,                            &
+   imasfl , bmasfl ,                                              &
+   viscf  , viscb  , viscf  , viscb  , rvoid  ,                   &
    rvoid  , rvoid  ,                                              &
    icvflb , ivoid  ,                                              &
-   smbdp  )
-
-  endif
-enddo
+   rovsdt , smbrp  , cvar_var        , dpvar  ,                   &
+   rvoid  , rvoid  )
 
+! Count clippings
 mmprpl = 0
+dismin =  grand
 do iel = 1, ncel
-  if (distpa(iel).lt.0.d0) then
-    mmprpl = 1
-    exit
+  if (cvar_var(iel).lt.0.d0) then
+    mmprpl = mmprpl + 1
+    dismin = min(cvar_var(iel), dismin)
+    cvar_var(iel) = epzero*(cell_f_vol(iel))**(1.d0/3.d0)
   endif
 enddo
 
-if (irangp.ge.0) call parcmx(mmprpl)
+if (irangp.ge.0) then
+  call parcpt(mmprpl)
+  call parmin(dismin)
+endif
+
+! Recompute wall distance without reconstruction (that ensure that it is positive)
+if (mmprpl.ge.1) then
+  if (nswrsp.gt.0) then
+    nswrsp = 0
+    ircflp = 0
+    ! Reset also in var_cal_opt structure because some basic routines
+    ! directly use the field options in vcopt...
+    vcopt%nswrsm = nswrsp
+    vcopt%ircflu = ircflp
+    call field_set_key_struct_var_cal_opt(f_id, vcopt)
+    write(nfecra,9000) mmprpl
+
+    ! Reset wall distance
+    do iel = 1, ncelet
+      cvar_var(iel)  = 0.d0
+    enddo
 
-if (mmprpl.eq.1) then
-  if (nswrsl.gt.0) then
-    nswrsl = 0
-    write(nfecra,9000)
     goto 110
   else
-    write(nfecra,9001) distpa(iel)
+
+    write(nfecra,9001) dismin
+
   endif
 endif
 
- 100  continue
-
-do iel=1,ncel
-  rtpdp(iel)  = distpa(iel)
+do iel = 1, ncel
+  dpvar(iel) = max(cvar_var(iel), 0.d0)
 enddo
 
 !===============================================================================
@@ -337,37 +332,33 @@ enddo
 !===============================================================================
 
 ! Allocate a temporary array for the gradient calculation
-allocate(grad(ncelet,3))
+allocate(grad(3,ncelet))
 
-! - Synchronization for parallelism and periodicity
-
-if (irangp.ge.0.or.iperio.eq.1) then
-  call synsca(rtpdp)
-  !==========
-endif
-
-! - Compute gradient
+! Compute current gradient
 
 inc    = 1
 iccocg = 1
-ivar   = 0
 
-call grdcel                                                       &
-!==========
- ( ivar   , imrgra , inc    , iccocg , nswrgy , imligy ,          &
-   iwarny , nfecra , epsrgy , climgy , extray ,                   &
-   rtpdp  , coefad , coefbd ,                                     &
-   grad   )
+call field_gradient_scalar(f_id, 0, imrgra, inc, iccocg, grad)
 
+counter = 0
 do iel = 1, ncel
-  w1(iel) = grad(iel,1)**2.d0+grad(iel,2)**2.d0+grad(iel,3)**2.d0
-  if(w1(iel)+2.d0*rtpdp(iel).gt.0.d0) then
-    distpa(iel) = - sqrt(w1(iel)) + sqrt(w1(iel)+2.d0*rtpdp(iel))
+  norm_grad = grad(1,iel)**2.d0+grad(2,iel)**2.d0+grad(3,iel)**2.d0
+  if (norm_grad+2.d0*dpvar(iel).ge.0.d0) then
+    cvar_var(iel) = sqrt(norm_grad + 2.d0*dpvar(iel)) - sqrt(norm_grad)
   else
-    write(nfecra,8000)iel, xyzcen(1,iel),xyzcen(2,iel),xyzcen(3,iel)
+    counter = counter + 1
   endif
 enddo
 
+if (irangp.ge.0) then
+  call parcpt(counter)
+endif
+
+if (counter.gt.0) then
+  write(nfecra,8000) counter
+endif
+
 ! Free memory
 deallocate(grad)
 
@@ -379,8 +370,8 @@ dismax = -grand
 dismin =  grand
 
 do iel = 1, ncel
-  dismin = min(distpa(iel),dismin)
-  dismax = max(distpa(iel),dismax)
+  dismin = min(cvar_var(iel),dismin)
+  dismax = max(cvar_var(iel),dismax)
 enddo
 
 if (irangp.ge.0) then
@@ -388,16 +379,13 @@ if (irangp.ge.0) then
   call parmax(dismax)
 endif
 
-write(nfecra,1000)dismin, dismax, nittot
+write(nfecra,1000) dismin, dismax
 
 ! Free memory
 deallocate(viscf, viscb)
-deallocate(coefad, coefbd)
-deallocate(cofafd, cofbfd)
-deallocate(dam, xam)
-deallocate(rtpdp, smbdp, rovsdp)
-deallocate(w1, w2, w3)
-deallocate(w7, w8, w9)
+deallocate(dpvar, smbrp, rovsdt)
+deallocate(imasfl, bmasfl)
+deallocate(w1)
 
 !===============================================================================
 ! 7. Formats
@@ -410,27 +398,21 @@ deallocate(w7, w8, w9)
 ' ** DISTANCE A LA PAROI                                      ',/,&
 '    -------------------                                      ',/,&
 '                                                             ',/,&
-'   Distance min = ',E14.5    ,'  Distance max = ',E14.5       ,/,&
-'                                                             ',/,&
-'     (Calcul de la distance realise en ',I10   ,' iterations)',/)
-
- 5000 format(1X,A8,' : SWEEP = ',I5,' NORME SECOND MEMBRE = ',E14.6)
+'   Distance min = ',E14.5    ,'  Distance max = ',E14.5       ,/)
 
  8000   format(                                                         &
 '@                                                            ',/,&
 '@ @@ ATTENTION : Calcul de la distance a la paroi            ',/,&
 '@    =========                                               ',/,&
-'@  La variable associee ne converge pas a la cellule ',I10    ,/,&
-'@       Coord X      Coord Y      Coord Z                    ',/,&
-'@ ',3E13.5                                                    ,/)
+'@  La variable associee ne converge dans ',I10    ,' cellules',/)
 
  9000   format(                                                         &
-'@                                                            ',/,&
-'@ @@ ATTENTION : Calcul de la distance a la paroi            ',/,&
-'@    =========                                               ',/,&
-'@  La solution du laplacien ne respecte pas le principe du   ',/,&
-'@  maximum. On recalcule le laplacien sans les               ',/,&
-'@  reconstructions.                                          ',/)
+'@'                                                            ,/,&
+'@ @@ ATTENTION : Calcul de la distance a la paroi'            ,/,&
+'@    ========='                                               ,/,&
+'@  La solution du laplacien ne respecte pas le principe du'   ,/,&
+'@  maximum en ', i10, ' cellules. On recalcule le laplacien'  ,/,&
+'@  sans les reconstructions.',/)
 
  9001   format(                                                         &
 '@                                                            ',/,&
@@ -447,27 +429,21 @@ deallocate(w7, w8, w9)
 ' ** WALL DISTANCE                                            ',/,&
 '    -------------                                            ',/,&
 '                                                             ',/,&
-'  Min distance = ',E14.5    ,' Max distance = ',E14.5         ,/,&
-'                                                             ',/,&
-'     (Distance calculation done in ',I10   ,' iterations)'    ,/)
-
- 5000 format(1X,A8,' : SWEEP = ',I5,' RIGHT HAND SIDE NORM = ',E14.6)
+'  Min distance = ',E14.5    ,' Max distance = ',E14.5         ,/)
 
  8000   format(                                                         &
 '@                                                            ',/,&
 '@ @@ WARNING: Wall distance calculation                      ',/,&
 '@    ========                                                ',/,&
-'@  The associated variable does not converge in cell ',I10    ,/,&
-'@       Coord X      Coord Y      Coord Z                    ',/,&
-'@ ',3E13.5                                                    ,/)
+'@  The associated variable does not converge in ',I10,' cells.'/)
 
  9000   format(                                                         &
-'@                                                            ',/,&
-'@ @@ WARNING: Wall distance calculation                      ',/,&
-'@    =========                                               ',/,&
-'@  The laplacian solution does not respect the maximum       ',/,&
-'@  principle. We recompute the laplacien without             ',/,&
-'@  reconstructions.                                          ',/)
+'@'                                                            ,/,&
+'@ @@ WARNING: Wall distance calculation'                      ,/,&
+'@    ========='                                               ,/,&
+'@  The laplacian solution does not respect the maximum'       ,/,&
+'@  principle in ', i10,' cells. We recompute the laplacien'   ,/,&
+'@  without reconstructions.',/)
 
  9001   format(                                                         &
 '@                                                            ',/,&
