@@ -1,8 +1,9 @@
+
 !-------------------------------------------------------------------------------
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,56 +21,48 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine dttvar &
-!================
+!===============================================================================
+! Purpose:
+! -------
 
+!> \file dttvar.f90
+!> \brief Compute the local time step and add the Courant and Fourier number to
+!the log.
+!>
+!> This function has access to the boundary face type, except for the first time
+!> step.
+
+!-------------------------------------------------------------------------------
+
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]     nvar          total number of variables
+!> \param[in]     nscal         total number of scalars
+!> \param[in]     ncepdp        number of cells with head loss terms
+!> \param[in]     ncesmp        number of cells with mass source terms
+!> \param[in]     iwarnp        verbosity
+!> \param[in]     icepdc        index number of cells with head loss terms
+!> \param[in]     icetsm        index number of cells with mass source terms
+!> \param[in]     itypsm        type of mass source term for each variable
+!>                               (see \ref cs_user_mass_source_terms)
+!> \param[in]     dt            time step (per cell)
+!> \param[in]     ckupdc        head loss coefficient
+!> \param[in]     smacel        value associated to each variable in the mass
+!>                               source terms or mass rate (see
+!>                               \ref cs_user_mass_source_terms)
+!_______________________________________________________________________________
+
+subroutine dttvar &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    iwarnp ,                                                       &
    icepdc , icetsm , itypsm ,                                     &
-   dt     , rtp    , rtpa   , propce ,                            &
+   dt     ,                                                       &
    ckupdc , smacel )
 
 !===============================================================================
-! FONCTION :
-! ----------
-
-! CALCUL DU PAS DE TEMPS LOCAL
-! AFFICHAGE DES NOMBRES DE COURANT + FOURIER MINIMUM, MAXIMUM
-! On dispose des types de faces de bord au pas de temps
-!   precedent (sauf au premier pas de temps, ou les tableaux
-!   ITYPFB et ITRIFB n'ont pas ete renseignes)
-
-! Sous programme utilise dans le cas une seule phase (ou
-! si seule la phase 1 pilote le pas de temps)
-!-------------------------------------------------------------------------------
-! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! nvar             ! i  ! <-- ! total number of variables                      !
-! nscal            ! i  ! <-- ! total number of scalars                        !
-! ncepdp           ! i  ! <-- ! number of cells with head loss                 !
-! ncesmp           ! i  ! <-- ! number of cells with mass source term          !
-! iwarnp           ! i  ! <-- ! verbosity                                      !
-! icepdc(ncelet    ! te ! <-- ! numero des ncepdp cellules avec pdc            !
-! icetsm(ncesmp    ! te ! <-- ! numero des cellules a source de masse          !
-! itypsm           ! te ! <-- ! type de source de masse pour les               !
-! (ncesmp,nvar)    !    !     !  variables (cf. ustsma)                        !
-! dt(ncelet)       ! ra ! <-- ! time step (per cell)                           !
-! rtp, rtpa        ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at current and previous time steps)          !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-! ckupdc           ! tr ! <-- ! tableau de travail pour pdc                    !
-!  (ncepdp,6)      !    !     !                                                !
-! smacel           ! tr ! <-- ! valeur des variables associee a la             !
-! (ncesmp,nvar)    !    !     !  source de masse                               !
-!                  !    !     ! pour ivar=ipr, smacel=flux de masse            !
-!__________________!____!_____!________________________________________________!
-
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
 
 !===============================================================================
 ! Module files
@@ -104,28 +97,26 @@ integer          iwarnp
 integer          icepdc(ncepdp)
 integer          icetsm(ncesmp), itypsm(ncesmp,nvar)
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
-double precision ckupdc(ncepdp,6), smacel(ncesmp,nvar)
+double precision dt(ncelet)
+double precision ckupdc(6,ncepdp), smacel(ncesmp,nvar)
 
 ! Local variables
 
-character*8      cnom
+character(len=8) :: cnom
 
-integer          ifac, iel, icfmax, icfmin, idiff0, iconv0, isym, flid
+integer          ifac, iel, icfmax(1), icfmin(1), idiff0, iconv0, isym, flid
 integer          modntl
-integer          ipcvis, ipcvst
 integer          iflmas, iflmab
 integer          icou, ifou , icoucf
 integer          inc, iccocg
 integer          nswrgp, imligp
-integer          iivar
+integer          f_id
 integer          nbrval, nclptr
-integer          ipccou, ipcfou, ntcam1
+integer          ntcam1
 
 double precision epsrgp, climgp, extrap
 double precision cfmax,cfmin, w1min, w2min, w3min
-double precision unpvdt, rom, dtloc
+double precision unpvdt, dtloc
 double precision xyzmax(3), xyzmin(3), vmin(1), vmax(1)
 double precision dtsdtm
 double precision hint
@@ -140,11 +131,14 @@ double precision, allocatable, dimension(:,:) :: grad
 double precision, allocatable, dimension(:) :: w1, w2, w3, dtsdt0
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: brom, crom
+double precision, dimension(:), pointer :: viscl, visct, cpro_cour, cpro_four
+
+type(var_cal_opt) :: vcopt_u, vcopt_p
 
 !===============================================================================
 
 !===============================================================================
-! 0.  INITIALISATION
+! 0. Initialisation
 !===============================================================================
 
 ! Pointers to the mass fluxes
@@ -166,12 +160,12 @@ endif
 ! Allocate work arrays
 allocate(w1(ncelet), w2(ncelet), w3(ncelet))
 
-ipcvis  = ipproc(iviscl)
-ipcvst  = ipproc(ivisct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
-ipccou  = ipproc(icour)
-ipcfou  = ipproc(ifour)
+call field_get_val_s(icour, cpro_cour)
+call field_get_val_s(ifour, cpro_four)
 
 if (ntlist.gt.0) then
   modntl = mod(ntcabs,ntlist)
@@ -181,17 +175,19 @@ else
   modntl = 1
 endif
 
-if (                                                             &
-   .not. (iconv(iu).ge.1 .and. ipccou.ge.1) .and.                &
-   .not. (idiff(iu).ge.1 .and. ipcfou.ge.1) .and.                &
-   .not. ((iconv(iu).ge.1 .or.idiff(iu).ge.1).and.               &
-           (iwarnp.ge.2.or.modntl.eq.0)) .and.                   &
-   .not. (ippmod(icompf).ge.0.and.                               &
-           (iwarnp.ge.2.or.modntl.eq.0)) .and.                   &
-   .not. (idtvar.eq.-1.or.idtvar.eq.1.or.idtvar.eq.2.or.         &
-           ((iwarnp.ge.2.or.modntl.eq.0).and.                    &
-             (idiff(iu).ge.1.or.iconv(iu).ge.1                   &
-                               .or.ippmod(icompf).ge.0)))        &
+call field_get_key_struct_var_cal_opt(ivarfl(iu), vcopt_u)
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt_p)
+
+if (.not. (vcopt_u%iconv.ge.1 .and. icour.ge.1) .and.              &
+    .not. (vcopt_u%idiff.ge.1 .and. ifour.ge.1) .and.              &
+    .not. ((vcopt_u%iconv.ge.1 .or.vcopt_u%idiff.ge.1).and.        &
+           (iwarnp.ge.2.or.modntl.eq.0)) .and.                     &
+    .not. (ippmod(icompf).ge.0.and.                                &
+           (iwarnp.ge.2.or.modntl.eq.0)) .and.                     &
+    .not. (idtvar.eq.-1.or.idtvar.eq.1.or.idtvar.eq.2.or.          &
+           ((iwarnp.ge.2.or.modntl.eq.0).and.                      &
+            (vcopt_u%idiff.ge.1.or.vcopt_u%iconv.ge.1.or.          &
+             ippmod(icompf).ge.0)))                                &
    ) then
 
   return
@@ -199,22 +195,19 @@ if (                                                             &
 endif
 
 !===============================================================================
-! 1.  CALCUL DE LA LIMITATION EN COMPRESSIBLE
+! 1. Compute CFL like condition on the time step for positive density for the
+!    compressible module
 !===============================================================================
 
-!     On commence par cela afin de disposer de VISCF VISCB comme
-!       tableaux de travail.
-
 if (ippmod(icompf).ge.0) then
 
   call cfdttv                                                   &
   !==========
  ( nvar   , nscal  , ncepdp , ncesmp ,                          &
    icepdc , icetsm , itypsm ,                                   &
-   dt     , rtp    , rtpa   , propce ,                          &
+   dt     ,                                                     &
    ckupdc , smacel ,                                            &
-   wcf    ,                                                     &
-   viscf  , viscb  , cofbft )
+   wcf    )
 
 endif
 
@@ -222,16 +215,12 @@ endif
 ! 2. Compute the diffusivity at the faces
 !===============================================================================
 
-!     On s'en sert dans les divers matrdt suivants
-
-!     "VITESSE" DE DIFFUSION FACETTE
-
-if (idiff(iu).ge. 1) then
+if (vcopt_u%idiff.ge.1) then
   do iel = 1, ncel
-    w1(iel) = propce(iel,ipcvis) + idifft(iu)*propce(iel,ipcvst)
+    w1(iel) = viscl(iel) + vcopt_u%idifft*visct(iel)
   enddo
   call viscfa(imvisf, w1, viscf, viscb)
-  !==========
+
 else
   do ifac = 1, nfac
     viscf(ifac) = 0.d0
@@ -242,7 +231,7 @@ else
 endif
 
 !===============================================================================
-! 3.  CONDITION LIMITE POUR MATRDT
+! 3. Boundary condition for matrdt
 !===============================================================================
 
 if (idtvar.ge.0) then
@@ -251,8 +240,8 @@ if (idtvar.ge.0) then
 
     if (bmasfl(ifac).lt.0.d0) then
       iel = ifabor(ifac)
-      hint = idiff(iu)*(  propce(iel,ipcvis)                         &
-                        + idifft(iu)*propce(iel,ipcvst))/distb(ifac)
+      hint = vcopt_u%idiff*(  viscl(iel)                         &
+                        + vcopt_u%idifft*visct(iel))/distb(ifac)
       coefbt(ifac) = 0.d0
       cofbft(ifac) = hint
     else
@@ -268,7 +257,7 @@ else
   !      (algorithm was probably broken since velocity components are
   !      coupled)
 
-  mult = 1.0d0 / 3.0d0
+  mult = 1.d0 / 3.d0
 
   call field_get_coefb_v (ivarfl(iu), coefbv)
   call field_get_coefbf_v(ivarfl(iu), cofbfv)
@@ -281,49 +270,48 @@ else
 endif
 
 !===============================================================================
-! 4.  ALGORITHME INSTATIONNAIRE
+! 4. Steady algorithm
 !===============================================================================
 
 if (idtvar.ge.0) then
 
 !===============================================================================
-! 4.1  PAS DE TEMPS VARIABLE A PARTIR DE COURANT ET FOURIER IMPOSES
+! 4.1 Variable time step from imposed Courant and Fourier
 !===============================================================================
 
   ! On calcule le pas de temps thermique max (meme en IDTVAR=0, pour affichage)
-  ! DTTMAX = 1/SQRT(MAX(0+,gradRO.g/RO) -> W3
+  ! dttmax = 1/sqrt(max(0+,gradRO.g/RO) -> w3
 
   if (iptlro.eq.1) then
 
     ! Allocate a temporary array for the gradient calculation
-    allocate(grad(ncelet,3))
+    allocate(grad(3,ncelet))
     allocate(coefbr(nfabor))
 
     do ifac = 1, nfabor
       coefbr(ifac) = 0.d0
     enddo
 
-    nswrgp = nswrgr(ipr)
-    imligp = imligr(ipr)
-    iwarnp = iwarni(ipr)
-    epsrgp = epsrgr(ipr)
-    climgp = climgr(ipr)
+    nswrgp = vcopt_p%nswrgr
+    imligp = vcopt_p%imligr
+    iwarnp = vcopt_p%iwarni
+    epsrgp = vcopt_p%epsrgr
+    climgp = vcopt_p%climgr
     extrap = 0.d0
 
-    iivar = 0
+    f_id = -1
     inc   = 1
     iccocg = 1
 
-    call grdcel                                                   &
-    !==========
- ( iivar  , imrgra , inc    , iccocg , nswrgp , imligp ,          &
-   iwarnp , nfecra , epsrgp , climgp , extrap ,                   &
-   crom, brom, coefbr ,                   &
+    call gradient_s &
+ ( f_id   , imrgra , inc    , iccocg , nswrgp , imligp ,          &
+   iwarnp , epsrgp , climgp , extrap ,                            &
+   crom, brom, coefbr ,                                           &
    grad )
 
     do iel = 1, ncel
-      w3(iel) = (grad(iel,1)*gx + grad(iel,2)*gy + grad(iel,3)*gz)&
-           /crom(iel)
+      w3(iel) = (grad(1,iel)*gx + grad(2,iel)*gy + grad(3,iel)*gz)&
+              / crom(iel)
       w3(iel) = 1.d0/sqrt(max(epzero,w3(iel)))
 
     enddo
@@ -339,15 +327,15 @@ if (idtvar.ge.0) then
     icou = 0
     ifou = 0
 
-! 4.1.1 LIMITATION PAR LE COURANT
-! =============================
+    ! 4.1.1 Courant limitation
+    ! ========================
 
-    if (coumax.gt.0.d0.and.iconv(iu).ge.1) then
+    if (coumax.gt.0.d0.and.vcopt_u%iconv.ge.1) then
 
       ! ICOU = 1 marque l'existence d'une limitation par le COURANT
       icou = 1
 
-      ! CONSTRUCTION DE U/DX           (COURANT       ) =W1
+      ! CONSTRUCTION DE U/DX (COURANT) = W1
 
       idiff0 = 0
 
@@ -355,19 +343,18 @@ if (idtvar.ge.0) then
       isym = 2
 
       call matrdt &
-      !==========
- (iconv(iu), idiff0, isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+     ( vcopt_u%iconv, idiff0, isym, coefbt, cofbft, imasfl, bmasfl, viscf, &
+       viscb, dam)
 
       do iel = 1, ncel
-        rom = crom(iel)
-        w1    (iel) = dam(iel)/(rom*volume(iel))
+        w1(iel) = dam(iel)/(crom(iel)*volume(iel))
       enddo
 
-      ! ---> CALCUL DE W1     = PAS DE TEMPS VARIABLE VERIFIANT
-      !       LE NOMBRE DE COURANT         MAXIMUM PRESCRIT PAR L'UTILISATEUR
+      ! ---> CALCUL DE W1 = PAS DE TEMPS VARIABLE VERIFIANT
+      !       LE NOMBRE DE COURANT MAXIMUM PRESCRIT PAR L'UTILISATEUR
 
       do iel = 1, ncel
-        w1    (iel) = coumax/max(w1    (iel), epzero)
+        w1(iel) = coumax/max(w1(iel), epzero)
       enddo
 
       ! PAS DE TEMPS UNIFORME : ON PREND LE MINIMUM DE LA CONTRAINTE
@@ -378,8 +365,7 @@ if (idtvar.ge.0) then
           w1min = min(w1min,w1(iel))
         enddo
         if (irangp.ge.0) then
-          call parmin (w1min)
-          !==========
+          call parmin(w1min)
         endif
         do iel = 1, ncel
           w1(iel) = w1min
@@ -388,10 +374,10 @@ if (idtvar.ge.0) then
 
     endif
 
-    ! 4.1.2 LIMITATION PAR LE FOURIER
-    ! =============================
+    ! 4.1.2 Fourier limitation
+    ! ========================
 
-    if (foumax.gt.0.d0.and.idiff(iu).ge.1) then
+    if (foumax.gt.0.d0.and.vcopt_u%idiff.ge.1) then
 
       ! IFOU = 1 marque l'existence d'une limitation par le FOURIER
       ifou = 1
@@ -399,24 +385,24 @@ if (idtvar.ge.0) then
       iconv0 = 0
       !                              2
       ! CONSTRUCTION DE      +2.NU/DX  (       FOURIER) =W2
-
       ! Matrice a priori symetrique
       isym = 1
 
       call matrdt &
       !==========
- (iconv0, idiff(iu), isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+ (iconv0, vcopt_u%idiff, isym, coefbt, cofbft, imasfl, bmasfl, &
+  viscf, viscb, dam)
 
       do iel = 1, ncel
-        rom = crom(iel)
-        w2    (iel) = dam(iel)/(rom*volume(iel))
+        w2(iel) = dam(iel)/(crom(iel)*volume(iel))
       enddo
 
+
       ! ---> CALCUL DE W2     = PAS DE TEMPS VARIABLE VERIFIANT
       !       LE NOMBRE DE         FOURIER MAXIMUM PRESCRIT PAR L'UTILISATEUR
 
       do iel = 1, ncel
-        w2    (iel) = foumax/max(w2    (iel), epzero)
+        w2(iel) = foumax/max(w2(iel), epzero)
       enddo
 
       ! ---> PAS DE TEMPS UNIFORME : ON PREND LE MINIMUM DE LA CONTRAINTE
@@ -463,7 +449,6 @@ if (idtvar.ge.0) then
         enddo
         if (irangp.ge.0) then
           call parmin (w3min)
-          !==========
         endif
         do iel = 1, ncel
           dam(iel) = w3min
@@ -503,7 +488,7 @@ if (idtvar.ge.0) then
     !              DESCENTE  IMMEDIATE   DU PAS DE TEMPS
 
     do iel = 1, ncel
-      if (w1    (iel).ge.dt(iel)) then
+      if (w1(iel).ge.dt(iel)) then
         unpvdt = 1.d0+varrdt
         dt(iel) = min(unpvdt*dt(iel), w1(iel))
       else
@@ -560,8 +545,8 @@ if (idtvar.ge.0) then
     ! 4.1.7 ON CLIPPE LE PAS DE TEMPS PAR RAPPORT A DTMIN ET DTMAX
     ! ==========================================================
 
-    icfmin = 0
-    icfmax = 0
+    icfmin(1) = 0
+    icfmax(1) = 0
 
     call field_get_id('dt', flid)
 
@@ -570,11 +555,11 @@ if (idtvar.ge.0) then
       dtloc = dt(1)
       if (dtloc.gt.dtmax) then
         dtloc = dtmax
-        icfmax = ncel
+        icfmax(1) = ncel
       endif
       if (dtloc.lt.dtmin) then
         dtloc = dtmin
-        icfmin = ncel
+        icfmin(1) = ncel
       endif
 
       ntcam1 = ntcabs - 1
@@ -584,10 +569,10 @@ if (idtvar.ge.0) then
         call csexit(1)
       endif
 
-      call log_iteration_clipping_field(flid, icfmin, icfmax, dt, dt)
+      call log_iteration_clipping_field(flid, icfmin(1), icfmax(1), dt, dt,icfmin(1), icfmax(1))
 
       ttcabs = ttcabs + (dtloc - dt(1))
-      if (imobil.eq.1 .or. iturbo.eq.2) then
+      if (iturbo.eq.2) then
         ttcmob = ttcmob + (dtloc - dt(1))
       endif
 
@@ -606,29 +591,29 @@ if (idtvar.ge.0) then
         vmax(1) = max(vmax(1),dt(iel))
 
         if (dt(iel).gt.dtmax) then
-          icfmax = icfmax +1
+          icfmax(1) = icfmax(1) +1
           dt(iel) = dtmax
         endif
         if (dt(iel).lt.dtmin) then
-          icfmin = icfmin +1
+          icfmin(1) = icfmin(1) +1
           dt(iel) = dtmin
         endif
 
       enddo
 
-      call log_iteration_clipping_field(flid, icfmin, icfmax, vmin, vmax)
+      call log_iteration_clipping_field(flid, icfmin(1), icfmax(1), vmin, vmax,icfmin(1), icfmax(1))
 
     endif
 
 
     if (iwarnp.ge.2) then
       if (irangp.ge.0) then
-        call parcpt (icfmin)
+        call parcpt (icfmin(1))
         !==========
-        call parcpt (icfmax)
+        call parcpt (icfmax(1))
         !==========
       endif
-      write (nfecra,1003) icfmin,dtmin,icfmax,dtmax
+      write (nfecra,1003) icfmin(1),dtmin,icfmax(1),dtmax
     endif
 
   endif
@@ -657,7 +642,7 @@ if (idtvar.ge.0) then
 ! 4.2  CALCUL DU NOMBRE DE COURANT POUR AFFICHAGE
 !===============================================================================
 
-  if (iconv(iu).ge.1 .and. ipccou.ge.1) then
+  if (vcopt_u%iconv.ge.1 .and. icour.ge.1) then
 
     idiff0 = 0
     cnom   =' COURANT'
@@ -670,54 +655,61 @@ if (idtvar.ge.0) then
 
     call matrdt &
     !==========
- (iconv(iu), idiff0, isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+ (vcopt_u%iconv, idiff0, isym, coefbt, cofbft, imasfl, bmasfl, &
+  viscf, viscb, dam)
 
     do iel = 1, ncel
-      rom = crom(iel)
-      w1    (iel) = dam(iel)/(rom*volume(iel))
+      w1(iel) = dam(iel)/(crom(iel)*volume(iel))
     enddo
 
     ! CALCUL DU NOMBRE DE COURANT/FOURIER MAXIMUM ET MINIMUM
 
     cfmax = -grand
     cfmin =  grand
-    icfmax= 1
-    icfmin= 1
+    icfmax(1)= 1
+    icfmin(1)= 1
 
     do iel = 1, ncel
-      propce(iel,ipccou) = w1(iel)*dt(iel)
+      cpro_cour(iel) = w1(iel)*dt(iel)
     enddo
 
     if (iwarnp.ge.2) then
 
       do iel = 1, ncel
-        if (propce(iel,ipccou).le.cfmin) then
-          cfmin  = propce(iel,ipccou)
-          icfmin = iel
+        if (cpro_cour(iel).le.cfmin) then
+          cfmin  = cpro_cour(iel)
+          icfmin(1) = iel
         endif
-        if (propce(iel,ipccou).ge.cfmax) then
-          cfmax  = propce(iel,ipccou)
-          icfmax = iel
+        if (cpro_cour(iel).ge.cfmax) then
+          cfmax  = cpro_cour(iel)
+          icfmax(1) = iel
         endif
       enddo
 
-      xyzmin(1) = xyzcen(1,icfmin)
-      xyzmin(2) = xyzcen(2,icfmin)
-      xyzmin(3) = xyzcen(3,icfmin)
-      xyzmax(1) = xyzcen(1,icfmax)
-      xyzmax(2) = xyzcen(2,icfmax)
-      xyzmax(3) = xyzcen(3,icfmax)
+      xyzmin(1) = xyzcen(1,max(icfmin(1), 1))
+      xyzmin(2) = xyzcen(2,max(icfmin(1), 1))
+      xyzmin(3) = xyzcen(3,max(icfmin(1), 1))
+      xyzmax(1) = xyzcen(1,max(icfmax(1), 1))
+      xyzmax(2) = xyzcen(2,max(icfmax(1), 1))
+      xyzmax(3) = xyzcen(3,max(icfmax(1), 1))
 
       if (irangp.ge.0) then
         nbrval = 3
         call parmnl (nbrval, cfmin, xyzmin)
-        !==========
         call parmxl (nbrval, cfmax, xyzmax)
-        !==========
       endif
 
-      write(nfecra,1001) cnom, cfmax, xyzmax(1), xyzmax(2), xyzmax(3)
-      write(nfecra,1002) cnom, cfmin, xyzmin(1), xyzmin(2), xyzmin(3)
+      if (icfmin(1).gt.0) then
+        write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
+
+      if (icfmax(1).gt.0) then
+        write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
 
     endif
 
@@ -727,10 +719,10 @@ if (idtvar.ge.0) then
 ! 4.3  CALCUL DU NOMBRE DE FOURIER POUR AFFICHAGE
 !===============================================================================
 
-  if (idiff(iu).ge.1 .and. ipcfou.ge.1) then
+  if (vcopt_u%idiff.ge.1 .and. ifour.ge.1) then
 
     iconv0 = 0
-    CNOM   =' FOURIER'
+    cnom   =' FOURIER'
 
     !                              2
     ! CONSTRUCTION DE      +2.NU/DX  (       FOURIER) =W1
@@ -741,43 +733,43 @@ if (idtvar.ge.0) then
 
     call matrdt &
     !==========
- (iconv0, idiff(iu), isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+ (iconv0, vcopt_u%idiff, isym, coefbt, cofbft, imasfl, bmasfl, &
+  viscf, viscb, dam)
 
     do iel = 1, ncel
-      rom = crom(iel)
-      w1    (iel) = dam(iel)/(rom*volume(iel))
+      w1(iel) = dam(iel)/(crom(iel)*volume(iel))
     enddo
 
     ! CALCUL DU NOMBRE DE COURANT/FOURIER MAXIMUM ET MINIMUM
 
     cfmax  = -grand
     cfmin  =  grand
-    icfmax = 0
-    icfmin = 0
+    icfmax(1) = 0
+    icfmin(1) = 0
 
     do iel = 1, ncel
-      propce(iel,ipcfou) = w1(iel)*dt(iel)
+      cpro_four(iel) = w1(iel)*dt(iel)
     enddo
 
     if (iwarnp.ge.2) then
 
       do iel = 1, ncel
-        if (propce(iel,ipcfou).le.cfmin) then
-          cfmin  = propce(iel,ipcfou)
-          icfmin = iel
+        if (cpro_four(iel).le.cfmin) then
+          cfmin  = cpro_four(iel)
+          icfmin(1) = iel
         endif
-        if (propce(iel,ipcfou).ge.cfmax) then
-          cfmax  = propce(iel,ipcfou)
-          icfmax = iel
+        if (cpro_four(iel).ge.cfmax) then
+          cfmax  = cpro_four(iel)
+          icfmax(1) = iel
         endif
       enddo
 
-      xyzmin(1) = xyzcen(1,icfmin)
-      xyzmin(2) = xyzcen(2,icfmin)
-      xyzmin(3) = xyzcen(3,icfmin)
-      xyzmax(1) = xyzcen(1,icfmax)
-      xyzmax(2) = xyzcen(2,icfmax)
-      xyzmax(3) = xyzcen(3,icfmax)
+      xyzmin(1) = xyzcen(1,icfmin(1))
+      xyzmin(2) = xyzcen(2,icfmin(1))
+      xyzmin(3) = xyzcen(3,icfmin(1))
+      xyzmax(1) = xyzcen(1,icfmax(1))
+      xyzmax(2) = xyzcen(2,icfmax(1))
+      xyzmax(3) = xyzcen(3,icfmax(1))
 
       if (irangp.ge.0) then
         nbrval = 3
@@ -801,9 +793,8 @@ if (idtvar.ge.0) then
 !     En incompressible uniquement (en compressible, on preferera
 !       afficher la contrainte liee a la masse volumique)
 
-  if ((iwarnp.ge.2.or.modntl.eq.0)                         &
-      .and. (idiff(iu).ge.1.or.iconv(iu).ge.1)             &
-      .and. (ippmod(icompf).lt.0))              then
+  if (      (vcopt_u%idiff.ge.1.or.vcopt_u%iconv.ge.1) &
+      .and. (ippmod(icompf).lt.0)) then
 
     cnom   =' COU/FOU'
 
@@ -813,23 +804,23 @@ if (idtvar.ge.0) then
     ! MATRICE A PRIORI NON SYMETRIQUE
 
     isym = 1
-    if (iconv(iu).gt.0) isym = 2
+    if (vcopt_u%iconv.gt.0) isym = 2
 
     call matrdt &
     !==========
- (iconv(iu), idiff(iu), isym, coefbt, cofbft, imasfl, bmasfl, viscf, viscb, dam)
+ (vcopt_u%iconv, vcopt_u%idiff, isym, coefbt, cofbft, imasfl, bmasfl, &
+  viscf, viscb, dam)
 
     do iel = 1, ncel
-      rom = crom(iel)
-      w1    (iel) = dam(iel)/(rom*volume(iel))
+      w1(iel) = dam(iel)/(crom(iel)*volume(iel))
     enddo
 
     ! CALCUL DU NOMBRE DE COURANT/FOURIER MAXIMUM ET MINIMUM
 
     cfmax  = -grand
     cfmin  =  grand
-    icfmax = 0
-    icfmin = 0
+    icfmax(1) = 0
+    icfmin(1) = 0
 
     do iel = 1, ncel
       w2(iel) = w1(iel)*dt(iel)
@@ -844,31 +835,38 @@ if (idtvar.ge.0) then
       do iel = 1, ncel
         if (w2(iel).le.cfmin) then
           cfmin  = w2(iel)
-          icfmin = iel
+          icfmin(1) = iel
         endif
         if (w2(iel).ge.cfmax) then
           cfmax  = w2(iel)
-          icfmax = iel
+          icfmax(1) = iel
         endif
       enddo
 
-      xyzmin(1) = xyzcen(1,icfmin)
-      xyzmin(2) = xyzcen(2,icfmin)
-      xyzmin(3) = xyzcen(3,icfmin)
-      xyzmax(1) = xyzcen(1,icfmax)
-      xyzmax(2) = xyzcen(2,icfmax)
-      xyzmax(3) = xyzcen(3,icfmax)
+      xyzmin(1) = xyzcen(1,max(icfmin(1), 1))
+      xyzmin(2) = xyzcen(2,max(icfmin(1), 1))
+      xyzmin(3) = xyzcen(3,max(icfmin(1), 1))
+      xyzmax(1) = xyzcen(1,max(icfmax(1), 1))
+      xyzmax(2) = xyzcen(2,max(icfmax(1), 1))
+      xyzmax(3) = xyzcen(3,max(icfmax(1), 1))
 
       if (irangp.ge.0) then
         nbrval = 3
         call parmnl (nbrval, cfmin, xyzmin)
-        !==========
         call parmxl (nbrval, cfmax, xyzmax)
-        !==========
       endif
 
-      write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
-      write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      if (icfmin(1).gt.0) then
+        write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
+
+      if (icfmax(1).gt.0) then
+        write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
 
     endif
 
@@ -880,7 +878,7 @@ if (idtvar.ge.0) then
 
   ! En Compressible uniquement
 
-  if ((iwarnp.ge.2.or.modntl.eq.0) .and. (ippmod(icompf).ge.0)) then
+  if (ippmod(icompf).ge.0) then
 
     cnom =' CFL/MAS'
 
@@ -888,8 +886,8 @@ if (idtvar.ge.0) then
 
     cfmax  = -grand
     cfmin  =  grand
-    icfmax = 0
-    icfmin = 0
+    icfmax(1) = 0
+    icfmin(1) = 0
 
     do iel = 1, ncel
       w2(iel) = wcf(iel)*dt(iel)
@@ -904,31 +902,38 @@ if (idtvar.ge.0) then
       do iel = 1, ncel
         if (w2(iel).le.cfmin) then
           cfmin  = w2(iel)
-          icfmin = iel
+          icfmin(1) = iel
         endif
         if (w2(iel).ge.cfmax) then
           cfmax  = w2(iel)
-          icfmax = iel
+          icfmax(1) = iel
         endif
       enddo
 
-      xyzmin(1) = xyzcen(1,icfmin)
-      xyzmin(2) = xyzcen(2,icfmin)
-      xyzmin(3) = xyzcen(3,icfmin)
-      xyzmax(1) = xyzcen(1,icfmax)
-      xyzmax(2) = xyzcen(2,icfmax)
-      xyzmax(3) = xyzcen(3,icfmax)
+      xyzmin(1) = xyzcen(1,max(icfmin(1), 1))
+      xyzmin(2) = xyzcen(2,max(icfmin(1), 1))
+      xyzmin(3) = xyzcen(3,max(icfmin(1), 1))
+      xyzmax(1) = xyzcen(1,max(icfmax(1), 1))
+      xyzmax(2) = xyzcen(2,max(icfmax(1), 1))
+      xyzmax(3) = xyzcen(3,max(icfmax(1), 1))
 
       if (irangp.ge.0) then
         nbrval = 3
         call parmnl (nbrval, cfmin, xyzmin)
-        !==========
         call parmxl (nbrval, cfmax, xyzmax)
-        !==========
       endif
 
-      write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
-      write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      if (icfmin(1).gt.0) then
+        write(nfecra,1001) cnom,cfmax,xyzmax(1),xyzmax(2),xyzmax(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
+
+      if (icfmax(1).gt.0) then
+        write(nfecra,1002) cnom,cfmin,xyzmin(1),xyzmin(2),xyzmin(3)
+      else
+        write(nfecra,*) cnom, "Too big to be displayed"
+      endif
 
     endif
 
@@ -940,17 +945,17 @@ if (idtvar.ge.0) then
 else
 
   isym = 1
-  if (iconv(iu).gt.0) isym = 2
+  if (vcopt_u%iconv.gt.0) isym = 2
 
   call matrdt &
   !==========
- ( iconv(iu), idiff(iu), isym,                                  &
+ ( vcopt_u%iconv, vcopt_u%idiff, isym,                          &
    coefbt, cofbft,                                              &
    imasfl, bmasfl,                                              &
    viscf, viscb, dt )
 
   do iel = 1, ncel
-    dt(iel) =  relaxv(iu)*crom(iel)*volume(iel)/max(dt(iel),epzero)
+    dt(iel) =  vcopt_u%relaxv*crom(iel)*volume(iel)/max(dt(iel),epzero)
   enddo
 
 endif
