@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,35 +26,14 @@
 
 module field_operator
 
+  use field
+
   !=============================================================================
 
   implicit none
 
   !=============================================================================
 
-  integer :: FIELD_INTENSIVE, FIELD_EXTENSIVE
-  integer :: FIELD_VARIABLE, FIELD_PROPERTY
-  integer :: FIELD_POSTPROCESS, FIELD_ACCUMULATOR, FIELD_USER
-
-  integer :: FIELD_OK, FIELD_INVALID_KEY_NAME, FIELD_INVALID_KEY_ID,   &
-             FIELD_INVALID_CATEGORY, FIELD_INVALID_TYPE
-
-  parameter (FIELD_INTENSIVE=1)
-  parameter (FIELD_EXTENSIVE=2)
-  parameter (FIELD_VARIABLE=4)
-  parameter (FIELD_PROPERTY=8)
-  parameter (FIELD_POSTPROCESS=16)
-  parameter (FIELD_ACCUMULATOR=32)
-  parameter (FIELD_USER=64)
-
-  parameter (FIELD_OK=0)
-  parameter (FIELD_INVALID_KEY_NAME=1)
-  parameter (FIELD_INVALID_KEY_ID=2)
-  parameter (FIELD_INVALID_CATEGORY=3)
-  parameter (FIELD_INVALID_TYPE=4)
-
-  !=============================================================================
-
   interface
 
     !> \brief  Compute cell gradient of scalar field or component of vector or
@@ -130,13 +109,49 @@ module field_operator
 
     !---------------------------------------------------------------------------
 
+    !> \brief  Compute cell gradient of tensor field.
+
+    !> \param[in]   f_id             field id
+    !> \param[in]   use_previous_t   1 if values at previous time step should
+    !>                               be used, 0 otherwise
+    !> \param[in]   imrgra           gradient computation mode
+    !> \param[in]   inc              0: increment; 1: do not increment
+    !> \param[out]  grad             gradient
+
+    subroutine field_gradient_tensor(f_id, use_previous_t, imrgra, inc,        &
+                                     grad)                                     &
+      bind(C, name='cs_f_field_gradient_tensor')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      integer(c_int), value                 :: f_id, use_previous_t, imrgra, inc
+      real(kind=c_double), dimension(6,3,*) :: grad
+    end subroutine field_gradient_tensor
+
+    !---------------------------------------------------------------------------
+
+    !> \brief  Shift field values in order to set its spatial average to a given
+    !>         value.
+
+    !> \param[in]   f_id  field id
+    !> \param[in]   va    real value of volume average to be set
+
+    subroutine field_set_volume_average(f_id, va)                              &
+      bind(C, name='cs_f_field_set_volume_average')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      integer(c_int), value      :: f_id
+      real(kind=c_double), value :: va
+    end subroutine field_set_volume_average
+
+    !---------------------------------------------------------------------------
+
     !> \cond DOXYGEN_SHOULD_SKIP_THIS
 
     !---------------------------------------------------------------------------
 
     !---------------------------------------------------------------------------
 
-    !> \endcond DOXYGEN_SHOULD_SKIP_THIS
+    !> (DOXYGEN_SHOULD_SKIP_THIS) \endcond
 
     !---------------------------------------------------------------------------
 
