@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,37 +20,41 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine findpt &
-!================
-
- ( ncelet , ncel   , xyzcen ,                                     &
-   xx     , yy     , zz     , node   , ndrang)
-
 !===============================================================================
+! Function:
+! ---------
 
-! FONCTION
-! --------
-!  RECHERCHE DU NOEUD LE PLUS PROCHE DU POINT XX,YY,ZZ
+!> \file findpt.f90
+!>
+!> \brief This subroutine looks for the nearest element to the position
+!>  (xx, yy, zz) among the element of xyzcen array.
+!>
+!-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! ncelet           ! i  ! <-- ! number of extended (real + ghost) cells        !
-! ncel             ! i  ! <-- ! number of cells                                !
-! xyzcen(ndim      ! tr !  -->! table des coordonnees du                       !
-!        ncelet    !    !     !           centre des volumes                   !
-! xx,yy,zz         ! tr !  -->! coordonnees du noeud cherche                   !
-! node             ! e  ! --> ! noeud cherche (numerotation globale)           !
-!                  !    !     !  zero si plantage                              !
-! ndrang           ! e  ! --> ! rang du processus associe                      !
-!__________________.____._____.________________________________________________.
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]     ncelet        number of extended (real + ghost) elements
+!> \param[in]     ncel          number of element
+!> \param[in]     xyzcen        interleaved array containing vector positions
+!> \param[in]     xx            X component of the point position to be compared
+!>                              with
+!> \param[in]     yy            Y component of the point position to be compared
+!>                              with
+!> \param[in]     zz            Z component of the point position to be compared
+!>                              with
+!> \param[in]     node          element of xyzcen which is the nearest to
+!>                              (xx,yy,zz)
+!> \param[in]     ndrang        index of the Rank containing the nearest element
+!_______________________________________________________________________________
+
+
+subroutine findpt &
+ ( ncelet , ncel   , xyzcen ,                                     &
+   xx     , yy     , zz     , node   , ndrang)
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
 !===============================================================================
 
 !===============================================================================
@@ -73,7 +77,7 @@ integer          ii
 double precision xx1, yy1, zz1, dis2, dis2mn
 
 !===============================================================================
-! 1. INITIALISATION
+! 1. Initialization
 !===============================================================================
 
 node = int((ncel+1)/2)
@@ -84,22 +88,21 @@ zz1 = xyzcen(3,node)
 dis2mn = (xx-xx1)**2+(yy-yy1)**2+(zz-zz1)**2
 
 do ii = 1, ncel
-   xx1 = xyzcen(1,ii)
-   yy1 = xyzcen(2,ii)
-   zz1 = xyzcen(3,ii)
-   dis2 = (xx-xx1)**2+(yy-yy1)**2+(zz-zz1)**2
-   if (dis2.lt.dis2mn) then
-      node = ii
-      dis2mn = dis2
-   endif
+  xx1 = xyzcen(1,ii)
+  yy1 = xyzcen(2,ii)
+  zz1 = xyzcen(3,ii)
+  dis2 = (xx-xx1)**2+(yy-yy1)**2+(zz-zz1)**2
+  if (dis2.lt.dis2mn) then
+    node = ii
+    dis2mn = dis2
+  endif
 enddo
 
 if (irangp.ge.0) then
-   call parfpt (node, ndrang, dis2mn)
-   !==========
+  call parfpt (node, ndrang, dis2mn)
 else
   ndrang = -1
 endif
 
 return
-end subroutine
+end subroutine findpt
