@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -35,9 +35,16 @@ use paramx
 use optcal
 use entsor
 use ihmpre
-use ppincl, only: ippmod, nmodmx
+use ppincl, only: ippmod, nmodmx, iatmos
 use post
 use cs_c_bindings
+use field
+use lagran
+use cpincl
+use dimens
+use radiat
+use cs_fuel_incl
+use cdomod
 
 !===============================================================================
 
@@ -48,17 +55,30 @@ implicit none
 ! Local variables
 
 integer          iok, ipp, nmodpp
+
 double precision ttsuit, wtsuit
 
 !===============================================================================
 
 interface
 
-  subroutine gui_postprocess_fields()  &
-      bind(C, name='cs_gui_postprocess_fields')
+  subroutine gui_output()  &
+      bind(C, name='cs_gui_output')
     use, intrinsic :: iso_c_binding
     implicit none
-  end subroutine gui_postprocess_fields
+  end subroutine gui_output
+
+  subroutine user_finalize_setup_wrapper()  &
+      bind(C, name='cs_user_finalize_setup_wrapper')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine user_finalize_setup_wrapper
+
+  subroutine user_syrthes_coupling()  &
+      bind(C, name='cs_user_syrthes_coupling')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine user_syrthes_coupling
 
 end interface
 
@@ -74,7 +94,6 @@ end interface
 !===============================================================================
 
 call iniini
-!==========
 
 nmodpp = 0
 do ipp = 2, nmodmx
@@ -85,77 +104,92 @@ enddo
 
 !===============================================================================
 ! 2. ENTREE DES DONNEES PAR L'UTILISATEUR
-!      ET POSITIONNEMENT DES VARIABLES (VARPOS)
+!      ET POSITIONNEMENT DES VARIABLES
 !===============================================================================
 
 call iniusi
-!==========
 
 call ppini1
-!==========
 
-call usipes(nmodpp)
-!==========
+!===============================================================================
+! Map Fortran pointers to C global data
+!===============================================================================
+
+call elec_option_init
+
+call cs_rad_transfer_options
+
+if (ippmod(iatmos).ge.0) call cs_at_data_assim_initialize
+
+! Lagrangian model options
+
+call lagran_init_map
+
+call lagopt(isuite, iccvfg, iscalt, dtref)
+
+! Additional fields if not in CDO mode only
+
+if (icdo.lt.2) then
+   call addfld
+endif
+
+! Time moments
+
+call cs_gui_time_moments
+call cs_user_time_moments
+
+! Postprocessing and logging
+
+if (iihmpr.eq.1) then
+  call gui_output
+endif
+
+! Restart
 
 ttsuit = -1.d0
 wtsuit = -1.d0
 
 call dflsui(ntsuit, ttsuit, wtsuit);
-!==========
-
-call rayopt
-!==========
-
-call lagopt
-!==========
 
 !===============================================================================
-! 3. DEFINITION DES COUPLAGES AVEC SYRTHES
+! 3. MODIFS APRES USINI1
 !===============================================================================
 
-! Le nombre de couplage SYRTHES doit etre connu avant MODINI a des fins
-! de verification de coherence avec la definition des scalaires
-
-if (iihmpr.eq.1) then
-  call uisyrc
-  !==========
+! Do not call this routine if CDO mode only (default variables and properties
+! are not defined anymore)
+if (icdo.lt.2) then
+   call modini
 endif
 
-call ussyrc
-!==========
-
-call ussatc
-!==========
-
 !===============================================================================
-! 4. MODIFS APRES USINI1
+! 4. Some additional fields and mappings
 !===============================================================================
 
-call modini
-!==========
+! Do not call this routine if CDO mode only (default variables and properties
+! are not defined anymore)
+if (icdo.lt.2) then
+   call fldini
+   call usipes(nmodpp)
+
+   ! Avoid a second spurious call to this function
+   ! Call in the C part if CDO is activated, i.e. when
+   ! additional geometric quantities and connectivities are built
+   if (icdo.lt.0) then
+      call user_finalize_setup_wrapper
+   endif
+endif
 
 !===============================================================================
-! 5. Initial definition of fields
-!===============================================================================
-
-call fldini
-!==========
-
-call gui_postprocess_fields
-
-call user_field_parameters
-!=========================
-
-call addfld
-
-!===============================================================================
-! 6. Coherency checks
+! 5. Coherency checks
 !===============================================================================
 
 iok = 0
 
-call verini (iok)
-!==========
+! No verification in CDO mode only. This done elsewhere
+if (icdo.lt.2) then
+   call verini (iok)
+   call parameters_check
+endif
 
 if(iok.gt.0) then
   write(nfecra,9999)iok
@@ -220,7 +254,6 @@ endif
 !===============================================================================
 
 call impini
-!==========
 
 return
 end subroutine
