@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -65,7 +65,7 @@ implicit none
 write(nfecra,1000)
 
 if (almax.le.0.d0) then
-  almax = voltot**.333d0
+  almax = voltot**(1.d0/3.d0)
   write(nfecra,1100) almax
   write(nfecra,1102)
   if(itytur.eq.2.or.itytur.eq.3                   &
