@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -33,19 +33,11 @@
 !------------------------------------------------------------------------------
 !   mode          name          role
 !------------------------------------------------------------------------------
-!> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
-!> \param[out]    dt            time step value
-!> \param[out]    rtp           calculation variables at cells center
-!> \param[out]    propce        physical properties at cell centers
-!> \param[out]    frcxt         external stress generating hydrostatic pressure
-!> \param[out]    prhyd         hydrostatic pressure predicted
 !______________________________________________________________________________
 
 subroutine iniva0 &
- ( nvar   , nscal  ,                                              &
-   dt     , rtp    , propce ,                                     &
-   frcxt  , prhyd)
+ ( nscal  )
 
 !===============================================================================
 ! Module files
@@ -68,6 +60,10 @@ use cplsat
 use field
 use mesh
 use cavitation
+use vof
+use cs_cf_bindings
+use cs_c_bindings
+use darcy_module
 
 !===============================================================================
 
@@ -75,60 +71,70 @@ implicit none
 
 ! Arguments
 
-integer          nvar   , nscal
-
-double precision frcxt(3,ncelet), prhyd(ncelet)
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), propce(ncelet,*)
+integer          nscal
 
 ! Local variables
 
-integer          iis   , ivar  , iscal , imom
-integer          iel   , ifac  , isou
-integer          iclip , ii    , jj    , idim
-integer          iivisl, iivist, iivisa, iivism
-integer          iicp  , iicpa
-integer          iiviss, iiptot
-integer          iptsna, iptsta, iptsca
-integer          nn
+integer          iis   , iscal
+integer          iel   , ifac
+integer          iclip , ii    , jj    , idim, f_dim
+integer          ifcvsl
 integer          iflid, nfld, ifmaip, bfmaip, iflmas, iflmab
-integer          f_id, f_id_prv, f_dim
 integer          kscmin, kscmax
+integer          f_type, idftnp
+integer          keyvar
 
-logical          interleaved
+logical          have_previous
 
-double precision xxk, xcmu, trii
+double precision xxk, xcmu, trii, clvfmn
 
-double precision rvoid(1)
-
-double precision, dimension(:), pointer :: brom, crom, crom_prev2
+double precision, dimension(:), pointer :: dt
+double precision, dimension(:), pointer :: brom, crom
 double precision, dimension(:), pointer :: cofbcp
 double precision, dimension(:), pointer :: porosi
 double precision, dimension(:,:), pointer :: porosf
 double precision, dimension(:), pointer :: field_s_v
-double precision, dimension(:,:), pointer :: field_v_v
+
+double precision, dimension(:), pointer :: cvar_pr
+double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_al
+double precision, dimension(:), pointer :: cvar_phi, cvar_fb, cvar_omg, cvar_nusa
+double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
+double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
+double precision, dimension(:,:), pointer :: cvar_rij
+double precision, dimension(:), pointer :: viscl, visct, cpro_cp, cpro_prtot
+double precision, dimension(:), pointer :: cpro_viscls, cproa_viscls, cvar_tempk
+double precision, dimension(:), pointer :: cpro_visma_s
+double precision, dimension(:), pointer :: mix_mol_mas
+double precision, dimension(:,:), pointer :: cpro_visma_v
+
+type(var_cal_opt) :: vcopt_uma
 
 !===============================================================================
+! Interfaces
+!===============================================================================
+
+interface
+
+  subroutine cs_gui_mesh_viscosity()  &
+    bind(C, name='cs_gui_mesh_viscosity')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_gui_mesh_viscosity
+
+end interface
 
 !===============================================================================
 ! 1.  INITIALISATION
 !===============================================================================
 
+call field_get_key_id("variable_id", keyvar)
+
+call field_get_val_s_by_name('dt', dt)
+
 ! Initialize variables to avoid compiler warnings
 
 jj = 0
 
-! Memoire
-
-
-! En compressible, ISYMPA initialise (= 1) car utile dans le calcul
-!     du pas de temps variable avant passage dans les C.L.
-
-if ( ippmod(icompf).ge.0 ) then
-  do ifac = 1, nfabor
-    isympa(ifac) = 1
-  enddo
-endif
-
 !===============================================================================
 ! 2. PAS DE TEMPS
 !===============================================================================
@@ -143,127 +149,132 @@ enddo
 ! 3.  INITIALISATION DES PROPRIETES PHYSIQUES
 !===============================================================================
 
+call field_get_val_s(ivarfl(ipr), cvar_pr)
+
 !     Masse volumique
 call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
 
 !     Masse volumique aux cellules (et au pdt precedent si ordre2 ou icalhy
-!     ou cavitation)
+!     ou algo. VOF)
 do iel = 1, ncel
   crom(iel)  = ro0
 enddo
-if (iroext.gt.0.or.icalhy.eq.1.or.idilat.gt.1.or.icavit.ge.0) then
-  call field_current_to_previous(icrom)
-endif
-if (icavit.ge.0) then
-  call field_get_val_s(icroaa, crom_prev2)
-  do iel = 1, ncelet
-    crom_prev2(iel) = crom(iel)
-  enddo
-endif
+
 !     Masse volumique aux faces de bord (et au pdt precedent si ordre2)
 do ifac = 1, nfabor
   brom(ifac) = ro0
 enddo
-if (iroext.gt.0.or.icavit.ge.0) then
-  call field_current_to_previous(ibrom)
-endif
 
-!     Viscosite moleculaire
-iivisl = ipproc(iviscl)
-iivist = ipproc(ivisct)
+! Note: for VOF or dilatable algorithms, density at twice previous time step is
+! also stored and written here with "current to previous" function
+call field_current_to_previous(icrom)
+call field_current_to_previous(icrom)
+call field_current_to_previous(ibrom)
+call field_current_to_previous(ibrom)
 
-!     Viscosite moleculaire aux cellules (et au pdt precedent si ordre2)
+! Molecular viscosity
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
+
+! Molecular viscosity at cells (and eventual previous value)
 do iel = 1, ncel
-  propce(iel,iivisl) = viscl0
+  viscl(iel) = viscl0
 enddo
-if(iviext.gt.0) then
-  iivisa = ipproc(ivisla)
-  do iel = 1, ncel
-    propce(iel,iivisa) = propce(iel,iivisl)
-  enddo
-endif
-!     Viscosite turbulente aux cellules (et au pdt precedent si ordre2)
-do iel = 1, ncel
-  propce(iel,iivist) = 0.d0
-enddo
-if(iviext.gt.0) then
-  iivisa = ipproc(ivista)
-  do iel = 1, ncel
-    propce(iel,iivisa) = propce(iel,iivist)
-  enddo
-endif
+call field_current_to_previous(iviscl)
 
-!     Chaleur massique aux cellules (et au pdt precedent si ordre2)
-if(icp.gt.0) then
-  iicp = ipproc(icp)
+! Specific heat at cells (and eventual previous value)
+if(icp.ge.0) then
+  call field_get_val_s(icp, cpro_cp)
   do iel = 1, ncel
-    propce(iel,iicp) = cp0
+    cpro_cp(iel) = cp0
   enddo
-  if(icpext.gt.0) then
-    iicpa  = ipproc(icpa)
-    do iel = 1, ncel
-      propce(iel,iicpa ) = propce(iel,iicp)
-    enddo
-  endif
+  call field_current_to_previous(icp)
 endif
 
 ! La pression totale sera initialisee a P0 + rho.g.r dans INIVAR
 !  si l'utilisateur n'a pas fait d'initialisation personnelle
 ! Non valable en compressible
-if (ippmod(icompf).lt.0) then
-  iiptot = ipproc(iprtot)
+! For groundwater flows, the field of index iprtot is the
+! pressure head (h = H - z). h is only used when gravity is taken
+! into account
+if ((ippmod(icompf).lt.0.and.ippmod(idarcy).lt.0).or.                          &
+    (ippmod(idarcy).ge.0.and.darcy_gravity.ge.1)) then
+  call field_get_val_s(iprtot, cpro_prtot)
   do iel = 1, ncel
-    propce(iel,iiptot) = - rinfin
+    cpro_prtot(iel) = - rinfin
   enddo
 endif
 
+! Initialization of mix_mol_mas with default values (air)
+! (used in cs_cf_thermo_default_init)
+if(ippmod(igmix).ge.0) then
+  call field_get_val_s(igmxml, mix_mol_mas)
+  do iel =1, ncel
+    mix_mol_mas(iel) = xmasmr
+  enddo
+endif
+
+! Default initialisations for the compressible model
+if (ippmod(icompf).ge.0) then
+  ! In compressible, for now, the temperature is not solved but is a field of
+  ! type variable anyway. The reference value has to be taken into account.
+  call field_get_val_s(ivarfl(isca(itempk)), cvar_tempk)
+  do iel = 1, ncel
+    cvar_tempk(iel) = t0
+  enddo
+
+  ! Default isochoric specific heat (cv0),
+  ! total energy and density
+  call cs_cf_thermo_default_init
+
+  ! Default diffusivity for total energy
+  visls0(ienerg) = visls0(itempk)/cv0
+endif
+
 ! Diffusivite des scalaires
 do iscal = 1, nscal
-  if (ivisls(iscal).gt.0) then
-    iiviss = ipproc(ivisls(iscal))
-    ! Diffusivite aux cellules (et au pdt precedent si ordre2)
+  call field_get_key_int (ivarfl(isca(iscal)), kivisl, ifcvsl)
+  ! Diffusivite aux cellules (et au pdt precedent si ordre2)
+  if (ifcvsl.ge.0) then
+    call field_get_val_s(ifcvsl, cpro_viscls)
     do iel = 1, ncel
-      propce(iel,iiviss) = visls0(iscal)
+      cpro_viscls(iel) = visls0(iscal)
     enddo
-    if (ivsext(iscal).gt.0) then
-      iivisa = ipproc(ivissa(iscal))
+    call field_have_previous(ifcvsl, have_previous)
+    if (have_previous) then
+      call field_get_val_prev_s(ifcvsl, cproa_viscls)
       do iel = 1, ncel
-        propce(iel,iivisa) = propce(iel,iiviss)
+        cproa_viscls(iel) = visls0(iscal)
       enddo
     endif
   endif
-
-  if (iscal.eq.iscalt.and.iturt(iscal).gt.0.and.irovar.eq.1) then
-    do iel = 1, ncelet
-      propce(iel,ipproc(ibeta)) = 0.d0
-    enddo
-  endif
 enddo
 
-! Initialisation of source terms for weakly compressible algorithm
-if (idilat.eq.4) then
-  do iel = 1, ncel
-    propce(iel,ipproc(iustdy(itsrho))) = 0.d0
-  enddo
-  do iscal = 1, nscal
-    do iel = 1, ncel
-      propce(iel,ipproc(iustdy(iscal))) = 0.d0
-    enddo
-  enddo
-endif
+! Mesh viscosity for ALE
+if (iale.ge.1) then
 
+  call field_get_key_struct_var_cal_opt(ivarfl(iuma), vcopt_uma)
+  idftnp = vcopt_uma%idften
 
-!     Viscosite de maillage en ALE
-if (iale.eq.1) then
-  nn = 1
-  if (iortvm.eq.1) nn = 3
-  do ii = 1, nn
-    iivism = ipproc(ivisma(ii))
+  if (iand(idftnp, ANISOTROPIC_LEFT_DIFFUSION).ne.0) then
+    call field_get_val_v(ivisma, cpro_visma_v)
     do iel = 1, ncel
-      propce(iel,iivism) = 1.d0
+      do ii = 1, 3
+        cpro_visma_v(ii  ,iel) = 1.d0
+        cpro_visma_v(ii+3,iel) = 0.d0
+      enddo
     enddo
-  enddo
+  else if (iand(idftnp, ISOTROPIC_DIFFUSION).ne.0) then
+    call field_get_val_s(ivisma, cpro_visma_s)
+    do iel = 1, ncel
+      cpro_visma_s(iel) = 1.d0
+    enddo
+  endif
+
+  call cs_gui_mesh_viscosity
+  call usvima
+
 endif
 
 ! Porosity
@@ -271,6 +282,8 @@ if (iporos.ge.1) then
   call field_get_val_s(ipori, porosi)
   do iel = 1, ncelet
     porosi(iel) = 1.d0
+    ! No solid cells
+    isolid_0(iel) = 0
   enddo
 
   ! Tensorial porosity
@@ -285,6 +298,9 @@ if (iporos.ge.1) then
       porosf(6, iel) = 0.d0
     enddo
   endif
+
+else
+  isolid_0(1) = 0
 endif
 
 !===============================================================================
@@ -295,50 +311,16 @@ endif
 !     On met la pression P* a PRED0
 !$omp parallel do
 do iel = 1, ncel
-  rtp(iel,ipr) = pred0
+  cvar_pr(iel) = pred0
 enddo
 
-!     Toutes les variables a 0
-f_id = -1
-
-do ivar = 1, nvar
-  f_id_prv = f_id
-  f_id = ivarfl(ivar)
-  if (f_id.eq.f_id_prv) cycle
-
-  call field_get_dim(f_id, f_dim, interleaved)
-
-  if (f_dim.gt.1) then
-    call field_get_val_v(f_id, field_v_v)
-  else if (f_dim.eq.1) then
-    call field_get_val_s(f_id, field_s_v)
-  endif
-
-  if (f_dim.gt.1) then
-    !$omp parallel do private(isou)
-    do iel = 1, ncel
-      do isou = 1, f_dim
-        field_v_v(isou,iel) = 0.d0
-      enddo
-    enddo
-  else
-    !$omp parallel do
-    do iel = 1, ncel
-      field_s_v(iel) = 0.d0
-    enddo
-  endif
-enddo
-
-! On definit les clipping du taux de vide et on initialize au clipping inf.
-if (icavit.ge.0) then
+! initialize void fraction to minimum clipping value
+if (ivofmt.gt.0) then
 
   call field_get_key_id("min_scalar_clipping", kscmin)
-  call field_get_key_id("max_scalar_clipping", kscmax)
+  call field_get_key_double(ivarfl(ivolf2), kscmin, clvfmn)
 
-  call field_set_key_double(ivarfl(ivoidf), kscmin, clvfmn)
-  call field_set_key_double(ivarfl(ivoidf), kscmax, clvfmx)
-
-  call field_get_val_s(ivarfl(ivoidf), field_s_v)
+  call field_get_val_s(ivarfl(ivolf2), field_s_v)
   do iel = 1, ncel
     field_s_v(iel) = clvfmn
   enddo
@@ -356,109 +338,161 @@ endif
 
 if(itytur.eq.2 .or. itytur.eq.5) then
 
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+
   xcmu = cmu
   if (iturb.eq.50) xcmu = cv2fmu
   if (iturb.eq.51) xcmu = cpalmu
 
   if (uref.ge.0.d0) then
     do iel = 1, ncel
-      rtp(iel,ik) = 1.5d0*(0.02d0*uref)**2
-      rtp(iel,iep) = rtp(iel,ik)**1.5d0*xcmu/almax
+      cvar_k(iel) = 1.5d0*(0.02d0*uref)**2
+      cvar_ep(iel) = cvar_k(iel)**1.5d0*xcmu/almax
     enddo
 
     iclip = 1
-    call clipke(ncelet , ncel   , nvar    ,     &
-         iclip  , iwarni(ik),                   &
-         propce , rtp    )
+    call clipke(ncelet, ncel, iclip)
 
   else
     do iel = 1, ncel
-      rtp(iel,ik) = -grand
-      rtp(iel,iep) = -grand
+      cvar_k(iel) = -grand
+      cvar_ep(iel) = -grand
     enddo
   endif
 
   if (iturb.eq.50) then
+    call field_get_val_s(ivarfl(iphi), cvar_phi)
+    call field_get_val_s(ivarfl(ifb), cvar_fb)
     do iel = 1, ncel
-      rtp(iel,iphi) = 2.d0/3.d0
-      rtp(iel,ifb) = 0.d0
+      cvar_phi(iel) = 2.d0/3.d0
+      cvar_fb(iel) = 0.d0
     enddo
   endif
   if (iturb.eq.51) then
+    call field_get_val_s(ivarfl(ial), cvar_al)
+    call field_get_val_s(ivarfl(iphi), cvar_phi)
     do iel = 1, ncel
-      rtp(iel,iphi) = 2.d0/3.d0
-      rtp(iel,ial) = 1.d0
+      cvar_phi(iel) = 2.d0/3.d0
+      cvar_al(iel) = 1.d0
     enddo
   endif
 
 elseif(itytur.eq.3) then
 
-  if (uref.ge.0.d0) then
+  call field_get_val_s(ivarfl(iep), cvar_ep)
 
-    trii   = (0.02d0*uref)**2
+  if (irijco.eq.1) then
+    call field_get_val_v(ivarfl(irij), cvar_rij)
 
-    do iel = 1, ncel
-      rtp(iel,ir11) = trii
-      rtp(iel,ir22) = trii
-      rtp(iel,ir33) = trii
-      rtp(iel,ir12) = 0.d0
-      rtp(iel,ir13) = 0.d0
-      rtp(iel,ir23) = 0.d0
-      xxk = 0.5d0*(rtp(iel,ir11)+                             &
-           rtp(iel,ir22)+rtp(iel,ir33))
-      rtp(iel,iep) = xxk**1.5d0*cmu/almax
-    enddo
-    iclip = 1
-    call clprij(ncelet , ncel   , nvar    ,     &
-               iclip  ,                         &
-               rtp    , rtp    )
+    if (uref.ge.0.d0) then
 
-  else
+      trii   = (0.02d0*uref)**2
 
-    do iel = 1, ncel
-      rtp(iel,ir11) = -grand
-      rtp(iel,ir22) = -grand
-      rtp(iel,ir33) = -grand
-      rtp(iel,ir12) = -grand
-      rtp(iel,ir13) = -grand
-      rtp(iel,ir23) = -grand
-      rtp(iel,iep)  = -grand
-    enddo
-
-    if(iturb.eq.32)then
       do iel = 1, ncel
-        rtp(iel,ial) = 1.d0
+        cvar_rij(1,iel) = trii
+        cvar_rij(2,iel) = trii
+        cvar_rij(3,iel) = trii
+        cvar_rij(4,iel) = 0.d0
+        cvar_rij(5,iel) = 0.d0
+        cvar_rij(6,iel) = 0.d0
+        xxk = 0.5d0*(cvar_rij(1,iel)+                             &
+             cvar_rij(2,iel)+cvar_rij(3,iel))
+        cvar_ep(iel) = xxk**1.5d0*cmu/almax
+      enddo
+      iclip = 1
+      call clprij2(ncelet, ncel, iclip)
+
+    else
+
+      do iel = 1, ncel
+        cvar_rij(1,iel) = -grand
+        cvar_rij(2,iel) = -grand
+        cvar_rij(3,iel) = -grand
+        cvar_rij(4,iel) = -grand
+        cvar_rij(5,iel) = -grand
+        cvar_rij(6,iel) = -grand
+        cvar_ep(iel)  = -grand
+      enddo
+
+    endif
+  else
+    call field_get_val_s(ivarfl(ir11), cvar_r11)
+    call field_get_val_s(ivarfl(ir22), cvar_r22)
+    call field_get_val_s(ivarfl(ir33), cvar_r33)
+    call field_get_val_s(ivarfl(ir12), cvar_r12)
+    call field_get_val_s(ivarfl(ir23), cvar_r23)
+    call field_get_val_s(ivarfl(ir13), cvar_r13)
+
+    if (uref.ge.0.d0) then
+
+      trii   = (0.02d0*uref)**2
+
+      do iel = 1, ncel
+        cvar_r11(iel) = trii
+        cvar_r22(iel) = trii
+        cvar_r33(iel) = trii
+        cvar_r12(iel) = 0.d0
+        cvar_r13(iel) = 0.d0
+        cvar_r23(iel) = 0.d0
+        xxk = 0.5d0*(cvar_r11(iel)+                             &
+             cvar_r22(iel)+cvar_r33(iel))
+        cvar_ep(iel) = xxk**1.5d0*cmu/almax
+      enddo
+      iclip = 1
+      call clprij(ncelet, ncel, iclip)
+
+    else
+
+      do iel = 1, ncel
+        cvar_r11(iel) = -grand
+        cvar_r22(iel) = -grand
+        cvar_r33(iel) = -grand
+        cvar_r12(iel) = -grand
+        cvar_r13(iel) = -grand
+        cvar_r23(iel) = -grand
+        cvar_ep(iel)  = -grand
       enddo
     endif
-
- endif
+  endif
+  if(iturb.eq.32)then
+    call field_get_val_s(ivarfl(ial), cvar_al)
+    do iel = 1, ncel
+      cvar_al(iel) = 1.d0
+    enddo
+  endif
 
 elseif(iturb.eq.60) then
 
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iomg), cvar_omg)
+
   if (uref.ge.0.d0) then
 
     do iel = 1, ncel
-      rtp(iel,ik ) = 1.5d0*(0.02d0*uref)**2
+      cvar_k(iel) = 1.5d0*(0.02d0*uref)**2
       !     on utilise la formule classique eps=k**1.5/Cmu/ALMAX et omega=eps/Cmu/k
-      rtp(iel,iomg) = rtp(iel,ik)**0.5d0/almax
+      cvar_omg(iel) = cvar_k(iel)**0.5d0/almax
     enddo
     !     pas la peine de clipper, les valeurs sont forcement positives
 
   else
 
     do iel = 1, ncel
-      rtp(iel,ik ) = -grand
-      rtp(iel,iomg) = -grand
+      cvar_k(iel) = -grand
+      cvar_omg(iel) = -grand
     enddo
 
   endif
 
 elseif(iturb.eq.70) then
 
+  call field_get_val_s(ivarfl(inusa), cvar_nusa)
+
   if (uref.ge.0.d0) then
 
     do iel = 1, ncel
-      rtp(iel,inusa ) = sqrt(1.5d0)*(0.02d0*uref)*almax
+      cvar_nusa(iel) = sqrt(1.5d0)*(0.02d0*uref)*almax
       !     on utilise la formule classique eps=k**1.5/Cmu/ALMAX
       !     et nusa=Cmu*k**2/eps
     enddo
@@ -467,7 +501,7 @@ elseif(iturb.eq.70) then
   else
 
     do iel = 1, ncel
-      rtp(iel,inusa ) = -grand
+      cvar_nusa(iel) = -grand
     enddo
 
   endif
@@ -482,9 +516,10 @@ if (nscal.gt.0) then
 
 !    Clipping des scalaires non variance
   do iis = 1, nscal
-    if(iscavr(iis).eq.0) then
+    call field_get_dim(ivarfl(isca(iis)), f_dim)
+    if(iscavr(iis).eq.0.and.f_dim.eq.1) then
       iscal = iis
-      call clpsca(ncelet, ncel, iscal, rvoid, rtp)
+      call clpsca(iscal)
       !==========
     endif
   enddo
@@ -494,7 +529,7 @@ if (nscal.gt.0) then
   do iis = 1, nscal
     if(iscavr(iis).ne.0.and.iclvfl(iis).ne.1) then
       iscal = iis
-      call clpsca(ncelet, ncel, iscal, rvoid, rtp)
+      call clpsca(iscal)
       !==========
     endif
   enddo
@@ -504,7 +539,7 @@ if (nscal.gt.0) then
   do iis = 1, nscal
     if (iscavr(iis).le.nscal.and.iscavr(iis).ge.1.and.iclvfl(iis).eq.1) then
       iscal = iis
-      call clpsca(ncelet, ncel, iscal, rtp(1,isca(iscavr(iis))), rtp)
+      call clpsca(iscal)
       !==========
     endif
   enddo
@@ -551,140 +586,42 @@ call field_get_n_fields(nfld)
 ifmaip = -1
 bfmaip = -1
 
-do ii = 1, nfld
+do iflid = 0, nfld - 1
+  call field_get_type(iflid, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    ! Is this field not managed by CDO ? Not useful with CDO
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
 
-  iflid = ii - 1
+      call field_get_key_int(iflid, kimasf, iflmas) ! interior mass flux
+      call field_get_key_int(iflid, kbmasf, iflmab) ! boundary mass flux
 
-  call field_get_key_int(iflid, kimasf, iflmas) ! interior mass flux
-  call field_get_key_int(iflid, kbmasf, iflmab) ! boundary mass flux
+      if (iflmas.ge.0 .and. iflmas.ne.ifmaip) then
+        call field_current_to_previous(iflid)
+        ifmaip = iflmas
+      endif
 
-  if (iflmas.ge.0 .and. iflmas.ne.ifmaip) then
-    call field_current_to_previous(iflid)
-    ifmaip = iflmas
-  endif
+      if (iflmab.ge.0 .and. iflmab.ne.bfmaip) then
+        call field_current_to_previous(iflid)
+        bfmaip = iflmab
+      endif
 
-  if (iflmab.ge.0 .and. iflmab.ne.bfmaip) then
-    call field_current_to_previous(iflid)
-    bfmaip = iflmab
-  endif
+    endif ! CDO ?
+  endif ! VARIABLE ?
 
 enddo
 
 !===============================================================================
-! 8.  INITIALISATION DES TERMES SOURCES SI EXTRAPOLES
+! 8.  INITIALISATIONS EN ALE
 !===============================================================================
 
-!     les termes sources de Navier Stokes
-if(isno2t.gt.0) then
-  iptsna = ipproc(itsnsa)
-  do ii = 1, ndim
-    do iel = 1, ncel
-      propce(iel,iptsna+ii-1) = 0.d0
-    enddo
-  enddo
-  if (icavit.ge.0) then
-    do iel = 1, ncel
-      propce(iel,iptsna+3) = 0.d0
-    enddo
-  endif
-endif
-
-!     les termes sources turbulents
-if(isto2t.gt.0) then
-  if(itytur.eq.2) jj = 2
-  if(itytur.eq.3) jj = 7
-  if(iturb.eq.30.or.iturb.eq.31) jj = 7
-  if(iturb.eq.32) jj = 8
-  if(iturb.eq.50) jj = 4
-  if(iturb.eq.60) jj = 2
-  if(iturb.eq.70) jj = 1
-  iptsta = ipproc(itstua)
-  do ii = 1, jj
-    do iel = 1, ncel
-      propce(iel,iptsta+ii-1) = 0.d0
-    enddo
-  enddo
-endif
-
-!     les termes sources des scalaires
-do iis = 1, nscal
-  if(isso2t(iis).gt.0) then
-    iptsca = ipproc(itssca(iis))
-    do iel = 1, ncel
-      propce(iel,iptsca) = 0.d0
-    enddo
-  endif
-enddo
-
-!===============================================================================
-! 9.  INITIALISATION DES MOYENNES
-!===============================================================================
-
-do imom = 1, nbmomt
-  do iel = 1, ncel
-    propce(iel,ipproc(icmome(imom))) = 0.d0
-  enddo
-enddo
-
-!===============================================================================
-! 10.  INITIALISATION CONSTANTE DE SMAGORINSKY EN MODELE DYNAMIQUE
-!===============================================================================
-
-if(iturb.eq.41) then
-  do iel = 1, ncel
-    propce(iel,ipproc(ismago)) = 0.d0
-  enddo
-endif
-
-!===============================================================================
-! 11.  INITIALISATION DU NUMERO DE LA FACE DE PAROI 5 LA PLUS PROCHE
-!===============================================================================
-
-!     Si IFAPAT existe,
-!     on suppose qu'il faut le (re)calculer : on init le tab a -1.
-
-if(abs(icdpar).eq.2) then
-  do iel = 1, ncel
-    ifapat(iel) = -1
-  enddo
-endif
-
-!===============================================================================
-! 12.  INITIALISATION DE LA FORCE EXTERIEURE QUAND IPHYDR=1
-!===============================================================================
-
-if(iphydr.eq.1) then
-  do iel = 1, ncel
-    frcxt(1,iel) = 0.d0
-    frcxt(2,iel) = 0.d0
-    frcxt(3,iel) = 0.d0
-  enddo
-endif
-
-!===============================================================================
-! 13.  INITIALISATION DE LA PRESSION HYDROSTATIQUE QUAND IPHYDR=2
-!===============================================================================
-
-if(iphydr.eq.2) then
-  do iel = 1, ncel
-    prhyd(iel) = 0.d0
-  enddo
-endif
-
-!===============================================================================
-! 14.  INITIALISATIONS EN ALE OU MAILLAGE MOBILE
-!===============================================================================
-
-if (iale.eq.1) then
+if (iale.ge.1) then
   do ii = 1, nnod
     impale(ii) = 0
-    do idim = 1, 3
-      depale(idim,ii) = 0.d0
-    enddo
   enddo
 endif
 
-if (iale.eq.1.or.imobil.eq.1) then
+if (iale.ge.1) then
   do ii = 1, nnod
     do idim = 1, 3
       xyzno0(idim,ii) = xyznod(idim,ii)
@@ -692,8 +629,25 @@ if (iale.eq.1.or.imobil.eq.1) then
   enddo
 endif
 
+!===============================================================================
+! 9 Current to previous for variables
+!===============================================================================
+
+do iflid = 0, nfld - 1
+  call field_get_type(iflid, f_type)
+  ! Is the field of type FIELD_VARIABLE?
+  if (iand(f_type, FIELD_VARIABLE).eq.FIELD_VARIABLE) then
+    ! Is this field not managed by CDO ? Perfomed elsewhere with CDO
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
+
+      call field_current_to_previous(iflid)
+
+    endif ! CDO ?
+  endif ! VARIABLE ?
+enddo
+
 !----
-! FIN
+! End
 !----
 
 return
