@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -34,15 +34,10 @@
 !------------------------------------------------------------------------------
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
-!> \param[in]     dt            valeur du pas de temps
-!> \param[in,out] rtp           variables de calcul au centre des
-!>                              cellules
-!> \param[in,out] propce        physical properties at cell centers
 !______________________________________________________________________________
 
 subroutine inivar &
- ( nvar   , nscal  ,                                              &
-   dt     , rtp    , propce )
+ ( nvar   , nscal )
 
 !===============================================================================
 ! Module files
@@ -65,6 +60,13 @@ use mesh
 use field
 use cfpoin, only:ithvar
 use cs_c_bindings
+use cs_cf_bindings
+use cs_f_interfaces
+use vof
+
+use, intrinsic :: iso_c_binding
+
+use darcy_module
 
 !===============================================================================
 
@@ -74,20 +76,17 @@ implicit none
 
 integer          nvar   , nscal
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), propce(ncelet,*)
-
 ! Local variables
 
-character*80     chaine
+character(len=80) :: chaine
 integer          ivar  , iscal
 integer          iel
 integer          iclip , iok   , ii
-integer          iiptot
-integer          ibormo(nbmomx), imodif
 integer          kscmin, kscmax, keyvar, n_fields
 integer          f_id, f_id_prv, c_id, f_dim
-
-logical          interleaved
+integer          iflid, iflidp
+integer          idimf
+integer          ivoid, uprtot
 
 double precision valmax, valmin, vfmin , vfmax
 double precision vdtmax, vdtmin
@@ -99,10 +98,20 @@ double precision xalmin, xalmax
 double precision scmaxp, scminp
 
 double precision rvoid(1)
-double precision, allocatable, dimension(:) :: w1, w2
+double precision vvoid(3)
 
+double precision, dimension(:), pointer :: dt
 double precision, dimension(:), pointer :: field_s_v
 double precision, dimension(:,:), pointer :: field_v_v
+double precision, dimension(:), pointer :: cvar_pr
+double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_al
+double precision, dimension(:), pointer :: cvar_phi, cvar_omg, cvar_nusa
+double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
+double precision, dimension(:,:), pointer :: cvar_rij
+double precision, dimension(:), pointer :: cvar_var
+double precision, dimension(:), pointer :: cpro_prtot
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -110,6 +119,8 @@ double precision, dimension(:,:), pointer :: field_v_v
 ! 1. Initialization
 !===============================================================================
 
+call field_get_val_s_by_name('dt', dt)
+
 call field_get_n_fields(n_fields)
 
 call field_get_key_id("variable_id", keyvar)
@@ -125,60 +136,54 @@ iok = 0
 !    INITIALISATIONS QUI LUI SONT PROPRES
 !===============================================================================
 
-! Initialized thermodynamic variables indicator
-ithvar = 0
-
 ! Indicateur d'initialisation des scalaires par l'utilisateur
 ! (mis a 1 si passage dans USINIV ou PPINIV ou dans l'IHM ; a 0 sinon)
 
 iusini = 1
 
-if  (ippmod(icompf).ge.0) then
+iflidp = -1
 
-  do iel = 1, ncel
-    rtp(iel,isca(itempk)) = t0
-  enddo
+do ivar = 1, nvar
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+  if (vcopt%iwgrec.eq.1) then
 
-  call cf_thermo_default_init(ncel, ncelet, rtp)
-  !==========================
+    if (vcopt%idiff.lt.1) cycle
+    iflid = ivarfl(ivar)
+    if (iflid.eq.iflidp) cycle
+    iflidp = iflid
 
-  ! On initialise la diffusivite thermique
-  visls0(ienerg) = visls0(itempk)/cv0
-
-  if(ivisls(ienerg).gt.0) then
-    if(ivisls(itempk).gt.0) then
-      if(icv.gt.0) then
-        do iel = 1, ncel
-          propce(iel,ipproc(ivisls(ienerg))) =                                &
-                                           propce(iel,ipproc(ivisls(itempk))) &
-                                         / propce(iel,ipproc(icv))
-        enddo
-      else
-        do iel = 1, ncel
-          propce(iel,ipproc(ivisls(ienerg))) =                                &
-                                           propce(iel,ipproc(ivisls(itempk))) &
-                                         / cv0
-        enddo
-      endif
-    else
-      do iel = 1, ncel
-        propce(iel,ipproc(ivisls(ienerg))) =  visls0(itempk) &
-                                            / propce(iel,ipproc(icv))
+    if (iand(vcopt%idften, ISOTROPIC_DIFFUSION).ne.0) then
+      idimf = 1
+    elseif (iand(vcopt%idften, ANISOTROPIC_DIFFUSION).ne.0) then
+      idimf = 6
+    endif
+    call field_get_key_int(iflid, kwgrec, f_id)
+    if (idimf.eq.6) then
+      call field_get_val_v(f_id, field_v_v)
+      do iel = 1, ncelet
+        field_v_v(1,iel) = 1.d0
+        field_v_v(2,iel) = 1.d0
+        field_v_v(3,iel) = 1.d0
+        field_v_v(4,iel) = 0.d0
+        field_v_v(5,iel) = 0.d0
+        field_v_v(6,iel) = 0.d0
+      enddo
+    else if (idimf.eq.1) then
+      call field_get_val_s(f_id, field_s_v)
+      do iel = 1, ncelet
+        field_s_v(iel) = 1.d0
       enddo
     endif
-  endif
 
-endif
+  endif
+enddo
 
 ! - Interface Code_Saturne
 !   ======================
 
 if (iihmpr.eq.1) then
 
-  call uiiniv (ncelet, isuite,                     &
-               ithvar,                             &
-               ro0, cp0, viscl0, uref,             &
-               almax, xyzcen)
+  call uiiniv (isuite, ippmod(idarcy), ithvar)
 
 endif
 
@@ -187,10 +192,7 @@ endif
 
 if (ippmod(iphpar).eq.0) then
 
-  call cs_user_initialization &
-  !==========================
-( nvar   , nscal  ,                                                             &
-  dt     )
+  call cs_user_f_initialization(nvar, nscal, dt)
 
   !     Avec l'interface, il peut y avoir eu initialisation,
   !       meme si usiniv n'est pas utilise.
@@ -208,58 +210,105 @@ else
 
   iusini = 1
 
-  call ppiniv(nvar, nscal, dt, rtp, propce)
-  !==========
-
-  if  (ippmod(icompf).ge.0) then
-
-    allocate(w1(ncelet), w2(ncelet))
-    imodif = 1
-
-    call cfther                                                                 &
-    !==========
-   ( nvar   ,                                                                   &
-     ithvar , imodif ,                                                          &
-     rtp    ,                                                                   &
-     w1     , w2     , rvoid )
-
-    deallocate(w1, w2)
-
-  endif
+  call ppiniv(nvar, nscal, dt)
 
 endif
 
+call user_initialization()
 
-! Si l'utilisateur a change Ptot, on change P* en consequence,
-! sinon on met Ptot a P0 + rho.g.r
-! A priori l'utilisateur remplira les NCEL valeurs ou rien du
-!  tout, mais on ne sait jamais ...
-! En compressible, Ptot n'est pas defini (correspond directement a RTP(.,IPR)
+! VoF algorithm
+if (ivofmt.gt.0) then
+  call vof_compute_linear_rho_mu
+  ! density is stored at the two previous time steps
+  call field_current_to_previous(icrom)
+  call field_current_to_previous(ibrom)
+  call field_current_to_previous(icrom)
+  call field_current_to_previous(ibrom)
+endif
+
+if (ippmod(icompf).ge.0.and.(    isuite.eq.0                 &
+                             .or.isuite.eq.1.and.ileaux.eq.0)) then
+
+  if (     ithvar.ne. 60000.and.ithvar.ne.100000                    &
+      .and.ithvar.ne.140000.and.ithvar.ne.150000.and.ithvar.ne.210000) then
+      write(nfecra,1000) ithvar
+      iok = iok + 1
+  endif
+
+  ivoid = -1
+  call cs_cf_thermo(ithvar, ivoid,  rvoid, rvoid, rvoid, vvoid)
+
+endif
+
+call user_extra_operations_initialize()
+
+! Pressure / Total pressure initialisation
+
+! Standard:
+! If the user has initialized the total pressure Ptot, P* is initialized
+! accordingly, only if the user has speficied the reference point.
+! (all values of the total pressure have to be initialized).
+! Otherwise, the total pressure is initialized using P*,
+! Ptot = P* + P0 + rho.g.r
+
+! In case of restart without auxiliary, Ptot is recomputed with P*.
+! (For EVM models, the shift by 2/3*rho*k is missing)
+! In case of restart with auxiliary, nothing is done.
+
+! Compressible:
+! The total pressure field does not need to be defined. The solved pressure is
+! the total pressure.
+
+! Ground water flow:
+! The field of index iprtot is the pressure head (h = H - z),
+! h is only used when gravity is taken into account.
+
+if (ippmod(icompf).lt.0.and.ippmod(idarcy).lt.0) then
+
+  call field_get_val_s(ivarfl(ipr), cvar_pr)
+  call field_get_val_s(iprtot, cpro_prtot)
+
+  uprtot = 0
+
+  if (ixyzp0.gt.-1.and.(isuite.eq.0.or.ileaux.eq.0)) then
+    uprtot = 1
+    do iel = 1, ncel
+      if (cpro_prtot(iel).le.-0.5d0*rinfin) then
+        uprtot = 0
+        exit
+      endif
+    enddo
+  endif
 
-if  (ippmod(icompf).lt.0) then
-  iiptot = ipproc(iprtot)
   xxp0   = xyzp0(1)
   xyp0   = xyzp0(2)
   xzp0   = xyzp0(3)
+
+  if (uprtot.gt.0) then
+    do iel = 1, ncel
+      cvar_pr(iel) =  cpro_prtot(iel)               &
+                    - ro0*( gx*(xyzcen(1,iel)-xxp0) &
+                    + gy*(xyzcen(2,iel)-xyp0)       &
+                    + gz*(xyzcen(3,iel)-xzp0) )     &
+                    + pred0 - p0
+    enddo
+  elseif (isuite.eq.0.or.ileaux.eq.0) then
+    call navstv_total_pressure
+  endif
+
+else if ((ippmod(idarcy).ge.0).and.(darcy_gravity.ge.1)) then
+
+  call field_get_val_s(ivarfl(ipr), cvar_pr)
+  call field_get_val_s(iprtot, cpro_prtot)
+
   do iel = 1, ncel
-    if (propce(iel,iiptot).gt.-0.5d0*rinfin) then
-      rtp(iel,ipr) = propce(iel,iiptot)                      &
-           - ro0*( gx*(xyzcen(1,iel)-xxp0)                   &
-           + gy*(xyzcen(2,iel)-xyp0)                         &
-           + gz*(xyzcen(3,iel)-xzp0) )                       &
-           + pred0 - p0
-    else
-      propce(iel,iiptot) = rtp(iel,ipr)                      &
-           + ro0*( gx*(xyzcen(1,iel)-xxp0)                   &
-           + gy*(xyzcen(2,iel)-xyp0)                         &
-           + gz*(xyzcen(3,iel)-xzp0) )                       &
-           + p0 - pred0
-    endif
+    cpro_prtot(iel) = cvar_pr(iel) - xyzcen(1,iel)*darcy_gravity_x &
+                                   - xyzcen(2,iel)*darcy_gravity_y &
+                                   - xyzcen(3,iel)*darcy_gravity_z
   enddo
+
 endif
 
-
-
 !===============================================================================
 ! 3.  CLIPPING DES GRANDEURS TURBULENTES (UTILISATEUR OU SUITE)
 !     (pour ITYTUR=2, 3, 5 ou 6)
@@ -295,29 +344,27 @@ endif
 !          fait au moins une erreur qui peut en cacher d'autres)
 !===============================================================================
 
-if(iusini.eq.1.or.isuite.eq.1) then
+if (iusini.eq.1.or.isuite.eq.1) then
 
   if(itytur.eq.2 .or. itytur.eq.5) then
 
-    xekmin = rtp(1,ik)
-    xepmin = rtp(1,iep)
+    call field_get_val_s(ivarfl(ik), cvar_k)
+    call field_get_val_s(ivarfl(iep), cvar_ep)
+
+    xekmin = cvar_k(1)
+    xepmin = cvar_ep(1)
     do iel = 1, ncel
-      xekmin = min(xekmin,rtp(iel,ik) )
-      xepmin = min(xepmin,rtp(iel,iep))
+      xekmin = min(xekmin,cvar_k(iel) )
+      xepmin = min(xepmin,cvar_ep(iel))
     enddo
     if (irangp.ge.0) then
       call parmin (xekmin)
-      !==========
       call parmin (xepmin)
-      !==========
     endif
 
     if(xekmin.ge.0.d0.and.xepmin.ge.0.d0) then
       iclip = 1
-      call clipke( ncelet , ncel   , nvar   ,          &
-      !==========
-                   iclip  , iwarni(ik) ,            &
-                   propce , rtp    )
+      call clipke(ncelet, ncel, iclip)
     else
       write(nfecra,3020) xekmin,xepmin
       iok = iok + 1
@@ -327,17 +374,17 @@ if(iusini.eq.1.or.isuite.eq.1) then
     !     compris entre 0 et 2
     if (itytur.eq.5) then
 
-      xphmin = rtp(1,iphi)
-      xphmax = rtp(1,iphi)
+      call field_get_val_s(ivarfl(iphi), cvar_phi)
+
+      xphmin = cvar_phi(1)
+      xphmax = cvar_phi(1)
       do iel = 1, ncel
-        xphmin = min(xphmin,rtp(iel,iphi) )
-        xphmax = max(xphmax,rtp(iel,iphi) )
+        xphmin = min(xphmin,cvar_phi(iel) )
+        xphmax = max(xphmax,cvar_phi(iel) )
       enddo
       if (irangp.ge.0) then
         call parmin (xphmin)
-        !==========
         call parmax (xphmax)
-        !==========
       endif
 
       !     Par coherence avec clpv2f, on ne clippe qu'a zero et pas a 2
@@ -350,17 +397,16 @@ if(iusini.eq.1.or.isuite.eq.1) then
       !     En v2-f, BL-v2/k, on verifie aussi que alpha est
       !     compris entre 0 et 1
       if (iturb.eq.51) then
-        xalmin = rtp(1,ial)
-        xalmax = rtp(1,ial)
+        call field_get_val_s(ivarfl(ial), cvar_al)
+        xalmin = cvar_al(1)
+        xalmax = cvar_al(1)
         do iel = 1, ncel
-          xalmin = min(xalmin,rtp(iel,ial) )
-          xalmax = max(xalmax,rtp(iel,ial) )
+          xalmin = min(xalmin,cvar_al(iel) )
+          xalmax = max(xalmax,cvar_al(iel) )
         enddo
         if (irangp.ge.0) then
           call parmin (xalmin)
-          !==========
           call parmax (xalmax)
-          !==========
         endif
 
         if(xalmin.lt.0.d0 .or. xalmax.gt.1.d0) then
@@ -374,49 +420,63 @@ if(iusini.eq.1.or.isuite.eq.1) then
 
   elseif(itytur.eq.3) then
 
-    x11min = rtp(1,ir11)
-    x22min = rtp(1,ir22)
-    x33min = rtp(1,ir33)
-    xepmin = rtp(1,iep)
-    do iel = 1, ncel
-      x11min = min(x11min,rtp(iel,ir11))
-      x22min = min(x22min,rtp(iel,ir22))
-      x33min = min(x33min,rtp(iel,ir33))
-      xepmin = min(xepmin,rtp(iel,iep) )
-    enddo
+    call field_get_val_s(ivarfl(iep), cvar_ep)
+    if (irijco.eq.1) then
+      call field_get_val_v(ivarfl(irij), cvar_rij)
+
+      x11min = cvar_rij(1,1)
+      x22min = cvar_rij(2,1)
+      x33min = cvar_rij(3,1)
+      xepmin = cvar_ep(1)
+      do iel = 1, ncel
+        x11min = min(x11min,cvar_rij(1,iel))
+        x22min = min(x22min,cvar_rij(2,iel))
+        x33min = min(x33min,cvar_rij(3,iel))
+        xepmin = min(xepmin,cvar_ep(iel) )
+      enddo
+    else
+      call field_get_val_s(ivarfl(ir11), cvar_r11)
+      call field_get_val_s(ivarfl(ir22), cvar_r22)
+      call field_get_val_s(ivarfl(ir33), cvar_r33)
+
+      x11min = cvar_r11(1)
+      x22min = cvar_r22(1)
+      x33min = cvar_r33(1)
+      xepmin = cvar_ep(1)
+      do iel = 1, ncel
+        x11min = min(x11min,cvar_r11(iel))
+        x22min = min(x22min,cvar_r22(iel))
+        x33min = min(x33min,cvar_r33(iel))
+        xepmin = min(xepmin,cvar_ep(iel) )
+      enddo
+    endif
     if (irangp.ge.0) then
       call parmin (x11min)
-      !==========
       call parmin (x22min)
-      !==========
       call parmin (x33min)
-      !==========
       call parmin (xepmin)
-      !==========
     endif
     if (x11min.ge.0.d0.and.x22min.ge.0.d0.and.                  &
          x33min.ge.0.d0.and.xepmin.ge.0.d0 ) then
       iclip = 1
-      call clprij( ncelet , ncel   , nvar   ,          &
-      !==========
-                   iclip  ,                            &
-                   rtp    , rtp    )
+      if (irijco.eq.0) then
+        call clprij(ncelet, ncel, iclip)
+      endif
     else
       write(nfecra,3030) x11min,x22min,x33min,xepmin
       iok = iok + 1
     endif
     if (iturb.eq.32) then
-      xalmin = rtp(1,ial)
-      xalmax = rtp(1,ial)
+      call field_get_val_s(ivarfl(ial), cvar_al)
+      xalmin = cvar_al(1)
+      xalmax = cvar_al(1)
       do iel = 1, ncel
-        xalmin = min(xalmin, rtp(iel,ial))
-        xalmax = max(xalmax, rtp(iel,ial))
+        xalmin = min(xalmin, cvar_al(iel))
+        xalmax = max(xalmax, cvar_al(iel))
       enddo
       if (irangp.ge.0) then
         call parmin (xalmin)
-        !==========
         call parmax (xalmax)
-        !==========
       endif
       if (xalmin.lt.0.or.xalmax.gt.1.d0) then
         write(nfecra,3033) xalmin, xalmax
@@ -426,17 +486,18 @@ if(iusini.eq.1.or.isuite.eq.1) then
 
   elseif(iturb.eq.60) then
 
-    xekmin = rtp(1,ik )
-    xomgmn = rtp(1,iomg)
+    call field_get_val_s(ivarfl(ik), cvar_k)
+    call field_get_val_s(ivarfl(iomg), cvar_omg)
+
+    xekmin = cvar_k(1)
+    xomgmn = cvar_omg(1)
     do iel = 1, ncel
-      xekmin = min(xekmin,rtp(iel,ik ))
-      xomgmn = min(xomgmn,rtp(iel,iomg))
+      xekmin = min(xekmin,cvar_k(iel))
+      xomgmn = min(xomgmn,cvar_omg(iel))
     enddo
     if (irangp.ge.0) then
       call parmin (xekmin)
-      !==========
       call parmin (xomgmn)
-      !==========
     endif
 
     !     En k-omega on clippe seulement a 0
@@ -447,13 +508,14 @@ if(iusini.eq.1.or.isuite.eq.1) then
 
   elseif(iturb.eq.70) then
 
-    xnumin = rtp(1,inusa)
+    call field_get_val_s(ivarfl(inusa), cvar_nusa)
+
+    xnumin = cvar_nusa(1)
     do iel = 1, ncel
-      xnumin = min(xnumin,rtp(iel,inusa))
+      xnumin = min(xnumin,cvar_nusa(iel))
     enddo
     if (irangp.ge.0) then
       call parmin (xnumin)
-      !==========
     endif
 
     !     En Spalart-Allmaras on clippe seulement a 0
@@ -513,34 +575,33 @@ if(nscal.gt.0.and.(iusini.eq.1.or.isuite.eq.1)) then
 !     Scalaires non variance
 
   do ii = 1, nscal
-    if(iscavr(ii).le.0.or.iscavr(ii).gt.nscal) then
+    f_id = ivarfl(isca(ii))
+    call field_get_dim(f_id, f_dim)
+    if((iscavr(ii).le.0.or.iscavr(ii).gt.nscal).and.f_dim.eq.1) then
 
       ! Get the min clipping
-      f_id = ivarfl(isca(ii))
       call field_get_key_double(f_id, kscmin, scminp)
       call field_get_key_double(f_id, kscmax, scmaxp)
 
       if (scminp.le.scmaxp) then
         ivar = isca(ii)
-        valmax = rtp(1  ,ivar)
-        valmin = rtp(1  ,ivar)
+        call field_get_val_s(ivarfl(ivar), cvar_var)
+        valmax = cvar_var(1)
+        valmin = cvar_var(1)
         do iel = 1, ncel
-          valmax = max(valmax,rtp(iel,ivar))
-          valmin = min(valmin,rtp(iel,ivar))
+          valmax = max(valmax,cvar_var(iel))
+          valmin = min(valmin,cvar_var(iel))
         enddo
         if (irangp.ge.0) then
           call parmax (valmax)
-          !==========
           call parmin (valmin)
-          !==========
         endif
 
 !     Verification de la coherence pour les clippings
 !                                           des scalaires non variance.
         if (valmin.ge.scminp.and.valmax.le.scmaxp) then
           iscal = ii
-          call clpsca(ncelet, ncel, iscal, rvoid, rtp)
-          !==========
+          call clpsca(iscal)
         else
           call field_get_label(ivarfl(isca(ii)), chaine)
           write(nfecra,3040) ii,chaine(1:16),                     &
@@ -566,17 +627,16 @@ if(nscal.gt.0.and.(iusini.eq.1.or.isuite.eq.1)) then
 
       if (scminp.le.scmaxp) then
         ivar = isca(ii)
-        valmax = rtp(1  ,ivar)
-        valmin = rtp(1  ,ivar)
+        call field_get_val_s(ivarfl(ivar), cvar_var)
+        valmax = cvar_var(1)
+        valmin = cvar_var(1)
         do iel = 1, ncel
-          valmax = max(valmax,rtp(iel,ivar))
-          valmin = min(valmin,rtp(iel,ivar))
+          valmax = max(valmax,cvar_var(iel))
+          valmin = min(valmin,cvar_var(iel))
         enddo
         if (irangp.ge.0) then
           call parmax (valmax)
-          !==========
           call parmin (valmin)
-          !==========
         endif
 
 !     Verification de la coherence pour les clippings de variance.
@@ -596,8 +656,7 @@ if(nscal.gt.0.and.(iusini.eq.1.or.isuite.eq.1)) then
 ! Ici on clippe pour etre coherent avec la valeur du scalaire
           if(valmin.ge.0.d0) then
             iscal = ii
-            call clpsca(ncelet, ncel, iscal, rtp(1,isca(iscavr(ii))), rtp)
-            !==========
+            call clpsca(iscal)
           else
             call field_get_name(ivarfl(isca(ii)), chaine)
             write(nfecra,3050)ii,chaine(1:16),valmin,scminp,valmax,scmaxp
@@ -624,9 +683,8 @@ if(nscal.gt.0.and.(iusini.eq.1.or.isuite.eq.1)) then
 
 endif
 
-
 !===============================================================================
-! 6.  IMPRESSIONS DE CONTROLE POUR LES INCONNUES, LE PAS DE TEMPS
+! 5.  IMPRESSIONS DE CONTROLE POUR LES INCONNUES, LE PAS DE TEMPS
 !        LE CUMUL DES DUREE POUR LES MOYENNES
 !===============================================================================
 
@@ -645,7 +703,7 @@ do ivar = 1, nvar
     c_id = 1
   endif
 
-  call field_get_dim(f_id, f_dim, interleaved)
+  call field_get_dim(f_id, f_dim)
 
   if (f_dim.gt.1) then
     call field_get_val_v(f_id, field_v_v)
@@ -671,9 +729,7 @@ do ivar = 1, nvar
 
   if (irangp.ge.0) then
     call parmax (valmax)
-    !==========
     call parmin (valmin)
-    !==========
   endif
   call field_get_label(f_id, chaine)
   write(nfecra,2010)chaine(1:16),valmin,valmax
@@ -691,9 +747,7 @@ if (idtvar.ge.0) then
   enddo
   if (irangp.ge.0) then
     call parmax (vdtmax)
-    !==========
     call parmin (vdtmin)
-    !==========
   endif
   write(nfecra,2010) 'dt', vdtmin, vdtmax
   write(nfecra,2020)
@@ -711,7 +765,7 @@ endif
 call time_moment_log_iteration
 
 !===============================================================================
-! 7.  ARRET GENERAL SI PB
+! 6.  ARRET GENERAL SI PB
 !===============================================================================
 
 if (iok.gt.0) then
@@ -728,6 +782,30 @@ write(nfecra,3000)
 
 #if defined(_CS_LANG_FR)
 
+ 1000 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@ ATTENTION:   ARRET A L''INITIALISATION DES VARIABLES    ',/,&
+'@    ==========   DANS LA THERMODYNAMIQUE COMPRESSIBLE.      ',/,&
+'@                                                            ',/,&
+'@    Le calcul ne peut etre execute.                         ',/,&
+'@                                                            ',/,&
+'@    Valeur de l''indicateur ithvar non prevue (',i10,').    ',/,&
+'@                                                            ',/,&
+'@    Deux et seulement deux variables parmi                  ',/,&
+'@    P, rho, T et E (sauf T et E) doivent etre imposees      ',/,&
+'@    a l''initialisation dans le GUI ou dans la routine      ',/,&
+'@    d''initialisation (cs_user_initialisation.f90) ou       ',/,&
+'@    encore via les deux.                                    ',/,&
+'@                                                            ',/,&
+'@    Verifier que ithvar n''ait pas ete partiellement        ',/,&
+'@    positionnee par le GUI de maniere non consistante       ',/,&
+'@    avec cs_user_initialisation.f90.                        ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+
  2000 format(                                                     &
                                                                 /,&
 ' -----------------------------------------------------------', /,&
@@ -1040,6 +1118,30 @@ write(nfecra,3000)
 
 #else
 
+ 1000 format(                                                     &
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/,&
+'@ @@ WARNING :     stop in compressible thermodynamics at    ',/,&
+'@    =========     initialisation.                           ',/,&
+'@                                                            ',/,&
+'@    The computation will stop.                              ',/,&
+'@                                                            ',/,&
+'@    Unexpected value of the indicator ithvar (',i10,').     ',/,&
+'@                                                            ',/,&
+'@    Two and only two independant variables among            ',/,&
+'@    P, rho, T and E (except T and E) should be imposed at   ',/,&
+'@    the initialisation in the GUI or in the user subroutine ',/,&
+'@    of initialisation (cs_user_initialisation.f90) or       ',/,&
+'@    in both.                                                ',/,&
+'@                                                            ',/,&
+'@    Check if iccfth has not been partially set through the  ',/,&
+'@    GUI in a non consistant manner with                     ',/,&
+'@    cs_user_initialisation.f90.                             ',/,&
+'@                                                            ',/,&
+'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
+'@                                                            ',/)
+
  2000 format(                                                     &
                                                                 /,&
 ' -----------------------------------------------------------', /,&
