@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -34,24 +34,10 @@
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]     ncelet        number of extended (real + ghost) cells
-!> \param[in]     ncel          number of cells
-!> \param[in]     nfac          number of inner faces
-!> \param[in]     nfabor        number of boundary faces
-!> \param[in]     nvar          total number of variables
-!> \param[in]     nscal         total number of scalars
-!> \param[out]    dt            time step (per cell)
-!> \param[out]    rtp           calculated variables at cell centers
-!> \param[out]    propce        physical properties at cell centers
-!> \param[out]    frcxt         external forces making hydrostatic pressure
-!> \param[out]    prhyd         predicted hydrostatic pressure
 !_______________________________________________________________________________
 
 
-subroutine lecamo &
- ( ncelet , ncel   , nfac   , nfabor , nvar   , nscal  ,          &
-   dt     , rtp    , propce ,                                     &
-   frcxt  , prhyd  )
+subroutine lecamo
 
 !===============================================================================
 
@@ -67,6 +53,8 @@ use optcal
 use pointe
 use numvar
 use parall
+use mesh
+use cs_c_bindings
 
 !===============================================================================
 
@@ -74,57 +62,39 @@ implicit none
 
 ! Arguments
 
-integer          ncelet , ncel   , nfac   , nfabor
-integer          nvar   , nscal
-
-double precision dt(ncelet), rtp(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
-double precision frcxt(3,ncelet), prhyd(ncelet)
-
 ! Local variables
 
+type(c_ptr)      oflmap
 
 !===============================================================================
 
 
-
 !===============================================================================
 ! 1. INITIALISATIONS
 !===============================================================================
 
 write(nfecra,1000)
 
-
 !===============================================================================
-! 2. LECTURE DU FICHIER SUITE PRINCIPAL
+! 2. Read main restart file
 !===============================================================================
 
-call lecamp &
-!==========
-( ncelet , ncel   ,                                    &
-  nvar   , nscal  ,                                    &
-  rtp    )
-
+call lecamp(oflmap)
 
 !===============================================================================
 ! 3. LECTURE DU FICHIER SUITE AUXILIAIRE
 !===============================================================================
 
 if (ileaux.eq.1) then
-
-  call lecamx &
-  !==========
-( ncelet , ncel   , nfac   , nfabor , nvar   , nscal  , &
-  dt     , propce ,                                     &
-  frcxt  , prhyd  )
-
+  call lecamx(oflmap)
 endif
 
-
 !===============================================================================
 ! 4. SORTIE
 !===============================================================================
 
+call cs_map_name_to_id_destroy(oflmap)
+
 write(nfecra,2000)
 
 !===============================================================================
