@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -39,7 +39,7 @@ subroutine mmtycl &
 !__________________!____!_____!________________________________________________!
 ! itypfb           ! ia ! <-- ! boundary face types                            !
 ! rcodcl           ! tr ! <-- ! valeur des conditions aux limites              !
-!  (nfabor,nvarcl) !    !     !  aux faces de bord                             !
+!                  !    !     !  aux faces de bord                             !
 !                  !    !     ! rcodcl(1) = valeur du dirichlet                !
 !                  !    !     ! rcodcl(2) = valeur du coef. d'echange          !
 !                  !    !     !  ext. (infinie si pas d'echange)               !
@@ -48,7 +48,7 @@ subroutine mmtycl &
 !                  !    !     ! pour les vitesses (vistl+visct)*gradu          !
 !                  !    !     ! pour la pression             dt*gradp          !
 !                  !    !     ! pour les scalaires                             !
-!                  !    !     !        cp*(viscls+visct/sigmas)*gradt          !
+!                  !    !     !        cp*(viscls+visct/turb_schmidt)*gradt    !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -65,6 +65,8 @@ use paramx
 use numvar
 use optcal
 use cstnum
+use dimens, only: nvar
+use rotation
 use turbomachinery
 use entsor
 use parall
@@ -79,7 +81,7 @@ implicit none
 
 integer          itypfb(nfabor)
 
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -88,7 +90,7 @@ integer          idefau
 
 double precision srfbnf, rnx, rny, rnz
 double precision rcodcx, rcodcy, rcodcz, rcodsn
-double precision vitbox, vitboy, vitboz
+double precision vr(3)
 double precision visclc, visctc, distbf, hint
 
 double precision, dimension(:), pointer :: viscl, visct
@@ -100,8 +102,8 @@ double precision, dimension(:), pointer :: viscl, visct
 !===============================================================================
 
 ! --- Physical quantities
-call field_get_val_s(iprpfl(iviscl), viscl)
-call field_get_val_s(iprpfl(ivisct), visct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 
 !===============================================================================
 ! VITESSE DE DEFILEMENT POUR LES PAROIS FLUIDES ET SYMETRIES
@@ -123,14 +125,12 @@ do ifac = 1, nfabor
 
     ! --- En turbomachine on connait la valeur exacte de la vitesse de maillage
 
-    vitbox = rotax(2)*cdgfbo(3,ifac) - rotax(3)*cdgfbo(2,ifac)
-    vitboy = rotax(3)*cdgfbo(1,ifac) - rotax(1)*cdgfbo(3,ifac)
-    vitboz = rotax(1)*cdgfbo(2,ifac) - rotax(2)*cdgfbo(1,ifac)
+    call rotation_velocity(irotce(iel), cdgfbo(:,ifac), vr)
 
     if (itypfb(ifac).eq.isymet) then
-      rcodcl(ifac,iu,1) = vitbox
-      rcodcl(ifac,iv,1) = vitboy
-      rcodcl(ifac,iw,1) = vitboz
+      rcodcl(ifac,iu,1) = vr(1)
+      rcodcl(ifac,iv,1) = vr(2)
+      rcodcl(ifac,iw,1) = vr(3)
     endif
 
     if (itypfb(ifac).eq.iparoi .or.                        &
@@ -141,9 +141,9 @@ do ifac = 1, nfabor
       if (rcodcl(ifac,iu,1).gt.rinfin*0.5d0 .and.              &
           rcodcl(ifac,iv,1).gt.rinfin*0.5d0 .and.              &
           rcodcl(ifac,iw,1).gt.rinfin*0.5d0) then
-        rcodcl(ifac,iu,1) = vitbox
-        rcodcl(ifac,iv,1) = vitboy
-        rcodcl(ifac,iw,1) = vitboz
+        rcodcl(ifac,iu,1) = vr(1)
+        rcodcl(ifac,iv,1) = vr(2)
+        rcodcl(ifac,iw,1) = vr(3)
       else
         ! On met a 0 les composantes de RCODCL non specifiees
         if (rcodcl(ifac,iu,1).gt.rinfin*0.5d0) rcodcl(ifac,iu,1) = 0.d0
@@ -157,9 +157,9 @@ do ifac = 1, nfabor
         rcodcx = rcodcl(ifac,iu,1)
         rcodcy = rcodcl(ifac,iv,1)
         rcodcz = rcodcl(ifac,iw,1)
-        rcodsn = (vitbox - rcodcx)*rnx                            &
-               + (vitboy - rcodcy)*rny                            &
-               + (vitboz - rcodcz)*rnz
+        rcodsn = (vr(1) - rcodcx)*rnx                            &
+               + (vr(2) - rcodcy)*rny                            &
+               + (vr(3) - rcodcz)*rnz
         rcodcl(ifac,iu,1) = rcodcx + rcodsn*rnx
         rcodcl(ifac,iv,1) = rcodcy + rcodsn*rny
         rcodcl(ifac,iw,1) = rcodcz + rcodsn*rnz
