@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,6 +27,8 @@ module numvar
 
   !=============================================================================
 
+  use, intrinsic :: iso_c_binding
+
   use paramx
 
   implicit none
@@ -43,7 +45,7 @@ module numvar
   !----------------------------------------------------------------------------
 
   !> \defgroup main_variables Main variables
-  !> \brief Main variables stored in rtp, rtpa.
+  !> \brief Main variable field indices (previously stored in rtp, rtpa).
 
   !> \addtogroup main_variables
   !> \{
@@ -64,9 +66,9 @@ module numvar
   !> velocity component \f$ u_z \f$
   integer, save :: iw
 
-  !> \anchor ivoidf
-  !> void fraction for cavitation modelling
-  integer, save :: ivoidf
+  !> \anchor ivolf2
+  !> void fraction for VOF method
+  integer, save :: ivolf2
 
   !> \anchor ik
   !> turbulent kinetic energy \f$ k \f$
@@ -100,6 +102,10 @@ module numvar
   !> Reynolds stress component \f$ R_{zz} \f$
   integer, save :: ir13
 
+  !> \anchor irij
+  !> Reynolds stress tenso \f$ R_{ij} \f$
+  integer, save :: irij
+
   !> \anchor iphi
   !> variable \f$ \phi \f$ of the \f$ \phi-f_b \f$ model
   integer, save :: iphi
@@ -129,7 +135,7 @@ module numvar
   integer, save :: iscapp(nscamx)
 
   !> \anchor nscaus
-  !> number of user scalars
+  !> number of user scalars solutions of an advection equation
   integer, save :: nscaus
 
   !> \anchor nscapp
@@ -138,7 +144,7 @@ module numvar
 
   !> \anchor nscasp
   !> number of species scalars
-  integer, save :: nscasp
+  integer(c_int), pointer, save :: nscasp
 
   !> \anchor iuma
   !> mesh velocity component \f$ w_x \f$
@@ -159,68 +165,53 @@ module numvar
   !----------------------------------------------------------------------------
 
   !> \defgroup physical_prop Physical properties
-  !> \brief Physical properties are stored in propce.
+  !> \brief Physical properties are using the field API.
   !> See \ref cs_user_boundary_conditions for some examples.
 
   !> \addtogroup physical_prop
   !> \{
 
-  !> pointer to cell properties (propce)
-  integer, save :: ipproc(npromx)
-
-  !> Density at the current time step
+  !> Density at the current time step (equal to icrom, kept for compatibility)
   integer, save :: irom
 
-  !> Density at the previous time step
-  integer, save :: iroma
-
-  !> Density at the second previous time
-  integer, save :: iromaa
-
   !> dynamic molecular viscosity (in kg/(m.s))
   integer, save :: iviscl
 
   !> dynamic turbulent viscosity
   integer, save :: ivisct
 
-  !> dynamic molecular viscosity (in kg/(m.s)) at the previous time-step
-  integer, save :: ivisla
-
-  !> dynamic turbulent viscosity at the previous time-step
-  integer, save :: ivista
-
-  !> specific heat \f$ C_p \f$
-  integer, save :: icp
-
-  !> specific heat \f$ C_p \f$ at the previous time-step
-  integer, save :: icpa
-
-  !> Navier-Stokes source terms at the previous time-step
-  integer, save :: itsnsa
-
-  !> Turbulent source terms at the previous time-step
-  integer, save :: itstua
-
-  !> transported scalars source terms at the previous time-step
-  integer, save :: itssca(nscamx)
-
   !> error estimator for Navier-Stokes
   integer, save :: iestim(nestmx)
 
   !> interior and boundary convective mass flux key ids of the variables
   integer, save :: kimasf, kbmasf
 
+  !> constant diffusivity field id key for scalars
+  integer, save :: kvisl0
+
   !> variable diffusivity field id key for scalars
   integer, save :: kivisl
 
+  !> variable density field id key for scalars
+  integer, save :: kromsl
+
+  !> source terms at previous time step for 2nd order
+  integer, save :: kstprv
+
+  !> source terms at the current time step (used for limiters)
+  integer, save :: kst
+
+  !> turbulent schmidt key for scalars
+  integer, save :: ksigmas
+
   !> convective mass flux of the variables at the previous time-step
   integer, save :: ifluaa(nvarmx)
 
-  !> cell and boundary density field ids of the variables
-  integer, save :: icrom, ibrom
+  !> cell density field ids of the variables
+  integer, save :: icrom
 
-  !> cell density at the second previous time step key id of the variables
-  integer, save :: icroaa
+  !> boundary density field ids of the variables
+  integer, save :: ibrom
 
   !> field ids of the cell porosity
   integer, save :: ipori, iporf
@@ -228,8 +219,9 @@ module numvar
   !> dynamic constant of Smagorinsky
   integer, save :: ismago
 
-  !> field id of the anisotropic turbulent viscosity
-  integer, save :: ivsten
+  !> field ids of the anisotropic viscosity
+  !> \remark turbulent or Darcy module anisotropic diffusion
+  integer, save :: ivsten, ivstes
 
   !> Courant number
   integer, save :: icour
@@ -237,13 +229,13 @@ module numvar
   !> Fourier number
   integer, save :: ifour
 
-  !> Total pressure at cell centres
+  !> Total pressure at cell centers
   !> \f$ P_{tot} = P^\star +\rho \vect{g} \cdot (\vect{x}-\vect{x}_0) \f$
   integer, save :: iprtot
 
   !> Mesh velocity viscosity for the ALE module
   !> \remark might be orthotropic
-  integer, save :: ivisma(3)
+  integer, save :: ivisma
 
   !> pointer for dilatation source terms
   integer, save :: iustdy(nscamx)
@@ -254,14 +246,51 @@ module numvar
   !> pointer for thermal expansion coefficient
   integer, save :: ibeta
 
+  !> pointer for deduced mass fraction in case of gas mix
+  integer, save :: iddgas
+
+  !> pointer for gas mix molar mass
+  integer, save :: igmxml
+
   !> field id of the stresses at boundary  (if post-processed)
   integer, save :: iforbr
 
   !>  field id of \f$y^+\f$ at boundary (if post-processed)
   integer, save :: iyplbr
 
-  !> \}
+  !>  field id of temperature at boundary
+  integer, save ::  itempb
 
+  !> field id of the square of the norm of the deviatoric
+  !> part of the deformation rate tensor (\f$S^2=2S_{ij}^D S_{ij}^D\f$).
+  !> Field defined only with the \f$k-\omega\f$ (SST) turbulence model
+  integer, save :: is2kw
+
+  !> field id of the divergence of the velocity. More precisely,
+  !> it is the trace of the velocity gradient (and not a finite volume
+  !> divergence term). In the cell \c iel,  \f$div(\vect{u})\f$ is given
+  !> by \c divukw(iel1). This array is defined only with the \f$k-\omega\f$ SST
+  !> turbulence model (because in this case it may be calculated at the same
+  !> time as \f$S^2\f$)
+  integer, save :: idivukw
+
+  !> field id of the strain rate tensor at the previous time step
+  integer, save :: istraio
+
+  !> \}
+  !----------------------------------------------------------------------------
+  ! Numerical properties
+  !----------------------------------------------------------------------------
+
+  !> \defgroup numerical_prop Numerical properties
+
+  !> \addtogroup numerical_prop
+  !> \{
+
+  !> Weighting for gradient calculation on variables
+  integer, save :: kwgrec
+
+  !> \}
   !----------------------------------------------------------------------------
   ! Mapping to field structures
   !----------------------------------------------------------------------------
@@ -274,9 +303,6 @@ module numvar
   !> Field id for variable i
   integer, save :: ivarfl(nvarmx)
 
-  !> Field id for property i
-  integer, save :: iprpfl(npromx)
-
   !> Field id for the dttens tensor
   integer, save :: idtten
 
@@ -284,6 +310,81 @@ module numvar
 
   !=============================================================================
 
+  interface
+
+    !---------------------------------------------------------------------------
+
+    !> \cond DOXYGEN_SHOULD_SKIP_THIS
+
+    !---------------------------------------------------------------------------
+
+    ! Interface to C function retrieving the number of species in the gas mix
+    ! if gas mix model is enabled (igmix)
+
+    subroutine cs_f_gas_mix_get_pointers(nscasp) &
+      bind(C, name='cs_f_gas_mix_get_pointers')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      type(c_ptr), intent(out) :: nscasp
+    end subroutine cs_f_gas_mix_get_pointers
+
+    !---------------------------------------------------------------------------
+
+    !> (DOXYGEN_SHOULD_SKIP_THIS) \endcond
+
+    !---------------------------------------------------------------------------
+
+  end interface
+
+  !=============================================================================
+
+contains
+
+  !=============================================================================
+
+  !> \addtogroup field_map
+  !> \{
+
+  !> \anchor iprpfl
+  !> Identity function for compatibility with deprecated iprpfl array
+
+  elemental pure function iprpfl(f_id) result(r_f_id)
+
+    implicit none
+
+    ! Parameters
+
+    integer, intent(in) :: f_id
+    integer             :: r_f_id
+
+    ! Function body
+
+    r_f_id = f_id
+
+  end function iprpfl
+
+  !> \}
+
+  !> \brief Initialize Fortran gas mix API.
+  !> This maps Fortran pointers to global C variables.
+
+  subroutine gas_mix_options_init
+
+    use, intrinsic :: iso_c_binding
+    implicit none
+
+    ! Local variables
+
+    type(c_ptr) :: c_nscasp
+
+    call cs_f_gas_mix_get_pointers(c_nscasp)
+
+    call c_f_pointer(c_nscasp, nscasp)
+
+  end subroutine gas_mix_options_init
+
+  !=============================================================================
+
   !> \}
 
 end module numvar
