@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,22 +36,6 @@ module paramx
 
   !=============================================================================
 
-  ! nprcmx : nombre max de proprietes physiques aux cellules (total) =
-  !          nscamx (Lambda) + 7 (rho,Cp,viscl,visct,cou,fou,iprtot)
-  !                          + 4 (estim)
-  ! nprfmx : nombre max de proprietes physiques aux faces internes =
-  !          nscamx (flumas) + 2*(flumas,alp)
-  ! nprbmx : nombre max de proprietes physiques aux faces de bord =
-  !          nscamx (flumab) + 3*(flumab,alp, romb)
-  ! npromx : nombre max de proprietes physiques tout confondu
-  !          majore par nprcmx+nprfmx+nprbmx
-  ! ngrdmx : nombre max de grandeurs =
-  !          nvarmx + npromx
-  ! nsmamx : nombre max de cases pour les tableaux termes source de masse
-  !          nvarmx + 1 pour smacel
-  ! nvppmx : nombre de variables pour affichages
-  !          ngrdmx + 20 (20 couvre dt, tpucou, et une marge de 16 ...)
-
   !> maximum number of scalars solutions of an
   !> advection equation, apart from the variables of the turbulence model
   !> \f$ (k, \varepsilon, R_{ij}, \omega, \varphi, \overline{f}, \alpha, \nu_t\f$)
@@ -62,44 +46,8 @@ module paramx
   !> maximal number of variables = nscamx + 12 (u,v,w,P,Rij,e,alp)
   integer   nvarmx
 
-  !> maximal number of physical properties at cells.
-  !> = nscamx (Lambda) + 7 (rho,Cp,viscl,visct,cou,fou,iprtot) + 4 (estim)
-  integer   nprcmx
-
-  !> maximal number of physical properties at internal faces.
-  !> = nscamx (flumas) + 2*(flumas,alp)
-  integer   nprfmx
-
-  !> maximal number of physical properties at external faces.
-  !> = nscamx (flumab) + 3*(flumab,alp, romb)
-  integer   nprbmx
-
-  !> maximal number of physical properties,
-  !> increased by nprcmx.
-  !> They will be stored in the arrays \c propce
-  integer   npromx
-
-  !> maximal number of physical quantities
-  !> = nvarmx + npromx
-  integer   ngrdmx
-
-  !> maximal size of mass source terms arrays.
-  !> (= nvarmx + 1 for smacel)
-  integer   nsmamx
-
-  !> number of displayed variables.
-  !> = ngrdmx + 20 (20 > dt, tpucou, increased by 16 ...)
-  integer   nvppmx
-
   parameter(nscamx=200)
   parameter(nvarmx=nscamx+12)
-  parameter(nprcmx=nscamx+11)
-  parameter(nprfmx=nscamx+ 2)
-  parameter(nprbmx=nscamx+ 3)
-  parameter(npromx=nprcmx+ nprfmx+nprbmx)
-  parameter(ngrdmx=nvarmx+ npromx)
-  parameter(nsmamx=nvarmx+ 1)
-  parameter(nvppmx=ngrdmx+20)
 
   !> Maximal possible boundary condition types
   integer    ntypmx
@@ -113,8 +61,8 @@ module paramx
   !> if \ref itypfb=ientre: inlet face.
   !> -  Zero-flux condition for pressure and Dirichlet condition for all
   !> other variables. The value of the Dirichlet must be given in
-  !> \ref rcodcl(ifac,ivar,1) for every value of \ref ivar, except for
-  !> \ref ivar=ipr. The other values of \ref rcodcl and
+  !> \ref rcodcl "rcodcl"(ifac,ivar,1) for every value of \c ivar, except for
+  !> \c ivar = \ref ipr. The other values of \ref rcodcl and
   !> \ref icodcl are filled automatically.
   integer   ientre
 
@@ -200,7 +148,7 @@ module paramx
   !> quantities.
   integer   isopcf
 
-  !> if \ref itypfb=isopcf: mixed inlet for compressible flow with given total pressure
+  !> if \ref itypfb=iephcf: mixed inlet for compressible flow with given total pressure
   !>                        and total enthalpy (reservoir boundary conditions).
   !>  - Boundary values are obtained by solving a Riemann problem between an inner (values
   !> at boundary cells center) and an outer state.
@@ -219,23 +167,51 @@ module paramx
   ! TODO : not available yet.
   integer   ieqhcf
 
-  ! TODO : mot absent de la doc
+  !> \anchor icscpl
+  !> code/code coupling condition
   integer   icscpl
 
+  !> \anchor icscpd
+  !> code/code coupling condition with decentered flux
+  integer   icscpd
+
   !> \anchor ifrent
   !> if \ref itypfb=ifrent: free entrance based on Bernoulli equation when
   !> the flow is incoming, standard outlet when outgoing
   integer   ifrent
 
+  !> \anchor ifresf
+  !> if \ref itypfb=ifresf: free surface for mobile mesh boundary condition
+  !>  - Homogeneous Neumann boundary condition for velocity and total energy (seen by the reconstruction
+  !> gradients and the diffusion operator).
+  !>  - Dirichlet (\ref icodcl=1) for the pressure.
+  !> Alse a boundary condition type for mesh velocity in ALE for modelling
+  !> free surface (\f$ \vect{u} \cdot \vect{S} = \vect{w} \cdot \vect{S} \f$).
+  integer   ifresf
+
+  !> \anchor i_convective_inlet
+  !> if \ref itypfb=i_convective_inlet: inlet face where the total mass flux is
+  !>                                    prescribed.
+  !> -  Zero-flux condition for pressure and Dirichlet condition for all
+  !> other variables. The value of the Dirichlet must be given in
+  !> \ref rcodcl "rcodcl"(ifac,ivar,1) for every value of \c ivar, except for
+  !> \c ivar = \ref numvar::ipr "ipr". The other values of \ref rcodcl and
+  !> \ref icodcl are filled automatically.
+  !> The diffusive flux is CANCELLED (therefore the total mass flux is due to
+  !> convection only).
+  integer   i_convective_inlet
+
+
   parameter(iindef=1, ientre=2, isolib=3, isymet=4, iparoi=5,       &
-            iparug=6, iesicf=7, isspcf=8, isopcf=9, iephcf=10,       &
-            ieqhcf=11, icscpl=12, ifrent=13)
+            iparug=6, iesicf=7, isspcf=8, isopcf=9, iephcf=10,      &
+            ieqhcf=11, icscpl=12, icscpd=13, ifrent=14, ifresf=15,  &
+            i_convective_inlet=16)
 
   !> maximal number of valuators for Navier-Stokes
   integer    nestmx
   parameter (nestmx=4)
 
-  !> error estimator for Navier-Stokes.
+  !> Error estimator for Navier-Stokes.
   !> iest = iespre: prediction, (default name: EsPre).
   !> After the velocity prediction step (yielding \f$\vect{u}^*\f$), the
   !> estimator \f$\eta^{\,pred}_{\,i,k}(\vect{u}^*)\f$, local variable calculated
@@ -265,7 +241,7 @@ module paramx
   !> reconstruction methods are perfect and the associated system is solved exactly.
   integer   iespre
 
-  !> error estimator for Navier-Stokes.
+  !> Error estimator for Navier-Stokes.
   !> iest = iesder: drift  (default name: EsDer).
   !> The estimator \f$\eta^{\,der}_{\,i,k}(\vect{u}^{\,n+1})\f$ is based on the
   !> following quantity (intrinsic to the code):
@@ -281,7 +257,8 @@ module paramx
   !> solved exactly.
   integer   iesder
 
-  !> error estimator for Navier-Stokes.  iest = iescor: correction, (default name: EsCor).
+  !> Error estimator for Navier-Stokes.
+  !> iest = iescor: correction, (default name: EsCor).
   !> The estimator \f$ \eta^{\,corr}_{\,i,k}(\vect{u}^{\,n+1})\f$ comes directly
   !> from the mass flow calculated with the updated velocity field:
   !> \f{eqnarray*}{
@@ -300,7 +277,7 @@ module paramx
   !> centers is made in a set of  functions with null divergence.
   integer   iescor
 
-  !> error estimator for Navier-Stokes. iest = iestot: total, (default name: EsTot).
+  !> Error estimator for Navier-Stokes. iest = iestot: total, (default name: EsTot).
   !> The estimator \f$ \eta^{\,tot}_{\,i,k}(\vect{u}^{\,n+1})\f$, local variable
   !> calculated at every cell \f$\Omega_i\f$, is based on the quantity
   !> \f$\vect{\mathcal R}^{\,tot}(\vect{u}^{\,n+1})\f$, which represents the
@@ -319,7 +296,7 @@ module paramx
   !> \f$ \eta^{\,tot}_{\,i,k}(\vect{u}^{\,n+1})= {|\Omega_i|}^{\,(k-2)/2}\ ||\vect{\mathcal R}^{\,tot}(\vect{u}^{\,n+1})||
   !> _{{I\hspace{-.25em}L}^{2}(\Omega_i)} \f$
   !> - The mass flux in the convective term is recalculated from \f$\vect{u}^{n+1}\f$
-  !> expressed at the cell centres (and not taken from the updated mass flow at the
+  !> expressed at the cell centers (and not taken from the updated mass flow at the
   !> faces).
   !> - As for the prediction estimator:
   !>   - The first family, k=1, suppresses the
@@ -331,13 +308,6 @@ module paramx
   integer   iestot
   parameter (iespre=1, iesder=2, iescor=3, iestot=4)
 
-  !> maximum number of calculated time-averages (default value: 50)
-  integer    nbmomx
-
-  !> maximum degree of the time-averages (default value: 5)
-  integer   ndgmox
-  parameter (nbmomx = 50, ndgmox = 5)
-
   ! conditions aux limites possibles pour la vitesse de maillage en ale
 
   !> \anchor ibfixe
@@ -353,8 +323,8 @@ module paramx
   !> - In the case where all the nodes of a face have a imposed displacement,
   !> it is not necessary to fill the tables with boundary conditions
   !> mesh velocity for this face, they will be erased. In the other case,
-  !> the value of the Dirichlet must be given in \ref rcodcl(ifac,ivar,1) for
-  !> every value of \ref ivar (\ref iuma, \ref ivma and \ref iwma).
+  !> the value of the Dirichlet must be given in \ref rcodcl "rcodcl"(ifac,ivar,1)
+  !> for every value of \c ivar (\ref iuma, \ref ivma and \ref iwma).
   !> The other boxes of \ref rcodcl and \ref icodcl are completed automatically.
   !> The tangential mesh velocity is taken like a tape speed under the
   !> boundary conditions of wall for the fluid, except if wall fluid velocity
@@ -362,22 +332,21 @@ module paramx
   !> (in which case it is this speed which is considered).
   integer   ivimpo
 
-  !> \anchor ifresf
-  !> boundary condition type for mesh velocity in ALE for modelling
-  !> free surface (\f$ \vect{u} \cdot \vect{S} = \vect{w} \cdot \vect{S} \f$).
-  integer   ifresf
-
-  parameter(ibfixe=1, igliss=2, ivimpo=3, ifresf=4)
+  parameter(ibfixe=1, igliss=2, ivimpo=3)
 
   !> maximum number of structures in ALE
   integer    nstrmx
   parameter (nstrmx=200)
 
-  !> number of variable of rtp moved in field structure (C transition)
-  integer    nflown
-  parameter (nflown=4)
   !=============================================================================
 
   !> \}
 
+  ! Temporary former Fortan subroutine names so that the user soubroutines
+  ! do not compile if the deprecated name (now used for the C routine) is used.
+  integer cs_user_boundary_conditions, cs_user_parameters,         &
+          cs_user_initialization, cs_user_physical_properties,     &
+          cs_user_extra_operations, ushist, cs_f_user_head_losses, &
+          usvpst
+
 end module paramx
