@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -37,13 +37,11 @@
 !______________________________________________________________________________!
 !> \param[in]     nfbrps        number of boundary faces to postprocess
 !> \param[in]     lstfbr        list of boundary faces to postprocess
-!> \param[in]     propce        physical properties at cell centers
 !> \param[out]    bflux         boundary heat flux at selected faces
 !_______________________________________________________________________________
 
 subroutine post_boundary_thermal_flux &
  ( nfbrps , lstfbr ,                                              &
-   propce ,                                                       &
    bflux )
 
 !===============================================================================
@@ -64,6 +62,7 @@ use period
 use mesh
 use field
 use field_operator
+use cs_c_bindings
 
 !===============================================================================
 
@@ -73,22 +72,27 @@ implicit none
 
 integer, intent(in)                                :: nfbrps
 integer, dimension(nfbrps), intent(in)             :: lstfbr
-double precision, dimension(ncelet, *), intent(in) :: propce
 double precision, dimension(nfbrps), intent(out)   :: bflux
 
 ! Local variables
 
-integer ::         inc, iccocg
+integer ::         inc, iccocg, f_id
 integer ::         ifac, iloc, ivar
 integer ::         iel
-integer ::         ipcvsl, ipcvst, iflmab
+integer ::         iflmab
 
-double precision :: xvsl  , srfbn
-double precision :: visct , flumab, diipbx, diipby, diipbz, tcel
+double precision :: cpp   , srfbn , heq
+double precision :: flumab, diipbx, diipby, diipbz
 
-double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
+double precision, allocatable, dimension(:) :: theipb
 double precision, allocatable, dimension(:,:) :: grad
-double precision, dimension(:), pointer :: bmasfl, tscalp
+double precision, dimension(:), pointer :: cpro_cp
+double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
+double precision, dimension(:), pointer :: bmasfl, tscalp, hextp, hintp, dist_theipb
+
+type(var_cal_opt) :: vcopt
+
+logical(c_bool), dimension(:), pointer ::  cpl_faces
 
 !===============================================================================
 
@@ -98,6 +102,8 @@ if (iscalt.gt.0) then
 
   ivar   = isca(iscalt)
 
+  f_id = ivarfl(ivar)
+
   ! Boundary condition pointers for gradients and advection
 
   call field_get_coefa_s(ivarfl(ivar), coefap)
@@ -108,23 +114,37 @@ if (iscalt.gt.0) then
   call field_get_coefaf_s(ivarfl(ivar), cofafp)
   call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
+  ! Boundary condition pointers for diffusion with coupling
+
+  call field_get_hext(f_id, hextp)
+  call field_get_hint(f_id, hintp)
+
   ! Pointers to fields and properties
 
   call field_get_val_prev_s(ivarfl(ivar), tscalp)
 
-  if (ivisls(iscalt).gt.0) then
-    ipcvsl = ipproc(ivisls(iscalt))
-  else
-    ipcvsl = 0
+  if (iscacp(iscalt).eq.1 .and. icp.ge.0) then
+    call field_get_val_s(icp, cpro_cp)
   endif
-  ipcvst = ipproc(ivisct)
+
   call field_get_key_int(ivarfl(ivar), kbmasf, iflmab)
   call field_get_val_s(iflmab, bmasfl)
 
   ! Compute variable values at boundary faces
 
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+  allocate(theipb(nfabor))
+  theipb = 0.d0
+
+  do iloc = 1, nfbrps
+    ifac = lstfbr(iloc)
+    iel = ifabor(ifac)
+    theipb(ifac) = tscalp(iel)
+  enddo
+
   ! Reconstructed fluxes
-  if (ircflu(ivar) .gt. 0 .and. itbrrb.eq.1) then
+  if (vcopt%ircflu .gt. 0 .and. itbrrb.eq.1) then
 
     ! Compute gradient of temperature / enthalpy
 
@@ -134,14 +154,13 @@ if (iscalt.gt.0) then
     iccocg = 1
 
     call field_gradient_scalar &
-      (ivarfl(ivar), 1, imrgra, inc,         &
+      (ivarfl(ivar), 0, imrgra, inc,         &
        iccocg,                               &
        grad)
 
-    ! Compute diffusive and convective flux using reconstructed temperature
+    ! Compute reconstructed temperature
 
     do iloc = 1, nfbrps
-
       ifac = lstfbr(iloc)
       iel = ifabor(ifac)
 
@@ -149,51 +168,57 @@ if (iscalt.gt.0) then
       diipby = diipb(2,ifac)
       diipbz = diipb(3,ifac)
 
-      tcel =   tscalp(iel)                                                  &
-             + diipbx*grad(1,iel) + diipby*grad(2,iel) + diipbz*grad(3,iel)
-
-      if (ipcvsl.gt.0) then
-        xvsl = propce(iel,ipcvsl)
-      else
-        xvsl = visls0(iscalt)
-      endif
-      srfbn = max(surfbn(ifac), epzero**2)
-      visct  = propce(iel,ipcvst)
-      flumab = bmasfl(ifac)
-
-      bflux(iloc) =                (cofafp(ifac) + cofbfp(ifac)*tcel)   &
-                    - flumab/srfbn*(coefap(ifac) + coefbp(ifac)*tcel)
-
+      theipb(ifac) = theipb(ifac)       &
+                   + diipbx*grad(1,iel) &
+                   + diipby*grad(2,iel) &
+                   + diipbz*grad(3,iel)
     enddo
 
     deallocate(grad)
+  endif
 
-  else ! If flux is not reconstructed
+  if (vcopt%icoupl.gt.0) then
+    call field_get_coupled_faces(ivarfl(ivar), cpl_faces)
+    allocate(dist_theipb(nfabor))
+    call cs_ic_field_dist_data_by_face_id(ivarfl(ivar), 1, theipb, dist_theipb)
+  endif
 
-    ! Compute diffusive and convective flux using non-reconstructed temperature
+  do iloc = 1, nfbrps
 
-    do iloc = 1, nfbrps
+    ifac = lstfbr(iloc)
+    iel = ifabor(ifac)
 
-      ifac = lstfbr(iloc)
-      iel = ifabor(ifac)
-
-      tcel = tscalp(iel)
-
-      if (ipcvsl.gt.0) then
-        xvsl = propce(iel,ipcvsl)
+    if (iscacp(iscalt).eq.1) then
+      if (icp.ge.0) then
+        cpp = cpro_cp(iel)
       else
-        xvsl = visls0(iscalt)
+        cpp = cp0
       endif
-      srfbn = max(surfbn(ifac), epzero**2)
-      visct  = propce(iel,ipcvst)
-      flumab = bmasfl(ifac)
+    else
+      cpp = 1.d0
+    endif
 
-      bflux(iloc) =                (cofafp(ifac) + cofbfp(ifac)*tcel)   &
-                    - flumab/srfbn*(coefap(ifac) + coefbp(ifac)*tcel)
+    srfbn = max(surfbn(ifac), epzero**2)
+    flumab = bmasfl(ifac)
 
-    enddo
+    bflux(iloc) = (cofafp(ifac) + cofbfp(ifac)*theipb(ifac))   &
+                - flumab*cpp/srfbn*(coefap(ifac) + coefbp(ifac)*theipb(ifac))
+    ! here bflux = 0 if current face is coupled
 
-  endif ! test on reconstruction
+    if (vcopt%icoupl.gt.0) then
+      if (cpl_faces(ifac)) then
+        heq = hextp(ifac) * hintp(ifac) / ((hextp(ifac) + hintp(ifac))*srfbn)
+        bflux(iloc) = heq*(theipb(ifac)-dist_theipb(ifac))
+      endif
+    endif
+
+  enddo
+
+  if (vcopt%icoupl.gt.0) then
+    deallocate(dist_theipb)
+  endif
+
+  deallocate(theipb)
 
 else ! if thermal variable is not available
 
@@ -218,156 +243,6 @@ end subroutine post_boundary_thermal_flux
 ! Function:
 ! ---------
 
-!> \brief Compute temperature at boundary.
-
-!> If working with enthalpy, compute an enthalpy.
-
-!-------------------------------------------------------------------------------
-! Arguments
-!______________________________________________________________________________.
-!  mode           name          role                                           !
-!______________________________________________________________________________!
-!> \param[in]     nfbrps        number of boundary faces to postprocess
-!> \param[in]     lstfbr        list of boundary faces to postprocess
-!> \param[out]    btemp         boundary temperature at selected faces
-!_______________________________________________________________________________
-
-subroutine post_boundary_temperature &
- ( nfbrps , lstfbr ,                                               &
-   btemp )
-
-!===============================================================================
-
-!===============================================================================
-! Module files
-!===============================================================================
-
-use paramx
-use pointe
-use entsor
-use cstnum
-use cstphy
-use optcal
-use numvar
-use parall
-use period
-use mesh
-use field
-use field_operator
-
-!===============================================================================
-
-implicit none
-
-! Arguments
-
-integer, intent(in)                                :: nfbrps
-integer, dimension(nfbrps), intent(in)             :: lstfbr
-double precision, dimension(nfbrps), intent(out)   :: btemp
-
-! Local variables
-
-integer ::         inc, iccocg
-integer ::         iel, ifac, iloc, ivar
-integer ::         itplus, itstar
-
-double precision :: diipbx, diipby, diipbz
-double precision :: tcel
-
-double precision, allocatable, dimension(:,:) :: grad
-double precision, dimension(:), pointer :: tplusp, tstarp
-double precision, dimension(:), pointer :: tscalp
-
-!===============================================================================
-
-! pointers to T+ and T* if saved
-
-call field_get_id_try('tplus', itplus)
-call field_get_id_try('tstar', itstar)
-
-if (itstar.ge.0 .and. itplus.ge.0) then
-
-  ivar = isca(iscalt)
-
-  call field_get_val_prev_s(ivarfl(ivar), tscalp)
-
-  call field_get_val_s (itplus, tplusp)
-  call field_get_val_s (itstar, tstarp)
-
-  ! Compute variable values at boundary faces
-
-  ! Reconstructed fluxes
-  if (ircflu(ivar) .gt. 0 .and. itbrrb.eq.1) then
-
-    ! Allocate a temporary array for the gradient calculation
-    allocate(grad(3,ncelet))
-
-    inc = 1
-    iccocg = 1
-
-    ! Compute gradient of temperature / enthalpy
-
-    call field_gradient_scalar &
-      (ivarfl(ivar), 1, imrgra, inc, iccocg,                               &
-       grad)
-
-    ! Compute reconstructed value in boundary cells
-
-    do iloc = 1, nfbrps
-
-      ifac = lstfbr(iloc)
-      iel = ifabor(ifac)
-
-      diipbx = diipb(1,ifac)
-      diipby = diipb(2,ifac)
-      diipbz = diipb(3,ifac)
-      tcel =   tscalp(iel)                                                  &
-             + diipbx*grad(1,iel) + diipby*grad(2,iel) + diipbz*grad(3,iel)
-
-      btemp(iloc) = tcel - tplusp(ifac)*tstarp(ifac)
-
-    enddo
-
-    deallocate(grad)
-
-  else ! If flux is not reconstructed
-
-    do iloc = 1, nfbrps
-
-      ifac = lstfbr(iloc)
-      iel = ifabor(ifac)
-
-      tcel = tscalp(iel)
-
-      btemp(iloc) = tcel - tplusp(ifac)*tstarp(ifac)
-
-    enddo
-
-  endif
-
-else ! default if not computable
-
-  do iloc = 1, nfbrps
-    btemp(iloc) = -1.d0
-  enddo
-
-endif
-
-!--------
-! Formats
-!--------
-
-!----
-! End
-!----
-
-return
-end subroutine post_boundary_temperature
-
-!===============================================================================
-! Function:
-! ---------
-
 !> \brief Compute Nusselt number near boundary.
 
 !-------------------------------------------------------------------------------
@@ -377,13 +252,11 @@ end subroutine post_boundary_temperature
 !______________________________________________________________________________!
 !> \param[in]     nfbrps        number of boundary faces to postprocess
 !> \param[in]     lstfbr        list of boundary faces to postprocess
-!> \param[in]     propce        physical properties at cell centers
 !> \param[out]    bnussl        Nusselt near boundary
 !_______________________________________________________________________________
 
 subroutine post_boundary_nusselt &
  ( nfbrps , lstfbr ,                                              &
-   propce ,                                                       &
    bnussl )
 
 !===============================================================================
@@ -404,6 +277,7 @@ use period
 use mesh
 use field
 use field_operator
+use cs_c_bindings
 
 !===============================================================================
 
@@ -413,23 +287,28 @@ implicit none
 
 integer, intent(in)                                :: nfbrps
 integer, dimension(nfbrps), intent(in)             :: lstfbr
-double precision, dimension(ncelet, *), intent(in) :: propce
 double precision, dimension(nfbrps), intent(out)   :: bnussl
 
 ! Local variables
 
 integer ::         inc, iccocg
 integer ::         iel, ifac, iloc, ivar
-integer ::         ipcvsl, ipcvst, itplus, itstar
+integer ::         ifcvsl, itplus, itstar
 
-double precision :: xvsl  , srfbn
-double precision :: visct , diipbx, diipby, diipbz
-double precision :: numer, denom, tcel
+double precision :: xvsl  , srfbn , heq
+double precision :: diipbx, diipby, diipbz
+double precision :: numer, denom
 
-double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
+double precision, allocatable, dimension(:) :: theipb
 double precision, allocatable, dimension(:,:) :: grad
+double precision, dimension(:), pointer :: cviscl
+double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
 double precision, dimension(:), pointer :: tplusp, tstarp
-double precision, dimension(:), pointer :: tscalp
+double precision, dimension(:), pointer :: tscalp, hextp, hintp, dist_theipb
+
+type(var_cal_opt) :: vcopt
+
+logical(c_bool), dimension(:), pointer ::  cpl_faces
 
 !===============================================================================
 
@@ -440,36 +319,38 @@ call field_get_id_try('tstar', itstar)
 
 if (itstar.ge.0 .and. itplus.ge.0) then
 
-  ivar   = isca(iscalt)
+  ivar = isca(iscalt)
 
   call field_get_val_prev_s(ivarfl(ivar), tscalp)
 
-  call field_get_val_s (itplus, tplusp)
-  call field_get_val_s (itstar, tstarp)
+  call field_get_val_s(itplus, tplusp)
+  call field_get_val_s(itstar, tstarp)
 
   ! Boundary condition pointers for diffusion
 
   call field_get_coefaf_s(ivarfl(ivar), cofafp)
   call field_get_coefbf_s(ivarfl(ivar), cofbfp)
 
-  ! Physical property pointers
+  ! Boundary condition pointers for diffusion with coupling
 
-  if (ivisls(iscalt).gt.0) then
-    ipcvsl = ipproc(ivisls(iscalt))
-  else
-    ipcvsl = 0
-  endif
-  ipcvst = ipproc(ivisct)
+  call field_get_hext(ivarfl(ivar), hextp)
+  call field_get_hint(ivarfl(ivar), hintp)
 
   ! Compute variable values at boundary faces
 
+  call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
+
+  allocate(theipb(nfabor))
+  theipb = 0.d0
+
+  do iloc = 1, nfbrps
+    ifac = lstfbr(iloc)
+    iel = ifabor(ifac)
+    theipb(ifac) = tscalp(iel)
+  enddo
+
   ! Reconstructed fluxes
-  if (ircflu(ivar) .gt. 0 .and. itbrrb.eq.1) then
-
-    ! Boundary condition pointers for gradients and advection
-
-    call field_get_coefa_s(ivarfl(ivar), coefap)
-    call field_get_coefb_s(ivarfl(ivar), coefbp)
+  if (vcopt%ircflu .gt. 0 .and. itbrrb.eq.1) then
 
     ! Compute gradient of temperature / enthalpy
 
@@ -479,75 +360,86 @@ if (itstar.ge.0 .and. itplus.ge.0) then
     iccocg = 1
 
     call field_gradient_scalar &
-      (ivarfl(ivar), 1, imrgra, inc, iccocg,                               &
+      (ivarfl(ivar), 0, imrgra, inc,         &
+       iccocg,                               &
        grad)
 
-    ! Compute using reconstructed temperature value in boundary cells
+    ! Compute reconstructed temperature
 
     do iloc = 1, nfbrps
-
       ifac = lstfbr(iloc)
       iel = ifabor(ifac)
 
       diipbx = diipb(1,ifac)
       diipby = diipb(2,ifac)
       diipbz = diipb(3,ifac)
-      tcel =   tscalp(iel)                                                 &
-             + diipbx*grad(1,iel) + diipby*grad(2,iel) + diipbz*grad(3,iel)
-
-      if (ipcvsl.gt.0) then
-        xvsl = propce(iel,ipcvsl)
-      else
-        xvsl = visls0(iscalt)
-      endif
-      srfbn = max(surfbn(ifac), epzero**2)
-      visct = propce(iel,ipcvst)
-
-      numer = (cofafp(ifac) + cofbfp(ifac)*tcel) * distb(ifac)
-      denom = xvsl * tplusp(ifac)*tstarp(ifac)
-
-      if (abs(denom).gt.1e-30) then
-        bnussl(iloc) = numer / denom
-      else
-        bnussl(iloc) = 0.d0
-      endif
 
+      theipb(ifac) = theipb(ifac)                                               &
+                   + diipbx*grad(1,iel) + diipby*grad(2,iel) + diipbz*grad(3,iel)
     enddo
 
     deallocate(grad)
-
-  else ! If flux is not reconstructed
-
-    ! Compute using non-reconstructed temperature value in boundary cells
-
-    do iloc = 1, nfbrps
-
-      ifac = lstfbr(iloc)
-      iel = ifabor(ifac)
-
-      tcel = tscalp(iel)
-
-      if (ipcvsl.gt.0) then
-        xvsl = propce(iel,ipcvsl)
-      else
-        xvsl = visls0(iscalt)
-      endif
-      srfbn = max(surfbn(ifac), epzero**2)
-      visct = propce(iel,ipcvst)
-
-      numer = (cofafp(ifac) + cofbfp(ifac)*tcel) * distb(ifac)
-      denom = xvsl * tplusp(ifac)*tstarp(ifac)
-
-      if (abs(denom).gt.1e-30) then
-        bnussl(iloc) = numer / denom
-      else
-        bnussl(iloc) = 0.d0
-      endif
-
-    enddo
-
   endif
 
+  if (vcopt%icoupl.gt.0) then
+    call field_get_coupled_faces(ivarfl(ivar), cpl_faces)
+    allocate(dist_theipb(nfabor))
+    call cs_ic_field_dist_data_by_face_id(ivarfl(ivar), 1, theipb, dist_theipb)
+  endif
+
+  ! Physical property pointers
+
+  call field_get_key_int (ivarfl(ivar), kivisl, ifcvsl)
+  if (ifcvsl .ge. 0) then
+    call field_get_val_s(ifcvsl, cviscl)
+  endif
+
+  ! Boundary condition pointers for gradients and advection
+
+  call field_get_coefa_s(ivarfl(ivar), coefap)
+  call field_get_coefb_s(ivarfl(ivar), coefbp)
+
+  ! Compute using reconstructed temperature value in boundary cells
+
+  do iloc = 1, nfbrps
+
+    ifac = lstfbr(iloc)
+    iel = ifabor(ifac)
+
+    if (ifcvsl.ge.0) then
+      xvsl = cviscl(iel)
+    else
+      xvsl = visls0(iscalt)
+    endif
+
+    srfbn = surfbn(ifac)
+
+    numer = (cofafp(ifac) + cofbfp(ifac)*theipb(ifac)) * distb(ifac)
+    ! here numer = 0 if current face is coupled
+
+    if (vcopt%icoupl.gt.0) then
+      if (cpl_faces(ifac)) then
+        heq = hextp(ifac) * hintp(ifac) / ((hextp(ifac) + hintp(ifac))*srfbn)
+        numer = heq*(theipb(ifac)-dist_theipb(ifac)) * distb(ifac)
+      endif
+    endif
+
+    denom = xvsl * tplusp(ifac)*tstarp(ifac)
+
+    if (abs(denom).gt.1e-30) then
+      bnussl(iloc) = numer / denom
+    else
+      bnussl(iloc) = 0.d0
+    endif
+
+  enddo
+
+  if (vcopt%icoupl.gt.0) then
+    deallocate(dist_theipb)
+  endif
+
+  deallocate(theipb)
+
 else ! default if not computable
 
   do iloc = 1, nfbrps
@@ -571,7 +463,7 @@ end subroutine post_boundary_nusselt
 ! Function:
 ! ---------
 
-!> \brief Compute efforts at boundary     .
+!> \brief Compute stress at boundary.
 
 !-------------------------------------------------------------------------------
 ! Arguments
@@ -580,12 +472,12 @@ end subroutine post_boundary_nusselt
 !______________________________________________________________________________!
 !> \param[in]     nfbrps        number of boundary faces to postprocess
 !> \param[in]     lstfbr        list of boundary faces to postprocess
-!> \param[out]    effort        efforts at selected faces
+!> \param[out]    stress        stress at selected faces
 !_______________________________________________________________________________
 
-subroutine post_efforts &
+subroutine post_stress &
  ( nfbrps , lstfbr ,                                                    &
-   effort )
+   stress )
 
 !===============================================================================
 
@@ -612,7 +504,7 @@ implicit none
 
 integer, intent(in)                                 :: nfbrps
 integer, dimension(nfbrps), intent(in)              :: lstfbr
-double precision, dimension(3, nfbrps), intent(out) :: effort
+double precision, dimension(3, nfbrps), intent(out) :: stress
 
 ! Local variables
 
@@ -627,9 +519,9 @@ call field_get_val_v(iforbr, forbr)
 do iloc = 1, nfbrps
   ifac = lstfbr(iloc)
   srfbn = surfbn(ifac)
-  effort(1,iloc) = forbr(1,ifac)/srfbn
-  effort(2,iloc) = forbr(2,ifac)/srfbn
-  effort(3,iloc) = forbr(3,ifac)/srfbn
+  stress(1,iloc) = forbr(1,ifac)/srfbn
+  stress(2,iloc) = forbr(2,ifac)/srfbn
+  stress(3,iloc) = forbr(3,ifac)/srfbn
 enddo
 
 !--------
@@ -641,13 +533,13 @@ enddo
 !----
 
 return
-end subroutine post_efforts
+end subroutine post_stress
 
 !===============================================================================
 ! Function:
 ! ---------
 
-!> \brief Extract efforts normal to the boundary     .
+!> \brief Extract stress normal to the boundary.
 
 !-------------------------------------------------------------------------------
 ! Arguments
@@ -656,10 +548,10 @@ end subroutine post_efforts
 !______________________________________________________________________________!
 !> \param[in]     nfbrps        number of boundary faces to postprocess
 !> \param[in]     lstfbr        list of boundary faces to postprocess
-!> \param[out]    effnrm        efforts normal to wall at selected faces
+!> \param[out]    effnrm        stress normal to wall at selected faces
 !_______________________________________________________________________________
 
-subroutine post_efforts_normal &
+subroutine post_stress_normal &
  ( nfbrps , lstfbr ,                                              &
    effnrm )
 
@@ -721,13 +613,13 @@ enddo
 !----
 
 return
-end subroutine post_efforts_normal
+end subroutine post_stress_normal
 
 !===============================================================================
 ! Function:
 ! ---------
 
-!> \brief Compute tangential efforts at boundary     .
+!> \brief Compute tangential stress at boundary.
 
 !-------------------------------------------------------------------------------
 ! Arguments
@@ -736,12 +628,12 @@ end subroutine post_efforts_normal
 !______________________________________________________________________________!
 !> \param[in]     nfbrps        number of boundary faces to postprocess
 !> \param[in]     lstfbr        list of boundary faces to postprocess
-!> \param[out]    effort        efforts at selected faces
+!> \param[out]    stress        stress at selected faces
 !_______________________________________________________________________________
 
-subroutine post_efforts_tangential &
+subroutine post_stress_tangential &
  ( nfbrps , lstfbr ,                                              &
-   effort )
+   stress )
 
 !===============================================================================
 
@@ -768,7 +660,7 @@ implicit none
 
 integer, intent(in)                                 :: nfbrps
 integer, dimension(nfbrps), intent(in)              :: lstfbr
-double precision, dimension(3, nfbrps), intent(out) :: effort
+double precision, dimension(3, nfbrps), intent(out) :: stress
 
 ! Local variables
 
@@ -790,9 +682,9 @@ do iloc = 1, nfbrps
   fornor =    forbr(1,ifac)*srfnor(1)                                 &
             + forbr(2,ifac)*srfnor(2)                                 &
             + forbr(3,ifac)*srfnor(3)
-  effort(1,iloc) = (forbr(1,ifac) - fornor*srfnor(1)) / srfbn
-  effort(2,iloc) = (forbr(2,ifac) - fornor*srfnor(2)) / srfbn
-  effort(3,iloc) = (forbr(3,ifac) - fornor*srfnor(3)) / srfbn
+  stress(1,iloc) = (forbr(1,ifac) - fornor*srfnor(1)) / srfbn
+  stress(2,iloc) = (forbr(2,ifac) - fornor*srfnor(2)) / srfbn
+  stress(3,iloc) = (forbr(3,ifac) - fornor*srfnor(3)) / srfbn
 enddo
 
 !--------
@@ -804,5 +696,4 @@ enddo
 !----
 
 return
-end subroutine post_efforts_tangential
-
+end subroutine post_stress_tangential
