@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -49,7 +49,6 @@
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 !> \param[in]     nvar          total number of variables
-!> \param[in]     nscal         total number of scalars
 !> \param[in]     ncesmp        number of cells with mass source term
 !> \param[in]     icetsm        index of cells with mass source term
 !> \param[in]     dt            time step (per cell)
@@ -79,12 +78,10 @@ use optcal
 use albase
 use parall
 use period
-use mltgrd
-use lagpar
-use lagran
 use cplsat
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -102,20 +99,17 @@ double precision smacel(ncesmp,nvar)
 
 ! Local variables
 
-character*80     chaine
+character(len=80) :: chaine
 integer          lchain
-integer          iccocg, inc   , init  , isym  , ipol  , isqrt
+integer          iccocg, inc   , init  , isym
 integer          ii, iel   , ifac
-integer          ireslp, nswmpr
-integer          isweep, niterf, icycle
+integer          nswmpr
+integer          isweep, niterf
 integer          nswrgp, imligp, iwarnp
 integer          iflmas, iflmab
 integer          idiffp, iconvp, ndircp
-integer          nitmap, imgrp , ncymap, nitmgp
-integer          iinvpe
-integer          nagmax, npstmg
 integer          ibsize, iesize, iphydp
-integer          imucpp
+integer          imucpp, f_id0, f_id
 double precision residu
 double precision thetap
 double precision epsrgp, climgp, extrap, epsilp
@@ -130,6 +124,10 @@ double precision, allocatable, dimension(:) :: cfapot, cfbpot
 double precision, allocatable, dimension(:) :: viscf, viscb
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: brom, crom, croma
+double precision, dimension(:), pointer :: cpro_rho_mass, bpro_rho_mass
+double precision, dimension(:), pointer :: brom_eos, crom_eos
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 
@@ -149,6 +147,9 @@ allocate(pot(ncelet), pota(ncelet), dpot(ncelet), rhs(ncelet))
 chaine = 'potential       '
 lchain = 16
 
+! --- non field pointer
+f_id0 = -1
+
 ! --- Physical quantities
 call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
@@ -166,18 +167,6 @@ isym  = 1
 ibsize = 1
 iesize = 1
 
-if (iresol(ipr).eq.-1) then
-  ireslp = 0
-  ipol   = 0
-  if (iconv(ipr).gt.0) then
-    ireslp = 1
-    ipol   = 0
-  endif
-else
-  ireslp = mod(iresol(ipr)+10000,1000)
-  ipol   = (iresol(ipr)-ireslp)/1000
-endif
-
 ! Boundary conditions on the potential (homogenous Neumann)
 do ifac = 1, nfabor
 
@@ -228,9 +217,7 @@ enddo
 ! 3. Residual of the system if needed
 !===============================================================================
 
-isqrt = 1
-
-call prodsc(ncel,isqrt,rhs,rhs,rnorm)
+rnorm = sqrt(cs_gdot(ncel,rhs,rhs))
 
 !===============================================================================
 ! 4. Building of the linear system to solve
@@ -244,7 +231,9 @@ enddo
 
 ! ---> Face diffusibility scalar
 
-if (idiff(ipr).ge.1) then
+call field_get_key_struct_var_cal_opt(ivarfl(ipr), vcopt)
+
+if (vcopt%idiff.ge.1) then
 
     call viscfa &
   ( imvisf ,                                                       &
@@ -262,8 +251,8 @@ else
 
 endif
 
-iconvp = iconv(ipr)
-idiffp = idiff(ipr)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
 ! Strengthen the diagonal
 ndircp = 0
 
@@ -279,33 +268,11 @@ call matrix &
    rvoid  , dam    , xam    )
 
 !===============================================================================
-! 5. Preparation of the Algebraic Multigrid
-!===============================================================================
-
-if (imgr(ipr).gt.0) then
-
-  ! --- Building of the mesh hierarchy
-
-  iwarnp = iwarni(ipr)
-  nagmax = nagmx0(ipr)
-  npstmg = ncpmgr(ipr)
-
-  call clmlga &
-  !==========
- ( chaine(1:16) ,    lchain ,                                     &
-   isym   , ibsize , iesize , nagmax , npstmg , iwarnp ,          &
-   ngrmax , ncegrm ,                                              &
-   rlxp1  ,                                                       &
-   dam    , xam    )
-
-endif
-
-!===============================================================================
-! 6. Solving (Loop over the non-orthogonalities)
+! 5. Solving (Loop over the non-orthogonalities)
 !===============================================================================
 
 ! --- Number of sweeps
-nswmpr = nswrsm(ipr)
+nswmpr = vcopt%nswrsm
 
 ! --- Variables are set to 0
 !       pot     is the potential
@@ -317,17 +284,22 @@ do iel = 1,ncel
   dpot(iel) = 0.d0
 enddo
 
-relaxp = relaxv(ipr)
+relaxp = vcopt%relaxv
 
-! (Test to modify if needed: must be scticter than
+! (Test to modify if needed: must be sctricly greater than
 ! the test in the conjugate gradient)
-tcrite = 10.d0*epsrsm(ipr)*rnorm
+tcrite = 10.d0*vcopt%epsrsm*rnorm
 
 ! Reconstruction loop (beginning)
 !--------------------------------
 isweep = 1
 residu = rnorm
 
+! Writing
+if (vcopt%iwarni.ge.2) then
+  write(nfecra,1400)chaine(1:16),isweep,residu, relaxp
+endif
+
 do while (isweep.le.nswmpr.and.residu.gt.tcrite)
 
   ! --- Solving on the increment dpot
@@ -335,31 +307,17 @@ do while (isweep.le.nswmpr.and.residu.gt.tcrite)
     dpot(iel) = 0.d0
   enddo
 
-  nitmap = nitmax(ipr)
-  imgrp  = imgr(ipr)
-  ncymap = ncymax(ipr)
-  nitmgp = nitmgf(ipr)
-  iwarnp = iwarni(ipr)
-  epsilp = epsilo(ipr)
+  epsilp = vcopt%epsilo
 
-  ! The potential is a scalar => no problem for the periodicity of rotation
-  ! (iinvpe=1)
-  iinvpe = 1
-
-  call invers &
-  !==========
- ( chaine(1:16)    , isym   , ibsize , iesize ,                   &
-   ipol   , ireslp , nitmap , imgrp  ,                            &
-   ncymap , nitmgp ,                                              &
-   iwarnp , niterf , icycle , iinvpe ,                            &
-   epsilp , rnorm  , residu ,                                     &
-   dam    , xam    , rhs   , dpot   )
+  call sles_solve_native(-1, chaine,                                &
+                         isym, ibsize, iesize, dam, xam,            &
+                         epsilp, rnorm, niterf, residu, rhs, dpot)
 
   ! Update the increment of potential
   if (idtvar.ge.0.and.isweep.le.nswmpr.and.residu.gt.tcrite) then
     do iel = 1, ncel
       pota(iel) = pot(iel)
-      pot(iel)  = pota(iel) + relaxv(ipr)*dpot(iel)
+      pot(iel)  = pota(iel) + vcopt%relaxv*dpot(iel)
     enddo
   ! If it is the last sweep, update with the total increment
   else
@@ -378,26 +336,26 @@ do while (isweep.le.nswmpr.and.residu.gt.tcrite)
     iccocg = 1
     init = 1
     inc  = 0
-    nswrgp = nswrgr(ipr)
-    imligp = imligr(ipr)
-    iwarnp = iwarni(ipr)
-    epsrgp = epsrgr(ipr)
-    climgp = climgr(ipr)
-    extrap = extrag(ipr)
+    nswrgp = vcopt%nswrgr
+    imligp = vcopt%imligr
+    iwarnp = vcopt%iwarni
+    epsrgp = vcopt%epsrgr
+    climgp = vcopt%climgr
+    extrap = vcopt%extrag
     ! This option should be adapted to iphydr = 1
     iphydp = 0
 
     call itrgrp &
     !==========
-   ( init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp , &
-     iwarnp ,                                                       &
-     epsrgp , climgp , extrap ,                                     &
-     rvoid  ,                                                       &
-     pot    ,                                                       &
-     clapot , clbpot ,                                              &
-     cfapot , cfbpot ,                                              &
-     viscf  , viscb  ,                                              &
-     dt     , dt     , dt     ,                                     &
+   ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp ,   &
+     iwarnp ,                                                                  &
+     epsrgp , climgp , extrap ,                                                &
+     rvoid  ,                                                                  &
+     pot    ,                                                                  &
+     clapot , clbpot ,                                                         &
+     cfapot , cfbpot ,                                                         &
+     viscf  , viscb  ,                                                         &
+     dt     ,                                                                  &
      rhs   )
 
     do iel = 1, ncel
@@ -405,13 +363,13 @@ do while (isweep.le.nswmpr.and.residu.gt.tcrite)
     enddo
 
     ! --- Convergence test
-    call prodsc(ncel, isqrt, rhs, rhs, residu)
+    residu = sqrt(cs_gdot(ncel, rhs, rhs))
 
-    if (iwarni(ipr).ge.2) then
+    if (vcopt%iwarni.ge.2) then
       if (rnorm.ge.epzero) then
-        write(nfecra,1440) chaine(1:16),isweep,residu/rnorm, relaxp
+        write(nfecra,1400) chaine(1:16),isweep,residu/rnorm, relaxp
       else
-        write(nfecra,1440) chaine(1:16),isweep,residu, relaxp
+        write(nfecra,1400) chaine(1:16),isweep,residu, relaxp
       endif
     endif
 
@@ -420,7 +378,7 @@ do while (isweep.le.nswmpr.and.residu.gt.tcrite)
 enddo
 ! --- Reconstruction loop (end)
 
-if (iwarni(ipr).ge.2) then
+if (vcopt%iwarni.ge.2) then
   if (isweep.gt.nswmpr) write(nfecra,1600) chaine(1:16), nswmpr
 endif
 
@@ -431,24 +389,23 @@ iccocg = 1
 init = 0
 inc  = 0
 iphydp = 0
-nswrgp = nswrgr(ipr)
-imligp = imligr(ipr)
-iwarnp = iwarni(ipr)
-epsrgp = epsrgr(ipr)
-climgp = climgr(ipr)
-extrap = extrag(ipr)
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+iwarnp = vcopt%iwarni
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
 
 call itrmas &
-!==========
- ( init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp , &
-   iwarnp ,                                                       &
-   epsrgp , climgp , extrap ,                                     &
-   rvoid  ,                                                       &
-   pota   ,                                                       &
-   clapot , clbpot ,                                              &
-   cfapot , cfbpot ,                                              &
-   viscf  , viscb  ,                                              &
-   dt     , dt     , dt     ,                                     &
+ ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp ,     &
+   0      , iwarnp ,                                                           &
+   epsrgp , climgp , extrap ,                                                  &
+   rvoid  ,                                                                    &
+   pota   ,                                                                    &
+   clapot , clbpot ,                                                           &
+   cfapot , cfbpot ,                                                           &
+   viscf  , viscb  ,                                                           &
+   dt     ,                                                                    &
    imasfl , bmasfl )
 
 ! The last increment is not reconstructed to fullfill exactly the continuity
@@ -459,25 +416,43 @@ nswrgp = 0
 inc = 0
 
 call itrmas &
-!==========
- ( init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp , &
-   iwarnp ,                                                       &
-   epsrgp , climgp , extrap ,                                     &
-   rvoid  ,                                                       &
-   dpot   ,                                                       &
-   clapot , clbpot ,                                              &
-   cfapot , cfbpot ,                                              &
-   viscf  , viscb  ,                                              &
-   dt     , dt     , dt     ,                                     &
+ ( f_id0  , init   , inc    , imrgra , iccocg , nswrgp , imligp , iphydp ,     &
+   0      , iwarnp ,                                                           &
+   epsrgp , climgp , extrap ,                                                  &
+   rvoid  ,                                                                    &
+   dpot   ,                                                                    &
+   clapot , clbpot ,                                                           &
+   cfapot , cfbpot ,                                                           &
+   viscf  , viscb  ,                                                           &
+   dt     ,                                                                    &
    imasfl , bmasfl )
 
+! Update density (which is coherent with the mass)
+!-------------------------------------------------
+
+if (irovar.eq.1) then
+  call field_get_val_s(icrom, crom_eos)
+  call field_get_val_s(ibrom, brom_eos)
+
+  call field_get_id("density_mass", f_id)
+  call field_get_val_s(f_id, cpro_rho_mass)
+  call field_get_id("boundary_density_mass", f_id)
+  call field_get_val_s(f_id, bpro_rho_mass)
+
+  do iel = 1, ncelet
+    cpro_rho_mass(iel) = crom_eos(iel)
+  enddo
+
+  do ifac = 1, nfabor
+    bpro_rho_mass(ifac) = brom_eos(ifac)
+  enddo
+endif
+
 !===============================================================================
-! 7. Suppression of the mesh hierarchy
+! 6. Free solver setup
 !===============================================================================
 
-if (imgr(ipr).gt.0) then
-  call dsmlga(chaine(1:16), lchain)
-endif
+call sles_free_native(-1, chaine)
 
 ! Free memory
 deallocate(dam, xam)
@@ -491,7 +466,7 @@ deallocate(viscf, viscb)
 !--------
 
 #if defined(_CS_LANG_FR)
- 1440 format(1X,A16,' : sweep = ',I5,' norme second membre = ',E14.6,&
+ 1400 format(1X,A16,' : sweep = ',I5,' norme second membre = ',E14.6,&
              ', relaxp = ',E14.6)
  1600 format(                                                     &
 '@                                                            ',/,&
@@ -502,7 +477,7 @@ deallocate(viscf, viscb)
 
 #else
 
- 1440 format(1X,A16,' : sweep = ',I5,' right hand side norm = ',E14.6,&
+ 1400 format(1X,A16,' : sweep = ',I5,' right hand side norm = ',E14.6,&
              ', relaxp = ',E14.6)
  1600 format(                                                     &
 '@'                                                            ,/,&
