@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -33,17 +33,13 @@
 !------------------------------------------------------------------------------
 !> \param[in]     nvar          total number of variables
 !> \param[in]     nscal         total number of scalars
+!> \param[in]     iterns        Navier-Stokes iteration number
 !> \param[in]     dt            time step (per cell)
-!> \param[in]     rtp           calculated variables at cell centers
-!>                              (at current and previous time steps)
-!> \param[in,out] rtpa          calculated variables at cell centers
-!>                              (at current and previous time steps)
-!> \param[in]     propce        physical properties at cell centers
 !______________________________________________________________________________
 
 subroutine scalai &
  ( nvar   , nscal  ,                                              &
-   dt     , rtp    , rtpa   , propce )
+   iterns , dt     )
 
 !===============================================================================
 ! Module files
@@ -55,7 +51,6 @@ use entsor
 use optcal
 use cstphy
 use cstnum
-use lagdim
 use pointe
 use parall
 use ppppar
@@ -63,10 +58,12 @@ use ppthch
 use coincl
 use cpincl
 use ppincl
-use elincl
 use mesh
 use atchem
 use siream
+use field
+use cs_cf_bindings
+use cfpoin, only: hgn_relax_eq_st
 
 !===============================================================================
 
@@ -74,20 +71,21 @@ implicit none
 
 ! Arguments
 
-integer          nvar   , nscal
-
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
+integer          nvar, nscal, iterns
+double precision dt(ncelet)
 
 ! Local variables
 
-integer          iscal, ivar, iel
+integer          iscal, ivar, iel, isou
 integer          ii, iisc, itspdv, icalc, iappel
-integer          ispecf
+integer          ispecf, scal_id, f_id, f_dim
 
 double precision, allocatable, dimension(:) :: dtr
 double precision, allocatable, dimension(:) :: viscf, viscb
-double precision, allocatable, dimension(:) :: smbrs, rovsdt
+
+double precision, dimension(:), pointer :: cvar_var, cvara_var
+double precision, dimension(:,:), pointer :: cvar_vav, cvara_vav
+integer :: keyvar
 
 ! NOMBRE DE PASSAGES DANS LA ROUTINE
 
@@ -101,10 +99,11 @@ save             ipass
 ! 1. Initialization
 !===============================================================================
 
+call field_get_key_id("scalar_id", keyvar)
+
 ! Allocate temporary arrays for the species resolution
 allocate(dtr(ncelet))
 allocate(viscf(nfac), viscb(nfabor))
-allocate(smbrs(ncelet), rovsdt(ncelet))
 
 ipass = ipass + 1
 
@@ -112,7 +111,7 @@ ipass = ipass + 1
 
 if (ichemistry.ge.1 .and. nscal.gt.0) then
   ! Computation of kinetics rates
-  call kinrates(rtp, propce)
+  call kinrates()
   !==========
 endif
 
@@ -126,7 +125,7 @@ if (nscapp.gt.0) then
 
   if (ippmod(iphpar).ge.1) then
 
-    call ppinv2(nvar, nscal, dt, rtp, propce)
+    call ppinv2(nvar, nscal, dt)
     !==========
 
     ! TODO: check that the following loop may be removed, as it is
@@ -141,9 +140,22 @@ if (nscapp.gt.0) then
         do ii = 1, nscapp
           iscal = iscapp(ii)
           ivar  = isca(iscal)
-          do iel = 1, ncelet
-            rtpa (iel,ivar) = rtp(iel,ivar)
-          enddo
+          call field_get_dim(ivarfl(ivar), f_dim)
+          if (f_dim.eq.1) then
+            call field_get_val_s(ivarfl(ivar), cvar_var)
+            call field_get_val_prev_s(ivarfl(ivar), cvara_var)
+            do iel = 1, ncelet
+              cvara_var(iel) = cvar_var(iel)
+            enddo
+          else
+            call field_get_val_v(ivarfl(ivar), cvar_vav)
+            call field_get_val_prev_v(ivarfl(ivar), cvara_vav)
+            do iel = 1, ncelet
+              do isou = 1, 3
+                cvara_vav(isou,iel)= cvar_vav(isou,iel)
+              enddo
+            enddo
+          endif
         enddo
       endif
     endif
@@ -157,7 +169,7 @@ if (nscapp.gt.0) then
     call cs_coal_masstransfer &
     !=======================
    ( ncelet , ncel   ,        &
-     rtpa   , propce , volume )
+     volume )
 
   endif
 
@@ -166,10 +178,7 @@ if (nscapp.gt.0) then
 
   if (ippmod(icfuel).ne.-1) then
 
-    call cs_fuel_masstransfer &
-    !=======================
-   ( ncelet , ncel   ,        &
-     rtpa   , propce )
+    call cs_fuel_masstransfer(ncelet, ncel)
 
   endif
 
@@ -178,11 +187,6 @@ if (nscapp.gt.0) then
 !                QUE LES SCALAIRES SOIENT RESOLUS AVANT
 !                LEUR VARIANCE ASSOCIEE
 
-
-
-
-
-
 ! ---> Boucle sur les scalaires physiques particulieres.
 !      On peut imaginer a la place des resolutions couplees.
 !      Ici, on ne donne qu'un exemple.
@@ -212,7 +216,7 @@ if (nscapp.gt.0) then
 
     ispecf = 0
 
-    if ( ippmod(icompf).ge.0 ) then
+    if (ippmod(icompf).ge.0 .and. iterns.eq.-1) then
 
       if ( iscal.eq.itempk ) then
         ispecf = 1
@@ -232,10 +236,8 @@ if (nscapp.gt.0) then
    ncepdc , ncetsm ,                                              &
    iscal  ,                                                       &
    icepdc , icetsm , itypsm ,                                     &
-   dt     , rtp    , rtpa   , propce ,                            &
-   ckupdc , smacel ,                                              &
-   viscf  , viscb  ,                                              &
-   smbrs  , rovsdt )
+   dt     ,                                                       &
+   ckupdc , smacel )
 
       endif
 
@@ -281,20 +283,39 @@ if (nscapp.gt.0) then
         call csexit (1)
       endif
 
+      call field_get_dim(ivarfl(isca(iscal)), f_dim)
+
+      if (f_dim.eq.1) then
 
 ! ---> Appel a covofi pour la resolution
 
-      call covofi                                                 &
-      !==========
- ( nvar   , nscal  ,                                              &
-   ncepdc , ncetsm ,                                              &
-   iisc   , itspdv ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
-   dtr    , rtp    , rtpa   , propce , tslagr ,                   &
-   ckupdc , smacel ,                                              &
-   viscf  , viscb  ,                                              &
-   smbrs  , rovsdt )
+        call covofi                                                 &
+        !==========
+   ( nvar   , nscal  ,                                              &
+     ncepdc , ncetsm , nfbpcd , ncmast ,                            &
+     iterns , iisc   , itspdv ,                                     &
+     icepdc , icetsm , ifbpcd , ltmast ,                            &
+     itypsm , itypcd , itypst ,                                     &
+     dtr    , tslagr ,                                              &
+     ckupdc , smacel , spcond , svcond , flxmst ,                   &
+     viscf  , viscb  )
 
+      else
+
+! ---> Appel a covofv pour la resolution
+
+        call covofv                                                 &
+        !==========
+   ( nvar   , nscal  ,                                              &
+     ncepdc , ncetsm ,                                              &
+     iterns , iisc   ,                                              &
+     icepdc , icetsm ,                                              &
+     itypsm ,                                                       &
+     dtr    ,                                                       &
+     ckupdc , smacel ,                                              &
+     viscf  , viscb  )
+
+      endif
 
 ! ---> Versions Electriques
 !             Effet Joule
@@ -303,9 +324,8 @@ if (nscapp.gt.0) then
 
 !     On calcule ici j, E, j.E reels et imagimaires
 
-      if ( ippmod(ieljou).ge.1 .or.                               &
-           ippmod(ielarc).ge.1 .or.                               &
-           ippmod(ielion).ge.1       ) then
+      if ((ippmod(ieljou).ge.1 .or. ippmod(ielarc).ge.1)  &
+        .and. iterns.eq.-1) then
 
 
 !     On utilise le  fait que les scalaires sont dans l'ordre
@@ -318,13 +338,17 @@ if (nscapp.gt.0) then
 
         if (ippmod(ielarc).ge.1.or.ippmod(ieljou).eq.1             &
              .or.ippmod(ieljou).eq.3) then
-          if (iscal.eq.ipotr) then
+          call field_get_id('elec_pot_r', f_id)
+          call field_get_key_int(f_id, keyvar, scal_id)
+          if (iscal.eq.scal_id) then
             icalc = 1
           endif
         endif
 !     On y va apres PotI si on est en Joule avec PotI
         if (ippmod(ieljou).eq.2 .or. ippmod(ieljou).eq.4) then
-          if (iscal.eq.ipoti) then
+          call field_get_id('elec_pot_i', f_id)
+          call field_get_key_int(f_id, keyvar, scal_id)
+          if (iscal.eq.scal_id) then
             icalc = 1
           endif
         endif
@@ -334,7 +358,7 @@ if (nscapp.gt.0) then
 !     Calcul de j, E et j.E
           iappel = 1
 
-          call elflux(iappel, propce)
+          call elflux(iappel)
           !==========
 
 
@@ -342,10 +366,7 @@ if (nscapp.gt.0) then
 
           if ( ielcor .eq.1  .and. ntcabs .gt. 1 ) then
 
-            call elreca(dt, rtp, propce)
-            !==========
-
-            call uselrc(nvar, nscal, dt)
+            call elreca(dt)
             !==========
 
           endif
@@ -364,18 +385,23 @@ endif
 
 !     On calcule ici A, B, jxB
 
-if (ippmod(ielarc).ge.1) then
+if (ippmod(ielarc).ge.1 .and. iterns.eq.-1) then
 
 !     On utilise le  fait que les scalaires sont dans l'ordre
 !       H, PotR, [PotI], [A] pour faire le calcul de A, B, jxB
 !       apres la determination et le recalage de j
   iappel = 2
 
-  call elflux(iappel, propce)
-  !==========
-
+  call elflux(iappel)
 endif
 
+! Compressible homogeneous two-phase model:
+! return to equilibrium source term step for volume, mass, energy fractions
+if (ippmod(icompf).eq.1.and.hgn_relax_eq_st.ge.0) then
+  call cs_cf_hgn_source_terms_step
+endif
+
+
 !===============================================================================
 ! 3. TRAITEMENT DES SCALAIRES UTILISATEURS STANDARD
 !     On voit que meme s'ils sont numerotes en premier, on peut les
@@ -439,19 +465,39 @@ if (nscaus.gt.0) then
       call csexit (1)
     endif
 
+    call field_get_dim(ivarfl(isca(iscal)), f_dim)
+
+    if (f_dim.eq.1) then
 
 ! ---> Appel a covofi pour la resolution
 
-    call covofi                                                   &
-    !==========
- ( nvar   , nscal  ,                                              &
-   ncepdc , ncetsm ,                                              &
-   iisc   , itspdv ,                                              &
-   icepdc , icetsm , itypsm ,                                     &
-   dtr    , rtp    , rtpa   , propce , tslagr ,                   &
-   ckupdc , smacel ,                                              &
-   viscf  , viscb  ,                                              &
-   smbrs  , rovsdt )
+      call covofi                                                   &
+      !==========
+   ( nvar   , nscal  ,                                              &
+     ncepdc , ncetsm , nfbpcd , ncmast ,                            &
+     iterns , iisc   , itspdv ,                                     &
+     icepdc , icetsm , ifbpcd , ltmast ,                            &
+     itypsm , itypcd , itypst ,                                     &
+     dtr    , tslagr ,                                              &
+     ckupdc , smacel , spcond , svcond , flxmst ,                   &
+     viscf  , viscb  )
+
+    else
+
+! ---> Appel a covofv pour la resolution
+
+        call covofv                                                 &
+        !==========
+   ( nvar   , nscal  ,                                              &
+     ncepdc , ncetsm ,                                              &
+     iterns , iisc   ,                                              &
+     icepdc , icetsm ,                                              &
+     itypsm ,                                                       &
+     dtr    ,                                                       &
+     ckupdc , smacel ,                                              &
+     viscf  , viscb  )
+
+      endif
 
 
 ! ---> Fin de la Boucle sur les scalaires utilisateurs.
@@ -461,19 +507,18 @@ endif
 
 ! Atmospheric gaseous chemistry
 ! Resolution of chemical evolution of species
-if (ichemistry.ge.1 .and. nscal.gt.0) then
-  call compute_gaseous_chemistry(dt, rtpa, rtp)
+if (ichemistry.ge.1 .and. nscal.gt.0 .and. iterns.eq.-1) then
+  call compute_gaseous_chemistry(dt)
 endif
 
 ! Atmospheric aerosol chemistry
-if (iaerosol.eq.1 .and. nscal.gt.0) then
-  call compute_siream(dt, rtp, propce)
+if (iaerosol.eq.1 .and. nscal.gt.0 .and. iterns.eq.-1) then
+  call compute_siream(dt)
 endif
 
 ! Free memory
 deallocate(dtr)
 deallocate(viscf, viscb)
-deallocate(smbrs, rovsdt)
 
 !===============================================================================
 ! 4.  FORMATS
@@ -488,17 +533,17 @@ deallocate(smbrs, rovsdt)
 '@ @@ ATTENTION : ARRET A LA RESOLUTION DES SCALAIRES         ',/,&
 '@    =========                                               ',/,&
 '@    SCALAIRE NUMERO ',I10                                    ,/,&
-'@    ISCAVR(',I10   ,') DOIT ETRE UN ENTIER                  ',/,&
+'@    iscavr(',I10   ,') DOIT ETRE UN ENTIER                  ',/,&
 '@      POSITIF OU NUL ET                                     ',/,&
 '@      INFERIEUR OU EGAL A NSCAL = ',I10                      ,/,&
 '@    IL VAUT ICI ',I10                                        ,/,&
 '@                                                            ',/,&
 '@  Le calcul ne peut etre execute.                           ',/,&
 '@                                                            ',/,&
-'@  Si ISCAVR(I) est nul, le scalaire I n est pas une variance',/,&
-'@  Si ISCAVR(I) est positif, le scalaire I est une variance :',/,&
+'@  Si iscavr(I) est nul, le scalaire I n est pas une variance',/,&
+'@  Si iscavr(I) est positif, le scalaire I est une variance :',/,&
 '@    il s agit de la variance des fluctuations du scalaire J ',/,&
-'@    dont le numero est ISCAVR(I)                            ',/,&
+'@    dont le numero est iscavr(I)                            ',/,&
 '@                                                            ',/,&
 '@  Verifier les parametres.                                  ',/,&
 '@  Contacter l''assistance.                                  ',/,&
@@ -515,16 +560,16 @@ deallocate(smbrs, rovsdt)
 '@ @@ WARNING: ABORT WHILE SOLVING SCALARS EQUATIONS          ',/,&
 '@    ========                                                ',/,&
 '@    SCALAR NUMBER ',I10                                      ,/,&
-'@    ISCAVR(',I10   ,') MUST BE A POSITIVE OR NULL INTEGER   ',/,&
+'@    iscavr(',I10   ,') MUST BE A POSITIVE OR NULL INTEGER   ',/,&
 '@      AND LOWER OR EQUAL THAN NSCAL = ', I10                 ,/,&
 '@    ITS VALUE IS ',I10                                       ,/,&
 '@                                                            ',/,&
 '@  The calculaton will not be run.                           ',/,&
 '@                                                            ',/,&
-'@  If ISCAVR(I) is null, the scalar I is not a variance      ',/,&
-'@  If ISCAVR(I) is positive, the scalar I is a variance:     ',/,&
+'@  If iscavr(I) is null, the scalar I is not a variance      ',/,&
+'@  If iscavr(I) is positive, the scalar I is a variance:     ',/,&
 '@    it is the variance of the fluctuations of the scalar J  ',/,&
-'@    whose number is ISCAVR(I)                               ',/,&
+'@    whose number is iscavr(I)                               ',/,&
 '@                                                            ',/,&
 '@  Check parameters.                                         ',/,&
 '@  Contact support.                                          ',/,&
@@ -535,7 +580,7 @@ deallocate(smbrs, rovsdt)
 #endif
 
 !----
-! FIN
+! End
 !----
 return
 
