@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,35 +20,31 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine schtmp &
-!================
-
- ( nscal  , iappel ,                                              &
-   propce )
-
 !===============================================================================
-! FONCTION :
+! Function :
 ! ----------
 
-! TRAITEMENT DU FLUX DE MASSE, DE LA VISCOSITE, DE LA MASSE
-! VOLUMIQUE, DE LA CHALEUR SPECIFIQUE ET DU TABLEAU TSNSA
-! DANS LE CAS D'UN THETA SCHEMA
+!> \file schtmp.f90
+!>
+!> \brief Management of the mass flux, the viscosity, the density, the specific
+!> heat  and the tsnsa array in case of a theta-scheme.
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#massflux"><b>mass flux</b></a> section
+!> of the theory guide for more informations.
+!-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
-! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! nscal            ! i  ! <-- ! total number of scalars                        !
-! iappel           ! e  ! <-- ! numero de l'appel (avant ou apres              !
-!                  !    !     ! phyvar                                         !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-!__________________!____!_____!________________________________________________!
+! Arguments                                                                    !
+!______________________________________________________________________________!
+!  mode           name               role                                      !
+!______________________________________________________________________________!
+!> \param[in]     nscal              total number of scalars
+!> \param[in]     iappel             call number (before of after phyvar
+!______________________________________________________________________________
 
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
+subroutine schtmp &
+ ( nscal  , iappel )
 
 !===============================================================================
 ! Module files
@@ -73,110 +69,110 @@ implicit none
 
 integer          nscal  , iappel
 
-double precision propce(ncelet,*)
-
 ! Local variables
 
 integer          iel    , ifac   , iscal
 integer          iflmas , iflmab
-integer          ipcvis , ipcvst
-integer          ipcvsa , ipcvta , ipcvsl
-integer          iicp   , iicpa
-double precision flux   , theta  , aa, bb, viscos, xmasvo, varcp
+integer          f_id
+integer          key_t_ext_id, icpext
+integer          iviext
+integer          iroext
+
+double precision flux   , theta  , viscos, varcp
+
 double precision, dimension(:), pointer :: i_mass_flux, b_mass_flux
 double precision, dimension(:), pointer :: i_mass_flux_prev, b_mass_flux_prev
-double precision, dimension(:), pointer :: brom, crom, broma, croma
+double precision, dimension(:), pointer :: brom, crom
+double precision, dimension(:), pointer :: cpro_viscl, cpro_visct
+double precision, dimension(:), pointer :: cpro_cp, cpro_visls
+double precision, dimension(:), pointer :: cproa_cp, cproa_visls, cproa_visct
+double precision, dimension(:), pointer :: cpro_rho_mass, bpro_rho_mass
+
 !===============================================================================
 
 !===============================================================================
-! 0. INITIALISATION
+! 0. Initialisation
 !===============================================================================
 
+call field_get_key_id("time_extrapolated", key_t_ext_id)
+
 !===============================================================================
-! 1.  AU TOUT DEBUT DE LA BOUCLE EN TEMPS
+! 1. At the really beginning of the time step
 !===============================================================================
 
 if (iappel.eq.1) then
 
-! --- Application du schema en temps sur le flux de masse
-!     Soit F le flux de masse
-!       - si istmpf = 2 (schema non standard, theta>0 en fait = 0.5)
-!         i_mass_flux      contient F_(n-2+theta) et on y met F(n-1+theta)
-!         i_mass_flux_prev contient F_(n-1+theta) et
-!                                    on y met une extrapolation en n+theta
-!       - si istmpf = 1 (schema non standard, theta=0)
-!         i_mass_flux      contient deja F_n et
-!         i_mass_flux_prev n'est pas utilise : on ne fait rien
-!       - sinon : istmpf = 0 (schema standard, theta= -999)
-!         i_mass_flux      et i_mass_flux_prev contiennent tous les deux F(n)
-!                                            : on ne fait rien
-
-
-!     Ordre 2 en temps pour le flux (theta = 0.5) a entrer dans navsto
-!       Flux convectif = 2F(n-1+theta)-F(n-2+theta)
-!       Flux conserve  =  F(n-1+theta)
-!     Au premier pas de temps, l'ancien a ete initialise dans inivar (a 0)
-!       en suite de calcul, les deux ont ete relus.
-
+  ! --- Store the previous mass flux (n-1->n) in *_mass_flux_prev
   if (istmpf.eq.2) then
     call field_get_key_int(ivarfl(iu), kimasf, iflmas)
     call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
-    call field_get_val_s(iflmas, i_mass_flux)
-    call field_get_val_s(iflmab, b_mass_flux)
-    call field_get_val_prev_s(iflmas, i_mass_flux_prev)
-    call field_get_val_prev_s(iflmab, b_mass_flux_prev)
-    do ifac = 1 , nfac
-      flux                   =                          i_mass_flux(ifac)
-      i_mass_flux(ifac)      = 2.d0*i_mass_flux(ifac) - i_mass_flux_prev(ifac)
-      i_mass_flux_prev(ifac) = flux
-    enddo
-    do ifac = 1 , nfabor
-      flux                   =                          b_mass_flux(ifac)
-      b_mass_flux(ifac)      = 2.d0*b_mass_flux(ifac) - b_mass_flux_prev(ifac)
-      b_mass_flux_prev(ifac) = flux
-    enddo
+    call field_current_to_previous(iflmas)
+    call field_current_to_previous(iflmab)
   endif
 
-!     Les valeurs courantes ecrasent les valeurs anterieures
-!       en cas d'extrapolation en temps (theta > 0)
-!       Pour RHO, on le fait en double si ICALHY = 1 (et sur NCELET)
-!     Au debut du calcul les flux nouveau et ancien ont ete initialises inivar
-  if (iroext.gt.0) then
+  ! If required, the density at time step n-1 is updated
+  ! Note that for VOF and dilatable algorithms, density at time step n-2
+  ! is also updated
+  ! Note that at the begining of the calculation, previous values have been
+  ! initialized by inivar
+  if (irovar.gt.0) then
     call field_current_to_previous(icrom)
     call field_current_to_previous(ibrom)
-  endif
-  if (iviext.gt.0) then
-    ipcvis = ipproc(iviscl)
-    ipcvst = ipproc(ivisct)
-    ipcvsa = ipproc(ivisla)
-    ipcvta = ipproc(ivista)
-    do iel = 1, ncel
-      propce(iel,ipcvsa) = propce(iel,ipcvis)
-      propce(iel,ipcvta) = propce(iel,ipcvst)
-    enddo
-  endif
-  if (icpext.gt.0) then
-    if (icp   .gt.0) then
-      iicp   = ipproc(icp   )
-      iicpa  = ipproc(icpa  )
-      do iel = 1, ncel
-        propce(iel,iicpa ) = propce(iel,iicp  )
+
+    if (idilat.gt.1.or.ivofmt.gt.0) then
+      ! Save the latest density field seen by the mass equation
+      ! it will be updated in the correction step.
+      call field_get_id("density_mass", f_id)
+      call field_get_val_s(f_id, cpro_rho_mass)
+      call field_get_id("boundary_density_mass", f_id)
+      call field_get_val_s(f_id, bpro_rho_mass)
+
+      call field_get_val_s(icrom, crom)
+      do iel = 1, ncelet
+        cpro_rho_mass(iel) = crom(iel)
+      enddo
+
+      call field_get_val_s(ibrom, brom)
+      do ifac = 1, nfabor
+        bpro_rho_mass(ifac) = brom(ifac)
       enddo
     endif
   endif
 
-!     Remarque : si on faisant cette operation pour tous les
-!       scalaires, on la ferait plusieurs fois pour les scalaires
-!       ayant une variance
+  call field_get_key_int(iviscl, key_t_ext_id, iviext)
+  if (iviext.gt.0) then
+    call field_current_to_previous(iviscl)
+  endif
+  call field_get_key_int(ivisct, key_t_ext_id, iviext)
+  if (iviext.gt.0) then
+    call field_current_to_previous(ivisct)
+  endif
+  if (icp.ge.0) then
+    call field_get_key_int(icp, key_t_ext_id, icpext)
+    if (icpext.gt.0) then
+      call field_current_to_previous(icp)
+    endif
+  endif
+
+  ! Remarque : si on faisait cette operation pour tous les
+  ! scalaires, on la ferait plusieurs fois pour les scalaires
+  ! ayant une variance
   if (nscal.ge.1) then
     do iscal = 1, nscal
-      if (ivisls(iscal).gt.0.and.iscavr(iscal).le.0) then
-        if (ivsext(iscal).gt.0) then
-          ipcvsl = ipproc(ivisls(iscal))
-          ipcvsa = ipproc(ivissa(iscal))
-          do iel = 1, ncel
-            propce(iel,ipcvsa) = propce(iel,ipcvsl)
-          enddo
+      ! Diffusivity
+      call field_get_key_int (ivarfl(isca(iscal)), kivisl, f_id)
+      if (f_id.ge.0.and.iscavr(iscal).le.0) then
+        call field_get_key_int(f_id, key_t_ext_id, iviext)
+        if (iviext.gt.0) then
+          call field_current_to_previous(f_id)
+        endif
+      endif
+      ! Densisty
+      call field_get_key_int (ivarfl(isca(iscal)), kromsl, f_id)
+      if (f_id.ge.0.and.iscavr(iscal).le.0) then
+        call field_get_key_int(f_id, key_t_ext_id, iroext)
+        if (iroext.gt.0) then
+          call field_current_to_previous(f_id)
         endif
       endif
     enddo
@@ -203,6 +199,7 @@ elseif (iappel.eq.2) then
 
   if (initro.ne.1) then
     initro = 1
+    call field_get_key_int(icrom, key_t_ext_id, iroext)
     if (iroext.gt.0) then
       call field_current_to_previous(icrom)
       call field_current_to_previous(ibrom)
@@ -210,44 +207,38 @@ elseif (iappel.eq.2) then
   endif
   if (initvi.ne.1) then
     initvi = 1
+    call field_get_key_int(iviscl, key_t_ext_id, iviext)
     if (iviext.gt.0) then
-      ipcvis = ipproc(iviscl)
-      ipcvst = ipproc(ivisct)
-      ipcvsa = ipproc(ivisla)
-      ipcvta = ipproc(ivista)
-      do iel = 1, ncel
-        propce(iel,ipcvsa) = propce(iel,ipcvis)
-        propce(iel,ipcvta) = propce(iel,ipcvst)
-      enddo
+      call field_current_to_previous(iviscl)
+    endif
+    call field_get_key_int(ivisct, key_t_ext_id, iviext)
+    if (iviext.gt.0) then
+      call field_current_to_previous(ivisct)
     endif
   endif
   if (initcp.ne.1) then
     initcp = 1
-    if (icpext.gt.0) then
-      if (icp   .gt.0) then
-        iicp   = ipproc(icp   )
-        iicpa  = ipproc(icpa  )
-        do iel = 1, ncel
-          propce(iel,iicpa ) = propce(iel,iicp  )
-        enddo
+    if (icp.gt.0) then
+      call field_get_key_int(icp, key_t_ext_id, icpext)
+      if (icpext.gt.0) then
+        call field_current_to_previous(icp)
       endif
     endif
   endif
 
-!     Remarque : si on faisant cette operation pour tous les
-!       scalaires, on la ferait plusieurs fois pour les scalaires
-!       ayant une variance
+  ! Remarque : si on faisant cette operation pour tous les
+  ! scalaires, on la ferait plusieurs fois pour les scalaires
+  ! ayant une variance
   if (nscal.ge.1) then
     do iscal = 1, nscal
+      ! Diffusivity
       if (initvs(iscal).ne.1) then
         initvs(iscal) = 1
-        if (ivisls(iscal).gt.0.and.iscavr(iscal).le.0) then
-          if (ivsext(iscal).gt.0) then
-            ipcvsl = ipproc(ivisls(iscal))
-            ipcvsa = ipproc(ivissa(iscal))
-            do iel = 1, ncel
-              propce(iel,ipcvsa) = propce(iel,ipcvsl)
-            enddo
+        call field_get_key_int (ivarfl(isca(iscal)), kivisl, f_id)
+        if (f_id.ge.0.and.iscavr(iscal).le.0) then
+          call field_get_key_int(f_id, key_t_ext_id, iviext)
+          if (iviext.gt.0) then
+            call field_current_to_previous(f_id)
           endif
         endif
       endif
@@ -258,89 +249,80 @@ elseif (iappel.eq.2) then
 ! 2.2 EXTRAPOLATION DES NOUVELLES VALEURS
 ! =======================================
 
-! --- Extrapolation de la viscosite et de la masse volumique dans le cas d'un
+! --- Extrapolation de la viscosite dans le cas d'un
 !     theta schema
 !     A partir de Fn-1 et Fn on calcule Fn+theta
 !     On conserve les nouvelles valeurs dans l'ancien tableau pour
 !     retablir en fin de pas de temps
 
-!     Le calcul pour Rho est fait sur NCELET afin d'economiser un echange.
-
-  if (iroext.gt.0) then
-    call field_get_val_s(icrom, crom)
-    call field_get_val_prev_s(icrom, croma)
-    theta  = thetro
-    do iel = 1, ncelet
-      xmasvo = crom(iel)
-      crom(iel) = (1.d0+theta)*crom(iel) - theta*croma(iel)
-      croma(iel) = xmasvo
-    enddo
-    call field_get_val_s(ibrom, brom)
-    call field_get_val_prev_s(ibrom, broma)
-    do ifac = 1, nfabor
-      xmasvo = brom(ifac)
-      brom(ifac) = (1.d0+theta)*brom(ifac) - theta*broma(ifac)
-      broma(ifac) = xmasvo
-    enddo
-  endif
-  if (iviext.gt.0) then
-    ipcvis = ipproc(iviscl)
-    ipcvst = ipproc(ivisct)
-    ipcvsa = ipproc(ivisla)
-    ipcvta = ipproc(ivista)
+  call field_get_key_int(iviscl, key_t_ext_id, iviext)
+   if (iviext.gt.0) then
+    call field_get_val_s(iviscl, cpro_viscl)
+    call field_get_val_prev_s(iviscl, cproa_visls)
     theta  = thetvi
     do iel = 1, ncel
-      viscos = propce(iel,ipcvis)
-      propce(iel,ipcvis) = (1.d0+theta) * propce(iel,ipcvis)    &
-           -       theta  * propce(iel,ipcvsa)
-      propce(iel,ipcvsa) = viscos
-      viscos = propce(iel,ipcvst)
-      propce(iel,ipcvst) = (1.d0+theta) * propce(iel,ipcvst)    &
-           -       theta  * propce(iel,ipcvta)
-      propce(iel,ipcvta) = viscos
+      viscos = cpro_viscl(iel)
+      cpro_viscl(iel) = (1.d0+theta) * cpro_viscl(iel)    &
+           -       theta  * cproa_visls(iel)
+      cproa_visls(iel) = viscos
     enddo
   endif
-  if (icpext.gt.0) then
-    if (icp.gt.0) then
-      iicp   = ipproc(icp   )
-      iicpa  = ipproc(icpa  )
+
+  call field_get_key_int(ivisct, key_t_ext_id, iviext)
+  if (iviext.gt.0) then
+    call field_get_val_s(ivisct, cpro_visct)
+    call field_get_val_prev_s(ivisct, cproa_visct)
+    theta  = thetvi
+    do iel = 1, ncel
+      viscos = cpro_visct(iel)
+      cpro_visct(iel) = (1.d0+theta) * cpro_visct(iel)    &
+           -       theta  * cproa_visct(iel)
+      cproa_visct(iel) = viscos
+    enddo
+  endif
+  if (icp.gt.0) then
+    call field_get_key_int(icp, key_t_ext_id, icpext)
+    if (icpext.gt.0) then
+      call field_get_val_s(icp, cpro_cp)
+      call field_get_val_prev_s(icp, cproa_cp)
       theta  = thetcp
       do iel = 1, ncel
-        varcp  = propce(iel,iicp  )
-        propce(iel,iicp  ) = (1.d0+theta) * propce(iel,iicp  )  &
-             -       theta  * propce(iel,iicpa )
-        propce(iel,iicpa ) = varcp
+        varcp  = cpro_cp(iel)
+        cpro_cp(iel) = (1.d0+theta) * cpro_cp(iel)      &
+                      -      theta  * cproa_cp(iel)
+        cproa_cp(iel) = varcp
       enddo
     endif
   endif
 
-!     Remarque : si on faisant cette operation pour tous les
-!       scalaires, on la ferait plusieurs fois pour les scalaires
-!       ayant une variance ET CE SERAIT FAUX
+  !     Remarque : si on faisant cette operation pour tous les
+  !       scalaires, on la ferait plusieurs fois pour les scalaires
+  !       ayant une variance ET CE SERAIT FAUX
   if (nscal.ge.1) then
     do iscal = 1, nscal
-      if (ivisls(iscal).gt.0.and.iscavr(iscal).le.0) then
-        if (ivsext(iscal).gt.0) then
+      ! Diffusivity
+      call field_get_key_int (ivarfl(isca(iscal)), kivisl, f_id)
+      if (f_id.ge.0.and.iscavr(iscal).le.0) then
+        call field_get_key_int(f_id, key_t_ext_id, iviext)
+        if (iviext.gt.0) then
           theta  = thetvs(iscal)
-          ipcvsl = ipproc(ivisls(iscal))
-          ipcvsa = ipproc(ivissa(iscal))
-          if (ipcvsl.gt.0) then
-            do iel = 1, ncel
-              viscos = propce(iel,ipcvsl)
-              propce(iel,ipcvsl) = (1.d0+theta)*propce(iel,ipcvsl)&
-                                 -       theta *propce(iel,ipcvsa)
-              propce(iel,ipcvsa) = viscos
-            enddo
-          endif
+          call field_get_val_s(f_id, cpro_visls)
+          call field_get_val_prev_s(f_id, cproa_visls)
+          do iel = 1, ncel
+            viscos = cpro_visls(iel)
+            cpro_visls(iel) = (1.d0+theta)*cpro_visls(iel) &
+                                    -theta *cproa_visls(iel)
+            cproa_visls(iel) = viscos
+          enddo
         endif
       endif
+
     enddo
   endif
 
   return
 
 
-
 !===============================================================================
 ! 3.  JUSTE APRES NAVSTO, DANS LES BOUCLES U/P ET ALE
 !===============================================================================
@@ -351,9 +333,7 @@ elseif (iappel.eq.3) then
 !        On suppose qu'il n'y en a qu'un seul.
 
 !     si istmpf = 1 : standard : on ne fait rien
-!     si istmpf = 2 : ordre 2 (thetfl > 0 : = 0.5)
-!       on calcule f(n+theta) par interpolation a partir
-!       de F_(n-1+theta) et f(n+1) et on le met dans i_mass_flux
+!     si istmpf = 2 : ordre 2 (thetfl > 0 : = 0.5) : on ne fait rien
 !     si istmpf = 0 : explicite (thetfl = 0) : on remet F(n) dans
 !       i_mass_flux sauf a la derniere iteration (un traitement
 !       complementaire sera fait en iappel=4)
@@ -362,24 +342,13 @@ elseif (iappel.eq.3) then
 !     - a toutes les iterations si ISTMPF.NE.0
 !     - a toutes les iterations sauf la derniere si ISTMPF.EQ.0
 
-  call field_get_key_int(ivarfl(iu), kimasf, iflmas)
-  call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
-  call field_get_val_s(iflmas, i_mass_flux)
-  call field_get_val_s(iflmab, b_mass_flux)
-  call field_get_val_prev_s(iflmas, i_mass_flux_prev)
-  call field_get_val_prev_s(iflmab, b_mass_flux_prev)
-
-  if (istmpf.eq.2) then
-    theta  = thetfl
-    aa = 1.d0/(2.d0-theta)
-    bb = (1.d0-theta)/(2.d0-theta)
-    do ifac = 1 , nfac
-      i_mass_flux(ifac) = aa * i_mass_flux(ifac) + bb * i_mass_flux_prev(ifac)
-    enddo
-    do ifac = 1 , nfabor
-      b_mass_flux(ifac) = aa * b_mass_flux(ifac) + bb * b_mass_flux_prev(ifac)
-    enddo
-  else if (istmpf.eq.0) then
+  if (istmpf.eq.0) then
+    call field_get_key_int(ivarfl(iu), kimasf, iflmas)
+    call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
+    call field_get_val_s(iflmas, i_mass_flux)
+    call field_get_val_s(iflmab, b_mass_flux)
+    call field_get_val_prev_s(iflmas, i_mass_flux_prev)
+    call field_get_val_prev_s(iflmab, b_mass_flux_prev)
     do ifac = 1 , nfac
       i_mass_flux(ifac) = i_mass_flux_prev(ifac)
     enddo
@@ -400,11 +369,9 @@ elseif (iappel.eq.4) then
 !        On suppose qu'il n'y en a qu'un seul.
 
 !     Si istmpf = 1 : standard : on ne fait rien
-!     Si istmpf = 2 : ordre 2 (thetfl > 0 : = 0.5)
-!       on calcule F(n+theta) par interpolation a partir
-!       de F_(n-1+theta) et F(n+1) et on le met dans i_mass_flux)
+!     Si istmpf = 2 : ordre 2 (thetfl > 0 : = 0.5) : on ne fait rien
 !     Si istmpf = 0 : explicite (thetfl = 0)
-!       On sauvegarde F_(n+1) dans i_mas_flux_prev,mais on continue
+!       On sauvegarde F_(n+1) dans i_mass_flux_prev, mais on continue
 !       les calculs avec F_(n) mis dans i_mass_flux
 
 !     On retablira au dernier appel de schtmp pour istmpf = 0
@@ -424,17 +391,8 @@ elseif (iappel.eq.4) then
   call field_get_val_prev_s(iflmas, i_mass_flux_prev)
   call field_get_val_prev_s(iflmab, b_mass_flux_prev)
 
-  if (istmpf.eq.2) then
-    theta  = thetfl
-    aa = 1.d0/(2.d0-theta)
-    bb = (1.d0-theta)/(2.d0-theta)
-    do ifac = 1 , nfac
-      i_mass_flux(ifac) = aa * i_mass_flux(ifac) + bb * i_mass_flux_prev(ifac)
-    enddo
-    do ifac = 1 , nfabor
-      b_mass_flux(ifac) = aa * b_mass_flux(ifac) + bb * b_mass_flux_prev(ifac)
-    enddo
-  else if (istmpf.eq.0) then
+  ! Already checked
+  if (istmpf.eq.0) then
     do ifac = 1 , nfac
       flux = i_mass_flux(ifac)
       i_mass_flux(ifac) = i_mass_flux_prev(ifac)
@@ -485,51 +443,49 @@ elseif (iappel.eq.5) then
 ! 3.1 RETABLISSEMENT POUR LES PROPRIETES PHYSIQUES
 ! ================================================
 
-!     Le calcul pour Rho est fait sur NCELET afin d'economiser un echange.
-
-  if (iroext.gt.0) then
-    call field_get_val_s(icrom, crom)
-    call field_get_val_prev_s(icrom, croma)
-    do iel = 1, ncelet
-      crom(iel) = croma(iel)
-    enddo
-    call field_get_val_s(ibrom, brom)
-    call field_get_val_prev_s(ibrom, broma)
-    do ifac = 1, nfabor
-      brom(ifac) = broma(ifac)
-    enddo
-  endif
+  call field_get_key_int(iviscl, key_t_ext_id, iviext)
   if (iviext.gt.0) then
-    ipcvis = ipproc(iviscl)
-    ipcvst = ipproc(ivisct)
-    ipcvsa = ipproc(ivisla)
-    ipcvta = ipproc(ivista)
+    call field_get_val_s(iviscl, cpro_viscl)
+    call field_get_val_prev_s(iviscl, cproa_visls)
     do iel = 1, ncel
-      propce(iel,ipcvis) = propce(iel,ipcvsa)
-      propce(iel,ipcvst) = propce(iel,ipcvta)
+      cpro_viscl(iel) = cproa_visls(iel)
     enddo
   endif
-  if (icpext.gt.0) then
-    if (icp.gt.0) then
-      iicp   = ipproc(icp   )
-      iicpa  = ipproc(icpa  )
+
+  call field_get_key_int(ivisct, key_t_ext_id, iviext)
+  if (iviext.gt.0) then
+    call field_get_val_s(ivisct, cpro_visct)
+    call field_get_val_prev_s(ivisct, cproa_visct)
+    do iel = 1, ncel
+      cpro_visct(iel) = cproa_visct(iel)
+    enddo
+  endif
+
+  if (icp.gt.0) then
+    call field_get_key_int(icp, key_t_ext_id, icpext)
+    if (icpext.gt.0) then
+      call field_get_val_s(icp, cpro_cp)
+      call field_get_val_prev_s(icp, cproa_cp)
       do iel = 1, ncel
-        propce(iel,iicp  ) = propce(iel,iicpa )
+        cpro_cp(iel) = cproa_cp(iel)
       enddo
     endif
   endif
 
-!     Remarque : si on faisant cette operation pour tous les
-!       scalaires, on la ferait plusieurs fois pour les scalaires
-!       ayant une variance
+  ! Remarque : si on faisant cette operation pour tous les
+  ! scalaires, on la ferait plusieurs fois pour les scalaires
+  ! ayant une variance
   if (nscal.ge.1) then
     do iscal = 1, nscal
-      if (ivisls(iscal).gt.0.and.iscavr(iscal).le.0) then
-        if (ivsext(iscal).gt.0) then
-          ipcvsl = ipproc(ivisls(iscal))
-          ipcvsa = ipproc(ivissa(iscal))
+      ! Diffusivity
+      call field_get_key_int (ivarfl(isca(iscal)), kivisl, f_id)
+      if (f_id.ge.0.and.iscavr(iscal).le.0) then
+        call field_get_key_int(f_id, key_t_ext_id, iviext)
+        if (iviext.gt.0) then
+          call field_get_val_s(f_id, cpro_visls)
+          call field_get_val_prev_s(f_id, cproa_visls)
           do iel = 1, ncel
-            propce(iel,ipcvsl) = propce(iel,ipcvsa)
+            cpro_visls(iel) = cproa_visls(iel)
           enddo
         endif
       endif
