@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -25,9 +25,8 @@ subroutine stdtcl &
 
  ( nbzfmx , nozfmx ,                                              &
    iqimp  , icalke , qimp   , dh     , xintur ,                   &
-   itypfb , iznfbr , ilzfbr ,                                     &
-   propce ,                                                       &
-   rcodcl , qcalc  )
+   itypfb , iznfbr ,                                              &
+   rcodcl )
 
 !===============================================================================
 ! FONCTION :
@@ -48,10 +47,8 @@ subroutine stdtcl &
 ! itypfb           ! ia ! <-- ! boundary face types                            !
 ! iznfbr           ! te ! <-- ! numero de zone de la face de bord              !
 ! (nfabor)         !    !     !                                                !
-! ilzfbr(nbzfmx    ! te ! <-- ! tableau de travail                             !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 ! rcodcl           ! tr ! --> ! valeur des conditions aux limites              !
-!  (nfabor,nvarcl) !    !     !  aux faces de bord                             !
+!                  !    !     !  aux faces de bord                             !
 !                  !    !     ! rcodcl(1) = valeur du dirichlet                !
 !                  !    !     ! rcodcl(2) = valeur du coef. d'echange          !
 !                  !    !     !  ext. (infinie si pas d'echange)               !
@@ -61,14 +58,9 @@ subroutine stdtcl &
 !                  !    !     ! pour les vitesses (vistl+visct)*gradu          !
 !                  !    !     ! pour la pression             dt*gradp          !
 !                  !    !     ! pour les scalaires                             !
-!                  !    !     !        cp*(viscls+visct/sigmas)*gradt          !
-! qcalc(nozfmx)    ! tr ! --- ! tab de travail (debit par zone)                !
+!                  !    !     !        cp*(viscls+visct/turb_schmidt)*gradt    !
 !__________________!____!_____!________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
 !===============================================================================
 
 !===============================================================================
@@ -80,10 +72,13 @@ use numvar
 use optcal
 use cstphy
 use cstnum
+use dimens, only: nvar
 use entsor
 use parall
 use mesh
 use field
+use cs_c_bindings
+
 !===============================================================================
 
 implicit none
@@ -95,21 +90,22 @@ integer          nbzfmx
 
 integer          iqimp(nozfmx), icalke(nozfmx)
 integer          itypfb(nfabor)
-integer          iznfbr(nfabor), ilzfbr(nbzfmx)
+integer          iznfbr(nfabor)
 
 double precision qimp(nozfmx), dh(nozfmx), xintur(nozfmx)
-double precision propce(ncelet,*)
-double precision rcodcl(nfabor,nvarcl,3)
-double precision qcalc(nozfmx)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
 integer          ifac, izone, ifvu, izonem
 integer          nozapm, nzfppp
-integer          icke, ipcvis, ii, iel, iok
-double precision qisqc, viscla, d2s3, uref2, rhomoy, dhy, xiturb
-double precision ustar2, xkent, xeent
+integer          icke, ii, iel, iok
+double precision qisqc, viscla, uref2, rhomoy, dhy, xiturb
 double precision, dimension(:), pointer :: brom
+double precision, dimension(:), pointer :: viscl
+integer, allocatable, dimension(:) :: ilzfbr
+double precision, allocatable, dimension(:) :: qcalc
+
 integer          ipass
 data             ipass /0/
 save             ipass
@@ -120,11 +116,12 @@ save             ipass
 ! 1.  INITIALISATIONS
 !===============================================================================
 
+allocate(ilzfbr(nbzfmx))
+allocate(qcalc(nozfmx))
+
 
 call field_get_val_s(ibrom, brom)
-ipcvis = ipproc(iviscl)
-
-d2s3 = 2.d0/3.d0
+call field_get_val_s(iviscl, viscl)
 
 ! --> On construit une liste des numeros des zones frontieres.
 !           (liste locale au processeur, en parallele)
@@ -385,7 +382,8 @@ do ifac = 1, nfabor
 
   if (izone .gt. 0) then
 
-    if ( itypfb(ifac).eq.ientre ) then
+    if (     itypfb(ifac).eq.ientre  &
+        .or. itypfb(ifac).eq.i_convective_inlet) then
 
 ! ----  Traitement automatique de la turbulence
 
@@ -404,62 +402,18 @@ do ifac = 1, nfabor
         uref2 = max(uref2,epzero)
         rhomoy = brom(ifac)
         iel    = ifabor(ifac)
-        viscla = propce(iel,ipcvis)
+        viscla = viscl(iel)
         icke   = icalke(izone)
         dhy    = dh(izone)
         xiturb = xintur(izone)
-        ustar2 = 0.d0
-        xkent = epzero
-        xeent = epzero
         if (icke.eq.1) then
-          call keendb                                             &
-          !==========
-        ( uref2, dhy, rhomoy, viscla, cmu, xkappa,                &
-          ustar2, xkent, xeent )
+          call turbulence_bc_inlet_hyd_diam(ifac,                            &
+                                            uref2, dhy, rhomoy, viscla,      &
+                                            rcodcl)
         else if (icke.eq.2) then
-          call keenin                                             &
-          !==========
-        ( uref2, xiturb, dhy, cmu, xkappa, xkent, xeent )
-        endif
-
-        if (itytur.eq.2) then
-
-          rcodcl(ifac,ik,1)  = xkent
-          rcodcl(ifac,iep,1) = xeent
-
-        elseif (itytur.eq.3) then
-
-          rcodcl(ifac,ir11,1) = d2s3*xkent
-          rcodcl(ifac,ir22,1) = d2s3*xkent
-          rcodcl(ifac,ir33,1) = d2s3*xkent
-          rcodcl(ifac,ir12,1) = 0.d0
-          rcodcl(ifac,ir13,1) = 0.d0
-          rcodcl(ifac,ir23,1) = 0.d0
-          rcodcl(ifac,iep,1)  = xeent
-          if (iturb.eq.32) then
-            rcodcl(ifac,ial,1)  = 1.d0
-          endif
-
-        elseif (itytur.eq.5) then
-
-          rcodcl(ifac,ik,1)   = xkent
-          rcodcl(ifac,iep,1)  = xeent
-          rcodcl(ifac,iphi,1) = d2s3
-          if(iturb.eq.50) then
-            rcodcl(ifac,ifb,1)  = 0.d0
-          elseif(iturb.eq.51) then
-            rcodcl(ifac,ial,1)  = 0.d0
-          endif
-
-        elseif (iturb.eq.60) then
-
-          rcodcl(ifac,ik,1)   = xkent
-          rcodcl(ifac,iomg,1) = xeent/cmu/xkent
-
-        elseif (iturb.eq.70) then
-
-          rcodcl(ifac,inusa,1) = cmu*xkent**2/xeent
-
+          call turbulence_bc_inlet_turb_intensity(ifac,                      &
+                                                  uref2, xiturb, dhy,        &
+                                                  rcodcl)
         endif
 
       endif
@@ -477,12 +431,14 @@ do ifac = 1, nfabor
 
 enddo
 
+! Free memory
+deallocate(ilzfbr)
+deallocate(qcalc)
 
 !----
 ! FORMATS
 !----
 
-
 !----
 ! FIN
 !----
