@@ -697,7 +697,7 @@ endif
 !     CALCUL DU PAS DE TEMPS SI VARIABLE
 !===============================================================================
 
-if(vcopt_u%iwarni.ge.1) then
+if (vcopt_u%iwarni.ge.1) then
   write(nfecra,1020)
 endif
 
@@ -710,7 +710,7 @@ call dttvar &
 
 if (nbaste.gt.0.and.itrale.gt.nalinf) then
   ntrela = ntcabs - ntpabs
-  call astpdt(dt, ncelet, ntrela)
+  call astpdt(dt, ntrela)
 endif
 
 ! Compute the pseudo tensorial time step if needed for the pressure solving
@@ -826,8 +826,14 @@ else
   xprale => null()
 endif
 
+icodcl => null()
+rcodcl => null()
+
 300 continue
 
+hbord => null()
+theipb => null()
+visvdr => null()
 
 ! --- Boucle sur navstv pour couplage vitesse/pression
 !     on s'arrete a NTERUP ou quand on a converge
@@ -845,12 +851,6 @@ if (nterup.gt.1.or.isno2t.gt.0) then
   if (nbccou.gt.0 .or. nfpt1t.gt.0 .or. iirayo.gt.0) itrfup = 0
 endif
 
-icodcl => null()
-rcodcl => null()
-hbord => null()
-theipb => null()
-visvdr => null()
-
 ! Allocate temporary arrays for boundary conditions
 if (italim .eq. 1) then
   allocate(icodcl(nfabor,nvar))
