@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2012 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,22 +20,28 @@
 
 !-------------------------------------------------------------------------------
 
-! Module for turbomachinery capabilities
+!> \file turbomachinery.f90
+!> \brief Module for turbomachinery computations
 
 module turbomachinery
 
   !=============================================================================
+  !> \defgroup at_turbomachinery Module for turbomachinery computations
 
-  ! Type of turbomachinery computation :
-  !   none (0), frozen rotor (1), transient (2)
+  !> \addtogroup at_turbomachinery
+  !> \{
+
+  !> Type of turbomachinery computation:
+  !>   none (0), frozen rotor (1), transient (2)
 
   integer, save :: iturbo
 
-  ! Rotation axis
+  !> Type of rotor/stator interface:
+  !>   joint internal faces (0), coupled boundary faces (1)
 
-  double precision, save :: rotax(3)
+  integer, save :: ityint
 
-  ! Flag on cells related to the rotor
+  !> Rotor identifier list (1:ncel)
 
   integer, dimension(:), pointer :: irotce
 
@@ -47,20 +53,31 @@ module turbomachinery
 
   double precision, dimension(:), allocatable :: coftur, hfltur
 
+  !> \}
+
   !=============================================================================
 
   interface
 
     ! Interface to C function mapping some data for turbomachinery
 
-    subroutine map_turbomachinery_module (iturbo2, rotax2, irotce2) &
-      bind(C, name='cs_f_map_turbomachinery_module')
+    subroutine map_turbomachinery_model (iturbo2, ityint2) &
+      bind(C, name='cs_f_map_turbomachinery_model')
+      use, intrinsic :: iso_c_binding
+      implicit none
+      integer(c_int), intent(out) :: iturbo2, ityint2
+    end subroutine map_turbomachinery_model
+
+    !---------------------------------------------------------------------------
+
+    ! Interface to C function mapping some data for turbomachinery
+
+    subroutine map_turbomachinery_rotor (irotce2) &
+      bind(C, name='cs_f_map_turbomachinery_rotor')
       use, intrinsic :: iso_c_binding
       implicit none
-      integer(c_int), intent(out) :: iturbo2
-      real(c_double), dimension(1:3), intent(out) :: rotax2
       type(c_ptr), intent(out) :: irotce2
-    end subroutine map_turbomachinery_module
+    end subroutine map_turbomachinery_rotor
 
     !---------------------------------------------------------------------------
 
@@ -106,40 +123,19 @@ contains
 
     use, intrinsic :: iso_c_binding
     use mesh
-    use cstphy
-    use cplsat, only: imobil
 
     implicit none
 
     ! Local variables
 
-    integer iel
     type(c_ptr) :: c_p
 
     ! Map turbomachinery module components to global c turbomachinery structure
 
-    call map_turbomachinery_module(iturbo, rotax, c_p)
+    call map_turbomachinery_rotor(c_p)
 
     call c_f_pointer(c_p, irotce, [ncelet])
 
-    ! In case of relative frame computation or turbomachinery computation
-    ! with code/code coupling, the data management is merged with the one
-    ! of turbomachinery module
-
-    if (iturbo.eq.0) then
-      if (icorio.eq.1.or.imobil.eq.1) then
-
-        rotax(1) = omegax
-        rotax(2) = omegay
-        rotax(3) = omegaz
-
-        allocate(irotce(ncelet))
-        do iel = 1, ncelet
-          irotce(iel) = 1
-        enddo
-      endif
-    endif
-
     ! Initialization of elapsed times
 
     rs_ell(1) = 0.d0
@@ -167,10 +163,8 @@ contains
     ! Local variables
 
     type(c_ptr) :: c_p
-    integer(c_int) :: iturbo2
-    real(c_double), dimension(1:3) :: rotax2
 
-    call map_turbomachinery_module(iturbo2, rotax2, c_p)
+    call map_turbomachinery_rotor(c_p)
 
     call turbomachinery_resize_cell_fields
 
@@ -186,13 +180,6 @@ contains
 
   subroutine turbomachinery_finalize
 
-    use cstphy, only: icorio
-    use cplsat, only: imobil
-
-    if (iturbo.eq.0) then
-      if (icorio.eq.1.or.imobil.eq.1)  deallocate(irotce)
-    endif
-
     if (iturbo.eq.2)  deallocate(coftur, hfltur)
 
     return
