@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -46,16 +46,18 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
+!>                               - 13 Dirichlet for the advection operator and
+!>                                    Neumann for the diffusion operator
 !> \param[in,out] rcodcl        boundary condition values:
 !>                               - rcodcl(1) value of the dirichlet
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradv \vect{u} \cdot \vect{n}  \f$
@@ -90,6 +92,7 @@ use ppincl
 use parall
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -100,16 +103,17 @@ implicit none
 integer          nvar   , nscal
 
 integer          itypfb(nfabor)
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
+character(len=80) :: chaine
+
 double precision grav2
-character*80     chaine
-integer          ifac, ivar, icode
-integer          nstoni        , nstvit, nstopp
+integer          ifac, ivar, icode, f_dim, f_type
+integer          nstoni, nstvit, nstopp
 integer          nstoke, nstosc, nstovf
 integer          nstuvw, nstoup, nstuke
 integer          nstrij, nsurij, nstov2
@@ -196,28 +200,40 @@ nstonu = 0
 ! --- Premiere boucle rapide
 iokcod = 0
 do ivar = 1, nvar
-  do ifac = 1, nfabor
-    icode = icodcl(ifac,ivar)
-    if(icode.eq. 0) then
-      iokcod = 1
-    endif
-  enddo
+  call field_get_type(ivarfl(ivar), f_type)
+  ! Is this field not managed by CDO?
+  if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
+
+    do ifac = 1, nfabor
+      icode = icodcl(ifac,ivar)
+      if(icode.eq. 0) then
+        iokcod = 1
+      endif
+    enddo
+
+  endif ! CDO ?
 enddo
 
 ! --- Seconde boucle lente si pb plus haut
 if (iokcod.ne.0) then
   do ivar = 1, nvar
-    do ifac = 1, nfabor
-      icode = icodcl(ifac,ivar)
-      if(icode.eq. 0) then
-        if (itypfb(ifac).gt.0) then
-          itypfb(ifac) = -itypfb(ifac)
+    call field_get_type(ivarfl(ivar), f_type)
+    ! Is this field not managed by CDO?
+    if (iand(f_type, FIELD_CDO)/=FIELD_CDO) then
+
+      do ifac = 1, nfabor
+        icode = icodcl(ifac,ivar)
+        if(icode.eq. 0) then
+          if (itypfb(ifac).gt.0) then
+            itypfb(ifac) = -itypfb(ifac)
+          endif
+          icodni(1) = ivar
+          icodni(2) = ifac
+          nstoni = nstoni + 1
         endif
-        icodni(1) = ivar
-        icodni(2) = ifac
-        nstoni = nstoni + 1
-      endif
-    enddo
+      enddo
+
+    endif ! CDO ?
   enddo
 endif
 
@@ -234,7 +250,8 @@ do ifac = 1, nfabor
 
   if(icodcu.ne. 1.and.icodcu.ne. 2.and.icodcu.ne. 3.and.        &
      icodcu.ne. 4.and.icodcu.ne. 5.and.icodcu.ne. 6.and.        &
-     icodcu.ne. 9.and.icodcu.ne.14) then
+     icodcu.ne.11.and.                                          &
+     icodcu.ne. 9.and.icodcu.ne.13.and.icodcu.ne.14) then
     if (itypfb(ifac).gt.0) then
       itypfb(ifac) = -itypfb(ifac)
     endif
@@ -245,7 +262,8 @@ do ifac = 1, nfabor
   endif
   if(icodcv.ne. 1.and.icodcv.ne. 2.and.icodcv.ne. 3.and.        &
      icodcv.ne. 4.and.icodcv.ne. 5.and.icodcv.ne. 6.and.        &
-     icodcv.ne. 9.and.icodcv.ne.14) then
+     icodcv.ne.11.and.                                          &
+     icodcv.ne. 9.and.icodcv.ne.13.and.icodcv.ne.14) then
     if (itypfb(ifac).gt.0) then
       itypfb(ifac) = -itypfb(ifac)
     endif
@@ -255,8 +273,9 @@ do ifac = 1, nfabor
     nstvit = nstvit + 1
   endif
   if(icodcw.ne. 1.and.icodcw.ne. 2.and.icodcw.ne. 3.and.        &
-     icodcw.ne. 4.and.icodcw.ne. 5.and.icodcv.ne. 6.and.        &
-     icodcw.ne. 9.and.icodcw.ne.14) then
+     icodcw.ne. 4.and.icodcw.ne. 5.and.icodcw.ne. 6.and.        &
+     icodcw.ne.11.and.                                          &
+     icodcw.ne. 9.and.icodcw.ne.13.and.icodcw.ne.14) then
     if (itypfb(ifac).gt.0) then
       itypfb(ifac) = -itypfb(ifac)
     endif
@@ -277,21 +296,15 @@ do ifac = 1, nfabor
     nstvit = nstvit + 1
   endif
 
-  ! --- on interdit les parois rugueuses en compressible
-  if (icodcu.eq.6 .and. ippmod(icompf).gt.0) then
-    icodvi(1) = iu
-    icodvi(2) = icodcl(ifac,iu)
-    icodvi(3) = 1
-    nstvit = nstvit + 1
-  endif
-
 enddo
 
 ! --- Conditions admissibles pour la pression
 do ifac = 1, nfabor
 
   if (icodcl(ifac,ipr).ne.1 .and. icodcl(ifac,ipr).ne.2 .and.     &
-      icodcl(ifac,ipr).ne.3 .and. icodcl(ifac,ipr).ne.13) then
+      icodcl(ifac,ipr).ne.3 .and. icodcl(ifac,ipr).ne.11.and.     &
+      icodcl(ifac,ipr).ne.12.and. icodcl(ifac,ipr).ne.13.and.     &
+      icodcl(ifac,ipr).ne.15) then
     if (itypfb(ifac).gt.0) then
       itypfb(ifac) = -itypfb(ifac)
     endif
@@ -311,11 +324,13 @@ if (itytur.eq.2) then
         icodcl(ifac,ik ).ne. 2.and.                          &
         icodcl(ifac,ik ).ne. 3.and.                          &
         icodcl(ifac,ik ).ne. 5.and.                          &
+        icodcl(ifac,ik ).ne.13.and.                          &
         icodcl(ifac,ik ).ne. 6     ).or.                     &
        (icodcl(ifac,iep).ne. 1.and.                          &
         icodcl(ifac,iep).ne. 2.and.                          &
         icodcl(ifac,iep).ne. 3.and.                          &
         icodcl(ifac,iep).ne. 5.and.                          &
+        icodcl(ifac,iep).ne.13.and.                          &
         icodcl(ifac,iep).ne. 6     ) )then
       if (itypfb(ifac).gt.0) then
         itypfb(ifac) = -itypfb(ifac)
@@ -336,7 +351,7 @@ elseif(itytur.eq.3) then
   do ifac = 1, nfabor
     icode = icodcl(ifac,ivar)
     if (icode.ne. 1.and.icode.ne. 2.and. icode.ne. 3.and.         &
-        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6     ) then
+        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6 .and.icode.ne.13) then
       if (itypfb(ifac).gt.0) then
         itypfb(ifac) = -itypfb(ifac)
       endif
@@ -350,7 +365,7 @@ elseif(itytur.eq.3) then
   do ifac = 1, nfabor
     icode = icodcl(ifac,ivar)
     if (icode.ne. 1.and.icode.ne. 2.and. icode.ne. 3.and.         &
-        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6     ) then
+        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6 .and.icode.ne.13) then
       if (itypfb(ifac).gt.0) then
         itypfb(ifac) = -itypfb(ifac)
       endif
@@ -364,7 +379,7 @@ elseif(itytur.eq.3) then
   do ifac = 1, nfabor
     icode = icodcl(ifac,ivar)
     if (icode.ne. 1.and.icode.ne. 2.and. icode.ne. 3.and.         &
-        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6     ) then
+        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6 .and.icode.ne.13) then
       if (itypfb(ifac).gt.0) then
         itypfb(ifac) = -itypfb(ifac)
       endif
@@ -378,21 +393,7 @@ elseif(itytur.eq.3) then
   do ifac = 1, nfabor
     icode = icodcl(ifac,ivar)
     if (icode.ne. 1.and.icode.ne. 2.and. icode.ne. 3.and.         &
-        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6     ) then
-      if (itypfb(ifac).gt.0) then
-        itypfb(ifac) = -itypfb(ifac)
-      endif
-      icodtb(1) = ivar
-      icodtb(2) = icode
-      nstrij = nstrij + 1
-    endif
-  enddo
-
-  ivar = ir13
-  do ifac = 1, nfabor
-    icode = icodcl(ifac,ivar)
-    if (icode.ne. 1.and.icode.ne. 2.and. icode.ne. 3.and.         &
-        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6     ) then
+        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6 .and.icode.ne.13) then
       if (itypfb(ifac).gt.0) then
         itypfb(ifac) = -itypfb(ifac)
       endif
@@ -406,7 +407,21 @@ elseif(itytur.eq.3) then
   do ifac = 1, nfabor
     icode = icodcl(ifac,ivar)
     if (icode.ne. 1.and.icode.ne. 2.and. icode.ne. 3.and.         &
-        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6     ) then
+        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6 .and.icode.ne.13) then
+      if (itypfb(ifac).gt.0) then
+        itypfb(ifac) = -itypfb(ifac)
+      endif
+      icodtb(1) = ivar
+      icodtb(2) = icode
+      nstrij = nstrij + 1
+    endif
+  enddo
+
+  ivar = ir13
+  do ifac = 1, nfabor
+    icode = icodcl(ifac,ivar)
+    if (icode.ne. 1.and.icode.ne. 2.and. icode.ne. 3.and.         &
+        icode.ne. 4.and.icode.ne. 5.and.icode.ne. 6 .and.icode.ne.13) then
       if (itypfb(ifac).gt.0) then
         itypfb(ifac) = -itypfb(ifac)
       endif
@@ -419,7 +434,7 @@ elseif(itytur.eq.3) then
   do ifac = 1, nfabor
     icode = icodcl(ifac,iep)
     if (icode.ne. 1.and.icode.ne. 2.and. icode.ne. 3.and.         &
-        icode.ne. 5.and.icode.ne.6) then
+        icode.ne. 5.and.icode.ne. 6 .and.icode.ne.13) then
       if (itypfb(ifac).gt.0) then
         itypfb(ifac) = -itypfb(ifac)
       endif
@@ -434,7 +449,7 @@ elseif(itytur.eq.3) then
     do ifac = 1, nfabor
       icode = icodcl(ifac,ial)
       if (icode.ne. 1.and.icode.ne. 2.and. icode.ne. 3.and.         &
-          icode.ne. 5.and.icode.ne.6) then
+          icode.ne. 5.and.icode.ne. 6 .and.icode.ne.13) then
         if (itypfb(ifac).gt.0) then
           itypfb(ifac) = -itypfb(ifac)
         endif
@@ -443,7 +458,7 @@ elseif(itytur.eq.3) then
         icodtb(3) = -1
         nstrij = nstrij + 1
       endif
-      ! No rought wall with EBRSM
+      ! No rough wall with EBRSM
       do ivar = 1, nvar
         if (icodcl(ifac,ivar).eq.6) then
           if (itypfb(ifac).gt.0) then
@@ -467,6 +482,7 @@ elseif (iturb.eq.50) then
         icodcl(ifac,ik ).ne. 2.and.                          &
         icodcl(ifac,ik ).ne. 3.and.                          &
         icodcl(ifac,ik ).ne. 5.and.                          &
+        icodcl(ifac,ik ).ne.13.and.                          &
         icodcl(ifac,ik ).ne. 6     ).or.                     &
        (icodcl(ifac,iep).ne. 1.and.                          &
         icodcl(ifac,iep).ne. 2.and.                          &
@@ -477,11 +493,13 @@ elseif (iturb.eq.50) then
         icodcl(ifac,iphi).ne. 2.and.                         &
         icodcl(ifac,iphi).ne. 3.and.                         &
         icodcl(ifac,iphi).ne. 5.and.                         &
+        icodcl(ifac,iphi).ne.13.and.                         &
         icodcl(ifac,iphi).ne. 6     ).or.                    &
        (icodcl(ifac,ifb).ne. 1.and.                          &
         icodcl(ifac,ifb).ne. 2.and.                          &
         icodcl(ifac,ifb).ne. 3.and.                          &
         icodcl(ifac,ifb).ne. 5.and.                          &
+        icodcl(ifac,ifb).ne.13.and.                          &
         icodcl(ifac,ifb).ne. 6     ) )then
 
       if (itypfb(ifac).gt.0) then
@@ -510,21 +528,25 @@ elseif (iturb.eq.51) then
         icodcl(ifac,ik ).ne. 2.and.                          &
         icodcl(ifac,ik ).ne. 3.and.                          &
         icodcl(ifac,ik ).ne. 5.and.                          &
+        icodcl(ifac,ik ).ne.13.and.                          &
         icodcl(ifac,ik ).ne. 6     ).or.                     &
        (icodcl(ifac,iep).ne. 1.and.                          &
         icodcl(ifac,iep).ne. 2.and.                          &
         icodcl(ifac,iep).ne. 3.and.                          &
         icodcl(ifac,iep).ne. 5.and.                          &
+        icodcl(ifac,iep).ne.13.and.                          &
         icodcl(ifac,iep).ne. 6     ).or.                     &
        (icodcl(ifac,iphi).ne. 1.and.                         &
         icodcl(ifac,iphi).ne. 2.and.                         &
         icodcl(ifac,iphi).ne. 3.and.                         &
         icodcl(ifac,iphi).ne. 5.and.                         &
+        icodcl(ifac,iphi).ne.13.and.                         &
         icodcl(ifac,iphi).ne. 6     ).or.                    &
        (icodcl(ifac,ial).ne. 1.and.                          &
         icodcl(ifac,ial).ne. 2.and.                          &
         icodcl(ifac,ial).ne. 3.and.                          &
         icodcl(ifac,ial).ne. 5.and.                          &
+        icodcl(ifac,ial).ne.13.and.                          &
         icodcl(ifac,ial).ne. 6     ) )then
 
       if (itypfb(ifac).gt.0) then
@@ -553,11 +575,13 @@ elseif (iturb.eq.60) then
         icodcl(ifac,ik ).ne. 2.and.                          &
         icodcl(ifac,ik ).ne. 3.and.                          &
         icodcl(ifac,ik ).ne. 5.and.                          &
+        icodcl(ifac,ik ).ne.13.and.                          &
         icodcl(ifac,ik ).ne. 6     ).or.                     &
        (icodcl(ifac,iomg).ne. 1.and.                         &
         icodcl(ifac,iomg).ne. 2.and.                         &
         icodcl(ifac,iomg).ne. 3.and.                         &
         icodcl(ifac,iomg).ne. 5.and.                         &
+        icodcl(ifac,iomg).ne.13.and.                         &
         icodcl(ifac,iomg).ne. 6     ) )then
 
       if (itypfb(ifac).gt.0) then
@@ -582,6 +606,7 @@ elseif (iturb.eq.70) then
        icodcl(ifac,inusa ).ne. 2.and.                          &
        icodcl(ifac,inusa ).ne. 3.and.                          &
        icodcl(ifac,inusa ).ne. 5.and.                          &
+       icodcl(ifac,inusa ).ne.13.and.                          &
        icodcl(ifac,inusa ).ne. 6           )then
 
       if (itypfb(ifac).gt.0) then
@@ -601,65 +626,100 @@ endif
 if(nscal.ge.1) then
   do iis = 1,nscal
     ivar = isca(iis)
-    do ifac = 1, nfabor
-      if(icodcl(ifac,ivar).ne. 1.and.                             &
-         icodcl(ifac,ivar).ne. 3.and.                             &
-         icodcl(ifac,ivar).ne. 5.and.                             &
-         icodcl(ifac,ivar).ne. 6 ) then
-        if (itypfb(ifac).gt.0) then
-          itypfb(ifac) = -itypfb(ifac)
+    call field_get_dim(ivarfl(ivar), f_dim)
+    if (f_dim.eq.1) then
+      do ifac = 1, nfabor
+        if(icodcl(ifac,ivar).ne. 1.and.                             &
+           icodcl(ifac,ivar).ne. 2.and.                             &
+           icodcl(ifac,ivar).ne. 3.and.                             &
+           icodcl(ifac,ivar).ne. 5.and.                             &
+           icodcl(ifac,ivar).ne.12.and.                             &
+           icodcl(ifac,ivar).ne.13.and.                             &
+           icodcl(ifac,ivar).ne. 6 ) then
+          if (itypfb(ifac).gt.0) then
+            itypfb(ifac) = -itypfb(ifac)
+          endif
+          icodsc(1) = ivar
+          icodsc(2) = icodcl(ifac,ivar)
+          nstosc = nstosc + 1
         endif
-        icodsc(1) = ivar
-        icodsc(2) = icodcl(ifac,ivar)
-        nstosc = nstosc + 1
-      endif
-      if(icodcl(ifac,ivar).eq. 5.and.                             &
-         iscavr(iis).gt.0        ) then
-        if (itypfb(ifac).gt.0) then
-          itypfb(ifac) = -itypfb(ifac)
+        if(icodcl(ifac,ivar).eq. 5.and.                             &
+           iscavr(iis).gt.0        ) then
+          if (itypfb(ifac).gt.0) then
+            itypfb(ifac) = -itypfb(ifac)
+          endif
+          icodvf(1) = ivar
+          icodvf(2) = icodcl(ifac,ivar)
+          nstovf = nstovf + 1
         endif
-        icodvf(1) = ivar
-        icodvf(2) = icodcl(ifac,ivar)
-        nstovf = nstovf + 1
-      endif
-      if(icodcl(ifac,ivar).eq. 6.and.                             &
-         iscavr(iis).gt.0        ) then
-        if (itypfb(ifac).gt.0) then
-          itypfb(ifac) = -itypfb(ifac)
+        if(icodcl(ifac,ivar).eq. 6.and.                             &
+           iscavr(iis).gt.0        ) then
+          if (itypfb(ifac).gt.0) then
+            itypfb(ifac) = -itypfb(ifac)
+          endif
+          icodvf(1) = ivar
+          icodvf(2) = icodcl(ifac,ivar)
+          nstovf = nstovf + 1
         endif
-        icodvf(1) = ivar
-        icodvf(2) = icodcl(ifac,ivar)
-        nstovf = nstovf + 1
-      endif
-! --- verification que la rugosite scalaire est initialisee si icodl=6
-      if(icodcl(ifac,ivar).eq.6.and.                              &
-         rcodcl(ifac,iv,3).lt.epzero)then
-        if (itypfb(ifac).gt.0) then
-          itypfb(ifac) = -itypfb(ifac)
+  ! --- verification que la rugosite scalaire est initialisee si icodl=6
+        if(icodcl(ifac,ivar).eq.6.and.                              &
+           rcodcl(ifac,iv,3).lt.epzero)then
+          if (itypfb(ifac).gt.0) then
+            itypfb(ifac) = -itypfb(ifac)
+          endif
+          icodsc(1) = -6
+          icodsc(2) = icodcl(ifac,ivar)
+          nstosc = nstosc + 1
         endif
-        icodsc(1) = -6
-        icodsc(2) = icodcl(ifac,ivar)
-        nstosc = nstosc + 1
-      endif
-    enddo
+      enddo
+    else ! additional vector variables
+      do ifac = 1, nfabor
+        if(icodcl(ifac,ivar).ne. 1.and.                             &
+           icodcl(ifac,ivar).ne. 2.and.                             &
+           icodcl(ifac,ivar).ne. 3.and.                             &
+           icodcl(ifac,ivar).ne. 4.and.                             &
+           icodcl(ifac,ivar).ne. 5.and.                             &
+           icodcl(ifac,ivar).ne.11.and.                             &
+           icodcl(ifac,ivar).ne.13.and.                             &
+           icodcl(ifac,ivar).ne.14.and.                             &
+           icodcl(ifac,ivar).ne. 6 ) then
+          if (itypfb(ifac).gt.0) then
+            itypfb(ifac) = -itypfb(ifac)
+          endif
+          icodsc(1) = ivar
+          icodsc(2) = icodcl(ifac,ivar)
+          nstosc = nstosc + 1
+        endif
+  ! --- verification que la rugosite scalaire est initialisee si icodl=6
+        if(icodcl(ifac,ivar).eq.6.and.                              &
+           rcodcl(ifac,iv,3).lt.epzero)then
+          if (itypfb(ifac).gt.0) then
+            itypfb(ifac) = -itypfb(ifac)
+          endif
+          icodsc(1) = -6
+          icodsc(2) = icodcl(ifac,ivar)
+          nstosc = nstosc + 1
+        endif
+      enddo
+    endif ! dim
   enddo
 endif
 
 ! --- Check if the gravity is non zero in case of free-surface
 iok = 0
-if (iale.eq.1) then
+if (iale.ge.1) then
+  grav2 = gx**2 + gy**2 + gz**2
   do ifac = 1, nfabor
-    grav2 = gx**2 + gy**2 + gz**2
-    if (ialtyb(ifac).eq.ifresf) then
-      if (grav2.le.epzero**2) then
-        write(nfecra,2001)
-        iok = 1
-      endif
+    if (ialtyb(ifac).eq.ifresf.and.grav2.le.epzero**2) then
+      iok = 1
     endif
   enddo
 endif
 
+if (irangp.ge.0) call parcmx(iok)
+
 if (iok.ne.0) then
+  write(nfecra,2001)
   call csexit (1)
   !==========
 endif
@@ -1210,6 +1270,7 @@ if (nstvit.gt.0 .or. nstopp.gt.0 .or. nstoke.gt.0 .or. nstrij.gt.0 .or.        &
 endif
 
 if (irangp.ge.0) call parcmx(iok)
+
 if(iok.ne.0) then
 
   call sync_bc_err(nstoni, 2, icodni)
@@ -1227,8 +1288,6 @@ if(iok.ne.0) then
     endif
     if (icodvi(3).eq.-1) then
       write(nfecra,1010) nstvit, chaine, icodvi(2)
-    elseif (icodvi(3).eq.1) then
-      write(nfecra,1015) nstvit, chaine, icodvi(2), ippmod(icompf)
     endif
   endif
 
@@ -1391,15 +1450,16 @@ if(iok.ne.0) then
     write (nfecra,1901) nstoni, nstosc, nstovf, nstusc
   endif
 
-if (nstvit.gt.0 .or. nstopp.gt.0 .or. nstoke.gt.0 .or. nstrij.gt.0 .or.        &
-    nstov2.gt.0 .or. nstonu.gt.0 .or. nstuvw.gt.0 .or. nstoup.gt.0 .or.        &
-    nstuke.gt.0 .or. nsurij.gt.0 .or. nstuv2.gt.0 .or. nstunu.gt.0     ) then
-  write (nfecra,1902)  nstvit,nstopp, nstoke,nstrij,             &
-                       nstov2,nstonu, nstuvw,nstoup,             &
-                       nstuke,nsurij, nstuv2,nstunu
-endif
-  call bcderr(itypfb)
-  !==========
+  if (nstvit.gt.0 .or. nstopp.gt.0 .or. nstoke.gt.0 .or. nstrij.gt.0 .or.    &
+      nstov2.gt.0 .or. nstonu.gt.0 .or. nstuvw.gt.0 .or. nstoup.gt.0 .or.    &
+      nstuke.gt.0 .or. nsurij.gt.0 .or. nstuv2.gt.0 .or. nstunu.gt.0     ) then
+    write (nfecra,1902)  nstvit,nstopp, nstoke,nstrij,                       &
+                         nstov2,nstonu, nstuvw,nstoup,                       &
+                         nstuke,nsurij, nstuv2,nstunu
+  endif
+
+  call boundary_conditions_error(itypfb)
+
 endif
 
 !===============================================================================
@@ -1419,14 +1479,6 @@ endif
 '@ COND. LIM. NON PREVUES                                     ',/,&
 '@   Nombre de faces de bord ',i10   ,'; variable ',a16        ,/,&
 '@     icodcl variable derniere face ', i10                    ,/,&
-'@                                                            '  )
- 1015 format(                                                     &
-'@                                                            ',/,&
-'@ CONDITIONS AUX LIMITES DE PAROI RUGUEUSE INCOMPATIBLES     ',/,&
-'@ AVEC LE MODULE COMPRESSIBLE                                ',/,&
-'@   Nombre de faces de bord ',i10   ,'; variable ',a16        ,/,&
-'@     icodcl variable derniere face ', i10                    ,/,&
-'@     ippmod(icompf)', i10                                    ,/,&
 '@                                                            '  )
  1020 format(                                                     &
 '@                                                            ',/,&
@@ -1564,14 +1616,6 @@ endif
 '@ UNEXPECTED BOUNDARY CONDITIONS                             ',/,&
 '@   Number of boundary faces ',i10   ,'; variable ',a16       ,/,&
 '@     icodcl variable last face ', i10                        ,/,&
-'@                                                            '  )
- 1015 format(                                                     &
-'@                                                            ',/,&
-'@ ROUGH WALL BOUNDARY CONDITIONS INCOMPATIBLE WITH THE       ',/,&
-'@ COMPRESSIBLE MODULE                                        ',/,&
-'@   Number of boundary faces ',i10   ,'; variable ',a16       ,/,&
-'@     icodcl variable last face ', i10                        ,/,&
-'@     ippmod(icompf)', i10                                    ,/,&
 '@                                                            '  )
  1020 format(                                                     &
 '@                                                            ',/,&
@@ -1639,7 +1683,7 @@ endif
 '@ @@ WARNING: ABORT DURING THE BOUNDARY CONDITIONS CHECK     ',/,&
 '@    ========                                                ',/,&
 '@                                                            ',/,&
-'@         Unexpeted boundary conditions:                     ',/,&
+'@         Unexpected boundary conditions:                    ',/,&
 '@             on the velocity                      : ',I10    ,/,&
 '@             on the pressure                      : ',I10    ,/,&
 '@             on k and epsilon                     : ',I10    ,/,&
@@ -1745,7 +1789,7 @@ if (irangp.ge.0) then
   if (nerloc.gt.0) irkerr = irangp
   call parcpt(nerloc)
   if (nerloc .ne. 0) then
-    call parimx(1, irkerr)
+    call parcmx(irkerr)
     call parbci(irkerr, nerrcd, errcod)
   endif
 endif
