@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -61,24 +61,26 @@
 
 BEGIN_C_DECLS
 
-/*-------------------------------------------------------------------------------
- * Local type definitions
- *-----------------------------------------------------------------------------*/
+/*=============================================================================
+ * Additional doxygen documentation
+ *============================================================================*/
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+/*!
+  \file bft_mem.c
+        Base memory allocation wrappers with optional tracing.
 
-/*
- * Structure defining an allocated memory block (for memory tracing)
- */
+  The memory managment function provided here provide optional logging,
+  and tracking of non-freed pointers.
 
-struct _bft_mem_block_t {
+  Since in most of the intended applications, failure to allocate memory
+  is considered fatal, failed allocations from these functions are
+  considedered as errors, which are fatal by default but can be handled
+  differently if an appropriate error handler is defined. So additional
+  checking of the return values in the calling code is not needed.
 
-  void    *p_bloc;  /* Allocated memory block start adress */
-  size_t   size;    /* Allocated memory block length */
-
-};
-
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
+  The functions provided here are otherwise based on the matching C library
+  functions.
+*/
 
 /*-------------------------------------------------------------------------------
  * Local macro documentation
@@ -129,6 +131,8 @@ struct _bft_mem_block_t {
  * \param [in]  _type  element type.
  */
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*-------------------------------------------------------------------------------
  * Local macro definitions
  *-----------------------------------------------------------------------------*/
@@ -139,6 +143,21 @@ struct _bft_mem_block_t {
 
 #define DIR_SEPARATOR '/'
 
+/*-------------------------------------------------------------------------------
+ * Local type definitions
+ *-----------------------------------------------------------------------------*/
+
+/*
+ * Structure defining an allocated memory block (for memory tracing)
+ */
+
+struct _bft_mem_block_t {
+
+  void    *p_bloc;  /* Allocated memory block start adress */
+  size_t   size;    /* Allocated memory block length */
+
+};
+
 /*-----------------------------------------------------------------------------
  * Local function prototypes
  *-----------------------------------------------------------------------------*/
@@ -188,6 +207,10 @@ static size_t  _bft_mem_global_n_frees = 0;
 static bft_error_handler_t  *_bft_mem_error_handler
                               = (_bft_mem_error_handler_default);
 
+#if defined(HAVE_OPENMP)
+static omp_lock_t _bft_mem_lock;
+#endif
+
 /*-----------------------------------------------------------------------------
  * Local function definitions
  *-----------------------------------------------------------------------------*/
@@ -545,6 +568,8 @@ _bft_mem_block_free(const void *p_free)
   }
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -566,6 +591,14 @@ bft_mem_init(const char *log_file_name)
 {
   size_t alloc_size;
 
+#if defined(HAVE_OPENMP)
+  if (omp_in_parallel()) {
+    if (omp_get_thread_num() != 0)
+      return;
+  }
+  omp_init_lock(&_bft_mem_lock);
+#endif
+
   if (_bft_mem_global_initialized == 1) {
     _bft_mem_error(__FILE__, __LINE__, 0,
                    _("bft_mem_init() has already been called"));
@@ -633,10 +666,17 @@ bft_mem_init(const char *log_file_name)
 
 void bft_mem_end(void)
 {
-  if (_bft_mem_global_initialized == 0) {
-    _bft_mem_error(__FILE__, __LINE__, 0,
-                   _("bft_mem_end() called before bft_mem_init()"));
+  if (_bft_mem_global_initialized == 0)
+    return;
+
+#if defined(HAVE_OPENMP)
+  if (omp_in_parallel()) {
+    if (omp_get_thread_num() != 0)
+      return;
   }
+  omp_destroy_lock(&_bft_mem_lock);
+#endif
+
   _bft_mem_global_initialized = 0;
 
   if (_bft_mem_global_file != NULL) {
@@ -747,26 +787,39 @@ bft_mem_malloc(size_t       ni,
 
   /* Memory allocation counting */
 
-  _bft_mem_global_alloc_cur += alloc_size;
+  {
+#if defined(HAVE_OPENMP)
+    int in_parallel = omp_in_parallel();
+    if (in_parallel)
+      omp_set_lock(&_bft_mem_lock);
+#endif
 
-  if (_bft_mem_global_alloc_max < _bft_mem_global_alloc_cur)
-    _bft_mem_global_alloc_max = _bft_mem_global_alloc_cur;
+    _bft_mem_global_alloc_cur += alloc_size;
 
-  if (_bft_mem_global_file != NULL) {
-    fprintf(_bft_mem_global_file, "\n  alloc: %-27s:%6d : %-39s: %9lu",
-            _bft_mem_basename(file_name), line_num,
-            var_name, (unsigned long)alloc_size);
-    fprintf(_bft_mem_global_file, " : (+%9lu) : %12lu : [%10p]",
-            (unsigned long)alloc_size,
-            (unsigned long)_bft_mem_global_alloc_cur,
-            p_loc);
-    fflush(_bft_mem_global_file);
+    if (_bft_mem_global_alloc_max < _bft_mem_global_alloc_cur)
+      _bft_mem_global_alloc_max = _bft_mem_global_alloc_cur;
+
+    if (_bft_mem_global_file != NULL) {
+      fprintf(_bft_mem_global_file, "\n  alloc: %-27s:%6d : %-39s: %9lu",
+              _bft_mem_basename(file_name), line_num,
+              var_name, (unsigned long)alloc_size);
+      fprintf(_bft_mem_global_file, " : (+%9lu) : %12lu : [%10p]",
+              (unsigned long)alloc_size,
+              (unsigned long)_bft_mem_global_alloc_cur,
+              p_loc);
+      fflush(_bft_mem_global_file);
+    }
+
+    _bft_mem_block_malloc(p_loc, alloc_size);
+
+    _bft_mem_global_n_allocs += 1;
+
+#if defined(HAVE_OPENMP)
+    if (in_parallel)
+      omp_unset_lock(&_bft_mem_lock);
+#endif
   }
 
-  _bft_mem_block_malloc(p_loc, alloc_size);
-
-  _bft_mem_global_n_allocs += 1;
-
   /* Return pointer to allocated memory */
 
   return p_loc;
@@ -819,8 +872,19 @@ bft_mem_realloc(void        *ptr,
 
   /* If the old size equals the new size, nothing needs to be done. */
 
+#if defined(HAVE_OPENMP)
+  int in_parallel = omp_in_parallel();
+  if (in_parallel)
+    omp_set_lock(&_bft_mem_lock);
+#endif
+
   old_size = _bft_mem_block_size(ptr);
 
+#if defined(HAVE_OPENMP)
+  if (in_parallel)
+    omp_unset_lock(&_bft_mem_lock);
+#endif
+
   if (new_size == old_size)
     return ptr;
 
@@ -852,30 +916,42 @@ bft_mem_realloc(void        *ptr,
     else if (_bft_mem_global_initialized == 0)
       return p_loc;
 
-    _bft_mem_global_alloc_cur += size_diff;
+    {
+#if defined(HAVE_OPENMP)
+      if (in_parallel)
+        omp_set_lock(&_bft_mem_lock);
+#endif
 
-    if (size_diff > 0) {
-      if (_bft_mem_global_alloc_max < _bft_mem_global_alloc_cur)
-        _bft_mem_global_alloc_max = _bft_mem_global_alloc_cur;
+      _bft_mem_global_alloc_cur += size_diff;
+
+      if (size_diff > 0) {
+        if (_bft_mem_global_alloc_max < _bft_mem_global_alloc_cur)
+          _bft_mem_global_alloc_max = _bft_mem_global_alloc_cur;
+      }
+
+      if (_bft_mem_global_file != NULL) {
+        char sgn = (size_diff > 0) ? '+' : '-';
+        fprintf(_bft_mem_global_file, "\nrealloc: %-27s:%6d : %-39s: %9lu",
+                _bft_mem_basename(file_name), line_num,
+                var_name, (unsigned long)new_size);
+        fprintf(_bft_mem_global_file, " : (%c%9lu) : %12lu : [%10p]",
+                sgn,
+                (unsigned long) ((size_diff > 0) ? size_diff : -size_diff),
+                (unsigned long)_bft_mem_global_alloc_cur,
+                p_loc);
+        fflush(_bft_mem_global_file);
+      }
+
+      _bft_mem_block_realloc(ptr, p_loc, new_size);
+
+      _bft_mem_global_n_reallocs += 1;
+
+#if defined(HAVE_OPENMP)
+      if (in_parallel)
+        omp_unset_lock(&_bft_mem_lock);
+#endif
     }
 
-    if (_bft_mem_global_file != NULL) {
-      char sgn = (size_diff > 0) ? '+' : '-';
-      fprintf(_bft_mem_global_file, "\nrealloc: %-27s:%6d : %-39s: %9lu",
-              _bft_mem_basename(file_name), line_num,
-              var_name, (unsigned long)new_size);
-      fprintf(_bft_mem_global_file, " : (%c%9lu) : %12lu : [%10p]",
-              sgn,
-              (unsigned long) ((size_diff > 0) ? size_diff : -size_diff),
-              (unsigned long)_bft_mem_global_alloc_cur,
-              p_loc);
-      fflush(_bft_mem_global_file);
-    }
-
-    _bft_mem_block_realloc(ptr, p_loc, new_size);
-
-    _bft_mem_global_n_reallocs += 1;
-
     return p_loc;
   }
 
@@ -915,6 +991,12 @@ bft_mem_free(void        *ptr,
 
   if (_bft_mem_global_initialized != 0) {
 
+#if defined(HAVE_OPENMP)
+    int in_parallel = omp_in_parallel();
+    if (in_parallel)
+      omp_set_lock(&_bft_mem_lock);
+#endif
+
     size_info = _bft_mem_block_size(ptr);
 
     _bft_mem_global_alloc_cur -= size_info;
@@ -933,6 +1015,11 @@ bft_mem_free(void        *ptr,
     _bft_mem_block_free(ptr);
 
     _bft_mem_global_n_frees += 1;
+
+#if defined(HAVE_OPENMP)
+    if (in_parallel)
+      omp_unset_lock(&_bft_mem_lock);
+#endif
   }
 
   free(ptr);
@@ -1002,26 +1089,39 @@ bft_mem_memalign(size_t       alignment,
 
   /* Memory allocation counting */
 
-  _bft_mem_global_alloc_cur += alloc_size;
+  {
+#if defined(HAVE_OPENMP)
+    int in_parallel = omp_in_parallel();
+    if (in_parallel)
+      omp_set_lock(&_bft_mem_lock);
+#endif
 
-  if (_bft_mem_global_alloc_max < _bft_mem_global_alloc_cur)
-    _bft_mem_global_alloc_max = _bft_mem_global_alloc_cur;
+    _bft_mem_global_alloc_cur += alloc_size;
 
-  if (_bft_mem_global_file != NULL) {
-    fprintf(_bft_mem_global_file, "\n  alloc: %-27s:%6d : %-39s: %9lu",
-            _bft_mem_basename(file_name), line_num,
-            var_name, (unsigned long)alloc_size);
-    fprintf(_bft_mem_global_file, " : (+%9lu) : %12lu : [%10p]",
-            (unsigned long)alloc_size,
-            (unsigned long)_bft_mem_global_alloc_cur,
-            p_loc);
-    fflush(_bft_mem_global_file);
+    if (_bft_mem_global_alloc_max < _bft_mem_global_alloc_cur)
+      _bft_mem_global_alloc_max = _bft_mem_global_alloc_cur;
+
+    if (_bft_mem_global_file != NULL) {
+      fprintf(_bft_mem_global_file, "\n  alloc: %-27s:%6d : %-39s: %9lu",
+              _bft_mem_basename(file_name), line_num,
+              var_name, (unsigned long)alloc_size);
+      fprintf(_bft_mem_global_file, " : (+%9lu) : %12lu : [%10p]",
+              (unsigned long)alloc_size,
+              (unsigned long)_bft_mem_global_alloc_cur,
+              p_loc);
+      fflush(_bft_mem_global_file);
+    }
+
+    _bft_mem_block_malloc(p_loc, alloc_size);
+
+    _bft_mem_global_n_allocs += 1;
+
+#if defined(HAVE_OPENMP)
+    if (in_parallel)
+      omp_unset_lock(&_bft_mem_lock);
+#endif
   }
 
-  _bft_mem_block_malloc(p_loc, alloc_size);
-
-  _bft_mem_global_n_allocs += 1;
-
   /* Return pointer to allocated memory */
 
   return p_loc;
