@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -113,6 +113,24 @@
 
 BEGIN_C_DECLS
 
+/*=============================================================================
+ * Additional doxygen documentation
+ *============================================================================*/
+
+/*!
+  \file bft_mem_usage.c
+        Base memory usage information (System and Library dependent).
+
+  The memory-usage measurement functions provided here may be system-dependent.
+  If they have not yet been ported to a given type of environment,
+  these functions should return 0.
+
+  The user should thus check for the return values of such functions to avoid
+  reporting invalid values, but the API is guaranteed.
+*/
+
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*-------------------------------------------------------------------------------
  * Local type definitions
  *-----------------------------------------------------------------------------*/
@@ -355,8 +373,8 @@ _bft_mem_usage_pr_size_init(void)
   char  buf[512]; /* should be large enough for "/proc/%lu/status"
                      then beginning of file content */
   size_t  r_size, i;
-  _Bool   status_has_size = false;
-  _Bool   status_has_peak = false;
+  bool    status_has_size = false;
+  bool    status_has_peak = false;
   int  fd;
   const pid_t  pid = getpid();
 
@@ -416,6 +434,8 @@ _bft_mem_usage_pr_size_init(void)
 
 #endif /* defined (__linux__) && ... */
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -737,6 +757,8 @@ bft_mem_usage_n_calls(size_t count[3])
   count[0] = _bft_mem_usage_n_allocs;
   count[1] = _bft_mem_usage_n_reallocs;
   count[2] = _bft_mem_usage_n_frees;
+#else
+  CS_UNUSED(count);
 #endif /* (HAVE_MALLOC_HOOKS) */
 }
 
