@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -42,14 +42,15 @@
 !______________________________________________________________________________.
 !  mode           name          role                                           !
 !______________________________________________________________________________!
-!> \param[in]     nvar          total number of variables
 !> \param[in]     imodif        indicator of what is computed
-!> \param[in,out] bval          dirichlet value for all variables
+!> \param[in,out] bc_en         dirichlet value for the total energy
+!> \param[in,out] bc_pr         dirichlet value for the pressure
+!> \param[in,out] bc_vel        dirichlet value for the velocity
 !_______________________________________________________________________________
 
 
 subroutine cffana &
- ( nvar   , imodif , bval   )
+ ( imodif , bc_en  , bc_pr  , bc_vel )
 
 !===============================================================================
 
@@ -78,18 +79,15 @@ implicit none
 
 ! Arguments
 
-integer          nvar
 integer          imodif
 
-double precision bval(nfabor,nvar)
+double precision bc_en(nfabor), bc_pr(nfabor), bc_vel(3,nfabor)
 
 ! Local variables
 
 integer          iel    , ifac
 integer          ien
-integer          iflmab
 double precision und    , rund
-double precision, dimension(:), pointer :: bmasfl
 double precision, dimension(:,:), pointer :: cofacv
 double precision, dimension(:), pointer :: coface
 double precision, dimension(:), pointer :: brom
@@ -102,9 +100,6 @@ double precision, dimension(:), pointer :: brom
 
 ien = isca(ienerg)
 
-call field_get_key_int(ivarfl(ien), kbmasf, iflmab)
-call field_get_val_s(iflmab, bmasfl)
-
 call field_get_coefac_v(ivarfl(iu), cofacv)
 call field_get_coefac_s(ivarfl(ien), coface)
 
@@ -117,9 +112,9 @@ iel   = ifabor(ifac)
 ! 1. COMPUTE VALUES NEEDED FOR ANALYTICAL FLUX
 !===============================================================================
 
-und   = (bval(ifac,iu)*surfbo(1,ifac)                          &
-       + bval(ifac,iv)*surfbo(2,ifac)                          &
-       + bval(ifac,iw)*surfbo(3,ifac))/surfbn(ifac)
+und   = (bc_vel(1,ifac)*surfbo(1,ifac)                          &
+       + bc_vel(2,ifac)*surfbo(2,ifac)                          &
+       + bc_vel(3,ifac)*surfbo(3,ifac))/surfbn(ifac)
 rund  = brom(ifac)*und
 
 !===============================================================================
@@ -131,20 +126,17 @@ rund  = brom(ifac)*und
 ! has to be imposed
 icvfli(ifac) = 1
 
-! Mass flux
-bmasfl(ifac) = rund * surfbn(ifac)
-
 ! Momentum flux (the centered pressure contribution is directly taken into account
 ! in the pressure BC)
-cofacv(1,ifac) = surfbn(ifac) * rund * bval(ifac,iu)
+cofacv(1,ifac) = suffbn(ifac) * rund * bc_vel(1,ifac)
 
-cofacv(2,ifac) = surfbn(ifac) * rund * bval(ifac,iv)
+cofacv(2,ifac) = suffbn(ifac) * rund * bc_vel(2,ifac)
 
-cofacv(3,ifac) = surfbn(ifac) * rund * bval(ifac,iw)
+cofacv(3,ifac) = suffbn(ifac) * rund * bc_vel(3,ifac)
 
 ! Total energy flux
-coface(ifac) = surfbn(ifac) * (rund * bval(ifac,ien) +              &
-                               und  * bval(ifac,ipr))
+coface(ifac) = suffbn(ifac) * (rund * bc_en(ifac) +  &
+                               und  * bc_pr(ifac))
 
 return
 
