@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -59,9 +59,9 @@ use cstnum
 use ppppar
 use ppthch
 use ppincl
-!bug Ba
 use ihmpre
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -73,6 +73,8 @@ integer          ii
 integer          iok, kscmin, kscmax
 double precision  scaclp(4)
 
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 ! 1. VARIABLES TRANSPORTEES
 !===============================================================================
@@ -109,8 +111,10 @@ iscacp(itempk) = 0
 !         - Schema convectif % schema 2ieme ordre
 !           = 0 : upwind
 !           = 1 : second ordre
-do ii = 1, nvarmx
-  blencv(ii) = 0.d0
+do ii = 1, nvar
+  call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+  vcopt%blencv = 0.d0
+  call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
 enddo
 
 !===============================================================================
@@ -131,9 +135,9 @@ endif
 !     Interdits en compressible
 
 if( (iescal(iespre).ne.0) .or.                            &
-     (iescal(iesder).ne.0) .or.                            &
-     (iescal(iescor).ne.0) .or.                            &
-     (iescal(iestot).ne.0) ) then
+    (iescal(iesder).ne.0) .or.                            &
+    (iescal(iescor).ne.0) .or.                            &
+    (iescal(iestot).ne.0) ) then
   write(nfecra,4000)
   call csexit (1)
 endif
@@ -371,6 +375,3 @@ endif
 
 return
 end subroutine
-
-
-
