@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,34 +20,27 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine cfprop
-!================
-
-!===============================================================================
-!  FONCTION  :
-!  ---------
-
-!     INIT DES POSITIONS DES VARIABLES D'ETAT POUR
-!              POUR LE COMPRESSIBLE SANS CHOC
-!         (DANS VECTEURS PROPCE)
+!> \file cfprop.f90
+!> \brief Properties definition initialization for the compressible module,
+!> according to calculation type selected by the user.
+!>
+!------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-!__________________!____!_____!________________________________________________!
+!-------------------------------------------------------------------------------
+!   mode          name          role
+!-------------------------------------------------------------------------------
+!______________________________________________________________________________!
 
-!     Type: i (integer), r (real), s (string), a (array), l (logical),
-!           and composite types (ex: ra real array)
-!     mode: <-- input, --> output, <-> modifies data, --- work array
-!===============================================================================
+subroutine cfprop
 
 !===============================================================================
 ! Module files
 !===============================================================================
 
 use paramx
+use ihmpre
 use dimens
 use numvar
 use optcal
@@ -57,6 +50,8 @@ use cstnum
 use ppppar
 use ppthch
 use ppincl
+use field
+use cs_cf_bindings
 
 !===============================================================================
 
@@ -64,42 +59,65 @@ implicit none
 
 ! Local variables
 
-integer       nprini
+integer       ifcvsl
 
 !===============================================================================
-! 1. POSITIONNEMENT DES PROPRIETES : PROPCE
-!    Physique particuliere : Compressible sans choc
+
+!===============================================================================
+! Interfaces
 !===============================================================================
 
-nprini = nproce
+interface
 
-if (ippmod(icompf).ge.0) then
+  subroutine cs_field_pointer_map_compressible()  &
+    bind(C, name='cs_field_pointer_map_compressible')
+    use, intrinsic :: iso_c_binding
+    implicit none
+  end subroutine cs_field_pointer_map_compressible
 
-  if (icv.gt.0) then
-    call add_property_field('specific_heat_const_vol', &
-                            'Specific_Heat_Const_Vol', &
-                            icv)
-    call hide_property(icv)
-    ihisvr(nvpp,1) = 0
-  endif
+end interface
 
-  if (iviscv.ne.0) then
-    call add_property_field('volume_viscosity', &
-                            'Volume_Viscosity', &
-                            iviscv)
-    call hide_property(iviscv)
-    ihisvr(nvpp,1) = 0
-  endif
+!===============================================================================
 
-! Nb algebraic (or state) variables
-!   specific to specific physic: nsalpp
-!   total: nsalto
-
-nsalpp = nproce - nprini
-nsalto = nproce
+! Variability of specific heat at constant volume Cv (constant by default)
+icv = -1
+cv0 = 0.d0
+call cs_cf_set_thermo_options
 
+! Variability of volumetric molecular viscosity (gui setting)
+if (iihmpr.eq.1) then
+  call csvvva(iviscv)
 endif
 
+! User settings: variability of molecular thermal conductivity and volume
+! viscosity
+call uscfx1
+
+! Dynamic viscosity of reference of the scalar total energy (ienerg).
+call field_get_key_int(ivarfl(isca(itempk)), kivisl, ifcvsl)
+if (ifcvsl.ge.0 .or. icv.ge.0) then
+  call field_set_key_int(ivarfl(isca(ienerg)), kivisl, 0)
+else
+  call field_set_key_int(ivarfl(isca(ienerg)), kivisl, -1)
+endif
+
+! Properties definition initialization according to their variability
+if (icv.ge.0) then
+  call add_property_field_1d('specific_heat_const_vol', &
+                          'Specific_Heat_Const_Vol', &
+                          icv)
+  call hide_property(icv)
+endif
+
+if (iviscv.ge.0) then
+  call add_property_field_1d('volume_viscosity', &
+                          'Volume_Viscosity', &
+                          iviscv)
+  call hide_property(iviscv)
+endif
+
+! MAP to C API
+call cs_field_pointer_map_compressible
 
 return
 end subroutine
