@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -23,10 +23,8 @@
 subroutine cfrusb &
 !================
 
- ( nvar   ,                                                       &
-   ifac   ,                                                       &
-   gammag ,                                                       &
-   rtp    , bval   )
+ ( ifac   ,                                                       &
+   bc_en  , bc_pr  , bc_vel    )
 
 !===============================================================================
 ! FUNCTION :
@@ -43,10 +41,7 @@ subroutine cfrusb &
 !__________________.____._____.________________________________________________.
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
-! nvar             ! i  ! <-- ! total number of variables                      !
 ! ifac             ! i  ! <-- ! face number                                    !
-! gammag           ! r  ! <-- ! gamma du gaz                                   !
-! rtp(ncelet,nvar) ! tr ! <-- ! variables de calcul au centre des cellules     !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -60,12 +55,12 @@ subroutine cfrusb &
 !===============================================================================
 
 use paramx
+use pointe, only:rvoid1
 use numvar
 use optcal
 use cstphy
 use cstnum
 use parall
-use pointe
 use entsor
 use ppppar
 use ppthch
@@ -73,6 +68,7 @@ use ppincl
 use cfpoin
 use mesh
 use field
+use cs_cf_bindings
 
 !===============================================================================
 
@@ -80,64 +76,92 @@ implicit none
 
 ! Arguments
 
-integer          nvar
 integer          ifac
 
-double precision gammag
-
-double precision rtp(ncelet,nflown:nvar)
-double precision bval(nfabor,nvar)
+double precision bc_en(nfabor), bc_pr(nfabor), bc_vel(3,nfabor)
 
 ! Local variables
 
-integer          iel
+integer          iel, l_size
 integer          ien
-integer          iflmab
 
-double precision und    , uni    , rund   , runi   , cd     , ci
-double precision rrus   , runb
-double precision, dimension(:), pointer :: bmasfl
+double precision und, uni, rund, runi, cd, ci, cd2(1), ci2(1)
+double precision rrus, runb
 double precision, dimension(:,:), pointer :: cofacv
 double precision, dimension(:), pointer :: coface
 double precision, dimension(:), pointer :: crom, brom
 double precision, dimension(:,:), pointer :: vel
+double precision, dimension(:), pointer :: cvar_pr, cvar_en, cpro_cp, cpro_cv
+double precision, dimension(:), pointer :: rvoid
+
+double precision cpi(1), cvi(1)
 
 !===============================================================================
 
-! Map field arrays
-call field_get_val_v(ivarfl(iu), vel)
+rvoid => null()
 
 !===============================================================================
 ! 0. INITIALISATION
 !===============================================================================
 
+! Map field arrays
+call field_get_val_v(ivarfl(iu), vel)
+
 ien = isca(ienerg)
 
 call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
 
-call field_get_key_int(ivarfl(ien), kbmasf, iflmab)
-call field_get_val_s(iflmab, bmasfl)
+call field_get_val_s(ivarfl(ipr), cvar_pr)
+call field_get_val_s(ivarfl(ien), cvar_en)
 
 call field_get_coefac_v(ivarfl(iu), cofacv)
 call field_get_coefac_s(ivarfl(ien), coface)
 
 iel   = ifabor(ifac)
 
+! Initialize local specific heat values and handle uniform cases
+
+if (icp.ge.0) then
+  call field_get_val_s(icp, cpro_cp)
+  cpi(1) = cpro_cp(iel)
+else
+  cpro_cp => rvoid1
+  cpi(1) = 0.d0
+endif
+
+if (icv.ge.0) then
+  call field_get_val_s(icv, cpro_cv)
+  cvi(1) = cpro_cv(iel)
+else
+  cpro_cv => rvoid1
+  cvi(1) = 0.d0
+endif
+
 !===============================================================================
 ! 1. COMPUTE VALUES NEEDED FOR RUSANOV SCHEME
 !===============================================================================
 
-und   = (bval(ifac,iu)*surfbo(1,ifac)                          &
-       + bval(ifac,iv)*surfbo(2,ifac)                          &
-       + bval(ifac,iw)*surfbo(3,ifac))/surfbn(ifac)
+und   = (bc_vel(1,ifac)*surfbo(1,ifac)                          &
+       + bc_vel(2,ifac)*surfbo(2,ifac)                          &
+       + bc_vel(3,ifac)*surfbo(3,ifac))/surfbn(ifac)
 uni   = (vel(1,iel)*surfbo(1,ifac)                            &
        + vel(2,iel)*surfbo(2,ifac)                            &
        + vel(3,iel)*surfbo(3,ifac))/surfbn(ifac)
 rund  = brom(ifac)*und
 runi  = crom(iel)     *uni
-cd    = sqrt(gammag*bval(ifac,ipr)/brom(ifac))
-ci    = sqrt(gammag*rtp(iel,ipr)/crom(iel))
+
+l_size = 1
+
+call cs_cf_thermo_c_square(cpi, cvi, &
+                           bc_pr(ifac:ifac), brom(ifac:ifac), &
+                           rvoid, rvoid, rvoid, cd2, l_size)
+call cs_cf_thermo_c_square(cpi, cvi, &
+                           cvar_pr(iel:iel), crom(iel:iel), &
+                           rvoid, rvoid, rvoid, ci2, l_size)
+
+cd    = sqrt(cd2(1))
+ci    = sqrt(ci2(1))
 rrus  = max(abs(und)+cd,abs(uni)+ci)
 
 runb  = 0.5d0*(brom(ifac)*und+crom(iel)*uni)          &
@@ -152,35 +176,32 @@ runb  = 0.5d0*(brom(ifac)*und+crom(iel)*uni)          &
 ! has to be imposed
 icvfli(ifac) = 1
 
-! Mass flux
-bmasfl(ifac) = runb*surfbn(ifac)
-
 ! Momentum flux (the centered pressure contribution is directly taken into account
 ! in the pressure BC)
-cofacv(1,ifac) = surfbn(ifac)*                                                  &
-                 0.5d0*( rund*bval(ifac,iu) + runi*vel(1,iel)                   &
-                         -rrus*(brom(ifac)*bval(ifac,iu)                        &
+cofacv(1,ifac) = suffbn(ifac)*                                                  &
+                 0.5d0*( rund*bc_vel(1,ifac) + runi*vel(1,iel)                   &
+                         -rrus*(brom(ifac)*bc_vel(1,ifac)                        &
                          -crom(iel)*vel(1,iel)) )
 
-cofacv(2,ifac) = surfbn(ifac)*                                                  &
-                 0.5d0*( rund*bval(ifac,iv) + runi*vel(2,iel)                   &
-                         -rrus*( brom(ifac)*bval(ifac,iv)                       &
+cofacv(2,ifac) = suffbn(ifac)*                                                  &
+                 0.5d0*( rund*bc_vel(2,ifac) + runi*vel(2,iel)                   &
+                         -rrus*( brom(ifac)*bc_vel(2,ifac)                       &
                          -crom(iel)*vel(2,iel)) )
 
-cofacv(3,ifac) = surfbn(ifac)*                                                  &
-                 0.5d0*( rund*bval(ifac,iw) + runi*vel(3,iel)                   &
-                         -rrus*(brom(ifac)*bval(ifac,iw)                        &
+cofacv(3,ifac) = suffbn(ifac)*                                                  &
+                 0.5d0*( rund*bc_vel(3,ifac) + runi*vel(3,iel)                   &
+                         -rrus*(brom(ifac)*bc_vel(3,ifac)                        &
                          -crom(iel)*vel(3,iel)) )
 
 ! BC for the pressure gradient in the momentum balance
-bval(ifac,ipr) = 0.5d0 * (bval(ifac,ipr) + rtp(iel,ipr))
+bc_pr(ifac) = 0.5d0 * (bc_pr(ifac) + cvar_pr(iel))
 
 ! Total energy flux
-coface(ifac) = surfbn(ifac)*                                                    &
-               0.5d0*( rund*bval(ifac,ien) + runi*rtp(iel,ien)                  &
-                       +und*bval(ifac,ipr) + uni*rtp(iel,ipr)                   &
-                       -rrus*(brom(ifac)*bval(ifac,ien)                         &
-                       -crom(iel)*rtp(iel,ien)) )
+coface(ifac) = suffbn(ifac)*                                                    &
+               0.5d0*( rund*bc_en(ifac) + runi*cvar_en(iel)                  &
+                       +und*bc_pr(ifac) + uni*cvar_pr(iel)                   &
+                       -rrus*(brom(ifac)*bc_en(ifac)                         &
+                       -crom(iel)*cvar_en(iel)) )
 
 return
 
