@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -70,8 +70,8 @@ implicit none
 
 ! Local variables
 
-integer        isc, f_id
-integer        kscmin, kscmax
+integer        f_id
+integer        kscmin, kscmax, kscavr
 
 !===============================================================================
 
@@ -87,12 +87,6 @@ interface
     implicit none
   end subroutine cs_field_pointer_map_gas_combustion
 
-  subroutine cs_gui_labels_gas_combustion()  &
-    bind(C, name='cs_gui_labels_gas_combustion')
-    use, intrinsic :: iso_c_binding
-    implicit none
-  end subroutine cs_gui_labels_gas_combustion
-
 end interface
 
 !===============================================================================
@@ -102,6 +96,7 @@ end interface
 ! Key ids for clipping
 call field_get_key_id("min_scalar_clipping", kscmin)
 call field_get_key_id("max_scalar_clipping", kscmax)
+call field_get_key_id("first_moment_id", kscavr)
 
 !===============================================================================
 ! 1. Define variable fields
@@ -115,13 +110,14 @@ if (ippmod(icod3p).ge.0) then
   ! Mixture fraction and its variance
 
   call add_model_scalar_field('mixture_fraction', 'Fra_MEL', ifm)
-  call add_model_scalar_field('mixture_fraction_variance', 'Var_FrMe', ifp2m)
-  iscavr(ifp2m) = ifm
-
   f_id = ivarfl(isca(ifm))
   call field_set_key_double(f_id, kscmin, 0.d0)
   call field_set_key_double(f_id, kscmax, 1.d0)
 
+  call add_model_scalar_field('mixture_fraction_variance', 'Var_FrMe', ifp2m)
+  f_id = ivarfl(isca(ifp2m))
+  call field_set_key_int(f_id, kscavr, ivarfl(isca(ifm)))
+
   ! Enthalpy
 
   if (ippmod(icod3p).eq.1) then
@@ -194,7 +190,9 @@ if (ippmod(icolwc).ge.0 ) then
   call field_set_key_double(f_id, kscmax, 1.d0)
 
   call add_model_scalar_field('mixture_fraction_variance', 'Var_FrMe', ifp2m)
-  iscavr(ifp2m) = ifm
+  f_id = ivarfl(isca(ifp2m))
+  call field_set_key_int(f_id, kscavr, ivarfl(isca(ifm)))
+
 
   call add_model_scalar_field('mass_fraction', 'Fra_Mas', iyfm)
   f_id = ivarfl(isca(iyfm))
@@ -202,7 +200,8 @@ if (ippmod(icolwc).ge.0 ) then
   call field_set_key_double(f_id, kscmax, 1.d0)
 
   call add_model_scalar_field('mass_fraction_variance', 'Var_FMa', iyfp2m)
-  iscavr(iyfp2m)= iyfm
+  f_id = ivarfl(isca(iyfp2m))
+  call field_set_key_int(f_id, kscavr, ivarfl(isca(iyfm)))
 
   if (ippmod(icolwc).ge.2 ) then
     call add_model_scalar_field('mass_fraction_covariance', 'COYF_PP4', icoyfp)
@@ -224,26 +223,12 @@ endif
 ! MAP to C API
 call cs_field_pointer_map_gas_combustion
 
-! Mapping for GUI
-if (iihmpr.eq.1) then
-  call cs_gui_labels_gas_combustion
-endif
-
 !===============================================================================
 ! 2. PROPRIETES PHYSIQUES
 !    A RENSEIGNER OBLIGATOIREMENT (sinon pb dans varpos)
-!      IVISLS, ICP
+!      ICP
 !===============================================================================
 
-do isc = 1, nscapp
-
-  if (iscavr(iscapp(isc)).le.0) then
-    ! Reference dynamic viscosity relative to this scalar
-    ivisls(iscapp(isc)) = 0
-  endif
-
-enddo
-
 if (ippmod(icod3p).eq.1 .or.   &
     ippmod(icoebu).eq.1 .or.   &
     ippmod(icoebu).eq.3 .or.   &
@@ -253,10 +238,9 @@ if (ippmod(icod3p).eq.1 .or.   &
 
   ! Although we are in enthalpy formulation, we keep Cp constant
 
-  icp = 0
+  icp = -1
 
 endif
 
 return
 end subroutine
-
