@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -44,15 +44,19 @@
 !> \param[in]     fdeb          abscissa of rectangle low boundary
 !> \param[in]     ffin          abscissa of rectangle high boundary
 !> \param[in]     hrec          rectangle height
-!> \param[in]     rtp           calculated variables at cell centers
-!> \param[in,out] propce        physical properties at cell centers
+!> \param[out]    tpdf          indicator for pdf shape:
+!                               - 0: Dirac at mean value
+!                               - 1: rectangle
+!                               - 2: Dirac's peak at \f$ f_{min} \f$
+!                               - 3: Dirac's peak at \f$ f_{max} \f$
+!                               - 4: rectangle and 2 Dirac's pics
 !> \param[in]     w1            work array
 !_______________________________________________________________________________
 
 subroutine d3pint &
  ( indpdf ,                                                       &
    dirmin , dirmax , fdeb   , ffin   , hrec   , tpdf ,            &
-   rtp    , propce , w1      )
+   w1      )
 
 !===============================================================================
 
@@ -63,7 +67,6 @@ subroutine d3pint &
 use paramx
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use entsor
 use cstnum
@@ -74,6 +77,8 @@ use ppincl
 use radiat
 use mesh
 use field
+use pointe
+
 !===============================================================================
 
 implicit none
@@ -83,14 +88,13 @@ implicit none
 integer          indpdf(ncelet)
 double precision dirmin(ncelet), dirmax(ncelet)
 double precision fdeb(ncelet), ffin(ncelet), hrec(ncelet), tpdf(ncelet)
-double precision rtp(ncelet,nflown:nvar), propce(ncelet,*), w1(ncelet)
+double precision w1(ncelet)
 
 
 ! Local variables
 
 integer          iel, icg
-integer          ih, if, jh, jf, iptsro
-integer          ipcsca, ipctem, ipckab, ipct4, ipct3
+integer          ih, if, jh, jf
 double precision aa1, bb1, aa2, bb2, f1, f2, a, b, fmini, fmaxi
 double precision u, v, c, d, temsmm, fsir
 double precision fm, fp2m
@@ -98,8 +102,15 @@ double precision fm, fp2m
 double precision dtsmdf  , dd1df  , dd2df  , df1df  , df2df  , dhrecdf
 double precision dtsmdfp2, dd1dfp2, dd2dfp2, df1dfp2, df2dfp2, dhrecdfp2
 double precision dtsmdd1, dtsmdd2, dtsmdf1, dtsmdf2, dtsmdhrec, dtsmdhs
-double precision dadhs, dbdhs, cotshs
-double precision, dimension(:), pointer ::  crom
+double precision dadhs, dbdhs, yprod
+double precision, dimension(:), pointer :: cpro_rho, cpro_tsrho
+double precision, dimension(:), pointer :: cproaa_rho
+double precision, dimension(:), pointer :: cvar_scalt, cpro_tsscalt
+double precision, dimension(:), pointer :: cvar_fm, cvar_fp2m
+double precision, dimension(:), pointer :: cpro_tsfm, cpro_tsfp2m, cpro_ym3
+double precision, dimension(:), pointer :: cpro_temp
+double precision, dimension(:), pointer :: cpro_ckabs, cpro_t4m, cpro_t3m
+type(pmapper_double_r1), dimension(:), pointer :: cpro_csca
 
 !===============================================================================
 
@@ -114,20 +125,24 @@ save          ipass
 ! 0. ON COMPTE LES PASSAGES
 !===============================================================================
 
-! Initialize variables to avoid compiler warnings
+allocate(cpro_csca(ngazg))
 
-ipckab = 0
-ipct3 = 0
-ipct4 = 0
+do icg = 1, ngazg
+  call field_get_val_s(iym(icg), cpro_csca(icg)%p)
+enddo
+
+! Initialize variables to avoid compiler warnings
 
 aa1 = 0.d0
 aa2 = 0.d0
 bb1 = 0.d0
 bb2 = 0.d0
 
-
 ipass = ipass + 1
 
+if (ippmod(icod3p).eq.1) call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
+call field_get_val_s(ivarfl(isca(ifm)), cvar_fm)
+call field_get_val_s(ivarfl(isca(ifp2m)), cvar_fp2m)
 
 !===============================================================================
 ! 1. INTEGRATION DES NGAZG FRACTIONS MASSIQUES D'ESPECES GLOBALES
@@ -142,8 +157,8 @@ fmaxi = 1.d0
 
 do iel = 1, ncel
 
-  fm   = rtp(iel,isca(ifm))
-  fp2m = rtp(iel,isca(ifp2m))
+  fm   = cvar_fm(iel)
+  fp2m = cvar_fp2m(iel)
 
   do icg = 1, ngazg
 
@@ -180,34 +195,32 @@ do iel = 1, ncel
       bb2 = -1.d0/(1.d0-fsir)
     endif
 
-    ipcsca = ipproc(iym(icg))
-
     if (indpdf(iel) .eq. 1) then
 
 ! ---> Integration de la PDF
 
-      propce(iel,ipcsca) = dirmin(iel) * ( aa1 + bb1 * fmini )  &
-                          + dirmax(iel) * ( aa2 + bb2 * fmaxi )
+      cpro_csca(icg)%p(iel) = dirmin(iel) * ( aa1 + bb1 * fmini )  &
+                            + dirmax(iel) * ( aa2 + bb2 * fmaxi )
       if (fdeb(iel).lt.fsir) then
         f1 = fdeb(iel)
         f2 = min( fsir,ffin(iel) )
-        propce(iel,ipcsca) = propce(iel,ipcsca)                 &
-             + hrec(iel)*(f2-f1)*(aa1+bb1*5.d-1*(f2+f1))
+        cpro_csca(icg)%p(iel) = cpro_csca(icg)%p(iel)                 &
+                              + hrec(iel)*(f2-f1)*(aa1+bb1*5.d-1*(f2+f1))
       endif
       if (ffin(iel).gt.fsir) then
         f1 = max(fsir,fdeb(iel))
         f2 = ffin(iel)
-        propce(iel,ipcsca) = propce(iel,ipcsca)                 &
-             + hrec(iel)*(f2-f1)*(aa2+bb2*5.d-1*(f2+f1))
+        cpro_csca(icg)%p(iel) = cpro_csca(icg)%p(iel)                 &
+                              + hrec(iel)*(f2-f1)*(aa2+bb2*5.d-1*(f2+f1))
       endif
     else
 
 ! ---> Degenerescence sur la valeur moyenne
 
       if (fm.le.fsir) then
-        propce(iel,ipcsca) = aa1+bb1*fm
+        cpro_csca(icg)%p(iel) = aa1+bb1*fm
       else
-        propce(iel,ipcsca) = aa2+bb2*fm
+        cpro_csca(icg)%p(iel) = aa2+bb2*fm
       endif
 
     endif
@@ -237,7 +250,7 @@ if (ippmod(icod3p).eq.1) then
   !==========
   ( ncelet , ncel    , indpdf ,                                   &
     dirmin , dirmax  , fdeb   , ffin   , hrec   ,                 &
-    rtp(1,isca(ifm)) , rtp(1,isca(iscalt)),                          &
+    cvar_fm          , cvar_scalt      ,                          &
     w1      )
 
 endif
@@ -253,20 +266,28 @@ endif
 
 ! ---> Positions des variables, coefficients
 
-ipctem = ipproc(itemp)
-if ( iirayo.gt.0 ) then
-  ipckab = ipproc(ickabs)
-  ipct4  = ipproc(it4m)
-  ipct3  = ipproc(it3m)
+call field_get_val_s(itemp, cpro_temp)
+call field_get_val_s(icrom, cpro_rho)
+
+if (idilat.ge.4) then
+  call field_get_val_prev2_s(icrom, cproaa_rho)
+  call field_get_val_s(iustdy(itsrho), cpro_tsrho)
+  call field_get_val_s(iustdy(ifm  ), cpro_tsfm)
+  call field_get_val_s(iustdy(ifp2m  ), cpro_tsfp2m)
+  call field_get_val_s(iustdy(iscalt), cpro_tsscalt)
+  call field_get_val_s(iym(3), cpro_ym3)
 endif
 
-call field_get_val_s(icrom, crom)
-if (idilat.eq.4) iptsro = ipproc(iustdy(itsrho))
+if ( iirayo.gt.0 ) then
+  call field_get_val_s(ickabs, cpro_ckabs)
+  call field_get_val_s(it4m, cpro_t4m)
+  call field_get_val_s(it3m, cpro_t3m)
+endif
 
 do iel = 1, ncel
 
-  fm   = rtp(iel,isca(ifm))
-  fp2m = rtp(iel,isca(ifp2m))
+  fm   = cvar_fm(iel)
+  fp2m = cvar_fp2m(iel)
 
   if (indpdf(iel).eq.1) then
 
@@ -279,24 +300,19 @@ do iel = 1, ncel
     enddo
     if (w1(iel) .ge. hh(1)) ih = 1
     if (w1(iel) .le. hh(nmaxh)) ih = nmaxh-1
-    propce(iel,ipctem) = dirmin(iel)*tinoxy +                   &
-                          dirmax(iel)*tinfue
-    temsmm = dirmin(iel)/wmolg(2)*tinoxy                         &
-           + dirmax(iel)/wmolg(1)*tinfue
+    cpro_temp(iel) = dirmin(iel)*tinoxy + dirmax(iel)*tinfue
+    temsmm = dirmin(iel)/wmolg(2)*tinoxy + dirmax(iel)/wmolg(1)*tinfue
 
     ! Weakly compressible algorithm: d T/M /d D1, d T/M /d D2
-    if (idilat.eq.4) then
+    if (idilat.ge.4) then
       dtsmdd1 = tinoxy/wmolg(2)
       dtsmdd2 = tinfue/wmolg(1)
     endif
 
     if (iirayo.gt.0) then
-      propce(iel,ipckab) =                                       &
-        dirmin(iel)*ckabsg(2)  + dirmax(iel)*ckabsg(1)
-      propce(iel,ipct4) =                                        &
-        dirmin(iel)*tinoxy**4 + dirmax(iel)*tinfue**4
-      propce(iel,ipct3) =                                        &
-        dirmin(iel)*tinoxy**3 + dirmax(iel)*tinfue**3
+      cpro_ckabs(iel) = dirmin(iel)*ckabsg(2)  + dirmax(iel)*ckabsg(1)
+      cpro_t4m(iel) =   dirmin(iel)*tinoxy**4 + dirmax(iel)*tinfue**4
+      cpro_t3m(iel) =   dirmin(iel)*tinoxy**3 + dirmax(iel)*tinfue**3
     endif
     if = 1
     do jf = 1, (nmaxf-1)
@@ -309,10 +325,9 @@ do iel = 1, ncel
     f1 = fdeb(iel)
 
     ! Weakly compressible algorithm: initialisation of d T/M /d Hrec d T/M /d Hs
-    if (idilat.eq.4) then
+    if (idilat.ge.4) then
       dtsmdhrec = 0.d0
       dtsmdhs   = 0.d0
-      cotshs    = 0.d0
     endif
 
     do while ( (ffin(iel)-f2).gt.epzero )
@@ -331,8 +346,7 @@ do iel = 1, ncel
 
 ! ----- Calcul de la temperature par integration
 
-      propce(iel,ipctem) = propce(iel,ipctem)                   &
-         + hrec(iel)*(f2-f1)*(a+b*(f1+f2)/2.d0)
+      cpro_temp(iel) = cpro_temp(iel) + hrec(iel)*(f2-f1)*(a+b*(f1+f2)/2.d0)
 
 ! ----- Preparation aux calculs du coefficient d'absorption
 !                               de T^4 et de T^3
@@ -376,23 +390,23 @@ do iel = 1, ncel
 ! ----- Calcul du coefficient d'absorption
 !           et des termes T^4 et de T^3 (si rayonnement)
 
-        propce(iel,ipckab) = propce(iel,ipckab) +               &
+        cpro_ckabs(iel) = cpro_ckabs(iel) +               &
           hrec(iel)*( u*(f2-f1) + v*(f2**2-f1**2)*0.5d0 )
 
-        propce(iel,ipct4) = propce(iel,ipct4) +                 &
-          hrec(iel)*                                             &
-      (      a**4            * (f2-f1)                            &
-   +   (4.d0*a**3  *b      ) * (f2**2-f1**2)/2.d0                 &
-   +   (6.d0*(a**2)*(b**2) ) * (f2**3-f1**3)/3.d0                 &
-   +   (4.d0*a     *(b**3) ) * (f2**4-f1**4)/4.d0                 &
-   +   (            (b**4) ) * (f2**5-f1**5)/5.d0  )
+        cpro_t4m(iel) = cpro_t4m(iel) +                             &
+                        hrec(iel)*                                  &
+                        (      a**4            * (f2-f1)            &
+                      +(4.d0*a**3  *b      ) * (f2**2-f1**2)/2.d0   &
+                      +(6.d0*(a**2)*(b**2) ) * (f2**3-f1**3)/3.d0   &
+                      +(4.d0*a     *(b**3) ) * (f2**4-f1**4)/4.d0   &
+                      +(            (b**4) ) * (f2**5-f1**5)/5.d0  )
 
-        propce(iel,ipct3) = propce(iel,ipct3) +                 &
-          hrec(iel)*                                             &
-      (      (a**3)          * (f2-f1)                            &
-   +   (3.d0*(a**2)*b      ) * (f2**2-f1**2)/2.d0                 &
-   +   (3.d0*a     *(b**2) ) * (f2**3-f1**3)/3.d0                 &
-   +   (            (b**3) ) * (f2**4-f1**4)/4.d0  )
+        cpro_t3m(iel) = cpro_t3m(iel) +                              &
+                        hrec(iel)*                                   &
+                        (      (a**3)          * (f2-f1)             &
+                      +   (3.d0*(a**2)*b      ) * (f2**2-f1**2)/2.d0 &
+                      +   (3.d0*a     *(b**2) ) * (f2**3-f1**3)/3.d0 &
+                      +   (            (b**3) ) * (f2**4-f1**4)/4.d0  )
 
       endif
 
@@ -406,7 +420,7 @@ do iel = 1, ncel
         ! Weakly compressible algorithm:
         ! d T/M /d f0 ; d T/M /d Hrec ; d T/M /d Hs ;
         ! d T/M /d f1 est calcule apres la boucle pour etre sur d'avoir f1 = ffin
-        if (idilat.eq.4) then
+        if (idilat.ge.4) then
 
           if (ippmod(icod3p).eq.1) then
 
@@ -435,27 +449,11 @@ do iel = 1, ncel
       f1 = f2
     enddo
 
-    if (idilat.eq.4) then
+    if (idilat.ge.4) then
 
       ! Weakly compressible algorithm: d T/M /d f1
       dtsmdf2 = hrec(iel) * ( a*c +(c*b+a*d)*ffin(iel) + b*d*ffin(iel)**2 )
 
-      ! Weakly compressible algorithm: factor for Hs source term
-      if (iirayo.ge.1) then
-
-        cotshs = 0.d0
-        if( ffin(iel).lt.fsir) then
-          cotshs = hrec(iel)/2.d0*(ffin(iel)**2-fdeb(iel)**2)/fsir
-        elseif( fdeb(iel).gt.fsir) then
-          cotshs = hrec(iel)/2.d0 * ( (ffin(iel)-fdeb(iel))          &
-                 *(2.d0-(ffin(iel)+fdeb(iel))/(1.d0-fsir)))
-        else
-          cotshs = hrec(iel)/2.d0 * ( (fsir**2-fdeb(iel)**2)/fsir       &
-                 + (ffin(iel)-fsir)*(2.d0-(ffin(iel)+fsir))/(1.d0-fsir))
-        endif
-
-      endif
-
     endif
 
   else
@@ -486,7 +484,7 @@ do iel = 1, ncel
 
 ! ----- Calcul de la temperature a partir de la valeur moyenne
 
-    propce(iel,ipctem) = a+b*fm
+    cpro_temp(iel) = a+b*fm
 
     if (fm.lt.fsir) then
 !         On a demarre cote pauvre
@@ -496,7 +494,7 @@ do iel = 1, ncel
 !         On termine cote riche (en commencant avec f1=fs)
       c = (  -fsir/wmolg(1)+1.d0/wmolg(3))/(1.d0-fsir)
       d = (   1.d0/wmolg(1)-1.d0/wmolg(3))/(1.d0-fsir)
-     endif
+    endif
 
     if (iirayo.gt.0) then
       if (fm.lt.fsir) then
@@ -513,17 +511,17 @@ do iel = 1, ncel
 !         et des termes T^4 et de T^3
 !         a partir de la valeur moyenne (si rayonnement)
 
-      propce(iel,ipckab) = u + v*fm
-      propce(iel,ipct4) = a**4                                   &
-       + (4.d0*(a**3)*b      ) * fm                              &
-       + (6.d0*(a**2)*(b**2) ) * fm**2                           &
-       + (4.d0*a     *(b**3) ) * fm**3                           &
-       + (            (b**4) ) * fm**4
+      cpro_ckabs(iel) = u + v*fm
+      cpro_t4m(iel) = a**4                                     &
+                    + (4.d0*(a**3)*b      ) * fm               &
+                    + (6.d0*(a**2)*(b**2) ) * fm**2            &
+                    + (4.d0*a     *(b**3) ) * fm**3            &
+                    + (            (b**4) ) * fm**4
 
-      propce(iel,ipct3) = a**3                                   &
-       + ( 3.d0*(a**2)*b      ) * fm                             &
-       + ( 3.d0*a     *(b**2) ) * fm**2                          &
-       + (             (b**3) ) * fm**3
+      cpro_t3m(iel) = a**3                                     &
+                    + ( 3.d0*(a**2)*b      ) * fm              &
+                    + ( 3.d0*a     *(b**2) ) * fm**2           &
+                    + (             (b**3) ) * fm**3
 
     endif
 
@@ -532,7 +530,7 @@ do iel = 1, ncel
     temsmm = a*c +(c*b+a*d)*fm + b*d*fm**2
 
     ! Weakly compressible algorithm: derivative
-    if (idilat.eq.4) then
+    if (idilat.ge.4) then
       dtsmdf   = (c*b+a*d) + 2.d0*b*d*fm
       dtsmdfp2 = 0.d0
 
@@ -545,12 +543,6 @@ do iel = 1, ncel
 
         dtsmdhs = (c + d * fm) * ( dadhs + fm * dbdhs )
 
-        if (fm.le.fsir) then
-          cotshs = fm/fsir
-        else
-          cotshs = (1.d0-fm)/(1.d0-fsir)
-        endif
-
       endif
 
     endif
@@ -559,14 +551,14 @@ do iel = 1, ncel
 
 ! ---> Calcul de la masse volumique
 
-  if (ipass.gt.1.or.(isuite.eq.1.and.initro.eq.1)) then
-    crom(iel) = srrom*crom(iel)               &
-                        + (1.d0-srrom)*                         &
-                        ( p0/(rr*temsmm) )
+  if (ipass.ge.1.or.(isuite.eq.1.and.initro.eq.1)) then
+    cpro_rho(iel) = srrom*cpro_rho(iel)               &
+                  + (1.d0-srrom)*                         &
+                  ( pther/(cs_physical_constants_r*temsmm) )
   endif
 
   ! Weakly compressible algorithm: Derivative calculation of pdf parameters
-  if (idilat.eq.4) then
+  if (idilat.ge.4) then
 
     ! dD0df, dD0df"2,
     ! dD1df, dD1df"2,
@@ -648,37 +640,37 @@ do iel = 1, ncel
     endif
 
     ! Scalar contribution is computed and add to the total source term
-    propce(iel,iptsro) =                                                &
-           (-rr/p0 * dtsmdf)   * propce(iel,ipproc(iustdy(ifm)))        &
-         + (-rr/p0 * dtsmdfp2) * propce(iel,ipproc(iustdy(ifp2m)))
+    cpro_tsrho(iel) =  (-cs_physical_constants_r/pther * dtsmdf)        &
+                       *cpro_tsfm(iel)                                    &
+                     + (-cs_physical_constants_r/pther * dtsmdfp2)      &
+                       *cpro_tsfp2m(iel)
 
-    if( ippmod(icod3p).eq.1 ) then
-    propce(iel,iptsro) = propce(iel,iptsro)                             &
-         + (-rr/p0 * dtsmdhs)  * propce(iel,ipproc(iustdy(iscalt)))
+    yprod = cpro_ym3(iel)
+
+    ! Note that h*=hm/Yp
+    if (ippmod(icod3p).eq.1.and.abs(yprod).gt.epzero) then
+
+      cpro_tsrho(iel) =  cpro_tsrho(iel)                                 &
+                       + (-cs_physical_constants_r/pther * dtsmdhs)      &
+                         /yprod*cpro_tsscalt(iel)
 
-      if( iirayo.ge.1 .and. abs(cotshs).gt.epzero ) then
-        propce(iel,iptsro) = propce(iel,iptsro)                         &
-        + (-rr/p0 * dtsmdhs) / cotshs                                   &
-                   * propce(iel,ipproc(itsre(1)))*volume(iel)
-      endif
     endif
 
     ! D(rho)/Dt = 1/rho d(rho)/dz Diff(z) = -rho d(1/rho)/dz Diff(z)
     ! iptsro contains -d(1/rho)/dz Diff(z) > x rho
-    propce(iel,iptsro) = propce(iel,iptsro) * crom(iel)
+    cpro_tsrho(iel) = cpro_tsrho(iel) * cpro_rho(iel)**2              &
+                                            / cproaa_rho(iel)
 
     ! arrays are re-initialize for source terms of next time step
-    propce(iel,ipproc(iustdy(ifm  ))) = 0.d0
-    propce(iel,ipproc(iustdy(ifp2m))) = 0.d0
-    ! array fo iscalt is used to store enthalpy gap from adiabatic Hs
-    ! to compute its diffusive term in tridim
-    if (ippmod(icod3p).ge.1) propce(iel,ipproc(iustdy(iscalt  ))) = w1(iel)
+    cpro_tsfm(iel) = 0.d0
+    cpro_tsfp2m(iel) = 0.d0
+    if (ippmod(icod3p).ge.1) cpro_tsscalt(iel) = 0.d0
 
   endif
 
 enddo
 
+deallocate(cpro_csca)
 
 return
 end subroutine
-
