@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,7 @@ subroutine ebuini &
 !================
 
  ( nvar   , nscal  ,                                              &
-   dt     , rtp    , propce )
+   dt     )
 
 !===============================================================================
 ! FONCTION :
@@ -59,9 +59,6 @@ subroutine ebuini &
 ! nvar             ! i  ! <-- ! total number of variables                      !
 ! nscal            ! i  ! <-- ! total number of scalars                        !
 ! dt(ncelet)       ! tr ! <-- ! valeur du pas de temps                         !
-! rtp              ! tr ! <-- ! variables de calcul au centre des              !
-! (ncelet,*)       !    !     !    cellules                                    !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -96,17 +93,25 @@ implicit none
 
 integer          nvar   , nscal
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), propce(ncelet,*)
+double precision dt(ncelet)
 
 ! Local variables
 
-character*80     chaine
+character(len=80) :: chaine
 integer          iel, mode, igg, izone
 integer          iscal, ivar, ii
 double precision hinit, coefg(ngazgm), hair, tinitk
 double precision sommqf, sommqt, sommq, tentm, fmelm
 double precision valmax, valmin, xkent, xeent, d2s3
 
+double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_phi
+double precision, dimension(:), pointer :: cvar_fb, cvar_omg, cvar_nusa
+double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
+double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
+double precision, dimension(:), pointer :: cvar_ygfm, cvar_fm
+double precision, dimension(:), pointer :: cvar_scalt
+double precision, dimension(:), pointer :: cvar_scal
+
 ! NOMBRE DE PASSAGES DANS LA ROUTINE
 
 integer          ipass
@@ -128,6 +133,37 @@ enddo
 
 d2s3 = 2.d0/3.d0
 
+call field_get_val_s(ivarfl(isca(iygfm)), cvar_ygfm)
+if ( ippmod(icoebu).eq.2 .or. ippmod(icoebu).eq.3 ) then
+  call field_get_val_s(ivarfl(isca(ifm)), cvar_fm)
+endif
+if ( ippmod(icoebu).eq.1 .or. ippmod(icoebu).eq.3 ) then
+  call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
+endif
+
+if (itytur.eq.2) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (itytur.eq.3) then
+  call field_get_val_s(ivarfl(ir11), cvar_r11)
+  call field_get_val_s(ivarfl(ir22), cvar_r22)
+  call field_get_val_s(ivarfl(ir33), cvar_r33)
+  call field_get_val_s(ivarfl(ir12), cvar_r12)
+  call field_get_val_s(ivarfl(ir13), cvar_r13)
+  call field_get_val_s(ivarfl(ir23), cvar_r23)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (iturb.eq.50) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+  call field_get_val_s(ivarfl(iphi), cvar_phi)
+  call field_get_val_s(ivarfl(ifb), cvar_fb)
+elseif (iturb.eq.60) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iomg), cvar_omg)
+elseif (iturb.eq.70) then
+  call field_get_val_s(ivarfl(inusa), cvar_nusa)
+endif
+
 !===============================================================================
 ! 2. INITIALISATION DES INCONNUES :
 !      UNIQUEMENT SI ON NE FAIT PAS UNE SUITE
@@ -173,51 +209,51 @@ if ( isuite.eq.0 ) then
 
       if (itytur.eq.2) then
 
-        rtp(iel,ik)  = xkent
-        rtp(iel,iep) = xeent
+        cvar_k(iel)  = xkent
+        cvar_ep(iel) = xeent
 
       elseif (itytur.eq.3) then
 
-        rtp(iel,ir11) = d2s3*xkent
-        rtp(iel,ir22) = d2s3*xkent
-        rtp(iel,ir33) = d2s3*xkent
-        rtp(iel,ir12) = 0.d0
-        rtp(iel,ir13) = 0.d0
-        rtp(iel,ir23) = 0.d0
-        rtp(iel,iep)  = xeent
+        cvar_r11(iel) = d2s3*xkent
+        cvar_r22(iel) = d2s3*xkent
+        cvar_r33(iel) = d2s3*xkent
+        cvar_r12(iel) = 0.d0
+        cvar_r13(iel) = 0.d0
+        cvar_r23(iel) = 0.d0
+        cvar_ep(iel)  = xeent
 
       elseif (iturb.eq.50) then
 
-        rtp(iel,ik)   = xkent
-        rtp(iel,iep)  = xeent
-        rtp(iel,iphi) = d2s3
-        rtp(iel,ifb)  = 0.d0
+        cvar_k(iel)   = xkent
+        cvar_ep(iel)  = xeent
+        cvar_phi(iel) = d2s3
+        cvar_fb(iel)  = 0.d0
 
       elseif (iturb.eq.60) then
 
-        rtp(iel,ik)   = xkent
-        rtp(iel,iomg) = xeent/cmu/xkent
+        cvar_k(iel)   = xkent
+        cvar_omg(iel) = xeent/cmu/xkent
 
       elseif(iturb.eq.70) then
 
-        rtp(iel,inusa) = cmu*xkent**2/xeent
+        cvar_nusa(iel) = cmu*xkent**2/xeent
 
       endif
 
 ! ----- Fraction massique de gaz frais
 
-      rtp(iel,isca(iygfm)) = 1.d0
+      cvar_ygfm(iel) = 1.d0
 
 ! ----- Fraction de melange
 
       if ( ippmod(icoebu).eq.2 .or. ippmod(icoebu).eq.3 ) then
-        rtp(iel,isca(ifm)) = zero
+        cvar_fm(iel) = zero
       endif
 
 ! ----- Enthalpie du melange
 
       if ( ippmod(icoebu).eq.1 .or. ippmod(icoebu).eq.3 ) then
-        rtp(iel,isca(iscalt)) = hair
+        cvar_scalt(iel) = hair
       endif
 
     enddo
@@ -263,41 +299,40 @@ if ( isuite.eq.0 ) then
 
 ! ----- Fraction massique de gaz frais
 
-      rtp(iel,isca(iygfm)) = 5.d-1
+      cvar_ygfm(iel) = 5.d-1
 
 ! ----- Fraction de melange
 
       if ( ippmod(icoebu).eq.2 .or. ippmod(icoebu).eq.3 ) then
-        rtp(iel,isca(ifm)) = fmelm
+        cvar_fm(iel) = fmelm
       endif
 
 ! ----- Enthalpie du melange
 
       if ( ippmod(icoebu).eq.1 .or. ippmod(icoebu).eq.3 ) then
-        rtp(iel,isca(iscalt)) = hinit
+        cvar_scalt(iel) = hinit
       endif
 
     enddo
 
 ! ----- On donne la main a l'utilisateur
-
-    call cs_user_initialization &
+    call cs_user_f_initialization &
     !==========================
   ( nvar   , nscal  ,                                            &
     dt     )
 
 ! ----- En periodique et en parallele,
-!       il faut echanger ces initialisations (qui sont en fait dans RTPA)
+!       il faut echanger ces initialisations
 
     if (irangp.ge.0.or.iperio.eq.1) then
-      call synsca(rtp(1,isca(iygfm)))
+      call synsca(cvar_ygfm)
       !==========
       if ( ippmod(icoebu).eq.2 .or. ippmod(icoebu).eq.3 ) then
-        call synsca(rtp(1,isca(ifm)))
+        call synsca(cvar_fm)
         !==========
       endif
       if ( ippmod(icoebu).eq.1 .or. ippmod(icoebu).eq.3 ) then
-        call synsca(rtp(1,isca(iscalt)))
+        call synsca(cvar_scalt)
         !==========
       endif
     endif
@@ -310,11 +345,12 @@ if ( isuite.eq.0 ) then
     do ii  = 1, nscapp
       iscal = iscapp(ii)
       ivar  = isca(iscal)
+      call field_get_val_s(ivarfl(isca(iscal)), cvar_scal)
       valmax = -grand
       valmin =  grand
       do iel = 1, ncel
-        valmax = max(valmax,rtp(iel,ivar))
-        valmin = min(valmin,rtp(iel,ivar))
+        valmax = max(valmax,cvar_scal(iel))
+        valmin = min(valmin,cvar_scal(iel))
       enddo
       call field_get_label(ivarfl(ivar), chaine)
       if (irangp.ge.0) then
