@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -23,8 +23,7 @@
 subroutine ebuphy &
 !================
 
- ( mbrom  , izfppp ,                                              &
-   rtp    , propce )
+ ( mbrom  , izfppp )
 
 !===============================================================================
 ! FONCTION :
@@ -41,9 +40,6 @@ subroutine ebuphy &
 ! mbrom            ! te ! <-- ! indicateur de remplissage de romb              !
 ! izfppp           ! te ! <-- ! numero de zone de la face de bord              !
 ! (nfabor)         !    !     !  pour le module phys. part.                    !
-! rtp              ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at current time step)                        !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________!____!_____!________________________________________________!
 
 !     Type: i (integer), r (real), s (string), a (array), l (logical),
@@ -58,7 +54,6 @@ subroutine ebuphy &
 use paramx
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use entsor
@@ -80,16 +75,11 @@ implicit none
 integer          mbrom
 integer          izfppp(nfabor)
 
-double precision rtp(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
-
 ! Local variables
 
-integer          ipctem, ipcfue, ipcoxy, ipcpro
 integer          igg, iel
-integer          ipckab, ipt4, ipt3
 integer          ifac, izone
-integer          ipcycg, mode
+integer          mode
 double precision coefg(ngazgm), ygfm, ygbm, epsi
 double precision nbmol , temsmm , fmel , ckabgf, ckabgb
 double precision masmgb, hgb, tgb, masmgf, masmg
@@ -97,8 +87,13 @@ double precision masmgb, hgb, tgb, masmgf, masmg
 double precision, allocatable, dimension(:) :: yfuegf, yoxygf, yprogf
 double precision, allocatable, dimension(:) :: yfuegb, yoxygb, yprogb
 double precision, allocatable, dimension(:) :: temp, masmel
-double precision, dimension(:), pointer ::  brom, crom
+double precision, dimension(:), pointer :: brom, crom
 double precision, dimension(:), pointer :: bsval
+double precision, dimension(:), pointer :: cvar_ygfm, cvar_fm
+double precision, dimension(:), pointer :: cvar_scalt
+double precision, dimension(:), pointer :: cpro_temp,cpro_ymgg
+double precision, dimension(:), pointer :: cpro_ym1, cpro_ym2, cpro_ym3
+double precision, dimension(:), pointer :: cpro_ckabs, cpro_t4m, cpro_t3m
 integer       ipass
 data          ipass /0/
 save          ipass
@@ -122,10 +117,6 @@ allocate(temp(ncelet), masmel(ncelet))
 
 ! Initialize variables to avoid compiler warnings
 
-ipckab = 0
-ipt3 = 0
-ipt4 = 0
-
 ckabgb = 0.d0
 ckabgf = 0.d0
 
@@ -142,16 +133,24 @@ enddo
 
 call field_get_val_s(icrom, crom)
 call field_get_val_s(ibrom, brom)
-ipctem = ipproc(itemp)
-ipcfue = ipproc(iym(1))
-ipcoxy = ipproc(iym(2))
-ipcpro = ipproc(iym(3))
+call field_get_val_s(itemp, cpro_temp)
+call field_get_val_s(iym(1), cpro_ym1)
+call field_get_val_s(iym(2), cpro_ym2)
+call field_get_val_s(iym(3), cpro_ym3)
+
 if ( iirayo.gt.0 ) then
-  ipckab = ipproc(ickabs)
-  ipt4 = ipproc(it4m)
-  ipt3 = ipproc(it3m)
+  call field_get_val_s(ickabs, cpro_ckabs)
+  call field_get_val_s(it4m, cpro_t4m)
+  call field_get_val_s(it3m, cpro_t3m)
 endif
 
+call field_get_val_s(ivarfl(isca(iygfm)), cvar_ygfm)
+if ( ippmod(icoebu).ne.0 .and. ippmod(icoebu).ne.1 ) then
+  call field_get_val_s(ivarfl(isca(ifm)), cvar_fm)
+endif
+if ( ippmod(icoebu).eq.1 .or. ippmod(icoebu).eq.3 ) then
+  call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
+endif
 
 !===============================================================================
 ! 2. DETERMINATION DES GRANDEURS THERMOCHIMIQUES
@@ -186,13 +185,13 @@ endif
 
 ! ---> Grandeurs MELANGE
 
-!       MASMEL           --> Masse molaire du melange
-!       PROPCE(    .,IPCTEM) --> Temperature du melange
-!       PROPCE(    .,IPCROM) --> Masse volumique du melange
-!       PROPCE(,.F,O,P ) --> Fractions massiques en F, O, P
-!       PROPCE(    .,IPCKAB) --> Coefficient d'absorption
-!       PROPCE(    .,IPT4  ) --> terme T^4
-!       PROPCE(    .,IPT3  ) --> terme T^3
+!       masmel           --> Masse molaire du melange
+!       cpro_temp        --> Temperature du melange
+!       crom             --> Masse volumique du melange
+!       cpro_.f,o,p      --> Fractions massiques en F, O, P
+!       cpro_ckabs       --> Coefficient d'absorption
+!       cpro_t4m         --> terme T^4
+!       cpro_t3m         --> terme T^3
 
 
 ! ---> Fractions massiques des gaz frais et brules en F, O, P
@@ -202,7 +201,7 @@ do iel = 1, ncel
   if ( ippmod(icoebu).eq.0 .or. ippmod(icoebu).eq.1 ) then
     fmel = frmel
   else
-    fmel = rtp(iel,isca(ifm))
+    fmel = cvar_fm(iel)
   endif
 
   yfuegf(iel) = fmel
@@ -243,7 +242,6 @@ do iel = 1, ncel
 
   mode    = -1
   call cothht                                                     &
-  !==========
   ( mode   , ngazg , ngazgm  , coefg  ,                           &
     npo    , npot   , th     , ehgazg ,                           &
     hgf    , tgf    )
@@ -259,7 +257,7 @@ do iel = 1, ncel
   enddo
   masmgb = 1.d0/nbmol
 
-  ygfm = rtp(iel,isca(iygfm))
+  ygfm = cvar_ygfm(iel)
   ygbm = 1.d0 - ygfm
 
 ! ---> Masse molaire du melange
@@ -275,13 +273,12 @@ do iel = 1, ncel
 ! ---- EBU Standard et modifie en conditions permeatiques (avec H)
     hgb = hgf
     if ( ygbm.gt.epsi ) then
-      hgb = ( rtp(iel,isca(iscalt))-hgf*ygfm ) / ygbm
+      hgb = ( cvar_scalt(iel)-hgf*ygfm ) / ygbm
     endif
   endif
 
   mode = 1
   call cothht                                                     &
-  !==========
   ( mode   , ngazg , ngazgm  , coefg  ,                           &
     npo    , npot   , th     , ehgazg ,                           &
     hgb    , tgb    )
@@ -293,7 +290,7 @@ do iel = 1, ncel
 
 ! ---> Temperature du melange
 !      Rq PPl : Il serait plus judicieux de ponderer par les CP (GF et GB)
-  propce(iel,ipctem) = ygfm*tgf + ygbm*tgb
+  cpro_temp(iel) = ygfm*tgf + ygbm*tgb
 
 ! ---> Temperature / Masse molaire
 
@@ -302,23 +299,23 @@ do iel = 1, ncel
 ! ---> Masse volumique du melange
 
   if (ipass.gt.1.or.(isuite.eq.1.and.initro.eq.1)) then
-    crom(iel) = srrom*crom(iel)                 &
-                       + (1.d0-srrom)*                            &
-                         ( p0/(rr*temsmm) )
+    crom(iel) = srrom*crom(iel)                          &
+              + (1.d0-srrom)*                            &
+                ( p0/(cs_physical_constants_r*temsmm) )
   endif
 
 ! ---> Fractions massiques des especes globales
 
-  propce(iel,ipcfue) = yfuegf(iel)*ygfm + yfuegb(iel)*ygbm
-  propce(iel,ipcoxy) = yoxygf(iel)*ygfm + yoxygb(iel)*ygbm
-  propce(iel,ipcpro) = yprogf(iel)*ygfm + yprogb(iel)*ygbm
+  cpro_ym1(iel) = yfuegf(iel)*ygfm + yfuegb(iel)*ygbm
+  cpro_ym2(iel) = yoxygf(iel)*ygfm + yoxygb(iel)*ygbm
+  cpro_ym3(iel) = yprogf(iel)*ygfm + yprogb(iel)*ygbm
 
 ! ---> Grandeurs relatives au rayonnement
 
   if ( iirayo.gt.0 ) then
-    propce(iel,ipckab) = ygfm*ckabgf + ygbm*ckabgb
-    propce(iel,ipt4)   = ygfm*tgf**4 + ygbm*tgb**4
-    propce(iel,ipt3)   = ygfm*tgf**3 + ygbm*tgb**3
+    cpro_ckabs(iel) = ygfm*ckabgf + ygbm*ckabgb
+    cpro_t4m(iel)  = ygfm*tgf**4 + ygbm*tgb**4
+    cpro_t3m(iel)  = ygfm*tgf**3 + ygbm*tgb**3
   endif
 
 enddo
@@ -378,7 +375,7 @@ if ( ipass.gt.1 .or. isuite.eq.1 ) then
         enddo
        masmg = 1.d0/nbmol
        temsmm = tkent(izone)/masmg
-       brom(ifac) = p0/(rr*temsmm)
+       brom(ifac) = p0/(cs_physical_constants_r*temsmm)
       endif
     endif
   enddo
@@ -388,16 +385,14 @@ endif
 ! --> Fractions massiques des especes globales au bord
 !     Uniquement si transport de H
 
-if ( iirayo.gt.0 ) then
-  do igg = 1, ngazg
-    call field_get_val_s(ibym(1), bsval)
-    ipcycg = ipproc(iym(igg))
-    do ifac = 1, nfabor
-      iel = ifabor(ifac)
-      bsval(ifac) = propce(iel,ipcycg)
-    enddo
+do igg = 1, ngazg
+  call field_get_val_s(ibym(1), bsval)
+  call field_get_val_s(iym(igg),cpro_ymgg)
+  do ifac = 1, nfabor
+    iel = ifabor(ifac)
+    bsval(ifac) = cpro_ymgg(iel)
   enddo
-endif
+enddo
 
 ! Free memory
 deallocate(yfuegf, yoxygf, yprogf)
