@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,58 +20,50 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine ebutcl &
-!================
-
- ( itypfb , izfppp ,                                              &
-   propce ,                                                       &
-   rcodcl )
-
 !===============================================================================
-! FONCTION :
+! Function :
 ! --------
 
-!    CONDITIONS AUX LIMITES AUTOMATIQUES
-
-!           COMBUSTION GAZ MODELE EBU
-
+!> \file ebutcl.f90
+!> \brief Automatic boundary conditions for perfect premixed flame combustion
+!>        model (EBU)
 
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! itypfb           ! ia ! <-- ! boundary face types                            !
-! izfppp           ! te ! <-- ! numero de zone de la face de bord              !
-! (nfabor)         !    !     !  pour le module phys. part.                    !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-! rcodcl           ! tr ! --> ! valeur des conditions aux limites              !
-!  (nfabor,nvar    !    !     !  aux faces de bord                             !
-!                  !    !     ! rcodcl(1) = valeur du dirichlet                !
-!                  !    !     ! rcodcl(2) = valeur du coef. d'echange          !
-!                  !    !     !  ext. (infinie si pas d'echange)               !
-!                  !    !     ! rcodcl(3) = valeur de la densite de            !
-!                  !    !     !  flux (negatif si gain) w/m2 ou                !
-!                  !    !     !  hauteur de rugosite (m) si icodcl=6           !
-!                  !    !     ! pour les vitesses (vistl+visct)*gradu          !
-!                  !    !     ! pour la pression             dt*gradp          !
-!                  !    !     ! pour les scalaires                             !
-!                  !    !     !        cp*(viscls+visct/sigmas)*gradt          !
-!__________________!____!_____!________________________________________________!
+!______________________________________________________________________________.
+!  mode           name          role                                           !
+!______________________________________________________________________________!
+!> \param[in]     nvar          total number of variables
+!> \param[in]     itypfb        boundary face types
+!> \param[out]    izfppp        boundary face zone number
+!> \param[out]    rcodcl        boundary condition values:
+!>                               - rcodcl(1) value of the dirichlet
+!>                               - rcodcl(2) value of the exterior exchange
+!>                                 coefficient (infinite if no exchange)
+!>                               - rcodcl(3) value flux density
+!>                                 (negative if gain) in w/m2 or roughness
+!>                                 in m if icodcl=6
+!>                                 -# for the velocity \f$ (\mu+\mu_T)
+!>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
+!>                                 -# for the pressure \f$ \Delta t
+!>                                    \grad P \cdot \vect{n}  \f$
+!>                                 -# for a scalar \f$ cp \left( K +
+!>                                     \dfrac{K_T}{\sigma_T} \right)
+!>                                     \grad T \cdot \vect{n} \f$
+!_______________________________________________________________________________
+
+subroutine ebutcl &
+ ( itypfb , izfppp ,                                              &
+   rcodcl )
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
 !===============================================================================
 
 !===============================================================================
 ! Module files
 !===============================================================================
 
-! Arguments
-
 use paramx
+use dimens, only : nvar
 use numvar
 use optcal
 use cstphy
@@ -85,6 +77,8 @@ use cpincl
 use ppincl
 use mesh
 use field
+use cs_c_bindings
+
 !===============================================================================
 
 implicit none
@@ -94,18 +88,19 @@ implicit none
 integer          itypfb(nfabor)
 integer          izfppp(nfabor)
 
-double precision propce(ncelet,*)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
 integer          igg, ifac, izone, mode
-integer          icke, ipcvis, ii, iel, iok
+integer          icke, ii, iel, iok
 double precision qisqc, viscla, d2s3, uref2, rhomoy, dhy, xiturb
-double precision ustar2, xkent, xeent, hgazf , tgazf, hgazb, tgazb
+double precision hgazf , tgazf, hgazb, tgazb
 double precision qcalc(nozppm), hgent(nozppm)
 double precision coefg(ngazgm)
 double precision, dimension(:), pointer ::  brom
+double precision, dimension(:), pointer :: viscl
+
 !===============================================================================
 !===============================================================================
 ! 1.  INITIALISATIONS
@@ -113,7 +108,7 @@ double precision, dimension(:), pointer ::  brom
 
 
 call field_get_val_s(ibrom, brom)
-ipcvis = ipproc(iviscl)
+call field_get_val_s(iviscl, viscl)
 
 d2s3 = 2.d0/3.d0
 
@@ -272,54 +267,26 @@ do ifac = 1, nfabor
       uref2 = max(uref2,epzero)
       rhomoy = brom(ifac)
       iel    = ifabor(ifac)
-      viscla = propce(iel,ipcvis)
+      viscla = viscl(iel)
       icke   = icalke(izone)
       dhy    = dh(izone)
       xiturb = xintur(izone)
-      ustar2 = 0.d0
-      xkent = epzero
-      xeent = epzero
+
       if (icke.eq.1) then
-        call keendb                                               &
-        !==========
-        ( uref2, dhy, rhomoy, viscla, cmu, xkappa,                &
-          ustar2, xkent, xeent )
+        !   Calculation of turbulent inlet conditions using
+        !     standard laws for a circular pipe
+        !     (their initialization is not needed here but is good practice).
+        call turbulence_bc_inlet_hyd_diam(ifac, uref2, dhy, rhomoy, viscla,  &
+                                          rcodcl)
       else if (icke.eq.2) then
-        call keenin                                               &
-        !==========
-        ( uref2, xiturb, dhy, cmu, xkappa, xkent, xeent )
-      endif
 
-      if (itytur.eq.2) then
+        ! Calculation of turbulent inlet conditions using
+        !   the turbulence intensity and standard laws for a circular pipe
+        !   (their initialization is not needed here but is good practice)
 
-        rcodcl(ifac,ik,1)  = xkent
-        rcodcl(ifac,iep,1) = xeent
+        call turbulence_bc_inlet_turb_intensity(ifac, uref2, xiturb, dhy,  &
+                                                rcodcl)
 
-      elseif (itytur.eq.3) then
-
-        rcodcl(ifac,ir11,1) = d2s3*xkent
-        rcodcl(ifac,ir22,1) = d2s3*xkent
-        rcodcl(ifac,ir33,1) = d2s3*xkent
-        rcodcl(ifac,ir12,1) = 0.d0
-        rcodcl(ifac,ir13,1) = 0.d0
-        rcodcl(ifac,ir23,1) = 0.d0
-        rcodcl(ifac,iep,1)  = xeent
-
-      elseif (iturb.eq.50) then
-
-        rcodcl(ifac,ik,1)   = xkent
-        rcodcl(ifac,iep,1)  = xeent
-        rcodcl(ifac,iphi,1) = d2s3
-        rcodcl(ifac,ifb,1)  = 0.d0
-
-      elseif (iturb.eq.60) then
-
-        rcodcl(ifac,ik,1)   = xkent
-        rcodcl(ifac,iomg,1) = xeent/cmu/xkent
-
-      elseif(iturb.eq.70) then
-
-        rcodcl(ifac,inusa,1) = cmu*xkent**2/xeent
 
       endif
 
