@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,6 @@ subroutine ebutss &
 !================
 
  ( iscal  ,                                                       &
-   rtpa   ,                                                       &
    smbrs  , rovsdt )
 
 !===============================================================================
@@ -67,8 +66,6 @@ subroutine ebutss &
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
 ! iscal            ! i  ! <-- ! scalar number                                  !
-! rtpa             ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at previous time step)                       !
 ! smbrs(ncelet)    ! tr ! --> ! second membre explicite                        !
 ! rovsdt(ncelet    ! tr ! --> ! partie diagonale implicite                     !
 !__________________!____!_____!________________________________________________!
@@ -87,7 +84,6 @@ use paramx
 use numvar
 use entsor
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use ppppar
@@ -97,6 +93,7 @@ use cpincl
 use ppincl
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -106,16 +103,21 @@ implicit none
 
 integer          iscal
 
-double precision rtpa(ncelet,nflown:nvar)
 double precision smbrs(ncelet), rovsdt(ncelet)
 
 ! Local variables
 
-character*80     chaine
+character(len=80) :: chaine
 integer          ivar, iel
 
 double precision, allocatable, dimension(:) :: w1, w2, w3
 double precision, dimension(:), pointer ::  crom
+double precision, dimension(:), pointer :: cvara_k, cvara_ep, cvara_omg
+double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
+double precision, dimension(:), pointer :: cvara_scal
+
+type(var_cal_opt) :: vcopt
+
 !===============================================================================
 !===============================================================================
 ! 1. INITIALISATION
@@ -136,6 +138,24 @@ call field_get_label(ivarfl(ivar), chaine)
 ! --- Numero des grandeurs physiques (voir cs_user_boundary_conditions)
 call field_get_val_s(icrom, crom)
 
+if ( ivar.eq.isca(iygfm) ) then
+  call field_get_val_prev_s(ivarfl(isca(iscal)), cvara_scal)
+endif
+
+if (itytur.eq.2.or.iturb.eq.50) then
+  call field_get_val_prev_s(ivarfl(ik), cvara_k)
+  call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+elseif (itytur.eq.3) then
+  call field_get_val_prev_s(ivarfl(ir11), cvara_r11)
+  call field_get_val_prev_s(ivarfl(ir22), cvara_r22)
+  call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
+  call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+elseif (iturb.eq.60) then
+  call field_get_val_prev_s(ivarfl(ik), cvara_k)
+  call field_get_val_prev_s(ivarfl(iomg), cvara_omg)
+endif
+
+call field_get_key_struct_var_cal_opt(ivarfl(ivar), vcopt)
 
 !===============================================================================
 ! 2. PRISE EN COMPTE DES TERMES SOURCES
@@ -145,7 +165,7 @@ if ( ivar.eq.isca(iygfm) ) then
 
 ! ---> Terme source pour la fraction massique moyenne de gaz frais
 
-  if (iwarni(ivar).ge.1) then
+  if (vcopt%iwarni.ge.1) then
     write(nfecra,1000) chaine(1:8)
   endif
 
@@ -154,31 +174,31 @@ if ( ivar.eq.isca(iygfm) ) then
   if (itytur.eq.2) then
 
     do iel = 1, ncel
-      w1(iel) = rtpa(iel,ik)
-      w2(iel) = rtpa(iel,iep)
+      w1(iel) = cvara_k(iel)
+      w2(iel) = cvara_ep(iel)
     enddo
 
   elseif (itytur.eq.3) then
 
     do iel = 1, ncel
-      w1(iel) = 0.5d0 *( rtpa(iel,ir11)                    &
-                        +rtpa(iel,ir22)                    &
-                        +rtpa(iel,ir33) )
-      w2(iel) = rtpa(iel,iep)
+      w1(iel) = 0.5d0 *( cvara_r11(iel)                    &
+                        +cvara_r22(iel)                    &
+                        +cvara_r33(iel) )
+      w2(iel) = cvara_ep(iel)
     enddo
 
   elseif (iturb.eq.50) then
 
     do iel = 1, ncel
-      w1(iel) = rtpa(iel,ik)
-      w2(iel) = rtpa(iel,iep)
+      w1(iel) = cvara_k(iel)
+      w2(iel) = cvara_ep(iel)
     enddo
 
   elseif (iturb.eq.60) then
 
     do iel = 1, ncel
-      w1(iel) = rtpa(iel,ik)
-      w2(iel) = cmu*rtpa(iel,ik)*rtpa(iel,iomg)
+      w1(iel) = cvara_k(iel)
+      w2(iel) = cmu*cvara_k(iel)*cvara_omg(iel)
     enddo
 
   endif
@@ -188,8 +208,8 @@ if ( ivar.eq.isca(iygfm) ) then
          w2(iel).gt.epzero       ) then
       w3(iel) = cebu*w2(iel)/w1(iel)                              &
                    *crom(iel)*volume(iel)                &
-                   *(1.d0 - rtpa(iel,ivar))
-      smbrs(iel) = smbrs(iel) - rtpa(iel,ivar)*w3(iel)
+                   *(1.d0 - cvara_scal(iel))
+      smbrs(iel) = smbrs(iel) - cvara_scal(iel)*w3(iel)
       rovsdt(iel) = rovsdt(iel) + max(w3(iel),zero)
     endif
   enddo
