@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,6 @@ subroutine lwctss &
 !================
 
  ( iscal  ,                                                       &
-   rtpa   , propce ,                                              &
    smbrs  , rovsdt )
 
 !===============================================================================
@@ -67,9 +66,6 @@ subroutine lwctss &
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
 ! iscal            ! i  ! <-- ! scalar number                                  !
-! rtpa             ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at previous time step)                       !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 ! smbrs(ncelet)    ! tr ! --> ! second membre explicite                        !
 ! rovsdt(ncelet    ! tr ! --> ! partie diagonale implicite                     !
 !__________________!____!_____!________________________________________________!
@@ -88,7 +84,6 @@ use paramx
 use numvar
 use entsor
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use parall
@@ -101,6 +96,7 @@ use ppincl
 use mesh
 use field
 use field_operator
+use pointe
 
 !===============================================================================
 
@@ -110,26 +106,28 @@ implicit none
 
 integer          iscal
 
-double precision rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
 double precision smbrs(ncelet), rovsdt(ncelet)
 
 ! Local variables
 
 integer          ivar, iel, idirac
 integer          inc , iccocg, iprev
-integer          ipcvst
 integer          ii
 
-integer          iptscl(ndracm), ipfmal(ndracm)
-integer          ipfmel(ndracm), iprhol(ndracm)
-
 double precision sum, epsi
 double precision tsgrad, tschim, tsdiss
+double precision turb_schmidt
 
 double precision, allocatable, dimension(:,:) :: gradf, grady
 double precision, allocatable, dimension(:) :: w10, w11
-double precision, dimension(:), pointer ::  crom
+double precision, dimension(:), pointer :: crom
+double precision, dimension(:), pointer :: visct
+double precision, dimension(:), pointer :: cvara_k, cvara_ep, cvara_omg
+double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
+double precision, dimension(:), pointer :: cvara_scal
+double precision, dimension(:), pointer :: cvara_yfm, cvara_fm
+type(pmapper_double_r1), dimension(:), pointer :: cpro_fmel, cpro_fmal
+type(pmapper_double_r1), dimension(:), pointer :: cpro_tscl, cpro_rhol
 
 !===============================================================================
 
@@ -146,14 +144,35 @@ ivar = isca(iscal)
 
 ! ---
 call field_get_val_s(icrom, crom)
-ipcvst = ipproc(ivisct)
+call field_get_val_s(ivisct, visct)
+
+call field_get_val_prev_s(ivarfl(isca(iscal)), cvara_scal)
+call field_get_val_prev_s(ivarfl(isca(iyfm)), cvara_yfm)
+call field_get_val_prev_s(ivarfl(isca(ifm)), cvara_fm)
+
+if (itytur.eq.2.or.iturb.eq.50) then
+  call field_get_val_prev_s(ivarfl(ik), cvara_k)
+  call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+elseif (itytur.eq.3) then
+  call field_get_val_prev_s(ivarfl(ir11), cvara_r11)
+  call field_get_val_prev_s(ivarfl(ir22), cvara_r22)
+  call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
+  call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+elseif (iturb.eq.60) then
+  call field_get_val_prev_s(ivarfl(ik), cvara_k)
+  call field_get_val_prev_s(ivarfl(iomg), cvara_omg)
+endif
+
+allocate(cpro_fmel(ndirac))
+allocate(cpro_fmal(ndirac))
+allocate(cpro_tscl(ndirac))
+allocate(cpro_rhol(ndirac))
 
-! --- Numero des grandeurs physiques (voir cs_user_boundary_conditions)
 do idirac = 1, ndirac
-  iptscl(idirac) = ipproc(itscl(idirac))
-  ipfmal(idirac) = ipproc(ifmal(idirac))
-  ipfmel(idirac) = ipproc(ifmel(idirac))
-  iprhol(idirac) = ipproc(irhol(idirac))
+  call field_get_val_s(ifmel(idirac), cpro_fmel(idirac)%p)
+  call field_get_val_s(ifmal(idirac), cpro_fmal(idirac)%p)
+  call field_get_val_s(irhol(idirac), cpro_rhol(idirac)%p)
+  call field_get_val_s(itscl(idirac), cpro_tscl(idirac)%p)
 enddo
 
 !===============================================================================
@@ -167,14 +186,14 @@ if (ivar.eq.isca(iyfm)) then
   do iel = 1, ncel
       sum = zero
       do idirac = 1, ndirac
-        sum  = sum + propce(iel,iprhol(idirac))                   &
-           *propce(iel,iptscl(idirac))*volume(iel)
+        sum  = sum + cpro_rhol(idirac)%p(iel)                   &
+           *cpro_tscl(idirac)%p(iel)*volume(iel)
       enddo
 
 ! terme implicite
 
-      if (rtpa(iel,ivar).gt.epsi) then
-        rovsdt(iel) = rovsdt(iel) + max(-sum/rtpa(iel,ivar),zero)
+      if (cvara_scal(iel).gt.epsi) then
+        rovsdt(iel) = rovsdt(iel) + max(-sum/cvara_scal(iel),zero)
       endif
 
 ! terme explicite
@@ -192,9 +211,9 @@ if (ivar.eq.isca(iyfp2m)) then
   do iel = 1, ncel
     sum = zero
     do idirac = 1, ndirac
-      sum  = sum + (propce(iel,iptscl(idirac))*volume(iel)        &
-        *(propce(iel,ipfmal(idirac)) - rtpa(iel,isca(iyfm)))      &
-             *propce(iel,iprhol(idirac)))
+      sum  = sum + (cpro_tscl(idirac)%p(iel)*volume(iel)        &
+        *(cpro_fmal(idirac)%p(iel) - cvara_yfm(iel))           &
+             *cpro_rhol(idirac)%p(iel))
     enddo
     smbrs(iel) = smbrs(iel) + sum
   enddo
@@ -216,7 +235,7 @@ if (ivar.eq.isca(icoyfp)) then
 
   ii = isca(ifm)
   do iel = 1, ncel
-    w10(iel) = rtpa(iel,ii)
+    w10(iel) = cvara_fm(iel)
   enddo
 
   iprev = 1
@@ -232,7 +251,7 @@ if (ivar.eq.isca(icoyfp)) then
 
   ii = isca(iyfm)
   do iel = 1, ncel
-    w11(iel) = rtpa(iel,ii)
+    w11(iel) = cvara_yfm(iel)
   enddo
 
   iprev = 1
@@ -255,35 +274,37 @@ if (ivar.eq.isca(icoyfp)) then
   if (itytur.eq.2) then
 
     do iel = 1, ncel
-      w10(iel) = rtpa(iel,ik)
-      w11(iel) = rtpa(iel,iep)
+      w10(iel) = cvara_k(iel)
+      w11(iel) = cvara_ep(iel)
     enddo
 
   elseif (itytur.eq.3) then
 
     do iel = 1, ncel
-      w10(iel) = ( rtpa(iel,ir11)                          &
-                  +rtpa(iel,ir22)                          &
-                  +rtpa(iel,ir33) ) / 2.d0
-      w11(iel) = rtpa(iel,iep)
+      w10(iel) = ( cvara_r11(iel)                          &
+                  +cvara_r22(iel)                          &
+                  +cvara_r33(iel) ) / 2.d0
+      w11(iel) = cvara_ep(iel)
     enddo
 
   elseif (iturb.eq.50) then
 
     do iel = 1, ncel
-      w10(iel) = rtpa(iel,ik)
-      w11(iel) = rtpa(iel,iep)
+      w10(iel) = cvara_k(iel)
+      w11(iel) = cvara_ep(iel)
     enddo
 
   elseif (iturb.eq.60) then
 
     do iel = 1, ncel
-      w10(iel) = rtpa(iel,ik)
-      w11(iel) = cmu*rtpa(iel,ik)*rtpa(iel,iomg)
+      w10(iel) = cvara_k(iel)
+      w11(iel) = cmu*cvara_k(iel)*cvara_omg(iel)
     enddo
 
   endif
 
+  call field_get_key_double(ivarfl(isca(iscal)), ksigmas, turb_schmidt)
+
   do iel=1,ncel
 
 !  A confirmer :
@@ -302,7 +323,7 @@ if (ivar.eq.isca(icoyfp)) then
 ! terme de gradient
 
     tsgrad =  (2.0d0                                              &
-         * propce(iel,ipcvst)/(sigmas(iscal))                     &
+         * visct(iel)/(turb_schmidt)                              &
          * (  gradf(1,iel)*grady(1,iel)                           &
             + gradf(2,iel)*grady(2,iel)                           &
             + gradf(3,iel)*grady(3,iel) ))                        &
@@ -311,16 +332,16 @@ if (ivar.eq.isca(icoyfp)) then
 
 ! terme de dissipation
 
-    tsdiss = -w11(iel) * rtpa(iel,ivar)
+    tsdiss = -w11(iel) * cvara_scal(iel)
 
 ! terme de chimique
 
     tschim = zero
     do idirac = 1, ndirac
       tschim =   tschim                                           &
-           + (propce(iel,iptscl(idirac))                          &
-           *(propce(iel,ipfmel(idirac))-rtpa(iel,isca(ifm)))      &
-           *volume(iel))*propce(iel,iprhol(idirac))
+           + (cpro_tscl(idirac)%p(iel)                          &
+           *(cpro_fmel(idirac)%p(iel)-cvara_fm(iel))      &
+           *volume(iel))*cpro_rhol(idirac)%p(iel)
     enddo
 
 ! --> Somme des termes
@@ -335,6 +356,9 @@ if (ivar.eq.isca(icoyfp)) then
 
 endif
 
+deallocate(cpro_fmel, cpro_fmal)
+deallocate(cpro_tscl, cpro_rhol)
+
 !----
 ! FIN
 !----
