@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,8 +24,7 @@ subroutine pdfpp3 &
 !================
 
  ( ncelet , ncel  ,                                               &
-   fm     , fp2m  , yfm    , yfp2m , coyfp  ,                     &
-   propce )
+   fm     , fp2m  , yfm    , yfp2m , coyfp )
 
 !===============================================================================
 ! FONCTION :
@@ -72,7 +71,6 @@ subroutine pdfpp3 &
 ! yfm              ! tr ! <-- ! moyenne de la fraction massique                !
 ! yfp2m            ! tr ! <-- ! variance de la fraction massique               !
 ! coyfp            ! tr !  ->           ! covariance
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -108,7 +106,6 @@ integer          ncelet, ncel
 double precision fm(ncelet)   , fp2m(ncelet)
 double precision yfm(ncelet)  , yfp2m(ncelet)
 double precision coyfp(ncelet)
-double precision propce(ncelet,*)
 
 ! Local variables
 
@@ -137,17 +134,15 @@ double precision y2p(ndracm)
 
 ! --- Pointeurs & autres
 
-integer          ipctem, ipcmam
-integer          ipampl(ndracm), ipfmel(ndracm)
-integer          ipfmal(ndracm), ipteml(ndracm)
-integer          ipmaml(ndracm)
-integer          iprhol(ndracm)
-integer          iptscl(ndracm)
-integer          ipcfue, ipcoxy, ipcpro, ipctsc
 double precision nbmol,  temsmm
 double precision sum7, sum8, sum9, sum10, sum11, sum12, sum17
 double precision sum1, sum2, sum3, sum4, sum5, sum6, sum16, sum15
 double precision, dimension(:), pointer ::  crom
+double precision, dimension(:), pointer ::  cpro_temp, cpro_tsc, cpro_mam
+double precision, dimension(:), pointer ::  cpro_ym1, cpro_ym2, cpro_ym3
+type(pmapper_double_r1), dimension(:), pointer :: cpro_fmel, cpro_fmal, cpro_teml
+type(pmapper_double_r1), dimension(:), pointer :: cpro_tscl, cpro_rhol, cpro_maml
+type(pmapper_double_r1), dimension(:), pointer :: cpro_ampl
 integer ipass
 data    ipass /0/
 save    ipass
@@ -156,22 +151,31 @@ save    ipass
 
 ! ---> Position des variables
 
-do idirac = 1, ndirac
-  ipampl(idirac) = ipproc(iampl(idirac))
-  ipfmel(idirac) = ipproc(ifmel(idirac))
-  ipfmal(idirac) = ipproc(ifmal(idirac))
-  ipteml(idirac) = ipproc(iteml(idirac))
-  ipmaml(idirac) = ipproc(imaml(idirac))
-  iprhol(idirac) = ipproc(irhol(idirac))
-  iptscl(idirac) = ipproc(itscl(idirac))
-enddo
-ipcfue = ipproc(iym(1))
-ipcoxy = ipproc(iym(2))
-ipcpro = ipproc(iym(3))
-ipctsc = ipproc(itsc)
-ipctem = ipproc(itemp)
 call field_get_val_s(icrom, crom)
-ipcmam = ipproc(imam)
+call field_get_val_s(itemp, cpro_temp)
+call field_get_val_s(iym(1), cpro_ym1)
+call field_get_val_s(iym(2), cpro_ym2)
+call field_get_val_s(iym(3), cpro_ym3)
+call field_get_val_s(itsc, cpro_tsc)
+call field_get_val_s(imam, cpro_mam)
+
+allocate(cpro_fmel(ndirac))
+allocate(cpro_fmal(ndirac))
+allocate(cpro_teml(ndirac))
+allocate(cpro_tscl(ndirac))
+allocate(cpro_rhol(ndirac))
+allocate(cpro_maml(ndirac))
+allocate(cpro_ampl(ndirac))
+
+do idirac = 1, ndirac
+  call field_get_val_s(iampl(idirac), cpro_ampl(idirac)%p)
+  call field_get_val_s(ifmel(idirac), cpro_fmel(idirac)%p)
+  call field_get_val_s(ifmal(idirac), cpro_fmal(idirac)%p)
+  call field_get_val_s(iteml(idirac), cpro_teml(idirac)%p)
+  call field_get_val_s(imaml(idirac), cpro_maml(idirac)%p)
+  call field_get_val_s(irhol(idirac), cpro_rhol(idirac)%p)
+  call field_get_val_s(itscl(idirac), cpro_tscl(idirac)%p)
+enddo
 
 ! ---> Initialisation
 
@@ -331,7 +335,7 @@ do iel =1, ncel
       if ( ipass.gt.1 .or.                                        &
           (isuite.eq.1.and.initro.eq.1) ) then
         rhol(idirac) = p0 * maml(idirac)                   &
-             / (rr*teml(idirac))
+             / (cs_physical_constants_r*teml(idirac))
       else
         rhol(idirac) = ro0
       endif
@@ -372,32 +376,32 @@ do iel =1, ncel
 
       sum16 = sum16 +w(idirac)
 
-! ---> Stockage des proprietes via PROPCE
+! ---> Stockage des proprietes
 
-      propce(iel,ipampl(idirac)) = d(idirac)
-      propce(iel,ipfmel(idirac)) = f(idirac)
-      propce(iel,ipfmal(idirac)) = y(idirac)
-      propce(iel,ipteml(idirac)) = teml(idirac)
-      propce(iel,ipmaml(idirac)) = maml(idirac)
-      propce(iel,iprhol(idirac)) = rhol(idirac)
-      propce(iel,iptscl(idirac)) = w(idirac)
+      cpro_ampl(idirac)%p(iel) = d(idirac)
+      cpro_fmel(idirac)%p(iel) = f(idirac)
+      cpro_fmal(idirac)%p(iel) = y(idirac)
+      cpro_teml(idirac)%p(iel) = teml(idirac)
+      cpro_maml(idirac)%p(iel) = maml(idirac)
+      cpro_rhol(idirac)%p(iel) = rhol(idirac)
+      cpro_tscl(idirac)%p(iel) = w(idirac)
 
     enddo
 
-    propce(iel,ipcmam) = sum1
-    propce(iel,ipctem) = sum2
-    temsmm             = sum3
-    propce(iel,ipcfue) = sum4
-    propce(iel,ipcoxy) = sum5
-    propce(iel,ipcpro) = sum6
-    propce(iel,ipctsc) = sum16
+    cpro_mam(iel)  = sum1
+    cpro_temp(iel) = sum2
+    temsmm         = sum3
+    cpro_ym1(iel)  = sum4
+    cpro_ym2(iel)  = sum5
+    cpro_ym3(iel)  = sum6
+    cpro_tsc(iel)  = sum16
 
 !---> Masse volumique du melange
 
     if ( ipass.gt.1 .or.                                          &
         (isuite.eq.1.and.initro.eq.1) ) then
       crom(iel) = srrom*crom(iel)               &
-                          +(1.d0-srrom)*(p0/(rr*temsmm))
+                          +(1.d0-srrom)*(p0/(cs_physical_constants_r*temsmm))
     endif
 
   else
@@ -544,7 +548,7 @@ do iel =1, ncel
       if ( ipass.gt.1 .or.                                        &
           (isuite.eq.1.and.initro.eq.1) ) then
         rhol(idirac) = p0 * maml(idirac)                   &
-             /(rr*teml(idirac))
+             /(cs_physical_constants_r*teml(idirac))
       else
         rhol(idirac) = ro0
       endif
@@ -586,15 +590,15 @@ do iel =1, ncel
 
       sum17 = sum17 + w(idirac)
 
-! ---> Stockage des proprietes via PROPCE
+! ---> Stockage des proprietes
 
-      propce(iel,ipampl(idirac)) = d(idirac)
-      propce(iel,ipfmel(idirac)) = f(idirac)
-      propce(iel,ipfmal(idirac)) = y(idirac)
-      propce(iel,ipmaml(idirac)) = maml(idirac)
-      propce(iel,ipteml(idirac)) = teml(idirac)
-      propce(iel,iprhol(idirac)) = rhol(idirac)
-      propce(iel,iptscl(idirac)) = w(idirac)
+      cpro_ampl(idirac)%p(iel) = d(idirac)
+      cpro_fmel(idirac)%p(iel) = f(idirac)
+      cpro_fmal(idirac)%p(iel) = y(idirac)
+      cpro_maml(idirac)%p(iel) = maml(idirac)
+      cpro_teml(idirac)%p(iel) = teml(idirac)
+      cpro_rhol(idirac)%p(iel) = rhol(idirac)
+      cpro_tscl(idirac)%p(iel) = w(idirac)
 
 
       if ((f(idirac).ne.zero).and.(y(idirac).ne.zero)) then
@@ -616,23 +620,27 @@ do iel =1, ncel
       endif
     enddo
 
-    propce(iel,ipcmam) = sum7
-    propce(iel,ipctem) = sum8
-    temsmm             = sum9
-    propce(iel,ipcfue) = sum10
-    propce(iel,ipcoxy) = sum11
-    propce(iel,ipcpro) = sum12
-    propce(iel,ipctsc) = sum17
+    cpro_mam(iel)  = sum7
+    cpro_temp(iel) = sum8
+    temsmm         = sum9
+    cpro_ym1(iel)  = sum10
+    cpro_ym2(iel)  = sum11
+    cpro_ym3(iel)  = sum12
+    cpro_tsc(iel)  = sum17
 
 ! ---> Masse volumique du melange
 
     if ( ipass.gt.1 .or.                                          &
         (isuite.eq.1.and.initro.eq.1) ) then
       crom(iel) = srrom * crom(iel)             &
-           + (1.d0-srrom) * (p0/(rr*temsmm))
+           + (1.d0-srrom) * (p0/(cs_physical_constants_r*temsmm))
     endif
 
   endif
 enddo
 
+deallocate(cpro_fmel, cpro_fmal, cpro_teml)
+deallocate(cpro_tscl, cpro_rhol, cpro_maml)
+deallocate(cpro_ampl)
+
 end subroutine
