@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,8 +24,7 @@ subroutine pdfpp4 &
 !================
 
  ( ncelet , ncel  ,                                               &
-   fm     , fp2m  , yfm    , yfp2m , coyfp  ,                     &
-   propce )
+   fm     , fp2m  , yfm    , yfp2m , coyfp )
 
 !===============================================================================
 ! FONCTION :
@@ -77,7 +76,6 @@ subroutine pdfpp4 &
 ! yfm              ! tr ! <-- ! moyenne de la fraction massique                !
 ! yfp2m            ! tr ! <-- ! variance de la fraction massique               !
 ! coyfp            ! tr ! <-- ! covariance                                     !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -103,6 +101,7 @@ use ppthch
 use ppincl
 use coincl
 use field
+
 !===============================================================================
 
 implicit none
@@ -115,7 +114,6 @@ integer          ncelet, ncel
 double precision fm(ncelet)   , fp2m(ncelet)
 double precision yfm(ncelet)  , yfp2m(ncelet)
 double precision coyfp(ncelet)
-double precision propce(ncelet,*)
 
 !===============================================================================
 ! Local variables
@@ -124,13 +122,6 @@ double precision propce(ncelet,*)
 integer          iel, igg, idirac
 integer          mode
 
-integer          ipctem, ipcmam
-integer          ipampl(ndracm), ipfmel(ndracm)
-integer          ipfmal(ndracm), ipteml(ndracm)
-integer          ipmaml(ndracm)
-integer          iprhol(ndracm)
-integer          iptscl(ndracm)
-integer          ipcfue, ipcoxy, ipcpro, ipctsc
 !      INTEGER          IPCKAB, IPT4 , IPT3
 
 double precision f(ndracm), y(ndracm), d(ndracm)
@@ -176,6 +167,12 @@ double precision mcy2p1, mcy2p3
 double precision mcy2p2, mcy2p4
 double precision my2p1 , my2p3, my2p2, my2p4
 double precision, dimension(:), pointer ::  crom
+double precision, dimension(:), pointer ::  cpro_temp, cpro_tsc, cpro_mam
+double precision, dimension(:), pointer ::  cpro_ym1, cpro_ym2, cpro_ym3
+type(pmapper_double_r1), dimension(:), pointer :: cpro_fmel, cpro_fmal, cpro_teml
+type(pmapper_double_r1), dimension(:), pointer :: cpro_tscl, cpro_rhol, cpro_maml
+type(pmapper_double_r1), dimension(:), pointer :: cpro_ampl
+
 integer ipass
 data    ipass /0/
 save    ipass
@@ -186,29 +183,37 @@ save    ipass
 !     0. POSITION ET INITIALISATION DES VARIABLES
 !===============================================================================
 
+call field_get_val_s(icrom, crom)
+call field_get_val_s(itemp, cpro_temp)
+call field_get_val_s(iym(1), cpro_ym1)
+call field_get_val_s(iym(2), cpro_ym2)
+call field_get_val_s(iym(3), cpro_ym3)
+call field_get_val_s(itsc, cpro_tsc)
+call field_get_val_s(imam, cpro_mam)
+
+allocate(cpro_fmel(ndirac))
+allocate(cpro_fmal(ndirac))
+allocate(cpro_teml(ndirac))
+allocate(cpro_tscl(ndirac))
+allocate(cpro_rhol(ndirac))
+allocate(cpro_maml(ndirac))
+allocate(cpro_ampl(ndirac))
+
 do idirac = 1, ndirac
-  ipampl(idirac) = ipproc(iampl(idirac))
-  ipfmel(idirac) = ipproc(ifmel(idirac))
-  ipfmal(idirac) = ipproc(ifmal(idirac))
-  ipteml(idirac) = ipproc(iteml(idirac))
-  ipmaml(idirac) = ipproc(imaml(idirac))
-  iprhol(idirac) = ipproc(irhol(idirac))
-  iptscl(idirac) = ipproc(itscl(idirac))
+  call field_get_val_s(iampl(idirac), cpro_ampl(idirac)%p)
+  call field_get_val_s(ifmel(idirac), cpro_fmel(idirac)%p)
+  call field_get_val_s(ifmal(idirac), cpro_fmal(idirac)%p)
+  call field_get_val_s(iteml(idirac), cpro_teml(idirac)%p)
+  call field_get_val_s(imaml(idirac), cpro_maml(idirac)%p)
+  call field_get_val_s(irhol(idirac), cpro_rhol(idirac)%p)
+  call field_get_val_s(itscl(idirac), cpro_tscl(idirac)%p)
 enddo
 
-ipcfue = ipproc(iym(1))
-ipcoxy = ipproc(iym(2))
-ipcpro = ipproc(iym(3))
-ipctsc = ipproc(itsc)
-ipctem = ipproc(itemp)
-call field_get_val_s(icrom, crom)
-ipcmam = ipproc(imam)
-
-!      IF ( IIRAYO.GT.0 ) THEN
-!        IPCKAB = IPPROC(ICKABS)
-!        IPT4   = IPPROC(IT4M)
-!        IPT3   = IPPROC(IT3M)
-!      ENDIF
+!if ( iirayo.gt.0 ) then
+!  call field_get_val_s(ickabs, cpro_ckabs)
+!  call field_get_val_s(it4m, cpro_t4m)
+!  call field_get_val_s(it3m, cpro_t3m)
+!endif
 
 ! ---> Initialisation des variables et compteurs
 
@@ -352,7 +357,7 @@ coyfpp=coyfp(iel)
       if ( ipass.gt.1 .or.                                        &
           (isuite.eq.1.and.initro.eq.1) ) then
         rhol(idirac) = p0 * maml(idirac)                   &
-             / (rr*teml(idirac))
+             / (cs_physical_constants_r*teml(idirac))
       else
         rhol(idirac) = ro0
       endif
@@ -390,33 +395,33 @@ coyfpp=coyfp(iel)
       sum15 = sum15 +rhol(idirac)*d(idirac)
       sum16 = sum16 +w(idirac)
 
-! ---> Stockage des proprietes via PROPCE
+! ---> Stockage des proprietes
 
-      propce(iel,ipampl(idirac)) = d(idirac)
-      propce(iel,ipfmel(idirac)) = f(idirac)
-      propce(iel,ipfmal(idirac)) = y(idirac)
-      propce(iel,ipteml(idirac)) = teml(idirac)
-      propce(iel,ipmaml(idirac)) = maml(idirac)
-      propce(iel,iprhol(idirac)) = rhol(idirac)
-      propce(iel,iptscl(idirac)) = w(idirac)
+      cpro_ampl(idirac)%p(iel) = d(idirac)
+      cpro_fmel(idirac)%p(iel) = f(idirac)
+      cpro_fmal(idirac)%p(iel) = y(idirac)
+      cpro_teml(idirac)%p(iel) = teml(idirac)
+      cpro_maml(idirac)%p(iel) = maml(idirac)
+      cpro_rhol(idirac)%p(iel) = rhol(idirac)
+      cpro_tscl(idirac)%p(iel) = w(idirac)
 
 !fin de boucle sur chaque DIRAC
     enddo
 
-    propce(iel,ipcmam) = sum1
-    propce(iel,ipctem) = sum2
-    temsmm             = sum3
-    propce(iel,ipcfue) = sum4
-    propce(iel,ipcoxy) = sum5
-    propce(iel,ipcpro) = sum6
-    propce(iel,ipctsc) = sum16
+    cpro_mam(iel)  = sum1
+    cpro_temp(iel) = sum2
+    temsmm         = sum3
+    cpro_ym1(iel)  = sum4
+    cpro_ym2(iel)  = sum5
+    cpro_ym3(iel)  = sum6
+    cpro_tsc(iel)  = sum16
 
 ! ---> Masse volumique du melange
 
     if ( ipass.gt.1.or.                                           &
         (isuite.eq.1.and.initro.eq.1) ) then
       crom(iel) = srrom*crom(iel)               &
-           +(1.d0-srrom)*(p0/(rr*temsmm))
+           +(1.d0-srrom)*(p0/(cs_physical_constants_r*temsmm))
     endif
 
   else
@@ -740,7 +745,7 @@ coyfpp=coyfp(iel)
     if ( ipass.gt.1.or.                                           &
         (isuite.eq.1.and.initro.eq.1)) then
       rhol(idirac) = p0 * maml(idirac)                     &
-           /(rr*teml(idirac))
+           /(cs_physical_constants_r*teml(idirac))
     else
       rhol(idirac) = ro0
     endif
@@ -790,42 +795,41 @@ coyfpp=coyfp(iel)
 
     sum17 = sum17 + w(idirac)
 
-! ---> Stockage des proprietes via PROPCE
+! ---> Stockage des proprietes
 
-    propce(iel,ipampl(idirac)) = d(idirac)
-    propce(iel,ipfmel(idirac)) = f(idirac)
-    propce(iel,ipfmal(idirac)) = y(idirac)
-    propce(iel,ipmaml(idirac)) = maml(idirac)
-    propce(iel,ipteml(idirac)) = teml(idirac)
-    propce(iel,iprhol(idirac)) = rhol(idirac)
-    propce(iel,iptscl(idirac)) = w(idirac)
+    cpro_ampl(idirac)%p(iel) = d(idirac)
+    cpro_fmel(idirac)%p(iel) = f(idirac)
+    cpro_fmal(idirac)%p(iel) = y(idirac)
+    cpro_maml(idirac)%p(iel) = maml(idirac)
+    cpro_teml(idirac)%p(iel) = teml(idirac)
+    cpro_rhol(idirac)%p(iel) = rhol(idirac)
+    cpro_tscl(idirac)%p(iel) = w(idirac)
 
 ! ---> Grandeurs relatives au rayonnement
 
-!           IF ( IIRAYO.GT.0 ) THEN
-!             PROPCE(IEL,IPCKAB) = YGFM*KABSGF + YGBM*KABSGB
-!             PROPCE(IEL,IPT4)   = YGFM*TGF**3+YGBM*TGB**3
-!             PROPCE(IEL,IPT3)   = YGFM*TGF**3+YGBM*TGB**3
-!           ENDIF
+!           if ( iirayo.gt.0 ) then
+!             cpro_ckab(iel) = ygfm*kabsgf + ygbm*kabsgb
+!             cpro_t4m(iel)  = ygfm*tgf**4+ygbm*tgb**4
+!             cpro_t3m(iel)  = ygfm*tgf**3+ygbm*tgb**3
+!           endif
 
 ! fin de boucle sur chaque DIRAC
   enddo
 
-
-  propce(iel,ipcmam) = sum7
-  propce(iel,ipctem) = sum8
-  temsmm             = sum9
-  propce(iel,ipcfue) = sum10
-  propce(iel,ipcoxy) = sum11
-  propce(iel,ipcpro) = sum12
-  propce(iel,ipctsc) = sum17
+  cpro_mam(iel)  = sum7
+  cpro_temp(iel) = sum8
+  temsmm         = sum9
+  cpro_ym1(iel)  = sum10
+  cpro_ym2(iel)  = sum11
+  cpro_ym3(iel)  = sum12
+  cpro_tsc(iel)  = sum17
 
 ! ---> Masse volumique du melange
 
   if ( ipass.gt.1 .or.                                            &
       (isuite.eq.1.and.initro.eq.1) ) then
     crom(iel) = srrom * crom(iel)               &
-         + (1.d0-srrom) * (p0/(rr*temsmm))
+         + (1.d0-srrom) * (p0/(cs_physical_constants_r*temsmm))
   endif
 
 ! de passage ou non par la PDF
@@ -834,6 +838,10 @@ endif
 ! fin de boucle sur les cellules
 enddo
 
+deallocate(cpro_fmel, cpro_fmal, cpro_teml)
+deallocate(cpro_tscl, cpro_rhol, cpro_maml)
+deallocate(cpro_ampl)
+
 ! ---> impression clipping
 
 if(irangp.ge.0) then
