@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,37 +19,35 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
+!===============================================================================
+! Function :
+! --------
+!> \file cs_coal_htconvers1.f90
+!>
+!> \brief - Calculation of gas temperature
+!>          Function with gas enthalpy and concentrations
+!>          if mode = 1
+!>        - Calculation of gas enthalpy
+!>          Function with gas temperature and concentrations
+!>          if mode = -1
+!-------------------------------------------------------------------------------
+
+!-------------------------------------------------------------------------------
+! Arguments
+!______________________________________________________________________________.
+!  mode           name          role
+!______________________________________________________________________________!
+!> \param[in]     mode          -1 : t -> h  ;   1 : h -> t
+!> \param[in,out] eh            gas enthalpy
+!>                                   (\f$ j . kg^{-1} \f$ of mixed gas)
+!> \param[in]     xesp          mass fraction of species
+!> \param[in]     f1mc          average f1
+!> \param[in]     f2mc          average f2
+!> \param[in,out] tp            gas temperature (in kelvin)
+!______________________________________________________________________________!
 
 subroutine cs_coal_htconvers1 &
-!============================
  ( mode , eh , xesp , f1mc , f2mc , tp )
-!===============================================================================
-! FONCTION :
-! --------
-! CALCUL DE LA TEMPERATURE DU GAZ
-!  EN FONCTION DE L'ENTHALPIE DU GAZ ET DES CONCENTRATIONS
-!  SI MODE = 1
-! CALCUL DE L'ENTHALPIE DU GAZ
-!  EN FONCTION DE LA TEMPERATURE DU GAZ ET DES CONCENTRATIONS
-!  SI MODE = -1
-
-! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! eh               ! tr ! <-- ! enthalpie du gaz                               !
-!                  !    !     ! (j/kg de melange gazeux)                       !
-! xesp             ! tr ! <-- ! fraction massique des especes                  !
-! f1mc             ! tr ! <-- ! f1 moyen                                       !
-! f2mc             ! tr ! <-- ! f2 moyen                                       !
-! tp               ! tr ! --> ! temperature du gaz (kelvin)                    !
-!__________________!____!_____!________________________________________________!
-
-!     TYPE : E (ENTIER), R (REEL), A (ALPHAMNUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
 
 !==============================================================================
 ! Module files
@@ -78,7 +76,6 @@ integer          mode
 double precision eh,tp
 double precision xesp(ngazem)
 double precision f1mc(ncharm),f2mc(ncharm)
-
 ! Local variables
 
 integer          i , icha
@@ -88,19 +85,21 @@ double precision den1   , den2
 double precision eh0 , eh1
 
 !===============================================================================
-! 1. CALCUL DE LA TEMPERATURE A PARTIR DE l'ENTHALPIE
+! 1. Calculation of temperature from enthalpy
 !===============================================================================
 
-if ( mode .eq. 1 ) then
+if (mode .eq. 1) then
 
   i = npo
 
-! --- Calcul de l'enthalpie de l'espece gazeuse CHx1m
-!                                            et CHx2m a TH(NPO)
+  ! Calculation of enthalpy of the gaseous species CHx1m
+  !                                            and CHx2m at TH(NPO)
+
   ehchx1 = zero
   ehchx2 = zero
   ychx10 = zero
   ychx20 = zero
+
   do icha = 1, ncharb
     den1   = 1.d0                                                 &
          / ( a1(icha)*wmole(ichx1c(icha))                         &
@@ -127,19 +126,19 @@ if ( mode .eq. 1 ) then
          ( ehgaze(ichx2c(icha),i)*                                &
          f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
   enddo
-  if ( ychx10.gt.epzero ) then
+
+  if (ychx10.gt.epzero) then
     ehchx1 = ehchx1 / ychx10
   else
     ehchx1 = ehgaze(ichx1,i)
   endif
-  if ( ychx20.gt.epzero ) then
+  if (ychx20.gt.epzero) then
     ehchx2 = ehchx2 / ychx20
   else
     ehchx2 = ehgaze(ichx2,i)
   endif
 
-! --- Clipping eventuel de TP a TH(NPO) si EH > EH1
-
+! --- Eventual clipping of temperature at TH(NPO) if EH > EH1
   eh1 = xesp(ichx1)*ehchx1                                         &
        + xesp(ichx2)*ehchx2                                        &
        + xesp(ico  )*ehgaze(ico  ,i)                               &
@@ -153,58 +152,54 @@ if ( mode .eq. 1 ) then
        + xesp(inh3 )*ehgaze(inh3 ,i)                               &
        + xesp(in2  )*ehgaze(in2  ,i)
 
-  if ( eh .ge. eh1 ) then
+  if (eh .ge. eh1) then
     tp = th(i)
     goto 501
   endif
 
   i = 1
 
-! --- Calcul de l'enthalpie de l'espece gazeuse CHx1m
-!                                            et CHx2m a TH(1)
+  ! Calculation of enthalpy of the gaseous species CHx1m
+  !                                             and CHx2m at TH(1)
   ehchx1 = zero
   ehchx2 = zero
   ychx10 = zero
   ychx20 = zero
+
   do icha = 1, ncharb
-    den1   = 1.d0                                                 &
-         / ( a1(icha)*wmole(ichx1c(icha))                         &
-            +b1(icha)*wmole(ico )                                 &
-            +c1(icha)*wmole(ih2o)                                 &
-            +d1(icha)*wmole(ih2s)                                 &
-            +e1(icha)*wmole(ihcn)                                 &
+    den1   = 1.d0                                                      &
+         / ( a1(icha)*wmole(ichx1c(icha))                              &
+            +b1(icha)*wmole(ico )                                      &
+            +c1(icha)*wmole(ih2o)                                      &
+            +d1(icha)*wmole(ih2s)                                      &
+            +e1(icha)*wmole(ihcn)                                      &
             +f1(icha)*wmole(inh3) )
-    ychx10 = ychx10 + den1 *                                      &
-     ( f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
-    ehchx1 = ehchx1 + den1 *                                      &
-     ( ehgaze(ichx1c(icha),i)*                                    &
-       f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
-    den2   = 1.d0                                                 &
+    ychx10 = ychx10 + den1*(f1mc(icha)*a1(icha)*wmole(ichx1c(icha)))
+    ehchx1 = ehchx1 + den1*(   ehgaze(ichx1c(icha),i)                  &
+                            * f1mc(icha)*a1(icha)*wmole(ichx1c(icha)))
+    den2   = 1.d0                                                      &
          / ( a2(icha)*wmole(ichx2c(icha))                         &
             +b2(icha)*wmole(ico )                                 &
             +c2(icha)*wmole(ih2o)                                 &
             +d2(icha)*wmole(ih2s)                                 &
             +e2(icha)*wmole(ihcn)                                 &
             +f2(icha)*wmole(inh3) )
-    ychx20 = ychx20 + den2 *                                      &
-     ( f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
-    ehchx2 = ehchx2 + den2 *                                      &
-     ( ehgaze(ichx2c(icha),i)*                                    &
-       f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
+    ychx20 = ychx20 + den2*(f2mc(icha)*a2(icha)*wmole(ichx2c(icha)))
+    ehchx2 = ehchx2 + den2*(  ehgaze(ichx2c(icha),i)                   &
+                            * f2mc(icha)*a2(icha)*wmole(ichx2c(icha)))
   enddo
-  if ( ychx10.gt.epzero ) then
+  if (ychx10.gt.epzero) then
     ehchx1 = ehchx1 / ychx10
   else
     ehchx1 = ehgaze(ichx1,i)
   endif
-  if ( ychx20.gt.epzero ) then
+  if (ychx20.gt.epzero) then
     ehchx2 = ehchx2 / ychx20
   else
     ehchx2 = ehgaze(ichx2,i)
   endif
 
-! --- Clipping eventuel de TP a TH(1) si EH < EH0
-
+  ! --- Eventual clipping of temperature at TH(1) if EH < EH0
   eh0 = xesp(ichx1)*ehchx1                                        &
       + xesp(ichx2)*ehchx2                                        &
       + xesp(ico  )*ehgaze(ico  ,i)                               &
@@ -223,52 +218,50 @@ if ( mode .eq. 1 ) then
     goto 501
   endif
 
-
  500    continue
   i = i + 1
 
-! --- Calcul de l'enthalpie de l'espece gazeuse CHx1m
-!                                            et CHx2m pour TH(I-1)
+! --- Calculation of enthalpy of the gaseous species CHx1m
+!                                            and CHx2m for TH(I-1)
   ehchx1 = zero
   ehchx2 = zero
   ychx10 = zero
   ychx20 = zero
+
   do icha = 1, ncharb
-    den1   = 1.d0                                                 &
-         / ( a1(icha)*wmole(ichx1c(icha))                         &
-            +b1(icha)*wmole(ico )                                 &
-            +c1(icha)*wmole(ih2o)                                 &
-            +d1(icha)*wmole(ih2s)                                 &
-            +e1(icha)*wmole(ihcn)                                 &
+    den1   = 1.d0                                                       &
+         / ( a1(icha)*wmole(ichx1c(icha))                               &
+            +b1(icha)*wmole(ico )                                       &
+            +c1(icha)*wmole(ih2o)                                       &
+            +d1(icha)*wmole(ih2s)                                       &
+            +e1(icha)*wmole(ihcn)                                       &
             +f1(icha)*wmole(inh3) )
-    ychx10 = ychx10 + den1 *                                      &
-      ( f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
-    ehchx1 = ehchx1 + den1 *                                      &
-      ( ehgaze(ichx1c(icha),i-1)*                                 &
-        f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
-    den2   = 1.d0                                                 &
-          / ( a2(icha)*wmole(ichx2c(icha))                         &
-             +b2(icha)*wmole(ico )                                 &
-             +c2(icha)*wmole(ih2o)                                 &
-             +d2(icha)*wmole(ih2s)                                 &
-             +e2(icha)*wmole(ihcn)                                 &
+    ychx10 = ychx10 + den1*(f1mc(icha)*a1(icha)*wmole(ichx1c(icha)))
+    ehchx1 = ehchx1 + den1*(   ehgaze(ichx1c(icha),i-1)                 &
+                            * f1mc(icha)*a1(icha)*wmole(ichx1c(icha)))
+    den2   = 1.d0                                                       &
+          / ( a2(icha)*wmole(ichx2c(icha))                              &
+             +b2(icha)*wmole(ico )                                      &
+             +c2(icha)*wmole(ih2o)                                      &
+             +d2(icha)*wmole(ih2s)                                      &
+             +e2(icha)*wmole(ihcn)                                      &
              +f2(icha)*wmole(inh3) )
-    ychx20 = ychx20 + den2 *                                      &
-       ( f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
-    ehchx2 = ehchx2 + den2 *                                      &
-       ( ehgaze(ichx2c(icha),i-1)*                                &
-         f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
+    ychx20 = ychx20 + den2*(f2mc(icha)*a2(icha)*wmole(ichx2c(icha)))
+    ehchx2 = ehchx2 + den2*(  ehgaze(ichx2c(icha),i-1)                  &
+                            * f2mc(icha)*a2(icha)*wmole(ichx2c(icha)))
   enddo
-  if ( ychx10.gt.epzero ) then
+
+  if (ychx10.gt.epzero) then
     ehchx1 = ehchx1 / ychx10
   else
     ehchx1 = ehgaze(ichx1,i-1)
   endif
-  if ( ychx20.gt.epzero ) then
+  if (ychx20.gt.epzero) then
     ehchx2 = ehchx2 / ychx20
   else
     ehchx2 = ehgaze(ichx2,i-1)
   endif
+
   eh0 = xesp(ichx1)*ehchx1                                          &
       + xesp(ichx2)*ehchx2                                          &
       + xesp(ico  )*ehgaze(ico  ,i-1)                               &
@@ -282,8 +275,9 @@ if ( mode .eq. 1 ) then
       + xesp(inh3 )*ehgaze(inh3 ,i-1)                               &
       + xesp(in2  )*ehgaze(in2  ,i-1)
 
-! --- Calcul de l'enthalpie de l'espece gazeuse CHx1m
-!                                            et CHx2m pour TH(I)
+  ! Calculation of the enthalpy of the gaseous species CHx1m
+  !                                                and CHx2m at TH(I)
+
   ehchx1 = zero
   ehchx2 = zero
   ychx10 = zero
@@ -344,18 +338,17 @@ if ( mode .eq. 1 ) then
     goto 501
   endif
   goto 500
+
  501    continue
 
 !===============================================================================
-! 1. CALCUL DE L'ENTHALPIE A PARTIR DE LA TEMPERATURE
+! 2. Calculation of the enthalpy from temperature
 !===============================================================================
 
 else if ( mode .eq. -1 ) then
 
   i = npo
-
-! --- Calcul en Max
-
+  ! --- Calculation at Max
   if ( tp .ge. th(i) ) then
     ehchx1 = zero
     ehchx2 = zero
@@ -411,9 +404,7 @@ else if ( mode .eq. -1 ) then
        + xesp(in2  )*ehgaze(in2  ,i)
     goto 601
   endif
-
-! Clipping en Min
-
+  ! Clipping at Min
   i = 1
   if ( tp .le. th(i) ) then
     ehchx1 = zero
@@ -456,9 +447,7 @@ else if ( mode .eq. -1 ) then
     else
       ehchx2 = ehgaze(ichx2,i)
     endif
-
-! --- Clipping eventuel de TP a TH(1) si EH < EH0
-
+    ! --- Eventual clipping of temperature at TH(1) if EH < EH0
     eh = xesp(ichx1)*ehchx1                                       &
        + xesp(ichx2)*ehchx2                                       &
        + xesp(ico  )*ehgaze(ico  ,i)                               &
@@ -473,16 +462,13 @@ else if ( mode .eq. -1 ) then
        + xesp(in2  )*ehgaze(in2  ,i)
     goto 601
   endif
-
-! Interpolation dans la table
-
+ ! Interpolation in the table
   i = 1
  600    continue
 
   i = i + 1
   if ( tp .le. th(i) ) then
-
-! --- Calcul de l'enthalpie de l'espece gazeuse TH(I-1)
+    ! --- Calculation of the enthalpy of the gaseous species TH(I-1)
     ehchx1 = zero
     ehchx2 = zero
     ychx10 = zero
@@ -536,9 +522,7 @@ else if ( mode .eq. -1 ) then
         + xesp(iso2 )*ehgaze(iso2 ,i-1)                           &
         + xesp(inh3 )*ehgaze(inh3 ,i-1)                           &
         + xesp(in2  )*ehgaze(in2  ,i-1)
-
-! --- Calcul de l'enthalpie de l'espece gazeuse TH(I)
-
+    ! --- Calculation of the enthalpy of the gaseous species TH(I)
     ehchx1 = zero
     ehchx2 = zero
     ychx10 = zero
@@ -592,7 +576,7 @@ else if ( mode .eq. -1 ) then
         + xesp(iso2 )*ehgaze(iso2 ,i)                            &
         + xesp(inh3 )*ehgaze(inh3 ,i)                            &
         + xesp(in2  )*ehgaze(in2  ,i)
-!
+
     eh =  eh0                                                    &
            + (eh1-eh0)*(tp-th(i-1))/(th(i)-th(i-1))
     goto 601
@@ -609,22 +593,20 @@ endif
 !--------
 ! Formats
 !--------
-
  1000 format(                                                     &
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/,&
-'@ @@ ATTENTION : ERREUR DANS CPTHP1                          ',/,&
+'@ @@ WARNING: Error in cpthp1                                ',/,&
 '@    =========                                               ',/,&
-'@    VALEUR INCORRECTE DE L''ARGUMENT MODE                   ',/,&
-'@    CE DOIT ETRE UN ENTIER EGAL A 1 OU -1                   ',/,&
-'@    IL VAUT ICI ',I10                                        ,/,&
+'@    Incorrect value of argument mode                        ',/,&
+'@    it must be an integer equal to 1 or -1                  ',/,&
+'@    it worths here ',I10                                     ,/,&
 '@                                                            ',/,&
-'@  Le calcul ne peut etre execute.                           ',/,&
+'@  The calculation can not run.                              ',/,&
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
-
 !----
 ! End
 !----
