@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -33,14 +33,13 @@ module cs_coal_incl
 
   ! Combustion du coke par H2O
 
-  integer, save :: ihth2o , ighh2o(nclcpm)
+  integer, save :: ihth2o , ighh2o(nclcpm), ipci(ncharm)
 
   ! Modele de NOx
   ! qpr : % d'azote libere pendant la devol.% de MV libere pendant la devol.
   ! fn : concentration en azote sur pur
 
-  double precision, save :: qpr(ncharm), fn(ncharm), ipci(ncharm),             &
-                            xashsec(ncharm)
+  double precision, save :: qpr(ncharm), fn(ncharm), xashsec(ncharm)
 
   ! Nouveau modele NOx
   ! ==================
