@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -64,6 +64,7 @@ use ihmpre
 use ppcpfu
 use cs_coal_incl
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -72,7 +73,9 @@ implicit none
 integer          ii , jj , iok
 integer          icha , isc
 
-double precision wmolme
+double precision wmolme, turb_schmidt
+
+type(var_cal_opt) :: vcopt
 
 !===============================================================================
 ! 1. VARIABLES TRANSPORTEES
@@ -112,51 +115,47 @@ do isc = 1, nscapp
 
 ! ------ Schmidt ou Prandtl turbulent
 
-  sigmas(jj) = 0.7d0
+  turb_schmidt = 0.7d0
+  call field_set_key_double(ivarfl(isca(jj)), ksigmas, turb_schmidt)
 
 ! ------ Coeff dissipation des fluctuations
 
   rvarfl(jj) = 0.8d0
 
-  ii = isca(iscapp(isc))
-
-! ------ Niveau de detail des impressions pour les variables et
-!          donc les scalaires (valeurs 0 ou 1)
-!          Si = -10000 non modifie par l'utilisateur -> niveau 1
-  if (iwarni(ii).eq.-10000) then
-    iwarni(ii) = 1
-  endif
-
 !   - Interface Code_Saturne:
 !     ======================
 
-!   ihisvr is already filled in UINUM1 routine
-
   if (iihmpr.ne.1) then
 
-! ------ Informations relatives a la resolution des scalaires
+    ii = isca(iscapp(isc))
 
-!         - Facteur multiplicatif du pas de temps
-  cdtvar(ii) = 1.d0
+    call field_get_key_struct_var_cal_opt(ivarfl(ii), vcopt)
+
+    ! ------ Informations relatives a la resolution des scalaires
+
+    ! - Facteur multiplicatif du pas de temps
+    cdtvar(ii) = 1.d0
 
 !         - Schema convectif % schema 2ieme ordre
 !           = 0 : upwind
 !           = 1 : second ordre
-  blencv(ii) = 0.d0
+    vcopt%blencv = 0.d0
 
 !         - Type de schema convetif second ordre (utile si BLENCV > 0)
 !           = 0 : Second Order Linear Upwind
 !           = 1 : Centre
-  ischcv(ii) = 1
+    vcopt%ischcv = 1
 
 !         - Test de pente pour basculer d'un schema centre vers l'upwind
-!           = 0 : utlisation automatique du test de pente
+!           = 0 : utilisation automatique du test de pente
 !           = 1 : calcul sans test de pente
-  isstpc(ii) = 0
+    vcopt%isstpc = 0
 
 !         - Reconstruction des flux de convetion et de diffusion aux faces
 !           = 0 : pas de reconstruction
-  ircflu(ii) = 0
+    vcopt%ircflu = 0
+
+    call field_set_key_struct_var_cal_opt(ivarfl(ii), vcopt)
 
   endif
 
@@ -177,7 +176,7 @@ wmolme = ( wmole(io2) *oxyo2(1) +wmole(in2) *oxyn2(1)             &
           +wmole(ih2o)*oxyh2o(1)+wmole(ico2)*oxyco2(1))           &
         /(oxyo2(1)+oxyn2(1)+oxyh2o(1)+oxyco2(1))
 
-ro0 = p0*wmolme / (rr*t0)
+ro0 = p0*wmolme / (cs_physical_constants_r*t0)
 
 ! ---- Initialisation pour la masse volumique du coke
 
@@ -186,7 +185,7 @@ do icha = 1, ncharb
 enddo
 
 ! On met SRROM et DIFTL0 a -GRAND pour forcer l'utilisateur a les
-! definir dans user_coal_ini1
+! definir dans cs_user_combustion
 ! ---> Coefficient de relaxation de la masse volumique
 !      RHO(n+1) = SRROM * RHO(n) + (1-SRROM) * RHO(n+1)
 srrom = -grand
@@ -219,8 +218,7 @@ if (iihmpr.eq.1) then
 
 endif
 
-call user_coal_ini1
-!==================
+call cs_user_combustion
 
 !===============================================================================
 ! 4. VERIFICATION DES DONNERS FOURNIES PAR L'UTLISATEUR
@@ -245,7 +243,7 @@ endif
  9998 format(                                                     &
 '                                                             ',/,&
 ' Pas d erreur detectee lors de la verification des donnees   ',/,&
-'                                            (user_coal_ini1).',/)
+'                                        (cs_user_combustion).',/)
  9999 format(                                                     &
 '@                                                            ',/,&
 '@                                                            ',/,&
@@ -260,7 +258,7 @@ endif
 '@                                                            ',/,&
 '@  Se reporter aux impressions precedentes pour plus de      ',/,&
 '@    renseignements.                                         ',/,&
-'@  Verifier user_coal_ini1.                                  ',/,&
+'@  Verifier cs_user_combustion.'                              ,/,&
 '@                                                            ',/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@                                                            ',/)
