@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,46 +20,31 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine cs_coal_thfieldconv1 &
-!==============================
- ( ncelet , ncel   ,                                              &
-   eh     , x2     , rtp    ,                                     &
-   fuel1  , fuel2  , fuel3  , fuel4 , fuel5 , fuel6 , fuel7 ,     &
-   oxyd   , prod1  , prod2  , prod3 , xiner ,                     &
-   tp     )
 !===============================================================================
-! FONCTION :
+! Function:
 ! --------
-! CALCUL DE LA TEMPERATURE DU GAZ
-!  EN FONCTION DE L'ENTHALPIE DU GAZ ET DES CONCENTRATIONS
+!> \file cs_coal_thfieldconv1.f90
+!>
+!> \brief Calculation of the gas temperature
+!>        Function with gas enthalpy and concentrations
+!-------------------------------------------------------------------------------
+
+!-------------------------------------------------------------------------------
+
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! ncelet           ! i  ! <-- ! number of extended (real + ghost) cells        !
-! ncel             ! i  ! <-- ! number of cells                                !
-! ntbmci           ! e  ! <-- ! taille du macro tableau mc entiers             !
-! ntbmcr           ! e  ! <-- ! taille du macro tableau mc reels               !
-! eh               ! tr ! <-- ! enthalpie du gaz                               !
-!                  !    !     ! (j/kg de melange gazeux)                       !
-! fuel1            ! tr ! <-- ! fraction massique chx1                         !
-! fuel2            ! tr ! <-- ! fraction massique chx2                         !
-! fuel3            ! tr ! <-- ! fraction massique co                           !
-! oxyd             ! tr ! <-- ! fraction massique o2                           !
-! prod1            ! tr ! <-- ! fraction massique co2                          !
-! prod2            ! tr ! <-- ! fraction massique h2o                          !
-! xiner            ! tr ! <-- ! fraction massique n2                           !
-! tp               ! tr ! --> ! temperature du gaz (kelvin)                    !
-! tbmci            ! tr ! <-- ! macro tableau entier mc travail                !
-! tbmcr            ! tr ! <-- ! macro tableau reel   mc travail                !
-! eh0              ! tr ! <-- ! tableau reel de travail                        !
-! eh1              ! tr ! <-- ! tableau reel de travail                        !
-!__________________!____!_____!________________________________________________!
-!     TYPE : E (ENTIER), R (REEL), A (ALPHAMNUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
+!______________________________________________________________________________.
+!  mode           name          role
+!______________________________________________________________________________!
+!> \param[in]     location_id     mesh location id (cells or boundary faces)
+!> \param[in]     eh              gas enthalpy
+!>                                (j/kg of gaseous mixture)
+!> \param[in,out] tp              gas temperature in kelvin
+!______________________________________________________________________________!
+
+subroutine cs_coal_thfieldconv1 &
+ ( location_id     ,                                              &
+   eh     ,                                                       &
+   tp     )
 
 !==============================================================================
 ! Module files
@@ -68,7 +53,6 @@ subroutine cs_coal_thfieldconv1 &
 use paramx
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use entsor
@@ -78,6 +62,10 @@ use coincl
 use cpincl
 use ppincl
 use ppcpfu
+use mesh
+use field
+use cs_c_bindings
+use pointe
 
 !===============================================================================
 
@@ -85,57 +73,81 @@ implicit none
 
 ! Arguments
 
-integer          ncelet , ncel
+integer          location_id
 
-double precision eh(ncelet)   , x2(ncelet)   , rtp(ncelet,nflown:nvar)
-double precision fuel1(ncelet), fuel2(ncelet), fuel3(ncelet)
-double precision fuel4(ncelet), fuel5(ncelet), fuel6(ncelet) , fuel7(ncelet)
-double precision oxyd(ncelet), xiner(ncelet)
-double precision prod1(ncelet),prod2(ncelet),prod3(ncelet)
-double precision tp(ncelet)
+double precision eh(*)
+double precision tp(*)
 
 ! Local variables
 
-integer          i, icel, icha
+integer          i, iel, ielt, nelt, icha
 
 double precision ychx10 , ychx20 , ehchx1 , ehchx2
 double precision den1   , den2 , eh0 , eh1
+double precision f1mc(ncharm), f2mc(ncharm)
 
-integer          iok
-double precision , dimension ( : , : )     , allocatable :: f1mc,f2mc
+double precision, dimension(:), pointer :: x1
+double precision, dimension(:), pointer :: fuel1, fuel2, fuel3, fuel4, fuel5
+double precision, dimension(:), pointer :: fuel6, fuel7, oxyd
+double precision, dimension(:), pointer :: prod1, prod2, prod3 , xiner
+type(pmapper_double_r1), dimension(:), allocatable :: cvar_f1m, cvar_f2m
 
 !===============================================================================
-!
-!===============================================================================
-! Deallocation dynamic arrays
-!----
-allocate(f1mc(1:ncel,1:ncharb),f2mc(1:ncel,1:ncharb),stat=iok)
-!----
-if ( iok > 0  ) then
-  write(nfecra,*) ' Memory allocation error inside: '
-  write(nfecra,*) '    cs_coal_thfieldconv1         '
+
+call field_get_val_s(iym1(ichx1), fuel1)
+call field_get_val_s(iym1(ichx2), fuel2)
+call field_get_val_s(iym1(ico  ), fuel3)
+call field_get_val_s(iym1(ih2s ), fuel4)
+call field_get_val_s(iym1(ihy  ), fuel5)
+call field_get_val_s(iym1(ihcn ), fuel6)
+call field_get_val_s(iym1(inh3 ), fuel7)
+call field_get_val_s(iym1(io2  ), oxyd)
+call field_get_val_s(iym1(ico2 ), prod1)
+call field_get_val_s(iym1(ih2o ), prod2)
+call field_get_val_s(iym1(iso2 ), prod3)
+call field_get_val_s(iym1(in2  ), xiner)
+
+! Massic fraction of gas
+call field_get_val_s_by_name("x_c", x1)
+
+if (location_id .eq. MESH_LOCATION_CELLS) then
+  nelt = ncel
+else if (location_id .eq. MESH_LOCATION_BOUNDARY_FACES) then
+  nelt = nfabor
+else
   call csexit(1)
 endif
-!===============================================================================
-!
-do icel = 1, ncel
-  do icha = 1, ncharb
-    f1mc(icel,icha) = rtp(icel,isca(if1m(icha))) /(1.d0-x2(icel))
-    f2mc(icel,icha) = rtp(icel,isca(if2m(icha))) /(1.d0-x2(icel))
-  enddo
-enddo
-!
-i = npo-1
-do icel = 1, ncel
 
-! --- Calcul de l'enthalpie de l'espece gazeuse CHx1m
-!                                            et CHx2m a TH(NPO)
+!===============================================================================
+
+allocate(cvar_f1m(ncharb))
+allocate(cvar_f2m(ncharb))
+do icha = 1, ncharb
+  call field_get_val_s(ivarfl(isca(if1m(icha))), cvar_f1m(icha)%p)
+  call field_get_val_s(ivarfl(isca(if2m(icha))), cvar_f2m(icha)%p)
+enddo
+
+i = npo-1
+
+do ielt = 1, nelt
+
+  if (location_id .eq. MESH_LOCATION_CELLS) then
+    iel = ielt
+  else
+    iel = ifabor(ielt)
+  endif
+
+! --- Calculation of enthalpy of the gaseous species CHx1m
+!                                            and CHx2m at TH(NPO)
   ehchx1 = zero
   ehchx2 = zero
   ychx10 = zero
   ychx20 = zero
   do icha = 1, ncharb
-!
+
+    f1mc(icha) = cvar_f1m(icha)%p(iel) / x1(iel)
+    f2mc(icha) = cvar_f2m(icha)%p(iel) / x1(iel)
+
     den1   = 1.d0                                                  &
          / ( a1(icha)*wmole(ichx1c(icha))                          &
             +b1(icha)*wmole(ico)                                   &
@@ -144,10 +156,10 @@ do icel = 1, ncel
             +e1(icha)*wmole(ihcn)                                  &
             +f1(icha)*wmole(inh3) )
     ychx10 = ychx10                                                &
-            +den1*(f1mc(icel,icha)*a1(icha)*wmole(ichx1c(icha)) )
+            +den1*(f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
     ehchx1 = ehchx1                                                &
             +den1*( ehgaze(ichx1c(icha),i+1)                       &
-                   *f1mc(icel,icha)*a1(icha)*wmole(ichx1c(icha)) )
+                   *f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
     den2   = 1.d0                                                  &
          / ( a2(icha)*wmole(ichx2c(icha))                          &
             +b2(icha)*wmole(ico)                                   &
@@ -156,50 +168,59 @@ do icel = 1, ncel
             +e2(icha)*wmole(ihcn)                                  &
             +f2(icha)*wmole(inh3) )
     ychx20 = ychx20                                                &
-            +den2 *(f2mc(icel,icha)*a2(icha)*wmole(ichx2c(icha)) )
+            +den2 *(f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
     ehchx2 = ehchx2 + den2 *                                       &
          ( ehgaze(ichx2c(icha),i+1)                                &
-          *f2mc(icel,icha)*a2(icha)*wmole(ichx2c(icha)) )
+          *f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
   enddo
-  if ( ychx10.gt.epzero ) then
+  if (ychx10.gt.epzero) then
     ehchx1 = ehchx1 / ychx10
   else
     ehchx1 = ehgaze(ichx1,i+1)
   endif
-  if ( ychx20.gt.epzero ) then
+  if (ychx20.gt.epzero) then
     ehchx2 = ehchx2 / ychx20
   else
     ehchx2 = ehgaze(ichx2,i+1)
   endif
 
-! --- Clipping eventuel de TP a TH(NPO) si EH > EH1
+  ! --- Eventual clipping of temperature at TH(NPO) if EH > EH1
 
-  eh1 = fuel1(icel)*ehchx1                                  &
-       +fuel2(icel)*ehchx2                                  &
-       +fuel3(icel)*ehgaze(ico ,i+1)                        &
-       +fuel4(icel)*ehgaze(ih2s,i+1)                        &
-       +fuel5(icel)*ehgaze(ihy ,i+1)                        &
-       +fuel6(icel)*ehgaze(ihcn,i+1)                        &
-       +fuel7(icel)*ehgaze(inh3,i+1)                        &
-       +oxyd(icel) *ehgaze(io2 ,i+1)                        &
-       +prod1(icel)*ehgaze(ico2,i+1)                        &
-       +prod2(icel)*ehgaze(ih2o,i+1)                        &
-       +prod3(icel)*ehgaze(iso2,i+1)                        &
-       +xiner(icel)*ehgaze(in2 ,i+1)
+  eh1 = fuel1(iel)*ehchx1                                  &
+       +fuel2(iel)*ehchx2                                  &
+       +fuel3(iel)*ehgaze(ico ,i+1)                        &
+       +fuel4(iel)*ehgaze(ih2s,i+1)                        &
+       +fuel5(iel)*ehgaze(ihy ,i+1)                        &
+       +fuel6(iel)*ehgaze(ihcn,i+1)                        &
+       +fuel7(iel)*ehgaze(inh3,i+1)                        &
+       +oxyd(iel) *ehgaze(io2 ,i+1)                        &
+       +prod1(iel)*ehgaze(ico2,i+1)                        &
+       +prod2(iel)*ehgaze(ih2o,i+1)                        &
+       +prod3(iel)*ehgaze(iso2,i+1)                        &
+       +xiner(iel)*ehgaze(in2 ,i+1)
 
-  if ( eh(icel).ge.eh1 ) tp(icel)= th(i+1)
+  if (eh(ielt).ge.eh1) tp(ielt) = th(i+1)
 enddo
 
 i = 1
-do icel = 1, ncel
 
-! --- Calcul de l'enthalpie de l'espece gazeuse CHx1m
-!                                            et CHx2m a TH(1)
+do ielt = 1, nelt
+
+  if (location_id .eq. MESH_LOCATION_CELLS) then
+    iel = ielt
+  else
+    iel = ifabor(ielt)
+  endif
+
+  ! --- Calculation of enthalpy of the gaseous species CHx1m
+  !                                            and CHx2m at TH(1)
   ehchx1 = zero
   ehchx2 = zero
   ychx10 = zero
   ychx20 = zero
   do icha = 1, ncharb
+    f1mc(icha) = cvar_f1m(icha)%p(iel) / x1(iel)
+    f2mc(icha) = cvar_f2m(icha)%p(iel) / x1(iel)
     den1   = 1.d0                                                    &
          / ( a1(icha)*wmole(ichx1c(icha))                            &
             +b1(icha)*wmole(ico)                                     &
@@ -208,10 +229,10 @@ do icel = 1, ncel
             +e1(icha)*wmole(ihcn)                                    &
             +f1(icha)*wmole(inh3) )
     ychx10 = ychx10                                                  &
-            +den1*(f1mc(icel,icha)*a1(icha)*wmole(ichx1c(icha)) )
+            +den1*(f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
     ehchx1 = ehchx1                                                  &
             +den1*( ehgaze(ichx1c(icha),i)                           &
-                   *f1mc(icel,icha)*a1(icha)*wmole(ichx1c(icha)) )
+                   *f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
     den2   = 1.d0                                                     &
          / ( a2(icha)*wmole(ichx2c(icha))                             &
             +b2(icha)*wmole(ico)                                      &
@@ -220,51 +241,62 @@ do icel = 1, ncel
             +e2(icha)*wmole(ihcn)                                     &
             +f2(icha)*wmole(inh3) )
     ychx20 = ychx20                                                   &
-            +den2*(f2mc(icel,icha)*a2(icha)*wmole(ichx2c(icha)) )
+            +den2*(f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
     ehchx2 = ehchx2                                                   &
             +den2*( ehgaze(ichx2c(icha),i)                            &
-                   *f2mc(icel,icha)*a2(icha)*wmole(ichx2c(icha)) )
+                   *f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
   enddo
-  if ( ychx10.gt.epzero ) then
+  if (ychx10.gt.epzero) then
     ehchx1 = ehchx1 / ychx10
   else
     ehchx1 = ehgaze(ichx1,i)
   endif
-  if ( ychx20.gt.epzero ) then
+  if (ychx20.gt.epzero) then
     ehchx2 = ehchx2 / ychx20
   else
     ehchx2 = ehgaze(ichx2,i)
   endif
 
-! --- Clipping eventuel de TP a TH(1) si EH < EH0
+  ! --- Eventual clipping of temperature at TH(1) if EH < EH0
 
-  eh0  = fuel1(icel)*ehchx1                                  &
-        +fuel2(icel)*ehchx2                                  &
-        +fuel3(icel)*ehgaze(ico ,i)                          &
-        +fuel4(icel)*ehgaze(ih2s,i)                          &
-        +fuel5(icel)*ehgaze(ihy ,i)                          &
-        +fuel6(icel)*ehgaze(ihcn,i)                          &
-        +fuel7(icel)*ehgaze(inh3,i)                          &
-        +oxyd(icel) *ehgaze(io2 ,i)                          &
-        +prod1(icel)*ehgaze(ico2,i)                          &
-        +prod2(icel)*ehgaze(ih2o,i)                          &
-        +prod3(icel)*ehgaze(iso2,i)                          &
-        +xiner(icel)*ehgaze(in2 ,i)
+  eh0  = fuel1(iel)*ehchx1                                  &
+        +fuel2(iel)*ehchx2                                  &
+        +fuel3(iel)*ehgaze(ico ,i)                          &
+        +fuel4(iel)*ehgaze(ih2s,i)                          &
+        +fuel5(iel)*ehgaze(ihy ,i)                          &
+        +fuel6(iel)*ehgaze(ihcn,i)                          &
+        +fuel7(iel)*ehgaze(inh3,i)                          &
+        +oxyd(iel) *ehgaze(io2 ,i)                          &
+        +prod1(iel)*ehgaze(ico2,i)                          &
+        +prod2(iel)*ehgaze(ih2o,i)                          &
+        +prod3(iel)*ehgaze(iso2,i)                          &
+        +xiner(iel)*ehgaze(in2 ,i)
 
-  if ( eh(icel) .le. eh0 ) tp(icel)= th(i)
+  if (eh(ielt) .le. eh0 ) tp(ielt)= th(i)
 
 enddo
 
 
 do i = 1, npo-1
-  do icel = 1, ncel
 
-! --- Calcul de l'enthalpie de l'espece gazeuse CHx1m
-!                                            et CHx2m pour TH(I)
+  do ielt = 1, nelt
+
+    if (location_id .eq. MESH_LOCATION_CELLS) then
+      iel = ielt
+    else
+      iel = ifabor(ielt)
+    endif
+
+    ! --- Calculation of enthalpy of the gaseous species CHx1m
+    !                                            and CHx2m for TH(I)
     ehchx1 = zero
     ehchx2 = zero
     ychx10 = zero
     ychx20 = zero
+    do icha = 1, ncharb
+      f1mc(icha) = cvar_f1m(icha)%p(iel) / x1(iel)
+      f2mc(icha) = cvar_f2m(icha)%p(iel) / x1(iel)
+    enddo
     do icha = 1, ncharb
       den1   = 1.d0                                                 &
              / ( a1(icha)*wmole(ichx1c(icha))                       &
@@ -274,10 +306,10 @@ do i = 1, npo-1
                 +e1(icha)*wmole(ihcn)                               &
                 +f1(icha)*wmole(inh3) )
       ychx10 = ychx10                                               &
-              +den1*f1mc(icel,icha)*a1(icha)*wmole(ichx1c(icha))
+              +den1*f1mc(icha)*a1(icha)*wmole(ichx1c(icha))
       ehchx1 = ehchx1                                               &
               +den1*( ehgaze(ichx1c(icha),i)                        &
-                     *f1mc(icel,icha)*a1(icha)*wmole(ichx1c(icha)) )
+                     *f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
       den2   = 1.d0                                                 &
              / ( a2(icha)*wmole(ichx2c(icha))                       &
                 +b2(icha)*wmole(ico)                                &
@@ -286,36 +318,36 @@ do i = 1, npo-1
                 +e2(icha)*wmole(ihcn)                               &
                 +f2(icha)*wmole(inh3) )
       ychx20 = ychx20                                               &
-              +den2*(f2mc(icel,icha)*a2(icha)*wmole(ichx2c(icha)) )
+              +den2*(f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
       ehchx2 = ehchx2                                               &
               +den2*( ehgaze(ichx2c(icha),i)                        &
-                     *f2mc(icel,icha)*a2(icha)*wmole(ichx2c(icha)) )
+                     *f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
     enddo
-    if ( ychx10.gt.epzero ) then
+    if (ychx10.gt.epzero) then
       ehchx1 = ehchx1 / ychx10
     else
       ehchx1 = ehgaze(ichx1,i)
     endif
-    if ( ychx20.gt.epzero ) then
+    if (ychx20.gt.epzero) then
       ehchx2 = ehchx2 / ychx20
     else
       ehchx2 = ehgaze(ichx2,i)
     endif
-    eh0 = fuel1(icel)*ehchx1                                  &
-         +fuel2(icel)*ehchx2                                  &
-         +fuel3(icel)*ehgaze(ico ,i)                          &
-         +fuel4(icel)*ehgaze(ih2s,i)                          &
-         +fuel5(icel)*ehgaze(ihy ,i)                          &
-         +fuel6(icel)*ehgaze(ihcn,i)                          &
-         +fuel7(icel)*ehgaze(inh3,i)                          &
-         +oxyd(icel) *ehgaze(io2 ,i)                          &
-         +prod1(icel)*ehgaze(ico2,i)                          &
-         +prod2(icel)*ehgaze(ih2o,i)                          &
-         +prod3(icel)*ehgaze(iso2,i)                          &
-         +xiner(icel)*ehgaze(in2 ,i)
+    eh0 = fuel1(iel)*ehchx1                                  &
+         +fuel2(iel)*ehchx2                                  &
+         +fuel3(iel)*ehgaze(ico ,i)                          &
+         +fuel4(iel)*ehgaze(ih2s,i)                          &
+         +fuel5(iel)*ehgaze(ihy ,i)                          &
+         +fuel6(iel)*ehgaze(ihcn,i)                          &
+         +fuel7(iel)*ehgaze(inh3,i)                          &
+         +oxyd(iel) *ehgaze(io2 ,i)                          &
+         +prod1(iel)*ehgaze(ico2,i)                          &
+         +prod2(iel)*ehgaze(ih2o,i)                          &
+         +prod3(iel)*ehgaze(iso2,i)                          &
+         +xiner(iel)*ehgaze(in2 ,i)
 
-! --- Calcul de l'enthalpie de l'espece gazeuse CHx1m
-!                                            et CHx2m pour TH(I+1)
+    ! --- Calculation of enthalpy of the gaseous species CHx1m
+    !                                            and CHx2m for TH(I+1)
     ehchx1 = zero
     ehchx2 = zero
     ychx10 = zero
@@ -329,10 +361,10 @@ do i = 1, npo-1
               +e1(icha)*wmole(ihcn)                                   &
               +f1(icha)*wmole(inh3) )
       ychx10 = ychx10                                                 &
-              +den1*f1mc(icel,icha)*a1(icha)*wmole(ichx1c(icha))
+              +den1*f1mc(icha)*a1(icha)*wmole(ichx1c(icha))
       ehchx1 = ehchx1                                                 &
               +den1*( ehgaze(ichx1c(icha),i+1)                        &
-                     *f1mc(icel,icha)*a1(icha)*wmole(ichx1c(icha)) )
+                     *f1mc(icha)*a1(icha)*wmole(ichx1c(icha)) )
       den2   = 1.d0                                                   &
              / ( a2(icha)*wmole(ichx2c(icha))                         &
                 +b2(icha)*wmole(ico)                                  &
@@ -341,38 +373,38 @@ do i = 1, npo-1
                 +e2(icha)*wmole(ihcn)                                 &
                 +f2(icha)*wmole(inh3) )
       ychx20 = ychx20                                                 &
-              +den2*f2mc(icel,icha)*a2(icha)*wmole(ichx2c(icha))
+              +den2*f2mc(icha)*a2(icha)*wmole(ichx2c(icha))
       ehchx2 = ehchx2                                                 &
               +den2*( ehgaze(ichx2c(icha),i+1)                        &
-                     *f2mc(icel,icha)*a2(icha)*wmole(ichx2c(icha)) )
+                     *f2mc(icha)*a2(icha)*wmole(ichx2c(icha)) )
     enddo
-    if ( ychx10.gt.epzero ) then
+    if (ychx10.gt.epzero) then
       ehchx1 = ehchx1 / ychx10
     else
       ehchx1 = ehgaze(ichx1,i+1)
     endif
-    if ( ychx20.gt.epzero ) then
+    if (ychx20.gt.epzero) then
       ehchx2 = ehchx2 / ychx20
     else
       ehchx2 = ehgaze(ichx2,i+1)
     endif
 
-    eh1 = fuel1(icel)*ehchx1                                    &
-         +fuel2(icel)*ehchx2                                    &
-         +fuel3(icel)*ehgaze(ico ,i+1)                          &
-         +fuel4(icel)*ehgaze(ih2s,i+1)                          &
-         +fuel5(icel)*ehgaze(ihy ,i+1)                          &
-         +fuel6(icel)*ehgaze(ihcn,i+1)                          &
-         +fuel7(icel)*ehgaze(inh3,i+1)                          &
-         +oxyd(icel) *ehgaze(io2 ,i+1)                          &
-         +prod1(icel)*ehgaze(ico2,i+1)                          &
-         +prod2(icel)*ehgaze(ih2o,i+1)                          &
-         +prod3(icel)*ehgaze(iso2,i+1)                          &
-         +xiner(icel)*ehgaze(in2 ,i+1)
+    eh1 = fuel1(iel)*ehchx1                                    &
+         +fuel2(iel)*ehchx2                                    &
+         +fuel3(iel)*ehgaze(ico ,i+1)                          &
+         +fuel4(iel)*ehgaze(ih2s,i+1)                          &
+         +fuel5(iel)*ehgaze(ihy ,i+1)                          &
+         +fuel6(iel)*ehgaze(ihcn,i+1)                          &
+         +fuel7(iel)*ehgaze(inh3,i+1)                          &
+         +oxyd(iel) *ehgaze(io2 ,i+1)                          &
+         +prod1(iel)*ehgaze(ico2,i+1)                          &
+         +prod2(iel)*ehgaze(ih2o,i+1)                          &
+         +prod3(iel)*ehgaze(iso2,i+1)                          &
+         +xiner(iel)*ehgaze(in2 ,i+1)
 
-    if ( eh(icel).ge.eh0 .and. eh(icel).le.eh1 ) then
-      tp(icel)= th(i) + (eh(icel)-eh0) *                        &
-                        (th(i+1)-th(i))/(eh1-eh0)
+    if (eh(ielt).ge.eh0 .and. eh(ielt).le.eh1) then
+      tp(ielt)= th(i) + (  eh(ielt)-eh0)                       &
+                         * (th(i+1)-th(i))/(eh1-eh0)
     endif
   enddo
 enddo
