@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,33 +20,25 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine cs_coal_thfieldconv2 &
-!================
- ( ncelet , ncel   ,                                              &
-   rtp    , propce )
-
 !===============================================================================
-! FONCTION :
+! Function:
 ! --------
-! CALCUL DE LA TEMPERATURE DES PARTICULES
-!  EN FONCTION DE L'ENTHALPIE DU SOLIDE ET DES CONCENTRATIONS
+!> \file cs_coal_thfieldconv2.f90
+!> \brief Calculation of the particles temperature
+!>        Function with the solid enthalpy and concentrations
+!-------------------------------------------------------------------------------
+
+!-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! ncelet           ! i  ! <-- ! number of extended (real + ghost) cells        !
-! ncel             ! i  ! <-- ! number of cells                                !
-! rtp              ! tr ! <-- ! variables de calcul au centre des              !
-! (ncelet,*)       !    !     !    cellules (instant courant)                  !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-! eh0              ! tr ! <-- ! tableau reel de travail                        !
-! eh1              ! tr ! <-- ! tableau reel de travail                        !
-!__________________!____!_____!________________________________________________!
-!     TYPE : E (ENTIER), R (REEL), A (ALPHAMNUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
+!______________________________________________________________________________.
+!   mode          name          role
+!______________________________________________________________________________!
+!> \param[in]     ncelet        number of extended (real + ghost) cells
+!> \param[in]     ncel          number of cells
+!______________________________________________________________________________!
+
+subroutine cs_coal_thfieldconv2 &
+ ( ncelet , ncel )
 
 !==============================================================================
 ! Module files
@@ -55,7 +47,6 @@ subroutine cs_coal_thfieldconv2 &
 use paramx
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use entsor
@@ -64,6 +55,7 @@ use ppthch
 use coincl
 use cpincl
 use ppincl
+use field
 
 !===============================================================================
 
@@ -72,12 +64,10 @@ implicit none
 ! Arguments
 
 integer          ncelet, ncel
-double precision rtp(ncelet,nflown:nvar), propce(ncelet,*)
 
 ! Local variables
 
-integer          i      , icla   , icha   , icel
-integer          ipcte1 , ipcte2
+integer          i      , icla   , icha   , iel
 integer          ihflt2
 double precision h2     , x2     , xch    , xck
 double precision xash   , xnp    , xtes   , xwat
@@ -85,13 +75,18 @@ double precision xash   , xnp    , xtes   , xwat
 integer          iok1
 double precision , dimension ( : )     , allocatable :: eh0,eh1
 
+double precision, dimension(:), pointer :: cvar_xchcl, cvar_xckcl, cvar_xnpcl
+double precision, dimension(:), pointer :: cvar_xwtcl
+double precision, dimension(:), pointer :: cvar_h2cl
+double precision, dimension(:), pointer :: cpro_temp2, cpro_temp1
+
 !===============================================================================
-! Methode de conversion
+! Conversion method
 !
 ihflt2 = 1
 !
 !===============================================================================
-! 1. CALCULS PRELIMINAIRES
+! 1. Preliminary calculations
 !===============================================================================
 
 !===============================================================================
@@ -106,34 +101,35 @@ if ( iok1 > 0 ) THEN
 endif
 !===============================================================================
 
-! --- Initialisation des tableaux
+! --- Tables initialization
 eh0( : ) = zero
 eh1( : ) = zero
 
-! --- Initialisation de T2 a T1
+! --- Initialization from T2 to T1
 
-ipcte1 = ipproc(itemp1)
+call field_get_val_s(itemp1,cpro_temp1)
 do icla = 1, nclacp
-  ipcte2 = ipproc(itemp2(icla))
-  do icel = 1, ncel
-    propce(icel,ipcte2) = propce(icel,ipcte1)
+  call field_get_val_s(itemp2(icla),cpro_temp2)
+  do iel = 1, ncel
+    cpro_temp2(iel) = cpro_temp1(iel)
   enddo
 enddo
 
 !===============================================================================
-! 2. CALCUL DE LA TEMPERATURE DES PARTICULES
+! 2. Particles temperature calculation
 !===============================================================================
 
 if ( ihflt2.eq.0 ) then
 
-! --> H2 fonction lineaire de T2
+! --> H2 linear function of T2
 
   do icla = 1, nclacp
-    ipcte2 = ipproc(itemp2(icla))
+    call field_get_val_s(ivarfl(isca(ih2(icla))), cvar_h2cl)
+    call field_get_val_s(itemp2(icla),cpro_temp2)
     icha = ichcor(icla)
-    do icel = 1, ncel
-      propce(icel,ipcte2) =                                       &
-            (rtp(icel,isca(ih2(icla)))-h02ch(icha))               &
+    do iel = 1, ncel
+      cpro_temp2(iel) =                                       &
+            (cvar_h2cl(iel)-h02ch(icha))                         &!FIXME divide by x2
             / cp2ch(icha) + trefth
     enddo
   enddo
@@ -144,37 +140,42 @@ else
 
   do icla = 1, nclacp
 
-    ipcte2 = ipproc(itemp2(icla))
+    call field_get_val_s(ivarfl(isca(ixch(icla))), cvar_xchcl)
+    call field_get_val_s(ivarfl(isca(ixck(icla))), cvar_xckcl)
+    call field_get_val_s(ivarfl(isca(inp(icla))), cvar_xnpcl)
+    if ( ippmod(iccoal) .eq. 1 ) then
+      call field_get_val_s(ivarfl(isca(ixwt(icla))), cvar_xwtcl)
+    endif
+    call field_get_val_s(ivarfl(isca(ih2(icla))), cvar_h2cl)
+    call field_get_val_s(itemp2(icla),cpro_temp2)
 
     i = npoc-1
-    do icel = 1, ncel
-      xch  = rtp(icel,isca(ixch(icla)))
-      xnp  = rtp(icel,isca(inp(icla)))
-      xck  = rtp(icel,isca(ixck(icla)))
+    do iel = 1, ncel
+      xch  = cvar_xchcl(iel)
+      xck  = cvar_xckcl(iel)
+      xnp  = cvar_xnpcl(iel)
       xash = xmash(icla)*xnp
       if ( ippmod(iccoal) .eq. 1 ) then
-        xwat = rtp(icel,isca(ixwt(icla)))
+        xwat = cvar_xwtcl(iel)
       else
         xwat = 0.d0
       endif
 
       x2   = xch + xck + xash + xwat
 
-      if ( x2 .gt. epsicp*100.d0 ) then
+      xtes = xmp0(icla)*xnp
 
-        h2   = rtp(icel,isca(ih2(icla)))/x2
+      if ( xtes.gt.epsicp .and. x2.gt.epsicp*100.d0 ) then
 
-        xtes = xmp0(icla)*xnp
+        h2   = cvar_h2cl(iel)/x2
 
-        if ( xtes.gt.epsicp .and. x2.gt.epsicp*100.d0 ) then
-          eh1(icel) = xch /x2 * ehsoli(ich(ichcor(icla) ),i+1)    &
-                    + xck /x2 * ehsoli(ick(ichcor(icla) ),i+1)    &
-                    + xash/x2 * ehsoli(iash(ichcor(icla)),i+1)    &
-                    + xwat/x2 * ehsoli(iwat(ichcor(icla)),i+1)
-          if ( h2.ge.eh1(icel) ) then
-            propce(icel,ipcte2) = thc(i+1)
+        eh1(iel) = xch /x2 * ehsoli(ich(ichcor(icla) ),i+1)    &
+                  + xck /x2 * ehsoli(ick(ichcor(icla) ),i+1)    &
+                  + xash/x2 * ehsoli(iash(ichcor(icla)),i+1)    &
+                  + xwat/x2 * ehsoli(iwat(ichcor(icla)),i+1)
+        if ( h2.ge.eh1(iel) ) then
+          cpro_temp2(iel) = thc(i+1)
 
-          endif
         endif
 
       endif
@@ -182,33 +183,31 @@ else
     enddo
 
     i = 1
-    do icel = 1, ncel
-      xch  = rtp(icel,isca(ixch(icla)))
-      xnp  = rtp(icel,isca(inp(icla)))
-      xck  = rtp(icel,isca(ixck(icla)))
+    do iel = 1, ncel
+      xch  = cvar_xchcl(iel)
+      xck  = cvar_xckcl(iel)
+      xnp  = cvar_xnpcl(iel)
       xash = xmash(icla)*xnp
       if ( ippmod(iccoal) .eq. 1 ) then
-        xwat = rtp(icel,isca(ixwt(icla)))
+        xwat = cvar_xwtcl(iel)
       else
         xwat = 0.d0
       endif
 
       x2   = xch + xck + xash + xwat
+      xtes = xmp0(icla)*xnp
 
-      if ( x2 .gt. epsicp*100.d0 ) then
+      if ( xtes.gt.epsicp .and. x2.gt.epsicp*100.d0 ) then
 
-        h2   = rtp(icel,isca(ih2(icla)))/x2
+        h2   = cvar_h2cl(iel)/x2
 
-        xtes = xmp0(icla)*xnp
 
-        if ( xtes.gt.epsicp .and. x2.gt.epsicp*100.d0 ) then
-          eh0(icel) = xch /x2 * ehsoli(ich(ichcor(icla) ),i)      &
-                    + xck /x2 * ehsoli(ick(ichcor(icla) ),i)      &
-                    + xash/x2 * ehsoli(iash(ichcor(icla)),i)      &
-                    + xwat/x2 * ehsoli(iwat(ichcor(icla)),i)
-          if ( h2.le.eh0(icel) ) then
-            propce(icel,ipcte2) = thc(i)
-          endif
+        eh0(iel) = xch /x2 * ehsoli(ich(ichcor(icla) ),i)      &
+                  + xck /x2 * ehsoli(ick(ichcor(icla) ),i)      &
+                  + xash/x2 * ehsoli(iash(ichcor(icla)),i)      &
+                  + xwat/x2 * ehsoli(iwat(ichcor(icla)),i)
+        if ( h2.le.eh0(iel) ) then
+          cpro_temp2(iel) = thc(i)
         endif
 
       endif
@@ -216,40 +215,37 @@ else
     enddo
 
     do i = 1, npoc-1
-      do icel = 1, ncel
-        xch  = rtp(icel,isca(ixch(icla)))
-        xnp  = rtp(icel,isca(inp(icla)))
-        xck  = rtp(icel,isca(ixck(icla)))
+      do iel = 1, ncel
+        xch  = cvar_xchcl(iel)
+        xck  = cvar_xckcl(iel)
+        xnp  = cvar_xnpcl(iel)
         xash = xmash(icla)*xnp
         if ( ippmod(iccoal) .eq. 1 ) then
-          xwat = rtp(icel,isca(ixwt(icla)))
+          xwat = cvar_xwtcl(iel)
         else
           xwat = 0.d0
         endif
 
         x2   = xch + xck + xash + xwat
+        xtes = xmp0(icla)*xnp
 
-        if ( x2 .gt. epsicp*100.d0 ) then
+        if ( xtes.gt.epsicp .and. x2.gt.epsicp*100.d0 ) then
 
-          h2   = rtp(icel,isca(ih2(icla)))/x2
+          h2   = cvar_h2cl(iel)/x2
 
-          xtes = xmp0(icla)*xnp
+          eh0(iel) = xch /x2 * ehsoli(ich(ichcor(icla) ),i  )  &
+                    + xck /x2 * ehsoli(ick(ichcor(icla) ),i  )  &
+                    + xash/x2 * ehsoli(iash(ichcor(icla)),i  )  &
+                    + xwat/x2 * ehsoli(iwat(ichcor(icla)),i  )
 
-          if ( xtes.gt.epsicp .and. x2.gt.epsicp*100.d0 ) then
-            eh0(icel) = xch /x2 * ehsoli(ich(ichcor(icla) ),i  )  &
-                      + xck /x2 * ehsoli(ick(ichcor(icla) ),i  )  &
-                      + xash/x2 * ehsoli(iash(ichcor(icla)),i  )  &
-                      + xwat/x2 * ehsoli(iwat(ichcor(icla)),i  )
+          eh1(iel) = xch /x2 * ehsoli(ich(ichcor(icla) ),i+1)  &
+                    + xck /x2 * ehsoli(ick(ichcor(icla) ),i+1)  &
+                    + xash/x2 * ehsoli(iash(ichcor(icla)),i+1)  &
+                    + xwat/x2 * ehsoli(iwat(ichcor(icla)),i+1)
 
-            eh1(icel) = xch /x2 * ehsoli(ich(ichcor(icla) ),i+1)  &
-                      + xck /x2 * ehsoli(ick(ichcor(icla) ),i+1)  &
-                      + xash/x2 * ehsoli(iash(ichcor(icla)),i+1)  &
-                      + xwat/x2 * ehsoli(iwat(ichcor(icla)),i+1)
-
-            if ( h2.ge.eh0(icel) .and. h2.le.eh1(icel) ) then
-              propce(icel,ipcte2) = thc(i) + (h2-eh0(icel)) *     &
-                    (thc(i+1)-thc(i))/(eh1(icel)-eh0(icel))
-            endif
+          if ( h2.ge.eh0(iel) .and. h2.le.eh1(iel) ) then
+            cpro_temp2(iel) = thc(i) + (h2-eh0(iel)) *     &
+                  (thc(i+1)-thc(i))/(eh1(iel)-eh0(iel))
           endif
 
         endif
