@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,7 @@ subroutine cs_coal_varini &
 !========================
 
  ( nvar   , nscal  ,                                            &
-   dt     , rtp    , propce )
+   dt     )
 
 !===============================================================================
 ! FONCTION :
@@ -59,9 +59,6 @@ subroutine cs_coal_varini &
 ! nvar             ! i  ! <-- ! total number of variables                      !
 ! nscal            ! i  ! <-- ! total number of scalars                        !
 ! dt(ncelet)       ! tr ! <-- ! valeur du pas de temps                         !
-! rtp              ! tr ! <-- ! variables de calcul au centre des              !
-! (ncelet,*)       !    !     !    cellules                                    !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -96,13 +93,11 @@ implicit none
 
 integer          nvar   , nscal
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), propce(ncelet,*)
+double precision dt(ncelet)
 
 ! Local variables
 
-character*80     name
-
-integer          iel, ige, mode, icla, icha
+integer          iel, ige, mode, icla, icha, ifac
 
 double precision t1init, h1init, coefe(ngazem)
 double precision t2init
@@ -112,6 +107,21 @@ double precision xkent, xeent, d2s3
 integer ioxy
 double precision wmh2o,wmco2,wmn2,wmo2,dmas
 
+double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_phi
+double precision, dimension(:), pointer :: cvar_fb, cvar_omg
+double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
+double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
+double precision, dimension(:), pointer :: cvar_xchcl, cvar_xckcl, cvar_xnpcl
+double precision, dimension(:), pointer :: cvar_xwtcl
+double precision, dimension(:), pointer :: cvar_h2cl
+double precision, dimension(:), pointer :: cvar_scalt, cvar_xch
+double precision, dimension(:), pointer :: cvar_f1m, cvar_f2m
+double precision, dimension(:), pointer :: cvar_f4m, cvar_f5m, cvar_f6m
+double precision, dimension(:), pointer :: cvar_f7m, cvar_f8m, cvar_f9m
+double precision, dimension(:), pointer :: cvar_fvp2m
+double precision, dimension(:), pointer :: cvar_yco2, cvar_yhcn, cvar_ynh3
+double precision, dimension(:), pointer :: cvar_yno, cvar_hox
+double precision, dimension(:), pointer :: cpro_x1, bpro_x1
 
 ! NOMBRE DE PASSAGES DANS LA ROUTINE
 
@@ -119,19 +129,71 @@ integer          ipass
 data             ipass /0/
 save             ipass
 
-double precision, dimension(:), pointer :: xagecpi, xagegas
+!===============================================================================
+! 1. Initializations
+!===============================================================================
 
-!===============================================================================
-! 1.  INITIALISATION VARIABLES LOCALES
-!===============================================================================
+! Massic fraction of gas
+call field_get_val_s_by_name("x_c", cpro_x1)
+call field_get_val_s_by_name("b_x_c", bpro_x1)
 
 ipass = ipass + 1
 
 d2s3 = 2.d0/3.d0
 
+if (itytur.eq.2) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (itytur.eq.3) then
+  call field_get_val_s(ivarfl(ir11), cvar_r11)
+  call field_get_val_s(ivarfl(ir22), cvar_r22)
+  call field_get_val_s(ivarfl(ir33), cvar_r33)
+  call field_get_val_s(ivarfl(ir12), cvar_r12)
+  call field_get_val_s(ivarfl(ir13), cvar_r13)
+  call field_get_val_s(ivarfl(ir23), cvar_r23)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (iturb.eq.50) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+  call field_get_val_s(ivarfl(iphi), cvar_phi)
+  call field_get_val_s(ivarfl(ifb), cvar_fb)
+elseif (iturb.eq.60) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iomg), cvar_omg)
+endif
+
+call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
+call field_get_val_s_by_name("x_c_h",cvar_xch)
+
+if (noxyd.ge.2) then
+  call field_get_val_s(ivarfl(isca(if4m)), cvar_f4m)
+endif
+if (noxyd.ge.3) then
+  call field_get_val_s(ivarfl(isca(if5m)), cvar_f5m)
+endif
+if (ippmod(iccoal).ge.1) then
+  call field_get_val_s(ivarfl(isca(if6m)), cvar_f6m)
+endif
+call field_get_val_s(ivarfl(isca(if7m)), cvar_f7m)
+if (ihtco2.eq.1) then
+  call field_get_val_s(ivarfl(isca(if8m)), cvar_f8m)
+endif
+if (ihth2o.eq.1) then
+  call field_get_val_s(ivarfl(isca(if9m)), cvar_f9m)
+endif
+call field_get_val_s(ivarfl(isca(ifvp2m)), cvar_fvp2m)
+if (ieqco2.ge.1) then
+  call field_get_val_s(ivarfl(isca(iyco2)), cvar_yco2)
+endif
+if (ieqnox.eq.1) then
+  call field_get_val_s(ivarfl(isca(iyhcn)), cvar_yhcn)
+  call field_get_val_s(ivarfl(isca(iynh3)), cvar_ynh3)
+  call field_get_val_s(ivarfl(isca(iyno)), cvar_yno)
+  call field_get_val_s(ivarfl(isca(ihox)), cvar_hox)
+endif
+
 !===============================================================================
-! 2. INITIALISATION DES INCONNUES :
-!      UNIQUEMENT SI ON NE FAIT PAS UNE SUITE
+! 2. Variable initialization
 !===============================================================================
 
 ! RQ IMPORTANTE : pour la combustion CP, 1 seul passage suffit
@@ -148,44 +210,44 @@ if ( isuite.eq.0 .and. ipass.eq.1 ) then
   if (itytur.eq.2) then
 
     do iel = 1, ncel
-      rtp(iel,ik)  = xkent
-      rtp(iel,iep) = xeent
+      cvar_k(iel)  = xkent
+      cvar_ep(iel) = xeent
     enddo
 
   elseif (itytur.eq.3) then
 
     do iel = 1, ncel
-      rtp(iel,ir11) = d2s3*xkent
-      rtp(iel,ir22) = d2s3*xkent
-      rtp(iel,ir33) = d2s3*xkent
-      rtp(iel,ir12) = 0.d0
-      rtp(iel,ir13) = 0.d0
-      rtp(iel,ir23) = 0.d0
-      rtp(iel,iep)  = xeent
+      cvar_r11(iel) = d2s3*xkent
+      cvar_r22(iel) = d2s3*xkent
+      cvar_r33(iel) = d2s3*xkent
+      cvar_r12(iel) = 0.d0
+      cvar_r13(iel) = 0.d0
+      cvar_r23(iel) = 0.d0
+      cvar_ep(iel)  = xeent
     enddo
 
   elseif (iturb.eq.50) then
 
     do iel = 1, ncel
-      rtp(iel,ik)   = xkent
-      rtp(iel,iep)  = xeent
-      rtp(iel,iphi) = d2s3
-      rtp(iel,ifb)  = 0.d0
+      cvar_k(iel)   = xkent
+      cvar_ep(iel)  = xeent
+      cvar_phi(iel) = d2s3
+      cvar_fb(iel)  = 0.d0
     enddo
 
   elseif (iturb.eq.60) then
 
     do iel = 1, ncel
-      rtp(iel,ik)   = xkent
-      rtp(iel,iomg) = xeent/cmu/xkent
+      cvar_k(iel)   = xkent
+      cvar_omg(iel) = xeent/cmu/xkent
     enddo
 
   endif
 
-! --> On initialise tout le domaine de calcul avec de l'air a TINITK
-!                   ================================================
+  ! --> All the domain is filled with the first oxidizer at TINITK
+  !                   ============================================
 
-! ---- Calculs de H1INIT et H2INIT
+  ! ---- Computation of H1INIT and H2INIT
 
   t1init = t0
   t2init = t0
@@ -196,35 +258,35 @@ if ( isuite.eq.0 .and. ipass.eq.1 ) then
   do icla = 1, nclacp
     icha = ichcor(icla)
 
-    if (i_coal_drift.eq.1) then
-      write(name,'(a,i2.2)')'x_age_coal_' ,icla
-      call field_get_val_s_by_name(name, xagecpi)
+    call field_get_val_s(ivarfl(isca(ixch(icla))), cvar_xchcl)
+    call field_get_val_s(ivarfl(isca(ixck(icla))), cvar_xckcl)
+    call field_get_val_s(ivarfl(isca(inp(icla))), cvar_xnpcl)
+    call field_get_val_s(ivarfl(isca(ih2(icla))), cvar_h2cl)
+    if ( ippmod(iccoal) .eq. 1 ) then
+      call field_get_val_s(ivarfl(isca(ixwt(icla))), cvar_xwtcl)
     endif
 
     do iel = 1, ncel
 
-      rtp(iel,isca(ixch(icla))) = zero
-      rtp(iel,isca(ixck(icla))) = zero
-      rtp(iel,isca(inp(icla) )) = zero
-      rtp(iel,isca(ih2(icla) )) = zero
+      cvar_xchcl(iel) = zero
+      cvar_xckcl(iel) = zero
+      cvar_xnpcl(iel) = zero
+      cvar_h2cl(iel) = zero
       if ( ippmod(iccoal) .eq. 1 ) then
-        rtp(iel,isca(ixwt(icla))) = zero
+        cvar_xwtcl(iel) = zero
       endif
 
-      if (i_coal_drift.eq.1) then
-        xagecpi(iel) = zero
-      endif
     enddo
   enddo
 
-! ------ Variables de transport relatives au melange
+  ! ------ Transported variables for the mix (solid+carrying gas)^2
 
   do ige = 1, ngazem
     coefe(ige) = zero
   enddo
 
-!       On considere l'oxydant 1
-
+  ! Oxidizer are mix of O2, N2 (air), CO2 and H2O (recycled exhaust)
+  ! the composition of the fisrt oxidiser is taken in account
   coefe(io2) = wmole(io2)*oxyo2(1)                                &
               /( wmole(io2) *oxyo2(1) +wmole(in2) *oxyn2(1)       &
                 +wmole(ih2o)*oxyh2o(1)+wmole(ico2)*oxyco2(1))
@@ -246,43 +308,44 @@ if ( isuite.eq.0 .and. ipass.eq.1 ) then
  !============================
 
   do iel = 1, ncel
-    rtp(iel,isca(iscalt)) = h1init
+    cvar_scalt(iel) = h1init
+    cvar_xch(iel)   = h1init
   enddo
 
-! ------ Variables de transport relatives au melange gazeux
+  ! ------ Transported variables for the mix (passive scalars, variance)
 !        (scalaires passifs et variances associees)
 
-  if (i_coal_drift.eq.1) then
-    call field_get_val_s_by_name('x_age_gas', xagegas)
-  endif
+  do icha = 1, ncharb
+    call field_get_val_s(ivarfl(isca(if1m(icha))), cvar_f1m)
+    call field_get_val_s(ivarfl(isca(if2m(icha))), cvar_f2m)
+    do iel = 1, ncel
+      cvar_f1m(iel) = zero
+      cvar_f2m(iel) = zero
+    enddo
+  enddo
 
   do iel = 1, ncel
-!
-    do icha = 1, ncharb
-      rtp(iel,isca(if1m(icha))) = zero
-      rtp(iel,isca(if2m(icha))) = zero
-    enddo
-!
+
     if ( noxyd .ge. 2 ) then
-      rtp(iel,isca(if4m)) = zero
+      cvar_f4m(iel) = zero
     endif
     if ( noxyd .ge. 3 ) then
-      rtp(iel,isca(if5m)) = zero
+      cvar_f5m(iel) = zero
     endif
     if ( ippmod(iccoal) .ge. 1 ) then
-      rtp(iel,isca(if6m)) = zero
+      cvar_f6m(iel) = zero
     endif
 !
-    rtp(iel,isca(if7m)) = zero
+    cvar_f7m(iel) = zero
 !
     if ( ihtco2 .eq. 1 ) then
-      rtp(iel,isca(if8m)) = zero
+      cvar_f8m(iel) = zero
     endif
     if ( ihth2o .eq. 1 ) then
-      rtp(iel,isca(if9m)) = zero
+      cvar_f9m(iel) = zero
     endif
 !
-    rtp(iel,isca(ifvp2m)) = zero
+    cvar_fvp2m(iel) = zero
 !
     if ( ieqco2.ge.1 ) then
 
@@ -294,42 +357,49 @@ if ( isuite.eq.0 .and. ipass.eq.1 ) then
       dmas = ( oxyo2 (ioxy)*wmo2 +oxyn2 (ioxy)*wmn2               &
               +oxyh2o(ioxy)*wmh2o+oxyco2(ioxy)*wmco2 )
       xco2 = oxyco2(ioxy)*wmco2/dmas
-      rtp(iel,isca(iyco2)) = oxyco2(ioxy)*wmco2/dmas
+      cvar_yco2(iel) = oxyco2(ioxy)*wmco2/dmas
     endif
-!
-    if ( ieqnox .eq. 1 ) then
-      rtp(iel,isca(iyhcn)) = zero
+    if ( ieqnox.eq.1 ) then
+      cvar_yhcn(iel) = zero
 ! Initialisation du NH3
-      rtp(iel,isca(iynh3)) = zero
-      rtp(iel,isca(iyno )) = zero
-      rtp(iel,isca(ihox )) = h1init
-    endif
-    if (i_coal_drift.eq.1) then
-      xagegas(iel) = zero
+      cvar_ynh3(iel) = zero
+      cvar_yno(iel) = zero
+      cvar_hox(iel) = h1init
     endif
+
   enddo
 
 endif
 
-!===============================================================================
-! 2.  ON DONNE LA MAIN A L'UTILISATEUR
-!===============================================================================
-
 if (ipass.eq.1) then
 
-  call cs_user_initialization &
+  do iel = 1, ncel
+    ! Initialization of the continuous mass fraction
+    cpro_x1(iel) = 1.d0
+  enddo
+
+  ! Initialization of the continuous mass fraction AT the BCs
+  do ifac = 1, nfabor
+    bpro_x1(ifac) = 1.d0
+  enddo
+
+!===============================================================================
+! 3. User initialization
+!===============================================================================
+
+  call cs_user_f_initialization &
   !==========================
 ( nvar   , nscal  ,                                            &
   dt     )
 
 endif
 
-!----
-! FORMATS
-!----
+!--------
+! Formats
+!--------
 
 !----
-! FIN
+! End
 !----
 
 return
