@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,42 +20,34 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine cs_fuel_physprop2 &
-!================
-
- ( ncelet , ncel   ,                              &
-   rtp    , propce )
-
 !===============================================================================
-! FONCTION :
+! Function:
 ! --------
+!> \file cs_fuel_physprop2.f90
+!>
+!> \brief Calculation of the physical properties of the dispersed phase
+!>
+!>
+!> Cell values
+!> -----------
+!> - Mass fraction of liquid
+!>   and eventual clippings
+!> - Diameter
+!> - Mass density
+!>   and eventual clippings
+!-------------------------------------------------------------------------------
 
-! CALCUL DES PROPRIETES PHYSIQUES DE LA PHASE DISPERSEE
-! VALEURS CELLULES
-! ----------------
-
-!   FRACTION MASSIQUE DE LIQUIDE
-!     ET CLIPPING EVENTUELS
-!   DIAMETRE
-!   MASSE VOLUMIQUE
-!     ET CLIPPING EVENTUELS
-
+!-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! ncelet           ! i  ! <-- ! number of extended (real + ghost) cells        !
-! ncel             ! i  ! <-- ! number of cells                                !
-! rtp              ! tr ! <-- ! variables de calcul au centre des              !
-! (ncelet,*)       !    !     !    cellules (instant courant)                  !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-!__________________!____!_____!________________________________________________!
+!______________________________________________________________________________.
+!  mode           name          role
+!______________________________________________________________________________!
+!> \param[in]     ncelet        number of extended (real + ghost) cells
+!> \param[in]     ncel          number of cells
+!______________________________________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
+subroutine cs_fuel_physprop2 &
+ ( ncelet , ncel )
 
 !===============================================================================
 ! Module files
@@ -64,7 +56,6 @@ subroutine cs_fuel_physprop2 &
 use paramx
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use entsor
 use cstnum
@@ -75,6 +66,7 @@ use coincl
 use cpincl
 use cs_fuel_incl
 use ppincl
+use field
 
 !===============================================================================
 
@@ -84,28 +76,30 @@ implicit none
 
 integer          ncelet , ncel
 
-double precision rtp(ncelet,nflown:nvar) , propce(ncelet,*)
-
 ! Local variables
 
 integer          iel
-integer          n1     , n2     , ipcdia , ipcro2 , icla
+integer          n1     , n2     , icla
+
 double precision xnp    ,  d1s3
 double precision diam2m , diam2x
 
+double precision, dimension(:), pointer :: cvar_yfolcl, cvar_ngcl
+double precision, dimension(:), pointer :: cpro_diam2, cpro_rom2
+
 !===============================================================================
 
 !===============================================================================
-! 1. INITIALISATIONS
+! 1. Initializations
 !===============================================================================
 
 d1s3 = 1.d0/3.d0
 
 !===============================================================================
-! 2. CALCUL POUR CHAQUE CLASSE
-!    DE LA MASSE VOLUMIQUE DU FOL
-!    DE LA FRACTION MASSIQUE DE FOL
-!    DU DIAMETRE DU COKE
+! 2. Calculation for each class
+!    - of mass density of fol
+!    - of mass fraction of fol
+!    - of the coke diameter
 !===============================================================================
 
 do icla = 1, nclafu
@@ -115,37 +109,39 @@ do icla = 1, nclafu
   diam2m =  1.d0
   diam2x =  0.d0
 
-  ipcdia = ipproc(idiam2(icla))
-  ipcro2 = ipproc(irom2 (icla))
+  call field_get_val_s(ivarfl(isca(iyfol(icla))), cvar_yfolcl)
+  call field_get_val_s(ivarfl(isca(ing(icla))), cvar_ngcl)
+  call field_get_val_s(idiam2(icla), cpro_diam2)
+  call field_get_val_s(irom2(icla), cpro_rom2)
 
   do iel = 1, ncel
 
-!  Masse Volumique
-    propce(iel,ipcro2) = rho0fl
-!
-! --- Calcul du diametre de la particule
-!
-    yfol   = rtp(iel,isca(iyfol(icla)))
-    xnp    = rtp(iel,isca(ing  (icla)))
+    !  Mass density
+    cpro_rom2(iel) = rho0fl
+
+    ! --- Calculation of the particle diameter
+
+    yfol   = cvar_yfolcl(iel)
+    xnp    = cvar_ngcl(iel)
     if ( yfol .gt. epsifl .and. (xnp*yfol) .gt. 0.d0) then
 
-      propce(iel,ipcdia) = ( (yfol / propce(iel,ipcro2) )           &
+      cpro_diam2(iel) = ( (yfol / cpro_rom2(iel) )           &
                             /(pi/6.d0 * xnp) ) ** d1s3
 
-      if ( propce(iel,ipcdia) .gt. dinifl(icla) ) then
+      if ( cpro_diam2(iel) .gt. dinifl(icla) ) then
         n1 = n1+1
-        diam2x = max(diam2x,propce(iel,ipcdia))
-        propce(iel,ipcdia) = dinifl(icla)
+        diam2x = max(diam2x,cpro_diam2(iel))
+        cpro_diam2(iel) = dinifl(icla)
       endif
 
-      if ( propce(iel,ipcdia) .lt. diniin(icla) ) then
+      if ( cpro_diam2(iel) .lt. diniin(icla) ) then
         n2 = n2+1
-        diam2m = min(diam2m,propce(iel,ipcdia))
-        propce(iel,ipcdia) = diniin(icla)
+        diam2m = min(diam2m,cpro_diam2(iel))
+        cpro_diam2(iel) = diniin(icla)
       endif
 
     else
-      propce(iel,ipcdia) = dinifl(icla)
+      cpro_diam2(iel) = dinifl(icla)
     endif
 
   enddo
@@ -172,16 +168,17 @@ do icla = 1, nclafu
 
 enddo
 
-!----
+!--------
 ! Formats
+!--------
 !----
 
- 1001 format(/,1X,' CLIPPING EN MAX DU DIAMETRE CLASSE :',I2,           &
-       /,10X,' Nombre de points : ',I8,                           &
-       /,10X,' Valeur Max       : ',G15.7)
- 1002 format(/,1X,' CLIPPING EN MIN DU DIAMETRE CLASSE :',I2,           &
-       /,10X,' Nombre de points : ',I8,                           &
-       /,10X,' Valeur Min       : ',G15.7)
+ 1001 format(/,1X,' clipping in max of class diameter:',I2,           &
+       /,10X,' Number of points: ',I8,                           &
+       /,10X,' Max value: ',G15.7)
+ 1002 format(/,1X,' clipping in min of class diametre:',I2,           &
+       /,10X,' Number of points: ',I8,                           &
+       /,10X,' Min value: ',G15.7)
 
 
 !----
