@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -23,7 +23,7 @@
 subroutine cs_fuel_radst &
 !=======================
   ( ivar   , ncelet , ncel   ,                                    &
-    volume , rtpa   , propce , smbrs  , rovsdt )
+    volume , smbrs  , rovsdt )
 
 
 !===============================================================================
@@ -46,9 +46,6 @@ subroutine cs_fuel_radst &
 ! ncelet           ! i  ! <-- ! number of extended (real + ghost) cells        !
 ! ncel             ! i  ! <-- ! number of cells                                !
 ! volume(ncelet    ! tr ! <-- ! volume des cellules                            !
-! rtp, rtpa        ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at current and previous time steps)          !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 ! smbrs(ncelet     ! tr ! <-- ! second membre du systeme                       !
 ! rovsdt(ncelet    ! tr ! <-- ! diagonale du systeme                           !
 !__________________!____!_____!________________________________________________!
@@ -66,13 +63,13 @@ subroutine cs_fuel_radst &
 use paramx
 use cstnum
 use cstphy
-use dimens, only: nvar
 use entsor
 use numvar
 use ppppar
 use ppthch
 use ppincl
 use radiat
+use field
 
 !===============================================================================
 
@@ -85,13 +82,16 @@ integer          ivar , ncelet, ncel
 double precision volume(ncelet)
 double precision smbrs(ncelet)
 double precision rovsdt(ncelet)
-double precision rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
 
 ! Local variables
 
 integer          iel , numcla , ipcl
 
+double precision, dimension(:), pointer :: cvara_yfolcl
+double precision, dimension(:), pointer :: cpro_tsri, cpro_yfol, cpro_tsre
+character(len=80) :: f_name
+integer              f_id
+
 !===============================================================================
 
 !===============================================================================
@@ -101,26 +101,36 @@ integer          iel , numcla , ipcl
 numcla = ivar-isca(ih2(1))+1
 ipcl   = 1+numcla
 
+call field_get_val_prev_s(ivarfl(isca(iyfol(numcla))), cvara_yfolcl)
+
 !===============================================================================
 ! 2. PRISE EN COMPTE DES TERMES SOURCES RADIATIFS
 !===============================================================================
 
+write(f_name,  '("rad_st_implicit_", i2.2)') ipcl
+call field_get_id(f_name, f_id)
+call field_get_val_s(f_id,cpro_tsri)
+write(f_name,  '("rad_st_", i2.2)') ipcl
+call field_get_id(f_name, f_id)
+call field_get_val_s(f_id,cpro_tsre)
+call field_get_val_s(iyfol(numcla),cpro_yfol)
+
 
 do iel = 1,ncel
-  propce(iel,ipproc(itsri(ipcl))) = max(-propce(iel,ipproc(itsri(ipcl))),zero)
+  cpro_tsri(iel) = max(-cpro_tsri(iel),zero)
 enddo
 
 do iel = 1,ncel
-  if ( propce(iel,ipproc(iyfol(numcla))) .gt. epzero ) then
+  if ( cpro_yfol(iel) .gt. epzero ) then
 
 !--> PARTIE EXPLICITE
 
-    smbrs(iel)  = smbrs(iel) +  propce(iel,ipproc(itsre(ipcl)))*volume(iel)  &
-                               *rtpa(iel,isca(iyfol(numcla)))
+    smbrs(iel)  = smbrs(iel) +  cpro_tsre(iel)*volume(iel)  &
+                               *cvara_yfolcl(iel)
 
 !--> PARTIE IMPLICITE
 
-    rovsdt(iel) = rovsdt(iel) + propce(iel,ipproc(itsri(ipcl)))*volume(iel)
+    rovsdt(iel) = rovsdt(iel) + cpro_tsri(iel)*volume(iel)
   endif
 
 enddo
