@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,7 @@ subroutine cs_fuel_varini &
 !========================
 
  ( nvar   , nscal  ,                                            &
-   dt     , rtp    , propce )
+   dt     )
 
 !===============================================================================
 ! FONCTION :
@@ -59,9 +59,6 @@ subroutine cs_fuel_varini &
 ! nvar             ! i  ! <-- ! total number of variables                      !
 ! nscal            ! i  ! <-- ! total number of scalars                        !
 ! dt(ncelet)       ! tr ! <-- ! valeur du pas de temps                         !
-! rtp              ! tr ! <-- ! variables de calcul au centre des              !
-! (ncelet,*)       !    !     !    cellules                                    !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -88,6 +85,7 @@ use cs_fuel_incl
 use ppincl
 use ppcpfu
 use mesh
+use field
 
 !===============================================================================
 
@@ -95,35 +93,81 @@ implicit none
 
 integer          nvar   , nscal
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), propce(ncelet,*)
+double precision dt(ncelet)
 
 ! Local variables
 
-integer          iel, ige, mode, icla , ioxy
+integer          iel, ige, mode, icla , ioxy, ifac
 
 double precision t1init, h1init, coefe(ngazem)
 double precision t2init
 double precision xkent, xeent, d2s3 , dmas
 double precision wmo2 , wmn2 , wmco2 , wmh2o
 
-
-! NOMBRE DE PASSAGES DANS LA ROUTINE
+double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_phi
+double precision, dimension(:), pointer :: cvar_fb, cvar_omg
+double precision, dimension(:), pointer :: cvar_r11, cvar_r22, cvar_r33
+double precision, dimension(:), pointer :: cvar_r12, cvar_r13, cvar_r23
+double precision, dimension(:), pointer :: cvar_yfolcl, cvar_ngcl, cvar_h2cl
+double precision, dimension(:), pointer :: cvar_scalt
+double precision, dimension(:), pointer :: cvar_fvap, cvar_f7m, cvar_fvp2m
+double precision, dimension(:), pointer :: cvar_yco2
+double precision, dimension(:), pointer :: cvar_yhcn, cvar_yno, cvar_hox
+double precision, dimension(:), pointer :: x1, b_x1
 
 integer          ipass
 data             ipass /0/
 save             ipass
 
 !===============================================================================
-! 1.  INITIALISATION VARIABLES LOCALES
+! 1. Initializations
 !===============================================================================
 
+! Massic fraction of gas
+call field_get_val_s_by_name("x_c", x1)
+call field_get_val_s_by_name("b_x_c", b_x1)
+
 ipass = ipass + 1
 
 d2s3 = 2.d0/3.d0
 
+if (itytur.eq.2) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (itytur.eq.3) then
+  call field_get_val_s(ivarfl(ir11), cvar_r11)
+  call field_get_val_s(ivarfl(ir22), cvar_r22)
+  call field_get_val_s(ivarfl(ir33), cvar_r33)
+  call field_get_val_s(ivarfl(ir12), cvar_r12)
+  call field_get_val_s(ivarfl(ir13), cvar_r13)
+  call field_get_val_s(ivarfl(ir23), cvar_r23)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+elseif (iturb.eq.50) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iep), cvar_ep)
+  call field_get_val_s(ivarfl(iphi), cvar_phi)
+  call field_get_val_s(ivarfl(ifb), cvar_fb)
+elseif (iturb.eq.60) then
+  call field_get_val_s(ivarfl(ik), cvar_k)
+  call field_get_val_s(ivarfl(iomg), cvar_omg)
+endif
+
+call field_get_val_s(ivarfl(isca(iscalt)), cvar_scalt)
+
+call field_get_val_s(ivarfl(isca(ifvap)), cvar_fvap)
+call field_get_val_s(ivarfl(isca(if7m)), cvar_f7m)
+call field_get_val_s(ivarfl(isca(ifvp2m)), cvar_fvp2m)
+if (ieqco2.ge.1) then
+  call field_get_val_s(ivarfl(isca(iyco2)), cvar_yco2)
+endif
+if (ieqnox.eq.1) then
+  call field_get_val_s(ivarfl(isca(iyhcn)), cvar_yhcn)
+  call field_get_val_s(ivarfl(isca(iyno)), cvar_yno)
+  call field_get_val_s(ivarfl(isca(ihox)), cvar_hox)
+endif
+
 !===============================================================================
-! 2. INITIALISATION DES INCONNUES :
-!      UNIQUEMENT SI ON NE FAIT PAS UNE SUITE
+! 2. Variable initialization
 !===============================================================================
 
 ! RQ IMPORTANTE : pour la combustion FU, 1 seul passage suffit
@@ -140,63 +184,68 @@ if ( isuite.eq.0 .and. ipass.eq.1 ) then
   if (itytur.eq.2) then
 
     do iel = 1, ncel
-      rtp(iel,ik)  = xkent
-      rtp(iel,iep) = xeent
+      cvar_k(iel)  = xkent
+      cvar_ep(iel) = xeent
     enddo
 
   elseif (itytur.eq.3) then
 
     do iel = 1, ncel
-      rtp(iel,ir11) = d2s3*xkent
-      rtp(iel,ir22) = d2s3*xkent
-      rtp(iel,ir33) = d2s3*xkent
-      rtp(iel,ir12) = 0.d0
-      rtp(iel,ir13) = 0.d0
-      rtp(iel,ir23) = 0.d0
-      rtp(iel,iep)  = xeent
+      cvar_r11(iel) = d2s3*xkent
+      cvar_r22(iel) = d2s3*xkent
+      cvar_r33(iel) = d2s3*xkent
+      cvar_r12(iel) = 0.d0
+      cvar_r13(iel) = 0.d0
+      cvar_r23(iel) = 0.d0
+      cvar_ep(iel)  = xeent
     enddo
 
   elseif (iturb.eq.50) then
 
     do iel = 1, ncel
-      rtp(iel,ik)   = xkent
-      rtp(iel,iep)  = xeent
-      rtp(iel,iphi) = d2s3
-      rtp(iel,ifb)  = 0.d0
+      cvar_k(iel)   = xkent
+      cvar_ep(iel)  = xeent
+      cvar_phi(iel) = d2s3
+      cvar_fb(iel)  = 0.d0
     enddo
 
   elseif (iturb.eq.60) then
 
     do iel = 1, ncel
-      rtp(iel,ik)   = xkent
-      rtp(iel,iomg) = xeent/cmu/xkent
+      cvar_k(iel)   = xkent
+      cvar_omg(iel) = xeent/cmu/xkent
     enddo
 
   endif
 
-! --> On initialise tout le domaine de calcul avec de l'air a TINITK
-!                   ================================================
+  ! --> All the domain is filled with the first oxidizer at TINITK
+  !                   ============================================
 
-! ---- Calculs de H1INIT et H2INIT
+  ! ---- Computation of H1INIT and H2INIT
 
   t1init = t0
   t2init = t0
 
-! ------ Variables de transport relatives a la phase liquide
+  ! ------ Variables de transport relatives a la phase liquide
   do icla = 1, nclafu
+    call field_get_val_s(ivarfl(isca(iyfol(icla))), cvar_yfolcl)
+    call field_get_val_s(ivarfl(isca(ing(icla))), cvar_ngcl)
+    call field_get_val_s(ivarfl(isca(ih2(icla))), cvar_h2cl)
     do iel = 1, ncel
-      rtp(iel,isca(iyfol(icla))) = zero
-      rtp(iel,isca(ing(icla) ))  = zero
-      rtp(iel,isca(ih2(icla) ))  = zero
+      cvar_yfolcl(iel) = zero
+      cvar_ngcl(iel) = zero
+      cvar_h2cl(iel) = zero
     enddo
   enddo
 
-! ------ Variables de transport relatives au melange
+  ! ------ Transported variables for the mix (liquid+carrying gas)^2
 
   do ige = 1, ngazem
     coefe(ige) = zero
   enddo
-!  On considere l'oxydant 1
+
+  ! Oxidizer are mix of O2, N2 (air), CO2 and H2O (recycled exhaust)
+  ! the composition of the fisrt oxidiser is taken in account
   coefe(io2) = wmole(io2)*oxyo2(1)                                &
               /( wmole(io2) *oxyo2(1) +wmole(in2) *oxyn2(1)       &
                 +wmole(ih2o)*oxyh2o(1)+wmole(ico2)*oxyco2(1))
@@ -207,22 +256,21 @@ if ( isuite.eq.0 .and. ipass.eq.1 ) then
               /( wmole(io2) *oxyo2(1) +wmole(in2) *oxyn2(1)       &
                 +wmole(ih2o)*oxyh2o(1)+wmole(ico2)*oxyco2(1))
   coefe(in2) = 1.d0-coefe(io2)-coefe(ih2o)-coefe(ico2)
-!
+
   mode = -1
 
   call cs_fuel_htconvers1 ( mode   , h1init , coefe  ,  t1init )
  !=======================
   do iel = 1, ncel
-    rtp(iel,isca(iscalt)) = h1init
+    cvar_scalt(iel) = h1init
   enddo
 
-! ------ Variables de transport relatives au melange gazeux
-!        (scalaires passifs et variances associees)
+  ! ------ Transported variables for the mix (passive scalars, variance)
 
   do iel = 1, ncel
-    rtp(iel,isca(ifvap))   = zero
-    rtp(iel,isca(if7m ))   = zero
-    rtp(iel,isca(ifvp2m))  = zero
+    cvar_fvap(iel) = zero
+    cvar_f7m(iel) = zero
+    cvar_fvp2m(iel) = zero
     if ( ieqco2.ge.1 ) then
       ioxy   =  1
       wmo2   = wmole(io2)
@@ -231,33 +279,42 @@ if ( isuite.eq.0 .and. ipass.eq.1 ) then
       wmn2   = wmole(in2)
       dmas = ( oxyo2 (ioxy)*wmo2 +oxyn2 (ioxy)*wmn2               &
               +oxyh2o(ioxy)*wmh2o+oxyco2(ioxy)*wmco2 )
-      rtp(iel,isca(iyco2)) = oxyco2(ioxy)*wmco2/dmas
+      cvar_yco2(iel) = oxyco2(ioxy)*wmco2/dmas
     endif
     if ( ieqnox.eq.1 ) then
-      rtp(iel,isca(iyhcn)) = zero
-      rtp(iel,isca(iyno))  = zero
-      rtp(iel,isca(ihox))  = h1init
+      cvar_yhcn(iel) = zero
+      cvar_yno(iel) = zero
+      cvar_hox(iel) = h1init
     endif
+
+    ! Initialization of the continuous mass fraction
+    x1(iel) = 1.d0
+
+  enddo
+
+  ! Initialization of the continuous mass fraction AT the BCs
+  do ifac = 1, nfabor
+    b_x1(ifac) = 1.d0
   enddo
 
 endif
 
 !===============================================================================
-! 2.  ON DONNE LA MAIN A L'UTILISATEUR
+! 3. User initialization
 !===============================================================================
 
 if (ipass.eq.1) then
 
-  call cs_user_initialization &
+  call cs_user_f_initialization &
   !==========================
 ( nvar   , nscal  ,                                            &
   dt     )
 
 endif
 
-!----
+!--------
 ! Formats
-!----
+!--------
 
 !----
 ! End
