@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,6 @@ subroutine cs_gascomb &
 !====================
  ( ncelet , ncel   , icb1 , icb2 ,                                        &
    indpdf ,                                                               &
-   rtp    , x2     ,                                                      &
    f1m    , f2m    , f3m      , f4m    , f5m  , f6m , f7m , f8m , f9m ,   &
    pdfm1  , pdfm2  , doxyd    , dfuel  , hrec ,                           &
    af1    , af2    , cx1m     , cx2m   , wmf1   , wmf2 ,                  &
@@ -86,7 +85,6 @@ subroutine cs_gascomb &
 use paramx
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use cstnum
 use entsor
@@ -98,6 +96,7 @@ use coincl
 use cpincl
 use ppincl
 use ppcpfu
+use field
 
 !===============================================================================
 
@@ -109,7 +108,6 @@ integer          ncelet , ncel
 integer          icb1   , icb2
 integer          indpdf(ncel)
 
-double precision rtp(ncelet,nflown:nvar), x2(ncel)
 double precision f1m(ncel)  , f2m(ncel) , f3m(ncel)
 double precision f4m(ncel)  , f5m(ncel) , f6m(ncel)
 double precision f7m(ncel)  , f8m(ncel) , f9m(ncel)
@@ -150,11 +148,20 @@ double precision somm  , sommax , sommin
 double precision zz(ngazem) , zzox(ngazem) , zzcl(ngazem)
 double precision zzs1(ngazem),zzs2(ngazem),zzs3(ngazem),zzs4(ngazem)
 
-!
+double precision, dimension(:), pointer :: x1
+double precision, dimension(:), pointer :: cvar_yco2
+
 !===============================================================================
 ! 1. CALCULS PRELIMINAIRES
 !===============================================================================
 
+if (ieqco2.eq.1) then
+  call field_get_val_s(ivarfl(isca(iyco2)), cvar_yco2)
+endif
+
+! Massic fraction of gas
+call field_get_val_s_by_name("x_c", x1)
+
 ! --- Masses molaires
 wmco   = wmole(ico  )
 wmh2s  = wmole(ih2s )
@@ -195,7 +202,7 @@ do iel = 1, ncel
 
 ! --> Calculs preliminaires
     if ( ieqco2 .eq. 1 ) then
-      zco2t = (rtp(iel,isca(iyco2))/ (1.d0-x2(iel)))/wmco2
+      zco2t = (cvar_yco2(iel)/ x1(iel))/wmco2
     else
       zco2t = 0.d0
     endif
@@ -286,7 +293,7 @@ do iel = 1, ncel
 ! --> Calculs preliminaires
 
     if ( ieqco2 .eq. 1 ) then
-      zco2t = (rtp(iel,isca(iyco2))/(1.d0-x2(iel)))/wmco2
+      zco2t = (cvar_yco2(iel)/x1(iel))/wmco2
     else
       zco2t = 0.d0
     endif
