@@ -10,7 +10,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -56,6 +56,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Local structure definitions
  *============================================================================*/
@@ -1948,6 +1950,8 @@ _convert_array_uint64_to_uint64(const cs_lnum_t               src_idx_start,
 
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
@@ -2047,8 +2051,7 @@ fvm_convert_array(const int                     src_dim,
                   "float to int32, int64, uint32, or uint64"));
       break;
 
-    case CS_CHAR:
-    case CS_DATATYPE_NULL:
+    default:
       break;
 
     }
@@ -2095,8 +2098,7 @@ fvm_convert_array(const int                     src_dim,
                   "double to int32, int64, uint32, or uint64"));
       break;
 
-    case CS_CHAR:
-    case CS_DATATYPE_NULL:
+    default:
       break;
 
     }
@@ -2173,8 +2175,7 @@ fvm_convert_array(const int                     src_dim,
                                      dest_data);
       break;
 
-    case CS_CHAR:
-    case CS_DATATYPE_NULL:
+    default:
       break;
 
     }
@@ -2251,8 +2252,7 @@ fvm_convert_array(const int                     src_dim,
                                      dest_data);
       break;
 
-    case CS_CHAR:
-    case CS_DATATYPE_NULL:
+    default:
       break;
 
     }
@@ -2329,8 +2329,7 @@ fvm_convert_array(const int                     src_dim,
                                       dest_data);
       break;
 
-    case CS_CHAR:
-    case CS_DATATYPE_NULL:
+    default:
       break;
 
     }
@@ -2407,15 +2406,13 @@ fvm_convert_array(const int                     src_dim,
                                       dest_data);
       break;
 
-    case CS_CHAR:
-    case CS_DATATYPE_NULL:
+    default:
       break;
 
     }
     break;
 
-  case CS_CHAR:
-  case CS_DATATYPE_NULL:
+  default:
     break;
   }
 }
