@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro and Type definitions
  *============================================================================*/
@@ -306,6 +308,8 @@ _group_class_dump(const fvm_group_class_t  *this_group_class,
   }
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -437,13 +441,14 @@ fvm_group_class_set_add(fvm_group_class_set_t   *this_group_class_set,
 fvm_group_class_set_t *
 fvm_group_class_set_destroy(fvm_group_class_set_t  *this_group_class_set)
 {
-  int i, j;
+  if (this_group_class_set == NULL)
+    return NULL;
 
-  for (i = 0; i < this_group_class_set->size; i++) {
+  for (int i = 0; i < this_group_class_set->size; i++) {
 
     fvm_group_class_t *_class = this_group_class_set->class + i;
 
-    for (j = 0; j < _class->n_groups; j++)
+    for (int j = 0; j < _class->n_groups; j++)
       BFT_FREE(_class->group_name[j]);
 
     _class->n_groups = 0;
