@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,6 +54,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro definitions
  *============================================================================*/
@@ -134,7 +136,7 @@ _local_to_global_extents(int         dim,
  *  true or false
  *----------------------------------------------------------------------------*/
 
-inline static _Bool
+inline static bool
 _a_ge_b(fvm_morton_code_t  code_a,
         fvm_morton_code_t  code_b)
 {
@@ -188,7 +190,7 @@ _a_ge_b(fvm_morton_code_t  code_a,
  *  true or false
  *----------------------------------------------------------------------------*/
 
-inline static _Bool
+inline static bool
 _a_gt_b(fvm_morton_code_t  code_a,
         fvm_morton_code_t  code_b)
 {
@@ -821,6 +823,8 @@ _bucket_sampling(int                      dim,
 
 #endif /* FM_HAVE_MPI */
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -864,7 +868,7 @@ fvm_morton_get_coord_extents(int               dim,
     for (j = 0; j < (size_t)dim; j++) {
       if (coords[i*dim + j] < g_extents[j])
         g_extents[j] = coords[i*dim + j];
-      else if (coords[i*dim + j] > g_extents[j + dim])
+      if (coords[i*dim + j] > g_extents[j + dim])
         g_extents[j + dim] = coords[i*dim + j];
     }
   }
@@ -950,7 +954,7 @@ fvm_morton_encode(int               dim,
   int  i;
   fvm_morton_code_t  morton_code;
 
-  fvm_morton_int_t  refinement = 1 << level;
+  fvm_morton_int_t  refinement = 1u << level;
 
   morton_code.L = level;
 
@@ -992,7 +996,7 @@ fvm_morton_encode_coords(int                dim,
   cs_coord_t s[3], d[3], n[3];
   cs_coord_t d_max = 0.0;
 
-  fvm_morton_int_t  refinement = 1 << level;
+  fvm_morton_int_t  refinement = 1u << level;
 
   for (i = 0; i < (size_t)dim; i++) {
     s[i] = extents[i];
@@ -1264,7 +1268,7 @@ fvm_morton_compare(int                dim,
  *  true or false
  *----------------------------------------------------------------------------*/
 
-_Bool
+bool
 fvm_morton_a_gt_b(fvm_morton_code_t  a,
                   fvm_morton_code_t  b)
 {
@@ -1282,7 +1286,7 @@ fvm_morton_a_gt_b(fvm_morton_code_t  a,
  *  true or false
  *----------------------------------------------------------------------------*/
 
-_Bool
+bool
 fvm_morton_a_ge_b(fvm_morton_code_t  a,
                   fvm_morton_code_t  b)
 {
@@ -1324,51 +1328,6 @@ fvm_morton_binary_search(cs_lnum_t           size,
   return start;
 }
 
-/*----------------------------------------------------------------------------
- * Get the quantile associated to a Morton code using a binary search.
- *
- * No check is done to ensure that the code is present in the quantiles.
- *
- * parameters:
- *   n_quantiles    <-- number of quantiles
- *   code           <-- code we are searching for
- *   quantile_start <-- first Morton code in each quantile (size: n_quantiles)
- *
- * returns:
- *   id associated to the given code in the codes array.
- *----------------------------------------------------------------------------*/
-
-size_t
-fvm_morton_quantile_search(size_t              n_quantiles,
-                           fvm_morton_code_t   code,
-                           fvm_morton_code_t  *quantile_start)
-{
-  size_t mid_id = 0;
-  size_t start_id = 0;
-  size_t end_id = n_quantiles;
-
-  /* use binary search */
-
-  while (start_id + 1 < end_id) {
-    mid_id = start_id + ((end_id -start_id) / 2);
-    if (_a_gt_b(quantile_start[mid_id], code))
-      end_id = mid_id;
-    else
-      start_id = mid_id;
-  }
-
-  /* We may have stopped short of the required value,
-     or have multiple occurences of a quantile start
-     (in case of empty quantiles), of which we want to
-     find the find highest one */
-
-  while (   start_id < n_quantiles - 1
-         && _a_ge_b(code, quantile_start[start_id+1]))
-    start_id++;
-
-  return start_id;
-}
-
 #if defined(HAVE_MPI)
 
 /*----------------------------------------------------------------------------
@@ -1460,6 +1419,64 @@ fvm_morton_build_rank_index(int                      dim,
 
 #endif /* HAVE_MPI */
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Function pointer for conversion of a double precision value in
+ * range [0, 1] to a given Morton code.
+ *
+ * \param[in]   s      coordinate between 0 and 1
+ * \param[out]  elt    pointer to element
+ * \param[in]   input  pointer to optional (untyped) value or structure;
+ *                     here, this is an interger representing the spatial
+ *                     dimension.
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+fvm_morton_s_to_code(double       s,
+                     void        *elt,
+                     const void  *input)
+{
+  const int level = sizeof(fvm_morton_int_t)*8 - 1;
+  const int dim = *((const int *)input);
+
+  fvm_morton_code_t  *hc = elt;
+
+  *hc = _double_to_code(dim, s, level);
+}
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Function pointer for comparison of 2 Morton codes.
+ *
+ * This function is the same type as that used by qsort_r.
+ *
+ * \param[in]  elt1   coordinate between 0 and 1
+ * \param[in]  elt2   pointer to optional (untyped) value or structure.
+ * \param[in]  input  pointer to optional (untyped) value or structure.
+ *
+ * \return < 0 if elt1 < elt2, 0 if elt1 == elt2, > 0 if elt1 > elt2
+ */
+/*----------------------------------------------------------------------------*/
+
+int
+fvm_morton_compare_o(const void  *elt1,
+                     const void  *elt2,
+                     const void  *input)
+{
+  CS_UNUSED(input);
+
+  int retval = 0;
+  if (_a_ge_b(*(const fvm_morton_code_t *)elt1,
+              *(const fvm_morton_code_t *)elt2) == false)
+    retval = -1;
+  else if (_a_gt_b(*(const fvm_morton_code_t *)elt1,
+                   *(const fvm_morton_code_t *)elt2) == true)
+    retval = 1;
+
+  return retval;
+}
+
 /*----------------------------------------------------------------------------
  * Dump a Morton to standard output or to a file.
  *
@@ -1475,7 +1492,7 @@ fvm_morton_dump(int                dim,
   int  i;
   double  coord[3];
 
-  const cs_gnum_t   n = 1 << code.L;
+  const cs_gnum_t   n = 1u << code.L;
   const double  stride = 1/(double)n;
 
   for (i = 0; i < dim; i++)
