@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -59,6 +59,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro and Type definitions
  *============================================================================*/
@@ -619,6 +621,8 @@ _sync_by_block(fvm_neighborhood_t  *n,
 
 #endif /* defined(HAVE_MPI) */
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -829,7 +833,7 @@ fvm_neighborhood_transfer_data(fvm_neighborhood_t   *n,
  * Determine intersecting boxes.
  *
  * Box global numbers and extents may be either copied for the structure's
- * internal use from the caller, or tranferred to the neighborhood management
+ * internal use from the caller, or transferred to the neighborhood management
  * structure: both the box_gnum and extents arguments have an "assigned"
  * variant, in which cas a pointer to a pointer is provided, and the
  * argument's property is transferred to the neighborhod management structure.
@@ -968,9 +972,10 @@ fvm_neighborhood_by_boxes(fvm_neighborhood_t  *n,
   n->n_elts = fvm_box_set_get_size(boxes);
 
   BFT_MALLOC(n->elt_num, n->n_elts, cs_gnum_t);
-  memcpy(n->elt_num,
-         fvm_box_set_get_g_num(boxes),
-         n->n_elts*sizeof(cs_gnum_t));
+  if (n->n_elts > 0)
+    memcpy(n->elt_num,
+           fvm_box_set_get_g_num(boxes),
+           n->n_elts*sizeof(cs_gnum_t));
 
   fvm_box_tree_get_intersects(bt,
                               boxes,
