@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -41,6 +41,7 @@
 
 #include "bft_mem.h"
 #include "bft_printf.h"
+#include "cs_mesh.h"
 
 #include "fvm_defs.h"
 #include "fvm_io_num.h"
@@ -59,6 +60,78 @@
 
 BEGIN_C_DECLS
 
+/*=============================================================================
+ * Additional doxygen documentation
+ *============================================================================*/
+
+/*!
+  \file fvm_nodal.c
+        Main structure for a nodal representation associated with a mesh.
+
+  The \c fvm_nodal_t structure is mostly used to handle post-processing
+  output in external formats, usually running in parallel using MPI.
+  This implies reconstructing a nodal connectivity (cells -> vertices) from
+  the main faces -> cells connectivity (using an intermediate
+  cells -> faces representation). It is also possible to directly
+  pass a nodal connectivity to such a structure.
+
+  So as to limit memory usage and avoid unecessary copies, the
+  \c fvm_nodal_t structure associated to a mesh defined by its nodal
+  connectivity is construed as a "view" on a mesh, and owns as little
+  data as possible. As such, most main structures associated with
+  this representation are defined by 2 arrays, one private, and one
+  shared. For example, an fvm_nodal_t structure has 2 coordinate
+  arrays:
+
+  <tt>const *cs_coord_t  vertex_coords;</tt>
+
+  <tt>*cs_coord_t       _vertex_coords;</tt>
+
+  If coordinates are shared with the calling code (and owned by that
+  code), we have  <tt>_vertex_coords = NULL</tt>, and \c vertex_coords
+  points to the array passed from the calling code. If the
+  coordinates array belongs to FVM (either having been "given" by
+  the calling code, or generated by an operation wich invalidates
+  coorindates sharing with the parent mesh, such as mesh extrusion),
+  we have <tt>vertex_coords = _vertex_coords</tt>, which points to the
+  private array.
+
+  When an fvm_nodal_t object is destroyed, it destroys its private
+  data and frees the corresponding memory, but obviously leaves its
+  shared data untouched.
+
+  If a \c fvm_nodal_t structure B is built from a structure A with
+  which it shares its data, and a second \c fvm_nodal_t mesh C
+  is a view on B (for example a restriction to a part of the domain
+  that we whish to post-process more frequently), C must be destroyed
+  before B, which must be destroyed before A.
+  FVM does not use reference counters or a similar mechanism, so
+  good managment of object lifecycles is of the responsiblity of
+  the calling code. In practice, this logic is simple enough that
+  no issues have been encountered with this model so far in the
+  intended uses of the code.
+
+  Another associated concept is that of "parent_number": if a mesh
+  constitutes a "view" on another, sur un autre, a list of
+  parent entity numbers allows accessing variables associated
+  with the "parent" mesh, without needing to extract or duplicate
+  values at the level of the calling code.
+
+  Note that an FVM structure is global from an MPI point of
+  view, as it may participate in collective parallel operations.
+  Thus, if an FVM mesh represents a subset of a global domain,
+  it may very well be empty for some ranks, but it must still exist
+  on all ranks of the main communicator associated with FVM.
+
+  For parallelism, another important concept is that of "global numbering",
+  corresponding to entity numbers in a "serial" or "global" version
+  of an object: two entities (vertices, elements, ...) on different
+  processors having a same global number correspond to the same
+  "absolute" object. Except when explicitely mentioned, all other
+  data defining an object is based on local definitions and numberings.
+  Parallelism is thus made quite "transparent" to the calling code.
+*/
+
 /*============================================================================
  * Static global variables
  *============================================================================*/
@@ -87,6 +160,8 @@ static int  fvm_nodal_n_edges_element[] = {1,   /* Edge */
                                            12,  /* Hexahedron */
                                            0};  /* Simple polyhedron */
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -165,6 +240,7 @@ _fvm_nodal_section_copy(const fvm_nodal_section_t *this_section)
   new_section->_vertex_num = NULL;
 
   new_section->gc_id = NULL;
+  new_section->tag = NULL;
 
   new_section->tesselation = NULL;  /* TODO: copy tesselation */
 
@@ -204,10 +280,10 @@ _fvm_nodal_section_copy(const fvm_nodal_section_t *this_section)
  *   true if connectivity has been reduced
  *----------------------------------------------------------------------------*/
 
-static _Bool
+static bool
 _fvm_nodal_section_reduce(fvm_nodal_section_t  * this_section)
 {
-  _Bool retval = false;
+  bool retval = false;
 
   /* If we have a tesselation of polyhedra (face index != NULL),
      we may need to keep the connectivity information, to
@@ -242,6 +318,9 @@ _fvm_nodal_section_reduce(fvm_nodal_section_t  * this_section)
   if (this_section->gc_id != NULL)
     BFT_FREE(this_section->gc_id);
 
+  if (this_section->tag != NULL)
+    BFT_FREE(this_section->tag);
+
   if (this_section->tesselation != NULL)
     fvm_tesselation_reduce(this_section->tesselation);
 
@@ -277,7 +356,7 @@ _renumber_parent_num(cs_lnum_t          parent_num_size,
   int  i;
   cs_lnum_t   old_num_id;
   cs_lnum_t *parent_num_p = _parent_num;
-  _Bool trivial = true;
+  bool trivial = true;
 
   if (parent_num_size > 0 && new_parent_num != NULL) {
 
@@ -540,13 +619,15 @@ _fvm_nodal_section_dump(const fvm_nodal_section_t  *this_section)
              "  _vertex_index:        %p\n"
              "  _vertex_num:          %p\n"
              "  _parent_element_num:  %p\n"
-             "  gc_id:                %p\n",
+             "  gc_id:                %p\n"
+             "  tag:                  %p\n",
              (const void *)this_section->_face_index,
              (const void *)this_section->_face_num,
              (const void *)this_section->_vertex_index,
              (const void *)this_section->_vertex_num,
              (const void *)this_section->_parent_element_num,
-             (const void *)this_section->gc_id);
+             (const void *)this_section->gc_id,
+             (const void *)this_section->tag);
 
   if (this_section->face_index != NULL) {
     bft_printf("\nPolyhedra -> faces (polygons) connectivity:\n\n");
@@ -633,6 +714,13 @@ _fvm_nodal_section_dump(const fvm_nodal_section_t  *this_section)
     bft_printf("\n");
   }
 
+  if (this_section->tag != NULL) {
+    bft_printf("\nTags:\n\n");
+    for (i = 0; i < this_section->n_elements; i++)
+      bft_printf("%10d : %10d\n", i + 1, this_section->tag[i]);
+    bft_printf("\n");
+  }
+
   /* Faces tesselation */
 
   if (this_section->tesselation != NULL)
@@ -656,6 +744,30 @@ _fvm_nodal_section_dump(const fvm_nodal_section_t  *this_section)
   }
 }
 
+/*----------------------------------------------------------------------------
+ * Remove global vertex labels from a nodal mesh.
+ *
+ * parameters:
+ *   this_nodal <-> nodal mesh structure
+ *----------------------------------------------------------------------------*/
+
+static void
+_remove_global_vertex_labels(fvm_nodal_t  *this_nodal)
+{
+  assert(this_nodal != NULL);
+
+  if (this_nodal->global_vertex_labels != NULL) {
+
+    cs_gnum_t n_g_vertices = fvm_nodal_n_g_vertices(this_nodal);
+
+    for (cs_gnum_t i = 0; i < n_g_vertices; i++)
+      BFT_FREE(this_nodal->global_vertex_labels[i]);
+
+    BFT_FREE(this_nodal->global_vertex_labels);
+
+  }
+}
+
 /*============================================================================
  * Semi-private function definitions (prototypes in fvm_nodal_priv.h)
  *============================================================================*/
@@ -710,6 +822,7 @@ fvm_nodal_section_create(const fvm_element_t  type)
   this_section->_vertex_num = NULL;
 
   this_section->gc_id = NULL;
+  this_section->tag = NULL;
 
   this_section->tesselation = NULL;
 
@@ -752,6 +865,9 @@ fvm_nodal_section_destroy(fvm_nodal_section_t  * this_section)
   if (this_section->gc_id != NULL)
     BFT_FREE(this_section->gc_id);
 
+  if (this_section->tag != NULL)
+    BFT_FREE(this_section->tag);
+
   if (this_section->tesselation != NULL)
     fvm_tesselation_destroy(this_section->tesselation);
 
@@ -787,10 +903,10 @@ fvm_nodal_section_destroy(fvm_nodal_section_t  * this_section)
 
 void
 fvm_nodal_section_copy_on_write(fvm_nodal_section_t  *this_section,
-                                _Bool                 copy_face_index,
-                                _Bool                 copy_face_num,
-                                _Bool                 copy_vertex_index,
-                                _Bool                 copy_vertex_num)
+                                bool                  copy_face_index,
+                                bool                  copy_face_num,
+                                bool                  copy_vertex_index,
+                                bool                  copy_vertex_num)
 {
   cs_lnum_t   n_faces;
   size_t  i;
@@ -995,6 +1111,8 @@ fvm_nodal_cell_face_connect(fvm_element_t   element_type,
   }
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -1053,6 +1171,10 @@ fvm_nodal_create(const char  *name,
 
   this_nodal->gc_set = NULL;
 
+  this_nodal->global_vertex_labels = NULL;
+
+  this_nodal->parent = NULL;
+
   return (this_nodal);
 }
 
@@ -1067,12 +1189,17 @@ fvm_nodal_create(const char  *name,
  *----------------------------------------------------------------------------*/
 
 fvm_nodal_t *
-fvm_nodal_destroy(fvm_nodal_t  * this_nodal)
+fvm_nodal_destroy(fvm_nodal_t   *this_nodal)
 {
   int           i;
 
+  if (this_nodal == NULL)
+    return NULL;
+
   /* Local structures */
 
+  _remove_global_vertex_labels(this_nodal);
+
   if (this_nodal->name != NULL)
     BFT_FREE(this_nodal->name);
 
@@ -1175,6 +1302,7 @@ fvm_nodal_copy(const fvm_nodal_t *this_nodal)
     new_nodal->sections[i] = _fvm_nodal_section_copy(this_nodal->sections[i]);
 
   new_nodal->gc_set = NULL;
+  new_nodal->global_vertex_labels = NULL;
 
   return (new_nodal);
 }
@@ -1198,7 +1326,7 @@ fvm_nodal_reduce(fvm_nodal_t  *this_nodal,
                  int           del_vertex_num)
 {
   int  i;
-  _Bool reduce_vertices = true;
+  bool reduce_vertices = true;
 
   /* Connectivity */
 
@@ -1355,12 +1483,14 @@ fvm_nodal_init_io_num(fvm_nodal_t       *this_nodal,
   int  i;
   fvm_nodal_section_t  *section;
 
-  if (entity_dim == 0)
+  if (entity_dim == 0) {
     this_nodal->global_vertex_num
       = fvm_io_num_create(this_nodal->parent_vertex_num,
                           parent_global_numbers,
                           this_nodal->n_vertices,
                           0);
+    _remove_global_vertex_labels(this_nodal);
+  }
 
   else {
     for (i = 0; i < this_nodal->n_sections; i++) {
@@ -1377,6 +1507,83 @@ fvm_nodal_init_io_num(fvm_nodal_t       *this_nodal,
 
 }
 
+/*----------------------------------------------------------------------------
+ * Transfer an existing global numbering structure to a nodal mesh.
+ *
+ * This assumes the local number of vertices is identical to that of
+ * the numberign structure.
+ *
+ * The argument pointer to the global numbering structure is set to NULL
+ * so the caller does not spuriously modify the structure after this call.
+ *
+ * A call to this function may be used instead of fvm_nodal_init_io_num
+ * (for vertices only).
+ *
+ * parameters:
+ *   this_nodal    <-- nodal mesh structure
+ *   io_num        <-> pointer to external numbering structure
+ *                     whose property is transferred to the mesh
+ *                     independent) parent entity numbers
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_transfer_vertex_io_num(fvm_nodal_t    *this_nodal,
+                                 fvm_io_num_t  **io_num)
+{
+  this_nodal->global_vertex_num = *io_num;
+  *io_num = NULL;
+  _remove_global_vertex_labels(this_nodal);
+}
+
+/*----------------------------------------------------------------------------
+ * Set entity tags (for non-vertex entities).
+ *
+ * The number of entities of the given dimension may be obtained
+ * through fvm_nodal_get_n_entities(), the tag[] array is populated
+ * in local section order, section by section).
+ *
+ * parameters:
+ *   this_nodal <-- nodal mesh structure
+ *   tag        <-- tag values to assign
+ *   entity_dim <-- 3 for cells, 2 for faces, 1 for edges
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_set_tag(fvm_nodal_t  *this_nodal,
+                  const int     tag[],
+                  int           entity_dim)
+{
+  cs_lnum_t entitiy_count = 0;
+  for (int i = 0; i < this_nodal->n_sections; i++) {
+    fvm_nodal_section_t  *section = this_nodal->sections[i];
+    if (section->entity_dim == entity_dim) {
+      BFT_REALLOC(section->tag, section->n_elements, int);
+      for (cs_lnum_t j = 0; j < section->n_elements; j++)
+        section->tag[j] = tag[entitiy_count + j];
+      entitiy_count += section->n_elements;
+    }
+  }
+}
+
+/*----------------------------------------------------------------------------
+ * Remove entity tags.
+ *
+ * parameters:
+ *   this_nodal <-- nodal mesh structure
+ *   entity_dim <-- 3 for cells, 2 for faces, 1 for edges
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_remove_tag(fvm_nodal_t  *this_nodal,
+                     int           entity_dim)
+{
+  for (int i = 0; i < this_nodal->n_sections; i++) {
+    fvm_nodal_section_t  *section = this_nodal->sections[i];
+    if (section->entity_dim == entity_dim)
+      BFT_FREE(section->tag);
+  }
+}
+
 /*----------------------------------------------------------------------------
  * Preset number and list of vertices to assign to a nodal mesh.
  *
@@ -1415,6 +1622,8 @@ fvm_nodal_define_vertex_list(fvm_nodal_t  *this_nodal,
     this_nodal->_parent_vertex_num = parent_vertex_num;
     this_nodal->parent_vertex_num = parent_vertex_num;
   }
+
+  _remove_global_vertex_labels(this_nodal);
 }
 
 /*----------------------------------------------------------------------------
@@ -1453,6 +1662,7 @@ fvm_nodal_set_shared_vertices(fvm_nodal_t       *this_nodal,
 
   _renumber_vertices(this_nodal);
 
+  _remove_global_vertex_labels(this_nodal);
 }
 
 /*----------------------------------------------------------------------------
@@ -1516,6 +1726,8 @@ fvm_nodal_transfer_vertices(fvm_nodal_t  *this_nodal,
   this_nodal->_vertex_coords = _vertex_coords;
   this_nodal->vertex_coords = _vertex_coords;
 
+  _remove_global_vertex_labels(this_nodal);
+
   return _vertex_coords;
 }
 
@@ -1523,7 +1735,7 @@ fvm_nodal_transfer_vertices(fvm_nodal_t  *this_nodal,
  * Make vertex coordinates of a nodal mesh private.
  *
  * If vertex coordinates were previously shared, those coordinates that
- * are actually refernces are copied, and the relation to parent vertices
+ * are actually references are copied, and the relation to parent vertices
  * is discarded.
  *
  * If vertices were already private, the mesh is not modified.
@@ -1544,7 +1756,7 @@ fvm_nodal_make_vertices_private(fvm_nodal_t  *this_nodal)
     const cs_lnum_t n_vertices = this_nodal->n_vertices;
     const int dim = this_nodal->dim;
 
-    BFT_MALLOC(vertex_coords, n_vertices * dim, cs_coord_t);
+    BFT_MALLOC(_vertex_coords, n_vertices * dim, cs_coord_t);
 
     /* If renumbering is necessary, update connectivity */
 
@@ -1664,6 +1876,37 @@ fvm_nodal_set_group_class_set(fvm_nodal_t                  *this_nodal,
   BFT_FREE(gc_renum);
 }
 
+/*----------------------------------------------------------------------------
+ * Assign global vertex labels to a nodal mesh.
+ *
+ * As these are expected to be used only for small sets (i.e. probes)
+ * where the point set is built from a global definition and data movement
+ * would add complexity and overhead, the labels refer to a global view
+ * on rank 0.
+ *
+ * The size of the labels pointers array should be the same as that
+ * returned by fvm_nodal_n_g_vertices();
+ *
+ * This function should only be called once the nodal mesh representation
+ * has been completed, as most functions modifying its vertex definitions
+ * will remove these labels.
+ *
+ * parameters:
+ *   this_nodal <-> nodal mesh structure
+ *   g_labels   <-- global vertex labels, or NULL
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_transfer_global_vertex_labels(fvm_nodal_t  *this_nodal,
+                                        char         *g_labels[])
+{
+  assert(this_nodal != NULL);
+
+  _remove_global_vertex_labels(this_nodal);
+
+  this_nodal->global_vertex_labels = g_labels;
+}
+
 /*----------------------------------------------------------------------------
  * Obtain the name of a nodal mesh.
  *
@@ -1902,6 +2145,63 @@ fvm_nodal_get_parent_num(const fvm_nodal_t  *this_nodal,
   }
 }
 
+/*----------------------------------------------------------------------------
+ * Return pointer to global vertex labels of a nodal mesh.
+ *
+ * As these are expected to be used only for small sets (i.e. probes)
+ * where the point set is built from a global definition and data movement
+ * would adds complexity and overhead, the labels refer to a global view
+ * on rank 0; for the same reason, only shared labels are needed.
+ *
+ * The size of the labels pointers array returned should be the same
+ * as that returned by fvm_nodal_n_g_vertices();
+ *
+ * parameters:
+ *   this_nodal <-> nodal mesh structure
+ *
+ * returns:
+ *   pointer to global vertex labels, or NULL
+ *----------------------------------------------------------------------------*/
+
+const char **
+fvm_nodal_get_global_vertex_labels(const fvm_nodal_t  *this_nodal)
+{
+  assert(this_nodal != NULL);
+
+  return (const char **)(this_nodal->global_vertex_labels);
+}
+
+/*----------------------------------------------------------------------------
+ * Return a const pointer to a nodal mesh representation's parent mesh.
+ *
+ * parameters:
+ *   this_nodal <-> pointer to structure that should be reduced
+ *
+ * return:
+ *   const pointer to parent mesh
+ *----------------------------------------------------------------------------*/
+
+const cs_mesh_t  *
+fvm_nodal_get_parent(const fvm_nodal_t  *this_nodal)
+{
+  return this_nodal->parent;
+}
+
+/*----------------------------------------------------------------------------
+ * Associate a parent mesh to a nodal mesh representation structure.
+ *
+ * parameters:
+ *   this_nodal <-> pointer to structure that should be reduced
+ *   parent     <-- const pointer to parent mesh
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_set_parent(fvm_nodal_t      *this_nodal,
+                     const cs_mesh_t  *parent)
+{
+  this_nodal->parent = parent;
+}
+
 /*----------------------------------------------------------------------------
  * Compute tesselation a a nodal mesh's sections of a given type, and add the
  * corresponding structure to the mesh representation.
@@ -2391,9 +2691,26 @@ fvm_nodal_dump(const fvm_nodal_t  *this_nodal)
   for (i = 0; i < this_nodal->n_sections; i++)
     _fvm_nodal_section_dump(this_nodal->sections[i]);
 
+  /* Dump metadata */
+  /*---------------*/
+
   /* Dump group class set (NULL allowed) */
 
   fvm_group_class_set_dump(this_nodal->gc_set);
+
+  /* Dump vertex labels */
+
+  if (this_nodal->global_vertex_labels != NULL) {
+    bft_printf("\nGlobal vertex labels:\n");
+    cs_lnum_t n_labels = (cs_lnum_t)fvm_nodal_n_g_vertices(this_nodal);
+    for (i = 0; i < n_labels; i++) {
+      if (this_nodal->global_vertex_labels[i] != NULL)
+        bft_printf("%10d : %s\n",
+                   i+1, this_nodal->global_vertex_labels[i]);
+      else
+        bft_printf("%10d : (null)\n", i+1);
+    }
+  }
 }
 
 /*----------------------------------------------------------------------------*/
