@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,7 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
+#include "cs_mesh.h"
 #include "fvm_defs.h"
 #include "fvm_group.h"
 #include "fvm_io_num.h"
@@ -179,6 +180,59 @@ fvm_nodal_init_io_num(fvm_nodal_t        *this_nodal,
                       const cs_gnum_t     parent_global_numbers[],
                       int                 entity_dim);
 
+/*----------------------------------------------------------------------------
+ * Transfer an existing global numbering structure to a nodal mesh.
+ *
+ * This assumes the local number of vertices is identical to that of
+ * the numberign structure.
+ *
+ * The argument pointer to the global numbering structure is set to NULL
+ * so the caller does not spuriously modify the structure after this call.
+ *
+ * A call to this function may be used instead of fvm_nodal_init_io_num
+ * (for vertices only).
+ *
+ * parameters:
+ *   this_nodal    <-- nodal mesh structure
+ *   io_num        <-> pointer to external numbering structure
+ *                     whose property is transferred to the mesh
+ *                     independent) parent entity numbers
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_transfer_vertex_io_num(fvm_nodal_t    *this_nodal,
+                                 fvm_io_num_t  **io_num);
+
+/*----------------------------------------------------------------------------
+ * Set entity tags (for non-vertex entities).
+ *
+ * The number of entities of the given dimension may be obtained
+ * through fvm_nodal_get_n_entities(), the tag[] array is populated
+ * in local section order, section by section).
+ *
+ * parameters:
+ *   this_nodal <-- nodal mesh structure
+ *   tag        <-- tag values to assign
+ *   entity_dim <-- 3 for cells, 2 for faces, 1 for edges
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_set_tag(fvm_nodal_t  *this_nodal,
+                  const int     tag[],
+                  int           entity_dim);
+
+/*----------------------------------------------------------------------------
+ * Remove entity tags.
+ *
+ * parameters:
+ *   this_nodal <-- nodal mesh structure
+ *   entity_dim <-- 3 for cells, 2 for faces, 1 for edges
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_remove_tag(fvm_nodal_t  *this_nodal,
+                     int           entity_dim);
+
 /*----------------------------------------------------------------------------
  * Preset number and list of vertices to assign to a nodal mesh.
  *
@@ -250,7 +304,7 @@ fvm_nodal_transfer_vertices(fvm_nodal_t  *this_nodal,
  * Make vertex coordinates of a nodal mesh private.
  *
  * If vertex coordinates were previously shared, those coordinates that
- * are actually refernces are copied, and the relation to parent vertices
+ * are actually references are copied, and the relation to parent vertices
  * is discarded.
  *
  * If vertices were already private, the mesh is not modified.
@@ -281,6 +335,30 @@ void
 fvm_nodal_set_group_class_set(fvm_nodal_t                  *this_nodal,
                               const fvm_group_class_set_t  *gc_set);
 
+/*----------------------------------------------------------------------------
+ * Assign global vertex labels to a nodal mesh.
+ *
+ * As these are expected to be used only for small sets (i.e. probes)
+ * where the point set is built from a global definition and data movement
+ * would add complexity and overhead, the labels refer to a global view
+ * on rank 0.
+ *
+ * The size of the labels pointers array should be the same as that
+ * returned by fvm_nodal_n_g_vertices();
+ *
+ * This function should only be called once the nodal mesh representation
+ * has been completed, as most functions modifying its vertex definitions
+ * will remove these labels.
+ *
+ * parameters:
+ *   this_nodal <-> nodal mesh structure
+ *   g_labels   <-- global vertex labels, or NULL
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_transfer_global_vertex_labels(fvm_nodal_t  *this_nodal,
+                                        char         *g_labels[]);
+
 /*----------------------------------------------------------------------------
  * Obtain the name of a nodal mesh.
  *
@@ -398,6 +476,52 @@ fvm_nodal_get_parent_num(const fvm_nodal_t  *this_nodal,
                          int                 entity_dim,
                          cs_lnum_t           parent_num[]);
 
+/*----------------------------------------------------------------------------
+ * Return pointer to global vertex labels of a nodal mesh.
+ *
+ * As these are expected to be used only for small sets (i.e. probes)
+ * where the point set is built from a global definition and data movement
+ * would adds complexity and overhead, the labels refer to a global view
+ * on rank 0; for the same reason, only shared labels are needed.
+ *
+ * The size of the labels pointers array returned should be the same
+ * as that returned by fvm_nodal_n_g_vertices();
+ *
+ * parameters:
+ *   this_nodal <-> nodal mesh structure
+ *
+ * returns:
+ *   pointer to global vertex labels, or NULL
+ *----------------------------------------------------------------------------*/
+
+const char **
+fvm_nodal_get_global_vertex_labels(const fvm_nodal_t  *this_nodal);
+
+/*----------------------------------------------------------------------------
+ * Return a const pointer to a nodal mesh representation's parent mesh.
+ *
+ * parameters:
+ *   this_nodal <-> pointer to structure that should be reduced
+ *
+ * return:
+ *   const pointer to parent mesh
+ *----------------------------------------------------------------------------*/
+
+const cs_mesh_t  *
+fvm_nodal_get_parent(const fvm_nodal_t  *this_nodal);
+
+/*----------------------------------------------------------------------------
+ * Associate a parent mesh to a nodal mesh representation structure.
+ *
+ * parameters:
+ *   this_nodal <-> pointer to structure that should be reduced
+ *   parent     <-- const pointer to parent mesh
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_nodal_set_parent(fvm_nodal_t      *this_nodal,
+                     const cs_mesh_t  *parent);
+
 /*----------------------------------------------------------------------------
  * Compute tesselation a a nodal mesh's sections of a given type, and add the
  * corresponding structure to the mesh representation.
