@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -61,6 +61,8 @@ BEGIN_C_DECLS
  * Static global variables
  *============================================================================*/
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -125,6 +127,11 @@ _extrude_strided_section(fvm_nodal_section_t  * this_section,
 
   this_section->connectivity_size = connectivity_size;
 
+  /* Remove old attributes */
+
+  BFT_FREE(this_section->gc_id);
+  BFT_FREE(this_section->tag);
+
   /* Remove old parent numbering */
 
   this_section->parent_element_num = NULL;
@@ -183,6 +190,8 @@ _extrude_strided_section(fvm_nodal_section_t  * this_section,
   this_section->stride *=2;
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -198,7 +207,7 @@ _extrude_strided_section(fvm_nodal_section_t  * this_section,
  *       {e_1_layer_1, ..., e_1_layer_m, e_2_layer_1, ... e_n_layer_m}
  *
  * parameters:
- *   this_section      <-> pointer to structure that should be extruded
+ *   this_nodal       <-> pointer to structure that should be extruded
  *   n_layers          <-> number of extruded layers
  *   extrusion_vectors <-> length and direction of extrusion for each vertex;
  *                         size: mesh_spatial_dim . n_vertices
