@@ -6,7 +6,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -60,6 +60,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Enumeration definitions
  *============================================================================*/
@@ -102,7 +104,7 @@ typedef enum {
  *   face_list_shift <-- face list to common number index shifts;
  *                       size: n_face_lists
  *   face_vertex_idx <-- face -> vertex indexes (per face list)
- *   face_vertex_num <-- face -> vertex numbers (per face list)
+ *   face_vertex     <-- face -> vertex ids (per face list)
  *   cell_face_idx   <-- cell -> face indexes (1 to n)
  *   cell_face_num   <-- cell -> face numbers (1 to n)
  *
@@ -115,7 +117,7 @@ _is_prism_or_poly(const cs_lnum_t    cell_id,
                   const int          n_face_lists,
                   const cs_lnum_t    face_list_shift[],
                   const cs_lnum_t   *face_vertex_idx[],
-                  const cs_lnum_t   *face_vertex_num[],
+                  const cs_lnum_t   *face_vertex[],
                   const cs_lnum_t    cell_face_idx[],
                   const cs_lnum_t    cell_face_num[])
 {
@@ -141,8 +143,8 @@ _is_prism_or_poly(const cs_lnum_t    cell_id,
     assert(fl > -1);
     face_id -= face_list_shift[fl];
 
-    vertex_id_start = face_vertex_idx[fl][face_id] - 1;
-    vertex_id_end   = face_vertex_idx[fl][face_id + 1] - 1;
+    vertex_id_start = face_vertex_idx[fl][face_id];
+    vertex_id_end   = face_vertex_idx[fl][face_id + 1];
     n_face_vertices = vertex_id_end - vertex_id_start;
 
     if (n_face_vertices == 3) {
@@ -150,12 +152,12 @@ _is_prism_or_poly(const cs_lnum_t    cell_id,
       if (cell_face_num[idx] > 0) {
         for (vtx = 0 ; vtx < 3 ; vtx++)
           vtx_tria[n_trias*3 + vtx]
-            = face_vertex_num[fl][vertex_id_start + vtx];
+            = face_vertex[fl][vertex_id_start + vtx] + 1;
       }
       else {
         for (vtx = 0 ; vtx < 3 ; vtx++)
           vtx_tria[n_trias*3 + vtx]
-            = face_vertex_num[fl][vertex_id_end - 1 - vtx];
+            = face_vertex[fl][vertex_id_end - 1 - vtx] + 1;
       }
 
       n_trias += 1;
@@ -199,7 +201,7 @@ _is_prism_or_poly(const cs_lnum_t    cell_id,
  *   face_list_shift <-- face list to common number index shifts;
  *                       size: n_face_lists
  *   face_vertex_idx <-- face -> vertex indexes (per face list)
- *   face_vertex_num <-- face -> vertex numbers (per face list)
+ *   face_vertex     <-- face -> vertex ids (per face list)
  *   cell_face_idx   <-- cell -> face indexes (1 to n)
  *   cell_face_num   <-- cell -> face numbers (1 to n)
  *   cell_vtx_tria   --> local triangle definitions (optional, 4 max.)
@@ -214,7 +216,7 @@ _nodal_cell_from_desc(const cs_lnum_t    cell_id,
                       const int          n_face_lists,
                       const cs_lnum_t    face_list_shift[],
                       const cs_lnum_t   *face_vertex_idx[],
-                      const cs_lnum_t   *face_vertex_num[],
+                      const cs_lnum_t   *face_vertex[],
                       const cs_lnum_t    cell_face_idx[],
                       const cs_lnum_t    cell_face_num[],
                       cs_lnum_t         *cell_vtx_tria,
@@ -261,8 +263,8 @@ _nodal_cell_from_desc(const cs_lnum_t    cell_id,
     assert(fl > -1);
     face_id -= face_list_shift[fl];
 
-    vertex_id_start = face_vertex_idx[fl][face_id] - 1;
-    vertex_id_end   = face_vertex_idx[fl][face_id + 1] - 1;
+    vertex_id_start = face_vertex_idx[fl][face_id];
+    vertex_id_end   = face_vertex_idx[fl][face_id + 1];
     n_face_vertices = vertex_id_end - vertex_id_start;
 
     if (n_face_vertices == 3) {
@@ -271,12 +273,12 @@ _nodal_cell_from_desc(const cs_lnum_t    cell_id,
         if (cell_face_num[idx] > 0) {
           for (vtx = 0 ; vtx < n_face_vertices ; vtx++)
             cell_vtx_tria[n_trias*3 + vtx]
-              = face_vertex_num[fl][vertex_id_start + vtx];
+              = face_vertex[fl][vertex_id_start + vtx] + 1;
         }
         else {
           for (vtx = 0 ; vtx < n_face_vertices ; vtx++)
             cell_vtx_tria[n_trias*3 + vtx]
-              = face_vertex_num[fl][vertex_id_end - 1 - vtx];
+              = face_vertex[fl][vertex_id_end - 1 - vtx] + 1;
         }
       }
 
@@ -289,12 +291,12 @@ _nodal_cell_from_desc(const cs_lnum_t    cell_id,
         if (cell_face_num[idx] > 0) {
           for (vtx = 0 ; vtx < n_face_vertices ; vtx++)
             cell_vtx_quad[n_quads*4 + vtx]
-              = face_vertex_num[fl][vertex_id_start + vtx];
+              = face_vertex[fl][vertex_id_start + vtx] + 1;
         }
         else {
           for (vtx = 0 ; vtx < n_face_vertices ; vtx++)
             cell_vtx_quad[n_quads*4 + vtx]
-              = face_vertex_num[fl][vertex_id_end - 1 - vtx];
+              = face_vertex[fl][vertex_id_end - 1 - vtx] + 1;
         }
       }
 
@@ -318,7 +320,7 @@ _nodal_cell_from_desc(const cs_lnum_t    cell_id,
                                     n_face_lists,
                                     face_list_shift,
                                     face_vertex_idx,
-                                    face_vertex_num,
+                                    face_vertex,
                                     cell_face_idx,
                                     cell_face_num);
     else if (n_trias == 4) {
@@ -353,7 +355,7 @@ _nodal_from_desc_cnv_cel_tetra(const cs_lnum_t   cell_vtx_tria[],
   cs_lnum_t   direction;
   cs_lnum_t   vtx_num, vtx_num_1, vtx_num_2;
 
-  _Bool  warn_orient = false;
+  bool  warn_orient = false;
 
 #if 0 && defined(DEBUG) && !defined(NDEBUG)
   bft_printf("face 1 : %d %d %d\n",
@@ -461,7 +463,7 @@ _nodal_from_desc_cnv_cel_pyram(const cs_lnum_t   cell_vtx_tria[],
   cs_lnum_t   direction;
   cs_lnum_t   vtx_num, vtx_num_1, vtx_num_2;
 
-  _Bool  warn_orient = false;
+  bool  warn_orient = false;
 
 #if 0 && defined(DEBUG) && !defined(NDEBUG)
   bft_printf("face 1 : %d %d %d %d\n",
@@ -572,7 +574,7 @@ _nodal_from_desc_cnv_cel_prism(const cs_lnum_t   cell_vtx_tria[],
   cs_lnum_t   ipass, direction;
   cs_lnum_t   vtx_num, vtx_num_1, vtx_num_2;
 
-  _Bool  warn_orient = false;
+  bool  warn_orient = false;
 
 #if 0 && defined(DEBUG) && !defined(NDEBUG)
   bft_printf("face 1 : %d %d %d\n",
@@ -691,7 +693,7 @@ _nodal_from_desc_cnv_cel_hexa(const cs_lnum_t   cell_vtx_quad[],
   cs_lnum_t   ipass, direction;
   cs_lnum_t   vtx_num, vtx_num_1, vtx_num_2;
 
-  _Bool  warn_orient = false;
+  bool  warn_orient = false;
 
 #if 0 && defined(DEBUG) && !defined(NDEBUG)
   bft_printf("face 1 : %d %d %d %d\n",
@@ -833,8 +835,8 @@ _nodal_face_from_desc_size(const cs_lnum_t    face_id,
   assert(fl > -1);
   _face_id -= face_list_shift[fl];
 
-  vertex_id_start = face_vertex_idx[fl][_face_id] - 1;
-  vertex_id_end   = face_vertex_idx[fl][_face_id + 1] - 1;
+  vertex_id_start = face_vertex_idx[fl][_face_id];
+  vertex_id_end   = face_vertex_idx[fl][_face_id + 1];
   n_face_vertices = vertex_id_end - vertex_id_start;
 
   return n_face_vertices;
@@ -842,9 +844,9 @@ _nodal_face_from_desc_size(const cs_lnum_t    face_id,
 }
 
 /*----------------------------------------------------------------------------
- * Copy the vertex numbers defining a given face.
+ * Copy the vertex ids defining a given face.
  *
- * The face_vtx[] array is filled with the vertex numbers of the face,
+ * The face_vtx[] array is filled with the vertex ids of the face,
  * and should be large enough to receive them.
  *
  * parameters:
@@ -853,7 +855,7 @@ _nodal_face_from_desc_size(const cs_lnum_t    face_id,
  *   face_list_shift <-- face list to common number index shifts;
  *                       size: n_face_lists
  *   face_vertex_idx <-- face -> vertex indexes (per face list)
- *   face_vertex_num <-- face -> vertex numbers (per face list)
+ *   face_vertex     <-- face -> vertex ids (per face list)
  *   face_vtx        --> local face definition
  *----------------------------------------------------------------------------*/
 
@@ -862,27 +864,27 @@ _nodal_face_from_desc_copy(const cs_lnum_t    face_id,
                            const int          n_face_lists,
                            const cs_lnum_t    face_list_shift[],
                            const cs_lnum_t   *face_vertex_idx[],
-                           const cs_lnum_t   *face_vertex_num[],
+                           const cs_lnum_t   *face_vertex[],
                            cs_lnum_t         *face_vtx)
 {
   cs_lnum_t   fl, _face_id;
   cs_lnum_t   vtx, vertex_id, vertex_id_start, vertex_id_end;
 
-  /* Copy vertex numbers */
-  /*---------------------*/
+  /* Copy vertex ids */
+  /*-----------------*/
 
   _face_id = face_id;
   for (fl = n_face_lists - 1 ; _face_id < face_list_shift[fl] ; fl--);
   assert(fl > -1);
   _face_id -= face_list_shift[fl];
 
-  vertex_id_start = face_vertex_idx[fl][_face_id] - 1;
-  vertex_id_end   = face_vertex_idx[fl][_face_id + 1] - 1;
+  vertex_id_start = face_vertex_idx[fl][_face_id];
+  vertex_id_end   = face_vertex_idx[fl][_face_id + 1];
 
   for (vtx = 0, vertex_id = vertex_id_start ;
        vertex_id < vertex_id_end ;
        vtx++, vertex_id++)
-    face_vtx[vtx] = face_vertex_num[fl][vertex_id];
+    face_vtx[vtx] = face_vertex[fl][vertex_id] + 1;
 }
 
 /*----------------------------------------------------------------------------
@@ -896,7 +898,7 @@ _nodal_face_from_desc_copy(const cs_lnum_t    face_id,
  *   face_list_shift <-- face list to common number number index shifts;
  *                       size: n_face_lists
  *   face_vertex_idx <-- face -> vertex indexes (per face list)
- *   face_vertex_num <-- face -> vertex numbers (per face list)
+ *   face_vertex     <-- face -> vertex ids (per face list)
  *   cell_face_idx   <-- cell -> face indexes (1 to n)
  *   cell_face_num   <-- cell -> face numbers (1 to n)
  *   cell_face_list  --> numbers of faces defining polyhedra
@@ -909,7 +911,7 @@ _fvm_nodal_extract_polyhedra(fvm_nodal_section_t  *this_section,
                              const int             n_face_lists,
                              const cs_lnum_t       face_list_shift[],
                              const cs_lnum_t      *face_vertex_idx[],
-                             const cs_lnum_t      *face_vertex_num[],
+                             const cs_lnum_t      *face_vertex[],
                              const cs_lnum_t       cell_face_idx[],
                              const cs_lnum_t       cell_face_num[],
                              cs_lnum_t            *cell_face_list[])
@@ -1059,11 +1061,11 @@ _fvm_nodal_extract_polyhedra(fvm_nodal_section_t  *this_section,
       assert(fl < n_face_lists);
       face_id -= face_list_shift[fl];
 
-      for (i = face_vertex_idx[fl][face_id] - 1 ;
-           i < face_vertex_idx[fl][face_id + 1] - 1 ;
+      for (i = face_vertex_idx[fl][face_id];
+           i < face_vertex_idx[fl][face_id + 1];
            i++)
         this_section->_vertex_num[c_cell_face_vertex_nums++]
-          = face_vertex_num[fl][i];
+          = face_vertex[fl][i] + 1;
 
       c_cell_face_vertex_idxs += 1;
       this_section->_vertex_index[c_cell_face_vertex_idxs]
@@ -1237,6 +1239,8 @@ _fvm_nodal_add_sections(fvm_nodal_t          *this_nodal,
 
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -1263,7 +1267,7 @@ _fvm_nodal_add_sections(fvm_nodal_t          *this_nodal,
  *   face_list_shift <-- face list to common number index shifts;
  *                       size: n_face_lists + 1
  *   face_vertex_idx <-- face -> vertex indexes (per face list)
- *   face_vertex_num <-- face -> vertex numbers (per face list)
+ *   face_vertex     <-- face -> vertex ids (per face list)
  *   cell_face_idx   <-- cell -> face indexes (1 to n)
  *   cell_face_num   <-- cell -> face numbers (1 to n)
  *   cell_gc_id      <-- cell -> group class ids, or NULL
@@ -1280,7 +1284,7 @@ fvm_nodal_from_desc_add_cells(fvm_nodal_t        *this_nodal,
                               const int           n_face_lists,
                               const cs_lnum_t     face_list_shift[],
                               const cs_lnum_t    *face_vertex_idx[],
-                              const cs_lnum_t    *face_vertex_num[],
+                              const cs_lnum_t    *face_vertex[],
                               const cs_lnum_t     cell_face_idx[],
                               const cs_lnum_t     cell_face_num[],
                               const int           cell_gc_id[],
@@ -1328,7 +1332,7 @@ fvm_nodal_from_desc_add_cells(fvm_nodal_t        *this_nodal,
                                       n_face_lists,
                                       face_list_shift,
                                       face_vertex_idx,
-                                      face_vertex_num,
+                                      face_vertex,
                                       cell_face_idx,
                                       cell_face_num,
                                       NULL,
@@ -1407,7 +1411,7 @@ fvm_nodal_from_desc_add_cells(fvm_nodal_t        *this_nodal,
                                       n_face_lists,
                                       face_list_shift,
                                       face_vertex_idx,
-                                      face_vertex_num,
+                                      face_vertex,
                                       cell_face_idx,
                                       cell_face_num,
                                       cell_vtx_tria,
@@ -1458,7 +1462,7 @@ fvm_nodal_from_desc_add_cells(fvm_nodal_t        *this_nodal,
       if (retcode == FVM_NODAL_FROM_DESC_FAILURE)
         bft_error(__FILE__, __LINE__, 0,
                   _("Incoherent connectivity for cell %d\n"),
-                  cell_id + 1);
+                  cell_id);
 
   }
 
@@ -1480,7 +1484,7 @@ fvm_nodal_from_desc_add_cells(fvm_nodal_t        *this_nodal,
        n_face_lists,
        face_list_shift,
        face_vertex_idx,
-       face_vertex_num,
+       face_vertex,
        cell_face_idx,
        cell_face_num,
        cell_face_list);
@@ -1538,7 +1542,7 @@ fvm_nodal_from_desc_add_cells(fvm_nodal_t        *this_nodal,
  *   face_list_shift <-- face list to common number index shifts;
  *                       size: n_face_lists
  *   face_vertex_idx <-- face -> vertex indexes (per face list)
- *   face_vertex_num <-- face -> vertex numbers (per face list)
+ *   face_vertex     <-- face -> vertex ids (per face list)
  *   face_gc_id      <-- face -> group class ids, or NULL (per face list)
  *   parent_face_num <-- face -> parent face number (1 to n) if non-trivial
  *                       (i.e. if face definitions correspond to a subset
@@ -1552,7 +1556,7 @@ fvm_nodal_from_desc_add_faces(fvm_nodal_t        *this_nodal,
                               const int           n_face_lists,
                               const cs_lnum_t     face_list_shift[],
                               const cs_lnum_t    *face_vertex_idx[],
-                              const cs_lnum_t    *face_vertex_num[],
+                              const cs_lnum_t    *face_vertex[],
                               const int          *face_gc_id[],
                               const cs_lnum_t     parent_face_num[])
 {
@@ -1710,7 +1714,7 @@ fvm_nodal_from_desc_add_faces(fvm_nodal_t        *this_nodal,
                                n_face_lists,
                                face_list_shift,
                                face_vertex_idx,
-                               face_vertex_num,
+                               face_vertex,
                                p_vertex_num);
 
     section->_parent_element_num[n_elements_type[face_type]] = face_id + 1;
@@ -1783,7 +1787,7 @@ fvm_nodal_from_desc_add_faces(fvm_nodal_t        *this_nodal,
  *   face_list_shift <-- face list to common number index shifts;
  *                       size: n_face_lists
  *   face_vertex_idx <-- face -> vertex indexes (per face list)
- *   face_vertex_num <-- face -> vertex numbers (per face list)
+ *   face_vertex     <-- face -> vertex ids (per face list)
  *   cell_face_idx   <-- cell -> face indexes (1 to n)
  *   cell_face_num   <-- cell -> face numbers (1 to n)
  *   vertex_num      --> nodal connectivity of cell, if not a general
@@ -1798,7 +1802,7 @@ fvm_nodal_from_desc_cell(const cs_lnum_t    cell_id,
                          const int          n_face_lists,
                          const cs_lnum_t    face_list_shift[],
                          const cs_lnum_t   *face_vertex_idx[],
-                         const cs_lnum_t   *face_vertex_num[],
+                         const cs_lnum_t   *face_vertex[],
                          const cs_lnum_t    cell_face_idx[],
                          const cs_lnum_t    cell_face_num[],
                          cs_lnum_t          vertex_num[8])
@@ -1811,7 +1815,7 @@ fvm_nodal_from_desc_cell(const cs_lnum_t    cell_id,
                                                   n_face_lists,
                                                   face_list_shift,
                                                   face_vertex_idx,
-                                                  face_vertex_num,
+                                                  face_vertex,
                                                   cell_face_idx,
                                                   cell_face_num,
                                                   cell_vtx_tria,
