@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -62,6 +62,8 @@ BEGIN_C_DECLS
  * Static global variables
  *============================================================================*/
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -222,6 +224,40 @@ _fvm_nodal_order_indexed_connect(cs_lnum_t           connect_idx[],
 
 }
 
+/*----------------------------------------------------------------------------
+ * Reorder group class id.
+ *
+ * parameters:
+ *   gc_id  <-> group class id
+ *   order  <-- ordering of entities (0 to n-1).
+ *   nb_ent <-- number of entities considered.
+ *----------------------------------------------------------------------------*/
+
+static void
+_fvm_nodal_order_gc_id(int              gc_id[],
+                       const cs_lnum_t  order[],
+                       const size_t     nb_ent)
+{
+  size_t  i;
+
+  int  *tmp_gc_id = NULL;
+
+  BFT_MALLOC(tmp_gc_id, nb_ent, int);
+
+  /* Temporary ordered copy */
+
+  for (i = 0 ; i < nb_ent ; i++)
+    tmp_gc_id[i] = gc_id[order[i]];
+
+  /* Now put back in initial location */
+
+  memcpy(gc_id, tmp_gc_id, nb_ent * sizeof(int));
+
+  BFT_FREE(tmp_gc_id);
+}
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -283,6 +319,10 @@ fvm_nodal_order_cells(fvm_nodal_t       *this_nodal,
                                            order,
                                            section->n_elements);
         }
+        if (section->gc_id != NULL)
+          _fvm_nodal_order_gc_id(section->gc_id,
+                                 order,
+                                 section->n_elements);
 
         BFT_FREE(order);
 
@@ -351,6 +391,10 @@ fvm_nodal_order_faces(fvm_nodal_t       *this_nodal,
                                            order,
                                            section->n_elements);
         }
+        if (section->gc_id != NULL)
+          _fvm_nodal_order_gc_id(section->gc_id,
+                                 order,
+                                 section->n_elements);
 
         BFT_FREE(order);
 
