@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,7 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
+#include "cs_mesh.h"
 #include "fvm_defs.h"
 #include "fvm_group.h"
 #include "fvm_nodal.h"
@@ -107,6 +108,10 @@ typedef struct _fvm_nodal_section_t {
 
   int         *gc_id;              /* Group class id, NULL if implicit 0 */
 
+  /* Optional tagging */
+
+  int         *tag;                /* Element tag */
+
   /* Auxiliary structure used to define subdivision of elements into
      simpler element types (usually polygons to triangles and
      polyhedra to tetrahedra and pyramids) */
@@ -201,10 +206,25 @@ struct _fvm_nodal_t {
 
   fvm_nodal_section_t  **sections;  /* Array of section descriptions */
 
+  /* Metadata */
+  /*----------*/
+
   /* Group class descriptions if present */
 
   fvm_group_class_set_t   *gc_set;  /* Pointer to group class set, or NULL */
 
+  /* Optional global vertex labels.
+     As these are expected to be used only for small sets (i.e. probes)
+     where the point set is built from a global definition and data movement
+     would adds complexity and overhead, the labels refer to a global view
+     on rank 0; for the same reason, only shared labels are needed */
+
+  char   **global_vertex_labels;  /* Pointer to vertex labels, or NULL */
+
+  /* Pointer to parent mesh, if defined */
+
+  const cs_mesh_t  *parent;
+
 };
 
 /*=============================================================================
@@ -251,10 +271,10 @@ fvm_nodal_section_destroy(fvm_nodal_section_t  * this_section);
 
 void
 fvm_nodal_section_copy_on_write(fvm_nodal_section_t  *this_section,
-                                _Bool                 copy_face_index,
-                                _Bool                 copy_face_num,
-                                _Bool                 copy_vertex_index,
-                                _Bool                 copy_vertex_num);
+                                bool                  copy_face_index,
+                                bool                  copy_face_num,
+                                bool                  copy_vertex_index,
+                                bool                  copy_vertex_num);
 
 /*----------------------------------------------------------------------------
  * Return global number of elements associated with section.
