@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -63,6 +63,8 @@ BEGIN_C_DECLS
  * Static global variables
  *============================================================================*/
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Local macro definitions
  *============================================================================*/
@@ -185,6 +187,7 @@ _triangulate_section(int                         dim,
 
       if (n_vertices == 4)
         n_triangles = fvm_triangulate_quadrangle(dim,
+                                                 1,
                                                  vertex_coords,
                                                  parent_vertex_num,
                                                  (  base_section->vertex_num
@@ -194,6 +197,7 @@ _triangulate_section(int                         dim,
 
       else {
         n_triangles = fvm_triangulate_polygon(dim,
+                                              1,
                                               n_vertices,
                                               vertex_coords,
                                               parent_vertex_num,
@@ -400,6 +404,7 @@ _triangulate_section_polygons(int                         dim,
     if (n_vertices > 4) {
 
       n_triangles = fvm_triangulate_polygon(dim,
+                                            1,
                                             n_vertices,
                                             vertex_coords,
                                             parent_vertex_num,
@@ -485,6 +490,8 @@ _triangulate_section_polygons(int                         dim,
 
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
