@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -55,6 +55,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro definitions
  *============================================================================*/
@@ -198,7 +200,7 @@ static int
 _combine_transforms_check(const fvm_periodicity_t  *this_periodicity,
                           int                       tr_id_0,
                           int                       tr_id_1,
-                          _Bool                     abort_on_error)
+                          bool                      abort_on_error)
 {
   int i;
   int eq[3];
@@ -509,6 +511,8 @@ _transform_dump(_transform_t  *this_transform,
              tr->m[2][0], tr->m[2][1], tr->m[2][2], tr->m[2][3]);
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
@@ -738,7 +742,7 @@ fvm_periodicity_add_by_matrix(fvm_periodicity_t       *this_periodicity,
 
     for (i = 0; i < this_periodicity->n_transforms - 1; i++) {
       const _transform_t *tr = this_periodicity->transform[i];
-      _Bool is_equiv = true;
+      bool  is_equiv = true;
       for (j = 0; j < 3; j++) {
         for (k = 0; k < 4; k++) {
           if (  CS_ABS(transform->m[j][k] - tr->m[j][k])
@@ -1259,7 +1263,7 @@ fvm_periodicity_combine(fvm_periodicity_t  *this_periodicity,
         retval = _combine_transforms_check(this_periodicity,
                                            i,
                                            j,
-                                           (_Bool)abort_on_error);
+                                           (bool)abort_on_error);
 
         if (retval == 0) {
 
