@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -60,6 +60,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Macro definitions
  *============================================================================*/
@@ -316,7 +318,7 @@ static int _compare_attributes(const void *x, const void *y)
  *   true if the string defines an integer, false otherwise
  *----------------------------------------------------------------------------*/
 
-static _Bool
+static bool
 _is_int(const char  *str,
         int         *value)
 {
@@ -324,7 +326,7 @@ _is_int(const char  *str,
   int retcode, int_len;
 
   *value = 0;
-  retcode = (_Bool)(sscanf(str, "%i%n", &_value, &int_len));
+  retcode = (bool)(sscanf(str, "%i%n", &_value, &int_len));
 
   if (retcode) {
     if (int_len != (int)strlen(str))
@@ -731,6 +733,8 @@ _get_criteria_id(fvm_selector_t  *selector,
   return op;
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -834,7 +838,7 @@ fvm_selector_create(int                           dim,
                    [selector->_n_group_class_elements
                                 [  group_class_id[j]
                                  - group_class_id_base]++]
-        = j+1;
+        = j;
 
     }
   }
@@ -913,6 +917,7 @@ fvm_selector_destroy(fvm_selector_t  *this_selector)
  * parameters:
  *   this_selector       <-> pointer to selector
  *   str                 <-- string defining selection criteria
+ *   elt_id_base         <-- element id base (usually 0 or 1)
  *   n_selected_elements <-- number of elements selected
  *   selected_elements   <-> selected elements list (1 to n numbering)
  *
@@ -923,6 +928,7 @@ fvm_selector_destroy(fvm_selector_t  *this_selector)
 int
 fvm_selector_get_list(fvm_selector_t  *this_selector,
                       const char      *str,
+                      cs_lnum_t        elt_id_base,
                       cs_lnum_t       *n_selected_elements,
                       cs_lnum_t       *selected_elements)
 
@@ -966,7 +972,7 @@ fvm_selector_get_list(fvm_selector_t  *this_selector,
                i++) {
             selected_elements[(*n_selected_elements)++]
               = ts->_group_class_elements
-                      [_criteria_group_class_set[gc_id]][i];
+                      [_criteria_group_class_set[gc_id]][i] + elt_id_base;
           }
         }
 
@@ -1021,7 +1027,7 @@ fvm_selector_get_list(fvm_selector_t  *this_selector,
                                     ts->attribute_ids[gc_id],
                                     ts->coords + (i*dim),
                                     ts->normals + (i*dim)))
-        selected_elements[(*n_selected_elements)++] = i+1;
+        selected_elements[(*n_selected_elements)++] = i + elt_id_base;
 
     }
   }
