@@ -82,6 +82,8 @@
 #include <vtkGenericDataObjectWriter.h>
 #include <vtkDoubleArray.h>
 
+#include <vtkFileOutputWindow.h>
+
 /*----------------------------------------------------------------------------
  *  Local headers
  *----------------------------------------------------------------------------*/
@@ -400,30 +402,39 @@ _init_coprocessor(bool      private_comm,
 _init_coprocessor(void)
 #endif
 {
-  if (_processor == NULL)
-    _processor = vtkCPProcessor::New();
-
   int mpi_flag = 0;
+  int mpi_rank = -1;
 
 #if defined(HAVE_MPI)
 
   MPI_Initialized(&mpi_flag);
 
   if (mpi_flag) {
-
     if (comm != _reference_comm) {
-
-      if (comm != MPI_COMM_NULL && _reference_comm != MPI_COMM_NULL) {
+      if (comm != MPI_COMM_NULL && _reference_comm != MPI_COMM_NULL)
         bft_error(__FILE__, __LINE__, 0,
                   _("All Catalyst writers must use the same MPI communicator"));
-      }
-      else {
-        _reference_comm = comm;
-        if (private_comm && comm != MPI_COMM_NULL)
-          MPI_Comm_dup(comm, &(_comm));
-        else
-          _comm = comm;
-      }
+    }
+  }
+
+#endif
+
+  if (_processor == NULL) {
+
+    _processor = vtkCPProcessor::New();
+
+#if defined(HAVE_MPI)
+
+    if (mpi_flag) {
+
+      _reference_comm = comm;
+      if (private_comm && comm != MPI_COMM_NULL)
+        MPI_Comm_dup(comm, &(_comm));
+      else
+        _comm = comm;
+
+      if (comm != MPI_COMM_NULL)
+        MPI_Comm_rank(_comm, &mpi_rank);
 
       vtkMPICommunicatorOpaqueComm vtk_comm
         = vtkMPICommunicatorOpaqueComm(&_comm);
@@ -431,12 +442,19 @@ _init_coprocessor(void)
 
     }
 
-  }
-
 #endif
 
-  if (!mpi_flag)
-    _processor->Initialize();
+    if (!mpi_flag)
+      _processor->Initialize();
+
+    vtkFileOutputWindow *log_output = vtkFileOutputWindow::New();
+    if (mpi_rank < 1)
+      log_output->SetFileName("./catalyst.log");
+    else
+      log_output->SetFileName("/dev/null");
+    vtkFileOutputWindow::SetInstance(log_output);
+
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -607,7 +625,6 @@ _get_norm_elt_type(const fvm_element_t fvm_elt_type)
  * parameters:
  *   norm_elt_type  <-- Catalyst element type.
  *   vertex_order  --> Pointer to vertex order array (0 to n-1).
- *
  *----------------------------------------------------------------------------*/
 
 static void
