@@ -11,7 +11,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2011 EDF S.A.
+  Copyright (C) 2005-2016 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -168,6 +168,18 @@ fvm_to_medcoupling_export_field(void                  *this_writer_p,
                                 double                 time_value,
                                 const void      *const field_values[]);
 
+/*----------------------------------------------------------------------------
+ * Flush files associated with a given writer.
+ *
+ * In this case, the effective call to coprocessing is done.
+ *
+ * parameters:
+ *   this_writer_p    <-- pointer to associated writer
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_to_medcoupling_flush(void  *this_writer_p);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
