@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,6 +54,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Local Type Definitions
  *============================================================================*/
@@ -77,8 +79,8 @@ struct _fvm_triangulate_state_t {
                                       - (x ,-1): boundary edge, triangle x
                                       - (x ,-1): internal edge, triangles x
                                                  and y */
-  _Bool        *edge_is_delaunay;  /* Delaunay edge indicator */
-  _Bool        *concave;           /* Concave vertex indicator */
+  bool         *edge_is_delaunay;  /* Delaunay edge indicator */
+  bool         *concave;           /* Concave vertex indicator */
 
   int           n_vertices_max;    /* Maximum number vertices; a larger
                                       size requires resizing work arrays */
@@ -291,7 +293,7 @@ _polygon_plane_3d(const int   n_vertices,
  *   coords      <-- coordinates of the polygon's vertices (2d).
  *----------------------------------------------------------------------------*/
 
-static _Bool
+static bool
 _polygon_vertex_is_convex(int               previous,
                           int               current,
                           int               next,
@@ -325,12 +327,12 @@ _polygon_vertex_is_convex(int               previous,
  *   epsilon       <-- associated relative tolerance.
  *----------------------------------------------------------------------------*/
 
-static _Bool
+static bool
 _polygon_vertex_is_ear(int               n_vertices,
                        int               current,
                        int               list_previous[],
                        int               list_next[],
-                       _Bool             concave[],
+                       bool              concave[],
                        const cs_coord_t  coords[],
                        double            epsilon)
 
@@ -368,7 +370,7 @@ _polygon_vertex_is_ear(int               n_vertices,
 
       for (i = list_next[next]; i != previous; i = list_next[i]) {
 
-        if (concave[i] == true) {
+        if (concave[i] == true && fabs(surf_2) > 1e-30) {
 
           vect1[0] = coords[i*2    ] - coords[previous*2    ];
           vect1[1] = coords[i*2 + 1] - coords[previous*2 + 1];
@@ -410,7 +412,7 @@ _polygon_vertex_is_ear(int               n_vertices,
  *   coords            <-- coordinates of the triangulation's vertices (2d).
  *----------------------------------------------------------------------------*/
 
-static _Bool
+static bool
 _edge_is_locally_delaunay(int               edge_vertex_0,
                           int               edge_vertex_1,
                           int               flip_vertex_0,
@@ -541,7 +543,7 @@ _polygon_delaunay_flip(int               n_vertices,
                        int               triangle_vertices[],
                        int               edge_vertices[],
                        int               edge_neighbors[],
-                       _Bool             edge_is_delaunay[],
+                       bool              edge_is_delaunay[],
                        const cs_coord_t  coords[])
 {
   int    triangle_id, edge_id, vertex_id;
@@ -553,8 +555,8 @@ _polygon_delaunay_flip(int               n_vertices,
 
   int    i, i_0, i_1, i_min, i_max, j;
 
-  _Bool   face_is_delaunay, edge_locally_delaunay;
-  _Bool   restart, convex_quad;
+  bool    face_is_delaunay, edge_locally_delaunay;
+  bool    restart, convex_quad;
 
   const int  n_edges = n_vertices*(n_vertices - 1)/2;
   const int  n_triangles = n_vertices - 2;
@@ -798,6 +800,160 @@ _polygon_delaunay_flip(int               n_vertices,
 
 }
 
+/*----------------------------------------------------------------------------
+ * Triangulate a quadrangle.
+ *
+ * A convex quadrangle is divided into two triangles along its shortest
+ * diagonal. A non-convex quadrangle may only be divided along the diagonal
+ * which lies inside the quadrangle.
+ *
+ * If the quadrangle_vertices argument is NULL, 1, 2, ...,n local numbering
+ * is implied.
+ *
+ * parameters:
+ *   dim                  <-- spatial dimension (2 or 3).
+ *   base                 <-- base numbering (usually 0 or 1)
+ *   coords               <-- coordinates of the triangulation's vertices.
+ *   parent_vertex_num    <-- optional indirection to vertex coordinates
+ *                            (base to n-base numbering).
+ *   quadrangle_vertices  <-- polygon connectivity; size: n_vertices or empty.
+ *   mode                 <-- triangles connectivity by vertex number or
+ *                            quadrangle vertex index.
+ *   triangle_vertices    --> triangles connectivity; size: 2 * 3.
+ *
+ * returns:
+ *   number of resulting triangles.
+ *----------------------------------------------------------------------------*/
+
+static int
+_triangulate_quadrangle(int                    dim,
+                        int                    base,
+                        const cs_coord_t       coords[],
+                        const cs_lnum_t        parent_vertex_num[],
+                        const cs_lnum_t        quadrangle_vertices[],
+                        fvm_triangulate_def_t  mode,
+                        cs_lnum_t              triangle_vertices[])
+{
+  int i, j;
+  double d2_02, d2_13;
+  int o_count = 0, o_id = 0;
+  cs_lnum_t   vertex_id[4] = {0, 1, 2, 3};
+  double v1[3] = {0.0, 0.0, 0.0}, v2[3] = {0.0, 0.0, 0.0};
+  double n0[3] = {0.0, 0.0, 0.0}, ni[3] = {0.0, 0.0, 0.0};
+
+  if (quadrangle_vertices != NULL) {
+    for (i = 0; i < 4 ; i++)
+      vertex_id[i] = quadrangle_vertices[i] - base;
+  }
+
+  if (parent_vertex_num != NULL) {
+    for (i = 0; i < 4 ; i++)
+      vertex_id[i] = parent_vertex_num[i] - base;
+  }
+
+  /* Check for an obtuse angle */
+
+  for (i = 0; i < dim; i++) {
+    v1[i] = coords[vertex_id[1]*dim + i] - coords[vertex_id[0]*dim + i];
+    v2[i] = coords[vertex_id[3]*dim + i] - coords[vertex_id[0]*dim + i];
+  }
+
+  _CROSS_PRODUCT_3D(n0, v1, v2);
+
+  for (j = 1; j < 4; j++) {
+    for (i = 0; i < dim; i++) {
+      v1[i] = coords[vertex_id[(j+1)%4]*dim + i] - coords[vertex_id[j]*dim + i];
+      v2[i] = coords[vertex_id[ j-1   ]*dim + i] - coords[vertex_id[0]*dim + i];
+    }
+    _CROSS_PRODUCT_3D(ni, v1, v2);
+    if (_DOT_PRODUCT_3D(n0, ni) < 0) {
+      o_count++;
+      o_id = j;
+    }
+  }
+
+  /* With an obtuse angle, only one diagonal lies inside the quadrangle;
+     we define it as "shorter" */
+
+  if (o_count > 0) {
+
+    if (o_count > 1)
+      o_id = 0;
+
+    if (o_id%2 == 0) {
+      d2_02 = 0.;
+      d2_13 = 1.;
+    }
+    else {
+      d2_02 = 1.;
+      d2_13 = 0.;
+    }
+
+  }
+
+  /* With no obtuse angle, we choose the true shortest diagonal */
+
+  else {
+
+    for (i = 0; i < dim; i++) {
+      v1[i] = coords[vertex_id[2]*dim + i] - coords[vertex_id[0]*dim + i];
+      v2[i] = coords[vertex_id[3]*dim + i] - coords[vertex_id[1]*dim + i];
+    }
+
+    /* Now compute diagonal lengths (squared) */
+
+    d2_02 = v1[0]*v1[0] + v1[1]*v1[1] + v1[2]*v1[2];
+    d2_13 = v2[0]*v2[0] + v2[1]*v2[1] + v2[2]*v2[2];
+
+  }
+
+  /* Now define triangulation */
+
+  if (quadrangle_vertices != NULL && mode == FVM_TRIANGULATE_MESH_DEF) {
+    if (d2_02 < d2_13) {
+      triangle_vertices[0] = quadrangle_vertices[0]; /* 1st triangle */
+      triangle_vertices[1] = quadrangle_vertices[1];
+      triangle_vertices[2] = quadrangle_vertices[2];
+      triangle_vertices[3] = quadrangle_vertices[2]; /* 2nd triangle */
+      triangle_vertices[4] = quadrangle_vertices[3];
+      triangle_vertices[5] = quadrangle_vertices[0];
+    }
+    else {
+      triangle_vertices[0] = quadrangle_vertices[0]; /* 1st triangle */
+      triangle_vertices[1] = quadrangle_vertices[1];
+      triangle_vertices[2] = quadrangle_vertices[3];
+      triangle_vertices[3] = quadrangle_vertices[2]; /* 2nd triangle */
+      triangle_vertices[4] = quadrangle_vertices[3];
+      triangle_vertices[5] = quadrangle_vertices[1];
+    }
+  }
+  else { /* if (   quadrangle_vertices == NULL
+                || mode == FVM_TRIANGULATE_ELT_DEF) */
+    if (d2_02 < d2_13) {
+      triangle_vertices[0] = base + 0; /* 1st triangle */
+      triangle_vertices[1] = base + 1;
+      triangle_vertices[2] = base + 2;
+      triangle_vertices[3] = base + 2; /* 2nd triangle */
+      triangle_vertices[4] = base + 3;
+      triangle_vertices[5] = base + 0;
+    }
+    else {
+      triangle_vertices[0] = base + 0; /* 1st triangle */
+      triangle_vertices[1] = base + 1;
+      triangle_vertices[2] = base + 3;
+      triangle_vertices[3] = base + 2; /* 2nd triangle */
+      triangle_vertices[4] = base + 3;
+      triangle_vertices[5] = base + 1;
+    }
+  }
+
+  /* Return number of triangles (for consistency with polygon triangulation) */
+
+  return 2;
+}
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -829,8 +985,8 @@ fvm_triangulate_state_create(const int  n_vertices_max)
     BFT_MALLOC(this_state->list_next, n_vertices_max, int);
     BFT_MALLOC(this_state->edge_vertices, n_edges_tot_max*2, int);
     BFT_MALLOC(this_state->edge_neighbors, n_edges_tot_max*2, int);
-    BFT_MALLOC(this_state->edge_is_delaunay, n_edges_tot_max, _Bool);
-    BFT_MALLOC(this_state->concave, n_vertices_max, _Bool);
+    BFT_MALLOC(this_state->edge_is_delaunay, n_edges_tot_max, bool);
+    BFT_MALLOC(this_state->concave, n_vertices_max, bool);
   }
   else {
     this_state->triangle_vertices = NULL;
@@ -887,12 +1043,14 @@ fvm_triangulate_state_destroy(fvm_triangulate_state_t  *this_state)
  *
  * parameters:
  *   dim               <-- spatial dimension (2 or 3).
+ *   base              <-- base numbering (usually 0 or 1)
  *   n_vertices        <-- number of vertices defining the polygon.
  *   coords            <-- coordinates of the triangulation's vertices.
- *   parent_vertex_num <-- optional indirection to vertex coordinates (1 to n).
+ *   parent_vertex_num <-- optional indirection to vertex coordinates
+ *                         (base to n-base).
  *   polygon_vertices  <-- polygon connectivity; size: n_vertices or empty.
  *   mode              <-- triangles connectivity by vertex number or
- *                         polygon vertex index (1 to n).
+ *                         polygon vertex index.
  *   triangle_vertices --> triangles connectivity;
  *                         size: (n_vertices - 2) * 3.
  *   state             <-> associated triangulation state structure.
@@ -903,6 +1061,7 @@ fvm_triangulate_state_destroy(fvm_triangulate_state_t  *this_state)
 
 int
 fvm_triangulate_polygon(int                             dim,
+                        int                             base,
                         int                             n_vertices,
                         const cs_coord_t                coords[],
                         const cs_lnum_t                 parent_vertex_num[],
@@ -917,9 +1076,16 @@ fvm_triangulate_polygon(int                             dim,
   int n_tries = 0;
   double epsilon[] = {1.0e-1, 1.0e-2, 0.0, -1.0e-2, -1.0e-1};
 
-  int  *const list_previous = state->list_previous;
-  int  *const list_next = state->list_next;
-  _Bool  *const concave = state->concave;
+  /* Special case for quadrangle */
+
+  if (n_vertices == 4)
+    return _triangulate_quadrangle(dim,
+                                   base,
+                                   coords,
+                                   parent_vertex_num,
+                                   polygon_vertices,
+                                   mode,
+                                   triangle_vertices);
 
   /* Initialize state structure */
 
@@ -936,21 +1102,26 @@ fvm_triangulate_polygon(int                             dim,
     BFT_REALLOC(state->list_next, n_vertices_max, int);
     BFT_REALLOC(state->edge_vertices, n_edges_tot_max*2, int);
     BFT_REALLOC(state->edge_neighbors, n_edges_tot_max*2, int);
-    BFT_REALLOC(state->edge_is_delaunay, n_edges_tot_max, _Bool);
-    BFT_REALLOC(state->concave, n_vertices_max, _Bool);
+    BFT_REALLOC(state->edge_is_delaunay, n_edges_tot_max, bool);
+    BFT_REALLOC(state->concave, n_vertices_max, bool);
+
   }
 
+  int  *const list_previous = state->list_previous;
+  int  *const list_next = state->list_next;
+  bool   *const concave = state->concave;
+
   if (parent_vertex_num != NULL) {
     if (polygon_vertices != NULL) {
       for (i = 0; i < n_vertices; i++) {
-        int vertex_id = parent_vertex_num[polygon_vertices[i]-1] - 1;
+        int vertex_id = parent_vertex_num[polygon_vertices[i]-base] - base;
         for (j = 0; j < dim; j++)
           state->coords[i*dim + j] = coords[vertex_id*dim + j];
       }
     }
     else {
       for (i = 0; i < (n_vertices * dim); i++) {
-        int vertex_id = parent_vertex_num[i] - 1;
+        int vertex_id = parent_vertex_num[i] - base;
         state->coords[i] = coords[vertex_id];
       }
     }
@@ -961,7 +1132,7 @@ fvm_triangulate_polygon(int                             dim,
     if (polygon_vertices != NULL) {
       for (i = 0; i < n_vertices; i++) {
         for (j = 0; j < dim; j++)
-          state->coords[i*dim + j] = coords[(polygon_vertices[i]-1)*dim + j];
+          state->coords[i*dim + j] = coords[(polygon_vertices[i]-base)*dim + j];
       }
     }
     else {
@@ -1055,7 +1226,6 @@ fvm_triangulate_polygon(int                             dim,
 
   }
 
-
   /* Now that we have an initial triangulation, apply flip algorithm
      to obtain a Delaunay triangulation */
 
@@ -1075,7 +1245,7 @@ fvm_triangulate_polygon(int                             dim,
   }
   else {
     for (i = 0; i < n_triangles * 3; i++)
-      triangle_vertices[i] = state->triangle_vertices[i] + 1;
+      triangle_vertices[i] = state->triangle_vertices[i] + base;
   }
 
   return n_triangles;
@@ -1093,8 +1263,10 @@ fvm_triangulate_polygon(int                             dim,
  *
  * parameters:
  *   dim                  <-- spatial dimension (2 or 3).
+ *   base                 <-- base numbering (usually 0 or 1)
  *   coords               <-- coordinates of the triangulation's vertices.
  *   parent_vertex_num    <-- optional indirection to vertex coordinates
+ *                            (base to n-base numbering).
  *   quadrangle_vertices  <-- polygon connectivity; size: n_vertices or empty.
  *   triangle_vertices    --> triangles connectivity; size: 2 * 3.
  *
@@ -1104,126 +1276,19 @@ fvm_triangulate_polygon(int                             dim,
 
 int
 fvm_triangulate_quadrangle(int               dim,
+                           int               base,
                            const cs_coord_t  coords[],
                            const cs_lnum_t   parent_vertex_num[],
                            const cs_lnum_t   quadrangle_vertices[],
                            cs_lnum_t         triangle_vertices[])
 {
-  int i, j;
-  double d2_02, d2_13;
-  int o_count = 0, o_id = 0;
-  cs_lnum_t   vertex_id[4] = {0, 1, 2, 3};
-  double v1[3] = {0.0, 0.0, 0.0}, v2[3] = {0.0, 0.0, 0.0};
-  double n0[3] = {0.0, 0.0, 0.0}, ni[3] = {0.0, 0.0, 0.0};
-
-  if (quadrangle_vertices != NULL) {
-    for (i = 0; i < 4 ; i++)
-      vertex_id[i] = quadrangle_vertices[i] - 1;
-  }
-
-  if (parent_vertex_num != NULL) {
-    for (i = 0; i < 4 ; i++)
-      vertex_id[i] = parent_vertex_num[i] - 1;
-  }
-
-  /* Check for an obtuse angle */
-
-  for (i = 0; i < dim; i++) {
-    v1[i] = coords[vertex_id[1]*dim + i] - coords[vertex_id[0]*dim + i];
-    v2[i] = coords[vertex_id[3]*dim + i] - coords[vertex_id[0]*dim + i];
-  }
-
-  _CROSS_PRODUCT_3D(n0, v1, v2);
-
-  for (j = 1; j < 4; j++) {
-    for (i = 0; i < dim; i++) {
-      v1[i] = coords[vertex_id[(j+1)%4]*dim + i] - coords[vertex_id[j]*dim + i];
-      v2[i] = coords[vertex_id[ j-1   ]*dim + i] - coords[vertex_id[0]*dim + i];
-    }
-    _CROSS_PRODUCT_3D(ni, v1, v2);
-    if (_DOT_PRODUCT_3D(n0, ni) < 0) {
-      o_count++;
-      o_id = j;
-    }
-  }
-
-  /* With an obtuse angle, only one diagonal lies inside the quadrangle;
-     we define it as "shorter" */
-
-  if (o_count > 0) {
-
-    if (o_count > 1)
-      o_id = 0;
-
-    if (o_id%2 == 0) {
-      d2_02 = 0.;
-      d2_13 = 1.;
-    }
-    else {
-      d2_02 = 1.;
-      d2_13 = 0.;
-    }
-
-  }
-
-  /* With no obtuse angle, we choose the true shortest diagonal */
-
-  else {
-
-    for (i = 0; i < dim; i++) {
-      v1[i] = coords[vertex_id[2]*dim + i] - coords[vertex_id[0]*dim + i];
-      v2[i] = coords[vertex_id[3]*dim + i] - coords[vertex_id[1]*dim + i];
-    }
-
-    /* Now compute diagonal lengths (squared) */
-
-    d2_02 = v1[0]*v1[0] + v1[1]*v1[1] + v1[2]*v1[2];
-    d2_13 = v2[0]*v2[0] + v2[1]*v2[1] + v2[2]*v2[2];
-
-  }
-
-  /* Now define triangulation */
-
-  if (quadrangle_vertices != NULL) {
-    if (d2_02 < d2_13) {
-      triangle_vertices[0] = quadrangle_vertices[0]; /* 1st triangle */
-      triangle_vertices[1] = quadrangle_vertices[1];
-      triangle_vertices[2] = quadrangle_vertices[2];
-      triangle_vertices[3] = quadrangle_vertices[2]; /* 2nd triangle */
-      triangle_vertices[4] = quadrangle_vertices[3];
-      triangle_vertices[5] = quadrangle_vertices[0];
-    }
-    else {
-      triangle_vertices[0] = quadrangle_vertices[0]; /* 1st triangle */
-      triangle_vertices[1] = quadrangle_vertices[1];
-      triangle_vertices[2] = quadrangle_vertices[3];
-      triangle_vertices[3] = quadrangle_vertices[2]; /* 2nd triangle */
-      triangle_vertices[4] = quadrangle_vertices[3];
-      triangle_vertices[5] = quadrangle_vertices[1];
-    }
-  }
-  else { /* if (quadrangle_vertices == NULL) */
-    if (d2_02 < d2_13) {
-      triangle_vertices[0] = 1; /* 1st triangle */
-      triangle_vertices[1] = 2;
-      triangle_vertices[2] = 3;
-      triangle_vertices[3] = 3; /* 2nd triangle */
-      triangle_vertices[4] = 4;
-      triangle_vertices[5] = 1;
-    }
-    else {
-      triangle_vertices[0] = 1; /* 1st triangle */
-      triangle_vertices[1] = 2;
-      triangle_vertices[2] = 4;
-      triangle_vertices[3] = 3; /* 2nd triangle */
-      triangle_vertices[4] = 4;
-      triangle_vertices[5] = 2;
-    }
-  }
-
-  /* Return number of triangles (for consistency with polygon triangulation) */
-
-  return 2;
+  return _triangulate_quadrangle(dim,
+                                 base,
+                                 coords,
+                                 parent_vertex_num,
+                                 quadrangle_vertices,
+                                 FVM_TRIANGULATE_MESH_DEF,
+                                 triangle_vertices);
 }
 
 /*----------------------------------------------------------------------------*/
