@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,7 @@
 
 #include "cs_base.h"
 #include "cs_file.h"
+#include "cs_fp_exception.h"
 #include "cs_timer.h"
 
 /*----------------------------------------------------------------------------
@@ -68,17 +69,24 @@
 #include "fvm_to_cgns.h"
 #include "fvm_to_med.h"
 #include "fvm_to_ensight.h"
+#include "fvm_to_histogram.h"
+#include "fvm_to_plot.h"
+#include "fvm_to_time_plot.h"
 
-#if !defined(HAVE_PLUGINS)
-#if defined(HAVE_CATALYST)
+#if defined(HAVE_CATALYST) && !defined(HAVE_PLUGIN_CATALYST)
 #include "fvm_to_catalyst.h"
 #endif
+
+#if defined(HAVE_MELISSA) && !defined(HAVE_PLUGIN_MELISSA)
+#include "fvm_to_melissa.h"
 #endif
 
 /*----------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Local Type Definitions
  *============================================================================*/
@@ -99,9 +107,9 @@ BEGIN_C_DECLS
 
 /* Number and status of defined formats */
 
-static const int _fvm_writer_n_formats = 6;
+static const int _fvm_writer_n_formats = 10;
 
-static fvm_writer_format_t _fvm_writer_format_list[6] = {
+static fvm_writer_format_t _fvm_writer_format_list[10] = {
 
   /* Built-in EnSight Gold writer */
   {
@@ -125,7 +133,7 @@ static fvm_writer_format_t _fvm_writer_format_list[6] = {
     NULL                               /* flush_func */
   },
 
-  /* MED 3.0 writer */
+  /* MED writer */
   {
     "MED",
     "3.0 +",
@@ -166,7 +174,7 @@ static fvm_writer_format_t _fvm_writer_format_list[6] = {
     "3.1 +",
     (  FVM_WRITER_FORMAT_USE_EXTERNAL
      | FVM_WRITER_FORMAT_HAS_POLYGON),
-    FVM_WRITER_FIXED_MESH,
+    FVM_WRITER_TRANSIENT_COORDS,
     0,                                 /* dynamic library count */
     NULL,                              /* dynamic library */
     NULL,                              /* dynamic library name */
@@ -197,16 +205,21 @@ static fvm_writer_format_t _fvm_writer_format_list[6] = {
   /* Catalyst (VTK) writer (plugin) */
   {
     "Catalyst",
-    "4.0 +",
+    "4.2 +",
     (  FVM_WRITER_FORMAT_USE_EXTERNAL
      | FVM_WRITER_FORMAT_HAS_POLYGON
      | FVM_WRITER_FORMAT_HAS_POLYHEDRON),
     FVM_WRITER_TRANSIENT_CONNECT,
-#if !defined(HAVE_CATALYST) || defined(HAVE_PLUGINS)
+#if !defined(HAVE_CATALYST) || defined(HAVE_PLUGIN_CATALYST)
     0,                                 /* dynamic library count */
     NULL,                              /* dynamic library */
+#if defined(HAVE_CATALYST)
     "fvm_catalyst",                    /* dynamic library name */
     "fvm_to_catalyst_",                /* dynamic library prefix */
+#else
+    NULL,
+    NULL,
+#endif
     NULL,
     NULL,
     NULL,
@@ -239,11 +252,16 @@ static fvm_writer_format_t _fvm_writer_format_list[6] = {
     "6.3 +",
     (  FVM_WRITER_FORMAT_USE_EXTERNAL
      | FVM_WRITER_FORMAT_HAS_POLYGON),
-    FVM_WRITER_FIXED_MESH,
+    FVM_WRITER_TRANSIENT_CONNECT,
     0,                                 /* dynamic library count */
     NULL,                              /* dynamic library */
+#if defined(HAVE_MEDCOUPLING)
     "fvm_medcoupling",                 /* dynamic library name */
     "fvm_to_medcoupling_",             /* dynamic library prefix */
+#else
+    NULL,
+    NULL,
+#endif
     NULL,
     NULL,
     NULL,
@@ -255,6 +273,122 @@ static fvm_writer_format_t _fvm_writer_format_list[6] = {
     NULL
   },
 
+  /* Built-in Melissa writer */
+  {
+    "Melissa",
+    "",
+    (  FVM_WRITER_FORMAT_USE_EXTERNAL
+     | FVM_WRITER_FORMAT_HAS_POLYGON
+     | FVM_WRITER_FORMAT_HAS_POLYHEDRON),
+    FVM_WRITER_TRANSIENT_CONNECT,
+#if !defined(HAVE_MELISSA) || defined(HAVE_PLUGIN_MELISSA)
+    0,                                 /* dynamic library count */
+    NULL,                              /* dynamic library */
+#if defined(HAVE_MELISSA)
+    "fvm_melissa",                     /* dynamic library name */
+    "fvm_to_melissa_",                 /* dynamic library prefix */
+#else
+    NULL,
+    NULL,
+#endif
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL,
+    NULL
+#else
+    0,                                 /* dynamic library count */
+    NULL,                              /* dynamic library */
+    NULL,                              /* dynamic library name */
+    NULL,                              /* dynamic library prefix */
+    NULL,                              /* n_version_strings_func */
+    NULL,                              /* version_string_func */
+    fvm_to_melissa_init_writer,        /* init_func */
+    fvm_to_melissa_finalize_writer,    /* finalize_func */
+    fvm_to_melissa_set_mesh_time,      /* set_mesh_time_func */
+    NULL,                              /* needs_tesselation_func */
+    fvm_to_melissa_export_nodal,       /* export_nodal_func */
+    fvm_to_melissa_export_field,       /* export_field_func */
+    NULL                               /* flush_func */
+#endif
+  },
+
+  /* Built-in histogram writer */
+  {
+    "histogram",
+    "",
+    (  FVM_WRITER_FORMAT_HAS_POLYGON
+     | FVM_WRITER_FORMAT_HAS_POLYHEDRON
+     | FVM_WRITER_FORMAT_SEPARATE_MESHES
+     | FVM_WRITER_FORMAT_NAME_IS_OPTIONAL),
+       FVM_WRITER_TRANSIENT_CONNECT,
+    0,                                 /* dynamic library count */
+    NULL,                              /* dynamic library */
+    NULL,                              /* dynamic library name */
+    NULL,                              /* dynamic library prefix */
+    NULL,                              /* n_version_strings_func */
+    NULL,                              /* version_string_func */
+    fvm_to_histogram_init_writer,      /* init_func */
+    fvm_to_histogram_finalize_writer,  /* finalize_func */
+    fvm_to_histogram_set_mesh_time,    /* set_mesh_time_func */
+    NULL,                              /* needs_tesselation_func */
+    NULL,                              /* export_nodal_func */
+    fvm_to_histogram_export_field,     /* export_field_func */
+    fvm_to_histogram_flush             /* flush_func */
+  },
+
+  /* Built-in plot writer */
+  {
+    "plot",
+    "",
+    (  FVM_WRITER_FORMAT_HAS_POLYGON
+     | FVM_WRITER_FORMAT_HAS_POLYHEDRON
+     | FVM_WRITER_FORMAT_SEPARATE_MESHES
+     | FVM_WRITER_FORMAT_NAME_IS_OPTIONAL),
+    FVM_WRITER_TRANSIENT_CONNECT,
+    0,                                 /* dynamic library count */
+    NULL,                              /* dynamic library */
+    NULL,                              /* dynamic library name */
+    NULL,                              /* dynamic library prefix */
+    NULL,                              /* n_version_strings_func */
+    NULL,                              /* version_string_func */
+    fvm_to_plot_init_writer,           /* init_func */
+    fvm_to_plot_finalize_writer,       /* finalize_func */
+    fvm_to_plot_set_mesh_time,         /* set_mesh_time_func */
+    NULL,                              /* needs_tesselation_func */
+    fvm_to_plot_export_nodal,          /* export_nodal_func */
+    fvm_to_plot_export_field,          /* export_field_func */
+    fvm_to_plot_flush                  /* flush_func */
+  },
+
+  /* Built-in time plot writer */
+  {
+    "time_plot",
+    "",
+    (  FVM_WRITER_FORMAT_HAS_POLYGON
+     | FVM_WRITER_FORMAT_HAS_POLYHEDRON
+     | FVM_WRITER_FORMAT_SEPARATE_MESHES
+     | FVM_WRITER_FORMAT_NAME_IS_OPTIONAL),
+    FVM_WRITER_FIXED_MESH,
+    0,                                 /* dynamic library count */
+    NULL,                              /* dynamic library */
+    NULL,                              /* dynamic library name */
+    NULL,                              /* dynamic library prefix */
+    NULL,                              /* n_version_strings_func */
+    NULL,                              /* version_string_func */
+    fvm_to_time_plot_init_writer,      /* init_func */
+    fvm_to_time_plot_finalize_writer,  /* finalize_func */
+    fvm_to_time_plot_set_mesh_time,    /* set_mesh_time_func */
+    NULL,                              /* needs_tesselation_func */
+    fvm_to_time_plot_export_nodal,     /* export_nodal_func */
+    fvm_to_time_plot_export_field,     /* export_field_func */
+    NULL                               /* flush_func */
+  },
+
   /* CCM-IO writer */
   {
     "CCM-IO",
@@ -273,7 +407,7 @@ static fvm_writer_format_t _fvm_writer_format_list[6] = {
     fvm_to_ccm_init_writer,           /* init_func */
     fvm_to_ccm_finalize_writer,       /* finalize_func */
     fvm_to_ccm_set_mesh_time,         /* set_mesh_time_func */
-    fvm_to_ccm_needs_tesselation,     /* needs_tesselation_func */
+    NULL,                             /* needs_tesselation_func */
     fvm_to_ccm_export_nodal,          /* export_nodal_func */
     fvm_to_ccm_export_field,          /* export_field_func */
     NULL                               /* flush_func */
@@ -292,6 +426,8 @@ static fvm_writer_format_t _fvm_writer_format_list[6] = {
 
 };
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Static global variables
  *============================================================================*/
@@ -304,6 +440,8 @@ const char  *fvm_writer_time_dep_name[] = {N_("fixed mesh"),
 
 const char _empty_string[] = "";
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -372,27 +510,23 @@ _get_dl_function_pointer(fvm_writer_format_t  *wf,
                          bool                  errors_are_fatal)
 {
   void  *retval = NULL;
-  char  *error = NULL;
 
   assert(wf != NULL);
   assert(wf->dl_lib != NULL);
 
-  dlerror();    /* Clear any existing error */
-
   if (wf->dl_name == NULL)
-    retval = dlsym(wf->dl_lib, name);
+    retval = cs_base_get_dl_function_pointer(wf->dl_lib,
+                                             name,
+                                             errors_are_fatal);
   else {
     char *_name;
     BFT_MALLOC(_name, strlen(wf->dl_prefix) + strlen(name) + 1, char);
     sprintf(_name, "%s%s", wf->dl_prefix, name);
-    retval = dlsym(wf->dl_lib, _name);
+    retval = cs_base_get_dl_function_pointer(wf->dl_lib,
+                                             _name,
+                                             errors_are_fatal);
     BFT_FREE(_name);
   }
-  error = dlerror();
-
-  if (error != NULL && errors_are_fatal)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error calling dlsym: %s\n"), error);
 
   return retval;
 }
@@ -407,19 +541,9 @@ _get_dl_function_pointer(fvm_writer_format_t  *wf,
 static void
 _load_plugin(fvm_writer_format_t  *wf)
 {
-  char  *lib_path = NULL;
-  const char *pkglibdir = cs_base_get_pkglibdir();
-
   /* Open from shared library */
 
-  BFT_MALLOC(lib_path,
-             strlen(pkglibdir) + 1 + 3 + strlen(wf->dl_name) + 3 + 1,
-             char);
-  sprintf(lib_path, "%s%c%s.so", pkglibdir, DIR_SEPARATOR, wf->dl_name);
-
-  wf->dl_lib = dlopen(lib_path, RTLD_LAZY);
-
-  BFT_FREE(lib_path);
+  wf->dl_lib = cs_base_dlopen_plugin(wf->dl_name);
 
   /* Increment reference count */
 
@@ -427,10 +551,6 @@ _load_plugin(fvm_writer_format_t  *wf)
 
   /* Load symbols from shared library */
 
-  if (wf->dl_lib == NULL)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error loading %s: %s."), lib_path, dlerror());
-
   /* Function pointers need to be double-casted so as to first convert
      a (void *) type to a memory address and then convert it back to the
      original type. Otherwise, the compiler may issue a warning.
@@ -479,9 +599,7 @@ _close_plugin(fvm_writer_format_t  *wf)
   if (wf->dl_lib == NULL)
     return;
 
-  if (dlclose(wf->dl_lib) != 0)
-    bft_error(__FILE__, __LINE__, 0,
-              _("Error unloading library: %s."), dlerror());
+  cs_base_dlclose(wf->name, wf->dl_lib);
 
   wf->dl_lib = NULL;
 
@@ -506,187 +624,170 @@ _close_plugin(fvm_writer_format_t  *wf)
 
 #endif /* defined(HAVE_DLOPEN)*/
 
+/*----------------------------------------------------------------------------
+ * Initialize specific format writer based on writer and optional
+ * mesh name info.
+ *
+ * parameters:
+ *   this_writer     <-- pointer to mesh and field output writer
+ *   mesh_name       <-- optional mesh name, or NULL
+ *
+ * returns:
+ *   pointer to mesh and field output writer
+ *----------------------------------------------------------------------------*/
+
+static void *
+_format_writer_init(fvm_writer_t  *this_writer,
+                    const char    *mesh_name)
+{
+  const  char empty[] = "";
+  const  char untitled[] = "[untitled]";
+
+  char   local_dir[] = ".";
+  char  *tmp_path = NULL, *tmp_name = NULL;
+
+  void  *format_writer = NULL;
+
+  /* Determine path and create directory if needed */
+
+  const char *path = local_dir;
+
+  if (this_writer->path != NULL) {
+
+    int l = strlen(this_writer->path);
+
+    if (l > 0) {
+      BFT_MALLOC(tmp_path, l + 2, char);
+      strcpy(tmp_path, this_writer->path);
+      if (tmp_path[l - 1] == DIR_SEPARATOR)
+        tmp_path[l - 1] = '\0';
+      if (cs_file_mkdir_default(this_writer->path) == 1)
+        tmp_path[0] = '\0';
+      else {
+        l = strlen(tmp_path);
+        tmp_path[l]   = DIR_SEPARATOR;
+        tmp_path[l+1] = '\0';
+      }
+      path = tmp_path;
+    }
+
+  }
+
+  const char *name = this_writer->name;
+
+  if (name != NULL) {
+    size_t lw = strlen(this_writer->name);
+    if (lw == 0)
+      name = mesh_name;
+    else if (mesh_name != NULL) {
+      if (strlen(mesh_name) > 0) {
+        size_t l = lw + 1 + strlen(mesh_name);
+        BFT_MALLOC(tmp_name, l + 1, char);
+        sprintf(tmp_name, "%s_%s", this_writer->name, mesh_name);
+        for (size_t i = lw + 1, j = 0; i < l; i++, j++) {
+          if (tmp_name[i] == ' ')
+            tmp_name[i] = '_';
+        }
+        name = tmp_name;
+      }
+    }
+  }
+  else
+    name = mesh_name;
+
+  if (name == NULL) {
+    if (this_writer->format->info_mask & FVM_WRITER_FORMAT_NAME_IS_OPTIONAL)
+      name = empty;
+    else
+      name = untitled;
+  }
+
+  /* Initialize format-specific writer */
+
+  fvm_writer_init_t  *init_func = this_writer->format->init_func;
+
+  if (init_func != NULL) {
+
+    cs_fp_exception_disable_trap();
+
+#if defined(HAVE_MPI)
+    format_writer = init_func(name,
+                              path,
+                              this_writer->options,
+                              this_writer->time_dep,
+                              cs_glob_mpi_comm);
+#else
+    format_writer = init_func(name,
+                              path,
+                              this_writer->options,
+                              this_writer->time_dep);
+#endif
+
+    cs_fp_exception_restore_trap();
+
+  }
+
+  BFT_FREE(tmp_name);
+  BFT_FREE(tmp_path);
+
+  /* Return pointer to initialized writer */
+
+  return format_writer;
+}
+
+/*----------------------------------------------------------------------------
+ * Find or add a specific format writer based on writer and optional
+ * mesh name info.
+ *
+ * parameters:
+ *   this_writer     <-- pointer to mesh and field output writer
+ *   mesh_name       <-- optional mesh name, or NULL
+ *
+ * returns:
+ *   pointer to mesh and field output writer
+ *----------------------------------------------------------------------------*/
+
+static void*
+_find_or_add_format_writer(fvm_writer_t        *this_writer,
+                           const fvm_nodal_t   *mesh)
+{
+  void  *format_writer = NULL;
+
+  assert(this_writer != NULL);
+  assert(this_writer->format != NULL);
+
+  /* Find or add single-mesh writer if required */
+
+  if  (   this_writer->n_format_writers == 0
+       || this_writer->mesh_names != NULL) {
+    int i;
+    const char empty[] = "";
+    const char *name = (mesh->name != NULL) ? mesh->name : empty;
+    for (i = 0; i < this_writer->n_format_writers; i++) {
+      if (strcmp(this_writer->mesh_names[i], name) == 0)
+        break;
+    }
+    if (i >= this_writer->n_format_writers) {
+      BFT_REALLOC(this_writer->format_writer, i + 1, void *);
+      BFT_REALLOC(this_writer->mesh_names, i + 1, char *);
+      BFT_MALLOC(this_writer->mesh_names[i], strlen(name) + 1, char);
+      strcpy(this_writer->mesh_names[i], name);
+      this_writer->format_writer[i] = _format_writer_init(this_writer, name);
+      this_writer->n_format_writers += 1;
+    }
+    format_writer = this_writer->format_writer[i];
+  }
+  else
+    format_writer = this_writer->format_writer[0];
+
+  return format_writer;
+}
+
 /*============================================================================
  * Semi-private function definitions (prototypes in fvm_writer_priv.h)
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Compute recommended buffer sizes to input or output a nodal mesh
- * definition by slices. This is especially useful when gathering the mesh for
- * output by slices using standard I/O in parallel mode.
- *
- * The global number of vertices and elements of each slice may also
- * be returned, if the pointers n_g_vertices and n_g_elements_section
- * are non-NULL respectively.
- *
- * The number of slices indicated is a minimum, and only a target;
- * computation is based primarily on cell and face connectivity, and the
- * target should be met for strided connectivities on those types of elements
- * only. Using an "optimistic" (i.e. small) mean number of vertices per
- * polyhedra or polygon will typically lead to requiring more slices, as
- * the connectivity slice size returned will be smaller than that truly
- * required for the corresponding slice size.
- * Slice sizes required for edges connectivity will meet the target only
- * when the global numbers of cells and faces given are zero, so as to
- * avoid generating too large connectivity slice sizes for cells should a mesh
- * contain both (as for example a hexahedral connectivity slice is 8 times
- * larger than the corresponding slice size, while an edges connectivity is
- * only 2 times as large).
- *
- * parameters:
- *   this_nodal                 <-- pointer to nodal mesh structure
- *   n_slices                   <-- target number of slices required
- *   n_polyhedron_vertices_mean <-- estimate of the mean number of vertices
- *                                  per polyhedron
- *   n_polygon_vertices_mean    <-- estimate of the mean number of vertices
- *                                  per polygon
- *   n_g_vertices               --> global number of vertices (or NULL)
- *   n_g_elements_section       --> array for global number of elements per
- *                                  section (or NULL)
- *   global_s_size              --> maximum number of entities defined per slice
- *   global_connect_s_size      --> maximum number of connectivity values
- *                                  per slice
- *----------------------------------------------------------------------------*/
-
-void
-fvm_writer_def_nodal_buf_size(const fvm_nodal_t  *this_nodal,
-                              int                 n_slices,
-                              int                 n_polyhedron_vertices_mean,
-                              int                 n_polygon_vertices_mean,
-                              cs_gnum_t          *n_g_vertices,
-                              cs_gnum_t           n_g_elements_section[],
-                              cs_gnum_t          *global_s_size,
-                              cs_gnum_t          *global_connect_s_size)
-{
-  int  i;
-  cs_gnum_t   n_g_elements = 0;
-  cs_gnum_t   n_g_cells = 0, n_g_faces = 0, n_g_edges = 0;
-  cs_gnum_t   _n_g_vertices = 0;
-
-  cs_gnum_t   connect_size = 0;
-  cs_gnum_t   *_n_g_elements_section = NULL;
-
-  const fvm_nodal_section_t  *section = NULL;
-
-  assert(this_nodal != NULL);
-
-  if (n_g_elements_section == NULL)
-    BFT_MALLOC(_n_g_elements_section, this_nodal->n_sections, cs_gnum_t);
-  else
-    _n_g_elements_section = n_g_elements_section;
-
-  /* Vertices */
-
-  if (this_nodal->global_vertex_num != NULL)
-    _n_g_vertices = fvm_io_num_get_global_count(this_nodal->global_vertex_num);
-  else
-    _n_g_vertices = this_nodal->n_vertices;
-
-  if (n_g_vertices != NULL)
-    *n_g_vertices = _n_g_vertices;
-
-  /* Edges, faces, and cells */
-
-  for (i = 0; i < this_nodal->n_sections; i++) {
-
-    section = this_nodal->sections[i];
-
-    n_g_elements = fvm_nodal_section_n_g_elements(section);
-
-    switch(section->entity_dim) {
-    case 1:
-      n_g_edges += n_g_elements;
-      break;
-    case 2:
-      n_g_faces += n_g_elements;
-      break;
-    default:
-      n_g_cells += n_g_elements;
-    }
-
-    _n_g_elements_section[i] = n_g_elements;
-
-  }
-
-  /* Define global slice size as (global_size / n_ranks) +  1 */
-
-  *global_s_size = CS_MAX(n_g_cells, n_g_faces);
-
-  if (*global_s_size == 0)
-    *global_s_size = n_g_edges;
-
-  if (*global_s_size == 0) /* Should the mesh contain only vertices */
-    *global_s_size = _n_g_vertices;
-
-  *global_s_size = (*global_s_size / n_slices) + 1;
-
-  /* Now compute best size for connectivity buffer,
-     using estimated "mean" numbers of nodes per polyhedron and per polygon */
-
-  *global_connect_s_size = 0;
-
-  for (i = 0; i < this_nodal->n_sections; i++) {
-
-    section = this_nodal->sections[i];
-
-    switch(section->type) {
-    case FVM_FACE_POLY:
-    case FVM_CELL_POLY:
-      if (section->type == FVM_FACE_POLY)
-        connect_size =   CS_MIN(_n_g_elements_section[i], *global_s_size)
-                       * n_polygon_vertices_mean;
-      else if (section->type == FVM_CELL_POLY)
-        connect_size =   CS_MIN(_n_g_elements_section[i], *global_s_size)
-                       * n_polyhedron_vertices_mean;
-
-      if (section->tesselation != NULL) {
-
-        int  i_type;
-
-        const fvm_tesselation_t  *tesselation = section->tesselation;
-        const int  n_sub_types = fvm_tesselation_n_sub_types(tesselation);
-
-        for (i_type = 0; i_type < n_sub_types; i_type++) {
-
-          int  stride;
-          cs_lnum_t   n_sub_elements_max;
-
-          fvm_element_t  sub_type = fvm_tesselation_sub_type(tesselation,
-                                                             i_type);
-
-          fvm_tesselation_get_global_size(tesselation,
-                                          sub_type,
-                                          NULL,
-                                          &n_sub_elements_max);
-
-          stride = fvm_nodal_n_vertices_element[sub_type];
-
-
-          connect_size = CS_MAX(connect_size,
-                                (cs_gnum_t)n_sub_elements_max * stride);
-
-        }
-
-      }
-
-      break;
-      break;
-    default:
-      connect_size =   CS_MIN(_n_g_elements_section[i], *global_s_size)
-                     * section->stride;
-    }
-
-    /* Buffer size is that required for the entity type
-       requiring the largest buffer size */
-
-    *global_connect_s_size = CS_MAX(*global_connect_s_size,
-                                    connect_size);
-
-  }
-
-  if (n_g_elements_section != _n_g_elements_section)
-    BFT_FREE(_n_g_elements_section);
-}
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
  * Public function definitions
@@ -737,6 +838,10 @@ fvm_writer_get_format_id(const char  *format_name)
     strcpy(closest_name, "Catalyst");
   else if (strncmp(tmp_name, "ccm", 3) == 0)
     strcpy(closest_name, "CCM-IO");
+  else if (strncmp(tmp_name, "melissa", 7) == 0)
+    strcpy(closest_name, "Melissa");
+  else
+    strcpy(closest_name, tmp_name);
 
   /* Find name in list */
 
@@ -881,6 +986,72 @@ fvm_writer_version_string(int format_index,
   return retval;
 }
 
+/*----------------------------------------------------------------------------
+ * Remove a given option from a format options list if present.
+ *
+ * The possible separators are also transformed to whitespace and merged.
+ *
+ * parameters:
+ *   format_options  <-> options for the selected format (case-independent,
+ *                       whitespace or comma separated list)
+ *   exclude_option  <-- option to be excluded
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_writer_filter_option(char        *format_options,
+                         const char  *exclude_option)
+{
+  char  *tmp_options = NULL;
+
+  size_t l = strlen(exclude_option);
+
+  /* Find corresponding format and check coherency */
+
+  tmp_options = _fvm_writer_option_list(format_options);
+
+  /* Parse top-level options (consuming those handled here);
+     the options string now contains options separated by a single
+     whitespace. */
+
+  if (tmp_options != NULL) {
+
+    int i0 = 0, i1;
+
+    while (tmp_options[i0] != '\0') {
+
+      for (i1 = i0; tmp_options[i1] != '\0' && tmp_options[i1] != ' '; i1++);
+      size_t l_opt = i1 - i0;
+
+      if (   (l_opt == l)
+          && (strncmp(tmp_options + i0, exclude_option, l_opt) == 0)) {
+        if (tmp_options[i1] == ' ')
+          strcpy(tmp_options + i0, tmp_options + i1 + 1);
+        else {
+          if (i0 > 1) {
+            assert(tmp_options[i0-1] = ' ');
+            i0--;
+          }
+          tmp_options[i0] = '\0';
+        }
+      }
+      else {
+        i0 = i1;
+        if (tmp_options[i0] == ' ')
+          i0++;
+      }
+
+      i1 = strlen(tmp_options);
+      BFT_REALLOC(tmp_options, i1+1, char);
+
+    }
+
+  }
+
+  strcpy(format_options, tmp_options);
+
+  BFT_FREE(tmp_options);
+}
+
 /*----------------------------------------------------------------------------
  * Initialize FVM mesh and field output writer.
  *
@@ -894,7 +1065,7 @@ fvm_writer_version_string(int format_index,
  *   divide_polygons     tesselate polygons with triangles
  *   divide_polyhedra    tesselate polyhedra with tetrahedra and pyramids
  *                       (adding a vertex near each polyhedron's center)
- *   split_tensors       write tensor values as separate scalars
+ *   separate_meshes     use a different writer for each mesh
  *
  * parameters:
  *   name            <-- base name of output
@@ -917,11 +1088,9 @@ fvm_writer_init(const char             *name,
                 fvm_writer_time_dep_t   time_dependency)
 {
   int  i;
-  char   local_dir[] = ".";
-  char  *tmp_path = NULL;
   char  *tmp_options = NULL;
   fvm_writer_t  *this_writer = NULL;
-  fvm_writer_init_t  *init_func = NULL;
+  bool separate_meshes = false;
 
   /* Find corresponding format and check coherency */
 
@@ -942,31 +1111,51 @@ fvm_writer_init(const char             *name,
               _("Format type \"%s\" required for case \"%s\" is not available"),
               format_name, name);
 
-  /* Create directory */
+  tmp_options = _fvm_writer_option_list(format_options);
 
-  if (path != NULL) {
+  /* Parse top-level options (consuming those handled here);
+     the options string now contains options separated by a single
+     whitespace. */
 
-    int l = strlen(path);
+  if (tmp_options != NULL) {
 
-    if (l > 0) {
-      BFT_MALLOC(tmp_path, l + 2, char);
-      strcpy(tmp_path, path);
-      if (tmp_path[l - 1] == DIR_SEPARATOR)
-        tmp_path[l - 1] = '\0';
-      if (cs_file_mkdir_default(path) == 1)
-        tmp_path[0] = '\0';
-      else {
-        l = strlen(tmp_path);
-        tmp_path[l]   = DIR_SEPARATOR;
-        tmp_path[l+1] = '\0';
+    int i0 = 0, i1;
+
+    while (tmp_options[i0] != '\0') {
+
+      for (i1 = i0; tmp_options[i1] != '\0' && tmp_options[i1] != ' '; i1++);
+      int l_opt = i1 - i0;
+
+      if (   (l_opt == 15)
+          && (strncmp(tmp_options + i0, "separate_meshes", l_opt) == 0)) {
+        separate_meshes = true;
+        if (tmp_options[i1] == ' ')
+          strcpy(tmp_options + i0, tmp_options + i1 + 1);
+        else {
+          if (i0 > 1) {
+            assert(tmp_options[i0-1] = ' ');
+            i0--;
+          }
+          tmp_options[i0] = '\0';
+        }
       }
+      else {
+        i0 = i1;
+        if (tmp_options[i0] == ' ')
+          i0++;
+      }
+
+      i1 = strlen(tmp_options);
+      if (i1 > 0)
+        BFT_REALLOC(tmp_options, i1+1, char);
+      else {
+        BFT_FREE(tmp_options);
+        break;
+      }
+
     }
 
   }
-  else
-    tmp_path = local_dir;
-
-  tmp_options = _fvm_writer_option_list(format_options);
 
   /* Initialize writer */
 
@@ -1001,30 +1190,26 @@ fvm_writer_init(const char             *name,
   CS_TIMER_COUNTER_INIT(this_writer->field_time);
   CS_TIMER_COUNTER_INIT(this_writer->flush_time);
 
+  if (this_writer->format->info_mask & FVM_WRITER_FORMAT_SEPARATE_MESHES)
+    separate_meshes = true;
+
+  if (separate_meshes)
+    this_writer->n_format_writers = 0; /* Delay contruction */
+  else
+    this_writer->n_format_writers = 1;
+
+  this_writer->mesh_names = NULL;
+
   /* Initialize format-specific writer */
 
-  init_func = this_writer->format->init_func;
-
-  if (init_func != NULL) {
-#if defined(HAVE_MPI)
-    this_writer->format_writer = init_func(name,
-                                           tmp_path,
-                                           this_writer->options,
-                                           this_writer->time_dep,
-                                           cs_glob_mpi_comm);
-#else
-    this_writer->format_writer = init_func(name,
-                                           tmp_path,
-                                           this_writer->options,
-                                           this_writer->time_dep);
-#endif
+  if  (this_writer->n_format_writers > 0) {
+    BFT_MALLOC(this_writer->format_writer, 1, void *);
+    this_writer->format_writer[0] = _format_writer_init(this_writer,
+                                                        NULL);
   }
   else
     this_writer->format_writer = NULL;
 
-  if (tmp_path != local_dir)
-    BFT_FREE(tmp_path);
-
   /* Return pointer to initialized writer */
 
   return this_writer;
@@ -1054,10 +1239,21 @@ fvm_writer_finalize(fvm_writer_t  *this_writer)
 
   finalize_func = this_writer->format->finalize_func;
 
-  if (finalize_func != NULL)
-    this_writer->format_writer = finalize_func(this_writer->format_writer);
+  if (finalize_func != NULL) {
+    cs_fp_exception_disable_trap();
+    for (int i = 0; i < this_writer->n_format_writers; i++)
+      finalize_func(this_writer->format_writer[i]);
+    cs_fp_exception_restore_trap();
+  }
   else
     this_writer->format_writer = NULL;
+  BFT_FREE(this_writer->format_writer);
+
+  if (this_writer->mesh_names != NULL) {
+    for (int i = 0; i < this_writer->n_format_writers; i++)
+      BFT_FREE(this_writer->mesh_names[i]);
+  }
+  BFT_FREE(this_writer->mesh_names);
 
   /* Unload plugin if required */
 
@@ -1178,10 +1374,14 @@ fvm_writer_set_mesh_time(fvm_writer_t  *this_writer,
 
   set_mesh_time_func = this_writer->format->set_mesh_time_func;
 
-  if (set_mesh_time_func != NULL)
-    set_mesh_time_func(this_writer->format_writer,
-                       time_step,
-                       time_value);
+  if (set_mesh_time_func != NULL) {
+    cs_fp_exception_disable_trap();
+    for (int i = 0; i < this_writer->n_format_writers; i++)
+      set_mesh_time_func(this_writer->format_writer[i],
+                         time_step,
+                         time_value);
+    cs_fp_exception_restore_trap();
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -1207,9 +1407,11 @@ fvm_writer_needs_tesselation(fvm_writer_t       *this_writer,
   int retval = 0;
   fvm_writer_needs_tesselation_t  *needs_tesselation_func = NULL;
 
+  void  *format_writer = _find_or_add_format_writer(this_writer, mesh);
+
   needs_tesselation_func = this_writer->format->needs_tesselation_func;
   if (needs_tesselation_func != NULL)
-    retval = needs_tesselation_func(this_writer->format_writer,
+    retval = needs_tesselation_func(format_writer,
                                     mesh,
                                     element_type);
   return retval;
@@ -1234,13 +1436,17 @@ fvm_writer_export_nodal(fvm_writer_t        *this_writer,
   assert(this_writer != NULL);
   assert(this_writer->format != NULL);
 
+  void  *format_writer = _find_or_add_format_writer(this_writer, mesh);
+
   t0 = cs_timer_time();
 
   export_nodal_func = this_writer->format->export_nodal_func;
 
-  if (export_nodal_func != NULL)
-    export_nodal_func(this_writer->format_writer,
-                      mesh);
+  if (export_nodal_func != NULL) {
+    cs_fp_exception_disable_trap();
+    export_nodal_func(format_writer, mesh);
+    cs_fp_exception_restore_trap();
+  }
 
   t1 = cs_timer_time();
 
@@ -1293,12 +1499,15 @@ fvm_writer_export_field(fvm_writer_t                 *this_writer,
   assert(this_writer != NULL);
   assert(this_writer->format != NULL);
 
+  void  *format_writer = _find_or_add_format_writer(this_writer, mesh);
+
   t0 = cs_timer_time();
 
   export_field_func = this_writer->format->export_field_func;
 
-  if (export_field_func != NULL)
-    export_field_func(this_writer->format_writer,
+  if (export_field_func != NULL) {
+    cs_fp_exception_disable_trap();
+    export_field_func(format_writer,
                       mesh,
                       name,
                       location,
@@ -1311,6 +1520,9 @@ fvm_writer_export_field(fvm_writer_t                 *this_writer,
                       time_value,
                       field_values);
 
+    cs_fp_exception_restore_trap();
+  }
+
   t1 = cs_timer_time();
 
   cs_timer_counter_add_diff(&(this_writer->field_time), &t0, &t1);
@@ -1340,7 +1552,12 @@ fvm_writer_flush(fvm_writer_t  *this_writer)
 
     t0 = cs_timer_time();
 
-    flush_func(this_writer->format_writer);
+    cs_fp_exception_disable_trap();
+
+    for (int i = 0; i < this_writer->n_format_writers; i++)
+      flush_func(this_writer->format_writer[i]);
+
+    cs_fp_exception_restore_trap();
 
     t1 = cs_timer_time();
 
