@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -188,6 +188,21 @@ fvm_writer_version_string(int format_index,
                           int string_index,
                           int compile_time_version);
 
+/*----------------------------------------------------------------------------
+ * Remove a given option from a format options list if present.
+ *
+ * The possible separators are also transformed to whitespace and merged.
+ *
+ * parameters:
+ *   format_options  <-> options for the selected format (case-independent,
+ *                       whitespace or comma separated list)
+ *   exclude_option  <-- option to be excluded
+ *----------------------------------------------------------------------------*/
+
+void
+fvm_writer_filter_option(char        *format_options,
+                         const char  *exclude_option);
+
 /*----------------------------------------------------------------------------
  * Initialize FVM mesh and field output writer.
  *
@@ -203,7 +218,7 @@ fvm_writer_version_string(int format_index,
  *   divide_polygons     tesselate polygons with triangles
  *   divide_polyhedra    tesselate polyhedra with tetrahedra and pyramids
  *                       (adding a vertex near each polyhedron's center)
- *   split_tensors       write tensor values as separate scalars
+ *   separate_meshes     use a different writer for each mesh
  *
  * parameters:
  *   name            <-- base name of output
