@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,7 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_volume_zone.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -45,53 +46,30 @@ BEGIN_C_DECLS
  * Public function prototypes for Fortran API
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Initialise the global 'vars' structure.
- *
- * Fortran Interface:
- *
- * subroutine uiinit
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uiinit, UIINIT) (void);
-
 /*----------------------------------------------------------------------------
  * Thermal model.
  *
  * Fortran Interface:
  *
- * SUBROUTINE CSTHER (ITHERM)
+ * SUBROUTINE CSTHER ()
  * *****************
  *
- * INTEGER          ITHERM  --> thermal model
- * integer          itpscl  --> temperature scale if itherm = 1
  *----------------------------------------------------------------------------*/
 
-
-void CS_PROCF (csther, CSTHER) (int  *itherm,
-                                int  *itpscl);
+void CS_PROCF (csther, CSTHER) (void);
 
 /*----------------------------------------------------------------------------
- * Turbulence model.
- *
- * Fortran Interface:
- *
- * SUBROUTINE CSTURB
- * *****************
- *
- * INTEGER          ITURB   -->   turbulence model
- * INTEGER          IDEUCH  -->   wall law treatment
- * INTEGER          IGRAKE  -->   k-eps gravity effects
- * INTEGER          IGRAKI  -->   Rij-eps gravity effects
- * DOUBLE PRECISION XLOMLG  -->   mixing_length_scale
+ * Turbulence model
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (csturb, CSTURB) (int *const iturb,
-                                int *const ideuch,
-                                int *const igrake,
-                                int *const igrari,
-                                double *const xlomlg);
+void cs_gui_turb_model(void);
+
+/*----------------------------------------------------------------------------
+ * Define reference length and reference velocity for the initialization of
+ * the turbulence variables
+ *----------------------------------------------------------------------------*/
+
+void cs_gui_turb_ref_values(void);
 
 /*----------------------------------------------------------------------------
  * Specific heat variable or constant indicator.
@@ -101,10 +79,9 @@ void CS_PROCF (csturb, CSTURB) (int *const iturb,
  * SUBROUTINE CSCPVA
  * *****************
  *
- * INTEGER          ICP     -->   Specific heat variable or constant indicator
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (cscpva, CSCPVA) (int *const icp);
+void CS_PROCF (cscpva, CSCPVA) (void);
 
 /*----------------------------------------------------------------------------
  * Volumic viscosity variable or constant indicator.
@@ -117,31 +94,7 @@ void CS_PROCF (cscpva, CSCPVA) (int *const icp);
  * INTEGER          IVISCV     -->   specific heat variable or constant indicator
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (csvvva, CSVVVA) (int *const iviscv);
-
-/*----------------------------------------------------------------------------
- * User scalars number.
- *
- * Fortran Interface:
- *
- * SUBROUTINE CSNSCA
- * *****************
- *
- * INTEGER          NSCAUS     -->   user scalars number
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (csnsca, CSNSCA) (int *const nscaus);
-
-/*----------------------------------------------------------------------------
- * User scalars labels
- *
- * Fortran Interface:
- *
- * SUBROUTINE UISCAU
- * *****************
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uiscau, UISCAU) (void);
+void CS_PROCF (csvvva, CSVVVA) (int *iviscv);
 
 /*----------------------------------------------------------------------------
  * User thermal scalar.
@@ -150,88 +103,44 @@ void CS_PROCF (uiscau, UISCAU) (void);
  *
  * SUBROUTINE UITHSC
  * *****************
- *
- * INTEGER          ISCALT     -->   thermal scalars number
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uithsc, UITHSC) (int *const iscalt);
-
-/*----------------------------------------------------------------------------
- * User scalars which are variance.
- *
- * Fortran Interface:
- *
- * SUBROUTINE CSISCA (ISCAVR)
- * *****************
- *
- * INTEGER          ISCAVR     -->   user scalars variance array
- * integer          itherm     <--  type of thermal model
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (csisca, CSISCA) (int        *iscavr,
-                                const int  *itherm);
+void CS_PROCF (uithsc, UITHSC) (void);
 
 /*----------------------------------------------------------------------------
  * Constant or variable indicator for the user scalar laminar viscosity.
  *
  * Fortran Interface:
  *
- * SUBROUTINE CSIVIS
+ * subroutine csivis
  * *****************
- *
- * INTEGER          ISCAVR  <->  number of the related variance if any
- * INTEGER          IVISLS  -->  indicator for the user scalar viscosity
- * INTEGER          ISCALT  <->  number of the user thermal scalar if any
- * INTEGER          ISCSTH  <->  type of the user thermal scalar
- * INTEGER          ITEMPK  -->  rtp index for temperature (in K)
  *----------------------------------------------------------------------------*/
 
-/*----------------------------------------------------------------------------
- * Constant or variable indicator for the user scalar laminar viscosity.
- *
- * Fortran Interface:
- *
- * subroutine csivis (iscavr, ivisls, iscalt, itherm, itempk)
- * *****************
- *
- * integer          iscavr  <-->  number of the related variance if any
- * integer          ivisls  <--   indicator for the user scalar viscosity
- * integer          iscalt  <-->  number of the user thermal scalar if any
- * integer          itherm  <-->  type of thermal model
- * integer          itempk   -->  rtp index for temperature (in K)
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (csivis, CSIVIS) (int *const iscavr,
-                                int *const ivisls,
-                                int *const iscalt,
-                                int *const itherm,
-                                int *const itempk);
+void CS_PROCF (csivis, CSIVIS) (void);
 
 /*----------------------------------------------------------------------------
  * Time passing parameter.
  *
  * Fortran Interface:
  *
- * SUBROUTINE CSIDTV (IDTVAR)
+ * SUBROUTINE CSIDTV ()
  * *****************
  *
- * INTEGER          IDTVAR  -->   fixed or variable time step
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF(csidtv, CSIDTV) (int *const idtvar);
+void CS_PROCF(csidtv, CSIDTV) (void);
 
 /*----------------------------------------------------------------------------
  * Hydrostatic pressure parameter.
  *
  * Fortran Interface:
  *
- * SUBROUTINE CSIPHY (IPHYDR)
+ * SUBROUTINE CSIPHY ()
  * *****************
  *
- * INTEGER          IPHYDR  -->   hydrostatic pressure
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (csiphy, CSIPHY) (int *const iphydr);
+void CS_PROCF (csiphy, CSIPHY) (void);
 
 /*----------------------------------------------------------------------------
  * Hydrostatic equilibrium parameter.
@@ -244,7 +153,7 @@ void CS_PROCF (csiphy, CSIPHY) (int *const iphydr);
  * INTEGER          ICFGRP  -->   hydrostatic equilibrium
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (cscfgp, CSCFGP) (int *const icfgrp);
+void CS_PROCF (cscfgp, CSCFGP) (int *icfgrp);
 
 /*----------------------------------------------------------------------------
  * Restart parameters.
@@ -260,9 +169,9 @@ void CS_PROCF (cscfgp, CSCFGP) (int *const icfgrp);
  *----------------------------------------------------------------------------*/
 
 
-void CS_PROCF (csisui, CSISUI) (int *const ntsuit,
-                                int *const ileaux,
-                                int *const iccvfg);
+void CS_PROCF (csisui, CSISUI) (int *ntsuit,
+                                int *ileaux,
+                                int *iccvfg);
 
 /*----------------------------------------------------------------------------
  * Time passing parameters.
@@ -272,30 +181,9 @@ void CS_PROCF (csisui, CSISUI) (int *const ntsuit,
  * SUBROUTINE CSTIME
  * *****************
  *
- * INTEGER          INPDT0  -->   zero tim step
- * INTEGER          IPTLTO  -->   thermal time step control
- * INTEGER          NTMABS  -->   iterations numbers
- * INTEGER          IDTVAR  -->   time step's options
- * DOUBLE PRECISION DTREF   -->   time step
- * DOUBLE PRECISION DTMIN   -->   minimal time step
- * DOUBLE PRECISION DTMAX   -->   maximal time step
- * DOUBLE PRECISION COUMAX  -->   maximal courant number
- * DOUBLE PRECISION FOUMAX  -->   maximal fournier number
- * DOUBLE PRECISION VARRDT  -->   max time step variation between 2 iterations
- * DOUBLE PRECISION RELXST  -->   relaxation coefficient if idtvar = -1
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (cstime, CSTIME) (int    *const inpdt0,
-                                int    *const iptlro,
-                                int    *const ntmabs,
-                                int    *const idtvar,
-                                double *const dtref,
-                                double *const dtmin,
-                                double *const dtmax,
-                                double *const coumax,
-                                double *const foumax,
-                                double *const varrdt,
-                                double *const relxst);
+void CS_PROCF (cstime, CSTIME) (void);
 
 /*----------------------------------------------------------------------------
  *
@@ -306,16 +194,7 @@ void CS_PROCF (cstime, CSTIME) (int    *const inpdt0,
  *
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uinum1, UINUM1) (double *const blencv,
-                                   int *const ischcv,
-                                   int *const isstpc,
-                                   int *const ircflu,
-                                double *const cdtvar,
-                                   int *const nitmax,
-                                double *const epsilo,
-                                   int *const iresol,
-                                   int *const imgrpr,
-                                   int *const nswrsm);
+void CS_PROCF (uinum1, UINUM1) (double *cdtvar);
 
 /*----------------------------------------------------------------------------
  * Global numerical parameters.
@@ -325,42 +204,19 @@ void CS_PROCF (uinum1, UINUM1) (double *const blencv,
  * SUBROUTINE CSNUM2
  * *****************
  *
- * INTEGER          IVISSE  -->   gradient transpose
  * INTEGER          RELAXP  -->   pressure relaxation
- * INTEGER          IPUCOU  -->   velocity pressure coupling
  * INTEGER          EXTRAG  -->   wall pressure extrapolation
  * INTEGER          IMRGRA  -->   gradient reconstruction
- * INTEGER          NTERUP  -->   piso sweep number
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (csnum2, CSNUM2) (   int *const ivisse,
-                                double *const relaxp,
-                                   int *const ipucou,
-                                double *const extrag,
-                                   int *const imrgra,
-                                   int *const nterup);
+void CS_PROCF (csnum2, CSNUM2) (double *relaxp,
+                                double *extrag,
+                                   int *imrgra);
 
-void CS_PROCF (csphys, CSPHYS) (const    int *const nmodpp,
-                                         int *const irovar,
-                                         int *const ivivar,
-                                         int *const icorio,
-                                      double *const gx,
-                                      double *const gy,
-                                      double *const gz,
-                                      double *const omegax,
-                                      double *const omegay,
-                                      double *const omegaz,
-                                      double *const ro0,
-                                      double *const viscl0,
-                                      double *const viscv0,
-                                      double *const visls0,
-                                      double *const cp0,
-                                      double *const t0,
-                                      double *const p0,
-                                      double *const xmasmr,
-                                         int *const itempk,
-                                         int *const itherm,
-                                         int *const itpscl);
+void CS_PROCF (csphys, CSPHYS) (
+                                double       *viscv0,
+                                double       *visls0,
+                                const    int *itempk);
 
 /*----------------------------------------------------------------------------
  * User scalar min and max values for clipping.
@@ -370,37 +226,12 @@ void CS_PROCF (csphys, CSPHYS) (const    int *const nmodpp,
  * subroutine cssca2
  * *****************
  *
- * integer          iscavr   <--  number of the related variance if any
+ * integer          iturt    -->  turbulent flux model
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (cssca2, CSSCA2) (const    int *const iscavr);
+void CS_PROCF (cssca2, CSSCA2) (int        *iturt);
 
-void CS_PROCF (cssca3, CSSCA3) (const    int *const itherm,
-                                const    int *const iscalt,
-                                const    int *const iscavr,
-                                double *const visls0,
-                                double *const t0,
-                                double *const p0,
-                                double *const cp0);
-
-/*----------------------------------------------------------------------------
- * Array of properties used in the calculation
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uiprop, UIPROP) (const int *const ivisls,
-                                const int *const ismago,
-                                const int *const iale,
-                                const int *const icp,
-                                const int *const iscavr);
-
-/*----------------------------------------------------------------------------
- * Temporal averaging treatment
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uimoyt, UIMOYT) (const int *const ndgmox,
-                                      int *const ntdmom,
-                                      int *const imoold,
-                                      int *const idfmom);
+void CS_PROCF (cssca3, CSSCA3) (double     *visls0);
 
 /*----------------------------------------------------------------------------
  * Turbulence initialization parameters.
@@ -410,25 +241,9 @@ void CS_PROCF (uimoyt, UIMOYT) (const int *const ndgmox,
  * SUBROUTINE CSTINI
  * *****************
  *
- * INTEGER          UREF   -->   reference velocity
- * INTEGER          ALMAX  -->   reference length
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (cstini, CSTINI) (double *const uref,
-                                double *const almax);
-
-/*----------------------------------------------------------------------------
- * Solver taking a scalar porosity into account
- *
- * Fortran Interface:
- *
- * SUBROUTINE UIIPSU
- * *****************
- *
- * INTEGER          IPOROS     -->   porosity
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uiipsu, UIIPSU) (int *iporos);
+void CS_PROCF (cstini, CSTINI) (void);
 
 /*----------------------------------------------------------------------------
  * Define porosity.
@@ -436,13 +251,9 @@ void CS_PROCF (uiipsu, UIIPSU) (int *iporos);
  * Fortran Interface:
  *
  * SUBROUTINE UIPORO
- * *****************
- *
- * INTEGER          IPOROS     <--   porosity
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uiporo, UIPORO) (const int *ncelet,
-                                const int *iporos);
+void CS_PROCF (uiporo, UIPORO) (void);
 
 /*----------------------------------------------------------------------------
  * User momentum source terms.
@@ -462,7 +273,6 @@ void CS_PROCF(uitsnv, UITSNV)(const cs_real_3_t *restrict vel,
                               cs_real_3_t       *restrict tsexp,
                               cs_real_33_t      *restrict tsimp);
 
-
 /*----------------------------------------------------------------------------
  * User scalar source terms.
  *
@@ -471,18 +281,19 @@ void CS_PROCF(uitsnv, UITSNV)(const cs_real_3_t *restrict vel,
  * subroutine uitssc (f_id, pvar, tsexp, tsimp)
  * *****************
  *
+ * integer          idarcy   <--  groundwater module activation
  * integer          f_id     <--  field id
  * double precision pvar     <--  scalar
  * double precision tsexp    -->  explicit source terms
  * double precision tsimp    -->  implicit source terms
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF(uitssc, UITSSC)(const int                  *f_id,
+void CS_PROCF(uitssc, UITSSC)(const int                  *idarcy,
+                              const int                  *f_id,
                               const cs_real_t   *restrict pvar,
                               cs_real_t         *restrict tsexp,
                               cs_real_t         *restrict tsimp);
 
-
 /*----------------------------------------------------------------------------
  * Thermal scalar source terms.
  *
@@ -510,147 +321,179 @@ void CS_PROCF(uitsth, UITSTH)(const int                  *f_id,
  * subroutine uiiniv
  * *****************
  *
- * integer          ncelet   <--  number of cells with halo
  * integer          isuite   <--  restart indicator
- * integer          iccfth   <--  type of initialisation(compressible model)
- * double precision ro0      <--  value of density if IROVAR=0
- * double precision cp0      <--  value of specific heat if ICP=0
- * double precision viscl0   <--  value of viscosity if IVIVAR=0
- * double precision uref     <--  value of reference velocity
- * double precision almax    <--  value of reference length
- * double precision xyzcen   <--  cell's gravity center
+ * integer          idarcy   <--  groundwater module activation
+ * integer          iccfth   <--  type of initialization (compressible model)
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF(uiiniv, UIINIV)(const int          *ncelet,
-                              const int          *isuite,
-                                    int          *iccfth,
-                              const cs_real_t    *ro0,
-                              const cs_real_t    *cp0,
-                              const cs_real_t    *viscl0,
-                              const cs_real_t    *uref,
-                              const cs_real_t    *almax,
-                              const double *const xyzcen);
+void CS_PROCF(uiiniv, UIINIV)(const int          *isuite,
+                              const int          *idarcy,
+                                    int          *iccfth);
 
 /*----------------------------------------------------------------------------
  * User law for material Properties
  *
  * Fortran Interface:
  *
- * SUBROUTINE UIPHYV (NCELET, ISCA, RTP)
+ * subroutine uiphyv
  * *****************
  *
- * INTEGER          NCEL     <--  number of cells whithout halo
- * INTEGER          NCELET   <--  number of cells whith halo
- * INTEGER          ICP      <--  pointer for predifined heat Cp
- * INTEGER          IVISLS   <--  pointer for Lambda/Cp
- * INTEGER          IROVAR   <--  =1 if rho variable, =0 if rho constant
- * INTEGER          IVIVAR   <--  =1 if mu variable, =0 if mu constant
- * INTEGER          ISCALT   <--  pointer for the thermal scalar in ISCA
- * INTEGER          ISCAVR   <--  scalars that are variance
- * INTEGER          IVISCV   <--  pointer for volumic viscosity viscv
- * INTEGER          ITEMPK   <--  pointer for temperature (in K)
- * DOUBLE PRECISION P0       <--  pressure reference value
- * DOUBLE PRECISION T0       <--  temperature reference value
- * DOUBLE PRECISION RO0      <--  density reference value
- * DOUBLE PRECISION CP0      <--  specific heat reference value
- * DOUBLE PRECISION VISCL0   <--  dynamic viscosity reference value
- * DOUBLE PRECISION VISLS0   <--  diffusion coefficient of the scalars
- * DOUBLE PRECISION VISCV0   <--  volumic viscosity
+ * integer          iviscv   <--  pointer for volumic viscosity viscv
+ * integer          itempk   <--  pointer for temperature (in K)
+ * double precision visls0   <--  diffusion coefficient of the scalars
+ * double precision viscv0   <--  volumic viscosity
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF(uiphyv, UIPHYV)(const cs_int_t  *ncel,
-                              const cs_int_t  *ncelet,
-                              const cs_int_t  *itherm,
-                              const cs_int_t  *icp,
-                              const cs_int_t   ivisls[],
-                              const cs_int_t  *irovar,
-                              const cs_int_t  *ivivar,
-                              const cs_int_t  *iscalt,
-                              const cs_int_t   iscavr[],
-                              const cs_int_t  *iviscv,
+void CS_PROCF(uiphyv, UIPHYV)(const cs_int_t  *iviscv,
                               const cs_int_t  *itempk,
-                              const cs_real_t *p0,
-                              const cs_real_t *t0,
-                              const cs_real_t *ro0,
-                              const cs_real_t *cp0,
-                              const cs_real_t *viscl0,
                               const cs_real_t *visls0,
                               const cs_real_t *viscv0);
 
 /*----------------------------------------------------------------------------
- * Head losses definition
+ * extra operations
  *
  * Fortran Interface:
  *
- * subroutine uikpdc
+ * SUBROUTINE UIEXOP
  * *****************
  *
- * integer          iappel   <--  number of calls during a time step
- * integer          ncelet   <--  number of cells with halo
- * integer          ncepdp  -->   number of cells with head losses
- * integer          icepdc  -->   ncepdp cells number with head losses
- * double precision ckupdc  -->   head losses matrix
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF(uikpdc, UIKPDC)(const int*   iappel,
-                              const int*   ncelet,
-                                    int*   ncepdp,
-                                    int    icepdc[],
-                                    double ckupdc[]);
+void CS_PROCF (uiexop, UIEXOP)(void);
 
 /*----------------------------------------------------------------------------
- * 1D profile postprocessing
+ * groundwater model : read laws for capacity, saturation and permeability
  *
  * Fortran Interface:
  *
- * SUBROUTINE UIPROF
+ * subroutine uidapp
  * *****************
- *
- * INTEGER          NCELET   <--  number of cells with halo
- * INTEGER          NCEL     <--  number of cells without halo
- * INTEGER          NTMABS   <--  max iterations numbers
- * INTEGER          NTCABS   <--  current iteration number
- * DOUBLE PRECISION TTCABS   <--  current physical time
- * DOUBLE PRECISION TTMABS   <--  max physical time
- * DOUBLE PRECISION TTPABS   <--  physical time at calculation beginning
- * DOUBLE PRECISION XYZCEN   <--  cell's gravity center
+ * integer         permeability    <--  permeability type
+ * integer         diffusion       <--  diffusion type
+ * integer         gravity         <--  check if gravity is taken into account
+ * double          gravity_x       <--  gravity direction
+ * double          gravity_y       <--  gravity direction
+ * double          gravity_z       <--  gravity direction
+ * integer         unsaturated     <--  unsaturated zone taken into account
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (uiprof, UIPROF)(const int    *const ncelet,
-                               const int    *const ncel,
-                               const int    *const ntmabs,
-                               const int    *const ntcabs,
-                               const double *const ttcabs,
-                               const double *const ttmabs,
-                               const double *const ttpabs,
-                               const double *const xyzcen);
+void CS_PROCF (uidapp, UIDAPP) (const int       *permeability,
+                                const int       *diffusion,
+                                const int       *gravity,
+                                const cs_real_t *gravity_x,
+                                const cs_real_t *gravity_y,
+                                const cs_real_t *gravity_z,
+                                const int       *unsaturated);
 
 /*----------------------------------------------------------------------------
- * Free memory: clean global private variables and libxml2 variables.
+ * Define fans with GUI
  *
  * Fortran Interface:
  *
- * SUBROUTINE MEMUI1
+ * SUBROUTINE UIFANS
  * *****************
  *
- * INTEGER          NCHARB  <-- number of coal
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (memui1, MEMUI1) (const int *const ncharb);
+void CS_PROCF (uifans, UIFANS) (void);
+
+/*----------------------------------------------------------------------------
+ * Define error estimators
+ *
+ * Fortran Interface:
+ *
+ * SUBROUTINE UIERES
+ * *****************
+ *
+ *----------------------------------------------------------------------------*/
+
+void CS_PROCF (uieres, UIERES) (int *iescal,
+                                int *iespre,
+                                int *iesder,
+                                int *iescor,
+                                int *iestot);
 
 /*=============================================================================
  * Public function prototypes
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- * Get thermal scalar model.
- *
- * return:
- *   value of itherm
+ * Initialize GUI reader structures.
  *----------------------------------------------------------------------------*/
 
-int
-gui_thermal_model(void);
+void
+cs_gui_init(void);
+
+/*-----------------------------------------------------------------------------
+ * Free memory: clean global private variables.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_finalize(void);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Add notebook variable to a formula.
+ *
+ * \param[in, out]  ev_law  pointer to MEI formula structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gui_add_notebook_variables(void  *ev_law);
+
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Compute GUI-defined head losses for a given volume zone.
+ *
+ * Head loss tensor coefficients for each cell are organized as follows:
+ * cku11, cku22, cku33, cku12, cku13, cku23.
+ *
+ * \param[in]       zone       pointer to zone structure
+ * \param[in]       cvara_vel  pointer to the velocity values of the previous time step
+ * \param[in, out]  cku        head loss coefficients
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_gui_head_losses(const cs_zone_t   *zone,
+                   const cs_real_3_t *cvara_vel,
+                   cs_real_t          cku[][6]);
+
+/*-----------------------------------------------------------------------------
+ * Selection of linear solvers.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_linear_solvers(void);
+
+/*-----------------------------------------------------------------------------
+ * Define parallel IO settings.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_parallel_io(void);
+
+/*-----------------------------------------------------------------------------
+ * Set partitioning options.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_partition(void);
+
+/*----------------------------------------------------------------------------
+ * Determine porosity model type
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_porous_model(void);
+
+/*----------------------------------------------------------------------------
+ * 1D profile postprocessing
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_profile_output(void);
 
 /*-----------------------------------------------------------------------------
  * Get initial value from property markup.
@@ -665,7 +508,21 @@ cs_gui_properties_value(const char  *property_name,
                         double      *value);
 
 /*-----------------------------------------------------------------------------
- * Initialization choice of the reference variables parameters.
+ * Get value of property markup for fluid of given id
+ *
+ * parameters:
+ *   fluid_id           <--   fluid index
+ *   property_name      <--   name of the property
+ *   value              -->   new initial value of the property
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_properties_value_by_fluid_id(const int    fluid_id,
+                                    const char  *property_name,
+                                    double      *value);
+
+/*-----------------------------------------------------------------------------
+ * Get value of reference fluid properties parameter.
  *
  * parameters:
  *   name            <--   parameter name
@@ -673,29 +530,39 @@ cs_gui_properties_value(const char  *property_name,
  *----------------------------------------------------------------------------*/
 
 void
-cs_gui_reference_initialization(const char  *param,
-                                double      *value);
+cs_gui_fluid_properties_value(const char  *param,
+                              double      *value);
 
-/*-----------------------------------------------------------------------------
- * Set partitioning options.
+/*----------------------------------------------------------------------------
+ * Get thermal scalar model.
+ *
+ * return:
+ *   value of itherm
+ *----------------------------------------------------------------------------*/
+
+int
+cs_gui_thermal_model(void);
+
+/*----------------------------------------------------------------------------
+ * Time moments definition
  *----------------------------------------------------------------------------*/
 
 void
-cs_gui_partition(void);
+cs_gui_time_moments(void);
 
 /*-----------------------------------------------------------------------------
- * Define parallel IO settings.
+ * Set turbomachinery model
  *----------------------------------------------------------------------------*/
 
 void
-cs_gui_parallel_io(void);
+cs_gui_turbomachinery(void);
 
 /*-----------------------------------------------------------------------------
- * Free memory: clean global private variables and libxml2 variables
+ * Set turbomachinery options.
  *----------------------------------------------------------------------------*/
 
 void
-cs_gui_clean_memory(void);
+cs_gui_turbomachinery_rotor(void);
 
 /*----------------------------------------------------------------------------
  * Logging output for MEI usage.
@@ -704,6 +571,59 @@ cs_gui_clean_memory(void);
 void
 cs_gui_usage_log(void);
 
+/*----------------------------------------------------------------------------
+ * Define user variables through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_user_variables(void);
+
+/*----------------------------------------------------------------------------
+ * Define user arrays through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_user_arrays(void);
+
+/*----------------------------------------------------------------------------
+ * Define balance by zone through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_balance_by_zone(void);
+
+/*----------------------------------------------------------------------------
+ * Define pressure drop through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_pressure_drop_by_zone(void);
+
+/*----------------------------------------------------------------------------
+ * Define fans through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_define_fans(void);
+
+/*----------------------------------------------------------------------------
+ * Define error estimator through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_error_estimator(int *iescal,
+                       int *iespre,
+                       int *iesder,
+                       int *iescor,
+                       int *iestot);
+
+/*----------------------------------------------------------------------------
+ * Define volume and boundary zones through the GUI.
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_zones(void);
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
