@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -36,26 +36,22 @@
 BEGIN_C_DECLS
 
 /*============================================================================
- * Public Fortran function prototypes
+ * Public Function prototypes
  *============================================================================*/
 
-/*-----------------------------------------------------------------------------
- * Define new SYRTHES coupling.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define SYRTHES couplings based on the GUI-generated setup.
  *
  * In the case of a single Code_Saturne and single SYRTHES instance, the
- * syrthes_app_num and syrthes_name arguments are ignored.
+ * syrthes_name argument is ignored.
  *
  * In case of multiple couplings, a coupling will be matched with available
- * SYRTHES instances prioritarily based on the syrthes_name argument, then
- * on the syrthes_app_num argument. If syrthes_name is empty, matching will
- * be based on syrthes_app_num only.
- *
- * subroutine uisyrc
- * *****************
- *
- *----------------------------------------------------------------------------*/
+ * SYRTHES instances prioritarily based on the syrthes_name argument.
+ */
+/*----------------------------------------------------------------------------*/
 
-void CS_PROCF (uisyrc, UISYRC) (void);
+void cs_gui_syrthes_coupling(void);
 
 /*----------------------------------------------------------------------------*/
 
