@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -38,75 +38,55 @@
 BEGIN_C_DECLS
 
 /*============================================================================
- * Public function prototypes for Fortran API
+ * Public function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- *
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uiray1, UIRAY1) (int *const iirayo,
-                                int *const isuird,
-                                int *const i_quad,
-                                int *const ndirec,
-                                int *const nfreqr,
-                                int *const idiver,
-                                int *const iimpar,
-                                int *const iimlum);
-
-
-/*----------------------------------------------------------------------------
- *
- *----------------------------------------------------------------------------*/
-
-
-void CS_PROCF (uiray2, UIRAY2) (const    int *const itypfb,
-                                const    int *const iparoi,
-                                const    int *const iparug,
-                                const    int *const ivart,
-                                         int *const izfrdp,
-                                         int *const isothp,
-                                const    int *const itpimp,
-                                const    int *const ipgrno,
-                                const    int *const iprefl,
-                                const    int *const ifgrno,
-                                const    int *const ifrefl,
-                                const    int *const nzoppm,
-                                const    int *const nfabor,
-                                const    int *const nvar,
-                                      double *const epsp,
-                                      double *const epap,
-                                      double *const tintp,
-                                      double *const textp,
-                                      double *const xlamp,
-                                      double *const rcodcl);
-
-/*----------------------------------------------------------------------------
- *
- *----------------------------------------------------------------------------*/
-
-
-void CS_PROCF (uiray3, UIRAY3) (      double *const ck,
-                                const    int *const ncel,
-                                         int *const imodak);
-/*----------------------------------------------------------------------------
- *
- *----------------------------------------------------------------------------*/
-
-void CS_PROCF (uiray4, UIRAY4) (int *const iirayo);
-
-
 /*-----------------------------------------------------------------------------
- * Free memory: clean global private variables.
- *
- * Fortran Interface:
- *
- * SUBROUTINE MEMUI2
- * *****************
- *
+ * Free GUI-defined radiative transfer parameters
  *----------------------------------------------------------------------------*/
 
-void CS_PROCF (memui2, MEMUI2) (void);
+void
+cs_gui_radiative_transfers_finalize(void);
+
+/*----------------------------------------------------------------------------
+ * Read GUI-defined radiative transfer parameters
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_radiative_transfer_parameters(void);
+
+/*----------------------------------------------------------------------------
+ * Set the radiative transfer absorption coefficient
+ *
+ * parameters:
+ *   ck --> absorption coefficient at cells
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_rad_transfer_absorption(cs_real_t  ck[]);
+
+/*----------------------------------------------------------------------------
+ * Postprocessing settings for radiative transfer
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_radiative_transfer_postprocess(void);
+
+/*----------------------------------------------------------------------------
+ * Radiative transfer boundary conditions
+ *----------------------------------------------------------------------------*/
+
+void
+cs_gui_radiative_transfer_bcs(const    int   itypfb[],
+                              int            nvar,
+                              int            ivart,
+                              int           *isothp,
+                              double        *epsp,
+                              double        *epap,
+                              double        *tintp,
+                              double        *textp,
+                              double        *xlamp,
+                              double        *rcodcl);
 
 /*----------------------------------------------------------------------------*/
 
