@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -50,19 +50,11 @@ typedef struct {
   char  *model_value;      /* predifined physics model value                  */
 } cs_var_t;
 
-
-typedef struct {
-  int     _cs_gui_max_vars;
-  int     _cs_gui_last_var;
-  char  **_cs_gui_var_name;
-} cs_label_t;
-
 /*============================================================================
  * Static global variables
  *============================================================================*/
 
 extern cs_var_t    *cs_glob_var;   /* Pointer to main variables structure */
-extern cs_label_t  *cs_glob_label; /* Pointer to main label structure */
 
 /*============================================================================
  * Public function prototypes for Fortran API
