@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -32,7 +32,7 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_defs.h"
-#include "cs_lagr_tracking.h"
+#include "cs_lagr_particle.h"
 
 /*----------------------------------------------------------------------------*/
 
@@ -44,26 +44,25 @@ BEGIN_C_DECLS
 
 typedef struct {
 
-  cs_real_t   faraday_cst;
-  cs_real_t   free_space_permit;
   cs_real_t   water_permit;
   cs_real_t   ionic_strength;
   cs_real_t   jamming_limit;
   cs_real_t   min_porosity;
-  cs_real_t   phi1;
-  cs_real_t   phi2;
+  cs_real_t   diam_mean;
+  cs_real_t   valen;
+  cs_real_t   phi_p;
+  cs_real_t   phi_s;
 
   cs_real_t*  temperature;
   cs_real_t*  debye_length;
   cs_real_t   cstham;
-  cs_real_t   dcutof;
-  cs_real_t   lambwl;
-  cs_real_t   kboltz;
+  cs_real_t   csthpp;
+  cs_real_t   lambda_vdw;
 
 } cs_lagr_clogging_param_t;
 
 /*============================================================================
- * Public function definitions for Fortran API
+ * Public function definitions
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
@@ -74,24 +73,19 @@ typedef struct {
  *----------------------------------------------------------------------------*/
 
 void
-CS_PROCF (cloginit, CLOGINIT)(const cs_real_t   *faraday_cst,
-                              const cs_real_t   *free_space_permit,
-                              const cs_real_t   *water_permit,
-                              const cs_real_t   *ionic_strength,
-                              const cs_real_t   *jamming_limit,
-                              const cs_real_t   *min_porosity,
-                              const cs_real_t    temperature[],
-                              const cs_real_t   *phi1,
-                              const cs_real_t   *phi2,
-                              const cs_real_t   *cstham,
-                              const cs_real_t   *dcutof,
-                              const cs_real_t   *lambwl,
-                              const cs_real_t   *kboltz
-);
-
-/*============================================================================
- * Public function definitions
- *============================================================================*/
+cloginit(const cs_real_t   *water_permit,
+         const cs_real_t   *ionic_strength,
+         const cs_real_t   *jamming_limit,
+         const cs_real_t   *min_porosity,
+         const cs_real_t   *diam_mean,
+         const cs_real_t    temperature[],
+         const cs_real_t   *valen,
+         const cs_real_t   *phi_p,
+         const cs_real_t   *phi_s,
+         const cs_real_t   *cstham,
+         const cs_real_t   *csthpp,
+         const cs_real_t   *lambda_vdw
+         );
 
 /*----------------------------------------------------------------------------
  * Clogging finalization.
@@ -112,7 +106,7 @@ cs_lagr_clogging_finalize(void);
  * parameters:
  *   particle         <-- pointer to particle data
  *   attr_map         <-- pointer to attribute map
- *   face_id          <-- id of face neighboring particle
+ *   iel              <-- id of cell where the particle is
  *   face_area        <-- area of face
  *   energy_barrier   <-> energy barrier
  *   surface_coverage <-> surface coverage
@@ -126,8 +120,7 @@ cs_lagr_clogging_finalize(void);
 int
 cs_lagr_clogging_barrier(const void                     *particle,
                          const cs_lagr_attribute_map_t  *attr_map,
-                         cs_lnum_t                       face_id,
-                         cs_real_t                       face_area,
+                         cs_lnum_t                       iel,
                          cs_real_t                      *energy_barrier,
                          cs_real_t                      *surface_coverage,
                          cs_real_t                      *limit,
