@@ -7,7 +7,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,14 +45,14 @@ extern "C" {
 #include "bft_mem.h"
 #include "bft_error.h"
 
-#include "mei_math_util.h"
-
 /*----------------------------------------------------------------------------
  * Header for the current file
  *----------------------------------------------------------------------------*/
 
 #include "mei_hash_table.h"
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Private functions definitions
  *============================================================================*/
@@ -109,6 +109,8 @@ _new_item(const char *const key)
   return new;
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public functions definitions
  *============================================================================*/
@@ -183,9 +185,6 @@ mei_hash_table_find(hash_table_t *const htable, const char *const key)
  * \param [in] value store a value if the record if a real
  * \param [in] f1 pointer on a one argument function
  * \param [in] f2 pointer on a two argument function
- * \param [in] f3 pointer on a three argument function
- * \param [in] f4 pointer on a four argument function
- * \param [in] i1d pointer on a 1D interpolation function
  */
 /*----------------------------------------------------------------------------*/
 
@@ -195,10 +194,7 @@ mei_hash_table_insert(hash_table_t *const htable,
                       const mei_flag_t type,
                       const double value,
                       const func1_t f1,
-                      const func2_t f2,
-                      const func3_t f3,
-                      const func4_t f4,
-                      const interp1d_t i1d)
+                      const func2_t f2)
 {
   unsigned v;
   struct item* item = mei_hash_table_find(htable, key);
@@ -221,9 +217,6 @@ mei_hash_table_insert(hash_table_t *const htable,
     } else if (type == FUNC4) {
       bft_error(__FILE__, __LINE__, 0, "not implemented yet \n");
 
-    } else if (type == INTERP1D) {
-      item->data->i1d = i1d;
-
     } else {
       item->data->value = value;
     }
@@ -350,15 +343,7 @@ mei_hash_table_init(hash_table_t *const htable)
   static const char *functions2_names[] = { "atan2", "min", "max", "mod" };
 
   /* predefined functions pointers to functions to calculate them */
-  static double (*functions2[]) (double, double) = { atan2, mei_min, mei_max, fmod };
-
-  /* predefined functions names with two arguments*/
-  static const char *interp1d_names[] = { "interp1d" };
-
-  /* predefined functions pointers to functions to calculate them */
-  static double (*interp1d[]) (char*, int, int, double) = { mei_interp1d };
-
-
+  static double (*functions2[]) (double, double) = { atan2, fmin, fmax, fmod };
 
   j = sizeof(constants_names) / sizeof(constants_names[0]);
   for (i = 0; i < j; i++)
@@ -367,9 +352,6 @@ mei_hash_table_init(hash_table_t *const htable)
                           CONSTANT,
                           constants[i],
                           NULL,
-                          NULL,
-                          NULL,
-                          NULL,
                           NULL);
 
   j = sizeof(functions_names) / sizeof(functions_names[0]);
@@ -379,9 +361,6 @@ mei_hash_table_init(hash_table_t *const htable)
                           FUNC1,
                           0,
                           functions[i],
-                          NULL,
-                          NULL,
-                          NULL,
                           NULL);
 
   j = sizeof(functions2_names) / sizeof(functions2_names[0]);
@@ -391,22 +370,7 @@ mei_hash_table_init(hash_table_t *const htable)
                           FUNC2,
                           0,
                           NULL,
-                          functions2[i],
-                          NULL,
-                          NULL,
-                          NULL);
-
-  j = sizeof(interp1d_names) / sizeof(interp1d_names[0]);
-  for (i = 0; i < j; i++)
-    mei_hash_table_insert(htable,
-                          interp1d_names[i],
-                          INTERP1D,
-                          0,
-                          NULL,
-                          NULL,
-                          NULL,
-                          NULL,
-                          interp1d[i]);
+                          functions2[i]);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -426,8 +390,7 @@ mei_hash_table_item_print(struct item *item)
     if (item->type != FUNC1 &&
         item->type != FUNC2 &&
         item->type != FUNC3 &&
-        item->type != FUNC4 &&
-        item->type != INTERP1D)
+        item->type != FUNC4)
       printf("valeur : %f\n", item->data->value);
     item = item->next;
   }
@@ -457,7 +420,7 @@ mei_hash_table_dump(hash_table_t *const htable)
  * Test
  *============================================================================*/
 
-#ifndef DOXYGEN_SHOULD_SKIP_THIS
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
 
 #undef _TEST_
 #ifdef _TEST_
@@ -483,8 +446,7 @@ main_test(int argc, char *argv[])
 
 #endif
 
-#endif /* DOXYGEN_SHOULD_SKIP_THIS */
-
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*----------------------------------------------------------------------------*/
 
