@@ -15,7 +15,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -54,7 +54,6 @@ typedef enum {
   FUNC2,
   FUNC3,
   FUNC4,
-  INTERP1D,
   OPR
 
 } mei_flag_t;
@@ -87,12 +86,6 @@ typedef double (*func3_t) (double, double, double);
 
 typedef double (*func4_t) (double, double, double, double);
 
-/*!
- * \brief Type definition for pointer to a function of for 1D interpolation
- */
-
-typedef double (*interp1d_t) (char*, int, int, double);
-
 /*!
  * \brief Type definition for data of each element contained in the hash table
  */
@@ -100,8 +93,7 @@ typedef double (*interp1d_t) (char*, int, int, double);
 typedef union {
   double value;    /*!< Constant or variable value.  */
   func1_t func;    /*!< Pointer to function with one argument */
-  func2_t f2;      /*!< Pointer to function with two argument */
-  interp1d_t i1d;  /*!< Pointer to function for 1D interpolation */
+  func2_t f2;      /*!< Pointer to function with two arguments */
 } data_t;
 
 /*!
@@ -193,21 +185,16 @@ struct item * mei_hash_table_lookup(hash_table_t *htable,
  * param [in] value store a value if the record if a real
  * param [in] f1 pointer on a one argument function
  * param [in] f2 pointer on a two argument function
- * param [in] f3 pointer on a three argument function
- * param [in] f4 pointer on a four argument function
- * param [in] i1d pointer on a 1D interpolation function
  */
 /*----------------------------------------------------------------------------*/
 
-void mei_hash_table_insert(hash_table_t *const htable,
-                           const char *const  key,
-                           const mei_flag_t type,
-                           const double value,
-                           const func1_t func,
-                           const func2_t f2,
-                           const func3_t f3,
-                           const func4_t f4,
-                           const interp1d_t i1d);
+void
+mei_hash_table_insert(hash_table_t *const htable,
+                      const char *const  key,
+                      const mei_flag_t type,
+                      const double value,
+                      const func1_t f1,
+                      const func2_t f2);
 
 /*----------------------------------------------------------------------------*/
 /*
