@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -33,6 +33,7 @@
 
 #include "cs_base.h"
 #include "cs_join_util.h"
+#include "cs_mesh.h"
 
 /*---------------------------------------------------------------------------*/
 
@@ -122,6 +123,21 @@ cs_join_all(bool  preprocess);
 void
 cs_join_finalize(void);
 
+/*----------------------------------------------------------------------------
+ * Flag boundary faces that will be selected for joining.
+ *
+ * parameters:
+ *   mesh          <-- pointer to mesh structure
+ *   preprocess    <-- true if we are in the preprocessing stage
+ *   b_select_flag <-> false for boundary faces not selected for joining,
+ *                     true for those selected
+ *---------------------------------------------------------------------------*/
+
+void
+cs_join_mark_selected_faces(const cs_mesh_t  *mesh,
+                            bool              preprocess,
+                            bool              b_select_flag[]);
+
 /*---------------------------------------------------------------------------*/
 
 END_C_DECLS
