@@ -10,7 +10,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -59,7 +59,7 @@ typedef struct {
 
   cs_lnum_t   edge_id;    /* id of the edge implied in this intersection */
   cs_lnum_t   vtx_id;     /* id of the vertex resulting of the intersection */
-  float       curv_abs;   /* curvilinear abscissa of the intersection */
+  cs_coord_t  curv_abs;   /* curvilinear abscissa of the intersection */
 
 } cs_join_inter_t;
 
@@ -92,13 +92,13 @@ typedef struct {
 
   cs_lnum_t   *vtx_lst;    /* List of new vertex num */
   cs_gnum_t   *vtx_glst;   /* List of new vertex global num */
-  float       *abs_lst;    /* List of curvilinear abscissa */
+  cs_coord_t  *abs_lst;    /* List of curvilinear abscissa */
 
   cs_lnum_t    max_sub_size;
 
 } cs_join_inter_edges_t;
 
-/*! \endcond (end ignore by Doxygen) */
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
  * Public function prototypes
@@ -292,17 +292,19 @@ cs_join_intersect_edges(cs_join_param_t         param,
  * storing potential intersections between face bounding boxes.
  *
  * parameters:
- *  param   <--  set of user-defined parameter
- *  mesh    <--  cs_join_mesh_t structure where faces are defined
+ *   param     <-- set of user-defined parameters
+ *   join_mesh <-- cs_join_mesh_t structure where faces are defined
+ *   stats     <-> joining statistics
  *
  * returns:
- *  a new allocated pointer to a cs_join_gset_t structure storing the
- *  face - face visibility.
+ *   a new allocated pointer to a cs_join_gset_t structure storing the
+ *   face - face visibility.
  *---------------------------------------------------------------------------*/
 
 cs_join_gset_t *
 cs_join_intersect_faces(const cs_join_param_t   param,
-                        const cs_join_mesh_t   *mesh);
+                        const cs_join_mesh_t   *join_mesh,
+                        cs_join_stats_t        *stats);
 
 /*----------------------------------------------------------------------------
  * Transform face visibility into edge visibility.
