@@ -1,11 +1,11 @@
 /*============================================================================
- * Management of post-treatment for joining operation
+ * Management of post-processing for joining operation
  *===========================================================================*/
 
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -44,21 +44,25 @@
 #include "fvm_nodal.h"
 #include "fvm_nodal_order.h"
 #include "fvm_nodal_from_desc.h"
+#include "fvm_writer.h"
 
+#include "cs_file.h"
 #include "cs_mesh_connect.h"
 #include "cs_post.h"
+#include "cs_timer_stats.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
  *---------------------------------------------------------------------------*/
 
-#include "cs_prototypes.h"
 #include "cs_join_post.h"
 
 /*---------------------------------------------------------------------------*/
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Macro and type definitions
  *===========================================================================*/
@@ -83,6 +87,7 @@ typedef struct {
 static  cs_join_post_t  _cs_join_post_param;
 
 static  bool            _cs_join_post_initialized = false;
+static  int             _post_stage_stat_id = -1;
 
 /*============================================================================
  * Private function definitions
@@ -94,7 +99,7 @@ static  bool            _cs_join_post_initialized = false;
  * post elements implied in the joining operations.
  *
  * returns:
- *   id of associated writer
+ *   id of associated writer (< 0, or 0 in case of failure)
  *----------------------------------------------------------------------------*/
 
 static int
@@ -102,13 +107,25 @@ _init_join_writer(void)
 {
   int  writer_id = cs_post_get_free_writer_id();
 
+  /* Special case for Catalyst: if matching co-processing script is
+     not available, revert to EnSight Gold format */
+
+  int default_format_id
+    = fvm_writer_get_format_id(cs_post_get_default_format());
+
+  if (default_format_id == fvm_writer_get_format_id("Catalyst")) {
+    if (! cs_file_isreg("error.py"))
+      return 0;
+  }
+
   cs_post_define_writer(writer_id,
                         "joining",
                         "postprocessing",
-                        cs_post_get_default_format(),
+                        fvm_writer_format_name(default_format_id),
                         cs_post_get_default_format_options(),
                         FVM_WRITER_FIXED_MESH,
                         false,
+                        false,
                         -1,
                         -1.0);
 
@@ -213,6 +230,8 @@ _post_elt_ifield(fvm_nodal_t  *mesh,
                           (const void **)var_ptr);
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *===========================================================================*/
@@ -224,20 +243,23 @@ _post_elt_ifield(fvm_nodal_t  *mesh,
 void
 cs_join_post_init(void)
 {
-  int  writer_num;
-
   if (_cs_join_post_initialized == true)
     return;
 
-  _cs_join_post_initialized = true;
+  _post_stage_stat_id = cs_timer_stats_id_by_name("postprocessing_stage");
 
-  writer_num = _init_join_writer();
+  int  writer_num = _init_join_writer();
 
-  cs_post_activate_writer(writer_num, 1);
+  if (writer_num != 0) {
 
-  _cs_join_post_param.writer = cs_post_get_writer(writer_num);
+    _cs_join_post_initialized = true;
 
-  _cs_join_post_param.writer_num = writer_num;
+    cs_post_activate_writer(writer_num, 1);
+
+    _cs_join_post_param.writer = cs_post_get_writer(writer_num);
+    _cs_join_post_param.writer_num = writer_num;
+
+  }
 }
 
 /*----------------------------------------------------------------------------
@@ -252,6 +274,11 @@ void
 cs_join_post_mesh(const char            *mesh_name,
                   const cs_join_mesh_t  *join_mesh)
 {
+  if (_cs_join_post_initialized == true)
+    return;
+
+  int t_top_id = cs_timer_stats_switch(_post_stage_stat_id);
+
   int  i, j;
   cs_lnum_t  n_vertices;
 
@@ -357,6 +384,8 @@ cs_join_post_mesh(const char            *mesh_name,
   BFT_FREE(dfield);
 
   post_mesh = fvm_nodal_destroy(post_mesh);
+
+  cs_timer_stats_switch(t_top_id);
 }
 
 /*----------------------------------------------------------------------------
@@ -375,6 +404,11 @@ cs_join_post_faces_subset(const char            *mesh_name,
                           cs_lnum_t              n_select_faces,
                           const cs_lnum_t        selected_faces[])
 {
+  if (_cs_join_post_initialized == true)
+    return;
+
+  int t_top_id = cs_timer_stats_switch(_post_stage_stat_id);
+
   cs_join_mesh_t  *subset_mesh = NULL;
 
   assert(parent_mesh != NULL);
@@ -387,6 +421,8 @@ cs_join_post_faces_subset(const char            *mesh_name,
   cs_join_post_mesh(subset_mesh->name, subset_mesh);
 
   cs_join_mesh_destroy(&subset_mesh);
+
+  cs_timer_stats_switch(t_top_id);
 }
 
 /*----------------------------------------------------------------------------
@@ -401,15 +437,17 @@ void
 cs_join_post_after_merge(cs_join_param_t          join_param,
                          const cs_join_select_t  *join_select)
 {
+  if (_cs_join_post_initialized == true)
+    return;
+
+  int t_top_id = cs_timer_stats_switch(_post_stage_stat_id);
+
   int  adj_mesh_id, sel_mesh_id;
 
   int  writer_ids[] = {_cs_join_post_param.writer_num};
   char  *mesh_name = NULL;
   fvm_nodal_t *adj_mesh = NULL, *sel_mesh = NULL;
 
-  if (_cs_join_post_initialized == false)
-    return;
-
   adj_mesh_id = cs_post_get_free_mesh_id();
 
   BFT_MALLOC(mesh_name, strlen("AdjacentJoinFaces_j") + 2 + 1, char);
@@ -461,6 +499,8 @@ cs_join_post_after_merge(cs_join_param_t          join_param,
   cs_post_free_mesh(adj_mesh_id);
 
   BFT_FREE(mesh_name);
+
+  cs_timer_stats_switch(t_top_id);
 }
 
 /*----------------------------------------------------------------------------
@@ -483,6 +523,11 @@ cs_join_post_after_split(cs_lnum_t         n_old_i_faces,
                          const cs_mesh_t  *mesh,
                          cs_join_param_t   join_param)
 {
+  if (join_param.visualization < 1 || _cs_join_post_initialized == false)
+    return;
+
+  int t_top_id = cs_timer_stats_switch(_post_stage_stat_id);
+
   cs_lnum_t  i, j;
 
   int  writer_ids[] = {_cs_join_post_param.writer_num};
@@ -495,12 +540,6 @@ cs_join_post_after_split(cs_lnum_t         n_old_i_faces,
   const int  n_new_i_faces = mesh->n_i_faces - n_old_i_faces;
   const int  n_new_b_faces = mesh->n_b_faces - n_old_b_faces + n_select_faces;
 
-  if (join_param.visualization < 1)
-    return;
-
-  if (_cs_join_post_initialized == false)
-    return;
-
   /* Define list of faces to post-treat */
 
   BFT_MALLOC(post_i_faces, n_new_i_faces, cs_lnum_t);
@@ -569,6 +608,8 @@ cs_join_post_after_split(cs_lnum_t         n_old_i_faces,
   BFT_FREE(post_i_faces);
   BFT_FREE(post_b_faces);
   BFT_FREE(mesh_name);
+
+  cs_timer_stats_switch(t_top_id);
 }
 
 /*----------------------------------------------------------------------------
@@ -589,14 +630,16 @@ cs_join_post_cleaned_faces(cs_lnum_t        n_i_clean_faces,
                            cs_lnum_t        b_clean_faces[],
                            cs_join_param_t  param)
 {
+  if (_cs_join_post_initialized == false)
+    return;
+
+  int t_top_id = cs_timer_stats_switch(_post_stage_stat_id);
+
   int  writer_ids[] = {_cs_join_post_param.writer_num};
   int  post_mesh_id = cs_post_get_free_mesh_id();
   char  *name = NULL;
   fvm_nodal_t *export_mesh = NULL;
 
-  if (_cs_join_post_initialized == false)
-    return;
-
   BFT_MALLOC(name, strlen("CleanFaces_j") + 2 + 1, char);
   sprintf(name,"%s%02d", "CleanFaces_j", param.num);
 
@@ -624,6 +667,8 @@ cs_join_post_cleaned_faces(cs_lnum_t        n_i_clean_faces,
   cs_post_free_mesh(post_mesh_id);
 
   BFT_FREE(name);
+
+  cs_timer_stats_switch(t_top_id);
 }
 
 /*----------------------------------------------------------------------------
