@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -62,6 +62,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Macro definitions
  *===========================================================================*/
@@ -217,9 +219,9 @@ _face_bbox(const cs_join_mesh_t  *m,
   const double  eps = 1e-5;
   const cs_join_vertex_t  *vertices = m->vertices;
 
-  for (i = m->face_vtx_idx[fid] - 1; i < m->face_vtx_idx[fid+1] - 1; i++) {
+  for (i = m->face_vtx_idx[fid]; i < m->face_vtx_idx[fid+1]; i++) {
 
-    int  vid = m->face_vtx_lst[i] - 1;
+    int  vid = m->face_vtx_lst[i];
     double  tol = (1 + eps) * vertices[vid].tolerance;
 
     for (k = 0; k < 3; k++) {
@@ -304,20 +306,24 @@ _renumber_local_ordered_i(cs_lnum_t         n_elts,
  * parameters:
  *   o2n_hist        <-- old -> new global face numbering
  *   gnum_rank_index <-- index on ranks for the old global face numbering
- *   send_rank_index --> index on ranks for sending face
+ *   n_send          --> number of faces/rank couples to send
+ *   send_rank       --> ids of ranks for sending faces
  *   send_faces      --> list of face ids to send
  *---------------------------------------------------------------------------*/
 
 static void
 _get_faces_to_send(const cs_join_gset_t  *o2n_hist,
                    const cs_gnum_t        gnum_rank_index[],
-                   cs_lnum_t             *send_rank_index[],
+                   cs_lnum_t             *n_send,
+                   cs_lnum_t             *send_rank[],
                    cs_lnum_t             *send_faces[])
 {
   cs_lnum_t  i, j, rank, start, end;
 
+  cs_lnum_t _n_send = 0;
   cs_lnum_t  reduce_size = 0;
-  cs_lnum_t  *_send_rank_index = NULL, *_send_faces = NULL, *reduce_ids = NULL;
+  int  *_send_rank = NULL;
+  cs_lnum_t  *_send_faces = NULL, *reduce_ids = NULL;
   cs_gnum_t  *reduce_index = NULL;
   cs_join_gset_t  *new_face_rank = NULL;
 
@@ -416,14 +422,18 @@ _get_faces_to_send(const cs_join_gset_t  *o2n_hist,
 
   /* Define arrays to return */
 
-  BFT_MALLOC(_send_rank_index, n_ranks + 1, cs_lnum_t);
+  _n_send = new_face_rank->index[n_ranks];
 
-  for (i = 0; i < n_ranks + 1; i++)
-    _send_rank_index[i] = new_face_rank->index[i];
+  BFT_MALLOC(_send_rank, _n_send, int);
 
-  BFT_MALLOC(_send_faces, _send_rank_index[n_ranks], cs_lnum_t);
+  for (i = 0; i < n_ranks; i++) {
+    for (j = new_face_rank->index[i]; j < new_face_rank->index[i+1]; j++)
+      _send_rank[j] = i;
+  }
 
-  for (i = 0; i < _send_rank_index[n_ranks]; i++)
+  BFT_MALLOC(_send_faces, _n_send, cs_lnum_t);
+
+  for (i = 0; i < _n_send; i++)
     _send_faces[i] = new_face_rank->g_list[i];
 
   cs_join_gset_destroy(&new_face_rank);
@@ -432,21 +442,16 @@ _get_faces_to_send(const cs_join_gset_t  *o2n_hist,
   if (cs_glob_join_log != NULL) {
     fprintf(cs_glob_join_log,
             "\n Exchange to do after the splitting operation:\n");
-    for (i = 0; i < n_ranks; i++) {
-      start = _send_rank_index[i];
-      end = _send_rank_index[i+1];
+    for (i = 0; i < _n_send; i++)
       fprintf(cs_glob_join_log,
-              " Send to rank %5d (n = %10d):", i, end - start);
-      for (j = start; j < end; j++)
-        fprintf(cs_glob_join_log, " %d ", _send_faces[j]);
-      fprintf(cs_glob_join_log, "\n");
-    }
+              " send %10d to rank %5d\n", _send_faces[i], _send_rank[i]);
   }
 #endif
 
   /* Return pointers */
 
-  *send_rank_index = _send_rank_index;
+  *n_send = _n_send;
+  *send_rank = _send_rank;
   *send_faces = _send_faces;
 }
 
@@ -479,8 +484,8 @@ _define_head_and_ext_edges(cs_lnum_t               face_id,
   cs_lnum_t  i, j, k, shift;
   cs_lnum_t  couple[2];
 
-  cs_lnum_t  start_id = mesh->face_vtx_idx[face_id]-1;
-  cs_lnum_t  end_id = mesh->face_vtx_idx[face_id+1]-1;
+  cs_lnum_t  start_id = mesh->face_vtx_idx[face_id];
+  cs_lnum_t  end_id = mesh->face_vtx_idx[face_id+1];
   cs_lnum_t  n_face_vertices = end_id - start_id;
 
   assert(perm < n_face_vertices);
@@ -490,8 +495,8 @@ _define_head_and_ext_edges(cs_lnum_t               face_id,
     couple[0] = mesh->face_vtx_lst[i];
     couple[1] = mesh->face_vtx_lst[i+1];
 
-    for (j = edges->vtx_idx[couple[0]-1];
-         j < edges->vtx_idx[couple[0]]; j++)
+    for (j = edges->vtx_idx[couple[0]];
+         j < edges->vtx_idx[couple[0]+1]; j++)
       if (edges->adj_vtx_lst[j] == couple[1])
         break;
 
@@ -507,8 +512,8 @@ _define_head_and_ext_edges(cs_lnum_t               face_id,
   couple[0] = mesh->face_vtx_lst[end_id-1];
   couple[1] = mesh->face_vtx_lst[start_id];
 
-  for (j = edges->vtx_idx[couple[0]-1];
-       j < edges->vtx_idx[couple[0]]; j++)
+  for (j = edges->vtx_idx[couple[0]];
+       j < edges->vtx_idx[couple[0]+1]; j++)
     if (edges->adj_vtx_lst[j] == couple[1])
       break;
 
@@ -863,7 +868,7 @@ _find_next(cs_join_param_t         param,
 
     for (i = v2v_s; i < v2v_e; i++) {
 
-      cs_lnum_t  vid3 = edges->adj_vtx_lst[i]-1;
+      cs_lnum_t  vid3 = edges->adj_vtx_lst[i];
 
 #if _DBGTST && defined(DEBUG) && !defined(NDEBUG)
       if (tst_dbg && cs_glob_join_log != NULL)
@@ -1011,7 +1016,7 @@ _find_next(cs_join_param_t         param,
 
     for (i = v2v_s; i < v2v_e; i++) {
 
-      cs_lnum_t  vid3 = edges->adj_vtx_lst[i]-1;
+      cs_lnum_t  vid3 = edges->adj_vtx_lst[i];
 
       if (vid3 != vid1) {
         *next_edge = edges->edge_lst[i];
@@ -1728,22 +1733,22 @@ _update_mesh_after_split(cs_block_dist_info_t    bi,
 
   } /* End of loop on new faces */
 
-  new_mesh->face_vtx_idx[0] = 1;
+  new_mesh->face_vtx_idx[0] = 0;
   for (i = 0; i < n_new_faces; i++)
     new_mesh->face_vtx_idx[i+1] += new_mesh->face_vtx_idx[i];
 
   BFT_MALLOC(new_mesh->face_vtx_lst,
-             new_mesh->face_vtx_idx[n_new_faces]-1,
+             new_mesh->face_vtx_idx[n_new_faces],
              cs_lnum_t);
 
   for (i = 0; i < n_new_faces; i++) {
 
     id = subfaces[i];
-    shift = new_mesh->face_vtx_idx[i] - 1;
+    shift = new_mesh->face_vtx_idx[i];
 
     for (j = builder->subface_index->array[id], k = 0;
          j < builder->subface_index->array[id+1]; j++, k++)
-      new_mesh->face_vtx_lst[shift + k] = builder->subface_connect->array[j];
+      new_mesh->face_vtx_lst[shift + k] = builder->subface_connect->array[j] - 1;
 
   } /* End of loop on new faces */
 
@@ -1814,6 +1819,8 @@ _update_mesh_after_split(cs_block_dist_info_t    bi,
 
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *===========================================================================*/
@@ -2043,7 +2050,7 @@ cs_join_split_faces(cs_join_param_t          param,
 
           for (j = 0; j < n_face_vertices; j++)
             builder->subface_connect->array[subface_s + j]
-              = w->face_vtx_lst[j + w->face_vtx_idx[fid] - 1];
+              = w->face_vtx_lst[j + w->face_vtx_idx[fid]] + 1;
 
           if (param.verbosity > 2) {
             fprintf(cs_glob_join_log,
@@ -2272,7 +2279,9 @@ cs_join_split_update_struct(const cs_join_param_t   param,
   if (n_ranks > 1) { /* Parallel mode */
 
     cs_lnum_t  j, subface_id;
-    cs_lnum_t  *send_rank_index = NULL, *send_faces = NULL;
+    cs_lnum_t   n_send = 0;
+    int        *send_rank = NULL;
+    cs_lnum_t  *send_faces = NULL;
     cs_gnum_t  *init_face_gnum = NULL;
     cs_join_gset_t  *distrib_sync_hist = NULL;
     cs_lnum_t  n_init_faces = _local_mesh->n_faces;
@@ -2313,20 +2322,21 @@ cs_join_split_update_struct(const cs_join_param_t   param,
 
     _get_faces_to_send(_o2n_hist, /* Local subface num is stored up to now */
                        gnum_rank_index,
-                       &send_rank_index,
+                       &n_send,
+                       &send_rank,
                        &send_faces);
 
     /* Get the new face connectivity from the distributed work_mesh */
 
-    cs_join_mesh_exchange(n_ranks,
-                          send_rank_index,
+    cs_join_mesh_exchange(n_send,
+                          send_rank,
                           send_faces,
                           work_mesh,
                           _local_mesh,
                           mpi_comm);
 
     BFT_FREE(send_faces);
-    BFT_FREE(send_rank_index);
+    BFT_FREE(send_rank);
 
     /* Order face by increasing global number */
 
