@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -63,6 +63,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Static global variables
  *===========================================================================*/
@@ -219,6 +221,46 @@ _join_param_define(int                      join_num,
    return param;
 }
 
+/*----------------------------------------------------------------------------
+ * Initialize a cs_join_stats_t structure.
+ *
+ * parameters:
+ *   join_num      <-- number of the current joining operation
+ *   fraction      <-- value of the fraction parameter
+ *   plane         <-- value of the plane parameter
+ *   perio_type    <-- periodicity type (FVM_PERIODICITY_NULL if none)
+ *   perio_matrix  <-- periodicity transformation matrix
+ *   verbosity     <-- level of verbosity required
+ *   visualization <-- level of visualization required
+ *   preprocessing <-- is joining part of the preprocessing stage ?
+ *
+ * returns:
+ *   a pointer to a cs_join_param_t structure
+ *---------------------------------------------------------------------------*/
+
+static cs_join_stats_t
+_join_stats_init(void)
+{
+  cs_join_stats_t  stats;
+
+  memset(&stats, 0, sizeof(cs_join_stats_t));
+
+  CS_TIMER_COUNTER_INIT(stats.t_box_build);
+  CS_TIMER_COUNTER_INIT(stats.t_box_query);
+  CS_TIMER_COUNTER_INIT(stats.t_inter_sort);
+
+  CS_TIMER_COUNTER_INIT(stats.t_l_join_mesh);
+  CS_TIMER_COUNTER_INIT(stats.t_edge_inter);
+  CS_TIMER_COUNTER_INIT(stats.t_new_vtx);
+  CS_TIMER_COUNTER_INIT(stats.t_merge_vtx);
+  CS_TIMER_COUNTER_INIT(stats.t_u_merge_vtx);
+  CS_TIMER_COUNTER_INIT(stats.t_split_faces);
+
+  CS_TIMER_COUNTER_INIT(stats.t_total);
+
+  return stats;
+}
+
 /*----------------------------------------------------------------------------
  * Initialize a structure for the synchronization of single
  * elements
@@ -381,8 +423,8 @@ _extract_contig_faces(cs_lnum_t          n_vertices,
     counter[i] = 0;
 
   for (i = 0; i < n_faces; i++) {
-    for (j = f2v_idx[i] - 1; j < f2v_idx[i+1] - 1; j++) {
-      vtx_id = f2v_lst[j] - 1;
+    for (j = f2v_idx[i]; j < f2v_idx[i+1]; j++) {
+      vtx_id = f2v_lst[j];
       counter[vtx_id] += 1;
     }
   } /* End of loop on faces */
@@ -404,9 +446,9 @@ _extract_contig_faces(cs_lnum_t          n_vertices,
 
   for (i = 0; i < n_faces; i++) {
 
-    for (j = f2v_idx[i] - 1; j < f2v_idx[i+1] - 1; j++) {
+    for (j = f2v_idx[i]; j < f2v_idx[i+1]; j++) {
 
-      vtx_id = f2v_lst[j] - 1;
+      vtx_id = f2v_lst[j];
       shift = v2f_idx[vtx_id] + counter[vtx_id];
       v2f_lst[shift] = i+1;
       counter[vtx_id] += 1;
@@ -1031,7 +1073,7 @@ _add_s_edge(cs_lnum_t        vertex_tag[],
 
   if (vertex_tag[v1_id] > 0 && vertex_tag[v2_id] > 0) {
 
-    _Bool  is_found = false;
+    bool  is_found = false;
 
     cs_lnum_t  tmp_size = *p_tmp_size;
     cs_lnum_t  max_size = *p_max_size;
@@ -1251,7 +1293,7 @@ _add_single_edges(cs_interface_set_t   *ifs,
                   cs_lnum_t             b_f2v_lst[],
                   cs_lnum_t             i_f2v_idx[],
                   cs_lnum_t             i_f2v_lst[],
-                  cs_lnum_t             i_face_cells[],
+                  cs_lnum_2_t           i_face_cells[],
                   cs_join_sync_t       *s_edges,
                   int                   verbosity)
 {
@@ -1281,13 +1323,13 @@ _add_single_edges(cs_interface_set_t   *ifs,
   for (i = 0; i < selection->n_b_adj_faces; i++) {
 
     fid = selection->b_adj_faces[i] - 1;
-    s = b_f2v_idx[fid] - 1;
-    e = b_f2v_idx[fid+1] - 1;
+    s = b_f2v_idx[fid];
+    e = b_f2v_idx[fid+1];
 
     for (j = s; j < e - 1; j++)
       _add_s_edge(vertex_tag,
-                  b_f2v_lst[j]-1,
-                  b_f2v_lst[j+1]-1,
+                  b_f2v_lst[j],
+                  b_f2v_lst[j+1],
                   sel_v2v_idx,
                   sel_v2v_lst,
                   &tmp_size,
@@ -1295,8 +1337,8 @@ _add_single_edges(cs_interface_set_t   *ifs,
                   &tmp_edges);
 
     _add_s_edge(vertex_tag,
-                b_f2v_lst[e-1]-1,
-                b_f2v_lst[s]-1,
+                b_f2v_lst[e-1],
+                b_f2v_lst[s],
                 sel_v2v_idx,
                 sel_v2v_lst,
                 &tmp_size,
@@ -1309,15 +1351,15 @@ _add_single_edges(cs_interface_set_t   *ifs,
 
     fid = selection->i_adj_faces[i] - 1;
 
-    if (i_face_cells[2*fid] == 0 || i_face_cells[2*fid+1] == 0) {
+    if (i_face_cells[fid][0] == -1 || i_face_cells[fid][1] == -1) {
 
-      s = i_f2v_idx[fid] - 1;
-      e = i_f2v_idx[fid+1] - 1;
+      s = i_f2v_idx[fid];
+      e = i_f2v_idx[fid+1];
 
       for (j = s; j < e - 1; j++)
         _add_s_edge(vertex_tag,
-                    i_f2v_lst[j]-1,
-                    i_f2v_lst[j+1]-1,
+                    i_f2v_lst[j],
+                    i_f2v_lst[j+1],
                     sel_v2v_idx,
                     sel_v2v_lst,
                     &tmp_size,
@@ -1325,8 +1367,8 @@ _add_single_edges(cs_interface_set_t   *ifs,
                     &tmp_edges);
 
       _add_s_edge(vertex_tag,
-                  i_f2v_lst[e-1]-1,
-                  i_f2v_lst[s]-1,
+                  i_f2v_lst[e-1],
+                  i_f2v_lst[s],
                   sel_v2v_idx,
                   sel_v2v_lst,
                   &tmp_size,
@@ -1915,7 +1957,7 @@ _get_missing_edges(cs_lnum_t            b_f2v_idx[],
                    cs_lnum_t            n_vertices,
                    cs_lnum_t            vtx_tag[],
                    cs_interface_set_t  *ifs,
-                   cs_lnum_t            i_face_cells[],
+                   cs_lnum_2_t          i_face_cells[],
                    cs_join_select_t    *selection,
                    int                  verbosity)
 {
@@ -2027,6 +2069,8 @@ _get_missing_edges(cs_lnum_t            b_f2v_idx[],
 }
 #endif /* defined(HAVE_MPI) */
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *===========================================================================*/
@@ -2095,6 +2139,8 @@ cs_join_create(int                      join_number,
                                    visualization,
                                    preprocessing);
 
+  join->stats = _join_stats_init();
+
   join->log_name = NULL;
 
   /* Copy the selection criteria for future use */
@@ -2161,6 +2207,7 @@ cs_join_destroy(cs_join_t  **join)
  *
  * parameters:
  *   selection_criteria <-- pointer to a cs_mesh_select_t structure
+ *   perio_type         <-- periodicity type (FVM_PERIODICITY_NULL if none)
  *   verbosity          <-- level of verbosity required
  *
  * returns:
@@ -2168,8 +2215,9 @@ cs_join_destroy(cs_join_t  **join)
  *---------------------------------------------------------------------------*/
 
 cs_join_select_t *
-cs_join_select_create(const char  *selection_criteria,
-                      int          verbosity)
+cs_join_select_create(const char              *selection_criteria,
+                      fvm_periodicity_type_t   perio_type,
+                      int                      verbosity)
 {
   cs_lnum_t  i;
 
@@ -2231,9 +2279,22 @@ cs_join_select_create(const char  *selection_criteria,
 
   BFT_MALLOC(selection->faces, mesh->n_b_faces, cs_lnum_t);
 
-  cs_selector_get_b_face_list(selection_criteria,
-                              &(selection->n_faces),
-                              selection->faces);
+  cs_selector_get_b_face_num_list(selection_criteria,
+                                  &(selection->n_faces),
+                                  selection->faces);
+
+  /* In case of periodicity, ensure no isolated faces are
+     selected */
+
+  if (perio_type != FVM_PERIODICITY_NULL) {
+    cs_lnum_t j = 0;
+    for (i = 0; i < selection->n_faces; i++) {
+      cs_lnum_t f_id = selection->faces[i]-1;
+      if (mesh->b_face_cells[f_id] > -1)
+        selection->faces[j++] = f_id+1;
+    }
+    selection->n_faces = j;
+  }
 
   BFT_MALLOC(order, selection->n_faces, cs_lnum_t);
   BFT_MALLOC(ordered_faces, selection->n_faces, cs_lnum_t);
@@ -2487,8 +2548,8 @@ cs_join_select_create(const char  *selection_criteria,
  *---------------------------------------------------------------------------*/
 
 void
-join_select_destroy(cs_join_param_t     param,
-                    cs_join_select_t  **join_select)
+cs_join_select_destroy(cs_join_param_t     param,
+                       cs_join_select_t  **join_select)
 {
   if (*join_select != NULL) {
 
@@ -2556,8 +2617,8 @@ cs_join_extract_vertices(cs_lnum_t         n_select_faces,
 
       face_id = select_faces[i] - 1;
 
-      for (j = f2v_idx[face_id] - 1; j < f2v_idx[face_id+1] - 1; j++)
-        counter[f2v_lst[j]-1] = 1;
+      for (j = f2v_idx[face_id]; j < f2v_idx[face_id+1]; j++)
+        counter[f2v_lst[j]] = 1;
 
     }
 
@@ -2653,13 +2714,13 @@ cs_join_build_edges_idx(cs_lnum_t        n_faces,
   for (i = 0; i < n_faces; i++) {
 
     fid = faces[i] - 1;
-    s = f2v_idx[fid] - 1;
-    e = f2v_idx[fid+1] - 1;
+    s = f2v_idx[fid];
+    e = f2v_idx[fid+1];
 
     for (j = s; j < e - 1; j++) { /* scan edges */
 
-      v1 = f2v_lst[j];
-      v2 = f2v_lst[j+1];
+      v1 = f2v_lst[j] + 1;
+      v2 = f2v_lst[j+1] + 1;
 
       if (v1 < v2)
         v2v_idx[v1] += 1;
@@ -2674,8 +2735,8 @@ cs_join_build_edges_idx(cs_lnum_t        n_faces,
 
     /* Last edge */
 
-    v1 = f2v_lst[e-1];
-    v2 = f2v_lst[s];
+    v1 = f2v_lst[e-1] + 1;
+    v2 = f2v_lst[s] + 1;
 
     if (v1 < v2)
       v2v_idx[v1] += 1;
@@ -2718,13 +2779,13 @@ cs_join_build_edges_lst(cs_lnum_t        n_faces,
   for (i = 0; i < n_faces; i++) {
 
     fid = faces[i] - 1;
-    s = f2v_idx[fid] - 1;
-    e = f2v_idx[fid+1] - 1;
+    s = f2v_idx[fid];
+    e = f2v_idx[fid+1];
 
     for (j = s; j < e - 1; j++) { /* Scan edges */
 
-      v1_id = f2v_lst[j] - 1;
-      v2_id = f2v_lst[j+1] - 1;
+      v1_id = f2v_lst[j];
+      v2_id = f2v_lst[j+1];
 
       if (v1_id < v2_id) {
         shift = v2v_idx[v1_id] + count[v1_id];
@@ -2741,8 +2802,8 @@ cs_join_build_edges_lst(cs_lnum_t        n_faces,
 
     /* Last edge */
 
-    v1_id = f2v_lst[e-1] - 1;
-    v2_id = f2v_lst[s] - 1;
+    v1_id = f2v_lst[e-1];
+    v2_id = f2v_lst[s];
 
     if (v1_id < v2_id) {
       shift = v2v_idx[v1_id] + count[v1_id];
