@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -61,6 +61,19 @@
 
 BEGIN_C_DECLS
 
+/*----------------------------------------------------------------------------*/
+/*! \file cs_mesh_bad_cells.c
+ *
+ * \brief Detect bad cells within meshes.
+ *
+ * Please refer to the
+ * <a href="../../theory.pdf#badcells"><b>flagging of bad cells</b></a>
+ * section of the theory guide for more informations.
+ */
+/*----------------------------------------------------------------------------*/
+
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -90,18 +103,13 @@ BEGIN_C_DECLS
  * Static global variables
  *============================================================================*/
 
-static const int _type_flag_mask[] = {CS_BAD_CELL_ORTHO_NORM,
-                                      CS_BAD_CELL_OFFSET,
-                                      CS_BAD_CELL_LSQ_GRAD,
-                                      CS_BAD_CELL_RATIO,
-                                      CS_BAD_CELL_GUILT,
-                                      CS_BAD_CELL_USER};
-
 /* compute and visualize flags (-1 initially, mask afterwards;
    first value: at initialization: second value: at each time step) */
 
 static int  _type_flag_compute[] = {-1, 0};
 static int  _type_flag_visualize[] = {0, 0};
+static int  _call_type_compute = 0;
+static int  _call_type_visualize = 0;
 
 /*============================================================================
  * Private function definitions
@@ -140,8 +148,8 @@ _compute_ortho_norm(const cs_mesh_t             *mesh,
 
   for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
 
-    cell1 = mesh->i_face_cells[2 * face_id] - 1;
-    cell2 = mesh->i_face_cells[2 * face_id + 1] - 1;
+    cell1 = mesh->i_face_cells[face_id][0];
+    cell2 = mesh->i_face_cells[face_id][1];
 
     /* Get information on mesh quantities */
 
@@ -166,8 +174,8 @@ _compute_ortho_norm(const cs_mesh_t             *mesh,
     i_face_ortho = cos_alpha;
 
     if (i_face_ortho < 0.1) {
-      bad_cell_flag[cell1] = bad_cell_flag[cell1] | _type_flag_mask[0];
-      bad_cell_flag[cell2] = bad_cell_flag[cell2] | _type_flag_mask[0];
+      bad_cell_flag[cell1] |= CS_BAD_CELL_ORTHO_NORM;
+      bad_cell_flag[cell2] |= CS_BAD_CELL_ORTHO_NORM;
     }
   }
 
@@ -176,7 +184,7 @@ _compute_ortho_norm(const cs_mesh_t             *mesh,
 
   for (face_id = 0; face_id < mesh->n_b_faces; face_id++) {
 
-    cell1 = mesh->b_face_cells[face_id] - 1;
+    cell1 = mesh->b_face_cells[face_id];
 
     /* Get information on mesh quantities */
 
@@ -202,7 +210,7 @@ _compute_ortho_norm(const cs_mesh_t             *mesh,
     b_face_ortho = cos_alpha;
 
     if (b_face_ortho < 0.1)
-      bad_cell_flag[cell1] = bad_cell_flag[cell1] | _type_flag_mask[0];
+      bad_cell_flag[cell1] |= CS_BAD_CELL_ORTHO_NORM;
   }
 
   if (mesh->halo != NULL)
@@ -239,23 +247,23 @@ _compute_offsetting(const cs_mesh_t             *mesh,
 
   for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
 
-    cell1 = mesh->i_face_cells[2 * face_id] - 1;
-    cell2 = mesh->i_face_cells[2 * face_id + 1] - 1;
+    cell1 = mesh->i_face_cells[face_id][0];
+    cell2 = mesh->i_face_cells[face_id][1];
 
     /* Compute center offsetting coefficient,
        in a manner consistent with iterative gradient reconstruction */
 
     v_of = &(mesh_quantities->dofij[face_id*3]);
     v_n = &(mesh_quantities->i_face_normal[face_id*3]);
-    of_n = fabs(_DOT_PRODUCT_3D(v_of, v_n));
+    of_n = _MODULE_3D(v_of) * _MODULE_3D(v_n);
 
     off_1 = 1 - pow(of_n / mesh_quantities->cell_vol[cell1], 1/3.);
     off_2 = 1 - pow(of_n / mesh_quantities->cell_vol[cell2], 1/3.);
 
     if (off_1 < 0.1)
-      bad_cell_flag[cell1] = bad_cell_flag[cell1] | _type_flag_mask[1];
+      bad_cell_flag[cell1] |= CS_BAD_CELL_OFFSET;
     if (off_2 < 0.1)
-      bad_cell_flag[cell2] = bad_cell_flag[cell2] | _type_flag_mask[1];
+      bad_cell_flag[cell2] |= CS_BAD_CELL_OFFSET;
 
   }
 
@@ -312,8 +320,8 @@ _compute_least_squares(const cs_mesh_t             *mesh,
 
   for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
 
-    cell1 = mesh->i_face_cells[2 * face_id] - 1;
-    cell2 = mesh->i_face_cells[2 * face_id + 1] - 1;
+    cell1 = mesh->i_face_cells[face_id][0];
+    cell2 = mesh->i_face_cells[face_id][1];
 
     /* Center of gravity for each cell */
 
@@ -348,7 +356,7 @@ _compute_least_squares(const cs_mesh_t             *mesh,
   /*------------------------*/
 
   for (face_id = 0; face_id < mesh->n_b_faces; face_id++) {
-    cell1 = mesh->b_face_cells[face_id] - 1;
+    cell1 = mesh->b_face_cells[face_id];
 
     surfn = _MODULE_3D(b_face_normal[face_id]);
 
@@ -443,7 +451,7 @@ _compute_least_squares(const cs_mesh_t             *mesh,
     lsq = min_diag / max_diag;
 
     if (lsq < 0.1)
-      bad_cell_flag[cell_id] = _type_flag_mask[2];
+      bad_cell_flag[cell_id] |= CS_BAD_CELL_LSQ_GRAD;
   }
 
   BFT_FREE(w1);
@@ -483,15 +491,15 @@ _compute_volume_ratio(const cs_mesh_t             *mesh,
 
   for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
 
-    cell1 = mesh->i_face_cells[2 * face_id] - 1;
-    cell2 = mesh->i_face_cells[2 * face_id + 1] - 1;
+    cell1 = mesh->i_face_cells[face_id][0];
+    cell2 = mesh->i_face_cells[face_id][1];
 
     vol_ratio = fmin(volume[cell1] / volume[cell2],
                      volume[cell2] / volume[cell1]);
 
     if (vol_ratio < 0.1*0.1) {
-      bad_cell_flag[cell1] = _type_flag_mask[3];
-      bad_cell_flag[cell2] = _type_flag_mask[3];
+      bad_cell_flag[cell1] |= CS_BAD_CELL_RATIO;
+      bad_cell_flag[cell2] |= CS_BAD_CELL_RATIO;
     }
   }
 
@@ -530,16 +538,18 @@ _bad_cells_post(const cs_mesh_t             *mesh,
                                 CS_BAD_CELL_LSQ_GRAD,
                                 CS_BAD_CELL_RATIO,
                                 CS_BAD_CELL_GUILT,
-                                CS_BAD_CELL_USER};
+                                CS_BAD_CELL_USER,
+                                CS_BAD_CELL_TO_REGULARIZE};
 
   const char *criterion_name[] = {N_("Bad Cell Ortho Norm"),
                                   N_("Bad Cell Offset"),
                                   N_("Bad Cell LSQ Gradient"),
                                   N_("Bad Cell Volume Ratio"),
                                   N_("Bad Cell Association"),
-                                  N_("Bad Cell by User")};
+                                  N_("Bad Cell by User"),
+                                  N_("Bad Cell to regularize")};
 
-  const int n_criteria = 6;
+  const int n_criteria = 7;
 
   if (_type_flag_visualize[call_type] == 0)
     return;
@@ -568,7 +578,8 @@ _bad_cells_post(const cs_mesh_t             *mesh,
       cs_parall_counter_max(&crit_flag, 1);
 
       if (crit_flag > 0)
-        cs_post_write_var(-1,
+        cs_post_write_var(CS_POST_MESH_VOLUME,
+                          CS_POST_WRITER_ALL_ASSOCIATED,
                           _(criterion_name[i]),
                           1,
                           false,
@@ -611,6 +622,107 @@ _bad_cells_post_function(void                  *mesh,
                   ts);
 }
 
+/*----------------------------------------------------------------------------
+ * Tag cell for regularisation
+ * if no bad cells, CS_BAD_CELLS_REGULARISATION is
+ * switch off.
+ *
+ * parameters:
+ *   mesh            <-- Void pointer to associated mesh structure
+ *   mq              <-- pointer to associated mesh quantities structure
+ *   bad_cell_flag   --> array of bad cell flags for various uses
+ *----------------------------------------------------------------------------*/
+
+static void
+_to_regularize(const cs_mesh_t             *mesh,
+               const cs_mesh_quantities_t  *mq,
+               unsigned                     bad_cell_flag[])
+{
+  cs_lnum_t n_cells_ext = mesh->n_cells_with_ghosts;
+  cs_lnum_t n_cells = mesh->n_cells;
+  cs_lnum_t n_i_faces = mesh->n_i_faces;
+  cs_lnum_t n_b_faces = mesh->n_b_faces;
+  const cs_lnum_2_t *i_face_cells = (const cs_lnum_2_t *)mesh->i_face_cells;
+  const int *b_face_cells = mesh->b_face_cells;
+
+  const cs_real_3_t *cdgfac = (const cs_real_3_t *) mq->i_face_cog;
+  const cs_real_3_t *cdgfbo = (const cs_real_3_t *) mq->b_face_cog;
+  const cs_real_3_t *surfac = (const cs_real_3_t *) mq->i_face_normal;
+  const cs_real_3_t *surfbo = (const cs_real_3_t *) mq->b_face_normal;
+
+  static cs_gnum_t nb_bad_cells = 0;
+
+  cs_real_3_t *vol;
+  BFT_MALLOC(vol, n_cells_ext, cs_real_3_t);
+
+  //FIXME tensor ?
+  for (cs_lnum_t cell_id = 0; cell_id < n_cells_ext; cell_id++) {
+    vol[cell_id][0] = 0.;
+    vol[cell_id][1] = 0.;
+    vol[cell_id][2] = 0.;
+  }
+  for (cs_lnum_t face_id = 0; face_id < n_i_faces; face_id++) {
+    cs_lnum_t cell_id1 = i_face_cells[face_id][0];
+    cs_lnum_t cell_id2 = i_face_cells[face_id][1];
+    vol[cell_id1][0] += cdgfac[face_id][0] * surfac[face_id][0];
+    vol[cell_id1][1] += cdgfac[face_id][1] * surfac[face_id][1];
+    vol[cell_id1][2] += cdgfac[face_id][2] * surfac[face_id][2];
+    vol[cell_id2][0] -= cdgfac[face_id][0] * surfac[face_id][0];
+    vol[cell_id2][1] -= cdgfac[face_id][1] * surfac[face_id][1];
+    vol[cell_id2][2] -= cdgfac[face_id][2] * surfac[face_id][2];
+  }
+
+  for (cs_lnum_t face_id = 0; face_id < n_b_faces; face_id++) {
+    cs_lnum_t cell_id = b_face_cells[face_id];
+    vol[cell_id][0] += cdgfbo[face_id][0] * surfbo[face_id][0];
+    vol[cell_id][1] += cdgfbo[face_id][1] * surfbo[face_id][1];
+    vol[cell_id][2] += cdgfbo[face_id][2] * surfbo[face_id][2];
+  }
+
+  for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
+
+    double determinant = mq->corr_grad_lin_det[cell_id];
+
+    int probleme = 0;
+    if (determinant <= 0.) {
+      probleme = 1;
+    }
+    else {
+      determinant = CS_MAX(determinant, 1.e-10);
+      determinant = CS_MAX(determinant, 1./determinant);
+      if (determinant > 2)
+        probleme = 1;
+    }
+
+    // FIXME not invariant by rotation
+    if (probleme > 0 || (bad_cell_flag[cell_id] & CS_BAD_CELL_RATIO)
+        || vol[cell_id][0] < 0. || vol[cell_id][1] < 0. || vol[cell_id][2] < 0.
+        || (bad_cell_flag[cell_id] & CS_BAD_CELL_ORTHO_NORM)) {
+      bad_cell_flag[cell_id] |= CS_BAD_CELL_TO_REGULARIZE;
+      nb_bad_cells ++;
+    }
+
+  }
+
+  if (mesh->halo != NULL)
+    cs_halo_sync_untyped(mesh->halo, CS_HALO_STANDARD,
+                         sizeof(unsigned), bad_cell_flag);
+
+  cs_parall_counter(&nb_bad_cells, 1);
+
+  BFT_FREE(vol);
+
+  /* If no bad cells, no need of regularisation */
+  if (nb_bad_cells <= 0) {
+    bft_printf("No need of regularisation\n");
+    cs_glob_mesh_quantities_flag -= CS_BAD_CELLS_REGULARISATION;
+  }
+
+  return;
+}
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -648,7 +760,7 @@ cs_mesh_bad_cells_set_options(int  type_flag_mask,
     _type_flag_visualize[i] = 0;
   }
 
-  for (i = 0; i < (int)(sizeof(int))*8; i++) {
+  for (i = 0; i < 7; i++) {
     int mask = (1 << i);
     if (type_flag_mask == 0 || (type_flag_mask & mask)) {
       if (compute > 0) {
@@ -729,9 +841,17 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
   cs_lnum_t i;
   cs_gnum_t n_cells_tot, iwarning, ibad;
 
+  const int call_type_log = _call_type_compute;
+
+  /* If bad cell data has been destroyed and this function is
+     called, we have a call type 0, even if it was called before */
+
+  if (mesh_quantities->bad_cell_flag == NULL)
+    _call_type_compute = 0;
+
   /* Initialization or per time step ? */
 
-  static int call_type = 0;
+  const int call_type = _call_type_compute;
 
   /* Set defaults if not done yet */
 
@@ -743,8 +863,8 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
 
   /* Check input data */
 
-  assert(mesh_quantities->i_face_normal != NULL);
-  assert(mesh_quantities->i_face_cog != NULL);
+  assert(mesh_quantities->i_face_normal != NULL || mesh->n_i_faces == 0);
+  assert(mesh_quantities->i_face_cog != NULL || mesh->n_i_faces == 0);
   assert(mesh_quantities->cell_cen != NULL);
   assert(mesh_quantities->cell_vol != NULL);
 
@@ -760,7 +880,7 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
   for (i = 0; i < n_cells_wghosts; i++)
     bad_cell_flag[i] = 0;
 
-  /* Possible warning printed in the listing --> flag initialization */
+  /* Possible warning printed in the log --> flag initialization */
 
   iwarning = 0;
   n_cells_tot = mesh->n_g_cells;
@@ -771,15 +891,17 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
   /* Condition 1: Orthogonal Normal */
   /*--------------------------------*/
 
-  if (_type_flag_compute[call_type] & CS_BAD_CELL_ORTHO_NORM) {
-
+  if (_type_flag_compute[call_type] & CS_BAD_CELL_ORTHO_NORM)
     _compute_ortho_norm(mesh,
                         mesh_quantities,
                         bad_cell_flag);
 
+
+  if (_type_flag_compute[call_type_log] & CS_BAD_CELL_ORTHO_NORM) {
+
     ibad = 0;
     for (i = 0; i < n_cells; i++) {
-      if (bad_cell_flag[i] & _type_flag_mask[0]) {
+      if (bad_cell_flag[i] & CS_BAD_CELL_ORTHO_NORM) {
         ibad++;
         iwarning++;
       }
@@ -802,15 +924,17 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
   /*---------------------------------*/
 
   if (   _type_flag_compute[call_type] & CS_BAD_CELL_OFFSET
-      && cs_glob_mesh_quantities->min_vol >= 0.) {
-
+      && cs_glob_mesh_quantities->min_vol >= 0.)
     _compute_offsetting(mesh,
                         mesh_quantities,
                         bad_cell_flag);
 
+  if (   _type_flag_compute[call_type_log] & CS_BAD_CELL_OFFSET
+      && cs_glob_mesh_quantities->min_vol >= 0.) {
+
     ibad = 0;
     for (i = 0; i < n_cells; i++) {
-      if (bad_cell_flag[i] & _type_flag_mask[1]) {
+      if (bad_cell_flag[i] & CS_BAD_CELL_OFFSET) {
         ibad++;
         iwarning++;
       }
@@ -821,7 +945,7 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
       cs_parall_counter(&iwarning, 1);
     }
 
-    /* Display listing output */
+    /* Display log output */
     bft_printf(_("\n  Criterion 2: Offset:\n"));
     bft_printf(_("    Number of bad cells detected: %llu --> %3.0f %%\n"),
                (unsigned long long)ibad,
@@ -832,15 +956,16 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
   /* Condition 3: Least Squares Gradient */
   /*-------------------------------------*/
 
-  if (_type_flag_compute[call_type] & CS_BAD_CELL_LSQ_GRAD) {
-
+  if (_type_flag_compute[call_type] & CS_BAD_CELL_LSQ_GRAD)
     _compute_least_squares(mesh,
                            mesh_quantities,
                            bad_cell_flag);
 
+  if (_type_flag_compute[call_type_log] & CS_BAD_CELL_LSQ_GRAD) {
+
     ibad = 0;
     for (i = 0; i < n_cells; i++) {
-      if (bad_cell_flag[i] & _type_flag_mask[2]) {
+      if (bad_cell_flag[i] & CS_BAD_CELL_LSQ_GRAD) {
         ibad++;
         iwarning++;
       }
@@ -862,15 +987,16 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
   /* Condition 4: Volume Ratio */
   /*---------------------------*/
 
-  if (_type_flag_compute[call_type] & CS_BAD_CELL_RATIO) {
-
+  if (_type_flag_compute[call_type] & CS_BAD_CELL_RATIO)
     _compute_volume_ratio(mesh,
                           mesh_quantities,
                           bad_cell_flag);
 
+  if (_type_flag_compute[call_type_log] & CS_BAD_CELL_RATIO) {
+
     ibad = 0;
     for (i = 0; i < n_cells; i++) {
-      if (bad_cell_flag[i] & _type_flag_mask[3]) {
+      if (bad_cell_flag[i] & CS_BAD_CELL_RATIO) {
         ibad++;
         iwarning++;
       }
@@ -881,7 +1007,7 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
       cs_parall_counter(&iwarning, 1);
     }
 
-    /* Display listing output */
+    /* Display log output */
     bft_printf(_("\n  Criterion 4: Cells Volume Ratio:\n"));
     bft_printf(_("    Number of bad cells detected: %llu --> %3.0f %%\n"),
                (unsigned long long)ibad,
@@ -889,8 +1015,8 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
 
   }
 
-  /* Guilt by association */
-  /*----------------------*/
+  /* 5: Guilt by association */
+  /*-------------------------*/
 
   if (_type_flag_compute[call_type] & CS_BAD_CELL_GUILT) {
 
@@ -906,8 +1032,8 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
     /* Loop on interior faces */
     for (face_id = 0; face_id < n_i_faces; face_id++) {
 
-      cell1 = mesh->i_face_cells[2 * face_id] - 1;
-      cell2 = mesh->i_face_cells[2 * face_id + 1] - 1;
+      cell1 = mesh->i_face_cells[face_id][0];
+      cell2 = mesh->i_face_cells[face_id][1];
 
       if (bad_cell_flag[cell2] != 0)
         bad_guilt_cells[cell1]++;
@@ -918,23 +1044,60 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
       if (bad_guilt_cells[i] >= 5 && bad_cell_flag[i] == 0) {
         ibad++;
         iwarning++;
-        bad_cell_flag[i] = _type_flag_mask[4];
+        bad_cell_flag[i] = CS_BAD_CELL_GUILT;
       }
     }
 
-    if (cs_glob_rank_id >= 0) {
-      cs_parall_counter(&ibad, 1);
-      cs_parall_counter(&iwarning, 1);
-    }
-
-    /* Display listing output */
-    bft_printf(_("\n  Criterion 5: Guilt by Association:\n"));
-    bft_printf(_("    Number of bad cells detected: %llu --> %3.0f %%\n"),
-               (unsigned long long)ibad,
-               (double)ibad / (double)n_cells_tot * 100.0);
-
     BFT_FREE(bad_guilt_cells);
 
+    if (_type_flag_compute[call_type_log] & CS_BAD_CELL_GUILT) {
+
+      if (cs_glob_rank_id >= 0) {
+        cs_parall_counter(&ibad, 1);
+        cs_parall_counter(&iwarning, 1);
+      }
+
+      /* Display log output */
+      bft_printf(_("\n  Criterion 5: Guilt by Association:\n"));
+      bft_printf(_("    Number of bad cells detected: %llu --> %3.0f %%\n"),
+                 (unsigned long long)ibad,
+                 (double)ibad / (double)n_cells_tot * 100.0);
+
+    }
+
+  }
+
+  /* Tag some cells to be regularized */
+  /*----------------------------------*/
+
+  if (cs_glob_mesh_quantities_flag & CS_BAD_CELLS_REGULARISATION) {
+    if (_type_flag_compute[call_type] & CS_BAD_CELL_TO_REGULARIZE)
+      _to_regularize(mesh,
+                     mesh_quantities,
+                     bad_cell_flag);
+
+    if (_type_flag_compute[call_type_log] & CS_BAD_CELL_TO_REGULARIZE) {
+
+      ibad = 0;
+      for (i = 0; i < n_cells; i++) {
+        if (bad_cell_flag[i] & CS_BAD_CELL_TO_REGULARIZE) {
+          ibad++;
+          iwarning++;
+        }
+      }
+
+      if (cs_glob_rank_id >= 0) {
+        cs_parall_counter(&ibad, 1);
+        cs_parall_counter(&iwarning, 1);
+      }
+
+      /* Display log output */
+      bft_printf(_("\n  Regularisation:\n"));
+      bft_printf(_("    Number of bad cells detected: %llu --> %3.0f %%\n"),
+                 (unsigned long long)ibad,
+                 (double)ibad / (double)n_cells_tot * 100.0);
+
+    }
   }
 
   /* Warning printed in the log file */
@@ -952,7 +1115,7 @@ cs_mesh_bad_cells_detect(const cs_mesh_t       *mesh,
 
   /* After first call, we assume others are done at each time step */
 
-  call_type = 1;
+  _call_type_compute = 1;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -971,7 +1134,7 @@ cs_mesh_bad_cells_postprocess(const cs_mesh_t             *mesh,
 {
   /* Initialization or per time step ? */
 
-  static int call_type = 0;
+  const int call_type = _call_type_visualize;
 
   /* Set defaults if not done yet */
 
@@ -988,7 +1151,7 @@ cs_mesh_bad_cells_postprocess(const cs_mesh_t             *mesh,
                   0,
                   NULL);
 
-  call_type = 1; /* Prevent future calls from doing anything */
+  _call_type_visualize = 1; /* Prevent future calls from doing anything */
 }
 
 /*----------------------------------------------------------------------------*/
@@ -1003,4 +1166,3 @@ cs_mesh_bad_cells_postprocess(const cs_mesh_t             *mesh,
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
-
