@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -44,6 +44,7 @@
 
 #include "cs_base.h"
 #include "cs_mesh.h"
+#include "cs_sort.h"
 
 #include "fvm_defs.h"
 #include "fvm_nodal.h"
@@ -60,6 +61,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -281,6 +284,8 @@ _add_faces_to_nodal(const cs_mesh_t  *mesh,
     BFT_FREE(num_glob_fac);
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -337,7 +342,7 @@ cs_mesh_connect_get_cell_faces(const cs_mesh_t         *mesh,
      as to ignore ghost cells */
 
   for (face_id = 0; face_id < mesh->n_b_faces; face_id++) {
-    cell_id = mesh->b_face_cells[face_id] - 1;
+    cell_id = mesh->b_face_cells[face_id];
     if (extr_cell_id != NULL)
       cell_id = extr_cell_id[cell_id];
     if (cell_id > -1)
@@ -345,8 +350,8 @@ cs_mesh_connect_get_cell_faces(const cs_mesh_t         *mesh,
   }
 
   for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
-    c_id1 = mesh->i_face_cells[face_id*2    ] - 1;
-    c_id2 = mesh->i_face_cells[face_id*2 + 1] - 1;
+    c_id1 = mesh->i_face_cells[face_id][0];
+    c_id2 = mesh->i_face_cells[face_id][1];
     if (extr_cell_id != NULL) {
       if (c_id1 < mesh->n_cells)
         c_id1 = extr_cell_id[c_id1];
@@ -378,7 +383,7 @@ cs_mesh_connect_get_cell_faces(const cs_mesh_t         *mesh,
     cell_face_count[cell_id] = 0;
 
   for (face_id = 0; face_id < mesh->n_b_faces; face_id++) {
-    cell_id = mesh->b_face_cells[face_id] - 1;
+    cell_id = mesh->b_face_cells[face_id];
     if (extr_cell_id != NULL)
       cell_id = extr_cell_id[cell_id];
     if (cell_id > -1) {
@@ -389,8 +394,8 @@ cs_mesh_connect_get_cell_faces(const cs_mesh_t         *mesh,
   }
 
   for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
-    c_id1 = mesh->i_face_cells[face_id*2    ] - 1;
-    c_id2 = mesh->i_face_cells[face_id*2 + 1] - 1;
+    c_id1 = mesh->i_face_cells[face_id][0];
+    c_id2 = mesh->i_face_cells[face_id][1];
     if (extr_cell_id != NULL) {
       if (c_id1 < mesh->n_cells)
         c_id1 = extr_cell_id[c_id1];
@@ -528,8 +533,8 @@ cs_mesh_connect_cells_to_nodal(const cs_mesh_t  *mesh,
     if (include_families) {
 
       for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
-        cs_lnum_t c_id_0 = mesh->i_face_cells[face_id*2] - 1;
-        cs_lnum_t c_id_1 = mesh->i_face_cells[face_id*2 + 1] - 1;
+        cs_lnum_t c_id_0 = mesh->i_face_cells[face_id][0];
+        cs_lnum_t c_id_1 = mesh->i_face_cells[face_id][1];
         if (   (extr_cell_idx[c_id_0] == 1 || extr_cell_idx[c_id_1] == 1)
             && (mesh->i_face_family[face_id] != null_family)) {
           i_face_list[i_face_count++] = face_id + 1;
@@ -538,7 +543,7 @@ cs_mesh_connect_cells_to_nodal(const cs_mesh_t  *mesh,
       BFT_REALLOC(i_face_list, i_face_count, cs_lnum_t);
 
       for (face_id = 0; face_id < mesh->n_b_faces; face_id++) {
-        cs_lnum_t c_id = mesh->b_face_cells[face_id] - 1;
+        cs_lnum_t c_id = mesh->b_face_cells[face_id];
         if (   (extr_cell_idx[c_id] == 1)
             && (mesh->b_face_family[face_id] != null_family)) {
           b_face_list[b_face_count++] = face_id + 1;
@@ -569,8 +574,8 @@ cs_mesh_connect_cells_to_nodal(const cs_mesh_t  *mesh,
     if (include_families && extr_cell_count > 0) {
 
       for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
-        cs_lnum_t c_id_0 = mesh->i_face_cells[face_id*2] - 1;
-        cs_lnum_t c_id_1 = mesh->i_face_cells[face_id*2 + 1] - 1;
+        cs_lnum_t c_id_0 = mesh->i_face_cells[face_id][0];
+        cs_lnum_t c_id_1 = mesh->i_face_cells[face_id][1];
         if (   (c_id_0 < extr_cell_count || c_id_1 < extr_cell_count)
             && (mesh->i_face_family[face_id] != null_family)) {
           i_face_list[i_face_count++] = face_id + 1;
@@ -579,7 +584,7 @@ cs_mesh_connect_cells_to_nodal(const cs_mesh_t  *mesh,
       BFT_REALLOC(i_face_list, i_face_count, cs_lnum_t);
 
       for (face_id = 0; face_id < mesh->n_b_faces; face_id++) {
-        cs_lnum_t c_id = mesh->b_face_cells[face_id] - 1;
+        cs_lnum_t c_id = mesh->b_face_cells[face_id];
         if (   (c_id < extr_cell_count)
             && (mesh->b_face_family[face_id] != null_family)) {
           b_face_list[b_face_count++] = face_id + 1;
@@ -614,6 +619,8 @@ cs_mesh_connect_cells_to_nodal(const cs_mesh_t  *mesh,
 
   extr_mesh = fvm_nodal_create(name, 3);
 
+  fvm_nodal_set_parent(extr_mesh, mesh);
+
   assert(sizeof(int) == sizeof(cs_lnum_t)); /* For families */
 
   if (include_families)
@@ -718,6 +725,8 @@ cs_mesh_connect_faces_to_nodal(const cs_mesh_t  *mesh,
 
   extr_mesh = fvm_nodal_create(name, 3);
 
+  fvm_nodal_set_parent(extr_mesh, mesh);
+
   _add_faces_to_nodal(mesh,
                       extr_mesh,
                       include_families,
@@ -744,6 +753,150 @@ cs_mesh_connect_faces_to_nodal(const cs_mesh_t  *mesh,
   return extr_mesh;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Build a vertex to cell connectivity for marked vertices only.
+ *
+ * It is the caller's responsibility to free the v2c_idx and v2c arrays,
+ * which are allocated by this function.
+ *
+ * \param[in]    mesh      pointer to mesh structure
+ * \param[in]    v_flag    vertex selection flag (0: not selected, 1: selected)
+ * \param[out]   v2c_idx   vertex to cells index (size: mesh->n_vertices +1)
+ * \param[out]   v2c       vertex to cells
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_mesh_connect_vertices_to_cells(cs_mesh_t    *mesh,
+                                  const char    v_flag[],
+                                  cs_lnum_t   **v2c_idx,
+                                  cs_lnum_t   **v2c)
+{
+  cs_lnum_t n_vertices = mesh->n_vertices;
+
+  /* Mark vertices which may be split (vertices lying on new boundary faces) */
+
+  cs_lnum_t  *_v2c_idx;
+  BFT_MALLOC(_v2c_idx, n_vertices+1, cs_lnum_t);
+
+  _v2c_idx[0] = 0;
+  for (cs_lnum_t i = 0; i < n_vertices; i++)
+    _v2c_idx[i+1] = 0;
+
+  /* Now build vertex -> cells index
+     (which will contain duplicate entries at first) */
+
+  for (cs_lnum_t f_id = 0; f_id < mesh->n_i_faces; f_id++) {
+    cs_lnum_t s_id = mesh->i_face_vtx_idx[f_id];
+    cs_lnum_t e_id = mesh->i_face_vtx_idx[f_id+1];
+    for (cs_lnum_t i = s_id; i < e_id; i++) {
+      cs_lnum_t vtx_id = mesh->i_face_vtx_lst[i];
+      if (v_flag[vtx_id] != 0) {
+        if (mesh->i_face_cells[f_id][0] > -1)
+          _v2c_idx[vtx_id + 1] += 1;
+        if (mesh->i_face_cells[f_id][1] > -1)
+          _v2c_idx[vtx_id + 1] += 1;
+      }
+    }
+  }
+
+  for (cs_lnum_t f_id = 0; f_id < mesh->n_b_faces; f_id++) {
+    cs_lnum_t s_id = mesh->b_face_vtx_idx[f_id];
+    cs_lnum_t e_id = mesh->b_face_vtx_idx[f_id+1];
+    for (cs_lnum_t i = s_id; i < e_id; i++) {
+      cs_lnum_t vtx_id = mesh->b_face_vtx_lst[i];
+      if (v_flag[vtx_id] != 0)
+        _v2c_idx[vtx_id + 1] += 1;
+    }
+  }
+
+  /* Transform count to index */
+
+  for (cs_lnum_t i = 0; i < n_vertices; i++)
+    _v2c_idx[i+1] += _v2c_idx[i];
+
+  /* Now define selected vertex->cell adjacency */
+
+  cs_lnum_t *_v2c;
+  BFT_MALLOC(_v2c, _v2c_idx[n_vertices], cs_lnum_t);
+
+  cs_lnum_t *v2c_count;
+  BFT_MALLOC(v2c_count, n_vertices, cs_lnum_t);
+  for (cs_lnum_t i = 0; i < n_vertices; i++)
+    v2c_count[i] = 0;
+
+  for (cs_lnum_t f_id = 0; f_id < mesh->n_i_faces; f_id++) {
+    cs_lnum_t s_id = mesh->i_face_vtx_idx[f_id];
+    cs_lnum_t e_id = mesh->i_face_vtx_idx[f_id+1];
+    for (cs_lnum_t i = s_id; i < e_id; i++) {
+      cs_lnum_t vtx_id = mesh->i_face_vtx_lst[i];
+      if (v_flag[vtx_id] != 0) {
+        cs_lnum_t c_id_0 = mesh->i_face_cells[f_id][0];
+        cs_lnum_t c_id_1 = mesh->i_face_cells[f_id][1];
+        cs_lnum_t j = _v2c_idx[vtx_id] + v2c_count[vtx_id];
+        if (c_id_0 > -1) {
+          _v2c[j++] = c_id_0;
+          v2c_count[vtx_id] += 1;
+        }
+        if (c_id_1 > -1) {
+          _v2c[j++] = c_id_1;
+          v2c_count[vtx_id] += 1;
+        }
+      }
+    }
+  }
+
+  for (cs_lnum_t f_id = 0; f_id < mesh->n_b_faces; f_id++) {
+    cs_lnum_t s_id = mesh->b_face_vtx_idx[f_id];
+    cs_lnum_t e_id = mesh->b_face_vtx_idx[f_id+1];
+    for (cs_lnum_t i = s_id; i < e_id; i++) {
+      cs_lnum_t vtx_id = mesh->b_face_vtx_lst[i];
+      if (v_flag[vtx_id] != 0) {
+        cs_lnum_t c_id_0 = mesh->b_face_cells[f_id];
+        cs_lnum_t j = _v2c_idx[vtx_id] + v2c_count[vtx_id];
+        _v2c[j] = c_id_0;
+        v2c_count[vtx_id] += 1;
+      }
+    }
+  }
+
+  BFT_FREE(v2c_count);
+
+  /* Order and compact adjacency array */
+
+  cs_sort_indexed(n_vertices, _v2c_idx, _v2c);
+
+  cs_lnum_t *tmp_v2c_idx = NULL;
+
+  BFT_MALLOC(tmp_v2c_idx, n_vertices+1, cs_lnum_t);
+  memcpy(tmp_v2c_idx, _v2c_idx, (n_vertices+1)*sizeof(cs_lnum_t));
+
+  cs_lnum_t k = 0;
+
+  for (cs_lnum_t i = 0; i < n_vertices; i++) {
+    cs_lnum_t js = tmp_v2c_idx[i];
+    cs_lnum_t je = tmp_v2c_idx[i+1];
+    cs_lnum_t v2c_prev = -1;
+    _v2c_idx[i] = k;
+    for (cs_lnum_t j = js; j < je; j++) {
+      if (v2c_prev != _v2c[j]) {
+        _v2c[k++] = _v2c[j];
+        v2c_prev = _v2c[j];
+      }
+    }
+  }
+  _v2c_idx[n_vertices] = k;
+
+  assert(_v2c_idx[n_vertices] <= tmp_v2c_idx[n_vertices]);
+
+  BFT_FREE(tmp_v2c_idx);
+  BFT_REALLOC(_v2c, _v2c_idx[n_vertices], cs_lnum_t);
+
+  *v2c_idx = _v2c_idx;
+  *v2c = _v2c;
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
