@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -65,6 +65,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -109,7 +111,7 @@ static void
 _face_type_g(cs_mesh_t                 *mesh,
              cs_lnum_t                  n_faces,
              const cs_interface_set_t  *face_ifs,
-             const cs_lnum_t            face_cell[],
+             const cs_lnum_2_t          face_cell[],
              const cs_lnum_t            face_vertices_idx[],
              char                       face_type[])
 {
@@ -121,11 +123,11 @@ _face_type_g(cs_mesh_t                 *mesh,
   /* Mark base interior faces */
 
   for (i = 0; i < n_faces; i++) {
-    if (face_cell[i*2] > 0 && face_cell[i*2+1] > 0)
+    if (face_cell[i][0] > -1 && face_cell[i][1] > -1)
       face_type[i] = '\0';
-    else if (face_cell[i*2] > 0)
+    else if (face_cell[i][0] > -1)
       face_type[i] = '\1';
-    else if (face_cell[i*2 + 1] > 0)
+    else if (face_cell[i][1] > -1)
       face_type[i] = '\2';
     else {
       face_type[i] = '\3';
@@ -191,7 +193,7 @@ _face_type_l(cs_mesh_t                  *mesh,
              cs_lnum_t                   n_faces,
              const cs_lnum_t             n_periodic_couples[],
              const cs_gnum_t      *const periodic_couples[],
-             const cs_lnum_t             face_cell[],
+             const cs_lnum_2_t           face_cell[],
              const cs_lnum_t             face_vertices_idx[],
              char                        face_type[])
 {
@@ -201,11 +203,11 @@ _face_type_l(cs_mesh_t                  *mesh,
   /* Mark base interior faces */
 
   for (i = 0; i < n_faces; i++) {
-    if (face_cell[i*2] > 0 && face_cell[i*2+1] > 0)
+    if (face_cell[i][0] > -1 && face_cell[i][1] > -1)
       face_type[i] = '\0';
-    else if (face_cell[i*2] > 0)
+    else if (face_cell[i][0] > -1)
       face_type[i] = '\1';
-    else if (face_cell[i*2 + 1] > 0)
+    else if (face_cell[i][1] > -1)
       face_type[i] = '\2';
     else
       face_type[i] = '\3';
@@ -270,7 +272,7 @@ _face_type_l(cs_mesh_t                  *mesh,
 static void
 _extract_face_cell(cs_mesh_t         *mesh,
                    cs_lnum_t          n_faces,
-                   const cs_lnum_t    face_cell[],
+                   const cs_lnum_2_t  face_cell[],
                    const char         face_type[])
 {
   cs_lnum_t i;
@@ -280,26 +282,26 @@ _extract_face_cell(cs_mesh_t         *mesh,
 
   /* Allocate arrays */
 
-  BFT_MALLOC(mesh->i_face_cells, mesh->n_i_faces * 2, cs_int_t);
-  BFT_MALLOC(mesh->b_face_cells, mesh->n_b_faces, cs_int_t);
+  BFT_MALLOC(mesh->i_face_cells, mesh->n_i_faces, cs_lnum_2_t);
+  BFT_MALLOC(mesh->b_face_cells, mesh->n_b_faces, cs_lnum_t);
 
   /* Now copy face -> cell connectivity */
 
   for (i = 0; i < n_faces; i++) {
 
     if (face_type[i] == '\0') {
-      mesh->i_face_cells[n_i_faces*2]     = face_cell[i*2];
-      mesh->i_face_cells[n_i_faces*2 + 1] = face_cell[i*2 + 1];
+      mesh->i_face_cells[n_i_faces][0] = face_cell[i][0];
+      mesh->i_face_cells[n_i_faces][1] = face_cell[i][1];
       n_i_faces++;
     }
 
     else if (face_type[i] == '\1') {
-      mesh->b_face_cells[n_b_faces] = face_cell[i*2];
+      mesh->b_face_cells[n_b_faces] = face_cell[i][0];
       n_b_faces++;
     }
 
     else if (face_type[i] == '\2') {
-      mesh->b_face_cells[n_b_faces] = face_cell[i*2 + 1];
+      mesh->b_face_cells[n_b_faces] = face_cell[i][1];
       n_b_faces++;
     }
 
@@ -344,10 +346,10 @@ _extract_face_vertices(cs_mesh_t         *mesh,
   BFT_MALLOC(mesh->i_face_vtx_idx, mesh->n_i_faces+1, cs_int_t);
   BFT_MALLOC(mesh->i_face_vtx_lst, mesh->i_face_vtx_connect_size, cs_int_t);
 
-  mesh->i_face_vtx_idx[0] = 1;
+  mesh->i_face_vtx_idx[0] = 0;
 
   BFT_MALLOC(mesh->b_face_vtx_idx, mesh->n_b_faces+1, cs_int_t);
-  mesh->b_face_vtx_idx[0] = 1;
+  mesh->b_face_vtx_idx[0] = 0;
 
   if (mesh->n_b_faces > 0)
     BFT_MALLOC(mesh->b_face_vtx_lst, mesh->b_face_vtx_connect_size, cs_int_t);
@@ -361,9 +363,9 @@ _extract_face_vertices(cs_mesh_t         *mesh,
 
     if (face_type[i] == '\0') {
       cs_lnum_t *_i_face_vtx =   mesh->i_face_vtx_lst
-                                + mesh->i_face_vtx_idx[n_i_faces] - 1;
+                               + mesh->i_face_vtx_idx[n_i_faces];
       for (j = 0; j < n_f_vertices; j++)
-        _i_face_vtx[j] = _face_vtx[j];
+        _i_face_vtx[j] = _face_vtx[j] - 1;
       mesh->i_face_vtx_idx[n_i_faces + 1] =   mesh->i_face_vtx_idx[n_i_faces]
                                             + n_f_vertices;
       n_i_faces++;
@@ -371,9 +373,9 @@ _extract_face_vertices(cs_mesh_t         *mesh,
 
     else if (face_type[i] == '\1' || face_type[i] == '\3') {
       cs_lnum_t *_b_face_vtx =   mesh->b_face_vtx_lst
-                                + mesh->b_face_vtx_idx[n_b_faces] - 1;
+                               + mesh->b_face_vtx_idx[n_b_faces];
       for (j = 0; j < n_f_vertices; j++)
-        _b_face_vtx[j] = _face_vtx[j];
+        _b_face_vtx[j] = _face_vtx[j] - 1;
       mesh->b_face_vtx_idx[n_b_faces + 1] =   mesh->b_face_vtx_idx[n_b_faces]
                                             + n_f_vertices;
       n_b_faces++;
@@ -381,9 +383,9 @@ _extract_face_vertices(cs_mesh_t         *mesh,
 
     else if (face_type[i] == '\2') {
       cs_lnum_t *_b_face_vtx =   mesh->b_face_vtx_lst
-                                + mesh->b_face_vtx_idx[n_b_faces] - 1;
+                               + mesh->b_face_vtx_idx[n_b_faces];
       for (j = 0; j < n_f_vertices; j++)
-        _b_face_vtx[j] = _face_vtx[n_f_vertices - j - 1];
+        _b_face_vtx[j] = _face_vtx[n_f_vertices - j - 1] - 1;
       mesh->b_face_vtx_idx[n_b_faces + 1] =   mesh->b_face_vtx_idx[n_b_faces]
                                             + n_f_vertices;
       n_b_faces++;
@@ -569,179 +571,6 @@ _face_ifs_to_interior(cs_interface_set_t  *face_ifs,
   BFT_FREE(i_face_id);
 }
 
-/*----------------------------------------------------------------------------
- * Compare periodic couples in global numbering form (qsort function).
- *
- * parameters:
- *   x <-> pointer to first couple
- *   y <-> pointer to second couple
- *
- * returns:
- *   lexicographical
- *----------------------------------------------------------------------------*/
-
-static int _compare_couples(const void *x, const void *y)
-{
-  int retval = 1;
-
-  const cs_gnum_t *c0 = x;
-  const cs_gnum_t *c1 = y;
-
-  if (c0[0] < c1[0])
-    retval = -1;
-
-  else if (c0[0] == c1[0]) {
-    if (c0[1] < c1[1])
-      retval = -1;
-    else if (c0[1] == c1[1])
-      retval = 0;
-  }
-
-  return retval;
-}
-
-/*----------------------------------------------------------------------------
- * Extract periodic face connectivity information for mesh builder when
- * running in parallel mode.
- *
- * parameters:
- *   mesh     <-- pointer to mesh structure
- *   mb       <-> pointer to mesh builder structure
- *   face_ifs <-- parallel and periodic faces interfaces set
- *----------------------------------------------------------------------------*/
-
-static void
-_extract_periodic_faces_g(const cs_mesh_t           *mesh,
-                          cs_mesh_builder_t         *mb,
-                          const cs_interface_set_t  *face_ifs)
-{
-  int i, j;
-  cs_lnum_t k, l;
-
-  int perio_count = 0;
-  cs_lnum_t  *send_index = NULL;
-  cs_gnum_t  *recv_num = NULL;
-  int  *tr_id = NULL;
-
-  cs_datatype_t gnum_type = CS_GNUM_TYPE;
-
-  const int n_perio = mesh->n_init_perio;
-  const int n_interfaces = cs_interface_set_size(face_ifs);
-  const cs_gnum_t *face_gnum = mesh->global_i_face_num;
-
-  /* Allocate arrays in mesh builder (initializing per_face_idx) */
-
-  assert(mesh->periodicity != NULL);
-  assert(mb != NULL);
-  assert(mb->n_g_per_face_couples == 0);
-
-  mb->n_perio = n_perio;
-
-  BFT_MALLOC(mb->n_per_face_couples, n_perio, cs_lnum_t);
-  BFT_MALLOC(mb->per_face_couples, n_perio, cs_gnum_t *);
-
-  for (i = 0; i < n_perio; i++) {
-    mb->n_per_face_couples[i] = 0;
-    mb->per_face_couples[i] = NULL;
-  }
-
-  /* List direct and reverse transforms */
-
-  BFT_MALLOC(tr_id, n_perio*2, int);
-
-  for (i = 0; i < n_perio*2; i++) {
-    int rev_id = fvm_periodicity_get_reverse_id(mesh->periodicity, i);
-    if (i < rev_id) {
-      int parent_ids[2];
-      fvm_periodicity_get_parent_ids(mesh->periodicity, i, parent_ids);
-      if (parent_ids[0] < 0 && parent_ids[1] < 0) {
-        tr_id[perio_count*2] = i + 1;
-        tr_id[perio_count*2 + 1] = rev_id + 1;
-        perio_count++;
-      }
-    }
-  }
-  assert(perio_count == n_perio);
-
-  for (i = 0; i < n_interfaces; i++) {
-    const cs_interface_t *face_if = cs_interface_set_get(face_ifs, i);
-    const cs_lnum_t *tr_index = cs_interface_get_tr_index(face_if);
-    for (j = 0; j < n_perio; j++) {
-      const cs_lnum_t n_tr_faces = (  tr_index[tr_id[j*2] + 1]
-                                    - tr_index[tr_id[j*2]]);
-      mb->n_per_face_couples[j] += n_tr_faces;
-    }
-  }
-
-  BFT_MALLOC(recv_num, cs_interface_set_n_elts(face_ifs), cs_gnum_t);
-
-  cs_interface_set_copy_array(face_ifs,
-                              gnum_type,
-                              1,
-                              true, /* src_on_parent */
-                              face_gnum,
-                              recv_num);
-
-  /* Prepare send buffer (send reverse transformation values) */
-
-  BFT_FREE(send_index);
-
-  for (i = 0; i < n_perio; i++)
-    BFT_MALLOC(mb->per_face_couples[i], mb->n_per_face_couples[i]*2, cs_gnum_t);
-
-  /* Reset couples count */
-
-  for (i = 0; i < n_perio; i++)
-    mb->n_per_face_couples[i] = 0;
-
-  /* Copy face couples to mesh builder */
-
-  for (i = 0, j = 0, l = 0; i < n_interfaces; i++) {
-
-    const cs_interface_t *face_if = cs_interface_set_get(face_ifs, i);
-    const cs_lnum_t *tr_index = cs_interface_get_tr_index(face_if);
-    const cs_lnum_t *elt_id = cs_interface_get_elt_ids(face_if);
-
-    l += tr_index[1];
-
-    for (j = 0; j < n_perio; j++) {
-
-      /* Count couples in direct periodicity */
-
-      cs_lnum_t nc = mb->n_per_face_couples[j]*2;
-      const cs_lnum_t start_id = tr_index[tr_id[j*2]];
-      const cs_lnum_t end_id = tr_index[tr_id[j*2] + 1];
-
-      for (k = start_id; k < end_id; k++) {
-        cs_lnum_t f_id = elt_id[k];
-        mb->per_face_couples[j][nc++] = face_gnum[f_id];
-        mb->per_face_couples[j][nc++] = recv_num[l++];
-      }
-      mb->n_per_face_couples[j] = nc/2;
-
-      /* Ignore couples in reverse periodicity */
-
-      l += tr_index[tr_id[j*2 + 1] + 1] - tr_index[tr_id[j*2 + 1]];
-
-    }
-
-  }
-
-  BFT_FREE(recv_num);
-  BFT_FREE(tr_id);
-
-  /* Now sort couples in place for future use (more for consistency
-     and ease of verification than absolutely necessary) */
-
-  for (i = 0; i < n_perio; i++) {
-    if (mb->n_per_face_couples[i] > 0)
-      qsort(mb->per_face_couples[i],
-            mb->n_per_face_couples[i],
-            sizeof(cs_gnum_t) * 2,
-            &_compare_couples);
-  }
-}
-
 #endif /* defined(HAVE_MPI) */
 
 /*----------------------------------------------------------------------------
@@ -878,14 +707,14 @@ _f_face_center(cs_lnum_t         n_f_faces,
 
     for (vtx_id = start_id; vtx_id < end_id; vtx_id++) {
 
-      cs_lnum_t shift = 3 * (face_vtx[vtx_id] - 1);
+      cs_lnum_t shift = 3 * (face_vtx[vtx_id]);
       for (i = 0; i < 3; i++)
         face_vtx_coord[n_face_vertices][i] = vtx_coord[shift + i];
       n_face_vertices++;
 
     }
 
-    /* Compute the barycentre of the face vertices */
+    /* Compute the barycenter of the face vertices */
 
     for (i = 0; i < 3; i++) {
       vtx_cog[i] = 0.0;
@@ -895,7 +724,7 @@ _f_face_center(cs_lnum_t         n_f_faces,
     }
 
     /* Loop on the triangles of the face (defined by an edge of the face
-       and its barycentre) */
+       and its barycenter) */
 
     for (i = 0; i < 3; i++) {
       ref_normal[i] = 0.;
@@ -1095,7 +924,7 @@ _default_face_rank(const cs_mesh_builder_t  *mb,
   n_free_faces = 0;
 
   for (i = 0; i < _n_faces; i++) {
-    if (mb->face_cells[i*2] == 0 && mb->face_cells[i*2 + 1] == 0)
+    if (mb->face_cells[2*i] == 0 && mb->face_cells[2*i+1] == 0)
       n_free_faces += 1;
   }
 
@@ -1132,7 +961,7 @@ _default_face_rank(const cs_mesh_builder_t  *mb,
 
   n_free_faces = 0;
   for (i = 0; i < _n_faces; i++) {
-    if (mb->face_cells[i*2] == 0 && mb->face_cells[i*2 + 1] == 0)
+    if (mb->face_cells[2*i] == 0 && mb->face_cells[2*i+1] == 0)
       free_face_ids[n_free_faces++] = i;
   }
 
@@ -1190,7 +1019,7 @@ _decompose_data_g(cs_mesh_t          *mesh,
   cs_gnum_t *_face_gcells = NULL;
   cs_gnum_t *_face_gvertices = NULL;
 
-  cs_lnum_t *_face_cells = NULL;
+  cs_lnum_2_t *_face_cells = NULL;
   cs_lnum_t *_face_gc_id = NULL;
   cs_lnum_t *_face_vertices_idx = NULL;
   cs_lnum_t *_face_vertices = NULL;
@@ -1254,6 +1083,8 @@ _decompose_data_g(cs_mesh_t          *mesh,
                 "(number of cells / number of processes ratio too low)."),
               (int)cs_glob_rank_id);
 
+  mesh->n_cells_with_ghosts = mesh->n_cells; /* will be increased later */
+
   /* Distribute faces */
   /*------------------*/
 
@@ -1288,15 +1119,15 @@ _decompose_data_g(cs_mesh_t          *mesh,
 
   /* Now convert face -> cell connectivity to local cell numbers */
 
-  BFT_MALLOC(_face_cells, _n_faces*2, cs_lnum_t);
+  BFT_MALLOC(_face_cells, _n_faces, cs_lnum_2_t);
 
   cs_block_to_part_global_to_local(_n_faces*2,
-                                   1,
+                                   0,
                                    mesh->n_cells,
                                    true,
                                    mesh->global_cell_num,
                                    _face_gcells,
-                                   _face_cells);
+                                   (cs_lnum_t *)_face_cells);
 
   BFT_FREE(_face_gcells);
 
@@ -1446,9 +1277,11 @@ _decompose_data_g(cs_mesh_t          *mesh,
 
   if (mb->n_perio > 0) {
     _face_ifs_to_interior(face_ifs, _n_faces, face_type);
-    _extract_periodic_faces_g(mesh,
-                              mb,
-                              face_ifs);
+    cs_mesh_builder_extract_periodic_faces_g(mesh->n_init_perio,
+                                             mb,
+                                             mesh->periodicity,
+                                             mesh->global_i_face_num,
+                                             face_ifs);
     cs_interface_set_destroy(&face_ifs);
   }
 
@@ -1480,7 +1313,7 @@ _decompose_data_l(cs_mesh_t          *mesh,
 
   cs_lnum_t _n_faces = 0;
 
-  cs_lnum_t *_face_cells = NULL;
+  cs_lnum_2_t *_face_cells = NULL;
   cs_lnum_t *_face_vertices_idx = NULL;
   cs_lnum_t *_face_vertices = NULL;
 
@@ -1491,6 +1324,7 @@ _decompose_data_l(cs_mesh_t          *mesh,
   assert((sizeof(cs_lnum_t) == 4) || (sizeof(cs_lnum_t) == 8));
 
   mesh->n_cells = mb->cell_bi.gnum_range[1] - 1;
+  mesh->n_cells_with_ghosts = mesh->n_cells; /* may be increased later */
 
   /* Cell families are already of the correct type,
      so they can simply be moved */
@@ -1505,11 +1339,11 @@ _decompose_data_l(cs_mesh_t          *mesh,
 
   /* Now copy face -> cell connectivity to local cell numbers */
 
-  BFT_MALLOC(_face_cells, _n_faces*2, cs_lnum_t);
+  BFT_MALLOC(_face_cells, _n_faces, cs_lnum_2_t);
 
   for (i = 0; i < _n_faces; i++) {
-    _face_cells[i*2] = mb->face_cells[i*2];
-    _face_cells[i*2 + 1] = mb->face_cells[i*2 + 1];
+    _face_cells[i][0] = mb->face_cells[i*2] - 1;
+    _face_cells[i][1] = mb->face_cells[i*2+1] - 1;
   }
 
   BFT_FREE(mb->face_cells);
@@ -1586,6 +1420,8 @@ _decompose_data_l(cs_mesh_t          *mesh,
   BFT_FREE(face_type);
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
