@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -58,8 +58,8 @@ void
 cs_mesh_halo_define(cs_mesh_t           *mesh,
                     cs_interface_set_t  *face_ifs,
                     cs_interface_set_t  *vertex_ifs,
-                    cs_int_t            *p_gcell_vtx_idx[],
-                    cs_int_t            *p_gcell_vtx_lst[]);
+                    cs_lnum_t           *p_gcell_vtx_idx[],
+                    cs_lnum_t           *p_gcell_vtx_lst[]);
 
 /*----------------------------------------------------------------------------*/
 
