@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -45,7 +45,9 @@
 #include "bft_error.h"
 #include "bft_printf.h"
 
+#include "cs_blas.h"
 #include "cs_interface.h"
+#include "cs_math.h"
 #include "cs_mesh.h"
 #include "cs_mesh_quantities.h"
 #include "cs_post.h"
@@ -60,6 +62,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -100,12 +104,12 @@ BEGIN_C_DECLS
  *----------------------------------------------------------------------------*/
 
 static void
-_compute_local_minmax(cs_int_t         n_vals,
+_compute_local_minmax(cs_lnum_t        n_vals,
                       const cs_real_t  var[],
                       cs_real_t       *min,
                       cs_real_t       *max)
 {
-  cs_int_t  i;
+  cs_lnum_t  i;
   cs_real_t  _min = DBL_MAX, _max = -DBL_MAX;
 
   for (i = 0; i < n_vals; i++) {
@@ -196,11 +200,11 @@ _display_histograms(int        n_steps,
  *----------------------------------------------------------------------------*/
 
 static void
-_histogram(cs_int_t         n_vals,
+_histogram(cs_lnum_t        n_vals,
            const cs_real_t  var[])
 {
-  cs_int_t  i;
-  int       j, k;
+  cs_lnum_t  i;
+  int        j, k;
 
   cs_real_t  step;
   cs_real_t  max, min, _max, _min;
@@ -271,8 +275,8 @@ static void
 _int_face_histogram(const cs_mesh_t  *mesh,
                     const cs_real_t   var[])
 {
-  cs_int_t  i;
-  int       j, k;
+  cs_lnum_t  i;
+  int        j, k;
 
   cs_real_t  step;
   cs_real_t  max, min, _max, _min;
@@ -316,7 +320,7 @@ _int_face_histogram(const cs_mesh_t  *mesh,
 
     for (i = 0; i < mesh->n_i_faces; i++) {
 
-      if (mesh->i_face_cells[i*2] > mesh->n_cells)
+      if (mesh->i_face_cells[i][0] >= mesh->n_cells)
         continue;
 
       /* Associated subdivision */
@@ -351,15 +355,14 @@ _compute_weighting_offsetting(const cs_mesh_t             *mesh,
                               cs_real_t                    weighting[],
                               cs_real_t                    offsetting[])
 {
-  cs_int_t  i, face_id, cell1, cell2;
-  cs_real_t  intersection;
+  cs_lnum_t  i, face_id, cell1, cell2;
   cs_real_t  cell_center1[3], cell_center2[3];
   cs_real_t  face_center[3], face_normal[3];
   cs_real_t  v0[3], v1[3], v2[3];
 
   double  coef0 = 0, coef1 = 0, coef2 = 0;
 
-  const cs_int_t  dim = mesh->dim;
+  const cs_lnum_t  dim = mesh->dim;
 
   /* Compute weighting coefficient */
   /*-------------------------------*/
@@ -370,8 +373,8 @@ _compute_weighting_offsetting(const cs_mesh_t             *mesh,
 
     /* Get local number of the cells in contact with the face */
 
-    cell1 = mesh->i_face_cells[2 * face_id] - 1;
-    cell2 = mesh->i_face_cells[2 * face_id + 1] - 1;
+    cell1 = mesh->i_face_cells[face_id][0];
+    cell2 = mesh->i_face_cells[face_id][1];
 
     /* Get information on mesh quantities */
 
@@ -411,20 +414,16 @@ _compute_weighting_offsetting(const cs_mesh_t             *mesh,
     /* Compute center offsetting coefficient */
     /*---------------------------------------*/
 
-    /* Compute intersection between face and segment defined by the two cell
-       centers */
-
     for (i = 0; i < dim; i++) {
-
-      intersection =  (1 - weighting[face_id]) * cell_center1[i]
-                    + weighting[face_id] * cell_center2[i];
-
-      v1[i] = intersection - face_center[i];
-      v2[i] = cell_center2[i] - cell_center1[i];
-
+      v1[i] = mesh_quantities->dofij[face_id*3 + i];
+      v2[i] = mesh_quantities->i_face_normal[face_id*3 + i];
     }
+    double of_s = _MODULE_3D(v1) * _MODULE_3D(v2);
 
-    offsetting[face_id] = _MODULE_3D(v1) / _MODULE_3D(v2);
+    offsetting[cell1] = CS_MAX(offsetting[cell1],
+        1. - pow(of_s / fabs(mesh_quantities->cell_vol[cell1]), 1./3.));
+    offsetting[cell2] = CS_MAX(offsetting[cell2],
+        1. - pow(of_s / fabs(mesh_quantities->cell_vol[cell2]), 1./3.));
 
   } /* End of loop on faces */
 
@@ -447,14 +446,14 @@ _compute_orthogonality(const cs_mesh_t             *mesh,
                        cs_real_t                    i_face_ortho[],
                        cs_real_t                    b_face_ortho[])
 {
-  cs_int_t  i, face_id, cell1, cell2;
+  cs_lnum_t  i, face_id, cell1, cell2;
   double  cos_alpha;
   cs_real_t  cell_center1[3], cell_center2[3];
   cs_real_t  face_center[3];
   cs_real_t  face_normal[3], vect[3];
 
   const double  rad_to_deg = 180. / acos(-1.);
-  const cs_int_t  dim = mesh->dim;
+  const cs_lnum_t  dim = mesh->dim;
 
   /* Loop on internal faces */
   /*------------------------*/
@@ -463,8 +462,8 @@ _compute_orthogonality(const cs_mesh_t             *mesh,
 
     /* Get local number of the cells beside the face */
 
-    cell1 = mesh->i_face_cells[2 * face_id] - 1;
-    cell2 = mesh->i_face_cells[2 * face_id + 1] - 1;
+    cell1 = mesh->i_face_cells[face_id][0];
+    cell2 = mesh->i_face_cells[face_id][1];
 
     /* Get information on mesh quantities */
 
@@ -504,7 +503,7 @@ _compute_orthogonality(const cs_mesh_t             *mesh,
 
     /* Get local number of the cell beside the face */
 
-    cell1 = mesh->b_face_cells[face_id] - 1;
+    cell1 = mesh->b_face_cells[face_id];
 
     /* Get information on mesh quantities */
 
@@ -548,21 +547,21 @@ _compute_orthogonality(const cs_mesh_t             *mesh,
  * parameters:
  *   idx_start       <-- first vertex index
  *   idx_end         <-- last vertex index
- *   face_vertex_num <-- face -> vertices connectivity
+ *   face_vertex_id  <-- face -> vertices connectivity
  *   face_normal     <-- face normal
  *   vertex_coords   <-- vertices coordinates
  *   face_warping    --> face warping angle
  *----------------------------------------------------------------------------*/
 
 static void
-_get_face_warping(cs_int_t         idx_start,
-                  cs_int_t         idx_end,
+_get_face_warping(cs_lnum_t        idx_start,
+                  cs_lnum_t        idx_end,
                   const cs_real_t  face_normal[],
-                  const cs_int_t   face_vertex_num[],
+                  const cs_lnum_t  face_vertex_id[],
                   const cs_real_t  vertex_coords[],
                   double           face_warping[])
 {
-  cs_int_t  i, idx, vertex_num1, vertex_num2;
+  cs_lnum_t  i, idx, vertex_id1, vertex_id2;
   double  edge_cos_alpha;
   cs_real_t  vect[3];
 
@@ -575,14 +574,14 @@ _get_face_warping(cs_int_t         idx_start,
 
   for (idx = idx_start; idx < idx_end - 1; idx++) {
 
-    vertex_num1 = face_vertex_num[idx] - 1;
-    vertex_num2 = face_vertex_num[idx + 1] - 1;
+    vertex_id1 = face_vertex_id[idx];
+    vertex_id2 = face_vertex_id[idx + 1];
 
     /* Get vertex coordinates */
 
     for (i = 0; i < dim; i++)
-      vect[i] =  vertex_coords[vertex_num2*dim + i]
-               - vertex_coords[vertex_num1*dim + i];
+      vect[i] =  vertex_coords[vertex_id2*dim + i]
+               - vertex_coords[vertex_id1*dim + i];
 
     edge_cos_alpha = _COSINE_3D(vect, face_normal);
     edge_cos_alpha = CS_ABS(edge_cos_alpha);
@@ -592,14 +591,14 @@ _get_face_warping(cs_int_t         idx_start,
 
   /* Last edge */
 
-  vertex_num1 = face_vertex_num[idx_end - 1] - 1;
-  vertex_num2 = face_vertex_num[idx_start] - 1;
+  vertex_id1 = face_vertex_id[idx_end - 1];
+  vertex_id2 = face_vertex_id[idx_start];
 
   /* Get vertex coordinates */
 
   for (i = 0; i < dim; i++)
-    vect[i] =  vertex_coords[vertex_num2*dim + i]
-             - vertex_coords[vertex_num1*dim + i];
+    vect[i] =  vertex_coords[vertex_id2*dim + i]
+             - vertex_coords[vertex_id1*dim + i];
 
   edge_cos_alpha = _COSINE_3D(vect, face_normal);
   edge_cos_alpha = CS_ABS(edge_cos_alpha);
@@ -629,7 +628,7 @@ _cell_from_max_face(const cs_mesh_t      *mesh,
                     const cs_real_t       b_face_val[],
                     cs_real_t             cell_val[])
 {
-  cs_int_t  i, j, cell_id;
+  cs_lnum_t  i, j, cell_id;
 
   /* Default initialization */
 
@@ -643,7 +642,7 @@ _cell_from_max_face(const cs_mesh_t      *mesh,
     for (i = 0; i < mesh->n_i_faces; i++) {
 
       for (j = 0; j < 2; j++) {
-        cell_id = mesh->i_face_cells[2*i + j] - 1;
+        cell_id = mesh->i_face_cells[i][j];
         if (i_face_val[i] > cell_val[cell_id])
           cell_val[cell_id] = i_face_val[i];
       }
@@ -656,7 +655,7 @@ _cell_from_max_face(const cs_mesh_t      *mesh,
 
     for (i = 0; i < mesh->n_b_faces; i++) {
 
-      cell_id = mesh->b_face_cells[i] - 1;
+      cell_id = mesh->b_face_cells[i];
       if (b_face_val[i] > cell_val[cell_id])
         cell_val[cell_id] = b_face_val[i];
 
@@ -685,7 +684,7 @@ _vtx_from_max_face(const cs_mesh_t     *mesh,
                    const cs_real_t      b_face_val[],
                    cs_real_t            vtx_val[])
 {
-  cs_int_t  i, j, idx_start, idx_end, vtx_id;
+  cs_lnum_t  i, j, idx_start, idx_end, vtx_id;
 
   /* Default initialization */
 
@@ -698,11 +697,11 @@ _vtx_from_max_face(const cs_mesh_t     *mesh,
 
     for (i = 0; i < mesh->n_i_faces; i++) {
 
-      idx_start = mesh->i_face_vtx_idx[i] - 1;
-      idx_end = mesh->i_face_vtx_idx[i + 1] - 1;
+      idx_start = mesh->i_face_vtx_idx[i];
+      idx_end = mesh->i_face_vtx_idx[i + 1];
 
       for (j = idx_start; j < idx_end; j++) {
-        vtx_id = mesh->i_face_vtx_lst[j] - 1;
+        vtx_id = mesh->i_face_vtx_lst[j];
         if (i_face_val[i] > vtx_val[vtx_id])
           vtx_val[vtx_id] = i_face_val[i];
       }
@@ -715,11 +714,11 @@ _vtx_from_max_face(const cs_mesh_t     *mesh,
 
     for (i = 0; i < mesh->n_b_faces; i++) {
 
-      idx_start = mesh->b_face_vtx_idx[i] - 1;
-      idx_end = mesh->b_face_vtx_idx[i + 1] - 1;
+      idx_start = mesh->b_face_vtx_idx[i];
+      idx_end = mesh->b_face_vtx_idx[i + 1];
 
       for (j = idx_start; j < idx_end; j++) {
-        vtx_id = mesh->b_face_vtx_lst[j] - 1;
+        vtx_id = mesh->b_face_vtx_lst[j];
         if (b_face_val[i] > vtx_val[vtx_id])
           vtx_val[vtx_id] = b_face_val[i];
       }
@@ -737,6 +736,180 @@ _vtx_from_max_face(const cs_mesh_t     *mesh,
                          vtx_val);
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief  Define the cell -> faces connectivity
+ *
+ * \param[in]  mesh        pointer to a cs_mesh_t structure
+ * \param[in]  p_c2f_idx   pointer to the array of indexes
+ * \param[in]  p_c2f_lst   pointer to the list of face ids
+ */
+/*----------------------------------------------------------------------------*/
+
+static void
+_build_c2f(const cs_mesh_t   *mesh,
+           cs_lnum_t         *p_c2f_idx[],
+           cs_lnum_t         *p_c2f_ids[])
+{
+  int  idx_size = 0;
+  int  *cell_shift = NULL;
+  cs_lnum_t  *c2f_idx = NULL;
+  cs_lnum_t  *c2f_ids = NULL;
+
+  const int  n_cells = mesh->n_cells;
+  const int  n_i_faces = mesh->n_i_faces;
+  const int  n_b_faces = mesh->n_b_faces;
+
+  BFT_MALLOC(c2f_idx, n_cells + 1, cs_lnum_t);
+  BFT_MALLOC(cell_shift, n_cells, int);
+
+# pragma omp parallel for if (n_cells > CS_THR_MIN)
+  for (cs_lnum_t i = 0; i < n_cells; i++)
+    cell_shift[i] = c2f_idx[i] = 0;
+  c2f_idx[n_cells] = 0;
+
+  for (cs_lnum_t i = 0; i < n_b_faces; i++) {
+    c2f_idx[mesh->b_face_cells[i]+1] += 1;
+    idx_size += 1;
+  }
+
+  for (cs_lnum_t i = 0; i < n_i_faces; i++) {
+
+    const int  c1_id = mesh->i_face_cells[i][0];
+    const int  c2_id = mesh->i_face_cells[i][1];
+
+    if (c1_id < n_cells) // cell owned by the local rank
+      c2f_idx[c1_id+1] += 1, idx_size += 1;
+    if (c2_id < n_cells) // cell owned by the local rank
+      c2f_idx[c2_id+1] += 1, idx_size += 1;
+
+  }
+
+  for (cs_lnum_t i = 0; i < n_cells; i++)
+    c2f_idx[i+1] += c2f_idx[i];
+
+  assert(c2f_idx[n_cells] == idx_size);
+
+  BFT_MALLOC(c2f_ids, idx_size, cs_lnum_t);
+
+  for (cs_lnum_t f_id = 0; f_id < n_i_faces; f_id++) {
+
+    const cs_lnum_t  c1_id = mesh->i_face_cells[f_id][0];
+    const cs_lnum_t  c2_id = mesh->i_face_cells[f_id][1];
+
+    if (c1_id < n_cells) { /* Don't want ghost cells */
+
+      const cs_lnum_t  shift = c2f_idx[c1_id] + cell_shift[c1_id];
+      c2f_ids[shift] = f_id;
+      cell_shift[c1_id] += 1;
+
+    }
+
+    if (c2_id < n_cells) { /* Don't want ghost cells */
+
+      const cs_lnum_t  shift = c2f_idx[c2_id] + cell_shift[c2_id];
+      c2f_ids[shift] = f_id;
+      cell_shift[c2_id] += 1;
+
+    }
+
+  } /* End of loop on internal faces */
+
+  for (cs_lnum_t  f_id = 0; f_id < n_b_faces; f_id++) {
+
+    const cs_lnum_t  c_id = mesh->b_face_cells[f_id];
+    const cs_lnum_t  shift = c2f_idx[c_id] + cell_shift[c_id];
+
+    c2f_ids[shift] = n_i_faces + f_id;
+    cell_shift[c_id] += 1;
+
+  } /* End of loop on border faces */
+
+  /* Free memory */
+  BFT_FREE(cell_shift);
+
+  /* Return pointers */
+  *p_c2f_idx = c2f_idx;
+  *p_c2f_ids = c2f_ids;
+}
+
+/*----------------------------------------------------------------------------
+ * Compute cellwise the warping error
+ *  Id = 1/|c| \sum_(f \in F_c) x_f \otimes \vect{f}
+ * Froebinus norm is used to get a scalar-valued quantity
+ *
+ * parameters:
+ *   mesh             <-- pointer to mesh structure.
+ *   mesh_quantities  <-- pointer to mesh quantities structures.
+ *   warp_error       <-> array of values to compute
+ *----------------------------------------------------------------------------*/
+
+static void
+_compute_warp_error(const cs_mesh_t              *mesh,
+                    const cs_mesh_quantities_t   *mesh_quantities,
+                    cs_real_t                     warp_error[])
+{
+  const cs_real_t  *vol = mesh_quantities->cell_vol;
+
+  cs_lnum_t  *c2f_ids = NULL;
+  cs_lnum_t  *c2f_idx = NULL;
+
+  /* Build cell -> face connectivity */
+  _build_c2f(mesh, &c2f_idx, &c2f_ids);
+
+  for (cs_lnum_t c_id = 0; c_id < mesh->n_cells; c_id++) {
+
+    const cs_real_t  invvol_c = 1/vol[c_id];
+    const cs_real_t  *xc = mesh_quantities->cell_cen + 3*c_id;
+
+    cs_real_33_t   tens = { {0, 0, 0}, {0, 0, 0}, {0, 0, 0} };
+
+    for (cs_lnum_t i = c2f_idx[c_id]; i < c2f_idx[c_id+1]; i++) {
+
+      cs_lnum_t  f_id = c2f_ids[i];
+      cs_real_t  *xf = NULL, *surf = NULL;
+      int  sgn = 1;
+
+      if (f_id < mesh->n_i_faces) {
+
+        const cs_lnum_t  c2_id = mesh->i_face_cells[f_id][1];
+        if (c_id == c2_id)
+          sgn = -1;
+
+        xf = mesh_quantities->i_face_cog + 3*f_id;
+        surf = mesh_quantities->i_f_face_normal + 3*f_id;
+
+      }
+      else {
+
+        f_id -= mesh->n_i_faces; // Border face
+        xf = mesh_quantities->b_face_cog + 3*f_id;
+        surf = mesh_quantities->b_f_face_normal + 3*f_id;
+
+      }
+
+      for (int ki = 0; ki < 3; ki++)
+        for (int kj = 0; kj < 3; kj++)
+          tens[ki][kj] += sgn*(xf[ki] - xc[ki]) * surf[kj];
+
+    } // Loop on face cells
+
+    for (int ki = 0; ki < 3; ki++)
+      for (int kj = 0; kj < 3; kj++)
+        tens[ki][kj] *= invvol_c;
+
+    warp_error[c_id] =
+      fabs(cs_math_33_determinant((const cs_real_t (*)[3])tens) - 1.);
+
+  } // Loop on cells
+
+  /* Free memory */
+  BFT_FREE(c2f_ids);
+  BFT_FREE(c2f_idx);
+}
+
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
@@ -761,12 +934,12 @@ cs_mesh_quality_compute_warping(const cs_mesh_t    *mesh,
                                 cs_real_t           i_face_warping[],
                                 cs_real_t           b_face_warping[])
 {
-  cs_int_t  i, face_id, idx_start, idx_end;
+  cs_lnum_t  i, face_id, idx_start, idx_end;
   cs_real_t  this_face_normal[3];
 
-  const cs_int_t  dim = mesh->dim;
-  const cs_int_t  *i_face_vtx_idx = mesh->i_face_vtx_idx;
-  const cs_int_t  *b_face_vtx_idx = mesh->b_face_vtx_idx;
+  const cs_lnum_t  dim = mesh->dim;
+  const cs_lnum_t  *i_face_vtx_idx = mesh->i_face_vtx_idx;
+  const cs_lnum_t  *b_face_vtx_idx = mesh->b_face_vtx_idx;
 
   assert(dim == 3);
 
@@ -782,8 +955,8 @@ cs_mesh_quality_compute_warping(const cs_mesh_t    *mesh,
 
     /* Evaluate warping for each edge of face. Keep the max. */
 
-    idx_start = i_face_vtx_idx[face_id] - 1;
-    idx_end = i_face_vtx_idx[face_id + 1] - 1;
+    idx_start = i_face_vtx_idx[face_id];
+    idx_end = i_face_vtx_idx[face_id + 1];
 
     _get_face_warping(idx_start,
                       idx_end,
@@ -806,8 +979,8 @@ cs_mesh_quality_compute_warping(const cs_mesh_t    *mesh,
 
     /* Evaluate warping for each edge */
 
-    idx_start = b_face_vtx_idx[face_id] - 1;
-    idx_end = b_face_vtx_idx[face_id + 1] - 1;
+    idx_start = b_face_vtx_idx[face_id];
+    idx_end = b_face_vtx_idx[face_id + 1];
 
     _get_face_warping(idx_start,
                       idx_end,
@@ -832,12 +1005,14 @@ void
 cs_mesh_quality(const cs_mesh_t             *mesh,
                 const cs_mesh_quantities_t  *mesh_quantities)
 {
-  cs_int_t  i;
+  cs_lnum_t  i;
 
   bool  compute_volume = true;
   bool  compute_weighting = true;
   bool  compute_orthogonality = true;
   bool  compute_warping = true;
+  bool  compute_thickness = true;
+  bool  compute_warp_error = true;
   bool  vol_fields = false;
   bool  brd_fields = false;
 
@@ -846,16 +1021,18 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
 
   double  *working_array = NULL;
 
-  const cs_int_t  n_vertices = mesh->n_vertices;
-  const cs_int_t  n_i_faces = mesh->n_i_faces;
-  const cs_int_t  n_b_faces = mesh->n_b_faces;
-  const cs_int_t  n_cells = mesh->n_cells;
-  const cs_int_t  n_cells_wghosts = mesh->n_cells_with_ghosts;
+  const cs_lnum_t  n_vertices = mesh->n_vertices;
+  const cs_lnum_t  n_i_faces = mesh->n_i_faces;
+  const cs_lnum_t  n_b_faces = mesh->n_b_faces;
+  const cs_lnum_t  n_cells = mesh->n_cells;
+  const cs_lnum_t  n_cells_wghosts = mesh->n_cells_with_ghosts;
+
+  const cs_time_step_t *ts = cs_glob_time_step;
 
   /* Check input data */
 
-  assert(mesh_quantities->i_face_normal != NULL);
-  assert(mesh_quantities->i_face_cog != NULL);
+  assert(mesh_quantities->i_face_normal != NULL || mesh->n_i_faces == 0);
+  assert(mesh_quantities->i_face_cog != NULL || mesh->n_i_faces == 0);
   assert(mesh_quantities->cell_cen != NULL);
   assert(mesh_quantities->cell_vol != NULL);
 
@@ -883,10 +1060,11 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
      For the moment, we export the mesh at this stage; this should be moved
      once PSTEMA has been moved from CALTRI to an earlier step. */
 
-  cs_post_write_meshes(NULL);
-
   cs_post_activate_writer(0, 1);
 
+  if (vol_fields || brd_fields)
+    cs_post_write_meshes(ts);
+
   /* Evaluate mesh quality criteria */
   /*--------------------------------*/
 
@@ -914,14 +1092,14 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
 
     /* Display histograms */
 
-    bft_printf(_("\n  Histogram of the interior faces warping:\n\n"));
-    _int_face_histogram(mesh, i_face_warping);
+    if (mesh->n_g_i_faces > 0) {
+      bft_printf(_("\n  Histogram of the interior faces warping:\n\n"));
+      _int_face_histogram(mesh, i_face_warping);
+    }
 
     if (mesh->n_g_b_faces > 0) {
-
       bft_printf(_("\n  Histogram of the boundary faces warping:\n\n"));
       _histogram(n_b_faces, b_face_warping);
-
     }
 
     /* Post processing */
@@ -936,7 +1114,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                             b_face_warping,
                             face_to_cell);
 
-        cs_post_write_var(-1,
+        cs_post_write_var(CS_POST_MESH_VOLUME,
+                          CS_POST_WRITER_ALL_ASSOCIATED,
                           "Face_Warp_c_max",
                           1,
                           false,
@@ -945,7 +1124,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                           face_to_cell,
                           NULL,
                           NULL,
-                          NULL);
+                          ts);
 
       }
       if (face_to_vtx != NULL) {
@@ -956,20 +1135,22 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                            b_face_warping,
                            face_to_vtx);
 
-        cs_post_write_vertex_var(-1,
+        cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                                 CS_POST_WRITER_ALL_ASSOCIATED,
                                  "Face_Warp_v_max",
                                  1,
                                  false,
                                  true,
                                  CS_POST_TYPE_cs_real_t,
                                  face_to_vtx,
-                                 NULL);
+                                 ts);
       }
 
     } /* End of post-processing on volume */
 
     if (brd_fields == true)
-      cs_post_write_var(-2,
+      cs_post_write_var(CS_POST_MESH_BOUNDARY,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
                         "Face_Warp",
                         1,
                         false,
@@ -978,7 +1159,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                         NULL,
                         NULL,
                         b_face_warping,
-                        NULL);
+                        ts);
 
     BFT_FREE(working_array);
 
@@ -994,9 +1175,9 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
 
     /* Only defined on internal faces */
 
-    BFT_MALLOC(working_array, 2*n_i_faces, double);
+    BFT_MALLOC(working_array, n_i_faces + n_cells_wghosts, double);
 
-    for (i = 0; i < 2*n_i_faces; i++)
+    for (i = 0; i < n_i_faces + n_cells_wghosts; i++)
       working_array[i] = 0.;
 
     weighting = working_array;
@@ -1009,13 +1190,15 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
 
     /* Display histograms */
 
-    bft_printf(_("\n  Histogram of the interior faces "
-                 "weighting coefficient:\n\n"));
-    _int_face_histogram(mesh, weighting);
+    if (mesh->n_g_i_faces > 0) {
+      bft_printf(_("\n  Histogram of the interior faces "
+                   "weighting coefficient:\n\n"));
+      _int_face_histogram(mesh, weighting);
+    }
 
-    bft_printf(_("\n  Histogram of the interior faces "
+    bft_printf(_("\n  Histogram of the cells "
                  "off-centering coefficient:\n\n"));
-    _int_face_histogram(mesh, offsetting);
+    _histogram(n_cells, offsetting);
 
     /* Post processing */
 
@@ -1024,7 +1207,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
       if (face_to_cell != NULL) {
 
         _cell_from_max_face(mesh, 0.5, weighting, NULL, face_to_cell);
-        cs_post_write_var(-1,
+        cs_post_write_var(CS_POST_MESH_VOLUME,
+                          CS_POST_WRITER_ALL_ASSOCIATED,
                           "Weighting_c_max",
                           1,
                           false,
@@ -1033,50 +1217,34 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                           face_to_cell,
                           NULL,
                           NULL,
-                          NULL);
+                          ts);
 
       }
       if (face_to_vtx != NULL) {
 
         _vtx_from_max_face(mesh, 0.5, weighting, NULL, face_to_vtx);
-        cs_post_write_vertex_var(-1,
+        cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                                 CS_POST_WRITER_ALL_ASSOCIATED,
                                  "Weighting_v_max",
                                  1,
                                  false,
                                  true,
                                  CS_POST_TYPE_cs_real_t,
                                  face_to_vtx,
-                                 NULL);
-
-      }
-      if (face_to_cell != NULL) {
-
-        _cell_from_max_face(mesh, 0., offsetting, NULL, face_to_cell);
-        cs_post_write_var(-1,
-                          "Offset_c_max",
-                          1,
-                          false,
-                          true,
-                          CS_POST_TYPE_cs_real_t,
-                          face_to_cell,
-                          NULL,
-                          NULL,
-                          NULL);
-
-      }
-      if (face_to_vtx != NULL) {
-
-        _vtx_from_max_face(mesh, 0., offsetting, NULL, face_to_vtx);
-        cs_post_write_vertex_var(-1,
-                                 "Offset_v_max",
-                                 1,
-                                 false,
-                                 true,
-                                 CS_POST_TYPE_cs_real_t,
-                                 face_to_vtx,
-                                 NULL);
+                                 ts);
 
       }
+      cs_post_write_var(CS_POST_MESH_VOLUME,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
+                        "Offset",
+                        1,
+                        false,
+                        true,
+                        CS_POST_TYPE_cs_real_t,
+                        offsetting,
+                        NULL,
+                        NULL,
+                        ts);
 
     } /* End of post-processing on volume */
 
@@ -1107,16 +1275,16 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
 
     /* Display histograms */
 
-    bft_printf(_("\n  Histogram of the interior faces "
-                 "non-orthogonality coefficient (in degrees):\n\n"));
-    _int_face_histogram(mesh, i_face_ortho);
+    if (mesh->n_g_i_faces > 0) {
+      bft_printf(_("\n  Histogram of the interior faces "
+                   "non-orthogonality coefficient (in degrees):\n\n"));
+      _int_face_histogram(mesh, i_face_ortho);
+    }
 
     if (mesh->n_g_b_faces > 0) {
-
       bft_printf(_("\n  Histogram of the boundary faces "
                    "non-orthogonality coefficient (in degrees):\n\n"));
       _histogram(n_b_faces, b_face_ortho);
-
     }
 
     /* Post processing */
@@ -1126,7 +1294,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
       if (face_to_cell != NULL) {
 
         _cell_from_max_face(mesh, 0., i_face_ortho, b_face_ortho, face_to_cell);
-        cs_post_write_var(-1,
+        cs_post_write_var(CS_POST_MESH_VOLUME,
+                          CS_POST_WRITER_ALL_ASSOCIATED,
                           "Non_Ortho_c_max",
                           1,
                           false,
@@ -1135,26 +1304,28 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                           face_to_cell,
                           NULL,
                           NULL,
-                          NULL);
+                          ts);
 
       }
       if (face_to_vtx != NULL) {
 
         _vtx_from_max_face(mesh, 0., i_face_ortho, b_face_ortho, face_to_vtx);
-        cs_post_write_vertex_var(-1,
+        cs_post_write_vertex_var(CS_POST_MESH_VOLUME,
+                                 CS_POST_WRITER_ALL_ASSOCIATED,
                                  "Non_Ortho_v_max",
                                  1,
                                  false,
                                  true,
                                  CS_POST_TYPE_cs_real_t,
                                  face_to_vtx,
-                                 NULL);
+                                 ts);
       }
 
     } /* End of post-processing on volume */
 
     if (brd_fields == true)
-      cs_post_write_var(-2,
+      cs_post_write_var(CS_POST_MESH_BOUNDARY,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
                         "Non_Ortho",
                         1,
                         false,
@@ -1163,7 +1334,7 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                         NULL,
                         NULL,
                         b_face_ortho,
-                        NULL);
+                        ts);
 
     BFT_FREE(working_array);
 
@@ -1186,7 +1357,8 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
     /* Post processing */
 
     if (vol_fields == true)
-      cs_post_write_var(-1,
+      cs_post_write_var(CS_POST_MESH_VOLUME,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
                         "Cell_Volume",
                         1,
                         false,
@@ -1195,7 +1367,83 @@ cs_mesh_quality(const cs_mesh_t             *mesh,
                         mesh_quantities->cell_vol,
                         NULL,
                         NULL,
-                        NULL);
+                        ts);
+
+  } /* End of cell volume treatment */
+
+  /*-------------------------*/
+  /* boundary cell thickness */
+  /*-------------------------*/
+
+  if (compute_thickness == true && mesh->n_g_b_faces > 0) {
+
+    cs_real_t *b_thickness;
+    BFT_MALLOC(b_thickness, mesh->n_b_faces, cs_real_t);
+
+    cs_mesh_quantities_b_thickness_f(mesh, mesh_quantities, 0, b_thickness);
+
+    /* Display histograms */
+
+    bft_printf(_("\n  Histogram of boundary cell thickness:\n\n"));
+    _histogram(mesh->n_b_faces, b_thickness);
+
+    /* Post processing */
+
+    if (vol_fields == true)
+      cs_post_write_var(CS_POST_MESH_BOUNDARY,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
+                        "Boundary_Cell_Thickness",
+                        1,
+                        false,
+                        true,
+                        CS_POST_TYPE_cs_real_t,
+                        NULL,
+                        NULL,
+                        b_thickness,
+                        ts);
+
+    BFT_FREE(b_thickness);
+
+  } /* End of boundary cell thickness treatment */
+
+  /*---------------*/
+  /* warping error */
+  /*---------------*/
+
+  if (compute_warp_error == true) {
+
+    cs_real_t  *warp_error = NULL;
+    BFT_MALLOC(warp_error, mesh->n_cells ,cs_real_t);
+
+    _compute_warp_error(mesh, mesh_quantities, warp_error);
+
+    /* Display histograms */
+
+    bft_printf(_("\n  Histogram of the cellwise warping error :\n\n"));
+    _histogram(n_cells, warp_error);
+
+    /* Post processing */
+
+    if (vol_fields == true)
+      cs_post_write_var(CS_POST_MESH_VOLUME,
+                        CS_POST_WRITER_ALL_ASSOCIATED,
+                        "Warp_Error",
+                        1,
+                        false,
+                        true,
+                        CS_POST_TYPE_cs_real_t,
+                        warp_error,
+                        NULL,
+                        NULL,
+                        ts);
+
+    double  l2_error = cs_gres(mesh->n_cells,
+                               mesh_quantities->cell_vol,
+                               warp_error,
+                               warp_error);
+
+    bft_printf(" L2-error norm induced by warping : %5.3e\n", sqrt(l2_error));
+    BFT_FREE(warp_error);
 
   } /* End of cell volume treatment */
 
