@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -52,12 +52,15 @@ BEGIN_C_DECLS
  * parameters:
  *   mesh     <-- pointer to mesh structure
  *   mb       <-- pointer to optional mesh builder structure, or NULL
+ *   path     <-- optional directory name for output, or NULL for default
+ *                (directory automatically created if necessary)
  *   filename <-- file name
  *----------------------------------------------------------------------------*/
 
 void
 cs_mesh_save(cs_mesh_t          *mesh,
              cs_mesh_builder_t  *mb,
+             const char         *path,
              const char         *filename);
 
 /*----------------------------------------------------------------------------*/
