@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -53,6 +53,7 @@
 #include "cs_mesh_quantities.h"
 #include "cs_parall.h"
 #include "cs_mesh_quality.h"
+#include "cs_all_to_all.h"
 
 /*----------------------------------------------------------------------------
  * Header for the current file
@@ -64,6 +65,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -314,7 +317,7 @@ _int_face_histogram(const cs_mesh_t      *mesh,
 
     for (i = 0; i < mesh->n_i_faces; i++) {
 
-      if (mesh->i_face_cells[i*2] > mesh->n_cells)
+      if (mesh->i_face_cells[i][0] >= mesh->n_cells)
         continue;
 
       /* Associated subdivision */
@@ -383,15 +386,15 @@ _get_local_tolerance(const cs_real_t   vtx_coords[],
 
   for (face_id = 0; face_id < n_faces; face_id++) {
 
-    start = face_vtx_idx[face_id] - 1;
-    end = face_vtx_idx[face_id + 1] - 1;
+    start = face_vtx_idx[face_id];
+    end = face_vtx_idx[face_id + 1];
 
     /* Loop on the vertices of the face */
 
     for (j = start; j < end - 1; j++) {
 
-      vtx_id1 = face_vtx_lst[j] - 1;
-      vtx_id2 = face_vtx_lst[j+1] - 1;
+      vtx_id1 = face_vtx_lst[j];
+      vtx_id2 = face_vtx_lst[j+1];
 
       for (k = 0; k < 3; k++) {
         a[k] = vtx_coords[3*vtx_id1 + k];
@@ -407,8 +410,8 @@ _get_local_tolerance(const cs_real_t   vtx_coords[],
 
     /* Case end - start */
 
-    vtx_id1 = face_vtx_lst[end-1] - 1;
-    vtx_id2 = face_vtx_lst[start] - 1;
+    vtx_id1 = face_vtx_lst[end-1];
+    vtx_id2 = face_vtx_lst[start];
 
     for (k = 0; k < 3; k++) {
       a[k] = vtx_coords[3*vtx_id1 + k];
@@ -439,14 +442,12 @@ static void
 _get_global_tolerance(cs_mesh_t            *mesh,
                       cs_real_t            *vtx_tolerance)
 {
-  cs_int_t  i, rank, vtx_id, block_size, shift;
+  cs_lnum_t  i, vtx_id;
   cs_gnum_t  first_vtx_gnum;
 
   cs_lnum_t n_vertices = mesh->n_vertices;
-  double  *g_vtx_tolerance = NULL, *send_list = NULL, *recv_list = NULL;
-  cs_int_t  *send_count = NULL, *recv_count = NULL;
-  cs_int_t  *send_shift = NULL, *recv_shift = NULL;
-  cs_gnum_t  *send_glist = NULL, *recv_glist = NULL;
+  double  *g_vtx_tolerance = NULL, *send_list = NULL;
+  cs_gnum_t  *send_glist = NULL;
   cs_gnum_t  n_g_vertices = mesh->n_g_vertices;
   const cs_gnum_t  *io_gnum = mesh->global_vtx_num;
 
@@ -456,75 +457,41 @@ _get_global_tolerance(cs_mesh_t            *mesh,
 
   /* Define a fvm_io_num_t structure on vertices */
 
-  block_size = n_g_vertices / n_ranks;
-  if (n_g_vertices % n_ranks > 0)
-    block_size += 1;
+  cs_block_dist_info_t
+    bi = cs_block_dist_compute_sizes(local_rank,
+                                     n_ranks,
+                                     1,
+                                     0,
+                                     n_g_vertices);
 
-  /* Count the number of vertices to send to each rank */
-  /* ------------------------------------------------- */
+  cs_int_t block_size = bi.block_size;
 
-  BFT_MALLOC(send_count, n_ranks, int);
-  BFT_MALLOC(recv_count, n_ranks, int);
-  BFT_MALLOC(send_shift, n_ranks + 1, int);
-  BFT_MALLOC(recv_shift, n_ranks + 1, int);
-
-  send_shift[0] = 0;
-  recv_shift[0] = 0;
-
-  for (rank = 0; rank < n_ranks; rank++)
-    send_count[rank] = 0;
-
-  for (i = 0; i < n_vertices; i++) {
-    rank = (io_gnum[i] - 1)/block_size;
-    send_count[rank] += 1;
-  }
-
-  MPI_Alltoall(send_count, 1, MPI_INT, recv_count, 1, MPI_INT, mpi_comm);
-
-  for (rank = 0; rank < n_ranks; rank++) {
-    send_shift[rank + 1] = send_shift[rank] + send_count[rank];
-    recv_shift[rank + 1] = recv_shift[rank] + recv_count[rank];
-  }
-
-  assert(send_shift[n_ranks] == n_vertices);
+  cs_all_to_all_t
+    *d = cs_all_to_all_create_from_block(n_vertices,
+                                         0, /* flags */
+                                         io_gnum,
+                                         bi,
+                                         mpi_comm);
 
   /* Send the global numbering for each vertex */
-  /* ----------------------------------------- */
 
-  BFT_MALLOC(send_glist, n_vertices, cs_gnum_t);
-  BFT_MALLOC(recv_glist, recv_shift[n_ranks], cs_gnum_t);
-
-  for (rank = 0; rank < n_ranks; rank++)
-    send_count[rank] = 0;
-
-  for (i = 0; i < n_vertices; i++) {
-    rank = (io_gnum[i] - 1)/block_size;
-    shift = send_shift[rank] + send_count[rank];
-    send_count[rank] += 1;
-    send_glist[shift] = io_gnum[i];
-  }
-
-  MPI_Alltoallv(send_glist, send_count, send_shift, CS_MPI_GNUM,
-                recv_glist, recv_count, recv_shift, CS_MPI_GNUM, mpi_comm);
+  cs_gnum_t *recv_glist = cs_all_to_all_copy_array(d,
+                                                   CS_GNUM_TYPE,
+                                                   1,
+                                                   false, /* reverse */
+                                                   io_gnum,
+                                                   NULL);
 
   /* Send the vertex tolerance for each vertex */
-  /* ----------------------------------------- */
 
-  BFT_MALLOC(send_list, n_vertices, double);
-  BFT_MALLOC(recv_list, recv_shift[n_ranks], double);
+  double *recv_list = cs_all_to_all_copy_array(d,
+                                               CS_REAL_TYPE,
+                                               1,
+                                               false, /* reverse */
+                                               vtx_tolerance,
+                                               NULL);
 
-  for (rank = 0; rank < n_ranks; rank++)
-    send_count[rank] = 0;
-
-  for (i = 0; i < n_vertices; i++) {
-    rank = (io_gnum[i] - 1)/block_size;
-    shift = send_shift[rank] + send_count[rank];
-    send_count[rank] += 1;
-    send_list[shift] = vtx_tolerance[i];
-  }
-
-  MPI_Alltoallv(send_list, send_count, send_shift, MPI_DOUBLE,
-                recv_list, recv_count, recv_shift, MPI_DOUBLE, mpi_comm);
+  cs_lnum_t n_recv = cs_all_to_all_n_elts_dest(d);
 
   /* Define the global tolerance array */
 
@@ -535,41 +502,33 @@ _get_global_tolerance(cs_mesh_t            *mesh,
 
   first_vtx_gnum = block_size * local_rank + 1;
 
-  for (i = 0; i < recv_shift[n_ranks]; i++) {
+  for (i = 0; i < n_recv; i++) {
     vtx_id = recv_glist[i] - first_vtx_gnum;
     g_vtx_tolerance[vtx_id] = CS_MIN(g_vtx_tolerance[vtx_id], recv_list[i]);
   }
 
   /* Replace local vertex tolerance by the new computed global tolerance */
 
-  for (i = 0; i < recv_shift[n_ranks]; i++) {
+  for (i = 0; i < n_recv; i++) {
     vtx_id = recv_glist[i] - first_vtx_gnum;
     recv_list[i] = g_vtx_tolerance[vtx_id];
   }
 
-  MPI_Alltoallv(recv_list, recv_count, recv_shift, MPI_DOUBLE,
-                send_list, send_count, send_shift, MPI_DOUBLE, mpi_comm);
-
-  for (rank = 0; rank < n_ranks; rank++)
-    send_count[rank] = 0;
-
-  for (i = 0; i < n_vertices; i++) {
-    rank = (io_gnum[i] - 1)/block_size;
-    shift = send_shift[rank] + send_count[rank];
-    send_count[rank] += 1;
-    vtx_tolerance[i] = send_list[shift];
-  }
+  cs_all_to_all_copy_array(d,
+                           CS_REAL_TYPE,
+                           1,
+                           true, /* reverse */
+                           recv_list,
+                           vtx_tolerance);
 
   /* Free memory */
 
+  cs_all_to_all_destroy(&d);
+
   BFT_FREE(recv_glist);
   BFT_FREE(send_glist);
   BFT_FREE(send_list);
   BFT_FREE(recv_list);
-  BFT_FREE(recv_count);
-  BFT_FREE(send_count);
-  BFT_FREE(recv_shift);
-  BFT_FREE(send_shift);
   BFT_FREE(g_vtx_tolerance);
 }
 
@@ -690,10 +649,10 @@ _unwarping_mvt(cs_mesh_t            *mesh,
 #endif
 
   for (face_id = 0; face_id < mesh->n_b_faces; face_id++) {
-    start_id = mesh->b_face_vtx_idx[face_id] -1;
-    end_id = mesh->b_face_vtx_idx[face_id + 1] -1;
+    start_id = mesh->b_face_vtx_idx[face_id];
+    end_id = mesh->b_face_vtx_idx[face_id + 1];
     for (i = start_id; i < end_id; i++) {
-      vtx = mesh->b_face_vtx_lst[i] -1;
+      vtx = mesh->b_face_vtx_lst[i];
       lambda = 0.0;
       for (coord_id = 0; coord_id < 3; coord_id++)
         lambda +=  (mesh->vtx_coord[3*vtx + coord_id]
@@ -711,11 +670,11 @@ _unwarping_mvt(cs_mesh_t            *mesh,
 
 
   for (face_id = 0; face_id < mesh->n_i_faces; face_id++) {
-    if (mesh->i_face_cells[2*face_id] <= mesh->n_cells) {
-      start_id = mesh->i_face_vtx_idx[face_id] -1;
-      end_id = mesh->i_face_vtx_idx[face_id + 1] -1;
+    if (mesh->i_face_cells[face_id][0] < mesh->n_cells) {
+      start_id = mesh->i_face_vtx_idx[face_id];
+      end_id = mesh->i_face_vtx_idx[face_id + 1];
       for (i = start_id; i < end_id; i++) {
-        vtx = mesh->i_face_vtx_lst[i] -1;
+        vtx = mesh->i_face_vtx_lst[i];
         lambda = 0.0;
         for (coord_id = 0; coord_id < 3; coord_id++)
           lambda += (mesh->vtx_coord[3*vtx + coord_id]
@@ -771,11 +730,11 @@ _compute_vtx_normals(cs_mesh_t           *mesh,
     b_vtx_norm[i] = 0.;
 
   for (i = 0; i < mesh->n_b_faces; i++) {
-    for (j = mesh->b_face_vtx_idx[i] -1;
-         j < mesh->b_face_vtx_idx[i+1] -1;
+    for (j = mesh->b_face_vtx_idx[i];
+         j < mesh->b_face_vtx_idx[i+1];
          j++) {
       for (coord_id = 0; coord_id < 3; coord_id++) {
-        b_vtx_norm[(mesh->b_face_vtx_lst[j]-1)*3 + coord_id]
+        b_vtx_norm[(mesh->b_face_vtx_lst[j])*3 + coord_id]
           += b_face_norm[i*3 + coord_id];
       }
     }
@@ -805,18 +764,27 @@ _compute_vtx_normals(cs_mesh_t           *mesh,
   }
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Set fixed vertices flag based on feature angle criterion.
+/*----------------------------------------------------------------------------*/
+/*! \brief Set fixed vertices flag based on feature angle criterion.
+ *
+ * <a name="fix_by_feature"></a>
+ *
+ * Please refer to the
+ * <a href="../../theory.pdf#fixbyfeature"><b>specific treatment for boundary faces</b></a>
+ * section of the theory guide for more informations.
  *
  * parameters:
- *   mesh           <--  pointer to a cs_mesh_t structure
- *   feature_angle  <--  feature angle (bounded between 0 and 90 degrees)
- *   vtx_is_fixed   -->  array to define vertices mobility (1: fixed, 0: free)
- *----------------------------------------------------------------------------*/
+ *  \param[in]  mesh           pointer to a cs_mesh_t structure
+ *  \param[in]  feature_angle  feature angle (bounded between 0 and 90 degrees)
+ *  \param[out] vtx_is_fixed   array to define vertices mobility (1: fixed, 0: free)
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_mesh_smoother_fix_by_feature(cs_mesh_t   *mesh,
@@ -858,15 +826,15 @@ cs_mesh_smoother_fix_by_feature(cs_mesh_t   *mesh,
     _vtx_is_fixed[j] = 0;
 
   for (face = 0; face < mesh->n_b_faces; face++) {
-    for (j = mesh->b_face_vtx_idx[face] -1;
-         j < mesh->b_face_vtx_idx[face +1] -1;
+    for (j = mesh->b_face_vtx_idx[face];
+         j < mesh->b_face_vtx_idx[face +1];
          j++) {
       face_norm = &b_face_norm[face*3];
-      vtx_norm = &b_vtx_norm[(mesh->b_face_vtx_lst[j] -1)*3];
+      vtx_norm = &b_vtx_norm[(mesh->b_face_vtx_lst[j])*3];
 
       if (_DOT_PRODUCT_3D(face_norm, vtx_norm) < cos(feature_angle*_PI_/180.0)
           || feature_angle < DBL_MIN)
-        _vtx_is_fixed[mesh->b_face_vtx_lst[j] -1] += 1;
+        _vtx_is_fixed[mesh->b_face_vtx_lst[j]] += 1;
     }
   }
 
@@ -892,13 +860,20 @@ cs_mesh_smoother_fix_by_feature(cs_mesh_t   *mesh,
   BFT_FREE(_vtx_is_fixed);
 }
 
-/*----------------------------------------------------------------------------
- * Unwarping smoother.
+/*----------------------------------------------------------------------------*/
+/*! \brief Unwarping smoother.
+ *
+ * <a name="unwarp"></a>
+ *
+ * Please refer to the
+ * <a href="../../theory.pdf#unwarp"><b>unwarping algorithm</b></a>
+ * section of the theory guide for more informations.
  *
  * parameters:
- *   mesh         <-- pointer to a cs_mesh_t structure
- *   vtx_is_fixed --> array to define vertices mobility (1 : fixed, 0 : free)
- *----------------------------------------------------------------------------*/
+ *   \param[in]  mesh          pointer to a cs_mesh_t structure
+ *   \param[out] vtx_is_fixed  array to define vertices mobility (1 : fixed, 0 : free)
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_mesh_smoother_unwarp(cs_mesh_t  *mesh,
