@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -51,23 +51,25 @@ BEGIN_C_DECLS
  * Public function prototypes
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Insert thin walls into the mesh.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Insert thin walls into the mesh.
  *
  * This is done by transforming interior faces into boundary faces.
+ * The created faces share vertices.
  *
  * Note that the list of selected faces is sorted by this function.
  *
- * parameters:
- *   mesh           <->  pointer to mesh structure to modify
- *   face_list      <-> list of selected (interior) faces (1 to n)
- *   face_list_size <-> number of selected (interior) faces
- *----------------------------------------------------------------------------*/
+ * \param[in]       mesh           pointer to mesh structure to modify
+ * \param[in, out]  face_list      list of selected (interior) faces (0 to n-1)
+ * \param[in]       face_list_size number of selected (interior) faces
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_create_thinwall(cs_mesh_t  *mesh,
                    cs_lnum_t  *face_list,
-                   int         face_list_size);
+                   cs_lnum_t   face_list_size);
 
 /*----------------------------------------------------------------------------*/
 
