@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -64,6 +64,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -84,7 +86,6 @@ BEGIN_C_DECLS
  * Private function definitions
  *============================================================================*/
 
-
 #if defined(HAVE_MPI)
 
 /*----------------------------------------------------------------------------
@@ -148,12 +149,12 @@ _mesh_to_builder_face_vertices_g(const cs_mesh_t     *mesh,
   for (i = 0; i < n_i_faces; i++) {
     for (j = mesh->i_face_vtx_idx[i]; j < mesh->i_face_vtx_idx[i+1]; j++)
       face_vtx_g[k++]
-        = mesh->global_vtx_num[mesh->i_face_vtx_lst[j - 1] - 1];
+        = mesh->global_vtx_num[mesh->i_face_vtx_lst[j]];
   }
   for (i = 0; i < n_b_faces; i++) {
     for (j = mesh->b_face_vtx_idx[i]; j < mesh->b_face_vtx_idx[i+1]; j++)
       face_vtx_g[k++]
-        = mesh->global_vtx_num[mesh->b_face_vtx_lst[j - 1] - 1];
+        = mesh->global_vtx_num[mesh->b_face_vtx_lst[j]];
   }
 
   n_block_faces = (mb->face_bi.gnum_range[1] - mb->face_bi.gnum_range[0]);
@@ -371,11 +372,11 @@ _mesh_to_builder_g(cs_mesh_t          *mesh,
   BFT_MALLOC(face_cell_g, n_faces*2, cs_gnum_t);
 
   for (i = 0; i < n_i_faces; i++) {
-    face_cell_g[i*2] = cell_gnum[mesh->i_face_cells[i*2] - 1];
-    face_cell_g[i*2 + 1] = cell_gnum[mesh->i_face_cells[i*2 + 1] - 1];
+    face_cell_g[i*2]     = cell_gnum[mesh->i_face_cells[i][0]];
+    face_cell_g[i*2 + 1] = cell_gnum[mesh->i_face_cells[i][1]];
   }
   for (i = 0, j = n_i_faces; i < n_b_faces; i++, j++) {
-    face_cell_g[j*2] = cell_gnum[mesh->b_face_cells[i] - 1];
+    face_cell_g[j*2] = cell_gnum[mesh->b_face_cells[i]];
     face_cell_g[j*2 + 1] = 0;
   }
 
@@ -649,8 +650,8 @@ _mesh_to_builder_l(cs_mesh_t          *mesh,
 
     for (i = 0; i < n_i_faces; i++) {
       k = i_order[i];
-      cell_id_0 = mesh->i_face_cells[k*2] - 1;
-      cell_id_1 = mesh->i_face_cells[k*2 + 1] - 1;
+      cell_id_0 = mesh->i_face_cells[k][0];
+      cell_id_1 = mesh->i_face_cells[k][1];
       if (cell_id_0 < mesh->n_cells)
         mb->face_cells[i*2] = mesh->global_cell_num[cell_id_0];
       else
@@ -662,7 +663,7 @@ _mesh_to_builder_l(cs_mesh_t          *mesh,
     }
     for (i = 0, j = n_i_faces; i < n_b_faces; i++, j++) {
       k = b_order[i];
-      mb->face_cells[j*2] = mesh->global_cell_num[mesh->b_face_cells[k] - 1];
+      mb->face_cells[j*2] = mesh->global_cell_num[mesh->b_face_cells[k]];
       mb->face_cells[j*2 + 1] = 0;
     }
 
@@ -674,8 +675,8 @@ _mesh_to_builder_l(cs_mesh_t          *mesh,
 
     for (i = 0; i < n_i_faces; i++) {
       k = i_order[i];
-      cell_id_0 = mesh->i_face_cells[k*2] - 1;
-      cell_id_1 = mesh->i_face_cells[k*2 + 1] - 1;
+      cell_id_0 = mesh->i_face_cells[k][0];
+      cell_id_1 = mesh->i_face_cells[k][1];
       if (cell_id_0 < mesh->n_cells)
         mb->face_cells[i*2] = cell_id_0 + 1;
       else
@@ -687,7 +688,7 @@ _mesh_to_builder_l(cs_mesh_t          *mesh,
     }
     for (i = 0, j = n_i_faces; i < n_b_faces; i++, j++) {
       k = b_order[i];
-      mb->face_cells[j*2] = mesh->b_face_cells[k];
+      mb->face_cells[j*2] = mesh->b_face_cells[k] + 1;
       mb->face_cells[j*2 + 1] = 0;
     }
 
@@ -883,7 +884,7 @@ _mesh_to_builder_l(cs_mesh_t          *mesh,
       k = b_order[i];
       for (j = mesh->b_face_vtx_idx[k]; j < mesh->b_face_vtx_idx[k+1]; j++)
         mb->face_vertices[l++]
-          = mesh->global_vtx_num[mesh->b_face_vtx_lst[j - 1] - 1];
+          = mesh->global_vtx_num[mesh->b_face_vtx_lst[j]];
     }
   }
   else { /* if (mesh->global_vtx_num == NULL) */
@@ -891,12 +892,12 @@ _mesh_to_builder_l(cs_mesh_t          *mesh,
     for (i = 0; i < n_i_faces; i++) {
       k = i_order[i];
       for (j = mesh->i_face_vtx_idx[k]; j < mesh->i_face_vtx_idx[k+1]; j++)
-        mb->face_vertices[l++] = mesh->i_face_vtx_lst[j - 1];
+        mb->face_vertices[l++] = mesh->i_face_vtx_lst[j] + 1;
     }
     for (i = 0; i < n_b_faces; i++) {
       k = b_order[i];
       for (j = mesh->b_face_vtx_idx[k]; j < mesh->b_face_vtx_idx[k+1]; j++)
-        mb->face_vertices[l++] = mesh->b_face_vtx_lst[j - 1];
+        mb->face_vertices[l++] = mesh->b_face_vtx_lst[j] + 1;
     }
   }
 
@@ -1036,13 +1037,18 @@ _write_dimensions(cs_mesh_t          *mesh,
     cs_io_write_global("n_groups", 1, 0, 0, 1, lnum_type,
                        &(mesh->n_groups), pp_out);
 
+    cs_lnum_t *_group_idx;
+    BFT_MALLOC(_group_idx, mesh->n_groups + 1, cs_lnum_t);
+    for (i = 0; i < mesh->n_groups + 1; i++)
+      _group_idx[i] = mesh->group_idx[i] + 1;
     cs_io_write_global("group_name_index",
                        mesh->n_groups + 1, 0, 0, 1, lnum_type,
-                       mesh->group_idx, pp_out);
+                       _group_idx, pp_out);
+    BFT_FREE(_group_idx);
 
     cs_io_write_global("group_name",
-                       mesh->group_idx[mesh->n_groups] - 1, 0, 0, 1, CS_CHAR,
-                       mesh->group_lst, pp_out);
+                       mesh->group_idx[mesh->n_groups], 0, 0, 1, CS_CHAR,
+                       mesh->group, pp_out);
   }
 
   cs_io_write_global("group_class_properties",
@@ -1287,6 +1293,8 @@ _write_mesh_perio_data_l(int         perio_num,
                              pp_out);
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*=============================================================================
  * Public function definitions
  *============================================================================*/
@@ -1381,7 +1389,8 @@ cs_mesh_to_builder(cs_mesh_t          *mesh,
   if (transfer) {
     if (mesh->vtx_interfaces != NULL)
       cs_interface_set_destroy(&(mesh->vtx_interfaces));
-    mesh->halo = cs_halo_destroy(mesh->halo);
+    if (mesh->halo != NULL)
+      cs_halo_destroy(&(mesh->halo));
   }
 
   /* Periodicity data */
@@ -1421,6 +1430,66 @@ cs_mesh_to_builder(cs_mesh_t          *mesh,
 
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Transfer mesh partitioning info to mesh builder structure.
+ *
+ * \param[in]       mesh      pointer to mesh structure
+ * \param[in, out]  mb        pointer to mesh builder structure
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_mesh_to_builder_partition(const cs_mesh_t    *mesh,
+                             cs_mesh_builder_t  *mb)
+{
+#if defined(HAVE_MPI)
+
+  if (cs_glob_n_ranks > 1) {
+
+    const cs_datatype_t int_type
+      = (sizeof(int) == 8) ? CS_INT64 : CS_INT32;
+
+    /* Distribute cell group class info to blocks */
+    /*---------------------------------------------*/
+
+    mb->cell_bi = cs_block_dist_compute_sizes(cs_glob_rank_id,
+                                              cs_glob_n_ranks,
+                                              mb->min_rank_step,
+                                              0,
+                                              mesh->n_g_cells);
+
+    mb->have_cell_rank = true;
+    BFT_REALLOC(mb->cell_rank,
+                (mb->cell_bi.gnum_range[1] - mb->cell_bi.gnum_range[0]),
+                int);
+
+    int *cell_rank;
+    BFT_MALLOC(cell_rank, mesh->n_cells, int);
+    for (cs_lnum_t i = 0; i < mesh->n_cells; i++)
+      cell_rank[i] = cs_glob_rank_id;
+
+    cs_part_to_block_t *d
+      = cs_part_to_block_create_by_gnum(cs_glob_mpi_comm,
+                                        mb->cell_bi,
+                                        mesh->n_cells,
+                                        mesh->global_cell_num);
+
+    cs_part_to_block_copy_array(d,
+                                int_type,
+                                1,
+                                cell_rank,
+                                mb->cell_rank);
+
+    cs_part_to_block_destroy(&d);
+
+    BFT_FREE(cell_rank);
+
+  }
+
+#endif
+}
+
 /*----------------------------------------------------------------------------*/
 /*!
  * \brief Reconstruct periodic faces info from mesh to builder.
