@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -86,6 +86,18 @@ cs_mesh_to_builder(cs_mesh_t          *mesh,
                    bool                transfer,
                    cs_io_t            *pp_out);
 
+/*----------------------------------------------------------------------------
+ * Transfer mesh partitioning info to mesh builder structure.
+ *
+ * parameters:
+ *   mesh <-- pointer to mesh structure
+ *   mb   <-> pointer to mesh builder structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_mesh_to_builder_partition(const cs_mesh_t    *mesh,
+                             cs_mesh_builder_t  *mb);
+
 /*----------------------------------------------------------------------------
  * Reconstruct periodic faces info from mesh to builder.
  *
