@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -52,6 +52,7 @@
 #include "cs_mesh_quality.h"
 #include "cs_mesh_connect.h"
 #include "cs_order.h"
+#include "cs_parall.h"
 #include "cs_post.h"
 
 /*----------------------------------------------------------------------------
@@ -64,6 +65,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*============================================================================
  * Global variables
  *============================================================================*/
@@ -259,10 +262,10 @@ _match_halo_face_cut(const cs_mesh_t           *mesh,
                               - mesh->i_face_vtx_idx[face_id]) - 2;
           new_face_id = old_to_new[face_id];
           for (t_id = 0; t_id < n_face_triangles; t_id++) {
-            cs_lnum_t start_id = new_face_vtx_idx[new_face_id + t_id] - 1;
-            cs_lnum_t end_id = new_face_vtx_idx[new_face_id + t_id] - 1;
+            cs_lnum_t start_id = new_face_vtx_idx[new_face_id + t_id];
+            cs_lnum_t end_id = new_face_vtx_idx[new_face_id + t_id];
             for (v_id = start_id; v_id < end_id; v_id++)
-              send_buf[l++] = new_face_vtx_lst[v_id];
+              send_buf[l++] = new_face_vtx_lst[v_id] + 1;
           }
         }
       }
@@ -279,10 +282,10 @@ _match_halo_face_cut(const cs_mesh_t           *mesh,
                               - mesh->i_face_vtx_idx[face_id]) - 2;
           new_face_id = old_to_new[face_id];
           for (t_id = 0; t_id < n_face_triangles; t_id++) {
-            cs_lnum_t start_id = new_face_vtx_idx[new_face_id + t_id] - 1;
-            cs_lnum_t end_id = new_face_vtx_idx[new_face_id + t_id] - 1;
+            cs_lnum_t start_id = new_face_vtx_idx[new_face_id + t_id];
+            cs_lnum_t end_id = new_face_vtx_idx[new_face_id + t_id];
             for (v_id = start_id; v_id < end_id; v_id++)
-              send_buf[l++] = new_face_vtx_lst[v_id];
+              send_buf[l++] = new_face_vtx_lst[v_id] + 1;
           }
         }
       }
@@ -329,10 +332,10 @@ _match_halo_face_cut(const cs_mesh_t           *mesh,
                               - mesh->i_face_vtx_idx[face_id]) - 2;
           new_face_id = old_to_new[face_id];
           for (t_id = 0; t_id < n_face_triangles; t_id++) {
-            cs_lnum_t start_id = new_face_vtx_idx[new_face_id + t_id] - 1;
-            cs_lnum_t end_id = new_face_vtx_idx[new_face_id + t_id] - 1;
+            cs_lnum_t start_id = new_face_vtx_idx[new_face_id + t_id];
+            cs_lnum_t end_id = new_face_vtx_idx[new_face_id + t_id];
             for (v_id = start_id; v_id < end_id; v_id++)
-              new_face_vtx_lst[v_id] = recv_buf[l++];
+              new_face_vtx_lst[v_id] = recv_buf[l++] - 1;
           }
         }
       }
@@ -349,10 +352,10 @@ _match_halo_face_cut(const cs_mesh_t           *mesh,
                               - mesh->i_face_vtx_idx[face_id]) - 2;
           new_face_id = old_to_new[face_id];
           for (t_id = 0; t_id < n_face_triangles; t_id++) {
-            cs_lnum_t start_id = new_face_vtx_idx[new_face_id + t_id] - 1;
-            cs_lnum_t end_id = new_face_vtx_idx[new_face_id + t_id] - 1;
+            cs_lnum_t start_id = new_face_vtx_idx[new_face_id + t_id];
+            cs_lnum_t end_id = new_face_vtx_idx[new_face_id + t_id];
             for (v_id = start_id; v_id < end_id; v_id++)
-              new_face_vtx_lst[v_id] = recv_buf[l++];
+              new_face_vtx_lst[v_id] = recv_buf[l++] - 1;
           }
         }
       }
@@ -388,7 +391,8 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
 
   fvm_triangulate_state_t  *triangle_state = NULL;
   cs_lnum_t  *new_face_vtx_idx = NULL, *new_face_vtx_lst = NULL;
-  cs_lnum_t  *new_face_cells = NULL, *new_face_family = NULL;
+  cs_lnum_2_t  *new_face_cells = NULL;
+  int        *new_face_family = NULL;
   cs_lnum_t  *cut_face_lst = NULL;
   cs_lnum_t  *new_face_shift = NULL;
   cs_lnum_t  *n_sub_elt_lst = NULL;
@@ -461,8 +465,8 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
 
   for (face_id = 0; face_id < n_init_faces; face_id++) {
 
-    idx_start = mesh->i_face_vtx_idx[face_id] - 1;
-    idx_end = mesh->i_face_vtx_idx[face_id + 1] - 1;
+    idx_start = mesh->i_face_vtx_idx[face_id];
+    idx_end = mesh->i_face_vtx_idx[face_id + 1];
 
     n_face_vertices = idx_end - idx_start;
     n_max_face_vertices = CS_MAX(n_max_face_vertices, n_face_vertices);
@@ -486,22 +490,10 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
 
   *p_n_sub_elt_lst = n_sub_elt_lst;
 
-  /* If no faces are cut, we are sure at this stage that we do
-     need to communicate face cutting info on the interfaces for
-     this rank, and do not otherwise call collective MPI operations,
-     so we may return immediately. */
-
-  if (n_cut_faces == 0) {
-    BFT_FREE(cut_flag);
-    BFT_FREE(new_face_shift);
-    cs_interface_set_destroy(&face_ifs);
-    return;
-  }
-
   BFT_MALLOC(new_face_vtx_idx, n_new_faces + 1, cs_lnum_t);
   BFT_MALLOC(new_face_vtx_lst, connect_size, cs_lnum_t);
-  BFT_MALLOC(new_face_cells, 2*n_new_faces, cs_lnum_t);
-  BFT_MALLOC(new_face_family, n_new_faces, cs_lnum_t);
+  BFT_MALLOC(new_face_cells, n_new_faces, cs_lnum_2_t);
+  BFT_MALLOC(new_face_family, n_new_faces, int);
 
   BFT_MALLOC(cut_face_lst, n_cut_faces, cs_lnum_t);
 
@@ -512,20 +504,21 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
      the local vertex positions in the parent faces, not to the true
      vertex numbers, so as to be synchronizable across interfaces. */
 
-  new_face_vtx_idx[0] = 1;
+  new_face_vtx_idx[0] = 0;
   connect_size = 0;
   n_new_faces = 0;
   n_cut_faces = 0;
 
   for (face_id = 0; face_id < n_init_faces; face_id++) {
 
-    idx_start = mesh->i_face_vtx_idx[face_id] - 1;
-    idx_end = mesh->i_face_vtx_idx[face_id + 1] - 1;
+    idx_start = mesh->i_face_vtx_idx[face_id];
+    idx_end = mesh->i_face_vtx_idx[face_id + 1];
     n_face_vertices = idx_end - idx_start;
 
     if (cut_flag[face_id] != 0) {
 
       n_triangles = fvm_triangulate_polygon(dim,
+                                            0,
                                             n_face_vertices,
                                             mesh->vtx_coord,
                                             NULL,
@@ -545,8 +538,7 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
         /* Update "face -> cells" connectivity */
 
         for (j = 0; j < 2; j++)
-          new_face_cells[2*n_new_faces + j]
-            = mesh->i_face_cells[2*face_id + j];
+          new_face_cells[n_new_faces][j] = mesh->i_face_cells[face_id][j];
 
         /* Update family for each face */
 
@@ -557,8 +549,7 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
 
         n_new_faces++;
         connect_size += 3;
-        new_face_vtx_idx[n_new_faces]
-          = new_face_vtx_idx[n_new_faces-1] + 3;
+        new_face_vtx_idx[n_new_faces] = new_face_vtx_idx[n_new_faces-1] + 3;
 
       } /* End of loop on triangles */
 
@@ -568,7 +559,7 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
       /* Update "face -> cells" connectivity */
 
       for (j = 0; j < 2; j++)
-        new_face_cells[2*n_new_faces + j] = mesh->i_face_cells[2*face_id + j];
+        new_face_cells[n_new_faces][j] = mesh->i_face_cells[face_id][j];
 
       /* Update family for each face */
 
@@ -621,22 +612,22 @@ _cut_warped_i_faces_halo(cs_mesh_t   *mesh,
 
     if (cut_flag[face_id] != 0) {
 
-      idx_start = mesh->i_face_vtx_idx[face_id] - 1;
-      idx_end = mesh->i_face_vtx_idx[face_id + 1] - 1;
+      idx_start = mesh->i_face_vtx_idx[face_id];
+      idx_end = mesh->i_face_vtx_idx[face_id + 1];
 
       n_face_vertices = idx_end - idx_start;
       n_triangles = n_face_vertices - 2;
 
       face_shift = new_face_shift[face_id];
 
-      old_face_idx = mesh->i_face_vtx_idx[face_id] - 1;
+      old_face_idx = mesh->i_face_vtx_idx[face_id];
 
       for (i = 0; i < n_triangles; i++) {
 
-        for (j = new_face_vtx_idx[face_shift] - 1;
-             j < new_face_vtx_idx[face_shift+1] - 1; j++) {
+        for (j = new_face_vtx_idx[face_shift];
+             j < new_face_vtx_idx[face_shift+1]; j++) {
 
-          cs_lnum_t v_id = new_face_vtx_lst[j] - 1;
+          cs_lnum_t v_id = new_face_vtx_lst[j];
           new_face_vtx_lst[j] = mesh->i_face_vtx_lst[old_face_idx + v_id];
         }
 
@@ -732,8 +723,8 @@ _cut_warped_faces(cs_mesh_t      *mesh,
 
   for (face_id = 0; face_id < n_init_faces; face_id++) {
 
-    idx_start = (*p_face_vtx_idx)[face_id] - 1;
-    idx_end = (*p_face_vtx_idx)[face_id + 1] - 1;
+    idx_start = (*p_face_vtx_idx)[face_id];
+    idx_end = (*p_face_vtx_idx)[face_id + 1];
 
     n_face_vertices = idx_end - idx_start;
     n_max_face_vertices = CS_MAX(n_max_face_vertices, n_face_vertices);
@@ -759,8 +750,10 @@ _cut_warped_faces(cs_mesh_t      *mesh,
 
   *p_n_sub_elt_lst = n_sub_elt_lst;
 
-  if (n_cut_faces == 0)
+  if (n_cut_faces == 0) {
+    BFT_FREE(cut_flag);
     return;
+  }
 
   BFT_MALLOC(new_face_vtx_idx, n_new_faces + 1, cs_lnum_t);
   BFT_MALLOC(new_face_vtx_lst, connect_size, cs_lnum_t);
@@ -771,22 +764,23 @@ _cut_warped_faces(cs_mesh_t      *mesh,
 
   triangle_state = fvm_triangulate_state_create(n_max_face_vertices);
 
-  /* Second loop : define the new connectivity after triangulation */
+  /* Second loop: define the new connectivity after triangulation */
 
-  new_face_vtx_idx[0] = 1;
+  new_face_vtx_idx[0] = 0;
   connect_size = 0;
   n_new_faces = 0;
   n_cut_faces = 0;
 
   for (face_id = 0; face_id < n_init_faces; face_id++) {
 
-    idx_start = (*p_face_vtx_idx)[face_id] - 1;
-    idx_end = (*p_face_vtx_idx)[face_id + 1] - 1;
+    idx_start = (*p_face_vtx_idx)[face_id];
+    idx_end = (*p_face_vtx_idx)[face_id + 1];
     n_face_vertices = idx_end - idx_start;
 
     if (cut_flag[face_id] != 0) {
 
       n_triangles = fvm_triangulate_polygon(dim,
+                                            0,
                                             n_face_vertices,
                                             mesh->vtx_coord,
                                             NULL,
@@ -806,15 +800,14 @@ _cut_warped_faces(cs_mesh_t      *mesh,
         /* Update "face -> cells" connectivity */
 
         for (j = 0; j < stride; j++)
-          new_face_cells[stride*n_new_faces + j]
-            = (*p_face_cells)[stride*face_id + j];
+          new_face_cells[stride*n_new_faces + j] = (*p_face_cells)[stride*face_id + j];
 
         /* Update family for each face */
 
         new_face_family[n_new_faces] = (*p_face_family)[face_id];
 
         /* Update "face -> vertices" connectivity index
-           (list has alread been defined by fvm_triangulate_polygon) */
+           (list has already been defined by fvm_triangulate_polygon) */
 
         n_new_faces++;
         connect_size += 3;
@@ -828,8 +821,8 @@ _cut_warped_faces(cs_mesh_t      *mesh,
       /* Update "face -> cells" connectivity */
 
       for (j = 0; j < stride; j++)
-        new_face_cells[stride*n_new_faces + j] =
-          (*p_face_cells)[stride*face_id + j];
+        new_face_cells[stride*n_new_faces + j]
+          = (*p_face_cells)[stride*face_id + j];
 
       /* Update family for each faces */
 
@@ -1056,6 +1049,8 @@ _post_after_cutting(cs_int_t       n_i_cut_faces,
   fvm_mesh = fvm_nodal_destroy(fvm_mesh);
 }
 
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
+
 /*============================================================================
  * Public function definitions for Fortran API
  *============================================================================*/
@@ -1212,7 +1207,7 @@ cs_mesh_warping_cut_faces(cs_mesh_t  *mesh,
                       &n_i_sub_elt_lst,
                       &mesh->n_i_faces,
                       &mesh->i_face_vtx_connect_size,
-                      &mesh->i_face_cells,
+                      (cs_lnum_t **)(&mesh->i_face_cells),
                       &mesh->i_face_family,
                       &mesh->i_face_vtx_idx,
                       &mesh->i_face_vtx_lst);
