@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -55,9 +55,16 @@ module ppcpfu
   ! prise en compte H2  , H2S , SO2 , HCN , NH3
   integer, save ::         ihy , ih2s , iso2  , ihcn , inh3
 
-  ! Equation sur YCO2
+  ! kinetic model for CO <=> CO2
+  ! 0  unused (maximal conversion in turbulent model)
+  ! 1  transport of CO2 mass fraction
+  ! 2  transport of CO mass fraction
 
-  integer, save ::         ieqco2 , iyco2
+  integer, save ::         ieqco2
+
+  ! scalar id of CO2 mass fraction
+
+  integer, save ::         iyco2
 
   ! Combustion heterogene avec le  CO2
 
@@ -102,7 +109,7 @@ module ppcpfu
   integer, save ::         iynh3
   !
   !
-  !   Propce supplementaires :
+  !   Propriétés supplementaires :
 
   !         Conversion HCN en NO       : EXP(-E1/RT)
   !         Conversion HCN en NO       : EXP(-E2/RT)
@@ -110,7 +117,7 @@ module ppcpfu
 
   integer, save ::         ighcn1 , ighcn2 , ignoth
   !
-  !   Propce supplementaires :
+  !   Propriétés supplementaires :
 
   !         Conversion NH3 en NO       : EXP(-E4/RT)
   !         Conversion NH3 en NO       : EXP(-E5/RT)
@@ -171,6 +178,9 @@ module ppcpfu
 
   integer, save :: inmoxy(nozppm)
 
+  ! The gas enthalpy is transported (x1 h1 to be correct)
+  integer, save :: ihgas
+
   !=============================================================================
 
 end module ppcpfu
