@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,7 @@ subroutine ppiniv &
 !================
 
  ( nvar   , nscal  ,                                              &
-   dt     , rtp    , propce )
+   dt     )
 
 !===============================================================================
 ! FONCTION :
@@ -58,9 +58,6 @@ subroutine ppiniv &
 ! nvar             ! i  ! <-- ! total number of variables                      !
 ! nscal            ! i  ! <-- ! total number of scalars                        !
 ! dt(ncelet)       ! tr ! <-- ! valeur du pas de temps                         !
-! rtp              ! tr ! <-- ! variables de calcul au centre des              !
-! (ncelet,*)       !    !     !    cellules                                    !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -93,7 +90,7 @@ implicit none
 
 integer          nvar   , nscal
 
-double precision dt(ncelet), rtp(ncelet,nflown:nvar), propce(ncelet,*)
+double precision dt(ncelet)
 
 ! Local variables
 
@@ -115,55 +112,40 @@ double precision dt(ncelet), rtp(ncelet,nflown:nvar), propce(ncelet,*)
 
  if ( ippmod(icod3p).ge.0 ) then
   call d3pini                                                     &
-  !==========
  ( nvar   , nscal  ,                                              &
-   dt     , rtp    , propce )
+   dt     )
   endif
 
 ! ---> Combustion gaz
 !      Flamme de premelange : modele EBU
 
  if (ippmod(icoebu).ge.0) then
-  call ebuini(nvar, nscal, dt, rtp, propce)
-  !==========
+  call ebuini(nvar, nscal, dt)
 endif
 
 ! ---> Combustion gaz
 !      Flamme de premelange : modele LWC
 
 if (ippmod(icolwc).ge.0) then
-  call lwcini(nvar, nscal, dt, rtp, propce)
-  !==========
+  call lwcini(nvar, nscal, dt)
 endif
 
 ! ---> Combustion charbon pulverise
 
 if (ippmod(iccoal).ge.0) then
-  call cs_coal_varini(nvar, nscal, dt, rtp, propce)
-  !==================
+  call cs_coal_varini(nvar, nscal, dt)
 endif
 
 ! ---> Combustion charbon pulverise couples Lagrangien
 
 if (ippmod(icpl3c).ge.0) then
-  call cplini(rtp)
-  !==========
+  call cplini
 endif
 
 ! ---> Combustion fuel
 
 if  (ippmod(icfuel).ge.0) then
-  call cs_fuel_varini(nvar, nscal, dt, rtp, propce)
-  !==================
-endif
-
-! ---> Compressible
-
-if ( ippmod(icompf).ge.0 ) then
-  call cfiniv                                                     &
-  !==========
- ( nvar   , nscal  ,                                              &
-   dt     , rtp    , propce )
+  call cs_fuel_varini(nvar, nscal, dt)
 endif
 
 ! ---> Version electrique
@@ -171,40 +153,59 @@ endif
 !      Conduction ionique
 
 if ( ippmod(ieljou).ge.1 .or.                                     &
-     ippmod(ielarc).ge.1 .or.                                     &
-     ippmod(ielion).ge.1       ) then
+     ippmod(ielarc).ge.1       ) then
 
-  call eliniv(nvar, nscal, dt, rtp, propce)
-  !==========
+  call eliniv(isuite)
+
+  ! give back the hand to the user... in fortran.
+  call cs_user_f_initialization(nvar, nscal, dt)
 
 endif
 
 ! ---> Ecoulements atmospheriques
 
-if ( ippmod(iatmos).ge.0 ) then
+if (ippmod(iatmos).ge.0) then
 
   call atiniv                                                     &
-  !==========
  ( nvar   , nscal  ,                                              &
-   dt     , rtp    , propce )
+   dt     )
 
 endif
 
-! ---> Ecoulements aerorefrigerants
+! ---> Cooling towers
 
-if ( ippmod(iaeros).ge.0 ) then
+if (ippmod(iaeros).ge.0) then
 
-  call ctiniv(nvar, nscal, dt, rtp, propce)
-  !==========
+  call ctiniv(nvar, nscal, dt)
+
+endif
+
+! Gas mixture modelling in presence of noncondensable gases and
+! condensable gas as stream.
+if (ippmod(igmix).ge.0) then
+
+  call cs_gas_mix_initialization &
+  ( nvar   , nscal  ,                                            &
+    dt     )
+
+endif
+
+! Compressible
+! Has to be called AFTER the gas mix initialization because the
+! mixture composition is taken into account in the thermodynamic
+! law, if gas mix specific physics is enabled.
+if (ippmod(icompf).ge.0) then
+
+  call cfiniv(nvar, nscal, dt)
 
 endif
 
 !----
-! FORMATS
+! Formats
 !----
 
 !----
-! FIN
+! End
 !----
 
 return
