@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -24,7 +24,7 @@ subroutine pptbht &
 !================
 
  ( ncoel  ,                                                       &
-   nomcoe , ehcoel , cpcoel , wmolce )
+   nomcel , ehcoel , cpcoel , wmolce )
 
 !===============================================================================
 ! FONCTION :
@@ -41,10 +41,11 @@ subroutine pptbht &
 !    nom        !type!mode !                   role                         !
 !_______________!____!_____!________________________________________________!
 ! ncoel         ! e  ! <-- ! nombre de const. elem.                         !
-! nomcoe(ngazem)! a  ! <-- ! nom des constituants elementaires              !
+! nomcel(ngazem)! a  ! <-- ! nom des constituants elementaires              !
 ! ehcoel        ! tr !  <- ! enthalpie pour chaque constituant              !
 ! (ngazem,npot) !    !     !                elementaire                     !
-! cpcoel(ngazem)! tr !  <- ! cp pour chaque constituant                     !
+! cpcoel        ! tr !  <- ! cp pour chaque constituant                     !
+! (ngazem,npot) !    !     !                elementaire                     !
 ! wmolce(ngazem)! tr !  <- ! masse molaire de chaque constituant            !
 !_______________!____!_____!________________________________________________!
 
@@ -75,16 +76,16 @@ implicit none
 
 integer          ncoel
 
-character*12     nomcoe(ngazem)
+character(len=12) :: nomcel(ngazem)
 
-double precision ehcoel(ngazem,npot) , cpcoel(ngazem)
+double precision ehcoel(ngazem,npot) , cpcoel(ngazem,npot)
 double precision wmolce (ngazem)
 
 ! Local variables
 
-character*256    pathdatadir
-character*40     dummy
-character*12     nomesp
+character(len=256) :: pathdatadir
+character(len=40) :: dummy
+character(len=12) :: nomesp
 
 integer          ind , iches , indtp , inicff , injcff, impjnf
 integer          ne   , nt  , nc , iok
@@ -118,8 +119,8 @@ do ne = 1 , ngazem
 enddo
 
 do ne = 1 , ncoel
-  cpcoel(ne)= 0.d0
   do nt = 1, npo
+    cpcoel(ne,nt)= 0.d0
     ehcoel(ne,nt)= 0.d0
   enddo
 enddo
@@ -154,7 +155,7 @@ read (impjnf,*) (wcoeff(2,injcff), injcff=4,7)
 !  l'espece consideree fait partie de l'exemple
 
 do ne = 1, ncoel
-  if ( nomcoe(ne).eq.nomesp ) then
+  if ( nomcel(ne).eq.nomesp ) then
     icoeff(ne) = 1
      do inicff = 1, 2
        do injcff = 1, 7
@@ -179,7 +180,7 @@ iok = 0
 do ne = 1, ncoel
   if(icoeff(ne).eq.0) then
     iok = iok + 1
-    write(nfecra,1000) nomcoe(ne)
+    write(nfecra,1000) nomcel(ne)
   endif
 enddo
 if(iok.ne.0) then
@@ -224,8 +225,7 @@ endif
 
 do nt = 1,npo
 
-! Determination du jeu de coefficients utilises
-
+  ! Determination du jeu de coefficients utilises
   if (th(nt) .gt. tlim(2)) then
     ind = 1
   else
@@ -234,24 +234,22 @@ do nt = 1,npo
 
   do ne = 1, ncoel
     ehcoel(ne,nt)  = coeff(ne,ind,6) + coeff(ne,ind,1) * th(nt)
-    cpcoel(ne)     =                   coeff(ne,ind,1)
+    cpcoel(ne,nt)  =                   coeff(ne,ind,1)
     cth = th(nt)
     ctc = 1.d0
 
-! Dans la table de Janaf, les COEFF sont adimensionnels (CP/R,H/R)
-
+    ! Dans la table de Janaf, les COEFF sont adimensionnels (CP/R,H/R)
     do nc = 2, 5
       cth = cth * th(nt)
       ctc = ctc * th(nt)
       ehcoel(ne,nt) = ehcoel(ne,nt)                               &
                     + coeff(ne,ind,nc) * cth / dble(nc)
-      cpcoel(ne) = cpcoel(ne) + coeff(ne,ind,nc) * ctc
+      cpcoel(ne,nt) = cpcoel(ne,nt) + coeff(ne,ind,nc) * ctc
     enddo
 
-!      Calcul du CP et du H pour chaque espece
-
-    ehcoel(ne,nt) = ehcoel(ne,nt) * rr / wmolce(ne)
-    cpcoel(ne)    = cpcoel(ne)    * rr / wmolce(ne)
+    ! Calcul du CP et du H pour chaque espece
+    ehcoel(ne,nt) = ehcoel(ne,nt) * cs_physical_constants_r / wmolce(ne)
+    cpcoel(ne,nt) = cpcoel(ne,nt) * cs_physical_constants_r / wmolce(ne)
   enddo
 
 enddo
