@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -23,8 +23,7 @@
 subroutine clipsa &
 !================
 
- ( ncelet , ncel   , nvar   ,                                     &
-   rtp    )
+ ( ncel )
 
 !===============================================================================
 ! Purpose:
@@ -37,12 +36,7 @@ subroutine clipsa &
 !__________________.____._____.________________________________________________.
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
-! nvar             ! e  ! <-- ! nombre de variables                            !
-! ncelet           ! i  ! <-- ! number of extended (real + ghost) cells        !
 ! ncel             ! i  ! <-- ! number of cells                                !
-!(ncelet,*         !    !     !                                                !
-! rtp              ! tr ! <-- ! tableaux des variables au pdt courant          !
-! (ncelet     )    !    !     !                                                !
 !__________________!____!_____!________________________________________________!
 
 !     Type: i (integer), r (real), s (string), a (array), l (logical),
@@ -59,6 +53,7 @@ use numvar
 use cstphy
 use cstnum
 use entsor
+use field
 use optcal
 use parall
 use cs_c_bindings
@@ -69,47 +64,69 @@ implicit none
 
 ! Arguments
 
-integer          nvar, ncelet, ncel
-double precision rtp(ncelet,nflown:nvar)
+integer          ncel
 
 ! Local variables
 
-integer          iclpnu,iel
+integer          iclpnu(1),iel, iclmx(1)
+integer          kclipp, clip_nusa_id
 double precision xnu, var, epz2
 double precision vmin(1), vmax(1)
 
+double precision, dimension(:), pointer :: cvar_nusa
+double precision, dimension(:), pointer :: cpro_nusa_clipped
+
 !===============================================================================
 
+call field_get_val_s(ivarfl(inusa), cvar_nusa)
+
+call field_get_key_id("clipping_id", kclipp)
+
+! Postprocess clippings?
+call field_get_key_int(ivarfl(inusa), kclipp, clip_nusa_id)
+if (clip_nusa_id.ge.0) then
+  call field_get_val_s(clip_nusa_id, cpro_nusa_clipped)
+endif
+
 ! Une petite valeur pour eviter des valeurs exactement nulles.
 
 epz2 = epzero**2
-
+iclmx(1) = 0
 !===============================================================================
-! ---> Stockage Min et Max pour listing
+! ---> Stockage Min et Max pour log
 !===============================================================================
 
 vmin(1) =  grand
 vmax(1) = -grand
 do iel = 1, ncel
-  var = rtp(iel,inusa)
+  var = cvar_nusa(iel)
   vmin(1) = min(vmin(1),var)
   vmax(1) = max(vmax(1),var)
 enddo
 
+if (clip_nusa_id.ge.0) then
+  do iel = 1, ncel
+    cpro_nusa_clipped(iel) = 0.d0
+  enddo
+endif
+
 !===============================================================================
 ! ---> Clipping "standard" NUSA>0
 !===============================================================================
 
-iclpnu = 0
+iclpnu(1) = 0
 do iel = 1, ncel
-  xnu = rtp(iel,inusa)
+  xnu = cvar_nusa(iel)
   if (xnu.lt.0.d0) then
-    iclpnu = iclpnu + 1
-    rtp(iel,inusa) = 0.d0
+    if (clip_nusa_id.ge.0) then
+      cpro_nusa_clipped(iel) = - xnu
+    endif
+    iclpnu(1) = iclpnu(1) + 1
+    cvar_nusa(iel) = 0.d0
   endif
 enddo
 
-call log_iteration_clipping_field(ivarfl(inusa), iclpnu, 0, vmin, vmax)
+call log_iteration_clipping_field(ivarfl(inusa), iclpnu(1), 0, vmin, vmax,iclpnu(1), iclmx(1))
 
 return
 
