@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,39 +20,30 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine clpalp &
-!================
-
- ( ncelet , ncel   , nvar   , rtp    )
-
 !===============================================================================
-! FONCTION :
+! Function :
 ! ----------
+!> \file clpalp.f90
+!> \brief Clipping of alpha in the framwork of the Rij-EBRSM model.
+!>        Also called for alpha of scalars for EB-DFM.
 
-! Clipping of Alpha
+!-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
-!ARGU                             ARGUMENTS
-!__________________.____._____.________________________________________________.
-!    nom           !type!mode !                   role                         !
-!__________________!____!_____!________________________________________________!
-! ncelet           ! e  ! <-- ! nombre d'elements halo compris                 !
-! ncel             ! e  ! <-- ! nombre de cellules                             !
-! nvar             ! e  ! <-- ! nombre de variables                            !
-! iclip            ! e  ! <-- ! indicateur = 1 on n'utilise pas rtpa           !
-!                  !    !     !  (inivar)                                      !
-!                  !    !     !            sinon on peut (turrij)              !
-! rtp              ! tr ! <-- ! tableaux des variables au pdt courant          !
-! (ncelet,nvar)    !    !     !                                                !
-!__________________!____!_____!________________________________________________!
+!______________________________________________________________________________.
+!  mode           nom           role
+!______________________________________________________________________________!
+!> \param[in]     f_id          field id of alpha variable
+!> \param[in]     ncelet        number of extended (real + ghost) cells
+!> \param[in]     ncel          number of cells
+!> \param[in]     alpha_min     minimum acceptable value for alpha
+!______________________________________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
 
-!-------------------------------------------------------------------------------
+subroutine clpalp &
+ ( f_id, ncelet , ncel  , alpha_min)
+
 !===============================================================================
 
 !===============================================================================
@@ -65,6 +56,7 @@ use numvar
 use cstnum
 use parall
 use cs_c_bindings
+use field
 
 !===============================================================================
 
@@ -72,47 +64,67 @@ implicit none
 
 ! Arguments
 
-integer          nvar, ncelet, ncel
-double precision rtp(ncelet,nflown:nvar)
+integer          f_id, ncelet, ncel
+double precision alpha_min(ncelet)
 
 ! VARIABLES LOCALES
 
-integer          iel, ivar, ipp
-integer          iclpmn, iclpmx
+integer          iel
+integer          iclpmn(1), iclpmx(1)
+integer          kclipp, clip_a_id
 double precision vmin(1), vmax(1), var
 
-!===============================================================================
+double precision, dimension(:), pointer :: cvar_al
+double precision, dimension(:), pointer :: cpro_a_clipped
 
-!===============================================================================
-!  ---> Stockage Min et Max pour listing
 !===============================================================================
 
-ivar = ial
-ipp = ipprtp(ivar)
+call field_get_val_s(f_id, cvar_al)
+
+call field_get_key_id("clipping_id", kclipp)
+
+! Postprocess clippings?
+call field_get_key_int(f_id, kclipp, clip_a_id)
+if (clip_a_id.ge.0) then
+  call field_get_val_s(clip_a_id, cpro_a_clipped)
+endif
+
+!===============================================================================
+!  ---> Stockage Min et Max pour log
+!===============================================================================
 
 vmin(1) =  grand
 vmax(1) = -grand
 do iel = 1, ncel
-  var = rtp(iel,ivar)
+  var = cvar_al(iel)
   vmin(1) = min(vmin(1),var)
   vmax(1) = max(vmax(1),var)
 enddo
 
+do iel = 1, ncel
+  if (clip_a_id.ge.0) &
+    cpro_a_clipped(iel) = 0.d0
+enddo
+
 ! ---> Clipping (modif pour eviter les valeurs exactement nulles)
 
-iclpmn = 0
-iclpmx = 0
+iclpmn(1) = 0
+iclpmx(1) = 0
 do iel = 1, ncel
-  if (rtp(iel,ial).lt.0.d0) then
-    iclpmn = iclpmn + 1
-    rtp(iel,ial) = 0.d0
-  elseif(rtp(iel,ial).gt.1.d0) then
-    iclpmx = iclpmx + 1
-    rtp(iel,ial) = 1.d0
+  if (cvar_al(iel).lt.alpha_min(iel)) then
+    if (clip_a_id.ge.0) &
+      cpro_a_clipped(iel) = alpha_min(iel) - cvar_al(iel)
+    iclpmn(1) = iclpmn(1) + 1
+    cvar_al(iel) = alpha_min(iel)
+  elseif(cvar_al(iel).gt.1.d0) then
+    if (clip_a_id.ge.0) &
+      cpro_a_clipped(iel) = cvar_al(iel) - 1.d0
+    iclpmx(1) = iclpmx(1) + 1
+    cvar_al(iel) = 1.d0
   endif
 enddo
 
-call log_iteration_clipping_field(ivarfl(ial), iclpmn, iclpmx, vmin, vmax)
+call log_iteration_clipping_field(f_id, iclpmn(1), iclpmx(1), vmin, vmax,iclpmn(1), iclpmx(1))
 
 return
 
