@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,37 +20,26 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine clpv2f &
-!================
+!> \file clpv2f.f90
+!> \brief Clipping of \f$ v^2\f$ and \f$ \phi \f$ for the Bl v2/k
+!> turbulence model (no clipping on \f$ f \f$).
+!>
+!------------------------------------------------------------------------------
 
- ( ncelet , ncel   , nvar   ,                                     &
-   iwaphi ,                                                       &
-   rtp    )
-
-!===============================================================================
-! FONCTION :
-! ----------
-
-! CLIPPING DE PHI EN V2F (PAS DE CLIPPING SUR F_BARRE)
-
-!-------------------------------------------------------------------------------
+!------------------------------------------------------------------------------
 ! Arguments
-!ARGU                             ARGUMENTS
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! ncelet           ! i  ! <-- ! number of extended (real + ghost) cells        !
-! ncel             ! e  ! <-- ! nombre de cellules                             !
-! nvar             ! e  ! <-- ! nombre de variables                            !
-! iwaphi           ! e  ! <-- ! niveau d'impression                            !
-! rtp              ! tr ! <-- ! tableaux des variables au pdt courant          !
-! (ncelet,nvar)    !    !     !                                                !
-!__________________!____!_____!________________________________________________!
+!------------------------------------------------------------------------------
+!   mode          name          role
+!------------------------------------------------------------------------------
+!> \param[in]     ncel          number of cells
+!> \param[in]     iwaphi        verbosity level
+!______________________________________________________________________________
+
+
+subroutine clpv2f &
+ ( ncel   ,                                                       &
+   iwaphi )
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
 !===============================================================================
 
 !===============================================================================
@@ -64,6 +53,7 @@ use cstnum
 use parall
 use optcal
 use cs_c_bindings
+use field
 
 !===============================================================================
 
@@ -71,18 +61,41 @@ implicit none
 
 ! Arguments
 
-integer          nvar, ncelet, ncel
+integer          ncel
 integer          iwaphi
-double precision rtp(ncelet,nflown:nvar)
 
 ! Local variables
 
-integer          iel, ipp
-integer          nclpmx, nclpmn
+integer          iel
+integer          kclipp, clip_a_id, clip_phi_id
+integer          nclpmx(1), nclpmn(1)
 double precision xphi, xal, vmin(1), vmax(1), var
 
+double precision, dimension(:), pointer :: cvar_al, cvar_phi
+double precision, dimension(:), pointer :: cpro_phi_clipped
+double precision, dimension(:), pointer :: cpro_a_clipped
+
 !===============================================================================
 
+call field_get_val_s(ivarfl(iphi), cvar_phi)
+
+call field_get_key_id("clipping_id", kclipp)
+
+! Postprocess clippings?
+call field_get_key_int(ivarfl(iphi), kclipp, clip_phi_id)
+if (clip_phi_id.ge.0) then
+  call field_get_val_s(clip_phi_id, cpro_phi_clipped)
+endif
+
+clip_a_id = -1
+if (iturb.eq.51) then
+  call field_get_val_s(ivarfl(ial), cvar_al)
+  call field_get_key_int(ivarfl(ial), kclipp, clip_a_id)
+  if (clip_a_id.ge.0) then
+    call field_get_val_s(clip_a_id, cpro_a_clipped)
+  endif
+endif
+
 !===============================================================================
 !  1. Pour le phi-fbar et BL-v2/k model, reperage des valeurs de phi
 !     superieures a 2 et clipping de phi en valeur absolue
@@ -90,47 +103,54 @@ double precision xphi, xal, vmin(1), vmax(1), var
 !===============================================================================
 
 !===============================================================================
-!     1.a Stockage Min et Max pour listing
+!     1.a Stockage Min et Max pour log
 !===============================================================================
 
-ipp = ipprtp(iphi)
-
 vmin(1) =  grand
 vmax(1) = -grand
 do iel = 1, ncel
-  var = rtp(iel,iphi)
+  var = cvar_phi(iel)
   vmin(1) = min(vmin(1),var)
   vmax(1) = max(vmax(1),var)
 enddo
 
+do iel = 1, ncel
+  if (clip_phi_id.ge.0) &
+    cpro_phi_clipped(iel) = 0.d0
+  if (clip_a_id.ge.0) &
+    cpro_a_clipped(iel) = 0.d0
+enddo
+
+
 !==============================================================================
 !     1.b Reperage des valeurs superieures a 2, pour affichage seulement
 !==============================================================================
 
 if (iwaphi.ge.2) then
-  nclpmx = 0
+  nclpmx(1) = 0
   do iel = 1, ncel
-    if (rtp(iel,iphi).gt.2.d0) nclpmx = nclpmx+1
+    if (cvar_phi(iel).gt.2.d0) nclpmx(1) = nclpmx(1)+1
   enddo
-  if(irangp.ge.0) call parcpt(nclpmx)
-                  !==========
-  if (nclpmx.gt.0) write(nfecra,1000) nclpmx
+  if (irangp.ge.0) call parcpt(nclpmx(1))
+  if (nclpmx(1).gt.0) write(nfecra,1000) nclpmx(1)
 endif
 
 !==============================================================================
 !     1.c Clipping en valeur absolue pour les valeurs negatives
 !==============================================================================
 
-nclpmn = 0
+nclpmn(1) = 0
 do iel = 1, ncel
-  xphi = rtp(iel,iphi)
+  xphi = cvar_phi(iel)
   if (xphi.lt.0.d0) then
-    rtp(iel,iphi) = -xphi
-    nclpmn = nclpmn + 1
+    if (clip_phi_id.ge.0) &
+      cpro_phi_clipped(iel) = -xphi
+    cvar_phi(iel) = -xphi
+    nclpmn(1) = nclpmn(1) + 1
   endif
 enddo
 
-call log_iteration_clipping_field(ivarfl(iphi), nclpmn, 0, vmin, vmax)
+call log_iteration_clipping_field(ivarfl(iphi), nclpmn(1), 0, vmin, vmax,nclpmn(1), nclpmx(1))
 
 !===============================================================================
 !  2. Pour le BL-v2/k model, clipping de alpha a 0 pour les valeurs negatives
@@ -140,15 +160,13 @@ call log_iteration_clipping_field(ivarfl(iphi), nclpmn, 0, vmin, vmax)
 if (iturb.eq.51) then
 
 !===============================================================================
-!     2.a Stockage Min et Max pour listing
+!     2.a Stockage Min et Max pour log
 !===============================================================================
 
-  ipp = ipprtp(ial)
-
   vmin(1) =  grand
   vmax(1) = -grand
   do iel = 1, ncel
-    var = rtp(iel,ial)
+    var = cvar_al(iel)
     vmin(1) = min(vmin(1),var)
     vmax(1) = max(vmax(1),var)
   enddo
@@ -158,21 +176,25 @@ if (iturb.eq.51) then
 !         superieures a 1
 !==============================================================================
 
-  nclpmn = 0
-  nclpmx = 0
+  nclpmn(1) = 0
+  nclpmx(1) = 0
   do iel = 1, ncel
-    xal = rtp(iel,ial)
+    xal = cvar_al(iel)
     if (xal.lt.0.d0) then
-      rtp(iel,ial) = 0.d0
-      nclpmn = nclpmn + 1
+      if (clip_a_id.ge.0) &
+        cpro_a_clipped(iel) = -xal
+      cvar_al(iel) = 0.d0
+      nclpmn(1) = nclpmn(1) + 1
     endif
     if (xal.gt.1.d0) then
-      rtp(iel,ial) = 1.d0
-      nclpmx = nclpmx + 1
+      if (clip_a_id.ge.0) &
+        cpro_a_clipped(iel) = 1.d0 - xal
+      cvar_al(iel) = 1.d0
+      nclpmx(1) = nclpmx(1) + 1
     endif
   enddo
 
-  call log_iteration_clipping_field(ivarfl(ial), nclpmn, nclpmx, vmin, vmax)
+  call log_iteration_clipping_field(ivarfl(ial), nclpmn(1), nclpmx(1), vmin,vmax,nclpmn(1), nclpmx(1))
 
 endif
 
