@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -52,10 +52,11 @@
 #include "cs_field_pointer.h"
 #include "cs_mesh.h"
 #include "cs_mesh_quantities.h"
-#include "cs_prototypes.h"
+#include "cs_random.h"
 #include "cs_timer.h"
 #include "cs_mesh_location.h"
 #include "cs_restart.h"
+#include "cs_restart_default.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
@@ -67,6 +68,8 @@
 
 BEGIN_C_DECLS
 
+/*! \cond DOXYGEN_SHOULD_SKIP_THIS */
+
 /*=============================================================================
  * Local Macro Definitions
  *============================================================================*/
@@ -224,10 +227,8 @@ _random_method(const cs_int_t   n_points,
 
   double    random[3];
 
-  int       three = 3;
-
   for (point_id = 0; point_id < n_points; point_id++) {
-    CS_PROCF(normalen, NORMALEN) (&three, random);
+    cs_random_normal(3, random);
     for (coo_id = 0; coo_id < 3; coo_id++)
       fluctuations[point_id*3 + coo_id] = random[coo_id];
   }
@@ -273,7 +274,6 @@ _batten_method(const cs_int_t       n_points,
 
   if (initialize == 1) {
 
-    const int     three           = 3;
     const int     three_n_modes   = 3*inflow->n_modes;
     const double  one_by_sqrt_two = sqrt(0.5);
 
@@ -282,7 +282,7 @@ _batten_method(const cs_int_t       n_points,
       /* Random generation of the n_modes frequencies following a normal
          law with a mean of 1 and a variance of 1 (i.e. N(1,1)). */
 
-      CS_PROCF(normalen, NORMALEN) (&inflow->n_modes, inflow->frequency);
+      cs_random_normal(inflow->n_modes, inflow->frequency);
 
       for (mode_id = 0; mode_id < inflow->n_modes; mode_id++)
         inflow->frequency[mode_id] += 1.;
@@ -290,7 +290,7 @@ _batten_method(const cs_int_t       n_points,
       /* Random generation of the n_modes wave vectors following a normal
          law with a mean of 0 and a variance of 0.5 (i.e. N(0,1/2)). */
 
-      CS_PROCF(normalen, NORMALEN) (&three_n_modes, inflow->wave_vector);
+      cs_random_normal(three_n_modes, inflow->wave_vector);
 
       for (mode_id = 0; mode_id < inflow->n_modes; mode_id++)
         for (coo_id = 0; coo_id < 3; coo_id++)
@@ -307,8 +307,8 @@ _batten_method(const cs_int_t       n_points,
         /* Temporary random vectors following a normal law N(0,1) necessary
            to compute the random amplitudes of the sines and cosines */
 
-        CS_PROCF(normalen, NORMALEN) (&three, rcos);
-        CS_PROCF(normalen, NORMALEN) (&three, rsin);
+        cs_random_normal(3, rcos);
+        cs_random_normal(3, rsin);
 
         _CROSS_PRODUCT_3D(inflow->amplitude_cos + mode_id*3,
                           rcos,
@@ -454,10 +454,8 @@ _synthetic_eddy_method(const cs_int_t    n_points,
 
   cs_lnum_t  j;
 
-  int        one = 1;
-
   double     alpha;
-  double     random;
+  double     random = -1.;
 
   double     pond_tot;
 
@@ -485,15 +483,15 @@ _synthetic_eddy_method(const cs_int_t    n_points,
   for (point_id = 0; point_id < n_points; point_id++) {
 
     cs_int_t  b_face_id = num_face[point_id] - 1;
-    cs_lnum_t cell_id = mesh->b_face_cells[b_face_id] - 1;
+    cs_lnum_t cell_id = mesh->b_face_cells[b_face_id];
 
     for (coo_id = 0; coo_id < 3; coo_id++) {
 
       double length_scale_min = -HUGE_VAL;
 
       for (j = mesh->b_face_vtx_idx[b_face_id];
-                 j < mesh->b_face_vtx_idx[b_face_id + 1]; j++) {
-              cs_lnum_t vtx_id = mesh->b_face_vtx_lst[j-1] - 1;
+           j < mesh->b_face_vtx_idx[b_face_id + 1]; j++) {
+              cs_lnum_t vtx_id = mesh->b_face_vtx_lst[j];
               length_scale_min = CS_MAX(length_scale_min,
                               2.*CS_ABS(mesh_q->cell_cen[3*cell_id + coo_id]
                                         - mesh->vtx_coord[3*vtx_id + coo_id]));
@@ -652,9 +650,9 @@ _synthetic_eddy_method(const cs_int_t    n_points,
 
         for (coo_id = 0; coo_id < 3; coo_id++) {
 
-          CS_PROCF(zufall, ZUFALL)(&one,&random);
-          inflow->energy[struct_id*3 + coo_id] =
-            (random < 0.5) ? -1. : 1.;
+          cs_random_uniform(1, &random);
+          inflow->energy[struct_id*3 + coo_id]
+            = (random < 0.5) ? -1. : 1.;
 
         }
 
@@ -662,7 +660,7 @@ _synthetic_eddy_method(const cs_int_t    n_points,
 
         for (coo_id = 0; coo_id < 3; coo_id++) {
 
-          CS_PROCF(zufall, ZUFALL)(&one,&random);
+          cs_random_uniform(1, &random);
           inflow->position[struct_id*3 + coo_id] =
             box_min_coord[coo_id] + random*box_length[coo_id];
 
@@ -793,9 +791,9 @@ _synthetic_eddy_method(const cs_int_t    n_points,
         for (coo_id = 0; coo_id < 3; coo_id++) {
 
           if (randomize[coo_id] == 1) {
-            CS_PROCF(zufall, ZUFALL)(&one, &random);
-            inflow->position[struct_id*3 + coo_id] =
-              box_min_coord[coo_id] + random*box_length[coo_id];
+            cs_random_uniform(1, &random);
+            inflow->position[struct_id*3 + coo_id]
+              = box_min_coord[coo_id] + random*box_length[coo_id];
           }
 
         }
@@ -804,9 +802,9 @@ _synthetic_eddy_method(const cs_int_t    n_points,
 
         for (coo_id = 0; coo_id < 3; coo_id++) {
 
-          CS_PROCF(zufall, ZUFALL)(&one, &random);
-          inflow->energy[struct_id*3 + coo_id] =
-            (random < 0.5) ? -1. : 1.;
+          cs_random_uniform(1, &random);
+          inflow->energy[struct_id*3 + coo_id]
+            = (random < 0.5) ? -1. : 1.;
         }
 
       }
@@ -963,7 +961,7 @@ _rescale_flowrate(cs_int_t     n_points,
     double dot_product = 0.;
 
     cs_lnum_t b_face_id = num_face[point_id] - 1;
-    cs_lnum_t cell_id = mesh->b_face_cells[b_face_id] - 1;
+    cs_lnum_t cell_id = mesh->b_face_cells[b_face_id];
 
     double fluct[3] = { fluctuations[point_id*3],
                         fluctuations[point_id*3 + 1],
@@ -996,11 +994,11 @@ _rescale_flowrate(cs_int_t     n_points,
 
     int coo_id;
     cs_lnum_t b_face_id = num_face[point_id] - 1;
-    cs_lnum_t cell_id = mesh->b_face_cells[b_face_id] - 1;
+    cs_lnum_t cell_id = mesh->b_face_cells[b_face_id];
 
     cs_lnum_t idx = mesh->b_face_vtx_idx[b_face_id];
-    cs_lnum_t vtx_id1 = mesh->b_face_vtx_lst[idx-1] - 1;
-    cs_lnum_t vtx_id2 = mesh->b_face_vtx_lst[idx] - 1;
+    cs_lnum_t vtx_id1 = mesh->b_face_vtx_lst[idx];
+    cs_lnum_t vtx_id2 = mesh->b_face_vtx_lst[idx+1];
 
     double norm = 0.;
     double normal_comp = 0., tangent_comp1 = 0., tangent_comp2 = 0.;
@@ -1202,31 +1200,7 @@ _cs_inflow_add_inlet(cs_inflow_type_t   type,
   cs_glob_inflow_n_inlets++;
 }
 
-/*----------------------------------------------------------------------------
- * Open the restart file associated to cs_les_inflow
- * Allocate cs_glob_inflow_suite
- *
- * parameters:
- *   nomsui <- name of the restart file
- *   lngnom <- name length
- *   ireawr <- 1 for reading, 2 for writing
- *----------------------------------------------------------------------------*/
-
-static void
-cs_loc_inflow_opnsuite(const char              *nomsui,
-                       const cs_int_t          *lngnom,
-                       const cs_restart_mode_t  ireawr)
-{
-  char *nombuf;
-
-  /* Name treatment for the C API */
-  nombuf = cs_base_string_f_to_c_create(nomsui, *lngnom);
-
-  cs_glob_inflow_suite = cs_restart_create(nombuf, NULL, ireawr);
-
-  /* Free the memory if necessary */
-  cs_base_string_f_to_c_free(&nombuf);
-}
+/*! (DOXYGEN_SHOULD_SKIP_THIS) \endcond */
 
 /*============================================================================
  *  Public function definitions for Fortran API
@@ -1239,7 +1213,6 @@ cs_loc_inflow_opnsuite(const char              *nomsui,
 void CS_PROCF(defsyn, DEFSYN)
 (
  cs_int_t            *n_inlets     /* --> number of inlets                    */
- CS_ARGF_SUPP_CHAINE
 )
 {
   cs_int_t inlet_id;
@@ -1351,11 +1324,7 @@ void CS_PROCF(synthe, SYNTHE)
  const cs_int_t  *const iw,        /* --> index of velocity component         */
  const cs_real_t *const ttcabs,    /* --> current physical time               */
  const cs_real_t        dt[],      /* --> time step                           */
- const cs_real_t        rtpa[],    /* --> variables at cellules (previous)    */
- const cs_real_t        rtp[],     /* --> variables at cellules               */
- const cs_real_t        propce[],  /* --> physical properties at cells        */
        cs_real_t        rcodcl[]   /* <-> boundary conditions array           */
- CS_ARGF_SUPP_CHAINE
 )
 {
   cs_int_t  face_id;
@@ -1541,21 +1510,17 @@ void CS_PROCF(synthe, SYNTHE)
  * SUBROUTINE LECSYN
  * *****************
  *
- * CHARACTER        FILNAM : <-- : Name of the restart file
- * INTEGER          LNGNAM : <-- : Name length
+ * character(kind=c_char)  filename : <-- : Name of the restart file
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF(lecsyn, LECSYN)
-     (
-      const char       *const filnam,
-      const cs_int_t   *const lngnam
-      CS_ARGF_SUPP_CHAINE
-      )
+(
+ const char  *filename
+)
 {
   bool                corresp_cel, corresp_fac, corresp_fbr, corresp_som;
   cs_int_t            nbvent;
   cs_int_t            indfac, ierror;
-  cs_int_t            version;    /* Not used at the moment */
 
   cs_restart_t             *suite;
   cs_mesh_location_type_t   support;
@@ -1566,16 +1531,15 @@ void CS_PROCF(lecsyn, LECSYN)
   ierror = CS_RESTART_SUCCESS;
 
   /* Open the restart file */
-  cs_loc_inflow_opnsuite(filnam,
-                         lngnam,
-                         CS_RESTART_MODE_READ);
+  cs_glob_inflow_suite
+    = cs_restart_create(filename, NULL, CS_RESTART_MODE_READ);
 
   if (cs_glob_inflow_suite == NULL)
     bft_error(__FILE__, __LINE__, 0,
               _("Abort while opening the LES inflow module restart file "
                 "in read mode.\n"
                 "Verify the existence and the name of the restart file: %s\n"),
-              filnam);
+              filename);
 
   /* Pointer to the global restart structure */
   suite = cs_glob_inflow_suite;
@@ -1621,9 +1585,7 @@ void CS_PROCF(lecsyn, LECSYN)
                   "\n"
                   "Verify that the restart file corresponds to a\n"
                   "restart file for the LES inflow module.\n"),
-                filnam);
-
-    version = *tabvar;
+                filename);
 
     BFT_FREE(tabvar);
   }
@@ -1949,9 +1911,10 @@ void CS_PROCF(lecsyn, LECSYN)
 
   }
 
+  cs_restart_read_fields(suite, CS_RESTART_LES_INFLOW);
+
   /* Close the restart file and free structures */
-  cs_restart_destroy(cs_glob_inflow_suite);
-  cs_glob_inflow_suite = NULL;
+  cs_restart_destroy(&cs_glob_inflow_suite);
 
   bft_printf(_(" ...completed\n"));
 }
@@ -1964,20 +1927,17 @@ void CS_PROCF(lecsyn, LECSYN)
  * SUBROUTINE ECRSYN
  * *****************
  *
- * CHARACTER        FILNAM : <-- : Name of the restart file
- * INTEGER          LNGNAM : <-- : Name length
+ * character(kind=c_char)  filename : <-- : Name of the restart file
  *----------------------------------------------------------------------------*/
 
 void CS_PROCF(ecrsyn, ECRSYN)
-     (
-      const char       *const filnam,
-      const cs_int_t   *const lngnam
-      CS_ARGF_SUPP_CHAINE
-      )
+(
+ const char  *filename
+)
 {
   int   inlet_id;
 
-  cs_int_t            nbvent, ierror;
+  cs_int_t            nbvent;
 
   cs_restart_t             *suite;
   cs_mesh_location_type_t   support;
@@ -1988,19 +1948,16 @@ void CS_PROCF(ecrsyn, ECRSYN)
 
   bft_printf(_("\n Writing the LES inflow module restart file...\n"));
 
-  ierror = CS_RESTART_SUCCESS;
-
   /* Open the restart file */
-  cs_loc_inflow_opnsuite(filnam,
-                         lngnam,
-                         CS_RESTART_MODE_WRITE);
+  cs_glob_inflow_suite
+    = cs_restart_create(filename, NULL, CS_RESTART_MODE_WRITE);
 
   if (cs_glob_inflow_suite == NULL)
     bft_error(__FILE__, __LINE__, 0,
               _("Abort while opening the LES inflow module restart "
                 "file in write mode.\n"
                 "Verify the existence and the name of the restart file: %s\n"),
-              filnam);
+              filename);
 
   /* Pointer to the global restart structure */
   suite = cs_glob_inflow_suite;
@@ -2235,9 +2192,10 @@ void CS_PROCF(ecrsyn, ECRSYN)
 
   }
 
+  cs_restart_write_fields(suite, CS_RESTART_LES_INFLOW);
+
   /* Close the restart file and free structures */
-  cs_restart_destroy(cs_glob_inflow_suite);
-  cs_glob_inflow_suite = NULL;
+  cs_restart_destroy(&cs_glob_inflow_suite);
 
   bft_printf(_(" ...completed\n"));
 
