@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,38 +20,26 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine resalp &
-!================
-
- ( nvar   ,                                                       &
-   rtp    , rtpa   , propce )
 
 !===============================================================================
 ! Function :
 ! ----------
+!> \file resalp.f90
+!> \brief Solving the equation on alpha in the framwork of the Rij-EBRSM model.
+!>        Also called for alpha of scalars for EB-DFM.
 
-! Solving the equation on Alpha in the framwork of the Rij-EBRSM model.
-! written from the equation of F_BARRE)
+!-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-!    nom           !type!mode !                   role                         !
-!__________________!____!_____!________________________________________________!
-! nvar             ! e  ! <-- ! nombre total de variables                      !
-! dt(ncelet)       ! tr ! <-- ! pas de temps                                   !
-! rtp, rtpa        ! tr ! <-- ! variables de calcul au centre des              !
-! (ncelet,*)       !    !     !    cellules (instant courant ou prec)          !
-! propce           ! tr ! <-- ! proprietes physiques au centre des             !
-! (ncelet,*)       !    !     !    cellules                                    !
-!__________________!____!_____!________________________________________________!
+!______________________________________________________________________________.
+!  mode           nom           role
+!______________________________________________________________________________!
+!> \param[in]     f_id          field id of alpha variable
+!> \param[in]     c_durbin_l    constant for the Durbin length
+!______________________________________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!-------------------------------------------------------------------------------
-!===============================================================================
+subroutine resalp(f_id, c_durbin_l )
 
 !===============================================================================
 ! Module files
@@ -68,51 +56,57 @@ use period
 use parall
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
 implicit none
 
 ! Arguments
-
-integer          nvar
-
-double precision rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
+integer          f_id
+double precision c_durbin_l
 
 ! Local variables
 
-integer          ivar  , iel
-integer          ipcvis, ipcvlo, ipcvst
+integer          iel
+integer          ii    , jj    , ifac
 integer          iflmas, iflmab
-integer          nswrgp, imligp, iwarnp, ipp
-integer          iconvp, idiffp, ndircp, ireslp
-integer          nitmap, nswrsp, ircflp, ischcp, isstpp, iescap
-integer          imgrp , ncymxp, nitmfp
+integer          nswrgp, imligp, iwarnp
+integer          iconvp, idiffp, ndircp
+integer          nswrsp, ircflp, ischcp, isstpp, iescap
 integer          imucpp, idftnp, iswdyp
 integer          icvflb
+integer          init
 integer          ivoid(1)
 double precision blencp, epsilp, epsrsp, epsrgp, climgp, extrap, relaxp
 double precision thetv , thetap
 double precision d1s4, d3s2, d1s2
 double precision xk, xnu, l2
 double precision xllke, xllkmg, xlldrb
+double precision normp
 
 double precision rvoid(1)
 
+type(var_cal_opt), target   :: vcopt
+
 character(len=80) :: label
 double precision, allocatable, dimension(:) :: viscf, viscb
+double precision, allocatable, dimension(:) :: alpha_min
 double precision, allocatable, dimension(:) :: smbr, rovsdt
 double precision, allocatable, dimension(:) :: w1
 double precision, allocatable, dimension(:) :: dpvar
 double precision, dimension(:), pointer :: imasfl, bmasfl
 double precision, dimension(:), pointer :: crom
 double precision, dimension(:), pointer :: coefap, coefbp, cofafp, cofbfp
+double precision, dimension(:), pointer :: cvar_al, cvara_al, cvara_ep
+double precision, dimension(:), pointer :: cvara_r11, cvara_r22, cvara_r33
+double precision, dimension(:), pointer :: viscl, visct
+double precision, dimension(:,:), pointer :: cvara_rij
 
 !===============================================================================
 
 !===============================================================================
-! 1. Initialisation
+! 1. Initialization
 !===============================================================================
 
 allocate(smbr(ncelet), rovsdt(ncelet), w1(ncelet))
@@ -120,8 +114,19 @@ allocate(viscf(nfac), viscb(nfabor))
 allocate(dpvar(ncelet))
 
 call field_get_val_s(icrom, crom)
-ipcvis = ipproc(iviscl)
-ipcvst = ipproc(ivisct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
+
+call field_get_val_s(f_id, cvar_al)
+call field_get_val_prev_s(f_id, cvara_al)
+call field_get_val_prev_s(ivarfl(iep), cvara_ep)
+if (irijco.eq.1) then
+  call field_get_val_prev_v(ivarfl(irij), cvara_rij)
+else
+  call field_get_val_prev_s(ivarfl(ir11), cvara_r11)
+  call field_get_val_prev_s(ivarfl(ir22), cvara_r22)
+  call field_get_val_prev_s(ivarfl(ir33), cvara_r33)
+endif
 
 call field_get_key_int(ivarfl(iu), kimasf, iflmas)
 call field_get_key_int(ivarfl(iu), kbmasf, iflmab)
@@ -132,36 +137,24 @@ d1s2 = 1.d0/2.d0
 d1s4 = 1.d0/4.d0
 d3s2 = 3.d0/2.d0
 
-!  test sur alpha qui ne doit pas etre superieur a 1
-if (iwarni(ial).ge.1) then
-  write(nfecra,1000)
-endif
-
 !===============================================================================
-! 2. Resolution de l'equation de ALPHA
+! 2. Resolving the equation of alpha
 !===============================================================================
 
-ivar = ial
-ipp    = ipprtp(ivar)
+! Get calculation options
+call field_get_key_struct_var_cal_opt(f_id, vcopt)
 
-call field_get_coefa_s(ivarfl(ivar), coefap)
-call field_get_coefb_s(ivarfl(ivar), coefbp)
-call field_get_coefaf_s(ivarfl(ivar), cofafp)
-call field_get_coefbf_s(ivarfl(ivar), cofbfp)
+call field_get_coefa_s (f_id, coefap)
+call field_get_coefb_s (f_id, coefbp)
+call field_get_coefaf_s(f_id, cofafp)
+call field_get_coefbf_s(f_id, cofbfp)
 
-if(iwarni(ivar).ge.1) then
-  call field_get_label(ivarfl(ivar), label)
+if(vcopt%iwarni.ge.1) then
+  call field_get_label(f_id, label)
   write(nfecra,1100) label
 endif
 
-thetv  = thetav(ivar)
-
-ipcvlo = ipcvis
-if(isto2t.gt.0) then
-  if(iviext.gt.0) then
-    ipcvlo = ipproc(ivisla)
-  endif
-endif
+thetv  = vcopt%thetav
 
 do iel = 1, ncel
   smbr(iel) = 0.d0
@@ -171,13 +164,13 @@ do iel = 1, ncel
 enddo
 
 !===============================================================================
-! 2.2 Terme source de ALPHA
-!     SMBR=1/L^2*(alpha) - 1/L^2
-!  En fait on met un signe "-" car l'eq resolue est
-!    -div(grad ) alpha = SMBR
+! 2.2 Source term of alpha
+!     \f$ smbr = \dfrac{1}{L^2 (\alpha)} - \dfrac{1}{L^2}\f$
+!  In fact there is a mark "-" because the solved equation is
+!    \f$-\div{\grad {alpha}} = smbr \f$
 !===============================================================================
 
-! ---> Matrice
+! ---> Matrix
 
 if (isto2t.gt.0) then
   thetap = thetv
@@ -186,115 +179,144 @@ else
 endif
 
 !FIXME the source term extrapolation is not well done!!!!
-do iel=1,ncel
+do iel = 1, ncel
+  if (irijco.eq.1) then
+    xk = d1s2*(cvara_rij(1,iel)+cvara_rij(2,iel)+cvara_rij(3,iel))
+  else
+    xk = d1s2*(cvara_r11(iel)+cvara_r22(iel)+cvara_r33(iel))
+  endif
+  xnu  = viscl(iel)/crom(iel)
 
-  xk = d1s2*(rtpa(iel,ir11)+rtpa(iel,ir22)+rtpa(iel,ir33))
-  xnu  = propce(iel,ipcvis)/crom(iel)
+  ! Integral length scale
+  xllke = xk**d3s2/cvara_ep(iel)
 
-  ! Echelle de longueur integrale
-  xllke = xk**d3s2/rtpa(iel,iep)
+  ! Kolmogorov length scale
+  xllkmg = xceta*(xnu**3/cvara_ep(iel))**d1s4
 
-  ! Echelle de longueur de Kolmogorov
-  xllkmg = xceta*(xnu**3/rtpa(iel,iep))**d1s4
+  ! Durbin length scale
+  xlldrb = c_durbin_l * max(xllke,xllkmg)
 
-  ! Echelle de longueur de Durbin
-  xlldrb = xcl*max(xllke,xllkmg)
+  ! For automatic initialization, the length scale is fixed at L^+ =50
+  if (ntcabs.eq.1.and.reinit_turb.eq.1) xlldrb=50.d0*viscl0/ro0/(0.05d0*uref)
 
-  l2      = xlldrb**2
+  l2 = xlldrb**2
 
-! Terme explicite
-  smbr(iel) = volume(iel)*(1.d0 -rtpa(iel,ial)) / l2
+  ! Explicit term
+  smbr(iel) = cell_f_vol(iel)*(1.d0 -cvara_al(iel)) / l2
 
-! Terme implicite
-  rovsdt(iel) = (rovsdt(iel) + volume(iel)*thetap) / l2
+  ! Implicit term
+  rovsdt(iel) = (rovsdt(iel) + cell_f_vol(iel)*thetap) / l2
 
 enddo
 
-! Calcul de viscf et viscb pour codits
+! Calculation of viscf and viscb for codits
 
 do iel = 1, ncel
   w1(iel) = 1.d0
 enddo
 
 call viscfa                                                       &
-!==========
  ( imvisf ,                                                       &
    w1     ,                                                       &
    viscf  , viscb  )
 
 !===============================================================================
-! 2.3 Resolution effective de l'equation de ALPHA
+! 2.3 Effective resolution of the equation of alpha
 !===============================================================================
 
-iconvp = iconv (ivar)
-idiffp = idiff (ivar)
-ireslp = iresol(ivar)
-ndircp = ndircl(ivar)
-nitmap = nitmax(ivar)
-nswrsp = nswrsm(ivar)
-nswrgp = nswrgr(ivar)
-imligp = imligr(ivar)
-ircflp = ircflu(ivar)
-ischcp = ischcv(ivar)
-isstpp = isstpc(ivar)
+iconvp = vcopt%iconv
+idiffp = vcopt%idiff
+ndircp = vcopt%ndircl ! Diagonal reinforcement: same as dynamic Alpha
+nswrsp = vcopt%nswrsm
+nswrgp = vcopt%nswrgr
+imligp = vcopt%imligr
+ircflp = vcopt%ircflu
+ischcp = vcopt%ischcv
+isstpp = vcopt%isstpc
 iescap = 0
 imucpp = 0
-idftnp = idften(ivar)
-iswdyp = iswdyn(ivar)
-imgrp  = imgr  (ivar)
-ncymxp = ncymax(ivar)
-nitmfp = nitmgf(ivar)
-iwarnp = iwarni(ivar)
-blencp = blencv(ivar)
-epsilp = epsilo(ivar)
-epsrsp = epsrsm(ivar)
-epsrgp = epsrgr(ivar)
-climgp = climgr(ivar)
-extrap = extrag(ivar)
-relaxp = relaxv(ivar)
+idftnp = vcopt%idften
+iswdyp = vcopt%iswdyn
+iwarnp = vcopt%iwarni
+blencp = vcopt%blencv
+epsilp = vcopt%epsilo
+epsrsp = vcopt%epsrsm
+epsrgp = vcopt%epsrgr
+climgp = vcopt%climgr
+extrap = vcopt%extrag
+relaxp = vcopt%relaxv
+
 ! all boundary convective flux with upwind
 icvflb = 0
+normp = -1.d0
+
+init   = 1
 
 call codits &
-!==========
- ( idtvar , ivar   , iconvp , idiffp , ireslp , ndircp , nitmap , &
+ ( idtvar , init   , f_id   , iconvp , idiffp , ndircp ,          &
    imrgra , nswrsp , nswrgp , imligp , ircflp ,                   &
    ischcp , isstpp , iescap , imucpp , idftnp , iswdyp ,          &
-   imgrp  , ncymxp , nitmfp , ipp    , iwarnp ,                   &
+   iwarnp , normp  ,                                              &
    blencp , epsilp , epsrsp , epsrgp , climgp , extrap ,          &
    relaxp , thetv  ,                                              &
-   rtpa(1,ivar)    , rtpa(1,ivar)    ,                            &
+   cvara_al        , cvara_al        ,                            &
    coefap , coefbp , cofafp , cofbfp ,                            &
    imasfl , bmasfl ,                                              &
-   viscf  , viscb  , rvoid  , viscf  , viscb  , rvoid  ,          &
+   viscf  , viscb  , viscf  , viscb  , rvoid  ,                   &
    rvoid  , rvoid  ,                                              &
    icvflb , ivoid  ,                                              &
-   rovsdt , smbr   , rtp(1,ivar)     , dpvar  ,                   &
+   rovsdt , smbr   , cvar_al         , dpvar  ,                   &
    rvoid  , rvoid  )
 
 !===============================================================================
 ! 3. Clipping
 !===============================================================================
 
-call clpalp(ncelet, ncel, nvar, rtp)
-!==========
+allocate(alpha_min(ncelet))
+
+! Compute a first estimator of the minimal value of alpha per cell.
+! This is deduced from "alpha/L^2 - div(grad alpha) = 1/L^2" and assuming that
+! boundary cell values are 0. This value is thefore non zero but
+! much smaller than the wanted value.
+do iel = 1, ncel
+  alpha_min(iel) = rovsdt(iel)
+enddo
+
+do iel = ncel +1, ncelet
+  alpha_min(iel) = 0.d0
+enddo
+
+do ifac = 1, nfac
+  ii = ifacel(1, ifac)
+  jj = ifacel(2, ifac)
+  alpha_min(ii) = alpha_min(ii) + viscf(ifac)
+  alpha_min(jj) = alpha_min(jj) + viscf(ifac)
+enddo
+
+do ifac = 1, nfabor
+  ii = ifabor(ifac)
+  alpha_min(ii) = alpha_min(ii) + viscb(ifac)/distb(ifac)
+enddo
+
+do iel = 1, ncel
+  alpha_min(iel) = rovsdt(iel)/alpha_min(iel)
+enddo
+
+call clpalp(f_id, ncelet, ncel, alpha_min)
 
 ! Free memory
 deallocate(smbr, rovsdt, w1)
-deallocate(viscf, viscb)
+deallocate(viscf, viscb,alpha_min)
 deallocate(dpvar)
 
 !--------
-! FORMATS
+! Formats
 !--------
 
- 1000    format(/,                                                &
-'   ** RESOLUTION DU ALPHA                                    ',/,&
-'      -----------------------------------------------        ',/)
- 1100    format(/,'           RESOLUTION POUR LA VARIABLE ',A8,/)
+ 1100    format(/,'           Solving the variable ',A8,/)
 
 !----
-! FIN
+! End
 !----
 
 return
