@@ -316,6 +316,7 @@ endif
 !===============================================================================
 ! 2. User source terms
 !===============================================================================
+
 call cs_user_turbulence_source_terms2 &
  ( nvar   , nscal  , ncepdp , ncesmp ,                            &
    ivarfl(ivar)    ,                                              &
@@ -323,6 +324,9 @@ call cs_user_turbulence_source_terms2 &
    ckupdc , smacel ,                                              &
    smbr   , rovsdt )
 
+! C version
+call user_source_terms(ivarfl(ivar), smbr, rovsdt)
+
 do isou = 1, dimrij
   ! If we extrapolate the source terms
   if (st_prv_id.ge.0) then
