@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,38 +20,27 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine visv2f &
-!================
-
- ( rtp    , rtpa   , propce )
 
 !===============================================================================
-! FONCTION :
+! Function:
 ! --------
+!> \file visv2f.f90
+!> \brief Calculation of turbulent viscosity for
+!>         the V2F-BL model
+!>
+!> Edge faces types are vailable at the previous time step
+!>  (except at the first time step, when the itypfb anf itrifb tables
+!>  have not been filled)
+!-------------------------------------------------------------------------------
 
-! CALCUL DE LA VISCOSITE TURBULENTE POUR
-!          LE MODELE K-OMEGA V2F-BL
-
-
-! On dispose des types de faces de bord au pas de temps
-!   precedent (sauf au premier pas de temps, ou les tableaux
-!   ITYPFB et ITRIFB n'ont pas ete renseignes)
-
+!-------------------------------------------------------------------------------
 ! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! icetsm(ncesmp    ! te ! <-- ! numero des cellules a source de masse          !
-! rtp, rtpa        ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at current and previous time steps)          !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-!__________________!____!_____!________________________________________________!
+!______________________________________________________________________________.
+!  mode           name          role
+!______________________________________________________________________________!
+!______________________________________________________________________________!
 
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
-!===============================================================================
+subroutine visv2f
 
 !===============================================================================
 ! Module files
@@ -60,7 +49,6 @@ subroutine visv2f &
 use paramx
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use entsor
 use mesh
@@ -73,13 +61,10 @@ implicit none
 
 ! Arguments
 
-double precision rtp(ncelet,nflown:nvar), rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
-
 ! Local variables
 
 integer          iel, inc
-integer          ipcvis, ipcvst, iprev
+integer          iprev
 
 double precision s11, s22, s33
 double precision dudy, dudz, dvdx, dvdz, dwdx, dwdy
@@ -91,26 +76,31 @@ double precision, dimension(:,:,:), allocatable :: gradv
 double precision, dimension(:,:), pointer :: coefau
 double precision, dimension(:,:,:), pointer :: coefbu
 double precision, dimension(:), pointer :: crom
+double precision, dimension(:), pointer :: viscl, visct
+double precision, dimension(:), pointer :: cvar_k, cvar_ep, cvar_phi
 
 !===============================================================================
 
 !===============================================================================
-! 1.  INITIALISATION
+! 1.  Initialization
 !===============================================================================
 
 call field_get_coefa_v(ivarfl(iu), coefau)
 call field_get_coefb_v(ivarfl(iu), coefbu)
 
-! --- Memoire
+! --- Memory
 allocate(s2(ncelet))
 
-! --- Rang des variables dans PROPCE (prop. physiques au centre)
-ipcvis = ipproc(iviscl)
-ipcvst = ipproc(ivisct)
+call field_get_val_s(iviscl, viscl)
+call field_get_val_s(ivisct, visct)
 call field_get_val_s(icrom, crom)
 
+call field_get_val_s(ivarfl(ik), cvar_k)
+call field_get_val_s(ivarfl(iep), cvar_ep)
+call field_get_val_s(ivarfl(iphi), cvar_phi)
+
 !===============================================================================
-! 2.  CALCUL DES GRADIENTS DE VITESSE ET DE
+! 2.  Calculation of velocity gradient and of
 !       S2 = 2* (S11**2+S22**2+S33**2+2*(S12**2+S13**2+S23**2)
 !===============================================================================
 
@@ -118,7 +108,7 @@ call field_get_val_s(icrom, crom)
 allocate(gradv(3,3,ncelet))
 
 inc = 1
-iprev = 1
+iprev = 0
 
 call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc,    &
                            gradv)
@@ -145,22 +135,22 @@ enddo
 deallocate(gradv)
 
 !===============================================================================
-! 3.  CALCUL DE LA VISCOSITE
+! 3.  Calculation of viscosity
 !===============================================================================
 
 do iel = 1, ncel
 
-  xk = rtp(iel,ik)
-  xe = rtp(iel,iep)
+  xk = cvar_k(iel)
+  xe = cvar_ep(iel)
   xrom = crom(iel)
-  xnu = propce(iel,ipcvis)/xrom
+  xnu = viscl(iel)/xrom
 
   ttke = xk / xe
   ttmin = cpalct*sqrt(xnu/xe)
-  ttlim = 0.6d0/rtp(iel,iphi)/sqrt(3.d0)/cpalmu/s2(iel)
+  ttlim = 0.6d0/cvar_phi(iel)/sqrt(3.d0)/cpalmu/s2(iel)
   tt = min(ttlim,sqrt(ttke**2 + ttmin**2))
 
-  propce(iel,ipcvst) = cpalmu*xrom*tt*rtp(iel,iphi)*rtp(iel,ik)
+  visct(iel) = cpalmu*xrom*tt*cvar_phi(iel)*cvar_k(iel)
 
 enddo
 
@@ -168,12 +158,12 @@ enddo
 deallocate(s2)
 
 !----
-! FORMAT
+! Format
 !----
 
 
 !----
-! FIN
+! End
 !----
 
 return
