@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -19,44 +19,26 @@
 ! Street, Fifth Floor, Boston, MA 02110-1301, USA.
 
 !-------------------------------------------------------------------------------
-
-subroutine viswal &
-!================
-
- ( rtpa   , propce )
-
 !===============================================================================
-! FONCTION :
-! --------
+! Function:
+! ---------
 
-! CALCUL DE LA VISCOSITE "TURBULENTE" POUR
-!          UN MODELE LES WALE
+!> \file viswal.f90
+!>
+!> \brief Compute the turbulent viscosity for the WALE LES model
+!>
+!> The turbulent viscosity is:
+!> \f$ \mu_T = \rho (C_{wale} L)^2 * \dfrac{(\tens{S}:\tens{Sd})^{3/2}}
+!>                                         {(\tens{S} :\tens{S})^(5/2)
+!>                                         +(\tens{Sd}:\tens{Sd})^(5/4)} \f$
+!> with \f$ \tens{S}  = \frac{1}{2}(\gradt \vect{u} + \transpose{\gradt \vect{u}})\f$
+!> and  \f$ \tens{Sd} = \deviator{(\symmetric{(\tens{S}^2)})}\f$
+!-------------------------------------------------------------------------------
 
-! PROPCE(1,IVISCT) = ROM * (CWALE*L)**2 *
-!   [(Sijd.Sijd)**(3/2)] / [(Sij.Sij)**(5/2) + (Sijd.Sijd)**(5/4)]
-!
-! avec
-!       Sij = 0.5*[DUi/Dxj + DUj/Dxi]
-! et
-!       Sijd = 0.5*[DUi/Dxk.DUk/Dxj + DUj/Dxk.DUk/Dxi] - 1/3*Delta_ij.Gkk**2
+!-------------------------------------------------------------------------------
 
-! On dispose des types de faces de bord au pas de temps
-!   precedent (sauf au premier pas de temps, ou les tableaux
-!   ITYPFB et ITRIFB n'ont pas ete renseignes)
+subroutine viswal
 
-! Arguments
-!__________________.____._____.________________________________________________.
-! name             !type!mode ! role                                           !
-!__________________!____!_____!________________________________________________!
-! rtpa             ! ra ! <-- ! calculated variables at cell centers           !
-!  (ncelet, *)     !    !     !  (at previous time step)                       !
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
-!__________________!____!_____!________________________________________________!
-
-!     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
-!            L (LOGIQUE)   .. ET TYPES COMPOSES (EX : TR TABLEAU REEL)
-!     MODE : <-- donnee, --> resultat, <-> Donnee modifiee
-!            --- tableau de travail
 !===============================================================================
 
 !===============================================================================
@@ -66,7 +48,6 @@ subroutine viswal &
 use paramx
 use numvar
 use optcal
-use dimens, only: nvar
 use cstphy
 use entsor
 use parall
@@ -80,60 +61,41 @@ implicit none
 
 ! Arguments
 
-double precision rtpa(ncelet,nflown:nvar)
-double precision propce(ncelet,*)
-
 ! Local variables
 
 integer          iel, inc
-integer          ipcvst, ipcvis, iprev
+integer          iprev
 integer          i, j, k
 
-double precision coef, deux, delta, tiers
-double precision sij, sijd, s, sd, sinv
-double precision xfil, xa  , xb  , radeux, con
-double precision dudx(ndim,ndim), kdelta(ndim,ndim)
+double precision coef, delta, third
+double precision sijd, s, sd, sinv
+double precision con, trace_g2
+double precision dudx(ndim,ndim), kdelta(ndim,ndim), g2(ndim,ndim)
 
 double precision, dimension(:,:,:), allocatable :: gradv
-double precision, dimension(:,:), pointer :: coefau
-double precision, dimension(:,:,:), pointer :: coefbu
 double precision, dimension(:), pointer :: crom
+double precision, dimension(:), pointer :: visct
 
 !===============================================================================
 
 !===============================================================================
-! 1.  INITIALISATION
+! 1. Initialization
 !===============================================================================
 
-call field_get_coefa_v(ivarfl(iu), coefau)
-call field_get_coefb_v(ivarfl(iu), coefbu)
-
-! --- Rang des variables dans PROPCE (prop. physiques au centre)
-ipcvis = ipproc(iviscl)
-ipcvst = ipproc(ivisct)
+call field_get_val_s(ivisct, visct)
 call field_get_val_s(icrom, crom)
 
-! --- Pour le calcul de la viscosite de sous-maille
-xfil   = xlesfl
-xa     = ales
-xb     = bles
-deux   = 2.d0
-radeux = sqrt(deux)
-tiers  = 1.d0/3.d0
-
+third  = 1.d0/3.d0
 
 !===============================================================================
-! 2.  CALCUL DU GRADIENT DE VITESSE
-!       W1 = DU/DX, W2 = DU/DY, W3 = DU/DZ
-!       W4 = DV/DX, W5 = DV/DY, W6 = DV/DZ
-!       W7 = DW/DX, W8 = DW/DY, W9 = DW/DZ
+! 2. Computation of the velocity gradient
 !===============================================================================
 
 ! Allocate temporary arrays for gradients calculation
 allocate(gradv(3,3,ncelet))
 
 inc = 1
-iprev = 1
+iprev = 0
 
 call field_gradient_vector(ivarfl(iu), iprev, imrgra, inc,    &
                            gradv)
@@ -150,7 +112,7 @@ kdelta(3,1) = 0
 kdelta(3,2) = 0
 kdelta(3,3) = 1
 
-coef = cwale**2 * radeux
+coef = sqrt(2.d0) * cwale**2
 
 do iel = 1, ncel
 
@@ -167,33 +129,45 @@ do iel = 1, ncel
   dudx(3,3) = gradv(3,3,iel)
 
   s  = 0.d0
-  sd = 0.d0
+
+  trace_g2 = 0.d0
 
   do i = 1, ndim
     do j = 1, ndim
 
-      ! Sij = 0.5 * (dUi/dXj + dUj/dXi)
+      ! s = 1/4 * (dUi/dXj + dUj/dXi) * (dUi/dXj + dUj/dXi)
 
-      sij = 0.5d0*(dudx(i,j)+dudx(j,i))
+      s = s + 0.25d0*(dudx(i,j)+dudx(j,i))**2
 
-      s = s + sij**2
 
+      ! g2 is the square tensor of the velocity gradient
+      g2(i,j) = 0.d0
       do k = 1, ndim
-
-!  traceless symmetric part of the square of the velocity gradient tensor
-!    Sijd = 0.5 * ( dUi/dXk dUk/dXj + dUj/dXk dUk/dXi) - 1/3 Dij dUk/dXk dUk/dXk
-
-        sijd = 0.5d0*(dudx(i,k)*dudx(k,j)+ dudx(j,k)*dudx(k,i)) &
-              -tiers*kdelta(i,j)*dudx(k,k)**2
-
-        sd = sd + sijd**2
-
+        g2(i,j) = g2(i,j) + dudx(i,k)*dudx(k,j)
       enddo
+
+    enddo
+
+    trace_g2 = trace_g2 + g2(i,i)
+
+  enddo
+
+  sd = 0.d0
+
+  do i = 1, ndim
+    do j = 1, ndim
+      ! traceless symmetric part of the square of the velocity gradient tensor
+      !   Sijd = 0.5*( dUi/dXk dUk/dXj + dUj/dXk dUk/dXi)
+      !        - 1/3 Dij dUk/dXl dUl/dXk
+
+      sijd = 0.5d0*(g2(i,j)+g2(j,i))-third*kdelta(i,j)*trace_g2
+
+      sd = sd + sijd**2
     enddo
   enddo
 
 !===============================================================================
-! 3.  CALCUL DE LA VISCOSITE TURBULENTE
+! 3. Computation of turbulent viscosity
 !===============================================================================
 
   ! Turbulent inverse time scale =
@@ -206,22 +180,22 @@ do iel = 1, ncel
     con = 0.d0
   endif
 
-  delta = xfil* (xa*volume(iel))**xb
+  delta = xlesfl* (ales*volume(iel))**bles
   delta = coef * delta**2
 
-  propce(iel,ipcvst) = crom(iel) * delta * con
+  visct(iel) = crom(iel) * delta * con
 
 enddo
 
 ! Free memory
 deallocate(gradv)
 
-!----
-! FORMAT
-!----
+!-------
+! Format
+!-------
 
 !----
-! FIN
+! End
 !----
 
 return
