@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -21,8 +21,6 @@
 !-------------------------------------------------------------------------------
 
 subroutine vorlgv &
-!================
-
  ( ncevor , ient   , dtref  ,                                     &
    yzc    , xu     , xv     , xw     )
 
@@ -63,6 +61,8 @@ use paramx
 use entsor
 use vorinc
 
+use cs_c_bindings
+
 !===============================================================================
 
 implicit none
@@ -121,7 +121,7 @@ do ii = 1, ncevor
 ! la fluctuation dans la direction normale
 ! a la paroi
 
-    call normalen(iun,dw1(1))
+    call cs_random_normal(1, dw1)
     ufluc = ( ufluc -                                             &
             (1.d0-2.d0/3.d0*cst2)*du_vor*vfluc*dtref +            &
              2.d0* sqrt(2.d0/3.d0*(cst1-1.d0)*ee_vor*dtref)       &
