@@ -2,7 +2,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -20,10 +20,7 @@
 
 !-------------------------------------------------------------------------------
 
-subroutine vorpre &
-!================
-
- ( propce )
+subroutine vorpre
 
 !===============================================================================
 ! FONCTION :
@@ -36,7 +33,6 @@ subroutine vorpre &
 !__________________.____._____.________________________________________________.
 ! name             !type!mode ! role                                           !
 !__________________!____!_____!________________________________________________!
-! propce(ncelet, *)! ra ! <-- ! physical properties at cell centers            !
 !__________________!____!_____!________________________________________________!
 
 !     TYPE : E (ENTIER), R (REEL), A (ALPHANUMERIQUE), T (TABLEAU)
@@ -67,12 +63,10 @@ implicit none
 
 ! Arguments
 
-double precision propce(ncelet,*)
-
 ! Local variables
 
 integer          ifac, iel, ii
-integer          ient, ipcvis
+integer          ient
 integer          isurf(nentmx)
 
 double precision xx, yy, zz
@@ -80,6 +74,8 @@ double precision xxv, yyv, zzv
 
 double precision, allocatable, dimension(:,:) :: w1x, w1y, w1z, w1v
 double precision, dimension(:), pointer :: crom
+double precision, dimension(:), pointer :: viscl
+
 !===============================================================================
 ! 1.  INITIALISATIONS
 !===============================================================================
@@ -115,7 +111,6 @@ if(irangp.ge.0) then
   do ient = 1, nnent
     icvor(ient) = icvor2(ient)
     call parcpt(icvor(ient))
-    !==========
     icvmax = max(icvmax,icvor(ient))
   enddo
 else
@@ -138,7 +133,7 @@ enddo
 ! Chaque processeur stocke dans les tableaux 'w1x', ...
 ! les coordonnees des faces ou il doit ensuite utiliser des vortex
 
-ipcvis = ipproc(iviscl)
+call field_get_val_s(iviscl, viscl)
 call field_get_val_s(icrom, crom)
 do ifac = 1, nfabor
   ient = irepvo(ifac)
@@ -148,7 +143,7 @@ do ifac = 1, nfabor
     w1x(icvor2(ient),ient)= cdgfbo(1,ifac)
     w1y(icvor2(ient),ient)= cdgfbo(2,ifac)
     w1z(icvor2(ient),ient)= cdgfbo(3,ifac)
-    w1v(icvor2(ient),ient) = propce(iel,ipcvis)/crom(iel)
+    w1v(icvor2(ient),ient) = viscl(iel)/crom(iel)
     xsurfv(ient) = xsurfv(ient) + sqrt(surfbo(1,ifac)**2          &
       + surfbo(2,ifac)**2 + surfbo(3,ifac)**2)
 !         Vecteur surface d'une face de l'entree
@@ -164,7 +159,6 @@ enddo
 if(irangp.ge.0) then
   do ient = 1, nnent
     call parsom(xsurfv(ient))
-    !==========
   enddo
 endif
 
@@ -174,21 +168,17 @@ endif
 if(irangp.ge.0) then
   do ient = 1, nnent
     call paragv &
-    !==========
  ( icvor2(ient) , icvor(ient)    ,  &
-   w1x(1,ient)  , xyzv(1,1,ient) )
+   w1x(:,ient)  , xyzv(:,1,ient) )
     call paragv &
-    !==========
  ( icvor2(ient) , icvor(ient)    ,  &
-   w1y(1,ient)  , xyzv(1,2,ient) )
+   w1y(:,ient)  , xyzv(:,2,ient) )
     call paragv &
-    !==========
  ( icvor2(ient) , icvor(ient)    ,  &
-   w1z(1,ient)  , xyzv(1,3,ient) )
+   w1z(:,ient)  , xyzv(:,3,ient) )
     call paragv &
-    !==========
  ( icvor2(ient) , icvor(ient)  ,  &
-   w1v(1,ient)  , visv(1,ient) )
+   w1v(:,ient)  , visv(:,ient) )
   enddo
 
 !  -> A la fin de cette etape, tous les processeurs connaissent
