@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -31,14 +31,15 @@
 !> \brief User subroutine which fills boundary conditions arrays
 !> (\c icodcl, \c rcodcl) for unknown variables.
 !>
+!> See \subpage cs_user_boundary_conditions_examples for examples.
 !>
-!> \section intro Introduction
+!> \section cs_user_boundary_conditions_intro Introduction
 !>
 !> Here one defines boundary conditions on a per-face basis.
 !>
 !> Boundary faces may be selected using the \ref getfbr subroutine.
 !>
-!> \code getfbr(string, nelts, eltlst) \endcode
+!> \code getfbr(string, nelts, lstelt) \endcode
 !>  - string is a user-supplied character string containing selection criteria;
 !>  - nelts is set by the subroutine. It is an integer value corresponding to
 !>    the number of boundary faces verifying the selection criteria;
@@ -190,11 +191,13 @@
 !>  - 5: Smooth wall    (usable for any variable except for pressure)
 !>  - 6: Rough wall     (usable for any variable except for pressure)
 !>  - 9: Free outlet    (usable only for velocity)
+!>  - 13: Dirichlet for the advection operator and
+!>        Neumann for the diffusion operator
 !>
 !> The values of the 3 \c rcodcl components are:
 !>  - \c rcodcl(ifac, ivar, 1):
-!>     - Dirichlet for the variable          if \c icodcl(ifac, ivar) =  1
-!>     - Wall value (sliding velocity, temp) if \c icodcl(ifac, ivar) =  5
+!>     - Dirichlet for the variable          if \c icodcl(ifac, ivar) = 1 or 13
+!>     - Wall value (sliding velocity, temp) if \c icodcl(ifac, ivar) = 5
 !>     .
 !>     The dimension of \c rcodcl(ifac, ivar, 1) is that of the
 !>     resolved variable, for instance:
@@ -211,32 +214,32 @@
 !>     - For the pressure P,              in  s/m:
 !>        \c rcodcl(ifac, ivar, 2) =                     dt / d
 !>     - For temperatures T,              in Watt/(m2 degres):
-!>        \c rcodcl(ifac, ivar, 2) = Cp*(viscls+visct/sigmas) / d
+!>        \c rcodcl(ifac, ivar, 2) = Cp*(viscls+visct/turb_schmidt) / d
 !>     - For enthalpies H,                in kg /(m2 s):
-!>        \c rcodcl(ifac, ivar, 2) =    (viscls+visct/sigmas) / d
+!>        \c rcodcl(ifac, ivar, 2) =    (viscls+visct/turb_schmidt) / d
 !>     - For other scalars F              in:
-!>        \c rcodcl(ifac, ivar, 2) =    (viscls+visct/sigmas) / d
+!>        \c rcodcl(ifac, ivar, 2) =    (viscls+visct/turb_schmidt) / d
 !>            (d has the dimension of a distance in m)
 !>
-!>  - \c rcodcl(ifac, ivar, 3) if \c icodcl(ifac, ivar) = 3:
+!>  - \c rcodcl(ifac, ivar, 3) if \c icodcl(ifac, ivar) = 3 or 13:
 !>      Flux density (< 0 if gain, n outwards-facing normal)
 !>     - For velocities U,                in kg/(m s2) = J:
 !>        \c rcodcl(ifac, ivar, 3) =         -(viscl+visct) * (grad U).n
 !>     - For pressure P,                  in kg/(m2 s):
 !>        \c rcodcl(ifac, ivar, 3) =                    -dt * (grad P).n
 !>     - For temperatures T,              in Watt/m2:
-!>        \c rcodcl(ifac, ivar, 3) = -Cp*(viscls+visct/sigmas) * (grad T).n
+!>        \c rcodcl(ifac, ivar, 3) = -Cp*(viscls+visct/turb_schmidt) * (grad T).n
 !>     - For enthalpies H,                in Watt/m2:
-!>        \c rcodcl(ifac, ivar, 3) = -(viscls+visct/sigmas) * (grad H).n
+!>        \c rcodcl(ifac, ivar, 3) = -(viscls+visct/turb_schmidt) * (grad H).n
 !>     - For other scalars F              in:
-!>        \c rcodcl(ifac, ivar, 3) = -(viscls+visct/sigmas) * (grad F).n
+!>        \c rcodcl(ifac, ivar, 3) = -(viscls+visct/turb_schmidt) * (grad F).n
 !>
 !>  - \c rcodcl(ifac, ivar, 3) if \c icodcl(ifac, ivar) = 6:
 !>      Roughness for the rough wall law
 !>     - For velocities U, dynamic roughness
 !>         \c rcodcl(ifac, iu, 3) = roughd
 !>     - For other scalars, thermal roughness
-!>         \c rcodcl(ifac, iv, 3) = rought
+!>         \c rcodcl(ifac, iv, 3) = rough
 !>
 !>
 !> Note that if the user assigns a value to \c itypfb equal to \c ientre, \c isolib,
@@ -323,25 +326,30 @@
 !>       entering the subroutine.
 !>
 !>
-!> \subsubsection cell_id Cells identification of some variables
+!> \subsubsection cs_user_bc_cell_id Cell values of some variables
 !>
-!> Cell values  (let \c iel = \c ifabor(ifac))
+!> Cell value field ids
 !>
-!> - Density:                               \c propce(iel, ipproc(irom))
-!> - Dynamic molecular viscosity:           \c propce(iel, ipproc(iviscl))
-!> - Turbulent viscosity:                   \c propce(iel, ipproc(ivisct))
-!> - Specific heat:                         \c propce(iel, ipproc(icp)
-!> - Diffusivity(lambda):                   \c propce(iel, ipproc(ivisls(iscal)))
+!> - Density:                        \c irom
+!> - Dynamic molecular viscosity:    \c iviscl
+!> - Turbulent viscosity:            \c ivisct
+!> - Specific heat:                  \c icp
+!> - Diffusivity(lambda):            \c field_get_key_int(ivarfl(isca(iscal)), &
+!>                                      kivisl, ...)
 !>
 !>
 !> \subsubsection fac_id Faces identification
 !>
 !> - Density:                               \c field id \c ibrom
-!> - Mass flux (for convecting \c ivar):
+!> - Boundary mass flux (for convecting \c ivar):
 !>     field id \c iflmab
-!>     using \c field_get_key_int(ivarfl(ivar), kimasf, iflmab)
+!>     using \c field_get_key_int(ivarfl(ivar), kbmasf, iflmab)
 !> - For other values: take as an approximation the value in the adjacent cell
 !>                     i.e. as above with \c iel = ifabor(ifac).
+!>
+!> Please refer to the
+!> <a href="../../theory.pdf#boundary"><b>boundary conditions</b></a>
+!> section of the theory guide for more informations.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -359,10 +367,12 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
+!>                               - 13 Dirichlet for the advection operator and
+!>                                    Neumann for the diffusion operator
 !> \param[in]     itrifb        indirection for boundary faces ordering
 !> \param[in,out] itypfb        boundary face types
 !> \param[out]    izfppp        boundary face zone number
@@ -372,7 +382,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
@@ -383,7 +393,7 @@
 !>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
-subroutine cs_user_boundary_conditions &
+subroutine cs_f_user_boundary_conditions &
  ( nvar   , nscal  ,                                              &
    icodcl , itrifb , itypfb , izfppp ,                            &
    dt     ,                                                       &
@@ -413,11 +423,12 @@ use ppcpfu
 use atincl
 use atsoil
 use ctincl
-use elincl
 use cs_fuel_incl
 use mesh
 use field
 use turbomachinery
+use iso_c_binding
+use cs_c_bindings
 
 !===============================================================================
 
@@ -427,12 +438,12 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itrifb(nfabor), itypfb(nfabor)
 integer          izfppp(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -453,7 +464,7 @@ integer, allocatable, dimension(:) :: lstelt
 
 if (iihmpr.eq.1) then
   return
-else
+else if (nfabor.gt.0) then
   write(nfecra,9000)
   call csexit (1)
 endif
@@ -464,11 +475,11 @@ endif
 '@',/,                                                            &
 '@ @@ WARNING:    stop in definition of boundary conditions',   /,&
 '@    =======',/,                                                 &
-'@  The user subroutine ''cs_user_boundary_conditions         ',/,&
-'@  must be completed.                                        ',/,&
-'@                                                            ',/,&
-'@  The calculation will not be run.                          ',/,&
-'@                                                            ',/,&
+'@  The user subroutine ''cs_f_user_boundary_conditions'       ,/,&
+'@  must be completed.'                                        ,/,&
+'@'                                                            ,/,&
+'@  The calculation will not be run.'                          ,/,&
+'@'                                                            ,/,&
 '@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
 '@',/)
 
@@ -504,7 +515,7 @@ allocate(lstelt(nfabor))  ! temporary array for boundary faces selection
 deallocate(lstelt)  ! temporary array for boundary faces selection
 
 return
-end subroutine cs_user_boundary_conditions
+end subroutine cs_f_user_boundary_conditions
 
 !> \section examples Examples
 !>   Several examples are provided
