@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -32,6 +32,9 @@
 !>  general purpose
 !>  (i.e. anything that does not have another dedicated user subroutine)
 !>
+!> See \subpage cs_user_extra_operations_examples and
+!> \subpage cs_user_extra_operations-nusselt_calculation for examples.
+!>
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -62,9 +65,7 @@ use optcal
 use cstphy
 use cstnum
 use entsor
-use lagpar
 use lagran
-use lagdim
 use parall
 use period
 use ppppar
@@ -74,6 +75,7 @@ use mesh
 use field
 use field_operator
 use turbomachinery
+use cs_c_bindings
 
 !===============================================================================
 
@@ -85,24 +87,6 @@ integer          nvar   , nscal
 
 double precision dt(ncelet)
 
-! Local variables
-
-! INSERT_VARIABLE_DEFINITIONS_HERE
-
-!===============================================================================
-
-!===============================================================================
-! Initialization
-!===============================================================================
-
-! INSERT_ADDITIONAL_INITIALIZATION_CODE_HERE
-
-!===============================================================================
-! User operations
-!===============================================================================
-
-! INSERT_MAIN_CODE_HERE
-
 !--------
 ! Formats
 !--------
@@ -111,6 +95,5 @@ double precision dt(ncelet)
 ! End
 !----
 
-
 return
 end subroutine cs_f_user_extra_operations
