@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -30,40 +30,41 @@
 !>
 !> \brief Initialize variables
 !>
+!> See \subpage cs_user_initialization for examples.
+!>
 !> This subroutine is called at beginning of the computation
 !> (restart or not) before the loop time step.
 !>
 !> This subroutine enables to initialize or modify (for restart)
 !> unkown variables and time step values.
 !>
-!> \c rom and \c viscl values are equal to \c ro0 and \c viscl0 or initialize
-!> by reading the restart file.
-!> viscls and cp variables (when there are defined) have no value
-!> excepted if they are read from a restart file.
-!>
-!> Physical quantities are defined in the following arrays:
-!> \code
-!>  propce ! physical quantities defined at cell center
-!> \endcode
-!>
-!> Examples:
-!> \code
-!>  propce(iel, ipproc(irom  )) ! means rom  (iel)
-!>  propce(iel, ipproc(iviscl)) ! means viscl(iel)
-!>  propce(iel, ipproc(icp   )) ! means cp   (iel)
-!>  propce(iel, ipproc(ivisls(iscal))) ! means visls(iel, iscal)
-!> \endcode
-!>
 !> Modification of the behaviour law of physical quantities (rom, viscl,
 !> viscls, cp) is not done here. It is the purpose of the user subroutine
-!> \ref cs_user_physical_properties.
+!> \ref cs_user_physical_properties
 !>
-!> \section cell_id Cells identification
+!> \c rom and \c viscl values are equal to \c ro0 and \c viscl0 or initialize
+!> by reading the restart file.
+!> Variables diffusivity and specific heat (when they are defined) have no value
+!> except if they are read from a restart file.
+!>
+!> \par cs_user_initialization_cell_id Cells identification
 !>
 !> Cells may be identified using the \ref getcel subroutine.
 !> The syntax of this subroutine is described in the
 !> \ref cs_user_boundary_conditions subroutine,
 !> but a more thorough description can be found in the user guide.
+!>
+!> Field cell values arrays can be retrieved using the appropriate access
+!> functions as described \ref field "here".
+!>
+!> Example of field ids:
+!> - Density:                        \c irom
+!> - Dynamic molecular viscosity:    \c iviscl
+!> - Turbulent viscosity:            \c ivisct
+!> - Specific heat:                  \c icp
+!> - Diffusivity(lambda):            \c field_get_key_int(ivarfl(isca(iscal)),
+!>                                      kivisl, ...)
+!>
 !
 !-------------------------------------------------------------------------------
 
@@ -77,7 +78,7 @@
 !> \param[in]     dt            time step (per cell)
 !_______________________________________________________________________________
 
-subroutine cs_user_initialization &
+subroutine cs_user_f_initialization &
  ( nvar   , nscal  ,                                              &
    dt     )
 
@@ -103,11 +104,11 @@ use cpincl
 use ppincl
 use atincl
 use ctincl
-use elincl
 use ppcpfu
 use cs_coal_incl
 use cs_fuel_incl
 use mesh
+use field
 use turbomachinery
 
 !===============================================================================
@@ -157,4 +158,4 @@ allocate(lstelt(ncel)) ! temporary array for cells selection
 deallocate(lstelt) ! temporary array for cells selection
 
 return
-end subroutine cs_user_initialization
+end subroutine cs_user_f_initialization
