@@ -1,7 +1,7 @@
 !===============================================================================
 ! User synthetic turbulence inlet definition.
 !
-! 1) Global caracteristics of synthetic turbulence inlets
+! 1) Global characteristics of synthetic turbulence inlets
 ! 2) Caracteristics of one specific inlet
 ! 3) Accurate specification of target statistics at inlet
 !===============================================================================
@@ -12,7 +12,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -36,10 +36,10 @@
 
 !> \file cs_user_les_inflow.f90
 !>
-!> \brief Generation of synthetic turbulence at LES inlets
-!>
 !> \brief Generation of synthetic turbulence at LES inlets initialization
 !>
+!> See \subpage les_inflow for examples.
+!>
 !> \c nent and \c isuisy might be defined.
 !>
 !> \c nent = Number of inlets
@@ -50,11 +50,11 @@
 
 !-------------------------------------------------------------------------------
 ! Arguments
-!______________________________________________________________________________.
+!______________________________________________________________________________!
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 !> \param[out]    nent          number of synthetic turbulence inlets
-!_______________________________________________________________________________
+!______________________________________________________________________________!
 
 subroutine cs_user_les_inflow_init (nent)
 
@@ -74,20 +74,6 @@ implicit none
 
 integer nent
 
-! Local variables
-
-!===============================================================================
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-
-if (1.eq.1) return
-
-!===============================================================================
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
-! INSERT_MAIN_CODE_HERE
-
 !--------
 ! Formats
 !--------
@@ -105,7 +91,7 @@ end subroutine cs_user_les_inflow_init
 ! Purpose :
 ! --------
 
-!> \brief Definition of the caracteristics of the synthetic turbulence inlet
+!> \brief Definition of the characteristics of the synthetic turbulence inlet
 !>  \c nument
 !>
 !> For each LES inlet, the following parameters might be defined:
@@ -121,13 +107,13 @@ end subroutine cs_user_les_inflow_init
 !>       .
 !>
 !>    - nelent indicates the number of "entities" relative to the method
-!>       (usefull only for the Batten method and the SEM):
+!>       (useful only for the Batten method and the SEM):
 !>
 !>       - for Batten : number of Fourier modes of the turbulent fluctuations
 !>       - for SEM    : number of synthetic eddies building the fluctuations
 !>       .
 !>
-!>    - iverbo indicates the verbosity level (listing)
+!>    - iverbo indicates the verbosity level (log)
 !>
 !>       -   0  no specific output
 !>       - > 0 additionnal output (only for SEM)
@@ -147,7 +133,7 @@ end subroutine cs_user_les_inflow_init
 !>     - dspent   : reference dissipation rate
 !>
 !>       \remarks
-!>       - dspent usefull only for typent = 2 (Batten) or typent = 3 (SEM).
+!>       - dspent useful only for typent = 2 (Batten) or typent = 3 (SEM).
 !>       - Strictly positive values are required for enrent and dspent.
 !>       - Accurate specification of the statistics of the flow at LES inlet
 !>         can be made via the user subroutine cs_user_les_inflow_advanced.
@@ -203,20 +189,8 @@ integer          lfbent(nfabor)
 
 double precision vitent(3), enrent, dspent
 
-! Local variables
-
 !===============================================================================
 
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-
-if (1.eq.1) return
-
-!===============================================================================
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
-! INSERT_MAIN_CODE_HERE
-
 !--------
 ! Formats
 !--------
@@ -316,20 +290,8 @@ double precision dt(ncelet)
 double precision uvwent(ndim,nfbent), rijent(6,nfbent)
 double precision epsent(nfbent)
 
-! Local variables
-
 !===============================================================================
 
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-
-if (1.eq.1) return
-
-!===============================================================================
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
-! INSERT_MAIN_CODE_HERE
-
 !--------
 ! Formats
 !--------
