@@ -10,7 +10,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -65,74 +65,54 @@
 
 BEGIN_C_DECLS
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \file cs_user_turbomachinery.c
+ *
+ * \brief Definition of turbomachinery related options.
+ *
+ * See \subpage turbomachinery for examples.
+*/
+/*----------------------------------------------------------------------------*/
+
 /*============================================================================
  * User function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Define rotor/stator model.
- *----------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define rotor/stator model.
+*/
+/*----------------------------------------------------------------------------*/
 
 void
 cs_user_turbomachinery(void)
 {
-  return; /* REMOVE_LINE_FOR_USE_OF_SUBROUTINE */
 
-  /* Set turbomachinery model type:
-
-     CS_TURBOMACHINERY_NONE,          No turbomachinery modeling
-     CS_TURBOMACHINERY_FROZEN,        Frozen rotor model
-     CS_TURBOMACHINERY_TRANSIENT      Full transient simulation
-  */
-
-  cs_turbomachinery_set_model(CS_TURBOMACHINERY_NONE);
 }
 
-/*----------------------------------------------------------------------------
- * Define rotor axes, associated cells, and rotor/stator faces.
- *----------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define rotor axes, associated cells, and rotor/stator faces.
+*/
+/*----------------------------------------------------------------------------*/
 
 void
 cs_user_turbomachinery_rotor(void)
 {
-  return; /* REMOVE_LINE_FOR_USE_OF_SUBROUTINE */
 
-  /* Define rotor axis and cells, with rotor/stator interface face joining */
-  /* --------------------------------------------------------------------- */
+}
 
-  if (false) {
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Define rotation velocity of rotor.
+*/
+/*----------------------------------------------------------------------------*/
 
-    /* Define cells belonging to rotor and associated axis */
+void
+cs_user_turbomachinery_set_rotation_velocity(void)
+{
 
-    double rotation_velocity = 2.;
-    double rotation_axis[3] = {0., 0., 1.};
-
-    const char cell_criteria[]  = "cylinder[0.0, 0.0, 0.0,"
-                                          " 0.0, 0.0, 1.0,"
-                                          " 2.0]";
-
-    cs_turbomachinery_add_rotor(cell_criteria,
-                                rotation_velocity,
-                                rotation_axis);
-
-    /* Define joining associated with rotor/stator interface */
-
-    const char faces_criteria[] = "rotor_interface or stator_interface";
-
-    int    verbosity = 0;     /* per-task dump if > 1, debug level if >= 3 */
-    int    visualization = 0; /* debug level if >= 3 */
-    float  fraction = 0.10, plane = 25.;
-
-    int join_num = cs_turbomachinery_join_add(faces_criteria,
-                                              fraction,
-                                              plane,
-                                              verbosity,
-                                              visualization);
-
-    /* Note that advanced parameters may be defined
-       using cs_join_set_advanced_param(),
-       just as for regular joinings or periodicities. */
-  }
 }
 
 /*----------------------------------------------------------------------------*/
