@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -28,7 +28,7 @@
 
 !> \file cs_user_boundary_conditions-advanced.f90
 !>
-!> Advanced example of cs_user_boundary_conditions.f90 subroutine
+!> \brief Advanced example of cs_user_boundary_conditions.f90 subroutine
 !>
 !-------------------------------------------------------------------------------
 
@@ -47,7 +47,7 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
@@ -60,7 +60,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
@@ -71,7 +71,7 @@
 !>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
-subroutine cs_user_boundary_conditions &
+subroutine cs_f_user_boundary_conditions &
  ( nvar   , nscal  ,                                              &
    icodcl , itrifb , itypfb , izfppp ,                            &
    dt     ,                                                       &
@@ -100,7 +100,6 @@ use ppincl
 use ppcpfu
 use atincl
 use ctincl
-use elincl
 use cs_fuel_incl
 use mesh
 use field
@@ -113,12 +112,12 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itrifb(nfabor), itypfb(nfabor)
 integer          izfppp(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -132,6 +131,7 @@ double precision xitur
 double precision xkent, xeent
 
 integer, allocatable, dimension(:) :: lstelt
+double precision, dimension(:), pointer :: boundary_roughness
 !< [loc_var_dec]
 
 !===============================================================================
@@ -256,6 +256,29 @@ do ilelt = 1, nlelt
 enddo
 !< [example_3]
 
+! Example of wall boundary condition with automatic continuous switch
+! between rough and smooth.
+
+!< [example_4]
+call field_get_val_s_by_name("boundary_roughness", boundary_roughness)
+
+call getfbr('6789', nlelt, lstelt)
+!==========
+do ilelt = 1, nlelt
+
+  ifac = lstelt(ilelt)
+
+  ! CAUTION: wall function keyword "iwallf" must be set to 6
+  !          in cs_user_parameters.
+
+  itypfb(ifac) = iparoi
+
+  ! Boundary roughtness (in meter)
+  boundary_roughness(ifac) = 0.05
+
+enddo
+!< [example_4]
+
 !--------
 ! Formats
 !--------
@@ -269,4 +292,4 @@ deallocate(lstelt)  ! temporary array for boundary faces selection
 !< [finalize]
 
 return
-end subroutine cs_user_boundary_conditions
+end subroutine cs_f_user_boundary_conditions
