@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -47,7 +47,7 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
@@ -60,7 +60,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
@@ -71,7 +71,7 @@
 !>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
-subroutine cs_user_boundary_conditions &
+subroutine cs_f_user_boundary_conditions &
  ( nvar   , nscal  ,                                              &
    icodcl , itrifb , itypfb , izfppp ,                            &
    dt     ,                                                       &
@@ -100,7 +100,6 @@ use ppincl
 use ppcpfu
 use atincl
 use ctincl
-use elincl
 use cs_fuel_incl
 use mesh
 use field
@@ -113,12 +112,12 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itrifb(nfabor), itypfb(nfabor)
 integer          izfppp(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -149,7 +148,7 @@ d2s3 = 2.d0/3.d0
 zref = 10.d0
 xuref = 10.d0
 rugd = 0.1d0
-rugt = 0.1d0
+rugt = rugd
 
 !===============================================================================
 ! Assign boundary conditions to boundary faces here
@@ -178,12 +177,18 @@ do ilelt = 1, nlelt
   ! - Zone to which the face belongs
   izfppp(ifac) = izone
 
-  ! - CAUTION: do not assign boundary condition type (itypfb(ifac))
-  !            as it is automatically set by the code.
-
   ! - Boundary conditions are prescribed from the meteo profile
   iprofm(izone) = 1
 
+  ! - boundary condition type can be set to ientre or i_convective_inlet
+
+  itypfb(ifac) = ientre
+
+  ! - automatic determination of type (inlet/outlet) according to sign of
+  !   mass flux
+
+  iautom(ifac) = 1
+
 enddo
 !< [example_1]
 
@@ -324,7 +329,6 @@ do ilelt = 1, nlelt
 
   ! Wall: zero flow (zero flux for pressure)
   !       rough friction for velocities (+ turbulent variables)
-  !       zero flux for scalars
 
   ! - Zone to which the zone belongs
   izfppp(ifac) = izone
@@ -334,24 +338,12 @@ do ilelt = 1, nlelt
   ! Roughness for velocity: rugd
   rcodcl(ifac,iu,3) = rugd
 
-  ! Roughness for scalars (if required):
-  ! rcodcl(ifac,iv,3) = rugd
+  ! Roughness for scalars:
+  rcodcl(ifac,iv,3) = rugt
+
+  ! By default zero flux for scalars
 
 
-  if(iscalt.ne.-1) then
-
-    ! If temperature prescribed to 20 with a rough wall law (scalar ii=1)
-    ! (with thermal roughness specified in rcodcl(ifac,iv,3)) :
-    ! ii = 1
-    ! icodcl(ifac, isca(ii))    = 6
-    ! rcodcl(ifac, isca(ii),1)  = 293.15d0
-
-    ! If flux prescribed to 4.d0 (scalar ii=2):
-    ! ii = 2
-    ! icodcl(ifac, isca(ii))    = 3
-    ! rcodcl(ifac, isca(ii), 3) = 4.D0
-
-  endif
 enddo
 !< [example_4]
 
@@ -386,4 +378,4 @@ enddo
 deallocate(lstelt)  ! temporary array for boundary faces selection
 
 return
-end subroutine cs_user_boundary_conditions
+end subroutine cs_f_user_boundary_conditions
