@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,7 @@
 ! Function:
 ! ---------
 !> \file  cs_user_boundary_conditions-compressible.f90
-!> \brief Example of cs_user_boundary_conditions.f90 for compressible
+!> \brief Example of cs_f_user_boundary_conditions.f90 for compressible
 !
 !-------------------------------------------------------------------------------
 
@@ -45,7 +45,7 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
@@ -58,7 +58,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
@@ -69,7 +69,7 @@
 !>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
-subroutine cs_user_boundary_conditions &
+subroutine cs_f_user_boundary_conditions &
  ( nvar   , nscal  ,                                              &
    icodcl , itrifb , itypfb , izfppp ,                            &
    dt     ,                                                       &
@@ -98,10 +98,10 @@ use ppincl
 use ppcpfu
 use atincl
 use ctincl
-use elincl
 use cs_fuel_incl
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -111,12 +111,12 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itrifb(nfabor), itypfb(nfabor)
 integer          izfppp(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -125,8 +125,8 @@ integer          ifac  , iel   , ii
 integer          izone , iutile
 integer          ilelt, nlelt
 
-double precision uref2 , dhyd  , rhomoy
-double precision ustar2, xkent , xeent , d2s3
+double precision uref2 , dhy  , rhomoy
+double precision d2s3
 
 integer, allocatable, dimension(:) :: lstelt
 double precision, dimension(:), pointer ::  cpro_rom
@@ -218,62 +218,20 @@ do ilelt = 1, nlelt
   !   We will also be careful if necessary to use a more precise
   !     formula for the dynamic viscosity use in the calculation of
   !     the Reynolds number (especially if it is variable, it may be
-  !     useful to take the law from 'usphyv'. Here, we use by default
+  !     useful to take the law from 'cs_user_physical_properties'. Here, we use by default
   !     the 'viscl0" value.
   !   Regarding the density, we have access to its value at boundary
   !     faces (romb) so this value is the one used here (specifically,
-  !     it is consistent with the processing in 'usphyv', in case of
+  !     it is consistent with the processing in 'cs_user_physical_properties', in case of
   !     variable density)
 
   !     Hydraulic diameter
-  dhyd   = 0.075d0
+  dhy   = 0.075d0
 
-  !   Calculation of friction velocity squared (ustar2)
-  !     and of k and epsilon at the inlet (xkent and xeent) using
-  !     standard laws for a circular pipe
-  !     (their initialization is not needed here but is good practice).
   rhomoy = cpro_rom(iel)
-  ustar2 = 0.d0
-  xkent  = epzero
-  xeent  = epzero
 
-  call keendb                                              &
-  !==========
-    ( uref2, dhyd, rhomoy, viscl0, cmu, xkappa,            &
-      ustar2, xkent, xeent )
-
-  if    (itytur.eq.2) then
-
-    rcodcl(ifac,ik,1)  = xkent
-    rcodcl(ifac,iep,1) = xeent
-
-  elseif(itytur.eq.3) then
-
-    rcodcl(ifac,ir11,1) = 2.d0/3.d0*xkent
-    rcodcl(ifac,ir22,1) = 2.d0/3.d0*xkent
-    rcodcl(ifac,ir33,1) = 2.d0/3.d0*xkent
-    rcodcl(ifac,ir12,1) = 0.d0
-    rcodcl(ifac,ir13,1) = 0.d0
-    rcodcl(ifac,ir23,1) = 0.d0
-    rcodcl(ifac,iep,1)  = xeent
-
-  elseif(iturb.eq.50) then
-
-    rcodcl(ifac,ik,1)   = xkent
-    rcodcl(ifac,iep,1)  = xeent
-    rcodcl(ifac,iphi,1) = d2s3
-    rcodcl(ifac,ifb,1)  = 0.d0
-
-  elseif(iturb.eq.60) then
-
-    rcodcl(ifac,ik,1)   = xkent
-    rcodcl(ifac,iomg,1) = xeent/cmu/xkent
-
-  elseif(iturb.eq.70) then
-
-    rcodcl(ifac,inusa,1) = cmu*xkent**2/xeent
-
-  endif
+  call turbulence_bc_inlet_hyd_diam &
+  ( ifac, uref2, dhy, rhomoy, viscl0, rcodcl )
 
   ! Handle scalars
   ! (do not loop on nscal to avoid modifying rho and energy)
@@ -376,6 +334,11 @@ do ilelt = 1, nlelt
   ! Total enthalpy
   rcodcl(ifac,isca(ienerg),1) = 294465.d0
 
+  ! Direction of the velocity: normal to inlet faces
+  rcodcl(ifac,iu,1) = -surfbo(1,ifac)
+  rcodcl(ifac,iv,1) = -surfbo(2,ifac)
+  rcodcl(ifac,iw,1) = -surfbo(3,ifac)
+
   ! Turbulence (no turbulence)
 
   ! Handle scalars
@@ -510,4 +473,4 @@ enddo
 deallocate(lstelt)  ! temporary array for boundary faces selection
 
 return
-end subroutine cs_user_boundary_conditions
+end subroutine cs_f_user_boundary_conditions
