@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,7 @@
 ! Function:
 ! ---------
 !> \file  cs_user_boundary_conditions-electric_arcs.f90
-!> \brief Example of cs_user_boundary_conditions subroutine for electric arcs
+!> \brief Example of cs_f_user_boundary_conditions subroutine for electric arcs
 !
 !-------------------------------------------------------------------------------
 
@@ -45,7 +45,7 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
@@ -58,7 +58,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
@@ -69,7 +69,7 @@
 !>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
-subroutine cs_user_boundary_conditions &
+subroutine cs_f_user_boundary_conditions &
  ( nvar   , nscal  ,                                              &
    icodcl , itrifb , itypfb , izfppp ,                            &
    dt     ,                                                       &
@@ -98,10 +98,10 @@ use ppincl
 use ppcpfu
 use atincl
 use ctincl
-use elincl
 use cs_fuel_incl
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -111,12 +111,12 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itrifb(nfabor), itypfb(nfabor)
 integer          izfppp(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -127,13 +127,15 @@ integer          izone,iesp
 integer          ilelt, nlelt
 
 double precision uref2, d2s3
-double precision rhomoy, dhy, ustar2
-double precision xkent, xeent
+double precision rhomoy, dhy
 double precision z1   , z2
+integer          ipotr, ipoti, f_id, ipotva
+character(len=80) :: f_name
 
 integer, allocatable, dimension(:) :: lstelt
 double precision, dimension(:), pointer :: bfpro_rom
-double precision, dimension(:), pointer :: cvara_potva
+double precision, dimension(:,:), pointer :: cvara_potva
+integer :: keyvar
 !< [loc_var_dec]
 
 !===============================================================================
@@ -144,8 +146,21 @@ double precision, dimension(:), pointer :: cvara_potva
 
 allocate(lstelt(nfabor))  ! temporary array for boundary faces selection
 
+call field_get_key_id("variable_id", keyvar)
+
 d2s3 = 2.d0/3.d0
 
+call field_get_id('elec_pot_r', f_id)
+call field_get_key_int(f_id, keyvar, ipotr)
+if (ippmod(ieljou).ge. 2) then
+  call field_get_id('elec_pot_i', f_id)
+  call field_get_key_int(f_id, keyvar, ipoti)
+endif
+if (ippmod(ielarc).ge.2) then
+  call field_get_id('vec_potential', f_id)
+  call field_get_key_int(f_id, keyvar, ipotva)
+endif
+
 !===============================================================================
 ! Assign boundary conditions to boundary faces here
 
@@ -202,64 +217,21 @@ do ilelt = 1, nlelt
     ! We will also be careful if necessary to use a more precise
     ! formula for the dynamic viscosity use in the calculation of
     ! the Reynolds number (especially if it is variable, it may be
-    ! useful to take the law from 'usphyv'. Here, we use by default
+    ! useful to take the law from 'cs_user_physical_properties'. Here, we use by default
     ! the 'viscl0" value.
 
     ! Regarding the density, we have acess to its value at boundary
     ! faces (romb) so this value is the one used here (specifically,
-    ! it is consistent with the processing in 'usphyv', in case of
+    ! it is consistent with the processing in 'cs_user_physical_properties', in case of
     ! variable density)
 
     ! Hydraulic diameter
     dhy     = 0.075d0
 
-    ! Calculation of friction velocity squared (ustar2)
-    ! and of k and epsilon at the inlet (xkent and xeent) using
-    ! standard laws for a circular pipe
-    ! (their initialization is not needed here but is good practice).
-
     rhomoy = bfpro_rom(ifac)
-    ustar2 = 0.d0
-    xkent  = epzero
-    xeent  = epzero
 
-    call keendb                                            &
-    !==========
-     ( uref2, dhy, rhomoy, viscl0, cmu, xkappa,            &
-       ustar2, xkent, xeent)
-
-    if (itytur.eq.2) then
-
-      rcodcl(ifac,ik,1)  = xkent
-      rcodcl(ifac,iep,1) = xeent
-
-    elseif (itytur.eq.3) then
-
-      rcodcl(ifac,ir11,1) = d2s3*xkent
-      rcodcl(ifac,ir22,1) = d2s3*xkent
-      rcodcl(ifac,ir33,1) = d2s3*xkent
-      rcodcl(ifac,ir12,1) = 0.d0
-      rcodcl(ifac,ir13,1) = 0.d0
-      rcodcl(ifac,ir23,1) = 0.d0
-      rcodcl(ifac,iep,1)  = xeent
-
-    elseif (iturb.eq.50) then
-
-      rcodcl(ifac,ik,1)   = xkent
-      rcodcl(ifac,iep,1)  = xeent
-      rcodcl(ifac,iphi,1) = d2s3
-      rcodcl(ifac,ifb,1)  = 0.d0
-
-    elseif (iturb.eq.60) then
-
-      rcodcl(ifac,ik,1)   = xkent
-      rcodcl(ifac,iomg,1) = xeent/cmu/xkent
-
-    elseif (iturb.eq.70) then
-
-      rcodcl(ifac,inusa,1) = cmu*xkent**2/xeent
-
-    endif
+    call turbulence_bc_inlet_hyd_diam &
+  ( ifac, uref2, dhy, rhomoy, viscl0, rcodcl )
 
   endif
 
@@ -284,17 +256,18 @@ do ilelt = 1, nlelt
   ! (What is important for electric arc is the difference between anode and
   ! cathode potentials)
 
-  ii = ipotr
-  icodcl(ifac,isca(ii))   = 1
-  rcodcl(ifac,isca(ii),1) = 0.d0
+  icodcl(ifac,ipotr)   = 1
+  rcodcl(ifac,ipotr,1) = 0.d0
 
   ! Mass fraction of the (n-1) gas mixture components
 
-  if (ngazg .gt. 1) then
-    do iesp=1,ngazg-1
-      ii = iycoel(iesp)
-      icodcl(ifac,isca(ii))   = 1
-      rcodcl(ifac,isca(ii),1) = 0.d0
+  if (ngazge .gt. 1) then
+    do iesp=1,ngazge-1
+      write(f_name,'(a13,i2.2)') 'esl_fraction_',iesp
+      call field_get_id(trim(f_name), f_id)
+      call field_get_key_int(f_id, keyvar, ii)
+      icodcl(ifac,ii)   = 1
+      rcodcl(ifac,ii,1) = 0.d0
     enddo
   endif
 
@@ -302,9 +275,8 @@ do ilelt = 1, nlelt
   ! Imaginary part of the potentiel (ipoti) is imposed to zero
 
   if (ippmod(ieljou).ge. 2) then
-    ii = ipoti
-    icodcl(ifac,isca(ii))   = 1
-    rcodcl(ifac,isca(ii),1) = 0.d0
+    icodcl(ifac,ipoti)   = 1
+    rcodcl(ifac,ipoti,1) = 0.d0
   endif
 
   ! Specific model for Electric arc:
@@ -319,11 +291,12 @@ do ilelt = 1, nlelt
   ! the electrodes (see above)
 
   if (ippmod(ielarc).ge.2) then
-    do idim= 1,ndimve
-      ii = ipotva(idim)
-      icodcl(ifac,isca(ii))   = 3
-      rcodcl(ifac,isca(ii),3) = 0.d0
-    enddo
+      icodcl(ifac,ipotva    )   = 3
+      rcodcl(ifac,ipotva    ,3) = 0.d0
+      icodcl(ifac,ipotva + 1)   = 3
+      rcodcl(ifac,ipotva + 1,3) = 0.d0
+      icodcl(ifac,ipotva + 2)   = 3
+      rcodcl(ifac,ipotva + 2,3) = 0.d0
   endif
 
 enddo
@@ -363,8 +336,8 @@ do ilelt = 1, nlelt
   ! Specific model for Joule Effect by direct conduction:
 
   ! If you want to make a simulation with an imposed Power PUISIM
-  ! (you want to get PUISIM imposed in useli1 and PUISIM = Amp x Volt)
-  ! you need to impose IELCOR=1 in useli1
+  ! (you want to get PUISIM imposed in cs_user_parameters.c and PUISIM = Amp x Volt)
+  ! you need to impose IELCOR=1 in cs_user_parameters.c
   ! The boundary conditions will be scaled by COEJOU coefficient
   ! for example the electrical potential will be multiplied bu COEJOU
   ! (both real and imaginary part of the electrical potential if needed)
@@ -379,22 +352,20 @@ do ilelt = 1, nlelt
   ! you can impose directly the value.
 
   if (ippmod(ieljou).ge. 1) then
-    ii = ipotr
-    icodcl(ifac,isca(ii))   = 1
+    icodcl(ifac,ipotr)   = 1
     if (ielcor.eq.1) then
-      rcodcl(ifac,isca(ii),1) = 500.d0*coejou
+      rcodcl(ifac,ipotr,1) = 500.d0*coejou
     else
-      rcodcl(ifac,isca(ii),1) = 500.d0
+      rcodcl(ifac,ipotr,1) = 500.d0
     endif
   endif
 
   if (ippmod(ieljou).ge. 2) then
-    ii = ipoti
-    icodcl(ifac,isca(ii))   = 1
+    icodcl(ifac,ipoti)   = 1
     if (ielcor.eq.1) then
-      rcodcl(ifac,isca(ii),1) = sqrt(3.d0)*500.d0*coejou
+      rcodcl(ifac,ipoti,1) = sqrt(3.d0)*500.d0*coejou
     else
-      rcodcl(ifac,isca(ii),1) = sqrt(3.d0)*500.d0
+      rcodcl(ifac,ipoti,1) = sqrt(3.d0)*500.d0
     endif
   endif
 
@@ -435,18 +406,18 @@ do ilelt = 1, nlelt
   !    intensity of the electrical current is imposed (COUIMP is the value of
   !    the imposed total current).
 
-  !    In that case, you need to impose IELCOR=1 in useli1
+  !    In that case, you need to impose IELCOR=1 in cs_user_parameters.c
   !    The "electrical variables" will be scaled by COEPOT coefficient :
   !    for example the electrical potential will be multiplied by COEPOT,
-  !    Joule effect will be multipied by COEPOT * COEPOT and so on (see uselrc.f90)
+  !    Joule effect will be multipied by COEPOT * COEPOT and so on (see cs_user_electric_scaling.c)
 
-  !    COEJOU is defined in uselrc.fr : different possibilities are described in
-  !    uselrc.f90, depending on the different physics you want to simulate
+  !    COEJOU is defined in cs_user_electric_scaling.c : different possibilities are described in
+  !    cs_user_electric_scaling.c, depending on the different physics you want to simulate
   !    (scaling from current, from power, special model for restriking ...)
 
   !    The variable DPOT is defined: it corresponds to the electrical potential
   !    difference between the electrodes (Anode potential - cathode Potential).
-  !    DPOT is calculated in uselrc.f90. DPOT is saved at each time step, and
+  !    DPOT is calculated in cs_user_electric_scaling.c. DPOT is saved at each time step, and
   !    for a following calculation
 
   !    DPOT is the value of the boundary condition on anode assuming that
@@ -455,13 +426,12 @@ do ilelt = 1, nlelt
   ! *  It is also possible to fix the value of the potential on the anode.
   !    (for example, 1000 Volts).
 
-  ii = ipotr
-  icodcl(ifac,isca(ii))   = 1
+  icodcl(ifac,ipotr)   = 1
 
   if (ippmod(ielarc).ge.1 .and. ielcor .eq.1) then
-    rcodcl(ifac,isca(ii),1) = dpot
+    rcodcl(ifac,ipotr,1) = pot_diff
   else
-    rcodcl(ifac,isca(ii),1) = 1000.d0
+    rcodcl(ifac,ipotr,1) = 1000.d0
   endif
 
   ! Mass fraction of the (n-1) gas mixture components
@@ -540,13 +510,16 @@ do ilelt = 1, nlelt
         cdgfbo(1,ifac) .ge.  2.249d-2  .or.                      &
         cdgfbo(3,ifac) .le. -2.249d-2  .or.                      &
         cdgfbo(3,ifac) .ge.  2.249d-2      ) then
+      call field_get_val_prev_v_by_name('vec_potential', cvara_potva)
       iel = ifabor(ifac)
-      do idim = 1, ndimve
-        ii = ipotva(idim)
-        call field_get_val_prev_s(ivarfl(isca(ii)), cvara_potva)
-        icodcl(ifac,isca(ii))   = 1
-        rcodcl(ifac,isca(ii),1) = cvara_potva(iel)
-      enddo
+      icodcl(ifac,ipotva)   = 1
+      rcodcl(ifac,ipotva,1) = cvara_potva(1, iel)
+
+      icodcl(ifac,ipotva + 1)   = 1
+      rcodcl(ifac,ipotva + 1,1) = cvara_potva(2, iel)
+
+      icodcl(ifac,ipotva + 2)   = 1
+      rcodcl(ifac,ipotva + 2,1) = cvara_potva(3, iel)
     endif
   endif
 
@@ -583,33 +556,31 @@ do ilelt = 1, nlelt
   rcodcl(ifac,isca(ii),2) = 1.d5
 
   ! Real electrical potential: anode boundary condition;
-  ! dpot calculated in uselrc.f
+  ! pot_diff calculated in cs_user_electric_scaling.c
 
-  ii = ipotr
-  icodcl(ifac,isca(ii))   = 1
+  icodcl(ifac,ipotr)   = 1
 
   if (ippmod(ielarc).ge.1 .and. ielcor .eq.1) then
-    rcodcl(ifac,isca(ii),1) = dpot
+    rcodcl(ifac,ipotr,1) = pot_diff
   else
-    rcodcl(ifac,isca(ii),1) = 100.d0
+    rcodcl(ifac,ipotr,1) = 100.d0
   endif
 
   ! Restriking modeling:
   ! ===================
   !    example to fit depending on the case, the geometry etc...
-  !     and also in agreement with uselrc.fr
+  !     and also in agreement with cs_user_electric_scaling.c
 
   if (ippmod(ielarc).ge.1 .and. ielcor .eq.1) then
-    if (iclaq.eq.1 .and. ntcabs.le.ntdcla+30) then
-
-      z1 = zclaq - 2.d-4
+    if (irestrike.eq.1 .and. ntcabs.le.ntdcla+30) then
+      z1 = restrike_point_z - 2.d-4
       if (z1.le.0.d0) z1 = 0.d0
-      z2 = zclaq + 2.d-4
+      z2 = restrike_point_z + 2.d-4
       if (z2.ge.2.d-2) z2 = 2.d-2
 
       if (cdgfbo(3,ifac).ge.z1 .and. cdgfbo(3,ifac).le.z2) then
         icodcl(ifac,isca(ii))   = 1
-        rcodcl(ifac,isca(ii),1) = dpot
+        rcodcl(ifac,isca(ii),1) = pot_diff
       else
         icodcl(ifac,isca(ii))   = 3
         rcodcl(ifac,isca(ii),3) = 0.d0
@@ -620,11 +591,12 @@ do ilelt = 1, nlelt
   ! Vector potential : Zero flux
 
   if (ippmod(ielarc).ge.2) then
-    do idim= 1,ndimve
-      ii = ipotva(idim)
-      icodcl(ifac,isca(ii))   = 3
-      rcodcl(ifac,isca(ii),3) = 0.d0
-    enddo
+      icodcl(ifac,ipotva    )   = 3
+      rcodcl(ifac,ipotva    ,3) = 0.d0
+      icodcl(ifac,ipotva + 1)   = 3
+      rcodcl(ifac,ipotva + 1,3) = 0.d0
+      icodcl(ifac,ipotva + 2)   = 3
+      rcodcl(ifac,ipotva + 2,3) = 0.d0
   endif
 
 enddo
@@ -659,9 +631,8 @@ do ilelt = 1, nlelt
   ! electrical potential depending on the electrode configuration:
 
   if (ippmod(ieljou).ge. 2) then
-    ii = ipoti
-    icodcl(ifac,isca(ii))   = 1
-    rcodcl(ifac,isca(ii),1) = 0.d0
+    icodcl(ifac,ipoti)   = 1
+    rcodcl(ifac,ipoti,1) = 0.d0
   endif
 
 enddo
@@ -678,4 +649,4 @@ enddo
 deallocate(lstelt)  ! temporary array for boundary faces selection
 
 return
-end subroutine cs_user_boundary_conditions
+end subroutine cs_f_user_boundary_conditions
