@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,7 @@
 ! Function:
 ! ---------
 !> \file  cs_user_boundary_conditions-fuel.f90
-!> \brief Example of cs_user_boundary_conditions subroutine.f90 for fuel
+!> \brief Example of cs_f_user_boundary_conditions subroutine.f90 for fuel
 !
 !-------------------------------------------------------------------------------
 
@@ -45,7 +45,7 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
@@ -58,7 +58,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
@@ -69,7 +69,7 @@
 !>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
-subroutine cs_user_boundary_conditions &
+subroutine cs_f_user_boundary_conditions &
  ( nvar   , nscal  ,                                              &
    icodcl , itrifb , itypfb , izfppp ,                            &
    dt     ,                                                       &
@@ -98,7 +98,6 @@ use ppincl
 use ppcpfu
 use atincl
 use ctincl
-use elincl
 use cs_fuel_incl
 use mesh
 use field
@@ -111,12 +110,12 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itrifb(nfabor), itypfb(nfabor)
 integer          izfppp(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -200,7 +199,7 @@ do ilelt = 1, nlelt
 !    - If ICALKE = 0 the boundary conditions of turbulence at
 !      the inlet are calculated as follows:
 
-  if(icalke(izone).eq.0) then
+  if (icalke(izone).eq.0) then
 
     uref2 = rcodcl(ifac,iu,1)**2                           &
            +rcodcl(ifac,iv,1)**2                           &
@@ -209,11 +208,6 @@ do ilelt = 1, nlelt
     xkent  = epzero
     xeent  = epzero
 
-    call keenin                                                   &
-    !==========
-      ( uref2, xintur(izone), dh(izone), cmu, xkappa,             &
-        xkent, xeent )
-
     if    (itytur.eq.2) then
 
       rcodcl(ifac,ik,1)  = xkent
@@ -244,8 +238,6 @@ do ilelt = 1, nlelt
     endif
 
   endif
-!
-!    - If ICALKE = 1 the boundary conditions of turbulence at
 !      the inlet refer to both, a hydraulic diameter and a
 !      reference velocity.
 !
@@ -331,11 +323,6 @@ do ilelt = 1, nlelt
     xkent  = epzero
     xeent  = epzero
 
-    call keenin                                                   &
-    !==========
-      ( uref2, xintur(izone), dh(izone), cmu, xkappa,             &
-        xkent, xeent )
-
     if    (itytur.eq.2) then
 
       rcodcl(ifac,ik,1)  = xkent
@@ -456,9 +443,6 @@ do ilelt = 1, nlelt
 enddo
 !< [example_5]
 
-! Deallocate the temporary array
-deallocate(lstelt)
-
 !--------
 ! Formats
 !--------
@@ -470,4 +454,4 @@ deallocate(lstelt)
 deallocate(lstelt)  ! temporary array for boundary faces selection
 
 return
-end subroutine cs_user_boundary_conditions
+end subroutine cs_f_user_boundary_conditions
