@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,8 @@
 ! Function:
 ! ---------
 !> \file  cs_user_boundary_conditions-gas_libby_williams.f90
-!> \brief Example of cs_user_boundary_conditions subroutine.f90 for Libby-Williams gas
+!> \brief Example of cs_f_user_boundary_conditions subroutine.f90 for
+!>        Libby-Williams gas
 !
 !-------------------------------------------------------------------------------
 
@@ -45,7 +46,7 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
@@ -58,7 +59,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
@@ -69,7 +70,7 @@
 !>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
-subroutine cs_user_boundary_conditions &
+subroutine cs_f_user_boundary_conditions &
  ( nvar   , nscal  ,                                              &
    icodcl , itrifb , itypfb , izfppp ,                            &
    dt     ,                                                       &
@@ -98,7 +99,6 @@ use ppincl
 use ppcpfu
 use atincl
 use ctincl
-use elincl
 use cs_fuel_incl
 use mesh
 use field
@@ -111,12 +111,12 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itrifb(nfabor), itypfb(nfabor)
 integer          izfppp(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -148,7 +148,7 @@ d2s3 = 2.d0/3.d0
 !   - set the boundary condition for each face
 !===============================================================================
 
-! Definition of a burned gas inlet (pilot flame) for each face of colour 11
+! Definition of a burned gas inlet (pilot flame) for each face of color 11
 
 !< [example_1]
 call getfbr('11', nlelt, lstelt)
@@ -201,11 +201,6 @@ do ilelt = 1, nlelt
     xkent  = epzero
     xeent  = epzero
 
-    call keenin                                                   &
-    !==========
-      ( uref2, xintur(izone), dh(izone), cmu, xkappa,             &
-        xkent, xeent )
-
     if    (itytur.eq.2) then
 
       rcodcl(ifac,ik,1)  = xkent
@@ -255,7 +250,7 @@ do ilelt = 1, nlelt
 enddo
 !< [example_1]
 
-! Definition of an unburned gas inlet for each face of colour 12
+! Definition of an unburned gas inlet for each face of color 12
 
 !< [example_2]
 call getfbr('12', nlelt, lstelt)
@@ -311,7 +306,7 @@ do ilelt = 1, nlelt
 enddo
 !< [example_2]
 
-! Definition of a wall for each face of colour 51 and 5
+! Definition of a wall for each face of color 51 and 5
 
 !< [example_3]
 call getfbr('51 or 5', nlelt, lstelt)
@@ -333,7 +328,7 @@ do ilelt = 1, nlelt
 enddo
 !< [example_3]
 
-! Definition of an exit for each face of colour 91 and 9
+! Definition of an exit for each face of color 91 and 9
 
 !< [example_4]
 call getfbr('91 or 9', nlelt, lstelt)
@@ -356,7 +351,7 @@ enddo
 !< [example_4]
 
 ! Definition of symmetric boundary conditions for each
-! face of colour 41 and 4.
+! face of color 41 and 4.
 
 !< [example_5]
 call getfbr('41 or 4', nlelt, lstelt)
@@ -389,4 +384,4 @@ enddo
 deallocate(lstelt)  ! temporary array for boundary faces selection
 
 return
-end subroutine cs_user_boundary_conditions
+end subroutine cs_f_user_boundary_conditions
