@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,7 +26,8 @@
 ! Function:
 ! ---------
 !> \file  cs_user_boundary_conditions-pulverized_coal.f90
-!> \brief Example of cs_user_boundary_conditions subroutine.f90 for pulverized coal
+!> \brief Example of cs_f_user_boundary_conditions subroutine.f90
+!>        for pulverized coal
 !
 !-------------------------------------------------------------------------------
 
@@ -45,7 +46,7 @@
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 5 smooth wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
-!>                               - 6 rought wall and
+!>                               - 6 rough wall and
 !>                                 \f$ \vect{u} \cdot \vect{n} = 0 \f$
 !>                               - 9 free inlet/outlet
 !>                                 (input mass flux blocked to 0)
@@ -58,7 +59,7 @@
 !>                               - rcodcl(2) value of the exterior exchange
 !>                                 coefficient (infinite if no exchange)
 !>                               - rcodcl(3) value flux density
-!>                                 (negative if gain) in w/m2 or roughtness
+!>                                 (negative if gain) in w/m2 or roughness
 !>                                 in m if icodcl=6
 !>                                 -# for the velocity \f$ (\mu+\mu_T)
 !>                                    \gradt \, \vect{u} \cdot \vect{n}  \f$
@@ -69,7 +70,7 @@
 !>                                     \grad T \cdot \vect{n} \f$
 !_______________________________________________________________________________
 
-subroutine cs_user_boundary_conditions &
+subroutine cs_f_user_boundary_conditions &
  ( nvar   , nscal  ,                                              &
    icodcl , itrifb , itypfb , izfppp ,                            &
    dt     ,                                                       &
@@ -98,7 +99,6 @@ use ppincl
 use ppcpfu
 use atincl
 use ctincl
-use elincl
 use cs_fuel_incl
 use mesh
 use field
@@ -111,12 +111,12 @@ implicit none
 
 integer          nvar   , nscal
 
-integer          icodcl(nfabor,nvarcl)
+integer          icodcl(nfabor,nvar)
 integer          itrifb(nfabor), itypfb(nfabor)
 integer          izfppp(nfabor)
 
 double precision dt(ncelet)
-double precision rcodcl(nfabor,nvarcl,3)
+double precision rcodcl(nfabor,nvar,3)
 
 ! Local variables
 
@@ -206,11 +206,6 @@ do ilelt = 1, nlelt
     xkent  = epzero
     xeent  = epzero
 
-    call keenin                                                   &
-    !==========
-      ( uref2, xintur(izone), dh(izone), cmu, xkappa,             &
-        xkent, xeent )
-
     if    (itytur.eq.2) then
 
       rcodcl(ifac,ik,1)  = xkent
@@ -301,7 +296,7 @@ do ilelt = 1, nlelt
 !      - Oxidizer's Temperature in K
   timpat(izone) = 800.d0  + tkelvi
 
-!        Coal inlet, initialisation
+!        Coal inlet, initialization
   do icha = 1, ncharm
     qimpcp(izone,icha) = zero
     timpcp(izone,icha) = zero
@@ -442,4 +437,4 @@ enddo
 deallocate(lstelt)  ! temporary array for boundary faces selection
 
 return
-end subroutine cs_user_boundary_conditions
+end subroutine cs_f_user_boundary_conditions
