@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,9 +26,17 @@
 ! Purpose:
 ! -------
 
-! \file cs_user_extra_operations-force_temperature.f90
-! This is an example of cs_user_extra_operations.f90 which
-! performs forced temperature
+!> \file cs_user_extra_operations-force_temperature.f90
+!>
+!> \brief This function is called at the end of each time step, and has a very
+!>  general purpose
+!>  (i.e. anything that does not have another dedicated user subroutine)
+!>
+!> See \subpage cs_user_extra_operations_examples and
+!> \subpage cs_user_extra_operations-nusselt_calculation for examples.
+!>
+!> This is an example of cs_user_extra_operations.f90 which
+!> performs forced temperature
 
 !-------------------------------------------------------------------------------
 
@@ -60,9 +68,7 @@ use optcal
 use cstphy
 use cstnum
 use entsor
-use lagpar
 use lagran
-use lagdim
 use parall
 use period
 use ppppar
