@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,9 +26,17 @@
 ! Purpose:
 ! -------
 
-! \file cs_user_extra_operations-parallel_operations.f90
-! This is an example of cs_user_extra_operations.f90 which
-! performs parallel operations
+!> \file cs_user_extra_operations-parallel_operations.f90
+!>
+!> \brief This function is called at the end of each time step, and has a very
+!>  general purpose
+!>  (i.e. anything that does not have another dedicated user subroutine)
+!>
+!> See \subpage cs_user_extra_operations_examples and
+!> \subpage cs_user_extra_operations-nusselt_calculation for examples.
+!>
+!> This is an example of cs_user_extra_operations.f90 which
+!> performs parallel operations
 
 !-------------------------------------------------------------------------------
 
@@ -60,9 +68,7 @@ use optcal
 use cstphy
 use cstnum
 use entsor
-use lagpar
 use lagran
-use lagdim
 use parall
 use period
 use ppppar
@@ -293,10 +299,10 @@ if (irangp.ge.0) then
 endif
 ! print the global maxima
 write(nfecra,5090) itab(1), itab(2), itab(3)
- 5090 format(' cs_user_extra_operations: Max. number of cells per proc. =    &
-             ', i10, /,  &
-             '         Max. number of interior faces per proc. = ', i10, /,  &
-             '         Max. number of boundary faces per proc. = ', i10)
+ 5090 format(' cs_user_extra_operations: Max. number of cells per rank = ',  &
+             i10, /,  &
+             '         Max. number of interior faces per rank = ', i10, /,   &
+             '         Max. number of boundary faces per rank = ', i10)
 !< [example_9]
 
 ! Minima from an array of integers;
@@ -314,10 +320,10 @@ if (irangp.ge.0) then
 endif
 ! print the global minima
 write(nfecra,5100) itab(1), itab(2), itab(3)
- 5100 format(' cs_user_extra_operations: Min. number of cells per proc. =    &
-             ', i10, /,  &
-             '         Min. number of interior faces per proc. = ', i10, /,  &
-             '         Min. number of boundary faces per proc. = ', i10)
+ 5100 format(' cs_user_extra_operations: Min. number of cells per rank = ',  &
+             i10, /,  &
+             '         Min. number of interior faces per rank = ', i10, /,   &
+             '         Min. number of boundary faces per rank = ', i10)
 !< [example_10]
 
 ! Sum of an array of reals;
