@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,8 +26,12 @@
 ! Purpose:
 ! -------
 
-!> \file cs_user_initialization.f90 atmospheric example
+!> \file cs_user_initialization-atmospheric.f90
 !>
+!> \brief Atmospheric example
+!>
+!> See \subpage cs_user_initialization for examples.
+!
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -41,7 +45,7 @@
 !_______________________________________________________________________________
 
 
-subroutine cs_user_initialization &
+subroutine cs_user_f_initialization &
 !================================
 
  ( nvar   , nscal  ,                                              &
@@ -69,7 +73,6 @@ use cpincl
 use ppincl
 use atincl
 use ctincl
-use elincl
 use ppcpfu
 use cs_coal_incl
 use cs_fuel_incl
@@ -239,4 +242,4 @@ endif
 deallocate(lstelt)
 
 return
-end subroutine cs_user_initialization
+end subroutine cs_user_f_initialization
