@@ -4,7 +4,7 @@
 !                      --------------------------
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,8 +27,11 @@
 ! -------
 
 !> \file cs_user_initialization-compressible.f90
+!>
 !> \brief Compressible example
 !>
+!> See \subpage cs_user_initialization for examples.
+!
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -42,7 +45,7 @@
 !_______________________________________________________________________________
 
 
-subroutine cs_user_initialization &
+subroutine cs_user_f_initialization &
  ( nvar   , nscal  ,                                              &
    dt     )
 
@@ -68,7 +71,6 @@ use cpincl
 use ppincl
 use atincl
 use ctincl
-use elincl
 use ppcpfu
 use cs_coal_incl
 use cs_fuel_incl
@@ -230,4 +232,4 @@ deallocate(w1, w2, w3, w4)
 !< [finalize]
 
 return
-end subroutine cs_user_initialization
+end subroutine cs_user_f_initialization
