@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,8 +27,11 @@
 ! -------
 
 !> \file cs_user_initialization-cooling_towers.f90
+!>
 !> \brief Cooling towers example
 !>
+!> See \subpage cs_user_initialization for examples.
+!
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -42,7 +45,7 @@
 !_______________________________________________________________________________
 
 
-subroutine cs_user_initialization &
+subroutine cs_user_f_initialization &
  ( nvar   , nscal  ,                                              &
    dt     )
 
@@ -68,7 +71,6 @@ use cpincl
 use ppincl
 use atincl
 use ctincl
-use elincl
 use ppcpfu
 use cs_coal_incl
 use cs_fuel_incl
@@ -96,7 +98,7 @@ double precision d2s3
 double precision, dimension(:,:), pointer :: vel
 
 integer, allocatable, dimension(:) :: lstelt
-double precision, dimension(:), pointer :: cvar_temp4, cvar_humid
+double precision, dimension(:), pointer :: cvar_temp, cpro_humid
 !< [loc_var_dec]
 
 !===============================================================================
@@ -121,27 +123,25 @@ call field_get_val_v(ivarfl(iu), vel)
 
 if (isuite.eq.0) then
 
-!   --- Initialisation de la temperature de l'air a 11 deg Celsius
-!                      de l'humidite de l'air a 0.0063
-!       pour toutes les cellules
+!   --- Initialize tmperature of humid air at 11 deg Celsius
+!       and of humidity at 0.0063
 
-  call field_get_val_s(ivarfl(isca(itemp4)), cvar_temp4)
-  call field_get_val_s(ivarfl(isca(ihumid)), cvar_humid)
+  call field_get_val_s(ivarfl(isca(iscalt)), cvar_temp)
+  call field_get_val_s(ihumid, cpro_humid)!FIXME
 
   do iel = 1, ncel
 
-    cvar_temp4(iel) = 11.d0
-    cvar_humid(iel) = 0.0063d0
+    cvar_temp(iel) = 11.d0
+    cpro_humid(iel) = 0.0063d0
 
   enddo
 
-!   --- Initialisation de la temperature de l'air a 20 deg Celsius
-!                      de l'humidite de l'air a 0.012
-!                      de la vitesse
-!       uniquement pour les cellules de couleur 6
+!   --- Initialize tmperature of humid air at 20 deg Celsius
+!       and of humidity at 0.012
+!       and of velocity at 0.5 m/s
+!       for cells of color 6
 
   call getcel('6', nlelt, lstelt)
-  !==========
 
   do ilelt = 1, nlelt
 
@@ -149,8 +149,8 @@ if (isuite.eq.0) then
 
     vel(1,iel) = -0.5d0
 
-    cvar_temp4(iel) = 20.d0
-    cvar_humid(iel) = 0.012d0
+    cvar_temp(iel) = 20.d0
+    cpro_humid(iel) = 0.012d0
 
   enddo
 
@@ -169,4 +169,4 @@ endif
 deallocate(lstelt)
 
 return
-end subroutine cs_user_initialization
+end subroutine cs_user_f_initialization
