@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -26,9 +26,13 @@
 ! Purpose:
 ! -------
 
-!> \file cs_user_initialization-electrics_arcs.f90
-!> \brief Electrics arcs example
+!> \file cs_user_initialization-eletric_arcs.f90
 !>
+!> \brief Electric arcs example
+!>
+!> See \subpage cs_user_initialization for examples.
+!>
+!
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -42,7 +46,7 @@
 !_______________________________________________________________________________
 
 
-subroutine cs_user_initialization &
+subroutine cs_user_f_initialization &
  ( nvar   , nscal  ,                                              &
    dt     )
 
@@ -68,7 +72,6 @@ use cpincl
 use ppincl
 use atincl
 use ctincl
-use elincl
 use ppcpfu
 use cs_coal_incl
 use cs_fuel_incl
@@ -90,67 +93,16 @@ integer          iel, mode
 integer          iesp , idimve
 
 double precision tinit, hinit, coefe(ngazem)
+character(len=80) :: f_name
 
 integer, allocatable, dimension(:) :: lstelt
 double precision, dimension(:), pointer :: cvar_scalt, cvar_ycoel
-double precision, dimension(:), pointer :: cvar_potr, cvar_poti, cvar_potva
+double precision, dimension(:), pointer :: cvar_potr, cvar_poti
+double precision, dimension(:,:), pointer :: cvar_potva
 !< [loc_var_dec]
 
 !===============================================================================
 
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_START
-!===============================================================================
-! 0.  This test allows the user to ensure that the version of this subroutine
-!       used is that from his case definition, and not that from the library.
-!     If a file from the GUI is used, this subroutine may not be mandatory,
-!       thus the default (library reference) version returns immediately.
-!===============================================================================
-!
-!< [init1]
-! For Joule heating by direct conduction, it stops
-!   you have tot be sure that the enthalpy function H(T) is the right one
-!
-if ( ippmod(ieljou).ge.1 ) then
-
-  write(nfecra,9010)
-  call csexit (1)
-
-! For electric arc, we continue because the value are given by defauft
-!       (H(T) is given from the data file dp_ELE)
-elseif (ippmod(ielarc).ge.1) then
-
-  if (ntcabs.eq.1) then
-    write(nfecra,9011)
-  endif
-
-  return
-
-endif
-!< [init1]
-
- 9010 format(                                                     &
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/,&
-'@ @@ CAUTION : Stop in the definition of Thermal properties  ',/,&
-'@    =========                                               ',/,&
-'@                      for Electric module                   ',/,&
-'@                                                            ',/,&
-'@     The user routine uselph has to be completed            ',/,&
-'@                                                            ',/,&
-'@     This user routine is used to define thermal properties ',/,&
-'@     It is unavoidable.                                     ',/,&
-'@                                                            ',/,&
-'@  The calculation will not be run.                          ',/,&
-'@                                                            ',/,&
-'@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@',/,&
-'@                                                            ',/)
- 9011 format(/,                                                   &
-' ELECTRIC ARC MODULE : THERMAL PROPERTIES ARE READ IN A FILE',/)
-
-! TEST_TO_REMOVE_FOR_USE_OF_SUBROUTINE_END
-
 !---------------
 ! Initialization
 !---------------
@@ -185,12 +137,12 @@ if ( isuite.eq.0 ) then
     mode = -1
     tinit = t0
     coefe(1) = 1.d0
-    if ( ngazg .gt. 1 ) then
-      do iesp = 2, ngazg
+    if ( ngazge .gt. 1 ) then
+      do iesp = 2, ngazge
         coefe(iesp) = 0.d0
       enddo
     endif
-    call elthht(mode,ngazg,coefe,hinit,tinit)
+    call elthht(mode,coefe,hinit,tinit)
   else
     mode = -1
     tinit = t0
@@ -207,13 +159,15 @@ if ( isuite.eq.0 ) then
 
 ! --> Mass fraction  = 1 ou 0
 
-  if ( ngazg .gt. 1 ) then
-    call field_get_val_s(ivarfl(isca(iycoel(1))), cvar_ycoel)
+  if ( ngazge .gt. 1 ) then
+    write(f_name,'(a13,i2.2)') 'esl_fraction_', 1
+    call field_get_val_s_by_name(trim(f_name), cvar_ycoel)
     do iel = 1, ncel
       cvar_ycoel(iel) = 1.d0
     enddo
-    do iesp = 2, ngazg-1
-      call field_get_val_s(ivarfl(isca(iycoel(iesp))), cvar_ycoel)
+    do iesp = 2, ngazge-1
+      write(f_name,'(a13,i2.2)') 'esl_fraction_',iesp
+      call field_get_val_s_by_name(trim(f_name), cvar_ycoel)
       do iel = 1, ncel
         cvar_ycoel(iel) = 0.d0
       enddo
@@ -224,26 +178,26 @@ if ( isuite.eq.0 ) then
 ! --> Electric potentials = 0
 
 !     -- Real Component
-  call field_get_val_s(ivarfl(isca(ipotr)), cvar_potr)
+  call field_get_val_s_by_name('elec_pot_r', cvar_potr)
   do iel = 1, ncel
     cvar_potr(iel) = 0.d0
   enddo
 
 !     -- Imaginary (for Joule heating by direct conduction)
   if ( ippmod(ieljou).eq.2 .or. ippmod(ieljou).eq.4 ) then
-    call field_get_val_s(ivarfl(isca(ipoti)), cvar_poti)
+    call field_get_val_s_by_name('elec_pot_r', cvar_poti)
     do iel = 1, ncel
       cvar_poti(iel) = 0.d0
     enddo
   endif
 
 !     -- Vector potential (3D electric arc 3D)
-  if ( ippmod(ielarc).ge.2 ) then
-    do idimve = 1, ndimve
-      call field_get_val_s(ivarfl(isca(ipotva(idimve))), cvar_potva)
-      do iel = 1, ncel
-        cvar_potva(iel) = 0.d0
-      enddo
+  if (ippmod(ielarc).ge.2) then
+    call field_get_val_v_by_name('vec_potential', cvar_potva)
+    do iel = 1, ncel
+      cvar_potva(1, iel) = 0.d0
+      cvar_potva(2, iel) = 0.d0
+      cvar_potva(3, iel) = 0.d0
     enddo
   endif
 
@@ -266,4 +220,4 @@ endif
 deallocate(lstelt) ! temporary array for cells selection
 
 return
-end subroutine cs_user_initialization
+end subroutine cs_user_f_initialization
