@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,8 +27,12 @@
 ! -------
 
 !> \file cs_user_initialization-fuel.f90
+!>
 !> \brief Fuel example
 !>
+!> See \subpage cs_user_initialization for examples.
+!>
+!
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -42,7 +46,7 @@
 !_______________________________________________________________________________
 
 
-subroutine cs_user_initialization &
+subroutine cs_user_f_initialization &
  ( nvar   , nscal  ,                                              &
    dt     )
 
@@ -68,7 +72,6 @@ use cpincl
 use ppincl
 use atincl
 use ctincl
-use elincl
 use ppcpfu
 use cs_coal_incl
 use cs_fuel_incl
@@ -116,7 +119,6 @@ double precision, dimension(:), pointer :: cvar_hox
 
 write(nfecra,9001)
 
-
 !< [init]
 d2s3 = 2.d0/3.d0
 
@@ -187,8 +189,8 @@ if ( isuite.eq.0 ) then
 
   endif
 
-! --> All the computation domain is initialised with air at TINITK
-!                   ================================================
+! --> All the computation domain is initialized with air at TINITK
+!             ====================================================
 
 ! ---- Computation of H1INIT and  H2INIT
 
@@ -251,7 +253,7 @@ if ( isuite.eq.0 ) then
     cvar_fvap(iel) = 0.d0
     cvar_f7m(iel) = zero
     cvar_fvp2m(iel) = zero
-    if ( ieqco2 .ge. 1 ) then
+    if (ieqco2 .ge. 1) then
       ioxy   =  1
       wmo2   = wmole(io2)
       wmco2  = wmole(ico2)
@@ -261,7 +263,7 @@ if ( isuite.eq.0 ) then
               +oxyh2o(ioxy)*wmh2o+oxyco2(ioxy)*wmco2 )
       cvar_yco2(iel) = oxyco2(ioxy)*wmco2/dmas
     endif
-    if ( ieqnox .eq. 1 ) then
+    if (ieqnox .eq. 1) then
       cvar_yhcn(iel) = zero
       cvar_yno(iel) = zero
       cvar_hox(iel) = h1init
@@ -285,4 +287,4 @@ endif
 !----
 
 return
-end subroutine cs_user_initialization
+end subroutine cs_user_f_initialization
