@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,8 +27,12 @@
 ! -------
 
 !> \file cs_user_initialization-gas_libby_williams.f90
+!>
 !> \brief Libby-Williams gas example
 !>
+!> See \subpage cs_user_initialization for examples.
+!>
+!
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -42,7 +46,7 @@
 !_______________________________________________________________________________
 
 
-subroutine cs_user_initialization &
+subroutine cs_user_f_initialization &
  ( nvar   , nscal  ,                                              &
    dt     )
 
@@ -68,7 +72,6 @@ use cpincl
 use ppincl
 use atincl
 use ctincl
-use elincl
 use ppcpfu
 use cs_coal_incl
 use cs_fuel_incl
@@ -93,9 +96,9 @@ double precision hinit, coefg(ngazgm)
 double precision sommqf, sommqt, sommq, tentm, fmelm
 
 
-character*80     chaine
-integer          iscal, ivar, ii
-double precision valmax, valmin
+character(len=80) :: chaine
+integer           :: iscal, ivar, ii
+double precision  :: valmax, valmin
 
 integer, allocatable, dimension(:) :: lstelt
 double precision, dimension(:), pointer :: cvar_yfm, cvar_fm, cvar_cyfp2m
@@ -203,7 +206,7 @@ if ( isuite.eq.0 ) then
 
   enddo
 
-  ! ---> Control Output of the user defined initialisation values
+  ! ---> Control Output of the user defined initialization values
 
   write(nfecra,2000)
 
@@ -235,14 +238,14 @@ endif
 !--------
 
  9001 format(                                                   /,&
-'  cs_user_initialization: variables initialisation by user'   ,/,&
+'  cs_user_initialization: variables initialization by user'   ,/,&
                                                                 /)
 
  2000 format(                                                   /,&
                                                                 /,&
 ' -----------------------------------------------------------' ,/,&
                                                                 /,&
-' ** INITIALISATION OF VARIABLES FOR Libby-Williams model'     ,/,&
+' ** INITIALIZATION OF VARIABLES FOR Libby-Williams model'     ,/,&
 '    --------------------------------------------------------' ,/,&
 '           ONLY ONE PASS'                                     ,/,&
 ' ---------------------------------'                           ,/,&
@@ -262,4 +265,4 @@ endif
 deallocate(lstelt) ! temporary array for cells selection
 
 return
-end subroutine cs_user_initialization
+end subroutine cs_user_f_initialization
