@@ -1,7 +1,7 @@
 !===============================================================================
 ! User synthetic turbulence inlet definition.
 !
-! 1) Global caracteristics of synthetic turbulence inlets
+! 1) Global characteristics of synthetic turbulence inlets
 ! 2) Caracteristics of one specific inlet
 ! 3) Accurate specification of target statistics at inlet
 !===============================================================================
@@ -12,7 +12,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -31,22 +31,26 @@
 !-------------------------------------------------------------------------------
 
 !===============================================================================
-! Purpose :
-! --------
+! Purpose:
+! -------
 
-!> Generation of synthetic turbulence at LES inlets
-
-!> Definition of global caracteristics of synthetic turbulence inlets
-
-!> nent and isuisy might be defined.
-
-!> nent = Number of inlets
-!> isuisy = 1: Reading of the LES inflow module restart file
-!>        = 0: not activated (synthetic turbulence reinitialized)
+!> \file cs_user_les_inflow-base.f90
+!>
+!> \brief Generation of synthetic turbulence at LES inlets initialization
+!>
+!> See \subpage les_inflow for examples.
+!>
+!> \c nent and \c isuisy might be defined.
+!>
+!> \c nent = Number of inlets
+!> \c isuisy = 1: Reading of the LES inflow module restart file
+!>           = 0: not activated (synthetic turbulence reinitialized)
+!>
+!-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
 ! Arguments
-!______________________________________________________________________________.
+!______________________________________________________________________________!
 !  mode           name          role                                           !
 !______________________________________________________________________________!
 !> \param[out]    nent          number of synthetic turbulence inlets
@@ -116,7 +120,7 @@ subroutine cs_user_les_inflow_define &
 
 ! Generation of synthetic turbulence at LES inlets
 
-! Definition of the caracteristics of the synthetic turbulence inlet 'nument'
+! Definition of the characteristics of the synthetic turbulence inlet 'nument'
 
 ! For each LES inlet, the following parameters might be defined:
 
@@ -130,12 +134,12 @@ subroutine cs_user_les_inflow_define &
 !         3 : Synthetic Eddy Method (SEM)
 
 !     * nelent indicates the number of "entities" relative to the method
-!       (usefull only for the Batten method and the SEM):
+!       (useful only for the Batten method and the SEM):
 
 !         for Batten : number of Fourier modes of the turbulent fluctuations
 !         for SEM    : number of synthetic eddies building the fluctuations
 
-!     * iverbo indicates the verbosity level (listing)
+!     * iverbo indicates the verbosity level (log)
 
 !          0  no specific output
 !         > 0 additionnal output (only for SEM)
@@ -155,7 +159,7 @@ subroutine cs_user_les_inflow_define &
 
 !       Note :
 !       ----
-!       - dspent usefull only for typent = 2 (Batten) or typent = 3 (SEM).
+!       - dspent useful only for typent = 2 (Batten) or typent = 3 (SEM).
 !       - Strictly positive values are required for enrent and dspent.
 !       - Accurate specification of the statistics of the flow at LES inlet
 !         can be made via the user subroutine cs_user_les_inflow_advanced.
@@ -258,7 +262,7 @@ if (nument.eq.1) then
   ! 2000 synthetic eddies contribute to the turbulent fluctuations
   nelent = 2000
 
-  ! Details concerning SEM in the listing
+  ! Details concerning SEM in the log
   iverbo = 1
 
   ! 2. Data relative to the LES inflow boundary faces
@@ -367,6 +371,7 @@ use cstphy
 use cstnum
 use mesh
 use field
+use cs_c_bindings
 
 !===============================================================================
 
@@ -396,7 +401,7 @@ double precision, dimension(:), pointer :: cpro_viscl
 !===============================================================================
 
 d2s3 = 2.d0/3.d0
-call field_get_val_s(iprpfl(iviscl), cpro_viscl)
+call field_get_val_s(iviscl, cpro_viscl)
 
 ! Example 1 : - mean velocity, Reynolds stresses an dissipation are deduced
 !==========   from a wall law for the first synthetic turbulence inlet,
@@ -490,9 +495,8 @@ if (nument.eq.1) then
     xkent = epzero
     xeent = epzero
 
-    call keenin &
-    !==========
-  ( uref2, xitur, xdh, cmu, xkappa, xkent, xeent )
+    call turbulence_bc_ke_turb_intensity&
+  ( uref2, xitur, xdh, xkent, xeent )
 
     rijent(1,ii) = d2s3*xkent
     rijent(2,ii) = d2s3*xkent
