@@ -10,7 +10,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -46,30 +46,7 @@
  *  Local headers
  *----------------------------------------------------------------------------*/
 
-#include <ple_locator.h>
-
-#include "bft_error.h"
-#include "bft_mem.h"
-#include "bft_printf.h"
-
-#include "fvm_defs.h"
-#include "fvm_selector.h"
-
-#include "cs_base.h"
-#include "cs_coupling.h"
-#include "cs_join.h"
-#include "cs_join_perio.h"
-#include "cs_mesh.h"
-#include "cs_mesh_connect.h"
-#include "cs_mesh_quantities.h"
-#include "cs_mesh_bad_cells.h"
-#include "cs_mesh_smoother.h"
-#include "cs_mesh_thinwall.h"
-#include "cs_mesh_warping.h"
-#include "cs_parall.h"
-#include "cs_post.h"
-#include "cs_preprocessor_data.h"
-#include "cs_selector.h"
+#include "cs_headers.h"
 
 /*----------------------------------------------------------------------------
  *  Header for the current file
@@ -81,6 +58,16 @@
 
 BEGIN_C_DECLS
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \file cs_user_mesh-groups_from_free_faces.c
+ *
+ * \brief Mesh modification example.
+ *
+ * See \subpage cs_user_mesh for examples.
+ */
+/*----------------------------------------------------------------------------*/
+
 /*============================================================================
  * Private function definitions
  *============================================================================*/
@@ -90,10 +77,11 @@ BEGIN_C_DECLS
  *
  * Values are transferred only to boundary faces with no group.
  *
- * parameters:
- *   mesh      <-> pointer to mesh structure
- *   tolerance <-- tolerance for mesh location
- *----------------------------------------------------------------------------*/
+ * parameters :
+ * - mesh : pointer to cs_mesh_t structure
+ * - tolerance : tolerance for mesh location
+ */
+/*----------------------------------------------------------------------------*/
 
 static void
 _mesh_groups_from_free_faces(cs_mesh_t  *mesh,
@@ -124,7 +112,7 @@ _mesh_groups_from_free_faces(cs_mesh_t  *mesh,
 
   for (i = 0; i < mesh->n_b_faces; i++) {
 
-    if (mesh->b_face_cells[i] < 1)
+    if (mesh->b_face_cells[i] < 0)
       n_free_faces += 1;
 
     else {
@@ -145,11 +133,11 @@ _mesh_groups_from_free_faces(cs_mesh_t  *mesh,
 
   for (i = 0; i < mesh->n_b_faces; i++) {
 
-    if (mesh->b_face_cells[i] < 1)
-      free_faces_list[n_free_faces++] = i+1;
+    if (mesh->b_face_cells[i] < 0)
+      free_faces_list[n_free_faces++] = i+1;    /* 1-based */
 
     else if (family_flag[mesh->b_face_family[i] - 1] == 1)
-      no_group_list[n_no_group++] = i+1;
+      no_group_list[n_no_group++] = i;          /* 0-based */
 
   }
 
@@ -168,12 +156,11 @@ _mesh_groups_from_free_faces(cs_mesh_t  *mesh,
   /* Associated PLE locator */
 
 #if defined(PLE_HAVE_MPI)
-  ple_locator_t *locator = ple_locator_create(1.0,
-                                              cs_glob_mpi_comm,
+  ple_locator_t *locator = ple_locator_create(cs_glob_mpi_comm,
                                               cs_glob_n_ranks,
                                               0);
 #else
-  ple_locator_t *locator = ple_locator_create(0.1);
+  ple_locator_t *locator = ple_locator_create();
 #endif
 
   cs_real_t *b_face_cog = NULL, * b_face_normal = NULL;
@@ -184,14 +171,17 @@ _mesh_groups_from_free_faces(cs_mesh_t  *mesh,
 
   ple_locator_set_mesh(locator,
                        free_faces,
+                       NULL,      /* options */
+                       0.,        /* absolute tolerance */
+                       tolerance, /* relative tolerance */
                        3,
                        n_no_group,
                        no_group_list,
+                       NULL,
                        b_face_cog,
                        NULL,
                        cs_coupling_mesh_extents,
-                       cs_coupling_point_in_mesh_p,
-                       NULL);
+                       cs_coupling_point_in_mesh_p);
 
   BFT_FREE(b_face_cog);
 
@@ -250,14 +240,16 @@ _mesh_groups_from_free_faces(cs_mesh_t  *mesh,
  * User function definitions
  *============================================================================*/
 
-/*----------------------------------------------------------------------------
- * Modify mesh.
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Modify geometry and mesh.
  *
  * In this example, group information from free faces is projected to
- * the neighboring boundary faces that are missing such information.
+ * the neighboring boundary faces that are missing such information. *
  *
- * The mesh structure is described in cs_mesh.h
- *----------------------------------------------------------------------------*/
+ * \param[in,out] mesh  pointer to a cs_mesh_t structure
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_user_mesh_modify(cs_mesh_t  *mesh)
