@@ -4,7 +4,7 @@
 
 ! This file is part of Code_Saturne, a general-purpose CFD tool.
 !
-! Copyright (C) 1998-2014 EDF S.A.
+! Copyright (C) 1998-2019 EDF S.A.
 !
 ! This program is free software; you can redistribute it and/or modify it under
 ! the terms of the GNU General Public License as published by the Free Software
@@ -27,8 +27,10 @@
 ! -------
 
 !> \file cs_user_physical_properties-coal_drift.f90
+!>
 !> \brief Definition of physical variable laws for scalars with a drift.
 !>
+!> See \subpage physical_properties for examples.
 !-------------------------------------------------------------------------------
 
 !-------------------------------------------------------------------------------
@@ -83,13 +85,12 @@ double precision dt(ncelet)
 
 !< [loc_var_dec]
 integer          ivart, iel, ifac
-integer          ipccp
 integer          ivar
 integer          f_id
 integer          icla
 integer          iscdri, keydri, iflid, nfld, keyccl
 
-double precision xrtp
+double precision xvart
 double precision aa1, bb1, cc1, dd1
 double precision aa2, bb2, cc2, dd2
 double precision aa3, bb3, cc3, dd3
@@ -99,19 +100,16 @@ double precision aa6, bb6, cc6, dd6
 double precision aa7, bb7, cc7, dd7
 double precision visco_O2, visco_CO, visco_H2, visco_N2
 double precision visco_SO2, visco_NH3, visco_CO2
-double precision g
 
 character*80     fname
 
-double precision, allocatable, dimension(:) :: x1, visco
+double precision, allocatable, dimension(:) :: visco
 
 double precision, dimension(:), pointer :: cpro_rom1, cpro_rom2, cpro_diam2
-double precision, dimension(:), pointer :: cpro_viscl
-double precision, dimension(:), pointer :: cpro_temp1, cpro_x2
+double precision, dimension(:), pointer :: cpro_temp1, cpro_x2, cpro_x1
 double precision, dimension(:), pointer :: cpro_ym1_3, cpro_ym1_5, cpro_ym1_7
 double precision, dimension(:), pointer :: cpro_ym1_8
 double precision, dimension(:), pointer :: cpro_ym1_9, cpro_ym1_11, cpro_ym1_12
-
 double precision, dimension(:), pointer :: cpro_taup
 double precision, dimension(:), pointer :: cpro_taupg
 !< [loc_var_dec]
@@ -123,16 +121,15 @@ double precision, dimension(:), pointer :: cpro_taupg
 !===============================================================================
 
 !< [init]
-call field_get_val_s(iprpfl(iviscl), cpro_viscl)
-allocate(x1(ncelet), visco(ncelet))
+allocate(visco(ncelet))
 
-call field_get_val_s(iprpfl(iym1(3)), cpro_ym1_3)
-call field_get_val_s(iprpfl(iym1(5)), cpro_ym1_5)
-call field_get_val_s(iprpfl(iym1(7)), cpro_ym1_7)
-call field_get_val_s(iprpfl(iym1(8)), cpro_ym1_8)
-call field_get_val_s(iprpfl(iym1(9)), cpro_ym1_9)
-call field_get_val_s(iprpfl(iym1(11)), cpro_ym1_11)
-call field_get_val_s(iprpfl(iym1(12)), cpro_ym1_12)
+call field_get_val_s(iym1(3), cpro_ym1_3)
+call field_get_val_s(iym1(5), cpro_ym1_5)
+call field_get_val_s(iym1(7), cpro_ym1_7)
+call field_get_val_s(iym1(8), cpro_ym1_8)
+call field_get_val_s(iym1(9), cpro_ym1_9)
+call field_get_val_s(iym1(11), cpro_ym1_11)
+call field_get_val_s(iym1(12), cpro_ym1_12)
 
 ! Key id for drift scalar
 call field_get_key_id("drift_scalar_model", keydri)
@@ -158,13 +155,11 @@ call field_get_n_fields(nfld)
 
 !< [example_1]
 
-g = sqrt(gx**2 + gy**2 + gz**2)
-
 ! Temperature
-call field_get_val_s(iprpfl(itemp1), cpro_temp1)
+call field_get_val_s(itemp1, cpro_temp1)
 
 ! Gas density
-call field_get_val_s(iprpfl(irom1), cpro_rom1)
+call field_get_val_s(irom1, cpro_rom1)
 
 ! First initialization
 if (ntcabs.le.1) then
@@ -173,8 +168,8 @@ if (ntcabs.le.1) then
     cpro_rom1(iel) = ro0
   enddo
   do icla = 1, nclacp
-    call field_get_val_s(iprpfl(irom2(icla)), cpro_rom2)
-    call field_get_val_s(iprpfl(idiam2(icla)), cpro_diam2)
+    call field_get_val_s(irom2(icla), cpro_rom2)
+    call field_get_val_s(idiam2(icla), cpro_diam2)
     do iel = 1, ncel
       cpro_rom2(iel)  = rho20(icla)
       cpro_diam2(iel) = diam20(icla)
@@ -223,20 +218,20 @@ dd7 = 2.9979d-15
 
 !-------------------------------------------------------------------------------
 !      law                    mu   = a + b T + c T**2 + d T**3
-!      so      cpro_viscl(iel) = a +b*xrtp+c*xrtp**2 + d*xrtp**3
+!      so      cpro_viscl(iel) = a +b*xvart+c*xvart**2 + d*xvart**3
 !-------------------------------------------------------------------------------
 
 if (ntcabs.gt.1) then
   do iel = 1, ncel
 
-    xrtp = cpro_temp1(iel)
-    visco_O2  = aa1 + xrtp*bb1 + cc1*xrtp**2 + dd1*xrtp**3
-    visco_CO  = aa2 + xrtp*bb2 + cc2*xrtp**2 + dd2*xrtp**3
-    visco_H2  = aa3 + xrtp*bb3 + cc3*xrtp**2 + dd3*xrtp**3
-    visco_N2  = aa4 + xrtp*bb4 + cc4*xrtp**2 + dd4*xrtp**3
-    visco_SO2 = aa5 + xrtp*bb5 + cc5*xrtp**2 + dd5*xrtp**3
-    visco_NH3 = aa6 + xrtp*bb6 + cc6*xrtp**2 + dd6*xrtp**3
-    visco_CO2 = aa7 + xrtp*bb7 + cc7*xrtp**2 + dd7*xrtp**3
+    xvart = cpro_temp1(iel)
+    visco_O2  = aa1 + xvart*bb1 + cc1*xvart**2 + dd1*xvart**3
+    visco_CO  = aa2 + xvart*bb2 + cc2*xvart**2 + dd2*xvart**3
+    visco_H2  = aa3 + xvart*bb3 + cc3*xvart**2 + dd3*xvart**3
+    visco_N2  = aa4 + xvart*bb4 + cc4*xvart**2 + dd4*xvart**3
+    visco_SO2 = aa5 + xvart*bb5 + cc5*xvart**2 + dd5*xvart**3
+    visco_NH3 = aa6 + xvart*bb6 + cc6*xvart**2 + dd6*xvart**3
+    visco_CO2 = aa7 + xvart*bb7 + cc7*xvart**2 + dd7*xvart**3
 
     ! Viscosity of the mixing
     visco(iel) = ( cpro_ym1_8(iel) * visco_O2                     &
@@ -246,25 +241,16 @@ if (ntcabs.gt.1) then
                  + cpro_ym1_11(iel)* visco_SO2                    &
                  + cpro_ym1_7(iel) * visco_NH3                    &
                  + cpro_ym1_9(iel) * visco_CO2 )/                 &
-                 ( cpro_ym1_8(iel) + cpro_ym1_3(iel)                   &
-                 + cpro_ym1_5(iel) + cpro_ym1_12(iel)                  &
-                 + cpro_ym1_11(iel)+ cpro_ym1_7(iel)                   &
+                 ( cpro_ym1_8(iel) + cpro_ym1_3(iel)              &
+                 + cpro_ym1_5(iel) + cpro_ym1_12(iel)             &
+                 + cpro_ym1_11(iel)+ cpro_ym1_7(iel)              &
                  + cpro_ym1_9(iel))
 
   enddo
 endif
 
-! Compute x1 = 1 - sum cpro_x2
-do iel = 1, ncel
-
-  x1(iel) = 1.d0
-
-  do icla = 1, nclacp
-    call field_get_val_s(iprpfl(ix2(icla)), cpro_x2)
-    x1(iel) = x1(iel) - cpro_x2(iel) !FIXME is x2(iel) initialized ?
-  enddo
-
-enddo
+! get x1 = 1 - sum cpro_x2
+call field_get_val_s_by_name("x_c", cpro_x1)
 
 ! All gas scalars have the same drift as if1m(1)
 !-----------------------------------------------
@@ -304,7 +290,7 @@ enddo
 
 do iflid = 0, nfld-1
 
-  ! index of the coal particle class
+  ! Index of the scalar class (<0 if the scalar belongs to the gas phase)
   call field_get_key_int(iflid, keyccl, icla)
 
   call field_get_key_int(iflid, keydri, iscdri)
@@ -312,9 +298,9 @@ do iflid = 0, nfld-1
   ! We only handle here one scalar with a drift per particle class
   if (icla.ge.1.and.btest(iscdri, DRIFT_SCALAR_ADD_DRIFT_FLUX)) then
 
-    call field_get_val_s(iprpfl(irom2(icla)), cpro_rom2)
-    call field_get_val_s(iprpfl(idiam2(icla)), cpro_diam2)
-    call field_get_val_s(iprpfl(ix2(icla)), cpro_x2)
+    call field_get_val_s(irom2(icla), cpro_rom2)
+    call field_get_val_s(idiam2(icla), cpro_diam2)
+    call field_get_val_s(ix2(icla), cpro_x2)
 
     ! Position of variables, coefficients
     ! -----------------------------------
@@ -333,9 +319,9 @@ do iflid = 0, nfld-1
     do iel = 1, ncel
 
       ! Simple model for Low Reynolds Numbers
-      cpro_taup(iel) = x1(iel) * cpro_rom2(iel)                               &
-                          * cpro_diam2(iel)**2                           &
-                          / (18.d0*visco(iel))
+      cpro_taup(iel) = cpro_x1(iel) * cpro_rom2(iel)                &
+                     * cpro_diam2(iel)**2                           &
+                     / (18.d0*visco(iel))
 
     enddo
 
@@ -344,17 +330,18 @@ do iflid = 0, nfld-1
     do iel = 1, ncel
 
       cpro_taupg(iel) = cpro_taupg(iel)                                  &
-                 - ( cpro_taup(iel) * cpro_x2(iel) )
+                      - ( cpro_taup(iel) * cpro_x2(iel) )
 
     enddo
 
-  endif
-enddo
+  endif ! test icla
+
+enddo ! loop on iflid
 
 !< [example_1]
 
 !Free memory
-deallocate(x1, visco)
+deallocate(visco)
 
 !===============================================================================
 
@@ -363,4 +350,4 @@ deallocate(x1, visco)
 !----
 
 return
-end subroutine usphyv
+end subroutine usphyv
\ No newline at end of file
