@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2014 EDF S.A.
+  Copyright (C) 1998-2019 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -61,6 +61,9 @@
 int
 main (int argc, char *argv[])
 {
+  CS_UNUSED(argc);
+  CS_UNUSED(argv);
+
   int n_groups = 6;
   int n_attributes = 6;
 
@@ -83,7 +86,7 @@ main (int argc, char *argv[])
   const char infix[] = "sphere[0, 0, 0, 2] and (not no_group[])";
 #endif
 
-  bft_mem_init(getenv("FVM_MEM_TRACE"));
+  bft_mem_init(getenv("CS_MEM_LOG"));
 
   pf = fvm_selector_postfix_create(infix,
                                    n_groups,
@@ -107,4 +110,3 @@ main (int argc, char *argv[])
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
-
